REPORT_NAME = 'oper_table'
REPORT_CAPTION = 'Операции за период'

import sguk_areports
from sguk_ide import *

api = None

#==============================================================================
#   Демонстрационный отчет
#==============================================================================
import m_repo
from m_repo import Attr, Param
from sguk_forms import *

class Report(sguk_areports.AReport):
    def __init__(self):
        super().__init__()
        self.areport_name = REPORT_CAPTION

        self.model.update({
            # --- Раздел описания параметров отчета --------------------------
            'params' : [
                    Param('first_date', 'Начало периода',     param_type='date'),
                    Param('last_date',  'Окончание периода',  param_type='date'),
                    Param('org_id',     'Организация',        param_type='OrgId', required=False),
                    Param('form_type',  'Вид отчетных форм',  param_type='form_type'),
                ],

            # --- Раздел описания таблицы данных, создаваемой отчетов --------
            'use_forms' : [],
            'use_vocs'  : ['organizations_voc_type'],
            'headers': [{'Операция':['OpCod', 'OpDate']}],
            'attrs' : [
                    Attr('report_form_id', 'ID формы'),
                    Attr('form_no', 'Отчетная форма', attr_type=api.repo.attr_types.FormNo),
                    Attr('form_period_start', 'Начало периода', attr_type='date'),
                    Attr('form_period_end', 'Конец периода', attr_type='date'),
                    Attr('table_row','Номер строки'),

                    # Атрибуты у которых сходные заголовки
                    Attr('OpCod', 'Код операции', inherit=True),
                    Attr('OpDate', 'Дата операции', inherit=True),
                    Attr('DocVid', 'Вид док-та', inherit=True),
                    Attr('DocDate', 'Дата док-та', inherit=True),
                    Attr('DocN', 'Номер док-та', inherit=True),
                    Attr('UktPrN', 'Номер прибора', inherit=True),
                    Attr('UktPrTyp', 'Тип прибора', inherit=True),
                    Attr('PrName', 'Наменование прибора', inherit=True),
                    Attr('Kateg', 'Категория ЗРИ', inherit=True),
                    Attr('Typ', 'Тип ЗРИ', inherit=True),
                    Attr('Numb', 'Номер источника', inherit=True),

                    'NAME_SHORT',
                    'KOD_ORG',
                    'OKPO',
                    'KOD_SRF',

                    '*'
                ],
            'attrs_drop': 'author_id amount mass_net mass_gross volume act_a act_b act_tr act_ur op_item org_id op_sign activity record_id',
            'attrs_drop_na': True,

            # --- Раздел описания локальных типов данных, для параметров и отчета -------------------
            'types' :{
                'form_type': m_repo.AttrTypeDict({
                    t_form_1_1: 'Форма 1.1', 
                    t_form_1_2: 'Форма 1.2',
                    t_form_1_3: 'Форма 1.3',
                    t_form_1_4: 'Форма 1.4',
                    t_form_1_5: 'Форма 1.5',
                    t_form_1_6: 'Форма 1.6',
                    }),
                'OrgId' : api.lib.core.repo.getTypeModel('organizations_voc_type')
            }
        })

    # Метод для создания отчета
    def build(self, context, **kwargs):
        params = context['params']['attrs']                                 # Извлечение параметров создания отчета
        form_type = params['form_type']

        reg_table = {
            t_form_1_1 : 'acc_o1',
            t_form_1_2 : 'acc_o2',
            t_form_1_3 : 'acc_o3',
            t_form_1_4 : 'acc_o4',
            t_form_1_5 : 'acc_o5',
            t_form_1_6 : 'acc_o6',
        }[form_type]


        sql = f'''
            SELECT 
                f.author_id, f.form_no, f.form_period_start, f.form_period_end,
                r.*, 
                vo."NAME_SHORT", vo."KOD_ORG", vo."OKPO", vo."KOD_SRF"
            FROM {reg_table} AS r 
            INNER JOIN "ReportForms" as f ON f.report_form_id = r.report_form_id
            LEFT OUTER JOIN "VocOrgs" as vo ON vo.record_id = f.author_id
        ''' 
 
        first_date = params.get('first_date')
        last_date = params.get('last_date')
        org_id = params.get('org_id')

        filters = []
        if first_date: filters.append(f''' "OpDate">='{first_date}' ''')
        if last_date: filters.append(f''' "OpDate"<='{last_date}' ''')
        if org_id: 
            kod_org = api.repo.Vocs.VocOrgs.ds.read(org_id).KOD_ORG
            filters.append(f''' r.kod_org='{kod_org}' ''')
         
        if filters: sql += ' WHERE ' + ' AND '.join(filters)

        report_data = api.context.db_connect.sql_exec(sql)                  # Расчет данных
        report_data = self.decode(report_data, ['form_no'])                 # Декодирование атрибутов  

        self.model['use_forms'] = [form_type + '_2022']                     # Подключение описаний атрибутов из форм

        return report_data      


#==============================================================================
#   Инициализация модуля
#==============================================================================

def init(_api:SgukApi, debug=False):
    global api
    api = _api

    #if api.get_config('API2_DEBUG', True):
    # Регистрация аналитического отчета в API
    api.repo.AReports.add_areport(REPORT_NAME, Report())

    # Добавление ссылки на создание отчета в главное меню
    api.app_menu.add_item('Аналитические отчеты', {
            'caption':REPORT_CAPTION, 
            'url':api.repo.AReports.get_create_url(REPORT_NAME)
        }, after='Журнал аналитических отчетов')
