import time

import m_utils
import m_rest
from sguk_api import api
from sguk_ide import *              # Используется только для подсветки синтаксиса в IDE


#========================================================================================
#   Обработчики тестовых действий
#========================================================================================
import os

import m_rec


def fillform_acion(context, **kwargs):
    report_form_id = int(context['report_form_id'])         # Выделаем ID текущей отчетной формы

    # Загружаем данные отчетной формы
    forms_ds = api.repo.ReportForms.core                    # Получаем интерфейс к работе с данными отчетных форм
    rec = forms_ds.read(report_form_id)                     # Читаем данные отчетной формы
    row_count = rec.table.count()                   

    # Обновляет OpCod для всех строк таблицы
    OpCod=str(int(rec.table[0].OpCod)+1)
    for row in rec.table:
        row.OpCod = OpCod

    # Добавляем новые строки
    row_dict = {'OpCod':'01'}
    rows_list = [{'OpCod':'02'}, {'OpCod':'03'}]
    rec.table.append(row_dict)                              # Одна строка в виде словаря
    rec.table.append(rows_list)                             # Несколько строк в виде списка
    rec.table.append( m_rec.Record(row_dict) )              # Одна строка в виде Record
    rec.table.append( m_rec.RecordSet(rows_list) )          # Несколько строк в виде RecordSet

    # Сохраняем изменения в отчетной форме
    rec = forms_ds.update(rec)

    #txt = f'report_form_id={report_form_id}<BR>{m_utils.to_html(m_utils.pretty(rec.to_dict()))}'
    txt = f'Добавлено строк: {rec.table.count() - row_count}'
    return m_rest.ApiRest.result_notify('Форма заполнена', f'{txt}', refresh='record')


def notify_action(context, api=None, **kwargs):
    action:m_actions.ActionThread = api.context.action

    steps = 10
    for i in range(steps):
        action.set_progress(i*10)           # Индикатор процесса выполнения - число меньше 100
        action.set_message(f'Tick {i}')     # Сообщение отображающее текущий статус выполнения действия
        time.sleep(0.5)

    return m_rest.ApiRest.result_notify('Тестовое действий', f'<pre>context={m_utils.to_html(m_utils.pretty(context))}</pre>')


def download_action(**kwargs):
    path = os.path.split(__file__)[0]
    file_path = os.path.join(path, 'demo_action.yaml')
    return m_rest.ApiRest.result_file(file_path)


def url_action(**kwargs):
    return m_rest.ApiRest.result_url('https://www.ya.ru')


#========================================================================================
#   Инициализация модуля
#========================================================================================

import sguk_forms

def init(_api):
    if api.get_config('API2_DEBUG', True):
        api.repo.ReportForms.add_action('ActionDemo', caption='Demo', index='0')

        api.repo.ReportForms.ActionDemo.add_action('demo_action', notify_action, caption='Notify', icon=None, text='Demo notification')
        api.repo.ReportForms.ActionDemo.add_action('download_action', download_action, caption='Download', icon=None, text='Demo download')
        api.repo.ReportForms.ActionDemo.add_action('url_action', url_action, caption='URL', icon=None, text='Demo URL')

        form_node = api.repo.ReportForms.find_form(sguk_forms.form_1_1_2022)
        form_node.add_action('FormActionDemo', caption='Demo', index='0', form_type='edit')
        form_node.FormActionDemo.add_action('demo_action', notify_action, caption='Notify', icon=None, text='Demo notification', form_type='edit')
        form_node.FormActionDemo.add_action('form fill', fillform_acion, caption='Fill form', icon=None, text='Demo', form_type='edit')

        api.repo.Vocs.add_action('demo_notify', notify_action, caption='Notify', icon=None, text='Demo notification')
