import m_ds
import m_rest
import m_repo
import sguk_areports
from sguk_ide import *

api = None

#==============================================================================
#   Демонстрационный отчет
#==============================================================================
from m_repo import Attr, Param
from sguk_forms import *

class DemoAReport(sguk_areports.AReport):
    def __init__(self):
        super().__init__()
        self.areport_name = 'Демонстрационный отчет'

        self.model.update({
            # --- Раздел описания параметров отчета --------------------------
            'params' : [
                    Param('OpCod',   'Код операции',   param_type='OpCod'),
                    Param('form_no', 'Отчетная форма', param_type='form_type'),
                    Param('org_form_no', 'Отчетная форма по организации', param_type='FormForOrgs'),
                    Param('year',    'Год',            param_type='years'),
                    Param('cod_rao', 'Код РАО',        param_type='calc_cod_rao'),
                    Param('modal_form', 'Выбор через MODAL', param_type='test_modal')
                ],

            # --- Раздел описания таблицы данных, создаваемой отчетов --------
            'attrs' : [
                    Attr('record_id'),
                    Attr('org_id', 'Организация', attr_type=api.repo.attr_types.OrgId),
                    Attr('OpCod', 'Код операции', attr_type='OpCodDesc')
                ],

            # --- Раздел описания локальных типов данных, для параметров и отчета -------------------
            'types' :{
                #'test_modal' : m_repo.AttrTypeModal('demo_modal'),

                'form_type': m_repo.AttrTypeDict({t_form_1_1:'Форма 1.1', t_form_1_2: 'Форма 1.2'}),

                'years'     : m_repo.AttrTypeList([2020, 2021, 2022]),

                'calc_cod_rao' : m_repo.AttrTypeCalc('CalcCodeRao'),    # Калькулятор на Frontend

                'OpCodDesc' : m_repo.AttrTypeDsDict(                        # Список из источника данных (Словарь кодов операций)
                        api.repo.Vocs.OpCod.ds, 
                        id_attr='value_id', 
                        label_func=lambda rec: f'{rec.value_id} : {rec.Op_Name}'
                    ),

                'FormForOrgs' : m_repo.AttrTypeDsDict(
                        m_ds.SqlDataSource('SELECT form_no, kod_org, count(*) FROM "ReportForms" GROUP BY form_no, kod_org'),
                        id_attr = 'form_no',
                        label_func=lambda rec: f'{rec.form_no} для KOD_ORG:{rec.kod_org}'
                    )
            }
        })

    # Метод для создания отчета
    def build(self, context, **kwargs):
        params = context['params']['attrs']                                 # Извлечение параметров создания отчета

        report_data = api.regs.acc_o1.select(filters={'OpCod': params['OpCod']})    # Расчет данных
        report_data = self.decode(report_data, ['org_id', 'OpCod'])         # Декодирование атрибутов  

        return report_data      


# Обработчик вызова действия по созданию отчета, возвращает URL для вызова диалога создания отчета
def demo_areport_endpoint(**kwargs):
    create_url = api.repo.AReports.get_create_url('demo_report')
    return m_rest.ApiRest.result_url(create_url)

#==============================================================================
#   Инициализация модуля
#==============================================================================

def init(_api:SgukApi, debug=False):
    global api
    api = _api

    if api.get_config('API2_DEBUG', True):
        # Регистрация аналитического отчета в API
        api.repo.AReports.add_areport('demo_report', DemoAReport())

        # Добавление ссылки на создание отчета в главное меню
        api.app_menu.add_item('Demo', {
                'caption':'demo_areport', 
                'url':api.repo.AReports.get_create_url('demo_report')
            })
