/* Динамически подгружаемый компонент, получает интерфейс к API*/

export default {
  inject: ["api"],
  template: `
    <div class="default-page-padding-inline app-font-default"
         style="display: flex; flex-direction: column; gap: 10px;">
    <h3 class="app-h-s22-h24-w7-c333">Экспорт в АЧ ({{(new Date(Date.now())).toLocaleString()}})</h3>
    <div style="display: flex; gap: 10px; flex-wrap: wrap;">
      <button-component
        class="form-button"
        @click="onDownload"
      >
        Скачать обменный файл АЧ
      </button-component>
      <button-component
        class="form-button"
        @click="onSetState"
      >
        Пометить формы как выгруженные
      </button-component>
    </div>
    <template v-if="exportData">
      <div class="app-h-s18-h24-w7-c333">Список выгружаемых отчетных форм:</div>
      <table class="unload-table" style="padding-bottom: 10px;">
        <thead>
        <tr class="table__header">
          <th class="table__th">Организация</th>
          <th class="table__th">Форма</th>
          <th class="table__th">Период с / по</th>
          <th class="table__th">№ корректировки</th>
          <th class="table__th">Статус</th>
        </tr>
        </thead>
        <tbody>
        <tr
          class="table__tr"
          :key="item.form_no + i"
          v-for="(item, i) of exportData.forms"
        >
          <td class="table__td">{{ item.author_id }}</td>
          <td class="table__td">{{ item.form_no }}</td>
          <td class="table__td">{{ item.period }}</td>
          <td class="table__td">{{ item.adjustment_no }}</td>
          <td class="table__td">{{ item.form_state }}</td>
        </tr>
        </tbody>
      </table>
      <template v-if="exportData.notify.length">
        <div class="app-h-s18-h24-w7-c333">Напоминания:</div>
        <table class="unload-table" >
          <tr class="table__tr" v-for="notify of exportData.notify">
            <td class="table__td" style="text-align:left;" v-html="toHtml(notify)">
            </td>
          </tr>
        </table>
      </template>
    </template>
    </div>
    <component :is="'style'">
    {{ css }}
    </component>
  `,
  props: {
    route: Object,
  },
  setup() {
    return {
      css: `
      .unload-table {
        border-spacing: 0;
      }
      
      .unload-table .table__td {
        text-align: center;
        line-height: 1.2em;
        padding: 5px 0;
      }
      `,
    };
  },

  data() {
    return {
      endpoint : '/api/exchange_ap/',     // URL доступа к REST API
      exportData: null,
    };
  },

  computed: {
    // --- Возвращает список форм в виде Array ----------------------------
    forms() {   
      let form_list = this.route.query.forms
      if (form_list === undefined) return []
      return Array.isArray(form_list) ? form_list : [form_list]; 
    },
  },

  methods: {                      
    // --- Преобразование строки для отображения в HTML --------------------
    toHtml(value) {
      return value.replaceAll('\n', '<BR>')
    },

    // --- Вызывает метод REST API, возвращает JSON -------------------------
    async fetchRest(method) {     
      const url = this.api.lib.ApiService.getUrl(
        this.endpoint +  method + '?',
        { forms: this.forms },
      );
      return await this.api.lib.HttpService.get(url).then((x) => x.json)
    },

    // --- Загружает данные через REST API -----------------------------------
    async loadData() {            // 
      this.exportData = await this.fetchRest('export_data')
    },

    // --- Устанавливает статус форм "Выгружена" -----------------------------
    async onSetState() {          
      const result = await this.fetchRest('export_state')

      this.api.lib.NotificationsService.send({
        title: result.caption,
        text: result.text,
        type: "info",
      });

      await this.loadData()
    },

    // --- Запускает загрузку обменного файла -------------------------------
    async onDownload() {          
      let url = this.api.lib.ApiService.getUrl(this.endpoint + "export_file?", {forms: this.forms,})
      await this.api.action.execute({ url });
    },
  },

  // --- Загрузка данных при создании страницы -------------------------------
  async created() {
    await this.loadData();
  },
};
