PAGE_FILE = 'export_ap'

import m_rest

from sguk_api import *
import sguk_forms

api = None

#============================================================================
#   Инициализация модуля
#============================================================================
def init(_api):
    global api
    api = _api

    # Регистрация компонента загрудаемого через /page/default/<имя компонента>
    api.pages.add_page_file(PAGE_FILE, PAGE_FILE + '.js', path=__file__)

    # Регистрация действия в журналах отчетных форм
    api.repo.ReportForms.ActionExchange.add_action(
        'export_ap', 
        action_export_ap, 
        caption='Выгрузить в АЧ', 
        icon='/interface/upload.svg', 
        text='Выгрузить в АЧ')

    # Регистрация endpoints для получения данных страницей
    api.add_url('/exchange_ap/export_data', ep_data)
    api.add_url('/exchange_ap/export_file', ep_file)
    api.add_url('/exchange_ap/export_state', ep_state)


#============================================================================
#   Обработка действий
#============================================================================

# Обработчик нажатия кнопки в журнале отчетных форм
def action_export_ap(context:dict=None, **kwargs):
    selected_id = [str(row['report_form_id']) for row in context.get('selected_rows',[])]

    if not selected_id:
        return m_rest.ApiRest.result_notify('Ошибка', 'Не выбраны формы для экспорта') 

    form_id_list = ",".join(selected_id)

    return m_rest.ApiRest.result_url( api.pages.get_url(PAGE_FILE, forms=form_id_list) )


#============================================================================
#   Методы REST API
#============================================================================

def get_forms(context):                     # Выделяет список id отчетных форм из параметра запроса forms
    forms_str = context['request'].args.get('forms')
    forms_id = [int(id) for id in forms_str.split(',')]
    return forms_id


# Endpoint - Формирование данных для страницы
def ep_data(context, **kwargs):
    forms = get_forms(context)
    forms_list = []                 # Список описаний отчетных форм
    notify_list = []                # Список уведомлений

    for id in forms:
        form_rec = api.repo.ReportForms.ds.read(id)
        form_name_str = 'Ф.' + '.'.join(sguk_forms.form_type(form_rec.form_no).split('_')[-2:])

        org = api.repo.Vocs.VocOrgs.ds.read(form_rec.author_id)
        forms_list.append({
            'author_id'     : org.NAME_SHORT + f' ({org.KOD_ORG})',
            'form_no'       : form_name_str,
            'period'        : m_utils.date_str_format(form_rec.form_period_start) + ' - ' + m_utils.date_str_format(form_rec.form_period_end),
            'adjustment_no' : form_rec.adjustment_no,
            'form_state'    : api.repo.attr_types.form_state.decode(form_rec.form_state)
        })

        notifyes = form_rec.get('form_notifications.main_table')
        if notifyes:
            notify_list.extend( [item.get('value',{}).get('text') for item in  notifyes.to_dict()] )

    return {
        'forms' : forms_list,
        'notify' : notify_list
    }


def calc_param_forms(context):
    res = ''
    for id in get_forms(context):
        res += f'&form_id={id}'

    return res


# Endpoint - Обработчик ссылки на закрузку обменного файла
# Создает ссылку вида /api/forms?action=unload&form_id=1&form_id=2 и т.д. (URL обрабатывается ядром)
def ep_file(context, **kwargs):
    url = '/api/forms?action=unload'
    url += calc_param_forms(context)

    return m_rest.ApiRest.result_download(url)


# Endpoint - Обработчик изменения состояний выгруженных отчетных форм
def ep_state(context, **kwargs):
    api.context['request'] = context['request']     # Пока Игнат не начал создавать api.context

    url = api.lib.rest.host_url()
    url += '/forms?action=set_forms_expt_time_unloaded_status'
    url += calc_param_forms(context)

    api.log.msg(f'ep_state: url={url}')

    core_response = api.lib.rest.post(url)
    
    try:
        core_response = core_response.json()
        if 'errors' not in core_response:
            raise Exception()
    except: 
        core_response = {'errors':[
                {'rus_message':f'Ошибочный ответ Core: {core_response.text}\nURL={url}'},
                {'rus_message':f'URL: {url}'},
            ]}

    if not core_response.get('errors'):
        return m_rest.ApiRest.result_notify('Действие выполнено', f'Статус форм изменен')
    else:
        err_txt = '\n'.join( [err.get('rus_message', '') for err in core_response.get('errors',[])] )
        if not err_txt:
            err_txt = f'Не корректный ответ функции изменения состояния: {core_response}'
        return m_rest.ApiRest.result_notify('Ошибка', err_txt, refresh='record')