/* Динамически подгружаемый компонент, получает интерфейс к API*/

export default {
  inject: ["api", "api2"],
  template: `
    <screen-spinner v-if="!is_loaded"/>
    <div v-else class="default-page-padding-inline app-font-default"
         style="display: flex; flex-direction: column; gap: 10px;">

        <h3 class="app-h-s22-h24-w7-c333">Проверка отчетных форм</h3>

        <div v-if="id_list?.length && !proc_id">
         <button-component v-if="result.status!='active'" @click="execCheck">Проверить отчетные формы</button-component>
         <b v-if="result.status=='active'">Выполняется проверка отчетных форм</b>
        </div>

        <div v-show="proc_id">
          <div style="display:inline-block; width:80%">
            <t-action-progress :proc-id="proc_id" @finish="onFinish" @progress="onProgress" />
            {{result.status_msg}}
            <t-action-progress v-if="proc_id" :progress="result.progress2" />
            {{result.status_msg2}}
          </div>
          <button-component class="form-button" @click="onCancel" style="display:inline-block; width:20%">Прервать</button-component>
        </div>

        <div v-show="log_txt.length">
          <b>Журнал сообщений</b><br>
          <textarea ref="log" v-model="log_txt" style="min-height:10em;width:90%"></textarea>
        </div>

        <a v-if="error_file_url" :href="error_file_url">Загрузить список ошибок</a>

        <div v-if="form_list">
          <b>Список отчетных форм для проверки</b>
          <table style="border-collapse: collapse">
            <tr>
              <td :style="td_style">ID отчетной формы</td>
              <td :style="td_style">Номер формы</td>
              <td :style="td_style">Начало периода</td>
              <td :style="td_style">Окончание периода</td>
              <td :style="td_style">Строк</td>
              <td :style="td_style">Состояние</td>
              <td :style="td_style">Кол-во замечаний</td>
            </tr>

            <tr v-for="[id, form] of Object.entries(form_list)">
              <td :style="td_style"><a :href="get_edit_link(id)" target="_blank">{{id}}</a></td>
              <td :style="td_style">{{form.form_no}}</td>
              <td :style="td_style">{{form.form_period_start}}</td>
              <td :style="td_style">{{form.form_period_end}}</td>
              <td :style="td_style">{{form.row_count}}</td>
              <td :style="td_style">{{form.form_state}}</td>
              <td :style="td_style">{{form.err_count!=null ? form.err_count : 'Не проверялась'}}</td>
            </tr>

            <tr>
              <td :style="td_style"><b>Итого</b></td>
              <td :style="td_style">{{id_list?.length}}</td>
              <td :style="td_style"></td>
              <td :style="td_style"></td>
              <td :style="td_style">{{total_rows}}</td>
              <td :style="td_style"></td>
              <td :style="td_style">{{total_errors}}</td>
            </tr>
          </table>  
        </div>

    </div>
  `,
 
  data() {
    return {
      panel_style : 'border: 1px solid black; margin:5px 0; padding: 5px;',
      td_style : 'border: 1px solid gray; text-align:center; padding: 0 5px;',

      endpoint : '/api/sguk_check_page/', // URL доступа к REST API модуля импорта исторических данных

      is_loaded : false,                  // Флаг окончания загрузки типов из репозитория

      progress_component : null,
      upload_component : null,

      result : {                          // Статус процесса полученный от backend
        progress: null,                       // Инидкатор прогресса :int 0...100
        progress2: null,
        status: null,                         // Код статуса :str
        status_msg: null,                     // Описание текущего этапа выполнения действия (РУС) : str
        status_msg2: null,                    // Описание текущего этапа выполнения действия (РУС) : str        
        log_msgs: null                        // Лог выполнения : str
      },                         

      proc_id : null,                     // Идентификатор действия импорта запущенного на сервере
      last_api_log_len: null,             // Длина последнего полученного result.log_msgs
      log_txt : '',

      id_list : null,                     // Список ID форм для проверки
      form_list : null,                   // Список id и ошибок в формах
      total_errors : null,                // Общее кол-во ошибок
      total_rows : null,                  // Общее кол-во строк в отчетных формах
      error_file_url : null               // URL для загрузки файла с ошибками
    };
  },

  watch: {
    result(new_val) {
      for (let i=this.last_api_log_len; i<new_val.log_msgs?.length; i++)
        this.log_add( new_val.log_msgs[i] )

      this.last_api_log_len = new_val.log_msgs?.length
    }
  },

  methods: {      
    log_add(txt) {
      this.log_txt += txt + '\r\n'
    },
    
    
    async execCheck() {
      for (let id in this.form_list)
        this.form_list[id].err_count = null
      this.total_errors = 0

      let params = {form_list: this.id_list}
      let action_url = this.api2.action.get_exec_url('sguk_check_page_check')
      this.result = await this.api.action.exec(action_url, params)
      this.proc_id = this.result.proc_id

      if (!this.proc_id) {
        this.api.action.process_reply(this.result)   // Обработка результата действия на Frontend
      } else 
        this.log_add(this.api2.ts_str()+' Проверка запущена')
    },


    update_totals() {
      if (!this.form_list) return
      
      this.total_errors = 0
      this.total_rows = 0
      for (let form of Object.values(this.form_list)) {
          this.total_errors += form.err_count
          this.total_rows += form.row_count
      } 
    },


    async load_forms() {
      this.is_loaded = false
      let reply = await this.api2.action.post('/api/sguk_check_page/get_forms', {form_list : this.id_list})
      this.form_list = reply.forms
      this.update_totals()
      this.is_loaded = true
    },


    onProgress(status) {
      this.result = status

      if (this.result.data.length) {
        this.api2.action.data_ack(this.proc_id, this.result.data.length)
        for (let item of this.result.data)
          this.form_list[item[0]]['err_count'] = item[1]
      }
      this.$refs.log.scrollTop = this.$refs.log.scrollHeight;
    },

    async onCancel() {
      this.result = await this.api.action.cancel(this.proc_id)
    },

    async onFinish(status) {
      this.onProgress(status)

      let result = await this.api.action.result(this.proc_id)
      this.proc_id = null
      this.last_api_log_len = 0

      this.form_list = result.forms
      this.error_file_url = result.error_file_url
      this.load_forms()

      this.log_add(this.api2.ts_str()+' Проверка завершена')
    },
    
    get_edit_link(id) {
      return `/editor-forms/${id}`
    }
  },

  created: async function() {
    let forms_str = this.api2.action.get_url_param('report_form_id')

    if (!forms_str) {
      this.id_list = {}
    } else {
      this.id_list = forms_str.split(',')
      this.id_list = this.id_list.map((i)=>Number(i))
    }
  
    this.is_loaded = true

    this.load_forms()
  },
};
