PAGE_NAME = 'sguk_check_page'

import m_rest

from sguk_api import *

api = None

#============================================================================
#   Инициализация модуля
#============================================================================
def init(_api):
    global api
    api = _api

    # Регистрация страницы-компонента загружаемой через /page/default/<имя компонента>
    api.pages.add_page_file(PAGE_NAME, PAGE_NAME+'.js', path=__file__)

    # Регистрация endpoints для получения данных страницей
    api.add_url('/sguk_check_page/get_forms', ep_get_forms)

    # Добавление ссылки в журналы отчетных форм
    api.repo.ReportForms.add_action(
            'sguk_check_page',                          # Название действия
            ep_check_page,                              # Функция выполнения действия
            caption='Проверить',                        # Название кнопки на панели
            index = '1',                                # Позиция при сортировке
            icon=None,                                  # Файл с иконкой для кнопки
            text='Страница проверки выбранных форм'     # Хинт к кнопку
        )

    api.repo.ReportForms.sguk_check_page.add('get_url', get_url)

    # Регистрация вызова импорта
    api.actions.add_action(None, 'sguk_check_page_check', action_check)


#============================================================================
#   Обработка действий
#============================================================================
import os

import sguk_setup_check

# Возвращает ссылку для загрузки страницы проверки
def get_url(id_list:list)->str:
    page_param_str = f'report_form_id={",".join(id_list)}'
    url = api.pages.get_url(PAGE_NAME) + '?' + page_param_str
    return url 


# Обработчик вызова страницы проверки из журнала отчетных форм 
def ep_check_page(context, **kwargs):
    id_list = [str(row['report_form_id']) for row in context['selected_rows']]

    if id_list:
        return m_rest.ApiRest.result_url( get_url(id_list) )
    else:
        return m_rest.ApiRest.result_notify('Ошибка', 'Не выбраны отчетные формы для проверки')


def get_forms(form_list):
    res = {}
    for form_id in form_list:
        form_rec = api.repo.ReportForms.ds.read(form_id)
        if not form_rec or form_rec.is_empty() or not 'main_table' in form_rec.form_table_data: 
            api.log.msg(f'Отстутствует main_table: form_id={form_id} form_rec={form_rec.to_dict()}')
            continue

        row_count = form_rec.form_table_data.main_table.count()
        res[form_id] = {
            'form_no' : form_rec.form_no,
            'form_period_start' : form_rec.form_period_start,
            'form_period_end' : form_rec.form_period_end,
            'row_count' : row_count,
            'form_state' : form_rec.form_state,
            'err_count': form_rec.form_errors.main_table.count() if form_rec.form_errors and 'main_table' in form_rec.form_errors  else None
            }

    return res


def ep_get_forms(api, context, **kwargs):
    args = api.lib.rest.get_args(context)

    if 'form_list' in args:
        form_list = args['form_list']
        res = get_forms(form_list)
        return m_rest.ApiRest.result({'forms':res})

    else:
        return m_rest.ApiRest.result_error('Не определен параметр form_list', f'{args}')


def save_errors_file(form_errors):    
    res = 'report_form_id, form_no, form_period_start, form_period_end, row, col_name, error\r'
    for id, form in form_errors.items():
        for err in form['errors']:
            res += f'{id}, {form["form_no"]}, {form["form_period_start"]}, {form["form_period_end"]},'
            res += f'{err["pointer"]["row"]}, {err["pointer"]["col_name"]},'
            try:
                err_str = f'{err["value"]["text"]}'
            except:
                raise Exception(err)

            res += err_str.replace(',', ' ').replace('\n', ' ').replace('\r', ' ')
            res += '\r'

    tmp_file_path = api.lib.files.new_tmp_file()
    with open(tmp_file_path, "w") as f:
        f.write(res)

    tmp_file_name = os.path.split(tmp_file_path)[-1]
    download_url = api.lib.files.add_download(tmp_file_name, tmp_file_path, 
            download_name='Ошибки импорта.csv'
        )
    return download_url


# Обработка действия проверки форм вызываемого со страницы проверки
def action_check(api, context, **kwargs):
    form_list = context['form_list']

    sguk_setup_check.load_vocs_cache()

    res = {}

    try:
        for n, form_id in enumerate(form_list):
            try:
                api.context.action.status_msg2 = 'Загрузка формы'

                form_rec = api.repo.ReportForms.ds.read(form_id)
                row_count = form_rec.form_table_data.main_table.count()
                api.context.action.set_message(f'Проверяется форма id={form_id} строк {row_count}')
                api.context.action.status_msg2 = 'Проверка формы'

                #sguk_setup_check.load_vocs_cache()
                check_res = api.repo.ReportForms.check_form(form_rec=form_rec)

                api.context.action.status_msg2 = 'Ошибки сохранены'
                api.context.action.status_last_msg2 = ''

                api.context.action.set_progress(round((n+1)*100/len(form_list), 1))
                api.context.action.add_data((form_id, len(check_res['errors']['main_table'])))

                res[form_id] = {
                    'form_no' : form_rec.form_no,
                    'form_period_start' : form_rec.form_period_start,
                    'form_period_end' : form_rec.form_period_end,
                    'row_count' : row_count,
                    'errors'   : check_res['errors']['main_table']
                    }

            except Exception as e:
                api.context.action.add_log(m_utils.exception_text(e))

        api.context.db_connect.commit()

        api.context.action.set_message(f'Сохранение файла ошибок')
        download_url = save_errors_file(res)

        return m_rest.ApiRest.result({'error_file_url':download_url})

    except:
        api.context.db_connect.rollback()
        raise
