/* Динамически подгружаемый компонент, получает интерфейс к API*/
export default {
    inject: ["api", "api2"],
    props: ["context", "form", "route"],
  
    template: `
      <div style="display: flex; flex-direction: column; gap: 10px; padding: 20px;">
    
        <context-input field="from_date" v-model="rec.from_date" style="width:200px"/>  
        <context-input field="to_date" v-model="rec.to_date" style="width:200px"/>
        <context-input field="op_codes" v-model="rec.op_codes" style="width:200px"/>
        <context-input field="org_id" v-model="rec.org_id" style="width:400px"/>
        
        <div style="display:flex; flex-direction:row-reverse; gap:10px;">
            <button-component class="form-button" @click="onClose">Закрыть</button-component>
            <button-component class="form-button" @click="onCreate">Создать</button-component>
        </div>
      </div>
    `,
  
    data() {
      return {
        rec: {
          from_date: "",
          to_date: "",
          op_codes: "",
          org_id: ""
        }
      };
    },
  
    methods: {
      async onCreate() {
        let params = {
                from_date : this.rec.from_date, 
                to_date : this.rec.to_date, 
                op_codes : this.rec.op_codes, 
                org_id : this.rec.org_id
            }

        let status = await this.api2.action.exec('forecast_create_form', 'ReportForms.Forecast.form_6_1_2022', params)
        let res = await this.api2.action.result(status.proc_id)

        //let url = '/reg-reports-v3/1_journal?preset=1'
        this.api.modal.commit(this, res)
      },    

      onClose() {
        this.api.modal.commit(this, {caption:'Отмена', text:'Действие отменено пользователем', refresh: 'all'})
      },    
    },
  
    async mounted() {
      this.form.set_caption('Создание формы прогноза РАО')
    },

    created: async function() {
        // Описание локальных типов полей
        this.api.repo.setAttrType('OpCodes', {base_type:'DICT', data_source:{'11':'Получение', '12':'Операция 12'}})
        this.api.repo.setAttrType('MyDate', {base_type:'DATE'})
        this.api.repo.setAttrType('MyOpCod', )
        
        // Делаем множественный выбор для кодов операций
        let OpCodes = this.api.repo.getAttrType('11_type')
        OpCodes['selection_mode'] = 'multiple'
        this.api.repo.setAttrType('OpCodes', OpCodes)

        // Описание типов используемых параметров
        this.api.repo.setAttr('from_date', {caption:'Начало периода', type:'MyDate'})
        this.api.repo.setAttr('to_date', {caption:'Окончание периода', type:'date'})
        this.api.repo.setAttr('op_codes', {caption:'Коды операций', type:'OpCodes'})
        this.api.repo.setAttr('org_id', {caption:'Организация', type:'organizations_voc_type'})
    }
  };
  