def date2str(dt):
    if not dt: 
        return None

    if type(dt) == str: 
        return dt

    if hasattr(dt, 'strftime'): 
        return dt.strftime('%Y-%m-%d')

    return None


def default(value):
    return '-' if not value else value

form_attrs = [
    'org_id', 'report_form_id', 'form_no',
    'OpCod', 'OpDate', 
    'RAOCod', 'BCod',
    'Kbm', 'Tonne', 'Sht',
    'ActTR', 'ActBG', 'ActA', 'ActUR',
    'DocVid', 'DocN', 'DocDate',
    'PH_Name', 'PH_Cod',
    'factory', 'UpCod',
    'PrName', 'UktPrTyp', 'UktPrN',
    'KbmGross', 'MassGross',
    'Salinity',
    'Subsid', 'FCP',
    'Obligations',
    'JobCod',
    'PTCName', 'TechOp', 'Stage',
    'KbmNetForecast', 'MassGrossForecast',
    'UktName', 'UktTyp', 'UktSht',
    'ClassRAO',
]


map_1_6 = {
    'report_form_id' : 'report_form_id',
    'form_no':  'form_no',
    'OpCod':	'OpCod',
    'OpDate':	('OpDate', date2str),
    'Kbm':	    'volume',
    'Tonne': 	'mass_net', 
    'Sht':	    ('amount', default),
    'ActTR':	('act_tr', default),
    'ActBG':	('act_b', default),
    'ActA':	    ('act_a', default),
    'ActUR':	('act_ur', default),

    'RAOCod':	'RAOCod',
    'BCod':	    'BCod',

    'DocVid':	'DocVid',
    'DocN':	    'DocN',
    'DocDate':	('DocDate', date2str),
    'PH_Name':	'PH_Name',
    'PH_Cod':	'PH_Cod',
    'factory':	'factory',
    'UpCod':	('UpCod', default),
    'PrName':	'PrName',
    'UktPrTyp':	'UktPrTyp',
    'UktPrN':	'UktPrN',
    'KbmGross':	[''],
    'MassGross':[''],
    'Salinity':	[''],
    'Subsid':	('Subsid', default),
    'FCP':	    ('FCP', default),
}

map_1_7 = {
    'report_form_id' : 'report_form_id',
    'form_no':  'form_no',
    'OpCod':	'OpCod',
    'OpDate':	('OpDate', date2str),
    'Kbm':	    'volume',
    'Tonne': 	'mass_net',
    'Sht':	    ('amount', default),
    'ActTR':	('act_tr', default),
    'ActBG':	('act_b', default),
    'ActA':	    ('act_a', default),
    'ActUR':	('act_ur', default),

    'RAOCod':	'g21',
    'BCod':	    'g22',
    'DocVid':	'g14',
    'DocN':	    'g15',
    'DocDate':	('g16', date2str),
    'PH_Name':	'g19',
    'PH_Cod':	'g20',
    'factory':	'factory',
    'UpCod':	('g30', default),
    'PrName':	'g4',
    'UktPrTyp':	'g5',
    'UktPrN':	'g6',
    'KbmGross':	'g10',
    'MassGross':'g11',
    'Salinity':	[''],
    'Subsid':	('g31', default),
    'FCP':	    ('g32', default),
}

map_1_8 = {
    'report_form_id' : 'report_form_id',
    'form_no':  'form_no',
    'OpCod':	'OpCod',
    'OpDate':	('OpDate', date2str),
    'Kbm':	    'volume',
    'Tonne': 	'mass_net',
    'Sht':	    ('amount', default),
    'ActTR':	('act_tr', default),
    'ActBG':	('act_b', default),
    'ActA':	    ('act_a', default),
    'ActUR':	('act_ur', default),

    'RAOCod':	'g18',
    'BCod':	    'g19',
    'DocVid':	'g11',
    'DocN':	    'g12',
    'DocDate':	('g13', date2str),
    'PH_Name':	'g16',
    'PH_Cod':	'g17',
    'factory':	'factory',
    'UpCod':	('g26', default), 
    'PrName':	[''],
    'UktPrTyp':	[''],
    'UktPrN':	'g4',
    'KbmGross':	'g6',
    'MassGross':'g7',
    'Salinity':	'g8',
    'Subsid':	('g27', default),
    'FCP':	    ('g28', default),
}

import m_rec
import sguk_forecast

def map_row(row, map):
    row.map(map, rename=True)
    row.attr_filter(map)


NO_OKPO = '12004368'

def load_sql(reg_name, okpo_attr, opCodes, fromDate=None, toDate=None, org_id=None, okpo_pip=None):

    opCodes = [f"'{code}'" for code in opCodes]
    opCodes = ','.join(opCodes)

    if okpo_pip == None: okpo_pip = NO_OKPO    # ОКПО национального оператора

    sql = f'''
        SELECT r.*, f.form_no 
        FROM {reg_name} as r 
        INNER JOIN "ReportForms" as f ON r.report_form_id = f.report_form_id
        WHERE "OpCod" in ({opCodes})
    '''

    if fromDate: sql += f''' AND "OpDate">='{fromDate}'\n'''
    if toDate:   sql += f''' AND "OpDate"<='{toDate}'\n'''
    if org_id:   sql += f''' AND org_id = {org_id}\n'''
    if okpo_pip: sql += f''' AND ( "OpCod" not in ('27','28') OR "{okpo_attr}" IN ({okpo_pip}) )\n'''

    return sql


def load_forms(op_codes, from_date=None, to_date=None, org_id=None, okpo_pip=None):

    form_list = [
        ('acc_o6', map_1_6, 'OkpoPIP'),
        ('acc_o7', map_1_7, 'g17'),
        ('acc_o8', map_1_8, 'g14'),
    ]

    db = sguk_forecast.api.context.db_connect
    res = []

    for params in form_list:
        reg_name = params[0]
        map_dict = params[1]
        okpo_attr = params[2]
        
        sql = load_sql(reg_name, okpo_attr, op_codes, from_date, to_date, org_id, okpo_pip)
        rs = db.sql_exec(sql)
        
        for rec in rs:
            rec.map(map_dict)
            rec.attr_filter(form_attrs)

        res.extend(rs.to_dict())

    return res

