from m_api import Api


from sguk_ide import *

t_form_6_1 = 't_form_6_1'
form_6_1_2022 = 'form_6_1_2022'

FORECAST_CREATE_MODAL_FORM = 'forecast_create_modal'

api = None

import os

import m_rest


def init(_api):
    global api
    api = _api

    # Создаем ветку в репозитории
    year_forms_node = api.repo.ReportForms.add_group('Forecast')
    class_node = year_forms_node.add_class(form_6_1_2022)

    # Регистрация действия по расчету прогноза
    class_node.add_action('action_forecast_calc', action_forecast_calc, caption='Рассчитать', form_type='edit')

    # Регистрация диалога создания формы
    journal_node = api.repo.ReportForms.add('6_journal')
    action_forecast_create_init(class_node, journal_node)

    # Регистрация генератора данных для заполнения атрибута Установка 
    select_techop_init()

    # Регистрируем справочники ПТЦ и Установки
    api.repo.Vocs.reg_voc('ptc_type', 'ptc_type', desc='ПТЦ')
    api.repo.Vocs.reg_voc('op_type', 'op_type', desc='Установки переработки РАО')

    # Добавление блока в главное меню
    api.app_menu.add_menu(
            {   'caption': 'Прогнозирование', 
                'items': [
                    {'description': 'Расчет прогнозируемого объема радиоактивных отходов для захоронения с учетом фактического образования радиоактивных отходов и их передачи на захоронение в предшествующие годы.'},

                    {'caption': 'Журнал прогнозов РАО', 'url': '/reg-reports-v3/6_journal'},
                    {'caption': 'Справочник ПТЦ', 'url': '/editing-voc/ptc_type'},
                    {'caption': 'Справочник технологических операций', 'url': '/editing-voc/op_type'},
                ]
            },
        )


import m_utils, m_rest

#===============================================================================================
#   Создание формы прогноза
#===============================================================================================
import forecast_map

def action_forecast_create_init(class_node, journal_node):
    class_node.add_action('forecast_create_form', action_forecast_create_form)

    # Регистрация модального диалога создания формы
    path = os.path.split(os.path.abspath(__file__))[0]
    api.modals.add_modal(FORECAST_CREATE_MODAL_FORM, file_name=os.path.join(path, 'forecast_create.js'))

    # Добавление вызова на панель действий журнала
    journal_node.add_action(
            'FormNew',                                      # Название действия
            action_forecast_create,                         # Функция выполнения действия
            caption='+ Новая форма',                        #  Название кнопки на панели
            icon=None,                                      # Файл с иконкой для кнопки
            text='Создать новую форму прогнозирования РАО', # Хинт к кнопку
            index='9',
            frontend_props={"data-type": "accent"},
        )
        #
        #
def action_forecast_create(context=None, **kwargs):
    '''
    Обработчик действия "Новая форма" для журнала прогнозов РАО
    '''
    return m_rest.ApiRest.result_modal(api.modals.get_url(FORECAST_CREATE_MODAL_FORM), context)
        #
        #
def action_forecast_create_form(api, context=None, **kwargs):
    '''
    Обработчик действия создания новой формы для журнала прогнозов РАО, вызывается из модальной формы
    '''
    if not context.get('org_id'):
        return m_rest.ApiRest.result_notify('Ошибка', 'Необходимо указать организацию')

    try:
        from_date = context.get('from_date')
        to_date = context.get('to_date')
        org_id = int(context.get('org_id'))
        op_codes = context.get('op_codes')

        if not op_codes.strip():
            op_codes = '11, 12, 13, 14, 27, 28, 31, 32, 35, 36, 38, 39, 41, 44, 45, 49, 64, 56, 57, 59, 55'

        if ';' in op_codes:
            op_codes = op_codes.split(';')
        elif ',' in op_codes:
            op_codes = op_codes.split(',')
        else:
            op_codes = op_codes.split(' ')

        op_codes = [f"{oper.strip()}" for oper in op_codes if oper.strip()]

        # Загрузка данных из регистров
        rows = forecast_map.load_forms(
            op_codes=op_codes, 
            from_date=from_date, 
            to_date=to_date, 
            org_id=org_id,
            okpo_pip="'12004368', '87810689', '13442643', '44230741', '12004368650002'" 
            )

        # Создание отчетной формы
        form = dict(
            form_no = form_6_1_2022,
            author_id = org_id,
            form_period_start = from_date,
            form_period_end = to_date,
            form_name = 'Прогноз образования РАО',
        #    form_table_data = {'main_table': rows}
        )
        form_rec = api.repo.ReportForms.core.create(form)

        if not form_rec or form_rec.is_empty():
            raise Exception('Ошибка создания формы прогноза')

        id = form_rec.report_form_id

        api.repo.ReportForms.ds.update_table(id, rows)

        api.context.db_connect.commit()

        return m_rest.ApiRest.result_url(f'/editor-forms/{id}')

    except Exception as e:
        return m_rest.ApiRest.result_notify('Ошибка', m_utils.exception_text(e))


    return m_rest.ApiRest.result_notify(
        'Завершено', 
        f'<pre>context={m_utils.to_html(m_utils.pretty(context))}</pre> form_rec type is {type(form_rec)} <br> {form_rec.to_dict()}')


#===============================================================================================
#   Формировние списков выбора установок для заданного ПТЦ
#===============================================================================================
'''
    При изменении кода РАО или ПТЦ обновляем список выбора установок 
    хранящийся в атрибуте строки 

    Выбор из списка производится отдельной модальной формой 
    которая загружает список для выбора из передаемоего контекста

    TODO
        - сделать формирование списка Установок на основании Кода РАО и ПТЦ
        - зарегистрировать обработчик события формирующий список выбора
        - зарегистрировать frontend-event на изменение Кода РАО и ПТЦ
        - сделать endpoint для предоставления списка выбора

        - сделать что бы можно было в описании типа указать URL 
            по которому форма выбора получает модель и данные, передавая контекст

        - обновить core-тип для поля Установка переработки для выбора из модальной формы

        Запасной вариант, если не получится задавать модель выбора в типе через URL
        - сделать модальную форму для выбора значения из списка загружаемого с endpoint
        - зарегистрировать модальную форму 
'''
TECHOP_LIST_URL = '/forecast/techop_list'

def select_techop_init():
    # Регистриуем endpoint для генератора списка
    api.add_url(TECHOP_LIST_URL, ep_device_list)

    # Регистрируем обработчик событие на автозаполнение атрибута TechOp при выборе PTCName
    api.repo.ReportForms.find_form(form_6_1_2022).add_frontend_event('set_TechOp', set_techop)
    api.repo.ReportForms.find_form(form_6_1_2022).add_frontend_event('set_Stage', set_stage)
    url = api.lib.frontend_events.get_url('ReportForms')
    api.lib.core.repo.updateTableFieldModel(form_6_1_2022, 'PTCName', on_change=url)
    api.lib.core.repo.updateTableFieldModel(form_6_1_2022, 'TechOp', on_change=url)

    # Регистрирует тип для выбора из списка предоставляемого по URL
    op_type = api.lib.core.repo.getTypeModel('op_type')
    #op_type['voc_type'] = 'op_type'
    op_type['data_url'] = '/api' + TECHOP_LIST_URL


def device_list(cod_rao, ptc_name):
    kat_rao_dict = {'0':'ОНАО', '1':'НАО', '2':'САО', '3':'ВАО', '4':'ОЗРИ', '9':'Прочие'}

    res = []
    remarks = []

    if cod_rao == None:
        return api.repo.Vocs.op_type.ds.select().to_dict()

    try:
        tech_op_list = None

        ptc_rec = api.repo.Vocs.ptc_type.ds.read(ptc_name)
        assert not ptc_rec.is_empty(), f'Не найден ПТЦ "{ptc_name}"'

        tech_op_str = ptc_rec.get('PTCTechOp')
        if not tech_op_str: return res
        tech_op_list = [op.strip() for op in tech_op_str.split(';')]

        for op in tech_op_list:
            op_rec = api.repo.Vocs.op_type.ds.read(op)

            if not op_rec or op_rec.is_empty(): continue

            match = {}
            match['Agr']      = cod_rao[1-1] in op_rec.Agr
            match['KatRAO']   = kat_rao_dict.get(cod_rao[2-1]) in op_rec.KatRAO 
            match['Nuclid']   = cod_rao[3-1] in op_rec.Nuclid 
            match['Yam']      = cod_rao[4-1] in op_rec.Yam
            match['HL']       = cod_rao[5-1] in op_rec.HL 
            match['PerDang']  = cod_rao[6-1] in op_rec.PerPotDang
            match['ProcMet']  = cod_rao[7-1] in op_rec.ProcMet
            match['ClassRAO'] = cod_rao[8-1] in op_rec.ClassRAO 
            match['CodTypRAO'] = cod_rao[9-1: 11-1] in op_rec.CodTypRAO 
            match['FireDang']  = cod_rao[11-1] in op_rec.FireDang

            mismatch = [k for k,v in match.items() if not v]
            if not mismatch:
                res.append(op_rec.to_dict())

            remarks.append(f'Операция: "{op_rec.value_id}"  несоответствия критериям: {mismatch}')

        remarks_str = "\n"+"\n".join(remarks)
        api.log.msg(f'Выбраны тех.операции {[op["value_id"] for op in res]} для кода РАО "{cod_rao}" и ПТЦ "{ptc_name}" {remarks_str}')
    except Exception as e:
        api.log.exception(e, f'код РАО "{cod_rao}"  ПТЦ "{ptc_name}"  tech_op_list "{tech_op_list}"')

    return res


def ep_device_list(context, **kwargs):
    try:
        args = api.lib.rest.get_args(context)

        selected_row = args.get('selected_row')
        if not selected_row: return m_rest.ApiRest.result({'result':[]})

        cod_rao = selected_row.get('RAOCod')
        ptc_name = selected_row.get('PTCName')

        data = device_list(cod_rao, ptc_name)
        return m_rest.ApiRest.result({'result':data})

    except Exception as e:
        return m_rest.ApiRest.result_exception(e)


import sguk_events

def set_techop(context: sguk_events.EventContext):
    #api.log.msg(f'set_techop: selected_cell={context.selected_cell}')

    if context.selected_cell.get('field') != 'PTCName': return

    cod_rao = context.selected_row.get('RAOCod')
    ptc_name = context.selected_row.get('PTCName')

    #api.log.msg(f'set_techop: cod_rao={cod_rao}  ptc_name={ptc_name}')

    data = device_list(cod_rao, ptc_name)

    if data and len(data)==1:
        techop = data[0]['value_id']
        context.selected_row['TechOp'] = techop
        return {'values':{'TechOp': techop}}        # , 'Stage': data[0]['StageTechOp']
    else:
        context.selected_row['TechOp'] = ''
        return {'values':{'TechOp': '', 'Stage':''}}


def set_stage(context: sguk_events.EventContext, techop=None):
    #api.log.msg(f'set_stage: selected_cell={context.selected_cell}')

    if not techop:
        if context.selected_cell.get('field') != 'TechOp': return
        techop = context.selected_cell.get('value')

    #api.log.msg(f'set_stage: techop={techop}  ')

    op_rec = api.repo.Vocs.op_type.ds.read(techop)

    if op_rec and not op_rec.is_empty():
        return {'values': {'Stage': op_rec['StageTechOp'], 'ClassRAO':op_rec['ClassRAO']}}
    else:
        return {'values':{'Stage':''}}

#===============================================================================================
#   Расчет объемов РАО для захоронения
#===============================================================================================

def form_calc(form_id):
    # Загружаем справочник ПТЦ
    # Загружаем форму
    # Заполняем форму
    # Сохраняем форму
    ptc_voc = api.repo.Vocs.ptc_type.ds.select()
    PrName_voc = api.repo.Vocs.PrName.ds.select()       # Справочника 27_voc - УКТ или иные упковки
    form    = api.repo.ReportForms.ds.read(form_id)

    api.repo.ReportForms.error_clear(form)
    add_error = lambda row_no, col_name, error_msg: api.repo.ReportForms.error_add(form, row_no, col_name, error_msg)

    for row_no, row in enumerate(form.table):
        row.set_default(None)   # Значения для отсутствующих атрибутов
        KbmNetForecast = UktSht = MassGrossForecast = 'ошибка'

        ptc_recs = ptc_voc.filter(lambda rec: rec.value_id==row.PTCName)

        if not ptc_recs.is_empty(): 
            ptc_rec = ptc_recs[0]
            OCoefKbm = m_utils.str2float(ptc_rec.OCoefKbm)
            if not OCoefKbm:
                add_error(row_no, 'PTCName', f'В справочнике ПТЦ, для ПТЦ "{row.PTCName}" не указан Общий коэф. изменения объема')

            KbmNetForecast = m_utils.str2float(row.Kbm) * OCoefKbm

            if 'UktName' not in row:
                add_error(row_no, 'UktName', f'Не указана упаковка') 
                continue

            PrName_recs = PrName_voc.filter(lambda rec: rec.value_id==row.UktName)
            if not PrName_recs.is_empty():
                container_net_volume = m_utils.str2float(PrName_recs[0]['7'])
                fill_density = m_utils.str2float(ptc_rec.PTCCase)

                if not container_net_volume:
                    add_error(row_no, 'UktTyp', f'Для установки "{row.UktName}" не определена графа 7') 
                if not fill_density:
                    add_error(row_no, 'PTCCase', f'Не определена плотность упаковки') 

                if container_net_volume and fill_density:
                    UktSht = KbmNetForecast / container_net_volume * fill_density / 100
                    
                container_gross_volume = m_utils.str2float(PrName_recs[0]['8'])
                if container_gross_volume:
                    MassGrossForecast = container_gross_volume * UktSht
                else:
                    add_error(row_no, 'UktTyp', f'Для установки "{row.UktName}" не определена графа 8') 

            else:
                add_error(row_no, 'UktTyp', f'Не найдена упаковка "{row.UktName}" в справочнике упаковок')
        else:
            add_error(row_no, 'PTCName', f'Не найден ПТЦ "{row.PTCName}"')


        row['KbmNetForecast'] = KbmNetForecast
        row['UktSht'] = UktSht
        row['MassGrossForecast'] = MassGrossForecast

    return form


def action_forecast_calc(context, **kwargs):
    src_form = context.get('report_form')
    if src_form:
        form_id = src_form['report_form_id']
    else:
        return m_rest.ApiRest.result_notify('Ошибка', 'Отсутствует параметр report_form')
    

    res_form = form_calc(form_id)

    #api.log.msg(m_utils.pretty(res_form.to_dict()))

    api.repo.ReportForms.core.update(res_form.to_dict())
    api.repo.ReportForms.ds.update_errors(form_id, res_form.form_errors, convert=False)
    api.context.db_connect.commit()

    if api.repo.ReportForms.error_count(res_form):
        return m_rest.ApiRest.result_notify('Расчет', f'Ошибки в процессе расчета', refresh='all')
    else:
        return m_rest.ApiRest.result_notify('Расчет', f'Расчет завершен', refresh='all')
