/* Динамически подгружаемый компонент, получает интерфейс к API*/
export default {
    inject: ["api", "api2"],
    props: ["context", "form", "route"],
  
    template: `
      <div style="display: flex; flex-direction: column; gap: 10px; padding: 20px;">
    
        <context-input field="org_id" v-model="rec.org_id" style="width:400px"/>
        <context-input field="tech_op" v-model="rec.tech_op" style="width:200px"/>
        <context-input field="op_codes" v-model="rec.op_codes" style="width:200px"/>
        <context-input field="start_year" v-model="rec.start_year" style="width:200px"/>
        
        <div style="display:flex; flex-direction:row-reverse; gap:10px;">
            <button-component class="form-button" @click="onClose">Закрыть</button-component>
            <button-component class="form-button" @click="onCreate">Создать</button-component>
        </div>
      </div>
    `,
  
    data() {
      return {
        rec: {
          from_date: "",
          to_date: "",
          op_codes: "",
          org_id: ""
        }
      };
    },
  
    methods: {
      async onCreate() {
        let params = {
              org_id : this.rec.org_id,
              tech_op : this.rec.tech_op,
              op_codes : this.rec.op_codes,
              start_year: this.rec.start_year,
            }

        let status = await this.api2.action.exec('forecast10_create_form', 'ReportForms.Forecast.form_6_2_2022', params)
        let res = await this.api2.action.result(status.proc_id)

        //let url = '/reg-reports-v3/1_journal?preset=1'
        this.api.modal.commit(this, res)
      },    

      onClose() {
        this.api.modal.commit(this, {caption:'Отмена', text:'Действие отменено пользователем', refresh: 'all'})
      },    
    },
  
    async mounted() {
      this.form.set_caption('Создание формы прогноза РАО')
    },

    created: async function() {
        // Описание локальных типов полей
        this.api.repo.setAttrType('MyDate', {base_type:'INT'})
        
        // Делаем множественный выбор для кодов операций
        let OpCodes = this.api.repo.getAttrType('11_type')
        OpCodes['selection_mode'] = 'multiple'
        this.api.repo.setAttrType('OpCodes', OpCodes)

        let OpTypes = this.api.repo.getAttrType('op_type')
        OpCodes['selection_mode'] = 'multiple'
        this.api.repo.setAttrType('OpTypes', OpTypes)


        // Описание типов используемых параметров
        this.api.repo.setAttr('org_id', {caption:'Организация', type:'organizations_voc_type'})
        this.api.repo.setAttr('tech_op', {caption:'Технологические операции', type:'OpTypes'})
        this.api.repo.setAttr('op_codes', {caption:'Коды операций', type:'OpCodes'})
        this.api.repo.setAttr('start_year', {caption:'Год начала расчета прогноза'})
    }
  };
  