from sguk_ide import *

t_form_6_2 = 't_form_6_2'
form_6_2_2022 = 'form_6_2_2022'

FORECAST_CREATE_MODAL_FORM = 'forecast10_create_modal'

api = None

import os

import m_rest


def init(_api):
    global api
    api = _api

    # Создаем ветку в репозитории
    journal_node = api.repo.ReportForms.Forecast
    class_node = journal_node.add_class(form_6_2_2022)

    # Регистрация диалога создания формы
    journal_node = api.repo.ReportForms['6_journal']
    action_forecast10_create_init(class_node, journal_node)


import m_utils, m_rest

#===============================================================================================
#   Создание формы прогноза
#===============================================================================================

def action_forecast10_create_init(class_node, journal_node):
    class_node.add_action('forecast10_create_form', action_forecast10_create_form)

    # Регистрация модального диалога создания формы
    path = os.path.split(os.path.abspath(__file__))[0]
    api.modals.add_modal(FORECAST_CREATE_MODAL_FORM, file_name=os.path.join(path, 'forecast10_create.js'))

    # Добавление вызова на панель действий журнала
    journal_node.add_action(
            'Forecast10mNew',                               # Название действия
            action_forecast10_create,                       # Функция выполнения действия
            caption='Прогноз 10 лет',                       #  Название кнопки на панели
            icon=None,                                      # Файл с иконкой для кнопки
            text='Создать новую форму прогнозирования РАО', # Хинт к кнопку
            index='9',
            frontend_props={"data-type": "accent"},
        )
        #
        #
def action_forecast10_create(context=None, **kwargs):
    '''
    Обработчик действия "Новая форма" для журнала прогнозов РАО
    '''
    return m_rest.ApiRest.result_modal(api.modals.get_url(FORECAST_CREATE_MODAL_FORM), context)
        #
        #
def action_forecast10_create_form(api, context=None, **kwargs):
    '''
    Обработчик действия создания новой формы для журнала прогнозов РАО, вызывается из модальной формы
    '''
    if not context.get('org_id'):
        return m_rest.ApiRest.result_notify('Ошибка', 'Необходимо указать организацию')

    try:
        # Чтение параметров заданных в модальном диалоге
        org_id = int(context.get('org_id'))
        tech_op = context.get('tech_op')
        op_codes = context.get('op_codes')
        try:
            start_year = int(context.get('start_year'))
        except:
            return m_rest.ApiRest.result_notify('Ошибка', 'Неправильный год начала периода')

        form, rows = create_form(org_id, tech_op, op_codes, start_year)     # Формироние данных для новой формы

        form_rec = api.repo.ReportForms.core.create(form)                   # Сохранение новой формы
        if not form_rec or form_rec.is_empty():
            raise Exception('Ошибка создания формы прогноза')
        id = form_rec.report_form_id

        api.repo.ReportForms.ds.update_table(id, rows)                      # Сохранение таблицы формы

        api.context.db_connect.commit()

        return m_rest.ApiRest.result_url(f'/editor-forms/{id}')

    except Exception as e:
        return m_rest.ApiRest.result_notify('Ошибка', m_utils.exception_text(e))


    return m_rest.ApiRest.result_notify(
        'Завершено', 
        f'<pre>context={m_utils.to_html(m_utils.pretty(context))}</pre> form_rec type is {type(form_rec)} <br> {form_rec.to_dict()}')


#========================================================================
#   Функция генерации данных для новой отчетной формы 
#========================================================================

def create_form(org_id, tech_op, op_codes, start_year):
    # Создание отчетной формы
    form = dict(
        form_no = form_6_2_2022,
        author_id = org_id,
        form_period_start = start_year,
        form_period_end = start_year+9,
        form_name = 'Прогноз образования РАО на 10 лет',
    )

    rows = []

    return form, rows