/* Динамически подгружаемый компонент, получает интерфейс к API*/

export default {
  inject: ["api", "api2"],
  template: `
    <screen-spinner v-if="!is_loaded"/>
    <div v-else class="default-page-padding-inline app-font-default"
         style="display: flex; flex-direction: column; gap: 10px;">

         <h3 class="app-h-s22-h24-w7-c333">Импорт исторических данных ({{(new Date(Date.now())).toLocaleString()}})</h3>

        <div>
          <context-input field="order" v-model="order" style="width:400px"/>
          <t-input-file @uploaded="onUploaded"/>
        </div>

        <button-component v-show="result.status!='active' && host_file" :disable="!host_file" class="form-button" @click="onImport">
          Импортировать
        </button-component>


        <div v-if="imported" :style="panel_style">
            <b>Результаты импорта</b>
            <p>{{result.status_msg}}</p>

            <div v-if="imported!=null">
              <p>Найдено:{{imported?.forms_count_in_file}}</p>
              <p>Импортировано ранее: {{imported?.forms_count_imported_before}}</p>
              <p>Импортировано повторно: {{imported?.forms_count_updated}}</p>
              <p>Импортировано первый раз: {{imported?.report_form_id?.length}}</p>
            </div>
        </div>

        <div v-if="form_list">
          <a :href="getCheckPageUrl()" target="_blank">Проверить импортированные формы</a>
          <br><br>
          <a :href="getOperFormsUrl()" target="_blank">Показать в журнале оперативных отчетных форм</a>
          <br><br>          
          <a :href="getYearFormsUrl()" target="_blank">Показать в журнале годовых отчетных форм</a>
          <br><br>
          
          <b>Список импортированных отчетных форм</b>
          <table style="border-collapse: collapse">
            <tr>
              <td :style="td_style">ID отчетной формы</td>
              <td :style="td_style">Номер формы</td>
              <td :style="td_style">Начало периода</td>
              <td :style="td_style">Окончание периода</td>
              <td :style="td_style">Строк</td>
            </tr>

            <tr v-for="[id, form] of Object.entries(form_list)">
              <td :style="td_style"><a :href="get_edit_link(id)" target="_blank">{{id}}</a></td>
              <td :style="td_style">{{form.form_no}}</td>
              <td :style="td_style">{{form.form_period_start}}</td>
              <td :style="td_style">{{form.form_period_end}}</td>
              <td :style="td_style">{{form.row_count}}</td>
            </tr>

            <tr>
              <td :style="td_style">Итого форм {{form_list?.length}}</td>
              <td :style="td_style"></td>
              <td :style="td_style"></td>
              <td :style="td_style"></td>
              <td :style="td_style">{{total_rows}}</td>
            </tr>
          </table>  
        </div>

        <div v-show="proc_id">
          <div style="display:inline-block; width:80%">
            <t-action-progress :proc-id="proc_id" @finish="onFinish" @progress="onProgress" />
            {{result.status_msg}}
          </div>
          <button-component class="form-button" @click="onCancel" style="display:inline-block; width:20%">Прервать</button-component>
        </div>

        <div v-show="log_txt.length">
          <b>Журнал сообщений</b><br>
          <textarea ref="log" v-model="log_txt" style="min-height:10em;width:90%"></textarea>
        </div>

        <a v-if="error_file_url" :href="error_file_url">Загрузить список ошибок</a>
    </div>
  `,
 
  data() {
    return {
      panel_style : 'border: 1px solid black; margin:5px 0; padding: 5px;',
      td_style : 'border: 1px solid gray; text-align:center; padding: 0 5px;',

      endpoint : '/api/import_hist/',     // URL доступа к REST API модуля импорта исторических данных
      order : '',                         // Код выбранного приказа
      host_file : null,                   // Путь к загруженному файлу на сервере

      is_loaded : false,                  // Флаг окончания загрузки типов из репозитория

      progress_component : null,
      upload_component : null,

      result : {                          // Статус процесса полученный от backend
        progress: null,                       // Инидкатор прогресса :int 0...100
        progress2: null,
        status: null,                         // Код статуса :str
        status_msg: null,                     // Описание текущего этапа выполнения действия (РУС) : str
        status_msg2: null,
        log_msgs: null                        // Лог выполнения : str
      },                         

      proc_id : null,                     // Идентификатор действия импорта запущенного на сервере
      last_api_log_len: null,             // Длина последнего полученного result.log_msgs
      log_txt : '',

      import_ts : null,                   // Штамп времени начала импорта
      log_import_id : null,               // ID операции импорта
      imported : null,
      updated_id : null,                  // Список ID форм для проверки
      form_list : null,                   // Список id и ошибок в формах
      total_rows : null,                  // Общее кол-во строк в отчетных формах
    };
  },

  watch: {
    result(new_val) {
      for (let i=this.last_api_log_len; i<new_val.log_msgs?.length; i++)
        this.log_add( new_val.log_msgs[i] )

      this.last_api_log_len = new_val.log_msgs?.length
    }
  },

  methods: {      
    log_add(txt) {
      this.log_txt += txt + '\r\n'
    },
    
    onUploaded(event) {
      this.host_file = Object.values(event.uploaded)[0]
      this.imported = null
      this.form_list = null
      this.total_errors = null
      this.error_file_url = null
      this.log_txt = ''
    },

    async exec(params) {
      this.result = await this.api.action.exec(this.api2.action.get_exec_url('import_hist_file'), params)
      this.proc_id = this.result.proc_id

      if (!this.proc_id) {
        this.api.action.process_reply(this.result)   // Обработка результата действия на Frontend
      } else 
        this.log_add(this.api2.ts_str()+'Импорт запущен')
    },

    getCheckPageUrl() {
      return '/page/default/sguk_check_page?report_form_id=' + this.updated_id.join(',')
    },

    getOperFormsUrl() {
      return `/reg-reports-v3/1_journal?filter=[%22and%22,[%22=%22,%22log_id%22,%22${this.log_import_id}%22]]`
    },

    getYearFormsUrl() {
      return `/reg-reports-v3/2_journal?filter=[%22and%22,[%22=%22,%22log_id%22,%22${this.log_import_id}%22]]`
    },

    update_totals() {
      if (!this.form_list) return
      
      this.total_errors = 0
      this.total_rows = 0
      for (let form of Object.values(this.form_list)) {
          this.total_errors += form.err_count
          this.total_rows += form.row_count
      } 
    },

    async load_forms(form_list=null) {
      if (form_list==null) form_list = this.updated_id

      let reply = await this.api2.action.post('/api/import_hist/get_forms', {form_list})

      if (form_list==this.updated_id)
        this.form_list = reply.forms
      else {
        for (let [id, form] of Object.entries(reply.forms))
          this.form_list[id] = form
      }

      this.update_totals()
    },


    onProgress(status) {
      this.result = status

      if (this.result.data.length) {
        this.api2.action.data_ack(this.proc_id, this.result.data.length)
      }
      this.$refs.log.scrollTop = this.$refs.log.scrollHeight;
    },

    async onCancel() {
      this.result = await this.api.action.cancel(this.proc_id)
    },

    async onFinish(status) {
      this.onProgress(status)

      let result = await this.api.action.result(this.proc_id)
      this.proc_id = null
      this.last_api_log_len = 0

      this.import_ts = result.form_import_ts
      this.log_import_id = result.log_import_id
      this.imported = result
      this.updated_id = result.report_form_id.concat(result.report_form_id_updated)
      this.load_forms()

      this.log_add(this.api2.ts_str()+' Импорт завершен')
    },
    

    async onImport() { 
      this.imported = null
      this.updated_id = null

      console.log('Import started')

      this.exec({order_id : this.order, file_id:this.host_file})
    },

    get_edit_link(id) {
      return `/editor-forms/${id}`
    }
  },

  created: async function() {
    // Загрузка типов атрибутов (orders_list - список приказов)
    await this.api.repo.loadModel('/api/import_hist/get_params')
    //this.api.repo.setAttrType('orders_list', {base_type:'DICT', data_source:{'1':'Приказ 1'}})

    // Описание типов используемых параметров
    this.api.repo.setAttr('order', {caption:'Формат файла', type:'orders_list'})
    this.api.repo.setAttr('file', {caption:'Файл для импорта', type:'OpCod'})
    this.is_loaded = true
  },
};
