PAGE_NAME = 'import_hist'

import m_rest, m_repo

from sguk_ide import *

api = None

#============================================================================
#   Инициализация модуля
#============================================================================
def init(_api:SgukApi):
    global api
    api = _api

    # Регистрация страницы-компонента загружаемой через /page/default/<имя компонента>
    api.pages.add_page_file(PAGE_NAME, PAGE_NAME+'.js', path=__file__)

    # Регистрация endpoints для получения данных страницей
    api.add_url('/import_hist/get_params', ep_get_params)
    api.add_url('/import_hist/get_forms', ep_get_forms)

    # Добавление ссылки в главное меню
    api.app_menu.add_item('Управление и настройки', {
            'caption':'Импорт исторических данных', 
            'url':api.pages.get_url(PAGE_NAME)
        })

    # Регистрация вызова импорта
    api.actions.add_action(None, 'import_hist_file', action_import_hist_file)


#============================================================================
#   Обработка действий
#============================================================================
import time, os

# Обработчик вызова импорта
def action_import_hist_file(api, context, **kwargs):
    action = api.context.action
    action.add_log(f'context: {context}')
    order_id = context['order_id']      # Код формата файла 
    host_file = context['file_id']        # Путь к файлу на backend

    # TODO Вызов функции импорта от Игната
    if order_id == 'test':
        dummy_import(api, order_id, server_file_name=host_file)
        import_res = 'Тест завершен'
    else:
        try:
            import_res = api.lib.import_hist.import_historical_data(
                api, 
                api.context.request, 
                api.context.user, 
                host_file,
                order_id, 
                kwargs['done'],  # ключ координации потоков для прерывания потока
                connection=api.context.db_connect.conn)

            api.context.db_connect.commit()

        except Exception as e:
            api.context.db_connect.rollback()
            raise

        finally:
            if host_file and os.path.isfile(host_file):
                os.remove(host_file)

    return m_rest.ApiRest.result(import_res)

        
def dummy_import(api, order_id, server_file_name):
    action = api.context.action
    action.add_log(f'order_id={order_id}  server_file_name={server_file_name}')

    for i in range(10+1):
        action.set_progress(i*10)
        action.set_message(f'{i*10}% завершено')
        action.add_log(f'Выполнено {i*10}%')

        time.sleep(0.5)

        if not action.is_active(): break


def get_forms(form_list):
    res = {}
    for form_id in form_list:
        form_rec = api.repo.ReportForms.ds.read(form_id)
        row_count = form_rec.form_table_data.main_table.count()
        res[form_id] = {
            'form_no' : form_rec.form_no,
            'form_period_start' : form_rec.form_period_start,
            'form_period_end' : form_rec.form_period_end,
            'row_count' : row_count,
            'err_count': form_rec.form_errors.main_table.count() if form_rec.get('form_errors') and form_rec.form_errors.get('main_table') else None
            }

    return res


def ep_get_forms(api, context, **kwargs):
    args = api.lib.rest.get_args(context)

    if 'form_list' in args:
        form_list = args['form_list']
        res = get_forms(form_list)
        return m_rest.ApiRest.result({'forms':res})

    else:
        return m_rest.ApiRest.result_error('Не определен параметр form_list', f'{args}')


#============================================================================
#   Методы REST API
#============================================================================

# Endpoint - Формирование данных для страницы
def ep_get_params(**kwargs):
    orders_list = {                     # Коды форматов файлов
        'test'        : 'Тест',
        'decree_449'  : 'Приказ №449 (MDB)',
        'decree_600'  : 'Приказ №600 (MDB)',
        'decree_1_19' : 'Приказ №1/19-НПА (MDB)',
        'decree_1_24' : 'Приказ №1/24-НПА (MDB)',
        'decree_1_13' : 'Приказ №1/13-НПА (МПЗФ)',
    }

    return m_rest.ApiRest.result({
        'attr_types' : {
            'orders_list' : m_repo.AttrTypeDict(orders_list).frontend_model(),
        },
    })

