// Хранилище данных используемых компонентами для ввода данных
export const compStorage = {
    template: '<slot></slot>',

    data() {return {
        storage: {
            model: {},
            record: {}
        },
        params: {}
    }},

    provide() {return {
        storage: Vue.computed(()=>this.storage),
        params: Vue.computed(()=>this.params)
    }}
}

// Компонент ввода данных для атрибута, показывает название атрибута
export const compEntry = { 
    props:['field', 'caption'], 
    inject:['storage'],
    template: `
        <span style="grid-column-start:1;margin:0 10px">{{capt}}</span>
        <input v-model="storage.record[field]" v-bind="$attrs">
    `,
    computed: {
        capt: function() {return this.caption ? this.caption : this.storage.model[this.field]}
    }
}

// Компонент панели действий для модальной формы
export const compPanel = {
    inject:['api', 'modal'],
    emits:['save', 'ok', 'mount'],
    template: `
        <hr>
        <div style="display:grid; grid-template-columns: repeat(4, 1fr)">
            <button @click="$emit('save')">Сохр.</button>
            <button @click="$emit('ok');close()">Ок</button>
            <span/>
            <button @click="close">Закрыть</button>
        </div>
    `,
    mounted() {this.$emit("mount")},
    methods: {
        close() {
            this.modal.close()
        }
    }
}

import {Page} from "/static/page.js"
import {modalPanel} from "/static/modal.js"

export function registerComponents(app) {
    app.component('storage', compStorage)
    app.component('page', Page)        
    app.component('modal', modalPanel)        
    app.component('entry', compEntry)
    app.component('panel', compPanel)
}