// Компонент для отображения загружаемой страницы или формы

export const Page = {
    template: `
        <storage>
            <component :is="dlg"/>
            <div v-if="msg!=null">{{msg}}</div>
            <div v-if="err!=null" style="background-color:red">{{err}}</div>
        </storage>
    `,

    props: ['page'],
    inject: ['api'],

    data() { return {
        dlg: null,              // Vue-структруа формы ввода
        err: null,              // Сообщение об ошибке при загрузке
        msg: null               // Сообщение статуса загрузки
    }},

    methods: {
        async load() {          // Загрузка компонента для отображения
            this.msg = 'Загрузка страницы ' + this.page + ' ...'
            this.err = null
            let url = this.api.page_url(this.page)

            try {
                let res = await import(url)
                this.dlg = Vue.shallowRef( res['default'] )    
                console.log('page.js:', this.url, 'loaded')
            } catch (e) {
                this.err = 'Ошибка загрузки страницы ' + this.page + '\nURL:' + url
                console.log('page.js: load exception', e)
            }

            this.msg = null
        }
    },

    watch: {
        page(newVal) { 
            this.load()
        }
    },

    mounted() {
        if (this.page) this.load()
    },
}