function parse_args(args) {
    let params = {}
    let content = ''
    for (let i=0; i<args.length; i++) {
        let item = args[i]
        if (typeof(item) == 'object')
            params = {...params, ...item}
        else
            content += ' ' + item
    }
    return {params, content}
}

export function h3(txt) {
    return `<h3 style="margin:0.5em">${txt}</h3>`
}

export function row() {
    let {params, content} = parse_args(arguments)

    let style_in = ''

    if (params['columns']) style_in += `display:grid; grid-template-columns: ${params['columns']}; grid-gap:10px; width:auto;`
    
    let attrs_in = (style_in) ? 'style="'+style_in + '"' : ''

    let html = `<div ${attrs_in}>${content}</div>`
    return html
}

export function panel() {
    let {params, content} = parse_args(arguments)

    let style_out = ''
    let style_in = ''

    if (params['columns']) style_in += `display:grid; grid-template-columns: ${params['columns']}; grid-gap:5px; width:auto;`
    
    let caption = params['caption'] ? `<div style="position:absolute; margin-top:-1.5em; background-color:lightgray;width:min-content;padding:0 5px"><b>${params['caption']}</b></div>` : ''

    if (params['border'])
        style_out += 'border: 1px solid black; margin: 0.5em 0; padding:1em;'

    let attrs_out = (style_out) ? 'style="'+style_out + '"' : ''
    let attrs_in = (style_in) ? 'style="'+style_in + '"' : ''

    let html = `<div ${attrs_out}>${caption}<div ${attrs_in}>${content}</div></div>`
    return html
}

export function entry(field, caption=null) {
    return `<entry field="${field}" ${caption ? 'caption="'+caption+'"' : ''}/>`
}

export function split() {
    return '<div style="grid-column:1/-1"><hr></div>'
}

export function button(caption, event) {
    return `<button @click="${event}">${caption}</button>`
}
