/* Динамически подгружаемый компонент

получает интерфейс к API
*/
export default {
    inject: ["api", "params", "storage","modal"],        // Импорт интерфейса к API

    // Описание формы ввода
    // используются компоненты entry и panel зарегистрированные глобально в Vue-приложении
    // из файла components.js
    template:`
        <div style="display:grid; grid-template-columns: auto 1fr">
            <entry field="docClass" disabled/> 
            <entry field="docId" disabled/> 
            <entry field="a"/> 
            <entry field="b"/>
            <div style="grid-column:1/-1"><hr></div>
            <entry field="c"/>
            Комментарии
            <textarea v-model="storage.record.c" style="grid-column:1/-1"/>
        </div>

        <panel @ok="save" @save="save" @mount="load"/>
        `,

    data() { return{
        docClass: 'test-doc'
    }},

    methods: {
        // Загрузка данных, обработчики события генерируемого компонентом panel
        load() {
            this.storage.model = this.api.model_load(this.docClass)         // Зарузка модели данных через API
            this.storage.model['c'] = 'Remark'                              // Добавление атрибуты не описанного в модели

            this.storage.record = this.api.data_load(this.docClass, this.params.id)  // Загрузка данных через API

            this.storage.record = {...this.storage.record, ...this.modal.params}
        },

        // Сохранение данных, обработчики события генерируемого компонентом panel
        save() {
            this.api.data_save(this.docClass, this.storage.record)          // Сохранение данных через API
            this.modal.setResult(this.storage.record)                       // Установка результата
        },
    },

    async mounted() {
        // Пример импорта дополнительной библиотеки
        let add_funcs = await import("/static/test-lib.js")
        this.api = {...this.api, ...add_funcs['default']}

        this.api.test()
    }
}


