def init(api):
    #api.set('frontend_models', ensure=True)

    # Регистрация класса модели данных для страницы
    api.frontend_models.add_page_type('browse_edit', BrowseEditFrontendModel)


import m_pages, m_web_ds, m_tree


class BrowseEditFrontendModel(m_pages.FrontendModel):
    '''
    Модель для страниц вида browse_edit реализованного на frontend
    Каждый вид отображаемой страницы является экземпляром этого класса

    BrowseEditFrontendModel(model_node, title, attrs_model)
        model_node - нода в которой создается модель
        title - заголовок страницы
        attrs_model - локальная модель данных обновлящая собираемую по-умолчанию для родительской ноды 
    '''
    def __init__(self, model_node:m_tree.Node, title, attrs_model:m_tree.Node=None, ds=None, headers=None):
        super().__init__(model_node, model_node._name, title)

        self._attrs_model = attrs_model
        self._headers = headers
        if ds==None: ds = self._node._parent.find_datasource()      

        self._web_ds = m_web_ds.WebDs(model_node, ds)              # Создание REST API к источнику данных


    def get_url(self):
        '''
        Возвращает URL страницы. Обработка маршрута релизована на frontend
        
        '''
        return f'/browse_edit/{self._node._parent._name}'


    def get_action_model(self, caption:str):
        return {'caption':caption, 'url': self.get_url()}


    def get_menu_item_model(self, title: str):
        return {'caption':title, 'url': self.get_url()}


    def frontend_model(self, *args): 
        # TODO собирает атрибуты для маршрута repo_path

        attrs_model = self._node._parent.gather_attrs(self._attrs_model)
        #api.log.msg(f'BrowseEditModel: attrs_model={attrs_model}')

        headers = self._headers or attrs_model.get_headers()
        types = attrs_model.get_types()

        return {
            "data_source_url"   : self._web_ds.get_url(),               # URL источника данных          
            "actions_list_url"  : '/api'+m_pages.FRONTEND_ACTIONS_URL,  # URL для загрузки списка действий
            "edit_action"       : None,                                 # URL для действия редактирования, по-умолчанию создается на Frontend
            "caption"           : self._title,                          # Заголовок формы
            "table_options"     : {"numCol":False, "numRow":False},     # Опции таблицы
            
            # Далее структура данных аналогичная аналогична получаемой по адресу 
            # /api/repository/vocs ? action=read
            "form" : {
                "table" : {
                    "model"     : attrs_model.frontend_model(),
                    "headers"   : headers
                }
            },
            "edit" : {"edit": True},
            "types" : types,
        }

