API_VERSION = '1.1'

MODULES_PATH = 'api.modules'

import sys
import importlib

import m_tree
import m_log
import m_context

api = None

def init(_api):
    global api
    api = _api


class ApiException(Exception):
    pass


from m_tree import Node

#====================================================================================    
disabled = []

class ModuleApi:
    def __init__(self):
        self._node:m_tree.Node = None
        self._debug = False


    def is_debug(self):
        return self._debug


    def set_debug(self, _debug):
        self._debug = _debug


    def log(self, *args, data=None):
        if self.is_debug():
            api.log.dump_add(*args, data=data) 

'''
    @property
    def api(self):
        if self._node:
            return self._node.root
        else:
            return api

def reload_module(_path):
    if type(_path) == str:
        module = api.find(_path)._data

    elif type(_path) == m_tree.Node:
        module = _path._data

    else:
        raise Exception(f'Не корректный маршрут {_path} : {type(_path)}')

    if type(module) == types.ModuleType:
        importlib.reload(_path._data)
    else:
        raise Exception(f'Не корректный тип модуля {type(module)} для пути {_path}')
'''

class Api:
    def __init__(self, config={}):
        self.node = Node('api', self)            # Ссылка на ноду
        self.node.add('versions.api', API_VERSION)
        self.node.add('modules', show_children=False)

        m_log.Log(self.node)
        self.node.add('config', config)
        self.node.add('context', m_context.Context())

        #self.node.add(MODULES_PATH + '.reload', reload_module)
        #self.node.find(MODULES_PATH).provide('reload')
            #
            #
    def get_config(self, key, default=None):                    # Возвращает значение параметра конфигурации
        config = self.node.find('api.config')
        if not config: return default

        if key in config:
            value = config.get_attr(key)
            if isinstance(value, m_tree.Node):
                value = value._data 
            return value
        else:
            return default
            #
            #
    def stdout(self, *args, **kwargs):
        if not self.get_config('API2_SILENT', False):
            print(*args, **kwargs)
            #
            #
    def add_module_api(self, module_path:str, module:ModuleApi) -> Node:    # Регистрирует API модуля
        if not isinstance(module, ModuleApi):
            raise Exception(f'app_module_api: модуль {module} для "{module_path}" должен быть потомком m_api.ModuleApi')

        module_node = self.node.add(module_path, module, node_type='ModuleAPI')
        module._node = module_node
        return module_node
            #
            #
    def add_module(self, module_name, **kwargs):                # Загружает модуль python и регистрирует его в дереве api
        if module_name in sys.modules:
            module = sys.modules[module_name]
        else:
            module = importlib.import_module(module_name)

        if not self.node.find(MODULES_PATH + '.' + module_name):
            if module_name not in disabled:
                if hasattr(module, 'init'):
                    module.init(self.node, **kwargs)
                self.node.add(MODULES_PATH + '.' + module_name, module, node_type='ModuleSrc')
                self.stdout(f'Module {module_name} loaded from {module.__file__} initialized')
            else:
                self.stdout(f'Module {module_name} loaded from {module.__file__} DISABLES')

        return module
            #
            #
    def add_url(self, rest_url, func):
        full_url = '/api' + rest_url
        self.node.add('web.'+ full_url, func)
        if self.node.web.get_attr('add_url'):
            self.node.web.add_url(rest_url, func)
        else:
            print(f'Не загружен модуль api.web. URL {full_url} не активен')


def make_node(config=None):
    node = Api(config=config).node
    return node
    

