import m_api
import m_db
import m_db_pg
import m_db_sqlite

SYSTEM_CONNECT_PATH = 'system.system_db_connect'

class ApiDb(m_api.Api):
    def __init__(self, config={}) -> None:
        super().__init__(config=config)

        self.loaded=False
            #
            #
    def create_db_connect(self, core_db_connect=None, db_name:str=None):

        db_type = self.get_config('DB_SYSTEM', 'postgresql')
        if db_type == 'postgresql':
            db = m_db_pg.DbPgConnect(core_db_connect=core_db_connect)
            if not db_name:
                if self.node.find('system.system_db_connect'):
                    db_name = self.node.system.system_db_connect.DB_NAME
                else:
                    db_name = self.get_config('POSTGRES_DATABASE')

            db.connect(db_name=db_name)
            # print(f'Connected to {db_name}')

        elif db_type == 'sqlite':
            db = None
            if core_db_connect:
                db = m_db_sqlite.DbSQLiteConnect(core_db_connect=core_db_connect)
                self.log.msg(f'create_db_connect: used core_db_connect={core_db_connect}')

            elif self.find('system.sqlite_connect'):
                sqlite_connect = api.system.sqlite_connect()
                if sqlite_connect:
                    db = m_db_sqlite.DbSQLiteConnect(core_db_connect=sqlite_connect)
                    db_path = db.db_path()
                    msg = f'create_db_connect: api.system.sqlite_connect() -> {sqlite_connect} : {type(sqlite_connect)} : {db} : {db_path}'
                    if self.loaded:
                        self.log.msg(msg)
                    else:
                        print(msg)

            if not db:
                db = m_db_sqlite.DbSQLiteConnect(db_file=api.get_config('SQLITE_DATABASE'))
                if self.get_config('API2_DEBUG', False):
                    self.log.msg(f'create_db_connect: created new DB connect {db}')

        else:
            raise Exception(f'Неизвестный тип БД DB_SYSTEM={db_type} доступны: "postgresql", "sqlite"')

        if not db:
            raise Exception('Ошибка создания подключения к БД')

        return db
            #
            #
    def build_context(self, core_db_connect=None):
        if not 'db_connect' in self.node.context:
            db_conn = self.create_db_connect(core_db_connect=core_db_connect)
            self.node.context['db_connect'] = db_conn
                #
                #
    def clear_context(self):
        if 'db_connect' in api.context:
            db_connect = api.context.db_connect
            if db_connect.conn_self_created:
                db_connect.commit()
                db_connect.disconnect()

        self.context.clear()
        #api.log.msg('clear_context: cleared')


def make_node(config=None):
    api_obj = ApiDb(config=config)
    api = api_obj.node

    m_db.init(api)
    m_db_pg.init(api)
    m_db_sqlite.init(api)
    api.build_context()
    api.add(SYSTEM_CONNECT_PATH, api.context.db_connect)

    return api