'''
Модуль хранения контекста разделенного между потоками
'''

import threading

class Context:
    def __init__(self, data:dict={}):
        self.__dict__['_data'] = {}
        self.set_data(data)


    def _get_th_id(self):
        return threading.get_ident()


    def _context_data(self, th_id=None):
        if not th_id:
            th_id = self._get_th_id()
        if th_id not in self._data:
            self._data[th_id] = {}

        return self._data[th_id]


    def get(self, attr_name, default=None, th_id=None):
        _data = self._context_data(th_id)
        return _data.get(attr_name, default)


    def set_data(self, data):
        th_id = self._get_th_id()
        self._data[th_id] = dict(data)


    def clear(self):
        th_id = self._get_th_id()
        if th_id in self._data:
            del self._data[th_id]


    def __getitem__(self, __name: str):
        _data = self._context_data()
        if __name not in _data:
            return super().__getitem__(__name)
        return _data[__name]


    def __getattr__(self, __name: str):
        _data = self._context_data()
        if __name not in _data:
            return super().__getattr__(__name)
        return _data[__name]


    def __setitem__(self, __name:str, __value):
        _data = self._context_data()
        _data[__name] = __value


    def __setattr__(self, __name: str, __value):
        _data = self._context_data()
        _data[__name] = __value


    def __contains__(self, name):
        return name in self._context_data()


    def __repr__(self) -> str:
        return f'Context: {self._context_data()}'


    def to_dict(self):
        return dict(self._context_data())


    def clear(self):
        del self._data[self._get_th_id()]



context = Context()