import sqlite3, json, os
import m_db

api = None

def init(_api):
    global api
    api = _api

class DbSQLiteConnect(m_db.DbAbstractConnect):
    def __init__(self, db_file=None, core_db_connect=None, autocreate=False, **kwargs):
        super().__init__(core_db_connect=core_db_connect)

        self.db_type = 'sqlite'
        self.autocommit = True
        self.autocreate= autocreate

        self.value_convertors = {
            'date' : lambda value: f"'{value}'",
            'time' : lambda value: f"'{value}'",
            'datetime' : lambda value: f"'{value}'", 
        }

        if core_db_connect:
            self.db_file = self.db_path()

        elif db_file:
            self.db_file = db_file if db_file else ''
            self.connect()
            api.log.msg(f'Создано поключение {self}')
        #
        #
    def __repr__(self):
        return f'DbSQLiteConnect({self.db_file}, {self.conn})'
            #
            #
    def is_closed(self):
        return not bool(self.conn)
            #
            #
    def connect(self):
        if self.conn == None:
            if self.autocreate or os.path.isfile(self.db_file):
                try:
                    self.conn = sqlite3.connect(self.db_file)
                except Exception as e:
                    raise Exception(f'Ошибка открытия файла БД {self.db_file}. {e}')
            else:
                raise Exception(f'Файл {self.db_file} не найден. Для создания свойство .autocreate=True')

        return self.conn        
            #
            #
    def connect_ensure(self):
        try:
            self.is_transaction_active()
        except:
            self.conn = None
        
        self.connect()
            #
            #
    def is_transaction_active(self):
        return self.conn.in_transaction
            #
            #
    def is_table_exist(self, db_table_name) -> bool:
        table = m_db.TableInfo(self, db_table_name)
        sql = f'''pragma table_info('{table.table_name}')'''
        return not self.sql_exec(sql).is_empty()

        
    def _load_table_model(self, db_table_name, is_assert=False) -> m_db.TableInfo:      # Возвращает список полей таблицы 

        type_translate = {
            'serial'  : 'int',
            'real'    : 'float',
            'int'     : 'int',
            'integer' : 'int',
            'text'    : 'str',
            'varchar' : 'str',
            'character varying' : 'str',
            'double precision' : 'float',
            'json'    : 'jsonb'
        }

        table = m_db.TableInfo(self, db_table_name)
        table.fields = {}
        table.primary_key = None

        sql = f'''pragma table_info('{table.table_name}')'''
        rows = self.sql_exec(sql)

        for rec in rows:
            field_name = rec.name
            type_str = rec.type.lower()

            table.fields[field_name] = type_translate.get(type_str, type_str)

            if 'pk' in rec and rec.pk and not table.primary_key: 
                table.primary_key = field_name

        if is_assert: assert table.fields, f'Не найдена таблица "{db_table_name}"'

        return table if table.fields else None
            #

    def returning_str(self, fields):
        return '; select last_insert_rowid();'

    def _pk_sql_type(self):
        return 'INTEGER PRIMARY KEY AUTOINCREMENT'

    def _date_encode(self, value):
        return f"date('{value}')"
        
        
    def _json_decode(self, value):     
        if not value: return {}
        return json.loads(value)
            #
            #
    def _json_encode(self, value):
        return super()._json_encode(value)


    '''
    def modify_table(self, table_name, table_model, recalc_func=None, check_only=False):
        if not super().modify_table(table_name, table_model): return

        id_list = self.load_id_list(table_name)
        self.drop_table(table_name)
        self.create_table(table_name, table_model)

        if recalc_func:
            for id in id_list:
                recalc_func(id)
    '''
    def sql_alter_column(self, table_name, field_name, type_name):
        sql = self.sql_drop_column(table_name, field_name)
        self.sql_exec(sql, res_type=None)

        return self.sql_add_column(table_name, field_name, type_name)


    def insert(self, table:m_db.TableInfo, data, return_id=False, **kwargs):
        super().insert(table, data, return_id=False, **kwargs)

        if return_id:
            rs = self.sql_exec(f'select last_insert_rowid() as {table.primary_key[0]}')
            if rs and not rs.is_empty():
                return rs[0]
        
        return None

    def db_path(self):
        rs = self.sql_exec('pragma database_list', res_type='')
        return rs[1][0][2]
            