'''
Модуль работы с меню приложения
'''
APP_MENU_URL = '/app-menu'

import m_api

api = None

class MenuNode:
    def __init__(self, caption, url):
        self.caption = caption
        self.url = url
        self.component = None
        self.props = None
        self.closed = None


class ApiMenu(m_api.ModuleApi):
    def __init__(self):
        super().__init__()

        self._menu = []


    def index(self, caption:str, root:list=None):
        root = self._menu if root==None else root

        i = 0
        while i<len(root):
            if root[i].get('caption') == caption:
                return i
            i += 1
        return None


    def add_menu(self, menu:dict, before:str=None, after:str=None, root:list=None):
        if type(menu) not in (list, dict):
            raise Exception(f'Блок меню "{menu}" должен иметь тип dict или list of dict')

        # Проверка на добавление существование блока с таким же заголовком
        for item in self._menu:
            if item.get('caption') == menu['caption']:
                return

        root = self._menu if root==None else root

        pos = None
        if before:  pos=self.index(before, root=root)
        elif after: 
            pos=self.index(after, root=root)
            if pos!=None: pos += 1
        if pos == None: pos = len(root)

        if type(menu) != list: menu = [menu]
        for item in menu:
            root.insert(pos, item)
            pos += 1


    def del_menu(self, menu_capt:str, root:list=None):
        root = self._menu if root==None else root

        idx = self.index(menu_capt, root=root)
        if idx != None:
            del root[idx]


    def add_item(self, menu_capt:str, menu_item:dict, before:str=None, after:str=None):
        # Поиск меню для добавления
        idx = self.index(menu_capt)

        if idx != None:
            menu = self._menu[idx]
        else:
            self._menu.append({'caption':menu_capt})
            menu = self._menu[-1]

        if 'items' not in menu: menu['items'] = []
        self.add_menu(menu_item, before=before, after=after, root=menu['items'])


    def del_item(self, menu_capt:str, item_capt:str):
        idx = self.index(menu_capt)
        if idx != None:
            self.del_menu(item_capt, root=self._menu[idx]['items'])


    def to_dict(self):
        return self._menu


def rest_app_menu(api, context, **kwargs):
    menu = api.app_menu._data.to_dict()
    return menu


def init(_api):
    global api
    api = _api

    api_menu = ApiMenu()
    node = api.add('app_menu', api_menu)
    api_menu.node = node

    api.add_url(APP_MENU_URL, rest_app_menu)
