import json
import requests

import m_utils
import m_rec

REMOTE_API_URL = '/remote'

api = None
rpc = None

def init(_api):
    global api
    api = _api

    rpc = Server()

    api.add_url(REMOTE_API_URL, rpc.endpoint)


def simple(data):
    if type(data) == list:
        for i in range(len(data)):
            data[i] = simple(data[i])

    elif type(data) == tuple:
        return simple(list(data))

    elif type(data) == dict:
        for key in data.keys():
            data[key] = simple(data[key])

    elif type(data) not in (int, float, str, bool):
        data = str(data)

    return data
    

class Server:
    def endpoint(self, context, **kwargs):
        request = context['request']
        args = request.args.to_dict()
        call_str = args.get('call')

        if 'db_connect' not in api.context:
            api.build_context()

        return self.exec(call_str)


    def exec(self, args_str):
        args_data = json.loads(args_str)
        path = args_data['path']
        args = args_data.get('args')
        kwargs = args_data.get('kwargs')
        
        try:
            if args==None and kwargs==None:
                # Извлекаем значений атрибута
                print(f'eval value: {path}')
                res = eval(path)
                if type(res) not in (int, float, str, bool, list, dict, tuple):
                    res = f'{path} {res.__class__.__name__}'
            else:
                print(f'eval func: {path}')
                res = eval(path)(*(args or []), **(kwargs or {}))
        except Exception as e:
            res = e
            
        return self.serialize(res)


    def serialize(self, data):
        if isinstance(data, Exception):
            txt = m_utils.exception_text(data)
            return f'Exception: {txt}'

        if hasattr(data, '__class__'):
            type_str = f'{data.__class__.__module__}.{data.__class__.__name__}'
        else:
            type_str = f'builtins.{type(data)}'

        if type_str == 'm_rec.Record':
            data = data.to_dict()
        elif type_str == 'm_rec.RecordSet':
            data = data.to_dict()
        elif not type_str.startswith('builtins.'):
            raise Exception(f'Неизвестный тип результата {type_str}')
            
        try:
            data = simple(data)
            return type_str + ': ' + json.dumps(data, ensure_ascii=False)
        except Exception as e:
            return f'Exception: Ошибка сериализации объекта {e}\nОбъект{type(data)} : {data}'


class RemoteNode:
    def __init__(self, parent=None, name='api', server=None, host=None, debug=False):
        self._parent = parent
        self._name = name
        self._debug = debug

        self._host = host
        if host:
            self._server = None
        else:
            self._server = server


    def request(self, call_str):
        url = self._host + '/api' + REMOTE_API_URL + '?call=' + call_str
        
        if self._debug:
            print(f'RemoteNode.request: url={url}')

        return requests.get(url).json()


    def __getattr__(self, name):
        if name == 'value':
            return self._remote_call(self._path())
        elif name == 'tree':
            return self._remote_call(self._path()+'.tree_str', [])[-1]
        else:
            return RemoteNode(self, name, server=self._server, host=self._host)
    
    def _path(self):
        names = []
        node = self
        while node:
            names.append(node._name)
            node = node._parent
        names.reverse()
        return '.'.join(names)
    
    def __repr__(self):
        return f'RemoteNode: {self._path()}'
    
    def __call__(self, *args, **kwargs):
        if args==None: args=[]
        return self._remote_call(self._path(), args, kwargs)
    
    def _remote_call(self, path, args=None, kwargs=None):
        call_data = {}
        call_data['path'] = path
        if args!=None: call_data['args'] = args
        if kwargs!=None: call_data['kwargs'] = kwargs
        
        call_str = json.dumps(call_data, ensure_ascii=False)
        
        if self._server:
            call_res = self._server.exec(call_str)
        elif self._host:
            call_res = self.request(call_str)
        else:
            raise Exception('Не указан сервер или хост')
        
        if call_res and call_res[0] == '"':
            call_res = call_res[1:-1]

        if call_res.startswith('Exception:'):
            raise Exception(f'Remote path: {path} - {call_res}\nfor {call_str}')

        return self.parse_value(call_res)
    
    
    def parse_value(self, value_str):
        words=value_str.split(': ', 1)

        type_str = words[0]
        
        translate_funcs = {
            'builtins.int': lambda s: int(s),
            'builtins.str': lambda s: s[1:-1],
            'builtins.dict': lambda s: json.loads(s),
            'builtins.tuple': lambda s: tuple(json.loads(s)),
            'm_rec.Record': lambda s: m_rec.Record( json.loads(s) ),
            'm_rec.RecordSet': lambda s: m_rec.RecordSet( json.loads(s) )
        }
        
        if type_str in translate_funcs:
            return translate_funcs[type_str](words[1])
        else:
            return f'RemoteNode: Неизвестный тип {value_str}'
