import traceback as tb
import os

def relative_path(this_file, path): 
    root_path = this_file if os.path.isdir(this_file) else os.path.split(this_file)[0]
    return os.path.join(root_path, path)

#=============================================================================================
#   Работа с датой и временем
#=============================================================================================
local_date_format = '%Y-%m-%d'
local_time_format = '%H:%M:%S'
local_datetime_format = '%Y-%m-%d %H:%M:%S'

import time
import datetime


def date_str(ts=None):                                      # Строка времени ГГГГ-ММ-ДД, по-умолчанию текущее время
    if ts==None: ts=time.time() 
    if type(ts) == str: return ts

    if type(ts) in (int, float):
        dt = datetime.datetime.fromtimestamp(ts)
    else: 
        dt = ts
    return dt.strftime('%Y-%m-%d')
        #
        #
def time_str(ts=None):                                      # Строка времени ЧЧ:ММ:СС, по-умолчанию текущее время
    if ts==None: ts=time.time() 

    if isinstance(ts, datetime.time):
        dt = ts
    elif type(ts) in (int, float):
        dt = datetime.datetime.fromtimestamp(ts)
    else:
        return f'<ошибка> {ts}'

    return dt.strftime('%H:%M:%S')
        #
        #
def datetime_str(ts=None):                                  # Строка времени ГГГГ-ММ-ДД ЧЧ:ММ:СС, по-умолчанию текущее время
    if ts==None: ts=time.time() 
    if type(ts) == str: return ts

    if type(ts) in (int, float):
        dt = datetime.datetime.fromtimestamp(ts)
    else:
        dt = ts

    return dt.strftime('%Y-%m-%d %H:%M:%S')
        #
        #
def str2date(date_str, default=''):                        # Преобразование строки в дату
    try:
        if type(date_str) == datetime.date:
            return date_str
        elif type(date_str) == datetime.datetime:
            return date_str.date()
        else:
            date_str = date_str.split(' ')[0]
            if '-' in date_str:
                return datetime.date.fromisoformat(date_str)   # Формат yyyy-mm-dd
            else:
                return datetime.datetime.strptime(date_str, '%d.%m.%Y').date()
    except:
        if type(default) == str:
            return str2date(default, None)
        else:
            return default
        #
        #
def str2datetime(datetime_str, default=None, isAssert=True):                        # Преобразование строки в дату
    try:
        if type(datetime_str) == datetime.date:
            return datetime_str
        elif type(datetime_str) == datetime.datetime:
            return datetime_str.date()
        else:
            if '-' in datetime_str:
                if ':' in datetime_str:
                    return datetime.datetime.strptime(datetime_str, '%Y-%m-%d %H:%M:%S')
                else:
                    return datetime.datetime.strptime(datetime_str, '%Y-%m-%d')
                
            elif '/' in datetime_str:
                if ':' in datetime_str:
                    return datetime.datetime.strptime(datetime_str, '%d/%m/%y %H:%M:%S')
                else:
                    return datetime.datetime.strptime(datetime_str, '%d/%m/%y')
                
            else:
                if ':' in datetime_str:
                    return datetime.datetime.strptime(datetime_str, '%d.%m.%y %H:%M:%S')
                else:
                    return datetime.datetime.strptime(datetime_str, '%d.%m.%y')
            
    except:
        if default or not isAssert: return default
        raise 
        #
        #
def str2time(time_str:str, default=None, isAssert=True):
    try:
        if time_str.count(':') == 1:
            return datetime.datetime.strptime(time_str, '%H:%M').time()
        else:
            return datetime.datetime.strptime(time_str, '%H:%M:%S').time()
    except:
        if default or not isAssert: return default
        raise 
        #
        #
def date_str_format(date_str, format='%d.%m.%Y', default=''):
    d = str2date(date_str, None)
    if d:
        return d.strftime(format)
    else:
        return default
        #
        #

def is_datetime(dt):
    return isinstance(dt, (datetime.date, datetime.time, datetime.datetime))


def datetime_local_str(dt):
    if dt == None:
        return None

    elif isinstance(dt, datetime.datetime):
        return dt.strftime(local_datetime_format)

    elif isinstance(dt, datetime.date):
        return dt.strftime(local_date_format)

    elif isinstance(dt, datetime.time):
        return dt.strftime(local_time_format)



# Строки времени для логов
def time_long_str(ts=None):
    if ts==None: ts=time.time() 
    dt = datetime.datetime.fromtimestamp(ts)
    return dt.strftime('%H:%M:%S')+('%.3f'%(ts%1))[1:]

#=============================================================================================
#   Преобразование типов
#=============================================================================================

def to_type(value, req_type, default=None):
    '''
    Преобразовывае value к типу данных data_type
    '''
    if value == None: return None
    if req_type == None: return value

    value_type = type(value)
    if value_type == req_type: return value

    def value_type_exception():
        raise TypeError(f'Неизвестное преобразование значения "{value}": {value_type} в {req_type}')

    if req_type==int:
        if value_type==str: value = int(value) if value.strip() else None
        elif value_type==datetime.datetime: value = round(value.timestamp())
        else: value_type_exception()

    elif req_type==float:
        if value_type==str: value = float(value) if value.strip() else None
        elif value_type==int: value = float(value)
        elif value_type==datetime.datetime: value = value.timestamp()
        else: value_type_exception()

    elif req_type==str:
        if value_type==datetime.date: value = date_str(value)
        elif value_type==datetime.datetime: value = datetime_str(value)
        elif value_type==datetime.time: value = time_str(value)
        elif value_type!=str: value = str(value)
        else: value_type_exception()

    elif req_type==datetime.date:
        if value_type==str: value = str2date(value)
        elif value_type==datetime.datetime: value = value.date()
        else: value_type_exception()

    elif req_type == datetime.datetime:
        if value_type==str: value = str2datetime(value)
        elif value_type==int: value = datetime.datetime.fromtimestamp(value)
        elif value_type==float: value = datetime.datetime.fromtimestamp(value)
        else: value_type_exception()

    elif req_type == datetime.time:
        if value_type==str: value = str2time(value)
        else: value_type_exception()

    else:
        raise Exception(f'Неизвестный тип данных {req_type}')
    
    return value


def str_compress(txt:str) -> str:
    '''
    Сжимает строку (заменяет \\n на пробелы, сдвоенные пробелы на одинарные)
    '''
    txt = txt.strip()
    txt = txt.replace('\n', ' ')
    while '  ' in txt:
        txt = txt.replace('  ', ' ')

    return txt


def simple_dict(a):
    if not a: return a
    r = {}
    for k,v in a.items():
        if type(v) not in (list, dict):
            r[k] = v

    return r


def simplify(data):                 # Преобразование данных в примитивные типы
    if hasattr(data, 'strftime'):
        data = data.strftime('%Y-%m-%d')
    
    if hasattr(data, 'to_dict'):
        data = data.to_dict()


    if data == None:
        res = None
        
    elif type(data) in (int, float, str, bool):
        res = data

    elif type(data) == dict:
        res = { k : simplify(v) for k,v in data.items() }

    elif type(data) == list:
        res = [simplify(i) for i in data]

    else:
        res = f'{data}'

    return res


def stringify(v):                   # Преобразование в строку, альтернатива json.dumps
    if type(v) in (list, tuple):
        res = f', '.join( [stringify(i) for i in v] )
        return f'[{res}]'
     
    if type(v) == dict:
        lst = [f'"{k}" : {stringify(v[k])}' for k in v.keys()]
        lst = f', '.join(lst)
        return '{' + lst + '}'
    
    if type(v) in (int, float):
        return f'{v}'
    
    return f"'{v}'"


def to_html(data):                  # Преобразование для отображения HTML
    data = data.replace('<', '&lsaquo;')
    data = data.replace('>','&rsaquo;')
    data = data.replace('"','&quot;')
    data = data.replace("'",'&prime;')
    data = data.replace('\\n', '\n')
    data = data.replace('\n', f'<BR>')

    return data


def to_html_code(data):
    style = 'margin:0;'
    data = to_html(data)
    data = data.replace('<BR>', f'</PRE><PRE style="{style}">')
    res = f'<PRE style="{style}">' + data + '</PRE>'
    return res



class Record:
    def __init__(self, hide:list=None, label=''):
        self._hide = hide if hide!=None else []
        self._label = label

    def __repr__(self):
        if self._label:
            res = self._label
        else:
            res = 'Record: '
            for key in self.__dict__:
                if key[0] != '_' and key not in self._hide:
                    value = self.__dict__[key]
                    if type(value)==str and len(value)>15:
                        value = value[:15] + '...'
                    if self._label:
                        res += self._label
                    res += f'{key}={value}  '
        return res


def exception_text(e, CR='\n', exclude=[]):
    res = f'{e}{CR}{type(e)}{CR}'
    for arg in e.args:
        res +=f'{arg}{CR}'
    
    
    for line in tb.format_tb(e.__traceback__):
        for p in exclude:
            if p in line: break

        else:
            res += CR + line

    return res


class CallerInfo:
    '''Возвращает информацию о вызвающей функции
        levels_up - кол-во уровней вверх от вызвавшей функции, по-умолчанию = 1
    '''
    def __init__(self, levels_up=1, deep=10):
        self.stack = []
        tb_info = tb.extract_stack(limit=levels_up+2+deep)
        tb_info.reverse()
        for caller in tb_info[levels_up+1:]:
            self.stack.append(self.parse_traceback(caller))


    def parse_traceback(self, tb_info):
        record = Record(hide=['file_name','file_path'])
        record.file_name = os.path.basename(tb_info.filename)
        record.file_path = os.path.dirname(tb_info.filename)
        record.module_name = record.file_name.split('.')[0]
        record.line_no = tb_info.lineno
        #record.line = tb_info.line
        record.func_name = tb_info.name
        record._label = f'{record.module_name}.{record.func_name}'
        return record
            
    def __repr__(self):
        caller = self.stack[0]
        return f'{caller.module_name}.{caller.func_name}'

    def outer_func(self, levels=3, exclude_modules=[], exclude_starts=None):
        module = self.stack[0].module_name
        idx = 0
        for idx in range(1, len(self.stack)):
            caller = self.stack[idx]
            if caller.module_name == module: continue
            if caller.module_name in exclude_modules: continue
            if exclude_starts and caller.module_name.startswith(exclude_starts): continue
            break  
        
        res = []
        while idx < len(self.stack) and levels:
            caller = self.stack[idx]
            res.append( f'{caller.module_name}.{caller.func_name}' )
            idx += 1

        return res         


def deep_update(a, b, base_only=False, update_to_none=True, update_to_blank=True):
    for k,v in b.items():
        if not k in a or a[k]==None and type(v)==dict: a[k]={}
            
        if type(v) == dict and not base_only:
            deep_update(a[k], v, base_only=base_only, update_to_none=update_to_none)
        else:
            if v == None and not update_to_none: continue
            if not v and not update_to_blank: continue
            
            a[k] = v


def is_match(text, pattern, rule='.*'):
    pattern = '^'+pattern.replace('*', rule)+'$'
    import re
    return bool(re.findall(pattern, text))



def str2float(float_str, default=0, is_assert=False):                                        # Преобразование строки в число
    if type(float_str) in (int, float): return float_str
    
    try:
        float_str = float_str.strip()
        if float_str[0]=='(' and float_str[-1]==')':
            float_str = float_str[1:-1]

        float_str = float_str.strip().replace(',','.').replace('e','E').replace('е','E').replace('Е','E')
        if '+' in float_str[1:] and 'E' not in float_str:
            pos = float_str[1:].index('+')+1
            float_str = float_str[:pos] + 'E' + float_str[pos:]
            
        if '-' in float_str[1:] and 'E' not in float_str:
            pos = float_str[1:].index('-')+1
            float_str = float_str[:pos] + 'E' + float_str[pos:]

        res = float(float_str)
        if str(res) == 'nan': res = None

        return res
    except:
        if is_assert: raise
        return default
        #


def str2hex(data_str, prefix='0x'):                     # Преобразует строку в строку байтов в шестнадцатеричном виде 
    if data_str==None: return None
    if type(data_str) != str: return str(data_str)

    res = [f'{prefix}{ord(ch):02x}' for ch in data_str]
    return ' '.join(res)


def is_chars(s: str):
    for ch in s:
        if (ord(ch)<ord(' ') or ord(ch)>127) and ch not in '\r\n':
            return False
    return True


# Перобразует словарь или массив в формат удобный для чтения
def pretty(data, prefix='', float_prec=4, hex=0, array_index=True, CR='\n', indent=8, max_len=80, hide=[], attrs=[], simple=False):

    def fit_str(line):
        return line if len(line) < max_len else line[:max_len] + '...'

    def pretty_value(value, hex=1):
        if type(value) == float:
            res = f'{round(value, float_prec)}'
        elif type(value) == int:
            if   hex==0: res = "%r"%value
            elif hex==1: res = f'{"%r"%value} ({value:02X}h)'
            elif hex==2: res = f'{value:02X}h'
        elif type(value) == str:
            if   hex==0: res = f'{"%r"%value}'
            elif hex==1: 
                if is_chars(value):
                    res = f'{"%r"%value}'
                else:    
                    res = f'{"%r"%value} (HEX: {str2hex(value, prefix="")})'
            elif hex==2: res = f'HEX: {str2hex(value, prefix="")}'
        else:
            try:
                res = f'{"%r"%value}'
            except:
                res = f'{value}'

        return res


    def pretty_dict(data, prefix='', path=[]):

        def key_string(key):
            row = (prefix + ' '*4) if len(CR) else ''
            row += pretty_value(key, hex=0) 
            return row


        #key_len = max([len(str(key)) for key in data.keys()] or [0])
        key_len = max([len(key_string(key)) for key in data.keys()] or [0])

        res = '{'+CR
        data_list = list(data.items())
        for key, value in data_list:

            key_str = key_string(key)

            if attrs and key not in attrs: continue         # Фильтрация по именам атрибутов

            row = key_str
            row += ' '*(key_len-len(key_str)) if len(CR) else ''
            row += ' : ' 

            key_path = path + [str(key)]
            if '.'.join(key_path) in hide: 
                res += row
                res += '<hidden>' + CR
            elif type(value) == dict : 
                if simple: continue
                res += row
                res += pretty_dict(value, prefix + ' '*indent, path=key_path)
            elif type(value) == list :
                if simple: continue
                res += row
                res += pretty_list(value, prefix + ' '*indent)
            else:
                row += pretty_value(value, hex=hex) + ','
                res += fit_str(row) + CR
        
        res += prefix if CR else ''
        res += '},'+CR
        return res


    def pretty_list(data, prefix=''):
        key_len = len(str(len(data)))

        res = '['+CR
        for i in range(len(data)):
            value = data[i]

            row = (prefix + " "*4) if len(CR) else ''
            if array_index:
                row += f'{" "*(key_len-len(str(i)))}' if len(CR) else '  '
                row +=f'{i} : '     
            
            if type(value) == dict: 
                res += row
                res += pretty_dict(value, prefix + ' '*indent)
            elif type(value) == list:
                res += row
                res += pretty_list(value, prefix + ' '*indent)
            else:
                row += pretty_value(value, hex=hex) + ', '
                res += fit_str(row) + CR

        res += prefix if CR else ''
        res += '],'+CR
        return res


    if type(data) == dict: 
        res = pretty_dict(data)
    elif type(data) == list:
        res = pretty_list(data)
    else:
        res = pretty_value(data, hex=hex)

    return res


def print_compact_dict(data, indent=''):
    lens = [len(key) for key,value in data.items() if value!=None]
    if lens: 
        max_key_len = max(lens)
    else:
        max_key_len = 0
    empty = []

    for key, value in data.items():
        if value != None and value != '':
            if type(value) == str:
                value_str = f"'{value}'"
            else:
                value_str = f'{value}'
            print(f'{indent}{key}{" "*(max_key_len-len(key))} : {value_str}')
        else:
            empty.append(key)

    if empty:
        print(f'{indent}empty: {empty}')



import importlib.util


extract_file_name = lambda file : os.path.split(file)[-1].split('.')[0]

def load_py_file(file_name):
    module_name = extract_file_name(file_name)
    spec = importlib.util.spec_from_file_location(module_name, file_name)
    m = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(m)
    return m


def load_py_settings(file_name, load_func=False):
    '''
    Загружает файл с настройками вида <variable>=<value>
    Возвращает DICT
    '''
    try:
        module = load_py_file(file_name)
        config = {}
        for k,v in module.__dict__.items():
            if k.startswith('__'): continue                 # Исключаем системные переменные
            if type(v) == type(module): continue            # Исключаем импортированные модули
            if callable(v) and not load_func: continue      # Исключаем фукнкции и классы
            config[k] = v

        #print(f'Настройки {os.path.abspath(file_name)}\n{config}')
        return config

    except Exception as e:
        print(f'Ошибка при загрузке файла {os.path.abspath(file_name)}.\n{e}')
        return {}


#=============================================================================================
#   Работа с атрибутами класса
#=============================================================================================

def get_class_attrs(this_class, attr_type, *, exclude=None, **kwargs) -> dict:
    res = {}

    if this_class == object:
        return res

    if this_class == exclude: return res

    for k, v in this_class.__dict__.items():
        if k.startswith('__'): continue
        if callable(v): continue
        if exclude and isinstance(v, exclude): continue 
        if attr_type and not isinstance(v, attr_type): continue

        res[k] = v

    try:
        kwargs['_fake_init'] = True
        item = this_class(**kwargs)     # Пытаемся инициализировать класс с переданными параметрами
    except TypeError as e:
        try:
            item = this_class(_fake_init=True)
        except TypeError as e:
            raise Exception(f'Класс не должен иметь конструкторов с параметрами. Ошибка для класса "{this_class.__name__}": {e}')

    if hasattr(item, '__annotations__'):
        for k, v in item.__annotations__.items():
            res[k] = (res.get(k), v)

    return res


def get_base_class_attrs(this_class, attr_type, *, exclude=None, **kwargs) -> dict:
    res = {}
    if not hasattr(this_class, '__bases__'): return res

    for cls in this_class.__bases__:
        res.update(get_base_class_attrs(cls, attr_type, exclude=exclude, **kwargs))
        res.update(get_class_attrs(cls, attr_type, exclude=exclude, **kwargs))
        
    return res


def get_all_class_attrs(cls, attr_type=None, *, exclude=None, **kwargs) -> dict:
    '''
    Загрузка описаний полей из атрибутов описания класса
        - attr_type - выбирать только атрибуты заданного типа
        - exclude - исключать атрибуты заданного типа
    '''
    
    attrs_map = get_base_class_attrs(cls, attr_type, exclude=exclude, **kwargs)
    attrs_map.update( get_class_attrs(cls, attr_type, exclude=exclude, **kwargs) )

    return attrs_map


def get_class_attrs_list(cls, attr_type=None) -> list:
    res = []
    this_attrs = get_class_attrs(cls, attr_type)
    for attr_name, attr_desc in this_attrs.items():
        res.append( (attr_name, attr_desc) )

    return res


def get_base_class_attrs_list(cls, attr_type=None) -> list:
    res = []
    if not hasattr(cls, '__bases__'): return res

    for cls in cls.__bases__:
        res.extend(get_base_class_attrs_list(cls, attr_type))
        res.extend(get_class_attrs_list(cls, attr_type))
        
    return res


def get_all_class_attrs_list(cls, attr_type=None) -> list:
    attrs_list = get_base_class_attrs_list(cls, attr_type)
    attrs_list.extend(get_class_attrs_list(cls, attr_type))

    return attrs_list
