api = None

def init(_api, **kwargs):
    global api
    api = _api

    api.add_module_api('web', ApiWeb(**kwargs))


import os
from flask import Flask, request, send_from_directory
import flask
import json

import m_api

routes = {}

def endpoint():
    if 'build_context' in api:
        api.build_context()

    url = request.base_url.split('//')[-1]
    path = '/' + url.split('/', 1)[-1]

    context = {'request': request, 'user':'test'}

    print(f'server request {request}')

    api.context['request'] = request
    api.context['user'] = 'test'

    func = routes[path]
    reply = func(api=api, context=context)
        
    if type(reply) in (list,dict):
        #reply = json.dumps(reply, ensure_ascii=False)
        reply = flask.make_response(flask.jsonify(reply))

    return reply


class ApiWeb(m_api.ModuleApi):
    def __init__(self, url_prefix='/api', frontend_dir='../frontend/editor-forms/public', port=5000):
        super().__init__()

        api2_path = api.get_config('API2_PATH', '.')
        api2_path = os.path.abspath(api2_path)

        kwargs = {}
        static_url = api.get_config('API2_WEB_STATIC_URL', '/static')
        static_folder = api.get_config('API2_WEB_STATIC_FOLDER', os.path.join(api2_path, 'static'))

        if static_url != None: kwargs['static_url_path'] = static_url
        if static_folder != None: kwargs['static_folder'] = static_folder

        print(f'm_web: {kwargs}')

        self.url_prefix = url_prefix
        self.frontend_dir = os.path.abspath(frontend_dir)
        self.port = port
        self.flask_app = Flask(__name__, **kwargs)
        self.flask_app.add_url_rule('/', view_func=self.ep_frontend, methods=['GET','POS'])
        self.flask_app.add_url_rule('/<path:path>', view_func=self.ep_frontend, methods=['GET','POS'])
        #self.flask_app = Flask(__name__, static_folder=static_folder, static_url_path=static_url)
        self.static_path = ''


    def add_url(self, url, func):
        if url[0] != '/': url = '/' + url
        url = self.url_prefix + url

        routes[url] = func
        self.flask_app.add_url_rule(url, view_func=endpoint, methods=['GET', 'POST'])

        #api.web.add(url, func)


    def add_static(self, path):
        self.static_path = path
        self.flask_app.add_url_rule('/static/<path:path>', self.ep_static, methods=['GET'] )


    def ep_static(self, path):
        print(f'serve static: {self.static_path} {path}')
        return send_from_directory(self.static_path, path)    


    def ep_frontend(self, path=None):
        if not path: path='index.html'

        if not os.path.isfile(os.path.join(self.frontend_dir, path)):
            path = 'index.html'

        print(f'requested frontend: {self.frontend_dir} {path}')
        return send_from_directory(self.frontend_dir, path)    


    def reg_actions_generator(self, url, func):
        self.add_url(url, func)

    
    def start(self):
        self.flask_app.run(host='0.0.0.0', port=self.port)

