'''
REST API к источнику данных типа TableDataSource 
'''
WEB_DS_URL_PATH = '/ds/'

api = None

def init(_api):
    global api
    api = _api


import m_tree, m_ds

class WebDs:
    def __init__(self, model_node:m_tree.Node, ds: m_ds.TableDataSource):
        
        ds_name = model_node.path().replace('.', '__')
        self._name = ds_name
        self._model_node = model_node

        if isinstance(ds, m_tree.Node): ds = ds._data
        if not isinstance(ds, m_ds.TableDataSource):
            raise Exception(f'Источник данных должен быть потомком m_ds.TableDataSource. type(ds)={type(ds)} ds={ds}')
        self._ds = ds

        api.web.add_url(WEB_DS_URL_PATH + ds_name, self.ep_rest)


    def get_url(self):
        return f'/api{WEB_DS_URL_PATH}{self._name}' 


    def get_pk_value(self, args):
        pk_attr = self._ds.table.primary_key
        selected_row = args['selected_row']

        if len(pk_attr) == 1:
            id = selected_row.get(pk_attr[0])
        else:
            raise Exception(f'Сложный первичный ключ не поддерживается. pk_attr={pk_attr}')

        return id


    def calcFront(self, rows):                      # Обработка вычисляемых полей
        attrs_model = self._model_node._parent.gather_attrs() or []

        for attr, model in attrs_model._attrs.items():
            calc_func = model._options.get('calcFront')
            if calc_func: 
                for row in rows:
                    row[attr] = calc_func(row)


    def ep_rest(self, api, context, **kwargs):
        args = api.lib.rest.get_args(context)

        action = args['action']
        uuid = args.get('__uuid')

        #raise Exception(f'Request {context["request"]}\nform:{context["request"].form.to_dict()}\njson: {context["request"].get_json()}')

        if action == 'select':
            rows = self._ds.select().to_dict()
            self.calcFront(rows)
            res = {'params':{'offset':0}, 'result':rows, 'values_remain':False}

        elif action == 'read':
            id = self.get_pk_value(args)
            res = self._ds.read(id)
            self.calcFront([res])
            res = res.to_dict()

        elif action == 'create':
            selected_row = args['selected_row']
            res = self._ds.insert(selected_row)
            self.calcFront([res])
            res = res.to_dict()
            api.context.db_connect.commit()

        elif action == 'save':
            selected_row = args['selected_row']
            id = self.get_pk_value(args)

            if id:
                self._ds.update(selected_row)
                res = self._ds.read(id)
            else:
                res = self._ds.insert(selected_row)

            self.calcFront([res])
            res = res.to_dict()
            api.context.db_connect.commit()

        elif action == 'change_cells':
            selected_row = args['selected_row']
            self._ds.update(selected_row)
            api.context.db_connect.commit()
            res = {}

        elif action == 'delete':
            id = self.get_pk_value(args)
            
            if id:
                res = self._ds.delete(id)
                #api.log.msg(f'm_web_ds: Удалена запись из таблицы {self._ds.table} id={id}')
                api.context.db_connect.commit()

            else:
                api.log.error(f'm_web_ds: Не определен первичный ключ {self._ds.table.primary_key} в параметрах {args}')

            res = {}

        else:
            raise Exception(f'Не корректное действие "{action}"')

        #api.log.msg(f'm_web_ds: rest call action={action}  selected_row={args.get("selected_row")}\nres={res}')
        if uuid:
            res['__uuid'] = uuid

        return res
