SYSTEM_CONNECT_PATH = 'system.system_db_connect'

import m_utils
import m_api
import m_db_pg
import m_db_sqlite

api = None

class SgukApi(m_api.Api):
    def __init__(self, config={}) -> None:
        super().__init__(config=config)

        self.loaded=False
            #
            #
    def create_db_connect(self, core_db_connect=None, db_name:str=None):

        db_type = api.get_config('DB_SYSTEM', 'postgresql')
        if db_type == 'postgresql':
            db = m_db_pg.DbPgConnect(core_db_connect=core_db_connect)
            if not db_name:
                if api.find('system.system_db_connect'):
                    db_name = self.node.system.system_db_connect.DB_NAME
                else:
                    db_name = api.get_config('POSTGRES_DATABASE')

            db.connect(db_name=db_name)
            # print(f'Connected to {db_name}')

        elif db_type == 'sqlite':
            db = None
            if core_db_connect:
                db = m_db_sqlite.DbSQLiteConnect(core_db_connect=core_db_connect)
                api.log.msg(f'create_db_connect: used core_db_connect={core_db_connect}')

            elif api.find('system.sqlite_connect'):
                sqlite_connect = api.system.sqlite_connect()
                if sqlite_connect:
                    db = m_db_sqlite.DbSQLiteConnect(core_db_connect=sqlite_connect)
                    db_path = db.db_path()
                    msg = f'create_db_connect: api.system.sqlite_connect() -> {sqlite_connect} : {type(sqlite_connect)} : {db} : {db_path}'
                    if api.loaded:
                        api.log.msg(msg)
                    else:
                        print(msg)

            if not db:
                db = m_db_sqlite.DbSQLiteConnect(db_file=api.get_config('SQLITE_DATABASE'))
                if api.get_config('API2_DEBUG', False):
                    api.log.msg(f'create_db_connect: created new DB connect {db}')

        else:
            raise Exception(f'Неизвестный тип БД DB_SYSTEM={db_type} доступны: "postgresql", "sqlite"')

        if not db:
            raise Exception('Ошибка создания подключения к БД')

        return db
            #
            #
    def build_context(self, core_db_connect=None):
        if not 'db_connect' in self.node.context:
            db_conn = self.create_db_connect(core_db_connect=core_db_connect)
            self.node.context['db_connect'] = db_conn
                #
                #
    def clear_context(self):
        if 'db_connect' in api.context:
            db_connect = api.context.db_connect
            if db_connect.conn_self_created:
                db_connect.commit()
                db_connect.disconnect()

        api.context.clear()
        #api.log.msg('clear_context: cleared')


def init_core(config={}):               # Инициализация модулей до модулей Core (Игнат)
    #print('sguk_api: init_core starting')

    global api
    api = SgukApi(config=config).node
    
    api.add_module('m_triggers')

    #print('sguk_api: init_core done')

    return api


def init(_api):                         # Инициализация после модулей Core
    global api
    api = _api

    if 'config' in api:
        print(f'{"-"*40}\nConfig: {m_utils.pretty( api.config._data)}\n{"-"*40}')

    # === Загрузка модулей ядра ==============================================
    api.add_module('m_repo')
    api.add_module('m_db')              # Интерфейс к БД
    api.add_module('m_db_pg')           # Интерфейс к БД PostgreSQL
    api.add_module('m_db_sqlite')       # Интерфейс к БД SQLite
    api.add_module('m_ds')              # Источники данных
    api.add_module('m_rest')            # Функции для REST API
    api.add_module('m_actions')         # Действия пользователя
    api.add_module('m_files')           # Работа с файлами
    api.add_module('m_debug')           # Веб-дебаггер
    api.add_module('m_remote')          # Удаленный доступ к API
    api.add_module('m_pages')           # Подгружаемые страницы 
    api.add_module('m_menu')            # Меню приложения
    api.add_module('m_loader')          # Загрузчик расширений
    api.add_module('m_web_ds')          # REST API к источнкам данных

    # === Создание системного подключения к БД ===============================
    db = api.create_db_connect()
    api.add(SYSTEM_CONNECT_PATH, db)
    #api.build_context()

    # === Загрузка надстроек СГУК ============================================
    api.add_module('sguk_repo')         # Регистрация объектов репозитория
    api.add_module('sguk_core_repo')    # Интерфейс к репозиторию в CORE
    api.add_module('sguk_events')       # Обработчик событий Frontend
    api.add_module('sguk_pages')        # Подгружаемые страницы
    api.add_module('sguk_menu')         # Меню приложения

    api.add_module('sguk_forms')        # Хранилище отчетных форм
    api.add_module('sguk_vocs')         # Хранилище справочников
    api.add_module('sguk_regs')         # Хранилище аналитических регистров

    api.add_module('frontend.browse_edit')      # Модель страницы browse_edit
    
    # === Загрузка настроек СГУК =============================================
    api.add_module('sguk_setup_api')

    # === Окончание инициализации ============================================
    api.build_context()
    print('sguk_api: init done')
    api.loaded = True


def make_test_node(db_name=None, disabled=[], config={}):       # Создание API для интерактивного режима (Jupyter Nitebook)
    m_api.disabled = disabled
    if db_name and type(db_name)!=str:
        raise Exception(f'Параметр db_name должен быть типа str. db_name={db_name}')

    if not config and db_name:
        config = {'DB_SYSTEM':'postgresql', 'POSTGRES_DATABASE':db_name}

    if not config:
        print(''' 
            Необходимо передать конфигурацию через параметр config.

            Параметры конфигурации:
                API2_PATH: <path to API2, default ''>
                DB_SYSTEM: sqlite | postgresql
                POSTGRES_DATABASE: <database name>
                SQLITE_DATABASE: <path to DB file>
                
        ''')
        return

    if 'API2_PATH' not in config:
        config['API2_PATH'] = ''

    api = init_core(config=config)
    api.add_module("m_web")
    init(api)

    return api