'''
Модуль Аналитических отчетов

api
+-- repo
    +-- AReports : ApiAReports
        +-- ds : AReportDS                  # Источник данных для журнала аналитических отчетов 
        +-- <report_type 1> : AReport       # Зарегистрированный аналитический отчет, потомок AReport
        ...

ApiAReports:
    add_areport( <report_type>, AReport() )     # Добавляет отчет в репозиторий
    get_create_url( <report_type> )             # Возвращает URL для вызова на frontend

Регистрирует URL:
    /api/areports2 ? 
        action=get_params & areport_class= <report_class>   # Возвращает модель параметров для построения отчета
        action=build                                        # Строит отчет и сохраняет его в хранилище
        action=make & name=<report_class> & <params>        # Строит отчет и возвращает в виде HTML=таблицы

    действие build запускает построение отчета в отдельном потоке, аналогично Actions и возвращает proc_id
    статус построения и результат аналогичен Actions
    результат - URL для просмотра отчета

Использует Fronend:
    //analytical-report-create ? init_values={
            "a_report_type":"<report_type>", 
            "a_report_params_url" : "/api/areports?action=get_params & areport_class= "<report_class>"
        }
'''

AREPORT_TYPE = 'AReports'
NODE_TYPE = 'AReport'

AREPORT_TABLE = 'AReports'      # Имя таблица в БД для хранения аналитических отчетов
PK_FIELD_NAME = 'areport_id'    
AREPORT_URL = '/areports2'      # URL для доступа к REST API модуля
CLASS_FIELD_NAME = None


api = None

def init(_api):
    global api
    api = _api

    api.add_module_api('api.repo.AReports', ApiAReports())

    # Регистрация источника данных - Аналитические отчеты
    ds = AReportDS()
    api.repo.add_datasource('AReports.ds', ds)
    api.add_url(AREPORT_URL, rest_endpoint)

    # Регистрация тестового действия
    

import m_rec
import m_tree
import m_rest
import m_actions
import m_repo
import m_api

class AReport:
    def __init__(self):
        self.areport_class = 'abstract'
        self.areport_name = 'abstract'
        self.model = {}                                             # Модель данных отчета
        self.report_data = None
        #
        #
    def build(self, **kwargs):                                      # Построение отчета
        pass
        #
        #
    def build_and_save(self, areport_id=None, **kwargs):            # Выполняет отчет и сохраняет его хранилище
        self.report_data = self.build(**kwargs)
        context = kwargs['context']

        areport_id = self.save(context['params'], areport_id=areport_id)        # Сохранение отчета в хранилище api.repo.AReports

        return self.result_open_areport(areport_id)                 # Возвращаем команду открытия отчета для просмотра
        #
        #
    def decode(self, data, attrs:list):                             # Декодирование атрибутов в текстовое представление
        if isinstance(data, m_rec.RecordSet):
            rs = data
        else:
            rs = m_rec.RecordSet(data)

        for attr in attrs:
            # Поиск описания атрибута в модели данных отчета
            for cur_attr in self.model['attrs']:
                if cur_attr.get('field')==attr: 
                    field_type = cur_attr.get('attr_type')
                    break
            else:
                continue

            # Определение функции декодирования
            if type(field_type)==str:
                if field_type in self.model['types']:
                    type_obj =  self.model['types'][field_type]
                else:
                    node = api.repo.attr_types.get_attr(field_type)
                    if node:
                        type_obj = node._data
                    else:
                        type_obj = None
            else:
                type_obj = field_type

            api.log.error(f'decode: attr={attr} type_obj={type_obj}')

            if isinstance(type_obj, m_repo.AttrType):
                decode_func = lambda x : type_obj.decode(x)

            elif type(type_obj) == dict:
                decode_func = lambda x : type_obj.get(x, x)

            else:
                raise Exception(f'Не определена функция декодирования для атрибута {attr}')

            for rec in rs:
                if attr in rec:
                    rec[attr] = decode_func(rec[attr]) or f'??? {rec[attr]}'
                
        if type(data) == list:
            return rs.to_dict()
        else:
            return rs
                #
                #
    def areport_model(self):
        return {
                'areport_name'   : self.areport_name,
                'areport_type'   : self.areport_class,
                'url_get_params' : f'/api/areports2?action=get_params&areport_class={self.areport_class}',
                'url_build'      : f'/api/areports2?action=build&areport_class={self.areport_class}',
                'url_rebuild'    : f'/api/areports2?action=rebuild',
            }


    def frontend_params_model(self):                                # Модель параметров отчета для frontend
        # === Загрузка описаний типов и репозитория ===
        # Загружаем список используемых типов
        types =  [param.get('attr_type') or param.get('type') for param in self.model['params']] 

        # Заменяем ссылки на типы в дереве API на имена типов
        for i in range(len(types)):
            if isinstance(types[i], m_tree.Node):
                types[i] = types[i]._name

        types = list(set(types))

        types_model = {}
        for attr_type in types:
            if not attr_type: continue

            front_type = {}

            # Определяет описание типа из репозитория
            if isinstance(attr_type, m_tree.Node) and isinstance(attr_type._data, m_repo.AttrType):
                repo_type = attr_type
            else:
                repo_type = api.repo.attr_types.get_attr(attr_type)

            if repo_type: front_type = repo_type.frontend_model()

            # Обновляем описание типа из локального описания типов
            custom_desc = self.model.get('types',{}).get(attr_type, {})
            if hasattr(custom_desc, 'frontend_model'): 
                custom_desc = custom_desc.frontend_model()
            front_type.update(custom_desc)

            types_model[attr_type] = front_type

        # === Формирование результата для Frontend ===
        params = []
        for param in self.model['params']:
            if hasattr(param, 'frontend_model'):
                params.append(param.frontend_model())

            elif type(param) == dict:
                param_desc = dict(param)
                if 'attr_type' in param_desc:
                    param_desc['type'] = param_desc.pop('attr_type')
                params.append(param_desc)
                
            else:
                raise Exception(f'Некорректный тип описания параметра "{type(param)}" описание {param}')

        res = {
            'areport_params' : self.areport_model(),
            'form': {
                'table' : {
                    'model' : params
                }
            },
            'types': types_model,
        }

        return res
            #
            #
    def get_attr_models(self):                                      # Создаем список отображаемых атрибутов
        # Загружаем список атрибутов из модели
        report_attrs = self.model.get('attrs',['*'])

        field_list = [attr_model if type(attr_model)==str else attr_model['field'] for attr_model in report_attrs]

        field_models = {}
        for field_model in report_attrs:
            if type(field_model) == str: continue

            field_models[field_model['field']] = field_model 


        # Если есть атрибут "*", загружаем список атрибутов из данных отчета
        data_fields = self.report_data.attr_keys() if self.report_data else []
        if '*' in field_list:
            field_list.remove('*')

            if self.report_data:
                for attr in data_fields:
                    if attr not in field_list:
                        field_list.append(attr)


        # Удаление полей перечисленных в атрибуте attrs_drop
        # Удаление полей отсутствующих в данных
        i = 0
        while i<len(field_list):
            if field_list[i] in self.model.get('attrs_drop',[]) or \
               (self.model.get('attrs_drop_na', False) and field_list[i] not in data_fields):
                del field_list[i]
            else:
                i += 1

        
        # Загружаем атрибуты из используемых форм и справочников
        extra_models = {}

        for form_class in self.model.get('use_forms', []):
            attrs = api.lib.core.repo.getTableAttrs(form_class)
            if not attrs: continue

            for attr in attrs:
                extra_models[attr['field']] = attr

        for voc_core_name in self.model.get('use_vocs', []):
            attrs = api.lib.core.repo.getVocAttrs(voc_core_name)
            if not attrs: continue

            for attr in attrs:
                extra_models[attr['field']] = attr

        #print('extra_models', extra_models)


        # Формируем результат  
        res = []
        for field_name in field_list:
            if field_name in field_models:
                field_model = field_models[field_name]
                if isinstance(field_model, m_repo.AttrBase):
                    field_model = field_model.frontend_model()

                # Механизм наследования от прототипа
                proto_field = field_models[field_name].get('inherit')
                if type(proto_field) == bool:
                    proto_field = field_name

                if proto_field:
                    proto_model = extra_models.get(proto_field, {})
                    if isinstance(proto_model, m_repo.AttrBase):
                        proto_model = proto_model.frontend_model()

                    if 'type' in field_model: field_model.pop('type')
                    if 'attr_type' in field_model: field_model.pop('attr_type')
                    proto_model.update(field_model)
                    field_model = proto_model                    

                res.append(field_model)

            elif field_name in extra_models:
                res.append(extra_models[field_name])

            else:
                res.append(m_repo.Attr(field_name))

        return res, field_list


    def core_areport_model(self):                                   # Модель описания отчета для Core
        attrs, field_list = self.get_attr_models()

        for i in range(len(attrs)):
            if isinstance(attrs[i], m_repo.AttrBase):
                attrs[i] = attrs[i].frontend_model()                # Подставляем модель атрибута

        res = {
            'areport_params': self.areport_model(),
            'name'          : self.areport_class,
            'module_name'   : __name__,
            'params'        : self.model.get('params', []),
            'table'  : {
                'model' : attrs
            },
            'types'  : self.model.get('types', [])                  # Подставляем типы из описания отчета
        }

        if 'headers' in self.model:
            res['table']['headers'] = self.model['headers']

        return res
            #

    def save(self, params:dict, areport_id=None):                   # Сохранение отчета в БД
        report_model = self.core_areport_model()
        data = self.report_data

        # Удаляем не используемые колонки
        attrs = report_model['table']['model']
        field_list = [attr['field'] for attr in attrs]
        if isinstance(data, m_rec.RecordSet):
            data.attr_filter(field_list)

        if hasattr(data, 'to_dict'):
            data = data.to_dict()

        # Преобразование дат в строки, иначе сериализаторы дают ошибку
        for rec in data:
            for key in rec.keys():
                if type(rec[key]) == datetime.datetime:
                    rec[key] = rec[key].strftime('%Y-%m-%d %H:%M:$S')

                elif type(rec[key]) == datetime.date:
                    rec[key] = rec[key].strftime('%Y-%m-%d')

        # Сохранение отчета через интерфейс к CORE
        to_save = {
                'areport_type' : self.areport_class,
                'areport_name' : self.areport_name,
                'create_url'   : api.repo.AReports.get_create_url(self.areport_class),
                'attrs'        : params.get('attrs'),
                'result'       : {
                    'data' : data
                }
            }

        api.log.dump_add('AreportSerializer.save', data=to_save)
        storage = api.lib.AreportsSerializer( 
                api.context.request, 
                api.context.user, 
                areport_repository=report_model
            )
        if areport_id:
            saved = storage.update(areport_id, to_save)
        else:
            saved = storage.save(to_save)

        saved_data = saved['data']

        if not 'areport_id' in saved_data or not saved_data['areport_id']:
            raise Exception(f'Ошибка сохранения отчета:\n{saved}')

        return saved_data['areport_id']
        #
        #
    def create(self, params):                                       # Создание отчета с сохранением в БД
        self.report_data = self.build(params)

        if api.context.db_connect.is_closed():
            raise Exception('Подключе к БД закрыто при построении отчета')

        #raise Exception(f'{report_data}')
        areport_id = self.save(params)

        return areport_id
            #
            #
    def result_open_areport(self, areport_id):
        url = f'/analytical-reports/{areport_id}'
        return m_rest.ApiRest.result_url(url, inplace=True)
        

#=============================================================================================    
#   API
#=============================================================================================    

class ApiAReports(m_api.ModuleApi):
    def add_areport(self, areport_class, areport: AReport=None):
        if isinstance(areport_class, AReport):
            areport = areport_class
            areport_class = areport.areport_class
        
        else:
            areport.areport_class = areport_class

        api.repo.AReports.add(areport_class, areport, node_type=NODE_TYPE)
        areport.areport_class = areport_class


    def encode_url(self, url):
        url = url.replace('/', '%2F')
        url = url.replace('=', '%3D')
        url = url.replace('?', '%3F')
        url = url.replace('&', '%26')
        return url


    def get_create_url(self, areport_class):
        params_url = f'/api/areports2?action=get_params&areport_class={areport_class}'
        params_url = self.encode_url(params_url)

        build_url = f'/api/areports2?action=build&areport_class={areport_class}'
        build_url = self.encode_url(build_url)

        url = f'/analytical-report-create?a_report_params_url={params_url}&a_report_create_url={build_url}'
        return url


    def find_report(self, report_class):
        assert(report_class)

        res = api.repo.AReports.children_find_down(report_class, node_type=NODE_TYPE)
        if not res or not res[0]:
            raise Exception(f'Отчет {report_class} не найден (res={res})')
        return res[0][0]


    def dump_models(self):         # Возвращает стурктуры описаний аналитических отчетов для регистрации в Core
        reports = self._node.find_down(None, node_type=NODE_TYPE, single=False)
        return [report.core_areport_model() for report in reports]


def rest_endpoint(api, context, **kwargs):
    args = api.lib.rest.get_args(context)

    action = args.get('action')
    
    if action == 'get_params':                                  # Возвращает модель параметров для построения отчета
        areport_class = args.get('areport_class')
        areport_node:AReport = api.repo.AReports.find_report(areport_class)

        model = areport_node.frontend_params_model()
        if not model:
            return m_rest.ApiRest.result_error(f'Не определена модель для аналитического отчета "{areport_class}"')

        return m_rest.ApiRest.result(model)

    elif action in ('build', 'rebuild'):                                     # Строит таблицу данных отчета
        params = args.get('context',{})
        areport_class = params.get('areport_type')
        if action == 'rebuild': 
            if 'areport_id' not in params:
                raise Exception(f'Не задан areport_id. params={params}')
            kwargs['areport_id'] = params['areport_id']

        areport_node:AReport = api.repo.AReports.find_report(areport_class)
        api.log.warning(f'areport_class={areport_class} areport_node={areport_node}')
        action_func = lambda **kwargs : areport_node.build_and_save(**kwargs)
        action_thread = api.actions.run(m_actions.ActionNodeData(action_func), {'params':params})

        return m_rest.ApiRest.result(action_thread.frontend_status())

    elif action == 'make':                                      # Создает таблицу для загрузки через HTTP
        areport_class = args.pop('name')
        params = args

        areport_node:AReport = api.repo.AReports.find_report(areport_class)
        data = areport_node.build({'params':params}, None)
        if hasattr(data, 'to_dict'):
            data = data.to_dict()

        res = '''
            <HTML><HEAD><STYLE>
                td {
                    border:1px solid grey;
                }
            </STYLE></HEAD>
            <BODY><TABLE style="border-collapse: collapse;">
        '''
        res += '<TR>'
        for attr in areport_node.model['attrs']:
            res += f'<TD>{attr.get("caption") or attr.get("field") }</TD>'
        res += '</TR>'

        for row in data:
            res += '<TR>'
            for attr in areport_node.model['attrs']:
                res += f'<TD>{row.get(attr.get("field"))}</TD>'
            res += '</TR>'
        res += '</TABLE></BODY></HTML>'

        #res += f'<BR><PRE>{m_utils.pretty(data)}</PRE>'

        return m_rest.ApiRest.result_html(res)

    else:
        return m_rest.ApiRest.result_error(f'Действий "{action}" не определено для аналитических отчетов')


#=============================================================================================    
#   Источник данных AReports
#=============================================================================================    

import datetime

import m_ds

class AReportDS(m_ds.TableDataSource):                   # Источник данных для отчетных форм
    def __init__(self):
        super().__init__(AREPORT_TYPE, AREPORT_TABLE, PK_FIELD_NAME, CLASS_FIELD_NAME)

        # Список обновляемыъ полей в таблице 
        self.fields = {
                'areport_class'  : str,
                'areport_name'   : str, 
                'areport_period' : str, 
                'areport_data'   : dict,
                'areport_state'  : str, 
                'areport_params' : dict
            } 
        #
        #
    def create_report(self, areport_class, data):
        rec = self.create(None)

        rec.areport_class = areport_class
        rec.areport_data = data
        self.update(rec.areport_id, rec)


