import json

import m_rec

from sguk_ide import *

MODULE_PATH = 'api.check_forms'
CHECK_NODE_TYPE = 'ReportFormCheck'

api = None
_debug = False

def init(_api:SgukApi, debug=False):
    global api
    api = _api

    global _debug
    _debug = debug

    # Регистрация модуля
    api.add_module_api(MODULE_PATH, CheckReportForms())

    # Тестовый триггер на обновление Отчетных форм
    #api.triggers.add_trigger('DS_AFTER_UPDATE', 'ReportForms', tg_check_form)

    api.repo.ReportForms.add('check_form', check_form)


def tg_check_form(context:dict, skip_check=False, **kwargs):               
    if skip_check: return 

    NEW = context.get('NEW')
    OLD = context.get('OLD')

    if not NEW or 'form_state' not in NEW: 
        raise Exception(f'Параметр NEW не содержит атрибута form_state: {NEW}')

    if NEW['form_state'] != 'verified': return

    if OLD and 'form_state' not in OLD:
        raise Exception(f'Параметр OLD не содержит атрибута form_state: {OLD}')

    if not OLD or NEW['form_state'] != OLD['form_state']:
        if OLD: del context['OLD']
        rec = m_rec.Record(NEW)
        rec.add_alias('table', 'form_table_data.main_table')
        module().check(rec)


class Check:                                                        # Прототип объекта проверки отчетной формы
    def __repr__(self):
        return f'{self.__module__}.{self.__class__.__name__}'
        #
        #
    def check(self, api, form_data):
        return {
            'errors' : [{'row':None, 'col_name':None, 'text':None}],
            'notify' : {'text': None}
        }
        

import m_api
import sguk_forms

class CheckReportForms(m_api.ModuleApi):
    def __init__(self):
        super().__init__()
        #
        #
    def add_check(self, form_class, check_obj):             # Регистрация проверки в дереве репозитория
        form_node = api.repo.ReportForms.find_form(form_class)
        form_node.add('check', check_obj, node_type=CHECK_NODE_TYPE)
        #
        #
    def check(self, form_data, use_progress=False):         # Проверка данных отчетной формы
        if type(form_data) == dict:
            form_data = m_rec.Record(form_data)
            form_data.add_alias('table', 'form_table_data.main_table')

        class_name = form_data['form_no']

        # Поиск правил проверки
        class_node = api.repo.ReportForms.find_form(class_name)
        
        _, check_list = class_node.children_find_up( api.repo, node_type=CHECK_NODE_TYPE)
        if not check_list:
            print(f'sguk_check.check: path=={class_node.path()} - остуствуют функции проверки')
            
        # Вызов функций для проверки
        check_results = {'errors':[], 'notify':{}}

        if not check_list: 
            check_results['notify'] = {'Автоматизированные проверки отсутствуют':None}

        for check in check_list:
            check_res = check.check(form_data, use_progress=use_progress)

            for rec in check_res['errors']:
                rec['source'] = f'{check.__module__}.{check.__class__.__name__}'

            check_results['errors'].extend(check_res['errors'])
            check_results['notify'].update(check_res['notify'])


        # Конвертация в формат ошибок для отчетной формы
        msg_texts = {}
        for msg in check_results['errors']:
            index = json.dumps({'row':msg['row'], 'col_name':msg['col_name']})
            cell_msgs = msg_texts.get(index, [])
            #cell_msgs.append(f'{msg["text"]}\n({msg["source"]})')    # Вывод сообщения с указанием функции-источника
            cell_msgs.append(f'{msg["text"]}')
            msg_texts[index] = cell_msgs
            
        errors = []
        for pointer, text in msg_texts.items():
            record = {
                    'value'   : '\n\n'.join(text),
                    'pointer' : json.loads(pointer),
                    'source'  : 'sguk_check'
                }
            errors.append(record)

        # Конвертация в формат уведомлений для отчетной формы
        notifications = check_results['notify']

        return {'errors':{'main_table':errors}, 'notifications':notifications}


def check_form(form_rec=None, form_id=None):                # Проверка и отображение формы api.repo.ReportForms.check_form
    if form_rec.form_state not in sguk_forms.FORM_TO_CHECK_STATES: return {'errors':{'main_table':[]}}

    if form_id and form_rec == None:
        form_rec = api.repo.ReportForms.ds.read(form_id)

    check_res = api.check_forms.check(form_rec)

    api.repo.ReportForms.ds.update_errors( 
            form_rec.report_form_id, 
            check_res['errors'], 
            check_res['notifications']
        )

    form_rec['form_errors'] = check_res['errors']
    form_rec['form_notifications'] = check_res['notifications']
    api.repo.ReportForms.try_verified(form_rec)

    api.context.db_connect.commit()    

    return check_res


def module() -> CheckReportForms: 
    return api.find(MODULE_PATH)
