'''
Интрефейс к данным репозитория в CORE
'''
def init(_api):
    global api
    api = _api

    api.add_module_api('api.lib.core.repo', CoreRepoApi())


import m_utils, m_api

api = None


class CoreData:
    def __init__(self, data):
        self._data = data
        
    def get(self, _path, field=None, **kwargs):
        attrs = _path.split('.')
        data = self._data
        for attr in attrs:
            if attr not in data: 
                api.log.error(f'Атрибут {attr} не найден. Марщрут {_path}')
                return None
            data = data.get(attr)
            
        if not field:
            return data
    
        for item in data:
            if item.get('field') == field:
                return item

        return None
    
    
    def set(self, _path, _value):
        attrs = _path.split('.')
        key = attrs[-1]
        path = '.'.join(attrs[:-1])

        data = self.get(path)
        data[key] = _value

        
    def update(self, _path, field, **kwargs):
        keys = list(kwargs.keys())
        values = list(kwargs.values())
        
        data = self.get(_path, field=field)
        if not data: 
            api.log.error(f'CoreData.update: не найден маршрут {_path} field={field} self._data={m_utils.pretty(self._data)}')
            return

        for k,v in zip(keys, values):
            data[k] = v
            
    
    def print(self):
        print(m_utils.pretty(self._data))


class RepoData(CoreData):
    def getFields(self):
        return self.get('model')

    def getTableFields(self):
        return self.get('tables.main_table.model')

    def getTableField(self, field):
        return self.get('tables.main_table.model', field=field)
    
    def updateTableField(self, field, **kwargs):
        self.update('tables.main_table.model',field=field, **kwargs)    


import sguk_setup_core_repo


def coreGetReportFormModel(form_class:str)->RepoData:
    if form_class == '_test_form':
        model = {
            'model':[{'field':'FormField', 'caption':'Form field'}],
            'tables': {'main_table': {'model':
                [ {'field':'TableField', 'caption':'Table field'} ]
            }}
        }

    elif 'formModelGet' in api.lib:
        model = api.lib.formModelGet(form_class)

    elif form_class in sguk_setup_core_repo.form_models:
        model = sguk_setup_core_repo.form_models[form_class]

    else:
        api.log.error(f'Модель формы {form_class} не доступна')
        return None

    return RepoData(model)


def coreGetVocModel(voc_name:str)->RepoData:
    if voc_name == '_test_voc':
        model = {'model': 
            [{'field':'VocField', 'caption':'Voc field'} ]
        }

    elif voc_name in sguk_setup_core_repo.type_models:
        model = sguk_setup_core_repo.type_models[voc_name]

    elif 'vocModelGet' in api.lib:
        model = api.lib.vocModelGet(voc_name) 

    else:
        api.log.error(f'Справочник {voc_name} не доступен')
        model = {}

    return RepoData(model)


def coreGetTypeModel(type_name:str)->RepoData:
    if type_name == '_test_type':
        model = {}

    elif 'typeModelGet' in api.lib:
        model = api.lib.typeModelGet(type_name)

    elif type_name in sguk_setup_core_repo.type_models:
        model = sguk_setup_core_repo.type_models[type_name]

    else:
        api.log.error(f'Тип {type_name} не доступен')
        model = {}

    return RepoData(model)


class CoreRepoApi(m_api.ModuleApi):
    def updateTableFieldModel(self, form_class, field, **kwargs):
        formModel = coreGetReportFormModel(form_class)
        if formModel:
            formModel.updateTableField(field, **kwargs)


    def getTableAttrs(self, form_class):
        formModel = coreGetReportFormModel(form_class)
        if formModel:
            return formModel.getTableFields()
        else:
            return None


    def getVocAttrs(self, voc_name):
        vocModel = coreGetVocModel(voc_name)
        if vocModel:
            return vocModel.getFields()
        else:
            api.log.error(f'Не наидена модель справочника {voc_name}')
            return None


    def getTypeModel(self, type_name):
        model = coreGetTypeModel(type_name)
        if model:
            return model._data
        else:
            return None 
