'''
Модуль обработки событий генерируемых на Frontend

для атрибутов в описании которых есть
    on_change 
Frontend вызывает REST API и передает event, page_data, selected_row, selected_cell
модуль возвращает на Frontend данные для обновления в структуре 
{
    values:{field:value, ...},
    defaults:{field:value, ...}
}

/api/frontend_events/ReportForms
    определяет список событий которые нужно вызвать по page_data['form_no']
    фомирует контекст для функций событий
    вызывает их последовательно от верхних к нижним
    формирует результат события объединяя результаты


Добавление обработчика:
    api.repo.ReportForms.OperForms.add_frontend_event('name', func)
'''
NODE_TYPE = 'FrontendEvent'
REST_URL = '/frontend_events/'


api = None

def init(_api):
    global api
    api = _api

    # Создание API модуля
    moduleApi = FrontendEvents()
    api.add_module_api('api.lib.frontend_events', moduleApi)

    # Регистрация API модуля
    api.repo.add('add_frontend_event', moduleApi.add_event)
    api.repo.provide('add_frontend_event')

    # Регистрация endpoint
    api.add_url(moduleApi.get_url('ReportForms', prefix=False), moduleApi.process_ReportForms_event)


    # Обработчик по умолчанию
    api.repo.ReportForms.add_frontend_event('event_default', lambda _: {'values':{}, 'defaults':{}})


import m_rec, m_api, m_rest

class EventContext(m_rec.Record):
    def __init__(self):
        self.event = ''
        self.page_data = m_rec.Record({})
        self.selected_row = m_rec.Record({})
        self.selected_cell = m_rec.Record({})


class FrontendEvents(m_api.ModuleApi):
    def add_event(self, event_name, func, _path):
        _path.add(event_name, func, node_type=NODE_TYPE)


    def get_url(self, storage, prefix=True):
        url = '/api' if prefix else ''
        return url + REST_URL + storage


    def process_ReportForms_event(self, context=None, args=None, **kwargs):                        # endpoint для обработки событий
        try:
            # Определение списка событий для вызова
            if context == None:
                context = api.context

            if args != None: 
                pass
            elif 'args' in api.context:
                args = api.context.args
            else:
                args = api.lib.rest.get_args()

            res = {'values':{}, 'defaults':{}}
            if args.get('selected_cell',{}).get('value')==None: return res

            page_data = args.get('page_data')
            form_class = (page_data or {}).get('form_no')
            if not form_class:
                return m_rest.ApiRest.result({'error':f'Не найден атрибут form_no. page_data={page_data}'})

            # Определение списка обработчиков событий для заданного класса формы
            form_node = api.repo.ReportForms.find_form(form_class)
            if form_node == None:
                return m_rest.ApiRest.result({'error':f'Не найдена форма {form_class}'})

            events = form_node.children_find_up(api.repo, node_type=NODE_TYPE)

            #api.log.msg(f'process_event: form_node={form_node}  events={events}')

            # Формирование контекста для функция обработки событий
            context = m_rec.Record(args)

            # Вызова функция обработки событий
            for event_func in events[1]:
                if not callable(event_func): continue

                event_res = event_func(context)

                #api.log.msg(f'process_event: event_func={event_func.__name__}  args={context}  event_res={event_res}')

                if event_res:
                    res['values'].update(event_res.get('values', {}))
                    res['defaults'].update(event_res.get('defaults', {}))


            # Формирование ответа
            return m_rest.ApiRest.result(res)

        except Exception as e:
            api.log.exception(e, {'args':args})
            return m_rest.ApiRest.result_exception(e, f'args={args}')
