'''
Надстройка для экспорта отчетных форм в ЦЧ в формате SQLite

api
+-- export_cp
    +-- <ReportForm.class_name> : ExportMap
    +-- <Vocs.class_name> : ExportMap

Карты отображения определяются в соответствии с описанием модуля m_map
'''
MODULE_PATH = 'api.export_forms_cp'

api = None
_debug = False


def init(_api, debug=False):
    global api
    api = _api

    global _debug
    _debug = debug

    api.add_module_api(MODULE_PATH, ExportFormsCP(debug=False))


import os

import m_utils
import m_map
import m_api
import m_rest
import m_db_sqlite

import sguk_forms

# Действие "Экспорт в ЦЧ" для регистрации в api
def action_export_forms_cp(context=None, **kwargs):                

    def progress_callback(line, msg):                       # Callback для отображения прогресса
        completed = int(line*100/form_count)
        if 'action' in api.context:
            api.context.action.set_progress(completed)
        print(f'=== action_export_forms_cp: completed {completed}% : {msg} ===\n')

    # Определяем имя файла с шаблоном
    selected = context.get('selected_rows')
    if not selected:
        raise m_api.ApiException('Не выбраны формы для экспорта') 

    module:ExportFormsCP = api.find(MODULE_PATH)
    template_name = module.get_template_name(selected[0]['report_form_id'])
    db_template = os.path.join( api.get_config('API2_PATH',''), 'TMP', template_name)

    # Создаем копию из шаблона и коннект к БД SQLite
    db_file = api.lib.files.copy(db_template)
    if not os.path.isfile(db_file):
        raise Exception(f'Ошибка создания файла {db_file} из шаблона {db_template}')

    db = m_db_sqlite.DbSQLiteConnect(db_file=db_file)
    print(f'Экспорт в ЦЧ: используется шаблон {db_template}')

    # Экспортируем формы
    try:
        id_list = [row['report_form_id'] for row in context['selected_rows']]
        form_count = len(id_list)

        module.export_forms(id_list, db, progress_callback)
        db.commit()

        return m_rest.ApiRest.result_file(db_file)                      # Возвращаем файл для загрузки на Frontend

    except Exception as e:
        db.rollback()                                                   
        api.lib.files.delete(db_file)                                 # Удаляем временный файл
        api.log.exception(e)
        raise e


class ExportFormsCP(m_map.Mapping, m_api.ModuleApi):                                # Экспорт в центральную часть
    '''
    Параметры карты для метода add_map:
        source - название таблицы в БД-источнике 
        target - название таблицы в БД-получателе
    '''
    def __init__(self, debug=False):
        super().__init__(debug)
        self.db = None

        self.export_templates = {}       # Словарь шаблонов фвйлов SQLite для экспорта

        self._debug_id = None
        #
        #
    def print_warns(self, caption, info):
        if not self.map_warns: return

        print(caption)
        for warn in self.map_warns:
            print(' '*4, warn)
        print(' '*4, info, '\n')

        self.map_warns = []
        #
        #
    def get_template_name(self, report_form_id):                               # Возвращает имя SQLite-файла с шаблоном 
        form_data = api.repo.ReportForms.ds.read(report_form_id)
        form_class = form_data.form_no

        for template_form, template_name in self.export_templates.items():
            if m_utils.is_match(form_class, template_form):
                return template_name

        raise Exception(f'Не найден шаблон экспорта для формы {form_data.form_no} report_form_id={report_form_id} доступны {self.export_templates}')
        #
        #
    def sqlite_insert(self, class_name, data, primary_key=None):               # Добавляет запись в таблицу обменного файла
        if not data: return None

        _, params = self.find_map(class_name)
        try:
            table_name = params['target']
            if not self._debug:
                self.db.insert(table_name, data)
                if primary_key:
                    new_id = self.db.sql_exec(f"SELECT max({primary_key}) Id FROM {table_name}")[0].Id
                else:
                    new_id = None
            else:
                self._debug_id += 1
                new_id = self._debug_id
            return new_id

        except Exception as e:
            api.log.var({
                'sqlite_insert.class_name':class_name, 
                'sqlite_insert.table_name':table_name, 
                'sqlite_insert.data':data})
            #print(f'sqlite_insert: class_name={class_name} table_name={table_name} data={data}')
            raise e
        #
        #
    def export_reference(self, ref_name, value, *args, **kwargs):              # override m_map.Maping.export_reference
        is_update_value = args[0] if args else False
        author_id = kwargs.get('author_id')

        new_value = self._export_voc(ref_name, value, is_update_value, author_id)

        if not is_update_value:
            new_value = value
        return new_value 
        #
        #
    def _export_voc(self, voc_class, value_id, is_update_value, author_id):    # Экспортирует запись об организации

        _, params = self.find_map(voc_class)
        voc_source_class = params.get('source') or voc_class


        def voc_desc():
            desc = api.repo.Vocs[voc_source_class]._desc or voc_class
            return f'"{desc}"'


        voc_node = api.repo.Vocs.find(voc_source_class)
        if not voc_node:
            msg = f'export_voc: Не найден справочник {voc_desc()}'
            api.log.error(msg)
            raise Exception(msg)

        voc_record = voc_node.ds.read(value_id)
        if not bool(voc_record.keys()): 
            msg = f'export_voc: значение "{value_id}" не найдено в справочнике {voc_desc()}'
            api.log.warning(msg)
            api.log.var('voc_class', 'value_id', for_warn=True)
            return

        voc_record['author_id'] = author_id
        row = self.map_record(voc_record, voc_class)
        api.log.warning(*self.map_warns)
        api.log.var('voc_class', 'value_id', for_warn=True)
        #self.print_warns('export_voc:', f'voc_class={voc_class} value_id={value_id}')

        try:
            primary_key = 'Id' if is_update_value else ''
            return self.sqlite_insert(voc_class, row, primary_key=primary_key)

        except Exception as e:
            api.log.var({
                'export_voc.voc_class'  : voc_class,
                'export_voc.value_id'   : value_id,
                'export_voc.voc_record' : voc_record,
                'export_voc.row'        : row,
            })
            #print(f'export_voc: voc_record={voc_record} row={row}')
            raise e
        #
        #
    def _export_form(self, form_record):                                       # Экспортирует запись о форме
        map_name = form_record.form_no+'.Forms'
        row = self.map_record(form_record, map_name)
        api.log.warning(*self.map_warns)
        api.log.var('form_record', for_warn=True)

        new_id = self.sqlite_insert(map_name, row, primary_key='Id')
        if self._debug: print(f'export_form: {form_record.form_no} - форма экспортирована')
        return new_id
        #
        #
    def _export_header(self, form_record):                                     # Экспорт заголовка формы
        map_name = form_record.form_no + '.header'
        row = self.map_record(form_record, map_name)
        self.print_warns('export_header:', f'form_id={form_record.report_form_id}')

        self.sqlite_insert(map_name, row)
        if self._debug: print(f'export_header: {map_name} экспортирован')
        #
        #
    def _export_comments(self, form_record):                                   # Экспорт комментариев
        comments = form_record.get('form_comments.main_table')
        if not comments: return

        for line in comments:
            row = form_record.to_dict()
            row.update(line)
            row = self.map_record(row, 'Comments')
            self.print_warns('export_comments:', f'form_id={form_record.report_form_id}')

            self.sqlite_insert('Comments', row)
        #
        #
    def export_forms(self, form_id_list: list, db=None, cb_func=None):         # Экспортирует форму со связанными данными
        self.db = db
        self.voc_values = {}                                            # Инициализация списков использованных значений справочников
        self._debug_id = 0

        try:

            for i, form_id in enumerate(form_id_list):

                # Получаем данные формы
                form_record = api.repo.ReportForms.ds.read(form_id)
                form_record.add_alias('', 'form_main_data')

                # Предобработка форм 1.7, 1.8
                form_record = sguk_forms.zip_form(form_record)          # сворачивание списка радионуклидов
                form_record = sguk_forms.stretch_form(form_record)      # растягивание значений на пустые ячейки

                export_form_id = self._export_form(form_record)
                form_record['FormId'] = export_form_id                  # Сохраняем ID экспортированной формы

                # Экспорт заголовка формы
                self._export_header(form_record)

                # Экспорт комментариев к форме
                self._export_comments(form_record)

                # Экспорт таблицы - Разворачивание по табличной части если есть атрибут table
                for line, row_data in enumerate(form_record.table):
                    source_row = form_record.to_dict()
                    source_row.update(row_data)
                    source_row['Index'] = line + 1

                    for key in source_row.keys():
                        if source_row[key]=='-': source_row[key] = None

                    target_row = self.map_record(source_row, form_record.form_no, export=self, author_id=form_record.author_id)
                    is_warns = bool(self.map_warns)
                    self.print_warns('export_forms:', f'line={line} form_id={form_record.report_form_id} form_no={form_record.form_no}')
                    if is_warns:
                        map_name = self.find_map(form_record.form_no)[0]
                        print(f'использована карта {map_name}')
                        print('form_record:'); form_record.print(indent='    ')
                        print('row_data:'); row_data.print(indent='    ')

                    self.sqlite_insert(form_record.form_no, target_row)            # Сохраняем запись в таблице-получателе

                if cb_func: cb_func(i+1, f'{form_record.form_no}')          # Вызов индикатора прогресса

        except Exception as e:
            api.log.var({'export_forms.report_form_id':form_id})
            raise e
