import m_utils, m_rec
import m_ds
import m_actions
import m_repo

from sguk_ide import *

def init(_api:SgukApi, debug=False):
    global sguk_forms_api
    sguk_forms_api = _api

    _api.repo.ReportForms.add('find_form', find_form)

    # Endpoint для предоставления списка действий доступных из формы
    _api.web.reg_actions_generator('/forms/actions?action=list', forms_actions_generator)


    ds = ReportFormsDS()

    ds.fields = [
            'form_state',
            'form_part',
            'form_main_data',
            'form_table_data',
            'form_remarks',
            'form_comments',
            'form_warnings',
            'sign_info',
            'fill_info',
            'form_period_start',
            'form_period_end',
            'adjustment_no',
            'user_id',
            'kod_org',
            'author_id'
        ]

    _api.repo.add_datasource('ReportForms.ds', ds)
    _api.repo.ReportForms.ds.add('update_errors', update_errors)
    _api.repo.ReportForms.add('error_add', error_add)
    _api.repo.ReportForms.add('error_count', error_count)
    _api.repo.ReportForms.add('error_clear', error_clear)
    _api.repo.ReportForms.add('set_state', set_state)
    _api.repo.ReportForms.add('try_verified', try_verified)

    _api.repo.add('ReportForms.core', ReportFormsCore())

    _api.repo.add_attr_type('form_state', m_repo.AttrTypeDict({
            'draft'     : 'Черновик',
            'verified'  : 'Проверена',
            'remarks'   : 'Замечания',  
            'confirmed' : 'Подтверждена',
            'loaded'    : 'Загружена',
            'unloaded'  : 'Выгружена',
            'canceled'  : 'Анулирована',
        })
    )

    _api.repo.add_attr_type('FormNo', FormNo('str'))

    _api.repo.ReportForms.add_action('action_forms_delete', action_forms_delete, caption='Удалить', index='2')

    _api.repo.ReportForms.add_action('action_forms_check', action_forms_check, caption='Проверить', form_type='edit')


    # Кнопка действий в журнале связанных с обменными файлами
    _api.repo.ReportForms.add_action('ActionExchange', caption='Обменный файл')

    # Кнопки действий реализованных на Frontend
    _api.repo.ReportForms.ActionExchange.add_action('LoadAP', caption='Загрузить АЧ', icon='/interface/download.svg', frontend_func='onDownloadOfflinePart')
    _api.repo.ReportForms.add_action('ExportToFile', caption='Экспорт в ...', frontend_func='onExport')
    _api.repo.ReportForms.add_action('FormCorrection', caption='Корректировка', index='8', frontend_func='onCreateAdjustmentReport')
    _api.repo.ReportForms.add_action('FormNew', caption='Новая форма', 
            frontend_func='onOpenCreateReportForm', 
            icon = '/interface/plus-white.svg',
            index='9', 
            frontend_props={"data-type": "accent"} #'style': 'background: #003274; color: #FFFFFF'})
        )



import sguk_forms


class FormNo(m_repo.AttrType):
    def decode(self, form_class):
        type_name = sguk_forms.form_type(form_class).split('_')
        class_year = sguk_forms.form_year(form_class)
        return f'Форма {type_name[1]}.{type_name[2]} ({class_year})'

    def frontend_model(self):
        res = super().frontend_model()
        res.update({
                'base_type'   : 'BASE',
            })

        return res



'''
    Регистрирует endpoint для предоставления списка действий в формах
'''
REPORT_FORMS_TYPE = 'ReportForms'
REPORT_FORMS_TABLE = 'ReportForms'
PK_FIELD_NAME = 'report_form_id'
CLASS_FIELD_NAME = 'form_no'

t_form_1_1 = 'form_1_1'
t_form_1_2 = 'form_1_2'
t_form_1_3 = 'form_1_3'
t_form_1_4 = 'form_1_4'
t_form_1_5 = 'form_1_5'
t_form_1_6 = 'form_1_6'
t_form_1_7 = 'form_1_7'
t_form_1_8 = 'form_1_8'
t_form_1_9 = 'form_1_9'

t_form_2_1 = 'form_2_1'
t_form_2_2 = 'form_2_2'
t_form_2_3 = 'form_2_3'
t_form_2_4 = 'form_2_4'
t_form_2_5 = 'form_2_5'
t_form_2_6 = 'form_2_6'
t_form_2_7 = 'form_2_7'
t_form_2_8 = 'form_2_8'
t_form_2_9 = 'form_2_9'
t_form_2_10 = 'form_2_10'
t_form_2_11 = 'form_2_11'
t_form_2_12 = 'form_2_12'

t_form_3_1 = 'form_3_1'
t_form_3_2 = 'form_3_2'

t_form_4_1 = 'form_4_1'

t_form_5_1 = 'form_5_1'
t_form_5_2 = 'form_5_2'
t_form_5_3 = 'form_5_3'
t_form_5_4 = 'form_5_4'
t_form_5_5 = 'form_5_5'
t_form_5_6 = 'form_5_6'
t_form_5_7 = 'form_5_7'


form_2022_year = ''

form_1_1_2022 = 'form_1_1' + form_2022_year
form_1_2_2022 = 'form_1_2' + form_2022_year
form_1_3_2022 = 'form_1_3' + form_2022_year
form_1_4_2022 = 'form_1_4' + form_2022_year
form_1_5_2022 = 'form_1_5' + form_2022_year
form_1_6_2022 = 'form_1_6' + form_2022_year
form_1_7_2022 = 'form_1_7' + form_2022_year
form_1_8_2022 = 'form_1_8' + form_2022_year
form_1_9_2022 = 'form_1_9' + form_2022_year

form_2_1_2022 = 'form_2_1' + form_2022_year
form_2_2_2022 = 'form_2_2' + form_2022_year
form_2_3_2022 = 'form_2_3' + form_2022_year
form_2_4_2022 = 'form_2_4' + form_2022_year
form_2_5_2022 = 'form_2_5' + form_2022_year
form_2_6_2022 = 'form_2_6' + form_2022_year
form_2_7_2022 = 'form_2_7' + form_2022_year
form_2_8_2022 = 'form_2_8' + form_2022_year
form_2_9_2022 = 'form_2_9' + form_2022_year
form_2_10_2022 = 'form_2_10' + form_2022_year
form_2_11_2022 = 'form_2_11' + form_2022_year
form_2_12_2022 = 'form_2_12' + form_2022_year

form_3_1_2022 = 'form_3_1' + form_2022_year
form_3_2_2022 = 'form_3_2' + form_2022_year

form_4_1_2022 = 'form_4_1' + form_2022_year

form_5_1_2022 = 'form_5_1' + form_2022_year
form_5_2_2022 = 'form_5_2' + form_2022_year
form_5_3_2022 = 'form_5_3' + form_2022_year
form_5_4_2022 = 'form_5_4' + form_2022_year
form_5_5_2022 = 'form_5_5' + form_2022_year
form_5_6_2022 = 'form_5_6' + form_2022_year
form_5_7_2022 = 'form_5_7' + form_2022_year


def form_type(form_no:str)->str:
    if '_' not in form_no[7:]: return form_no

    pos = form_no.index('_',7) 
    return form_no[:pos]


def form_year(form_no:str) -> str:
    if '_' not in form_no[7:]: return form_no

    pos = form_no.index('_',7)
    return form_no[pos+1:]


class ReportFormsDS(m_ds.TableDataSource):                   # Источник данных для отчетных форм
    def __init__(self):
        aliases = {
                'table': 'form_table_data.main_table',
                'remarks' : 'form_remarks.main_table',
                'comments': 'form_comments.main_table',
                'warnings': 'form_warnings.main_table'
            }         
        super().__init__(REPORT_FORMS_TYPE, REPORT_FORMS_TABLE, PK_FIELD_NAME, CLASS_FIELD_NAME, aliases=aliases)
            #
            #
    def create(self, form_class):
        data = dict(
            form_no = form_class,
            form_main_data = {},
            form_table_data = {'main_table':[]},
            form_remarks = {},
            form_comments = {},
            form_warnings = {}
        )
        return super().insert(data)
            #
            #
    def update_table(self, report_form_id, table_data):
        db = sguk_forms_api.context.db_connect

        if hasattr(table_data, 'to_dict'):
            table_data = table_data.to_dict()

        value_str =  db.value_sql_str( {'main_table': table_data} , 'jsonb')

        sql = f'''
            UPDATE "ReportForms" SET
                form_table_data ={value_str}
                WHERE report_form_id = {report_form_id}
        '''
        db.sql_exec(sql, res_type=None)


class ReportFormsCore:                          # Интерфейс к фукнция Core для работы с таблицей ReportForms
    def preset(self, rs):                       # Найстройка алиасов
        rs.add_alias('table','form_table_data.main_table')
            #
            #
    def read(self, report_form_id):             # Чтение отчетной формы
        _api = sguk_forms_api
        pg_conn = _api.context.db_connect.conn
        rec_dict = _api.lib.FormsSerializer( _api.context.request, _api.context.user, connection=pg_conn).read(report_form_id)

        rec = m_rec.Record(rec_dict)
        self.preset(rec)
        return rec 
            #
            #
    def create(self, data):                     # Создание отчетной формы
        _api = sguk_forms_api
        if hasattr(data, 'to_dict'):
            data = data.to_dict()

        pg_conn = _api.context.db_connect.conn
        rec_dict = _api.lib.FormsSerializer( _api.context.request, _api.context.user, connection=pg_conn).save(data)
        _api.context.db_connect.commit()

        rec = m_rec.Record(rec_dict)
        self.preset(rec)
        return rec
            #
            #
    def update(self, data):                     # Обновление отчетной формы
        _api = sguk_forms_api

        if not _api.lib.find('FormsSerializer'): return None

        report_form_id = data['report_form_id']

        if type(data) in (m_rec.Record, m_rec.RecordSet):
            data = data.to_dict()

        pg_conn = _api.context.db_connect.conn
        rec_dict = _api.lib.FormsSerializer( _api.context.request, _api.context.user, connection=pg_conn).update(report_form_id, data)
        _api.context.db_connect.commit()

        rec = m_rec.Record(rec_dict)
        self.preset(rec)
        return rec


sguk_forms_api = None

def zip_form(form_rec):                                 # Сворачивает значения полей радионуклиды и активность из нескольких строк
    fields = {             # Поле для определения записи об операции и сворачиваемые поля
        'form_1_7_2022' : ('g2', ['g12', 'g13']),    # OpCod, радионуклиды, активность
        'form_1_8_2022' : ('g2', ['g9' , 'g10'])     # OpCod, радионуклиды, активность
    }.get(form_rec.form_no)

    if fields:
        form_rec.table = zip_values(form_rec.table, fields[0], fields[1])

    return form_rec
        #
        #
def zip_values(table, oper_field, value_fields):        # Сворачиваем значения из нескольких строк в одну
    # Консолидируем данные о радионуклидах
    r_buf = []
    oper_line = None

    def add_values(row):
        for field in value_fields:
            if row[field]:
                if field not in r_buf: r_buf[field] = []
                r_buf[field].append(row[field])


    for line, row in enumerate(table):

        if row[oper_field] or line == table.count()-1:
            if oper_line != None:
                # Сохраняем список радионуклидов во всех строках
                if not row[oper_field]: 
                    end_line = line+1
                    add_values(row)
                else: 
                    end_line = line 
                    
                for field in value_fields:
                    r_str = '; '.join(r_buf[field])
                    for i in range(oper_line, end_line):
                        table[i][field] = r_str

            oper_line = line
            r_buf = {}

        add_values(row)

    return table
        #

def stretch_form(form_rec):                                 # Заполняет пустые ячеки, удаляет пустые строки
    fields = {       # Определение поля для (сброса буфера, идентификации пустых строк)
        'form_1_7_2022' : ('g2', 'g21'),     # код операции, код РАО
        'form_1_8_2022' : ('g2', 'g18')      # код операции, код РАО
    }.get(form_rec.form_no)

    if fields:
        form_rec.form_table_data.main_table = stretch_rows(form_rec.form_table_data.main_table, fields[0], fields[1])
        #
        #
def stretch_rows(src_table, reset_field, clear_field):      # Заполняет пустные ячейки из заполненных ячеек колонки
    dst_table = []

    buf = None

    for _row in src_table:
        row = m_rec.Record(m_utils.simplify(_row))

        if row[clear_field]:
            if not buf or row[reset_field]:
                buf = row
            else:
                buf.update(row, update_to_none=False, update_to_blank=False)
                row.update(buf, update_to_none=True)

            dst_table.append(row)
        
    return m_rec.RecordSet(dst_table)
        

def forms_actions_generator(context:dict, **kwargs):
    form_no = context['form']['form_no']
    form_node = sguk_forms_api.repo.find_class('ReportForms', form_no, is_assert=False)
    if not form_node:
        form_no = form_type(form_no)

    forms = sguk_forms_api.repo.children_find_down(node_name=form_no)
    if not forms[0]:
        return []

    path = forms[0][0].path()
    path = path.split('.repo.')[-1]

    return m_actions.rest_list(path, form_type='edit')


def find_form(form_class):
    api = sguk_forms_api

    form_node = api.repo.find_class('ReportForms', form_class, is_assert=False)
    if form_node: return form_node

    form_class = form_type(form_class)
    form_node = api.repo.find_class('ReportForms', form_class, is_assert=False)
    if form_node: return form_node

    form_class = form_class + '_2022'
    form_node = api.repo.find_class('ReportForms', form_class, is_assert=False)
    if form_node: return form_node

    return None

import m_rest

def action_forms_delete(context, **kwargs):
    api = sguk_forms_api

    selected = context.get('selected_rows')
    if not selected:
        return m_rest.ApiRest.result_notify('Ошибка', 'Не выбраны отчетные формы для удаления')

    id_list = [row['report_form_id'] for row in selected]

    api.context.action.status_msg = 'Удаление отчетных форм'
    for no, id in enumerate(id_list):
        api.repo.ReportForms.ds.delete(id)
        api.context.action.progress = (no+1) / len(id_list)

    api.context.action.status_msg = 'Очистка регистров'
    api.regs.reg_clean(autocommit=False)

    api.context.db_connect.commit()

    return m_rest.ApiRest.result_notify('Удаление выполнено', f'Удалено отчетных форм {len(id_list)}', refresh='all')


def action_forms_check(context, **kwargs):
    api = sguk_forms_api

    selected = context.get('report_form')
    if not selected:
        return m_rest.ApiRest.result_notify('Ошибка', 'Не выбрана отчетная форма')

    form_id = selected['report_form_id']

    form_rec = api.repo.ReportForms.ds.read(form_id)

    if form_rec.form_state not in FORM_TO_CHECK_STATES:
        return m_rest.ApiRest.result_notify('Проверка формы', f'Проверка недоступна для состояния формы {form_rec.form_state}')

    api.repo.ReportForms.check_form(form_rec)
    try_verified(form_rec)

    form_rec = api.repo.ReportForms.ds.read(form_id)
    err_count = form_rec.form_errors.main_table.count()
    err_msg = f'Ошибок: {err_count}' if err_count else 'Ошибки не найдены'

    return m_rest.ApiRest.result_notify('Результат проверки отчетной формы', err_msg, refresh='all')


def update_errors(report_form_id, errors=None, notify=None, convert=True):
    api = sguk_forms_api
    update_pairs = []

    if convert and errors:
        for i in range(len(errors['main_table'])):
            errors['main_table'][i]['value'] = {'text': errors['main_table'][i]['value']}
    if errors != None:
        errors_str = api.context.db_connect.value_sql_str(errors, 'jsonb')
        update_pairs.append(f'form_errors = {errors_str}')

    new_notify = []
    if convert and notify:
        for msg, row_no in notify.items():
            new_notify.append({'value':{'text':msg}, 'pointer':{'row':row_no}})
        new_notify = {'main_table': new_notify}
    else:
        new_notify = notify

    if notify != None:
        notify_str = api.context.db_connect.value_sql_str(new_notify, 'jsonb')
        update_pairs.append(f'form_notifications = {notify_str}')


    if not update_pairs: return

    sql = f'''
        UPDATE "ReportForms" SET 
            {', '.join(update_pairs)}
        WHERE report_form_id={report_form_id}
    '''
    api.context.db_connect.sql_exec(sql, res_type=None)

# FORM_CANCELED = {'canceled'}

FORM_TO_CHECK_STATES = {'draft', 'verified', 'loaded', 'remarks'}

FORM_ACTIVE_STATES =  ('verified', 'confirmed', 'unloaded', 'accepted')


def error_add(form_rec:m_rec.Record, row_no:int, col_name:str, error_msg:str, source:str=None):
    error_msg = error_msg.replace('"', '\"')

    form_rec.form_errors.main_table.append({
        'pointer' : {'row': row_no, 'col_name':col_name},
        'source'  : source,
        'value'   : {'text': error_msg},
    })


def error_clear(form_rec:m_rec.Record):
    form_rec.form_errors.main_table = []


def error_count(form_rec:m_rec.Record):
    try:
        return len(form_rec.form_errors.main_table)
    except:
        return 0

def set_state(form_rec=None, new_state=None):
    api = sguk_forms_api

    new_dict = form_rec.to_dict()
    new_dict['form_state'] = new_state
    context = {
        'OLD' : form_rec,
        'NEW' : new_dict
    }

    api.log.msg(f'set_state: id={form_rec.report_form_id} NEW={new_dict["form_state"]} OLD={form_rec.form_state}')

    api.triggers.emit('DS_BEFORE_UPDATE', form_rec.form_no, context)

    sql = f'''
        UPDATE "ReportForms" SET
            form_state = '{new_state}'
            WHERE report_form_id = {form_rec.report_form_id}
    '''
    api.context.db_connect.sql_exec(sql, res_type=None)

    api.triggers.emit('DS_AFTER_UPDATE', form_rec.form_no, context)


def try_verified(form_rec):
    api = sguk_forms_api

    is_errors = bool( form_rec['form_errors'].get('main_table') )

    if is_errors and form_rec.form_state in FORM_TO_CHECK_STATES:
        api.repo.ReportForms.set_state(form_rec = form_rec, new_state = 'remarks')

    elif not is_errors and form_rec.form_state not in FORM_ACTIVE_STATES:
        api.repo.ReportForms.set_state(form_rec = form_rec, new_state = 'verified')


    api.context.db_connect.commit()


