'''
Модуль для описания регистров и способов отображения в регистры

Для отображения создается триггер на DS_AFTER_UPDATE. 

Триггер выполняет правила отображения заданные в репозитории в виде объекта типа RegMap
на для всех элементов и текущего и до api.repo

Классы:
    Register - класс описывающий регистр и методы работы с ним
    RegMap  - класс описывающий способ отображения
'''

#==================================================================================
#   Инициализация
#==================================================================================
def init(_api):
    global api
    api = _api

    api.add_module_api('regs', ApiRegs())
    api.triggers.add_trigger('DS_AFTER_UPDATE', '', tg_map_report_form)  

    api.add('api.regs.acc_o1', Register('acc_o1'))   
    api.add('api.regs.acc_o2', Register('acc_o2'))   
    api.add('api.regs.acc_o3', Register('acc_o3'))   
    api.add('api.regs.acc_o4', Register('acc_o4'))   
    api.add('api.regs.acc_o5', Register('acc_o5'))   
    api.add('api.regs.acc_o6', Register('acc_o6'))   
    api.add('api.regs.acc_o7', Register('acc_o7'))   
    api.add('api.regs.acc_o8', Register('acc_o8'))   
    #api.add('api.regs.acc_o9', Register('acc_o9'))   

    api.add('regs.acc_snk_1', RegisterSQL('acc_snk_1', snk_sql('acc_o1')))
    api.add('regs.acc_snk_2', RegisterSQL('acc_snk_2', snk_sql('acc_o2')))
    api.add('regs.acc_snk_3', RegisterSQL('acc_snk_3', snk_sql('acc_o3')))
    api.add('regs.acc_snk_4', RegisterSQL('acc_snk_4', snk_sql('acc_o4')))
    api.add('regs.acc_snk_5', RegisterSQL('acc_snk_5', snk_sql('acc_o5')))
    api.add('regs.acc_snk_6', RegisterSQL('acc_snk_6', snk_sql('acc_o6')))
    api.add('regs.acc_snk_7', RegisterSQL('acc_snk_7', snk_sql('acc_o7')))
    api.add('regs.acc_snk_8', RegisterSQL('acc_snk_8', snk_sql('acc_o8')))


import datetime

def snk_sql(reg='acc_o1', snk_date: str=None, org_id=None):
    field_sets = {
        'acc_o1': {
            'balance_fields' : 'balance_amount',
            'fields' : '''             
                "PaspN", "Typ", "Nuclid", "Numb", "Activn", "IzgotOKPO", "IzgotDate", "Kateg", "Nss", 
                "FormSobst", "Pravoobl", "PrName", "UktPrTyp", "UktPrN"
            ''',
            'condition' : 'balance_amount > 0'
        },
        'acc_o2': {
            'balance_fields' : 'balance_mass_net',
            'fields' : '''
                "PaspN", "Typ", "Numb", "IzgotOKPO", "IzgotDate", "Nss",
                "FormSobst", "Pravoobl", "PrName", "UktPrTyp", "UktPrN"
            ''',
            'condition' : 'balance_mass_net > 0'
        },
        'acc_o3': {
            'balance_fields' : 'balance_amount',
            'fields' : '''
                "PaspN", "Typ", "Nuclid", "Numb", "Activn", "IzgotOKPO", "IzgotDate", "Agr",
                "FormSobst", "Pravoobl", "PrName", "UktPrTyp", "UktPrN"        
            ''',
            'condition' : 'balance_amount > 0'
        },
        'acc_o4': {
            'balance_fields' : 'balance_volume, balance_mass_net',
            'fields' : '''
                "PaspN", "IstName", "Typ", "Nuclid", "Activn", "ActDate", "Arg",
                "FormSobst", "Pravoobl", "PrName", "UktPrTyp", "UktPrN"
            ''',
            'condition' : 'balance_volume > 0 OR balance_mass_net'
        },
        'acc_o5': {
            'balance_fields' : 'balance_amount',
            'fields' :'''
                "PaspN", "Typ", "Nuclid", "Numb", "Activn", "IzgotDate", "BCod",
                "PrName", "UktPrTyp", "UktPrN",
                "PH_Name", "PH_Cod", "CodRAO", "Subsid", "FCP"        
            ''',
            'condition' : 'balance_amount > 0'
        },
        'acc_o6': {
            'balance_fields' : 'balance_volume, balance_mass_net, balance_amount',
            'fields' :'''
                "RAOCod", "BCod", "RAOCodMax",
                "ActTR", "ActBG", "ActA", "ActUR", "ActDate",
                "PH_Name", "PH_Cod", "UpCod",
                "PrName", "UktPrTyp", "UktPrN",
                "Subsid", "FCP"        
            ''',
            'condition' : 'balance_amount > 0 OR balance_mass_net OR balance_amount > 0'
        },
        'acc_o7': {
            'balance_fields' : 'balance_volume, balance_mass_gross',
            'fields' :'''
                g4, g5, g6, g7, g8, g9, 
                g12, g13,
                g19, g20, g21, g22, g23, g24, g25, g26, g27, g28, g29, g30, g31, g32
            ''',
            'condition' : 'balance_volume > 0 OR balance_mass_gross > 0'
        },
        'acc_o8': {
            'balance_fields' : 'balance_volume, balance_mass_net',
            'fields' :'''
                g4, g5, 
                g8, g9, g10, 
                g16, g17, g18, g19, g20, g21, g22, g23, g24, g25, g26, g27, g28
            ''',
            'condition' : 'balance_volume > 0 OR balance_mass_net > 0'

        },
    }
    fields = field_sets[reg]['fields']
    balance_fields = field_sets[reg]['balance_fields']
    condition = field_sets[reg]['condition']

    date_where_str = f' AND "OpDate"<={snk_date} ' if snk_date else ''

    if org_id: 
        org = api.repo.Vocs.VocOrgs.ds.read(org_id)
        if not org or org.is_empty():
            raise Exception(f'Не найдена организация org_id={org_id}')
        kod_org_cond = f"kod_org='{org.KOD_ORG}'"
    else:
        kod_org_cond = '1=1' 


    sql = f'''
        SELECT kod_org, op_item, {balance_fields}, {fields} 
        FROM (
            SELECT  
                items.kod_org, items.op_item, reg.org_id,
                (SELECT sum(amount*op_sign) FROM {reg} 
                    WHERE kod_org=items.kod_org AND op_item=items.op_item AND "OpDate">=items.inv_date {date_where_str}
                ) + items.inv_amount balance_amount,
                (SELECT sum(volume*op_sign) FROM {reg} 
                    WHERE kod_org=items.kod_org AND op_item=items.op_item AND "OpDate">=items.inv_date {date_where_str}
                ) + items.inv_volume balance_volume,
                (SELECT sum(mass_net*op_sign) FROM {reg} 
                    WHERE kod_org=items.kod_org AND op_item=items.op_item AND "OpDate">=items.inv_date {date_where_str}
                ) + items.inv_mass_net balance_mass_net,
                (SELECT sum(mass_gross*op_sign) FROM {reg} 
                    WHERE kod_org=items.kod_org AND op_item=items.op_item AND "OpDate">=items.inv_date {date_where_str}
                ) + items.inv_mass_gross balance_mass_gross,
                {fields}
            FROM 
                (
                SELECT 
                    all_items.kod_org, all_items.op_item, 
                    coalesce(inv.last_inv_date, '2000-01-01') inv_date,
                    coalesce(inv.amount, 0) inv_amount,
                    coalesce(inv.volume, 0) inv_volume,
                    coalesce(inv.mass_net, 0) inv_mass_net,
                    coalesce(inv.mass_gross, 0) inv_mass_gross,

                    (SELECT max(record_id) 
                        FROM {reg} 
                        WHERE kod_org=all_items.kod_org AND op_item=all_items.op_item AND op_sign>=0 {date_where_str}) last_in_rec
                        
                FROM
                    (SELECT distinct op_item, kod_org FROM {reg} WHERE {kod_org_cond}) as all_items
                LEFT OUTER JOIN 
                    (SELECT kod_org, op_item, "OpDate" last_inv_date, amount, volume, mass_net, mass_gross 
                        FROM {reg} 
                        WHERE "OpCod"='10' {date_where_str} AND {kod_org_cond}
                        ORDER BY "OpDate" desc limit 1) as inv
                    ON inv.kod_org=all_items.kod_org AND inv.op_item=all_items.op_item
                ) as items
            INNER JOIN 
                {reg} as reg on reg.record_id = items.last_in_rec 
        ) as res
        WHERE {condition}
    '''

    return sql


import m_rec
import m_db, m_ds
import m_triggers
import m_api

import sguk_forms

api = None
debug = True


class ApiRegs(m_api.ModuleApi):
    def find_maps(self, form_class):
        # Поиск объектов RegMap для отображения
        class_node = api.repo.ReportForms.find_form(form_class)
        if not class_node:
            return 
        
        _, mapping_list = class_node.children_find_up( api.repo, node_type='RegMap')

        return mapping_list            
            #
            #
    def map_form(self, form_rec, src='', clear_only=False, nosave=False) -> dict:
        if form_rec.form_state not in sguk_forms.FORM_ACTIVE_STATES and not nosave:
            clear_only = True

        # Содаем клон исходной записи
        if hasattr(form_rec, 'to_dict'):
            form_rec = form_rec.to_dict()
        form_rec = m_rec.Record(form_rec)
        
        sguk_forms.stretch_form(form_rec)

        # Поиск карт отображения в репозитории
        class_name = form_rec.form_no
        mapping_list = self.find_maps(class_name) or []

        api.log.msg(f'map_form started {class_name} clear_only={clear_only} карты отображения {mapping_list}')

        # Подготовка контекста для вызова маппинга
        form_id = form_rec.report_form_id

        mapping = {}
        try:
            # Очистка предыдущего мапинга
            if not nosave:
                for map in mapping_list:
                    map.clear(form_id)

            # Отображение объектов
            if not clear_only:
                for map in mapping_list:                
                    mapped_data = map.map_obj(form_rec, nosave=nosave)

                    # Удаляем атрибуты не используемые в регистре
                    mapped_data = m_rec.RecordSet(mapped_data, convert=False)
                    mapped_data.attr_filter(map.reg.table.fields)

                    mapping[map.reg.reg_name] = mapped_data

        except m_triggers.TriggerException as e:
            api.log.error(f'map_form: TriggerException {e} \n{e.args}\n')
            api.log.exception(e)

        except Exception as e:
            api.log.exception(e)
            raise e

        if self.is_debug() and not nosave:
            api.log.msg(f'map_form completed')

        return mapping
            #
            #
    def remap_form(self, report_form_id, clear_only=False):
        form_rec = api.repo.ReportForms.ds.read(report_form_id)
        self.map_form(form_rec, clear_only=clear_only)
            #
            #
    def remap_forms(self, pattern=None, autocommit=True):
        filter = {'form_no': 'LIKE %'+pattern+'%'} if pattern else {}
        forms = api.repo.ReportForms.ds.select(filters=filter, fields=['report_form_id','form_no', 'form_state'])

        res = {'errors':{}, 'remapped':{}}
        for rec in forms:

            clear_only = rec.form_state not in sguk_forms.FORM_ACTIVE_STATES
            form_id = rec.report_form_id
            form_title = {
                'report_form_id': form_id, 
                'form_state': rec.form_state
            }

            try:
                self.remap_form(form_id, clear_only=clear_only)
                form_title['action'] = 'cleared' if clear_only else 'mapped'
            
            except Exception as e:
                form_title['error'] = m_utils.exception_text(e)
               
            container = res['errors'] if 'error' in form_title else res['remapped']
            if rec.form_no not in container: container[rec.form_no] = []
            container[rec.form_no].append(form_title)

        if autocommit:
            api.context.db_connect.commit()
        return res


    def get_snk_sql(self, form_no, report_date, org_id=None):
        api.log.msg(f'get_snk_sql: form_no={form_no}  report_date="{report_date}"  org_id={org_id}')
        reg = RegisterSnk(form_no=form_no, report_date=report_date, org_id=org_id)
        return reg.sql
            #
            #
    def create_snk(self, reg_name, report_date, org_id=None):
        return RegisterSnk(reg_name, report_date, org_id=org_id)

    def reg_clean(self, autocommit:bool):
        '''
        Удаляет из всех регистров записи для которых отсутствует запись в ReportForms
        '''
        try:
            reg_list = []
            db = api.context.db_connect

            for reg in api.regs.children_find_down()[1]:
                if not isinstance(reg, m_ds.TableDataSource): continue
                if 'report_form_id' not in reg.table.fields: continue

                table_name = reg.table.table_name
                sql = f'''
                    DELETE FROM {table_name} WHERE report_form_id NOT IN (
                        SELECT report_form_id FROM "ReportForms"
                    );        
                '''
                try:
                    db.sql_exec(sql, res_type=None)
                    reg_list.append(table_name)
                except Exception as e:
                    api.log.exception(e)

            if autocommit: db.commit()
            api.log.msg(f'reg_clean: Проверены регистры {reg_list}')

        except Exception as e:
            api.log.exception(e)


class Register(m_ds.TableDataSource):                    # Класс описания регистра хранящегося в таблице
    '''
    Регистр создается из структуры таблицы в БД, имя которой соответствует имени регистра

    insert - добавляет в таблицу регистра данные всех атрибуты которые есть в таблице
             атрибуты отсутсвующие в таблице игнорируются
             параметр map позволяет переименовать атрибуты в исходных данных {<new_name>:<old_name>}

             создание записей происходит через m_db.DbAbstractConnect.insert(...)
    '''
    def __init__(self, reg_name):
        self.reg_name = reg_name
        self.table_name = reg_name

        super().__init__(self.table_name, self.table_name, 'record_id')
        #
        #
    def __repr__(self):
        return f'Register( {self.table_name} )'
        #
        #
    def insert(self, data:m_rec.Record):      # Добавление отображения в регистр
        if type(data) == dict:
            data = m_rec.Record(data)

        data.attr_filter(self.table.fields)            # Удаление атрибутов отсутствующих в модели

        # Атрибуты содержание "-" становятся NULL
        for k, v in data._data.items():
            if v=='-':
                data._data[k] = None

        if debug or api.context.db_connect.db_type != 'sqlite':
            super().insert(data)
        #
        #
    def clear(self, obj_id):                 # Очистка отображения в регистр
        db = self._get_connect()

        if debug or api.context.db_connect.db_type != 'sqlite':
            db.delete(self.table, filters={ 'report_form_id' : obj_id })


class RegisterSQL:
    def __init__(self, reg_name, sql):
        self.reg_name = reg_name
        self.sql = sql
            #
            #
    def __repr__(self):
        return f'RegisterSQL( {self.reg_name} )'
        #
        #
    def _db_connect(self, use_system_connect=False) -> m_db.DbAbstractConnect:    
        if use_system_connect:
            return api.system.system_db_connect

        return api.context.db_connect
        #
        #
    def select(self, filters=[], orders=[], limit=None, use_system_connect=False) -> m_rec.RecordSet:    # Возвращает RecordSet из регистра
        sql_res = self.sql
        db = self._db_connect(use_system_connect=use_system_connect)

        if filters:
            filter_sql = db.sql_pairs(filters, ' AND ')
            if filter_sql:
                sql_res  += f' WHERE {filter_sql}'

        if limit:
            sql_res += f' LIMIT {limit}'

        return db.sql_exec(sql_res)


class RegisterSnk(RegisterSQL):
    def __init__(self, reg_name=None, report_date=None, form_no=None, org_id=None):
        regs = {
            sguk_forms.t_form_1_1 : 'acc_o1',
            sguk_forms.t_form_1_2 : 'acc_o2',
            sguk_forms.t_form_1_3 : 'acc_o3',
            sguk_forms.t_form_1_4 : 'acc_o4',
            sguk_forms.t_form_1_5 : 'acc_o5',
            sguk_forms.t_form_1_6 : 'acc_o6',
            sguk_forms.t_form_1_7 : 'acc_o7',
            sguk_forms.t_form_1_8 : 'acc_o8',
        }

        if form_no:
            form_type = sguk_forms.form_type(form_no)
            if form_type in regs: 
                reg_name = regs[form_type]
            else:
                raise Exception(f'Неизвестный класс отчетной формы {form_no}')

        if report_date == None:
            report_date = m_utils.date_str()

        if type(report_date) == str:
            report_date = m_utils.str2date(report_date)

        elif type(report_date) == datetime.date:
            pass

        else:
            raise Exception(f'Не корректный тип параметра report_date {type(report_date)} должен быть str или datetime.date')

        date_str = api.system.system_db_connect.value_sql_str(report_date, 'date')
        sql = snk_sql(reg_name, date_str, org_id=org_id)

        super().__init__(reg_name, sql)


class RegMap:                                      # Класс описания отображения               
    '''
    Класс для описания отображения регистр
        - отображает в регистр заданный в конструкторе по соответствию имен атрибутов
        - после создания добавляется в дерево репозитория
        - вызывается из триггера для отображения (tg_map_report_form)
        - использует методы Register.insert, Register.clear
    '''
    def __init__(self, reg, map={}):
        '''
        Создает класс для отображения в регистр.
            reg - ссылка на регистр. Пример: api.regs.acc_o1
        '''
        self.reg = reg
        self.map = map
        #
        #
    def __repr__(self):
        if self.__class__.__name__ == 'RegMap':
            return f'RegMap( {self.reg} )'
        else:
            return f'{self.__class__.__name__}'
        #
        #
    def clear(self, obj_id):                   # Очистка предыдущего отображения
        self.reg.clear(obj_id)                       
        #
        #
    def map_obj(self, form_rec:m_rec.RecordSet, nosave=False):              # Отображение объекта
        pass


class FormMapping(RegMap):

    def map_obj(self, form_rec:m_rec.Record, nosave=False) -> list:           # Отображает форму в регистры
        # Отображение строк таблицы
        form_attrs = {}                 # Извлекает только атрибуты "простого" типа
        for k, v in form_rec.items():
            if type(v) not in (list, dict) and not isinstance(v, m_rec.Record) and not isinstance(v, m_rec.RecordSet):
                form_attrs[k] = v


        org_id = form_attrs['author_id']
        org_rec = api.repo.Vocs.VocOrgs.ds.read(org_id)
        if org_rec.is_empty():
            raise Exception(f'Не определен KOD_ORG для org_id={org_id}')
        
        form_attrs['kod_org'] = org_rec.KOD_ORG

        mapped_rows = []
        rows:m_rec.RecordSet = form_rec['form_table_data']['main_table']
        for row_no, table_row in enumerate(rows): 

            row_ext_dict = dict(form_attrs)
            row_ext_dict.update( table_row.to_dict() )
            row = m_rec.Record( row_ext_dict , convert=False )

            row['table_row'] = row_no
            row.set_default('')

            try:
                row_data = self.map_row(row, nosave=nosave)
                mapped_rows.append(row_data)
            except AttributeError:
                if not nosave: raise

            if 'action' in api.context:
                api.context.action.status_msg2 = 'Отображение формы'
                api.context.action.progress2 = int((row_no+1) * 100 / rows.count())

        return mapped_rows
        #
        #
    def map_row(self, row_data: m_rec.Record, nosave=False) -> m_rec.Record:   # Отображает строку таблицы в регистр        
        row_data.map(self.map, rename=True)

        if not nosave:
            self.reg.insert(row_data)

        return row_data



# Состояния ReportForms при переходе в которые происходит отображение в регистры
MAPPED_FORM_STATES = sguk_forms.FORM_ACTIVE_STATES

import m_utils
import sguk_api

def tg_map_report_form(context, **kwargs):     # Триггер для обработки отображения
    '''
    Запускает обработчики маппинга типа RegMap размещенные в репозитории между api.repo 
    и классом обрабатываемого документа.

    Запускается только при именении form_state на значение verified. 
    Для принудительного запуска установить context['force_run'] = True
    '''
    #core_db_connect = kwargs.get('core_db_connect')
    #if not core_db_connect:
    #    api.log.error(f'tg_map_report_form: не передан core_db_connect')

    #api.build_context(core_db_connect=core_db_connect)

    NEW = context.get('NEW')
    OLD = context.get('OLD')

    if not NEW or not OLD: return
    if 'form_state' not in NEW or 'form_state' not in OLD: return
    if NEW.get('form_state') == OLD.get('form_state'): return

    form_new = m_rec.Record(NEW) if type(NEW) == dict else NEW
    form_old = m_rec.Record(OLD) if type(OLD) == dict else OLD

    #api.log.msg(f'tg_map_report_form: id={form_new.report_form_id} new_state={form_new.form_state} old_state={form_old.form_state}')

    # Проверяем условие для запуска отображения
    if form_new.form_state in MAPPED_FORM_STATES and form_old.form_state not in MAPPED_FORM_STATES:
        api.regs.map_form(form_new, clear_only=False)

    elif form_new.form_state not in MAPPED_FORM_STATES and form_old.form_state in MAPPED_FORM_STATES:
        api.regs.map_form(form_new, clear_only=True)

    api.context.db_connect.commit()
    

#==================================================================================
#   Описание регистров для IDE
#==================================================================================
if 1==2:
    class RegList:
        #acc_proto = Register()
        #acc_oper = Register()
        #acc_items = RegisterSQL()
        acc_o1 = Register()
        acc_o2 = Register()
        acc_o3 = Register()
        acc_o4 = Register()
        acc_o5 = Register()
        acc_o6 = Register()
        acc_o7 = Register()
        acc_o8 = Register()
        acc_o9 = Register()



