from sguk_api import api
import sguk_areports
from sguk_ide import *

import sguk_forms
from m_repo import Attr, Param

#==============================================================================
#   26. История учетной единицы
#==============================================================================
class Ar26(sguk_areports.AReport):
    def __init__(self):
        super().__init__()
        self.areport_name = 'История ЗРИ'
        self.areport_class = 'ar_26d'
        self.model = {
                'name' : '26. История учетной единицы',
                'module_name' : 'ar_26d',
                'params': [
                    Param('op_item', 'Код учетной единицы'),
                ],
                'attrs' : [
                    { 'field' : 'org_id',       'caption' : 'Организация',          'type' : 'OrgId', },
                    { 'field' : 'OpDate',       'caption' : 'Дата операции',        'width':'10em',     'type' : 'date', },
                    { 'field' : 'OpCod',        'caption' : 'Код операции',         'type' : '11_type'},
                    { 'field' : 'PaspN',        'caption' : 'Номер паспорт', },
                    { 'field' : 'Typ',          'caption' : 'Тип ЗРИ',              'typr' : '23_type' },
                    { 'field' : 'Nuclid',       'caption' : 'Нуклиды',              'type':'radionuclide_voc_type',},
                    { 'field' : 'Numb',         'caption' : 'Номер ЗРИ', },
                    { 'field' : 'Sht',          'caption' : 'Кол-во', },
                    { 'field' : 'Activn',       'caption' : 'Активность, Бк',       'type': 'exponential', },
                    { 'field' : 'IzgotOKPO',    'caption' : 'ОКПО изготовителя', },
                    { 'field' : 'IzgotDate',    'caption' : 'Дата изготовления', },
                    { 'field' : 'Kateg',        'caption' : 'Категория ЗРИ', },
                    { 'field' : 'Nss',          'caption' : 'НCC, мес.', },
                    { 'field' : 'DocVid',       'caption' : 'Вид док-та',           'type': '12_type',},
                    { 'field' : 'DocN',         'caption' : 'Номер док-та',     },
                    { 'field' : 'DocDate',      'caption' : 'Дата док-та',          'type' : 'date' },
                    { 'field' : 'OkpoPIP',      'caption' : 'ОКПО поставщ. или получ.', 'type' : 'OKPO_type' },
                    { 'field' : 'OkpoPrv',      'caption' : 'ОКПО перевозч.',       'type' : 'OKPO_type' },
                    { 'field' : 'PrName',       'caption' : 'Наимен. прибора',      'type': '26_type',},
                    { 'field' : 'UktPrTyp',     'caption' : 'Тип УКТ или прибора', },
                    { 'field' : 'UktPrN',       'caption' : 'Номер УКТ или прибора', },
                    { 'field' : 'sign',         'caption' : 'Знак операции',        'type' : 'int' },
                    { 'field' : 'form_state',   'caption' : 'Состояние ОФ', },
                    { 'field' : 'is_active',    'caption' : 'Активность ОФ', },
                ]
            }

    def build(self, params):
        form_no = params['form_no']
        if form_no.startswith(sguk_forms.t_form_1_1):
            reg = api.regs.acc_o1

        elif form_no.startswith(sguk_forms.t_form_1_5):
            reg = api.regs.acc_o5

        else:
            raise Exception(f'История ЗРИ недоступна для формы {form_no}')

        op_item_wo_org = params['op_item'].split('#', 1)[-1]
        return reg.select(filters={'op_item': f"LIKE %{op_item_wo_org}"})



#==============================================================================
#   Инициализация модуля
#==============================================================================

def init(_api:SgukApi, debug=False):
    global api
    api = _api

    api.repo.AReports.add('ar_26', Ar26())