'''
Используется api.context:
    db_connect
    form_data   данные формы
    row_data
    row_no
    row_result
'''
from dateutil.relativedelta import relativedelta
import re

import m_utils
from m_utils import str2date, str2float
from m_rec import Record, RecordSet

import sguk_vocs
import sguk_setup_regs
import sguk_forms
from sguk_check import *

from sguk_ide import *

api = None
_debug = False

# Проверяет непрерывность относительно периода предыдущей формы
def check_form_period():
    form = api.context.form_data

    if not form.form_period_end:        
        return 'Должна быть указана дата окончания периода'

    period_start = str2date(form.form_period_start)
    period_end = str2date(form.form_period_end)
    if period_start and period_end and period_end > period_start:
        return 'Окончание периода должно быть позже начала периода'

    try:
        db = api.context.db_connect
    except:
        db = api.system.system_db_connect

    period_start_str = db._date_encode(form.form_period_start)

    sql = f'''
            SELECT {period_start_str} - (
                SELECT  max(form_period_end) 
                    FROM "ReportForms" f
                    INNER JOIN "VocOrgs" o ON o.record_id = f.author_id 
                    WHERE f.form_state in ('verified', 'confirmed', 'unloaded')
                        AND o."KOD_ORG" = (SELECT "KOD_ORG" FROM "VocOrgs" WHERE value_id='{form.author_id}')
                        AND f.form_no = '{form.form_no}'
                        AND f.form_period_start < {period_start_str}
            ) AS pause, {period_start_str}
        '''
        
    rs = db.sql_exec(sql)
    if rs[0].pause:
        return 'Начала периода не соответствует окончанию предыдущего периода'


from sguk_api import api


class CheckFormData:
    def __init__(self):
        self._errors = []

    def add_error(self, attr_name, text):
        self._errors.append({'col_name':attr_name, 'row':'', 'text':text})

    def errors(self):
        return self._errors


class CheckRowData:
    def __init__(self, row_no):
        self.row_no = row_no

        self._errors = []
        self._notify = {}

    def add_error(self, column, text):
        self._errors.append({'col_name':column, 'row':self.row_no, 'text':text})

    def add_notify(self, text):
        if text not in self._notify:
            self._notify.update({text:self.row_no})

    def errors(self):
        return self._errors

    def notify(self):
        return self._notify


def load_vocs_cache():                          # Загружает словари в кэш
    vocs_cache = sguk_vocs.VocCache()
    vocs_cache.reg_voc('VocOrgs', ['attrs'], preload=False, indexes=['record_id'])
    vocs_cache.reg_voc('DocVid', ['attrs'])
    vocs_cache.reg_voc('PH_Name', ['attrs']) 
    vocs_cache.reg_voc('radionuclide_voc1', ['attrs'], preload=False)

    #vocs_cache.reg_voc('VocOrgs', ['''attrs->'OKPO' as "OKPO" '''], preload=False, indexes=['record_id'])
    #vocs_cache.reg_voc('DocVid', ['''attrs->'DocName' as "DocName" '''])
    #vocs_cache.reg_voc('PH_Name', ['''attrs->'7' as "7" ''']) 
    #vocs_cache.reg_voc('radionuclide_voc1', [
    #    '''attrs->'Kod_gruppy' as "Kod_gruppy" ''', 
    #    '''attrs->'Period_p_r' as "Period_p_r" ''', 
    #    '''attrs->'Edinica_izmer_p_r' as "Edinica_izmer_p_r"'''])

    vocs_cache.reg_voc('OKSM', [], indexes=['value_id', 'label'])
    vocs_cache.reg_voc('SubRF', [], indexes=['value_id', 'label'])
    vocs_cache.reg_voc('FormSobst', [])
    vocs_cache.reg_voc('PrName', [])
    vocs_cache.reg_voc('BCod', [])
    vocs_cache.reg_voc('UpCod', [])
    vocs_cache.reg_voc('ORI', [])

    api.context['vocs'] = vocs_cache
        #
        #
def voc_check(voc_name, value_id):              # Проверяет наличие словаря в кэше
    return api.context.vocs.exist(voc_name, value_id)
        #
        #
def voc_read(voc_name, value_id):               # Читает значение из кэша
    return api.context.vocs.read(voc_name, value_id)


class ReportFormCheck(Check):
    def check(self, form_data, use_progress=False):

        def set_action_attr(**kwargs):
            if not 'action' in api.context: return

            for attr, value in kwargs.items():
                setattr(api.context.action, attr, value)

        if 'db_connect' not in api.context:
            Exception(f'Не найден api.context.db_connect. Context={api.context.to_dict()}')

        api.context['form_data'] = form_data
        api.context['form_result'] = CheckFormData()

        if 'vocs' not in api.context or not api.context.vocs:
            load_vocs_cache()

        set_action_attr( status_msg2 = 'Загружаем данные формы' )

        self.check_form()        

        errors = api.context.form_result.errors()
        notify = {}

        assert form_data.table!=None, f'check: form_data.table is None. form_data={form_data}'

        for row_no, row_data in enumerate(form_data.table):
            self.add_row_aliases(row_data)
            api.context['row_data'] = row_data
            api.context['row_no'] = row_no
            api.context['row_result'] = CheckRowData(row_no)

            try:
                self.check_row()

                errors.extend( list(api.context.row_result.errors()) ) 
                notify.update( dict(api.context.row_result.notify()) )

            except Exception as e:
                errors.append({
                    'row' : row_no,
                    'col_name': '',
                    'text' : f'Ошибка на сервере {m_utils.exception_text(e)}'

                })

            set_action_attr(
                progress2 = round((row_no+1) * 100 / form_data.table.count(),1),
                status_msg2 = 'Проверка формы'
            )

            if 'action' in api.context and not api.context.action.is_active():
                raise Exception('Выполнение прервано')

        set_action_attr( status_msg2 = 'Сохраняем ошибки' )

        return {'errors':errors, 'notify':notify}
        #
        #
    def check_form(self): 
        check_form_attr_not_empty('email')
        check_form_attr_not_empty('phone')
        check_form_attr_not_empty('position')
        check_form_attr_not_empty('full_name')
        #
        #
    def add_row_aliases(self, row_data):
        # Добаляет алиасы к названия столбцов таблицы
        # row_data.add_alias('new name', 'exitst name')
        pass


    def check_row(self):
        result:CheckRowData = api.context.row_result
        row_data:Form_1_1_table = api.context.row_data

        self.add_notify(result, row_data)


    def add_notify(self, result:CheckRowData, row_data):
        '''
        Генерация напоминаний при проверке формы
        '''
        pass


def form_start_year()->str:                     # Выделяет строку с годом начала применения формы содержащейся в контексте
    form_no = api.context.form_data.get('form_no')
    return sguk_forms.form_year(form_no)
        #
        #
def check_form_attr_not_empty(attr_name):
    value = api.context.form_data.get('form_main_data.'+attr_name,'')
    result:CheckFormData = api.context.form_result

    if type(value)==str:
        if len(value.strip()) == 0:
            result.add_error(attr_name, 'Значение не должно быть пустым')

        elif len(value) != len(value.strip()):
            result.add_error(attr_name, f'Значение "{value}" не должно содержать скрытых символов, лидирующих и замыкающих пробелов')
        #
        #
def check_contain_any(value:str, required:list) -> bool:
    for item in required:
        if item in value: return True
    return False
        #
        #
def check_okpo(okpo_str):                                                   # Проверка кода ОКПО (8,10,14 символов)
    try:
        int(okpo_str)
        return len(okpo_str) in (8, 10, 14)
    except:
        return False
        #
        #
def check_num(value:str, allow_brackets=False) -> bool:
    if type(value) not in (int,float,str):
        return False

    if type(value)==str:
        value = value.strip()

    if allow_brackets and value:
        if value[0] == '(' and value[-1] == ')': value = value[1:-1]

    try:
        str2float(value, is_assert=True)
        return True
    except:
        return False
        #
        #
def check_separated(value_str:str, pattern:str='[^A-Za-zА-Яа-я\-\d/()\s\.]'):
    values = value_str.split(';')
    for value in values:
        if re.findall(pattern, value.strip()): 
            return False
    return True
        #
        #
def in_ranges(x, *lst):
    for cond in lst:
        if type(cond)==tuple and x>=cond[0] and x<=cond[1]: return True
        elif type(cond)==list and x in cond: return True
        elif x == cond: return True 
    return False
        #
        #
def any(*conds):
    for cond in conds:
        if cond: return True
    return False
        #
        #
def contain_word(s:str, words:str):
    s = s.lower()
    for i in words.split(' '):
        if i.lower() in s: return True
    return False
    
# --- Проверки внутри формы ---
def check_remark(attr_name:str) -> bool:                        # Проверка наличия примечания
    form_data = api.context.form_data
    row_no = api.context.row_no

    for remark in form_data.form_remarks.main_table:
        if remark['pointer']['col_name'] == attr_name and remark['pointer']['row'] == row_no:
            return True

    return False
        #

# --- Проверки по аналитическим регистрам
def check_related_oper(form_type:str, filter:dict) -> bool:     # Проверка: была ли отображена операция в заданном типе отчетных форм
    reg = {
            sguk_forms.t_form_1_1: api.regs.acc_o1,
            sguk_forms.t_form_1_2: api.regs.acc_o2,
            sguk_forms.t_form_1_3: api.regs.acc_o3,
            sguk_forms.t_form_1_4: api.regs.acc_o4,
            sguk_forms.t_form_1_5: api.regs.acc_o5,
            sguk_forms.t_form_1_6: api.regs.acc_o6,
            }[form_type]
    return not reg.select(filters=filter).is_empty()

# --- Проверки по СНК ---
def get_form_cache() -> dict:                                               # Возвращает отображение формы по регистрам
    form_data:Record = api.context.form_data

    res = api.regs.map_form(form_data, nosave=True) or {}

    #if not res and form_data:
    #    raise Exception(f'Ошибка отображения формы {form_data.form_no} report_form_id={form_data.report_form_id}')

    return res
        #
        #
def merge_records(records:RecordSet, cache:RecordSet) -> RecordSet:         # Объединяем записи регистра с cache
    forms = cache.attr_values('report_form_id')         # Список report_form_id которые присутствуют в cache
    records = RecordSet(records.to_dict())              # Клонируем RecordSet

    for form_id in forms:
        records.delete({'report_form_id': form_id})

    records.append( cache )

    return records
        #
        #
def get_item_history(item_id, reg_node, cache:RecordSet=None) -> RecordSet: # Вовзращает RecordSet операций с УЕ обновленный отображением текущей формы
    rs = reg_node.select(filters={'op_item':item_id}) 

    if cache:
        cache = cache.filter({'op_item': item_id})

        rs = merge_records(rs, cache)

    return rs
        #
        #
def check_snk(item_id:str, reg_node, cache:RecordSet=None) -> bool:      # Проверяет присутствие УЕ в СНК
    rs = get_item_history(item_id, reg_node, cache=cache)
    return not rs.is_empty()
        #
        #
def check_snk_amount_balance(item_id, reg_node, cache=None) -> float:    # Проверяет баланс УЕ по атрибуту amount
    hist = get_item_history(item_id, reg_node, cache=cache)
    return hist.calc_sum(lambda rec: float(rec.amount)*float(rec.op_sign))
        #
        #
def check_prev_oper(item_id, OpCod, reg_name:str,  cache:RecordSet, OpCodAttr:str='OpCod'):     # Проверка: была отображена операция для УЕ (acc_o1)
    reg = api.regs.find(reg_name)
    
    if not cache.filter({'op_item':item_id, OpCodAttr:OpCod}).is_empty(): return True
    if not reg.select(filters={'op_item':item_id, OpCodAttr:OpCod}).is_empty(): return True

    return False
        #
        #

# --- Проверки по справочникам ---
def org_OKPO() -> str:                                  # Возвращает ОКПО организации
    form_data:Record = api.context.form_data

    org_id = form_data.author_id
    org_rec = voc_read('VocOrgs', org_id)
    try:
        return org_rec.OKPO
    except:
        raise Exception(f'Не найдена организация. org_id={org_id} form_data={org_rec.to_dict()}')
        #
        #
def is_in_OKSM(value_id) -> bool:                       # Проверка наличия в справочнике ОКСМ
    if voc_check('OKSM', value_id): return True
    if not api.repo.Vocs.OKSM.ds.select(filters={'label':value_id}).is_empty(): return True
    return False
        #
        #
def is_in_SubRF(value_id) -> bool:                      # Проверка наличия в справочнике субьектов РФ
    if voc_check('SubRF', value_id): return True
    if not api.repo.Vocs.SubRF.ds.select(filters={'label':value_id}).is_empty(): return True
    return False
        #
        #
def is_nuclide(value_id) -> bool:
    if value_id in ('', '-') : return True
    return voc_check('radionuclide_voc1', value_id.lower())

# --- Проверка стандартных полей ---
def check_attr_not_empty(AttrName:str) -> bool:         # Проверка на не пустую строку
    value = api.context.row_data.get(AttrName,'')
    result:CheckRowData = api.context.row_result

    if type(value)==str:
        if len(value.strip()) == 0:
            result.add_error(AttrName, 'Значение не должно быть пустым')
            return False

        elif len(value) != len(value.strip()):
            result.add_error(AttrName, f'Значение "{value}" не должно содержать скрытых символов, лидирующих и замыкающих пробелов')
            return False

    return True
        #
        #
def check_attrs_not_empty(attrs):
    if type(attrs) == str:
        attrs = attrs.split(' ')
        attrs = [attr.strip() for attr in attrs]

    res = True
    for attr in attrs:
        if attr:
            check_res = check_attr_not_empty(attr)
            res = res and check_res

    return res
        #
        #
def check_attr_num(attrName:str, minValue=0, maxValue=None, allowNone=False, msg=None):
    result:CheckRowData = api.context.row_result
    valueStr = api.context.row_data.get(attrName)

    if valueStr=='-' and allowNone: return

    value = str2float(valueStr, None)

    if msg==None:
        if minValue!=None and maxValue!=None:
            msg = f'Должно быть числовое значение от {minValue} до {maxValue}'
        elif minValue!=None:
            msg = f'Должно быть числовое значение больше {minValue}'
        else:
            msg = f'Должно быть числовое значение'

        if allowNone:
            msg += ' либо «-» без кавычек'

    if value==None or (minValue!=None and value<minValue) or (maxValue!=None and value>maxValue):
        result.add_error(attrName, msg)
        #
        #
def check_attr_date(attr_name: str, blank_str:str='-'):
    row_data = api.context.row_data
    result:CheckRowData = api.context.row_result
    date_str = row_data[attr_name]

    if date_str == blank_str: return

    if not m_utils.str2date(date_str, default=None):
        result.add_error(attr_name, f'Не корректный формат даты "{date_str}"')
        #
        #
def check_date_between(check_date_attr, min_date_attr, max_date_attr, msg):
    form_data = api.context.form_data
    row_data = api.context.row_data
    result:CheckRowData = api.context.row_result

    check_date = str2date(row_data[check_date_attr])
    if not check_date: return

    if min_date_attr:
        min_date = str2date(row_data.get(min_date_attr, form_data.get(min_date_attr)))
        if min_date and min_date > check_date:
            result.add_error(check_date_attr, msg)

    if max_date_attr:
        max_date = str2date(row_data.get(max_date_attr, form_data.get(max_date_attr)))
        if max_date and max_date < check_date:
            result.add_error(check_date_attr, msg)
        #
        #
def check_attr_DocVid(attr_name:str):    # Проверка наличия в справочнике "Виды документов"
    result:CheckRowData = api.context.row_result
    row_data = api.context.row_data
    DocVid = row_data.get(attr_name,'')

    if not voc_check('DocVid', DocVid):
        result.add_error(attr_name, 'Значение должно быть из справочника "Виды документов сопровождающих операцию"')

    if row_data.DocVid == '19' and not check_remark(attr_name):
        result.add_error(attr_name, 'При выборе этого кода в примечании к ячейке формы должно быть приведено наименование документа')
        #
        #
def check_attr_DocDate(DocDateAttr:str='DocDate', OpDateAttr:str='OpDate', OpCodAttr:str='OpCod'):
    result:CheckRowData = api.context.row_result
    form_data = api.context.form_data
    row_data = api.context.row_data
    DocDate = row_data.get(DocDateAttr,'')
    OpDate = row_data.get(OpDateAttr,'')
    OpCod = row_data.get(OpCodAttr,'')

    if OpCod == '41':
        if str2date(OpDate) != str2date(DocDate):
            result.add_error(DocDateAttr, 'Дата документа должна соответствовать дате операции')
    elif OpCod == '10':
        check_date_between(DocDateAttr, 'form_period_start', 'form_period_end', 'Дата документа выходит за границы периода')
    else:
        check_date_between(DocDateAttr, None, OpDateAttr, 'Дата документа не может быть позже даты операции')
        #
        #
def check_attr_Nss(nssAttr:str='Nss', izgotDateAttr:str='IzgotDate', operDateAttr:str='OpDate', item_name:str='ЗРИ'):     # Проверка: Дата операции внутри срока пользования
    result:CheckRowData = api.context.row_result
    row_data = api.context.row_data
    oper_date = str2date(row_data.get(operDateAttr))
    izgot_date = str2date(row_data.get(izgotDateAttr))
    nss_str = row_data.get(nssAttr)

    try:
        nss = int(nss_str)
        assert(nss<=600)
        assert(nss>0)
    except:
        result.add_error(nssAttr, 'Графа НСС должна содержать кол-во месяцев не более 600')
        return False

    if oper_date and izgot_date and oper_date > izgot_date + relativedelta(months=nss):
        result.add_error(nssAttr, f'Для {item_name} истек НСС, следует продлить НСС либо снять с учета с одновременной постановкой на учет как РАО (при выполнении критериев отнесения к РАО)')
        return False

    return True
        #
        #
def check_attr_Agr(agrAttr:str):
    result:CheckRowData = api.context.row_result
    agr = api.context.row_data.get(agrAttr, '')

    if str(agr) not in '1 2 3':
        result.add_error(agrAttr, 'Значение должно быть из справочника "Агрегатное состояние"')
        #
        #
def check_attr_OkpoPIP(oksmCodes:str, selfOkpoCodes:str, OkpoPipAttr:str='OkpoPIP', OpCodAttr:str='OpCod'):
    result:CheckRowData = api.context.row_result
    row_data = api.context.row_data
    OkpoPIP = row_data.get(OkpoPipAttr,'')
    OpCod = row_data.get(OpCodAttr,'')

    if OkpoPIP == 'Минобороны':
        pass
    elif OpCod in selfOkpoCodes:
        selfOkpo = org_OKPO()
        if OkpoPIP != selfOkpo:
            result.add_error(OkpoPipAttr, f'Должен быть код ОКПО отчитывающейся организации {selfOkpo}')
    elif OpCod in oksmCodes:
        if not is_in_OKSM(OkpoPIP):
            result.add_error(OkpoPipAttr, 'Код поставщика/получателя должен быть из справочника ОКСМ')
        if not check_remark(OkpoPipAttr):
            result.add_error(OkpoPipAttr, 'Должно быть примечание к ячейке "Код поставщика/получателя"')
    else:
        if not check_okpo(OkpoPIP):
            result.add_error(OkpoPipAttr, 'Длина значения должна быть 8 или 14 символов')
        #
        #
def check_attr_OkpoPrv(noneOkpoCodes:str, OkpoPrvAttr:str='OkpoPrv', OpCodAttr:str='OpCod'):
    result:CheckRowData = api.context.row_result
    row_data = api.context.row_data
    OkpoPrv = row_data.get(OkpoPrvAttr)
    OpCod = row_data.get(OpCodAttr)

    if OpCod in noneOkpoCodes:
        if OkpoPrv != '-':
            result.add_error(OkpoPrvAttr, 'Должно быть "-"')
    elif OkpoPrv == 'Минобороны' and OpCod in '22 32':
        pass
    elif OpCod in '21 22 25 27 28 29 31 32 35 36 37 38 39 61 62 81 82 83 84 85 86 87 88':
        if not check_okpo(OkpoPrv):
            result.add_error(OkpoPrvAttr, 'Длина значения должна быть 8, 10 или 14 символов')
        #
        #
def check_attr_IzgotOKPO(IzgotOkpoAttr:str, opCodAttr:str, itemName:str=''):
    result:CheckRowData = api.context.row_result
    row_data = api.context.row_data
    IzgotOKPO = row_data.get(IzgotOkpoAttr,'')
    OpCod = row_data.get(opCodAttr,'')

    if OpCod in '11':
        okpo = org_OKPO()
        if IzgotOKPO != okpo:
            result.add_error(IzgotOkpoAttr, f'Должен быть код ОКПО отчитывающейся организации {okpo}')

    elif IzgotOKPO.isdigit():
        if not check_okpo(IzgotOKPO):
            result.add_error(IzgotOkpoAttr, 'Длина кода ОКПО 8, 10 или 14 символов')

    elif is_in_OKSM(IzgotOKPO):
        pass

    elif IzgotOKPO.lower() == 'прим.':
        if not check_remark(IzgotOkpoAttr):
            result.add_error(IzgotOkpoAttr, f'Необходимо указать в примечании наименование и адрес организацииизготовителя {itemName}')

    else:
        result.add_error(IzgotOkpoAttr, f'Должен быть код ОКПО, или код ОКСМ или "прим."')
        #
        #
def check_attr_FormSobst(row_data:Record, formSobstAttr:str):
    if form_start_year()<'2022': return

    result:CheckRowData = api.context.row_result
    form_sobst = row_data.get(formSobstAttr)

    if not voc_check('FormSobst', form_sobst):
        result.add_error(formSobstAttr, 'Значение должно быть из справочника "Формы собственности"')
        #
        #
def check_attr_Pravobl(PravooblAttr:str='Pravoobl', FormSobstAttr:str='FormSobst', itemName:str=''):
    if form_start_year()<'2022': return

    result:CheckRowData = api.context.row_result
    row_data = api.context.row_data
    Pravoobl = row_data.get(PravooblAttr)
    FormSobst = row_data.get(FormSobstAttr)

    if FormSobst == '1' and not check_okpo(Pravoobl):
        if Pravoobl != 'Российская Федерация':
            result.add_error(PravooblAttr, 'Должен быть код ОКПО или "Российская Федерация"')
    elif FormSobst == '2':
        if check_okpo(Pravoobl):
            if not check_remark(FormSobstAttr): 
                result.add_error(FormSobstAttr, 'В примечании необходимо указать наименование субъекта Российской Федерации')
        elif not is_in_SubRF(Pravoobl):
            result.add_error(PravooblAttr, 'Должен быть код ОКПО или Субьект РФ из списка')
    elif FormSobst == '3':
        if check_okpo(Pravoobl):
            if not check_remark(FormSobstAttr):
                result.add_error(FormSobstAttr, 'В примечании необходимо указать наименование муниципального образования')
    elif FormSobst == '4' and not check_okpo(Pravoobl):
        result.add_error(PravooblAttr, 'Должен быть код ОКПО')
    elif FormSobst == '5' and not is_in_OKSM(Pravoobl):
        result.add_error(PravooblAttr, 'Значение должно быть из справочника ОКСМ')
    elif FormSobst == '6':
        if not is_in_OKSM(Pravoobl):
            result.add_error(PravooblAttr, 'Значение должно быть из справочника ОКСМ')
        if not check_remark(PravooblAttr):
            result.add_error(PravooblAttr, f'Необходимо указать в примечании наименование и адрес правообладателя (собственника или обладателя иного вещного права) на {itemName}')
    elif FormSobst == '9':
        if not check_remark(PravooblAttr):
            result.add_error(PravooblAttr, f'Необходимо указать в примечании наименование и адрес правообладателя (собственника или обладателя иного вещного права) на {itemName}')
        #
        #
def check_attrs_PrName(PrNameAttr:str, UktPrTypAttr:str):
    result:CheckRowData = api.context.row_result
    row_data = api.context.row_data
    PrName = row_data.get(PrNameAttr)
    UktPrTyp = row_data.get(UktPrTypAttr)

    if not voc_check('PrName', PrName):
        result.add_error(PrNameAttr, 'Значение должно быть из справочника "Приборы (установки), УКТ или иные упаковки"')
        #
        #
def check_attrs_PH_Name(PH_NameAttr:str, PH_CodAttr:str):
    if form_start_year() < '2022': return 

    result:CheckRowData = api.context.row_result
    row_data = api.context.row_data
    PH_Name = row_data.get(PH_NameAttr)
    PH_Cod = row_data.get(PH_CodAttr)

    vocRec = voc_read('PH_Name', PH_Name)

    if vocRec.is_empty():
        result.add_error(PH_NameAttr, 'Значение должно быть из справочника "Пункты хранения"')
    elif form_start_year()>='2014':
        if not vocRec.get('7'):
            result.add_error(PH_NameAttr, f'Для пункта хранения {PH_Name} не определен "Код пункта хранения" в справочнике "Пункты хранения" {vocRec}')
        elif vocRec['7'] != PH_Cod:
            result.add_error(PH_NameAttr, f'Код пункта хранения "{PH_Cod}" не соответствут записи в справочнике "Пункты хранения" - "{vocRec["7"]}"')
            #result.add_error(PH_NameAttr, f'Данные пункта хранения: {vocRec.to_dict()}')
        #
        #
def check_attr_BCod(bCodAttr:str, opCodAttr:str=None, OpCod=None):               # Проверка Статуса РАО
    if form_start_year()<'2022': return

    result:CheckRowData = api.context.row_result
    row_data = api.context.row_data
    BCod = row_data.get(bCodAttr)
    
    if not OpCod:
        OpCod = row_data.get(opCodAttr)
        if not OpCod: return

    if OpCod in '11, 12, 13, 14, 28, 38, 41':
        okpo = org_OKPO()
        if BCod != okpo:
            result.add_error(bCodAttr, f'Для операции с кодом "{OpCod}" должен быть ОКПО отчитывающейся организации ({okpo})')
    elif OpCod in '76':
        if BCod != '6':
            result.add_error(bCodAttr, 'Для кода операции 76 значение "Статус РАО" должно быть 6')
    else:
        if not check_okpo(BCod) and not voc_check('BCod', BCod):
            result.add_error(bCodAttr, 'Значение должно быть из справочника "Статус РАО"')

    if BCod == '3' and not check_remark(bCodAttr):
        result.add_error(bCodAttr, 'В примечании к ячейке необходимо указать наименование субъекта Российской Федерации')

    if BCod == '4' and not check_remark(bCodAttr):
        result.add_error(bCodAttr, 'В примечании необходимо указать наименование муниципалитета')
        #result.add_error(bCodAttr, f'Запись {row_data.to_dict()}')

    if BCod == '9' and not check_remark(bCodAttr):
        result.add_error(bCodAttr, 'Необходимо примечание')
        #
        #
def check_attr_UpCod(upCodAttr:str, raoCodAttr:str, opCodAttr:str):
    result:CheckRowData = api.context.row_result
    row_data = api.context.row_data
    UpCod = row_data.get(upCodAttr)
    RaoCod = row_data.get(raoCodAttr)
    OpCod = row_data.get(opCodAttr)

    if UpCod in '52, 72, 74':
        if OpCod not in '49, 59':
            result.add_error('UpCod', 'Для выбранных кодов способа сортировки, необходимо выбрать код операции 49 или 59')
    elif OpCod in '01, 10, 11, 12, 13, 14, 16, 18, 21, 22, 25, 26, 27, 28, 29, 31, 32, 35, 36, 37, 38, 39, 42, 43, 48, 51, 52, 63, 64, 68, 71, 72, 73, 74, 75, 76,  97, 98, 99' and UpCod!='-':
        result.add_error('UpCod', 'Для указанной операции должен быть символ "-"')
    elif OpCod == '44' and UpCod == '22':
        if len(RaoCod)>=11 and RaoCod[11-1] != '1':
            result.add_error('UpCod', 'Проверьте значение 11 символа кода РАО')
    elif OpCod in '44, 45, 56, 57' or (OpCod in '49, 59' and UpCod != '-'):
        if not voc_check('UpCod', UpCod):
            result.add_error('UpCod', 'Значение должно быть из справочника установок переработки/сортировки')
        #
        #
def check_attr_Nuclides(NuclidesAttr:str):
    result:CheckRowData = api.context.row_result
    data = api.context.row_data.get(NuclidesAttr)
    if data=='-': return

    if not check_separated(data):
        result.add_error(NuclidesAttr, 'Названия радионуклидова должны быть разделены точкой с запятой')

    values = data.split(';')
    for value in values:
        if not is_nuclide(value.strip()):
            result.add_error(NuclidesAttr, f'Название "{value}" отсутствует в справочнике радионуклидов')
        #
        #
def check_attr_Subsid(subsidAttr:str):
    check_attr_num(subsidAttr, minValue=0, maxValue=100, allowNone=True)
        #
        #
def check_attr_FCP(fcpAttr:str):
    check_attr_num(fcpAttr, allowNone=True)
        #
        #
def calcNuclideGroup(nlist):
    '''
    Порядок групп: ABU
    '''
    return { 'U':1, 'A':2, 'AU':3, 'B':4, 'AB':5, 'ABU':6, 'BU':6,  '':0}[nlist] 
        #
        #
def calcNS(row_data, nuclidesAttr):                                      # Расчет показателей радионуклидов для проверки кода РАО
    result:CheckRowData = api.context.row_result
    nuclides = row_data.get(nuclidesAttr)
    if nuclides=='-':
        is_err = True
        nuclides = []
    else:
        is_err = False
        nuclides = nuclides.split(';')
        nuclides = [nuclide.strip() for nuclide in nuclides]

    groups = set()
    period_max = {'A':0, 'B':0, 'U':0}

    for nuclide in nuclides:
        if not nuclide.strip(): continue

        try:
            rec = voc_read('radionuclide_voc1', nuclide.lower())
            group = {'а':'A', 'б':'B', 'Трансурановый':'U', 'Тритий':'T'}[rec.Kod_gruppy]
            if group: groups.add(group)

            period = rec.Period_p_r * {'лет':365, 'мин':1/24/60, 'сек':1/24/60/60, 'сут':1, 'час':1/24}[rec.Edinica_izmer_p_r]

            if period > period_max.get(group, period): period_max[group] = period

        except Exception as e:
            api.log.exception(e)
            result.add_error(nuclidesAttr, f'Ошибка определения параметров радионуклида {nuclide}')
            is_err = True

    nlist = ''
    if 'A' in groups: nlist += 'A'
    if 'B' in groups or 'T' in groups: nlist += 'B'
    if 'U' in groups: nlist += 'U'

    period_cod = 1 if max(period_max.values()) > 31*365 else 2

    if not is_err:
        return {
            'nucl_gr'        : calcNuclideGroup(nlist), 
            'period_gr'      : period_cod, 
            'period_max_gr'  : period_max,
            'period_max_all' : max(period_max.values()), 
            'groups'         : groups
        }
    else:
        return {
            'nucl_gr'   : 0, 
            'period_gr' : 0, 
            'period_max_gr'  : 0,
            'period_max_all' : 0, 
            'groups'         : ''
        }
        #
        #
def calcRaoCategory(codRao, mass, actTr, actA, actBG, actU):    # Определение категории РАО
    mass  = str2float(mass)

    if not mass: return 0

    actTr = str2float(actTr);   actTr_U = actTr/(mass*1E6)
    actA  = str2float(actA);    actA_U  = actA /(mass*1E6)
    actBG = str2float(actBG);   actBG_U = actBG/(mass*1E6)
    actU  = str2float(actU);    actU_U  = actU /(mass*1E6)

    if codRao[0] == '2':
        # Расчет категорий для ТРО
        if   actTr_U < 1E7:  catTr = 0
        elif actTr_U < 1E8:  catTr = 1
        elif actTr_U < 1E11: catTr = 2
        else: catTr = 3 

        if   actBG_U < 1E3: catBG = 0
        elif actBG_U < 1E4: catBG = 1
        elif actBG_U < 1E7: catBG = 2
        else: catBG = 3

        if   actA_U < 1E2: catA = 0
        elif actA_U < 1E3: catA = 1
        elif actA_U < 1E6: catA = 2
        else: catA = 3

        if   actU_U < 1E1: catU = 0
        elif actU_U < 1E2: catU = 1
        elif actU_U < 1E5: catU = 2
        else: catU = 3

    
    elif codRao[0] == '1':
        # Расчет категорий для ЖРО
        if   actTr_U < 1E4:  catTr = 1
        elif actTr_U < 1E8:  catTr = 2
        else: catTr = 3 

        if   actBG_U < 1E3: catBG = 1
        elif actBG_U < 1E7: catBG = 2
        else: catBG = 3

        if   actA_U < 1E2: catA = 1
        elif actA_U < 1E6: catA = 2
        else: catA = 3

        if   actU_U < 1E1: catU = 1
        elif actU_U < 1E5: catU = 2
        else: catU = 3

    else:
        catTr = catBG = catA = catU = 0

    return max(catTr, catBG, catA, catU)
        #
        #

def check_attrs_activity(codRaoAttr, nuclidesAttr, actT_attr, actB_attr, actA_attr, actU_attr, gAct, gNucl):
    result:CheckRowData = api.context.row_result
    row_data = api.context.row_data
    codRao = row_data.get(codRaoAttr)

    # Радионуклиды
    if len(codRao)>3 and codRao[3-1]=='0' and codRao!='-':
        nuclideGroups=''
    else:
        check_attr_Nuclides(NuclidesAttr=nuclidesAttr)
        nuclide_desc = calcNS(row_data, nuclidesAttr=nuclidesAttr)
        nuclideGroups = nuclide_desc['groups']        

    # Активности
    check_attr_num(actB_attr, allowNone='B' not in nuclideGroups)
    check_attr_num(actA_attr, allowNone='A' not in nuclideGroups)

    if form_start_year()>='2022':
        check_attr_num(actT_attr, allowNone='T' not in nuclideGroups)
        check_attr_num(actU_attr, allowNone='U' not in nuclideGroups)

    if f'{row_data[actT_attr]}{row_data[actB_attr]}{row_data[actA_attr]}{row_data[actU_attr]}' =='----':
        msg = f'Должна быть заполнена хотя бы одна из граф {gAct} в зависимости от радионуклидов, указанных в графе {gNucl}'
        result.add_error(actB_attr, msg)
        result.add_error(actA_attr, msg)

def check_rao_criteria(codRaoAttr, massAttr, actT_attr, actB_attr, actA_attr, opCodAttr):
    result:CheckRowData = api.context.row_result
    row_data = api.context.row_data
    codRao = row_data.get(codRaoAttr)
    if not codRao: return

    mass = str2float(row_data.get(massAttr)) 
    if mass==0: return

    actA = str2float(row_data.get(actA_attr))
    actB = str2float(row_data.get(actB_attr)) + str2float(row_data.get(actT_attr))

    actA_u = actA / (mass * 1E6)
    actB_u = actB / (mass * 1E6)

    agr = codRao[0]

    if agr == '1':
        limit_A = 0.05
        limit_B = 0.5
    elif agr == '2':
        limit_A = 1
        limit_B = 100
    else:
        return

    if actA_u < limit_A and actB_u < limit_B and not check_remark(opCodAttr):
        result.add_error(opCodAttr, 
            'Не соответствует критериям отнесения к РАО. Необходимо указать примечание для операции '
            f'(уд.акт. альфа = {actA_u} < {limit_A} Бк/г, уд.акт. бета = {actB_u:.1e} < {limit_B} Бк/г)'
        )


# Проверка Кода РАО 
def check_attr_RaoCod(opCodAttr, codRaoAttr, massAttr, nuclidesAttr, upCodAttr, ActTr, ActA, ActBG, ActU, PH_Cod, BCod):  
    if form_start_year() < '2014': return

    result:CheckRowData = api.context.row_result
    form_data = api.context.form_data
    row_data = api.context.row_data
    mass = str2float(row_data.get(massAttr))
    nuclides = row_data.get(nuclidesAttr)
    UpCod = row_data.get(upCodAttr)
    OpCod = row_data.get(opCodAttr)

    rc = row_data.get(codRaoAttr)

    if not re.findall('^\d{11}$', rc):
        result.add_error(codRaoAttr, 'Код РАО должен состоять из 11 цифр')
        return 

    rcAgr   = int(rc[0])     # Агрегатное состояние
    rcCat   = int(rc[2-1])   # Категория РАО
    rcNucl  = int(rc[3-1])   # Радионуклидный состав
    rcNM    = int(rc[4-1])   # Содержание ядерных материалов
    rcPP    = int(rc[5-1])   # Период полураспада
    rcPD    = int(rc[6-1])   # Период потенциальной опасности
    rcPT    = int(rc[7-1])   # Способ переработки
    rcClass = int(rc[8-1])   # Класс РАО
    rcType  = int(rc[8:10])  # Код типа РАО
    rcFD    = int(rc[11-1])  # Горючесть

    is_form_1_78 =  sguk_forms.form_type(form_data.form_no) in (sguk_forms.t_form_1_7, sguk_forms.t_form_1_8)

    nuclide_desc = calcNS(row_data, nuclidesAttr)

    # 1-й символ: Агрегатное состояние
    if not any(
            rcAgr==1 and in_ranges(rcType, (11, 39), [99]),
            rcAgr==2 and in_ranges(rcType, (41, 99)),
            rcAgr==2 and in_ranges(rcType, (11, 39)) and rcPT!=0,
            rcAgr==3 and rcType==1
        ): 
        result.add_error(codRaoAttr, f'символ[1]="{rcAgr}": Агрегатное состояние не соответствует типу выбранных РАО (символы 9-10 кода РАО)')

    # 2-й символ: Категория РАО
    if rcCat==4:
        if rcType not in [81, 82, 85, 86, 87, 88, 89]:
            result.add_error(codRaoAttr, f'символ[2]="{rcCat}": Значение 2-ого символ кода РАО 4 используется только для отработавших ЗРИ (символы[9-10] должны быть из списка 81, 82, 85, 86, 87, 88, 89)')

    elif rcCat==9:
        if not check_remark(codRaoAttr):
            result.add_error(codRaoAttr, 'Необходимо дать пояснение для 2-ого символа кода РАО')

    elif (rcAgr==1 and rcCat not in (1,2,3,9)) or (rcAgr==2 and rcCat not in (0,1,2,3,9)):
        result.add_error(codRaoAttr, f'символ[2]="{rcCat}": Неправильно указана категория РАО. Должна быть {"1,2,3,9" if rcAgr==1 else "0,1,2,3,9"}')
    else:
        calcCat = calcRaoCategory(rc, mass, ActTr, ActA, ActBG, ActU)
        if rcCat != calcCat:
            result.add_error(codRaoAttr, f'символ[2]="{rcCat}": Неправильно указана категория РАО. Должна быть {calcCat}')

    # 3-й символ: Радионуклидный состав
    if not is_form_1_78 and (rcNucl!=0 or nuclides!=''):
        try:
            ns = nuclide_desc['nucl_gr'] 
            if ns != rcNucl:
                result.add_error(codRaoAttr, f'Радионуклиды, указанные в графе 9 не соответствуют 3-му символу кода РАО. Должна быть {ns}')
        except Exception as e:
            result.add_error(codRaoAttr, f'Ошибка определения группы для радионуклида {e.args}')

    # 4-й символ: Содержание ядерных материалов
    ym_nuclides = 'плутоний уран-233 уран-235 уран-238 нептуний-237 америций-241 америций-243 калифорний-252 торий литий-6 тритий'
    if not is_form_1_78 and rcNM==2 and not contain_word(nuclides.lower(), ym_nuclides):
        result.add_error(codRaoAttr, f'символ[4]="{rcNM}": Не указаны радионуклиды, которые могут быть отнесены к ЯМ ({ym_nuclides})')

    # 5-й символ: Период полураспада (1-более 31 год, 2-менее)
    if not is_form_1_78 and rcPP != nuclide_desc['period_gr'] and nuclide_desc['period_gr']>0:
        result.add_error(codRaoAttr, f'5-й символ должен быть {nuclide_desc["period_gr"]} период полураспада {int(nuclide_desc["period_max_all"]/365)} год(а)')

    # 6-й символ: Период потенциальной опасности
    if rcAgr==3:
        pass
    elif rcAgr in (1, 2) and rcNucl==0:
        if not check_remark(codRaoAttr):
            result.add_error(codRaoAttr, f'символ[6]="{rcNucl}": Укажите причины невозможности определения периода потенциальной опасности')
    else:
        pass #TODO

    # 7-й символ: Способ переработки
    if OpCod == 56:
        if not any( 
                rcPT==0 and in_ranges(UpCod, (11,17),19,(21,24),29,(51,55),[61,72,73,74,99]),
                rcPT==1 and in_ranges(UpCod, [31,32,39]),
                rcPT==2 and UpCod=='41',
                rcPT==3 and UpCod in (42,71),
                rcPT==4 and UpCod=='43',
                rcPT==9 and UpCod=='49'
            ):
            result.add_error(codRaoAttr, '7-ой символ кода РАО не соответствует коду переработки/сортировки, указанному в графе 22')

    if rcAgr==1 and rcPT!=0:
        result.add_error(codRaoAttr, 'Для жидких РАО 7-й символ кода РАО не может быть равным 1, 2, 3, 4, 9')

    # 8-й символ: Класс РАО
    if form_start_year()>='2022' and rcClass not in (0,7,9) and form_data.form_no not in 'form_1_7_2022 form_1_8_2022':
        result.add_error(codRaoAttr, f'символ[8]="{rcClass}": Сведения о кондиционированных отходах необходимо представлять в формах 1.7 и 1.8 (допустимы значения 0,7,9)')

    if rcClass==7 and PH_Cod[0] in '3 5 6' and BCod!='1':
        result.add_error(codRaoAttr, f'символ[8]="{rcClass}": Особые РАО могут быть размещены только в ПРОРАО либо ПКОРАО (допустимое значение "Статус РАО"=1)')

    if rcClass==9 and PH_Cod[0]=='21' and BCod!='1':
        result.add_error(codRaoAttr, f'символ[8]="{rcClass}": Идентификатор 9 используется только для тех РАО, по которым решение об отнесении к особым или удаляемым отложено в ходе проведения первичной регистрации (допустимое значение "Статус РАО"=1)')

    # 9-10 символы: Код типа РАО
    if form_start_year()>='2022' and rcType == 94:
        result.add_error(codRaoAttr, f'символы[9-10]="{rcType}": Сведения о РАО, подготовленных для передачи национальному оператору предоставляются в форме 1.8')

    if int(rcType) in (19, 29,39, 59, 69, 78, 79, 89, 99) and not check_remark(codRaoAttr):
        result.add_error(codRaoAttr, f'символы[9-10]="{rcType}": Необходимо заполнить примечание к коду типа РАО')



class Form_1_1_2022_check(ReportFormCheck):      # ЗРИ (ZRI)
    def check_form(self):
        super().check_form()

        self.cache = get_form_cache().get('acc_o1')       # Получаем кэш с ожидаемым отображением формы в регистрах

        msg = check_form_period()
        if msg:
            return [{'row':'', 'col_name':'', 'text':msg}]
        else:
            return []
            #
            #
    def add_notify(self, result:CheckRowData, row_data:Form_1_1_table):

        notify_list = {
            '11' : 'Для 1, 2, 3 категории ЗРИ: «К отчету должны быть приложены скан-копии паспортов на ЗРИ”',
            '18' : 'Необходимо представить копию документа, отражающего результаты проведенных измерений',
            '66' : 'Необходимо приложить к отчёту скан-копию документа о продлении назначенного срока службы',
            '68' : 'Необходимо представить копию документа, отражающего результаты проведенных измерений',
            '71' : 'Необходимо представить копию документа (протокола, акта, прочего документа), характеризующего обстоятельства установления факта утери',
            '72' : 'Необходимо представить копию документа (протокола, акта, прочего документа), характеризующего обстоятельства установления факта утери контроля над учетной единицей',
            '73' : 'Необходимо представить копию документа (протокола, акта, прочего документа), характеризующего обстоятельства обнаружения учетной единицы',
            '74' : 'Необходимо представить копию документа (протокола, акта, прочего документа), характеризующего обстоятельства изъятия РВ или РАО',
            '75' : 'Необходимо представить копию документа, характеризующего обстоятельства обнаружения учетной единицы',
            '76' : 'Необходимо представить копию документа, характеризующего обстоятельства изъятия РАО.',
            '85' : 'Для 1, 2, 3 категории ЗРИ: «К отчету должны быть приложены скан-копии паспортов (сертификатов) на ЗРИ”',
            '86' : 'Для 1, 2, 3 категории ЗРИ: «К отчету должны быть приложены скан-копии паспортов (сертификатов) на ЗРИ”',
        }

        OpCod = row_data.OpCod

        if OpCod not in notify_list: return
        if OpCod in '11 85 86' and row_data.get('Kateg') not in '1 2 3': return

        DocVidRec = voc_read('DocVid', row_data.DocVid)
        if DocVidRec and 'DocName' in DocVidRec:
            DocVidStr = DocVidRec.DocName
        else:
            DocVidStr = f'код вида документа"{row_data.DocVid}"'
        
        result.add_notify(f'{notify_list[OpCod]}\n{DocVidStr} №{row_data.DocN} от {m_utils.date_str_format(row_data.DocDate)}')
            #
            #
    def check_row(self):
        ReportFormCheck.check_row(self)

        result:CheckRowData = api.context.row_result
        form_data:Record = api.context.form_data
        row_data:Form_1_1_table = api.context.row_data

        if not check_attrs_not_empty(  
            'OpCod OpDate '
            'PaspN Typ Nuclid Numb Sht Activn IzgotOKPO IzgotDate Kateg Nss '
            'FormSobst Pravoobl '
            'DocVid DocN DocDate '
            'OkpoPIP OkpoPrv '
            'PrName UktPrTyp UktPrN'): 
            return
        
        item_id = sguk_setup_regs.calc_item_id(row_data, form_data, silent=True)
        
        check_attr_date('OpDate', blank_str=None)
        check_attr_date('IzgotDate')
        check_attr_date('DocDate')

    # п.2 - OpCod
        if row_data.OpCod not in '10, 11, 12, 15, 17, 18, 21, 22, 25, 27, 28, 29, 31, 32, 35, 37, 38, 39, 41, 42, 43, 46, 53, 54, 58, 61, 62, 63, 64, 65, 66, 67, 68, 71, 72, 73, 74, 75, 81, 82, 83, 84, 85, 86, 87, 88, 97, 98, 99':
            result.add_error('OpCod', 'Неверный код операции')
        else:
            if row_data.OpCod == '10': 
                if not check_snk(item_id, api.regs.acc_o1, self.cache):
                    result.add_error('OpCod', 'Сведения, представленные в инвентаризации не соответствуют СНК')

            if row_data.OpCod in '12 42':
                if not check_contain_any(row_data.Nuclid.lower(), ['плутоний', 'уран-233', 'уран-235', 'нептуний-237', 'америций-241', 'америций-243', 'калифорний-252', 'торий', 'литий-6', 'тритий']):
                    result.add_error('OpCod', 'В графе 6 не представлены  сведения о  радионуклидах, которые могут быть отнесены к ЯМ. Проверьте правильность выбранного кода операции')
            
            if row_data.OpCod in '29 39 97 98 99':
                if not check_remark('OpCod'):
                    result.add_error('OpCod', 'Необходимо дать пояснение об осуществленной операции')

            if row_data.OpCod in '84 85':
                if not is_in_OKSM(row_data.IzgotOKPO):
                    result.add_error('OpCod', 'Код используется для предоставления сведений о ЗРИ, произведенных за пределами Российской Федерации')

            if row_data.OpCod == '37':
                if not check_prev_oper(item_id, '27', 'acc_o1', self.cache, OpCodAttr='OpCod'):
                    result.add_error('OpCod', 'В отчетах не найдена строка об осуществлении передачи учетной единицы. Проверьте правильность выбранного кода операции')
                
            if row_data.OpCod == '41':
                if not check_related_oper(sguk_forms.t_form_1_5, {
                    'OpCod'     : row_data.OpCod,
                    'OpDate'    : row_data.OpDate,
                    'PaspN'     : row_data.PaspN,
                    'Typ'       : row_data.Typ,
                    'IzgotDate' : row_data.IzgotDate
                    }):
                    result.add_error('OpCod', 'Заполните форму 1.5')

            if row_data.OpCod == '54':
                if not check_okpo(row_data.OkpoPIP) or row_data.OkpoPIP == org_OKPO():
                    result.add_error('OpCod', 'В графе 19 необходимо указать ОКПО подрядной организации')

            if row_data.OpCod == '58':
                filter = {
                        'OpCod'     : '46', 
                        'Nuclid'    : row_data.Nuclid , 
                        'Typ'       : row_data.Typ,
                        'IzgotOKPO' : row_data.IzgotOKPO,
                        'IzgotDate' : row_data.IzgotDate
                    }
                if self.cache.filter(filter).is_empty() and api.regs.acc_o1.select(filters=filter).is_empty():
                    result.add_error('OpCod', 'В отчетах не найдены сведения о снятии с учета учетной единицы для разукомплектования. Проверьте правильность выбранного кода операции')

            if row_data.OpCod == '62':
                if not check_prev_oper(item_id, '61', 'acc_o1', self.cache, OpCodAttr='OpCod'):
                    result.add_error('OpCod', 'В отчетах не найдены сведения о вывозе учетной единицы. Проверьте правильность выбранного кода операции')

            if row_data.OpCod in '81 88':
                if not (check_okpo(row_data.IzgotOKPO) and row_data.IzgotOKPO.isnumeric()):
                    result.add_error('OpCod', 'Код используется для предоставления сведений о ЗРИ, произведенных в Российской Федерации')

            if row_data.OpCod in '21 22 25 27 28 29 41 42 43 46 53 54 65 66 67 68 71 72 81 82 83 84 88 98':
                if not check_snk(item_id, api.regs.acc_o1, self.cache):
                    result.add_error('OpCod', 'Учетной единицы с такими параметрами нет в организации. Проверьте правильность указываемых сведений для ЗРИ')

    # п.3 - OpDate
        if row_data.OpCod == '10':
            check_date_between('DocDate', 'form_period_start', 'form_period_end', 'Дата документа не входит в отчетный период')
        else:
            check_date_between('OpDate', 'form_period_start', 'form_period_end', 'Дата операции не входит в отчетный период')

    # п.4 - PaspN
        if not row_data.PaspN:
            result.add_error('PaspN', 'Формат ввода данных не соответствует приказу')

    # 6: Nuclid
        check_attr_Nuclides(NuclidesAttr='Nuclid')

    # 7: Numb (Номер ЗРИ)
        if not check_separated(row_data.Numb) and form_start_year()>='2022':
            result.add_error('Numb', 'Значения должны быть разделены ";"')

    # п.8 - Sht
        check_attr_num('Sht', allowNone=True)

    # п.9 - Activn
        check_attr_num('Activn', allowNone=True, maxValue=10E+20, msg='Проверьте правильность предоставления сведений по суммарной активности')

    # п.10 - IzgotOKPO
        check_attr_IzgotOKPO(IzgotOkpoAttr='IzgotOKPO', opCodAttr='OpCod', itemName='ЗРИ')

    # п.11 - IzgotDate
        check_date_between('IzgotDate', None, 'OpDate', 'Дата изготовления не может быть позже даты операции')

    # п.12 - Kateg
        if form_start_year()>='2014' and (str(row_data.Kateg)<'1' or str(row_data.Kateg)>'5'):
            result.add_error('Kateg', 'Необходимо указать категорию ЗРИ от 1 до 5')

    # п.13 - Nss
        check_attr_Nss()            

    # п.14 - FormSobst
        if form_start_year() >= '2022':
            check_attr_FormSobst(row_data, formSobstAttr='FormSobst')

    # п.15 - Pravoobl
        if form_start_year() >= '2022':
            check_attr_Pravobl(itemName='ЗРИ')

    # п.16 - DocVid
        check_attr_DocVid('DocVid')

    # п.18 - DocDate
        check_attr_DocDate()

    # п.19 - OkpoPIP
        check_attr_OkpoPIP( \
            selfOkpoCodes='10, 11, 12, 15, 17,18, 41, 42, 43, 46, 53, 58, 61, 62, 65, 67, 68, 71, 72, 72, 73, 74, 75, 76', \
            oksmCodes='81, 82, 83, 84, 85, 86, 87, 88', \
            OkpoPipAttr='OkpoPIP', OpCodAttr='OpCod')
                
    # п.20 - OkpoPrv
        check_attr_OkpoPrv( \
            noneOkpoCodes='10, 11,12, 15, 17, 18, 41, 42, 43, 46, 53, 54, 58, 65, 66, 67, 68, 71, 72, 73, 74, 75, 76, 97, 98', \
            OkpoPrvAttr='OkpoPrv', OpCodAttr='OpCod')

    # п.21 - PrName, UktPrTyp
        check_attrs_PrName(PrNameAttr='PrName', UktPrTypAttr='UktPrTyp')


class Form_1_2_2022_check(ReportFormCheck):      # ИОУ (IOU)
    def check_form(self):
        super().check_form()

        self.cache = get_form_cache().get('acc_o2')

        msg = check_form_period()
        if msg:
            return [{'row':'', 'col_name':'', 'text':msg}]
        else:
            return []
            #
            #
    def add_notify(self, result:CheckRowData, row_data:Form_1_2_table):

        notify_list = {
            '66' : 'Необходимо приложить к отчёту скан-копию документа о продлении назначенного срока службы',
            '68' : 'Необходимо представить копию документа, отражающего результаты проведенных измерений',
            '71' : 'Необходимо представить копию документа (протокола, акта, прочего документа), характеризующего обстоятельства установления факта утери',
            '72' : 'Необходимо представить копию документа (протокола, акта, прочего документа), характеризующего обстоятельства установления факта утери контроля над учетной единицей',
            '73' : 'Необходимо представить копию документа (протокола, акта, прочего документа), характеризующего обстоятельства обнаружения учетной единицы',
            '74' : 'Необходимо представить копию документа (протокола, акта, прочего документа), характеризующего обстоятельства изъятия РВ или РАО',
            '75' : 'Необходимо представить копию документа, характеризующего обстоятельства обнаружения учетной единицы',
        }

        OpCod = row_data.OpCod
        if OpCod not in notify_list: return

        DocVidRec = voc_read('DocVid', row_data.DocVid)
        if DocVidRec and 'DocName' in DocVidRec:
            DocVidStr = DocVidRec.DocName
        else:
            DocVidStr = f'код вида документа"{row_data.DocVid}"'
        
        result.add_notify(f'{notify_list[OpCod]}\n{DocVidStr} №{row_data.DocN} от {m_utils.date_str_format(row_data.DocDate)}')
            #
            #
    def check_row(self):
        ReportFormCheck.check_row(self)

        result:CheckRowData = api.context.row_result
        form_data:Record = api.context.form_data
        row_data:Form_1_2_table = api.context.row_data

        if not check_attrs_not_empty(  
            'OpCod OpDate '
            'PaspN Typ Numb Kg IzgotOKPO IzgotDate Nss '
            'FormSobst Pravoobl '
            'DocVid DocN DocDate '
            'OkpoPIP OkpoPrv '
            'PrName UktPrTyp UktPrN'):
            return

        item_id = sguk_setup_regs.calc_item_id(row_data, form_data, silent=True)

        check_attr_date('OpDate', blank_str=None)
        check_attr_date('IzgotDate')
        check_attr_date('DocDate')

    # п.2 - OpCod
        if row_data.OpCod not in '10, 11, 12, 17, 18, 21, 22, 25, 27, 28, 29, 31, 32, 35, 37, 38, 39, 41, 42, 46, 48, 53, 54, 58, 61, 62, 63, 64, 66, 67, 68, 71, 72, 73, 74, 75, 81, 82, 83, 85, 86, 87, 97, 98, 99':
            result.add_error('OpCod', 'Неверный код операции')
        else:
            if row_data.OpCod == '10':
                if not check_snk(item_id, api.regs.acc_o2, self.cache):
                    result.add_error('OpCod', 'Сведения, представленные в инвентаризации не соответствуют СНК')

            if row_data.OpCod in '29 39 97 98 99':
                if not check_remark('OpCod'):
                    result.add_error('OpCod', 'Необходимо дать пояснение об осуществленной операции')

            if row_data.OpCod in '21 22 25 27 28 29 41 46 53 54 66 67 68 71 72 81 82 83 98':
                if not check_snk(item_id, api.regs.acc_o2, self.cache):
                    result.add_error('OpCod', 'Учетной единицы с такими параметрами нет в организации. Проверьте правильность указываемых сведений для ИОУ')

            if row_data.OpCod == '37':
                if not check_prev_oper(item_id, '27', 'acc_o2', self.cache, OpCodAttr='OpCod'):
                    result.add_error('OpCod', 'В отчетах не найдена строка об осуществлении передачи учетной единицы. Проверьте правильность выбранного кода операции')

            if row_data.OpCod == '41':
                if not check_related_oper(sguk_forms.t_form_1_6, {'OpCod':row_data.OpCod, 'OpDate':row_data.OpDate, 'DocN':row_data.DocN, 'DocDate':row_data.DocDate}):
                    result.add_error('OpCod', 'Заполните форму 1.6')

            if row_data.OpCod == '54':
                if not check_okpo(row_data.OkpoPIP) or row_data.OkpoPIP == org_OKPO():
                    result.add_error('OpCod', 'В графе 16 необходимо указать ОКПО подрядной организации')

            if row_data.OpCod == '58':
                if not check_prev_oper(item_id, '46', 'acc_o2', self.cache, OpCodAttr='OpCod'):
                    result.add_error('OpCod', 'В отчетах не найдены сведения о снятии с учета учетной единицы для разукомплектования. Проверьте правильность выбранного кода операции')

            if row_data.OpCod == '62':
                if not check_prev_oper(item_id, '61', 'acc_o2', self.cache, OpCodAttr='OpCod'):
                    result.add_error('OpCod', 'В отчетах не найдены сведения о вывозе учетной единицы. Проверьте правильность выбранного кода операции')

            if row_data.OpCod == '81':
                if not check_okpo(row_data.IzgotOKPO) or not check_okpo(row_data.Pravoobl):
                    result.add_error('OpCod', 'Код используется для предоставления сведений о ИОУ, произведенных в Российской Федерации. В графах 8 и 12 должны быть коды ОКПО')

            if row_data.OpCod == '85':
                if not is_in_OKSM(row_data.IzgotOKPO) or not is_in_OKSM(row_data.Pravoobl):
                    result.add_error('OpCod', 'Код используется для предоставления сведений о ИОУ, произведенных за пределами Российской Федерации')
        
    # п.3 - OpDate
        if row_data.OpCod == '10':
            check_date_between('DocDate', 'form_period_start', 'form_period_end', 'Дата документа не входит в отчетный период')
        else:
            check_date_between('OpDate', 'form_period_start', 'form_period_end', 'Дата операции не входит в отчетный период')

    # п.7 - Kg
        check_attr_num('Kg', msg='Необходимо заполнить сведения о массе ИОУ')

    # п.8 - IzgotOKPO
        check_attr_IzgotOKPO(IzgotOkpoAttr='IzgotOKPO', opCodAttr='OpCod', itemName='ИОУ')

    # п.9 - IzgotDate
        check_date_between('IzgotDate', None, 'OpDate', 'Дата изготовления не может быть позже даты операции')

    # п.10 - Nss
        check_attr_Nss(item_name='ИОУ')

    # п.11 - FormSobst
        if form_start_year() >= '2022':
            check_attr_FormSobst(row_data, formSobstAttr='FormSobst')

    # п.12 - Pravoobl
        if form_start_year() >= '2022':
            check_attr_Pravobl(itemName='ИОУ')

    # п.13 - DocVid
        check_attr_DocVid('DocVid')

    # п.15 - DocDate
        check_attr_DocDate()

    # п.16 - OkpoPIP
        check_attr_OkpoPIP(
            selfOkpoCodes='10, 11, 12, 15, 17,18, 41, 42, 46, 53, 58, 61, 62, 67, 68, 71, 72, 72, 73, 74, 75, 76', \
            oksmCodes='81, 82, 83, 84, 85, 86, 87, 88', \
            OkpoPipAttr='OkpoPIP', OpCodAttr='OpCod')

    # п.17 - OkpoPrv
        check_attr_OkpoPrv(
            noneOkpoCodes='10, 11,12, 17, 18, 41, 42, 46, 53, 54, 58, 65, 66, 67, 68, 71, 72, 73, 74, 75, 76, 97, 98', \
            OkpoPrvAttr='OkpoPrv',OpCodAttr='OpCod')

    # п.18 - PrName, UktPrTyp
        check_attrs_PrName(PrNameAttr='PrName', UktPrTypAttr='UktPrTyp')


class Form_1_3_2022_check(ReportFormCheck):      # ОРИ в виде отдельных изделий (ORI)
    def check_form(self):
        super().check_form()

        self.cache = get_form_cache().get('acc_o3')

        msg = check_form_period()
        if msg:
            return [{'row':'', 'col_name':'', 'text':msg}]
        else:
            return []
            #
            #
    def add_row_aliases(self, row_data):
        #row_data.add_alias('OpCod', 'code')
        #row_data.add_alias('OpDate', 'date')
        
        #row_data.add_alias('DocVid', 'vid')
        #row_data.add_alias('DocN', 'num2')
        #row_data.add_alias('DocDate', 'date2')
        pass
            #
            #
    def check_row(self):
        ReportFormCheck.check_row(self)

        result:CheckRowData = api.context.row_result
        form_data:Record = api.context.form_data
        row_data:Form_1_3_table = api.context.row_data

        if not check_attrs_not_empty(
            'OpCod OpDate '
            'PaspN Typ Nuclid Numb Activn IzgotOKPO IzgotDate Agr '
            'FormSobst Pravoobl '
            'DocVid DocN DocDate '
            'OkpoPIP OkpoPrv '
            'PrName UktPrTyp UktPrN'):
            return

        item_id = sguk_setup_regs.calc_item_id(row_data, form_data, silent=True)

        check_attr_date('OpDate', blank_str=None)
        check_attr_date('IzgotDate')
        check_attr_date('DocDate')

    # п.2 - code
        if row_data.OpCod not in '10, 11, 12, 15, 17, 18, 21, 22, 25, 27, 28, 29, 31, 32, 35, 37, 38, 39, 41, 42, 43, 46, 48, 53, 54, 58, 61, 62, 63, 64, 65, 67, 68, 71, 72, 73, 74, 75, 81, 82, 83, 85, 86, 87, 97, 98, 99':
            if row_data.OpCod in '47' and form_start_year() < '2022':
                pass
            else:
                result.add_error('OpCod', 'Неверный код операции')
        else:
            if row_data.OpCod == '10':
                if not check_snk(item_id, api.regs.acc_o3, self.cache):
                    result.add_error('OpCod', 'Сведения, представленные в инвентаризации не соответствуют СНК')

            if row_data.OpCod in '12 42':
                if not check_contain_any(row_data.Nuclid.lower(), ['плутоний', 'уран-233', 'уран-235', 'нептуний-237', 'америций-241', 'америций-243', 'калифорний-252', 'торий', 'литий-6', 'тритий']):
                    result.add_error('OpCod', 'В графе 6 не представлены  сведения о  радионуклидах, которые могут быть отнесены к ЯМ. Проверьте правильность выбранного кода операции')

            if row_data.OpCod in '29 39 97 98 99':
                if not check_remark('OpCod'):
                    result.add_error('OpCod', 'Необходимо дать пояснение об осуществленной операции')

            if row_data.OpCod in '21 22 25 27 28 29 41 42  43 46 53 54 65 67 68 71 72 81 82 83 98':
                if not check_snk(item_id, api.regs.acc_o3, self.cache):
                    result.add_error('OpCod', 'Учетной единицы с такими параметрами нет в организации. Проверьте правильность указываемых сведений для ОРИ')

            if row_data.OpCod == '37':
                if not check_prev_oper(item_id, '27', 'acc_o3', self.cache, OpCodAttr='OpCod'):
                    result.add_error('OpCod', 'В отчетах не найдена строка об осуществлении передачи учетной единицы. Проверьте правильность выбранного кода операции')

            if row_data.OpCod == '41':
                if not check_related_oper(sguk_forms.t_form_1_6, {'OpCod':row_data.OpCod, 'OpDate':row_data.OpDate, 'RAOCodMax':row_data.Nuclid, 'ActDate':row_data.IzgotDate, 'DocN':row_data.DocN, 'DocDate':row_data.DocDate}):
                    result.add_error('OpCod', 'Заполните форму 1.6')

            if row_data.OpCod == '54':
                if not check_okpo(row_data.OkpoPIP) or row_data.OkpoPIP == org_OKPO():
                    result.add_error('OpCod', 'В графе 16 необходимо указать ОКПО подрядной организации')

            if row_data.OpCod == '58':
                if not check_prev_oper(item_id, '46', 'acc_o3', self.cache, OpCodAttr='OpCod'):
                    result.add_error('OpCod', 'В отчетах не найдены сведения о снятии с учета учетной единицы для разукомплектования. Проверьте правильность выбранного кода операции')

            if row_data.OpCod == '62':
                if not check_prev_oper(item_id, '61', 'acc_o3', self.cache, OpCodAttr='OpCod'):
                    result.add_error('OpCod', 'В отчетах не найдены сведения о вывозе учетной единицы. Проверьте правильность выбранного кода операции')

            if row_data.OpCod == '65':
                if not check_related_oper(sguk_forms.t_form_1_1, {'op_item':item_id}):
                    result.add_error('OpCod', 'Заполните форму 1.1')

            if row_data.OpCod == '81':
                if not check_okpo(row_data.IzgotOKPO):
                    result.add_error('OpCod', f'Код используется для предоставления сведений об ОРИ, произведенных в Российской Федерации.')
                if not check_okpo(row_data.Pravoobl):
                    result.add_error('OpCod', f'Код используется для предоставления сведений об ОРИ, произведенных в Российской Федерации.')

            if row_data.OpCod == '85':
                if not is_in_OKSM(row_data.IzgotOKPO):
                    result.add_error('OpCod', f'Код используется для предоставления сведений об ОРИ, произведенных за пределами Российской Федерации. Графа 9 должна содержать код из справочника ОКСМ. Указано "{row_data.IzgotOKPO}"')
        
    # п.3 - date
        if row_data.OpCod == '10':
            check_date_between('DocDate', 'form_period_start', 'form_period_end', 'Дата документа не входит в отчетный период')
        else:
            check_date_between('OpDate', 'form_period_start', 'form_period_end', 'Дата операции не входит в отчетный период')

    # п.6 - radio (Радионуклиды)
        check_attr_Nuclides(NuclidesAttr='Nuclid')

    # п.7 - num (Номер ОРИ)
        if not check_separated(row_data.Numb):
            result.add_error('Numb', 'Формат ввода данных не соответствует приказу')

    # п.8 - active
        check_attr_num('Activn', allowNone=True, maxValue=10E+20, msg='Проверьте правильность предоставления сведений по суммарной активности')

    # п.9 - okpo (Код ОКПО изготовителя)
        if row_data.OpCod != '85':
            check_attr_IzgotOKPO(IzgotOkpoAttr='IzgotOKPO', opCodAttr='OpCod', itemName='ОРИ')

    # п.10 - date_v
        check_date_between('IzgotDate', None, 'OpDate', 'Дата изготовления не может быть позже даты операции')

    # 11: Arg (агрегатное состояние)
        check_attr_Agr(agrAttr='Agr')

    # п.12 - form_code (Код формы собственности)
        if form_start_year() >= '2022':
            check_attr_FormSobst(row_data, formSobstAttr='FormSobst')

    # п.13 - prav
        if form_start_year() >= '2022':
            check_attr_Pravobl(itemName='ОРИ', PravooblAttr='Pravoobl', FormSobstAttr='FormSobst')

    # п.14 - vid (Вид документа)
        check_attr_DocVid('DocVid')

    # п.15 - num2 (Номер документа)

    # п.16 - date2 (Дата документа)
        check_attr_DocDate(DocDateAttr='DocDate', OpDateAttr='OpDate', OpCodAttr='OpCod')

    # п.17 - post (Код ОКПО поставщика или получателя)
        check_attr_OkpoPIP(
            selfOkpoCodes='10, 11, 12, 15, 17,18, 41, 42, 43, 46, 53, 58, 61, 62, 65, 67, 68, 71, 72, 72, 73, 74, 75, 76', \
            oksmCodes='81, 82, 83, 85, 86, 87', \
            OkpoPipAttr='OkpoPIP', OpCodAttr='OpCod')

    # п.18 - per (Код ОКПО перевозчика)
        check_attr_OkpoPrv(
            noneOkpoCodes='10, 11,12, 15, 17, 18, 41, 42, 43, 46, 53, 54, 58, 65, 66, 67, 68, 71, 72, 73, 74, 75, 76, 97, 98', \
            OkpoPrvAttr='OkpoPrv', OpCodAttr='OpCod')

    # п.19 - naim, type2 (Наименование прибора (установки) УКТ)
        check_attrs_PrName(PrNameAttr='PrName', UktPrTypAttr='UktPrTyp')


class Form_1_4_2022_check(ReportFormCheck):      # ОРИ кроме отдельных изделий (ORP)
    def check_form(self):
        super().check_form()

        self.cache:RecordSet = get_form_cache().get('acc_o4')   # Отображение текущей формы на регистра acc_o4

        msg = check_form_period()
        if msg:
            return [{'row':'', 'col_name':'', 'text':msg}]
        else:
            return []
            #
            #
    def check_row(self):
        ReportFormCheck.check_row(self)

        result:CheckRowData = api.context.row_result
        form_data:Record = api.context.form_data
        row_data:Form_1_4_table = api.context.row_data

        if not check_attrs_not_empty(
            'OpCod OpDate '
            'PaspN IstName Typ Nuclid Activn ActDate Kbm Kg Arg '
            'FormSobst Pravoobl ' 
            'DocVid DocN DocDate '
            'OkpoPIP OkpoPrv '
            'PrName UktPrTyp UktPrN'):
            return

        item_id = sguk_setup_regs.calc_item_id(row_data, form_data, silent=True)

        check_attr_date('OpDate', blank_str=None)
        check_attr_date('ActDate')
        check_attr_date('DocDate')

    # п.2 - OpCod
        if row_data.OpCod not in '10, 11, 12, 15, 17, 18, 21, 22, 25, 27, 28, 29, 31, 32, 35, 37, 38, 39, 41, 42, 43, 46, 47, 48,  53, 54, 58, 61, 62, 63, 64, 67, 68, 71, 72, 73, 74, 75, 81, 82, 83, 85, 86, 87, 97, 98, 99':
            result.add_error('OpCod', 'Неверный код операции')
        else:
            if row_data.OpCod == '10':
                if not check_snk(item_id, api.regs.acc_o4, self.cache):
                    result.add_error('OpCod', 'Сведения, представленные в инвентаризации не соответствуют СНК')

            if row_data.OpCod in '12 42':
                if not check_contain_any(row_data.Nuclid.lower(), ['плутоний', 'уран-233', 'уран-235', 'нептуний-237', 'америций-241', 'америций-243', 'калифорний-252', 'торий', 'литий-6', 'тритий']):
                    result.add_error('OpCod', 'В графе 7 не представлены  сведения о  радионуклидах, которые могут быть отнесены к ЯМ. Проверьте правильность выбранного кода операции')
            
            if row_data.OpCod in '29 39 97 98 99':
                if not check_remark('OpCod'):
                    result.add_error('OpCod', 'Необходимо дать пояснение об осуществленной операции')

            if row_data.OpCod in '21 22 25 27 28 29 41 42 43 46 61 65 67 68 71 72 81 82 83 98':
                if not check_snk(item_id, api.regs.acc_o4, self.cache):
                    result.add_error('OpCod', 'Учетной единицы с такими параметрами нет в организации. Проверьте правильность указываемых сведений для ИОУ')

            if row_data.OpCod == '37':
                if not check_prev_oper(item_id, '27', 'acc_o4', self.cache, OpCodAttr='OpCod'):
                    result.add_error('OpCod', 'В отчетах не найдена строка об осуществлении передачи учетной единицы. Проверьте правильность выбранного кода операции')

            if row_data.OpCod == '41':
                if not check_related_oper(sguk_forms.t_form_1_6, {'OpCod':row_data.OpCod, 'OpDate':row_data.OpDate, 'RAOCodMax':row_data.Nuclid, 'ActDate':row_data.ActDate, 'DocN':row_data.DocN, 'DocDate':row_data.DocDate}):
                    result.add_error('OpCod', 'Заполните форму 1.6')

            if row_data.OpCod == '54':
                if not check_okpo(row_data.OkpoPIP) or row_data.OkpoPIP == org_OKPO():
                    result.add_error('OpCod', 'В графе 18 необходимо указать ОКПО подрядной организации')

            if row_data.OpCod == '58':
                # Нужна проверка на совпадение по IstName, Nuclid, Arg и OpCod=46
                filter = {'IstName':row_data.IstName, 'Nuclid':row_data.Nuclid, 'Arg':row_data.Arg, 'OpCod':'46'}
                if self.cache.filter(filter).is_empty() and api.regs.acc_o4.select(filter).is_empty():                  
                    result.add_error('OpCod', 'В отчетах не найдены сведения о снятии с учета учетной единицы для разукомплектования. Проверьте правильность выбранного кода операции')

            if row_data.OpCod == '62':
                if not check_prev_oper(item_id, '61', 'acc_o4', self.cache, OpCodAttr='OpCod'):
                    result.add_error('OpCod', 'В отчетах не найдены сведения о вывозе учетной единицы. Проверьте правильность выбранного кода операции')
        
    # п.3 - OpDate
        if row_data.OpCod == '10':
            check_date_between('DocDate', 'form_period_start', 'form_period_end', 'Дата документа не входит в отчетный период')
        else:
            check_date_between('OpDate', 'form_period_start', 'form_period_end', 'Дата операции не входит в отчетный период')

    # п.6 - Typ
        if not voc_check('ORI', row_data.Typ): 
            result.add_error('Typ', 'Вид ОРИ не найден в справочнике видов ОРИ')

    # п.7 - Nuclid
        check_attr_Nuclides(NuclidesAttr='Nuclid')

    # п.8 - Activn
        check_attr_num('Activn', allowNone=True, maxValue=10E+20, msg='Проверьте правильность предоставления сведений по суммарной активности')

    # п.9 - ActDate
        check_date_between('ActDate', None, 'OpDate', 'Дата изммерения не может быть позже даты операции')

    # 12: Arg
        check_attr_Agr(agrAttr='Arg')

    # п.13 - FormSobst
        if form_start_year() >= '2022':
            check_attr_FormSobst(row_data, formSobstAttr='FormSobst')

    # п.14 - Pravoobl
        if form_start_year() >= '2022':
            check_attr_Pravobl(itemName='ОРИ', PravooblAttr='Pravoobl')

    # п.15 - DocVid
        check_attr_DocVid('DocVid')

    # п.17 - DocDate
        check_attr_DocDate(DocDateAttr='DocDate', OpDateAttr='OpDate', OpCodAttr='OpCod')

    # п.18 - OkpoPIP
        check_attr_OkpoPIP(
            selfOkpoCodes='10, 11, 12, 15, 17,18, 41, 42, 43, 46, 53, 58, 61, 62, 67, 68, 71, 72, 72, 73, 74, 75, 76', \
            oksmCodes='81, 82, 83, 85, 86, 87', \
            OkpoPipAttr='OkpoPIP', OpCodAttr='OpCod')

    # п.19 - OkpoPrv
        check_attr_OkpoPrv(
            noneOkpoCodes='10, 11,12, 15, 17, 18, 41, 42, 43, 46, 53, 54, 58, 65, 67, 68, 71, 72, 73, 74, 75, 76, 97, 98', \
            OkpoPrvAttr='OkpoPrv', OpCodAttr='OpCod')

    # п.20 - PrName, UktPrTyp
        check_attrs_PrName(PrNameAttr='PrName', UktPrTypAttr='UktPrTyp')


class Form_1_5_2022_check(ReportFormCheck):      # ОЗРИ (ROZ)
    def check_form(self):
        super().check_form()

        self.cache:RecordSet = get_form_cache().get('acc_o5')   # Отображение текущей формы на регистра acc_o4

        msg = check_form_period()
        if msg:
            return [{'row':'', 'col_name':'', 'text':msg}]
        else:
            return []
            #
            #
    def add_notify(self, result:CheckRowData, row_data:Form_1_1_table):

        notify_list = {
            '18' : 'Необходимо представить копию документа, отражающего результаты проведенных измерений',
            '68' : 'Необходимо представить копию документа, отражающего результаты проведенных измерений',
            '71' : 'Необходимо представить копию документа (протокола, акта, прочего документа), характеризующего обстоятельства установления факта утери',
            '72' : 'Необходимо представить копию документа (протокола, акта, прочего документа), характеризующего обстоятельства установления факта утери контроля над учетной единицей',
            '73' : 'Необходимо представить копию документа (протокола, акта, прочего документа), характеризующего обстоятельства обнаружения учетной единицы',
            '74' : 'Необходимо представить копию документа (протокола, акта, прочего документа), характеризующего обстоятельства изъятия РВ или РАО',
            '75' : 'Необходимо представить копию документа, характеризующего обстоятельства обнаружения учетной единицы',
            '76' : 'Необходимо представить копию документа, характеризующего обстоятельства изъятия РАО.',
        }

        OpCod = row_data.OpCod
        if OpCod not in notify_list: return

        DocVidRec = voc_read('DocVid', row_data.DocVid)
        if DocVidRec and 'DocName' in DocVidRec:
            DocVidStr = DocVidRec.DocName
        else:
            DocVidStr = f'код вида документа"{row_data.DocVid}"'
        
        result.add_notify(f'{notify_list[OpCod]}\n{DocVidStr} №{row_data.DocN} от {m_utils.date_str_format(row_data.DocDate)}')
            #
            #
    def check_row(self):
        ReportFormCheck.check_row(self)

        result:CheckRowData = api.context.row_result
        form_data:Record = api.context.form_data
        row_data:Form_1_5_table = api.context.row_data

        if not check_attrs_not_empty(  
            'OpCod OpDate '                                         # п.2 - 3
            'PaspN Typ Nuclid Numb Sht Activn IzgotDate BCod '      # п.4 - 11
            'DocVid DocN DocDate '                                  # п.12 = 14
            'OkpoPIP OkpoPrv '                                      # п.15 - 16
            'PrName UktPrTyp UktPrN '                               # п.17 - 19
            'PH_Name PH_Cod '                                       # п.20 - 21
            'CodRAO Subsid FCP'):                                    # п.22 - 24
            return

        item_id = sguk_setup_regs.calc_item_id(row_data, form_data, silent=True)

        check_attr_date('OpDate', blank_str=None)
        check_attr_date('IzgotDate')
        check_attr_date('DocDate')

    # п.2 - OpCod
        if row_data.OpCod not in '01, 10, 14, 18, 21, 22, 25, 26, 27, 28, 29, 31, 32, 35, 36, 37, 38, 39, 41, 43, 44, 45, 48, 49, 51, 52, 57, 59, 63, 64, 68, 71, 72, 73, 74, 75, 76,  84, 88, 97, 98, 99':
            result.add_error('OpCod', 'Неверный код операции')
        else:
            if row_data.OpCod == '10':
                if not check_snk(item_id, api.regs.acc_o5, self.cache):
                    result.add_error('OpCod', 'Сведения, представленные в инвентаризации не соответствуют СНК')

            if row_data.OpCod in '29 39 49 59 97 98 99':
                if not check_remark('OpCod'):
                    result.add_error('OpCod', 'Необходимо дать пояснение об осуществленной операции')

            if row_data.OpCod in '21, 22, 25, 26, 27, 28, 29, 42, 43, 44, 45, 48, 49, 51, 68, 71, 72, 84, 98':
                if not check_snk(item_id, api.regs.acc_o5, self.cache):
                    result.add_error('OpCod', 'Учетной единицы с такими параметрами нет в организации. Проверьте правильность указываемых сведений для ОЗРИ')

            if row_data.OpCod == '37':
                if not check_prev_oper(item_id, '27', 'acc_o5', self.cache, OpCodAttr='OpCod'):
                    result.add_error('OpCod', 'В отчетах не найдена строка об осуществлении передачи учетной единицы. Проверьте правильность выбранного кода операции')

            if row_data.OpCod == '52':
                if not check_prev_oper(item_id, '51', 'acc_o5', self.cache, OpCodAttr='OpCod'):
                    result.add_error('OpCod', 'В отчетах не найдена строка об изъятии РАО из пункта хранения. Проверьте правильность выбранного кода операции')

            if row_data.OpCod == '57':
                if not check_prev_oper(item_id, '45', 'acc_o5', self.cache, OpCodAttr='OpCod'):
                    result.add_error('OpCod', 'В отчетах не найдена строка снятии учетной единицы для упаковки/переупаковки. Проверьте правильность выбранного кода операции')

            if row_data.OpCod == '84':
                if row_data.BCod != '7':
                    result.add_error('OpCod', 'Для ОЗРИ, которые возвращаются в страну поставщика код статус РАО – 7')
        
    # п.3 - OpDate
        if row_data.OpCod == '10':
            check_date_between('DocDate', 'form_period_start', 'form_period_end', 'Дата документа не входит в отчетный период')
        else:
            check_date_between('OpDate', 'form_period_start', 'form_period_end', 'Дата операции не входит в отчетный период')

    # п.5 - Typ
        if str(row_data.Typ).strip()=='': 
            result.add_error('Typ', 'Заполните сведения о типе ЗРИ, который переведен в ОЗИИИ')

    # п.6 - Nuclid
        if not check_separated(row_data.Nuclid):
            result.add_error('Nuclid', 'Формат ввода данных не соответствует приказу')

    # п.7 - Numb
        if not check_separated(row_data.Numb):
            result.add_error('Numb', 'Формат ввода данных не соответствует приказу')
        if str(row_data.Numb).strip()=='':
            result.add_error('Numb', 'Заполните сведения о номере ЗРИ, который переведен в ОЗИИИ')

    # п.8 - Sht
        if form_start_year() >= '2014':  # После приказа №600
            if str(row_data.Sht).strip()=='' or not check_num(row_data.Sht):
                result.add_error('Sht', 'Заполните сведения о количестве ЗРИ, переведенных в ОЗИИИ')

    # п.9 - Activn
        activn = str(row_data.get('Activn','')).strip()
        if not check_num(activn, allow_brackets=True):
            result.add_error('Activn', 'Заполните сведения о суммарной активности ЗРИ, переведенных в ОЗИИИ. Оценочные сведения приводятся в круглых скобках')

    # п.10 - IzgotDate
        check_date_between('IzgotDate', None, 'OpDate', 'Дата изготовления не может быть позже даты операции')

    # п.11 - BCod
        if form_start_year() >= '2014':
            check_attr_BCod('BCod', 'OpCod')

    # п.12 - DocVid
        check_attr_DocVid('DocVid')

    # п.14 - DocDate
        check_attr_DocDate(DocDateAttr='DocDate', OpDateAttr='OpDate', OpCodAttr='OpCod')

    # п.15 - OkpoPIP
        check_attr_OkpoPIP(
            selfOkpoCodes='01, 10, 14, 18, 41, 43, 44, 45, 48, 49, 51, 52, 57, 59, 68, 71, 72, 73, 74, 75, 76, 97, 98, 99', \
            oksmCodes='84, 88', \
            OkpoPipAttr='OkpoPIP', OpCodAttr='OpCod')

    # п.16 - OkpoPrv
        check_attr_OkpoPrv(
            noneOkpoCodes='01, 10, 14, 18, 41, 43, 44, 45, 48, 49, 51, 52, 57, 59, 71, 72, 73, 74, 75, 76,  97, 98, 99', \
            OkpoPrvAttr='OkpoPrv', OpCodAttr='OpCod')

    # п.17 - PrName, UktPrTyp
        check_attrs_PrName(PrNameAttr='PrName', UktPrTypAttr='UktPrTyp')

    # п.20 - PH_Name, PH_Cod
        check_attrs_PH_Name(PH_NameAttr='PH_Name', PH_CodAttr='PH_Cod')

    # п.22 - CodRAO
        if form_start_year()>='2022' and row_data.OpCod in '49, 59' and row_data.CodRAO not in '52, 72, 74':
            result.add_error('CodRAO', 'Коду операции сортировка соответствуют коды сортировки 52, 72, 74')

    # п.23 - Subsid
        check_attr_Subsid(subsidAttr='Subsid')

    # п.24 - FCP
        check_attr_FCP(fcpAttr='FCP')


class Form_1_6_2022_check(ReportFormCheck):      # РАО - некондиционированные (RAO)
    def check_form(self):
        super().check_form()

        self.cache:RecordSet = get_form_cache().get('acc_o6')   # Отображение текущей формы на регистра acc_o4

        return []

        msg = check_form_period()
        if msg:
            return [{'row':'', 'col_name':'', 'text':msg}]
        else:
            return []
            #
            #
    def check_row(self):
        ReportFormCheck.check_row(self)

        result:CheckRowData = api.context.row_result
        form_data:Record = api.context.form_data
        row_data:Form_1_6_table = api.context.row_data

        if not check_attrs_not_empty(
            'OpCod OpDate '                                         # п.2 - 3
            'RAOCod BCod Kbm Tonne Sht RAOCodMax '                  # п.4 - 9
            'ActTR ActBG ActA ActUR ActDate '                       # п.10 - 14
            'DocVid DocN DocDate '                                  # п.15 - 17
            'OkpoPIP OkpoPrv '                                      # п.18 - 19
            'PH_Name PH_Cod UpCod '                                 # п.20 - 22
            'PrName UktPrTyp UktPrN '                               # п.23 - 25
            'Subsid FCP'):                                          # п.26 - 27
            return

        item_id = sguk_setup_regs.calc_item_id(row_data, form_data, silent=True)

        check_attr_date('OpDate', blank_str=None)
        check_attr_date('ActDate')
        check_attr_date('DocDate')

    # п.2 - OpCod
        if row_data.OpCod not in '01, 10, 11, 12, 13, 14, 16, 18, 21, 22, 25, 26, 27, 28, 29, 31, 32, 35, 36, 37, 38, 39, 41, 42, 43, 44, 45, 48, 49, 51, 52, 56, 57, 59, 63, 64, 68, 71, 72, 73, 74, 75, 76,  97, 98, 99':
            if row_data.OpCod in '55' and form_start_year() < '2022':
                pass
            else:
                result.add_error('OpCod', 'Неверный код операции')
        else:
            if row_data.OpCod == '10':
                if not check_snk(item_id, api.regs.acc_o6, self.cache):
                    result.add_error('OpCod', 'Сведения, представленные в инвентаризации не соответствуют СНК')

            if row_data.OpCod in '11, 12, 13, 14, 16, 18, 41' and len(row_data.RAOCod)>=10 and row_data.RAOCod[8:10] == '99':
                result.add_error('OpCod', 'Для вновь образованных РАО код типа РАО «99» не может быть использован')

            if row_data.OpCod in '12 42':
                if not check_contain_any(row_data.RAOCodMax.lower(), ['плутоний', 'уран-233', 'уран-235', 'уран-238', 'нептуний-237', 'америций-241', 'америций-243', 'калифорний-252', 'торий', 'литий-6', 'тритий']):
                    result.add_error('OpCod', 'В графе 9 не представлены  сведения о  радионуклидах, которые могут быть отнесены к ЯМ. Проверьте правильность выбранного кода операции')
            
            if row_data.OpCod in '29 39 49 59 97 98 99':
                if not check_remark('OpCod'):
                    result.add_error('OpCod', 'Необходимо дать пояснение об осуществленной операции')

            if row_data.OpCod in '21, 22, 25, 26, 27, 28, 29, 42, 43, 44, 45, 48, 49, 51, 68, 71, 72, 98':
                if not check_snk(item_id, api.regs.acc_o6, self.cache):
                    result.add_error('OpCod', 'Учетной единицы с такими параметрами нет в организации. Проверьте правильность указываемых сведений для РАО')

            if row_data.OpCod == '37':
                if not check_prev_oper(item_id, '27', 'acc_o6', self.cache, OpCodAttr='OpCod'):
                    result.add_error('OpCod', 'В отчетах не найдена строка об осуществлении передачи учетной единицы. Проверьте правильность выбранного кода операции')

            if row_data.OpCod == '52':
                if not check_prev_oper(item_id, '51', 'acc_o6', self.cache, OpCodAttr='OpCod'):
                    result.add_error('OpCod', 'В отчетах не найдена строка об изъятии РАО из пункта хранения. Проверьте правильность выбранного кода операции')

            #if row_data.OpCod == '57':
            #    if not check_prev_oper(item_id, '45', 'acc_o6', self.cache, OpCodAttr='OpCod'):
            #        result.add_error('OpCod', 'В отчетах не найдена строка снятии учетной единицы для упаковки/переупаковки. Проверьте правильность выбранного кода операции')

            if row_data.OpCod in '49, 59' and row_data.UpCod not in '- 52, 72, 74':
                result.add_error('OpCod', 'Код операции не соответствует коду переработки/сортировки, указанному в графе 22')

    # п.3 - OpDate
        if row_data.OpCod == '10':
            check_date_between('DocDate', 'form_period_start', 'form_period_end', 'Дата документа не входит в отчетный период')
        else:
            check_date_between('OpDate', 'form_period_start', 'form_period_end', 'Дата операции не входит в отчетный период')

    # 4: RAOCod
        check_attr_RaoCod(
            opCodAttr='OpCod', codRaoAttr='RAOCod', massAttr='Tonne', nuclidesAttr='RAOCodMax', upCodAttr='UpCod', 
            ActTr=row_data.ActTR, ActA=row_data.ActA, ActBG=row_data.ActBG, ActU=row_data.ActUR, PH_Cod=row_data.PH_Cod, BCod=row_data.BCod)        

    # п.5 - BCod (Статус РАО)
        if form_start_year() >= '2014':
            check_attr_BCod(bCodAttr='BCod', opCodAttr='OpCod')

    # п.6 - Kbm
        check_attr_num('Kbm', msg='Необходимо заполнить сведения об объеме РАО, если представляемые данные являются расчетными, то соответствующие значения указываются в круглых скобках')

    # п.7 - Tonne
        Tonne = str2float(row_data.Tonne)
        if Tonne == 0:
            result.add_error('Tonne', 'Необходимо заполнить сведения о массе РАО, если представляемые данные являются расчетными, то соответствующие значения указываются в круглых скобках')
        else:
            Kbm = str2float(row_data.Kbm)
            if Kbm > 0 and Tonne/Kbm > 21:
                result.add_error('Tonne', 'Проверьте значение массы и объема. Расчетное значение плотности слишком большое')
            if Kbm > 0 and Tonne/Kbm < 0.005:
                result.add_error('Tonne', 'Проверьте значение массы и объема. Расчетное значение плотности слишком маленькое')

    # п.8 - Sht
        if len(row_data.RAOCod)>=10:
            raoType = row_data.RAOCod[8:10]
            if raoType in '81, 82, 84, 85, 86, 87, 88, 89':
                if str2float(row_data.Sht) == 0:
                    result.add_error('Sht', 'Необходимо заполнить сведения о количестве ОЗИИИ')
            elif raoType != '99':
                if row_data.RAOCod[0] in '1 3' and str2float(row_data.Sht)>0:
                    result.add_error('Sht', 'Графа заполняется только для ОЗИИИ')

    # 9-13: Проверка радионуклидов и активности 
        check_rao_criteria(codRaoAttr='RAOCod', massAttr='Tonne', actT_attr='ActTR', actB_attr='ActBG', actA_attr='ActA', opCodAttr='OpCod')
        check_attrs_activity(codRaoAttr='RAOCod', nuclidesAttr='RAOCodMax', actT_attr='ActTR', actB_attr='ActBG', actA_attr='ActA', actU_attr='ActUR', gAct='10-13', gNucl='9')

    # п.14 - ActDate
        check_date_between('ActDate', None, 'OpDate', 'Проверьте правильность указанной даты измерения активности')

    # п.15 - DocVid
        check_attr_DocVid('DocVid')

    # п.17 - DocDate
        check_attr_DocDate(DocDateAttr='DocDate', OpDateAttr='OpDate', OpCodAttr='OpCod')

    # п.18 - OkpoPIP
        check_attr_OkpoPIP(
            selfOkpoCodes='01, 10, 11, 12, 13, 14, 16, 18, 41, 42, 43, 44, 45, 48, 49, 51, 52, 56, 57, 59, 68, 71, 72, 73, 74, 75, 76, 97, 98', \
            oksmCodes='', \
            OkpoPipAttr='OkpoPIP', OpCodAttr='OpCod')

    # п.19 - OkpoPrv
        check_attr_OkpoPrv(
            noneOkpoCodes='01, 10, 11, 12, 13, 14, 16, 18, 41, 42, 43, 44, 45, 48, 49, 51, 52, 56, 57, 59, 71, 72, 73, 74, 75, 76,  97, 98, 99', \
            OkpoPrvAttr='OkpoPrv', OpCodAttr='OpCod')

    # п.22 - UpCod
        check_attr_UpCod(upCodAttr='UpCod', raoCodAttr='RAOCod', opCodAttr='OpCod')

    # п.20 - PH_Name, PH_Cod
        check_attrs_PH_Name(PH_NameAttr='PH_Name', PH_CodAttr='PH_Cod')

    # п.22 - PrName, UktPrTyp
        check_attrs_PrName(PrNameAttr='PrName', UktPrTypAttr='UktPrTyp')

    # п.26 - Subsid
        check_attr_Subsid(subsidAttr='Subsid')

    # п.27 - FCP
        check_attr_FCP(fcpAttr='FCP')


class Form_1_7_2022_check(ReportFormCheck):      # ТРО - кондиционированные
    def check_form(self):
        api.context['OpCod'] = ''
        super().check_form()

        self.cache:RecordSet = get_form_cache().get('acc_o7')   # Отображение текущей формы на регистр acc_o7

        msg = check_form_period()
        if msg:
            return [{'row':'', 'col_name':'', 'text':msg}]
        else:
            return []
            #
            #
    def add_row_aliases(self, row_data):
        row_data.add_alias('OpCod', 'g2')
        row_data.add_alias('OpDate', 'g3')
        row_data.add_alias('DocVid', 'g14')
        row_data.add_alias('DocN', 'g15')
        row_data.add_alias('DocDate', 'g16')
        pass
            #
            #
    def check_row(self):
        ReportFormCheck.check_row(self)

        result:CheckRowData = api.context.row_result
        form_data:Record = api.context.form_data

        row_data:Form_1_7_table = Record( api.context.row_data)    # Создаем клон
        row_no:int = api.context.row_no

        #result.add_error('g4', f'Запись: {row_data.to_dict()}')

        if row_data.g2:
            api.context['OpCod'] = row_data.g2

        if not api.context['OpCod']:
            result.add_error('g2', 'Не указан код операции')

        # Проверка радионуклида и его активности
        if row_data.g12 or row_data.g13: 
            if not check_attrs_not_empty('g12 g13'): return
            check_attr_Nuclides(NuclidesAttr='g12')
            check_attr_num(attrName='g13')


        # Определяем строки относящиеся к одной операции
        form_table:RecordSet = form_data.form_table_data.main_table
        
        first_line = row_no
        while first_line>0 and not form_table[first_line].g2: 
            first_line -= 1

        last_line = row_no + 1
        while last_line<form_table.count() and not form_table[last_line].g2: last_line += 1
        last_line -= 1

        #res.append({'col_name':'', 'text':f'first_line={first_line} last_line={last_line}'})

        # Формируем список радионуклидов
        nuclides = []
        for i in range(first_line, last_line+1):
            if form_table[i].g12:
                nuclides.append(form_table[i].g12)
        row_data.g12 = ';'.join(nuclides)


        # Заполняем графы из строки с операцией
        if not row_data.g2:
            is_oper_check = False 
            for g in 'g2 g3 g4 g5 g6 g7 g8 g9 g10 g11 g14 g15 g16 g17 g18 g19 g20'.split(' '):
                row_data[g] = form_data.form_table_data.main_table[first_line][g]
        else:
            is_oper_check = True


        # Проверка сведений о РАО
        if row_data.g21 or row_data.g22:
            if not check_attrs_not_empty('g21 g22 g23 g24 g25 g26 g27 g28 g29 g30 g31 g32'): return

            # 21: g21 (код РАО)
            if len(row_data.g21) < 8:
                result.add_error('g21', 'Не корректная длина кода РАО')
            else:
                if row_data.g21[1-1] != '2':
                    result.add_error('g21', f'символ[1]="{row_data.g21[1-1]}": В форме 1.7 приводятся сведения только о твердых кондиционированных РАО. Разрешено только значение 2')
                if row_data.g21[2-1] == '9':
                    result.add_error('g21', f'символ[2]="{row_data.g21[2-1]}": Для кондиционированных РАО должна быть определена категория')
                if row_data.g21[3-1] == '0':
                    result.add_error('g21', f'символ[3]="{row_data.g21[3-1]}": Укажите 3-й символ кода РАО в соответствии с радионуклидном составом')
                if row_data.g21[6-1] == '0':
                    result.add_error('g21', f'символ[6]="{row_data.g21[6-1]}": Для кондиционированных РАО должен быть определен период потенциальной опасности')
                if row_data.g21[8-1] not in '1 2 3 4 6':
                    result.add_error('g21', f'символ[8]="{row_data.g21[8-1]}": Сведения о некондиционированных отходах необходимо представлять в формах 1.5 и 1.6')
                if row_data.g21[8-1] == '6' and row_data.g21[2-1] != '0':
                    result.add_error('g21', f'символ[8]="{row_data.g21[8-1]}": Твердые РАО, содержащие природные радионуклиды относятся к ОНАО')

                # Проверка 3-го символа кода РАО - радионуклидный состав
                nlist = ''
                if str2float(row_data.g28): nlist += 'A'
                if str2float(row_data.g26) or str2float(row_data.g27): nlist += 'B'
                if str2float(row_data.g29): nlist += 'U'
                req_rao_cod = calcNuclideGroup(nlist)
                if row_data.g21[3-1] != str(req_rao_cod):
                    result.add_error('g21', f'символ[3]="{row_data.g21[3-1]}": Код радионуклидного состава не соответствует введенным значениям суммарной активности (г.26-29). Должно быть {req_rao_cod}')


            check_attr_RaoCod(
                opCodAttr='g2', codRaoAttr='g21', massAttr='g24', nuclidesAttr='g12', upCodAttr='g30', 
                ActTr=row_data.g26, ActA=row_data.g28, ActBG=row_data.g27, ActU=row_data.g29, PH_Cod=row_data.g20, BCod=row_data.g22)


            # п.22 - g22 (BCod)
            check_attr_BCod(bCodAttr='g22', OpCod=api.context.OpCod)

            # п.23 - g23 (объем без упаковки)
            if str2float(row_data.g23) <= 0:
                result.add_error('g23', 'Необходимо заполнить сведения об объеме РАО.')

            # п.25 - g25 (Sht)
            if len(row_data.g21)>=10:
                raoType = row_data.g21[8:10]
                if raoType in '81, 82, 84, 85, 86, 87, 88, 89':
                    if str2float(row_data.g25) == 0:
                        result.add_error('g25', 'Необходимо заполнить сведения о количестве ОЗИИИ')
                elif raoType != '99':
                    if str2float(row_data.g25)>0:
                        result.add_error('g25', 'Графа заполняется только для ОЗИИИ')

            # п.26 - g26 (ActTR)
            # п.27 - g27 (ActBG)
            # п.28 - g28 (ActA)
            # п.29 - g29 (ActUR)
            #check_rao_criteria(codRaoAttr='g21', massAttr='g11', actT_attr='g26', actB_attr='g27', actA_attr='g28', opCodAttr='g2')
            #check_attrs_activity(codRaoAttr='g21', nuclidesAttr='g12', actT_attr='g26', actB_attr='g27', actA_attr='g28', actU_attr='g29', gAct='26-29', gNucl='12')

            # п.30 - g30 (UpCod)
            if row_data.g2 in '01, 10, 11, 12, 13, 14, 16, 21, 22, 25, 26, 27, 28, 29, 31, 32, 35, 36, 37, 38, 39, 43, 51, 52, 63, 64, 97, 98':
                if row_data.g30 != '-':
                    result.add_error('g30',  f'Для операции {row_data.g2} код переработки не указавается, должен быть "-"')
            else:
                if row_data.g2 == '55' and row_data.g30 == '-':
                    pass
                else:    
                    if not voc_check('UpCod', row_data.g30):
                        result.add_error('g30', 'Значение должно быть из справочника установок переработки/сортировки')

            # п.31 - g31 (Subsid)
            check_attr_Subsid(subsidAttr='g31')

            # п.32 - g32 (FCP)
            check_attr_FCP(fcpAttr='g32')


        if not is_oper_check: 
            return 


        item_id = sguk_setup_regs.calc_item_id(row_data, form_data, silent=True)

        if not check_attrs_not_empty('g2 g3 g4 g5 g6 g7 g8 g9 g10 g11 g14 g15 g16 g17 g18 g19 g20 '): return

        check_attr_date('OpDate', blank_str=None)
        check_attr_date('DocDate')

    #  2: g2 (OpCod)
        if row_data.g2 not in '01, 10, 11, 12, 13, 14, 16, 21, 22, 25, 26, 27, 28, 29, 31, 32, 35, 36, 37, 38, 39, 43, 51, 52, 55, 63, 64, 97, 98':
            result.add_error('g2', 'Неверный код операции')
        else:
            if row_data.g2 == '10':
                if not check_snk(item_id, api.regs.acc_o7, self.cache):
                    result.add_error('g2', 'Сведения, представленные в инвентаризации не соответствуют СНК')

            if row_data.g2 in '11, 12, 13, 14' and len(row_data.g21)>=10 and row_data.g21[8:10] == '99':
                result.add_error('g2', 'Для вновь образованных РАО код типа РАО «99» не может быть использован')

            if row_data.g2 in '12':
                if not check_contain_any(row_data.g12.lower(), ['плутоний', 'уран-233', 'уран-235', 'нептуний-237', 'америций-241', 'америций-243', 'калифорний-252', 'торий', 'литий-6', 'тритий']):
                    result.add_error('g2', 'В графе 12 не представлены  сведения о  радионуклидах, которые могут быть отнесены к ЯМ. Проверьте правильность выбранного кода операции')
            
            if row_data.g2 in '29 39 97 98':
                if not check_remark('g2'):
                    result.add_error('g2', 'Необходимо дать пояснение об осуществленной операции')

            if row_data.g2 in '21, 22, 25, 26, 27, 28, 29, 51, 98':
                if not check_snk(item_id, api.regs.acc_o7, self.cache):
                    result.add_error('g2', 'Учетной единицы с такими параметрами нет в организации. Проверьте правильность указываемых сведений для РАО')

            if row_data.g2 == '37':
                if not check_prev_oper(item_id, '27', 'acc_o7', self.cache):
                    result.add_error('g2', 'В отчетах не найдена строка об осуществлении передачи учетной единицы. Проверьте правильность выбранного кода операции')

            if row_data.g2 == '52':
                if not check_prev_oper(item_id, '51', 'acc_o7', self.cache):
                    result.add_error('g2', 'В отчетах не найдена строка об изъятии РАО из пункта хранения. Проверьте правильность выбранного кода операции')

            if row_data.g2 in '10, 11, 12, 13, 14' and len(row_data.g21)>=5 and \
                (row_data.g21[2-1]=='0' or (row_data.g21[2-1]=='1' and row_data.g21[5-1]=='2')):
                result.add_error('g2', 'Нет сведений о кондиционировании РАО, проверьте правильность выбранной формы для предоставления отчета')

    #  3: g3 (OpDate)
        if row_data.g2 == '10':
            check_date_between('g16', 'form_period_start', 'form_period_end', 'Дата документа не входит в отчетный период')
        else:
            check_date_between('g3', 'form_period_start', 'form_period_end', 'Дата операции не входит в отчетный период')

    #  4: g4 - Наименование упаковки
    #  5: g5 - Тип упаковки
        if row_data.g4.lower() != 'без упаковки':
            check_attrs_PrName(PrNameAttr='g4', UktPrTypAttr='g5')

    #  8: g8 - Дата формирования упаковки
        check_date_between('g8', None, 'g3', 'Дата формирования упаковки должна быть меньше или равна дате операции')
        if str(row_data.g8).strip() == '':
            result.add_error('g8', 'Проверьте правильность даты формирования упаковки')

    # 10: g10 - Объем, м3
        if str2float(row_data.g10) <= 0:
            result.add_error('g10', 'Должно быть число больше 0')

    # 11: g11 - Масса, т
        if str2float(row_data.g11) <= 0:
            result.add_error('g11', 'Должно быть число больше 0')

    # 12: g12 - Радионуклиды

    # 13: g13
        check_attr_num('g13')

    # 14: g14 - Вид документа
        check_attr_DocVid('g14')

    # 16: g16 - Дата документа
        check_attr_DocDate(DocDateAttr='g16', OpDateAttr='g3', OpCodAttr='g2')

    # 17: g17 (OkpoPIP)
        check_attr_OkpoPIP(
            selfOkpoCodes='01, 10, 11, 12, 13, 14, 16, 43, 51, 52, 55', \
            oksmCodes='', \
            OkpoPipAttr='g17', OpCodAttr='g2')

    # 18: g18 (OkpoPrv)
        check_attr_OkpoPrv(
            noneOkpoCodes='01, 10, 11, 12, 13, 14, 16, 43, 51, 52, 55, 97, 98', \
            OkpoPrvAttr='g18', OpCodAttr='g2')

    # 19: g19, g20 (PH_Name, PH_Cod)
        check_attrs_PH_Name(PH_NameAttr='g19', PH_CodAttr='g20')


class Form_1_8_2022_check(ReportFormCheck):      # ЖРО - кондиционированные
    def check_form(self):
        api.context['OpCod'] = ''
        super().check_form()

        self.cache:RecordSet = get_form_cache().get('acc_o8')   # Отображение текущей формы на регистра acc_o4

        #api.log.msg(f'ReportFormCheck.check: 1.2.form_data.table keys = {list(api.context.form_data.table[0]._data.keys())}')

        msg = check_form_period()
        if msg:
            return [{'row':'', 'col_name':'', 'text':msg}]
        else:
            return []
            #
            #
    def add_row_aliases(self, row_data):
        row_data.add_alias('OpCod', 'g2')
        row_data.add_alias('OpDate', 'g3')
        row_data.add_alias('DocVid', 'g11')
        row_data.add_alias('DocN', 'g12')
        row_data.add_alias('DocDate', 'g13')
            #
            #
    def check_row(self):
        ReportFormCheck.check_row(self)

        result:CheckRowData = api.context.row_result
        form_data:Record = api.context.form_data
        row_data:Form_1_8_table = api.context.row_data
        row_no:int = api.context.row_no

        if row_data.g2:
            api.context['OpCod'] = row_data.g2

        if not api.context['OpCod']:
            result.add_error('g2', 'Не указан код операции')

        row_data = Record(row_data.to_dict())

        if not row_data.g9 and not row_data.g18:
            result.add_error('g9', 'Необходимо указать радионуклид или характеристики ЖРО')
            result.add_error('g18', 'Необходимо указать радионуклид или характеристики ЖРО')
            return


    # Проверка радионуклида и его активности
        if row_data.g9 or row_data.g10: 
            if not check_attrs_not_empty('g9 g10'): return
            check_attr_Nuclides(NuclidesAttr='g9')
            check_attr_num(attrName='g10')


        # Определяем строки относящиеся к одной операции
        form_table:RecordSet = form_data.form_table_data.main_table
        
        first_line = row_no
        while first_line>0 and not form_table[first_line].g2: first_line -= 1

        last_line = row_no + 1
        while last_line<form_table.count() and not form_table[last_line].g2: last_line += 1
        last_line -= 1


        # Формируем список радионуклидов
        nuclides = []
        for i in range(first_line, last_line+1):
            if form_table[i].g9:
                nuclides.append(form_table[i].g9)
        row_data.g9 = ';'.join(nuclides)


        # Заполняем графы из строки с операцией
        if not row_data.g2:
            is_oper_check = False 
            for g in 'g2 g3 g4 g5 g6 g7 g8 g11 g14 g15 g16 g17'.split(' '):
                row_data[g] = form_data.form_table_data.main_table[first_line][g]
        else:
            is_oper_check = True


    # Проверка характеристик ЖРО
        if row_data.g19:
            if not check_attrs_not_empty('g18 g19 g20 g21 g22 g23 g24 g25 g26 g27 g28'): return

        # 18: g18 (код РАО)
            if row_data.g18 and len(row_data.g18) < 8:
                result.add_error('g18', 'Не корректная длина кода РАО')
            elif row_data.g18:
                if not row_data.g18[1-1] == '1':
                    result.add_error('g18', 'В форме 1.8 приводятся сведения только о жидких кондиционированных РАО')
                if not row_data.g18[2-1] in '1 2':
                    result.add_error('g18', 'К пятому классу могут быть отнесены только среднеактивные и низкоактивные ЖРО')
                if row_data.g18[3-1] == '0':
                    result.add_error('g18', 'Укажите 3-й символ кода РАО в соответствии с радионуклидном составом')
                if row_data.g18[6-1] == '0':
                    result.add_error('g18', 'Для кондиционированных РАО должен быть определен период потенциальной опасности')
                if not row_data.g18[7-1] == '0':
                    result.add_error('g18', 'К пятому классу относятся только жидкие РАО. Сведения об омоноличеных (отвержденных) РАО приводятся в формах 1.6 или 1.7')
                if not row_data.g18[8-1] == '5':
                    result.add_error('g18', 'Проверьте правильность выбранной формы для предоставления отчета')

                # Проверка 3-го символа кода РАО - радионуклидный состав
                nlist = ''
                if str2float(row_data.g24): nlist += 'A'
                if str2float(row_data.g22) or str2float(row_data.g23): nlist += 'B'
                if str2float(row_data.g25): nlist += 'U'
                req_rao_cod = calcNuclideGroup(nlist)
                if row_data.g18[3-1] != str(req_rao_cod):
                    result.add_error('g18', f'символ[3]="{row_data.g18[3-1]}": Код радионуклидного состава не соответствует введенным значениям суммарной активности (г.22-25). Должно быть {req_rao_cod}')


            check_attr_RaoCod(  
                opCodAttr='g2', codRaoAttr='g18', massAttr='g21', nuclidesAttr='g9', upCodAttr='g26',
                ActTr=row_data.g22, ActA=row_data.g24, ActBG=row_data.g23, ActU=row_data.g25, PH_Cod=row_data.g17, BCod=row_data.g19)

        # 19: g19 (BCod - статус РАО)
            check_attr_BCod(bCodAttr='g19', OpCod=api.context.OpCod)

        # 20: g20 (объем)
            if str2float(row_data.g20) <= 0:
                result.add_error('g20', 'Необходимо заполнить сведения об объему ЖРО')

        # 21: g21 (масса без упаковки)
            Tonne = str2float(row_data.g21)
            if Tonne <= 0:
                result.add_error('g21', 'Необходимо заполнить сведения о массе ЖРО')
            else:
                Kbm = str2float(row_data.g20)
                if Kbm > 0 and Tonne/Kbm > 21:
                    result.add_error('g21', 'Проверьте значение массы и объема. Расчетное значение плотности слишком большое')
                if Kbm > 0 and Tonne/Kbm < 0.005:
                    result.add_error('g21', 'Проверьте значение массы и объема. Расчетное значение плотности слишком маленькое')

        # 22: g22 (ActTR)
        # 23: g23 (ActBG)
        # 24: g24 (ActA)
        # 25: g25 (ActUR)
            #check_rao_criteria(codRaoAttr='g18', massAttr='g7', actT_attr='g22', actB_attr='g23', actA_attr='g24', opCodAttr='g2')
            #check_attrs_activity(codRaoAttr='g18', nuclidesAttr='g9', actT_attr='g22', actB_attr='g23', actA_attr='g24', actU_attr='g25', gAct='22-25', gNucl='9')

        # 26: g26 (UpCod)
            if row_data.g2 in '01, 10, 11, 12, 13, 18,21, 25, 26, 27, 28, 29, 31, 32, 35, 36, 37, 38, 39, 51, 52, 63, 64, 68, 97, 98':
                if row_data.g26 != '-':
                    result.add_error('g26',  f'Для операции {row_data.g2} код переработки не указавается, должен быть "-"')
            else:
                if row_data.g2 == '55' and row_data.g26 == '-':
                    pass
                else:    
                    if not voc_check('UpCod', row_data.g26):
                        result.add_error('g26', 'Значение должно быть из справочника установок переработки/сортировки')

        # 27: g27 (Subsid)
            check_attr_Subsid(subsidAttr='g27')

        # 28: g28 (FCP)
            check_attr_FCP(fcpAttr='g28')


        if not is_oper_check: 
            return 

    # Проверка граф операции
        if not check_attrs_not_empty('g2 g3 g4 g5 g6 g7 g8 g11 g12 g13 g14 g15 g16 g17'): return

        check_attr_date('OpDate', blank_str=None)
        check_attr_date('DocDate')

        item_id = sguk_setup_regs.calc_item_id(row_data, form_data, silent=True)

    #  2: g2 (OpCod)
        if row_data.g2 not in '01, 10, 11, 12, 13, 18, 21, 25, 26, 27, 28, 29, 31, 32, 35, 36, 37, 38, 39, 51, 52, 55, 63, 64, 68, 97, 98':
            result.add_error('g2', 'Неверный код операции')
        else:
            if row_data.g2 == '10':
                if not check_snk(item_id, api.regs.acc_o8, self.cache):
                    result.add_error('g2', 'Сведения, представленные в инвентаризации не соответствуют СНК')

            if row_data.g2 in '12':
                if not check_contain_any(row_data.g9.lower(), ['плутоний', 'уран-233', 'уран-235', 'нептуний-237', 'америций-241', 'америций-243', 'калифорний-252', 'торий', 'литий-6', 'тритий']):
                    result.add_error('g2', 'В графе 9 не представлены  сведения о  радионуклидах, которые могут быть отнесены к ЯМ. Проверьте правильность выбранного кода операции')
    
            if row_data.g2 == '11, 12, 13, 18' and len(row_data.g21)>=10 and row_data.g21[8:10] == '99':
                result.add_error('g2', 'Для вновь образованных РАО код типа РАО «99» не может быть использован')

            if row_data.g2 in '29 39 97 98':
                if not check_remark('g2'):
                    result.add_error('g2', 'Необходимо дать пояснение об осуществленной операции')

            if row_data.g2 in '21, 25, 26, 27, 28, 29, 51, 98':
                if not check_snk(item_id, api.regs.acc_o8, self.cache):
                    result.add_error('g2', 'Учетной единицы с такими параметрами нет в организации. Проверьте правильность указываемых сведений для РАО')

            if row_data.g2 == '37':
                if not check_prev_oper(item_id, '27', 'acc_o8', self.cache):
                    result.add_error('g2', 'В отчетах не найдена строка об осуществлении передачи учетной единицы. Проверьте правильность выбранного кода операции')

            if row_data.g2 == '52':
                if not check_prev_oper(item_id, '51', 'acc_o8', self.cache):
                    result.add_error('g2', 'В отчетах не найдена строка об изъятии РАО из пункта хранения. Проверьте правильность выбранного кода операции')

    #  3: g3 (OpDate)
        if row_data.g2 == '10':
            check_date_between('g13', 'form_period_start', 'form_period_end', 'Дата документа не входит в отчетный период')
        else:
            check_date_between('g3', 'form_period_start', 'form_period_end', 'Дата операции не входит в отчетный период')

    #  6: g6 (объем)
        if str2float(row_data.g6) <= 0:
            result.add_error('g6', 'Необходимо заполнить сведения об объеме партии ЖРО.')

    #  7: g7 (масса)
        if str2float(row_data.g7) <= 0:
            result.add_error('g7', 'Необходимо заполнить сведения о массе брутто партии ЖРО.')

    #  8: g8 (солесодержание)
        check_attr_num('g8', msg='Необходимо заполнить сведения о солесодержании.')

    #  9: g9 (радионуклиды)

    # 10: g10 (удельная активность)
        check_attr_num('g10')

    # 11: g11 (DocVid)
        check_attr_DocVid('g11')

    # 13: g13 (DocDate)
        check_attr_DocDate(DocDateAttr='g13', OpDateAttr='g3', OpCodAttr='g2')

    # 14: g14 (OkpoPIP)
        check_attr_OkpoPIP(
            selfOkpoCodes='01, 10, 11, 12, 13, 18, 51, 52, 55, 68', \
            oksmCodes='', \
            OkpoPipAttr='g14', OpCodAttr='g2')

    # 15: g15 (OkpoPrv)
        check_attr_OkpoPrv(
            noneOkpoCodes='01, 10, 11, 12, 13, 18, 51, 52, 55, 68, 97, 98', \
            OkpoPrvAttr='g15', OpCodAttr='g2')

    # 16: g16, g17 (PH_Name, PH_Cod)
        check_attrs_PH_Name(PH_NameAttr='g16', PH_CodAttr='g17')



def init(_api:SgukApi, debug=False):
    global _debug
    _debug = debug

    global api
    api = _api

    api.check_forms.add_check(sguk_forms.form_1_1_2022, Form_1_1_2022_check())
    api.check_forms.add_check(sguk_forms.form_1_2_2022, Form_1_2_2022_check())
    api.check_forms.add_check(sguk_forms.form_1_3_2022, Form_1_3_2022_check())
    api.check_forms.add_check(sguk_forms.form_1_4_2022, Form_1_4_2022_check())
    api.check_forms.add_check(sguk_forms.form_1_5_2022, Form_1_5_2022_check())
    api.check_forms.add_check(sguk_forms.form_1_6_2022, Form_1_6_2022_check())
    api.check_forms.add_check(sguk_forms.form_1_7_2022, Form_1_7_2022_check())
    api.check_forms.add_check(sguk_forms.form_1_8_2022, Form_1_8_2022_check())
