from sguk_ide import * 

api = None

import m_repo

def init(_api):
    global api
    api = _api

    api.repo.add_attr_type('date', CoreType('date') )
    api.repo.add_attr_type('exponential', CoreType('exponential') )


class CoreType(m_repo.AttrType):

    def __init__(self, core_type):
        super().__init__(core_type)

        core_model = api.lib.core.repo.getTypeModel(core_type)
        if core_model:
            self._type_attrs = core_model
        else:
            self._type_attr = {}


type_models = {
    'date': {'base_type': 'DATE'},

    'exponential': {"base_type": "BASE",
        "handlers": { "before_label": ["exponential"], "before_save": ["exponential"] },
        "style": { "textAlign": "right" },
        "validation": { "handlers": ["exponential"], "allow_list": ["-"] }
    },

    'organizations_voc_type': {
        'lazy_load_size': 50,
        'value_id': 'record_id',
        'label': '{NAME_SHORT} ({KOD_ORG})',
        'base_type': 'VOC',
        'voc_subtype': 'settings',
        'voc_name': 'Организации',
        'voc_type': 'organizations',
        'translate_value_id': True,
        # пусто, сформируется автоматически на основе model, чтобы не дублировать
        'headers': [
            'KOD_ORG', 'KOD_GOL_ORG', 'NAME_FULL', 'NAME_SHORT', 'ORG_TYPE',
            'KOD_SRF', 'INN', 'OKPO', 'ADDRESS_U', 'status'
        ],
        'model': [
            {
                'field': 'KOD_ORG',
                'caption': 'Регистрационный номер организации',
                'type': 'str',
                'hidden': True,
            },
            {
                'field': 'NAME_SHORT',
                'caption': 'Краткое наименование организации',
            },
            {
                'field': 'ORG_TYPE',
                'caption': 'Тип организации',
                'type': 'ORG_TYPE_type',
            },
            {
                'field': 'KOD_GOL_ORG',
                'caption': 'Код головной организации',
                'type': 'organizations_voc_type'
            },
            {
                'field': 'KOD_SRF',
                'caption': 'Субъект Российской Федерации',
                'type': '115_type'
            },
            {
                'field': 'NAME_FULL',
                'caption': 'Полное наименование организации',
            },
            {
                'field': 'ADDRESS_U',
                'caption': 'Юридический адрес организации',
                'style': "width: 550px;",
            },
            {
                'field': 'INN',
                'caption': 'ИНН',
            },
            {
                'field': 'OKPO',
                'caption': 'ОКПО',
                'type': 'OKPO_type',
            },
            {
                'field': 'DT_REG',
                'caption': 'дата регистрации',
                'type': 'date',
            },
            {
                'field': 'DT_SU',
                'caption': 'дата снятия с учёта',
                'type': 'date',
            },
            {
                'field': 'status',
                'caption': 'Статус',
                'type': 'voc_status_type',
                'readonly': True,
                'hide_selection': True,
                'filter': ['=']
            }
        ]
    }
}



test_form1 = {
        'module_type': 'report_form',
        'form_name': '1.1: Сведения о закрытых радионуклидных источниках',
        'form_num': '1.1',
        'form_main_attrs': [
            {'field': 'position','caption': 'Исполнитель: Должность'},
            {'field': 'full_name','caption': 'Исполнитель: Фамилия, Имя, Отчество'},
            {'field': 'phone', 'caption': 'Исполнитель: Телефон','type': 'phone'},
            {'field': 'email','caption': 'Исполнитель: Электронная почта','type': 'email'},
        ],
        'tables': {
            'main_table': {
                'tab_name': 'Таблица',
                'headers': [
                    {'Операция': ['OpCod', 'OpDate']},
                    {'Закрытый радионуклидный источник': ['PaspN', 'Typ', 'Nuclid', 'Numb', 'Sht', 'Activn', 'IzgotOKPO', 'IzgotDate', 'Kateg', 'Nss']},
                    {'Право собственности на ЗРИ': ['FormSobst', 'Pravoobl']},
                    {'Документ': ['DocVid', 'DocN', 'DocDate']},
                    {'ОКПО': ['OkpoPIP', 'OkpoPrv']},
                    {'Прибор (установка УКТ или иная упаковка)': ['PrName', 'UktPrTyp', 'UktPrN']}
                ],
                'model': [
                    {'field': 'OpCod','caption': 'Код','type': '11_type'},
                    {'field': 'OpDate','caption': 'Дата','type': 'date'},
                    {'field': 'PaspN','caption': 'Номер паспорта','type': 'str'},
                    {'field': 'Typ','caption': 'Тип','type': '23_type', 'map': {'Nuclid': '4'}},
                    {'field': 'Nuclid','caption': 'Радионуклиды','type': 'radionuclide_voc_type'},
                    {'field': 'Numb','caption': 'Номер','type': 'str'},
                    {'field': 'Sht','caption': 'Количество, шт.','type': 'int'},
                    {'field': 'Activn','caption': 'Активность, Бк','type': 'exponential'},
                    {'field': 'IzgotOKPO','caption': 'ОКПО Изготовителя','type': 'OKPO_type'},
                    {'field': 'IzgotDate','caption': 'Дата выпуска','type': 'date'},
                    {'field': 'Kateg','caption': 'Категория','type': 'CalcCategoryZri'},
                    {'field': 'Nss','caption': 'НСС, месяцев','type': 'int'},
                    {'field': 'FormSobst','caption': 'код формы собственности','type': 'FormSobst_type'},
                    {'field': 'Pravoobl','caption': 'правообладатель','type': 'OKPO_type'},
                    {'field': 'DocVid','caption': 'Вид','type': '12_type'},
                    {'field': 'DocN','caption': 'Номер'},
                    {'field': 'DocDate','caption': 'Дата','type': 'date'},
                    {'field': 'OkpoPIP','caption': 'Поставщика или получателя','type': 'OKPO_type'},
                    {'field': 'OkpoPrv','caption': 'Перевозчика','type': 'OKPO_type'},
                    {'field': 'PrName','caption': 'Наименование','type': '27_type','map': {'UktPrTyp': '2'}},
                    {'field': 'UktPrTyp','caption': 'Тип'},
                    {'field': 'UktPrN','caption': 'Номер','type': 'str'},
                ]
            }
        },
        'export': [
            {
                'form_name': 'Отчетная форма 1.1',
                'object_type': 'f1-1_2021_exp',
                'output_ext': ['docx', 'pdf', 'xlsx']
            }
        ]
    }


form_models = {
    'test_form1' : test_form1
}

