'''
Обработчики событий.

Интерфейс к fronend-событиям определен в sguk_events.py
Интерфейс к core-репозиторию в sguk_core_repo.py
'''

from sguk_forms import *

api = None

def init(_api):
    global api
    api = _api

    # Регистрируем обработчики событий в репозитории
    api.repo.ReportForms.find_form(form_1_1_2022).add_frontend_event('OpCod_OKPO', form_1_1_event)
    api.repo.ReportForms.find_form(form_1_2_2022).add_frontend_event('OpCod_OKPO', form_1_2_event)
    api.repo.ReportForms.find_form(form_1_3_2022).add_frontend_event('OpCod_OKPO', form_1_3_event)
    api.repo.ReportForms.find_form(form_1_4_2022).add_frontend_event('OpCod_OKPO', form_1_4_event)
    api.repo.ReportForms.find_form(form_1_5_2022).add_frontend_event('OpCod_OKPO', form_1_5_event)
    api.repo.ReportForms.find_form(form_1_6_2022).add_frontend_event('OpCod_OKPO', form_1_6_event)
    api.repo.ReportForms.find_form(form_1_7_2022).add_frontend_event('OpCod_OKPO', form_1_7_event)
    api.repo.ReportForms.find_form(form_1_8_2022).add_frontend_event('OpCod_OKPO', form_1_8_event)

    # Получаем URL обработчика для хранилища
    url = api.lib.frontend_events.get_url('ReportForms')

    # Добавляем в описания полей вызовы REST API на события
    api.lib.core.repo.updateTableFieldModel(form_1_1_2022, 'OpCod', on_change=url)
    api.lib.core.repo.updateTableFieldModel(form_1_2_2022, 'OpCod', on_change=url)
    api.lib.core.repo.updateTableFieldModel(form_1_3_2022, 'OpCod', on_change=url)
    api.lib.core.repo.updateTableFieldModel(form_1_4_2022, 'OpCod', on_change=url)
    api.lib.core.repo.updateTableFieldModel(form_1_5_2022, 'OpCod', on_change=url)
    api.lib.core.repo.updateTableFieldModel(form_1_6_2022, 'OpCod', on_change=url)
    api.lib.core.repo.updateTableFieldModel(form_1_7_2022, 'g2', on_change=url)
    api.lib.core.repo.updateTableFieldModel(form_1_8_2022, 'g2', on_change=url)


okpo_cache = {}

from sguk_events import EventContext

def get_okpo(org_id):
    if org_id not in okpo_cache:
        okpo_cache[org_id] = api.repo.Vocs.VocOrgs.ds.read(org_id).OKPO
    
    return okpo_cache[org_id]


def get_OpCod_OKPO(context:EventContext):
    OpCod = context.selected_cell.value
    OKPO = get_okpo(context.page_data.author_id)

    return OpCod, OKPO


def form_1_1_event(context:EventContext):
    if context.selected_cell.field != 'OpCod': return

    OpCod, OKPO = get_OpCod_OKPO(context)
    defs = {}

    if OpCod in '11 58':
        defs.update(IzgotOKPO=OKPO)

    if OpCod in '10, 11, 12, 15, 17, 18, 41, 42, 43, 46, 53, 58, 61, 62, 65, 67, 68, 71, 72, 72, 73, 74, 75, 76':
        defs.update(OkpoPIP=OKPO)

    if OpCod in '10, 11,12, 15, 17, 18, 41, 42, 43, 46, 53, 54, 58, 65, 66, 67, 68, 71, 72, 73, 74, 75, 76, 97, 98':
        defs.update(OkpoPrv='-')

    return {'defaults':defs}


def form_1_2_event(context:EventContext):
    if context.selected_cell.field != 'OpCod': return

    OpCod, OKPO = get_OpCod_OKPO(context)
    defs = {}

    if OpCod in '11, 58':
        defs.update(IzgotOKPO=OKPO)

    if OpCod in '10, 11, 12, 15, 17, 18, 41, 42, 46, 53, 58, 61, 62, 67, 68, 71, 72, 72, 73, 74, 75, 76':
        defs.update(OkpoPIP=OKPO)

    if OpCod in '10, 11,12, 17, 18, 41, 42, 46, 53, 54, 58, 65, 66, 67, 68, 71, 72, 73, 74, 75, 76, 97, 98':
        defs.update(OkpoPrv='-')

    return {'defaults':defs}


def form_1_3_event(context:EventContext):
    if context.selected_cell.field != 'OpCod': return

    OpCod, OKPO = get_OpCod_OKPO(context)
    defs = {}

    if OpCod in '11, 58':
        defs.update(okpo=OKPO)

    if OpCod in '10, 11, 12, 15, 17, 18, 41, 42, 43, 46, 53, 58, 61, 62, 65, 67, 68, 71, 72, 72, 73, 74, 75, 76':
        defs.update(post=OKPO)

    if OpCod in '10, 11,12, 17, 18, 41, 42, 46, 53, 54, 58, 65, 66, 67, 68, 71, 72, 73, 74, 75, 76, 97, 98':
        defs.update(per='-')

    return {'defaults':defs}


def form_1_4_event(context:EventContext):
    if context.selected_cell.field != 'OpCod': return

    OpCod, OKPO = get_OpCod_OKPO(context)
    defs = {}

    if OpCod in '10, 11, 12, 15, 17, 18, 41, 42, 43, 46, 53, 58, 61, 62, 67, 68, 71, 72, 72, 73, 74, 75, 76':
        defs.update(OkpoPIP=OKPO)

    if OpCod in '10, 11,12, 15, 17, 18, 41, 42, 43, 46, 53, 54, 58, 65, 67, 68, 71, 72, 73, 74, 75, 76, 97, 98':
        defs.update(OkpoPrv='-')

    return {'defaults':defs}


def form_1_5_event(context:EventContext):
    if context.selected_cell.field != 'OpCod': return

    OpCod, OKPO = get_OpCod_OKPO(context)
    defs = {}

    if OpCod in '14, 28, 38, 41':
        defs.update(BCod=OKPO)

    if OpCod in '76':
        defs.update(BCod=6)

    if OpCod in '01, 10, 14, 18, 41, 43, 44, 45, 48, 49, 51, 52, 57, 59, 68, 71, 72, 73, 74, 75, 76, 97, 98, 99':
        defs.update(OkpoPIP=OKPO)

    if OpCod in '01, 10, 14, 18, 41, 43, 44, 45, 48, 49, 51, 52, 57, 59, 71, 72, 73, 74, 75, 76,  97, 98, 99':
        defs.update(OkpoPrv='-')

    if OpCod in '01, 10, 14, 18, 21, 22, 25, 26, 27, 28, 29, 31, 32, 35, 36, 37, 38, 39, 43, 48, 51, 52, 63, 64, 68, 71, 72, 73, 74, 75, 76,  84, 88, 97, 98, 99':
        defs.update(CodRAO='-')

    return {'defaults':defs}


def form_1_6_event(context:EventContext):
    if context.selected_cell.field != 'OpCod': return

    OpCod, OKPO = get_OpCod_OKPO(context)
    defs = {}

    if OpCod in '11, 12, 13, 14, 28, 38, 41':
        defs.update(BCod=OKPO)

    if OpCod in '76':
        defs.update(BCod=6)

    if OpCod in '01, 10, 11, 12, 13, 14, 16, 18, 41, 42, 43, 44, 45, 48, 49, 51, 52, 56, 57, 59, 68, 71, 72, 73, 74, 75, 76, 97, 98':
        defs.update(OkpoPIP=OKPO)

    if OpCod in '01, 10, 11, 12, 13, 14, 16, 18, 41, 42, 43, 44, 45, 48, 49, 51, 52, 56, 57, 59, 71, 72, 73, 74, 75, 76,  97, 98, 99':
        defs.update(OkpoPrv='-')

    if OpCod in '01, 10, 11, 12, 13, 14, 16, 18, 21, 22, 25, 26, 27, 28, 29, 31, 32, 35, 36, 37, 38, 39, 42, 43, 48, 51, 52, 63, 64, 68, 71, 72, 73, 74, 75, 76,  97, 98, 99':
        defs.update(UpCod='-')

    return {'defaults':defs}


def form_1_7_event(context:EventContext):
    if context.selected_cell.field != 'g2': return

    OpCod, OKPO = get_OpCod_OKPO(context)
    defs = {}

    if OpCod in '01, 10, 11, 12, 13, 14, 16, 43, 51, 52, 55':
        defs.update(g17=OKPO)

    if OpCod in '01, 10, 11, 12, 13, 14, 16, 43, 51, 52, 55, 97, 98':
        defs.update(g18='-')

    if OpCod in '11, 12, 13, 14':
        defs.update(g22=OKPO)

    if OpCod in '01, 10, 11, 12, 13, 14, 16, 21, 22, 25, 26, 27, 28, 29, 31, 32, 35, 36, 37, 38, 39, 43, 51, 52, 63, 64, 97, 98':
        defs.update(g30='-')

    return {'defaults':defs}


def form_1_8_event(context:EventContext):
    if context.selected_cell.field != 'g2': return

    OpCod, OKPO = get_OpCod_OKPO(context)
    defs = {}

    if OpCod in '01, 10, 11, 12, 13, 18, 51, 52, 55, 68':
        defs.update(g14=OKPO)

    if OpCod in '01, 10, 11, 12, 13, 18, 51, 52, 55, 68, 97, 98':
        defs.update(g15='-')

    if OpCod in '11, 12, 13, 14':
        defs.update(g19=OKPO)

    if OpCod in '01, 10, 11, 12, 13, 18,21, 25, 26, 27, 28, 29, 31, 32, 35, 36, 37, 38, 39, 51, 52, 63, 64, 68, 97, 98':
        defs.update(g26='-')

    return {'defaults':defs}
