import datetime

from sguk_ide import *

api = None

def julian_date(date, *args, **kwargs):
    if not date: return None
    
    return (f"julianday('{date}')", 'raw')


def default(value, row, *args, **kwargs):
    if value!=None:
        return value
    else:
        return args[0]

def bool_int(value, *args, **kwargs):
    if type(value)==str and value.lower() in ('да yes y true'):
        return 1
    elif type(value)==int and value!=0:
        return 1
    else:
        return 0


def to_int(value, *args, **kwargs):
    if type(value) in (int, float):
        return int(value)

    elif type(value)==str:
        try:
            return int(value)
        except:
            pass
    
        try:
            value = float(value)
            return int(value)
        except:
            pass
    
    return value 


def form_num(value, *args, **kwargs):                       # Определение номера отчетной формы ReportForms.form_1_(1)
    if type(value) == str:
        value = value.split('.')[0]
        value = value.split('_')[-1]
    num = int(value)
    return num

def update_territories(value, row, *args, **kwargs):
    export = kwargs['export']
    rec = export.map_record(row, 'Territories')
    export.sqlite_insert(kwargs['api'], 'Territories', rec)
    return value

def update_receivers(value, row, *args, **kwargs):
    export = kwargs['export']
    rec = export.map_record(row, 'Receivers')
    export.sqlite_insert(kwargs['api'], 'Receivers', rec)
    return value


from sguk_forms import *
import sguk_export_cp

def init(_api:SgukApi):
    global api
    api = _api

    api.repo.ReportForms.ActionExchange.add_action('delimiter', caption='-------------------------', index='8')
    api.repo.ReportForms.ActionExchange.add_action(
            'export_cp',                                # Название действия
            sguk_export_cp.action_export_forms_cp,      # Функция выполнения действия
            caption='Выгрузить в ЦЧ',                   # Название кнопки на панели
            index = '9',
            icon='/interface/upload.svg',               # Файл с иконкой для кнопки
            text='Выгрузить в ЦЧ'                       # Хинт к кнопку
        )


    module:sguk_export_cp.ExportFormsCP = api.find(sguk_export_cp.MODULE_PATH)._data

    module.export_templates = {         # Имена файлов БД с шаблонами
            'form_1_*' : 'OperForms.db',
            'form_2_*' : 'YearForms.db',
            'form_3_*' : 'BorderForms.db',
            'form_4_*' : 'RiacForms.db',
            'form_5_*' : 'ViacForms.db',
        }

    module.add_map('Comments', {        # Отображение на таблицу Comments
            'FormType' : (0,),
            'RowId'    : 'pointer.row',
            'CellIndex': ('pointer.col', default, 0),
            'Comment'  : 'value.text'
        }, {'target':'Comments'})

    module.add_map('VocOrgs', {         # Отображение на таблицу Organizations
            'Id'                 : 'value_id',
            'OrganizationType'   : ('ORG_TYPE', default, 0),
            'RegistrationNumber' : 'KOD_ORG',
            'Regions'            : 'KOD_SRF',
            'ParentId'           :	'KOD_GOL_ORG',
            'ShortName'          : 'NAME_SHORT',
            'Name'               : 'NAME_FULL',
            'JrAddressCity'      : 'ADDRESS_U',
            'FactAddressCity'    : 'ADDRESS_P',
            'Phone'              : 'TLFON',
            'Fax'                : 'FAX',
            'EMail'              : 'EMAIL',
            'OKPO'               : 'OKPO',
            'OKVED'              : 'OKVED',
            'OKOGU'              : 'OKOGU',
            'OKTMO'              : 'OKTMO',
            'INN'                : 'INN',
            'KPP'                : 'KPP',
            'OKOPF'              : 'OKOPF',
            'OKFS'               : 'OKFS',
            'HeadPosition'       : 'RUKOVODITEL',
            'DateRegistration'   : 'DT_REG'
        }, {'target':'Organizations'})

    module.add_map('voc_24', {          # Справочник ИОУ (voc_24)
            'OrganizationId'   : 'author_id',
            'DeviceName'       : '2',
            'DeviceType'       : '3',
            'Name'             : '6',
            'DeviceMass'       : ('5', default, 0),
            'Manufacturer'     : ('',),
            'Nss'              : (0,),	
            'Category'         : None,	
            'DeviceDimensions' : '4',
            'Index'            : '7',
            'MinMass'          : None,	
            'MaxMass'          : '8',
            'Document'         : '9',
            'Comment'          : None	
        }, {'target':'DictIOU'})

    module.add_map('PH_Name', {         # Справочник ПХ 
            'OrganizationId'   : 'author_id',	
            'Name'             : '6',
            'Code'             : '7',
            'DateStart'        : ('10', julian_date),
            'DateEnd'          : ('11', julian_date),
            'Term'             : None,	
            'ActName'          : None,	
            'UseInCadastre'    : ('26', bool_int),
            'CadastreNumber'   : '9'
        }, {'target':'DictPh'})

    module.add_map('PrName', {          # Справочник УКТ 
            'OrganizationId' : 'author_id',     # Первичный ключ: OrganizationId, Name, Type
            'Name'           : 'x',
            'Type'           : '2',
            'Purpose'        : '4',
            'InnerVolume'    : '8',
            'OuterVolume'    : '7',
            'MassNetto'      : None,
            'MassBrutto'     : None,
            'NSS'            : None,	
            'Cost'           : None,	
            'Material'       : '5',
            'Manufacturer'   : None,	
            'ClassRao'       : None,	
            'PZRO'           : None	
        }, {'target':'DictUkt'})

    module.add_map('voc_23_full', {     # Справочник ЗРИ (23_voc1)
            'OrganizationId'     : 'author_id',
            'TypeModel'          : '3',
            'Radionuclides'      : '4',
            'TypeCapsule'        : None,	
            'RadiationModeIndex' : '5',
            'Dimensions'         : '6',
            'MinActivity'        : '7',
            'MaxActivity'        : '8',
            'RadiationCharacteristics' : None,
            'Category'           : '11',
            'StrengthClass'      : None,	
            'ManufacturerOKPO'   :'10',
            'Purpose'            : None,	
        }, {'target':'DictZri'})

    module.add_map('Territories', {     # Участки загрязненных территорий (из Ф.2.10)
            'OrganizationId' : 'author_id',
            'Name'           : 'g3',
            'CadastreNumber' : 'g4',
            'Code'           : 'g5',
        }, {'target':'DictContaminatedTerritories'})

    module.add_map('UpCod', {           # Установки переработки 
            'OrganizationId'    : 'author_id', 
            'Name'              : '1',
            'Code'              : '2',
            'PackType'          : None, 
            'TempStorage'       : None, 
            'DateUsageBegin'    : None, 
            'DateUsageEnd'      : None, 
            'Power'             : '3',
            'WorkHours'         : '4',
            'VolumeCoefficient' : '5',
            'MassCoefficient'   : '6',
            'Salinity'          : '7',
        }, {'target':'DictProcessingPlants'})

    module.add_map('Receivers', {       # Приемники отведенных вод (Ф.2.8)
            'OrganizationId' : 'author_id',
            'Name'           : 'g3',
            'Code'           : 'g4',
            'Pool'           : 'g5',
        }, {'target':'DictWaterReceivers'})

# --- Оперативные отчеты ------------------------------------------------------------

    module.add_map('form_1_*.Forms', {  # Отображение на таблицу Forms
            'Type'             : ('form_no', form_num),
            'OrganizationId'   : ('author_id', 'VocOrgs', False),        # Экспорт ссылки на справочник Организаций
            'ApprovalState'    : (0,),
            'DateStart'        : ('form_period_start', julian_date),
            'DateEnd'          : ('form_period_end', julian_date),
            'CorrectionNumber' : ('adjustment_no', default, 0),
            'RiacState'        : (0,),
            'ViacState'        : (0,),
            'CiacState'        : (0,),
        }, {'target':'Forms'}) 

    module.add_map('form_1_*.header', { # Отображение форм на таблицу FormHeader
            'FormId'            : 'FormId',
            'PerformerPosition' : ('',),
            'PefrormerFio'      : ('',),
            'PerformerPhone'    : ('',),
            'PerformerEmail'    : ('',),
        }, {'target':'FormHeader'})

    module.add_map(form_1_1_2022, {        # Отображение на таблицу Form1
            'FormId'            : 'FormId',
            'Index'             : 'Index',
            'OperationCode'     : 'OpCod', 
            'OperationDate'     : ('OpDate', julian_date),

            'PassportNumber'    : 'PaspN',
            'ZriType'           : ('Typ', 'voc_23_full', False),
            'Radionuclides'     : 'Nuclid',
            'ZriNumber'         : 'Numb',
            'ZriCount'          : 'Sht',
            'Activity'          : 'Activn',
            'ManufacturerOkpo'  : 'IzgotOKPO',
            'ReleaseDate'       : 'IzgotDate',
            'Category'          : 'Kateg',
            'NSS'               : 'Nss',
            'OwnershipCode'     : 'FormSobst',
            'OwnerOkpo'         : 'Pravoobl',
            'DocumentType'      : 'DocVid',
            'DocumentNumber'    : 'DocN',
            'DocumentDate'      : 'DocDate',
            'SupplierOkpo'      : 'OkpoPIP',
            'TransportOkpo'     : 'OkpoPrv',
            'UktName'           : ('PrName', 'PrName', False),
            'UktType'           : 'UktPrTyp',
            'UktNumber'         : 'UktPrN',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form1'})

    module.add_map(form_1_2_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',
            'OperationCode'     : 'OpCod',
            'OperationDate'     : ('OpDate', julian_date),

            'PassportNumber'    : 'PaspN',
            'IouName'           : ('Typ', 'voc_24', False),
            'IouNumber'         : 'Numb',
            'IouMass'           : 'Kg',
            'ManufacturerOkpo'  : 'IzgotOKPO',
            'ReleaseDate'       : 'IzgotDate',
            'Nss'               : 'Nss',
            'OwnershipCode'     : 'FormSobst',
            'OwnerOkpo'         : 'Pravoobl',
            'DocumentType'      : 'DocVid',
            'DocumentNumber'    : 'DocN',
            'DocumentDate'      : 'DocDate',
            'SupplierOkpo'      : 'OkpoPIP',
            'TransportOkpo'     : 'OkpoPrv',
            'UktName'           : ('PrName', 'PrName', False),
            'UktType'           : 'UktPrTyp',
            'UktNumber'         : 'UktPrN',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form2'})

    module.add_map(form_1_3_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',
            'OperationCode'     : 'OpCod',
            'OperationDate'     : ('OpDate', julian_date),

            'PassportNumber'	: 'PaspN',
            'OriType'           : ('Typ', 'voc_23_full', False),
            'Radionuclides'     : 'Nuclid',
            'OriNumber'         : 'Numb',
            'Activity'          : 'Activn',
            'ManufacturerOkpo'  : 'IzgotOKPO',
            'ReleaseDate'       : 'IzgotDate',
            'AggregateState'    : 'Agr',
            'OwnershipCode'     : 'FormSobst',
            'OwnerOkpo'         : 'Pravoobl',
            'DocumentType'      : 'DocVid',
            'DocumentNumber'    : 'DocN',
            'DocumentDate'      : 'DocDate',
            'SupplierOkpo'      : 'OkpoPIP',
            'TransportOkpo'     : 'OkpoPrv',
            'UktName'           : ('PrName', 'PrName', False),
            'UktType'           : 'UktPrTyp',
            'UktNumber'         : 'UktPrN',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form3'})

    module.add_map(form_1_4_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',
            'OperationCode'     : 'OpCod',
            'OperationDate'     : ('OpDate', julian_date),

            'PassportNumber'    : 'PaspN',
            'OriName'           : 'IstName',
            'OriType'           : 'Typ',
            'Radionuclides'     : 'Nuclid',
            'Activity'          : 'Activn',
            'MeasureDate'       : 'ActDate',
            'Volume'            : 'Kbm',
            'Mass'              : 'Kg',
            'AggregateState'    : 'Arg',
            'OwnershipCode'     : 'FormSobst',
            'OwnerOkpo'         : 'Pravoobl',
            'DocumentType'      : 'DocVid',
            'DocumentNumber'    : 'DocN',
            'DocumentDate'      : 'DocDate',
            'SupplierOkpo'      : 'OkpoPIP',
            'TransportOkpo'     : 'OkpoPrv',
            'UktName'           : ('PrName', 'PrName', False),
            'UktType'           : 'UktPrTyp',
            'UktNumber'         : 'UktPrN',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form4'})

    module.add_map(form_1_5_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',
            'OperationCode'     : 'OpCod',
            'OperationDate'     : ('OpDate', julian_date),

            'PassportNumber'    : 'PaspN',
            'OzriType'          : ('Typ', 'voc_23_full', False),
            'Radionuclides'     : 'Nuclid',
            'OzriNumber'        : 'Numb',
            'OzriCount'         : 'Sht',
            'Activity'          : 'Activn',
            'ReleaseDate'       : 'IzgotDate',
            'StatusRao'         : 'BCod',
            'DocumentType'      : 'DocVid',
            'DocumentNumber'    : 'DocN',
            'DocumentDate'      : 'DocDate',
            'SupplierOkpo'      : 'OkpoPIP',
            'TransportOkpo'     : 'OkpoPrv',
            'UktName'           : ('PrName', 'PrName', False),
            'UktType'           : 'UktPrTyp',
            'UktNumber'         : 'UktPrN',
            'PhName'            : 'PH_Name',
            'PhCode'            : 'PH_Cod',
            'ProcessingCode'    : 'CodRAO',
            'Subsidy'           : 'Subsid',
            'FCP'               : 'FCP',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form5'})

    module.add_map(form_1_6_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',
            'OperationCode'     : 'OpCod',
            'OperationDate'     : ('OpDate', julian_date),

            'CodeRao'           : 'RAOCod',
            'StatusRao'         : 'BCod',
            'Volume'            : 'Kbm',
            'Mass'              : 'Tonne',
            'Count'             : 'Sht',
            'Radionuclides'     : 'RAOCodMax',
            'SumActivityTritiy' : 'ActTR',
            'SumActivityBeta'   : 'ActBG',
            'SumActivityAlpha'  : 'ActA',
            'SumActivityTrans'  : 'ActUR',
            'MeasureDate'       : 'ActDate',
            'DocumentType'      : 'DocVid',
            'DocumentNumber'    : 'DocN',
            'DocumentDate'      : 'DocDate',
            'SupplierOkpo'      : 'OkpoPIP',
            'TransportOkpo'     : 'OkpoPrv',
            'PhName'            : 'PH_Name',
            'PhCode'            : 'PH_Cod',
            'ProcessingCode'    : 'UpCod',
            'UktName'           : ('PrName', 'PrName', False),
            'UktType'           : 'UktPrTyp',
            'UktNumber'         : 'UktPrN',
            'Subsidy'           : 'Subsid',
            'FCP'               : 'FCP',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form6'})

    module.add_map(form_1_7_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',
            'OperationCode'     : 'g2',
            'OperationDate'     : ('g3', julian_date),

            'UktName'           : ('g4', 'PrName', False),
            'UktType'           : 'g5',
            'UktNumber'         : 'g6',
            'PackCode'          : 'g7',
            'PackDate'          : 'g8',
            'PackPassportNumber' : 'g9',
            'VolumeBrutto'      : 'g10',
            'MassBrutto'        : 'g11',
            'Radionuclides'     : 'g12',
            'Activitiy'         : 'g13',
            'DocumentType'      : 'g14',
            'DocumentNumber'    : 'g15',
            'DocumentDate'      : 'g16',
            'SupplierOkpo'      : 'g17',
            'TransportOkpo'     : 'g18',
            'PhName'            : 'g19',
            'PhCode'            : 'g20',
            'CodeRao'           : 'g21',
            'StatusRao'         : 'g22',
            'Volume'            : 'g23',
            'Mass'              : 'g24',
            'Count'             : 'g25',
            'SumActivityTritiy' : 'g26',
            'SumActivityBeta'   : 'g27',
            'SumActivityAlpha'  : 'g28',
            'SumActivityTrans'  : 'g29',
            'ProcessingCode'    : 'g30',
            'Subsidy'           : 'g31',
            'FCP'               : 'g32',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form7'})

    module.add_map(form_1_8_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',
            'OperationCode'     : 'g2',
            'OperationDate'     : ('g3', julian_date),

            'UktNumber'         : 'g4',
            'PassportNumber'    : 'g5',
            'VolumeBrutto'      : 'g6',
            'MassBrutto'        : 'g7',
            'Salinity'          : 'g8',
            'Radionuclides'     : 'g9',
            'Activity'          : 'g10',
            'DocumentType'      : 'g11',
            'DocumentNumber'    : 'g12',
            'DocumentDate'      : 'g13',
            'SupplierOkpo'      : 'g14',
            'TransportOkpo'     : 'g15',
            'PhName'            : 'g16',
            'PhCode'            : 'g17',
            'CodeRao'           : 'g18',
            'StatusRao'         : 'g19',
            'Volume'            : 'g20',
            'Mass'              : 'g21',
            'SumActivityTritiy' : 'g22',
            'SumActivityBeta'   : 'g23',
            'SumActivityAlpha'  : 'g24',
            'SumActivityTrans'  : 'g25',
            'ProcessingCode'    : 'g26',
            'Subsidy'           : 'g27',
            'FCP'               : 'g28',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form8'})

    module.add_map(form_1_9_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',
            'OperationCode'     : 'g2',
            'OperationDate'     : ('g3', julian_date),

            'DocumentType'      : 'g4',
            'DocumentNumber'    : 'g5',
            'DocumentDate'      : 'g6',
            'ObjectTypeCode'    : 'g7',
            'Radionuclides'     : 'g8',
            'Activity'          : 'g9',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form9'})

# --- Годовые отчеты ------------------------------------------------------------

    module.add_map('form_2_*.Forms', {           # Отображение на таблицу Forms
            'Type'             : ('form_no', form_num),
            'OrganizationId'   : ('author_id', 'VocOrgs', True),        # Экспорт ссылки на справочник Организаций
            'Year'             : ('reporting_year', default, 2022),
            'CorrectionNumber' : ('adjustment_no', default, 0),
            'ApprovalState'    : (0,),
            'RiacState'        : (0,),
            'ViacState'        : (0,),
            'CiacState'        : (0,),
            'FillDate'         : None,            
        }, {'target':'Forms'}) 

    module.add_map('form_2_*.header', {        # Отображение форм на таблицу FormHeader
            'FormId'   : 'FormId',
            'PerformerPosition' : ('',),
            'PefrormerFio'      : ('',),
            'PerformerPhone'    : ('',),
            'PerformerEmail'    : ('',),
        }, {'target':'FormHeader'})

    module.add_map(form_2_1_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',

            'PlantName'               : ('g2', 'voc_28', False),
            'PlantCode'               : 'g3',
            'Power'                   : 'g19',
            'WorkHours'               : 'g4',
            'ReceivedCodeRao'         : 'g5',
            'ReceivedStatusRao'       : 'g6',
            'ReceivedVolume'          : 'g7',
            'ReceivedMass'            : 'g8',
            'ReceivedCount'           : ('g9', to_int),
            'ReceivedActivityTritiy'  : 'g20',
            'ReceivedActivityBeta'    : 'g21',
            'ReceivedActivityAlpha'   : 'g10',
            'ReceivedActivityTrans'   : 'g11',
            'ProcessedCodeRao'        : 'g12',
            'ProcessedStatusRao'      : 'g13',
            'ProcessedVolume'         : 'g14',
            'ProcessedMass'           : 'g15',
            'ProcessedCount'          : 'g16',
            'ProcessedActivityTritiy' : 'g22',
            'ProcessedActivityBeta'   : 'g23',
            'ProcessedActivityAlpha'  : 'g17',
            'ProcessedActivityTrans'  : 'g18',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form1'})

    module.add_map(form_2_2_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',

            'PhName'            : ('g2', 'PH_Name', False),
            'PhCode'            : 'g3',
            'UktName'           : ('g11', 'PrName', False),
            'UktType'           : 'g12',
            'Count'             : 'g13',
            'CodeRao'           : 'g4',
            'StatusRao'         : 'g5',
            'VolumeWithoutPack' : 'g6',
            'VolumeWithPack'    : 'g7',
            'MassWithoutPack'   : 'g8',
            'MassWithPack'      : 'g9',
            'OzriCount'         : 'g10',
            'SumActivityTritiy' : 'g19',
            'SumActivityBeta'   : 'g15',
            'SumActivityAlpha'  : 'g14',
            'SumActivityTrans'  : 'g20',
            'Radionuclides'     : 'g16',
            'Subsidy'           : 'g21',
            'FCP'               : 'g22',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form2'})

    module.add_map(form_2_3_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',

            'PhName'            : 'g2',
            'PhCode'            : 'g3',
            'ProjectVolume'     : 'g13',
            'CodeRao'           : 'g4',
            'AllowedVolume'     : 'g5',
            'AllowedMass'       : 'g6',
            'AllowedCount'      : 'g7',
            'SumActivity'       : 'g8',
            'DocumentNumber'    : 'g9',
            'DocumentDate'      : 'g10',
            'DocumentTerm'      : 'g11',
            'DocumentName'      : 'g12',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form3'})

    module.add_map(form_2_4_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',

            'CodeOyat'          : 'g2',
            'FCP'               : 'g17',
            'FormedMass'        : 'g3',
            'FormedCount'       : 'g4',
            'ReceivedMass'      : 'g5',
            'ReceivedCount'     : 'g6',
            'ImportMass'        : 'g7',
            'ImportCount'       : 'g8',
            'RegisteredMass'    : 'g9',
            'RegisteredCount'   : 'g10',
            'TransferredMass'   : 'g11',
            'TransferredCount'  : 'g12',
            'ProcessedMass'     : 'g13',
            'ProcessedCount'    : 'g14',
            'UnregisteredMass'  : 'g15',
            'UnregisteredCount' : 'g16',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form4'})

    module.add_map(form_2_5_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',

            'PhName'            : 'g2',
            'PhCode'            : 'g3',
            'CodeOyat'          : 'g4',
            'FCP'               : 'g10',
            'Mass'              : 'g5',
            'MassBrutto'        : 'g6',
            'Count'             : 'g7',
            'SumActivityAlpha'  : 'g8',
            'SumActivityBeta'   : 'g9',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form5'})

    module.add_map(form_2_6_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',

            'WellName'          : 'num_s',
            'ZoneName'          : 'zone',
            'RwSource'          : 'ist',
            'Distance'          : 'rust',
            'Depth'             : 'gl',
            'Radionuclides'     : 'nuk',
            'AverageRadionuclideContent' : 'radio',
            'WellCount'         : 'quantity',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form6'})

    module.add_map(form_2_7_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',

            'Name'              : 'g2',
            'Radionuclide'      : 'g3',
            'AllowedActivity'   : 'g4',
            'ActualActivity'    : 'g5',
            'PreviousActivity'  : 'g6',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form7'})

    module.add_map(form_2_7_2022+'.header', {           # Таблица Form7Header
            'FormId'            : 'FormId',

            'PermissionName'    : 'g7',
            'PermissionFrom'    : 'g8',
            'PermissionTermFrom' : 'form_period_start',
            'PermissionTermTo'  : 'form_period_end',
            'DocumentName'      : 'g9',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form7Header'})

    module.add_map(form_2_8_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',

            'WastewaterName'    : ('g2', update_receivers),
            'Name'              : 'g3',
            'ReceiverCode'      : 'g4',
            'WaterObjectCode'   : 'g5',
            'AllowedVolume'     : 'g6',
            'ActualVolume'      : 'g7',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form8'})

    module.add_map(form_2_8_2022+'.header', {
            'FormId'            : 'FormId',

            'Permission1Name'   : 'g11',
            'Permission1From'   : 'g12',
            'Permission1TermFrom' : 'per1',
            'Permission1TermTo' : 'to1',
            'Permission1DocumentName' : 'g13',
            'Permission2Name'   : 'g21',
            'Permission2From'   : 'g22',
            'Permission2TermFrom' : 'per2',
            'Permission2TermTo' : 'to2',
            'Permission2DocumentName' : 'g23',
            'ContractName'      : 'g31',
            'ContractFrom'      : 'g32',
            'ContractTermFrom'  : 'per3',
            'ContractTermTo'    : 'to3',
            'ContractOrganization' : 'g33',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form8'})

    module.add_map(form_2_9_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',

            'Name'              : 'g2',
            'Radionuclide'      : 'g3',
            'AllowedActivity'   : 'g4',
            'ActualActivity'    : 'g5',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form9'})

    module.add_map(form_2_10_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',

            'IndicatorName'     : 'g2',
            'Name'              : ('g3', update_territories),
            'CadastreNumber'    : 'g4',
            'Code'              : 'g5',
            'Area'              : 'g6',
            'AverageDose'       : 'g7',
            'MaxDose'           : 'g8',
            'DensityAlpha'      : 'g9',
            'DensityBeta'       : 'g10',
            'FCP'               : 'g11',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form10'})

    module.add_map(form_2_11_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',

            'Name'              : 'g2',
            'CadastreNumber'    : 'g3',
            'Code'              : 'g4',
            'Area'              : 'square',
            'Radionuclide'      : 'g5',
            'LandPlot'          : 'g6',
            'LiquidPhase'       : 'g7',
            'BottomSediments'   : 'g8',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form11'})

    module.add_map(form_2_12_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',

            'OperationCode'     : 'g2',
            'ObjectType'        : 'g3',
            'Radionuclides'     : 'g4',
            'Activity'          : 'g5',
            'SupplierOkpo'      : 'g6',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form12'})

# --- Отчеты экспортера ------------------------------------------------------

    module.add_map('form_3_*.Forms', {           # Отображение на таблицу Forms
            'Type'             : ('form_no', form_num),
            'OrganizationId'   : ('author_id', 'VocOrgs', True),        # Экспорт ссылки на справочник Организаций
            'Year'             : ('reporting_year', default, 2022),
            'CorrectionNumber' : ('adjustment_no', default, 0),
            'ApprovalState'    : (0,),
            'RiacState'        : (0,),
            'ViacState'        : (0,),
            'CiacState'        : (0,),
            'FillDate'         : None,            
        }, {'target':'Forms'}) 


    module.add_map(form_3_1_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',

            'Radionuclides'     : 'g17',
            'Count'             : 'g18',
            'Activity'          : 'g19',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,

        }, {'target':'Form1'})

    module.add_map(form_3_1_2022+'.header', {
            'FormId'            : 'FormId',

            'ReceiverName'      : 'g2',
            'ReceiverJrAddress' : 'g3',
            'ReceiverFactAddress' : 'g4',
            'LicenseNumber'     : 'g5',
            'LicenseTerm'       : 'g6',
            'RequestTerm'       : 'g7',
            'UserName'          : 'g8',
            'UserJrAddress'     : 'g9',
            'UserFactAddress'   : 'g10',
            'UserPhone'         : 'g11',
            'UserEmail'         : 'g12',
            'UsageArea'         : 'g13',
            'ContractNumber'    : 'g14',
            'ContractDate'      : 'g15',
            'OKSM'              : 'g16',
            'PerformerPosition' : 'position',
            'PerformerFio'      : 'full_name',
            'PerformerPhone'    : 'phone',
            'PerformerEmail'    : 'email',

        }, {'target':'Form1Header'})

    module.add_map(form_3_2_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',

            'PassportNumber'    : 'q13',
            'Type'              : ('q14', 'voc_23_full', False),
            'Number'            : 'q15',
            'Radionuclides'     : 'q16',
            'ManufacturingDate' : 'q17',
            'Activity'          : 'q18',
            'NuclearMaterials'  : 'q19',
            'Category'          : 'q20',
            'ManufacturerOkpo'  : 'q21',
            'CertificateNumber' : 'q22',
            'CertificateDate'   : 'q23',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form2'})

    module.add_map(form_3_2_2022+'.header', {
            'FormId'            : 'FormId',

            'IdentificationNumber' : 'q1',
            'SupplyDate'        : 'q2',
            'ReceiverName'      : 'q3',
            'RwHandling'        : 'q4',
            'RaoHandling'       : 'q5',
            'RwLicenseNumber'   : 'q6',
            'RaoLicenseNumber'  : 'q7',
            'RwLicenseTerm'     : 'q8',
            'RaoLicenseTerm'    : 'q9',
            'SupplyAddress'     : 'q10',
            'Radionuclides'     : 'q11',
            'Activity'          : 'q12',
            'Count'             : 'q13',
            'Ukt1Name'          : ('q24', 'PrName', False),
            'Ukt1Type'          : 'q25',
            'Ukt1Number'        : 'q26',
            'Ukt1SupplyDate'    : 'q27',
            'Ukt1Mass'          : 'q28',
            'Ukt2Name'          : None,
            'Ukt2Type'          : None,
            'Ukt2Number'        : None,
            'Ukt2SupplyDate'    : None,
            'Ukt2Mass'          : None,
            'Ukt3Name'          : None,
            'Ukt3Type'          : None,
            'Ukt3Number'        : None,
            'Ukt3SupplyDate'    : None,
            'Ukt3Mass'          : None,
            'Identifier1Name'   : 'q29',
            'Identifier1Value'  : 'q30',
            'Identifier2Name'   : None,
            'Identifier2Value'  : None,
            'Identifier3Name'   : None,
            'Identifier3Value'  : None,
            'PerformerPosition' : 'position',
            'PerformerFio'      : 'full_name',
            'PerformerPhone'    : 'phone',
            'PerformerEmail'    : 'email',

        }, {'target':'Form2Header'})

# --- Отчеты РИАЦ ------------------------------------------------------------

    module.add_map('RiacOrgs', {         # Отображение на таблицу Organizations
            'Id'                 : 'value_id',

            'AuthorityName'	        : 'AuthorityName',
            'AuthorityShortName'	: 'AuthorityShortName',
            'AuthorityAddress'	    : 'AuthorityAddress',
            'AuthorityHeadName'	    : 'AuthorityHeadName',
            'AuthorityPerformerName': 'AuthorityPerformerName',
            'AuthorityPhone'	    : 'AuthorityPhone',
            'AuthorityFax'	        : 'AuthorityFax',
            'AuthorityEmail'	    : 'AuthorityEmail',

            'Name'              : 'NAME_FULL',
            'ShortName'         : 'NAME_SHORT',
            'Subject'           : 'KOD_SRF',
            'Address'           : 'ADDRESS_U',
            'HeadName'          : 'RUKOVODITEL',
            'PerformerName'     : ('',),
            'Phone'             : 'TLFON',
            'Fax'               : 'FAX',
            'Email'             : 'EMAIL',
            'DateStart'         : None,
            'DateEnd'           : None,

        }, {'target':'Organizations', 'source':'VocOrgs'})

    module.add_map('form_4_*.Forms', {           # Отображение на таблицу Forms
            'Type'             : ('form_no', form_num),
            'OrganizationId'   : ('author_id', 'RiacOrgs', True),        # Экспорт ссылки на справочник Организаций
            'Year'             : ('reporting_year', default, 2022),
            'CorrectionNumber' : ('adjustment_no', default, 0),
            'ApprovalState'    : (0,),
            'RiacState'        : (0,),
            'ViacState'        : (0,),
            'CiacState'        : (0,),
            'FillDate'         : None,            
        }, {'target':'Forms'}) 

    module.add_map('form_4_*.header', { # Отображение форм на таблицу FormHeader
            'FormId'            : 'FormId',
            'PerformerPosition' : ('',),
            'PefrormerFio'      : ('',),
            'PerformerPhone'    : ('',),
            'PerformerEmail'    : ('',),
        }, {'target':'FormHeader'})

    module.add_map(form_4_1_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',

            'RegNumber'         : 'g2',
            'Okpo'              : 'g3',
            'Name'              : 'g4',
            'Licenses'          : 'g5',
            'Forms1_1_1_4WithoutInventory'  : 'g6',
            'Forms1_1_1_4WithInventory'     : 'g7',
            'Forms1_12'         : 'g8',
            'Comments'          : 'g9',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form1'})

# --- Отчеты ВИАЦ ------------------------------------------------------------

    module.add_map('ViacOrgs', {         # Отображение на таблицу Organizations
            'Id'                 : 'value_id',
            'Name'              : 'NAME_FULL',
            'ShortName'         : 'NAME_SHORT',
            'HeadName'          : 'RUKOVODITEL',
            'PerformerName'     : ('',),
            'Phone'             : 'TLFON',
            'Fax'               : 'FAX',
            'Email'             : 'EMAIL',

        }, {'target':'Organizations', 'source':'VocOrgs'})

    module.add_map('form_5_*.Forms', {           # Отображение на таблицу Forms
            'Type'             : ('form_no', form_num),
            'OrganizationId'   : ('author_id', 'ViacOrgs', True),        # Экспорт ссылки на справочник Организаций
            'Year'             : ('reporting_year', default, 2022),
            'CorrectionNumber' : ('adjustment_no', default, 0),
            'ApprovalState'    : (0,),
            'RiacState'        : (0,),
            'ViacState'        : (0,),
            'CiacState'        : (0,),
            'FillDate'         : None,            
        }, {'target':'Forms'}) 

    module.add_map('form_5_*.header', { # Отображение форм на таблицу FormHeader
            'FormId'            : 'FormId',
            'PerformerPosition' : ('',),
            'PefrormerFio'      : ('',),
            'PerformerPhone'    : ('',),
            'PerformerEmail'    : ('',),
        }, {'target':'FormHeader'})

    module.add_map(form_5_1_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',

            'OperationCode'     : 'g2',
            'Category'          : 'g3',
            'Radionuclides'     : 'g4',
            'Count'             : 'g5',
            'Activity'          : 'g6',
            'SupplierOkpo'      : 'g7',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form1'})

    module.add_map(form_5_2_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',

            'Category'          : 'g2',
            'Radionuclides'     : 'g3',
            'Count'             : 'g4',
            'Activity'          : 'g5',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form2'})

    module.add_map(form_5_3_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',

            'OperationCode'     : 'g2',
            'OriType'           : 'g3',
            'OriKind'           : 'g4',
            'AggregateState'    : 'g5',
            'SupplierOkpo'      : 'g6',
            'Radionuclides'     : 'g7',
            'Activity'          : 'g8',
            'Mass'              : 'g9',
            'Volume'            : 'g10',
            'Count'             : 'g11',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form3'})

    module.add_map(form_5_4_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',

            'OriType'           : 'g2',
            'OriKind'           : 'g3',
            'AggregateState'    : 'g4',
            'Radionuclides'     : 'g5',
            'Activity'          : 'g6',
            'Mass'              : 'g7',
            'Volume'            : 'g8',
            'Count'             : 'g9',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form4'})

    module.add_map(form_5_5_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',

            'IouName'           : 'g2',
            'OperationCode'     : 'g3',
            'SupplierOkpo'      : 'g4',
            'Count'             : 'g5',
            'Mass'              : 'g6',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form5'})

    module.add_map(form_5_6_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',

            'IouName'           : 'g2',
            'Count'             : 'g3',
            'Mass'              : 'g4',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form6'})

    module.add_map(form_5_7_2022, {
            'FormId'            : 'FormId',
            'Index'             : 'Index',

            'RegNumber'         : 'g2',
            'Okpo'              : 'g3',
            'Name'              : 'g4',
            'Document'          : 'g5',
            'Licenses'          : 'g6',
            'AllowedActivities' : 'g7',
            'Comments'          : 'g8',

            'ParenthesisFlags'  : None,
            'CommentFlags'      : None,
            'HyphenFlags'       : None,
        }, {'target':'Form7'})
