'''
    Определяет действия доступные в формах:
     - вызов аналитического отчета с передачей параметров
'''
import m_rest, m_rec

from sguk_ide import *

api = None
_debug = False

# Интерфейс вызова аналитического отчета из действия в отчетной форме
def action_form_zri_history(context:dict=None, **kwargs):
    '''
    Обработчик действия "История ЗРИ"
    '''

    # Определение параметров для построение отчета
    report_form_id = context['report_form_id']
    form_rec = api.repo.ReportForms.ds.read(report_form_id)

    selected = m_rec.Record(context['selected_row'])
    selected['author_id'] = form_rec.author_id
    selected['form_no'] = form_rec.form_no
    item_id = api.modules.sguk_setup_regs.calc_item_id(selected)

    # Создание аналитического отчета
    areport_id = api.repo.AReports.ar_26.create({'op_item':item_id, 'form_no':form_rec.form_no})

    # Возврат ссылки для просмотра созданного аналитического отчета
    return m_rest.ApiRest.result_url( f'/analytical-reports/{areport_id}' )


import sguk_forms
from sguk_forms import *

def init(_api:SgukApi, debug=False):
    global api
    api = _api

    global _debug
    _debug = debug

    
    oper_forms_node = api.repo.ReportForms.add_group('OperForms')
    oper_forms_node.add_class(form_1_1_2022)
    oper_forms_node.add_class(form_1_2_2022)
    oper_forms_node.add_class(form_1_3_2022)
    oper_forms_node.add_class(form_1_4_2022)
    oper_forms_node.add_class(form_1_5_2022)
    oper_forms_node.add_class(form_1_6_2022)
    oper_forms_node.add_class(form_1_7_2022)
    oper_forms_node.add_class(form_1_8_2022)
    oper_forms_node.add_class(form_1_9_2022)

    year_forms_node = api.repo.ReportForms.add_group('YearForms')
    year_forms_node.add_class(form_2_1_2022)
    year_forms_node.add_class(form_2_2_2022)
    year_forms_node.add_class(form_2_3_2022)
    year_forms_node.add_class(form_2_4_2022)
    year_forms_node.add_class(form_2_5_2022)
    year_forms_node.add_class(form_2_6_2022)
    year_forms_node.add_class(form_2_7_2022)
    year_forms_node.add_class(form_2_8_2022)
    year_forms_node.add_class(form_2_9_2022)
    year_forms_node.add_class(form_2_10_2022)
    year_forms_node.add_class(form_2_11_2022)
    year_forms_node.add_class(form_2_12_2022)

    border_moves_node = api.repo.ReportForms.add_group('BorderMoves')
    border_moves_node.add_class(form_3_1_2022)
    border_moves_node.add_class(form_3_2_2022)

    riac_forms_node = api.repo.ReportForms.add_group('RIACforms')
    riac_forms_node.add_class(form_4_1_2022)
    
    viac_forms_node = api.repo.ReportForms.add_group('VIACforms')
    viac_forms_node.add_class(form_5_1_2022)
    viac_forms_node.add_class(form_5_2_2022)
    viac_forms_node.add_class(form_5_3_2022)
    viac_forms_node.add_class(form_5_4_2022)
    viac_forms_node.add_class(form_5_5_2022)
    viac_forms_node.add_class(form_5_6_2022)
    viac_forms_node.add_class(form_5_7_2022)
    

    # Ф.1.1 - История ЗРИ
    api.repo.find_class('ReportForms', sguk_forms.t_form_1_1).add_action(
        'ar_26', 
        action_form_zri_history, 
        caption='История ЗРИ', 
        icon=None, 
        form_type='edit')

    api.repo.find_class('ReportForms', sguk_forms.t_form_1_5).add_action(
        'ar_26', 
        action_form_zri_history, 
        caption='История ЗРИ', 
        icon=None, 
        form_type='edit')

    # Действие "Перевести в"
    form_node = _api.repo.find_class('ReportForms', 'form_1_1', is_assert=False)
    if form_node != None:
        form_node.add_action('ZriMoveTo', caption='Перевести в', form_type='edit')
        form_node.ZriMoveTo.add_action('ZriToOri', action_zri_to_ori, caption='ЗРИ в ОРИ', form_type='edit') # , url='/api/forms/actions?action=exec&path=forms_actions&action_name=zri_to_ori'
        form_node.ZriMoveTo.add_action('ZriToOzri', action_zri_to_ozri, caption='ЗРИ в ОЗРИ', form_type='edit') # , url='/api/forms/actions?action=exec&path=forms_actions&action_name=zri_to_ozri'


def action_zri_to_ori(context, **kwargs):
    request = api.context.request
    form_id = context['report_form_id']
    try:
        new_form = api.lib.sguk.zri_to_ori(request, form_id, connection=api.context.db_connect.conn)
        api.context.db_connect.commit()
        return m_rest.ApiRest.result_url(f'/editor-forms/{new_form["report_form_id"]}')
    except Exception as e:
        return m_rest.ApiRest.result_notify('Перевод ЗРИ в ОРИ', e)


def action_zri_to_ozri(context, **kwargs):
    request = api.context.request
    form_id = context['report_form_id']
    try:
        new_form = api.lib.sguk.zri_to_ozri(request, form_id, connection=api.context.db_connect.conn)
        api.context.db_connect.commit()
        return m_rest.ApiRest.result_url(f'/editor-forms/{new_form["report_form_id"]}')
    except Exception as e:
        return m_rest.ApiRest.result_notify('Перевод ЗРИ в ОЗРИ', e)
