'''
Модуль регистрации справочников 

Значения справочников хранятся в таблице VocValues. Доступ к данным предоставляется 
источником данных - экземпляром класса VocDS, тип справочника определяется при инициализации.

Регистрируются типы данных со значениями выбранными из справочников
'''
from sguk_ide import *

api = None

import m_repo
import sguk_vocs

def init(_api:SgukApi):
    global api
    api = _api

    class_node = api.repo.Vocs.add_class('VocOrgs', desc='Справочник организаций')
    ds = class_node.add('ds', sguk_vocs.VocOrgsDS())
    
    vocs_node = api.repo.Vocs
    vocs_node.reg_voc('OpCod',       '11_voc',  type_name='OpCod', desc='Коды операций')
    vocs_node.reg_voc('DocVid',      '12_voc',  type_name='DocVid', desc='Виды документов сопровождающих операцию', label_attr='DocName')
    vocs_node.reg_voc('ORI',         '13_voc',  desc='Виды ОРИ')
    vocs_node.reg_voc('BCod',        '16_voc',  desc='Код "Статус РАО"')
    vocs_node.reg_voc('UpCod',       '17_voc',  desc='Коды переработки/сортировки')
    vocs_node.reg_voc('F28g5',       '110_voc', desc='Наименование бассейного округа')
    vocs_node.reg_voc('PH_Name',     '21_voc',  desc='Пункты хранения')
    vocs_node.reg_voc('voc_23',      '23_voc',  desc='Источники излучения')
    vocs_node.reg_voc('voc_23_full', '23_voc1', desc='Источники излучения (расширенный)')
    vocs_node.reg_voc('voc_24',      '24_voc',  desc='Изделия из обедненного ураня (ИОУ)')
    vocs_node.reg_voc('PrName',      '27_voc',  desc='Приборы (установки), УКТ или иные упаковки')
    vocs_node.reg_voc('Factory',     '28_voc',  desc='Установки переработки')
    vocs_node.reg_voc('voc_110',     '110_voc', desc='Названия водных бассейнов')
    vocs_node.reg_voc('voc_112',     '112_voc', desc='Коды типа РВ не в составе ЗРИ')
    vocs_node.reg_voc('voc_113',     '113_voc', desc='Наименование зоны контроля')
    vocs_node.reg_voc('voc_114',     '114_voc', desc='Орган управления использованием атомной энергии (ОУИАЭ)')
    vocs_node.reg_voc('SubRF',       '115_voc', desc='Субъект Российской Федерации')
    vocs_node.reg_voc('voc_116',     '116_voc', desc='Общероссийский Классификатор Органов Государственной власти и Управления (ОКОГУ)')
    vocs_node.reg_voc('voc_117',     '117_voc', desc='Общероссийский Классификатор Организационно-Правовых Форм (ОКОПФ)')
    vocs_node.reg_voc('voc_118',     '118_voc', desc='Общероссийский Классификатор Организационно-Правовых Форм (ОКФС)')
    vocs_node.reg_voc('OKSM',        '119_voc', desc='Общероссийский Классификатор Стран Мира (ОКСМ)')
    vocs_node.reg_voc('voc_120',     '120_voc', desc='Агрегатное состояние')
    vocs_node.reg_voc('FormSobst',   'FormSobst_voc', desc='Формы собственности')
    vocs_node.reg_voc('radionuclide_voc1', 'radionuclide_voc1', desc='Радионуклиды')

    api.repo.add_attr_type('OrgId', m_repo.AttrTypeDsDict(api.repo.Vocs.VocOrgs.ds, id_attr='record_id', label_attr='NAME_SHORT'))
    api.repo.add_attr_type('UpCod', m_repo.AttrTypeDsDict(api.repo.Vocs.UpCod.ds, id_attr='value_id', label_attr='c11'))
    api.repo.add_attr_type('Nuclid', m_repo.AttrTypeDsDict(api.repo.Vocs.radionuclide_voc1.ds, id_attr='value_id', label_attr='label'))

