import m_utils
import m_rec

import m_rest

from sguk_api import *

api = None


def init(_api):
    global api
    api = _api

    if api.get_config('API2_DEBUG', True):
        endpoint_test_init()             # endpoint /test
        triggers_test_init()             # trigger ReportForms.DS_AFTER_UPDATE
        actions_test_init()              # actions: _test_global, _test_form (ReportForms, form_1_1)

#=============================================================================================    
#   Actions
#=============================================================================================    
def action_test(**kwargs):
    return m_rest.ApiRest.result_notify('Тестовое действий', '"Успешно выполнено<BR>строка 1<br>строка 2"')


def action_download(**kwargs):
    import os
    tmp_path = api.get_config('API2_FILES_TMP', '')
    file_path = os.path.join( tmp_path, 'OperForms.db')
    return m_rest.ApiRest.result_file(file_path)

import sguk_forms

def actions_test_init():
    api.actions.add_action('', '_test_global', action_test, caption='Global test action', icon='/interface/upload.svg', text='Global test action')
    #api.repo.ReportForms.add_action('_test_form', action_test, caption='Form test action', icon='/interface/upload.svg', text='Form test action')
    #api.repo.ReportForms.add_action('_test_download', action_download, caption='Form download action', icon='/interface/upload.svg', text='Form download action')

    form_node = api.repo.ReportForms.find_form(sguk_forms.form_1_1_2022)
    form_node.add_action('_test_form', action_test, caption='Form 1.1 test action', icon=None, text='Form 1.1 test action')
    form_node.add_action('_test_form2', action_test, caption='Form 1.1 test action 2', icon=None, text='Form 1.1 test action 2')

    areport_node = api.repo.AReports
    areport_node.add_action('_test_areport_action', action_test, caption='Test')

#=============================================================================================    
#   Endpoint
#=============================================================================================    
from werkzeug.wrappers import Response

def ep_test(context, **kwargs):
    print('=== Endpoint /test ===')

    request = {
        'method' : context['request'].method,
        'path'   : context['request'].path,
        'args'   : context['request'].args.to_dict()
    }
    print('request:', m_utils.pretty(request))
    print('kwargs:', m_utils.pretty(kwargs), '\n')

    html = f'''
            <html><body>
            <h1>context</h1>
            <pre>
{m_utils.pretty(context)}
            </pre>
            <h1>kwargs</h1>
            <pre>
{m_utils.pretty(kwargs)}
            </pre>
            <h1>api</h1>
            <pre>
{m_utils.to_html(_api.tree_str()[-1])}
            </pre>
            </body></html>
        '''
    #request = context['request']

    return Response(html, mimetype='text/html')


def ep_tree(api, **kwargs):
    return Response(api.tree_str()[-1], mimetype='text/plain')

def endpoint_test_init():
    api.add_url('/test', ep_test)
    api.add_url('/tree', ep_tree)


#=============================================================================================    
#   Триггер 
#=============================================================================================    
def triggers_test_init():
    # Тестовый триггер на обновление Отчетных форм
    api.triggers.add_trigger('DS_AFTER_UPDATE', 'ReportForms', tg_after_update)


def tg_after_update(context, **kwargs):               
    NEW = context.get('NEW')
    OLD = context.get('OLD')
    if NEW and OLD and NEW['form_state'] != OLD['form_state'] and NEW['form_state'] == 'verified':
        del context['OLD']
        rec = m_rec.Record(NEW)
        rec.add_alias('table', 'form_table_data.main_table')
        #print('context:', m_utils.pretty(context), '\n')
        #print('kwargs:', m_utils.pretty(kwargs), '\n')

#=============================================================================================    
#   Функции
#=============================================================================================    

def forms_info():
    forms = {}
    for row in api.repo.ReportForms.ds.select():
        if row.form_no not in forms:
            forms[row.form_no] = ''
            
        table_len = row.form_table_data.main_table.count()
        if table_len:
            forms[row.form_no] += f'{row.report_form_id}({table_len}) '

    keys = list(forms.keys())
    keys.sort()

    for form in keys:
        print(f'{form} : {forms[form]}')

