def init(_api):
    global api
    api = _api

    api.repo.add_storage(VOCS_TYPE)
    api.repo.Vocs.add('reg_voc', reg_voc)


VOCS_TYPE = 'Vocs'              # Имя хранилища в репозитории
VOCS_TABLE = 'VocValues'        # Имя таблицы для хранения данных
PK_FIELD_NAME = 'record_id'     # Поле первичного ключа
CLASS_FIELD_NAME = 'voc_type'   # Поле класса записи

VOC_ORG_TABLE = 'VocOrgs'
VOC_ORG_CLASS = 'VocOrgs'

api = None

import m_rec
import m_ds
import sguk_repo

    
def reg_voc(voc_name, voc_type, type_name=None, desc=None, label_attr=None, pk_field='value_id'):
    class_node = api.repo.Vocs.add_class(voc_name, desc=desc)
    ds = class_node.add_datasource('ds', VocDS(voc_type, pk_field=pk_field))
    if type_name:
        api.repo.add_attr_type(type_name, sguk_repo.SgukVocType(ds, label_attr=label_attr))

    api.actions.aliases[voc_type] = voc_name



class VocDS(m_ds.TableDataSource):
    '''
    Источник данных для справочников хранящихся в таблице VocValues

    VocDS(<voc_type>)
        voc_type    - значение поле voc_type, определяющее тип справочника 
    '''
    def __init__(self, voc_type, pk_field=None):
        if not pk_field: pk_field = PK_FIELD_NAME
        super().__init__( VOCS_TYPE, VOCS_TABLE, pk_field, CLASS_FIELD_NAME)

        self.voc_type = voc_type

        self.fields = [
                ('value_id', str),
                ('label', str),
                ('attrs', 'jsonb'),
                ('valid_from', 'date'),
                ('valid_to', 'date'),
                ('value_status', int)
            ]

        self.container = 'attrs'

        self.aliases = {'':'attrs'}
            #
            #
    def db_read(self, id, **kwargs):
        filters = kwargs.pop('filters', {})
        filters[self.class_field_name] = self.voc_type
        return super().db_read(id, filters=filters, **kwargs)
            #
            #
    def db_create(self, data):
        data['voc_type'] = self.voc_type
        return super().db_create(data)
            #
            #
    def db_update(self, rec):
        filters = {self.class_field_name: self.voc_type}
        return super().db_update(rec, filters)
            #
            #
    def db_select(self, **kwargs):
        if 'filters' not in kwargs or kwargs['filters'] is None:
            kwargs['filters'] = {}
        
        kwargs['filters']['voc_type'] = self.voc_type

        if 'debug' in kwargs:
            print(f'VocDS.db_select: filters={kwargs["filters"]}')

        return super().db_select(**kwargs)
            #
            #
    

class VocOrgsDS(m_ds.TableDataSource):
    '''
    Источник данных для справочника организаций
    '''
    def __init__(self):
        super().__init__( VOC_ORG_CLASS, VOC_ORG_TABLE, 'record_id')


class VocCache:
    def __init__(self):
        self._cache = {}
        self._props = {}

    def get_prop(self, voc_class, prop_name):
        value = self._props.get(voc_class, {}).get(prop_name)
        if value != None:
            return value
        elif prop_name == 'indexes':
            return ['value_id']
        elif prop_name == 'fields':
            return None
        elif prop_name == 'preload':
            return True

        return None


    def reg_voc(self, voc_class, fields=None, preload=True, indexes=['value_id']):
        if fields!=None:
            # Добавляет поля из индекса не попавшие в список полей
            for field in indexes:
                if field not in fields:
                    fields.append(field)

            if 'value_id' not in fields: fields.append('value_id')

        self._props[voc_class] = dict(fields=fields, preload=preload, indexes=indexes)


    def get_voc_ds(self, voc_class):
        voc_node = api.repo.Vocs.find(voc_class)
        if not voc_node:
            raise Exception(f'Не найден справочник {voc_class}')

        return voc_node.ds._data
            #
            #
    def load_voc(self, voc_class):
        voc_ds = self.get_voc_ds(voc_class)

        fields = self.get_prop(voc_class, 'fields')
        params = {'fields':fields} if fields else {}
        records = voc_ds.select(**params)

        voc_data = {}
        for rec in records:
            for field in self.get_prop(voc_class, 'indexes'):
                key = rec[field]
                voc_data[key] = rec

        self._cache[voc_class] = voc_data


    def load_rec(self, voc_class, value_id):
        voc_ds = self.get_voc_ds(voc_class)

        for field in self.get_prop(voc_class, 'indexes'):
            if field=='value_id':  
                rec = voc_ds.read(value_id)
            else:
                recs = voc_ds.select(filters={field:value_id})
                rec = recs[0] if recs.count() else None

            if rec and not rec.is_empty(): 
                if voc_class not in self._cache: self._cache[voc_class] = {}
                #rec.attr_filter(self.get_prop(voc_class, 'fields'))
                self._cache[voc_class][value_id] = rec
                return


    def exist(self, voc_class, value_id):
        if voc_class not in self._cache:
            self._cache[voc_class] = {}
            if self.get_prop(voc_class, 'preload'):
                self.load_voc(voc_class)

        if value_id not in self._cache[voc_class] and not self.get_prop(voc_class, 'preload'):
            self.load_rec(voc_class, value_id)

        return value_id and value_id in self._cache[voc_class]


    def read(self, voc_class, value_id):
        if self.exist(voc_class, value_id):
            return self._cache[voc_class][value_id]
        else:
            return m_rec.Record({})

