hex_digits = '0123456789ABCDEF'

#----------------------------------------------------
#   hex...
#----------------------------------------------------
def hexPack(hexStr:str) -> str:
    '''
    Исключает из hex-строки символы не относящиеся к шестнадцатиричным
    '''
    res = ''
    for c in hexStr:
        if c in hex_digits:
            res += c
    return res


def hexLenPack(hexStr:str, silent=False):
    '''
    Удаляет из строки не щестнадцатеричные символы и определяет длину данных в байтах
    '''
    packStr = hexPack(hexStr)
    res = len(packStr)
    if not silent and res % 2 != 0:
        raise Exception('Нечетное количество символов')
    return int(res/2), packStr


def hexLen(hexStr:str, silent=False) -> int:
    '''
    Возвращает длину данных в байтах
    '''
    l, _ = hexLenPack(hexStr, silent)
    return l


def hexGet(hexStr, idx, count=1, silent=False):
    '''
    Возвращает строку шестнадцатеричных символов соответсвующих 
    заданному номер начального байта и заданному количеству байт
    '''
    l, src = hexLenPack(hexStr, silent)    

    if idx < 0:
        idx = l + idx

    if not silent and l <= idx:
        raise Exception(f'Не корректный индекс. Запрошено {idx} при длине {l}')
    if not silent and l <= idx+count-1:
        raise Exception(f'Не достаточно данных. Запрошено {idx+count-1} при длине {l}')

    firstPos = idx*2
    lastPos = firstPos + (count-1)*2 + 2
    return src[firstPos: lastPos]


def hex2byte(src:str) -> int:
    '''
    Преобразует шестнадцатеричное представление (один или два символа) в байт
    '''
    if len(src) == 2:
        return int(src[0], 16) * 16 + int(src[1], 16)
    elif len(src) == 1:
        return int(src[0], 16)
    else:
        raise Exception(f'hex2byte: Не корректная длина строки "{src}"')


def hex2bytes(src:str) -> list:
    '''
    Преобразует шестнадцатеричную строку в массив байтов
    '''
    l, s = hexLenPack(src)
    res = []
    for p in range(l):
        res.append(hex2byte(s[p*2: p*2+2]))

    return res 


def hex2bstr(src:str) -> bytes:
    data = hex2bytes(src)
    data = [chr(b) for b in data]
    return b''.join(data)


def hex2int(dataHex: str) -> int:
    '''
    Преобразует шестнадцатеричную строку в целое число
    '''
    data = hex2bytes(dataHex)
    return bytes2int(data)


def hex2bstr(hexStr: str) -> bytes:
    '''
    Преобразует шестнадцатеричную строку в байт-строку
    '''
    return bytes( hex2bytes(hexStr) )


def hexBitLen(hexStr:str) -> int:
    '''
    Возвращает количество байт в шестнадцатеричной строке
    '''
    return hexLen(hexStr)*8


def hex2bits(hexStr:str) -> str:
    '''
    Преобразует шестнадцатеричную строку в двоичную
    '''
    l, s = hexLenPack(hexStr)
    res = ''
    for i in range(0, l*2, 2):
        b = hex2int(s[i]+s[i+1])
        for bitIdx in range(7, -1, -1):
            res += '1' if b & (1<<bitIdx) else '0'
    return res


def hexBitFromStart(hexStr:str, idx:int) -> int:
    '''
    Возвращает значение указанного бита в шестнадцатеричной строке
    '''
    s=hexPack(hexStr)
    byteIdx = idx >> 3
    bitAtByte = 7 - (idx - byteIdx*8) 
    #print(f'idx={idx} byteIdx={byteIdx} bitAtByte={bitAtByte} s={s}')
    return 1 if hex2int(hexGet(s, byteIdx)) & (1<<bitAtByte) else 0

#----------------------------------------------------
#   bytes...
#----------------------------------------------------
def bytes2hex(data:list, line_size=255) -> str:
    '''
    Преобразует массив байт в hex-строку
    '''
    res = []
    res_str = ''
    for b in data:
        res.append(hex_digits[b>>4] + hex_digits[b & 0x0F])
    
    i = 0
    while i+line_size< len(res):
        res_str += ' '.join(res[i:i+line_size]) + '\n'
        i += line_size
    res_str += ' '.join(res[i:])
    return res_str


def bytes2int(data: bytes) -> int:
    '''
    Преобразует массив байт в целое
    '''
    n = 0
    while data:
        n = n*256 + data[0]
        data = data[1:]
    return n


def bytes2str(data:list) -> str:
    '''
    Преобразует массив байт в строку символов, соответсвующих байтам
    '''
    return ''.join([chr(b) for b in data])


def bytes2bstr(data:list) -> bytes:
    '''
    Преобразует массив байт в байтовую строку
    '''
    return bytes(data)


#----------------------------------------------------
#   int...
#----------------------------------------------------
def int2bytes(data):
    res = []
    while data:
        res.append(data % 256)
        data //= 256
    res.reverse()
    return res


def int2hex(data, line_size=256):
    return bytes2hex(int2bytes(data), line_size)

def hex2str(data):
    return bytes2str( hex2bytes(data) )


#----------------------------------------------------
#   class HexStr
#----------------------------------------------------
class HexStr:
    '''
    Класс для работы с hex-строкой
    '''
    STR = 'str'

    def __init__(self, src=None, bits=None, format=None):
        self.set(src, bits, format)

    def __repr__(self):
        if self.format == HexStr.STR:
            return bytes2str( hex2bytes(self.data) )

        else:
            res = ''
            for i in range(self.length()):
                res += self.get(i).toStr()
                res += '\n' if (i+1) % 16 == 0 else ' '
            return ''.join(res[:-1])


    def set(self, src:str=None, bits:str=None, format=None) -> None:
        if src==None and bits!=None:
            src = int2hex(int(bits, 2))

        if isinstance(src, HexStr):
            src = src.data

        if type(src) == int:
            src = int2hex(src)

        l, self.data = hexLenPack(src.upper())
        self.format = format
        self.firstByteIdx = 0
        self.lastByteIdx = l-1


    def length(self) -> int:
        return hexLen(self.data)


    def get(self, idx:int, count:int=1):
        return HexStr( hexGet(self.data, idx, count) )


    def getRange(self, firstByte:int, lastByte:int=None):
        if lastByte==None: lastByte = self.length()-1
        return self.get(firstByte, lastByte-firstByte+1)


    def bitLen(self) -> int:
        return hexBitLen(self.data)


    def getBitFromStart(self, idx) -> int:
        return int(hexBitFromStart(self.data, idx))


    def toBits(self) -> str:
        return hex2bits(self.data)


    def toInt(self) -> int:
        return hex2int(self.data)


    def toBCDInt(self) -> int:
        r = 0
        for d in self.data:
            r = r * 10 + int(d)
        return r


    def toStr(self) -> str:
        return self.data


    def popNext(self, count=1):
        res = self.get(self.firstByteIdx, count)
        self.firstByteIdx += count
        return res


    def popFirst(self, count=1):
        self.firstPos = 0
        return self.popNext(count)


    def restLen(self):
        return self.lastByteIdx-self.firstByteIdx+1


    def popRest(self):
        return self.popNext(self.lastByteIdx-self.firstByteIdx+1)
    
    

def pretty(data, formats={}, prefix=''):
    keys = list(data.keys())
    keys.sort()
    for key in keys:
        value = data[key]
        print(prefix+key+': ', end='')
        if type(value)==dict:
            print('{')
            pretty(value, formats, prefix+' '*(len(key)+4))
            print(prefix+' '*(len(key)+2)+'}')
        else:
            fmt = formats.get(key)
            if type(fmt) == tuple:
                desc = fmt[1]
                fmt = fmt[0]
            else:
                desc = ''

            if fmt=='STR':
                value = f'{hex2str(value)} :STR'
            elif fmt=='BCD':
                value = f'{value.replace(" ", "")} :BCD'
            elif fmt=='HEX':
                value = str(HexStr(value)) + ' :HEX'
            elif fmt=='LEN':
                value = f'{hexLen(value)} bytes'
            else:
                value = f'{value}  "{hex2str(value)}"' 

            if desc:
                value += f' ({desc})'

            print(value)


class TLV:
    def __init__(self, data: list, tags=None):
        self.tags = tags
        self.data = self._parse(data)

    def _parse(self, data):
        p=0
        res={}
        while p<len(data):
            tag = data[p]; p+=1
            isContainer = tag & 0x20
            isExtTag = tag & 0x1F == 0x1F
            
            if isExtTag:
                tag = tag * 256 + data[p]; p+=1
                
            dataLen = data[p]; p+=1
            if dataLen==129:
                dataLen = data[p]; p+=1
            elif dataLen==130:
                dataLen = dataLen*256 + data[p]; p+=1
                
            tagName = int2hex(tag).replace(' ','')
            tagValue = data[p: p+dataLen]; p+=dataLen
            if isContainer:
                tagValue = self._parse(tagValue)
            else:
                tagValue = bytes2hex(tagValue)
                if self.tags != None:
                    self.tags[tagName] = tagValue
            
            res[tagName] = tagValue

        return res  


    def get(self, path, data=None):
        tags = path.split('.')
        if data==None: data=self.data
        value = data[tags[0]]

        if len(tags) == 1:
            return value
        else:
            return self.get('.'.join(tags[1:]), value)


    def print(self):
        pretty(self.data)
