import os, sys
import re


def get_main_module():
    '''
    Возвращает путь к основному модулю приложения
    '''
    for module in sys.modules.values():
        if module.__name__ == '__main__':
            return module
        
    return None
        

def get_main_path():
    main_file = get_main_module()
    if main_file: 
        return os.path.split(main_file.__file__)[0]
    else:    
        return os.getcwd()


def find_path(file_path, *, relative=None, env_list=None, extension=None, dir_allow=False):
    '''
    Находит файл используя пути указанные в env_list.
    Возвращает полный абсолютный путь
    '''
    if env_list==None: env_list = ['PYTHONPATH', 'PATH']

    path_list = []

    if relative:
        path_list.append(os.path.split(os.path.abspath(relative))[0])

    path_list.append(get_main_path())
        
    # Загрузка путей из переменных окружения
    for env_name in env_list:
        env_value = os.environ.get(env_name)
        if not env_value: continue
        
        split_char = ';'
        if ':' in env_value and ';' not in env_value: split_char = ':'

        path_list.extend(env_value.split(split_char))

    # Поиск файла
    for path in path_list:

        search_path = os.path.join(path, file_path)
        if os.path.isfile(search_path):
            return os.path.abspath(search_path)

        if extension and not search_path.endswith(extension) and os.path.isfile(search_path+extension):
            return os.path.abspath(search_path+extension)

        if dir_allow and os.path.isdir(search_path):
            return os.path.abspath(search_path)
        
        
    return None


def path_extract(pattern, src, dst=None):
    '''
    Извлекает по шаблону вида 'path0/{path1}/path3/{path4}'
    значения соответствующие path1 и path4 в заданном src
    '''
    var_names=re.findall('\{(\w+)\}', pattern)
    if var_names==None:
        return dst or {}
    
    for i in var_names or []:
        pattern=pattern.replace('{'+i+'}', '([/\w]+)')

    #print(f'{pattern=}')
    sr=re.search(pattern, src)
    if sr==None: return None

    res = {}
    for i, v in enumerate(sr.groups()):
        res[var_names[i]] = v
        if dst: dst=dst.replace(f'${var_names[i]}', v)
    return dst or res


def uri_split(remote_path:str) -> tuple:
    '''
    Разделяет URI вида http://domain.my:2222/path на домент, порт, путь 
    допустимы аргументы 
        /path => '', 0, /path
        c:\path => '', 0, c:\path
        path1   => 

    '''
    # Проверка на пути в стиле windows
    if ':\\' in remote_path or ':/' in remote_path:
        return '', 0, remote_path

    remote_path = remote_path.replace('\\', '/')

    if '/' in remote_path:
        if '.' in remote_path.split('/')[0]:
            host, path = remote_path.split('/', 1)
        else:
            host = ''
            path = remote_path
    else:
        host = remote_path
        path = ''

    path = '/' + path

    if ':' in host:
        host, port = host.split(':')
    else:
        port = 22

    return host, port, path


def test_find_path():
    res = find_path('path.py')
    res = find_path('utils/path.py')
    res = find_path('utils/path.py', env_list=[])
    pass


if __name__=='__main__':
    test_find_path()