import settings

from datasource.datasource import get_db_connection, close_connection

from repository.types.types_repository import get_type_repository

from repository.utils import import_module_
from repository.vocs.vocs_repository import get_voc_repository
from settings import API2_PATH
from web import Web

from loguru import logger

from wsgi.apps.reporting_forms.services.forms_actions.funcs import zri_to_ozri, zri_to_ori
from wsgi.apps.user_logs.services import delete_db_items_with_log_id


m_api = import_module_(API2_PATH, 'm_api')
sguk_api = import_module_(API2_PATH, 'sguk_api')
m_db_module = import_module_(API2_PATH, 'm_db')

api2 = None

users_log_levels = {}


def trace_level_view(api2_obj, context, *args, **kwargs):
    """Задать уровень логирования для пользователя"""
    args = context['args']
    levels = ('0', '1', '2', '3', '4', '5')
    if args['level'] not in levels:
        return {
            'error': f'Уровень логирования должен быть одним из: {levels}'
        }
    users_log_levels[args['user']] = args['level']
    return {'message': f'Уровень логирования успешно задан: Логин: {args["user"]}, уровень: {args["level"]}'}


def get_api2():
    return api2


def get_api2_clone(conn, request, user):
    api2.build_context(core_db_connect=conn)
    api2.context['request'] = request
    api2.context['user'] = user
    return api2


def get_connect():
    return get_db_connection()


def connect_release(conn):
    close_connection(conn)


def init_api2(app, context_for_api2):
    global api2
    api2 = sguk_api.init_core()

    api2.add('system.pg_connect', get_connect)
    api2.add('system.pg_release', connect_release)
    api2.add('system.sqlite_connect', get_connect)
    api2.add('system.sqlite_release', connect_release)

    api2.add('lib')

    from repository.forms.forms_repository import get_form_repository
    api2.lib.add('formModelGet', get_form_repository)

    api2.lib.add('typeModelGet', get_type_repository)
    api2.lib.add('vocModelGet', get_voc_repository)

    settings_as_dict = {}
    for var in dir(settings):
        if not var.startswith('__') and not var.endswith('__'):
            settings_as_dict[var] = getattr(settings, var)
    api2.set('config', settings_as_dict, ensure=True)

    web = Web(app)
    api2.add('web', web)
    api2.triggers.add_event('REST_BEFORE_PROCESS')
    api2.triggers.add_event('REST_AFTER_PROCESS')

    api2.triggers.add_event('CUSTOM_FILE_INIT')
    api2.triggers.add_event('AFTER_INIT')

    # регистрация на стороне api2
    sguk_api.init(api2)

    for context in context_for_api2:
        api2.triggers.emit('CUSTOM_FILE_INIT', context=context)
    api2.triggers.emit('AFTER_INIT', context={'api': api2})

    web.add_url('/trace_level', trace_level_view)

    from wsgi.apps.areports.serializers import AReportsSerializer
    api2.lib.add('AreportsSerializer', AReportsSerializer)

    from wsgi.apps.reporting_forms.serializers import FormSerializer
    api2.lib.add('FormsSerializer', FormSerializer)

    from wsgi.apps.historical_data.services.services import run_import_api2
    api2.lib.add('import_hist.import_historical_data', run_import_api2)

    from wsgi.apps.reporting_forms.services.services import validate_form
    api2.lib.add('validate_form', validate_form)

    api2.lib.add('clear_action', delete_db_items_with_log_id)

    # Кнопки для форм (переводы и т.д.)
    api2.lib.add('sguk')
    api2.lib.sguk.add('zri_to_ozri', zri_to_ozri)
    api2.lib.sguk.add('zri_to_ori', zri_to_ori)

    api2.print()

    logger.info(f'INIT API2 DONE, api2: {api2}')

    return api2
