from services.exceptions import CoreBaseException


class DatabaseRepositoryException(CoreBaseException):
    def __init__(self, *args: object, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.base_type = 'database_repository_excpetion'


class DatabaseRepositoryException2(CoreBaseException):
    def __init__(self, *args: object, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.base_type = 'database_repository_excpetion2'


class UnknownTypeException(DatabaseRepositoryException):
    def __init__(self, type_) -> None:
        super().__init__(
            type_='unknown_type',
            message=f'Unknown type: {type_}',
            rus_message=f'...'
        )


class RecordNotFoundException(DatabaseRepositoryException):
    def __init__(self, record_id, model_data=None) -> None:
        super().__init__(
            type_='record_not_found_exception',
            message=f'Record not found with id: {record_id}. Model data: {model_data}',
            rus_message=f'Не найдена запись с id: {record_id}'
        )


class IncorrectRecordIdFormatException(DatabaseRepositoryException):
    def __init__(self, record_id) -> None:
        super().__init__(
            type_='incorrect_record_format_exception',
            message=f'Incorrect record format: {record_id}',
            rus_message=f'Неверный формат id записи: {record_id}'
        )


class TypeException(DatabaseRepositoryException):
    def __init__(self, attr, type_, value) -> None:
        super().__init__(
            type_='convert_to_type',
            message=f'can not convert value: {value} to type: {type_}',
            rus_message=f'...'
        )
        self.attr = attr


class ValidationException(DatabaseRepositoryException):
    def __init__(self, validation_errors, *args: object) -> None:
        self.validation_errors = validation_errors


class UnsupportedFilterParamException(DatabaseRepositoryException):
    def __init__(self, params) -> None:
        super().__init__(
            type_='UnsupportedFilterParamException',
            message=f'UnsupportedFilterParamException: Params: {params}',
            rus_message=f'Невозможно выполнить фильтрацию по данным параметрам: {params}'
        )


class ForeignKeyViolationException(DatabaseRepositoryException):
    def __init__(self, params) -> None:
        super().__init__(
            type_='UnsupportedFilterParamException',
            message=f'UnsupportedFilterParamException: Params: {params}',
            rus_message=f'Невозможно выполнить фильтрацию по данным параметрам: {params}'
        )


class UniqueViolationException(DatabaseRepositoryException):
    def __init__(self, model) -> None:
        super().__init__(
            type_='UniqueViolationException',
            message=f'UniqueViolationException: Model: {model}',
            rus_message=f'Невозможно сохранить данные. Указанное значение или их комбинация уже существует.'
        )
