import os.path
import sqlite3
import sys
import time
from os import listdir
from os.path import isfile, join

import psycopg2
import psycopg2.errors
from loguru import logger

import settings


def get_db_connection():
    if settings.DB_SYSTEM == 'sqlite':
        conn = sqlite3.connect(settings.SQLITE_DATABASE)
        return conn
    elif settings.DB_SYSTEM == 'postgresql':
        return psycopg2.connect(user=settings.DB_USER['user'],
                                password=settings.DB_USER['password'],
                                host=settings.DB_HOST,
                                port=settings.DB_PORT,
                                database=settings.POSTGRES_DATABASE)
    raise Exception(f'В файле настроек указан неверный DB_SYSTEM. Доступные значения: sqlite, postgresql.')


def close_connection(conn):
    conn.close()


def init_database_sqlite(folder_path):
    sql_script_files = [f for f in listdir(folder_path) if isfile(join(folder_path, f))]
    sql_script_files.sort()

    logger.info(f'Найдены файлы для иницализации базы: {sql_script_files}')

    for sql_script_file in sql_script_files:
        conn = get_db_connection()
        cur = conn.cursor()

        sql_script_file_full_path = os.path.join(folder_path, sql_script_file)

        try:
            try:
                cur.execute('select * from patch_control').fetchall()
                patch_control_exists = True
            except sqlite3.OperationalError:
                patch_control_exists = False

            if patch_control_exists:
                cur.execute(f"select * from patch_control where file_name = '{sql_script_file}';")
                if cur.fetchone():
                    logger.info(f'Файл {sql_script_file} уже проинициализирован в базе ранее.')
                    continue
                logger.info(f'Файл {sql_script_file} будет проинициализирован в базе.')

            logger.info(f'Иницализация файла {sql_script_file}...')

            cur.execute('begin')

            with open(sql_script_file_full_path, 'r') as f:
                for line in f.readlines():
                    line = line.strip()
                    if not line:
                        continue
                    cur.execute(line)
            cur.execute('insert into patch_control (file_name) values (?);', (sql_script_file,))
            logger.info(f'В таблицу patch_control добавлен файл {sql_script_file}.')
        except Exception as e:
            conn.execute('rollback')
            logger.error(f'При инициализации базы возникла ошибка: {e}')
            sys.exit()
        except:
            try:
                conn.execute('rollback')
            except:
                pass
            logger.error('Прервано выполнение.')
            sys.exit()
        else:
            conn.execute('commit')
        finally:
            cur.close()
            conn.close()
