from services.exceptions import CoreBaseException


class ExportManagerException(CoreBaseException):
    ...


class TemplateFileNotFoundException(ExportManagerException):
    def __init__(self, template_file_path, *args: object, **kwargs) -> None:
        super().__init__(
            type_='template_file_not_found',
            msg=f'can not find the template file at the specified path: {template_file_path}',
            pretty_msg=f'Файл шаблона не найден: {template_file_path}'
        )


class TemplateIDNotFoundException(ExportManagerException):
    def __init__(self, template_id, *args: object, **kwargs) -> None:
        super().__init__(
            type_='template_id_not_found',
            msg=f'can not find template file with id: {template_id}',
            pretty_msg=f'Не найдена форма с id: {template_id}'
        )


class ExportTypeNotFoundException(ExportManagerException):
    def __init__(self, export_type, *args: object, **kwargs) -> None:
        super().__init__(
            type_='export_type_not_found',
            msg=f'this template id does not have export type: {export_type}',
            pretty_msg=f'Данный тип объекта не поддерживает тип экспорта {export_type}'
        )


class UnknownExportTypeException(ExportManagerException):
    def __init__(self, export_type, *args: object, **kwargs) -> None:
        super().__init__(
            type_='unknown_export_type',
            msg=f'unknown export type: {export_type}',
            pretty_msg=f'Неизвестный тип экспорта: {export_type}'
        )


class ExportItemNotFoundException(ExportManagerException):
    def __init__(self, export_item, *args: object, **kwargs) -> None:
        super().__init__(
            type_='export_item_not_found',
            msg=f'export_item_not_found: {export_item}',
            pretty_msg=f'Не найден экспорт с object_id: {export_item}'
        )


class PdfExportException(ExportManagerException):
    def __init__(self, form_id, *args: object, **kwargs) -> None:
        super().__init__(
            type_='pdf_export_exception',
            msg=f'pdf_export_exception: {form_id}',
            pretty_msg=f'Невозможно получить экспорт документа в формате PDF для form_id: {form_id}, так как '
                       f'отсутствует промежуточный формат DOCX '
        )
