import os
from datetime import date

from openpyxl import Workbook
from openpyxl.styles.borders import Border, Side

from exportmanager.excpetions import UnknownExportTypeException
from exportmanager.templateengine.docx.docx import create_docx_document
from exportmanager.templateengine.xlsx.xlsx import create_xlsx_document2
from repository.forms import forms_repository
from repository.forms.exceptions import ModelItemNotFoundException
from repository.reports import reports_repository
from repository.types import types_repository
from wsgi.apps.reporting_forms.models import FormModel
from wsgi.apps.voc.models import OrganizationVocModel, VocValueModel


class ExportFormManager:

    def __init__(self, request, form_data, form_repository):
        self.request = request
        self.form_data = form_data
        self.form_repository = form_repository

    def _prepare(self):

        def iso_date_to_normal():
            for k in ('form_period_start', 'form_period_end'):
                if self.form_data['form_main_data'][k]:
                    self.form_data['form_main_data'][k] = self.form_data['form_main_data'][k].strftime('%d.%m.%Y')

        self.form_data['form_main_data']['cor_num'] = self.form_data['adjustment_no'] if self.form_data[
                                                                                             'adjustment_no'] is not None else ''

        for k in ('form_period_start', 'form_period_end', 'notification_date'):
            if self.form_data[k] is not None:
                self.form_data['form_main_data'][k] = self.form_data[k].strftime('%d.%m.%Y')

        for k in ('reporting_year', 'uniq_numb'):
            if self.form_data[k] is not None:
                self.form_data['form_main_data'][k] = self.form_data[k]

        for form_main_attr_model in self.form_repository['form_main_attrs']:
            if 'type' not in form_main_attr_model:
                continue
            try:
                type_ = types_repository.get_type_repository(form_main_attr_model['type'])
                if type_['base_type'] == 'VOC' and type_.get('refresh_label') and type_.get(
                        'translate_value_id') and type_.get('refresh_label_to_export_forms'):
                    voc_type = type_.get('voc_type_db')
                    if voc_type is None:
                        voc_type = type_['voc_type']
                    current_value = self.form_data['form_main_data'][form_main_attr_model['field']]
                    res_value = []
                    for value in current_value.split('; '):
                        new_value = VocValueModel(self.request).filter([
                            'and',
                            ['=', 'voc_type', voc_type],
                            ['=', 'value_id', value]
                        ])[0]
                        new_value = ' '.join(str(new_value.attrs[k]) for k in type_['label'])
                        res_value.append(new_value)
                    self.form_data['form_main_data'][form_main_attr_model['field']] = '; '.join(res_value)
            except:
                continue

        for table_name, table_repo in self.form_repository['tables'].items():
            for model in table_repo['model']:
                if 'type' not in model:
                    continue
                try:
                    type_ = types_repository.get_type_repository(model['type'])
                    if type_['base_type'] == 'VOC' and type_.get('refresh_label') and type_.get(
                            'translate_value_id') and type_.get('refresh_label_to_export_forms'):
                        voc_type = type_.get('voc_type_db')
                        if voc_type is None:
                            voc_type = type_['voc_type']
                        for row in self.form_data['form_table_data'][table_name]:
                            current_value = row[model['field']]
                            res_value = []
                            for value in current_value.split('; '):
                                v = VocValueModel(self.request).filter([
                                    'and',
                                    ['=', 'voc_type', voc_type],
                                    ['=', 'value_id', value]
                                ])[0]
                                new_value = ' '.join(str(v.attrs[k]) for k in type_['label'])
                                res_value.append(new_value)
                            row[model['field']] = '; '.join(res_value)
                except:
                    continue

        # date
        for table_name, table_data in self.form_data['form_table_data'].items():
            for row in table_data:
                for attr, attr_value in row.items():
                    try:
                        type_ = types_repository.get_type_from_form_repository_by_field_name(self.form_repository, attr,
                                                                                             table_name)
                    except:
                        type_ = None
                    if type_ is not None and type_['base_type'] == 'DATE' and attr_value:
                        try:
                            date_ = date.fromisoformat(attr_value)
                            row[attr] = date_.strftime('%d.%m.%Y')
                        except ValueError:
                            pass

        for form_main_attr_repo_model in self.form_repository['form_main_attrs']:
            if not form_main_attr_repo_model.get('type'):
                continue
            try:
                type_ = types_repository.get_type_repository(form_main_attr_repo_model['type'])
            except:
                type_ = None
            if type_ is not None and type_['base_type'] == 'DATE':
                field = form_main_attr_repo_model['field']
                try:
                    date_ = date.fromisoformat(self.form_data['form_main_data'][field])
                    self.form_data['form_main_data'][field] = date_.strftime('%d.%m.%Y')
                except (ValueError, KeyError):
                    pass

        # заполнение нумерации колонок у примечаний, комментариев на основе порядке в репозитории
        for t in ('form_remarks', 'form_comments'):
            item = self.form_data[t]
            for table_name, t_data in item.items():
                for si in t_data:
                    try:
                        model_item = forms_repository.get_model_item_by_field(self.form_data['form_no'],
                                                                              table_name, si['pointer']['col_name'])
                    except (KeyError, ModelItemNotFoundException):
                        si['pointer']['col'] = ''
                        continue

                    if 'col_num' in model_item:
                        if model_item['col_num'] == '':
                            si['ignore_at_templates'] = True  # не включать в печатную форму
                        else:
                            si['pointer']['col'] = model_item['col_num']  # нумерация указана в репозитории
                    else:
                        si['pointer']['col'] = forms_repository.get_field_order_num(self.form_data['form_no'],
                                                                                    table_name,
                                                                                    si['pointer']['col_name']) + 2
                    si['pointer']['row'] = str(int(si['pointer']['row']) + 1)

        for table_name, table_data in self.form_repository['tables'].items():
            if table_data.get('convert_to_main_attrs'):
                try:
                    self.form_data['form_main_data'].update(self.form_data['form_table_data'][table_name][0])
                except (KeyError, IndexError):
                    pass

    def export(self, template_file_path, export_type, result_file_path):  # DO NOT USED !!!!!
        if export_type == 'docx':
            # self._prepare()
            return self.export_to_docx(template_file_path, result_file_path)
        elif export_type == 'xlsx':
            # self._prepare()
            return self.export_to_xlsx(template_file_path, result_file_path)
        elif export_type == 'pdf':
            return self.export_to_pdf(template_file_path, result_file_path)
        else:
            raise UnknownExportTypeException(export_type)

    def export_to_docx(self, template_file_path, result_file_path):
        self._prepare()
        create_docx_document(self.form_data, template_file_path, result_file_path)

    def export_to_xlsx(self, template_file_path, result_file_path):
        self._prepare()
        create_xlsx_document2(self.form_data, template_file_path, result_file_path)

    @staticmethod
    def export_to_pdf(docx_file_path, result_file):
        os.system(
            f'libreoffice --headless --convert-to pdf {docx_file_path} --outdir {result_file}')


def export_areport_to_xlsx(areport, file_):
    rep = areport['repository']
    data = areport['data']

    wb = Workbook()
    ws = wb.active

    thin_border = Border(left=Side(style='thin'),
                         right=Side(style='thin'),
                         top=Side(style='thin'),
                         bottom=Side(style='thin'))

    u = 0

    for table, table_rep in rep['tables'].items():

        if not data['form_table_data'][table]:
            continue

        row_c = 1

        tab_name = table_rep['tab_name'].replace(':', '')[:min(len(table_rep['tab_name']), 25)] + '...'
        if not u:
            try:
                ws.title = tab_name
            except ValueError:  # ValueError: Invalid character / found in sheet title
                ws.title = 'Аналитический отчет'
        else:
            ws = wb.create_sheet(tab_name)
        if not data['form_table_data'][table]:
            row_c += 1
            ws.cell(row_c, 1).value = 'нет данных'
            row_c += 1
            continue
        ws.merge_cells(start_row=row_c, start_column=1,
                       end_row=row_c, end_column=1 + len(data['form_table_data'][table][0]))

        headers = table_rep['headers']

        def count_headers_depth(header_, dept_):
            if isinstance(header_, dict):
                return dept_ + count_headers_depth(header_, dept_ + 1)
            return dept_

        headers_level_depth = count_headers_depth(headers, 1)

        row_c += 1

        ws.cell(row=row_c, column=1).value = 'п/п'
        ws.cell(row=row_c, column=1).border = thin_border
        ws.merge_cells(start_row=row_c, start_column=1,
                       end_row=row_c + headers_level_depth - 1, end_column=1)
        col_c = 2

        for header in headers:
            for model in table_rep['model']:
                if model['field'] == header:
                    ws.cell(row_c, col_c).value = model['caption']
                    ws.cell(row_c, col_c).border = thin_border
                    col_c += 1

        row_c += 1
        ws.cell(row_c, 1).value = 1

        for i in range(2, len(headers) + 2):
            ws.cell(row_c, i).value = i
            ws.cell(row_c, i).border = thin_border

        row_c += 1

        for i, row in enumerate(data['form_table_data'][table]):
            ws.cell(row_c, 1).value = (i + 1)
            ws.cell(row_c, 1).border = thin_border
            col_c = 2
            for header in headers:
                for model in table_rep['model']:
                    if model['field'] == header:
                        value = row[header]
                        ws.cell(row_c, col_c).value = value
                        ws.cell(row_c, col_c).border = thin_border
                        col_c += 1

            row_c += 1

        row_c += 1
        u += 1

    wb.save(file_)


class ExportTitlePages:

    def __init__(self, org: OrganizationVocModel.Data, template_file_path, request, form: FormModel.Data):
        self.conn = request
        self.org = org
        self.template_file_path = template_file_path
        self.data_to_export = {'form_main_data': {}}
        self.data_to_export['form_main_data']['reporting_year'] = str(form.reporting_year)
        self._prepare()

    def _prepare_fields(self, org: OrganizationVocModel.Data, postfix=''):
        """Подготовить поля организации к выгрузке"""
        # орган управления атомной энергией - вывод наименования, не кода
        try:
            if org.KOD_IAE is not None:
                voc_values = VocValueModel(self.conn).filter([
                    'and',
                    ['=', 'voc_type', '114_voc'],
                ])
                KOD_IAE_value = next(filter(lambda v: v.attrs['a'] == org.KOD_IAE, voc_values)).attrs['2']
                self.data_to_export['form_main_data']['KOD_IAE'] = KOD_IAE_value
        except (IndexError, KeyError, StopIteration):
            pass
        # субъект РФ - вывод наименования, не кода
        if org.KOD_SRF is not None:
            try:
                voc_values = VocValueModel(self.conn).filter([
                    'and',
                    ['=', 'voc_type', '115_voc'],
                ])
                for v in voc_values:
                    if v.attrs['s'] == org.KOD_SRF or str(v.attrs['s']) == org.KOD_SRF:
                        self.data_to_export['form_main_data']['KOD_SRF' + postfix] = str(v.attrs['2'])
                        break
            except (IndexError, KeyError, StopIteration):
                pass

    def _prepare(self):
        # если головная организация
        if self.org.PRZN_OP == OrganizationVocModel.Data.PrznOpTypes.ENTITY:
            self._set_data_to_export(self.org.as_dict(), '_G')
            self._set_data_to_export(self.org.as_dict()['attrs'], '_G')
            self._prepare_fields(self.org, '_G')
        else:  # подразделение
            # находим головную организацию
            try:
                gol_org = OrganizationVocModel(self.conn).filter({'record_id': self.org.KOD_GOL_ORG})[0]
                self._set_data_to_export(gol_org.as_dict(), '_G')
                self._set_data_to_export(gol_org.as_dict()['attrs'], '_G')
                self._prepare_fields(gol_org, '_G')
            except IndexError:  # не найдена головная организация с таким кодом
                pass
            self._set_data_to_export(self.org.as_dict())
            self._prepare_fields(self.org)
            self._set_data_to_export(self.org.as_dict()['attrs'])

    def _set_data_to_export(self, source, postfix=''):
        if source:
            for k, v in source.items():
                if k == 'attrs':
                    continue
                if v is None:
                    v = ''
                if k != 'KOD_ORG':
                    k += postfix
                self.data_to_export['form_main_data'][k] = v

    def export_to_docx(self, result_file_path):
        create_docx_document(self.data_to_export, self.template_file_path, result_file_path)

    def export_to_pdf(self, docx_file_path, result_file):
        os.system(
            f'libreoffice --headless --convert-to pdf {docx_file_path} --outdir {result_file}')
