from docx import Document
from docx.opc.exceptions import PackageNotFoundError

from exportmanager.templateengine.docx.docx_table import set_document_tables, set_remarks2
from exportmanager.templateengine.docx.docx_text import set_document_text, delete_all_unused_tags
from exportmanager.excpetions import TemplateFileNotFoundException


def create_docx_document(data: dict, template_file_path: str, result_file_path: str):
    """Создает новый документ docx на основе docx шаблона и данных словаря

    Args:
        data (dict): Данные для заполнения
        template_file_path (str): Путь к файлу docx шаблона
        result_file_path (str): Путь к файлу для сохранения нового документа

    Warnings:
        FixedElementTagNotFoundWarning: В шаблоне не найден фиксированный элемент с таким тэгом
        TableTagNotFoundWarning: В шаблоне не найдена таблица с таким тэгом
        CellTagNotFoundWarning: В шаблоне не найдена ячейка с таким тэгом
    """
    try:
        doc = Document(template_file_path)
    except PackageNotFoundError:
        raise TemplateFileNotFoundException(template_file_path)

    # text_section = data.get('text')
    text_section = data.get('form_main_data')  # TODO: возможно экспорт будет не только отчетных форм, ключ может передаваться извне
    # tables_section = data.get('tables')
    tables_section = data.get('form_table_data')
    repository_section = data.get('repository')

    set_document_text(doc, text_section, repository_section)
    set_document_tables(doc.tables, tables_section)

    set_remarks2(doc.tables, data.get('form_remarks'))

    delete_all_unused_tags(doc)

    doc.save(result_file_path)
