import os

from api2 import get_api2
from repository.areports.exceptions import AreportRepositoryNotFoundException
from repository.utils import is_py_file, import_module_

areports_repository = {}  # { areport_file_name -> areport_data }

areports_types_list_repository_f = []  # список типов аналитических отчетов для выпадающего списка на фронте


def _init_areport_repo(areport_data, name):
    params_types = {}
    for i, param in enumerate(areport_data['params']):
        if 'type' in param and isinstance(param['type'], dict):
            params_types[param['field'] + '_type_'] = {
                'base_type': 'DICT',
                'data_source': param['type'],
            }
            areport_data['params'][i]['type'] = param['field'] + '_type_'

    areport_data['params_types'] = params_types

    areports_repository[name] = areport_data

    areport_type = {
        'label': areport_data['name'],
        'value': name,
    }
    areports_types_list_repository_f.append(areport_type)


def init_areports_repository(areports_repository_files_dirs):
    storage = []

    global areports_repository
    for rep_dir in areports_repository_files_dirs:
        for file_ in os.listdir(rep_dir):
            if not is_py_file(file_):
                continue

            try:
                file_name, file_ext = file_.split('.')
            except Exception:
                continue

            full_path = os.path.join(rep_dir, file_)
            report_file_module = import_module_(rep_dir, file_name)
            areport_file_data = report_file_module.init(api=get_api2())

            storage.append({
                'path': full_path,
                'file_data': areport_file_data,
            })

            params_types = {}
            for i, param in enumerate(areport_file_data['params']):
                if 'type' in param and isinstance(param['type'], dict):
                    params_types[param['field'] + '_type_'] = {
                        'base_type': 'DICT',
                        'data_source': param['type'],
                    }
                    areport_file_data['params'][i]['type'] = param['field'] + '_type_'

            areport_file_data['params_types'] = params_types

            areports_repository[file_name] = areport_file_data

            areport_type = {
                'label': areport_file_data['name'],
                'value': file_name,
            }
            areports_types_list_repository_f.append(areport_type)

    return storage


def add_extra_areports_repositories(areport_repositories):
    global areports_repository
    for areport_repo in areport_repositories:
        _init_areport_repo(areport_repo, areport_repo['name'])


def get_areport_repository(areport_name):
    try:
        return areports_repository[areport_name]
    except KeyError:
        raise AreportRepositoryNotFoundException(areport_name)


def get_areport_types_list_f():
    return areports_types_list_repository_f
