# decrees_types_list_repository_f = []  # список типов приказов для выпадающего списка на фронте
import os

from repository.decrees.exceptions import DecreeRepositoryNotFoundByYearFromException, DecreeRepositoryNotFoundException

decrees_repository = {}


def init_decrees_repository(items):
    for item in items:
        item_data = item['file_data']
        path = item['path']
        file_name = os.path.splitext(os.path.basename(path))[0]
        decrees_repository[file_name] = item_data


def get_decrees_types_list_repository():
    decrees_types_list = []
    for decree_repo in decrees_repository.values():
        if not decree_repo.get('choice_allowed'):
            continue
        decree_type = {
            'label': decree_repo['name'],
            'value': decree_repo['year_from'],
        }
        if decree_repo.get('default'):
            decree_type['default'] = decree_repo['default']
        decrees_types_list.append(decree_type)
    return decrees_types_list


def get_decree_repository_by_year_from(decree_year_from):
    try:
        return next(filter(lambda decree_repo: decree_repo['year_from'] == decree_year_from, decrees_repository.values()))
    except StopIteration:
        raise DecreeRepositoryNotFoundByYearFromException(decree_year_from)


def get_decree_repository_by_name(name):
    try:
        return next(filter(lambda decree_repo: decree_repo['name'] == name, decrees_repository.values()))
    except StopIteration:
        raise DecreeRepositoryNotFoundException(f'Not found by name: {name}')
