from services.exceptions import CoreBaseException


class RepositoryException(CoreBaseException):
    ...


class ModuleTypeNotFoundException(RepositoryException):
    def __init__(self, file_name, *args: object, **kwargs) -> None:
        super().__init__(
            type='ModuleTypeNotFoundException',
            message=f'Not found required key "module_type" in file: {file_name}',
        )


class UnknownModuleTypeException(RepositoryException):
    def __init__(self, module_type, *args: object, **kwargs) -> None:
        super().__init__(
            type='UnknownModuleTypeException',
            message=f'Unknown module type: {module_type}',
        )


class RepositoryItemNotFoundException(RepositoryException):
    def __init__(self, object_id, *args: object, **kwargs) -> None:
        super().__init__(
            type='repository_item_not_found',
            message=f'repository_item_not_found: {object_id}',
        )


class VocTypeNotFoundException(RepositoryException):
    def __init__(self, voc_type, *args: object, **kwargs) -> None:
        super().__init__(
            type='voc_type_not_found_exception',
            message=f'voc_type_not_found_exception: {voc_type}',
        )


class VocKeyNotFoundException(RepositoryException):
    def __init__(self, key, *args: object, **kwargs) -> None:
        super().__init__(
            type='VocKeyNotFoundException',
            message=f'VocKeyNotFoundException. Required: {key}',
        )


class RepositoryValidationException(RepositoryException):
    ...


class RequiredKeyNotFoundException(RepositoryValidationException):
    def __init__(self, key, path, *args: object, **kwargs) -> None:
        super().__init__(
            type='RequiredKeyNotFoundException',
            message=f'RequiredKeyNotFoundException. Key: {key}, path: {path}',
        )


class InvalidTypeException(RepositoryValidationException):
    def __init__(self, key, path, type_required, *args: object, **kwargs) -> None:
        super().__init__(
            type='InvalidTypeException',
            message=f'InvalidTypeException. Key: {key}, path: {path}, type_required: {type_required}',
        )


# class CustomTypeDoesNotExistsException(RepositoryValidationException):
#     def __init__(self, type_, path, *args: object, **kwargs) -> None:
#         super().__init__(
#             type='CustomTypeDoesNotExistsException',
#             message=f'CustomTypeDoesNotExistsException. Type: {type_} Path: {path}',
#         )


class UnknownSelectModeException(RepositoryValidationException):
    def __init__(self, select_mode, *args: object, **kwargs) -> None:
        super().__init__(
            type='UnknownSelectModeException',
            message=f'UnknownSelectModeException. select_mode: {select_mode}',
        )


class RegisterTypeNotFoundException(RepositoryException):
    def __init__(self, register_type, *args: object, **kwargs) -> None:
        super().__init__(
            type='RegisterTypeNotFoundException',
            message=f'RegisterTypeNotFoundException: {register_type}',
        )
