from repository.forms.exceptions import FormRefersToNonExistenceTypeException
from repository.types import types_repository
from repository.validation import check_config


def check_form_configuration(form_data, file_path):
    """Проверить структуру формы в пользовательском файле"""

    config = {
        'form_name': str,
        'form_num': str,
        'form_main_attrs': [
            {
                'field': str,
                'caption': str,
            }
        ],
        'tables': {
            '_': {
                'tab_name': str,
                'headers': list,
                'model': [
                    {
                        'field': str,
                        'caption': str,
                    }
                ]
            },
        }
    }
    check_config(config, form_data, file_path)


def check_form_types(form_name, form_data):
    models_section = form_data['tables']
    for table_name, table_data in models_section.items():
        for cell in table_data['model']:
            if 'type' in cell:
                try:
                    types_repository.get_types_repository()[cell['type']]
                except KeyError:
                    raise FormRefersToNonExistenceTypeException(form_name, table_name, cell['field'], cell['type'])
