import os

import settings
from repository.forms_journals.exceptions import FormsJournalRepositoryNotFoundException, \
    FormsJournalPresetNotFoundException
from repository.forms_journals.validation import check_report_form_journal_configuration
from repository.types import types_repository

forms_journals_repository = {}
forms_journals_repository_f = {}
forms_journals_list_repository_f = []  # список журналов форм для выпадающего списка на фронте


def init_forms_journals_repository(items):
    global forms_journals_list_repository_f
    for item in items:
        file_data = item['file_data']
        path = item['path']
        file_name = os.path.splitext(os.path.basename(path))[0]

        check_report_form_journal_configuration(file_data, path)

        forms_journals_repository[file_name] = file_data

        file_data['model'].append({
            'field': 'context_menu',
            'caption': '',
            'type': 'slot',
            'width': '50px',
            'slot': 'context_menu',
        })

        if not file_data.get('headers'):
            file_data['headers'] = [model['field'] for model in file_data['model']]

        forms_journals_repository_f[file_name] = {
            'edit': {
                'create_report_form': file_data.get('create_report_form', True),
            },
            'types': types_repository.get_types_repository(),
            'table': {
                'headers': file_data['headers'],
                'model': file_data['model'],
                'options': {
                    'server_filter_sort': False if settings.DB_SYSTEM == 'sqlite' else True,
                }
            },
        }
        if 'lazy_load_size' in file_data:
            forms_journals_repository_f[file_name]['lazy_load_size'] = file_data['lazy_load_size']

        forms_journal_list_repository_f = {
            'label': file_data['journal_name'],
            'value': file_name,
        }

        if 'presets' in file_data:
            forms_journal_list_repository_f['presets'] = [
                {'label': v['label'], 'value': k} for k, v in file_data['presets'].items()]

        # формирование filter и sort - варианты фильтрации для полей справочника (модели) и сортировки
        for model_item in filter(lambda x: x['field'] != 'context_menu', file_data['model']):
            # if 'filter' not in model_item:
            #     model_item['filter'] = ['=', 'like']
            if 'sort' not in model_item:
                model_item['sort'] = True

        forms_journals_list_repository_f.append(forms_journal_list_repository_f)

    def sort_forms_journals_list_repository_f(item_):
        value = item_['value']
        try:
            return int(value.split('_')[0])
        except (IndexError, ValueError):
            return 10000

    forms_journals_list_repository_f.sort(key=sort_forms_journals_list_repository_f)


def get_forms_journals():
    return forms_journals_repository


def get_forms_journal(journal_name):
    try:
        return forms_journals_repository[journal_name]
    except KeyError:
        raise FormsJournalRepositoryNotFoundException(journal_name)


def get_forms_journals_types_list_repository_f():
    return forms_journals_list_repository_f


def get_forms_journal_repository_f(journal_name):
    try:
        return forms_journals_repository_f[journal_name]
    except KeyError:
        raise FormsJournalRepositoryNotFoundException(journal_name)


def get_preset_query(journal_name, preset):
    try:
        return get_forms_journal(journal_name)['presets'][preset]['query']
    except KeyError:
        raise FormsJournalPresetNotFoundException(journal_name, preset)
