import os

from repository.registers.exceptions import FormRegisterNotFoundException, RegisterNotFoundException
from repository.utils import is_py_file, import_module_

register_repository = {}  # file_name -> init()
forms_registers = {}  # form_name ->


def init_registers_repository(items):
    for item in items:
        register_file_data = item['file_data']
        path = item['path']
        file_name = os.path.splitext(os.path.basename(path))[0]

        register_repository[file_name] = register_file_data

        for form_name, value in register_file_data['forms'].items():
            forms_registers[form_name] = {
                'register': file_name,
                'table_name': register_file_data['table_name'],
                'map': value['map'],
                'acc_obj': value.get('acc_obj'),
                'sum_attrs': value.get('sum_attrs'),
                'plus_operations': value.get('plus_operations'),
                'minus_operations': value.get('minus_operations'),
            }


def get_form_register_by_form_name(form_name):
    try:
        return forms_registers[form_name]
    except KeyError:
        raise FormRegisterNotFoundException(form_name)


def get_register_repository(register_name):
    try:
        return register_repository[register_name]
    except KeyError:
        raise RegisterNotFoundException(register_name)
