from repository.exceptions import RepositoryException


class RepositoryTypesException(RepositoryException):
    ...


class TypeRepositoryNotFoundException(RepositoryTypesException):
    def __init__(self, type_name, *args: object, **kwargs) -> None:
        super().__init__(
            type='VocRepositoryNotFoundException',
            message=f'Type repository not found by type_name = "{type_name}"',
        )


class RequiredKeyTypesNotFoundException(RepositoryTypesException):
    def __init__(self, file_path, *args: object, **kwargs) -> None:
        super().__init__(
            type='RequiredKeyTypesNotFoundException',
            message=f'Not found required key "types" in type file: {file_path}',
        )


class CustomTypeDoesNotExistsException(RepositoryTypesException):
    def __init__(self, type_, path, *args: object, **kwargs) -> None:
        super().__init__(
            type='CustomTypeDoesNotExistsException',
            message=f'CustomTypeDoesNotExistsException. Type: {type_} Path: {path}',
        )


class CustomTypeNotFoundException(RepositoryTypesException):
    def __init__(self, type_, *args: object, **kwargs) -> None:
        super().__init__(
            type='CustomTypeNotFoundException',
            message=f'CustomTypeNotFoundException. Type: {type_}',
        )


class BaseOverrideTypeNotFoundException(RepositoryTypesException):
    def __init__(self, base_type, *args: object, **kwargs) -> None:
        super().__init__(
            type='BaseOverrideTypeNotFoundException',
            message=f'Base type not found: {base_type}',
        )
