from repository.exceptions import RepositoryException


class RepositoryVocsException(RepositoryException):
    ...


class VocTypeKeyNotFoundInVocTypeException(RepositoryVocsException):
    def __init__(self, voc_type_name, *args: object, **kwargs):
        super().__init__(
            type='VocTypeKeyNotFoundInVocTypeException',
            message=f'Required key "voc_type". Not found in voc type: {voc_type_name}'
        )


class CustomVocTypeKeyNotFoundException(RepositoryVocsException):
    def __init__(self, voc_type_name, voc_type, *args: object, **kwargs):
        super().__init__(
            type='VocTypeNotFoundException',
            message=f'Can not find "voc_name", in custom voc type: {voc_type_name}. (can be it is not custom,'
                    f'but I can not find it in Vocs database table by type: {voc_type}. If you are sure that the '
                    f'voc is not custom, then check for a typo in the {voc_type}, it must match the value in the '
                    f'database) '
        )


class VocTypeNotFoundException(RepositoryVocsException):
    def __init__(self, voc_type, *args: object, **kwargs) -> None:
        super().__init__(
            type='voc_type_not_found_exception',
            message=f'voc_type_not_found_exception: {voc_type}',
        )


class VocRepositoryNotFoundException(RepositoryVocsException):
    def __init__(self, voc_type, *args: object, **kwargs) -> None:
        super().__init__(
            type='VocRepositoryNotFoundException',
            message=f'Voc repository not found with voc_type = "{voc_type}"',
        )


class ValueIdNotFoundInModelException(RepositoryVocsException):
    def __init__(self, value_id, *args: object, **kwargs) -> None:
        super().__init__(
            type='ValueIdNotFoundInModelException',
            message=f'Value_id = {value_id} refers to a non-existent field. Field not found in model section',
        )


class LabelFieldNotFoundInModelException(RepositoryVocsException):
    def __init__(self, label_field, label, *args: object, **kwargs) -> None:
        super().__init__(
            type='LabelFieldNotFoundInModelException',
            message=f'Label field = {label_field} refers to a non-existent field. Field not found in model section. '
                    f'Label: {label}',
        )


class SortFieldNotFoundInModelException(RepositoryVocsException):
    def __init__(self, sort_field, path, *args: object, **kwargs) -> None:
        super().__init__(
            type='SortFieldNotFoundInModelException',
            message=f'Sort field = {sort_field} refers to a non-existent field. Field not found in model section. '
                    f'Path: {path}',
        )


class VocModelItemNotFound(RepositoryVocsException):
    def __init__(self, voc_type, field, *args: object, **kwargs) -> None:
        super().__init__(
            type='VocModelItemNotFound',
            message=f'voc_type = {voc_type}, field = {field}',
        )
