import re

from repository.validation import check_config
from repository.vocs.exceptions import ValueIdNotFoundInModelException, LabelFieldNotFoundInModelException, \
    SortFieldNotFoundInModelException, RepositoryVocsException
from services.exceptions import CoreBaseException


def check_voc_configuration(voc_data, file_path):
    """Проверить структуру справочника в пользовательском файле"""

    config = {
        'base_type': str,
        'label': [],
        'value_id': str,
        'voc_type': str,
        # 'voc_name': str,
        'headers': [
        ],
        'model': [

        ]
    }
    check_config(config, voc_data, file_path)

    if 'sort' in voc_data:
        for item in voc_data['sort']:
            if item.startswith('-'):
                item = item[1:]
            if item not in (model_item['field'] for model_item in voc_data['model']):
                raise SortFieldNotFoundInModelException(item, file_path)

    # проверка model
    for model_item in voc_data['model']:
        if 'field' not in model_item:
            raise RepositoryVocsException(message=f'Ошибка в одном из элементов секции "model" справочника, не найден '
                                                  f'обязательный параметр "field". Проверьте, возможно в имени '
                                                  f'параметра опечатка. Имя справочника: {file_path}, элемент "model": '
                                                  f'{model_item}')


def check_voc_value_id(value_id_field, model_section):
    """Проверить, что value_id ссылается на существующий field"""
    if value_id_field == 'record_id':
        return
    for model in model_section:
        if model['field'] == value_id_field:
            return
    raise ValueIdNotFoundInModelException(value_id_field)


def check_voc_label(label, model_section):
    """Проверить, что value_id ссылается на существующий field"""
    if isinstance(label, list):
        for field in label:
            if field not in (model['field'] for model in model_section):
                raise LabelFieldNotFoundInModelException(field, label)
    elif isinstance(label, str):
        fields = [model_item['field'] for model_item in model_section]
        regex = r'{\w+}'
        fields_in_label = re.findall(regex, label)
        fields_in_label = [f[1:-1] for f in fields_in_label]
        fields_not_found_in_model = list(filter(lambda f: f not in fields, fields_in_label))
        if fields_not_found_in_model:
            raise LabelFieldNotFoundInModelException(fields_not_found_in_model, label)
    else:
        raise CoreBaseException(message=f'unknown voc label format: {type(label)}, label: {label}')
