import os
import sys
import pathlib

try:
    project_dir = pathlib.Path(__file__).parent.resolve().parents[0]
    site_packages_path = os.path.join(project_dir, 'site-packages')
    sys.path.append(site_packages_path)
except Exception as e:
    print(f'ERROR: can not add site-packages, exception: {e}')

import settings

from loguru import logger
from waitress import serve

import pandas as pd  # do not remove
from api2 import init_api2
from repository.areports.areports_repository import init_areports_repository, add_extra_areports_repositories
from repository.extensions.extensions_repository import init_extensions_repository
from repository.repository import init as init_repository
from repository.types import types_repository
from settings import HOST, PORT, THREADS, AREPORTS_REPOSITORY_FILES_DIRS, \
    EXTENSIONS_FILES_DIRS, RVRAO_MODULE_PATH, FC_RAO_MODULE_PATH, CUSTOM_FILES_DIRS
from wsgi.wsgi_app import make_app

from logger import init_logger


def init(serve_static):
    os.environ['PATH'] += os.pathsep + '/usr/bin'

    sys.path.insert(1, RVRAO_MODULE_PATH)
    __import__('rvrao', globals(), locals(), ['rvrao'])

    sys.path.insert(1, FC_RAO_MODULE_PATH)
    __import__('form_fc_rao', globals(), locals(), ['form_fc_rao'])

    context_for_api2 = init_repository(
        custom_files_dirs=CUSTOM_FILES_DIRS,
        areports_repository_files_dirs=AREPORTS_REPOSITORY_FILES_DIRS,
        extensions_files_dirs=EXTENSIONS_FILES_DIRS,
    )

    app_obj, app_f = make_app(serve_static=serve_static)

    api2 = init_api2(app_obj, context_for_api2)

    logger.info(f'api2 initialization finished')

    init_areports_repository(AREPORTS_REPOSITORY_FILES_DIRS)
    init_extensions_repository(EXTENSIONS_FILES_DIRS)

    extra_types = api2.repo.attr_types.dump_models()
    types_repository.add_extra_types(extra_types)

    extra_areports_repositories_from_api2 = api2.repo.AReports.dump_models()
    add_extra_areports_repositories(extra_areports_repositories_from_api2)

    return app_obj


@logger.catch()
def main():
    init_logger()
    app_obj = init(settings.SERVE_STATIC)
    logger.info('====SERVER STARTED=====')
    if settings.DEBUG:
        import multiprocessing
        print(f'process PID: {multiprocessing.current_process().pid}')
    serve(app_obj, host=HOST, port=PORT, threads=THREADS, max_request_body_size=1073741824 * 5)


if __name__ == '__main__':
    main()
