class CoreBaseException(Exception):
    def __init__(self, *args: object, **kwargs) -> None:
        super().__init__(*args)
        for kwarg, kwarg_data in kwargs.items():
            setattr(self, kwarg, kwarg_data)

    def __str__(self) -> str:
        try:
            return self.message
        except AttributeError:
            return self.rus_message


class PermissionDeniedException(CoreBaseException):
    def __init__(self, role, action) -> None:
        super().__init__(
            type_='PermissionDeniedException',
            rus_message=f'Пользователь не обладает необходимыми правами на выполнение операции. Роль пользователя: {role}, действие: {action}'
        )


class CanNotReadJsonExportFileException(CoreBaseException):
    def __init__(self, *args: object, **kwargs) -> None:
        super().__init__(
            type='CanNotReadJsonExportFileException',
            message='Can not read file as JSON',
            rus_message='Ошибка при чтении данных из файла. Проверьте его правильность'
        )
