import datetime
import json

import settings
from services.services import is_str_in_date_format


def build_filter_in(param, values):
    return [
        'or',
        *[['=', param, str(v)] for v in values]
    ]


def build_filter_sql_line(f, database_table, allowed_filter_opers_for_field=None, select_join_map=None, fields_to_types=None, change_filter_objs=None,
                          request=None):
    logical_op = ('and', 'or')
    comparison_op = ('=', '<', '>', '<=', '>=', 'in', 'like', 'sql')

    if isinstance(f, str):
        f = json.loads(f)

    logical_op_count = 0
    params_to_values = {}  # имена параметров фильтрации к их значениям
    warnings_ = []

    def parse(x):
        nonlocal logical_op_count
        if not x:
            return

        # если нет операндов (['and'])
        if len(x) == 1:
            return

        if x[0] not in logical_op and x[0] not in comparison_op:
            raise Exception(f'первый элемент списка должен быть логической операцией или операцией сравнения: {x[0]}')

        if x[0] in logical_op:
            sub_results = []
            for sub_x in x[1:]:
                sub_res = parse(sub_x)
                if sub_res:
                    sub_results.append('(' + sub_res + ')')

            res = f' {x[0]} '.join(sub_results)

            if logical_op_count:
                res = f'({res})' if res else ''
            logical_op_count += 1
            return res

        elif x[0] in comparison_op:
            com, par, val, *mods = x

            add_database_table_name_to_par = True

            # Заменяем объект фильтрации, если пришел
            # Журнал отчетных форм
            # change_filter_objs = {
            #     ('=', 'decree'): ('like', 'form_no', lambda filter_value: f'%_{filter_value}'),
            # }
            # Поля decree нет в базе, поэтому фильтруем по form_no
            if change_filter_objs is not None and (com, par) in change_filter_objs:
                com, par, lambda_func_change_val = change_filter_objs[(com, par)]
                val = lambda_func_change_val(request, val)

            if val is not None and not isinstance(val, list):
                val = str(val)

            par_clear = par.split('.')[1] if '.' in par else par
            # if select_join_map and par_clear not in select_join_map - если поле из смежной таблицы
            # то не делаем проверку на допустимость операции фильтрации
            # ----
            # если поля нет в правиле разрешенных операций по фильтрации, но есть в join правилах, то не применяем
            # валидацию по фильтрации
            if allowed_filter_opers_for_field and par_clear not in allowed_filter_opers_for_field and select_join_map and par_clear in select_join_map:
                pass
            # иначе проверяем, разрешена ли операция по фильтрации для поля
            elif allowed_filter_opers_for_field is not None:
                # если поля нет в allowed_filter_opers_for_field, по полю не разрешена фильтрация
                # по умолчанию allowed_filter_opers_for_field = список полей модели
                if par_clear not in allowed_filter_opers_for_field:
                    warnings_.append(f'Unsupported filter param: \'{par}\'')
                    return
                # если указано 'all' для поля, по нему разрешены все операции
                # иначе указан список разрешенных операция для этого поля
                if (allowed_filter_opers_for_field[par_clear] != 'all' and
                        com not in allowed_filter_opers_for_field[par_clear]):
                    warnings_.append(f'Unsupported filter operation: \'{com}\' for param: \'{par}\'. '
                                     f'Allowed operations: {allowed_filter_opers_for_field[par_clear]}')
                    return

                if fields_to_types:
                    if com == 'like' and fields_to_types.get(par_clear) in (bool, datetime.date, datetime.datetime):
                        return

            if '.' not in par:
                # сохраняем список всех полей их их значения, по которым производится фильтрация
                params_to_values[par] = val
                if select_join_map:
                    if par in select_join_map:
                        add_database_table_name_to_par = False
                        par = select_join_map[par]
                    else:
                        par = f'"{par}"'
            else:
                json_field, json_par = par.split('.')
                # пример для справочника организаций: если attrs.NAME_FULL, проверяем, что NAME_FULL
                # является системным полем (поле таблицы), тогда убираем обертку из
                # с фронта приходит обретка attrs, поэтому необходимо делать проверку
                if fields_to_types is not None and json_par in fields_to_types:
                    par = f'"{json_par}"'
                else:
                    if settings.DB_SYSTEM == 'postgresql':
                        par = f'"{json_field}"->>\'{json_par}\''
                    elif settings.DB_SYSTEM == 'sqlite':
                        par = f'json_extract({json_field}, "$.{json_par}")'

            if is_str_in_date_format(val) or isinstance(val, str) and com != 'sql':
                val = val.replace("'", "''")
                val = f"'{val}'"

            line = []

            for mod in mods:
                if mod == 'not':
                    line.insert(0, 'not')

            if com == 'like':
                if 'complete_match' not in mods:
                    val = "'%" + val[1:-1] + "%'"
                    if 'not_ignore_case' not in mods:
                        val = f'lower({val})'
                        par = f'lower({par})'
                        add_database_table_name_to_par = False

            if com == 'sql':
                com = ''

            if com == 'in':
                val = '(' + ', '.join(f"'{v}'" for v in val) + ')'

            if fields_to_types is not None:
                if val == "''" and fields_to_types.get(par_clear) in (datetime.date, datetime.datetime, int):
                    com = 'is'
                    val = 'NULL'

                elif fields_to_types.get(par_clear) in (datetime.datetime, ) and val:
                    try:
                        time_value = datetime.date.fromisoformat(val[1:-1])  # срез - без одинарных кавычек
                        next_day_value = time_value + datetime.timedelta(days=1)
                        com = 'between'
                        val = f"'{time_value}' and '{next_day_value}'"
                    except ValueError:
                        pass

            if val is None:
                com = 'is'
                val = 'NULL'

            if add_database_table_name_to_par:
                if not par.endswith('"') and not par.startswith('"'):
                    par = f'"{par}"'
                if settings.DB_SYSTEM == 'postgresql':
                    par = f'"{database_table}".{par}'
                elif settings.DB_SYSTEM == 'sqlite' and 'json_extract' not in par:
                    par = f'"{database_table}".{par}'

            line.extend((par, com, val))

            return ' '.join(str(l) for l in line)

        else:
            raise Exception('ошибка фильтрации')

    filter_res = parse(f)
    filter_res = '(' + parse(f) + ')' if filter_res else ''

    return filter_res, params_to_values, warnings_
