import logging
import os.path

from settings import LOGGING_DIR_PATH

api2_error_format = logging.Formatter(
    f"======================================\n"
    f"%(asctime)s - %(url)s - %(levelname)-4s\n"
    f"\tException message: %(exception_message)s\n"
    f"\tAPI2 message: %(message)s\n"
    f"\t%(traceback)s"
)

api2_info_format = logging.Formatter(
    f"======================================\n"
    f"%(asctime)s - %(url)s - %(levelname)-4s\n"
    f"\tAPI2 message: %(message)s\n"
)

level_names_to_level = {
    'ERROR': logging.ERROR,
    'INFO': logging.INFO,
    'WARNING': logging.WARNING,
    'DEBUG': logging.DEBUG,
}

level_names_to_format = {
    'ERROR': api2_error_format,
    'INFO': api2_info_format,
}


def api2_log(user_login, log_level, url, exception_message, traceback, message):
    logger = logging.getLogger('api2_logger')
    logger.setLevel(logging.DEBUG)
    format_ = level_names_to_format[log_level]
    handler = logging.FileHandler(os.path.join(LOGGING_DIR_PATH, f'{user_login}.log'), 'a', 'utf-8')
    handler.setFormatter(format_)
    logger.addHandler(handler)

    logger = logging.LoggerAdapter(
        logger,
        {
            'user_login': user_login,
            'exception_message': exception_message,
            'url': url,
            'traceback': traceback,
        }
    )

    logger.log(level_names_to_level[log_level], message)
