import os
import random
import string


class TempFile:

    def __init__(self, dir_path, suffix, file_name=None, close_others=True, close_current=True):
        self.dir_path = dir_path
        self.suffix = suffix

        self.close_others = close_others
        self.close_current = close_current

        self._file_name_length = 20  # длина рандомного имени файла

        self._create_dir_if_not_exists()

        self.file_name = self._gen_file_name(file_name)
        self.file_name_only = self.file_name.split('.')[0]
        self.full_path = os.path.join(self.dir_path, self.file_name)
        self.file = open(self.full_path, 'w')

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        if self.close_others:
            for filename in os.listdir(self.dir_path):
                file_path = os.path.join(self.dir_path, filename)
                try:
                    os.remove(file_path)
                except:
                    pass
        elif self.close_current:
            try:
                os.remove(self.full_path)
            except:
                pass

    def delete_file(self):
        try:
            os.remove(self.full_path)
        except:
            pass

    def _create_dir_if_not_exists(self):
        try:
            os.mkdir(self.dir_path)
        except FileExistsError:
            pass

    def _gen_file_name(self, file_name):
        if file_name is None:
            file_name = ''.join(
                random.choice(string.ascii_uppercase + string.digits) for _ in range(self._file_name_length))
        file_name += self.suffix
        return file_name
