import os
import pathlib

from dotenv import load_dotenv
dotenv_path = os.path.join(os.path.dirname(__file__), '.env')
if os.path.exists(dotenv_path):
    load_dotenv(dotenv_path)

PROJECT_DIR = pathlib.Path(__file__).parent.resolve().parents[0]
SITE_PACKAGES_PATH = os.path.join(PROJECT_DIR, 'site-packages')

HOST = os.environ.get('HOST', '127.0.0.1')
PORT = os.environ.get('PORT', '8000')

WIN = False
DEBUG = False

THREADS = int(os.environ.get('THREADS', 4))

TEMP_DIR_PATH = os.getenv('TEMP_DIR_PATH', os.path.join(PROJECT_DIR, 'TMP', 'tmp'))
TEMP_DIR_DECREES_PATH = os.getenv('TEMP_DIR_DECREES_PATH', os.path.join(PROJECT_DIR, 'TMP', 'temp_decrees'))

CUSTOM_FILES_DIRS = os.environ.get('CUSTOM_FILES_DIRS')
if CUSTOM_FILES_DIRS is None:
    CUSTOM_FILES_DIRS = [os.path.join(PROJECT_DIR, 'custom')]
else:
    CUSTOM_FILES_DIRS = CUSTOM_FILES_DIRS.split(', ')

STATIC_FILES_DIR = os.environ.get('STATIC_FILES_DIR', os.path.join(CUSTOM_FILES_DIRS[0], 'static_files'))


RVRAO_MODULE_PATH = os.environ.get('RVRAO_MODULE_PATH', os.path.join(PROJECT_DIR, 'rvrao-ext'))
FC_RAO_MODULE_PATH = os.environ.get('FC_RAO_MODULE_PATH', os.path.join(RVRAO_MODULE_PATH, 'forecast'))


AREPORTS_REPOSITORY_FILES_DIRS = [
    os.path.join(RVRAO_MODULE_PATH, 'reports')
]
EXTENSIONS_FILES_DIRS = [
    os.path.join(RVRAO_MODULE_PATH, 'actions')
]

DB_SYSTEM = 'postgresql'

POSTGRES_DATABASE = os.environ.get('POSTGRES_DATABASE')
DB_HOST = os.environ.get('POSTGRES_HOST', '127.0.0.1')
DB_PORT = os.environ.get('POSTGRES_PORT', '5432')

DB_USER = {
    'user': os.environ.get('POSTGRES_USER'),
    'password': os.environ.get('POSTGRES_PASSWORD'),
}

API2_PATH = os.environ.get('API2_PATH', os.path.join(PROJECT_DIR, 'api2'))
EXT_PATH = os.environ.get('EXT_PATH', os.path.join(PROJECT_DIR, 'api2', 'extensions'))
API2_FILES_TMP = os.path.join(PROJECT_DIR, 'TMP', 'temp_decrees')

API2_DEBUG = False

VIAC_ENABLED = True
RIAC_ENABLED = True

LOGGING_DIR_PATH = os.environ.get('LOGGING_DIR_PATH', os.path.join(PROJECT_DIR, 'logging'))
RADIO_CSV_PATH = os.environ.get('RADIO_CSV_PATH', os.path.join(PROJECT_DIR, 'custom', 'static_files', 'radionuclides.csv'))
RAISE_EX = False

SERVE_STATIC = os.environ.get('SERVE_STATIC', False)

LOG_TO = os.path.join(PROJECT_DIR, 'logs', 'logs.txt')
CONSOLE_LOG = os.environ.get('CONSOLE_LOG', True)

LOCAL_RUN = os.environ.get('LOCAL_RUN', False)
LOG_LEVEL = os.environ.get('LOG_LEVEL', 'INFO')

FRONT_BUILD_PATH = os.environ.get('FRONT_BUILD_PATH', os.path.join(PROJECT_DIR, 'frontend'))

FIREBIRD_USER = os.environ.get('FIREBIRD_USER', 'SYSDBA')
FIREBIRD_PASSWORD = os.environ.get('FIREBIRD_PASSWORD')

BUG_REPORT = False
WIN_DEBUG = False