import datetime

from repository.vocs import vocs_repository
from wsgi.apps.bugreports import services
from wsgi.apps.bugreports.models import BugreportModel
from wsgi.apps.users.models import UserModel
from wsgi.serializers import BaseSerializer


class BugreportsSerializer(BaseSerializer):
    model = BugreportModel(None)

    def __init__(self, request, user: UserModel.Data):
        BaseSerializer.__init__(self, request)
        self.user = user

    def save(self, data):
        obj = self.model.Data.from_dict(data)
        obj.creator_id = self.user.login  # todo: вынос в map
        obj.bug_id = services.generate_bug_id(self.request)
        obj.attrs.update({
            'environment': services.generate_environment(self.request),
            'bug_id': obj.bug_id,
            'creation_dt': str(datetime.datetime.now()),
            'creator_id': self.user.login,
        })
        return obj

    def update(self, record_id, data):
        instance = self.model.read(record_id)
        obj = instance.replace(data)
        obj.attrs = instance.attrs
        obj.attrs.update({k: v for k, v in data['attrs'].items() if v})
        return obj


class BugreportsAdminVocSerializer(BugreportsSerializer):

    def _response(self, resp):
        resp['voc_type'] = 'bugreports_admin'
        resp['label'] = resp['attrs']['creator_id']
        resp['value_id'] = resp['attrs']['creator_id']
        resp['edit'] = {
            'edit': True,
        }
        return resp


class BugreportsVocSerializer(BugreportsSerializer):
    fields = ['attrs', 'record_id', 'creator_id']

    def __init__(self, request, user: UserModel.Data):
        super().__init__(request, user)
        self.allowed_fields = vocs_repository.get_voc_fields('bugreports')

    def _response(self, resp):
        resp['voc_type'] = 'bugreports'
        resp['attrs'] = {k: v for k, v in resp['attrs'].items() if k in self.allowed_fields}
        resp['edit'] = {
            'edit': self.user.login == resp['creator_id']
        }
        resp['label'] = resp['creator_id']
        resp['value_id'] = resp['creator_id']
        return resp

    def list(self):
        users_logins_with_same_org_id = [user_raw['login'] for user_raw in
                                         UserModel(self.request).filter_raw({'org_id': self.user.org_id}, fields_=['login'])]
        return super().filter({'creator_id': users_logins_with_same_org_id})
