from wsgi.apps.bugreports.rules import bugreports_rules
from wsgi.apps.bugreports.serializers import BugreportsAdminVocSerializer, BugreportsVocSerializer, BugreportsSerializer
from wsgi.utils import parse_request, response, api_view


@parse_request(bugreports_rules)
def bugreports_view_manager(request, *args, **kwargs):
    action = kwargs['action']
    # GET
    if action == 'voc_list':
        return voc_list_bugreports_view(request)
    elif action == 'voc_read':
        return voc_read_bugreports_view(request, kwargs['record_id'])
    # POST
    elif action == 'voc_save':
        return voc_save_bugreports_view(request, kwargs['body'])
    elif action == 'voc_update':
        return voc_update_bugreports_view(request, kwargs['record_id'], kwargs['body'])
    elif action == 'delete':
        return delete_bugreports_view(request, kwargs['record_id'])


@api_view()
def voc_list_bugreports_view(request, *args, **kwargs):
    """Получить список багрепортов для отображения в таблице справочника"""
    if kwargs['user'].is_admin:
        return response(request, BugreportsAdminVocSerializer(request, kwargs['user']).list(), many=True)
    return response(request, BugreportsVocSerializer(request, kwargs['user']).list(), many=True)


@api_view()
def voc_read_bugreports_view(request, record_id, *args, **kwargs):
    """Получить багрепорт для отображения в таблице справочника"""
    if kwargs['user'].is_admin:
        return response(request, BugreportsAdminVocSerializer(request, kwargs['user']).read(record_id))
    return response(request, BugreportsVocSerializer(request, kwargs['user']).read(record_id))


@api_view()
def voc_save_bugreports_view(request, body, *args, **kwargs):
    """Сохранить багрепорт пользователя. Вызов из компонента справочников"""
    if kwargs['user'].is_admin:
        return response(request, BugreportsAdminVocSerializer(request, kwargs['user']).save(body))
    return response(request, BugreportsVocSerializer(request, kwargs['user']).save(body))


@api_view()
def voc_update_bugreports_view(request, record_id, body, *args, **kwargs):
    """Обновить багрепорт пользователя. Вызов из компонента справочников"""
    if kwargs['user'].is_admin:
        return response(request, BugreportsAdminVocSerializer(request, kwargs['user']).update(record_id, body))
    return response(request, BugreportsVocSerializer(request, kwargs['user']).update(record_id, body))


@api_view()
def delete_bugreports_view(request, record_id, *args, **kwargs):
    """Удалить багрепорт"""
    return response(request, BugreportsSerializer(request, kwargs['user']).delete(record_id))
