from exportmanager.export_manager import ExportFormManager, ExportTitlePages
from repository.forms import forms_repository
from services.exceptions import CoreBaseException
from settings import TEMP_DIR_PATH
from services.temp import TempFile
from wsgi.apps.export_manager.rules import export_manager_app_rules
from wsgi.apps.export_manager.services.services import build_download_name_for_form_export_file
from wsgi.apps.reporting_forms.models import FormModel
from wsgi.apps.user_logs.services import UserLogsLogger
from wsgi.utils import file_response, parse_request, api_view, response


@parse_request(export_manager_app_rules)
def export_manager_app_view_manager(request, *args, **kwargs):
    action = kwargs['action']
    # === GET ===
    if action == 'export_to_file':
        return export_to_file_view(request, kwargs['export_form_id'], kwargs['object_id'], kwargs['object_type'])
    elif action == 'export_with_title_pages':
        return export_with_title_pages_view(request, request.args.getlist('form_id'), kwargs.get('export_type', 'docx'))


@api_view()
def export_to_file_view(request, export_form_id, object_id, object_type, *args,
                        **kwargs):
    """Выгрузить в xlsx, docx, pdf отечтную форму"""
    form = FormModel(request).read(object_id)
    export_type = export_form_id.split('.')[-1]
    if export_type == 'pdf':
        export_form_id = export_form_id.split('.')[0] + '.docx'
    template_full_path = forms_repository.get_form_template_full_path(export_form_id)
    form_repository = forms_repository.get_form_repository(form.form_no)
    UserLogsLogger(request, kwargs['user']).log_export_form(export_form_id, export_type)
    if object_type == 'form':
        if export_type == 'docx':
            with TempFile(TEMP_DIR_PATH, '.docx') as res_file:
                ExportFormManager(request, form.as_dict(), form_repository).export_to_docx(template_full_path,
                                                                                  res_file.full_path)
                return file_response(res_file.full_path,
                                     download_name=build_download_name_for_form_export_file(res_file.full_path, form))
        elif export_type == 'xlsx':
            with TempFile(TEMP_DIR_PATH, '.xlsx') as res_file:
                ExportFormManager(request, form.as_dict(), form_repository).export_to_xlsx(template_full_path,
                                                                                  res_file.full_path)
                return file_response(res_file.full_path,
                                     download_name=build_download_name_for_form_export_file(res_file.full_path, form))
        elif export_type == 'pdf':
            with TempFile(TEMP_DIR_PATH, '.docx') as res_docx_file, TempFile(TEMP_DIR_PATH, '.pdf',
                                                                             res_docx_file.file_name_only) as res_pdf_file:
                ExportFormManager(request, form.as_dict(), form_repository).export_to_docx(template_full_path,
                                                                                  res_docx_file.full_path)
                ExportFormManager(request, form.as_dict(), form_repository).export_to_pdf(res_docx_file.full_path,
                                                                                 res_pdf_file.dir_path)
                return file_response(res_pdf_file.full_path,
                                     download_name=build_download_name_for_form_export_file(res_pdf_file.full_path, form))
    elif object_type == 'form_title':
        org = form.author_id_obj
        if export_type == 'docx':
            with TempFile(TEMP_DIR_PATH, '.docx') as res_docx_file:
                ExportTitlePages(org, template_full_path, request, form).export_to_docx(
                    res_docx_file.full_path)
                return file_response(res_docx_file.full_path,
                                     download_name=build_download_name_for_form_export_file(res_docx_file.full_path, form, form_title=True))
        elif export_type == 'pdf':
            with TempFile(TEMP_DIR_PATH, '.docx') as res_docx_file, TempFile(TEMP_DIR_PATH, '.pdf',
                                                                             res_docx_file.file_name_only) as res_pdf_file:
                ExportTitlePages(org, template_full_path, request, form).export_to_docx(
                    res_docx_file.full_path)
                ExportTitlePages(org, template_full_path, request, form).export_to_pdf(
                    res_docx_file.full_path,
                    res_pdf_file.dir_path)
                return file_response(res_pdf_file.full_path,
                                     download_name=build_download_name_for_form_export_file(res_pdf_file.full_path, form, form_title=True))


@api_view()
def export_with_title_pages_view(request, forms_ids, export_type='docx', *args, **kwargs):
    """Экспортировать отчетные формы в печатные с заполнением титульных листов"""
    if export_type != 'docx':
        raise CoreBaseException(message='export_type has not supported yet')
    forms = FormModel(request).filter({'report_form_id': forms_ids})

    resp_data = []
    for form in forms:
        for export_item in forms_repository.get_form_exports_types(form.form_no):
            resp_data.append({
                'title': export_item['title'],
                'info': '',
                'link': f'/api/export_manager?action=export_to_file&export_form_id={export_item["form_id"]}&'
                        f'object_id={form.report_form_id}&object_type={export_item["object_type"]}'
            })
        resp_data.append({})

    resp_repository = {
        'headers': ['title', 'info', 'link'],
        'model': [
            {
                'field': 'title',
                'caption': 'Форма',
            },
            {
                'field': 'info',
                'caption': 'Информация',
            },
            {
                'field': 'link',
                'caption': 'Ссылка на скачивание',
                'type': 'slot',
                'slot': 'link',
            }
        ],
        'types': {
            'slot': {
                'base_type': 'SLOT',
            }
        }
    }

    resp = {
        'repository': resp_repository,
        'data': resp_data,
    }

    return response(request, resp)
