import datetime

from services.services import to_exp
from wsgi.apps.historical_data.services.decrees.base_decree import BaseDecree
from wsgi.apps.reporting_forms.models import FormModel
from wsgi.apps.users.models import UserModel
from wsgi.apps.users.serializers import UsersSerializer


class Decree1_19(BaseDecree):
    def __init__(self, thread_runner, request, user, decrees, decree_name, db_file_path, db_file_type, api2_obj=None):
        super().__init__(thread_runner, request, user, decrees, decree_name, db_file_path, db_file_type,
                         api2_obj=api2_obj)
        self.decree_num = '19'
        self.form_rows_rules = {
            'ZRI': {
                'form_no': 'form_1_1',
                'convert': self.ZRI_convert,
            },
            'ROZ': {
                'form_no': 'form_1_5',
                'map': {
                    'Subsid': 'Subsidy',
                    'FCP': 'FCPnumber',
                },
                'convert': self.ROZ_convert,
            },
            'RAO': {
                'form_no': 'form_1_6',
                'map': {
                    'Subsid': 'Subsidy',
                    'FCP': 'FCPnumber',
                },
                'convert': self.RAO_convert,
            },
            'ORP': {
                'form_no': 'form_1_4',
                'map': {
                    'Arg': 'Agr',
                },
                'convert': self.ORP_convert,
            },
            'ORI': {
                'form_no': 'form_1_3',
                'convert': self.ORI_convert,
            },
            'IOU': {
                'form_no': 'form_1_2',
                'convert': self.IOU_convert,
            },
            'F1_24': {
                'form_no': 'form_2_1',
                'map': {
                    'g19': 'Power',
                    'g21': 'g11',
                    'g23': 'g18',
                },
                'convert': self.F1_24_convert,
            },
            'F2_24': {
                'form_no': 'form_2_2',
                'map': {
                    'g21': 'Subsidy',
                    'g22': 'FCPnumber',
                },
                'convert': self.F2_24_convert,
            },
            'F3_24': {
                'form_no': 'form_2_3',
                'map': {
                    'g13': 'ProjectVolume',
                },
                'convert': self.F3_24_convert,
            },
            'F4_24': {
                'form_no': 'form_2_4',
                'map': {
                    'g17': 'FCPnumber',
                },
                'convert': self.F4_24_convert,
            },
            'F5_24': {
                'form_no': 'form_2_5',
                'map': {
                    'g10': 'FCPnumber',
                },
                'convert': self.F5_24_convert,
            },
            'F6_24': {
                'form_no': 'form_2_6',
                'map': {
                    'num_s': 'g2',
                    'zone': 'g3',
                    'ist': 'g4',
                    'rust': 'g5',
                    'gl': 'g6',
                    'nuk': 'g7',
                    'radio': 'g8',
                },
                'convert': self.F6_24_convert,
            },
            'F7_24': {
                'form_no': 'form_2_7',
                'convert': self.F7_24_convert,
            },
            'F8_24': {
                'form_no': 'form_2_8',
                'map': {
                    'g5': 'PoolDistrict',
                },
                'convert': self.F8_24_convert,
            },
            'F9_24': {
                'form_no': 'form_2_9',
                'convert': self.F9_24_convert,
            },
            'F10_24': {
                'form_no': 'form_2_10',
                'map': {
                    'g11': 'FCPnumber',
                },
                'convert': self.F10_24_convert,
            },
            'F11_24': {
                'form_no': 'form_2_11',
                'convert': self.F11_24_convert,
            },
        }

        #

        # mdb_tables_names_24_to_19 = {
        #     'ZRI': 'F1'
        # }

        # Приводим имена годовой отчетности приказа 19 к виду 24 приказа
        # например, F2 -> F2_24
        table_names_to_pop = []
        extra_readers_data = {}
        for table_name, table_data in self.readers.items():
            if table_name.startswith('F') and table_name not in ('Forms', 'FORMP') and '_' not in table_name:
                table_names_to_pop.append(table_name)
                extra_readers_data[table_name + '_24'] = table_data

        for table_name in table_names_to_pop:
            self.readers.pop(table_name)

        for table_name, data in extra_readers_data.items():
            for d in data:
                self.readers[table_name].append(d)

        # ключ - MDB название таблицы
        self.forms_map_system_to_mdb = {}
        for mdb_table_name, rule in self.form_rows_rules.items():
            self.forms_map_system_to_mdb[mdb_table_name] = {}
            if 'map' in rule:
                self.forms_map_system_to_mdb[mdb_table_name]['map'] = {}
                for k, v in rule['map'].items():
                    self.forms_map_system_to_mdb[mdb_table_name]['map'][v] = k

        self.org_period_from = datetime.date(2017, 1, 1)
        self.org_period_to = datetime.date(2021, 12, 31)

        self._set_tables_cols_order()

        self.users = {}

        # for decree_name in ('decree_1_24', 'decree_1_19'):
        #     users = UserModel(self.request).filter({'login': decree_name})
        #     if users:
        #         self.users[decree_name] = users[0]
        #         # self.user = users[0]
        #     else:
        #         user_saved = UsersSerializer(request, user).save({
        #             'login': decree_name,
        #             'password': decree_name + 'ABCDEF',
        #             'fio': decrees[decree_name]['label'],
        #         })
        #         self.users[decree_name] = UserModel(self.request).read(user_saved['record_id'])
                # self.user = UserModel(self.request).read(user_saved['record_id'])

        self.after_init()

    def _is_19_decree(self, form):
        return UserModel(self.request).read(form.user_id).login == 'decree_1_19'

    def ZRI_convert(self, row, row_mdb, form):
        if row['Activn'] != '' and row['Sht'] != '':
            row['Activn'] = to_exp(float(row['Activn']) * int(row['Sht']))
        if row['Nss'] != '':
            row['Nss'] = str(int(float(row['Nss']) * 12))
        # if row['DocVid']:
        #     row['DocVid'] = row['DocVid'].zfill(2)
        #
        row['FormSobst'] = row['Pravoobl'] = '-'

    def ROZ_convert(self, row, row_mdb, form):
        if row['Activn'] != '' and row['Sht'] != '':
            row['Activn'] = to_exp(float(row['Activn']) * int(row['Sht']))
        # if row['DocVid']:
        #     row['DocVid'] = row['DocVid'].zfill(2)
        #
        row['factory'] = row['CodRAO'] = '-'

    def RAO_convert(self, row, row_mdb, form):
        if row['Kbm']:
            row['Kbm'] = "{:e}".format(float(row['Kbm']))
        if row['Tonne']:
            row['Tonne'] = "{:e}".format(float(row['Tonne']))
        # if row['DocVid']:
        #     row['DocVid'] = row['DocVid'].zfill(2)
        # тритий = трансурановые радионуклиды = '-'
        row['ActTR'] = row['ActUR'] = '-'
        #
        row['factory'] = '-'

        if self._is_19_decree(form):
            row['FCP'] = row['Subsid'] = '-'

        # raos = set()
        # for k in ('RAOCodMax', 'RAOCodCat', 'RAOCodDang'):
        #     for y in row_mdb[k].split('; '):
        #         for x in y.split(';'):
        #             if x:
        #                 x = x.strip()
        #                 # x.strip(';')
        #                 if x.endswith(';'):
        #                     x = x[:-1]
        #                 if x.endswith('m'):
        #                     x = x[:-1] + 'м'
        #
        #                 if x == 'смесь нуклидов урана':
        #                     x = 'сумма нуклидов урана'
        #
        #                 raos.add(x)
        #
        # row['RAOCodMax'] = '; '.join(raos)

        kod_grs = []

        # print(self.radios)

        # perenos = False
        #
        # for rad in raos:
        #     try:
        #         # rad_in_spr = next(filter(lambda x: x['\ufeffName_RN'] == rad, self.radios))
        #         # perenos = True
        #         # kod_rg = rad_in_spr['Kod_gruppy']
        #         # kod_grs.append(kod_rg)
        #         rad_in_spr = self.radios[rad.lower()]
        #         perenos = True
        #         kod_rg = rad_in_spr['Kod_gruppy']
        #         kod_grs.append(kod_rg)
        #     except KeyError as e:
        #         raise e
        #         pass
        #         # raise CoreBaseException(
        #         #     message=f'rad: -{rad}- not foudn in radio csv'
        #         # )
        #
        # if perenos:
        #     if len(kod_grs) == 1 and kod_grs[0] == 'Тритий':
        #         # print('----1')
        #         row['ActTR'] = row['ActBG']
        #         row['ActBG'] = row['ActUR'] = '-'
        #     elif all(i in kod_grs for i in ('Тритий', 'а', 'б', 'Трансурановый')):
        #         # print('----2')
        #         row['ActTR'] = row['ActUR'] = '???'
        #     elif all(i in kod_grs for i in ('Тритий', 'а', 'б')):
        #         # print('----3')
        #         row['ActTR'] = '???'
        #         row['ActUR'] = '-'
        #     elif all(i in kod_grs for i in ('Тритий', 'б')):
        #         # print('----4')
        #         row['ActTR'] = '???'
        #         row['ActUR'] = '-'
        #     elif all(i in kod_grs for i in ('Тритий', 'а')):
        #         # print('----5')
        #         row['ActTR'] = row['ActBG']
        #         row['ActBG'] = row['ActUR'] = '-'
        #     elif all(i in kod_grs for i in ('Тритий', 'Трансурановый')):
        #         # print('----6')
        #         row['ActUR'] = row['ActA']
        #         row['ActTR'] = row['ActBG']
        #         row['ActA'] = row['ActBG'] = '-'
        #     elif all(i in kod_grs for i in ('Трансурановый', 'а', 'б')):
        #         # print('----7')
        #         row['ActUR'] = '???'
        #         row['ActTR'] = '-'
        #     elif all(i in kod_grs for i in ('Трансурановый', 'б')):
        #         # print('----8')
        #         row['ActUR'] = row['ActA']
        #         row['ActTR'] = row['ActA'] = '-'
        #     elif all(i in kod_grs for i in ('Трансурановый', 'а')):
        #         # print('----9')
        #         row['ActUR'] = '???'
        #         row['ActTR'] = '-'
        #     elif all(i in kod_grs for i in ('Трансурановый',)):
        #         # print('----10')
        #         row['ActUR'] = row['ActA']
        #         row['ActA'] = row['ActTR'] = '-'
        #     elif len(kod_grs) == 1 and kod_grs[0] in ('а', 'б') or all(i in ('а', 'б') for i in kod_grs):
        #         # print('----11')
        #         row['ActTR'] = row['ActUR'] = '-'
        #     else:
        #         pass
        #         # print('-----12------', kod_grs)

    def ORP_convert(self, row, row_mdb, form):
        if row['Activn']:
            row['Activn'] = "{:e}".format(float(row['Activn']))
        if row['Kg']:
            row['Kg'] = "{:e}".format(float(row['Kg']))
        if row['Kbm']:
            row['Kbm'] = "{:e}".format(float(row['Kbm']))
        #
        row['FormSobst'] = row['Pravoobl'] = '-'
        # if row['DocVid']:
        #     row['DocVid'] = row['DocVid'].zfill(2)
        # if row['Typ']:
        #     row['Typ'] = row['Typ'].zfill(2)

    def ORI_convert(self, row, row_mdb, form):
        if row['Activn']:
            row['Activn'] = "{:e}".format(float(row['Activn']))
        #
        row['FormSobst'] = row['Pravoobl'] = '-'
        # if row['vid']:
        #     row['vid'] = row['vid'].zfill(2)

    def IOU_convert(self, row, row_mdb, form):
        if row['Nss'] != '':
            row['Nss'] = str(int(float(row['Nss']) * 12))
        if row['Kg'] != '':  # todo: !!!!!!!!!!!!!!!!!!!!!!!!!~
            row['Kg'] = "{:e}".format(float(row['Kg']))  # todo: float, не int
        #
        row['FormSobst'] = row['Pravoobl'] = '-'
        # if row['DocVid']:
        #     row['DocVid'] = row['DocVid'].zfill(2)

    def F1_24_convert(self, row, row_mdb, form):
        for k in ('g14', 'g15', 'g23', 'g17', 'g21', 'g10', 'g7', 'g8', 'g19'):
            if row[k]:
                row[k] = self._to_exp(row[k])
        for k in ('g20', 'g11', 'g22', 'g18'):
            row[k] = '-'

        if self._is_19_decree(form):
            row['g19'] = '-'

    def F2_24_convert(self, row, row_mdb, form):
        for k in ('g6', 'g7', 'g8', 'g9', 'g19', 'g14', 'g15', 'g20'):
            if row[k]:
                row[k] = self._to_exp(row[k])
        for k in ('g19', 'g20'):
            row[k] = '-'
        row['g16'] = self._build_set(row_mdb, ['g16', 'g17', 'g18'])

        # print("\t\t\t=>>>>>>>", self._is_19_decree(form))

        if self._is_19_decree(form):
            row['g21'] = row['g22'] = '-'

    def F3_24_convert(self, row, row_mdb, form):
        for k in ('g5', 'g6', 'g8'):
            if row[k]:
                row[k] = self._to_exp(row[k])

        if self._is_19_decree(form):
            row['g13'] = '-'

    def F4_24_convert(self, row, row_mdb, form):
        for k in ('g3', 'g5', 'g7', 'g9', 'g11', 'g13', 'g15'):
            if row[k]:
                row[k] = self._to_exp(row[k])

        if self._is_19_decree(form):
            row['g17'] = '-'

    def F5_24_convert(self, row, row_mdb, form):
        for k in ('g5', 'g6', 'g8', 'g9'):
            if row[k]:
                row[k] = self._to_exp(row[k])

        if self._is_19_decree(form):
            row['g10'] = '-'

    def F6_24_convert(self, row, row_mdb, form: FormModel.Data):
        for k in ('radio',):
            if row[k]:
                row[k] = self._to_exp(row[k])
        # обращение к основным данным формы
        try:
            form_data = next(filter(lambda f: f['ID'] == row_mdb['IDF'], self.get_reader('Forms')))
            form.form_main_data['quantity'] = form_data['SKV_F4']
        except StopIteration:
            # print('FORM IS', form)
            # print(f'F6_24_convert не найдена форма с id ')
            raise Exception('[[[[')

    def F7_24_convert(self, row, row_mdb, form):
        for k in ('g4', 'g5', 'g6'):
            if row[k]:
                row[k] = self._to_exp(row[k])

        if 'Allows' in self.readers:
            for row_allows in self.readers['Allows']:
                if row_allows['IDF'] == row_mdb['IDF'] and row_allows['TYPE_A'] == '5':
                    form.form_main_data['g7'] = row_allows['SERIAL']
                    form.form_main_data['g8'] = self._to_date(row_allows['date_a'])
                    form.form_main_data['g9'] = row_allows['WHO_TAKE']
                    form.form_main_data['per_from'] = row_allows['date_from']
                    form.form_main_data['per_to'] = row_allows['date_to']

    def F8_24_convert(self, row, row_mdb, form):
        for k in ('g6', 'g7'):
            if row[k]:
                row[k] = self._to_exp(row[k])

        if 'Allows' in self.readers:
            for row_allows in self.get_reader('Allows'):
                if row_allows['IDF'] == row_mdb['IDF']:
                    if row_allows['TYPE_A'] == '6':
                        form.form_main_data['g11'] = row_allows['SERIAL']
                        form.form_main_data['g12'] = self._to_date(row_allows['date_a'])
                        form.form_main_data['g13'] = row_allows['WHO_TAKE']
                        form.form_main_data['per1'] = self._to_date(row_allows['date_from'])
                        form.form_main_data['to1'] = self._to_date(row_allows['date_to'])
                    elif row_allows['TYPE_A'] == '7':
                        form.form_main_data['g21'] = row_allows['SERIAL']
                        form.form_main_data['g22'] = self._to_date(row_allows['date_a'])
                        form.form_main_data['g23'] = row_allows['WHO_TAKE']
                        form.form_main_data['per2'] = self._to_date(row_allows['date_from'])
                        form.form_main_data['to2'] = self._to_date(row_allows['date_to'])
                    elif row_allows['TYPE_A'] == '8':
                        form.form_main_data['g31'] = row_allows['SERIAL']
                        form.form_main_data['g32'] = self._to_date(row_allows['date_a'])
                        form.form_main_data['g33'] = row_allows['WHO_TAKE']
                        form.form_main_data['per3'] = self._to_date(row_allows['date_from'])
                        form.form_main_data['to3'] = self._to_date(row_allows['date_to'])

    def F9_24_convert(self, row, row_mdb, form):
        for k in ('g4', 'g5'):
            if row[k]:
                row[k] = self._to_exp(row[k])

    def F10_24_convert(self, row, row_mdb, form):
        for k in ('g6', 'g7', 'g8', 'g9', 'g10'):
            if row[k]:
                row[k] = self._to_exp(row[k])

        if self._is_19_decree(form):
            row['g11'] = '-'

    def F11_24_convert(self, row, row_mdb, form):
        for k in ('square', 'g6', 'g7', 'g8'):
            if row[k]:
                row[k] = self._to_exp(row[k])

    def _import_main_form_data_row(self, row, convert_func, user=None, decree_year=None):
        if row.get('USE124ORDER', False) in ('0', False):
            super()._import_main_form_data_row(row, convert_func, user=None, decree_year='2014')

    def import_historical_data(self):
        super()._import_data()
        # примечание к годовому отчету - комментарий к форме в системе
        if 'PRIMYear' in self.csv_table_files_temp:
            self._import_form_comment(self.readers['PRIMYear'], 'PRIMYear')
        self._save_data()
        self._log_after()
