import datetime

from services.services import to_exp
from wsgi.apps.historical_data.services.decrees.base_decree import BaseDecree
from wsgi.apps.historical_data.services.decrees.utils import get_reader_from_mdb_gen
from wsgi.apps.reporting_forms.models import FormModel
from wsgi.apps.users.models import UserModel
from wsgi.apps.users.serializers import UsersSerializer


class Decree_24_19(BaseDecree):
    def __init__(self, thread_runner, request, user, decrees, decree_name, db_file_path, db_file_type, api2_obj=None, done=None):
        super().__init__(thread_runner, request, user, decrees, decree_name, db_file_path, db_file_type,
                         api2_obj=api2_obj, done=done)
        self.decree_num = '19_24'
        self.form_rows_rules = {
            'ZRI': {
                'form_no': 'form_1_1',
                'convert': self.ZRI_convert,
            },
            'ROZ': {
                'form_no': 'form_1_5',
                'map': {
                    'Subsid': 'Subsidy',
                    'FCP': 'FCPnumber',
                },
                'convert': self.ROZ_convert,
            },
            'RAO': {
                'form_no': 'form_1_6',
                'map': {
                    'Subsid': 'Subsidy',
                    'FCP': 'FCPnumber',
                },
                'convert': self.RAO_convert,
            },
            'ORP': {
                'form_no': 'form_1_4',
                'map': {
                    'Arg': 'Agr',
                },
                'convert': self.ORP_convert,
            },
            'ORI': {
                'form_no': 'form_1_3',
                'convert': self.ORI_convert,
            },
            'IOU': {
                'form_no': 'form_1_2',
                'convert': self.IOU_convert,
            },
            'F1_24': {
                'form_no': 'form_2_1',
                'map': {
                    'g19': 'Power',
                    'g21': 'g11',
                    'g23': 'g18',
                },
                'convert': self.F1_24_convert,
            },
            'F2_24': {
                'form_no': 'form_2_2',
                'map': {
                    'g21': 'Subsidy',
                    'g22': 'FCPnumber',
                },
                'convert': self.F2_24_convert,
            },
            'F3_24': {
                'form_no': 'form_2_3',
                'map': {
                    'g13': 'ProjectVolume',
                },
                'convert': self.F3_24_convert,
            },
            'F4_24': {
                'form_no': 'form_2_4',
                'map': {
                    'g17': 'FCPnumber',
                },
                'convert': self.F4_24_convert,
            },
            'F5_24': {
                'form_no': 'form_2_5',
                'map': {
                    'g10': 'FCPnumber',
                },
                'convert': self.F5_24_convert,
            },
            'F6_24': {
                'form_no': 'form_2_6',
                'map': {
                    'num_s': 'g2',
                    'zone': 'g3',
                    'ist': 'g4',
                    'rust': 'g5',
                    'gl': 'g6',
                    'nuk': 'g7',
                    'radio': 'g8',
                },
                'convert': self.F6_24_convert,
            },
            'F7_24': {
                'form_no': 'form_2_7',
                'convert': self.F7_24_convert,
            },
            'F8_24': {
                'form_no': 'form_2_8',
                'map': {
                    'g5': 'PoolDistrict',
                },
                'convert': self.F8_24_convert,
            },
            'F9_24': {
                'form_no': 'form_2_9',
                'convert': self.F9_24_convert,
            },
            'F10_24': {
                'form_no': 'form_2_10',
                'map': {
                    'g11': 'FCPnumber',
                },
                'convert': self.F10_24_convert,
            },
            'F11_24': {
                'form_no': 'form_2_11',
                'convert': self.F11_24_convert,
            },
        }

        # ключ - MDB название таблицы
        self.forms_map_system_to_mdb = {}
        for mdb_table_name, rule in self.form_rows_rules.items():
            self.forms_map_system_to_mdb[mdb_table_name] = {}
            if 'map' in rule:
                self.forms_map_system_to_mdb[mdb_table_name]['map'] = {}
                for k, v in rule['map'].items():
                    self.forms_map_system_to_mdb[mdb_table_name]['map'][v] = k

        self.org_period_from = datetime.date(2017, 1, 1)
        self.org_period_to = datetime.date(2021, 12, 31)

        self.users = {}
        self.after_init()

        self.F6_24_convert_after_proccess_forms = {}
        self.F6_24_forms = {}
        self.F7_24_forms = {}  # хранилище форм F7_24 для пост обработки
        self.F8_24_forms = {}

    def ZRI_convert(self, row, row_mdb, form):
        if row['Activn'] != '' and row['Sht'] != '':
            row['Activn'] = to_exp(float(row['Activn']) * int(row['Sht']))
        if row['Nss'] != '':
            row['Nss'] = str(int(float(row['Nss']) * 12))
        row['FormSobst'] = row['Pravoobl'] = '-'

    def ROZ_convert(self, row, row_mdb, form):
        if row['Activn'] != '' and row['Sht'] != '':
            row['Activn'] = to_exp(float(row['Activn']) * int(row['Sht']))
        row['factory'] = row['CodRAO'] = '-'

    def RAO_convert(self, row, row_mdb, form):
        if row['Kbm']:
            row['Kbm'] = "{:e}".format(float(row['Kbm']))
        if row['Tonne']:
            row['Tonne'] = "{:e}".format(float(row['Tonne']))
        # тритий = трансурановые радионуклиды = '-'
        row['ActTR'] = row['ActUR'] = '-'
        #
        row['factory'] = '-'

    def ORP_convert(self, row, row_mdb, form):
        if row['Activn']:
            row['Activn'] = "{:e}".format(float(row['Activn']))
        if row['Kg']:
            row['Kg'] = "{:e}".format(float(row['Kg']))
        if row['Kbm']:
            row['Kbm'] = "{:e}".format(float(row['Kbm']))
        #
        row['FormSobst'] = row['Pravoobl'] = '-'

    def ORI_convert(self, row, row_mdb, form):
        if row['Activn']:
            row['Activn'] = "{:e}".format(float(row['Activn']))
        #
        row['FormSobst'] = row['Pravoobl'] = '-'

    def IOU_convert(self, row, row_mdb, form):
        if row['Nss'] != '':
            row['Nss'] = str(int(float(row['Nss']) * 12))
        if row['Kg'] != '':  # todo: !!!!!!!!!!!!!!!!!!!!!!!!!~
            row['Kg'] = "{:e}".format(float(row['Kg']))  # todo: float, не int
        #
        row['FormSobst'] = row['Pravoobl'] = '-'

    def F1_24_convert(self, row, row_mdb, form):
        for k in ('g14', 'g15', 'g23', 'g17', 'g21', 'g10', 'g7', 'g8', 'g19'):
            if row[k]:
                row[k] = self._to_exp(row[k])
        for k in ('g20', 'g11', 'g22', 'g18'):
            row[k] = '-'

    def F2_24_convert(self, row, row_mdb, form):
        for k in ('g6', 'g7', 'g8', 'g9', 'g19', 'g14', 'g15', 'g20'):
            if row[k]:
                row[k] = self._to_exp(row[k])
        for k in ('g19', 'g20'):
            row[k] = '-'
        row['g16'] = self._build_set(row_mdb, ['g16', 'g17', 'g18'])

    def F3_24_convert(self, row, row_mdb, form):
        for k in ('g5', 'g6', 'g8'):
            if row[k]:
                row[k] = self._to_exp(row[k])

    def F4_24_convert(self, row, row_mdb, form):
        for k in ('g3', 'g5', 'g7', 'g9', 'g11', 'g13', 'g15'):
            if row[k]:
                row[k] = self._to_exp(row[k])

    def F5_24_convert(self, row, row_mdb, form):
        for k in ('g5', 'g6', 'g8', 'g9'):
            if row[k]:
                row[k] = self._to_exp(row[k])

    def F6_24_convert(self, row, row_mdb, form):
        for k in ('radio',):
            if row[k]:
                row[k] = self._to_exp(row[k])

        if int(row_mdb['IDF']) not in self.F6_24_forms:
            self.F6_24_forms[int(row_mdb['IDF'])] = form

    def F6_24_convert_after(self):
        if 'Forms' in self.table_names:
            for row in get_reader_from_mdb_gen(self.db_file_path, 'Forms', self.mdb_tables_headers):
                if int(row['ID']) in self.F6_24_forms:
                    form = self.F6_24_forms[int(row['ID'])]
                    form['form_main_data']['quantity'] = row['SKV_F4']

    def F7_24_convert(self, row, row_mdb, form):
        for k in ('g4', 'g5', 'g6'):
            if row[k]:
                row[k] = self._to_exp(row[k])

        if int(row_mdb['IDF']) not in self.F7_24_forms:
            self.F7_24_forms[int(row_mdb['IDF'])] = form

    def F7_24_convert_after(self):
        if 'Allows' in self.table_names:
            for row_allows in get_reader_from_mdb_gen(self.db_file_path, 'Allows', self.mdb_tables_headers):
                if int(row_allows['IDF']) in self.F7_24_forms:
                    form = self.F7_24_forms[int(row_allows['IDF'])]
                    form['form_main_data']['g7'] = row_allows['SERIAL']
                    form['form_main_data']['g8'] = self._to_date(row_allows['date_a'])
                    form['form_main_data']['g9'] = row_allows['WHO_TAKE']
                    form['form_main_data']['per_from'] = self._to_date(row_allows['date_from'])
                    form['form_main_data']['per_to'] = self._to_date(row_allows['date_to'])

    def F8_24_convert(self, row, row_mdb, form):
        for k in ('g6', 'g7'):
            if row[k]:
                row[k] = self._to_exp(row[k])

        if int(row_mdb['IDF']) not in self.F7_24_forms:
            self.F8_24_forms[int(row_mdb['IDF'])] = form

    def F8_24_convert_after(self):
        if 'Allows' in self.table_names:
            for row_allows in get_reader_from_mdb_gen(self.db_file_path, 'Allows', self.mdb_tables_headers):
                if int(row_allows['IDF']) in self.F8_24_forms:
                    form = self.F8_24_forms[int(row_allows['IDF'])]
                    if row_allows['TYPE_A'] == '6':
                        form['form_main_data']['g11'] = row_allows['SERIAL']
                        form['form_main_data']['g12'] = self._to_date(row_allows['date_a'])
                        form['form_main_data']['g13'] = row_allows['WHO_TAKE']
                        form['form_main_data']['per1'] = self._to_date(row_allows['date_from'])
                        form['form_main_data']['to1'] = self._to_date(row_allows['date_to'])
                    elif row_allows['TYPE_A'] == '7':
                        form['form_main_data']['g21'] = row_allows['SERIAL']
                        form['form_main_data']['g22'] = self._to_date(row_allows['date_a'])
                        form['form_main_data']['g23'] = row_allows['WHO_TAKE']
                        form['form_main_data']['per2'] = self._to_date(row_allows['date_from'])
                        form['form_main_data']['to2'] = self._to_date(row_allows['date_to'])
                    elif row_allows['TYPE_A'] == '8':
                        form['form_main_data']['g31'] = row_allows['SERIAL']
                        form['form_main_data']['g32'] = self._to_date(row_allows['date_a'])
                        form['form_main_data']['g33'] = row_allows['WHO_TAKE']
                        form['form_main_data']['per3'] = self._to_date(row_allows['date_from'])
                        form['form_main_data']['to3'] = self._to_date(row_allows['date_to'])

    def F9_24_convert(self, row, row_mdb, form):
        for k in ('g4', 'g5'):
            if row[k]:
                row[k] = self._to_exp(row[k])

    def F10_24_convert(self, row, row_mdb, form):
        for k in ('g6', 'g7', 'g8', 'g9', 'g10'):
            if row[k]:
                row[k] = self._to_exp(row[k])

    def F11_24_convert(self, row, row_mdb, form):
        for k in ('square', 'g6', 'g7', 'g8'):
            if row[k]:
                row[k] = self._to_exp(row[k])

    def _import_main_form_data_row(self, row, convert_func, user=None, decree_year=None):
        if 'USE124ORDER' not in row:
            super()._import_main_form_data_row(row, convert_func, user=None, decree_year='2017')
        else:
            if isinstance(row["USE124ORDER"], str):
                if row['USE124ORDER'] == '1':
                    super()._import_main_form_data_row(row, convert_func, user=None, decree_year='2017')
                else:
                    super()._import_main_form_data_row(row, convert_func, user=None, decree_year='2014')
            else:
                if row["USE124ORDER"]:
                    super()._import_main_form_data_row(row, convert_func, user=None, decree_year='2017')
                else:
                    super()._import_main_form_data_row(row, convert_func, user=None, decree_year='2014')

    def import_historical_data(self):
        super()._import_data()
        # примечание к годовому отчету - комментарий к форме в системе
        if 'PRIMYear' in self.table_names:
            self._import_form_comment(None, 'PRIMYear')
        self.F6_24_convert_after()
        self.F7_24_convert_after()
        self.F8_24_convert_after()
        self._save_data()
        self._log_after()
