import datetime

from repository.forms import forms_repository
from wsgi.apps.historical_data.services.decrees.base_decree import BaseDecree
from wsgi.apps.reporting_forms.models import FormModel
from wsgi.apps.users.models import UserModel
from wsgi.apps.voc.models import OrganizationVocModel, VocValueModel
from wsgi.apps.voc.services import build_voc_label


class Decree449(BaseDecree):

    decree_year = '2001'

    def __init__(self, thread_runner, request, user, decrees, decree_name, db_file_path, db_file_type, api2_obj=None):
        super().__init__(thread_runner, request, user, decrees, decree_name, db_file_path, db_file_type, api2_obj=api2_obj)
        self.decree_num = '449'
        self.form_rows_rules = {
            'ОП_ЗРИ': {
                'form_no': 'form_1_1',
                'convert': self.OP_ZRI_convert,
                'map': {
                    'Typ': 'ЗРИ_2',
                    'Numb': 'ЗРИ_3',
                    'PaspN': 'ЗРИ_4',
                    'IzgotDate': 'ЗРИ_5',
                    'Nss': 'ЗРИ_6',
                    'Activn': 'ЗРИ_7',
                    'OpDate': 'ЗРИ_8',
                    'OpCod': 'ЗРИ_9',
                    'DocN': 'ЗРИ_10',
                    'DocDate': 'ЗРИ_11',
                    'IzgotOPKO': 'ЗРИ_12',
                    'OkpoPIP': 'ЗРИ_13',
                    'PrName': 'ЗРИ_14',
                    'UktPrTyp': 'ЗРИ_15',
                    'UktPrN': 'ЗРИ_16',
                }
            },
            'ОП_ОРИ': {
                'form_no': 'form_1_4',
                'convert': self.OP_ORI_convert,
                'map': {
                    'IstName': 'ОРИ_2',
                    'PaspN': 'ОРИ_3',
                    'Nuclid': 'ОРИ_4',
                    'ActDate': 'ОРИ_5',
                    'Activn': 'ОРИ_6',
                    'Kbm': 'ОРИ_7',
                    'Kg': 'ОРИ_8',
                    'OpDate': 'ОРИ_9',
                    'OpCod': 'ОРИ_10',
                    'DocN': 'ОРИ_11',
                    'DocDate': 'ОРИ_12',
                    'OkpoPIP': 'ОРИ_13',
                    'PrName': 'ОРИ_14',
                }
            },
            'ОП_РАО': {
                'form_no': 'form_1_6',
                'convert': self.OP_RAO_convert,
                'map': {
                    'Kbm': 'РАО_6',
                    'Tonne': 'РАО_7',
                    'RAOCodMax': 'РАО_8',
                    'ActA': 'РАО_9',
                    'ActBG': 'РАО_10',
                    'ActDate': 'РАО_11',
                    'OpCod': 'РАО_12',
                    'OpDate': 'РАО_13',
                    'DocN': 'РАО_14',
                    'DocDate': 'РАО_15',
                    'OkpoPIP': 'РАО_16',
                    'PH_Cod': 'РАО_17',
                    'PH_Name': 'РАО_18',
                    'UpCod': 'РАО_19',
                    'PrName': 'РАО_20',
                }
            }
        }

        self.org_period_from = datetime.date(2001, 1, 1)
        self.org_period_to = datetime.date(2009, 12, 31)

        # self.form_ids_to_orgs_okpo = {}
        self.okpo_codes_to_orgs = {}
        self.forms_names_to_db_form_id_to_form = {}
        self.after_init()

    def OP_ZRI_convert(self, row, row_mdb, form):
        if row['Nss']:
            row['Nss'] = str(int(row['Nss']) * 12)
        if row['OpCod']:
            # (row['OpCod'])
            mdb_op_code_to_system = {
                1: '38',
                2: '85',
                3: '11',
                4: '27',
                5: '37',
                6: '28',
                7: '81',
                8: '61',
                9: '62',
                10: '29',
                11: '66',
                12: '53',
                19: '41',
                99: '99',
            }
            row['OpCod'] = mdb_op_code_to_system[int(row['OpCod'])]

        self.forms_names_to_db_form_id_to_form.setdefault('form_1_1', {})[row['OpCod']] = form

        if row['Activn']:
            row['Activn'] = self._to_exp(row['Activn'])

    def OP_ORI_convert(self, row, row_mdb, form):
        if row['OpCod']:
            mdb_op_code_to_system = {
                1: '38',
                2: '85',
                3: '11',
                6: '28',
                7: '81',
                8: '27',
                9: '37',
                10: '29',
                11: '46',
                12: '47',
                19: '41',
                99: '99',
            }
            row['OpCod'] = mdb_op_code_to_system[int(row['OpCod'])]

        self.forms_names_to_db_form_id_to_form.setdefault('form_1_4', {})[row['OpCod']] = form
        # 8 10 11
        if row['Activn']:
            row['Activn'] = self._to_exp(row['Activn'])
        if row['Kbm']:
            row['Kbm'] = self._to_exp(row['Kbm'])
        if row['Kg']:
            row['Kg'] = self._to_exp(row['Kg'])

    def OP_RAO_convert(self, row, row_mdb, form):
        # for k in ('ActTR', 'ActUR'):
        #     row[k] = '-'
        if row['OpCod']:
            mdb_op_code_to_system = {
                1: '38',
                3: '11',
                4: '28',
                7: '26',
                10: '44',
                99: '99',
            }
            row['OpCod'] = mdb_op_code_to_system[int(row['OpCod'])]
        if row['UpCod']:
            mdb_up_code_to_system = {
                1: '42',
                2: '41',
                3: '43',
                4: '22',
                5: '31',
                6: '24',
                99: '99',
            }
            row['UpCod'] = mdb_up_code_to_system.get(int(row['UpCod']), str(row['UpCod']))

        self.forms_names_to_db_form_id_to_form.setdefault('form_1_6', {})[row['OpCod']] = form

        raos = set()
        for k in ('РАО_8', ):
            for y in row_mdb[k].split('; '):
                for x in y.split(';'):
                    if x:
                        x = x.strip()
                        # x.strip(';')
                        if x.endswith(';'):
                            x = x[:-1]
                        if x.endswith('m'):
                            x = x[:-1] + 'м'

                        if x == 'смесь нуклидов урана':
                            x = 'сумма нуклидов урана'

                        raos.add(x)

        row['RAOCodMax'] = '; '.join(raos)

        kod_grs = []

        # print(self.radios)

        perenos = False

        for rad in raos:
            try:
                # rad_in_spr = next(filter(lambda x: x['\ufeffName_RN'] == rad, self.radios))
                # perenos = True
                # kod_rg = rad_in_spr['Kod_gruppy']
                # kod_grs.append(kod_rg)

                rad_in_spr = self.radios[rad]
                perenos = True
                kod_rg = rad_in_spr['Kod_gruppy']
                kod_grs.append(kod_rg)
            except (StopIteration, KeyError):
                pass
                # raise CoreBaseException(
                #     message=f'rad: -{rad}- not foudn in radio csv'
                # )

        if perenos:
            if len(kod_grs) == 1 and kod_grs[0] == 'Тритий':
                print('----1')
                row['ActTR'] = row['ActBG']
                row['ActBG'] = row['ActUR'] = '-'
            elif all(i in kod_grs for i in ('Тритий', 'а', 'б', 'Трансурановый')):
                print('----2')
                row['ActTR'] = row['ActUR'] = '???'
            elif all(i in kod_grs for i in ('Тритий', 'а', 'б')):
                print('----3')
                row['ActTR'] = '???'
                row['ActUR'] = '-'
            elif all(i in kod_grs for i in ('Тритий', 'б')):
                print('----4')
                row['ActTR'] = '???'
                row['ActUR'] = '-'
            elif all(i in kod_grs for i in ('Тритий', 'а')):
                print('----5')
                row['ActTR'] = row['ActBG']
                row['ActBG'] = row['ActUR'] = '-'
            elif all(i in kod_grs for i in ('Тритий', 'Трансурановый')):
                print('----6')
                row['ActUR'] = row['ActA']
                row['ActTR'] = row['ActBG']
                row['ActA'] = row['ActBG'] = '-'
            elif all(i in kod_grs for i in ('Трансурановый', 'а', 'б')):
                print('----7')
                row['ActUR'] = '???'
                row['ActTR'] = '-'
            elif all(i in kod_grs for i in ('Трансурановый', 'б')):
                # print('----8')
                row['ActUR'] = row['ActA']
                row['ActTR'] = row['ActA'] = '-'
            elif all(i in kod_grs for i in ('Трансурановый', 'а')):
                # print('----9')
                row['ActUR'] = '???'
                row['ActTR'] = '-'
            elif all(i in kod_grs for i in ('Трансурановый',)):
                # print('----10')
                row['ActUR'] = row['ActA']
                row['ActA'] = row['ActTR'] = '-'
            elif len(kod_grs) == 1 and kod_grs[0] in ('а', 'б') or all(i in ('а', 'б') for i in kod_grs):
                # print('----11')
                row['ActTR'] = row['ActUR'] = '-'
            else:
                print('-----12------', kod_grs)

    def OP_PERIOD_convert(self, form: FormModel.Data, row_mdb):
        pass

    def _save_data(self):
        # print(self.okpo_codes_to_orgs)
        # exit()
        # for okpo_code, org in self.okpo_codes_to_orgs.items():
        #     # if org.record_id is None:  # если организации еще нет в базе
        #     # org = OrganizationVocModel(self.request).save(org, commit=True)
        #     # формирование value_id
        #     # d = org.as_dict()
        #     # org.value_id = str(d.get(self.org_repository['value_id']))
        #     # org = OrganizationVocModel(self.request).update(org.record_id, org, commit=True)
        #     self.okpo_codes_to_orgs[okpo_code] = org
        #
        #     # self.orgs
        #
        # # сохранение отчетных форм
        # for form_id, form in self.result_forms.items():
        #     form.author_id = self.okpo_codes_to_orgs[str(form.author_id)].record_id
        #     # form.author_id = OrganizationVocModel(self.conn).filter({'OKPO': [str(form.author_id)]})[0]
        #     # self.result_forms[form_id] = FormModel(self.request).save(form)

        self.orgs = self.okpo_codes_to_orgs

        super()._save_data()

    def _import_data(self):
        # импорт основных таблиц форм
        self._set_progress(0, 'Импорт основных таблиц форм')
        if 'ОП_ПЕРИОД' in self.csv_table_files_temp:
            self._import_main_form_data(self.readers['ОП_ПЕРИОД'], self.OP_PERIOD_convert)
        # импорт таблицы данных форм (строки)
        table_count = 0
        for table_name, rules in self.form_rows_rules.items():
            self._set_progress(1, 'Импорт основных таблиц форм', table_count, len(self.form_rows_rules))
            if table_name in self.table_names:
                form_repository = forms_repository.get_form_repository(rules['form_no'])
                self._import_forms_rows(
                    reader=self.get_reader(table_name),
                    form_repository=forms_repository.get_form_repository(rules['form_no']),
                    form_no=rules['form_no'],
                    map_=rules.get('map'),
                    convert_func=rules.get('convert'),
                    id_key='ОП_Код',
                    table_name=table_name,
                    model_fields=[model_item['field'] for model_item in
                                  form_repository['tables']['main_table']['model']],
                    field_to_model_item={model_item['field']: model_item for model_item in
                                         form_repository['tables']['main_table']['model']},
                    tables_keys=list(form_repository['tables'].keys()),
                )
                table_count += 1
        # импорт примечаний к ячейкам форм
        self._set_progress(2, 'Импорт примечаний')
        note_ = {
            'OП_ЗРИк': 'ОП_ЗРИ',
            'ОП_ОРИк': 'ОП_ОРИ',
            'ОП_РАОк': 'ОП_РАО',
        }
        for remarks_table, table in note_.items():
            if remarks_table in self.readers:
                self._import_cell_remarks2(self.get_reader(remarks_table), self.form_rows_rules[table]['form_no'])

        if 'NoteCell' in self.csv_table_files_temp:
            self._import_cell_remarks(self.readers['NoteCell'], 'NoteCell')
        # импорт справочника организаций
        self._set_progress(3, 'Импорт справочника организаций')
        if 'СпрОрг' in self.csv_table_files_temp:
            self._import_orgs(self.get_reader('СпрОрг'), 'СпрОрг')

    def _import_main_form_data(self, reader, table_name, convert_func=None):
        for row in reader:
            self.result_forms[int(row['ОП_Код'])] = form = FormModel.Data.empty()
            form.form_period_start = datetime.datetime.strptime(row['ОП_BEG'], '%m/%d/%y %H:%M:%S')
            form.form_period_end = datetime.datetime.strptime(row['ОП_END'], '%m/%d/%y %H:%M:%S')
            form.adjustment_no = 0
            form.form_main_data = {
                'full_name': row['ОП_ISP'],
                'phone': row['ОП_TLISP'],
            }
            form.user_id = self.user.record_id
            form.author_id = int(row['F01_04'])

            # сохраняем в базе id формы из исходной базы
            form.import_src_id = row['ОП_Код']
            # org_id=XXX/YYY, где XXX - org_id из БД, YYY - номер импортируемого приказа (449, 600, 24, 19, 13).
            # Пример: import_author := org_id=001/24 или org_id=555/600
            form.import_author = self._build_import_author_id(row['F01_04'])  # f'org_id={self.decree_num}/{row["ID"]}'

            if convert_func is not None:
                convert_func(form, row)

    def _import_orgs(self, reader, table_name):
        for row in reader:
            # orgs = OrganizationVocModel(self.conn).filter({'OKPO': row['F01_04']})
            # if orgs:  # организация с таким кодом уже существует
            #     # self.orgs[int(row['ID'])] = orgs[0]
            #     self.okpo_codes_to_orgs[row['F01_04']] = orgs[0]
            #     continue

            okpo = row['F01_04']

            F01_04 = row['F01_04']

            db_to_system_attrs = {
                'F01_01': 'NAME_FULL',
                'F01_02': 'NAME_SHORT',
                'F01_04': 'OKPO',
                'F01_06': 'OKVED',
                'F01_07': 'OKATO',
                'F01_08': 'OKOGU',
                'F01_09': 'OKOPF',
                'F01_10': 'OKFS',
                'F01_11': 'KOD_IAE',
                'F01_12_1': 'INDEX_U',
                'F01_12_2': 'KOD_SRF',
                'F01_12_3': 'GOROD_U',
                'F01_12_4': 'ULITSA_U',
                'F01_12_5': 'DOM_U',
                'F01_12_6': 'KORPUS_U',
                'F01_13': 'TLFON',
                'F01_14': 'FAX',
                'F01_15': 'EMAIL',
            }

            for db_k, sys_k in db_to_system_attrs.items():
                row[sys_k] = row[db_k]
                row.pop(db_k)

            row['RUKOVODITEL'] = ', '.join(row[k] for k in ('F01_16', 'F01_RUK'))

            org = OrganizationVocModel.Data.from_dict(row)
            org.value_status = VocValueModel.Data.ValueStatusesTypes.ACTIVE
            org.voc_type = 'organizations'
            org.label = build_voc_label(org.as_dict(), self.org_repository, attrs_wrap=False)

            org.import_src_id = F01_04
            org.import_author = self._build_import_author_id(F01_04)

            org.valid_from = self.org_period_from
            org.valid_to = self.org_period_to

            self.okpo_codes_to_orgs[okpo] = org

    def _import_cell_remarks2(self, reader, form_no):
        for row in reader:
            try:
                form: FormModel.Data = self.forms_names_to_db_form_id_to_form[form_no]['Ключ']
            except KeyError:
                print(
                    'WARNING: _import_cell_remarks2 строка ссылается по несуществующему ключ в основной таблице формы')
                return

            mdb_table_name = None
            for k, v in self.form_rows_rules.items():
                if v['form_no'] == form.form_no:
                    mdb_table_name = k
                    break

            cols = self.tables_cols_order[mdb_table_name]
            remark = {
                'value': {
                    'text': row['Комм']
                },
                'pointer': {
                    'row': int(row['Nrow']),
                    # 'col_name': cols[int(row['Ncol']) - 1],  # todo:  как определить
                }
            }
            form.form_remarks['main_table'].append(remark)
