import datetime

from wsgi.apps.historical_data.services.decrees.base_decree import BaseDecree
from wsgi.apps.users.models import UserModel


class Decree600(BaseDecree):

    decree_year = '2010'

    def __init__(self, thread_runner, request, user, decrees, decree_name, db_file_path, db_file_type, done=None, api2_obj=None):
        super().__init__(thread_runner, request, user, decrees, decree_name, db_file_path, db_file_type, done=done, api2_obj=api2_obj)
        self.decree_num = '600'
        self.form_rows_rules = {
            'RAO': {
                'form_no': 'form_1_6',
                'convert': self.RAO_convert,
                'map': {
                    'RAOCodMax': 'Nuclid',
                    'Tonne': 'Kg'
                }
            },
            'IOU': {
                'form_no': 'form_1_2',
                'convert': self.IOU_convert,
            },
            'ORP': {
                'form_no': 'form_1_4',
                'map': {
                    'Arg': 'Agr',
                },
                'convert': self.ORP_convert,
            },
            'ROZ': {
                'form_no': 'form_1_5',
                'convert': self.ROZ_convert,
            },
            'ZRI': {
                'form_no': 'form_1_1',
                'convert': self.ZRI_convert,
            },
            'ORI': {
                'form_no': 'form_1_3',
                'convert': self.ORI_convert,
            },
        }

        self.org_period_from = datetime.date(2010, 1, 1)
        self.org_period_to = datetime.date(2013, 12, 31)

        # self._set_tables_cols_order()
        self.after_init()

    def RAO_convert(self, row, row_mdb, form):
        if row['Kbm']:
            row['Kbm'] = "{:e}".format(float(row['Kbm']))
        if row['Tonne']:
            row['Tonne'] = self._to_exp(str(float(row_mdb['Kg']) / 1000))
        # if row['DocVid']:
        #     row['DocVid'] = row['DocVid'].zfill(2)
        #
        self._fill_by_dash(row, ['BCod', 'factory', 'Subsid', 'FCP'])
        form['form_main_data']['position'] = '-'

        # raos = set()
        # for k in ('Nuclid', ):
        #     for x in row_mdb[k].split('; '):
        #         raos.add(x)

        raos = set()
        for k in ('Nuclid', ):
            for y in row_mdb[k].split('; '):
                for x in y.split(';'):
                    if x:
                        x = x.strip()
                        # x.strip(';')
                        if x.endswith(';'):
                            x = x[:-1]
                        if x.endswith('m'):
                            x = x[:-1] + 'м'

                        if x == 'смесь нуклидов урана':
                            x = 'сумма нуклидов урана'

                        raos.add(x)

        row['RAOCodMax'] = '; '.join(raos)

        kod_grs = []

        # for rad in row['RAOCodMax'].split('; '):
        #     rad_in_spr = next(filter(lambda x: x['\ufeffName_RN'] == rad, self.radios))
        #     kod_rg = rad_in_spr['Kod_gruppy']
        #     kod_grs.append(kod_rg)

        perenos = False

        for rad in raos:
            try:
                # rad_in_spr = next(filter(lambda x: x['\ufeffName_RN'] == rad, self.radios))
                rad_in_spr = self.radios[rad]
                perenos = True
                kod_rg = rad_in_spr['Kod_gruppy']
                kod_grs.append(kod_rg)
            except (StopIteration, KeyError):
                pass

        # print(kod_grs)

        if perenos:

            if len(kod_grs) == 1 and kod_grs[0] == 'Тритий':
                # print('----1')
                row['ActTR'] = row['ActBG']
                row['ActBG'] = row['ActUR'] = '-'
            elif all(i in kod_grs for i in ('Тритий', 'а', 'б', 'Трансурановый')):
                # print('----2')
                row['ActTR'] = row['ActUR'] = '???'
            elif all(i in kod_grs for i in ('Тритий', 'а', 'б')):
                # print('----3')
                row['ActTR'] = '???'
                row['ActUR'] = '-'
            elif all(i in kod_grs for i in ('Тритий', 'б')):
                # print('----4')
                row['ActTR'] = '???'
                row['ActUR'] = '-'
            elif all(i in kod_grs for i in ('Тритий', 'а')):
                # print('----5')
                row['ActTR'] = row['ActBG']
                row['ActBG'] = row['ActUR'] = '-'
            elif all(i in kod_grs for i in ('Тритий', 'Трансурановый')):
                # print('----6')
                row['ActUR'] = row['ActA']
                row['ActTR'] = row['ActBG']
                row['ActA'] = row['ActBG'] = '-'
            elif all(i in kod_grs for i in ('Трансурановый', 'а', 'б')):
                # print('----7')
                row['ActUR'] = '???'
                row['ActTR'] = '-'
            elif all(i in kod_grs for i in ('Трансурановый', 'б')):
                # print('----8')
                row['ActUR'] = row['ActA']
                row['ActTR'] = row['ActA'] = '-'
            elif all(i in kod_grs for i in ('Трансурановый', 'а')):
                # print('----9')
                row['ActUR'] = '???'
                row['ActTR'] = '-'
            elif all(i in kod_grs for i in ('Трансурановый',)):
                # print('----10')
                row['ActUR'] = row['ActA']
                row['ActA'] = row['ActTR'] = '-'
            elif len(kod_grs) == 1 and kod_grs[0] in ('а', 'б') or all(i in ('а', 'б') for i in kod_grs):
                # print('----11')
                row['ActTR'] = row['ActUR'] = '-'
            else:
                pass
                # print('-----12------', kod_grs)

    def IOU_convert(self, row, row_mdb, form):
        if row['Nss'] != '':
            row['Nss'] = str(int(float(row['Nss']) * 12))
        if row['Kg'] != '':  # todo: !!!!!!!!!!!!!!!!!!!!!!!!!~
            row['Kg'] = "{:e}".format(float(row['Kg']))  # todo: float, не int
        # if row['DocVid']:
        #     row['DocVid'] = row['DocVid'].zfill(2)
        #
        self._fill_by_dash(row, ['FormSobst', 'Pravoobl'])
        form['form_main_data']['position'] = '-'

    def ORP_convert(self, row, row_mdb, form):
        # if row['Typ']:
        #     typ = {
        #         '4': '04',
        #         '5': '05',
        #         '6': '06',
        #         '7': '07',
        #         '8': '08',
        #         '9': '09'
        #     }
        #     row['Typ'] = typ.get(row['Typ'], row['Typ'])
        if row['Activn']:
            row['Activn'] = "{:e}".format(float(row['Activn']))
        if row['Kg']:
            row['Kg'] = "{:e}".format(float(row['Kg']))
        if row['Kbm']:
            row['Kbm'] = "{:e}".format(float(row['Kbm']))
        # if row['DocVid']:
        #     row['DocVid'] = row['DocVid'].zfill(2)
        #
        self._fill_by_dash(row, ['FormSobst', 'Pravoobl'])
        form['form_main_data']['position'] = '-'

    def ROZ_convert(self, row, row_mdb, form):
        if row['Activn'] != '':
            row['Activn'] = self._to_exp(row['Activn'])
        # if row['DocVid']:
        #     row['DocVid'] = row['DocVid'].zfill(2)
        for k in ('Subsid', 'FCP', 'CodRAO'):
            row[k] = '-'
        #
        self._fill_by_dash(row, ['factory', 'BCod', 'Sht'])
        form['form_main_data']['position'] = '-'

    def ZRI_convert(self, row, row_mdb, form):
        if row['Activn'] != '':
            row['Activn'] = "{:e}".format(float(row['Activn']))
        if row['Nss'] != '':
            row['Nss'] = str(int(float(row['Nss']) * 12))
        # if row['DocVid']:
        #     row['DocVid'] = row['DocVid'].zfill(2)
        #
        self._fill_by_dash(row, ['Sht', 'Kateg', 'FormSobst', 'Pravoobl'])
        form['form_main_data']['position'] = '-'

    def ORI_convert(self, row, row_mdb, form):
        if row['Activn']:
            row['Activn'] = "{:e}".format(float(row['Activn']))
        # if row['vid']:
        #     row['vid'] = row['vid'].zfill(2)
        #
        self._fill_by_dash(row, ['FormSobst', 'Pravoobl'])
        form['form_main_data']['position'] = '-'
