from services.exceptions import CoreBaseException
from services.temp import TempFile
from settings import TEMP_DIR_DECREES_PATH
from wsgi.apps.historical_data.rules import historical_data_rules
from wsgi.apps.historical_data.services.repositories import IMPORT_HISTORICAL_DATA_INFO_REPOSITORY
from wsgi.apps.historical_data.services.services import get_decree_types, run_import, import_decree_threads_storage
from wsgi.utils import parse_request, response, api_view


@parse_request(historical_data_rules)
def historical_data_views_manager(request, *args, **kwargs):
    action = kwargs['action']
    # GET
    if action == 'list_decrees_types':
        return list_decrees_types_view(request)
    elif action == 'get_import_info':
        return get_import_info_view(request)
    # POST
    elif action == 'import':
        return import_view(request, kwargs['decree'])


@api_view()
def list_decrees_types_view(request, *arg, **kwargs):
    """Получить список доступных приказов для импорта исторических данных"""
    return response(request, get_decree_types(), many=True)


@api_view()
def import_view(request, decree, *args, **kwargs):
    """Импортировать исторические данные"""
    if 'file' not in request.files:
        raise CoreBaseException(message='required form-data (file) with key = file')
    file_ = request.files['file']
    ext = '.RAODB' if decree == 'decree_1_13' else '.mdb'
    db_file_type = 'firebird' if decree == 'decree_1_13' else 'mdb'
    with TempFile(TEMP_DIR_DECREES_PATH, ext, close_others=False, close_current=False) as temp:
        file_.save(temp.full_path)
        run_import(request, kwargs['user'], decree, temp.full_path, db_file_type)

    return response(request, {
        'message': 'Начат импорт исторических данных'
    })


@api_view()
def get_import_info_view(request, *args, **kwargs):
    """Получить текущую информацию об импорте истор. данных (процесс импорта)"""
    resp = {
        'data': {
            'import_historical_data_info': [v.as_dict() for _, v in import_decree_threads_storage.items()],
            'edit': {
                'forms_conflicts_table': {
                    'rowCreate': False,
                    'rowDelete': False,
                    'rowMove': False,
                }
            }
        },
        'repository': IMPORT_HISTORICAL_DATA_INFO_REPOSITORY,
    }
    return response(request, resp)
