from services.temp import TempFile
from settings import TEMP_DIR_PATH
from web import extra_actions
from wsgi.apps.reporting_forms.models import FormModel
from wsgi.apps.reporting_forms.rules import forms_actions_rules
from wsgi.apps.reporting_forms.services.forms_actions.actions import exec_action, \
    FormActionsResponseTypes, FORMS_ACTIONS
from wsgi.utils import parse_request, response, api_view, file_response


@parse_request(forms_actions_rules)
def reporting_forms_actions_view_manager(request, *args, **kwargs):
    action = kwargs['action']
    # === GET ===
    if action == 'list':
        return list_view(request, kwargs['report_form_id'])
    # === POST ===
    elif action == 'exec':
        return exec_view(request, kwargs['action_name'], kwargs['path'], kwargs['body'])


@api_view()
def list_view(request, report_form_id, *args, **kwargs):
    """Получить список действий для формы"""
    form = FormModel(request).read(report_form_id)
    if extra_actions.get(request.full_path_with_action) is not None:
        actions_for_form_gen_func = extra_actions[request.full_path_with_action]
        context = {
            'form': form.as_dict(),
        }
        api2_clone = request.storage['api2_clone']
        actions_for_form = FORMS_ACTIONS.dict_(form, kwargs['user'], actions_for_form_gen_func(context))
    else:
        actions_for_form = FORMS_ACTIONS.dict_(form, kwargs['user'])
    return response(request, actions_for_form)


@api_view()
def exec_view(request, action_name, path, body, *args, **kwargs):
    """Выполнить действие над отчетной формой"""
    form_id = body['context']['report_form_id']
    response_type, result, extra_kwargs = exec_action(request, action_name, kwargs['user'], form_id)
    if response_type == FormActionsResponseTypes.FILE:
        return file_response(request, result)
    elif response_type == FormActionsResponseTypes.XLSX_WORKBOOK:
        with TempFile(TEMP_DIR_PATH, '.xlsx') as temp_file:
            result.save(temp_file.full_path)
            return file_response(
                temp_file.full_path,
                download_name=extra_kwargs['download_name_gen_func'](temp_file.full_path, FormModel(request).read(form_id))
            )
    return response(request, result)
