from services.exceptions import PermissionDeniedException, CoreBaseException
from wsgi.apps.reporting_forms.models import FormModel
from wsgi.apps.users.models import UserModel
from wsgi.apps.users.serializers import UsersSerializer
from wsgi.permissions import _check_permission


def check_edit_report_form_permission(user, form):
    _check_permission(user, UsersSerializer.PermissionActions.EDIT_CREATED_REPORT_FORMS)
    if form.user_id != user.record_id:
        _check_permission(user, UsersSerializer.PermissionActions.EDIT_ALL_REPORT_FORMS)


def check_unload_upload_exchange_files(user):
    _check_permission(user, UsersSerializer.PermissionActions.UPLOAD_UNLOAD_FILE)


def check_add_remarks_to_reports_forms_permission(user):
    """Добавление примечаний к отчетным формам"""
    _check_permission(user, UsersSerializer.PermissionActions.EDIT_REMAKRS_reports_forms)


def check_add_comments_to_reports_forms_permission(user):
    """Добавление клмментариев к отчетным формам"""
    _check_permission(user, UsersSerializer.PermissionActions.EDIT_COMMENTS_REPORTS_FORMS)


def check_confirm_cancel_confirm_form_permission(user: UserModel.Data):
    """Подтверждение/отмена подтверждения отчётной формы"""
    if user.user_role not in UserModel.ACTIONS_TO_ROLES[UserModel.GlobalActions.CONFIRM_CANCEL_CONFIRM_FORM]:
        raise PermissionDeniedException(user.user_role.value, 'подтверждение/отмена подтверждения отчётной формы')


def check_change_form_hint_permission(user: UserModel.Data):
    """Изменение подсказок отчетной формы"""
    if not user.is_admin:
        raise PermissionDeniedException(user.user_role.value, 'создание/изменение подсказок отчетной формы')


def check_read_form_permission(user: UserModel.Data, form: FormModel.Data):
    if user.is_admin or not user.all_kod_orgs:
        return
    if form.kod_org not in user.all_kod_orgs:
        raise CoreBaseException(
            rus_message='Пользователь не обладает достаточными правами для открытия формы.'
        )
