from wsgi.apps.reporting_forms.models import FormModel
from wsgi.apps.users.models import UserModel
from wsgi.apps.users.serializers import UsersSerializer


def calc_permissions_for_form_actions(request, report_form_id: int, user: UserModel.Data, form: FormModel.Data):
    try:
        allowed_actions = FormModel.actions_for_statuses[form.form_state]
    except KeyError:
        return {
            'edit': False,
            'remark': False,
            'comment': False,
            'warning': False,  # пока не реализованы
            'unloadingExchangeFile': False,  # НЕ АЧ (возможно word, pfd, xlsx)
            'unloading': False,  # обменный файл
            'hint': False,
        }
    act = FormModel.FormActions

    user_perm_b = user.permission_as_binary
    user_perm_act = UsersSerializer.PermissionActions
    # права пользователя на действия над отчетной формой
    if form.user_id == user.record_id:
        user_edit_allowed = True
    else:
        user_edit_allowed = user_perm_b[user_perm_act.EDIT_ALL_REPORT_FORMS.value[0]] != '0'
    user_remark_allowed = user_perm_b[user_perm_act.EDIT_REMAKRS_reports_forms.value[0]] != '0'
    user_comment_allowed = user_perm_b[user_perm_act.EDIT_COMMENTS_REPORTS_FORMS.value[0]] != '0'
    user_exch_allowed = user_perm_b[user_perm_act.UPLOAD_UNLOAD_FILE.value[0]] != '0'
    # права на действия относительно статуса формы и прав пользователя
    return {
        'edit': (form.form_export_ts is None and form.form_import_ts is None and
                 act.EDITING in allowed_actions and user_edit_allowed),
        'remark': (form.form_export_ts is None and form.form_import_ts is None and
                   act.EDITING_REMARKS in allowed_actions and user_remark_allowed),
        'comment': act.EDITING_COMMENTS in allowed_actions and user_comment_allowed,
        'warning': False,  # пока не реализованы
        'unloadingExchangeFile': act.EXPORT_EXCHANGE_FILE in allowed_actions and user_exch_allowed,
        'unloading': False, #  act.EXPORT_DOCX_XLSX_PDF in allowed_actions,
        'hint': user.is_admin,
    }
