from services.exceptions import CoreBaseException


class FormsActionsException(CoreBaseException):
    ...


class FormsActionsPermissionsDeniedException(FormsActionsException):
    def __init__(self, allowed_user_roles, *args: object, **kwargs) -> None:
        super().__init__(
            type='FormsActionsPermissionsDeniedException',
            message=f'Пользователь не обладает необходимыми правами на выполнение действия. '
                    f'Пользователь должен иметь одну из следующих ролей: '
                    f'{", ".join(st.value for st in allowed_user_roles)}',
        )


class FormsActionsFormStateWrongException(FormsActionsException):
    def __init__(self, allowed_form_states, *args: object, **kwargs) -> None:
        super().__init__(
            type='FormsActionsFormStateWrongException',
            message=f'Форма имеет некорректный статус, невозможно выполнить действие. '
                    f'Форма должна иметь один из следующих статусов: '
                    f'{", ".join(st.value for st in allowed_form_states)}',
        )


class FormsActionNotFoundException(FormsActionsException):
    def __init__(self, action_name, *args: object, **kwargs) -> None:
        super().__init__(
            type='FormsActionNotFoundException',
            message=f'FormsActionNotFoundException. Action name: {action_name}',
        )
