from services.exceptions import CoreBaseException
from wsgi.apps.reporting_forms.models import FormModel
from wsgi.apps.reports.rules import reports_rules
from wsgi.utils import parse_request, response, api_view


@parse_request(reports_rules)
def reports_view_manager(request, *args, **kwargs):
    action = kwargs['action']
    if action == 'create_adjustment_report':
        return create_adjustment_report_view(request, kwargs['body'])

@api_view()
def create_adjustment_report_view(request, body, *args, **kwargs):
    """Создать корректирующие отчетные формы на основе отчетных форм"""
    forms = [FormModel(request).read(id_) for id_ in body['report_form_ids']]

    st = FormModel.Data.FormStateStatuses
    # разрешенные статусы форм, которые могут быть включены в кор. отчет
    allowed_statuses = [st.UNLOADED, st.LOADED, st.REJECTED, st.ACCEPTED]

    if not all(form.form_state in allowed_statuses for form in forms):
        raise CoreBaseException(rus_message='Не все формы имеют разрешенный статус. '
                                            'Формы должны иметь один из следующих статусов: Выгружена, '
                                            'Загружена, Отклонена, Принята')

    new_form_ids = []
    for form in forms:
        form.form_state = st.CANCELED  # задаем для исходных форм статус "Аннулирована"
        FormModel(request).update(form.report_form_id, form)

        new_form = form.copy()
        new_form.form_state = st.DRAFT  # для новых форм задаем статус "Черновик"
        new_form.form_export_ts = None  # убираем время экспорта
        new_form.form_import_ts = None  # убираем время импорта
        new_form.log_id = None  # убираем log_id импорта формы
        new_form.user_id = kwargs['user'].record_id

        if new_form.adjustment_no is None:
            new_form.adjustment_no = 1
        else:
            new_form.adjustment_no += 1

        new_form = FormModel(request).save(new_form)
        new_form_ids.append(new_form.report_form_id)

    return response(request, new_form_ids, many=True)
