from repository.areports import areports_repository
from repository.types import types_repository
from wsgi.apps.repository.areports_rules import areports_rules
from wsgi.apps.repository.areports_rules import areports_types_rules
from wsgi.utils import api_view, response
from wsgi.utils import parse_request


@parse_request(areports_rules)
def areports_view_manager(request, *args, **kwargs):
    action = kwargs['action']
    if action == 'get_params':
        return get_params_areport_repository_view(request, kwargs['areport_type'])


@parse_request(areports_types_rules)
def areports_types_view_manager(request, *args, **kwargs):
    action = kwargs['action']
    if action == 'list':
        return list_areports_types_repository_view(request)


@api_view()
def list_areports_types_repository_view(request, *args, **kwargs):
    """Получить список типов аналитических отчетов"""
    return response(request, areports_repository.get_areport_types_list_f(), many=True)


@api_view()
def get_params_areport_repository_view(request, areport_type, *args, **kwargs):
    """Получить список параметров для аналитического отчета areport_type"""
    areport_rep = areports_repository.get_areport_repository(areport_type)
    types = types_repository.get_types_repository()
    types.update(areport_rep['params_types'])
    res = {
        'form': {
            'table': {
                'headers': [],
                'model': areport_rep['params']
            }
        },
        'types': types,
    }
    if 'areport_params' in areport_rep:
        res['areport_params'] = areport_rep['areport_params']
    else:
        res['areport_params'] = {
            'areport_name': areport_rep['name'],
            'areport_type': areport_type,
        }
    return response(request, res)
