import copy

from repository.forms import forms_repository
from repository.forms_journals import forms_journals_repository
from repository.forms_journals.exceptions import FormsJournalRepositoryNotFoundException
from wsgi.apps.hints.models import HintModel
from wsgi.apps.repository.forms_rules import forms_rules, forms_types_rules, forms_exports_types_rules
from wsgi.utils import parse_request
from wsgi.utils import response, api_view


@parse_request(forms_rules)
def forms_view_manager(request, *args, **kwargs):
    action = kwargs['action']
    if action == 'read':
        return read_form_repository_view(request, kwargs['object_type'])


@parse_request(forms_types_rules)
def forms_types_view_manager(request, *args, **kwargs):
    action = kwargs['action']
    if action == 'list':
        return list_forms_types_repository_view(request)
    elif action == 'list_by_report_num':
        return list_by_report_type_forms_types_repository_view(request, kwargs['report_num'], kwargs.get('journal_name'),
                                                               kwargs.get('preset'), kwargs.get('decree'))


@parse_request(forms_exports_types_rules)
def forms_exports_types_view_manager(request, *args, **kwargs):
    action = kwargs['action']
    if action == 'list':
        return list_forms_exports_types_repository_view(request)
    if action == 'read':
        return read_forms_exports_types_repository_view(request, kwargs['object_type'])


@api_view()
def read_form_repository_view(request, form_name, *args, **kwargs):
    """Получить репозиторий формы по имени"""
    form_repository = forms_repository.get_form_repository_f(form_name)
    # добавление текстовых подсказок из бд к атрибутам формы по ее типу (form_1_1, form_1_2, ...)
    form_hints = HintModel(request).filter({
        'item_type': [HintModel.Data.ItemTypeTypes.FORM.value],
        'item_class': [form_name],
    })
    if form_hints:
        form_hint = form_hints[0]
        # подсказки к ячейкам таблицы
        for table_name, table_hints in form_hint.attrs['tables'].items():
            for attr_name, attr_hint in table_hints.items():
                try:
                    attr_model = next(filter(lambda model_item: model_item['field'] == attr_name,
                                             form_repository['form']['tables'][table_name]['model']))
                    attr_model['help'] = attr_hint
                except StopIteration:  # есть подсказка для поля, но поля больше нет в форме
                    continue
        # подсказки для атрибутов формы
        for attr_name, attr_hint in form_hint.attrs['form_main_attrs'].items():
            try:
                attr_model = next(filter(lambda model_item: model_item['field'] == attr_name,
                                         form_repository['form']['form_main_attrs']))
                attr_model['help'] = attr_hint
            except StopIteration:  # есть подсказка для поля, но поля больше нет в форме
                continue

    return response(request, form_repository)


@api_view()
def list_forms_types_repository_view(request, *args, **kwargs):
    """Получить список типов форм"""
    forms_types = forms_repository.get_forms_types_list_repository()
    return response(request, forms_types, many=True)


@api_view()
def list_by_report_type_forms_types_repository_view(request, report_num, journal_name, preset, decree, *args, **kwargs):
    """Получить список типов форм принадлежащих отчету с номером report_num"""
    forms_types = forms_repository.get_forms_types_list_repository_by_report_num_and_decree_year(report_num, decree)
    if preset is not None:
        try:
            form_journal_rep = forms_journals_repository.get_forms_journal(journal_name)
            preset_query = form_journal_rep['presets'][preset]['query']
        except (FormsJournalRepositoryNotFoundException, KeyError):
            return response(request, forms_types, many=True)
        forms_types = copy.deepcopy(forms_types)
        prev_form_item = None
        for form_type in forms_types:
            form_type_value = form_type['value'][:-4] if form_type['value'].count('_') > 2 else form_type
            from loguru import logger
            logger.debug(form_type_value)
            preset_query = preset_query.replace('\\', '')  # убрать экранирование символов PostgreSQL
            if form_type_value in preset_query:
                # отображать по умолчанию на фронтенде отчет. форму относительно журнала,
                # в котором находится пользователь
                form_type['default'] = True
                if prev_form_item is not None:
                    prev_form_item.pop('default')
                prev_form_item = form_type

    return response(request, forms_types, many=True)


@api_view()
def list_forms_exports_types_repository_view(request, *args, **kwargs):
    """Получить список типов эксплрта форм"""
    exports_types = forms_repository.get_forms_exports_types()
    return response(request, exports_types)


@api_view()
def read_forms_exports_types_repository_view(request, form_name, *args, **kwargs):
    """Получить список типов экспорты для формы form_name"""
    export_type = forms_repository.get_form_exports_types(form_name)
    return response(request, export_type)