from repository.vocs import vocs_repository
from wsgi.apps.hints.models import HintModel
from wsgi.apps.repository.vocs_rules import vocs_types_rules, vocs_rules
from wsgi.apps.users.models import UserModel
from wsgi.utils import parse_request
from wsgi.utils import response, api_view


@parse_request(vocs_types_rules)
def vocs_repository_types_view_manager(request, *args, **kwargs):
    action = kwargs['action']
    if action == 'list':
        return vocs_types_list_view(request)
    if action == 'dict':
        return vocs_types_dict_view(request)


@parse_request(vocs_rules)
def vocs_view_manager(request, *args, **kwargs):
    action = kwargs['action']
    if action == 'read':
        return read_voc_repository_view(request, kwargs['voc_type'])


@api_view()
def vocs_types_list_view(request, *args, **kwargs):
    """Получить типы справочников в формате списка"""
    vocs_types = vocs_repository.get_vocs_types_list()
    user: UserModel.Data = kwargs['user']
    if user.is_admin:
        vocs_types = list(filter(lambda x: x['voc_type'] != 'bugreports', vocs_types))
    else:
        vocs_types = list(filter(lambda x: x['voc_type'] != 'bugreports_admin', vocs_types))

    return response(request, vocs_types, many=True)


@api_view()
def vocs_types_dict_view(request, *args, **kwargs):
    """Получить типы справочников в формате словаря"""
    return response(request, vocs_repository.get_vocs_types_dict())


@api_view()
def read_voc_repository_view(request, voc_type, *args, **kwargs):
    """Получить репозиторий справочника по `voc_type`"""
    voc_repository = vocs_repository.get_voc_repository_f(voc_type)
    # добавление текстовых подсказок из бд к атрибутам формы по ее типу (form_1_1, form_1_2, ...)
    form_hints = HintModel(request).filter({
        'item_type': [HintModel.Data.ItemTypeTypes.VOC.value],
        'item_class': [voc_type],
    })
    if form_hints:
        form_hint = form_hints[0]
        for attr_name, attr_hint in form_hint.attrs.items():
            try:
                attr_model = next(filter(lambda model_item: model_item['field'] == attr_name,
                                         voc_repository['form']['table']['model']))
                attr_model['help'] = attr_hint
            except StopIteration:  # есть подсказка для поля, но поля больше нет в форме
                continue

    return response(request, vocs_repository.get_voc_repository_f(voc_type))
