from wsgi.apps.reporting_forms.models import FormModel
from wsgi.apps.voc.models import VocValueModel, OrganizationVocModel


def save_import_item(request, item_data, model, filter_=None, commit=False, filter_lambda=None):
    """Метод проверяет данные и сохраняет/обновляет/игнорирует запись.

    Args:
        request: Запрос.
        item_data: Данные сущности в формате словаря.
        model: Класс модели данных.
        commit: Делать коммит запроса сразу.
    """

    if filter_:
        items = model(request).filter(filter_)
        if filter_lambda:
            items = list(filter(filter_lambda, items))
        if items:
            return 'found_original', items[0]

    id_name = 'report_form_id' if model == FormModel else 'record_id'

    if model == OrganizationVocModel:
        item_found = model(request).filter_raw(params=None, query_condition=f'import_author = \'{item_data["import_author"]}\'')
    else:
        item_found = model(request).filter_raw(
            params=None,
            query_condition=f'import_src_id = \'{item_data["import_src_id"]}\' AND import_author = \'{item_data["import_author"]}\''
        )
    if item_found:
        item_old = item_found[0]
        if item_old['import_hash'] == item_data['import_hash']:
            return 'found', item_old
        else:
            f, s = 'updated', model(request).update(item_old[id_name], item_data, commit=commit, return_as_dict=True)
            return f, s
    else:
        f, s = 'created', model(request).save(item_data, commit=commit, return_as_dict=True)
        return f, s


def save_import_voc(request, item_data, commit=False):
    filter_ = ['and',
               # ['=', 'creator_id', str(item_data['creator_id'])],
               ['=', 'voc_type', str(item_data['voc_type'])],
               ['=', 'value_id', str(item_data['value_id'])],
               ]
    vocs = VocValueModel(request).filter(filter_)
    if vocs:
        if item_data['attrs']['code_OKPO_org']:
            vocs_with_same_okpo = list(filter(
                lambda x: x.attrs.get('code_OKPO_org') == item_data['attrs']['code_OKPO_org'],
                vocs
            ))
            if vocs_with_same_okpo:
                vocs_with_same_okpo = vocs_with_same_okpo[0]
                for attr, attr_value in item_data['attrs'].items():
                    vocs_with_same_okpo.attrs[attr] = attr_value
                updated_voc = VocValueModel(request).update(vocs_with_same_okpo.record_id, vocs_with_same_okpo)
                return 'updated', updated_voc.as_dict()
    return 'created', VocValueModel(request).save(item_data).as_dict()

