from repository.vocs import vocs_repository
from wsgi.apps.user_logs.models import UserLogsModel
from wsgi.serializers import BaseSerializer


class UserLogsVocSerializer(BaseSerializer):
    model = UserLogsModel(None)

    def __init__(self, request):
        super().__init__(request)
        self.extract_field_changed_from_value_id = False
        self.voc_repository = vocs_repository.get_voc_repository('users_logs')
        self.allowed_filter_opers_for_fields.update({m['field']: m.get('filter', None) for m in self.voc_repository['model']})

    def _after_init(self):
        # Замена необходима, т.к. фронт работает с логами пользователя как со справочниками, ожидает универсального
        # поведения. Т.к. у логов пользователя другая таблица, отличающаяся по структуре, заменяем вручную.
        if self.extract_field == 'value_id':
            self.extract_field = 'record_id'
            self.extract_field_changed_from_value_id = True
        super()._after_init()

    def _extract(self, resp):
        res = super()._extract(resp)
        if self.extract_field_changed_from_value_id:
            return list(map(str, res))

    def _response(self, resp):
        resp = super()._response(resp)
        resp['edit'] = False
        resp['voc_type'] = 'users_logs'
        resp['label'] = resp['attrs']['date_time']
        # Для того, чтобы поддерживался механизм работы как со справочниками.
        resp['value_id'] = str(resp['record_id'])
        return resp
