from wsgi.apps.user_settings.const import USER_SETTINGS_VOC_TYPE
from wsgi.apps.user_settings.rules import user_settings_rules
from wsgi.apps.user_settings.services import get_user_settings_voc
from wsgi.apps.voc.models import VocValueModel
from wsgi.utils import response, parse_request, api_view


@parse_request(user_settings_rules)
def user_settings_view_manager(request, *args, **kwargs):
    action = kwargs['action']
    # POST
    if action == 'read':
        return read_view(request, kwargs['body'])
    elif action == 'update':
        return update_view(request, kwargs['body'])


@api_view()
def read_view(request, body, *args, **kwargs):
    """Получить настройки интерфейса пользователя.

    Args:
        body: Ключи настроек пользователя - список ключей. Если заданы, вернется только
            значение по данному ключу внутри attrs,
            т.е. ``voc.attrs[setting_key]``. Если = ``None``, вернутся все настройки.
    """
    user_settings_voc = get_user_settings_voc(request, kwargs['user'])
    if user_settings_voc is None:
        return response(request, None)

    if body:
        resp = {k: user_settings_voc.attrs.get(k, None) for k in body}
        return response(request, resp)

    return response(request, user_settings_voc.attrs)


@api_view()
def update_view(request, body, *args, **kwargs):
    """Обновить настройки интерфейса пользователя. Обновляются только те параметры, которые пришли в теле.
    (Поле attrs не затирается полностью, а обновляется).
    """
    user_settings_voc = get_user_settings_voc(request, kwargs['user'])
    if user_settings_voc is not None:
        user_settings_voc.attrs.update(body)
        user_settings_voc = VocValueModel(request).update(user_settings_voc.record_id, user_settings_voc) # user_settings_voc.attrs.update(body)
        return response(request, user_settings_voc.as_dict())

    # не найдены настройки пользователя
    data = {
        'voc_type': USER_SETTINGS_VOC_TYPE,
        'attrs': {
            'user_id': kwargs['user'].record_id,
        }
    }
    data['attrs'].update(body)
    user_settings_voc = VocValueModel(request).save(VocValueModel.Data.from_dict(data))
    return response(request, user_settings_voc.as_dict())
