from wsgi.apps.users.models import UserModel
from wsgi.apps.voc.models import OrganizationVocModel


def bind_new_valid_org_version_to_users(request, old_org_v: OrganizationVocModel.Data, new_org_v: OrganizationVocModel.Data):
    """Функция осуществляет обновление привязанной основной организации у пользователя, если у организации
    обновилась версия. Делает:
        1. Замену основной организации у пользователей (org_id) на новый record_id.
        2. Замену в привязанные орг. (organizations_ids) на новый record_id, если был найден старый среди них.

    Args:
        request: Запрос.
        old_org_v: Версия организации (старая версия), которая изменяется.
        new_org_v: Новая версия организации.

    """
    users_belong_to_org = UserModel(request).filter({'org_id': old_org_v.record_id})
    for user in users_belong_to_org:
        user.org_id = new_org_v.record_id
        UserModel(request).update(user.record_id, user)

    for user in UserModel(request).list():
        if user.organizations_ids and str(old_org_v.record_id) in user.organizations_ids:
            user.organizations_ids = user.organizations_ids.replace(str(old_org_v.record_id), str(new_org_v.record_id))
            UserModel(request).update(user.record_id, user)


def get_org_active_version(request, kod_org: str) -> OrganizationVocModel.Data:
    """Получить активную версию организаии по коду.

    Args:
        request: Запрос
        kod_org: Код.

    Returns:
        OrganizationVocModel.Data: Организация. Если не найдена активная - вернется None.

    """
    orgs = OrganizationVocModel(request).filter(['=', 'KOD_ORG', kod_org])
    for org in orgs:
        if not org.valid_to:
            return org


def get_orgs_ids_line(request, kod_orgs_line: str) -> str:
    """Получить строку, состоящую из id организаций, разделенных `; `.

    Args:
        request: Запрос.
        kod_orgs_line: Строка кодов организаций, разделенных `; `.

    Returns:
        str: id организаций, разделенных `; `.

    """
    if not kod_orgs_line:
        return ''
    orgs_ids = set()
    for kod_org_ in kod_orgs_line.split('; '):
        org = get_org_active_version(request, kod_org_)
        if org is not None:
            orgs_ids.add(str(org.record_id))
    return '; '.join(orgs_ids)
