from wsgi.apps.users.models import UserModel
from wsgi.apps.users.permissions import check_edit_users_voc_permissions
from wsgi.apps.users.rules import users_rules
from wsgi.apps.users.serializers import UsersSerializer, UsersVocSerializer
from wsgi.utils import parse_request, response, api_view


@parse_request(users_rules)
def users_view_manager(request, *args, **kwargs):
    action = kwargs['action']
    # GET
    if action == 'read':
        return read_users_view(request, kwargs['record_id'])
    elif action == 'list':
        return list_users_view(request)
    elif action == 'filter':
        return filter_users_view(request)
    elif action == 'voc_list':
        return voc_list_users_view(request)
    elif action == 'voc_read':
        return voc_read_users_view(request, kwargs['record_id'])
    elif action == 'list_choice':
        return list_choice_users_view(request)
    # POST
    elif action == 'voc_save':
        return voc_save_users_view(request, kwargs['body'])
    elif action == 'voc_update':
        return voc_update_users_view(request, kwargs['record_id'], kwargs['body'])
    elif action == 'delete':
        return delete_users_view(request, kwargs['record_id'])


@api_view()
def read_users_view(request, record_id, *args, **kwargs):
    """Получть запись пользователя"""
    return response(request, UsersSerializer(request, kwargs['user']).read(record_id))


@api_view()
def list_users_view(request, *args, **kwargs):
    """Получть все записи пользователей"""
    return response(request, UsersSerializer(request, kwargs['user']).list(), many=True)


@api_view()
def filter_users_view(request, *args, **kwargs):
    """Получть список пользователей по фильтру"""
    return response(request, UsersSerializer(request, kwargs['user']).filter(request.args), many=True)


@api_view()
def voc_list_users_view(request, *args, **kwargs):
    """Получить список пользователей для отображения в таблице"""
    return response(request, UsersVocSerializer(request, kwargs['user']).list(), many=True)


@api_view()
def voc_read_users_view(request, record_id, *args, **kwargs):
    """Получить запись пользователя для таблицы справочника"""
    return response(request, UsersVocSerializer(request, kwargs['user']).read(record_id))


@api_view()
def list_choice_users_view(request, *args, **kwargs):
    """Получить список пользователей для выбора"""
    return response(
        request,
        [{'label': user.fio, 'value': user.record_id} for user in UserModel(request).list()],
        many=True
    )


@api_view()
def voc_save_users_view(request, body, *args, **kwargs):
    """Сохранить запись пользователя. Вызов из компонента справочников"""
    check_edit_users_voc_permissions(kwargs['user'])
    return response(request, UsersVocSerializer(request, kwargs['user']).save(body))


@api_view()
def voc_update_users_view(request, record_id, body, *args, **kwargs):
    """Обновить запись пользователя. Вызов из компонента справочников"""
    check_edit_users_voc_permissions(kwargs['user'])
    return response(request, UsersVocSerializer(request, kwargs['user']).update(record_id, body))


@api_view()
def delete_users_view(request, record_id, *args, **kwargs):
    """Удалить запись пользователя"""
    return response(request, UsersSerializer(request).delete(record_id))
