from repository.vocs import vocs_repository
from services.exceptions import CoreBaseException
from wsgi.apps.users.models import UserModel
from wsgi.apps.users.serializers import UsersSerializer
from wsgi.apps.voc.models import VocValueModel
from wsgi.permissions import _check_permission


def check_edit_system_vocs_permission(user):
    """Редатикрование системных справочников"""
    _check_permission(user, UsersSerializer.PermissionActions.EDIT_SYSTEM_VOCS)


def check_edit_vocs_permissions(user):
    """Редатикрование общих справочников"""
    _check_permission(user, UsersSerializer.PermissionActions.EDIT_NOT_SYSTEM_VOCS)


def check_edit_users_voc_permissions(user):
    """Создание и редактирование учетных записей пользователей"""
    _check_permission(user, UsersSerializer.PermissionActions.EDIT_USERS_VOCS)


def gl_check_voc_update_permissions(request, user: UserModel.Data, voc_type, voc: VocValueModel.Data):
    voc_repository = vocs_repository.get_voc_repository(voc_type)
    if voc_repository.get('voc_subtype', 'custom') == 'system':
        check_edit_system_vocs_permission(user)
    if voc.voc_type == 'users':
        check_edit_users_voc_permissions(user)
    check_edit_vocs_permissions(user)
    if voc.creator_id != user.login and user.user_role != UserModel.Data.UserRoleTypes.ADMINISTRATOR:
        raise CoreBaseException(
            rus_message='Редактировать запись справочника может '
                        'только ее создатель или администратор',
            not_repeat=True,  # для фронта, чтобы запросы не слались повторно
        )
