from repository.forms import forms_repository
from repository.snk.snk import snk_repository
from repository.types import types_repository
from repository.vocs import vocs_repository
from repository.vocs.exceptions import VocRepositoryNotFoundException
from services.exceptions import CoreBaseException
from wsgi.apps.bugreports.views import voc_list_bugreports_view, voc_save_bugreports_view, voc_read_bugreports_view, \
    voc_update_bugreports_view, delete_bugreports_view
from wsgi.apps.hints.models import HintModel
from wsgi.apps.user_logs.views import voc_read_users_logs_view, voc_list_users_logs_view
from wsgi.apps.users.models import UserModel
from wsgi.apps.users.views import voc_list_users_view, voc_read_users_view, voc_update_users_view, voc_save_users_view, \
    delete_users_view
from wsgi.apps.voc import services
from wsgi.apps.voc.models import VocValueModel, OrganizationVocModel
from wsgi.apps.voc.permissions import check_edit_vocs_permissions, check_edit_system_vocs_permission, \
    check_edit_users_voc_permissions, gl_check_voc_update_permissions
from wsgi.apps.voc.rules import voc_app_rules, voc_organizations_rules
from wsgi.apps.voc.serializers import OrganizationsVocVersionsSerializer
from wsgi.apps.voc.serializers import VocValuesSerializer, OrganizationsVocSerializer
from wsgi.permissions import bool_permission
from wsgi.utils import response, parse_request, api_view

from wsgi.apps.users import services as users_services


@parse_request(voc_app_rules)
def voc_app_view_manager(request, *args, **kwargs):
    action = kwargs['action']
    # GET
    if action == 'filter':
        return filter_vocs_view(request)
    if action == 'read':
        return read_voc_view(request, kwargs['record_id'], kwargs['voc_type'])
    if action == 'list_by_type':
        return list_by_type_voc_view(request, kwargs['voc_type'], kwargs.get('only_active'))
    if action == 'list_by_value_id':
        return list_by_value_id_vocs_view(request, kwargs['voc_type'], request.args.getlist('value_id'))
    elif action == 'list_organizations':
        return list_organizations_voc_view(request, kwargs.get('for_current_user'))
    elif action == 'list_users':
        return list_users_voc_view(request)
    elif action == 'list_stock_list':
        return list_stock_list_view(request, kwargs['form_no'], kwargs['date'], kwargs['kod_org'])
    elif action == 'get_extra_info':
        return get_extra_info_view(request, kwargs['voc_type'])
    elif action == 'search_by_value_id':
        return search_by_value_id_view(request, kwargs['voc_type'], kwargs['value_id'],
                                       kwargs.get('only_active'), int(kwargs['limit']))
    # POST
    elif action == 'save':
        return save_voc_view(request, kwargs['body'])
    elif action == 'update':
        return update_voc_view(request, kwargs['record_id'], kwargs['voc_type'], kwargs['body'])
    elif action == 'save_new_version':
        return save_new_version_voc_view(request, kwargs['record_id'], kwargs['body'])
    elif action == 'delete':
        return delete_voc_view(request, kwargs['record_id'], kwargs['voc_type'])
    elif action == 'to_archive':
        return to_archive_view(request, kwargs['record_id'], kwargs['voc_type'], kwargs['body'])
    elif action == 'to_make_active':
        return to_make_active_view(request, kwargs['record_id'], kwargs['voc_type'], kwargs['body'])
    elif action == 'change_hint':
        return change_hint_view(request, kwargs['voc_type'], kwargs['attr'], kwargs['body'])
    # -- change table ---
    elif action == 'change_cells':
        return change_cells_voc_view(request, kwargs['body'])


@parse_request(voc_organizations_rules)
def voc_organizations_view_manager(request, *args, **kwargs):
    action = kwargs['action']
    if action == 'list_versions':
        return list_versions_voc_organizations_view(request, kwargs['record_id'])
    # elif action == 'save_new_version':
    #     return save_new_version_voc_view(request, kwargs['kod_org'], kwargs['body'])


# ____VIEWS____


@api_view()
def read_voc_view(request, record_id, voc_type, *args, **kwargs):
    """Прочитать значение справочника из бд по record_id"""
    # redirect
    if voc_type in ('bugreports', 'bugreports_admin'):
        return voc_read_bugreports_view(request, record_id, args, kwargs)
    if voc_type == 'users':
        return voc_read_users_view(request, record_id, args, kwargs)
    if voc_type == 'users_logs':
        return voc_read_users_logs_view(request, record_id, args, kwargs)
    # view
    user: UserModel.Data = kwargs['user']
    # voc = VocValueModel(request).read(record_id)
    if voc_type == 'organizations':
        return response(request, OrganizationsVocSerializer(request, user, voc_type).read(record_id))
    return response(request, VocValuesSerializer(request, user, voc_type).read(record_id))


@api_view()
def filter_vocs_view(request, *args, **kwargs):
    """Отфильтровать значения справочника по параметрам"""
    return response(
        request,
        VocValuesSerializer(request, kwargs['user'], request.args.get('voc_type')).filter(request.args),
        many=True
    )


@api_view()
def list_by_value_id_vocs_view(request, voc_type, value_id_list, *args, **kwargs):
    """Получить список записей справочников со значениями из value_id_list"""
    # todo: больше не используется
    pass


@api_view()
def list_organizations_voc_view(request, for_current_user, *args, **kwargs):
    """Получить список организаций для выбора"""
    user: UserModel.Data = kwargs['user']

    if user.is_admin or not user.all_kod_orgs:
        return response(
            request,
            [{'label': org.NAME_SHORT, 'value': org.record_id}
             for org in OrganizationVocModel(request).filter(
                {
                    'value_status': VocValueModel.Data.ValueStatusesTypes.ACTIVE.value,
                    'PRZN_ACTIV': OrganizationVocModel.Data.PrznActvnTypes.SHOW_IN_ORGS_LIST.value,
                }
            )]
        )

    orgs_choices = []
    for kod_org in user.all_kod_orgs:
        org = users_services.get_org_active_version(request, kod_org)
        if org is None:
            continue
        orgs_choices.append({
            'label': org.NAME_SHORT,
            'value': org.record_id,
        })

    return response(request, orgs_choices, many=True)


@api_view()
def list_users_voc_view(request, *args, **kwargs):
    """Получить список пользователей для выбора"""
    users_choices = [{'label': user.fio, 'value': user.record_id} for user in UserModel(request).list()]
    return response(request, users_choices, many=True)


@api_view()
def save_voc_view(request, body, *args, **kwargs):
    """Сохранить новую запись справочника с voc_type"""
    voc_type = body['voc_type']
    # redirect
    if body['voc_type'] in ('bugreports_admin', 'bugreports'):
        return voc_save_bugreports_view(request, body, args, kwargs)
    if body['voc_type'] == 'users':
        return voc_save_users_view(request, body, args, kwargs)
    # permissions
    voc_repository = vocs_repository.get_voc_repository(body['voc_type'])
    if voc_repository.get('voc_subtype') and voc_repository['voc_subtype'] in ('system', 'settings'):
        check_edit_system_vocs_permission(kwargs['user'])
    check_edit_vocs_permissions(kwargs['user'])
    # view
    vocs_repository.check_voc_type_existence(body['voc_type'])
    if body['voc_type'] == 'organizations':
        return response(request, OrganizationsVocSerializer(request, kwargs['user'], voc_type).save(body))
    return response(request, VocValuesSerializer(request, kwargs['user'], voc_type).save(body))


@api_view()
def update_voc_view(request, record_id, voc_type, body, *args, **kwargs):
    """Обновить запись справочника по record_id"""
    # redirect
    if voc_type in ('bugreports', 'bugreports_admin'):
        return voc_update_bugreports_view(request, record_id, body, args, kwargs)
    if voc_type == 'users':
        return voc_update_users_view(request, record_id, body, args, kwargs)
    # permissions
    user: UserModel.Data = kwargs['user']
    # voc_type = VocValueModel(request).read(record_id).voc_type
    voc_repository = vocs_repository.get_voc_repository(voc_type)
    if voc_repository.get('voc_subtype') and voc_repository['voc_subtype'] in ('system', 'settings'):
        check_edit_system_vocs_permission(kwargs['user'])
    check_edit_vocs_permissions(kwargs['user'])
    if voc_type != 'organizations' and (VocValueModel(request).read(int(record_id)).creator_id != user.login and
            user.user_role != UserModel.Data.UserRoleTypes.ADMINISTRATOR):
        raise CoreBaseException(rus_message='Редактировать запись справочника может '
                                            'только ее создатель или администратор')
    # view
    if body['voc_type'] == 'bugreport_admin':  # todo:
        body['voc_type'] = 'bugreports'

    vocs_repository.check_voc_type_existence(body['voc_type'])
    if body['voc_type'] == 'organizations':
        return response(request, OrganizationsVocSerializer(request, user, voc_type).update(record_id, body))
    return response(request, VocValuesSerializer(request, user, voc_type).update(record_id, body))


@api_view()
def save_new_version_voc_view(request, record_id, body, *args, **kwargs):  # (ограничение по KOD_ORG)
    user: UserModel.Data = kwargs['user']
    # permissions
    voc_type = VocValueModel(request).read(record_id).voc_type
    voc_repository = vocs_repository.get_voc_repository(voc_type)
    if voc_repository.get('is_system'):
        check_edit_system_vocs_permission(kwargs['user'])
    if body['voc_type'] == 'users':
        check_edit_users_voc_permissions(kwargs['user'])
    check_edit_vocs_permissions(kwargs['user'])
    if (VocValueModel(request).read(record_id).creator_id != user.login and
            user.user_role != UserModel.Data.UserRoleTypes.ADMINISTRATOR):
        raise CoreBaseException(rus_message='Редактировать запись справочника может '
                                            'только ее создатель или администратор')

    # view
    return response(request, OrganizationsVocVersionsSerializer(request, kwargs['user'], record_id).save(body))


@api_view()
def delete_voc_view(request, record_id, voc_type, *args, **kwargs):
    """Удалить запись справочника (объект VocValue)"""
    # redirect
    if voc_type in ('bugreports', 'bugreports_admin'):
        return delete_bugreports_view(request, record_id, args, kwargs)
    if voc_type == 'users':
        return delete_users_view(request, record_id, args, kwargs)
    # permissions
    user: UserModel.Data = kwargs['user']
    voc_type = VocValueModel(request).read(record_id).voc_type
    voc_repository = vocs_repository.get_voc_repository(voc_type)
    voc = VocValueModel(request).read(int(record_id))
    if voc_repository.get('is_system'):
        check_edit_system_vocs_permission(kwargs['user'])
    if voc.voc_type == 'users':
        check_edit_users_voc_permissions(kwargs['user'])
    check_edit_vocs_permissions(kwargs['user'])
    if (VocValueModel(request).read(int(record_id)).creator_id != user.login and
            user.user_role != UserModel.Data.UserRoleTypes.ADMINISTRATOR):
        raise CoreBaseException(rus_message='Редактировать запись справочника может '
                                            'только ее создатель или администратор')
    # view
    if voc.voc_type in ('organizations',):
        raise CoreBaseException(
            message='Delete for organizations voc is not supported now',
            rus_message='Удаление записей для данного справочника не поддерживается',
        )

    return response(request, VocValuesSerializer(request, user, voc_type).delete(record_id))


@api_view()
def to_archive_view(request, record_id, voc_type, body, *args, **kwargs):
    """Архивировать запись справочника"""
    user: UserModel.Data = kwargs['user']
    # permissions
    gl_check_voc_update_permissions(request, user, voc_type, VocValueModel(request).read(record_id))
    # view
    if voc_type == 'organizations':
        org = OrganizationVocModel(request).read(record_id)
        if org.value_status != OrganizationVocModel.Data.ValueStatusesTypes.ACTIVE:
            raise CoreBaseException(rus_message='Архивирование доступно только для активных записей организаций')
        org.value_status = OrganizationVocModel.Data.ValueStatusesTypes.OUTDATED
        OrganizationVocModel(request).update(org.record_id, org)
        return response(request, OrganizationsVocSerializer(request, user, voc_type).read(org.record_id))
    voc = VocValueModel(request).read(record_id)
    if voc.value_status != VocValueModel.Data.ValueStatusesTypes.ACTIVE:
        raise CoreBaseException(rus_message='Архивирование доступно только для активных записей')
    voc.value_status = VocValueModel.Data.ValueStatusesTypes.OUTDATED
    VocValueModel(request).update(voc.record_id, voc)
    return response(request, VocValuesSerializer(request, user, voc_type).read(voc.record_id))


@api_view()
def to_make_active_view(request, record_id, voc_type, body, *args, **kwargs):
    """Сделать запись справочника активной"""
    user: UserModel.Data = kwargs['user']
    # permissions
    gl_check_voc_update_permissions(request, user, voc_type, VocValueModel(request).read(record_id))
    # view
    if voc_type == 'organizations':
        org = OrganizationVocModel(request).read(record_id)
        if org.value_status not in (OrganizationVocModel.Data.ValueStatusesTypes.OUTDATED, OrganizationVocModel.Data.ValueStatusesTypes.DRAFT) or \
                org.valid_to:
            raise CoreBaseException(
                rus_message='Сделать активной можно запись организации, имеющую статус "Черновик" или "Архивирована", также '
                            'запись не должна иметь дату окончания действия.')
        org.value_status = OrganizationVocModel.Data.ValueStatusesTypes.ACTIVE
        OrganizationVocModel(request).update(org.record_id, org)
        return response(request, OrganizationsVocSerializer(request, user, voc_type).read(org.record_id))
    voc = VocValueModel(request).read(record_id)
    if voc.value_status not in (VocValueModel.Data.ValueStatusesTypes.OUTDATED, VocValueModel.Data.ValueStatusesTypes.DRAFT):
        raise CoreBaseException(rus_message='Сделать активной можно только архивированную запись справочника или черновик')
    voc.value_status = VocValueModel.Data.ValueStatusesTypes.ACTIVE
    VocValueModel(request).update(voc.record_id, voc)
    return response(request, VocValuesSerializer(request, user, voc_type).read(voc.record_id))


@api_view()
def change_hint_view(request, voc_type, attr, body, *args, **kwargs):
    """Изменить подсказку у ячейки справочника"""
    try:
        voc_repository = vocs_repository.get_voc_repository(voc_type)
    except VocRepositoryNotFoundException:
        raise CoreBaseException(message=f'voc repository not found with voc_type: {voc_type}')

    voc_attrs = tuple(item['field'] for item in voc_repository['model'])
    if attr not in voc_attrs:
        raise CoreBaseException(message=f'attr: {attr} not found in voc with voc_type: {voc_type}. '
                                        f'allowed attrs: {voc_attrs}')

    hints = HintModel(request).filter({
        'item_type': [HintModel.Data.ItemTypeTypes.VOC.value],
        'item_class': [voc_type],
    })

    if hints:
        hint = hints[0]
        hint.attrs[attr] = body['text']
        HintModel(request).update(hint.record_id, hint)
    else:
        hint_data = {
            'item_type': HintModel.Data.ItemTypeTypes.VOC.value,
            'item_class': voc_type,
            'attrs': {},
        }
        hint_data['attrs'][attr] = body['text']
        HintModel(request).save(HintModel.Data.from_dict(hint_data))

    return response(request, None)


@api_view()
def list_by_type_voc_view(request, voc_type, only_active, *args, **kwargs):
    """Получить записи справочника с voc_type для отображения в таблице"""
    # redirect
    if voc_type in ('bugreports', 'bugreports_admin'):
        return voc_list_bugreports_view(request, args, kwargs)
    if voc_type == 'users':
        return voc_list_users_view(request, args, kwargs)
    if voc_type == 'users_logs':
        return voc_list_users_logs_view(request, args, kwargs)
    # view
    voc_repo = vocs_repository.get_voc_repository(voc_type)
    voc_type_final = voc_repo['voc_type_db'] if 'voc_type_db' in voc_repo else  voc_repo['voc_type']
    if voc_type_final == 'organizations':
        return response(request, OrganizationsVocSerializer(request, kwargs['user'], voc_type).list(only_active=only_active), many=True)
    return response(
        request,
        VocValuesSerializer(request, kwargs['user'], voc_type).list(only_active=only_active),
        many=True
    )


@api_view()
def list_versions_voc_organizations_view(request, record_id, *args, **kwargs):  #
    """Получить список версий спарвочника орг. по kod_org"""
    return response(request, OrganizationsVocVersionsSerializer(request, kwargs['user'], record_id).list(), many=True)


@api_view()
def list_stock_list_view(request, form_no, date, kod_org, *args, **kwargs):
    """Получить список наличного количества по номеру формы, и дате"""
    try:
        org = OrganizationVocModel(request).filter({'KOD_ORG': kod_org})[0]
    except IndexError:
        raise CoreBaseException(rus_message='Организация с таким кодом не найдена')

    stock_list_data = services.get_stock_list(request, form_no, org.record_id, date, connection=request.storage['conn'])
    form_repository = forms_repository.get_form_repository(form_no)
    rep = {
        'types': types_repository.get_types_repository(),
        'form': {
            'table': {
                'headers': [],
                'model': []
            }
        }
    }

    if not stock_list_data:
        res = {
            'repository': rep,
            'form': {
                'form_table_data': {
                    'table': stock_list_data
                }
            }
        }
        return response(request, res)

    # Добавление доп данных модели 'module_type': 'snk'
    form_no_clear = forms_repository.get_clear_form_no(form_no)
    if form_no_clear in snk_repository:
        rep['form']['table']['model'].extend(snk_repository[form_no_clear]['extra_model'])
        rep['form']['table']['headers'].extend(m['field'] for m in snk_repository[form_no_clear]['extra_model'])

        # Переименование полей приходящих с api2 под формат репозитория формы
        rename_fields = {m['source']: m['field'] for m in snk_repository[form_no_clear]['extra_model']}
        for row in stock_list_data:
            for k, v in rename_fields.items():
                if k in row:
                    row[v] = row[k]
                    row.pop(k)

    keys_not_in_form_rep = []
    for k in stock_list_data[0]:
        if k in rep['form']['table']['headers']:
            continue
        found = False
        for item in form_repository['tables']['main_table']['model']:
            if k == item['field']:
                rep['form']['table']['model'].append(item)
                rep['form']['table']['headers'].append(item['field'])
                found = True
        if not found:
            keys_not_in_form_rep.append(k)

    for item in stock_list_data:
        for k in keys_not_in_form_rep:
            item.pop(k)

    res = {
        'repository': rep,
        'form': {
            'form_table_data': {
                'table': stock_list_data
            }
        }
    }
    return response(request, res)


@api_view()
def search_by_value_id_view(request, voc_type, value_id, only_active, limit, *args, **kwargs):
    """Поиск voc по voc_type и value_id (частичное совпадение, без учета регистра)"""
    filter_params = [
        'and',
        ['like', 'value_id', value_id],
        ['=', 'voc_type', voc_type],
    ]
    if only_active:
        filter_params.append(['=', 'value_status', VocValueModel.Data.ValueStatusesTypes.ACTIVE.value])
    vocs = VocValueModel(request).filter_raw(
        filter_params,
        fields_=['value_id'],
    )

    try:
        vocs.pop(limit)
        values_remain = True
    except IndexError:
        values_remain = False
    resp = {
        'values_remain': values_remain,
        'result': [voc['value_id'] for voc in vocs],
    }
    return response(request, resp, ignore_format=True)


# --- change table ---

@api_view()
def change_cells_voc_view(request, body, *args, **kwargs):
    # permissions
    for item in body['table_cells']:
        voc = VocValueModel(request).read(item['pointer']['record_id'])
        gl_check_voc_update_permissions(request, kwargs['user'], voc.voc_type, voc)
    # view
    for item in body['table_cells']:
        pointer = item['pointer']
        table_cell = item['table_cell']
        col_name = pointer['col_name']
        value = table_cell.get('value', '')

        voc = VocValueModel(request).read(pointer['record_id'])
        if voc.voc_type in ('organizations', 'users'):
            raise CoreBaseException(rus_message='Обновление данного справочника из таблицы не поддерживается')
        else:
            voc.attrs[col_name] = value
            VocValuesSerializer(request, kwargs['user'], voc.voc_type).update(voc.record_id, voc.as_dict())

    return response(request, None)


@api_view()
def get_extra_info_view(request, voc_type, *args, **kwargs):
    """Получить доп информацию"""
    resp = {
        'edit': {
            'create_update': services.calc_permissions_for_voc_edit(voc_type, kwargs['user']),
            'hint': bool_permission(lambda: check_edit_users_voc_permissions(kwargs['user'])),
        }
    }
    return response(request, resp)
