from services.exceptions import CoreBaseException


class ApiException(CoreBaseException):
    ...


class ActionNotAllowedException(ApiException):
    def __init__(self, *args: object, **kwargs) -> None:
        self.action = kwargs['action']
        self.method = kwargs['method']
        super().__init__(
            type='action_not_allowed',
            message=f'Action: {self.action} не поддерживается для метода: {self.method}. Попробуйте выбрать другой '
                    f'метод (GET или POST).',
        )


class ActionRequiredException(ApiException):
    def __init__(self, *args: object, **kwargs) -> None:
        super().__init__(
            type='action_required',
            message=f'action required',
        )


class ArgRequired(ApiException):
    def __init__(self, *args: object, **kwargs) -> None:
        self.arg = kwargs.get('arg')
        if len(self.arg) == 1:
            self.arg = self.arg[0]
        super().__init__(
            type='arg_required',
            message=f'arg required: {self.arg}',
        )


class BodyParamsRequired(ApiException):
    def __init__(self, *args: object, **kwargs) -> None:
        self.body_params = kwargs.get('body_params')
        self.required_structure = kwargs.get('required_structure')
        if len(self.body_params) == 1:
            self.body_params = self.body_params[0]
        super().__init__(
            type='body_params_required',
            message=f'body params required: {self.body_params}',
            required_structure=f'required_structure: {self.required_structure}'
        )


class BodyRequiredException(ApiException):
    def __init__(self, *args: object, **kwargs) -> None:
        super().__init__(
            type='request_body_required',
            message=f'request body required',
        )
