def init():
    return {
        'module_type': 'types',
        'types': {
            'organizations_voc_type': {
                'lazy_load_size': 50,
                'value_id': 'record_id',
                'base_type': 'VOC',
                # 'voc_subtype': 'settings',
                'label': '{NAME_SHORT} ({KOD_ORG})',
                'voc_name': 'Организации',
                'voc_type': 'organizations',
                'translate_value_id': True,
                'group': [
                    {
                        'type': 'details',
                        'fields': [
                            'PRZN_ORG', 'ORG_TYPE', 'PRZN_ACTIV', 'YROO',
                            'VIAC'
                        ]
                    },
                    {
                        'caption': 'Информация об организации',
                        'type': 'details',  # раздел - выпадаюший список
                        'fields': [
                            {
                                'caption': 'Общая информация',
                                'type': 'header',  # заголовок
                                'fields': [
                                    'KOD_GOL_ORG', 'KOD_ORG', 'KOD_IAE', 'KOD_SRF',
                                    'AuthorityName', 'AuthorityShortName', 'AuthorityAddress', 'AuthorityHeadName',
                                    'AuthorityPerformerName', 'AuthorityPhone', 'AuthorityFax', 'AuthorityEmail',
                                    'NAME_FULL', 'NAME_SHORT', 'ADDRESS_U', 'ADDRESS_P',

                                ]
                            },
                            {
                                'caption': 'Контакты',
                                'type': 'header',
                                'fields': [
                                    'RUKOVODITEL', 'PerformerName', 'TLFON', 'FAX', 'EMAIL',
                                ]
                            },
                            {
                                'caption': 'Коды',
                                'type': 'header',
                                'fields': [
                                    'INN', 'KPP', 'OKPO', 'OKVED', 'OKOGU', 'OKTMO', 'OKOPF', 'OKFS', 'OKPO_PRP'
                                ]
                            },
                            # {
                            #    'caption': 'Руководитель организации',
                            #    'type': 'header',
                            #    'fields': [
                            #        'DOLJN', 'fio', ,
                            #    ]
                            # }
                        ]
                    },
                    {
                        'caption': 'Служебная информация',
                        'type': 'details',
                        'fields': [
                            'DT_REG', 'DT_SU', 'PRZN_PRBL', 'PRIM', 'category1',
                            'category2', 'category3', 'category4', 'category5',
                        ]
                    }
                ],
                'headers': [
                    'KOD_ORG', 'KOD_GOL_ORG', 'NAME_FULL', 'NAME_SHORT', 'ORG_TYPE',
                    'YROO', 'KOD_IAE', 'KOD_SRF', 'INN', 'KPP', 'OKPO', 'OKVED', 'OKOGU', 'OKTMO',
                    'OKOPF', 'OKFS', 'OKPO_PRP', 'ADDRESS_U', 'ADDRESS_P', 'RUKOVODITEL', 'TLFON', 'DT_REG', 'DT_SU',
                    'status'
                ],
                'model': [
                    {
                        'field': 'KOD_ORG',
                        'caption': 'Регистрационный номер организации',
                        'type': 'str',
                        'hidden': True,
                        'style': "width: 550px;",
                        'show_if_enable_attrs': True,
                    },
                    {
                        # фиктивное поле, не хранится в явном виде в бд.
                        # отображает информацию на основе PRZN_OP и ORG_TYPE
                        'field': 'PRZN_ORG',
                        'caption': 'Признак организации',
                        'type': 'PRZN_ORG_type',
                        'style': "width: 550px;"
                    },
                    {
                        'field': 'YROO',
                        'caption': 'Организация ЯРОО',
                        'type': 'YROO_type',
                    },
                    {
                        'field': 'VIAC',
                        'caption': 'Тип ВИАЦ',
                        'show_if_enable_attrs': True,
                        'type': 'VIAC_type',
                    },
                    {
                        'field': 'ORG_TYPE',
                        'caption': 'Тип организации',
                        'type': 'ORG_TYPE_type',
                        'style': "width: 550px;",
                        'enable_attrs': {
                            '0': [
                                # ЮрЛицо
                                'KOD_ORG', 'KOD_IAE', 'KOD_SRF', 'ADDRESS_P',
                                'INN', 'KPP', 'OKPO', 'OKVED', 'OKOGU', 'OKTMO',
                                'OKOPF', 'OKFS', 'OKPO_PRP', 'DT_REG', 'DT_SU',
                                'PRZN_PRBL',
                            ],
                            '1': [
                                # Филиал
                                'KOD_GOL_ORG', 'KOD_ORG', 'KOD_IAE', 'KOD_SRF',
                                'ADDRESS_P', 'INN', 'KPP', 'OKPO', 'OKVED', 'OKOGU',
                                'OKTMO', 'OKOPF', 'OKFS', 'OKPO_PRP', 'DT_REG', 'DT_SU',
                                'PRZN_PRBL',
                            ],
                            '2': [
                                # РИАЦ
                                'KOD_SRF', 'AuthorityName', 'AuthorityShortName', 'AuthorityAddress',
                                'AuthorityHeadName', 'AuthorityPerformerName', 'AuthorityPhone',
                                'AuthorityFax', 'AuthorityEmail', 'PerformerName',
                            ],
                            '3': [
                                # ВИАЦ
                                'VIAC', 'ADDRESS_P', 'PerformerName',
                            ],
                            '4': [
                                # ЦИАЦ
                                'KOD_SRF', 'AuthorityName', 'AuthorityShortName', 'AuthorityAddress',
                                'AuthorityHeadName', 'AuthorityPerformerName', 'AuthorityPhone',
                                'AuthorityFax', 'AuthorityEmail', 'PerformerName',
                            ]
                        },
                    },
                    {
                        'field': 'PRZN_ACTIV',
                        'caption': 'Признак вывода в список',
                        # 'help':'Тест'
                        'type': 'PRZN_ACTIV_type',
                        'style': "width: 550px;"
                    },
                    {
                        'field': 'KOD_GOL_ORG',
                        'caption': 'Код головной организации',
                        'style': "width: 550px;",
                        'show_if_enable_attrs': True,
                        'type': 'organizations_voc_type'
                    },
                    {
                        'field': 'KOD_IAE',
                        'caption': 'Код органа управления использованием атомной энергии',
                        'style': "width: 550px;",
                        'show_if_enable_attrs': True,
                        'type': '114_label_2_type',
                    },
                    {
                        'field': 'KOD_SRF',
                        'caption': 'Субъект Российской Федерации',
                        'style': "width: 550px;",
                        'show_if_enable_attrs': True,
                        'type': '115_type'
                    },
                    {
                        'field': 'AuthorityName',
                        'caption': 'Наименование органа власти субъекта Российской федерации',
                        'show_if_enable_attrs': True,
                    },
                    {
                        'field': 'AuthorityShortName',
                        'caption': 'Сокращённое наименование органа исполнительной власти субъекта Российской федерации',
                        'show_if_enable_attrs': True,
                    },
                    {
                        'field': 'AuthorityAddress',
                        'caption': 'Место нахождения органа управления',
                        'show_if_enable_attrs': True,
                    },
                    {
                        'field': 'AuthorityHeadName',
                        'caption': 'Должность, фамилия, имя, отчество руководителя органа управления',
                        'show_if_enable_attrs': True,
                    },
                    {
                        'field': 'AuthorityPerformerName',
                        'caption': 'Должность, фамилия, имя, отчество ответственного исполнителя органа управления',
                        'show_if_enable_attrs': True,
                    },
                    {
                        'field': 'AuthorityPhone',
                        'caption': 'Телефон органа исполнительной власти',
                        'show_if_enable_attrs': True,
                    },
                    {
                        'field': 'AuthorityFax',
                        'caption': 'Факс органа исполнительной власти',
                        'show_if_enable_attrs': True,
                    },
                    {
                        'field': 'AuthorityEmail',
                        'caption': 'Электронная почта органа исполнительной власти',
                        'show_if_enable_attrs': True,
                    },
                    {
                        'field': 'NAME_FULL',
                        'caption': 'Полное наименование организации',
                        'style': "width: 550px;",
                    },
                    {
                        'field': 'NAME_SHORT',
                        'caption': 'Краткое наименование организации',
                        'style': "width: 550px;",
                    },
                    {
                        'field': 'ADDRESS_U',
                        'caption': 'Юридический адрес организации',
                        'style': "width: 550px;",
                        # 'type': 'address_json'
                    },
                    {
                        'field': 'ADDRESS_P',
                        'caption': 'Фактический адрес организации',
                        'style': "width: 550px;",
                        'show_if_enable_attrs': True,
                        # 'type': 'address_json'
                    },
                    {
                        'field': 'TLFON',
                        'caption': 'Телефон',
                        'type': 'phone',
                        'style': "width: 284px;",
                    },
                    {
                        'field': 'FAX',
                        'caption': 'Факс',
                        'style': "width: 284px;"
                    },
                    {
                        'field': 'EMAIL',
                        'caption': 'Электронная почта',
                        'type': 'email',
                        'style': "width: 284px;"
                    },
                    {
                        'field': 'INN',
                        'caption': 'ИНН',
                        'style': "width: 284px;",
                        'show_if_enable_attrs': True,
                        'type': 'INN_t',
                    },
                    {
                        'field': 'KPP',
                        'caption': 'КПП',
                        'style': "width: 284px;",
                        'show_if_enable_attrs': True,
                        'type': 'KPP_t',
                    },
                    {
                        'field': 'OKPO',
                        'caption': 'ОКПО',
                        'type': 'OKPO_type',
                        'style': "width: 284px;",
                        'show_if_enable_attrs': True,
                        'type': 'OKPO_t',
                    },
                    {
                        'field': 'OKVED',
                        'caption': 'ОКВЭД',
                        'style': "width: 284px;",
                        'show_if_enable_attrs': True,
                    },
                    {
                        'field': 'OKOGU',
                        'caption': 'ОКОГУ',
                        'style': "width: 284px;",
                        'show_if_enable_attrs': True,
                        'type': '116_type'
                    },
                    {
                        'field': 'OKTMO',
                        'caption': 'ОКТМО',
                        'style': "width: 284px;",
                        'show_if_enable_attrs': True,
                        'type': 'OKTMO_t',
                    },
                    {
                        'field': 'OKOPF',
                        'caption': 'ОКОПФ',
                        'style': "width: 284px;",
                        'show_if_enable_attrs': True,
                        'type': '117_type'
                    },
                    {
                        'field': 'OKFS',
                        'caption': 'ОКФС',
                        'style': "width: 284px;",
                        'show_if_enable_attrs': True,
                        'type': '118_type'
                    },
                    {
                        'field': 'OKPO_PRP',
                        'caption': 'ОКПО правопреемника',
                        'type': 'OKPO_type',
                        'style': "width: 284px;",
                        'show_if_enable_attrs': True,
                    },
                    {
                        'field': 'RUKOVODITEL',
                        'caption': 'Должность, фамилия, имя, отчество (при наличии) руководителя',
                        'type': 'str'
                    },
                    {
                        'field': 'PerformerName',
                        'caption': 'Ответственный исполнитель',
                        'show_if_enable_attrs': True,
                    },
                    {
                        'field': 'DT_REG',
                        'caption': 'дата регистрации',
                        'type': 'date',
                        'show_if_enable_attrs': True,
                    },
                    {
                        'field': 'DT_SU',
                        'caption': 'дата снятия с учёта',
                        'type': 'date',
                        'show_if_enable_attrs': True,
                    },
                    # {
                    #    'field': 'PRZN_PRBL',
                    #    'caption': 'Признак проблемности',
                    #    'show_if_enable_attrs': True,
                    # },
                    {
                        'field': 'PRIM',
                        'caption': 'Примечания'
                    },
                    {
                        'field': 'category1',
                        'caption': 'Служебное поле 1'  # Аналитическая категория №1
                    },
                    {
                        'field': 'category2',
                        'caption': 'Служебное поле 2'
                        # Аналитическая категория №2
                    },
                    {
                        'field': 'category3',
                        'caption': 'Служебное поле 3'
                        # Аналитическая категория №3
                    },
                    {
                        'field': 'category4',
                        'caption': 'Служебное поле 4'
                        # Аналитическая категория №4
                    },
                    {
                        'field': 'category5',
                        'caption': 'Служебное поле 5'
                        # Аналитическая категория №5
                    },
                    {
                        'field': 'status',
                        'caption': 'Статус',
                        'type': 'voc_status_type',
                        'readonly': True,
                        'hide_selection': True,
                        'filter': ['=']
                    }
                    # {
                    #     'field': 'PRZN_OP',
                    #     'caption': 'Признак обособленного подразделения'
                    # },
                ]
            },
            'organizations_voc_type_value_id_label': {
                'override_type': 'organizations_voc_type',
                'voc_type': 'organizations_voc_type_return_label_as_value_id',
                'voc_type_db': 'organizations',
                'translate_value_id': False,
                'value_id_return': 'label',
                'voc': {
                    'config_url': {
                        'params': {
                            'only_active': False,
                        }
                    }
                }
            },
            'organizations_voc_type_value_id_KOD_ORG': {
                'override_type': 'organizations_voc_type',
                'voc_type': 'organizations_voc_value_id_KOD_ORG',
                'voc_type_db': 'organizations',
                'translate_value_id': False,
                'value_id_return': 'KOD_ORG',
            },
            #DICT (org)    
            'PRZN_ORG_type': {
                'base_type': 'DICT',
                'data_source': {
                    '4': 'Действующая',
                    '1': 'Юр. лицо без объектов учета',
                    '2': 'Снята с учета',
                    '3': 'Разовая организация'
                }
            },
            'PRZN_OP_type': {
                'base_type': 'DICT',
                'data_source': {
                    '0': 'Юридическое лицо',
                    '1': 'Филиал, представительство, обособленное подразделение'
                }
            },
            'ORG_TYPE_type': {
                'base_type': 'DICT',
                'data_source': {
                    '0': 'Юридическое лицо',
                    '1': 'Филиал, представительство, обособленное подразделение',
                    '2': 'РИАЦ',
                    '3': 'ВИАЦ',
                    '4': 'ЦИАЦ',
                },
                'default': '0'
            },
            'PRZN_ACTIV_type': {
                'base_type': 'DICT',
                'data_source': {
                    '0': 'Не выводить в списке организаций',
                    '1': 'Выводить в списке организаций'
                }
            },
            'LVL_ORG_t': {
                'base_type': 'DICT',
                'data_source': {
                    '0': 'Юр. лицо',
                    '1': 'Филиал',
                    '2': 'Подфилиал'
                }
            },
            #STR (org)
            'INN_t': {
                'base_type': 'BASE',
                'validation': {
                    'type': 'str',
                    'regex': r'^\d{10}$'
                },
                'help': ''
            },
            'KPP_t': {
                'base_type': 'BASE',
                'validation': {
                    'type': 'str',
                    'regex': r'^\d{9}$'
                },
                'help': ''
            },
            'OKPO_t': {
                'base_type': 'BASE',
                'validation': {
                    'type': 'str',
                    'regex': r'^(\d{8}|\d{10}|\d{14})$'
                },
                'help': ''
            },
            'OKTMO_t': {
                'base_type': 'BASE',
                'validation': {
                    'type': 'str',
                    'regex': r'^(\d{8}|\d{11})$'
                },
                'help': ''
            },
            'YROO_type': {
                'base_type': 'DICT',
                'data_source': {
                    False: 'нет',
                    True: 'да',
                }
            },
            'VIAC_type': {
                'base_type': 'DICT',
                'data_source': {
                    '0': 'Федеральный орган исполнительной власти',
                    '1': 'ВИАЦ Госкорпорации «Росатом»',
                    '2': 'ВИАЦ Министерства обороны Российской Федерации',
                }
            }
        }
    }
