/* Динамически подгружаемый компонент, получает интерфейс к API*/

export default {
  inject: ["api"],
  template: `
    <div class="default-page-padding-inline app-font-default"
         style="display: flex; flex-direction: column; gap: 10px;">
    <h3 class="app-h-s22-h24-w7-c333">Экспорт в АЧ</h3>
    <div style="display: flex; gap: 10px; flex-wrap: wrap;">
      <button-component
        class="form-button"
        @click="executeExport"
      >
        Ссылка на обменный файл АЧ
      </button-component>
      <button-component
        class="form-button"
        @click="onExportState"
      >
        Пометить формы как выгруженные
      </button-component>
    </div>
    <template v-if="exportData">
      <table class="unload-table" style="padding-bottom: 10px;">
        <thead>
        <tr class="table__header">
          <th class="table__th">Автор</th>
          <th class="table__th">Форма</th>
          <th class="table__th">Период с / по</th>
          <th class="table__th">№ корректировки</th>
          <th class="table__th">Статус</th>
        </tr>
        </thead>
        <tbody>
        <tr
          class="table__tr"
          :key="item.form_no + i"
          v-for="(item, i) of exportData.forms"
        >
          <td class="table__td">{{ item.author_id }}</td>
          <td class="table__td">{{ item.form_no }}</td>
          <td class="table__td">{{ item.period }}</td>
          <td class="table__td">{{ item.adjustment_no }}</td>
          <td class="table__td">{{ item.form_state }}</td>
        </tr>
        </tbody>
      </table>
      <template v-if="exportData.notify.length">
        <div class="app-h-s18-h24-w7-c333">Напоминания:</div>
        <ul style="margin-top: 0px;">
          <li :key="notify" v-for="notify of exportData.notify">{{ notify }}</li>
        </ul>
      </template>
    </template>
    </div>
    <component :is="'style'">
    {{ css }}
    </component>
  `,
  props: {
    route: Object,
  },
  setup() {
    return {
      css: `
      .unload-table {
        border-spacing: 0;
      }
      
      .unload-table .table__td {
        text-align: center;
      }
      `,
    };
  },
  data() {
    return {
      exportData: null,
    };
  },
  computed: {
    forms() {
      return this.toArray(this.route.query.forms);
    },

    exportFileUrl() {
      return this.api.lib.ApiService.getUrl("/api/exchange_ap/export_file?", {
        forms: this.forms,
      });
    },
  },
  methods: {
    toArray(value) {
      if (value === undefined) {
        return [];
      }

      return Array.isArray(value) ? value : [value];
    },

    getExportData(forms) {
      const url = this.api.lib.ApiService.getUrl(
        "/api/exchange_ap/export_data?",
        { forms },
      );
      return this.api.lib.HttpService.get(url).then((x) => x.json);
    },

    async onExportState() {
      const url = this.api.lib.ApiService.getUrl(
        "/api/exchange_ap/export_state?",
        { forms: this.forms },
      );
      const result = await this.api.lib.HttpService.get(url).then(
        (x) => x.json,
      );
      this.api.lib.NotificationsService.send({
        title: "Результат",
        text: result,
        type: "info",
      });
    },

    async executeExport() {
      await this.api.frontend.ApiActionsService.executeExec({
        url: this.exportFileUrl,
        caption: "Ссылка на обменный файл АЧ",
      });
    },
  },
  async created() {
    this.exportData = await this.getExportData(this.forms);
  },
  async mounted() {},
};
