/* Динамически подгружаемый компонент, получает интерфейс к API*/

export default {
  inject: ["api", "api2"],
  props: ["context", "form", "route"],
  template: `
    <div style="display: flex; flex-direction: column; gap: 10px; padding: 20px;">
    <editing-form
      :current-data="currentData"
      :default-open-headers="true"
      :read-only="false"
      :voc-extra-info="vocExtraInfo"
      :voc-repository-dto="vocRepositoryDto"
      unique-id="template-form"
      voc-type="template-form"
      @reset="onResetField"
      @change-field="onChangeField"
    />
    <button-component class="form-button" @click="onCommit">Сохранить</button-component>
    </div>
  `,

  data() {
    return {
      currentData: {},
      initialData: {},
      vocExtraInfo: {
        edit: {
          create_update: true,
          hint: true,
        },
      },
      vocRepositoryDto: {
        types: {
          date: {
            base_type: "DATE",
          },
          "11_voc": {
            base_type: "VOC",
            voc_type: "11_voc",
            selection_mode: "single",
            lazy_load_size: 100,
          },
          CalcCodeRao: {
            base_type: "CALC",
            calc_type: "CalcCodeRao",
          },
          dict: {
            base_type: "DICT",
            data_source: {
              1: "1",
              2: "2",
            },
          },
          list: {
            base_type: "LIST",
            data_source: ["1", "2"],
          },
          exponential: {
            base_type: "BASE",
            validation: {
              handlers: ["exponential"],
            },
            handlers: {
              before_save: ["exponential"],
              before_label: [
                {
                  handler: "exponential",
                  params: 2,
                },
              ],
            },
          },
        },
        form: {
          table: {
            model: [
              {
                field: "Op",
                caption: "Код операции (выбор из справочника)",
                style: "width: 284px;",
                type: "11_voc",
              },
              {
                field: "OpDate",
                caption: "Дата операции",
                style: "width: 284px;",
                type: "date",
              },
              {
                field: "Subdiv",
                caption: "Код РАО",
                style: "width: 284px;",
                type: "CalcCodeRao",
                hide_reset: true,
              },
              {
                field: "Sector",
                caption: "Участок (readonly)",
                style: "width: 284px;",
                readonly: true,
              },
              {
                field: "exponential",
                caption: "Экспоненциальная форма",
                style: "width: 284px;",
                type: "exponential",
              },
              {
                field: "DocVid",
                caption: "Вид (словарь)",
                style: "width: 284px;",
                type: "dict",
              },
              {
                field: "DocN",
                caption: "Номер (список)",
                style: "width: 284px;",
                type: "list",
              },
              {
                field: "DocDate",
                caption: "Дата",
                style: "width: 284px;",
                type: "date",
              },
              {
                field: "DocFile",
                caption: "Файл",
                style: "width: 284px;",
              },
              {
                field: "Plant",
                caption: "Установка переработки",
                style: "width: 284px;",
              },
              {
                field: "DescriptionCodPlant",
                caption: "Примечание",
                height: 300,
                style: "resize: vertical !important; width: 300px",
                hide_reset: true,
              },
              {
                field: "UktTyp",
                caption: "Тип",
                style: "width: 284px;",
              },
              {
                field: "UktNumb",
                caption: "Номер",
                style: "width: 284px;",
              },
              {
                field: "UktName",
                caption: "Наименование",
                style: "width: 284px;",
              },
              {
                field: "NumbYE",
                caption: "Номер учётной единицы",
                style: "width: 284px;",
              },
              {
                field: "PaspN",
                caption: "№ акта/паспорта",
                style: "width: 284px;",
              },
              {
                field: "Mass",
                caption: "Масса, т",
                style: "width: 284px;",
              },
              {
                field: "Kbm",
                caption: "Объём, м3",
                style: "width: 284px;",
              },
              {
                field: "NameLockDevice",
                caption: "NameLockDevice",
                style: "width: 284px;",
              },
              {
                field: "NumbLockDevice2",
                caption: "NumbLockDevice",
                style: "width: 284px;",
              },

              {
                field: "FullNamePerformer",
                caption: "ФИО",
                style: "width: 284px;",
              },
              {
                field: "SignaturePeformer",
                caption: "Подпись",
                style: "width: 284px;",
              },
              {
                field: "SignatureDate",
                caption: "Дата",
                style: "width: 284px;",
              },
            ],
          },
        },
        edit: {
          edit: true,
        },
        translate_value_id: true,
        lazy_load_size: 50,
        group: [
          {
            type: "details",
            fields: ["Op", "OpDate"],
          },
          {
            caption: "Сведения об операции:",
            type: "details",
            fields: [
              {
                caption: "Описание операции:",
                type: "header",
                fields: ["exponential"],
                attrsStyle:
                  "display: block; width: max-content; padding: 10px; border: 2px solid black; border-radius: 3px;",
              },
              {
                caption: "Место образования:",
                type: "header",
                fields: ["Subdiv", "Sector"],
                attrsStyle:
                  "display: block; width: max-content; padding: 10px; border: 2px solid black; border-radius: 3px;",
                captionStyle: "font-weight: bold;",
              },
              {
                caption: "Документ:",
                type: "header",
                fields: ["DocVid", "DocN", "DocDate", "DocFile"],
                attrsStyle:
                  "display: block; width: max-content; padding: 10px; border: 2px solid black; border-radius: 3px;",
              },
              {
                caption: "Сведения о переработке/сортировке:",
                type: "header",
                fields: ["Plant", "DescriptionCodPlant"],
                attrsStyle:
                  "display: block; width: max-content; padding: 10px; border: 2px solid black; border-radius: 3px;",
              },
            ],
          },
          {
            caption: "Сведения о РАО:",
            type: "details",
            fields: [
              {
                caption: "Сведения об упаковке:",
                type: "header",
                fields: ["UktTyp", "UktNumb", "UktName"],
                attrsStyle:
                  "display: block; width: max-content; padding: 10px; border: 2px solid black; border-radius: 3px;",
              },
              {
                caption: "Сведения о РАО:",
                type: "header",
                fields: ["NumbYE", "PaspN", "Mass", "Kbm"],
                attrsStyle:
                  "display: block; width: max-content; padding: 10px; border: 2px solid black; border-radius: 3px;",
              },
              {
                caption: "Пломбировочное устройство:",
                type: "header",
                fields: ["NameLockDevice", "NumbLockDevice"],
                attrsStyle:
                  "display: block; width: max-content; padding: 10px; border: 2px solid black; border-radius: 3px;",
              },
              {
                caption: "Исполнитель операции:",
                type: "header",
                fields: [
                  "FullNamePerformer",
                  "SignaturePeformer",
                  "SignatureDate",
                ],
                attrsStyle:
                  "display: block; width: max-content; padding: 10px; border: 2px solid black; border-radius: 3px;",
              },
            ],
          },
        ],
      },
    };
  },

  methods: {
    onCommit() {
      this.api.modal.commit(this, this.currentData);
    },

    onClose() {
      this.api.modal.close(this);
    },

    onResetField(model) {
      this.currentData[model.field] = this.initialData[model.field];
    },

    onChangeField(event) {
      this.currentData[event.model.field] = event.value;
    },
  },

  async mounted() {
    this.form.set_caption("Форма образования. Операции: 11, 56, 59");
  },
};
