/* Динамически подгружаемый компонент, получает интерфейс к API*/

export default {
  inject: ["api"],
  template: `
    <div class="default-page-container table-scroll-container" style="gap: 10px; height: 100%;">
    <div class="default-page-padding-inline" style="display: flex; gap: 10px; flex-wrap: wrap;">
      <switch-component
        :key="emitName"
        v-for="emitName of logEmitNames"
        style="width: 200px;"
        :checked="logEmitNamesSelected.has(emitName)"
        :label="emitName"
        @click="onLogChange(emitName)"
      />
      <switch-component
        style="width: 200px;"
        :checked="multiselect"
        label="multiselect"
        @click="multiselect = !multiselect"
      />
      <switch-component
        style="width: 200px;"
        :checked="numCol"
        label="num-col"
        @click="numCol = !numCol"
      />
      <switch-component
        style="width: 200px;"
        :checked="numRow"
        label="num-row"
        @click="numRow = !numRow"
      />
      <button-component class="button-blue" @click="saveSettings()">
        Сохранить
      </button-component>
    </div>
    <t-table
      :model="model"
      :headers="headers"
      :multiselect="multiselect"
      :num-col="numCol"
      :num-row="numRow"
      v-model:data="data"
      @update:data="logEmit('update:data', $event)"
      @update:data:event="logEmit('update:data:event', $event)"
      @row:update="logEmit('row:update', $event)"
      @row:create="logEmit('row:create', $event)"
      @row:delete="logEmit('row:delete', $event)"
      @row:check="logEmit('row:check', $event)"
      @row:click="logEmit('row:click', $event)"
      @row:dblclick="logEmit('row:dblclick', $event)"
    />
    </div>
  `,
  props: {
    route: Object,
  },
  setup() {
    return {
      logEmitNames: new Set([
        "update:data",
        "update:data:event",
        "row:update",
        "row:create",
        "row:delete",
        "row:check",
        "row:click",
        "row:dblclick",
        "row:keydown",
      ]),
    };
  },
  data() {
    return {
      logEmitNamesSelected: new Set(this.logEmitNames),
      multiselect: true,
      numCol: true,
      numRow: true,
      inputText: "",
      model: {
        test: { caption: "Test" },
        test2: { caption: "Test 2" },
      },
      data: [{ test: 123, test2: "abc" }],
      headers: ["test", "test2"],
    };
  },
  created() {
    this.loadSettings();
  },
  methods: {
    loadSettings() {
      try {
        const settings = JSON.parse(localStorage.getItem("test-t-table"));
        this.setSettings(settings);
      } catch (e) {}
    },
    saveSettings() {
      localStorage.setItem(
        "test-t-table",
        JSON.stringify(this.getCurrentSettings()),
      );
    },
    getCurrentSettings() {
      return {
        logEmitNamesSelectedArray: Array.from(this.logEmitNamesSelected),
        multiselect: this.multiselect,
        numCol: this.numCol,
        numRow: this.numRow,
      };
    },
    setSettings(settings) {
      this.logEmitNamesSelected = new Set(
        settings.logEmitNamesSelectedArray || [],
      );
      this.multiselect = settings.multiselect;
      this.numCol = settings.numCol;
      this.numRow = settings.numRow;
    },
    onLogChange(emitName) {
      this.logEmitNamesSelected.has(emitName)
        ? this.logEmitNamesSelected.delete(emitName)
        : this.logEmitNamesSelected.add(emitName);
    },
    logEmit(name, event) {
      if (!this.logEmitNamesSelected.has(name)) return;
      this.api.lib.Logger.log(`TTable ${name}:`, event);
    },
  },
};
