# Действие для Отчетной формы

def init():
    return [
        {
            'caption'   : 'Загрузить операции',
            'method'    : 'load_opers',
            'forms'     : ['form_6_0'],
            'params': [
                # {
                #     'field'         : 'date_from',
                #     'caption'       : 'Начало периода',
                #     'type'          : 'date',
                #     'required'      : True
                # },
                # {
                #     'field'         : 'date_to',
                #     'caption'       : 'Конец периода',
                #     'type'          : 'date',
                #     'required'      : True
                # },
                # { 
                #     'field'         : 'opers',   
                #     'caption'       : 'Коды операций',  
                #     'type'          : '11_type',    
                #     'required'      : True,
                #     'multiselect'   : True 
                # }
            ]
        },
        {
            'caption'   : 'Расчитать',
            'method'    : 'calc_forecast',
            'forms'     : ['form_6_0']
        }
    ]
    

def exec(method, params):       # Стандартный маршрутизатор вызовов
    '''
    method  - название метода из актрибута method
    params  - словарь параметров и контекстных значений
                параметры описаны в репозитарии, в разделе params
                контекстные параметры определяются на frontend:
                    - report_form_id    - id редактируемой формы
                    - row               - номер выбранной строки в табличной части
    '''
    if method == 'load_opers':
        load_opers(params)

    elif method == 'calc_forecast':
        calc_forecast(params)

    else:
        raise Exception(f'Неизвестный метод {method}')

#======================================================================================
#   Реализация расширения
#======================================================================================
import sys
import os

sys.path.insert(1, os.path.join(sys.path[0], '..'))    

import pandas as pd
import rvrao


def load_opers(params):         # Загрузка операций
    form_id = params['report_form_id']
    if type(form_id) == str:
        form_id = int(form_id)

    # Загрузка данных формы
    form = rvrao.rf_read(form_id)

    # date_from = params.get('date_from') or form.get('form_period_start') or '2021-01-01'
    # date_to   = params.get('date_to')   or form.get('form_period_end') or '2025-01-01'
    # op_codes  = params.get('opers')     or form.get('opers') or '1,10,22,44'
    date_from = form.get('form_period_start') or '2021-01-01'
    date_to   = form.get('form_period_end') or '2025-01-01'
    op_codes  = form.get('op_codes') or '1,10,22,44'
    op_codes  = op_codes.replace(' ', '')

    ar_16 = load_ar16(date_from, date_to, op_codes)
    ar_17 = load_ar17(date_from, date_to, op_codes)
    ar_18 = load_ar18(date_from, date_to, op_codes)

    # Объединение загруженных операций
    opers = ar_16.append(ar_17).append(ar_18)

    # Заполнение пустых ячеек
    opers.Kbm.fillna(0, inplace=True)
    opers.Tonne.fillna(0, inplace=True)
    opers.Sht.fillna(0, inplace=True)
    opers.ActTR.fillna(0, inplace=True)
    opers.ActBG.fillna(0, inplace=True)
    opers.ActA.fillna(0, inplace=True)
    opers.ActUR.fillna(0, inplace=True)

    # Сохранение табличных данных
    rvrao.rf_table_update(form_id, opers, 'main_table')


ar_16_columns = [
    "OpCod", "OpDate", "RAOCod", "BCod", "Kbm", "Tonne", "Sht", "RAOCodMax",
    "ActTR", "ActBG", "ActA", "ActUR",
    "DocN", "UpCod", "UktPrN", "PH_Name", "PH_Cod",
    "Subsid", "FCP"
 ] 
    #
def load_ar16(date_from, date_to, op_codes):    # Загрузка операций из регистра формы 1.6
    sql = f'''
        SELECT *
        FROM "AccRAO"
        WHERE op_date BETWEEN '{date_from}' AND '{date_to}'
            AND position("OpCod" IN '{op_codes}')>0
    '''
    df = rvrao.sql_dataframe(sql)    
    rvrao.drop_unused_columns(df, ar_16_columns)
    
    if 'OpDate' in df.columns.values:
        df.OpDate = pd.to_datetime(df.OpDate).dt.strftime('%Y-%m-%d')

    return df
    #
    #
ar_17_columns = {
    'g2':'OpCod', 
    'g3':'OpDate',
    'g21': 'RAOCod',
    'g22': 'BCod',
    'g23': 'Kbm',
    'g24': 'Tonne',
    'g12': 'RAOCodMax',
    'g25': 'Sht',
    'g26': 'ActTR',
    'g27': 'ActBG',
    'g28': 'ActA',
    'g29': 'ActUR',
    'g15': 'DocN',
    'g30': 'UpCod',
    'g7' : 'UktPrN',
    'g19': 'PH_Name',
    'g20': 'PH_Cod',
    'g31': 'Subsid',
    'g32': 'FCP'
 }
    #
    #
def load_ar17(date_from, date_to, op_codes):    # Загрузка операций из регистра формы 1.6
    sql = f'''
        SELECT *
        FROM "AccO7"
        WHERE g3 BETWEEN '{date_from}' AND '{date_to}'
            AND position(g2 IN '{op_codes}')>0
    '''
    df = rvrao.sql_dataframe(sql)    
    rvrao.drop_unused_columns(df, list(ar_17_columns.keys()))
    df = df.rename(columns=ar_17_columns)

    if 'OpDate' in df.columns.values:
        df.OpDate = pd.to_datetime(df.OpDate).dt.strftime('%Y-%m-%d')

    return df
    #
    #
ar_18_columns = {
    'g2':'OpCod', 
    'g3':'OpDate',
    'g18': 'RAOCod',
    'g19': 'BCod',
    'g20': 'Kbm',
    'g21': 'Tonne',
    'g9': 'RAOCodMax',
    'g22': 'ActTR',
    'g23': 'ActBG',
    'g24': 'ActA',
    'g25': 'ActUR',
    'g12': 'DocN',
    'g26': 'UpCod',
    'g5' : 'UktPrN',
    'g16': 'PH_Name',
    'g17': 'PH_Cod',
    'g27': 'Subsid',
    'g28': 'FCP'
 }
    #
    #
def load_ar18(date_from, date_to, op_codes):    # Загрузка операций из регистра формы 1.6
    sql = f'''
        SELECT *
        FROM "AccO8"
        WHERE g3 BETWEEN '{date_from}' AND '{date_to}'
            AND position(g2 IN '{op_codes}')>0
    '''
    df = rvrao.sql_dataframe(sql)    
    rvrao.drop_unused_columns(df, list(ar_18_columns.keys()))
    df = df.rename(columns=ar_18_columns)
    if 'OpDate' in df.columns.values:
        df.OpDate = pd.to_datetime(df.OpDate).dt.strftime('%Y-%m-%d')

    return df


def calc_forecast(params):      # Расчет вычисляемых колонок
    form_id = params['report_form_id']

    # Загрузка справочников
    voc_op = rvrao.voc_values('11_voc')         # Справочник операция - только для теста

    # Загрузка табличных данных формы
    table = rvrao.rf_table_read(form_id)

    # Добавление вычисляемых колонок
    table['op_desc'] = ''
    table['volume_mult'] = 0.0
    table['fc_volume'] = 0.0
    table['fc_kont_qty'] = 0
    table['fc_kont_volume'] = 0.0

    # Вычисление колонок
    for idx in table.index.values:
        # Загрузка аргументов
        op_cod = str(table.loc[idx, 'OpCod'])
        kbm = table.loc[idx, 'Kbm']

        # Поиск значений в справочниках
        op_desc = voc_op.find_rec({'Op_Cod':op_cod}, 'Op_Name')
        kont_volume_net = 2     # Объем РАО в контейнере (выбрать из справочника)
        kont_volume_gross = 2.5 # Объем контейнерf (выбрать из справочника)

        volume_mult = 0.8
        fc_volume = kbm * volume_mult
        fc_kont_qty = int(fc_volume / kont_volume_net + 0.99)
        fc_kont_volume = fc_kont_qty * kont_volume_gross

        # Сохранение вычисленных значений
        table.loc[idx, 'op_desc'] = op_desc
        table.loc[idx, 'volume_mult'] = volume_mult
        table.loc[idx, 'fc_volume'] = fc_volume
        table.loc[idx, 'fc_kont_qty'] = fc_kont_qty
        table.loc[idx, 'fc_kont_volume'] = fc_kont_volume


    # Сохранение табличных данных
    rvrao.rf_table_update(form_id, table, 'main_table')

    # Отображение табличных данных в регистр AccRaoForecast
    rvrao.sql_rows(f'SELECT fc_pub({form_id})', commit=True)



