def init( context=None, api=None ):
    model = {
        'name' : '01. Отчет о наличном количестве',
        'module_name' : 'ar_01',
        'params': [
            { 'field' : 'op_date',    'caption' : 'Дата отчета',  'type' : 'date',    'required' : True },
            { 'field' : 'form_no',    'caption' : 'Отчетная форма',
                'type' : {
                    'form_1_1' : 'Форма 1.1',
                    'form_1_2' : 'Форма 1.2',
                    'form_1_3' : 'Форма 1.3',
                    'form_1_4' : 'Форма 1.4',
                    'form_1_5' : 'Форма 1.5',
                },
                'required' : True
            },
            { 'field' : 'org_id',     'caption' : 'Организация',  'type' : 'organizations_voc_type', }
        ],
        'table' : {
            'model' : [
                { 'field' : 'org_id',       'caption' : 'Организация',      'type' : 'organizations_voc_type', },
                { 'field' : 'form_no',      'caption' : 'Номер формы'},

                { 'field' : 'Typ',          'caption' : 'Тип источника'},
                { 'field' : 'PrName',       'caption' : 'Название прибора'},
                { 'field' : 'Arg',          'caption' : 'Агрегатное состояние'},
                { 'field' : 'Numb',         'caption' : 'Номер ЗРИ'},
                { 'field' : 'PaspN',        'caption' : 'Номер паспорта'},
                { 'field' : 'IzgotDate',    'caption' : 'Дата изготовления', 'type': 'date'},


                { 'field' : 'end_amount',   'caption' : 'Кол-во в наличии'},
                { 'field' : 'end_mass_net', 'caption' : 'Масса в наличии'},
                { 'field' : 'end_volume',   'caption' : 'Объем в наличии'},

                { 'field' : 'inv_date',     'caption' : 'Дата инвенторизации', 'type': 'date'},

                { 'field' : 'inv_amount',   'caption' : 'Масса по инверторизации'},
                { 'field' : 'flow_amount',  'caption' : 'Изменение массы'},

                { 'field' : 'inv_mass_net', 'caption' : 'Масса по инверторизации'},
                { 'field' : 'flow_mass_net','caption' : 'Изменение массы'},
            ]
        }
    }
    api.add('repo.AReports.ar_01', {'model':model, 'build':build}, node_type='rvrao-ext')
    return model


import sys
import os
import pandas as pd

sys.path.insert(1, os.path.join(sys.path[0], '..'))    

rvrao = None        # Ссылка на модуль rvrao.py

def exec(params, context=None, api=None ):   # Интерфейс вызова
    # Загрузка параметров
    op_date = params.get('op_date')
    forms = params.get('form_no')
    org_id = params.get('org_id')

    # Приведение параметров к требуемому виду
    if not org_id or len(org_id)==0:
        org_id = 'NULL'

    # Загрузка модуля rvrao
    global rvrao
    rvrao = context.get_plugin('rvrao')

    # Создание отчета
    res_df = pd.DataFrame()

    for form_no in ['form_1_1', 'form_1_2', 'form_1_3', 'form_1_4', 'form_1_5']:
        if form_no in forms:
            df = build_for_form(form_no, op_date, org_id, context=context)
            res_df = res_df.append(df)

    return res_df.reset_index().to_json(orient="table", force_ascii=False)


def build(params, api=None):                              # Старый интерфейс вызова
    import rvrao
    return exec(params, context=rvrao.def_context, api=api)


def build_for_form(form_no, op_date, org_id, context):

    if form_no == 'form_1_1':
        sql_fields = ',inv_amount, flow_amount, end_amount'
        sql = 'INNER JOIN "acc_o1" r ON r.record_id = b.last_record_id'

    elif form_no == 'form_1_2':
        sql_fields = ',inv_mass_net, flow_mass_net, end_mass_net'
        sql = 'INNER JOIN "acc_o2" r ON r.record_id = b.last_record_id'

    elif form_no == 'form_1_3':
        sql_fields = ''
        sql = 'INNER JOIN "acc_o3" r ON r.record_id = b.last_record_id'

    elif form_no == 'form_1_4':
        sql_fields = ',end_mass_net, end_volume'
        sql = 'INNER JOIN "acc_o4" r ON r.record_id = b.last_record_id'

    elif form_no == 'form_1_5':
        sql_fields = ',inv_amount, flow_amount, end_amount'
        sql = 'INNER JOIN "acc_o5" r ON r.record_id = b.last_record_id'

    sql = f"SELECT r.*, b.inv_date {sql_fields} FROM acc_items_balance('{op_date}', '{form_no}', {org_id}) AS b {sql}"
    print(sql)

    df = rvrao.sql_dataframe(sql, context=context)
    
    return df

