def init( context=None, db_name=None, api=None ):
    return {
        'module_type' : 'lib'
    }


import sys
import os

sys.path.insert(1, os.path.join(sys.path[0], '..', '..'))    

try:
    import settings 
except:
    pass

try:
    HOST = settings.DB_HOST
except:
    HOST = 'wiki.robolab.msk.ru'
    print(f'rvrao: использован хост по-умолчанию {HOST}')

try:
    DATABASE = settings.POSTGRES_DATABASE
except:
    DATABASE = 'rvirao-build' #'dev'
    print(f'rvrao: использована БД по-умолчанию {DATABASE}')

try:
    PORT = settings.DB_PORT
except:
    PORT = "5432"
    print(f'rvrao: использован порт по-умолчанию {PORT}')

try:
    USER = settings.DB_USER['user']
    PASSWORD = settings.DB_USER['password']
except:
    USER = 'rvrao'
    PASSWORD = 'rvrao'
    print(f'rvrao: использован пользователь по-умолчанию {USER}')

import psycopg2
from psycopg2.extras import NamedTupleCursor, RealDictCursor, DictCursor

import pandas as pd

#======================================================================================
#   Источники данных справочников
#======================================================================================
def str2float(df, attr):            # Преобразует столбец DataFrame из текстового типа в числовой
    if attr not in df.columns.values: return
    
    res = []
    for x in df[attr].values:
        if type(x) == str:
            try:
                y = float(x)
            except:
                y = None
        else:
            y = x
        res.append(y)

    df[attr]=pd.Series(res)
    #    
    #
def dict2value(values, voc_df):     # Расшифровка DICT кодов из справочников в STR значения (устарело)
    if type(values) == dict:
        values = list(values.keys())
        if len(values):
            value_str = ''
            for value_id in values:
                try:
                    value_lbl = voc_df.value_id.loc[int(values[0])]
                except:
                    value_lbl = values[0]
                if value_lbl != None:
                    if len(value_str): value_str += ', '
                    value_str += value_lbl    
            value = value_str
        else:
            value = None
    else:
        value = values
    return value

def old_get_connect():                  # Возвращает подключение к БД
    conn = psycopg2.connect(user=USER, password=PASSWORD, database=DATABASE, host=HOST, port=PORT)
    conn.set_client_encoding('UTF8')
    return conn
    #
    #
#======================================================================================
#   Контекст вызова
#======================================================================================
class Context:
    def __init__(self, **kvargs):
        self.USER_LOGIN = kvargs.get('USER_LOGIN' '<None>')
        self.DB_HOST = kvargs.get('DB_HOST', HOST)
        self.DB_NAME = kvargs.get('DB_NAME', DATABASE)
        self.DB_PORT = kvargs.get('DB_PORT', PORT)

        self.LOGIN   = kvargs.get('LOGIN', USER)                # Логин для подключения к БД
        self.PWD     = kvargs.get('PWD', PASSWORD)              # Пароль для подключения к БД
        #
        #
    def log(self, msg, log_type='debug'):
        print(f'DEBUG: {msg}')
        #
        #
    def set_msg(self, msg):
        print(f'RESULT: {msg}')
        #
        #
    def exception(self, msg):
        return Exception(msg)
        #
        #
    def get_connect(self):
        conn = psycopg2.connect(
                user=self.LOGIN, 
                password=self.PWD, 
                database=self.DB_NAME, 
                host=self.DB_HOST, 
                port=self.DB_PORT
            )

        conn.set_client_encoding('UTF8')
        return conn
        #
        #
    def get_plugin(self, plugin_name):
        if plugin_name == 'rvrao':
            return sys.modules[__name__]

def_context = Context()

def sql_rows(sql, cursor_factory=RealDictCursor, commit=False, context=def_context):
    conn = context.get_connect()
    cur = conn.cursor(cursor_factory=cursor_factory)
    try:
        cur.execute(sql)
        rows = cur.fetchall()
    finally:
        cur.close()

    if commit:
        conn.commit()

    return rows
    #
    #
def sql_dataframe(sql, context=def_context):                                 # DF из SQL запроса
    rows = sql_rows(sql, context=context)
    return pd.DataFrame(rows)
    #
    #


#======================================================================================
#   Источники данных справочников
#======================================================================================
def voc_dataframe(voc_type=None, load_attrs=False, context=def_context):     # Загрузка справочника в DataFrame
    '''
    voc_type    - тип справочника
    load_attrs  - флаг загрузки атрибутов записей
    '''
    sql = 'SELECT '
    if load_attrs:
        sql += ' * '
    else:
        sql += ' record_id, value_id, label, voc_type '

    sql += ' FROM "VocValues"'

    if voc_type:
        sql += f"WHERE voc_type='{voc_type}'"

    df = sql_dataframe(sql, context=context)

    if voc_type:
        df.set_index('value_id')
    else:
        df.set_index('record_id')

    return df
    #
    #
def voc_unpack(voc_df, voc_type=None):                  # Распаковка атрибутов записей в DataFrame
    if voc_type:
        voc_df = voc_df[voc_df.voc_type==voc_type]
    
    new_rows = []
    for idx in voc_df.index.values:
        record = voc_df.loc[idx]
        new_row = {}
        new_row['record_id'] = record['record_id']
        new_row['voc_type'] = record['voc_type']
        new_row['value_id'] = record['value_id']
        new_row['label'] = record['label']
        new_row.update(record['attrs'])

        new_rows.append(new_row)

    df = pd.DataFrame(new_rows)
    df.index = df.record_id
    
    return df
    #
    #
def voc_find(df, conditions):
    cond = ''
    for key, value in conditions.items():
        if type(value)==str:
            cond += f'{key}=="{value}" and '
        else:
            cond += f'{key}=={value} and '
    cond = cond[:-4]
    
    return df.query(cond)
    #
    #
def voc_find_rec(df, conditions, field=None):
    res_df = voc_find(df, conditions)

    res_dict = res_df.to_dict(orient='records')
    if not res_dict:
        return None
    
    res_dict = res_dict[0]

    if field:
        return res_dict.get(field)
    else:
        return res_dict
    #
    #

from types import MethodType
def voc_values(voc_type, context=def_context):
    df = voc_dataframe(voc_type, load_attrs=True, context=def_context)
    voc = voc_unpack(df).set_index('value_id')

    # Добавление методов для поиск значений
    voc.find = MethodType(voc_find, voc)
    voc.find_rec = MethodType(voc_find_rec, voc)

    return voc
    #
    #
def voc_orgs_dataframe(context=def_context):            # Загрузка справочника организаций в DataFrame
    sql = 'select * from "VocOrgs"'
    orgs_df = sql_dataframe(sql, context)
    
    str2float(orgs_df, 'record_id')
    orgs_df.index = orgs_df.record_id
    del orgs_df['attrs']

    return orgs_df
    #
    #
def voc_orgs_merge(df, org_field='org_id'):             # Присоединение DataFrame организаций
    '''
    df      - DataFrame к которому присоединяется справочник организаций
    org_id  - имя поля DataFrame содержащего org_id 
    '''
    orgs_df = voc_orgs_dataframe()
    del orgs_df['voc_type']
    del orgs_df['valid_from']
    del orgs_df['valid_to']
    del orgs_df['value_id']
    del orgs_df['value_status']
    del orgs_df['record_id']
    orgs_df.rename(columns={'label':'org_label'}, inplace=True)
    return df.merge(orgs_df, left_on=org_field, right_index=True)

#======================================================================================
#   Источники данных отчетных форм
#======================================================================================
def rf_table_unpack(reportforms_rows):      # Преобразование табличной части отчетной формы в DataFrame
    
    new_rows = []
    for record in reportforms_rows:
        for row in record['form_table_data']['main_table']:
            new_row = row
            new_row['org_id'] = record['org_id']
            new_row['user_id'] = record['user_id']
            new_row['form_no'] = record['form_no']
            new_row['form_state'] = record['form_state']
            new_row['is_active'] = record['is_active']

            new_rows.append(new_row)

    return pd.DataFrame(new_rows)

    #
def rf_table_decode(table_df):              # Декодирование кодов из справочников

    voc_df = voc_dataframe()

    for idx in table_df.index.values:
        rec = table_df.loc[idx]

        # Декодирование числовых значений
        
        
        # Декодирование кодов из справочников
        attrs = {
                'form_1_1': ['Typ', 'OpCod', 'Nuclid', 'PrName', 'DocVid', 'FormSobst'],
                'form_1_2': ['Typ', 'OpCod', 'Nuclid', 'PrName', 'DocVid', 'FormSobst'],
                'form_1_3': ['Arg', 'vid', 'code', 'naim', 'radio', 'form_code'],
                'form_1_4': ['Typ', 'OpCod', 'Nuclid', 'PrName', 'DocVid', 'FormSobst', 'Arg', 'Agr'],
                'form_1_5': ['Typ', 'BCod', 'OpCod', 'CodRAO','DocVid', 'Nuclid', 'PrName', 'PH_Name'],
                'form_1_6': ['BCod', 'OpCod', 'CodRAO','DocVid', 'PrName', 'RAOCod', 'PH_Name'],
                'form_1_7': ['g2', 'g4', 'g12', 'g14', 'g19', 'g21', 'g22', 'g30'],
                }.get(rec['form_no'])
        
        if attrs:
            for attr_name in attrs:
                if attr_name in rec: table_df.loc[idx, (attr_name)] = dict2value(rec[attr_name], voc_df)         

    #
def rf_convert_nums(df):                    # Преобразование столбцов с числовыми данными в числа
    str2float(df, 'org_id')
    str2float(df, 'Sht')
    str2float(df, 'Kg')
    str2float(df, 'Kbm')
    str2float(df, 'Activn')
    #
    
#======================================================================================
#   Генераторы DataFrame
#======================================================================================
def rf_dataframe(forms=None, active_only=True, context=def_context):        # Список отчетных форм из ReportForms
    sql =  'SELECT r.author_id as org_id, user_id, '
    sql += 'form_main_data, form_table_data, '
    sql += 'form_no, form_state, rf_is_active(f.report_form_id) is_active '
    sql += 'FROM reportforms f LEFT OUTER JOIN reports r ON r.report_id = f.report_id '

    conditions = []
    if forms:
        conditions.append( f"position(f.form_no in '{forms}')>0" )

    if active_only:
        conditions.append( ' rf_is_active(f.report_form_id) ' )

    if len(conditions):
        sql += ' WHERE ' + ' AND '.join(conditions)

    rows = sql_rows(sql, context=context)
    df = rf_table_unpack(rows)
    rf_convert_nums(df)
    #if decode: rf_table_decode(df)

    return df
    #
    #
def rf_read(form_id, context=def_context):
    sql = f'''
        SELECT * FROM reportforms WHERE report_form_id={form_id}
    '''
    rows = sql_rows(sql, DictCursor, context=context)

    if len(rows) == 0:
        raise Exception(f'Отчетная форма не найдена report_form_id={form_id}')

    rows = dict(rows[0].items())

    # Добавляем атрибуты отчетной формы
    form_attrs = rows['form_main_data']
    if form_attrs: 
        rows.update(form_attrs)

    return rows
    #
    #
def rf_table_read(form_id, table_key='main_table', context=def_context):
    sql = f'''
        SELECT form_table_data->'{table_key}'
            FROM reportforms
            WHERE report_form_id = {form_id};
    '''
    rows = sql_rows(sql, context=context)
    rows = list(rows[0].values())[0]

    return pd.DataFrame(rows)
    #
    #
def rf_table_update(form_id, df, table_key='main_table', context=def_context):
    conn = context.get_connect()
    cur = conn.cursor()

    data_str = df.to_json(orient='records', force_ascii=False, date_format='iso')

    sql = f'''
        SELECT rf_table_update({form_id}, '{data_str}', '{table_key}');
    '''

    cur.execute(sql)
    conn.commit()

#======================================================================================
#   Источники данных учетных регистров
#======================================================================================
def ar_f11_dataframe(context = def_context):    # DF Формы 1.1 / AccZRI
    sql =  ' SELECT r.*,  f.form_no, f.form_state, rf_is_active(f.report_form_id) is_active '
    sql += ' FROM acc_o1 as r '
    sql += ' INNER JOIN reportforms f ON f.report_form_id=r.report_form_id'
    return sql_dataframe(sql, context=context)
    #
def ar_f12_dataframe(context = def_context):    # DF Формы 1.2 / AccIOU
    sql =  ' SELECT r.*,  f.form_no, f.form_state, rf_is_active(f.report_form_id) is_active '
    sql += ' FROM "AccIOU" as r '
    sql += ' INNER JOIN reportforms f ON f.report_form_id=r.report_form_id'
    return sql_dataframe(sql, context=context)
    #
def ar_f13_dataframe(context = def_context):    # DF Формы 1.3 / AccORI
    sql =  ' SELECT r.*,  f.form_no, f.form_state, rf_is_active(f.report_form_id) is_active '
    sql += ' FROM "AccORI" as r '
    sql += ' INNER JOIN reportforms f ON f.report_form_id=r.report_form_id'
    return sql_dataframe(sql, context=context)
    #
def ar_f14_dataframe(context = def_context):    # DF Формы 1.4 / AccORP
    sql =  ' SELECT r.*,  f.form_no, f.form_state, rf_is_active(f.report_form_id) is_active '
    sql += ' FROM "AccORP" as r '
    sql += ' INNER JOIN reportforms f ON f.report_form_id=r.report_form_id'
    return sql_dataframe(sql, context=context)
    #
def ar_f15_dataframe(context = def_context):    # DF Формы 1.5 / AccROZ
    sql =  ' SELECT r.*,  f.form_no, f.form_state, rf_is_active(f.report_form_id) is_active '
    sql += ' FROM "AccROZ" as r '
    sql += ' INNER JOIN reportforms f ON f.report_form_id=r.report_form_id'
    return sql_dataframe(sql, context=context)
    #
def ar_f16_dataframe(context = def_context):    # DF Формы 1.6 / AccRAO
    sql =  ' SELECT r.*,  f.form_no, f.form_state, rf_is_active(f.report_form_id) is_active '
    sql += ' FROM "AccRAO" as r '
    sql += ' INNER JOIN reportforms f ON f.report_form_id=r.report_form_id'
    return sql_dataframe(sql, context=context)
    #
def ar_f17_dataframe(context = def_context):    # DF Формы 1.7 / AccO7
    sql =  ' SELECT r.*,  f.form_no, f.form_state, rf_is_active(f.report_form_id) is_active '
    sql += ' FROM "AccO7" as r '
    sql += ' INNER JOIN reportforms f ON f.report_form_id=r.report_form_id'
    return sql_dataframe(sql, context=context)
    #
def ar_f18_dataframe(context = def_context):    # DF Формы 1.8 / AccO8
    sql =  ' SELECT r.*,  f.form_no, f.form_state, rf_is_active(f.report_form_id) is_active '
    sql += ' FROM "AccO8" as r '
    sql += ' INNER JOIN reportforms f ON f.report_form_id=r.report_form_id'
    return sql_dataframe(sql, context=context)
    #
def ar_f19_dataframe(context = def_context):    # DF Формы 1.9 / AccO9
    sql =  ' SELECT r.*,  f.form_no, f.form_state, rf_is_active(f.report_form_id) is_active '
    sql += ' FROM "AccO9" as r '
    sql += ' INNER JOIN reportforms f ON f.report_form_id=r.report_form_id'
    return sql_dataframe(sql, context=context)
    #
def ar_f21_dataframe(context = def_context):    # DF Формы 2.1 / AccF1
    sql =  ' SELECT r.*,  f.form_no, f.form_state, rf_is_active(f.report_form_id) is_active '
    sql += ' FROM "AccF1" as r '
    sql += ' INNER JOIN reportforms f ON f.report_form_id=r.report_form_id'
    return sql_dataframe(sql, context=context)
    #
def ar_f22_dataframe(context = def_context):    # DF Формы 2.2 / AccF2
    sql =  ' SELECT r.*,  f.form_no, f.form_state, rf_is_active(f.report_form_id) is_active '
    sql += ' FROM "AccF2" as r '
    sql += ' INNER JOIN reportforms f ON f.report_form_id=r.report_form_id'
    return sql_dataframe(sql, context=context)
    #
def ar_f23_dataframe(context = def_context):    # DF Формы 2.3 / AccF3
    sql =  ' SELECT r.*,  f.form_no, f.form_state, rf_is_active(f.report_form_id) is_active '
    sql += ' FROM "AccF3" as r '
    sql += ' INNER JOIN reportforms f ON f.report_form_id=r.report_form_id'
    return sql_dataframe(sql, context=context)
    #
def ar_f24_dataframe(context = def_context):    # DF Формы 2.4 / AccF4
    sql =  ' SELECT r.*,  f.form_no, f.form_state, rf_is_active(f.report_form_id) is_active '
    sql += ' FROM "AccF4" as r '
    sql += ' INNER JOIN reportforms f ON f.report_form_id=r.report_form_id'
    return sql_dataframe(sql, context=context)
    #
def ar_f25_dataframe(context = def_context):    # DF Формы 2.5 / AccF5
    sql =  ' SELECT r.*,  f.form_no, f.form_state, rf_is_active(f.report_form_id) is_active '
    sql += ' FROM "AccF5" as r '
    sql += ' INNER JOIN reportforms f ON f.report_form_id=r.report_form_id'
    return sql_dataframe(sql, context=context)
    #
def ar_f26_dataframe(context = def_context):    # DF Формы 2.6 / AccF6
    sql =  ' SELECT r.*,  f.form_no, f.form_state, rf_is_active(f.report_form_id) is_active '
    sql += ' FROM "AccF6" as r '
    sql += ' INNER JOIN reportforms f ON f.report_form_id=r.report_form_id'
    return sql_dataframe(sql, context=context)
    #
def ar_f27_dataframe(context = def_context):    # DF Формы 2.7 / AccF7
    sql =  ' SELECT r.*,  f.form_no, f.form_state, rf_is_active(f.report_form_id) is_active '
    sql += ' FROM "AccF7" as r '
    sql += ' INNER JOIN reportforms f ON f.report_form_id=r.report_form_id'
    return sql_dataframe(sql, context=context)
    #
def ar_f28_dataframe(context = def_context):    # DF Формы 2.8 / AccF8
    sql =  ' SELECT r.*,  f.form_no, f.form_state, rf_is_active(f.report_form_id) is_active '
    sql += ' FROM "AccF8" as r '
    sql += ' INNER JOIN reportforms f ON f.report_form_id=r.report_form_id'
    return sql_dataframe(sql, context=context)
    #
def ar_f29_dataframe(context = def_context):    # DF Формы 2.9 / AccF9
    sql =  ' SELECT r.*,  f.form_no, f.form_state, rf_is_active(f.report_form_id) is_active '
    sql += ' FROM "AccF9" as r '
    sql += ' INNER JOIN reportforms f ON f.report_form_id=r.report_form_id'
    return sql_dataframe(sql, context=context)
    #
def ar_f210_dataframe(context = def_context):   # DF Формы 2.10 / AccF10
    sql =  ' SELECT r.*,  f.form_no, f.form_state, rf_is_active(f.report_form_id) is_active '
    sql += ' FROM "AccF10" as r '
    sql += ' INNER JOIN reportforms f ON f.report_form_id=r.report_form_id'
    return sql_dataframe(sql, context=context)
    #
def ar_f211_dataframe(context = def_context):   # DF Формы 2.11 / AccF11
    sql =  ' SELECT r.*,  f.form_no, f.form_state, rf_is_active(f.report_form_id) is_active '
    sql += ' FROM "AccF11" as r '
    sql += ' INNER JOIN reportforms f ON f.report_form_id=r.report_form_id'
    return sql_dataframe(sql, context=context)
    #
def ar_f212_dataframe(context = def_context):   # DF Формы 2.12 / AccF12
    sql =  ' SELECT r.*,  f.form_no, f.form_state, rf_is_active(f.report_form_id) is_active '
    sql += ' FROM "AccF12" as r '
    sql += ' INNER JOIN reportforms f ON f.report_form_id=r.report_form_id'
    return sql_dataframe(sql, context=context)
    #

def fc_dataframe(context = def_context):        # DF Регистра прогнозирования РАО
    sql = '''
        SELECT  *
        FROM "AccRaoForecast"
    '''

    df = sql_dataframe(sql, context=context)
    df['op_year'] = pd.DatetimeIndex(df.OpDate).year

    return df
#======================================================================================
#   
#======================================================================================
def drop_unused_columns(df, use_list):
    drop_list = []
    for col in df.columns.values:
        if col not in use_list:
            drop_list.append(col)

    for col in drop_list:
        del df[col]
    #
    #

#======================================================================================
#   Группировка данных
#======================================================================================
def group(df, group_list, main_agg_list, sub_agg_list):
    '''
    df              - pandas.DataFrame
    group_list      - список полей для группировки, например ['field1', 'field2']
    main_agg_list   - основное правило агрегации, например {'total_qty': ('qty', 'sum'), } 
    sub_agg_list    - правило агрегации для промежуточных итогов, например {'total_qty': 'sum', } 

    Возвращает группированный DataFrame с промежуточными итогами. 
    Уровень промежуточного итога указан в колоне "level", где 0 - общий итог, 1-первый уровень, 10-дательные данные
    '''

    # Группировка основных данные
    g2_df = df.groupby(group_list).agg(**main_agg_list)
    g2_df['level'] = 10
    g2_df = g2_df.reset_index()
    g_df = g2_df.copy()

    # Группировка промежуточных итогов
    for l in range(1,len(group_list)):
        gs_df = g2_df.groupby(group_list[0:l]).agg(sub_agg_list)
        gs_df = gs_df.reset_index()
        gs_df['level'] = l
        g_df = g_df.append(gs_df, ignore_index=True)

    # Главный итог
    gt_df = g2_df.agg(sub_agg_list)
    gt_df['level'] = 0
    g_df = g_df.append(gt_df.T, ignore_index=True)


    # Объединение данных и промежуточных итогов в единую таблицу
    g_df = g_df.sort_values(group_list)
    g_df = g_df.reset_index()
    del g_df['index']
    g_df = g_df.set_index(group_list)

    return g_df

#======================================================================================
#   Форматирование группированных данных
#======================================================================================
def format(df, fmt_dict):
    row_styles = []
    for i in range(df.index.size):
        level = df.level.iloc[i]
        if level==0:
            style = [('background-color', '#FFA0A0'),('border-bottom','1px solid black')]            
        elif level==1:
            style = [('background-color', '#C0FFC0'),('border-bottom','1px solid black')]
        elif level==2:
            style = [('background-color', '#E0E0F0'),('border-bottom','1px solid black')]
        else:
            style = []
        row_styles.append({'selector':f'.row{i}', 'props':style})

    row_styles.append({'selector':'.level0', 'props':[('border-bottom','1px solid black')]})

    s = df.style.set_table_styles(row_styles)
    s = s.format(fmt_dict)

    return s
