CREATE ROLE rw_user;
GRANT SELECT, INSERT, UPDATE, DELETE ON ALL TABLES IN SCHEMA public TO rw_user;

CREATE ROLE ro_user;
GRANT SELECT ON ALL TABLES IN SCHEMA public TO ro_user;

CREATE USER rvrao WITH PASSWORD 'rvrao';
GRANT ro_user TO rvrao;

CREATE ROLE db_admin;
GRANT  db_admin TO rw_user;

CREATE USER user1 WITH password 'user1';
GRANT rw_user TO user1;


--
-- PostgreSQL database dump
--

-- Dumped from database version 11.16 (Debian 11.16-0+deb10u1)
-- Dumped by pg_dump version 14.5 (Homebrew)

-- Started on 2022-11-30 13:43:05 MSK

SET statement_timeout = 0;
SET lock_timeout = 0;
SET idle_in_transaction_session_timeout = 0;
SET client_encoding = 'UTF8';
SET standard_conforming_strings = on;
SELECT pg_catalog.set_config('search_path', '', false);
SET check_function_bodies = false;
SET xmloption = content;
SET client_min_messages = warning;
SET row_security = off;

--
-- TOC entry 3 (class 2615 OID 2200)
-- Name: public; Type: SCHEMA; Schema: -; Owner: postgres
--



ALTER SCHEMA public OWNER TO postgres;

--
-- TOC entry 3155 (class 0 OID 0)
-- Dependencies: 3
-- Name: SCHEMA public; Type: COMMENT; Schema: -; Owner: postgres
--

COMMENT ON SCHEMA public IS 'standard public schema';


--
-- TOC entry 677 (class 1247 OID 20135)
-- Name: areport_state_type; Type: TYPE; Schema: public; Owner: db_admin
--

CREATE TYPE public.areport_state_type AS ENUM (
    'wait',
    'temp',
    'saved',
    'published',
    'archived'
);


ALTER TYPE public.areport_state_type OWNER TO db_admin;

--
-- TOC entry 662 (class 1247 OID 19759)
-- Name: form_state_type; Type: TYPE; Schema: public; Owner: db_admin
--

CREATE TYPE public.form_state_type AS ENUM (
    'draft',
    'remarks',
    'verified',
    'confirmed',
    'unloaded',
    'rejected',
    'accepted',
    'canceled',
    'loaded'
);


ALTER TYPE public.form_state_type OWNER TO db_admin;

--
-- TOC entry 680 (class 1247 OID 20242)
-- Name: user_role_type; Type: TYPE; Schema: public; Owner: db_admin
--

CREATE TYPE public.user_role_type AS ENUM (
    'operator',
    'chief_operator',
    'administrator',
    'voc_operator',
    'analyst'
);


ALTER TYPE public.user_role_type OWNER TO db_admin;

--
-- TOC entry 254 (class 1255 OID 20053)
-- Name: acc_inventory_exist(date); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.acc_inventory_exist(p_date date) RETURNS TABLE(org_id integer, form_no character varying, inv_date date, last_record_id integer, end_mass_net double precision, end_volume double precision, end_amount bigint, end_activity double precision, "PaspN" character varying, "Typ" character varying, "Numb" character varying, "IzgotOKPO" character varying, "IzgotDate" date, "IstName" character varying)
    LANGUAGE sql
    AS $$

        SELECT b.org_id, b.form_no, inv_date, last_record_id, end_mass_net, end_volume, end_amount, end_activity,
            COALESCE(f1."PaspN", f2."PaspN", f3."PaspN", f4."PaspN") AS "PaspN",
            COALESCE(f1."Typ", f2."Typ", f3."Typ", f4."Typ") AS "Typ",
            COALESCE(f1."Numb", f2."Numb", f3."Numb") AS "Numb",
            COALESCE(f1."IzgotOKPO", f2."IzgotOKPO", f3."IzgotOKPO") AS "IzgotOKPO",
            COALESCE(f1."IzgotDate", f2."IzgotDate", f3."IzgotDate") AS "IzgotDate",
            f4."IstName" 
        FROM acc_items_balance(p_date, 'form_1_1, form_1_2, form_1_3, form_1_4') AS b
            LEFT OUTER JOIN acc_o1 AS f1 ON f1.record_id = b.last_record_id
            LEFT OUTER JOIN acc_o2 AS f2 ON f2.record_id = b.last_record_id
            LEFT OUTER JOIN acc_o3 AS f3 ON f3.record_id = b.last_record_id
            LEFT OUTER JOIN acc_o4 AS f4 ON f4.record_id = b.last_record_id

        WHERE p_date - inv_date > (CASE 
                                        WHEN form_no in ('form_1_1', 'form_1_2', 'form_1_3', 'form_1_4') THEN 365
                                        ELSE 365*5
                                    END)
    $$;


ALTER FUNCTION public.acc_inventory_exist(p_date date) OWNER TO db_admin;

--
-- TOC entry 257 (class 1255 OID 32280)
-- Name: acc_item_inv(character varying, date); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.acc_item_inv(p_item character varying, p_date date) RETURNS TABLE(org_id integer, op_item character varying, "OpDate" date, mass_net double precision, volume double precision, amount bigint, activity double precision)
    LANGUAGE sql
    AS $$
        SELECT 
            p.org_id, p.op_item, p1."OpDate",
            p1.mass_net, p1.volume, P1.amount::BIGINT, p1.activity
            FROM 
                acc_oper AS p 
                LEFT OUTER JOIN acc_oper AS p1 ON p.record_id = p1.record_id AND p1.op_sign=0 AND p1."OpDate" <= p_date
            WHERE
                p.op_item = p_item
                
            ORDER BY p1."OpDate" DESC NULLS LAST
            LIMIT 1
    $$;


ALTER FUNCTION public.acc_item_inv(p_item character varying, p_date date) OWNER TO db_admin;

--
-- TOC entry 266 (class 1255 OID 20038)
-- Name: acc_items_balance(date, character varying, integer); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.acc_items_balance(p_date date, p_form_no character varying DEFAULT NULL::character varying, p_org_id integer DEFAULT NULL::integer) RETURNS TABLE(org_id integer, acc_id character varying, form_no character varying, inv_date date, last_record_id integer, inv_mass_net double precision, flow_mass_net double precision, end_mass_net double precision, inv_volume double precision, flow_volume double precision, end_volume double precision, inv_amount bigint, flow_amount bigint, end_amount bigint, inv_activity double precision, flow_activity double precision, end_activity double precision)
    LANGUAGE sql
    AS $$
        SELECT 
            list.org_id, list.op_item, list.form_no,
            inv."OpDate" AS inv_date,
            (SELECT record_id FROM acc_oper WHERE op_item=list.op_item ORDER BY record_id DESC LIMIT 1) last_record_id,
            inv.mass_net AS inv_mass_net, flow.mass_net AS flow_mass_net, COALESCE(inv.mass_net,0) + flow.mass_net AS end_mass_net,
            inv.volume AS inv_volume, flow.volume AS flow_volume, COALESCE(inv.volume,0) + flow.volume AS end_volume,
            inv.amount AS inv_amount, flow.amount AS flow_amount, COALESCE(inv.amount,0) + flow.amount AS end_amount,
            inv.activity AS inv_activity, flow.activity AS flow_activity, COALESCE(inv.activity,0) + flow.activity AS end_activity
        FROM 
            (SELECT DISTINCT p.org_id, p.op_item, f.form_no              -- список объектов учета
                FROM acc_oper p INNER JOIN reportforms f ON p.report_form_id=f.report_form_id
                WHERE f.form_no LIKE (COALESCE(p_form_no, f.form_no)::VARCHAR || '%')
                  AND p.org_id = COALESCE(p_org_id, p.org_id) 
                  AND p.op_item IS NOT NULL
            ) list

            LEFT OUTER JOIN LATERAL acc_item_inv(list.op_item, p_date) AS inv ON inv.op_item=list.op_item           -- данные инвенторизации

            LEFT OUTER JOIN LATERAL (                                                   -- обороты по объектам учета
                    SELECT 
                        op_item,
                        sum(op_sign * mass_net) AS mass_net,
                        sum(op_sign * volume) AS volume,
                        sum(op_sign * amount) AS amount,
                        sum(op_sign * activity) AS activity
                    FROM acc_oper 
                    WHERE op_item = list.op_item
                        AND "OpDate" <= p_date 
                        AND "OpDate" >= COALESCE(inv."OpDate", "OpDate")
                    GROUP BY op_item
                    ) flow ON flow.op_item=list.op_item;
    $$;


ALTER FUNCTION public.acc_items_balance(p_date date, p_form_no character varying, p_org_id integer) OWNER TO db_admin;

--
-- TOC entry 258 (class 1255 OID 32281)
-- Name: acc_items_check(date, character varying, integer); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.acc_items_check(p_date date, p_form_no character varying DEFAULT NULL::character varying, p_org_id integer DEFAULT NULL::integer) RETURNS TABLE(org_id integer, op_item character varying, form_no character varying, last_inv_date date, prev_inv_date date, last_record_id integer, calc_mass_net double precision, inv_mass_net double precision, calc_volume double precision, inv_volume double precision, calc_amount bigint, inv_amount bigint)
    LANGUAGE sql
    AS $$
        SELECT 
            list.org_id, list.op_item, list.form_no,
            last_inv."OpDate" AS last_inv_date,
            prev_inv."OpDate" AS prev_inv_date,
            (SELECT record_id FROM acc_oper WHERE op_item=list.op_item ORDER BY record_id DESC LIMIT 1) last_record_id,

            prev_inv.mass_net + flow.mass_net AS calc_mass_net, 
            last_inv.mass_net AS inv_mass_net,

            prev_inv.volume + flow.volume AS calc_volume,
            last_inv.volume AS inv_volume,

            prev_inv.amount + flow.amount AS calc_amount,
            last_inv.amount AS inv_amount

        FROM 
            (SELECT DISTINCT p.org_id, p.op_item, f.form_no                                      -- список объектов учета
                FROM acc_oper p INNER JOIN reportforms f ON p.report_form_id=f.report_form_id
                WHERE f.form_no = COALESCE(p_form_no, f.form_no)
                  AND p.org_id = COALESCE(p_org_id, p.org_id) 
                  AND p.op_item IS NOT NULL
            ) list

            LEFT OUTER JOIN LATERAL acc_item_inv(list.op_item, p_date) AS last_inv               -- данные последней инвенторизации
            ON last_inv.op_item=list.op_item  

            LEFT OUTER JOIN LATERAL acc_item_inv(list.op_item, last_inv."OpDate"-1) AS prev_inv   -- данные предыдущей инвенторизации
            ON prev_inv.op_item = last_inv.op_item  

            LEFT OUTER JOIN LATERAL (                                                           -- обороты по объектам учета
                    SELECT op_item,
                        sum(op_sign * mass_net) AS mass_net,
                        sum(op_sign * volume) AS volume,
                        sum(op_sign * amount) AS amount,
                        sum(op_sign * activity) AS activity
                    FROM acc_oper
                    WHERE   "OpDate" >  COALESCE(prev_inv."OpDate", "OpDate")
                        AND "OpDate" <  COALESCE(last_inv."OpDate", "OpDate")
                        AND "OpDate" <= p_date
                        AND op_item = list.op_item
                    GROUP BY op_item
                    ) flow
            ON flow.op_item=list.op_item;
    $$;


ALTER FUNCTION public.acc_items_check(p_date date, p_form_no character varying, p_org_id integer) OWNER TO db_admin;

--
-- TOC entry 247 (class 1255 OID 20004)
-- Name: acc_move_replay(); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.acc_move_replay() RETURNS void
    LANGUAGE sql
    AS $$
        UPDATE "AccProto" SET row=row;           
    $$;


ALTER FUNCTION public.acc_move_replay() OWNER TO db_admin;

--
-- TOC entry 246 (class 1255 OID 20003)
-- Name: acc_move_update(integer, double precision, double precision, integer, double precision, double precision, double precision, double precision, double precision, double precision); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.acc_move_update(p_record_id integer, p_mass_net double precision, p_volume double precision, p_amount integer, p_activity double precision, p_mass_gross double precision DEFAULT NULL::double precision, p_act_tr double precision DEFAULT NULL::double precision, p_act_ur double precision DEFAULT NULL::double precision, p_act_a double precision DEFAULT NULL::double precision, p_act_b double precision DEFAULT NULL::double precision) RETURNS void
    LANGUAGE plpgsql
    AS $$
    DECLARE v_is_active BOOLEAN;
    BEGIN
        v_is_active = (SELECT rf_is_active(report_form_id) FROM "AccProto" WHERE record_id=p_record_id AND sign IS NOT NULL);

        IF v_is_active THEN
            IF NOT EXISTS(SELECT 1 FROM "AccMoves" WHERE record_id = p_record_id) THEN
                INSERT INTO "AccMoves"(record_id) VALUES (p_record_id);
            END IF;

            UPDATE "AccMoves" SET
                mass_net = p_mass_net,
                volume = p_volume,
                amount = p_amount,
                activity = p_activity,
                mass_gross = p_mass_gross,
                act_tr = p_act_tr,
                act_ur = p_act_ur,
                act_a = p_act_a,
                act_b = p_act_b
                WHERE record_id = p_record_id;

        ELSE
            DELETE FROM "AccMoves" WHERE record_id = p_record_id;

        END IF;
    END $$;


ALTER FUNCTION public.acc_move_update(p_record_id integer, p_mass_net double precision, p_volume double precision, p_amount integer, p_activity double precision, p_mass_gross double precision, p_act_tr double precision, p_act_ur double precision, p_act_a double precision, p_act_b double precision) OWNER TO db_admin;

--
-- TOC entry 260 (class 1255 OID 20039)
-- Name: ar_balance_f11(date, integer); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.ar_balance_f11(p_date date, p_org_id integer DEFAULT NULL::integer) RETURNS TABLE(org_id integer, "PaspN" character varying, "Typ" character varying, "Nuclid" character varying, "Numb" character varying, "Activn" double precision, "IzgotOKPO" character varying, "IzgotDate" date, "Kateg" character varying, "Nss" double precision, "Sht" bigint)
    LANGUAGE sql
    AS $$

        SELECT
            r."org_id", r."PaspN", r."Typ", r."Nuclid", r."Numb", r."Activn", r."IzgotOKPO", r."IzgotDate", r."Kateg", r."Nss",
            b.end_amount AS "Sht" 
            FROM acc_items_balance(p_date, 'form_1_1', p_org_id) AS b INNER JOIN acc_o1 AS r ON b.last_record_id = r.record_id

    $$;


ALTER FUNCTION public.ar_balance_f11(p_date date, p_org_id integer) OWNER TO db_admin;

--
-- TOC entry 250 (class 1255 OID 20040)
-- Name: ar_balance_f12(date, integer); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.ar_balance_f12(p_date date, p_org_id integer DEFAULT NULL::integer) RETURNS TABLE(org_id integer, "PaspN" character varying, "Typ" character varying, "Numb" character varying, "IzgotOKPO" character varying, "IzgotDate" date, "Nss" integer, "Kg" double precision)
    LANGUAGE sql
    AS $$

        SELECT 
            r."org_id", r."PaspN", r."Typ", r."Numb", r."IzgotOKPO", r."IzgotDate", r."Nss",
            b.end_mass_net AS "Kg" 
            FROM acc_items_balance(p_date, 'form_1_2', p_org_id) AS b INNER JOIN acc_o2 AS r ON b.last_record_id = r.record_id

    $$;


ALTER FUNCTION public.ar_balance_f12(p_date date, p_org_id integer) OWNER TO db_admin;

--
-- TOC entry 267 (class 1255 OID 91023)
-- Name: ar_balance_f13(date, integer); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.ar_balance_f13(p_date date, p_org_id integer DEFAULT NULL::integer) RETURNS TABLE(org_id integer, "PaspN" character varying, "Typ" character varying, "Nuclid" character varying, "Numb" character varying, "IzgotOKPO" character varying, "IzgotDate" date, "Agr" integer, "Activn" double precision)
    LANGUAGE sql
    AS $$

        SELECT 
            r."org_id", r."PaspN", r."Typ", r."Nuclid", r."Numb", r."IzgotOKPO", r."IzgotDate", r."Agr",
            b.end_activity AS "Activn" 
            FROM acc_items_balance(p_date, 'form_1_3', p_org_id) AS b INNER JOIN acc_o3 AS r ON b.last_record_id = r.record_id

    $$;


ALTER FUNCTION public.ar_balance_f13(p_date date, p_org_id integer) OWNER TO db_admin;

--
-- TOC entry 261 (class 1255 OID 20042)
-- Name: ar_balance_f14(date, integer); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.ar_balance_f14(p_date date, p_org_id integer DEFAULT NULL::integer) RETURNS TABLE(org_id integer, "PaspN" character varying, "IstName" character varying, "Typ" character varying, "Nuclid" character varying, "Arg" integer, "Kbm" double precision, "Kg" double precision, "Activn" double precision)
    LANGUAGE sql
    AS $$

        SELECT 
            r."org_id", r."PaspN", r."IstName", r."Typ", r."Nuclid", r."Arg",
            b.end_volume AS "Kbm", b.end_mass_net AS "Kg", b.end_activity AS "Activn" 
            FROM acc_items_balance(p_date, 'form_1_4', p_org_id) AS b INNER JOIN acc_o4 AS r ON b.last_record_id = r.record_id

    $$;


ALTER FUNCTION public.ar_balance_f14(p_date date, p_org_id integer) OWNER TO db_admin;

--
-- TOC entry 262 (class 1255 OID 20043)
-- Name: ar_balance_f15(date, integer); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.ar_balance_f15(p_date date, p_org_id integer DEFAULT NULL::integer) RETURNS TABLE(org_id integer, "PaspN" character varying, "Typ" character varying, "Nuclid" character varying, "Numb" character varying, "IzgotDate" date, "Sht" bigint, "Activn" double precision)
    LANGUAGE sql
    AS $$

        SELECT 
            r."org_id", r."PaspN", r."Typ", r."Nuclid", r."Numb", r."IzgotDate",
            b.end_amount, b.end_activity 
            FROM acc_items_balance(p_date, 'form_1_5', p_org_id) AS b INNER JOIN acc_o5 AS r ON b.last_record_id = r.record_id

    $$;


ALTER FUNCTION public.ar_balance_f15(p_date date, p_org_id integer) OWNER TO db_admin;

--
-- TOC entry 263 (class 1255 OID 20044)
-- Name: ar_balance_f16(date, integer); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.ar_balance_f16(p_date date, p_org_id integer DEFAULT NULL::integer) RETURNS TABLE(org_id integer, "RAOCod" character varying, "BCod" character varying, "PH_Name" character varying, "PH_Cod" character varying, "PrName" character varying, "UktPrType" character varying, "UktPrN" character varying, "Kbm" double precision, "Tonne" double precision, "Sht" bigint)
    LANGUAGE sql
    AS $$

        SELECT 
            r."org_id", r."RAOCod", r."BCod", r."PH_Name", r."PH_Cod", r."PrName", r."UktPrTyp", r."UktPrN",
            b.end_volume, b.end_mass_net, b.end_amount 
            FROM acc_items_balance(p_date, 'form_1_6', p_org_id) AS b INNER JOIN acc_o6 AS r ON b.last_record_id = r.record_id

    $$;


ALTER FUNCTION public.ar_balance_f16(p_date date, p_org_id integer) OWNER TO db_admin;

--
-- TOC entry 264 (class 1255 OID 20045)
-- Name: ar_balance_f17(date, integer); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.ar_balance_f17(p_date date, p_org_id integer DEFAULT NULL::integer) RETURNS TABLE(org_id integer, g4 character varying, g5 character varying, g6 character varying, g7 character varying, g8 date, g9 character varying, volume double precision, mass_net double precision, activity double precision)
    LANGUAGE sql
    AS $$

        SELECT 
            r."org_id", r."g4", r."g5", r."g6", r."g7", r."g8", r."g9",
            b.end_volume, b.end_mass_net, b.end_activity 
            FROM acc_items_balance(p_date, 'form_1_7', p_org_id) AS b INNER JOIN acc_o7 AS r ON b.last_record_id = r.record_id

    $$;


ALTER FUNCTION public.ar_balance_f17(p_date date, p_org_id integer) OWNER TO db_admin;

--
-- TOC entry 265 (class 1255 OID 20046)
-- Name: ar_balance_f18(date, integer); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.ar_balance_f18(p_date date, p_org_id integer DEFAULT NULL::integer) RETURNS TABLE(org_id integer, g4 character varying, g5 character varying, g18 character varying, g19 character varying, volume double precision, mass_net double precision, activity double precision)
    LANGUAGE sql
    AS $$

        SELECT 
            r."org_id", r."g4", r."g5", r."g18", r."g19", 
            b.end_volume, b.end_mass_net, b.end_activity 
            FROM acc_items_balance(p_date, 'form_1_8', p_org_id) AS b INNER JOIN acc_o8 AS r ON b.last_record_id = r.record_id

    $$;


ALTER FUNCTION public.ar_balance_f18(p_date date, p_org_id integer) OWNER TO db_admin;

--
-- TOC entry 251 (class 1255 OID 20047)
-- Name: ar_balance_f19(date, integer); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.ar_balance_f19(p_date date, p_org_id integer DEFAULT NULL::integer) RETURNS TABLE(org_id integer, g7 character varying)
    LANGUAGE sql
    AS $$

        SELECT 
            r."org_id", r."g7"
            FROM acc_items_balance(p_date, 'form_1_9', p_org_id) AS b INNER JOIN acc_o9 AS r ON b.last_record_id = r.record_id

    $$;


ALTER FUNCTION public.ar_balance_f19(p_date date, p_org_id integer) OWNER TO db_admin;

--
-- TOC entry 259 (class 1255 OID 20049)
-- Name: ar_inv_check_f11(date, integer); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.ar_inv_check_f11(p_date date, p_org_id integer DEFAULT NULL::integer) RETURNS TABLE(org_id integer, "PaspN" character varying, "Typ" character varying, "Nuclid" character varying, "Numb" character varying, "Activn" double precision, "IzgotOKPO" character varying, "IzgotDate" date, "Kateg" character varying, "Nss" double precision, last_inv_date date, prev_inv_date date, "calc_Sht" bigint, "inv_Sht" bigint)
    LANGUAGE sql
    AS $$

        SELECT
            r."org_id", r."PaspN", r."Typ", r."Nuclid", r."Numb", r."Activn", r."IzgotOKPO", r."IzgotDate", r."Kateg", r."Nss",
            b.last_inv_date, b.prev_inv_date, b.calc_amount, b.inv_amount 
            FROM acc_items_check(p_date, 'form_1_1', p_org_id) AS b INNER JOIN acc_o1 AS r ON b.last_record_id = r.record_id

    $$;


ALTER FUNCTION public.ar_inv_check_f11(p_date date, p_org_id integer) OWNER TO db_admin;

--
-- TOC entry 253 (class 1255 OID 20050)
-- Name: ar_inv_check_f12(date, integer); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.ar_inv_check_f12(p_date date, p_org_id integer DEFAULT NULL::integer) RETURNS TABLE(org_id integer, "PaspN" character varying, "Typ" character varying, "Numb" character varying, "IzgotOKPO" character varying, "IzgotDate" date, "Nss" integer, last_inv_date date, prev_inv_date date, "calc_Kg" double precision, "inv_Kg" double precision)
    LANGUAGE sql
    AS $$

        SELECT 
            r."org_id", r."PaspN", r."Typ", r."Numb", r."IzgotOKPO", r."IzgotDate", r."Nss",
            b.last_inv_date, b.prev_inv_date, b.calc_mass_net, b.inv_mass_net
            FROM acc_items_check(p_date, 'form_1_2', p_org_id) AS b INNER JOIN acc_o2 AS r ON b.last_record_id = r.record_id

    $$;


ALTER FUNCTION public.ar_inv_check_f12(p_date date, p_org_id integer) OWNER TO db_admin;

--
-- TOC entry 230 (class 1255 OID 20195)
-- Name: areport_status_update(); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.areport_status_update() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
    BEGIN
        IF NEW.areport_data IS NOT NULL AND NEW.areport_state='wait' THEN
            NEW.areport_state = 'temp';
        END IF;

        RETURN NEW;
    END $$;


ALTER FUNCTION public.areport_status_update() OWNER TO db_admin;

--
-- TOC entry 252 (class 1255 OID 20061)
-- Name: fc_pub(integer); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.fc_pub(p_form_id integer) RETURNS void
    LANGUAGE plpgsql SECURITY DEFINER
    AS $$
    DECLARE
        v_table JSONB;
        v_rec RECORD;
        v_row JSONB;
        v_opcod VARCHAR;
        v_rao_oper INTEGER;
        v_reg_rec_id INTEGER;
        v_cur CURSOR FOR SELECT * FROM jsonb_array_elements(v_table);
    BEGIN
        DELETE FROM "AccRaoForecast" WHERE report_form_id = p_form_id;

        SELECT form_table_data->'main_table' INTO v_table
            FROM "ReportForms"
            WHERE report_form_id = p_form_id;

        
        FOR v_rec IN v_cur LOOP

            INSERT INTO "AccRaoForecast"(report_form_id) 
                VALUES (p_form_id)
                RETURNING record_id INTO v_reg_rec_id;

            v_row = v_rec.value;

            v_opcod = v_row->>'OpCod';

            CASE 
                WHEN v_opcod IN ('11','12','13','14','41') THEN
                    v_rao_oper = 1;
                WHEN v_opcod IN ('44','45','49') THEN
                    v_rao_oper = 2;
                WHEN v_opcod IN ('56','57','59') THEN
                    v_rao_oper = 3;
                ELSE
                    v_rao_oper = 0;
            END CASE;

            UPDATE "AccRaoForecast" SET

                "OpCod"     = v_opcod, 
                "rao_oper"  = v_rao_oper,
                "OpDate"    = (v_row->>'OpDate')::DATE,
                "RAOCod"    = v_row->'RAOCod',
                "BCod"      = v_row->'BCod',
                "RAOCodMax" = v_row->>'RAOCodMax',

                "UpCod"     = v_row->>'UpCod',
                "PH_Name"   = v_row->>'PH_Name',
                "PH_Cod"    = v_row->>'PH_Cod',
                "FCP"       = v_row->>'FCP',
                "obligations" = v_row->>'obligations',
                "work_code" = v_row->>'work_code',
                "ptc_name"  = v_row->>'ptc_name',
                "tech_stage" = v_row->>'tech_stage',
                "tech_oper" = v_row->>'tech_oper',
                "kont_type" = v_row->>'kont_type',
                "rao_class" = v_row->>'rao_class',

                "volume_mult" = (v_row->>'volume_mult')::FLOAT,   

                "Kbm"       = (v_row->>'Kbm')::FLOAT,     
                "Tonne"     = (v_row->>'Tonne')::FLOAT,   
                "Sht"       = ((v_row->>'Sht')::FLOAT)::INTEGER,
                "ActTR"     = (v_row->>'ActTR')::FLOAT,   
                "ActBG"     = (v_row->>'ActBG')::FLOAT,   
                "ActA"      = (v_row->>'ActA')::FLOAT,    
                "ActU"      = (v_row->>'ActU')::FLOAT,    

                "fc_volume"     = (v_row->>'fc_volume')::FLOAT,   
                "fc_kont_qty"   = ((v_row->>'fc_kont_qty')::FLOAT)::INTEGER, 
                "fc_kont_volume" = (v_row->>'fc_kont_volume')::FLOAT, 

                "Subsid"    = (v_row->>'Subsig')::FLOAT   

                WHERE record_id = v_reg_rec_id;

        END LOOP;

    END $$;


ALTER FUNCTION public.fc_pub(p_form_id integer) OWNER TO db_admin;

--
-- TOC entry 255 (class 1255 OID 20255)
-- Name: fc_rao_load(character varying); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.fc_rao_load(p_opcodes character varying) RETURNS json
    LANGUAGE plpgsql
    AS $$
    DECLARE 
        v_cur CURSOR FOR SELECT * FROM "AccRAO" AS r WHERE position(r."OpCod" IN p_opcodes)>0;
        v_res JSONB DEFAULT '[]'::JSONB;
        v_row JSONB;
        v_rec RECORD;
        v_recj JSONB;
        v_field VARCHAR;
        v_fields VARCHAR[];
    BEGIN
        v_fields = string_to_array('OpCod,OpDate', ',');
        RAISE NOTICE 'v_field=%', v_field;

        FOR v_rec IN v_cur LOOP
            v_row = '{}'::JSONB;

            v_recj = row_to_json(v_rec);
            FOREACH v_field IN ARRAY v_fields LOOP
                v_row = v_row || jsonb_build_object(v_field, v_recj->v_field);
            END LOOP;

            v_res = v_res || v_row;
        END LOOP;

        RETURN jsonb_pretty(v_res);
    END $$;


ALTER FUNCTION public.fc_rao_load(p_opcodes character varying) OWNER TO db_admin;

--
-- TOC entry 233 (class 1255 OID 28412)
-- Name: get_info(); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.get_info() RETURNS jsonb
    LANGUAGE sql
    AS $$
        SELECT '{"version":"1.9", "patch":"2022-09-10"}'::JSONB;
    $$;


ALTER FUNCTION public.get_info() OWNER TO db_admin;

--
-- TOC entry 248 (class 1255 OID 20005)
-- Name: get_oper_codes(character varying); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.get_oper_codes(p_value_id character varying) RETURNS character varying
    LANGUAGE sql
    AS $$
        SELECT label 
            FROM "VocValues"
            WHERE voc_type = 'OperSignCodes' AND value_id = p_value_id;
    $$;


ALTER FUNCTION public.get_oper_codes(p_value_id character varying) OWNER TO db_admin;

--
-- TOC entry 249 (class 1255 OID 20006)
-- Name: get_oper_sign(character varying, character varying); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.get_oper_sign(p_form character varying, p_opcode character varying) RETURNS integer
    LANGUAGE plpgsql
    AS $$
    DECLARE 
        v_inc_codes VARCHAR;
        v_dec_codes VARCHAR;
        v_sign INT;
    BEGIN    
        SELECT get_oper_codes(p_form || '-inc') INTO v_inc_codes;
        SELECT get_oper_codes(p_form || '-dec') INTO v_dec_codes;
        
        CASE 
            WHEN position(p_opcode IN v_inc_codes)>0 THEN
                v_sign = 1;

            WHEN position(p_opcode IN v_dec_codes)>0 THEN
                v_sign = -1;

            WHEN position(p_opcode IN '01 10')>0 THEN           -- Начальный ввод или инвенторизация
                v_sign = 0;

            ELSE
                v_sign = NULL;
        END CASE;

        RETURN v_sign;
    END $$;


ALTER FUNCTION public.get_oper_sign(p_form character varying, p_opcode character varying) OWNER TO db_admin;

--
-- TOC entry 256 (class 1255 OID 28254)
-- Name: rename1(jsonb); Type: FUNCTION; Schema: public; Owner: postgres
--

CREATE FUNCTION public.rename1(attrs jsonb) RETURNS jsonb
    LANGUAGE plpgsql
    AS $$
    DECLARE 
        res JSONB;
    BEGIN
        RETURN jsonb_build_object(
    'creation_dt' , attrs->>'date',
    'creator_id' , attrs->>'login',
    'bug_id', attrs->>'bug_id',
    'type', attrs->>'bug_type',
    'status', attrs->>'bug_status',
    'environment', attrs->>'enviroment',
    'description', attrs->>'bug_comments',
    'dev_response', attrs->>'bug_response',
    'user_priority', attrs->>'bug_user_priority',
    'dev_priority', attrs->>'bug_priority'
        );
    END $$;


ALTER FUNCTION public.rename1(attrs jsonb) OWNER TO postgres;

--
-- TOC entry 232 (class 1255 OID 20008)
-- Name: reportform_org_id(integer); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.reportform_org_id(p_report_form_id integer) RETURNS integer
    LANGUAGE sql
    AS $$
        SELECT COALESCE(r.author_id, NULL) 
            FROM "ReportForms" AS f
                LEFT OUTER JOIN "Reports" AS r ON r.report_id = f.report_id 
            WHERE report_form_id = p_report_form_id
    $$;


ALTER FUNCTION public.reportform_org_id(p_report_form_id integer) OWNER TO db_admin;

--
-- TOC entry 231 (class 1255 OID 19814)
-- Name: rf_is_active(integer); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.rf_is_active(p_rf_id integer) RETURNS boolean
    LANGUAGE sql
    AS $$
        SELECT EXISTS(
            SELECT 1 
                FROM reportforms
                WHERE report_form_id = p_rf_id
                    AND form_state IN ('verified', 'confirmed', 'accepted')
        )
    $$;


ALTER FUNCTION public.rf_is_active(p_rf_id integer) OWNER TO db_admin;

--
-- TOC entry 228 (class 1255 OID 19815)
-- Name: rf_table_update(integer, character varying, character varying); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.rf_table_update(p_form_id integer, p_data_str character varying, p_table_key character varying) RETURNS void
    LANGUAGE plpgsql SECURITY DEFINER
    AS $$
    DECLARE
        v_table_data JSONB;
    BEGIN
        SELECT form_table_data INTO v_table_data
            FROM "ReportForms"
            WHERE report_form_id = p_form_id;

        IF v_table_data IS NULL THEN
            v_table_data = '{}'::JSONB;
        ELSE
            v_table_data = v_table_data - p_table_key;
        END IF;

        v_table_data = v_table_data || jsonb_build_object(p_table_key, p_data_str::JSONB);

        UPDATE "ReportForms" SET
            form_table_data = v_table_data
            WHERE report_form_id = p_form_id;

    END $$;


ALTER FUNCTION public.rf_table_update(p_form_id integer, p_data_str character varying, p_table_key character varying) OWNER TO db_admin;

--
-- TOC entry 229 (class 1255 OID 20002)
-- Name: tgfn_acc_moves_delete(); Type: FUNCTION; Schema: public; Owner: db_admin
--

CREATE FUNCTION public.tgfn_acc_moves_delete() RETURNS trigger
    LANGUAGE plpgsql SECURITY DEFINER
    AS $$
    BEGIN
        DELETE FROM "AccMoves" WHERE record_id = OLD.record_id;
        RAISE NOTICE 'Deleted %', OLD.record_id;
        RETURN OLD;
    END $$;


ALTER FUNCTION public.tgfn_acc_moves_delete() OWNER TO db_admin;

SET default_tablespace = '';

--
-- TOC entry 204 (class 1259 OID 19823)
-- Name: AReports; Type: TABLE; Schema: public; Owner: db_admin
--

CREATE TABLE public."AReports" (
    areport_id integer NOT NULL,
    areport_name character varying,
    areport_ts timestamp without time zone DEFAULT CURRENT_TIMESTAMP,
    areport_period jsonb,
    areport_data jsonb,
    areport_state public.areport_state_type DEFAULT 'wait'::public.areport_state_type,
    areport_creator integer,
    areport_params jsonb
);


ALTER TABLE public."AReports" OWNER TO db_admin;

--
-- TOC entry 203 (class 1259 OID 19821)
-- Name: AReports_areport_id_seq; Type: SEQUENCE; Schema: public; Owner: db_admin
--

CREATE SEQUENCE public."AReports_areport_id_seq"
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public."AReports_areport_id_seq" OWNER TO db_admin;

--
-- TOC entry 3157 (class 0 OID 0)
-- Dependencies: 203
-- Name: AReports_areport_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: db_admin
--

ALTER SEQUENCE public."AReports_areport_id_seq" OWNED BY public."AReports".areport_id;


--
-- TOC entry 205 (class 1259 OID 19997)
-- Name: AccMoves; Type: TABLE; Schema: public; Owner: db_admin
--

CREATE TABLE public."AccMoves" (
    record_id integer NOT NULL,
    mass_net double precision,
    mass_gross double precision,
    volume double precision,
    amount integer,
    activity double precision,
    act_tr double precision,
    act_ur double precision,
    act_a double precision,
    act_b double precision
);


ALTER TABLE public."AccMoves" OWNER TO db_admin;

--
-- TOC entry 202 (class 1259 OID 19794)
-- Name: ReportForms; Type: TABLE; Schema: public; Owner: db_admin
--

CREATE TABLE public."ReportForms" (
    report_form_id integer NOT NULL,
    report_id integer,
    form_state public.form_state_type DEFAULT 'draft'::public.form_state_type,
    form_no character varying,
    form_part character varying,
    form_version character varying,
    form_name character varying,
    form_main_data jsonb,
    form_table_data jsonb,
    form_remarks jsonb,
    form_comments jsonb,
    form_warnings jsonb,
    kod_org character varying,
    sign_info jsonb,
    fill_info jsonb,
    form_period_start date,
    form_period_end date,
    adjustment_no integer DEFAULT 0,
    user_id integer,
    author_id integer,
    form_export_ts timestamp without time zone,
    form_import_ts timestamp without time zone,
    reporting_year integer,
    notification_date date,
    form_comment jsonb,
    form_errors jsonb,
    uniq_numb character varying,
    import_author character varying,
    import_src_id character varying,
    import_hash character varying,
    form_notifications jsonb,
    log_id integer
);


ALTER TABLE public."ReportForms" OWNER TO db_admin;

--
-- TOC entry 201 (class 1259 OID 19792)
-- Name: ReportForms_report_form_id_seq; Type: SEQUENCE; Schema: public; Owner: db_admin
--

CREATE SEQUENCE public."ReportForms_report_form_id_seq"
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public."ReportForms_report_form_id_seq" OWNER TO db_admin;

--
-- TOC entry 3160 (class 0 OID 0)
-- Dependencies: 201
-- Name: ReportForms_report_form_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: db_admin
--

ALTER SEQUENCE public."ReportForms_report_form_id_seq" OWNED BY public."ReportForms".report_form_id;


--
-- TOC entry 199 (class 1259 OID 19730)
-- Name: VocValues; Type: TABLE; Schema: public; Owner: db_admin
--

CREATE TABLE public."VocValues" (
    record_id integer NOT NULL,
    voc_type character varying,
    value_id character varying,
    label character varying,
    attrs jsonb,
    valid_from date,
    valid_to date,
    value_status integer,
    creator_id character varying,
    kod_org character varying DEFAULT '0'::character varying,
    attrs1 jsonb,
    import_author character varying,
    import_src_id character varying,
    import_hash character varying,
    log_id integer
);


ALTER TABLE public."VocValues" OWNER TO db_admin;

--
-- TOC entry 200 (class 1259 OID 19749)
-- Name: VocOrgs; Type: TABLE; Schema: public; Owner: db_admin
--

CREATE TABLE public."VocOrgs" (
    "KOD_ORG" character varying,
    "ORG_TYPE" character varying,
    "KOD_IAE" character varying,
    "KOD_SRF" character varying,
    "NAME_FULL" character varying,
    "NAME_SHORT" character varying,
    "ADDRESS_U" character varying,
    "ADDRESS_P" character varying,
    "RUKOVODITEL" character varying,
    "OKPO" character varying,
    "OKVED" character varying,
    "OKATO" character varying,
    "OKTMO" character varying,
    "INN" character varying,
    "KPP" character varying,
    "OKOPF" character varying,
    "OKFS" character varying,
    "PRIM" character varying,
    "PRZN_ORG" integer,
    "PRZN_OP" integer,
    "KOD_GOL_ORG" character varying,
    "PRZN_ACTIV" integer,
    "LVL_ORG" integer,
    "OKPO_PRP" character varying,
    "RIAC" character varying,
    "VIAC" character varying,
    "OUIAE" character varying,
    "DT_KORR" date,
    "USER_KORR" character varying,
    "DT_REG" date,
    "DT_SU" date,
    category1 character varying,
    category2 character varying,
    category3 character varying,
    category4 character varying,
    category5 character varying,
    "YROO" boolean
)
INHERITS (public."VocValues");


ALTER TABLE public."VocOrgs" OWNER TO db_admin;

--
-- TOC entry 198 (class 1259 OID 19728)
-- Name: VocValues_record_id_seq; Type: SEQUENCE; Schema: public; Owner: db_admin
--

CREATE SEQUENCE public."VocValues_record_id_seq"
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public."VocValues_record_id_seq" OWNER TO db_admin;

--
-- TOC entry 3163 (class 0 OID 0)
-- Dependencies: 198
-- Name: VocValues_record_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: db_admin
--

ALTER SEQUENCE public."VocValues_record_id_seq" OWNED BY public."VocValues".record_id;


--
-- TOC entry 197 (class 1259 OID 19719)
-- Name: Vocs; Type: TABLE; Schema: public; Owner: db_admin
--

CREATE TABLE public."Vocs" (
    voc_id integer NOT NULL,
    voc_name character varying,
    voc_type character varying
);


ALTER TABLE public."Vocs" OWNER TO db_admin;

--
-- TOC entry 196 (class 1259 OID 19717)
-- Name: Vocs_voc_id_seq; Type: SEQUENCE; Schema: public; Owner: db_admin
--

CREATE SEQUENCE public."Vocs_voc_id_seq"
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public."Vocs_voc_id_seq" OWNER TO db_admin;

--
-- TOC entry 3165 (class 0 OID 0)
-- Dependencies: 196
-- Name: Vocs_voc_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: db_admin
--

ALTER SEQUENCE public."Vocs_voc_id_seq" OWNED BY public."Vocs".voc_id;


--
-- TOC entry 227 (class 1259 OID 100132)
-- Name: acc_fc; Type: TABLE; Schema: public; Owner: user1
--

CREATE TABLE public.acc_fc (
    record_id integer NOT NULL,
    report_form_id integer,
    "OpCod" character varying,
    "OpDate" date,
    "RAOCod" character varying,
    "BCod" character varying,
    "Kbm" double precision,
    "Tonne" double precision,
    "Sht" integer,
    "ActTR" double precision,
    "ActBG" double precision,
    "ActA" double precision,
    "ActU" double precision,
    "DocN" character varying,
    "UpCod" character varying,
    "UktPrN" character varying,
    "PH_Name" character varying,
    "PH_Cod" character varying,
    "Subsid" character varying,
    "FCP" character varying,
    obligations character varying,
    work_code character varying,
    ptc_name character varying,
    tech_stage character varying,
    tech_oper character varying,
    kont_type character varying,
    rao_class character varying,
    volume_mult double precision,
    fc_volume double precision,
    fc_kont_qty double precision,
    fc_kont_volume double precision
);


ALTER TABLE public.acc_fc OWNER TO user1;

--
-- TOC entry 226 (class 1259 OID 100130)
-- Name: acc_fc_record_id_seq; Type: SEQUENCE; Schema: public; Owner: user1
--

CREATE SEQUENCE public.acc_fc_record_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.acc_fc_record_id_seq OWNER TO user1;

--
-- TOC entry 3166 (class 0 OID 0)
-- Dependencies: 226
-- Name: acc_fc_record_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: user1
--

ALTER SEQUENCE public.acc_fc_record_id_seq OWNED BY public.acc_fc.record_id;


--
-- TOC entry 213 (class 1259 OID 32198)
-- Name: acc_proto; Type: TABLE; Schema: public; Owner: db_admin
--

CREATE TABLE public.acc_proto (
    record_id integer NOT NULL,
    report_form_id integer,
    table_row integer,
    org_id integer,
    kod_org character varying
);


ALTER TABLE public.acc_proto OWNER TO db_admin;

--
-- TOC entry 214 (class 1259 OID 32209)
-- Name: acc_oper; Type: TABLE; Schema: public; Owner: db_admin
--

CREATE TABLE public.acc_oper (
    "OpCod" character varying,
    "OpDate" date,
    op_item character varying,
    op_sign integer,
    mass_net double precision,
    mass_gross double precision,
    volume double precision,
    amount integer,
    activity double precision,
    act_tr double precision,
    act_ur double precision,
    act_a double precision,
    act_b double precision
)
INHERITS (public.acc_proto);


ALTER TABLE public.acc_oper OWNER TO db_admin;

--
-- TOC entry 215 (class 1259 OID 32216)
-- Name: acc_o1; Type: TABLE; Schema: public; Owner: db_admin
--

CREATE TABLE public.acc_o1 (
    "PaspN" character varying,
    "Typ" character varying,
    "Nuclid" character varying,
    "Numb" character varying,
    "Sht" integer,
    "Activn" double precision,
    "IzgotOKPO" character varying,
    "IzgotDate" date,
    "Kateg" character varying,
    "Nss" double precision,
    "DocVid" integer,
    "DocN" character varying,
    "DocDate" date,
    "OkpoPIP" character varying,
    "OkpoPrv" character varying,
    "PrName" character varying,
    "UktPrTyp" character varying,
    "UktPrN" character varying,
    "FormSobst" character varying,
    "Pravoobl" character varying
)
INHERITS (public.acc_oper);


ALTER TABLE public.acc_o1 OWNER TO db_admin;

--
-- TOC entry 216 (class 1259 OID 32224)
-- Name: acc_o2; Type: TABLE; Schema: public; Owner: db_admin
--

CREATE TABLE public.acc_o2 (
    "PaspN" character varying,
    "Typ" character varying,
    "Nuclid" character varying,
    "Numb" character varying,
    "Kg" double precision,
    "IzgotOKPO" character varying,
    "IzgotDate" date,
    "Nss" integer,
    "VidIzd" character varying,
    "DocVid" integer,
    "DocN" character varying,
    "DocDate" date,
    "OkpoPIP" character varying,
    "OkpoPrv" character varying,
    "PrName" character varying,
    "UktPrTyp" character varying,
    "UktPrN" character varying,
    "FormSobst" character varying,
    "Pravoobl" character varying
)
INHERITS (public.acc_oper);


ALTER TABLE public.acc_o2 OWNER TO db_admin;

--
-- TOC entry 217 (class 1259 OID 32231)
-- Name: acc_o3; Type: TABLE; Schema: public; Owner: db_admin
--

CREATE TABLE public.acc_o3 (
    "PaspN" character varying,
    "Typ" character varying,
    "Nuclid" character varying,
    "Numb" character varying,
    "Activn" double precision,
    "IzgotOKPO" character varying,
    "IzgotDate" date,
    "Agr" integer,
    "DocVid" integer,
    "DocN" character varying,
    "DocDate" date,
    "OkpoPIP" character varying,
    "OkpoPrv" character varying,
    "PrName" character varying,
    "UktPrTyp" character varying,
    "UktPrN" character varying,
    "FormSobst" character varying,
    "Pravoobl" character varying
)
INHERITS (public.acc_oper);


ALTER TABLE public.acc_o3 OWNER TO db_admin;

--
-- TOC entry 218 (class 1259 OID 32238)
-- Name: acc_o4; Type: TABLE; Schema: public; Owner: db_admin
--

CREATE TABLE public.acc_o4 (
    "PaspN" character varying,
    "IstName" character varying,
    "Typ" character varying,
    "Nuclid" character varying,
    "Activn" double precision,
    "ActDate" date,
    "Kbm" double precision,
    "Kg" double precision,
    "Arg" integer,
    "DocVid" integer,
    "DocN" character varying,
    "DocDate" date,
    "OkpoPIP" character varying,
    "OkpoPrv" character varying,
    "PrName" character varying,
    "UktPrTyp" character varying,
    "UktPrN" character varying,
    "FormSobst" character varying,
    "Pravoobl" character varying
)
INHERITS (public.acc_oper);


ALTER TABLE public.acc_o4 OWNER TO db_admin;

--
-- TOC entry 219 (class 1259 OID 32245)
-- Name: acc_o5; Type: TABLE; Schema: public; Owner: db_admin
--

CREATE TABLE public.acc_o5 (
    "PaspN" character varying,
    "Typ" character varying,
    "Nuclid" character varying,
    "Numb" character varying,
    "Sht" integer,
    "Activn" double precision,
    "IzgotDate" date,
    "Kateg" character varying,
    "BCod" character varying,
    "DocVid" integer,
    "DocN" character varying,
    "DocDate" date,
    "OkpoPIP" character varying,
    "OkpoPrv" character varying,
    "PrName" character varying,
    "UktPrTyp" character varying,
    "UktPrN" character varying,
    "PH_Name" character varying,
    "PH_Cod" character varying,
    factory character varying,
    "CodRAO" character varying,
    "Subsid" character varying,
    "FCP" character varying
)
INHERITS (public.acc_oper);


ALTER TABLE public.acc_o5 OWNER TO db_admin;

--
-- TOC entry 220 (class 1259 OID 32252)
-- Name: acc_o6; Type: TABLE; Schema: public; Owner: db_admin
--

CREATE TABLE public.acc_o6 (
    "RAOCod" character varying,
    "BCod" character varying,
    "Kbm" double precision,
    "Tonne" double precision,
    "Sht" double precision,
    "RAOCodMax" character varying,
    "RAOCodCat" character varying,
    "RAOCodDang" character varying,
    "ActTR" double precision,
    "ActBG" double precision,
    "ActA" double precision,
    "ActUR" double precision,
    "ActDate" date,
    "DocVid" integer,
    "DocN" character varying,
    "DocDate" date,
    "OkpoPIP" character varying,
    "OkpoPrv" character varying,
    "PH_Name" character varying,
    "PH_Cod" character varying,
    factory character varying,
    "UpCod" character varying,
    "PrName" character varying,
    "UktPrTyp" character varying,
    "UktPrN" character varying,
    "Subsid" double precision,
    "FCP" character varying
)
INHERITS (public.acc_oper);


ALTER TABLE public.acc_o6 OWNER TO db_admin;

--
-- TOC entry 221 (class 1259 OID 32259)
-- Name: acc_o7; Type: TABLE; Schema: public; Owner: db_admin
--

CREATE TABLE public.acc_o7 (
    g4 character varying,
    g5 character varying,
    g6 character varying,
    g7 character varying,
    g8 date,
    g9 character varying,
    g10 double precision,
    g11 double precision,
    g12 character varying,
    g13 double precision,
    g14 character varying,
    g15 character varying,
    g16 date,
    g17 character varying,
    g18 character varying,
    g19 character varying,
    g20 character varying,
    g21 character varying,
    g22 character varying,
    g23 double precision,
    g24 double precision,
    g25 integer,
    g26 double precision,
    g27 double precision,
    g28 double precision,
    g29 double precision,
    factory character varying,
    g30 character varying,
    g31 double precision,
    g32 character varying
)
INHERITS (public.acc_oper);


ALTER TABLE public.acc_o7 OWNER TO db_admin;

--
-- TOC entry 222 (class 1259 OID 32266)
-- Name: acc_o8; Type: TABLE; Schema: public; Owner: db_admin
--

CREATE TABLE public.acc_o8 (
    g4 character varying,
    g5 character varying,
    g6 double precision,
    g7 double precision,
    g8 double precision,
    g9 character varying,
    g10 double precision,
    g11 character varying,
    g12 character varying,
    g13 date,
    g14 character varying,
    g15 character varying,
    g16 character varying,
    g17 character varying,
    g18 character varying,
    g19 character varying,
    g20 double precision,
    g21 double precision,
    g22 double precision,
    g23 double precision,
    g24 double precision,
    g25 double precision,
    factory character varying,
    g26 character varying,
    g27 double precision,
    g28 character varying
)
INHERITS (public.acc_oper);


ALTER TABLE public.acc_o8 OWNER TO db_admin;

--
-- TOC entry 223 (class 1259 OID 32273)
-- Name: acc_o9; Type: TABLE; Schema: public; Owner: db_admin
--

CREATE TABLE public.acc_o9 (
    g4 character varying,
    g5 character varying,
    g6 date,
    g7 character varying,
    g8 character varying,
    g9 double precision
)
INHERITS (public.acc_oper);


ALTER TABLE public.acc_o9 OWNER TO db_admin;

--
-- TOC entry 212 (class 1259 OID 32196)
-- Name: acc_proto_record_id_seq; Type: SEQUENCE; Schema: public; Owner: db_admin
--

CREATE SEQUENCE public.acc_proto_record_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.acc_proto_record_id_seq OWNER TO db_admin;

--
-- TOC entry 3167 (class 0 OID 0)
-- Dependencies: 212
-- Name: acc_proto_record_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: db_admin
--

ALTER SEQUENCE public.acc_proto_record_id_seq OWNED BY public.acc_proto.record_id;


--
-- TOC entry 207 (class 1259 OID 28183)
-- Name: bugreports; Type: TABLE; Schema: public; Owner: db_admin
--

CREATE TABLE public.bugreports (
    record_id integer NOT NULL,
    attrs jsonb DEFAULT '{}'::jsonb,
    bug_id character varying,
    creator_id character varying,
    status character varying DEFAULT 'draft'::character varying,
    creation_dt timestamp without time zone DEFAULT now()
);


ALTER TABLE public.bugreports OWNER TO db_admin;

--
-- TOC entry 206 (class 1259 OID 28181)
-- Name: bugreports_record_id_seq; Type: SEQUENCE; Schema: public; Owner: db_admin
--

CREATE SEQUENCE public.bugreports_record_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.bugreports_record_id_seq OWNER TO db_admin;

--
-- TOC entry 3168 (class 0 OID 0)
-- Dependencies: 206
-- Name: bugreports_record_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: db_admin
--

ALTER SEQUENCE public.bugreports_record_id_seq OWNED BY public.bugreports.record_id;


--
-- TOC entry 225 (class 1259 OID 40186)
-- Name: hints; Type: TABLE; Schema: public; Owner: db_admin
--

CREATE TABLE public.hints (
    record_id integer NOT NULL,
    item_type character varying,
    item_class character varying,
    attrs jsonb
);


ALTER TABLE public.hints OWNER TO db_admin;

--
-- TOC entry 224 (class 1259 OID 40184)
-- Name: hints_record_id_seq; Type: SEQUENCE; Schema: public; Owner: db_admin
--

CREATE SEQUENCE public.hints_record_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.hints_record_id_seq OWNER TO db_admin;

--
-- TOC entry 3169 (class 0 OID 0)
-- Dependencies: 224
-- Name: hints_record_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: db_admin
--

ALTER SEQUENCE public.hints_record_id_seq OWNED BY public.hints.record_id;


--
-- TOC entry 211 (class 1259 OID 28228)
-- Name: user_logs; Type: TABLE; Schema: public; Owner: db_admin
--

CREATE TABLE public.user_logs (
    record_id integer NOT NULL,
    status character varying DEFAULT 'draft'::character varying,
    attrs jsonb DEFAULT '{}'::jsonb,
    action character varying
);


ALTER TABLE public.user_logs OWNER TO db_admin;

--
-- TOC entry 210 (class 1259 OID 28226)
-- Name: user_logs_record_id_seq; Type: SEQUENCE; Schema: public; Owner: db_admin
--

CREATE SEQUENCE public.user_logs_record_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.user_logs_record_id_seq OWNER TO db_admin;

--
-- TOC entry 3170 (class 0 OID 0)
-- Dependencies: 210
-- Name: user_logs_record_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: db_admin
--

ALTER SEQUENCE public.user_logs_record_id_seq OWNED BY public.user_logs.record_id;


--
-- TOC entry 209 (class 1259 OID 28212)
-- Name: users; Type: TABLE; Schema: public; Owner: db_admin
--

CREATE TABLE public.users (
    record_id integer NOT NULL,
    fio character varying,
    is_blocked boolean DEFAULT false,
    login character varying,
    password character varying,
    cookie character varying,
    user_role character varying,
    permissions integer,
    organizations character varying,
    creator_id character varying,
    kod_org character varying DEFAULT '0'::character varying,
    kod_org_default character varying,
    attrs jsonb DEFAULT '{}'::jsonb,
    org_id integer,
    organizations_ids character varying
);


ALTER TABLE public.users OWNER TO db_admin;

--
-- TOC entry 208 (class 1259 OID 28210)
-- Name: users_record_id_seq; Type: SEQUENCE; Schema: public; Owner: db_admin
--

CREATE SEQUENCE public.users_record_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.users_record_id_seq OWNER TO db_admin;

--
-- TOC entry 3171 (class 0 OID 0)
-- Dependencies: 208
-- Name: users_record_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: db_admin
--

ALTER SEQUENCE public.users_record_id_seq OWNED BY public.users.record_id;


--
-- TOC entry 2945 (class 2604 OID 19826)
-- Name: AReports areport_id; Type: DEFAULT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public."AReports" ALTER COLUMN areport_id SET DEFAULT nextval('public."AReports_areport_id_seq"'::regclass);


--
-- TOC entry 2942 (class 2604 OID 19797)
-- Name: ReportForms report_form_id; Type: DEFAULT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public."ReportForms" ALTER COLUMN report_form_id SET DEFAULT nextval('public."ReportForms_report_form_id_seq"'::regclass);


--
-- TOC entry 2940 (class 2604 OID 19752)
-- Name: VocOrgs record_id; Type: DEFAULT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public."VocOrgs" ALTER COLUMN record_id SET DEFAULT nextval('public."VocValues_record_id_seq"'::regclass);


--
-- TOC entry 2941 (class 2604 OID 20214)
-- Name: VocOrgs kod_org; Type: DEFAULT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public."VocOrgs" ALTER COLUMN kod_org SET DEFAULT '0'::character varying;


--
-- TOC entry 2938 (class 2604 OID 19733)
-- Name: VocValues record_id; Type: DEFAULT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public."VocValues" ALTER COLUMN record_id SET DEFAULT nextval('public."VocValues_record_id_seq"'::regclass);


--
-- TOC entry 2937 (class 2604 OID 19722)
-- Name: Vocs voc_id; Type: DEFAULT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public."Vocs" ALTER COLUMN voc_id SET DEFAULT nextval('public."Vocs_voc_id_seq"'::regclass);


--
-- TOC entry 2971 (class 2604 OID 100135)
-- Name: acc_fc record_id; Type: DEFAULT; Schema: public; Owner: user1
--

ALTER TABLE ONLY public.acc_fc ALTER COLUMN record_id SET DEFAULT nextval('public.acc_fc_record_id_seq'::regclass);


--
-- TOC entry 2961 (class 2604 OID 32219)
-- Name: acc_o1 record_id; Type: DEFAULT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public.acc_o1 ALTER COLUMN record_id SET DEFAULT nextval('public.acc_proto_record_id_seq'::regclass);


--
-- TOC entry 2962 (class 2604 OID 32227)
-- Name: acc_o2 record_id; Type: DEFAULT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public.acc_o2 ALTER COLUMN record_id SET DEFAULT nextval('public.acc_proto_record_id_seq'::regclass);


--
-- TOC entry 2963 (class 2604 OID 32234)
-- Name: acc_o3 record_id; Type: DEFAULT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public.acc_o3 ALTER COLUMN record_id SET DEFAULT nextval('public.acc_proto_record_id_seq'::regclass);


--
-- TOC entry 2964 (class 2604 OID 32241)
-- Name: acc_o4 record_id; Type: DEFAULT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public.acc_o4 ALTER COLUMN record_id SET DEFAULT nextval('public.acc_proto_record_id_seq'::regclass);


--
-- TOC entry 2965 (class 2604 OID 32248)
-- Name: acc_o5 record_id; Type: DEFAULT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public.acc_o5 ALTER COLUMN record_id SET DEFAULT nextval('public.acc_proto_record_id_seq'::regclass);


--
-- TOC entry 2966 (class 2604 OID 32255)
-- Name: acc_o6 record_id; Type: DEFAULT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public.acc_o6 ALTER COLUMN record_id SET DEFAULT nextval('public.acc_proto_record_id_seq'::regclass);


--
-- TOC entry 2967 (class 2604 OID 32262)
-- Name: acc_o7 record_id; Type: DEFAULT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public.acc_o7 ALTER COLUMN record_id SET DEFAULT nextval('public.acc_proto_record_id_seq'::regclass);


--
-- TOC entry 2968 (class 2604 OID 32269)
-- Name: acc_o8 record_id; Type: DEFAULT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public.acc_o8 ALTER COLUMN record_id SET DEFAULT nextval('public.acc_proto_record_id_seq'::regclass);


--
-- TOC entry 2969 (class 2604 OID 32276)
-- Name: acc_o9 record_id; Type: DEFAULT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public.acc_o9 ALTER COLUMN record_id SET DEFAULT nextval('public.acc_proto_record_id_seq'::regclass);


--
-- TOC entry 2960 (class 2604 OID 32212)
-- Name: acc_oper record_id; Type: DEFAULT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public.acc_oper ALTER COLUMN record_id SET DEFAULT nextval('public.acc_proto_record_id_seq'::regclass);


--
-- TOC entry 2959 (class 2604 OID 32201)
-- Name: acc_proto record_id; Type: DEFAULT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public.acc_proto ALTER COLUMN record_id SET DEFAULT nextval('public.acc_proto_record_id_seq'::regclass);


--
-- TOC entry 2948 (class 2604 OID 28186)
-- Name: bugreports record_id; Type: DEFAULT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public.bugreports ALTER COLUMN record_id SET DEFAULT nextval('public.bugreports_record_id_seq'::regclass);


--
-- TOC entry 2970 (class 2604 OID 40189)
-- Name: hints record_id; Type: DEFAULT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public.hints ALTER COLUMN record_id SET DEFAULT nextval('public.hints_record_id_seq'::regclass);


--
-- TOC entry 2956 (class 2604 OID 28231)
-- Name: user_logs record_id; Type: DEFAULT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public.user_logs ALTER COLUMN record_id SET DEFAULT nextval('public.user_logs_record_id_seq'::regclass);


--
-- TOC entry 2952 (class 2604 OID 28215)
-- Name: users record_id; Type: DEFAULT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public.users ALTER COLUMN record_id SET DEFAULT nextval('public.users_record_id_seq'::regclass);


--
-- TOC entry 3126 (class 0 OID 19823)
-- Dependencies: 204
-- Data for Name: AReports; Type: TABLE DATA; Schema: public; Owner: db_admin
--



--
-- TOC entry 3127 (class 0 OID 19997)
-- Dependencies: 205
-- Data for Name: AccMoves; Type: TABLE DATA; Schema: public; Owner: db_admin
--



--
-- TOC entry 3124 (class 0 OID 19794)
-- Dependencies: 202
-- Data for Name: ReportForms; Type: TABLE DATA; Schema: public; Owner: db_admin
--



--
-- TOC entry 3122 (class 0 OID 19749)
-- Dependencies: 200
-- Data for Name: VocOrgs; Type: TABLE DATA; Schema: public; Owner: db_admin
--



--
-- TOC entry 3121 (class 0 OID 19730)
-- Dependencies: 199
-- Data for Name: VocValues; Type: TABLE DATA; Schema: public; Owner: db_admin
--

INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'скандий-44', 'скандий-44', '{"AM": 44, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.03, "Num_TM": 21, "Name_RN": "скандий-44", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.93, "Vid_izluch": "бета", "Name_RN_Lat": "Sc-44", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сурьма-120м', 'сурьма-120м', '{"AM": 120, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 51, "Name_RN": "сурьма-120м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 5.76, "Vid_izluch": "бета", "Name_RN_Lat": "Sb-120m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сурьма-124м', 'сурьма-124м', '{"AM": 124, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 51, "Name_RN": "сурьма-124м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.337, "Vid_izluch": "гамма", "Name_RN_Lat": "Sb-124m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'теллур-129', 'теллур-129', '{"AM": 129, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 1, "Num_TM": 52, "Name_RN": "теллур-129", "UdA_GRO": "", "UdA_TRO": 100.0, "ObA_GaRO": 2300.0, "Kod_gruppy": "б", "Period_p_r": 1.16, "Vid_izluch": "бета", "Name_RN_Lat": "Te-129", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иод-124', 'иод-124', '{"AM": 124, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.06, "Num_TM": 53, "Name_RN": "иод-124", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 4.18, "Vid_izluch": "бета", "Name_RN_Lat": "I-124", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ксенон-133', 'ксенон-133', '{"AM": 133, "MOI": "", "MZA": 10000.0, "MZUA": 1000000.0, "D_val": 3, "Num_TM": 54, "Name_RN": "ксенон-133", "UdA_GRO": "", "UdA_TRO": 1000.0, "ObA_GaRO": 2200.0, "Kod_gruppy": "б", "Period_p_r": 5.27, "Vid_izluch": "бета", "Name_RN_Lat": "Xe-133", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цезий-134м', 'цезий-134м', '{"AM": 134, "MOI": "", "MZA": 100000.0, "MZUA": 1000000.0, "D_val": 0.04, "Num_TM": 55, "Name_RN": "цезий-134м", "UdA_GRO": 680.0, "UdA_TRO": 1000.0, "ObA_GaRO": 6100.0, "Kod_gruppy": "б", "Period_p_r": 2.9, "Vid_izluch": "гамма", "Name_RN_Lat": "Cs-134m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ксенон-125', 'ксенон-125', '{"AM": 125, "MOI": "", "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 54, "Name_RN": "ксенон-125", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 16.9, "Vid_izluch": "бета", "Name_RN_Lat": "Xe-125", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'церий-139', 'церий-139', '{"AM": 139, "MOI": 1.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.6, "Num_TM": 58, "Name_RN": "церий-139", "UdA_GRO": 5.3, "UdA_TRO": 100.0, "ObA_GaRO": 65.0, "Kod_gruppy": "б", "Period_p_r": 138.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Ce-139", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'церий-134', 'церий-134', '{"AM": 134, "MOI": "", "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 58, "Name_RN": "церий-134", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Ce-134", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'прометий-148м', 'прометий-148м', '{"AM": 148, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.03, "Num_TM": 61, "Name_RN": "прометий-148м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 41.3, "Vid_izluch": "бета", "Name_RN_Lat": "Pm-148m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'европий-148', 'европий-148', '{"AM": 148, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.03, "Num_TM": 63, "Name_RN": "европий-148", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 54.5, "Vid_izluch": "бета", "Name_RN_Lat": "Eu-148", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'самарий-155', 'самарий-155', '{"AM": 155, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 62, "Name_RN": "самарий-155", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.368, "Vid_izluch": "бета", "Name_RN_Lat": "Sm-155", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тербий-160', 'тербий-160', '{"AM": 160, "MOI": 1.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.06, "Num_TM": 65, "Name_RN": "тербий-160", "UdA_GRO": 8.6, "UdA_TRO": 10.0, "ObA_GaRO": 16.0, "Kod_gruppy": "б", "Period_p_r": 72.3, "Vid_izluch": "бета", "Name_RN_Lat": "Tb-160", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тербий-156м2', 'тербий-156м2', '{"AM": 156, "MOI": "", "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 65, "Name_RN": "тербий-156м2", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 5.3, "Vid_izluch": "гамма", "Name_RN_Lat": "Tb-156m2", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гольмий-162', 'гольмий-162', '{"AM": 162, "MOI": "", "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": "", "Num_TM": 67, "Name_RN": "гольмий-162", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.25, "Vid_izluch": "бета", "Name_RN_Lat": "Ho-162", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тулий-173', 'тулий-173', '{"AM": 173, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 69, "Name_RN": "тулий-173", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 8.24, "Vid_izluch": "бета", "Name_RN_Lat": "Tm-173", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гафний-182', 'гафний-182', '{"AM": 182, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.05, "Num_TM": 72, "Name_RN": "гафний-182", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 8900000.0, "Vid_izluch": "бета", "Name_RN_Lat": "Hf-182", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гафний-180м', 'гафний-180м', '{"AM": 180, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 72, "Name_RN": "гафний-180м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 5.5, "Vid_izluch": "гамма", "Name_RN_Lat": "Hf-180m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'вольфрам-188', 'вольфрам-188', '{"AM": 188, "MOI": "", "MZA": 100000.0, "MZUA": 100000.0, "D_val": 1, "Num_TM": 74, "Name_RN": "вольфрам-188", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 69.78, "Vid_izluch": "бета", "Name_RN_Lat": "W-188", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тантал-186', 'тантал-186', '{"AM": 186, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 73, "Name_RN": "тантал-186", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.175, "Vid_izluch": "бета", "Name_RN_Lat": "Ta-186", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рений-189', 'рений-189', '{"AM": 189, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 1, "Num_TM": 75, "Name_RN": "рений-189", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.01, "Vid_izluch": "бета", "Name_RN_Lat": "Re-189", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рений-182м', 'рений-182м', '{"AM": 182, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 75, "Name_RN": "рений-182м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 12.7, "Vid_izluch": "бета", "Name_RN_Lat": "Re-182m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рений-182', 'рений-182', '{"AM": 182, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 75, "Name_RN": "рений-182", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.67, "Vid_izluch": "гамма", "Name_RN_Lat": "Re-182", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иридий-190м', 'иридий-190м', '{"AM": 190, "MOI": "", "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": "", "Num_TM": 77, "Name_RN": "иридий-190м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.2, "Vid_izluch": "гамма", "Name_RN_Lat": "Ir-190m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иридий-186м', 'иридий-186м', '{"AM": 186, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 77, "Name_RN": "иридий-186м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.75, "Vid_izluch": "бета", "Name_RN_Lat": "Ir-186m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'платина-200', 'платина-200', '{"AM": 200, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 78, "Name_RN": "платина-200", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 12.5, "Vid_izluch": "бета", "Name_RN_Lat": "Pt-200", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'плутоний-238', 'плутоний-238', '{"AM": 238, "MOI": 0.1, "MZA": 10000.0, "MZUA": 1000.0, "D_val": 0.06, "Num_TM": 94, "Name_RN": "плутоний-238", "UdA_GRO": 0.061, "UdA_TRO": 1.0, "ObA_GaRO": 0.0027, "Kod_gruppy": "Трансурановый", "Period_p_r": 87.7, "Vid_izluch": "альфа", "Name_RN_Lat": "Pu-238", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кюрий-247', 'кюрий-247', '{"AM": 247, "MOI": 0.1, "MZA": 10000.0, "MZUA": 1000.0, "D_val": 0.001, "Num_TM": 96, "Name_RN": "кюрий-247", "UdA_GRO": 0.072, "UdA_TRO": 1.0, "ObA_GaRO": 0.0032, "Kod_gruppy": "Трансурановый", "Period_p_r": 15600000.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Cm-247", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'висмут-202', 'висмут-202', '{"AM": 202, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 83, "Name_RN": "висмут-202", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.67, "Vid_izluch": "бета", "Name_RN_Lat": "Bi-202", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'висмут-201', 'висмут-201', '{"AM": 201, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 83, "Name_RN": "висмут-201", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.8, "Vid_izluch": "бета", "Name_RN_Lat": "Bi-201", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'полоний-216', 'полоний-216', '{"AM": 216, "MOI": 0.0, "MZA": 0.0, "MZUA": 0.0, "D_val": 0, "Num_TM": 84, "Name_RN": "полоний-216", "UdA_GRO": 0.0, "UdA_TRO": 0.0, "ObA_GaRO": 0.0, "Kod_gruppy": "а", "Period_p_r": 0.145, "Vid_izluch": "альфа", "Name_RN_Lat": "Po-216", "Edinica_izmer_p_r": "сек", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'золото-199', 'золото-199', '{"AM": 199, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.9, "Num_TM": 79, "Name_RN": "золото-199", "UdA_GRO": 31.0, "UdA_TRO": 100.0, "ObA_GaRO": 140.0, "Kod_gruppy": "б", "Period_p_r": 3.14, "Vid_izluch": "бета", "Name_RN_Lat": "Au-199", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'уран-238', 'уран-238', '{"AM": 238, "MOI": "", "MZA": 10000.0, "MZUA": 10000.0, "D_val": "неогран.", "Num_TM": 92, "Name_RN": "уран-238", "UdA_GRO": 0.3, "UdA_TRO": 10.0, "ObA_GaRO": 0.041, "Kod_gruppy": "а", "Period_p_r": 4470000000.0, "Vid_izluch": "альфа", "Name_RN_Lat": "U-238", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'радон-222', 'радон-222', '{"AM": 222, "MOI": "", "MZA": 100000000.0, "MZUA": 10000.0, "D_val": 0.04, "Num_TM": 86, "Name_RN": "радон-222", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 210.0, "Kod_gruppy": "а", "Period_p_r": 3.825, "Vid_izluch": "альфа", "Name_RN_Lat": "Rn-222", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'уран-234', 'уран-234', '{"AM": 234, "MOI": "", "MZA": 10000.0, "MZUA": 10000.0, "D_val": 0.1, "Num_TM": 92, "Name_RN": "уран-234", "UdA_GRO": 0.28, "UdA_TRO": 10.0, "ObA_GaRO": 0.033, "Kod_gruppy": "а", "Period_p_r": 244000.0, "Vid_izluch": "альфа", "Name_RN_Lat": "U-234", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'америций-239', 'америций-239', '{"AM": 239, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 95, "Name_RN": "америций-239", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 11.9, "Vid_izluch": "гамма", "Name_RN_Lat": "Am-239", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'плутоний-244', 'плутоний-244', '{"AM": 244, "MOI": 0.1, "MZA": 10000.0, "MZUA": 1000.0, "D_val": 0.0003, "Num_TM": 94, "Name_RN": "плутоний-244", "UdA_GRO": 0.057, "UdA_TRO": 1.0, "ObA_GaRO": 0.026, "Kod_gruppy": "Трансурановый", "Period_p_r": 81300000.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Pu-244", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'берклий-249', 'берклий-249', '{"AM": 249, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": 10, "Num_TM": 97, "Name_RN": "берклий-249", "UdA_GRO": 24.0, "UdA_TRO": 1000.0, "ObA_GaRO": 0.77, "Kod_gruppy": "Трансурановый", "Period_p_r": 330.0, "Vid_izluch": "бета", "Name_RN_Lat": "Bk-249", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'эйнштейний-251', 'эйнштейний-251', '{"AM": 251, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 99, "Name_RN": "эйнштейний-251", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 1.38, "Vid_izluch": "гамма", "Name_RN_Lat": "Es-251", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'уран обогащенный', 'уран обогащенный', '{"AM": "", "MOI": "", "MZA": "", "MZUA": "", "D_val": "", "Num_TM": "", "Name_RN": "уран обогащенный", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "а", "Period_p_r": 704000000.0, "Vid_izluch": "альфа", "Name_RN_Lat": "U-ОБОГ", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": ""}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сумма нуклидов иода', 'сумма нуклидов иода', '{"AM": "", "MOI": "", "MZA": "", "MZUA": "", "D_val": "", "Num_TM": "", "Name_RN": "сумма нуклидов иода", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": "", "Vid_izluch": "", "Name_RN_Lat": "I", "Edinica_izmer_p_r": "", "Osn_vid_raspad_text": ""}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'железо-60', 'железо-60', '{"AM": 60, "MOI": "", "MZA": 100000.0, "MZUA": 100000.0, "D_val": 0.06, "Num_TM": 26, "Name_RN": "железо-60", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2620000.0, "Vid_izluch": "бета", "Name_RN_Lat": "Fe-60", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'калий-45', 'калий-45', '{"AM": 45, "MOI": 100.0, "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 19, "Name_RN": "калий-45", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.333, "Vid_izluch": "бета", "Name_RN_Lat": "K-45", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '540.0', 'НОВАЯ КАЛЕДОНИЯ', '{"2": "НОВАЯ КАЛЕДОНИЯ", "v": 540.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1326500', '1326500 Министерство труда и социальной защиты РФ', '{"2": "Министерство труда и социальной защиты РФ", "m": 1326500}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '548.0', 'ВАНУАТУ', '{"2": "ВАНУАТУ", "v": 548.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '554.0', 'НОВАЯ ЗЕЛАНДИЯ', '{"2": "НОВАЯ ЗЕЛАНДИЯ", "v": 554.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '408.0', 'КОРЕЯ (НАРОДНО-ДЕМОКРАТИЧЕСКАЯ РЕСПУБЛИКА)', '{"2": "КОРЕЯ (НАРОДНО-ДЕМОКРАТИЧЕСКАЯ РЕСПУБЛИКА)", "v": 408.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '410.0', 'КОРЕЯ (РЕСПУБЛИКА)', '{"2": "КОРЕЯ (РЕСПУБЛИКА)", "v": 410.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '414.0', 'КУВЕЙТ', '{"2": "КУВЕЙТ", "v": 414.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '010', 'АНТАРКТИДА', '{"2": "АНТАРКТИДА", "v": "010"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1326080', '1326080 Федеральное агентство морского и речного транспорта', '{"2": "Федеральное агентство морского и речного транспорта", "m": 1326080}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '04', 'Республика Алтай', '{"2": "Республика Алтай", "3": 7, "s": "04"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '417', 'КИРГИЗИЯ', '{"2": "КИРГИЗИЯ", "3": "СНГ", "v": 417, "attrs1": null, "status": "1"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '01', 'Республика Адыгея', '{"2": "Республика Адыгея", "3": 3, "s": "01"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '06', 'Республика Ингушетия', '{"2": "Республика Ингушетия", "3": 4, "s": "06"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '50', 'Московская область', '{"2": "Московская область", "3": 1, "s": 50}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '100', 'БОЛГАРИЯ', '{"2": "БОЛГАРИЯ", "v": "100"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '422.0', 'ЛИВАН', '{"2": "ЛИВАН", "v": 422.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '426.0', 'ЛЕСОТО', '{"2": "ЛЕСОТО", "v": 426.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '428.0', 'ЛАТВИЯ', '{"2": "ЛАТВИЯ", "v": 428.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1326060', '1326060 Федеральное дорожное агентство', '{"2": "Федеральное дорожное агентство", "m": 1326060}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '434.0', 'ЛИВИЯ', '{"2": "ЛИВИЯ", "v": 434.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '438.0', 'ЛИХТЕНШТЕЙН', '{"2": "ЛИХТЕНШТЕЙН", "v": 438.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '442.0', 'ЛЮКСЕМБУРГ', '{"2": "ЛЮКСЕМБУРГ", "v": 442.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '446.0', 'МАКАО', '{"2": "МАКАО", "v": 446.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'германий-71', 'германий-71', '{"label": "германий-71"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рений-182м', 'рений-182м', '{"label": "рений-182м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кжн (смесь короткоживущих нуклидов)*', 'кжн (смесь короткоживущих нуклидов)*', '{"label": "кжн (смесь короткоживущих нуклидов)*"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'плутоний-245', 'плутоний-245', '{"label": "плутоний-245"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'плутоний-244', 'плутоний-244', '{"label": "плутоний-244"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'уран-232', 'уран-232', '{"label": "уран-232"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'полоний-210', 'полоний-210', '{"label": "полоний-210"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'свинец-195м', 'свинец-195м', '{"label": "свинец-195м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'платина-200', 'платина-200', '{"label": "платина-200"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'осмий-194', 'осмий-194', '{"label": "осмий-194"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'осмий-189м', 'осмий-189м', '{"label": "осмий-189м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сумма нуклидов тория', 'сумма нуклидов тория', '{"label": "сумма нуклидов тория"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'америций-246м', 'америций-246м', '{"label": "америций-246м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'джн (смесь долгоживущих нуклидов)*', 'джн (смесь долгоживущих нуклидов)*', '{"label": "джн (смесь долгоживущих нуклидов)*"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'протактиний-231', 'протактиний-231', '{"label": "протактиний-231"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'висмут-213', 'висмут-213', '{"label": "висмут-213"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'таллий-199', 'таллий-199', '{"label": "таллий-199"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'платина-193', 'платина-193', '{"label": "платина-193"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рений-186м', 'рений-186м', '{"label": "рений-186м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рений-186', 'рений-186', '{"label": "рений-186"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'вольфрам-188', 'вольфрам-188', '{"label": "вольфрам-188"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тантал-183', 'тантал-183', '{"label": "тантал-183"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гафний-184', 'гафний-184', '{"label": "гафний-184"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'лютеций-174м', 'лютеций-174м', '{"label": "лютеций-174м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тулий-172', 'тулий-172', '{"label": "тулий-172"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тулий-167', 'тулий-167', '{"label": "тулий-167"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гольмий-164м', 'гольмий-164м', '{"label": "гольмий-164м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тербий-153', 'тербий-153', '{"label": "тербий-153"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гадолиний-149', 'гадолиний-149', '{"label": "гадолиний-149"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'европий-157', 'европий-157', '{"label": "европий-157"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'самарий-141', 'самарий-141', '{"label": "самарий-141"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'неодим-147', 'неодим-147', '{"label": "неодим-147"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'празеодим-142', 'празеодим-142', '{"label": "празеодим-142"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'лантан-142', 'лантан-142', '{"label": "лантан-142"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'барий-133м', 'барий-133м', '{"label": "барий-133м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'барий-126', 'барий-126', '{"label": "барий-126"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цезий-132', 'цезий-132', '{"label": "цезий-132"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ксенон-133м', 'ксенон-133м', '{"label": "ксенон-133м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иод-134', 'иод-134', '{"label": "иод-134"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иод-133', 'иод-133', '{"label": "иод-133"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иод-132м', 'иод-132м', '{"label": "иод-132м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иод-128', 'иод-128', '{"label": "иод-128"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иод-124', 'иод-124', '{"label": "иод-124"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иод-120м', 'иод-120м', '{"label": "иод-120м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иод-120', 'иод-120', '{"label": "иод-120"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'теллур-134', 'теллур-134', '{"label": "теллур-134"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'теллур-133м', 'теллур-133м', '{"label": "теллур-133м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'теллур-133', 'теллур-133', '{"label": "теллур-133"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'теллур-132', 'теллур-132', '{"label": "теллур-132"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'теллур-131м', 'теллур-131м', '{"label": "теллур-131м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'теллур-131', 'теллур-131', '{"label": "теллур-131"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'теллур-129м', 'теллур-129м', '{"label": "теллур-129м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'теллур-129', 'теллур-129', '{"label": "теллур-129"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'теллур-125м', 'теллур-125м', '{"label": "теллур-125м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'теллур-123м', 'теллур-123м', '{"label": "теллур-123м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иод-121', 'иод-121', '{"label": "иод-121"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'теллур-123', 'теллур-123', '{"label": "теллур-123"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'теллур-121м', 'теллур-121м', '{"label": "теллур-121м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'теллур-121', 'теллур-121', '{"label": "теллур-121"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'теллур-116', 'теллур-116', '{"label": "теллур-116"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сурьма-131', 'сурьма-131', '{"label": "сурьма-131"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сурьма-130', 'сурьма-130', '{"label": "сурьма-130"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'олово-113', 'олово-113', '{"label": "олово-113"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'индий-111', 'индий-111', '{"label": "индий-111"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'серебро-115', 'серебро-115', '{"label": "серебро-115"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'родий-102', 'родий-102', '{"label": "родий-102"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'технеций-98', 'технеций-98', '{"label": "технеций-98"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'технеций-97м', 'технеций-97м', '{"label": "технеций-97м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'технеций-96', 'технеций-96', '{"label": "технеций-96"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ниобий-95', 'ниобий-95', '{"label": "ниобий-95"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цирконий-88', 'цирконий-88', '{"label": "цирконий-88"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'стронций-92', 'стронций-92', '{"label": "стронций-92"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рубидий-82м', 'рубидий-82м', '{"label": "рубидий-82м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'углерод-14', 'углерод-14', '{"label": "углерод-14"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20210', '20210 Общественные движения', '{"2": "Общественные движения", "n": 20210}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '860', 'УЗБЕКИСТАН', '{"2": "УЗБЕКИСТАН", "3": "СНГ", "v": 860, "attrs1": null, "status": "1"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '581.0', 'МАЛЫЕ ТИХООКЕАНСКИЕ ОТДАЛЕННЫЕ ОСТРОВА СОЕДИНЕННЫХ ШТАТОВ', '{"2": "МАЛЫЕ ТИХООКЕАНСКИЕ ОТДАЛЕННЫЕ ОСТРОВА СОЕДИНЕННЫХ ШТАТОВ", "v": 581.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '570.0', 'НИУЭ', '{"2": "НИУЭ", "v": 570.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '462.0', 'МАЛЬДИВЫ', '{"2": "МАЛЬДИВЫ", "v": 462.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '458.0', 'МАЛАЙЗИЯ', '{"2": "МАЛАЙЗИЯ", "v": 458.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '454.0', 'МАЛАВИ', '{"2": "МАЛАВИ", "v": 454.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '450.0', 'МАДАГАСКАР', '{"2": "МАДАГАСКАР", "v": 450.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1324700', '1324700 Министерство РФ по делам Северного Кавказа', '{"2": "Министерство РФ по делам Северного Кавказа", "m": 1324700}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '466.0', 'МАЛИ', '{"2": "МАЛИ", "v": 466.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '470.0', 'МАЛЬТА', '{"2": "МАЛЬТА", "v": 470.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '478.0', 'МАВРИТАНИЯ', '{"2": "МАВРИТАНИЯ", "v": 478.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '474.0', 'МАРТИНИКА', '{"2": "МАРТИНИКА", "v": 474.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1324500', '1324500 Министерство связи и массовых коммуникаций РФ', '{"2": "Министерство связи и массовых коммуникаций РФ", "m": 1324500}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '480.0', 'МАВРИКИЙ', '{"2": "МАВРИКИЙ", "v": 480.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '498', 'МОЛДОВА (РЕСПУБЛИКА)', '{"2": "МОЛДОВА (РЕСПУБЛИКА)", "3": "СНГ", "v": 498, "attrs1": null, "status": "1"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '499.0', 'ЧЕРНОГОРИЯ', '{"2": "ЧЕРНОГОРИЯ", "v": 499.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '500.0', 'МОНТСЕРРАТ', '{"2": "МОНТСЕРРАТ", "v": 500.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '504.0', 'МАРОККО', '{"2": "МАРОККО", "v": 504.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '024', 'АНГОЛА', '{"2": "АНГОЛА", "v": "024"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '031', 'АЗЕРБАЙДЖАН', '{"2": "АЗЕРБАЙДЖАН", "3": "СНГ", "v": "031", "attrs1": null, "status": "1"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '512.0', 'ОМАН', '{"2": "ОМАН", "v": 512.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '036', 'АВСТРАЛИЯ', '{"2": "АВСТРАЛИЯ", "v": "036"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '040', 'АВСТРИЯ', '{"2": "АВСТРИЯ", "v": "040"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '048', 'БАХРЕЙН', '{"2": "БАХРЕЙН", "v": "048"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '044', 'БАГАМЫ', '{"2": "БАГАМЫ", "v": "044"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '051', 'АРМЕНИЯ', '{"2": "АРМЕНИЯ", "3": "СНГ", "v": "051", "attrs1": null, "status": "1"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '052', 'БАРБАДОС', '{"2": "БАРБАДОС", "v": "052"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '060', 'БЕРМУДЫ', '{"2": "БЕРМУДЫ", "v": "060"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '064', 'БУТАН', '{"2": "БУТАН", "v": "064"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '072', 'БОТСВАНА', '{"2": "БОТСВАНА", "v": "072"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '084', 'БЕЛИЗ', '{"2": "БЕЛИЗ", "v": "084"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '108.0', 'БУРУНДИ', '{"2": "БУРУНДИ", "v": 108.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '120.0', 'КАМЕРУН', '{"2": "КАМЕРУН", "v": 120.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '132.0', 'КАБО-ВЕРДЕ', '{"2": "КАБО-ВЕРДЕ", "v": 132.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '156.0', 'КИТАЙ', '{"2": "КИТАЙ", "v": 156.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('211_voc_vsp', 'Пункт хранения РАО', 'Пункт хранения РАО', '{"kod_obj": 1, "vid_obj": "Пункт хранения РАО", "description": "Стационарные объекты и сооружения, не относящиеся к ядерным установкам, радиационным источникам и предназначенные для хранения хранения или захоронения радиоактивных отходов. На эксплуатацию таких объектов (отдельно или в составе иного объекта, например, ядерной установки) выдана лицензия Ростехнадзора на вид деятельности - эксплуатация пункта хранения РАО или такое разрешение указано в условиях действия лицензии, выданной на эксплуатацию иного объекта, например, ядерной установки."}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('211_voc_vsp', 'Место сбора РАО', 'Место сбора РАО', '{"kod_obj": 2, "vid_obj": "Место сбора РАО", "description": "Специально выделенные места непосредственно в местах образования радиоактивных отходов, которые используются для сбора радиоактивных отходов отдельно от обычных отходов с использованием пластиковых или бумажных мешков или сборников-контейнеров для твердых РАО или специальные емкости для жидких РАО. На эксплуатацию таких мест должно быть получено санитарно-эпидемиологическое заключение."}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('211_voc_vsp', 'Место временного хранения РАО', 'Место временного хранения РАО', '{"kod_obj": 3, "vid_obj": "Место временного хранения РАО", "description": "Стационарные объекты, сооружения, площадки, предназначенные для временного хранения РАО в целях обеспечения безопасного хранения РАО на время проведения каких либо работ и для которых не предусмотрено хранение РАО, после завершения указанных работ. Например, объекты хранения РАО, созданные для хранения РАО при проведении работ по реабилитации территорий, вывода из эксплуатации ОИАЭ. На эксплуатацию таких мест должно быть получено санитарно-эпидемиологическое заключение."}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('211_voc_vsp', 'Прочее', 'Прочее', '{"kod_obj": "", "vid_obj": "Прочее", "description": "Прочие пункты хранения"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('kat_rao_voc', 'Прочие', 'Прочие', '{"prim": "Прочие категории РАО", "kat_rao": "Прочие", "znach_rao": "9"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('kat_rao_voc', 'ОЗРИ', 'ОЗРИ', '{"prim": "Отработавшие закрытые источники ионизирующего излучения", "kat_rao": "ОЗРИ", "znach_rao": "4"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('kat_rao_voc', 'ВАО', 'ВАО', '{"prim": "Высокоактивные радиоактивные отходы", "kat_rao": "ВАО", "znach_rao": "3"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('kat_rao_voc', 'САО', 'САО', '{"prim": "Среднеактивные радиоактивные отходы", "kat_rao": "САО", "znach_rao": "2"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('kat_rao_voc', 'НАО', 'НАО', '{"prim": "Низкоактивные радиоактивные отходы", "kat_rao": "НАО", "znach_rao": "1"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('kat_rao_voc', 'ОНАО', 'ОНАО', '{"prim": "Очень низкоактивные радиоактивные отходы", "kat_rao": "ОНАО", "znach_rao": "0"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_yam', '2', '2', '{"prim": "Содержит ЯМ", "number": 2}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_yam', '1', '1', '{"prim": "Не содержит ЯМ", "number": 1}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '162.0', 'ОСТРОВ РОЖДЕСТВА', '{"2": "ОСТРОВ РОЖДЕСТВА", "v": 162.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '158.0', 'ТАЙВАНЬ (КИТАЙ)', '{"2": "ТАЙВАНЬ (КИТАЙ)", "v": 158.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_per_raspad', '2', '2', '{"prim": "Короткоживующие (период полураспада менее 31 года)", "number": 2}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_per_raspad', '1', '1', '{"prim": "Долгоживующие (период полураспада более 31 года)", "number": 1}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_per_op', '3', '3', '{"prim": "Более 500 лет", "number": 3}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_per_op', '2', '2', '{"prim": "От 100 до 500 лет", "number": 2}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_per_op', '1', '1', '{"prim": "Менее 100 лет", "number": 1}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_per_op', '0', '0', '{"prim": "Не определён", "number": 0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '66', '66', '{"prim": "Продление назначенного срока службы ЗРИ/ИОУ. При использовании операции необходимо к отчету приложить скан-копию документа о продлении назначенного срока службы.", "Op_Cod": "66", "Op_Name": "Продление НСС ЗРИ/ИОУ", "deadline": "10 – для РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '65', '65', '{"prim": "Используется для обозначения операции, связанной со снятием с учета ЗРИ (в форме ЗРИ) и постановкой этой учетной единицы как ОРИ (в форме ОРИ). Операция не предусматривает каких-либо действий, изменяющих агрегатное состояние, конструкцию или физико-химические характеристики источника", "Op_Cod": "65", "Op_Name": "Перевод из ЗРИ в ОРИ", "deadline": "10 – для РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_sposob_pererabotki', '9', '9', '{"prim": "Омоноличенные (отверждённые) другим способом", "number": 9}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_sposob_pererabotki', '4', '4', '{"prim": "Остеклованные", "number": 4}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_sposob_pererabotki', '3', '3', '{"prim": "Цементированные", "number": 3}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_sposob_pererabotki', '2', '2', '{"prim": "Битумированные", "number": 2}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_sposob_pererabotki', '1', '1', '{"prim": "Спрессованные (компактированные)", "number": 1}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_sposob_pererabotki', '0', '0', '{"prim": "РАО, не подвергавшиеся переработке способами, перечисленными ниже", "number": 0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '166.0', 'КОКОСОВЫЕ (КИЛИНГ) ОСТРОВА', '{"2": "КОКОСОВЫЕ (КИЛИНГ) ОСТРОВА", "v": 166.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_class_rao', '7', '7', '{"prim": "Особые РАО", "number": 7}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_class_rao', '6', '6', '{"prim": "Удаляемые 6-го класса", "number": 6}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_class_rao', '5', '5', '{"prim": "Удаляемые 5-го класса", "number": 5}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_class_rao', '4', '4', '{"prim": "Удаляемые 4-го класса", "number": 4}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_class_rao', '3', '3', '{"prim": "Удаляемые 3-го класса", "number": 3}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_class_rao', '2', '2', '{"prim": "Удаляемые 2-го класса", "number": 2}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_class_rao', '1', '1', '{"prim": "Удаляемые 1-го класса", "number": 1}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_class_rao', '0', '0', '{"prim": "Удаляемые РАО, класс которых не установлен", "number": 0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3900500', '3900500 Избирательные комиссии сельских поселений', '{"2": "Избирательные комиссии сельских поселений", "m": 3900500}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1322600', '1322600 Министерство науки и высшего образования Российской Федерации', '{"2": "Министерство науки и высшего образования Российской Федерации", "m": 1322600}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '25', '25', '{"number": 25, "typ_RAO": "спирты"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '92', '92', '{"number": 92, "typ_RAO": "электрокабели"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '94', '94', '{"number": 94, "typ_RAO": "жидкие РАО, подготовленные к передаче национальному оператору"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '5100029', '5100029 Шанхайская организация сотрудничества', '{"2": "Шанхайская организация сотрудничества", "m": 5100029}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '99', '99', '{"prim": "Не используется для РАО, образованных после вступления в силу Федерального закона от 11.07.2011 № 190-ФЗ «Об обращении с радиоактивными отходами и о внесении изменений в отдельные законодательные акты Российской Федерации». Данный вариант выбирается, если РАО не могут быть классифицированы по позициям, приведенным выше. При выборе этого значения в примечании необходимо указать наименование и происхождение РАО.", "number": 99, "typ_RAO": "прочие типы РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '98', '98', '{"prim": "Используется сокращенное наименование: КНИИТ.", "number": 98, "typ_RAO": "канал измерения нейтронного потока и температуры"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '97', '97', '{"prim": "Используется сокращенное наименование: ПЭЛы (СП).", "number": 97, "typ_RAO": "поглощающие элементы (стержни-поглотители)"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '96', '96', '{"number": 96, "typ_RAO": "загрязненный грунт, включая керны"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '95', '95', '{"prim": "ИК, содержащие ОЗИИИ, обозначаются (в зависимости от массы) кодами 87 или 88.", "number": 95, "typ_RAO": "ионизационные камеры (ИК), не содержащие ОЗИИИ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '93', '93', '{"number": 93, "typ_RAO": "стройматериалы, строительный и прочий мусор"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_pojar_op', '2', '2', '{"prim": "Негорючие", "number": 2}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_pojar_op', '1', '1', '{"prim": "Горючие", "number": 1}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '174.0', 'КОМОРЫ', '{"2": "КОМОРЫ", "v": 174.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '170.0', 'КОЛУМБИЯ', '{"2": "КОЛУМБИЯ", "v": 170.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '5100025', '5100025 Евразийский экономический союз', '{"2": "Евразийский экономический союз", "m": 5100025}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '5100024', '5100024 Евразийская группа по противодействию легализации преступных доходов и финансированию терроризма', '{"2": "Евразийская группа по противодействию легализации преступных доходов и финансированию терроризма", "m": 5100024}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('27_voc_sert', 'Н', 'Н', '{"full_sert": "Несертифицированный", "short_sert": "Н"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('27_voc_sert', 'С', 'С', '{"full_sert": "Сертифицированный", "short_sert": "С"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '175.0', 'МАЙОТТА', '{"2": "МАЙОТТА", "v": 175.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '178.0', 'КОНГО', '{"2": "КОНГО", "v": 178.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '5100021', '5100021 Координационный совет генеральных прокуроров государств - участников Содружества Независимых Государств', '{"2": "Координационный совет генеральных прокуроров государств - участников Содружества Независимых Государств", "m": 5100021}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '5100019', '5100019 Исполнительный комитет Союза Беларуси и России', '{"2": "Исполнительный комитет Союза Беларуси и России", "m": 5100019}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '5100014', '5100014 Международный инвестиционный банк', '{"2": "Международный инвестиционный банк", "m": 5100014}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '5100018', '5100018 Организация Договора о коллективной безопасности', '{"2": "Организация Договора о коллективной безопасности", "m": 5100018}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '180.0', 'КОНГО (ДЕМОКРАТИЧЕСКАЯ РЕСПУБЛИКА)', '{"2": "КОНГО (ДЕМОКРАТИЧЕСКАЯ РЕСПУБЛИКА)", "v": 180.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '5100013', '5100013 Международный банк экономического сотрудничества', '{"2": "Международный банк экономического сотрудничества", "m": 5100013}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '11', '11', '{"prim": "Используется для постановки на учёт: 1) изделий (ЗРИ, ОРИ, ИОУ) при поступлении на склад готовой продукции; 2) РВ и РАО (кроме ОЗИИИ), образовавшихся при осуществлении деятельности. Код не может быть использован для отражения фактов постановки на учет: 1) материалов, которые стояли на учете в СГУК ЯМ. Для этого следует использовать операцию с кодом 12; 2) РАО, образовавшихся при обслуживании и эксплуатации пунктов хранения/захоронения РАО в штатном режиме. Для этого следует использовать операцию с кодом 13; 3) РАО, образовавшихся из РВ, не подлежащих учету в СГУК РВ и РАО. Для этого следует использовать операцию с кодом 14; 4) РВ, образовавшихся при переработке РАО в виде ОЗИИИ. Для этого следует использовать операцию с кодом 15; 5) РАО, образовавшихся при проведении работ по выводу из эксплуатации объектов использования атомной энергии или при реабилитации загрязненных территорий. Для этого следует использовать операцию с кодом 16; 6) РАО образовавшихся из РВ, подлежащих учету в СГУК РВ и РАО. Для этого следует использовать операцию с кодом 41; 7) РАО, образовавшихся из других учетных единиц, стоявших на учете в СГУК РВ и РАО, после приведения в соответствие критериям приемлемости РАО для захоронения,  переработки, постановки на учет при операциях упаковки/переупаковки или сортировки. Для этого следует использовать операции с кодами 55, 56, 57, 59; 8) РВ, изготовленных (образовавшихся) из других учетных единиц. Для этого следует использовать операцию с кодом 58; 9) ОРИ, образовавшихся в результате перевода в эту категорию ЗРИ. Для этого следует использовать операцию с кодом 65", "Op_Cod": "11", "Op_Name": "Изготовление / образование", "deadline": "10 – для РВ; 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '5100011', '5100011 Межгосударственный статистический комитет Содружества Независимых Государств', '{"2": "Межгосударственный статистический комитет Содружества Независимых Государств", "m": 5100011}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '36', '36', '{"prim": "Используется для постановки на учет при получении одним юридическим лицом, осуществляющим деятельность на территории Российской Федерации от другого юридического лица на территории Российской Федерации учетной единицы, с обязательствами по безопасному обращению, включая передачу федеральному государственному унитарному предприятию «Национальный оператор по обращению с радиоактивными отходами» для захоронения с оплатой захоронения (при необходимости)", "Op_Cod": "36", "Op_Name": "Получение от юридического лица, зарегистрированного на территории Российской Федерации, с обязательствами по обеспечению безопасности при обращении, вплоть до их передачи национальному оператору", "deadline": "10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '37', '37', '{"prim": "Получение учетной единицы, которая ранее передавалась другому юридическому лицу по основаниям, не связанным с переходом права собственности (иного вещного права). Используется для отражения факта возврата учетной единицы, информация о передаче которой представлялась с кодом 27", "Op_Cod": "37", "Op_Name": "Получение (возврат) ранее переданного юридическому лицу, зарегистрированному на территории в Российской Федерации", "deadline": "10 – для РВ; 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '53', '53', '{"prim": "Используется для отражения факта перемещения отчитывающейся организацией учетной единицы из контейнера в контейнер, из контейнера в установку (прибор) и наоборот. При заполнении формы указываются наименование, тип и номер прибора (установки), УКТ или иной упаковки, в которую размещена учетная единица.", "Op_Cod": "53", "Op_Name": "Зарядка (загрузка)/разрядка (выгрузка) организацией", "deadline": "10 – для РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '81', '81', '{"prim": "Вывоз с территории Российской Федерации с передачей права собственности иностранному получателю", "Op_Cod": "81", "Op_Name": "Экспорт", "deadline": "10 – для РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '76', '76', '{"prim": "Постановка на учет РАО, которые изымаются по указанию (предписанию) надзорных органов, органов исполнительной власти, в том числе органов ГО и ЧС в ходе выполнения радиационно-аварийных работ, включая ликвидацию участков радиоактивного загрязнения, при обнаружении и изъятии бесхозяйной учетной единицы РАО. В этом случае необходимо представить копию документа, характеризующего обстоятельства изъятия РАО. Отчет с использованием данного кода предоставляет организация, которой учетная единица РАО передана на хранение", "Op_Cod": "76", "Op_Name": "Постановка на учет РАО, изымаемых по указанию органов исполнительной власти, надзорных органов", "deadline": "10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '82', '82', '{"prim": "Используется при временном вывозе с территории Российской Федерации учетной единицы без передачи права собственности иностранной организации: 1) если работники организации, являющейся собственником вывозимой учетной единицы, вывозят ее для производства работ за пределами Российской Федерации; 2) если учетная единица передается в аренду иностранной организации, осуществляющей деятельность за пределами Российской Федерации; 3) если осуществляется вывоз по иным причинам, с гарантией возврата учетной единицы на территорию Российской Федерации.Возврат на территорию Российской Федерации временно ввезенного ранее РВ отражается посредством операции с кодом 87. Учетная единица снимается с учета в организации, осуществившей временный вывоз", "Op_Cod": "82", "Op_Name": "Временный вывоз с территории Российской Федерации", "deadline": "10 – для РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '218.0', 'ЭКВАДОР', '{"2": "ЭКВАДОР", "v": 218.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '5100009', '5100009 Главное командование объединенных вооруженных сил СНГ', '{"2": "Главное командование объединенных вооруженных сил СНГ", "m": 5100009}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '22', '22', '{"prim": "Используется для снятия с учета учетной единицы при ее передаче воинской части (организации) Вооруженных сил Российской Федерации", "Op_Cod": "22", "Op_Name": "Передача в Министерство обороны Российской Федерации", "deadline": "10 – для РВ; 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '21', '21', '{"prim": "Используется для снятия с учета при передаче учетной единицы между обособленными подразделениями одного юридического лица. Операция с этим кодом может использоваться только в отчете обособленного подразделения, передающего учетную единицу. Подразделение, на территории которого расположен постоянно действующий исполнительный орган юридического лица, рассматривается так же как обособленное подразделение. Обособленное подразделение, получающее учетную единицу, отражает факт постановки на учет операцией с кодом 31", "Op_Cod": "21", "Op_Name": "Передача одним обособленным подразделением другому обособленному подразделению того же юридического лица", "deadline": "10 – для РВ; 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '25', '25', '{"prim": "Используется для отражения факта возврата учетной единицы, информация о получении которой представлялась с кодом 35. Учетная единица передается (возвращается) юридическому лицу, от которого она была получена", "Op_Cod": "25", "Op_Name": "Передача (возврат) ранее полученного от юридического лица, зарегистрированного на территории Российской Федерации", "deadline": "10 – для РВ; 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '75', '75', '{"prim": "Используется для отражения факта обнаружения учетной единицы и постановки ее на учет, об утрате которой ранее сообщалось операцией с кодом 71. В этом случае необходимо представить копию документа, характеризующего обстоятельства обнаружения учетной единицы", "Op_Cod": "75", "Op_Name": "Обнаружение утерянного ранее", "deadline": "5 – для РВ; 5 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '68', '68', '{"prim": "Используется для снятия с учета учетной единицы, сведения о количественных или качественных характеристиках которой изменились в результате проведения измерений. Учетная единица с новыми количественными и/или качественными характеристиками ставится на учет с кодом операции 18. При использовании данной операции необходимо представить копию документа, отражающего результаты проведенных измерений", "Op_Cod": "68", "Op_Name": "Снятие с учета при изменении характеристик, сведения о которых получены в результате проведения измерений", "deadline": "10 – для РВ; 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '74', '74', '{"prim": "Постановка на учет РВ или РАО, изъятого из незаконного оборота в результате проведения специальных мероприятий включая мероприятия, проводимые правоохранительными органами, органами государственной безопасности и, таможенного контроля. В этом случае необходимо представить копию документа (протокола, акта, прочего документа), характеризующего обстоятельства изъятия РВ или РАО. Отчет предоставляет организация, которой передана учетная единица на хранение", "Op_Cod": "74", "Op_Name": "Изъятие из незаконного оборота", "deadline": "5 – для РВ; 5 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '5100007', '5100007 Исполнительный комитет Содружества Независимых Государств', '{"2": "Исполнительный комитет Содружества Независимых Государств", "m": 5100007}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '32', '32', '{"prim": "Используется для постановки на учет учетных единиц, ранее находившихся на учете в воинской части (организации) Вооруженных сил Российской Федерации", "Op_Cod": "32", "Op_Name": "Получение из Министерства обороны Российской Федерации", "deadline": "10 – для РВ; 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '31', '31', '{"prim": "Используется для постановки на учет при передаче учетной единицы между обособленными подразделениями одного юридического лица. Для этой операции подразделение, на территории которого расположен орган управления юридического лица, рассматривается так же как обособленное подразделение. Операция с этим кодом может использоваться только в отчете обособленного подразделения, получающего учетную единицу. Обособленное подразделение, передающее учетную единицу, отражает факт снятия с учета учетной единицы операцией с кодом 21", "Op_Cod": "31", "Op_Name": "Получение одним обособленным подразделением от другого обособленного подразделения того же юридического лица", "deadline": "10 – для РВ; 10– для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '57', '57', '{"prim": "Операция используется для отражения постановки на учет новых учетных единиц в результате упаковки, распаковки или переупаковки РАО. Операция используется так же при размещении первичных упаковок РАО в упаковку (контейнер). При этом учетная единица РАО, поступившая на переупаковку, снимается с учета с кодом операции 45", "Op_Cod": "57", "Op_Name": "Постановка на учет РАО при операциях упаковки или переупаковки", "deadline": "10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '5100004', '5100004 Межгосударственный банк', '{"2": "Межгосударственный банк", "m": 5100004}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '5100006', '5100006 Межгосударственное евроазиатское объединение угля и металла', '{"2": "Межгосударственное евроазиатское объединение угля и металла", "m": 5100006}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '222.0', 'ЭЛЬ-САЛЬВАДОР', '{"2": "ЭЛЬ-САЛЬВАДОР", "v": 222.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '226.0', 'ЭКВАТОРИАЛЬНАЯ ГВИНЕЯ', '{"2": "ЭКВАТОРИАЛЬНАЯ ГВИНЕЯ", "v": 226.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '231.0', 'ЭФИОПИЯ', '{"2": "ЭФИОПИЯ", "v": 231.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '232.0', 'ЭРИТРЕЯ', '{"2": "ЭРИТРЕЯ", "v": 232.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '233.0', 'ЭСТОНИЯ', '{"2": "ЭСТОНИЯ", "v": 233.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('112_voc', '1:', '1:', '{"prim": "", "attrs1": null, "g1_cod": "1:", "status": "1", "g1_types": "ОРИ  в виде отдельных изделий, за исключением радиофармпрепаратов"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '234.0', 'ФАРЕРСКИЕ ОСТРОВА', '{"2": "ФАРЕРСКИЕ ОСТРОВА", "v": 234.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '5100000', '5100000 МЕЖГОСУДАРСТВЕННЫЕ ОРГАНЫ УПРАВЛЕНИЯ, ОБРАЗОВАННЫЕ СТРАНАМИ С УЧАСТИЕМ РФ', '{"2": "МЕЖГОСУДАРСТВЕННЫЕ ОРГАНЫ УПРАВЛЕНИЯ, ОБРАЗОВАННЫЕ СТРАНАМИ С УЧАСТИЕМ РФ", "m": 5100000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('13_voc', '1', '1 Приказ №600: Промышленные отходы с повешенным содержанием природных радионуклидов', '{"ORICod": 1, "ORI_ty": "Приказ №600: Промышленные отходы с повешенным содержанием природных радионуклидов", "attrs1": null, "status": "2", "import_hash": null, "import_author": null, "import_src_id": null}', 2, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4220004', '4220004 Международные общественные объединения', '{"2": "Международные общественные объединения", "m": 4220004}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '5000000', '5000000 МЕЖГОСУДАРСТВЕННЫЕ ОРГАНЫ УПРАВЛЕНИЯ', '{"2": "МЕЖГОСУДАРСТВЕННЫЕ ОРГАНЫ УПРАВЛЕНИЯ", "m": 5000000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('13_voc', '2', '2 Приказ №600: Промышленные отходы с повышенным содержанием техногенных радионуклидов, кроме металлических отходов, имеющих поверхностное загрязнение', '{"ORICod": 2, "ORI_ty": "Приказ №600: Промышленные отходы с повышенным содержанием техногенных радионуклидов, кроме металлических отходов, имеющих поверхностное загрязнение", "status": "0"}', 2, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('13_voc', '6', '6 Плав', '{"ORICod": 6, "ORI_ty": "Плав", "attrs1": null}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4220002', '4220002 Межрегиональные общественные объединения', '{"2": "Межрегиональные общественные объединения", "m": 4220002}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4220003', '4220003 Региональные и местные общественные объединения', '{"2": "Региональные и местные общественные объединения", "m": 4220003}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('13_voc', '3', '3 Приказ №600: Металлические отходы, имеющие поверхностное загрязнение техногенными радионуклидами', '{"ORICod": 3, "ORI_ty": "Приказ №600: Металлические отходы, имеющие поверхностное загрязнение техногенными радионуклидами", "status": "0"}', 2, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '516.0', 'НАМИБИЯ', '{"2": "НАМИБИЯ", "v": 516.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4220001', '4220001 Общероссийские общественные объединения', '{"2": "Общероссийские общественные объединения", "m": 4220001}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('16_voc', 'код ОКПО: ХХХХХХХХ', 'код ОКПО: ХХХХХХХХ', '{"status": "0", "BCodCod": "код ОКПО: ХХХХХХХХ", "BCod_Name": "код ОКПО юридического лица, в результате деятельности которого образовались РАО", "BCod_Clarif": "используется для РАО, образовавшихся после вступления в силу Федерального закона от 11.07.2011  № 190-ФЗ «Об обращении с радиоактивными отходами и о внесении изменений в отдельные законодательные акты Российской Федерации» (Собрание законодательства Российской Федерации 2011, № 29, ст. 4281)"}', 2, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4220000', '4220000 Общественные объединения', '{"2": "Общественные объединения", "m": 4220000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '520.0', 'НАУРУ', '{"2": "НАУРУ", "v": 520.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '524.0', 'НЕПАЛ', '{"2": "НЕПАЛ", "v": 524.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '531.0', 'КЮРАСАО', '{"2": "КЮРАСАО", "v": 531.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '533.0', 'АРУБА', '{"2": "АРУБА", "v": 533.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '242.0', 'ФИДЖИ', '{"2": "ФИДЖИ", "v": 242.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '192.0', 'КУБА', '{"2": "КУБА", "v": 192.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '191.0', 'ХОРВАТИЯ', '{"2": "ХОРВАТИЯ", "v": 191.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100400', '4100400 Государственные компании', '{"2": "Государственные компании", "m": 4100400}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '248.0', 'ЭЛАНДСКИЕ ОСТРОВА', '{"2": "ЭЛАНДСКИЕ ОСТРОВА", "v": 248.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '250.0', 'ФРАНЦИЯ', '{"2": "ФРАНЦИЯ", "v": 250.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100306', '4100306 Фонд содействия реформированию жилищно-коммунального хозяйства', '{"2": "Фонд содействия реформированию жилищно-коммунального хозяйства", "m": 4100306}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100305', '4100305 Агентство по страхованию вкладов', '{"2": "Агентство по страхованию вкладов", "m": 4100305}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '260.0', 'ФРАНЦУЗСКИЕ ЮЖНЫЕ ТЕРРИТОРИИ', '{"2": "ФРАНЦУЗСКИЕ ЮЖНЫЕ ТЕРРИТОРИИ", "v": 260.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '258.0', 'ФРАНЦУЗСКАЯ ПОЛИНЕЗИЯ', '{"2": "ФРАНЦУЗСКАЯ ПОЛИНЕЗИЯ", "v": 258.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '91', '91', '{"number": 91, "typ_RAO": "фильтры вентиляционные в сборе"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '89', '89', '{"number": 89, "typ_RAO": "прочие учетные единицы, содержащие ОЗИИИ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '88', '88', '{"number": 88, "typ_RAO": "приборы и установки с массой 30 кг и более, содержащие ОЗИИИ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '87', '87', '{"number": 87, "typ_RAO": "приборы и установки с массой менее 30 кг, содержащие ОЗИИИ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '86', '86', '{"number": 86, "typ_RAO": "дымо-, пожароизвещатели"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '85', '85', '{"number": 85, "typ_RAO": "радиоизотопные термоэлектрические генераторы (РИТЭГ)"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '84', '84', '{"number": 84, "typ_RAO": "открытые радионуклидные источники в виде отдельных изделий"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '83', '83', '{"number": 83, "typ_RAO": "изделия с радиоактивной светомассой постоянного действия (ампулированные источники для ночного обозначения габаритов, аварийные и автономные осветители, указатели, источники света для шкал и циферблатов приборов, подсветки оружейных прицелов и подобное)"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '82', '82', '{"number": 82, "typ_RAO": "ОЗИИИ в защитном контейнере"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '81', '81', '{"number": 81, "typ_RAO": "ОЗИИИ без защитного контейнера"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '79', '79', '{"number": 79, "typ_RAO": "Твердые РАО органические прочие"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '78', '78', '{"number": 78, "typ_RAO": "Твердые РАО биологические"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '77', '77', '{"number": 77, "typ_RAO": "древесина, бумага, картон"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '76', '76', '{"number": 76, "typ_RAO": "графит"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '75', '75', '{"prim": "Кроме аналогичных изделий из неорганических материалов.", "number": 75, "typ_RAO": "спецодежда и другие средства индивидуальной защиты, обувь, обтирочные материалы, ветошь, вата, фильтроэлементы (фильтровальная ткань) фильтров вентиляции"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '74', '74', '{"number": 74, "typ_RAO": "сажа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '73', '73', '{"prim": "Изделия из полиэтилена, полихлорвинила, резины.", "number": 73, "typ_RAO": "полимеры"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '72', '72', '{"number": 72, "typ_RAO": "уголь активированный, сульфоуголь"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '71', '71', '{"number": 71, "typ_RAO": "смолы отработанные ионообменные"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '69', '69', '{"number": 69, "typ_RAO": "Твердые РАО неорганические прочие"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '68', '68', '{"number": 68, "typ_RAO": "рудные материалы"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '67', '67', '{"number": 67, "typ_RAO": "плав солевой"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '66', '66', '{"number": 66, "typ_RAO": "зола"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '65', '65', '{"number": 65, "typ_RAO": "отходы плавильного производства (включая шлаки, футеровку.)"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '64', '64', '{"number": 64, "typ_RAO": "изделия из стекла и керамики, лабораторная посуда"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '63', '63', '{"number": 63, "typ_RAO": "теплоизоляционные материалы неорганические"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '62', '62', '{"number": 62, "typ_RAO": "песок"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '61', '61', '{"prim": "Данный вариант выбирается, если отработанные сорбенты находятся в сухом виде.", "number": 61, "typ_RAO": "сорбенты и фильтроматериалы неорганические"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '59', '59', '{"prim": "Металлические отходы, которые не могут быть классифицированы по приведенным выше позициям, в т. ч. несортированный лом черных и цветных металлов.", "number": 59, "typ_RAO": "металлический лом прочий"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '58', '58', '{"prim": "Несортированный лом нескольких цветных металлов.", "number": 58, "typ_RAO": "лом цветных металлов несортированный"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '57', '57', '{"number": 57, "typ_RAO": "лом цветных металлов прочих, не перечисленных в пунктах 51-57"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '56', '56', '{"number": 56, "typ_RAO": "цирконий и его сплавы"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '55', '55', '{"number": 55, "typ_RAO": "свинец и его сплавы"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '54', '54', '{"number": 54, "typ_RAO": "никель и его сплавы, кроме медно-никелевых сплавов, отнесенных к коду 53"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '53', '53', '{"number": 53, "typ_RAO": "медь и сплавы меди"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '52', '52', '{"number": 52, "typ_RAO": "уран металлический обедненный и изделия из него"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '51', '51', '{"number": 51, "typ_RAO": "алюминий и его сплавы, изделия и оборудование из алюминия и его сплавов"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '46', '46', '{"number": 46, "typ_RAO": "металлы щелочные, в том числе отработавший теплоноситель"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '45', '45', '{"prim": "Выбирается в тех случаях, когда не могут быть выбраны типы с кодами 41-44.", "number": 45, "typ_RAO": "лом черных металлов"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '44', '44', '{"number": 44, "typ_RAO": "металлы черные, кроме нержавеющей стали и крупногабаритных изделий"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '43', '43', '{"prim": "Размеры отдельных элементов превышают 350х350х400 мм или превышают 2 м в одном измерении.", "number": 43, "typ_RAO": "металлы черные, кроме нержавеющей стали: крупногабаритные изделия и оборудование"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '42', '42', '{"number": 42, "typ_RAO": "сталь нержавеющая, кроме крупногабаритных изделий"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '41', '41', '{"prim": "Размеры отдельных элементов превышают 350х350х400 мм или превышают 2 м в одном измерении", "number": 41, "typ_RAO": "сталь нержавеющая: крупногабаритные изделия и оборудование"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '39', '39', '{"number": 39, "typ_RAO": "прочие пульпы, шламы технологические"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '38', '38', '{"number": 38, "typ_RAO": "шламы после очистки трапных вод"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '37', '37', '{"number": 37, "typ_RAO": "илы, иловые осадки водоемов-накопителей, осадки, кеки"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '36', '36', '{"number": 36, "typ_RAO": "взвеси, содержащие продукты коррозии"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '35', '35', '{"number": 35, "typ_RAO": "кубовый остаток после упаривания с солесодержанием более 250 г/л, концентрат солевой"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '34', '34', '{"number": 34, "typ_RAO": "солевой остаток с солесодержанием до 250 г/л, кубовый остаток"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '33', '33', '{"number": 33, "typ_RAO": "шламы и пульпы отработавших сорбентов неорганических, фильтрующих материалов"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '32', '32', '{"number": 32, "typ_RAO": "пульпы отработавших ионообменных смол и прочих сорбентов органических"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '31', '31', '{"number": 31, "typ_RAO": "хвосты гидрометаллургического, химико-металлургического, разделительного и сублиматного производств, пульпы, образующиеся после нейтрализации"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '29', '29', '{"number": 29, "typ_RAO": "жидкости органические прочие"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '26', '26', '{"number": 26, "typ_RAO": "сцинцилляторы органические жидкостные"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '24', '24', '{"number": 24, "typ_RAO": "моющие средства (детергенты) за исключением спиртов"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '23', '23', '{"number": 23, "typ_RAO": "эмульсии смазочно-охлаждающих жидкостей (СОЖ)"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '22', '22', '{"number": 22, "typ_RAO": "масла"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '21', '21', '{"number": 21, "typ_RAO": "экстрагенты"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '19', '19', '{"number": 19, "typ_RAO": "жидкости неорганические прочие"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '18', '18', '{"number": 18, "typ_RAO": "воды промысловые"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '17', '17', '{"number": 17, "typ_RAO": "воды (декантаты) бассейнов-хранилищ, а также емкостей хранения пульп, фильтратов"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '16', '16', '{"number": 16, "typ_RAO": "воды лабораторий, трапные, обмывочные воды, растворы после дезактивации, включая воды санпропускников и спецпрачечных, сточные воды прочие"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '15', '15', '{"number": 15, "typ_RAO": "растворы регенерационные (сливы), растворы промывки экстрагентов, сорбентов"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '14', '14', '{"number": 14, "typ_RAO": "конденсаты после упаривания, газоочистки, конденсаты технологические прочие"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '13', '13', '{"number": 13, "typ_RAO": "растворы хвостовые"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '12', '12', '{"number": 12, "typ_RAO": "воды охлаждения реакторного производства, контурные воды, воды цистерн биологической защиты, воды бассейнов выдержки ОЯТ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '262.0', 'ДЖИБУТИ', '{"2": "ДЖИБУТИ", "v": 262.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '11', '11', '{"number": 11, "typ_RAO": "воды загрязненные подземные, в том числе грунтовые и дренажных систем"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('28_voc_cod_type_rao', '1', '1', '{"number": 1, "typ_RAO": "газы, аэрозоли"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'уран природный', 'уран природный', '{"AM": "", "MOI": "", "MZA": 1000.0, "MZUA": 1000.0, "D_val": "неогран.", "Num_TM": "", "Name_RN": "уран природный", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "а", "Period_p_r": 4470000000.0, "Vid_izluch": "альфа", "Name_RN_Lat": "U-ЕСТ", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": ""}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'торий природный, включая Th-232 ', 'торий природный, включая Th-232 ', '{"AM": "", "MOI": "", "MZA": 1000.0, "MZUA": 1000.0, "D_val": "", "Num_TM": "", "Name_RN": "торий природный, включая Th-232 ", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "а", "Period_p_r": 14000000000.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Th-ЕСТ", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": ""}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сумма нуклидов тория', 'сумма нуклидов тория', '{"AM": "", "MOI": "", "MZA": "", "MZUA": "", "D_val": "", "Num_TM": "", "Name_RN": "сумма нуклидов тория", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "а", "Period_p_r": "", "Vid_izluch": "", "Name_RN_Lat": "Th", "Edinica_izmer_p_r": "", "Osn_vid_raspad_text": ""}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сумма нуклидов плутония', 'сумма нуклидов плутония', '{"AM": "", "MOI": "", "MZA": "", "MZUA": "", "D_val": "", "Num_TM": "", "Name_RN": "сумма нуклидов плутония", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": "", "Vid_izluch": "", "Name_RN_Lat": "Pu", "Edinica_izmer_p_r": "", "Osn_vid_raspad_text": ""}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'менделеевий-258', 'менделеевий-258', '{"AM": 258, "MOI": "", "MZA": 100000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 101, "Name_RN": "менделеевий-258", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 51.5, "Vid_izluch": "альфа", "Name_RN_Lat": "Md-258", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'менделеевий-257', 'менделеевий-257', '{"AM": 257, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 101, "Name_RN": "менделеевий-257", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 5.2, "Vid_izluch": "гамма", "Name_RN_Lat": "Md-257", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'фермий-257', 'фермий-257', '{"AM": 257, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 100, "Name_RN": "фермий-257", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 101.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Fm-257", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'фермий-255', 'фермий-255', '{"AM": 255, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 100, "Name_RN": "фермий-255", "UdA_GRO": 5.4, "UdA_TRO": 1000.0, "ObA_GaRO": 0.4, "Kod_gruppy": "Трансурановый", "Period_p_r": 20.1, "Vid_izluch": "альфа", "Name_RN_Lat": "Fm-255", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'фермий-254', 'фермий-254', '{"AM": 254, "MOI": 10000.0, "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": "", "Num_TM": 100, "Name_RN": "фермий-254", "UdA_GRO": 31.0, "UdA_TRO": 10000.0, "ObA_GaRO": 1.8, "Kod_gruppy": "Трансурановый", "Period_p_r": 3.24, "Vid_izluch": "альфа", "Name_RN_Lat": "Fm-254", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'фермий-253', 'фермий-253', '{"AM": 253, "MOI": "", "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": "", "Num_TM": 100, "Name_RN": "фермий-253", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 3.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Fm-253", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'фермий-252', 'фермий-252', '{"AM": 252, "MOI": "", "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 100, "Name_RN": "фермий-252", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 25.39, "Vid_izluch": "альфа", "Name_RN_Lat": "Fm-252", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'эйнштейний-254', 'эйнштейний-254', '{"AM": 254, "MOI": 0.1, "MZA": 10000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 99, "Name_RN": "эйнштейний-254", "UdA_GRO": 0.49, "UdA_TRO": 10.0, "ObA_GaRO": 0.014, "Kod_gruppy": "Трансурановый", "Period_p_r": 276.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Es-254", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'эйнштейний-253', 'эйнштейний-253', '{"AM": 253, "MOI": 100.0, "MZA": 100000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 99, "Name_RN": "эйнштейний-253", "UdA_GRO": 2.2, "UdA_TRO": 100.0, "ObA_GaRO": 0.04, "Kod_gruppy": "Трансурановый", "Period_p_r": 20.5, "Vid_izluch": "альфа", "Name_RN_Lat": "Es-253", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'эйнштейний-250', 'эйнштейний-250', '{"AM": 250, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 99, "Name_RN": "эйнштейний-250", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 8.6, "Vid_izluch": "гамма", "Name_RN_Lat": "Es-250", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '268.0', 'ГРУЗИЯ', '{"2": "ГРУЗИЯ", "v": 268.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '266.0', 'ГАБОН', '{"2": "ГАБОН", "v": 266.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'эйнштейний-254м', 'эйнштейний-254м', '{"AM": 254, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 99, "Name_RN": "эйнштейний-254м", "UdA_GRO": 3.3, "UdA_TRO": 100.0, "ObA_GaRO": 0.23, "Kod_gruppy": "Трансурановый", "Period_p_r": 1.64, "Vid_izluch": "бета", "Name_RN_Lat": "Es-254m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'калифорний-246', 'калифорний-246', '{"AM": 246, "MOI": 1000.0, "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 98, "Name_RN": "калифорний-246", "UdA_GRO": 4.2, "UdA_TRO": 1000.0, "ObA_GaRO": 0.24, "Kod_gruppy": "Трансурановый", "Period_p_r": 1.49, "Vid_izluch": "альфа", "Name_RN_Lat": "Cf-246", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'калифорний-254', 'калифорний-254', '{"AM": 254, "MOI": 1.0, "MZA": 1000.0, "MZUA": 1000.0, "D_val": 0.0003, "Num_TM": 98, "Name_RN": "калифорний-254", "UdA_GRO": 0.034, "UdA_TRO": 1.0, "ObA_GaRO": 0.0027, "Kod_gruppy": "Трансурановый", "Period_p_r": 60.5, "Vid_izluch": "осколки деления", "Name_RN_Lat": "Cf-254", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "спонтанное деление"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'калифорний-253', 'калифорний-253', '{"AM": 253, "MOI": 100.0, "MZA": 100000.0, "MZUA": 100000.0, "D_val": 0.4, "Num_TM": 98, "Name_RN": "калифорний-253", "UdA_GRO": 9.8, "UdA_TRO": 100.0, "ObA_GaRO": 0.081, "Kod_gruppy": "Трансурановый", "Period_p_r": 17.8, "Vid_izluch": "бета", "Name_RN_Lat": "Cf-253", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'калифорний-252', 'калифорний-252', '{"AM": 252, "MOI": 1.0, "MZA": 10000.0, "MZUA": 10000.0, "D_val": 0.02, "Num_TM": 98, "Name_RN": "калифорний-252", "UdA_GRO": 0.15, "UdA_TRO": 10.0, "ObA_GaRO": 0.0056, "Kod_gruppy": "Трансурановый", "Period_p_r": 2.64, "Vid_izluch": "альфа", "Name_RN_Lat": "Cf-252", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'калифорний-251', 'калифорний-251', '{"AM": 251, "MOI": 0.1, "MZA": 1000.0, "MZUA": 1000.0, "D_val": 0.1, "Num_TM": 98, "Name_RN": "калифорний-251", "UdA_GRO": 0.038, "UdA_TRO": 1.0, "ObA_GaRO": 0.0017, "Kod_gruppy": "Трансурановый", "Period_p_r": 898.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Cf-251", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'калифорний-250', 'калифорний-250', '{"AM": 250, "MOI": 1.0, "MZA": 10000.0, "MZUA": 10000.0, "D_val": 0.1, "Num_TM": 98, "Name_RN": "калифорний-250", "UdA_GRO": 0.086, "UdA_TRO": 10.0, "ObA_GaRO": 0.0036, "Kod_gruppy": "Трансурановый", "Period_p_r": 13.1, "Vid_izluch": "альфа", "Name_RN_Lat": "Cf-250", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'калифорний-249', 'калифорний-249', '{"AM": 249, "MOI": 0.1, "MZA": 1000.0, "MZUA": 1000.0, "D_val": 0.1, "Num_TM": 98, "Name_RN": "калифорний-249", "UdA_GRO": 0.039, "UdA_TRO": 1.0, "ObA_GaRO": 0.0018, "Kod_gruppy": "Трансурановый", "Period_p_r": 350.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Cf-249", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'калифорний-248', 'калифорний-248', '{"AM": 248, "MOI": 1.0, "MZA": 10000.0, "MZUA": 10000.0, "D_val": 0.1, "Num_TM": 98, "Name_RN": "калифорний-248", "UdA_GRO": 0.49, "UdA_TRO": 10.0, "ObA_GaRO": 0.014, "Kod_gruppy": "Трансурановый", "Period_p_r": 334.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Cf-248", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'калифорний-244', 'калифорний-244', '{"AM": 244, "MOI": "", "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": "", "Num_TM": 98, "Name_RN": "калифорний-244", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 0.323, "Vid_izluch": "альфа", "Name_RN_Lat": "Cf-244", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'берклий-250', 'берклий-250', '{"AM": 250, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 97, "Name_RN": "берклий-250", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 3.22, "Vid_izluch": "бета", "Name_RN_Lat": "Bk-250", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'берклий-246', 'берклий-246', '{"AM": 246, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 97, "Name_RN": "берклий-246", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 1.83, "Vid_izluch": "гамма", "Name_RN_Lat": "Bk-246", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'берклий-247', 'берклий-247', '{"AM": 247, "MOI": "", "MZA": 10000.0, "MZUA": 1000.0, "D_val": 0.08, "Num_TM": 97, "Name_RN": "берклий-247", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 1380.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Bk-247", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'берклий-245', 'берклий-245', '{"AM": 245, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 97, "Name_RN": "берклий-245", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 4.94, "Vid_izluch": "гамма", "Name_RN_Lat": "Bk-245", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кюрий-250', 'кюрий-250', '{"AM": 250, "MOI": "", "MZA": 1000.0, "MZUA": 100.0, "D_val": "", "Num_TM": 96, "Name_RN": "кюрий-250", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 8300.0, "Vid_izluch": "осколки деления", "Name_RN_Lat": "Cm-250", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "спонтанное деление"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кюрий-249', 'кюрий-249', '{"AM": 249, "MOI": "", "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 96, "Name_RN": "кюрий-249", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 1.07, "Vid_izluch": "бета", "Name_RN_Lat": "Cm-249", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'америций-244', 'америций-244', '{"AM": 244, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.09, "Num_TM": 95, "Name_RN": "америций-244", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 10.1, "Vid_izluch": "бета", "Name_RN_Lat": "Am-244", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'америций-243', 'америций-243', '{"AM": 243, "MOI": 0.1, "MZA": 1000.0, "MZUA": 1000.0, "D_val": 0.2, "Num_TM": 95, "Name_RN": "америций-243", "UdA_GRO": 0.069, "UdA_TRO": 1.0, "ObA_GaRO": 0.003, "Kod_gruppy": "Трансурановый", "Period_p_r": 7380.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Am-243", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'америций-242м', 'америций-242м', '{"AM": 242, "MOI": 0.1, "MZA": 10000.0, "MZUA": 1000.0, "D_val": 0.3, "Num_TM": 95, "Name_RN": "америций-242м", "UdA_GRO": 0.072, "UdA_TRO": 1.0, "ObA_GaRO": 0.0033, "Kod_gruppy": "Трансурановый", "Period_p_r": 141.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Am-242m", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кюрий-238', 'кюрий-238', '{"AM": 238, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 96, "Name_RN": "кюрий-238", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 2.2, "Vid_izluch": "гамма", "Name_RN_Lat": "Cm-238", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'америций-241', 'америций-241', '{"AM": 241, "MOI": 0.1, "MZA": 10000.0, "MZUA": 1000.0, "D_val": 0.06, "Num_TM": 95, "Name_RN": "америций-241", "UdA_GRO": 0.069, "UdA_TRO": 1.0, "ObA_GaRO": 0.0029, "Kod_gruppy": "Трансурановый", "Period_p_r": 432.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Am-241", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'америций-246', 'америций-246', '{"AM": 246, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 95, "Name_RN": "америций-246", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 0.65, "Vid_izluch": "бета", "Name_RN_Lat": "Am-246", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'америций-245', 'америций-245', '{"AM": 245, "MOI": "", "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 95, "Name_RN": "америций-245", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 2.05, "Vid_izluch": "бета", "Name_RN_Lat": "Am-245", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'америций-244м', 'америций-244м', '{"AM": 244, "MOI": "", "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": "", "Num_TM": 95, "Name_RN": "америций-244м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 0.433, "Vid_izluch": "бета", "Name_RN_Lat": "Am-244m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'америций-242', 'америций-242', '{"AM": 242, "MOI": 1000.0, "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 95, "Name_RN": "америций-242", "UdA_GRO": 46.0, "UdA_TRO": 1000.0, "ObA_GaRO": 6.5, "Kod_gruppy": "Трансурановый", "Period_p_r": 16.0, "Vid_izluch": "бета", "Name_RN_Lat": "Am-242", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'америций-240', 'америций-240', '{"AM": 240, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 95, "Name_RN": "америций-240", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 2.12, "Vid_izluch": "гамма", "Name_RN_Lat": "Am-240", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'америций-238', 'америций-238', '{"AM": 238, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 95, "Name_RN": "америций-238", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 1.63, "Vid_izluch": "гамма", "Name_RN_Lat": "Am-238", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'нептуний-235', 'нептуний-235', '{"AM": 235, "MOI": "", "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": 100, "Num_TM": 93, "Name_RN": "нептуний-235", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 1.08, "Vid_izluch": "гамма", "Name_RN_Lat": "Np-235", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'америций-237', 'америций-237', '{"AM": 237, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 95, "Name_RN": "америций-237", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 1.22, "Vid_izluch": "гамма", "Name_RN_Lat": "Am-237", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'америций-246м', 'америций-246м', '{"AM": 246, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 95, "Name_RN": "америций-246м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 0.417, "Vid_izluch": "бета", "Name_RN_Lat": "Am-246m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'плутоний-246', 'плутоний-246', '{"AM": 246, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 94, "Name_RN": "плутоний-246", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 10.9, "Vid_izluch": "бета", "Name_RN_Lat": "Pu-246", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'плутоний-245', 'плутоний-245', '{"AM": 245, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 94, "Name_RN": "плутоний-245", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 10.5, "Vid_izluch": "бета", "Name_RN_Lat": "Pu-245", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'плутоний-243', 'плутоний-243', '{"AM": 243, "MOI": 1000.0, "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 94, "Name_RN": "плутоний-243", "UdA_GRO": 160.0, "UdA_TRO": 1000.0, "ObA_GaRO": 1300.0, "Kod_gruppy": "Трансурановый", "Period_p_r": 4.95, "Vid_izluch": "бета", "Name_RN_Lat": "Pu-243", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'плутоний-235', 'плутоний-235', '{"AM": 235, "MOI": 100.0, "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 94, "Name_RN": "плутоний-235", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 0.422, "Vid_izluch": "гамма", "Name_RN_Lat": "Pu-235", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'плутоний-234', 'плутоний-234', '{"AM": 234, "MOI": 100.0, "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 94, "Name_RN": "плутоний-234", "UdA_GRO": 85.0, "UdA_TRO": 100.0, "ObA_GaRO": 5.2, "Kod_gruppy": "Трансурановый", "Period_p_r": 8.8, "Vid_izluch": "гамма", "Name_RN_Lat": "Pu-234", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'уран-236', 'уран-236', '{"AM": 236, "MOI": 10.0, "MZA": 10000.0, "MZUA": 10000.0, "D_val": 0.2, "Num_TM": 92, "Name_RN": "уран-236", "UdA_GRO": 0.29, "UdA_TRO": 10.0, "ObA_GaRO": 0.035, "Kod_gruppy": "а", "Period_p_r": 23400000.0, "Vid_izluch": "альфа", "Name_RN_Lat": "U-236", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'уран-235', 'уран-235', '{"AM": 235, "MOI": "", "MZA": 10000.0, "MZUA": 10000.0, "D_val": 0.00008, "Num_TM": 92, "Name_RN": "уран-235", "UdA_GRO": 0.29, "UdA_TRO": 10.0, "ObA_GaRO": 0.037, "Kod_gruppy": "а", "Period_p_r": 704000000.0, "Vid_izluch": "альфа", "Name_RN_Lat": "U-235", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'уран-233', 'уран-233', '{"AM": 233, "MOI": 1.0, "MZA": 10000.0, "MZUA": 10000.0, "D_val": 0.07, "Num_TM": 92, "Name_RN": "уран-233", "UdA_GRO": 0.27, "UdA_TRO": 10.0, "ObA_GaRO": 0.032, "Kod_gruppy": "а", "Period_p_r": 158000.0, "Vid_izluch": "альфа", "Name_RN_Lat": "U-233", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'уран-232', 'уран-232', '{"AM": 232, "MOI": 0.1, "MZA": 1000.0, "MZUA": 1000.0, "D_val": 0.06, "Num_TM": 92, "Name_RN": "уран-232", "UdA_GRO": 0.042, "UdA_TRO": 1.0, "ObA_GaRO": 0.014, "Kod_gruppy": "а", "Period_p_r": 72.0, "Vid_izluch": "альфа", "Name_RN_Lat": "U-232", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'нептуний-236', 'нептуний-236', '{"AM": 236, "MOI": "", "MZA": 100000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 93, "Name_RN": "нептуний-236", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 153000.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Np-236", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'уран-230', 'уран-230', '{"AM": 230, "MOI": 10.0, "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.04, "Num_TM": 92, "Name_RN": "уран-230", "UdA_GRO": 0.25, "UdA_TRO": 10.0, "ObA_GaRO": 0.0081, "Kod_gruppy": "а", "Period_p_r": 20.8, "Vid_izluch": "альфа", "Name_RN_Lat": "U-230", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'актиний-227', 'актиний-227', '{"AM": 227, "MOI": "", "MZA": 1000.0, "MZUA": 100.0, "D_val": 0.04, "Num_TM": 89, "Name_RN": "актиний-227", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 21.772, "Vid_izluch": "бета", "Name_RN_Lat": "Ac-227", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'нептуний-234', 'нептуний-234', '{"AM": 234, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 93, "Name_RN": "нептуний-234", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 4.4, "Vid_izluch": "бета", "Name_RN_Lat": "Np-234", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'актиний-225', 'актиний-225', '{"AM": 225, "MOI": "", "MZA": 10000.0, "MZUA": 10000.0, "D_val": 0.09, "Num_TM": 89, "Name_RN": "актиний-225", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "а", "Period_p_r": 9.9203, "Vid_izluch": "альфа", "Name_RN_Lat": "Ac-225", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '270.0', 'ГАМБИЯ', '{"2": "ГАМБИЯ", "v": 270.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'нептуний-233', 'нептуний-233', '{"AM": 233, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 93, "Name_RN": "нептуний-233", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 0.603, "Vid_izluch": "гамма", "Name_RN_Lat": "Np-233", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'радий-228', 'радий-228', '{"AM": 228, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.03, "Num_TM": 88, "Name_RN": "радий-228", "UdA_GRO": 0.02, "UdA_TRO": 10.0, "ObA_GaRO": 0.031, "Kod_gruppy": "б", "Period_p_r": 5.75, "Vid_izluch": "бета", "Name_RN_Lat": "Ra-228", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'нептуний-232', 'нептуний-232', '{"AM": 232, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 93, "Name_RN": "нептуний-232", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 0.245, "Vid_izluch": "гамма", "Name_RN_Lat": "Np-232", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'радий-226', 'радий-226', '{"AM": 226, "MOI": "", "MZA": 10000.0, "MZUA": 10000.0, "D_val": 0.04, "Num_TM": 88, "Name_RN": "радий-226", "UdA_GRO": 0.049, "UdA_TRO": 10.0, "ObA_GaRO": 0.03, "Kod_gruppy": "а", "Period_p_r": 1600.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Ra-226", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'радий-225', 'радий-225', '{"AM": 225, "MOI": 10.0, "MZA": 100000.0, "MZUA": 100000.0, "D_val": 0.1, "Num_TM": 88, "Name_RN": "радий-225", "UdA_GRO": 0.14, "UdA_TRO": 100.0, "ObA_GaRO": 0.017, "Kod_gruppy": "б", "Period_p_r": 14.8, "Vid_izluch": "бета", "Name_RN_Lat": "Ra-225", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'радий-224', 'радий-224', '{"AM": 224, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.05, "Num_TM": 88, "Name_RN": "радий-224", "UdA_GRO": 0.21, "UdA_TRO": 10.0, "ObA_GaRO": 0.037, "Kod_gruppy": "а", "Period_p_r": 3.66, "Vid_izluch": "альфа", "Name_RN_Lat": "Ra-224", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'радий-223', 'радий-223', '{"AM": 223, "MOI": "", "MZA": 100000.0, "MZUA": 100000.0, "D_val": 0.1, "Num_TM": 88, "Name_RN": "радий-223", "UdA_GRO": 0.14, "UdA_TRO": 100.0, "ObA_GaRO": 0.015, "Kod_gruppy": "а", "Period_p_r": 11.4, "Vid_izluch": "альфа", "Name_RN_Lat": "Ra-223", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'нептуний-240', 'нептуний-240', '{"AM": 240, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 93, "Name_RN": "нептуний-240", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 1100.0, "Kod_gruppy": "Трансурановый", "Period_p_r": 1.08, "Vid_izluch": "бета", "Name_RN_Lat": "Np-240", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'нептуний-238', 'нептуний-238', '{"AM": 238, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 93, "Name_RN": "нептуний-238", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 2.12, "Vid_izluch": "бета", "Name_RN_Lat": "Np-238", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'астат-211', 'астат-211', '{"AM": 211, "MOI": 1000.0, "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": 0.5, "Num_TM": 85, "Name_RN": "астат-211", "UdA_GRO": 1.2, "UdA_TRO": 1000.0, "ObA_GaRO": 1.05, "Kod_gruppy": "а", "Period_p_r": 7.21, "Vid_izluch": "альфа", "Name_RN_Lat": "At-211", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'нептуний-236м', 'нептуний-236м', '{"AM": 236, "MOI": "", "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 93, "Name_RN": "нептуний-236м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 22.5, "Vid_izluch": "бета", "Name_RN_Lat": "Np-236m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'полоний-210', 'полоний-210', '{"AM": 210, "MOI": "", "MZA": 10000.0, "MZUA": 10000.0, "D_val": 0.06, "Num_TM": 84, "Name_RN": "полоний-210", "UdA_GRO": 0.011, "UdA_TRO": 10.0, "ObA_GaRO": 0.034, "Kod_gruppy": "а", "Period_p_r": 138.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Po-210", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'свинец-212', 'свинец-212', '{"AM": 212, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.05, "Num_TM": 82, "Name_RN": "свинец-212", "UdA_GRO": 2.2, "UdA_TRO": 10.0, "ObA_GaRO": 0.62, "Kod_gruppy": "б", "Period_p_r": 10.6, "Vid_izluch": "бета", "Name_RN_Lat": "Pb-212", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'уран-240', 'уран-240', '{"AM": 240, "MOI": 100.0, "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 92, "Name_RN": "уран-240", "UdA_GRO": 12.0, "UdA_TRO": 1000.0, "ObA_GaRO": 160.0, "Kod_gruppy": "б", "Period_p_r": 14.1, "Vid_izluch": "бета", "Name_RN_Lat": "U-240", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'свинец-210', 'свинец-210', '{"AM": 210, "MOI": "", "MZA": 10000.0, "MZUA": 10000.0, "D_val": 0.3, "Num_TM": 82, "Name_RN": "свинец-210", "UdA_GRO": 0.02, "UdA_TRO": 10.0, "ObA_GaRO": 0.11, "Kod_gruppy": "б", "Period_p_r": 22.3, "Vid_izluch": "бета", "Name_RN_Lat": "Pb-210", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'уран-239', 'уран-239', '{"AM": 239, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 92, "Name_RN": "уран-239", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.392, "Vid_izluch": "бета", "Name_RN_Lat": "U-239", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'свинец-203', 'свинец-203', '{"AM": 203, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.2, "Num_TM": 82, "Name_RN": "свинец-203", "UdA_GRO": 57.0, "UdA_TRO": 100.0, "ObA_GaRO": 530.0, "Kod_gruppy": "б", "Period_p_r": 2.17, "Vid_izluch": "гамма", "Name_RN_Lat": "Pb-203", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'уран-237', 'уран-237', '{"AM": 237, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 92, "Name_RN": "уран-237", "UdA_GRO": 18.0, "UdA_TRO": 100.0, "ObA_GaRO": 65.0, "Kod_gruppy": "б", "Period_p_r": 6.75, "Vid_izluch": "бета", "Name_RN_Lat": "U-237", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'свинец-202', 'свинец-202', '{"AM": 202, "MOI": "", "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": 0.2, "Num_TM": 82, "Name_RN": "свинец-202", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 52500.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Pb-202", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'свинец-201', 'свинец-201', '{"AM": 201, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.09, "Num_TM": 82, "Name_RN": "свинец-201", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 9.4, "Vid_izluch": "бета", "Name_RN_Lat": "Pb-201", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'уран-231', 'уран-231', '{"AM": 231, "MOI": 100.0, "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 92, "Name_RN": "уран-231", "UdA_GRO": 49.0, "UdA_TRO": 100.0, "ObA_GaRO": 300.0, "Kod_gruppy": "а", "Period_p_r": 4.2, "Vid_izluch": "альфа", "Name_RN_Lat": "U-231", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'протактиний-234м', 'протактиний-234м', '{"AM": 234, "MOI": "", "MZA": "", "MZUA": "", "D_val": "", "Num_TM": 91, "Name_RN": "протактиний-234м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.18, "Vid_izluch": "бета", "Name_RN_Lat": "Pa-234м", "Edinica_izmer_p_r": "мин", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'протактиний-234', 'протактиний-234', '{"AM": 234, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 91, "Name_RN": "протактиний-234", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 6.7, "Vid_izluch": "бета", "Name_RN_Lat": "Pa-234", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'протактиний-232', 'протактиний-232', '{"AM": 232, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 91, "Name_RN": "протактиний-232", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.31, "Vid_izluch": "бета", "Name_RN_Lat": "Pa-232", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'таллий-204', 'таллий-204', '{"AM": 204, "MOI": 1.0, "MZA": 10000.0, "MZUA": 10000000.0, "D_val": 20, "Num_TM": 81, "Name_RN": "таллий-204", "UdA_GRO": 11.0, "UdA_TRO": 10000.0, "ObA_GaRO": 160.0, "Kod_gruppy": "б", "Period_p_r": 3.78, "Vid_izluch": "бета", "Name_RN_Lat": "Tl-204", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'таллий-202', 'таллий-202', '{"AM": 202, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.2, "Num_TM": 81, "Name_RN": "таллий-202", "UdA_GRO": 30.0, "UdA_TRO": 100.0, "ObA_GaRO": 440.0, "Kod_gruppy": "б", "Period_p_r": 12.2, "Vid_izluch": "бета", "Name_RN_Lat": "Tl-202", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'таллий-201', 'таллий-201', '{"AM": 201, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 1, "Num_TM": 81, "Name_RN": "таллий-201", "UdA_GRO": "", "UdA_TRO": 100.0, "ObA_GaRO": 1600.0, "Kod_gruppy": "б", "Period_p_r": 3.04, "Vid_izluch": "гамма", "Name_RN_Lat": "Tl-201", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'протактиний-228', 'протактиний-228', '{"AM": 228, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 91, "Name_RN": "протактиний-228", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 22.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Pa-228", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'протактиний-227', 'протактиний-227', '{"AM": 227, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 91, "Name_RN": "протактиний-227", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "а", "Period_p_r": 0.638, "Vid_izluch": "альфа", "Name_RN_Lat": "Pa-227", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'торий-232', 'торий-232', '{"AM": 232, "MOI": "", "MZA": 1000.0, "MZUA": 1000.0, "D_val": "неогран.", "Num_TM": 90, "Name_RN": "торий-232", "UdA_GRO": 0.061, "UdA_TRO": 1.0, "ObA_GaRO": 0.0049, "Kod_gruppy": "а", "Period_p_r": 14000000000.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Th-232", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'торий-226', 'торий-226', '{"AM": 226, "MOI": 1000.0, "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 90, "Name_RN": "торий-226", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "а", "Period_p_r": 0.515, "Vid_izluch": "альфа", "Name_RN_Lat": "Th-226", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'актиний-226', 'актиний-226', '{"AM": 226, "MOI": "", "MZA": 100000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 89, "Name_RN": "актиний-226", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.21, "Vid_izluch": "бета", "Name_RN_Lat": "Ac-226", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'золото-198', 'золото-198', '{"AM": 198, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.2, "Num_TM": 79, "Name_RN": "золото-198", "UdA_GRO": 14.0, "UdA_TRO": 100.0, "ObA_GaRO": 120.0, "Kod_gruppy": "б", "Period_p_r": 2.69, "Vid_izluch": "бета", "Name_RN_Lat": "Au-198", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'золото-195', 'золото-195', '{"AM": 195, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": 2, "Num_TM": 79, "Name_RN": "золото-195", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 186.01, "Vid_izluch": "гамма", "Name_RN_Lat": "Au-195", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'золото-194', 'золото-194', '{"AM": 194, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.07, "Num_TM": 79, "Name_RN": "золото-194", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.64, "Vid_izluch": "бета", "Name_RN_Lat": "Au-194", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'золото-193', 'золото-193', '{"AM": 193, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": 0.6, "Num_TM": 79, "Name_RN": "золото-193", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 17.6, "Vid_izluch": "бета", "Name_RN_Lat": "Au-193", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'актиний-224', 'актиний-224', '{"AM": 224, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 89, "Name_RN": "актиний-224", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.9, "Vid_izluch": "гамма", "Name_RN_Lat": "Ac-224", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'радий-227', 'радий-227', '{"AM": 227, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 88, "Name_RN": "радий-227", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.703, "Vid_izluch": "бета", "Name_RN_Lat": "Ra-227", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'платина-197м', 'платина-197м', '{"AM": 197, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.9, "Num_TM": 78, "Name_RN": "платина-197м", "UdA_GRO": "", "UdA_TRO": 100.0, "ObA_GaRO": 2900.0, "Kod_gruppy": "б", "Period_p_r": 1.57, "Vid_izluch": "гамма", "Name_RN_Lat": "Pt-197m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'платина-197', 'платина-197', '{"AM": 197, "MOI": 1000.0, "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": 4, "Num_TM": 78, "Name_RN": "платина-197", "UdA_GRO": 34.0, "UdA_TRO": 1000.0, "ObA_GaRO": 720.0, "Kod_gruppy": "б", "Period_p_r": 18.3, "Vid_izluch": "бета", "Name_RN_Lat": "Pt-197", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'платина-195м', 'платина-195м', '{"AM": 195, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 2, "Num_TM": 78, "Name_RN": "платина-195м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 4.02, "Vid_izluch": "гамма", "Name_RN_Lat": "Pt-195m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'платина-193м', 'платина-193м', '{"AM": 193, "MOI": 1000.0, "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": 10, "Num_TM": 78, "Name_RN": "платина-193м", "UdA_GRO": 30.0, "UdA_TRO": 1000.0, "ObA_GaRO": 530.0, "Kod_gruppy": "б", "Period_p_r": 4.33, "Vid_izluch": "гамма", "Name_RN_Lat": "Pt-193m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иридий-189', 'иридий-189', '{"AM": 189, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": 1, "Num_TM": 77, "Name_RN": "иридий-189", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 13.3, "Vid_izluch": "гамма", "Name_RN_Lat": "Ir-189", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'франций-223', 'франций-223', '{"AM": 223, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 87, "Name_RN": "франций-223", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.363, "Vid_izluch": "бета", "Name_RN_Lat": "Fr-223", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'франций-222', 'франций-222', '{"AM": 222, "MOI": "", "MZA": 100000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 87, "Name_RN": "франций-222", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.24, "Vid_izluch": "бета", "Name_RN_Lat": "Fr-222", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'астат-207', 'астат-207', '{"AM": 207, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 85, "Name_RN": "астат-207", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.8, "Vid_izluch": "бета", "Name_RN_Lat": "At-207", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'полоний-212', 'полоний-212', '{"AM": 212, "MOI": 0.0, "MZA": 0.0, "MZUA": 0.0, "D_val": 0, "Num_TM": 84, "Name_RN": "полоний-212", "UdA_GRO": 0.0, "UdA_TRO": 0.0, "ObA_GaRO": 0.0, "Kod_gruppy": "а", "Period_p_r": 0.0000299, "Vid_izluch": "альфа", "Name_RN_Lat": "Po-212", "Edinica_izmer_p_r": "сек", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'полоний-208', 'полоний-208', '{"AM": 208, "MOI": "", "MZA": 10000.0, "MZUA": 10000.0, "D_val": 0, "Num_TM": 84, "Name_RN": "полоний-208", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": "", "Kod_gruppy": "а", "Period_p_r": 2.9, "Vid_izluch": "альфа", "Name_RN_Lat": "Po-208", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'полоний-209', 'полоний-209', '{"AM": 209, "MOI": "", "MZA": 10000.0, "MZUA": 10000.0, "D_val": 0, "Num_TM": 84, "Name_RN": "полоний-209", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 103.0, "Vid_izluch": "", "Name_RN_Lat": "Po-209", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": ""}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'осмий-194', 'осмий-194', '{"AM": 194, "MOI": "", "MZA": 100000.0, "MZUA": 100000.0, "D_val": 0.7, "Num_TM": 76, "Name_RN": "осмий-194", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 6.0, "Vid_izluch": "бета", "Name_RN_Lat": "Os-194", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'осмий-193', 'осмий-193', '{"AM": 193, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 1, "Num_TM": 76, "Name_RN": "осмий-193", "UdA_GRO": 17.0, "UdA_TRO": 100.0, "ObA_GaRO": 190.0, "Kod_gruppy": "б", "Period_p_r": 1.25, "Vid_izluch": "бета", "Name_RN_Lat": "Os-193", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'осмий-191м', 'осмий-191м', '{"AM": 191, "MOI": 1000.0, "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": 1, "Num_TM": 76, "Name_RN": "осмий-191м", "UdA_GRO": 140.0, "UdA_TRO": 1000.0, "ObA_GaRO": 680.0, "Kod_gruppy": "б", "Period_p_r": 13.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Os-191m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'осмий-191', 'осмий-191', '{"AM": 191, "MOI": 100.0, "MZA": 10000000.0, "MZUA": 100000.0, "D_val": 2, "Num_TM": 76, "Name_RN": "осмий-191", "UdA_GRO": 24.0, "UdA_TRO": 100.0, "ObA_GaRO": 60.0, "Kod_gruppy": "б", "Period_p_r": 15.4, "Vid_izluch": "бета", "Name_RN_Lat": "Os-191", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'полоний-218', 'полоний-218', '{"AM": 218, "MOI": "", "MZA": "", "MZUA": "", "D_val": "", "Num_TM": 84, "Name_RN": "полоний-218", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "а", "Period_p_r": 3.1, "Vid_izluch": "альфа", "Name_RN_Lat": "Po-218", "Edinica_izmer_p_r": "мин", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'осмий-185', 'осмий-185', '{"AM": 185, "MOI": 1.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.1, "Num_TM": 76, "Name_RN": "осмий-185", "UdA_GRO": 27.0, "UdA_TRO": 10.0, "ObA_GaRO": 72.0, "Kod_gruppy": "б", "Period_p_r": 94.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Os-185", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'полоний-207', 'полоний-207', '{"AM": 207, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 84, "Name_RN": "полоний-207", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 1000.0, "Kod_gruppy": "б", "Period_p_r": 5.83, "Vid_izluch": "бета", "Name_RN_Lat": "Po-207", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'полоний-205', 'полоний-205', '{"AM": 205, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 84, "Name_RN": "полоний-205", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 1600.0, "Kod_gruppy": "б", "Period_p_r": 1.8, "Vid_izluch": "бета", "Name_RN_Lat": "Po-205", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'полоний-203', 'полоний-203', '{"AM": 203, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 84, "Name_RN": "полоний-203", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.612, "Vid_izluch": "бета", "Name_RN_Lat": "Po-203", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'висмут-214', 'висмут-214', '{"AM": 214, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 83, "Name_RN": "висмут-214", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.332, "Vid_izluch": "бета", "Name_RN_Lat": "Bi-214", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'висмут-213', 'висмут-213', '{"AM": 213, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 83, "Name_RN": "висмут-213", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.761, "Vid_izluch": "бета", "Name_RN_Lat": "Bi-213", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'висмут-203', 'висмут-203', '{"AM": 203, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 83, "Name_RN": "висмут-203", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 11.8, "Vid_izluch": "бета", "Name_RN_Lat": "Bi-203", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'висмут-200', 'висмут-200', '{"AM": 200, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 83, "Name_RN": "висмут-200", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.606, "Vid_izluch": "бета", "Name_RN_Lat": "Bi-200", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'свинец-211', 'свинец-211', '{"AM": 211, "MOI": "", "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 82, "Name_RN": "свинец-211", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.601, "Vid_izluch": "бета", "Name_RN_Lat": "Pb-211", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'свинец-209', 'свинец-209', '{"AM": 209, "MOI": "", "MZA": 1000000.0, "MZUA": 100000000.0, "D_val": "", "Num_TM": 82, "Name_RN": "свинец-209", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.25, "Vid_izluch": "бета", "Name_RN_Lat": "Pb-209", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'свинец-205', 'свинец-205', '{"AM": 205, "MOI": "", "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": "неогран.", "Num_TM": 82, "Name_RN": "свинец-205", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 17300000.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Pb-205", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'свинец-202м', 'свинец-202м', '{"AM": 202, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 82, "Name_RN": "свинец-202м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.62, "Vid_izluch": "гамма", "Name_RN_Lat": "Pb-202m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'свинец-200', 'свинец-200', '{"AM": 200, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 82, "Name_RN": "свинец-200", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 21.5, "Vid_izluch": "гамма", "Name_RN_Lat": "Pb-200", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'свинец-199', 'свинец-199', '{"AM": 199, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 82, "Name_RN": "свинец-199", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.5, "Vid_izluch": "бета", "Name_RN_Lat": "Pb-199", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'свинец-198', 'свинец-198', '{"AM": 198, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 82, "Name_RN": "свинец-198", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.4, "Vid_izluch": "гамма", "Name_RN_Lat": "Pb-198", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'свинец-195м', 'свинец-195м', '{"AM": 195, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 82, "Name_RN": "свинец-195м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.263, "Vid_izluch": "бета", "Name_RN_Lat": "Pb-195m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'свинец-214', 'свинец-214', '{"AM": 214, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 82, "Name_RN": "свинец-214", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.447, "Vid_izluch": "бета", "Name_RN_Lat": "Pb-214", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'таллий-199', 'таллий-199', '{"AM": 199, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 81, "Name_RN": "таллий-199", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 7.42, "Vid_izluch": "гамма", "Name_RN_Lat": "Tl-199", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'таллий-198м', 'таллий-198м', '{"AM": 198, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 81, "Name_RN": "таллий-198м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.87, "Vid_izluch": "гамма", "Name_RN_Lat": "Tl-198m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'таллий-198', 'таллий-198', '{"AM": 198, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 81, "Name_RN": "таллий-198", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 5.3, "Vid_izluch": "бета", "Name_RN_Lat": "Tl-198", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'таллий-197', 'таллий-197', '{"AM": 197, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 81, "Name_RN": "таллий-197", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.84, "Vid_izluch": "бета", "Name_RN_Lat": "Tl-197", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кюрий-248', 'кюрий-248', '{"AM": 248, "MOI": 0.1, "MZA": 1000.0, "MZUA": 1000.0, "D_val": 0.005, "Num_TM": 96, "Name_RN": "кюрий-248", "UdA_GRO": 0.018, "UdA_TRO": 1.0, "ObA_GaRO": 0.00082, "Kod_gruppy": "Трансурановый", "Period_p_r": 348000.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Cm-248", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кюрий-246', 'кюрий-246', '{"AM": 246, "MOI": 0.1, "MZA": 1000.0, "MZUA": 1000.0, "D_val": 0.2, "Num_TM": 96, "Name_RN": "кюрий-246", "UdA_GRO": 0.065, "UdA_TRO": 1.0, "ObA_GaRO": 0.0029, "Kod_gruppy": "Трансурановый", "Period_p_r": 4730.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Cm-246", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кюрий-245', 'кюрий-245', '{"AM": 245, "MOI": 0.1, "MZA": 1000.0, "MZUA": 1000.0, "D_val": 0.09, "Num_TM": 96, "Name_RN": "кюрий-245", "UdA_GRO": 0.065, "UdA_TRO": 1.0, "ObA_GaRO": 0.0029, "Kod_gruppy": "Трансурановый", "Period_p_r": 8500.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Cm-245", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кюрий-244', 'кюрий-244', '{"AM": 244, "MOI": 1.0, "MZA": 10000.0, "MZUA": 10000.0, "D_val": 0.05, "Num_TM": 96, "Name_RN": "кюрий-244", "UdA_GRO": 0.11, "UdA_TRO": 10.0, "ObA_GaRO": 0.0046, "Kod_gruppy": "Трансурановый", "Period_p_r": 18.1, "Vid_izluch": "альфа", "Name_RN_Lat": "Cm-244", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кюрий-243', 'кюрий-243', '{"AM": 243, "MOI": 1.0, "MZA": 10000.0, "MZUA": 1000.0, "D_val": 0.2, "Num_TM": 96, "Name_RN": "кюрий-243", "UdA_GRO": 0.091, "UdA_TRO": 1.0, "ObA_GaRO": 0.004, "Kod_gruppy": "Трансурановый", "Period_p_r": 28.5, "Vid_izluch": "альфа", "Name_RN_Lat": "Cm-243", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кюрий-242', 'кюрий-242', '{"AM": 242, "MOI": 10.0, "MZA": 100000.0, "MZUA": 100000.0, "D_val": 0.04, "Num_TM": 96, "Name_RN": "кюрий-242", "UdA_GRO": 1.4, "UdA_TRO": 100.0, "ObA_GaRO": 0.021, "Kod_gruppy": "Трансурановый", "Period_p_r": 163.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Cm-242", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кюрий-241', 'кюрий-241', '{"AM": 241, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.1, "Num_TM": 96, "Name_RN": "кюрий-241", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 32.8, "Vid_izluch": "гамма", "Name_RN_Lat": "Cm-241", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кюрий-240', 'кюрий-240', '{"AM": 240, "MOI": "", "MZA": 100000.0, "MZUA": 100000.0, "D_val": 0.3, "Num_TM": 96, "Name_RN": "кюрий-240", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "Трансурановый", "Period_p_r": 27.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Cm-240", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'таллий-195', 'таллий-195', '{"AM": 195, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 81, "Name_RN": "таллий-195", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.16, "Vid_izluch": "бета", "Name_RN_Lat": "Tl-195", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'таллий-194м', 'таллий-194м', '{"AM": 194, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 81, "Name_RN": "таллий-194м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.546, "Vid_izluch": "бета", "Name_RN_Lat": "Tl-194m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'таллий-194', 'таллий-194', '{"AM": 194, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 81, "Name_RN": "таллий-194", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.55, "Vid_izluch": "бета", "Name_RN_Lat": "Tl-194", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'плутоний-242', 'плутоний-242', '{"AM": 242, "MOI": 0.1, "MZA": 10000.0, "MZUA": 1000.0, "D_val": 0.07, "Num_TM": 94, "Name_RN": "плутоний-242", "UdA_GRO": 0.057, "UdA_TRO": 1.0, "ObA_GaRO": 0.0026, "Kod_gruppy": "Трансурановый", "Period_p_r": 376000.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Pu-242", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'плутоний-241', 'плутоний-241', '{"AM": 241, "MOI": 10.0, "MZA": 100000.0, "MZUA": 100000.0, "D_val": 3, "Num_TM": 94, "Name_RN": "плутоний-241", "UdA_GRO": 2.9, "UdA_TRO": 100.0, "ObA_GaRO": 0.14, "Kod_gruppy": "Трансурановый", "Period_p_r": 14.4, "Vid_izluch": "бета", "Name_RN_Lat": "Pu-241", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'плутоний-240', 'плутоний-240', '{"AM": 240, "MOI": 0.1, "MZA": 1000.0, "MZUA": 1000.0, "D_val": 0.06, "Num_TM": 94, "Name_RN": "плутоний-240", "UdA_GRO": 0.055, "UdA_TRO": 1.0, "ObA_GaRO": 0.0025, "Kod_gruppy": "Трансурановый", "Period_p_r": 6540.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Pu-240", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'плутоний-239', 'плутоний-239', '{"AM": 239, "MOI": 0.1, "MZA": 10000.0, "MZUA": 1000.0, "D_val": 0.06, "Num_TM": 94, "Name_RN": "плутоний-239", "UdA_GRO": 0.055, "UdA_TRO": 1.0, "ObA_GaRO": 0.0025, "Kod_gruppy": "Трансурановый", "Period_p_r": 24100.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Pu-239", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'плутоний-237', 'плутоний-237', '{"AM": 237, "MOI": 100.0, "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": 2, "Num_TM": 94, "Name_RN": "плутоний-237", "UdA_GRO": 140.0, "UdA_TRO": 1000.0, "ObA_GaRO": 320.0, "Kod_gruppy": "Трансурановый", "Period_p_r": 45.3, "Vid_izluch": "гамма", "Name_RN_Lat": "Pu-237", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'плутоний-236', 'плутоний-236', '{"AM": 236, "MOI": 1.0, "MZA": 10000.0, "MZUA": 10000.0, "D_val": 0.1, "Num_TM": 94, "Name_RN": "плутоний-236", "UdA_GRO": 0.16, "UdA_TRO": 10.0, "ObA_GaRO": 0.0062, "Kod_gruppy": "Трансурановый", "Period_p_r": 2.85, "Vid_izluch": "альфа", "Name_RN_Lat": "Pu-236", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ртуть-193', 'ртуть-193', '{"AM": 193, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 80, "Name_RN": "ртуть-193", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.5, "Vid_izluch": "бета", "Name_RN_Lat": "Hg-193", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ртуть-199м', 'ртуть-199м', '{"AM": 199, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 80, "Name_RN": "ртуть-199м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.71, "Vid_izluch": "гамма", "Name_RN_Lat": "Hg-199m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ртуть-195', 'ртуть-195', '{"AM": 195, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 80, "Name_RN": "ртуть-195", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 9.9, "Vid_izluch": "гамма", "Name_RN_Lat": "Hg-195", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'нептуний-239', 'нептуний-239', '{"AM": 239, "MOI": 100.0, "MZA": 10000000.0, "MZUA": 100000.0, "D_val": 0.5, "Num_TM": 93, "Name_RN": "нептуний-239", "UdA_GRO": 17.0, "UdA_TRO": 100.0, "ObA_GaRO": 110.0, "Kod_gruppy": "Трансурановый", "Period_p_r": 2.36, "Vid_izluch": "бета", "Name_RN_Lat": "Np-239", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ртуть-193м', 'ртуть-193м', '{"AM": 193, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 80, "Name_RN": "ртуть-193м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 11.1, "Vid_izluch": "бета", "Name_RN_Lat": "Hg-193m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'нептуний-237', 'нептуний-237', '{"AM": 237, "MOI": 1.0, "MZA": 1000.0, "MZUA": 1000.0, "D_val": 0.07, "Num_TM": 93, "Name_RN": "нептуний-237", "UdA_GRO": 0.13, "UdA_TRO": 1.0, "ObA_GaRO": 0.0054, "Kod_gruppy": "Трансурановый", "Period_p_r": 2140000.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Np-237", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'золото-201', 'золото-201', '{"AM": 201, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 79, "Name_RN": "золото-201", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.44, "Vid_izluch": "бета", "Name_RN_Lat": "Au-201", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'золото-200м', 'золото-200м', '{"AM": 200, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 79, "Name_RN": "золото-200м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 18.7, "Vid_izluch": "бета", "Name_RN_Lat": "Au-200m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'протактиний-233', 'протактиний-233', '{"AM": 233, "MOI": 10.0, "MZA": 10000000.0, "MZUA": 100000.0, "D_val": 0.4, "Num_TM": 91, "Name_RN": "протактиний-233", "UdA_GRO": 16.0, "UdA_TRO": 100.0, "ObA_GaRO": 28.0, "Kod_gruppy": "б", "Period_p_r": 27.0, "Vid_izluch": "бета", "Name_RN_Lat": "Pa-233", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'золото-200', 'золото-200', '{"AM": 200, "MOI": "", "MZA": 100000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 79, "Name_RN": "золото-200", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.807, "Vid_izluch": "бета", "Name_RN_Lat": "Au-200", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'протактиний-231', 'протактиний-231', '{"AM": 231, "MOI": "", "MZA": 1000.0, "MZUA": 1000.0, "D_val": 0.06, "Num_TM": 91, "Name_RN": "протактиний-231", "UdA_GRO": 0.019, "UdA_TRO": 1.0, "ObA_GaRO": 0.00088, "Kod_gruppy": "а", "Period_p_r": 32700.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Pa-231", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'протактиний-230', 'протактиний-230', '{"AM": 230, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.1, "Num_TM": 91, "Name_RN": "протактиний-230", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 0.14, "Kod_gruppy": "б", "Period_p_r": 17.4, "Vid_izluch": "гамма", "Name_RN_Lat": "Pa-230", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'золото-198м', 'золото-198м', '{"AM": 198, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 79, "Name_RN": "золото-198м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.3, "Vid_izluch": "гамма", "Name_RN_Lat": "Au-198m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'торий-234', 'торий-234', '{"AM": 234, "MOI": "", "MZA": 100000.0, "MZUA": 1000000.0, "D_val": 2, "Num_TM": 90, "Name_RN": "торий-234", "UdA_GRO": 4.0, "UdA_TRO": 1000.0, "ObA_GaRO": 15.0, "Kod_gruppy": "б", "Period_p_r": 24.1, "Vid_izluch": "бета", "Name_RN_Lat": "Th-234", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'платина-199', 'платина-199', '{"AM": 199, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 78, "Name_RN": "платина-199", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.513, "Vid_izluch": "бета", "Name_RN_Lat": "Pt-199", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'торий-231', 'торий-231', '{"AM": 231, "MOI": "", "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": 10, "Num_TM": 90, "Name_RN": "торий-231", "UdA_GRO": 40.0, "UdA_TRO": 1000.0, "ObA_GaRO": 310.0, "Kod_gruppy": "б", "Period_p_r": 1.06, "Vid_izluch": "бета", "Name_RN_Lat": "Th-231", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'торий-230', 'торий-230', '{"AM": 230, "MOI": "", "MZA": 10000.0, "MZUA": 1000.0, "D_val": 0.07, "Num_TM": 90, "Name_RN": "торий-230", "UdA_GRO": 0.065, "UdA_TRO": 1.0, "ObA_GaRO": 0.0088, "Kod_gruppy": "а", "Period_p_r": 77000.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Th-230", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'торий-229', 'торий-229', '{"AM": 229, "MOI": 0.1, "MZA": 1000.0, "MZUA": 1000.0, "D_val": 0.01, "Num_TM": 90, "Name_RN": "торий-229", "UdA_GRO": 0.028, "UdA_TRO": 1.0, "ObA_GaRO": 0.0017, "Kod_gruppy": "а", "Period_p_r": 7340.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Th-229", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'торий-228', 'торий-228', '{"AM": 228, "MOI": "", "MZA": 10000.0, "MZUA": 1000.0, "D_val": 0.04, "Num_TM": 90, "Name_RN": "торий-228", "UdA_GRO": 0.19, "UdA_TRO": 1.0, "ObA_GaRO": 0.0029, "Kod_gruppy": "а", "Period_p_r": 1.91, "Vid_izluch": "альфа", "Name_RN_Lat": "Th-228", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'торий-227', 'торий-227', '{"AM": 227, "MOI": "", "MZA": 10000.0, "MZUA": 10000.0, "D_val": 0.08, "Num_TM": 90, "Name_RN": "торий-227", "UdA_GRO": 1.6, "UdA_TRO": 10.0, "ObA_GaRO": 0.011, "Kod_gruppy": "а", "Period_p_r": 18.7, "Vid_izluch": "альфа", "Name_RN_Lat": "Th-227", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '288.0', 'ГАНА', '{"2": "ГАНА", "v": 288.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'платина-189', 'платина-189', '{"AM": 189, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 78, "Name_RN": "платина-189", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 10.9, "Vid_izluch": "бета", "Name_RN_Lat": "Pt-189", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'актиний-228', 'актиний-228', '{"AM": 228, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.03, "Num_TM": 89, "Name_RN": "актиний-228", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 3.2, "Kod_gruppy": "б", "Period_p_r": 6.13, "Vid_izluch": "бета", "Name_RN_Lat": "Ac-228", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'платина-186', 'платина-186', '{"AM": 186, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 78, "Name_RN": "платина-186", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Pt-186", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иридий-188', 'иридий-188', '{"AM": 188, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 77, "Name_RN": "иридий-188", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.73, "Vid_izluch": "гамма", "Name_RN_Lat": "Ir-188", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иридий-187', 'иридий-187', '{"AM": 187, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 77, "Name_RN": "иридий-187", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 10.5, "Vid_izluch": "гамма", "Name_RN_Lat": "Ir-187", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иридий-186', 'иридий-186', '{"AM": 186, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 77, "Name_RN": "иридий-186", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 15.8, "Vid_izluch": "бета", "Name_RN_Lat": "Ir-186", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'висмут-212', 'висмут-212', '{"AM": 212, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.05, "Num_TM": 83, "Name_RN": "висмут-212", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 3.6, "Kod_gruppy": "б", "Period_p_r": 1.01, "Vid_izluch": "бета", "Name_RN_Lat": "Bi-212", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'висмут-210м', 'висмут-210м', '{"AM": 210, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.3, "Num_TM": 83, "Name_RN": "висмут-210м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "а", "Period_p_r": 3000000.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Bi-210m", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'висмут-210', 'висмут-210', '{"AM": 210, "MOI": "", "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": 8, "Num_TM": 83, "Name_RN": "висмут-210", "UdA_GRO": 11.0, "UdA_TRO": 1000.0, "ObA_GaRO": 1.2, "Kod_gruppy": "б", "Period_p_r": 5.01, "Vid_izluch": "бета", "Name_RN_Lat": "Bi-210", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'висмут-207', 'висмут-207', '{"AM": 207, "MOI": 0.1, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.05, "Num_TM": 83, "Name_RN": "висмут-207", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 21.0, "Kod_gruppy": "б", "Period_p_r": 31.55, "Vid_izluch": "бета", "Name_RN_Lat": "Bi-207", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'висмут-206', 'висмут-206', '{"AM": 206, "MOI": 1.0, "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.02, "Num_TM": 83, "Name_RN": "висмут-206", "UdA_GRO": 7.2, "UdA_TRO": 10.0, "ObA_GaRO": 65.0, "Kod_gruppy": "б", "Period_p_r": 6.24, "Vid_izluch": "бета", "Name_RN_Lat": "Bi-206", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'висмут-205', 'висмут-205', '{"AM": 205, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.04, "Num_TM": 83, "Name_RN": "висмут-205", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 15.3, "Vid_izluch": "бета", "Name_RN_Lat": "Bi-205", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иридий-185', 'иридий-185', '{"AM": 185, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 77, "Name_RN": "иридий-185", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 14.0, "Vid_izluch": "бета", "Name_RN_Lat": "Ir-185", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иридий-184', 'иридий-184', '{"AM": 184, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 77, "Name_RN": "иридий-184", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.02, "Vid_izluch": "бета", "Name_RN_Lat": "Ir-184", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иридий-182', 'иридий-182', '{"AM": 182, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 77, "Name_RN": "иридий-182", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.25, "Vid_izluch": "бета", "Name_RN_Lat": "Ir-182", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иридий-195м', 'иридий-195м', '{"AM": 195, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 77, "Name_RN": "иридий-195м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.8, "Vid_izluch": "бета", "Name_RN_Lat": "Ir-195m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иридий-195', 'иридий-195', '{"AM": 195, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 77, "Name_RN": "иридий-195", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.5, "Vid_izluch": "бета", "Name_RN_Lat": "Ir-195", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'таллий-200', 'таллий-200', '{"AM": 200, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.05, "Num_TM": 81, "Name_RN": "таллий-200", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 600.0, "Kod_gruppy": "б", "Period_p_r": 1.09, "Vid_izluch": "бета", "Name_RN_Lat": "Tl-200", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иридий-194м', 'иридий-194м', '{"AM": 194, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 77, "Name_RN": "иридий-194м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 171.0, "Vid_izluch": "бета", "Name_RN_Lat": "Ir-194m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иридий-193м', 'иридий-193м', '{"AM": 193, "MOI": "", "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": "", "Num_TM": 77, "Name_RN": "иридий-193м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 10.53, "Vid_izluch": "гамма", "Name_RN_Lat": "Ir-193m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иридий-192м', 'иридий-192м', '{"AM": 192, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 77, "Name_RN": "иридий-192м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 241.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Ir-192m", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иридий-190м2', 'иридий-190м2', '{"AM": 190, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 77, "Name_RN": "иридий-190м2", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.1, "Vid_izluch": "бета", "Name_RN_Lat": "Ir-190m2", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'осмий-189м', 'осмий-189м', '{"AM": 189, "MOI": "", "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": "", "Num_TM": 76, "Name_RN": "осмий-189м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 6.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Os-189m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'осмий-182', 'осмий-182', '{"AM": 182, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 76, "Name_RN": "осмий-182", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 22.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Os-182", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ртуть-203', 'ртуть-203', '{"AM": 203, "MOI": 10.0, "MZA": 100000.0, "MZUA": 100000.0, "D_val": 0.3, "Num_TM": 80, "Name_RN": "ртуть-203", "UdA_GRO": 7.2, "UdA_TRO": 100.0, "ObA_GaRO": 46.0, "Kod_gruppy": "б", "Period_p_r": 46.6, "Vid_izluch": "бета", "Name_RN_Lat": "Hg-203", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'осмий-181', 'осмий-181', '{"AM": 181, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 76, "Name_RN": "осмий-181", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.75, "Vid_izluch": "бета", "Name_RN_Lat": "Os-181", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ртуть-197м', 'ртуть-197м', '{"AM": 197, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.7, "Num_TM": 80, "Name_RN": "ртуть-197м", "UdA_GRO": 29.0, "UdA_TRO": 100.0, "ObA_GaRO": 200.0, "Kod_gruppy": "б", "Period_p_r": 23.8, "Vid_izluch": "гамма", "Name_RN_Lat": "Hg-197m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ртуть-197', 'ртуть-197', '{"AM": 197, "MOI": 100.0, "MZA": 10000000.0, "MZUA": 100000.0, "D_val": 2, "Num_TM": 80, "Name_RN": "ртуть-197", "UdA_GRO": 60.0, "UdA_TRO": 100.0, "ObA_GaRO": 360.0, "Kod_gruppy": "б", "Period_p_r": 2.67, "Vid_izluch": "гамма", "Name_RN_Lat": "Hg-197", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ртуть-195м', 'ртуть-195м', '{"AM": 195, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.2, "Num_TM": 80, "Name_RN": "ртуть-195м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.73, "Vid_izluch": "гамма", "Name_RN_Lat": "Hg-195m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'осмий-180', 'осмий-180', '{"AM": 180, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 76, "Name_RN": "осмий-180", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.366, "Vid_izluch": "гамма", "Name_RN_Lat": "Os-180", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ртуть-194', 'ртуть-194', '{"AM": 194, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.07, "Num_TM": 80, "Name_RN": "ртуть-194", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 444.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Hg-194", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рений-188м', 'рений-188м', '{"AM": 188, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 75, "Name_RN": "рений-188м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.31, "Vid_izluch": "гамма", "Name_RN_Lat": "Re-188m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'платина-193', 'платина-193', '{"AM": 193, "MOI": "", "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": 3000, "Num_TM": 78, "Name_RN": "платина-193", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 50.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Pt-193", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '292.0', 'ГИБРАЛТАР', '{"2": "ГИБРАЛТАР", "v": 292.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'платина-191', 'платина-191', '{"AM": 191, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.3, "Num_TM": 78, "Name_RN": "платина-191", "UdA_GRO": 40.0, "UdA_TRO": 100.0, "ObA_GaRO": 670.0, "Kod_gruppy": "б", "Period_p_r": 2.8, "Vid_izluch": "гамма", "Name_RN_Lat": "Pt-191", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рений-186м', 'рений-186м', '{"AM": 186, "MOI": "", "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 75, "Name_RN": "рений-186м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 200000.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Re-186m", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'платина-188', 'платина-188', '{"AM": 188, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.04, "Num_TM": 78, "Name_RN": "платина-188", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 10.2, "Vid_izluch": "гамма", "Name_RN_Lat": "Pt-188", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рений-181', 'рений-181', '{"AM": 181, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 75, "Name_RN": "рений-181", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 20.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Re-181", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рений-178', 'рений-178', '{"AM": 178, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 75, "Name_RN": "рений-178", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.22, "Vid_izluch": "бета", "Name_RN_Lat": "Re-178", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иридий-194', 'иридий-194', '{"AM": 194, "MOI": 100.0, "MZA": 100000.0, "MZUA": 100000.0, "D_val": 0.7, "Num_TM": 77, "Name_RN": "иридий-194", "UdA_GRO": 10.0, "UdA_TRO": 100.0, "ObA_GaRO": 140.0, "Kod_gruppy": "б", "Period_p_r": 19.1, "Vid_izluch": "бета", "Name_RN_Lat": "Ir-194", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рений-177', 'рений-177', '{"AM": 177, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 75, "Name_RN": "рений-177", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.233, "Vid_izluch": "бета", "Name_RN_Lat": "Re-177", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'вольфрам-179', 'вольфрам-179', '{"AM": 179, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 74, "Name_RN": "вольфрам-179", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.625, "Vid_izluch": "гамма", "Name_RN_Lat": "W-179", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иридий-192', 'иридий-192', '{"AM": 192, "MOI": 1.0, "MZA": 10000.0, "MZUA": 10000.0, "D_val": 0.08, "Num_TM": 77, "Name_RN": "иридий-192", "UdA_GRO": 9.8, "UdA_TRO": 10.0, "ObA_GaRO": 17.0, "Kod_gruppy": "б", "Period_p_r": 74.0, "Vid_izluch": "бета", "Name_RN_Lat": "Ir-192", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'вольфрам-177', 'вольфрам-177', '{"AM": 177, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 74, "Name_RN": "вольфрам-177", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.25, "Vid_izluch": "бета", "Name_RN_Lat": "W-177", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'вольфрам-176', 'вольфрам-176', '{"AM": 176, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 74, "Name_RN": "вольфрам-176", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.3, "Vid_izluch": "гамма", "Name_RN_Lat": "W-176", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иридий-190', 'иридий-190', '{"AM": 190, "MOI": 1.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.05, "Num_TM": 77, "Name_RN": "иридий-190", "UdA_GRO": 11.0, "UdA_TRO": 10.0, "ObA_GaRO": 46.0, "Kod_gruppy": "б", "Period_p_r": 12.1, "Vid_izluch": "гамма", "Name_RN_Lat": "Ir-190", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тантал-185', 'тантал-185', '{"AM": 185, "MOI": "", "MZA": 100000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 73, "Name_RN": "тантал-185", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.816, "Vid_izluch": "бета", "Name_RN_Lat": "Ta-185", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рений-188', 'рений-188', '{"AM": 188, "MOI": 100.0, "MZA": 100000.0, "MZUA": 100000.0, "D_val": 1, "Num_TM": 75, "Name_RN": "рений-188", "UdA_GRO": 9.7, "UdA_TRO": 100.0, "ObA_GaRO": 110.0, "Kod_gruppy": "б", "Period_p_r": 17.0, "Vid_izluch": "бета", "Name_RN_Lat": "Re-188", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тантал-184', 'тантал-184', '{"AM": 184, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 73, "Name_RN": "тантал-184", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 8.7, "Vid_izluch": "бета", "Name_RN_Lat": "Ta-184", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рений-186', 'рений-186', '{"AM": 186, "MOI": 1000.0, "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": 4, "Num_TM": 75, "Name_RN": "рений-186", "UdA_GRO": 9.1, "UdA_TRO": 1000.0, "ObA_GaRO": 92.0, "Kod_gruppy": "б", "Period_p_r": 3.78, "Vid_izluch": "бета", "Name_RN_Lat": "Re-186", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '296.0', 'КИРИБАТИ', '{"2": "КИРИБАТИ", "v": 296.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рений-184м', 'рений-184м', '{"AM": 184, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.07, "Num_TM": 75, "Name_RN": "рений-184м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 169.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Re-184m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рений-184', 'рений-184', '{"AM": 184, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.08, "Num_TM": 75, "Name_RN": "рений-184", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 38.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Re-184", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тантал-183', 'тантал-183', '{"AM": 183, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 73, "Name_RN": "тантал-183", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 5.1, "Vid_izluch": "бета", "Name_RN_Lat": "Ta-183", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тантал-182м', 'тантал-182м', '{"AM": 182, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 73, "Name_RN": "тантал-182м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.264, "Vid_izluch": "гамма", "Name_RN_Lat": "Ta-182m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тантал-180м', 'тантал-180м', '{"AM": 180, "MOI": "", "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 73, "Name_RN": "тантал-180м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1200000000000000.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Ta-180m", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тантал-180', 'тантал-180', '{"AM": 180, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 73, "Name_RN": "тантал-180", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 8.154, "Vid_izluch": "гамма", "Name_RN_Lat": "Ta-180", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тантал-177', 'тантал-177', '{"AM": 177, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 73, "Name_RN": "тантал-177", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.36, "Vid_izluch": "бета", "Name_RN_Lat": "Ta-177", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'вольфрам-187', 'вольфрам-187', '{"AM": 187, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.1, "Num_TM": 74, "Name_RN": "вольфрам-187", "UdA_GRO": 21.0, "UdA_TRO": 100.0, "ObA_GaRO": 350.0, "Kod_gruppy": "б", "Period_p_r": 23.9, "Vid_izluch": "бета", "Name_RN_Lat": "W-187", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'вольфрам-185', 'вольфрам-185', '{"AM": 185, "MOI": 1000.0, "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": 100, "Num_TM": 74, "Name_RN": "вольфрам-185", "UdA_GRO": 31.0, "UdA_TRO": 10000.0, "ObA_GaRO": 530.0, "Kod_gruppy": "б", "Period_p_r": 75.1, "Vid_izluch": "бета", "Name_RN_Lat": "W-185", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'вольфрам-181', 'вольфрам-181', '{"AM": 181, "MOI": 10.0, "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": 5, "Num_TM": 74, "Name_RN": "вольфрам-181", "UdA_GRO": 180.0, "UdA_TRO": 1000.0, "ObA_GaRO": 2800.0, "Kod_gruppy": "б", "Period_p_r": 121.0, "Vid_izluch": "бета", "Name_RN_Lat": "W-181", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тантал-176', 'тантал-176', '{"AM": 176, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 73, "Name_RN": "тантал-176", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 8.08, "Vid_izluch": "бета", "Name_RN_Lat": "Ta-176", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'вольфрам-178', 'вольфрам-178', '{"AM": 178, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.9, "Num_TM": 74, "Name_RN": "вольфрам-178", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 21.7, "Vid_izluch": "гамма", "Name_RN_Lat": "W-178", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тантал-175', 'тантал-175', '{"AM": 175, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 73, "Name_RN": "тантал-175", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 10.5, "Vid_izluch": "бета", "Name_RN_Lat": "Ta-175", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тантал-174', 'тантал-174', '{"AM": 174, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 73, "Name_RN": "тантал-174", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.2, "Vid_izluch": "бета", "Name_RN_Lat": "Ta-174", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тантал-173', 'тантал-173', '{"AM": 173, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 73, "Name_RN": "тантал-173", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.65, "Vid_izluch": "бета", "Name_RN_Lat": "Ta-173", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тантал-172', 'тантал-172', '{"AM": 172, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 73, "Name_RN": "тантал-172", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.613, "Vid_izluch": "бета", "Name_RN_Lat": "Ta-172", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '300.0', 'ГРЕЦИЯ', '{"2": "ГРЕЦИЯ", "v": 300.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гафний-184', 'гафний-184', '{"AM": 184, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 72, "Name_RN": "гафний-184", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 4.12, "Vid_izluch": "бета", "Name_RN_Lat": "Hf-184", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гафний-183', 'гафний-183', '{"AM": 183, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 72, "Name_RN": "гафний-183", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.07, "Vid_izluch": "бета", "Name_RN_Lat": "Hf-183", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гафний-182м', 'гафний-182м', '{"AM": 182, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 72, "Name_RN": "гафний-182м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.02, "Vid_izluch": "бета", "Name_RN_Lat": "Hf-182m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тантал-182', 'тантал-182', '{"AM": 182, "MOI": 0.1, "MZA": 10000.0, "MZUA": 10000.0, "D_val": 0.06, "Num_TM": 73, "Name_RN": "тантал-182", "UdA_GRO": 9.1, "UdA_TRO": 10.0, "ObA_GaRO": 11.0, "Kod_gruppy": "б", "Period_p_r": 115.0, "Vid_izluch": "бета", "Name_RN_Lat": "Ta-182", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гафний-179м', 'гафний-179м', '{"AM": 179, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 72, "Name_RN": "гафний-179м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 25.1, "Vid_izluch": "гамма", "Name_RN_Lat": "Hf-179m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тантал-179', 'тантал-179', '{"AM": 179, "MOI": "", "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": 6, "Num_TM": 73, "Name_RN": "тантал-179", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.82, "Vid_izluch": "гамма", "Name_RN_Lat": "Ta-179", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тантал-178', 'тантал-178', '{"AM": 178, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.07, "Num_TM": 73, "Name_RN": "тантал-178", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.36, "Vid_izluch": "бета", "Name_RN_Lat": "Ta-178", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гафний-178м', 'гафний-178м', '{"AM": 178, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 72, "Name_RN": "гафний-178м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 31.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Hf-178m", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гафний-177м', 'гафний-177м', '{"AM": 177, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 72, "Name_RN": "гафний-177м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.856, "Vid_izluch": "гамма", "Name_RN_Lat": "Hf-177m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гафний-173', 'гафний-173', '{"AM": 173, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 72, "Name_RN": "гафний-173", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 24.0, "Vid_izluch": "бета", "Name_RN_Lat": "Hf-173", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гафний-170', 'гафний-170', '{"AM": 170, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 72, "Name_RN": "гафний-170", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 16.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Hf-170", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'лютеций-179', 'лютеций-179', '{"AM": 179, "MOI": "", "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 71, "Name_RN": "лютеций-179", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 4.59, "Vid_izluch": "бета", "Name_RN_Lat": "Lu-179", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'лютеций-178м', 'лютеций-178м', '{"AM": 178, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 71, "Name_RN": "лютеций-178м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.378, "Vid_izluch": "бета", "Name_RN_Lat": "Lu-178m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'лютеций-178', 'лютеций-178', '{"AM": 178, "MOI": "", "MZA": 100000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 71, "Name_RN": "лютеций-178", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.473, "Vid_izluch": "бета", "Name_RN_Lat": "Lu-178", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'лютеций-177м', 'лютеций-177м', '{"AM": 177, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 71, "Name_RN": "лютеций-177м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 161.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Lu-177m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'лютеций-176м', 'лютеций-176м', '{"AM": 176, "MOI": "", "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 71, "Name_RN": "лютеций-176м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.68, "Vid_izluch": "бета", "Name_RN_Lat": "Lu-176m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '308.0', 'ГРЕНАДА', '{"2": "ГРЕНАДА", "v": 308.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гафний-181', 'гафний-181', '{"AM": 181, "MOI": 1.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.1, "Num_TM": 72, "Name_RN": "гафний-181", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 22.0, "Kod_gruppy": "б", "Period_p_r": 42.4, "Vid_izluch": "бета", "Name_RN_Lat": "Hf-181", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'лютеций-176', 'лютеций-176', '{"AM": 176, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 71, "Name_RN": "лютеций-176", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 37600000000.0, "Vid_izluch": "бета", "Name_RN_Lat": "Lu-176", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'лютеций-171', 'лютеций-171', '{"AM": 171, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 71, "Name_RN": "лютеций-171", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 8.22, "Vid_izluch": "бета", "Name_RN_Lat": "Lu-171", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'лютеций-170', 'лютеций-170', '{"AM": 170, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 71, "Name_RN": "лютеций-170", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.0, "Vid_izluch": "бета", "Name_RN_Lat": "Lu-170", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'лютеций-169', 'лютеций-169', '{"AM": 169, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 71, "Name_RN": "лютеций-169", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.42, "Vid_izluch": "бета", "Name_RN_Lat": "Lu-169", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гафний-175', 'гафний-175', '{"AM": 175, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.2, "Num_TM": 72, "Name_RN": "гафний-175", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 70.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Hf-175", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иттербий-178', 'иттербий-178', '{"AM": 178, "MOI": "", "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 70, "Name_RN": "иттербий-178", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.23, "Vid_izluch": "бета", "Name_RN_Lat": "Yb-178", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гафний-172', 'гафний-172', '{"AM": 172, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.04, "Num_TM": 72, "Name_RN": "гафний-172", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.87, "Vid_izluch": "гамма", "Name_RN_Lat": "Hf-172", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иттербий-177', 'иттербий-177', '{"AM": 177, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 70, "Name_RN": "иттербий-177", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.9, "Vid_izluch": "бета", "Name_RN_Lat": "Yb-177", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иттербий-167', 'иттербий-167', '{"AM": 167, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 70, "Name_RN": "иттербий-167", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.292, "Vid_izluch": "бета", "Name_RN_Lat": "Yb-167", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иттербий-166', 'иттербий-166', '{"AM": 166, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 70, "Name_RN": "иттербий-166", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.36, "Vid_izluch": "гамма", "Name_RN_Lat": "Yb-166", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иттербий-162', 'иттербий-162', '{"AM": 162, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 70, "Name_RN": "иттербий-162", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.315, "Vid_izluch": "бета", "Name_RN_Lat": "Yb-162", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тулий-175', 'тулий-175', '{"AM": 175, "MOI": "", "MZA": 1.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 69, "Name_RN": "тулий-175", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.253, "Vid_izluch": "бета", "Name_RN_Lat": "Tm-175", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'лютеций-177', 'лютеций-177', '{"AM": 177, "MOI": 100.0, "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": 2, "Num_TM": 71, "Name_RN": "лютеций-177", "UdA_GRO": 25.0, "UdA_TRO": 1000.0, "ObA_GaRO": 91.0, "Kod_gruppy": "б", "Period_p_r": 6.71, "Vid_izluch": "бета", "Name_RN_Lat": "Lu-177", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тулий-172', 'тулий-172', '{"AM": 172, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 69, "Name_RN": "тулий-172", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.65, "Vid_izluch": "бета", "Name_RN_Lat": "Tm-172", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'лютеций-174м', 'лютеций-174м', '{"AM": 174, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": 0.6, "Num_TM": 71, "Name_RN": "лютеций-174м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 142.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Lu-174m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'лютеций-174', 'лютеций-174', '{"AM": 174, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": 0.8, "Num_TM": 71, "Name_RN": "лютеций-174", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.31, "Vid_izluch": "бета", "Name_RN_Lat": "Lu-174", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'лютеций-173', 'лютеций-173', '{"AM": 173, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": 0.9, "Num_TM": 71, "Name_RN": "лютеций-173", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.36, "Vid_izluch": "гамма", "Name_RN_Lat": "Lu-173", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'лютеций-172', 'лютеций-172', '{"AM": 172, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.04, "Num_TM": 71, "Name_RN": "лютеций-172", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 6.7, "Vid_izluch": "гамма", "Name_RN_Lat": "Lu-172", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тулий-166', 'тулий-166', '{"AM": 166, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 69, "Name_RN": "тулий-166", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 7.7, "Vid_izluch": "бета", "Name_RN_Lat": "Tm-166", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тулий-162', 'тулий-162', '{"AM": 162, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 69, "Name_RN": "тулий-162", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.362, "Vid_izluch": "бета", "Name_RN_Lat": "Tm-162", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'эрбий-172', 'эрбий-172', '{"AM": 172, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 68, "Name_RN": "эрбий-172", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.05, "Vid_izluch": "бета", "Name_RN_Lat": "Er-172", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'эрбий-165', 'эрбий-165', '{"AM": 165, "MOI": "", "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 68, "Name_RN": "эрбий-165", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 10.4, "Vid_izluch": "гамма", "Name_RN_Lat": "Er-165", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'эрбий-161', 'эрбий-161', '{"AM": 161, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 68, "Name_RN": "эрбий-161", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.24, "Vid_izluch": "бета", "Name_RN_Lat": "Er-161", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иттербий-175', 'иттербий-175', '{"AM": 175, "MOI": 100.0, "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": 2, "Num_TM": 70, "Name_RN": "иттербий-175", "UdA_GRO": 31.0, "UdA_TRO": 1000.0, "ObA_GaRO": 150.0, "Kod_gruppy": "б", "Period_p_r": 4.19, "Vid_izluch": "бета", "Name_RN_Lat": "Yb-175", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иттербий-169', 'иттербий-169', '{"AM": 169, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": 0.3, "Num_TM": 70, "Name_RN": "иттербий-169", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 32.018, "Vid_izluch": "гамма", "Name_RN_Lat": "Yb-169", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гольмий-167', 'гольмий-167', '{"AM": 167, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 67, "Name_RN": "гольмий-167", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.1, "Vid_izluch": "бета", "Name_RN_Lat": "Ho-167", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гольмий-164м', 'гольмий-164м', '{"AM": 164, "MOI": "", "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 67, "Name_RN": "гольмий-164м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.625, "Vid_izluch": "гамма", "Name_RN_Lat": "Ho-164m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гольмий-164', 'гольмий-164', '{"AM": 164, "MOI": "", "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 67, "Name_RN": "гольмий-164", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.483, "Vid_izluch": "гамма", "Name_RN_Lat": "Ho-164", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гольмий-162м', 'гольмий-162м', '{"AM": 162, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 67, "Name_RN": "гольмий-162м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.13, "Vid_izluch": "гамма", "Name_RN_Lat": "Ho-162m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гольмий-161', 'гольмий-161', '{"AM": 161, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 67, "Name_RN": "гольмий-161", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.5, "Vid_izluch": "гамма", "Name_RN_Lat": "Ho-161", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тулий-171', 'тулий-171', '{"AM": 171, "MOI": 1000.0, "MZA": 100000000.0, "MZUA": 10000000.0, "D_val": 300, "Num_TM": 69, "Name_RN": "тулий-171", "UdA_GRO": 120.0, "UdA_TRO": 10000.0, "ObA_GaRO": 86.0, "Kod_gruppy": "б", "Period_p_r": 1.92, "Vid_izluch": "бета", "Name_RN_Lat": "Tm-171", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '316.0', 'ГУАМ', '{"2": "ГУАМ", "v": 316.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тулий-170', 'тулий-170', '{"AM": 170, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": 20, "Num_TM": 69, "Name_RN": "тулий-170", "UdA_GRO": 10.0, "UdA_TRO": 1000.0, "ObA_GaRO": 16.0, "Kod_gruppy": "б", "Period_p_r": 129.0, "Vid_izluch": "бета", "Name_RN_Lat": "Tm-170", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тулий-167', 'тулий-167', '{"AM": 167, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.6, "Num_TM": 69, "Name_RN": "тулий-167", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 9.24, "Vid_izluch": "гамма", "Name_RN_Lat": "Tm-167", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гольмий-159', 'гольмий-159', '{"AM": 159, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 67, "Name_RN": "гольмий-159", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.55, "Vid_izluch": "бета", "Name_RN_Lat": "Ho-159", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гольмий-157', 'гольмий-157', '{"AM": 157, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 67, "Name_RN": "гольмий-157", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.21, "Vid_izluch": "бета", "Name_RN_Lat": "Ho-157", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гольмий-155', 'гольмий-155', '{"AM": 155, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 67, "Name_RN": "гольмий-155", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.8, "Vid_izluch": "бета", "Name_RN_Lat": "Ho-155", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'эрбий-171', 'эрбий-171', '{"AM": 171, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.2, "Num_TM": 68, "Name_RN": "эрбий-171", "UdA_GRO": 38.0, "UdA_TRO": 100.0, "ObA_GaRO": 430.0, "Kod_gruppy": "б", "Period_p_r": 7.52, "Vid_izluch": "бета", "Name_RN_Lat": "Er-171", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'эрбий-169', 'эрбий-169', '{"AM": 169, "MOI": 1000.0, "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": 200, "Num_TM": 68, "Name_RN": "эрбий-169", "UdA_GRO": 37.0, "UdA_TRO": 10000.0, "ObA_GaRO": 110.0, "Kod_gruppy": "б", "Period_p_r": 9.3, "Vid_izluch": "бета", "Name_RN_Lat": "Er-169", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'диспрозий-157', 'диспрозий-157', '{"AM": 157, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 66, "Name_RN": "диспрозий-157", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 8.1, "Vid_izluch": "гамма", "Name_RN_Lat": "Dy-157", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'диспрозий-155', 'диспрозий-155', '{"AM": 155, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 66, "Name_RN": "диспрозий-155", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 10.0, "Vid_izluch": "бета", "Name_RN_Lat": "Dy-155", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тербий-161', 'тербий-161', '{"AM": 161, "MOI": "", "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 65, "Name_RN": "тербий-161", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 6.91, "Vid_izluch": "бета", "Name_RN_Lat": "Tb-161", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гольмий-166м', 'гольмий-166м', '{"AM": 166, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.04, "Num_TM": 67, "Name_RN": "гольмий-166м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1200.0, "Vid_izluch": "бета", "Name_RN_Lat": "Ho-166m", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гольмий-166', 'гольмий-166', '{"AM": 166, "MOI": 100.0, "MZA": 100000.0, "MZUA": 1000000.0, "D_val": 2, "Num_TM": 67, "Name_RN": "гольмий-166", "UdA_GRO": 9.7, "UdA_TRO": 1000.0, "ObA_GaRO": 130.0, "Kod_gruppy": "б", "Period_p_r": 1.12, "Vid_izluch": "бета", "Name_RN_Lat": "Ho-166", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тербий-156м', 'тербий-156м', '{"AM": 156, "MOI": "", "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": "", "Num_TM": 65, "Name_RN": "тербий-156м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 24.4, "Vid_izluch": "бета", "Name_RN_Lat": "Tb-156m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тербий-156', 'тербий-156', '{"AM": 156, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 65, "Name_RN": "тербий-156", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 5.34, "Vid_izluch": "гамма", "Name_RN_Lat": "Tb-156", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тербий-155', 'тербий-155', '{"AM": 155, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 65, "Name_RN": "тербий-155", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 5.32, "Vid_izluch": "гамма", "Name_RN_Lat": "Tb-155", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тербий-154', 'тербий-154', '{"AM": 154, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 65, "Name_RN": "тербий-154", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 21.4, "Vid_izluch": "бета", "Name_RN_Lat": "Tb-154", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тербий-153', 'тербий-153', '{"AM": 153, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 65, "Name_RN": "тербий-153", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.34, "Vid_izluch": "бета", "Name_RN_Lat": "Tb-153", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тербий-151', 'тербий-151', '{"AM": 151, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 65, "Name_RN": "тербий-151", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 17.6, "Vid_izluch": "бета", "Name_RN_Lat": "Tb-151", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тербий-150', 'тербий-150', '{"AM": 150, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 65, "Name_RN": "тербий-150", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.27, "Vid_izluch": "бета", "Name_RN_Lat": "Tb-150", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'диспрозий-166', 'диспрозий-166', '{"AM": 166, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": 1, "Num_TM": 66, "Name_RN": "диспрозий-166", "UdA_GRO": 8.5, "UdA_TRO": 1000.0, "ObA_GaRO": 60.0, "Kod_gruppy": "б", "Period_p_r": 3.4, "Vid_izluch": "бета", "Name_RN_Lat": "Dy-166", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'диспрозий-165', 'диспрозий-165', '{"AM": 165, "MOI": 1000.0, "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": 3, "Num_TM": 66, "Name_RN": "диспрозий-165", "UdA_GRO": 120.0, "UdA_TRO": 1000.0, "ObA_GaRO": 1500.0, "Kod_gruppy": "б", "Period_p_r": 2.33, "Vid_izluch": "бета", "Name_RN_Lat": "Dy-165", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'диспрозий-159', 'диспрозий-159', '{"AM": 159, "MOI": "", "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": 6, "Num_TM": 66, "Name_RN": "диспрозий-159", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 144.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Dy-159", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тербий-149', 'тербий-149', '{"AM": 149, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 65, "Name_RN": "тербий-149", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 4.15, "Vid_izluch": "бета", "Name_RN_Lat": "Tb-149", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тербий-147', 'тербий-147', '{"AM": 147, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 65, "Name_RN": "тербий-147", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.65, "Vid_izluch": "бета", "Name_RN_Lat": "Tb-147", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гадолиний-152', 'гадолиний-152', '{"AM": 152, "MOI": "", "MZA": 10000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 64, "Name_RN": "гадолиний-152", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "а", "Period_p_r": 108000000000000.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Gd-152", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тербий-158', 'тербий-158', '{"AM": 158, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.09, "Num_TM": 65, "Name_RN": "тербий-158", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 180.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Tb-158", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тербий-157', 'тербий-157', '{"AM": 157, "MOI": "", "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": 100, "Num_TM": 65, "Name_RN": "тербий-157", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 71.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Tb-157", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гадолиний-151', 'гадолиний-151', '{"AM": 151, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 64, "Name_RN": "гадолиний-151", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 120.0, "Vid_izluch": "бета", "Name_RN_Lat": "Gd-151", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гадолиний-149', 'гадолиний-149', '{"AM": 149, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 64, "Name_RN": "гадолиний-149", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 9.4, "Vid_izluch": "гамма", "Name_RN_Lat": "Gd-149", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гадолиний-147', 'гадолиний-147', '{"AM": 147, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 64, "Name_RN": "гадолиний-147", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.59, "Vid_izluch": "бета", "Name_RN_Lat": "Gd-147", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гадолиний-145', 'гадолиний-145', '{"AM": 145, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 64, "Name_RN": "гадолиний-145", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.382, "Vid_izluch": "бета", "Name_RN_Lat": "Gd-145", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'европий-158', 'европий-158', '{"AM": 158, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 63, "Name_RN": "европий-158", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.765, "Vid_izluch": "бета", "Name_RN_Lat": "Eu-158", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'европий-157', 'европий-157', '{"AM": 157, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 63, "Name_RN": "европий-157", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 15.1, "Vid_izluch": "бета", "Name_RN_Lat": "Eu-157", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '320.0', 'ГВАТЕМАЛА', '{"2": "ГВАТЕМАЛА", "v": 320.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'европий-150м', 'европий-150м', '{"AM": 150, "MOI": "", "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 63, "Name_RN": "европий-150м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 12.6, "Vid_izluch": "бета", "Name_RN_Lat": "Eu-150m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'европий-150', 'европий-150', '{"AM": 150, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 63, "Name_RN": "европий-150", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 36.9, "Vid_izluch": "гамма", "Name_RN_Lat": "Eu-150", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'европий-146', 'европий-146', '{"AM": 146, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 63, "Name_RN": "европий-146", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 4.61, "Vid_izluch": "бета", "Name_RN_Lat": "Eu-146", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'европий-145', 'европий-145', '{"AM": 145, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 63, "Name_RN": "европий-145", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 5.94, "Vid_izluch": "бета", "Name_RN_Lat": "Eu-145", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гадолиний-159', 'гадолиний-159', '{"AM": 159, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": 2, "Num_TM": 64, "Name_RN": "гадолиний-159", "UdA_GRO": 27.0, "UdA_TRO": 1000.0, "ObA_GaRO": 350.0, "Kod_gruppy": "б", "Period_p_r": 18.6, "Vid_izluch": "бета", "Name_RN_Lat": "Gd-159", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гадолиний-153', 'гадолиний-153', '{"AM": 153, "MOI": 10.0, "MZA": 10000000.0, "MZUA": 100000.0, "D_val": 1, "Num_TM": 64, "Name_RN": "гадолиний-153", "UdA_GRO": 51.0, "UdA_TRO": 100.0, "ObA_GaRO": 44.0, "Kod_gruppy": "б", "Period_p_r": 242.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Gd-153", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'самарий-156', 'самарий-156', '{"AM": 156, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 62, "Name_RN": "самарий-156", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 9.4, "Vid_izluch": "бета", "Name_RN_Lat": "Sm-156", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'самарий-146', 'самарий-146', '{"AM": 146, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 62, "Name_RN": "самарий-146", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "а", "Period_p_r": 68000000.0, "Vid_izluch": "альфа", "Name_RN_Lat": "Sm-146", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гадолиний-148', 'гадолиний-148', '{"AM": 148, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.4, "Num_TM": 64, "Name_RN": "гадолиний-148", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "а", "Period_p_r": 71.1, "Vid_izluch": "альфа", "Name_RN_Lat": "Gd-148", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'самарий-142', 'самарий-142', '{"AM": 142, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 62, "Name_RN": "самарий-142", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.21, "Vid_izluch": "бета", "Name_RN_Lat": "Sm-142", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'гадолиний-146', 'гадолиний-146', '{"AM": 146, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.03, "Num_TM": 64, "Name_RN": "гадолиний-146", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 48.3, "Vid_izluch": "гамма", "Name_RN_Lat": "Gd-146", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'самарий-141м', 'самарий-141м', '{"AM": 141, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 62, "Name_RN": "самарий-141м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.377, "Vid_izluch": "бета", "Name_RN_Lat": "Sm-141m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'самарий-141', 'самарий-141', '{"AM": 141, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 62, "Name_RN": "самарий-141", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.17, "Vid_izluch": "бета", "Name_RN_Lat": "Sm-141", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'прометий-150', 'прометий-150', '{"AM": 150, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 61, "Name_RN": "прометий-150", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.68, "Vid_izluch": "бета", "Name_RN_Lat": "Pm-150", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'европий-156', 'европий-156', '{"AM": 156, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.05, "Num_TM": 63, "Name_RN": "европий-156", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 15.2, "Vid_izluch": "бета", "Name_RN_Lat": "Eu-156", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'европий-155', 'европий-155', '{"AM": 155, "MOI": 1.0, "MZA": 10000000.0, "MZUA": 100000.0, "D_val": 2, "Num_TM": 63, "Name_RN": "европий-155", "UdA_GRO": 43.0, "UdA_TRO": 100.0, "ObA_GaRO": 18.0, "Kod_gruppy": "б", "Period_p_r": 4.96, "Vid_izluch": "бета", "Name_RN_Lat": "Eu-155", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'европий-154', 'европий-154', '{"AM": 154, "MOI": 0.1, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.06, "Num_TM": 63, "Name_RN": "европий-154", "UdA_GRO": 6.9, "UdA_TRO": 10.0, "ObA_GaRO": 2.3, "Kod_gruppy": "б", "Period_p_r": 8.8, "Vid_izluch": "бета", "Name_RN_Lat": "Eu-154", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'европий-152м', 'европий-152м', '{"AM": 152, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.2, "Num_TM": 63, "Name_RN": "европий-152м", "UdA_GRO": 27.0, "UdA_TRO": 100.0, "ObA_GaRO": 400.0, "Kod_gruppy": "б", "Period_p_r": 9.32, "Vid_izluch": "бета", "Name_RN_Lat": "Eu-152m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'европий-152', 'европий-152', '{"AM": 152, "MOI": 0.1, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.06, "Num_TM": 63, "Name_RN": "европий-152", "UdA_GRO": 9.8, "UdA_TRO": 10.0, "ObA_GaRO": 2.9, "Kod_gruppy": "б", "Period_p_r": 13.3, "Vid_izluch": "бета", "Name_RN_Lat": "Eu-152", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'прометий-148', 'прометий-148', '{"AM": 148, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 61, "Name_RN": "прометий-148", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 5.37, "Vid_izluch": "бета", "Name_RN_Lat": "Pm-148", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'прометий-146', 'прометий-146', '{"AM": 146, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 61, "Name_RN": "прометий-146", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 5.53, "Vid_izluch": "бета", "Name_RN_Lat": "Pm-146", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'европий-149', 'европий-149', '{"AM": 149, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": 2, "Num_TM": 63, "Name_RN": "европий-149", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 93.1, "Vid_izluch": "гамма", "Name_RN_Lat": "Eu-149", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'европий-147', 'европий-147', '{"AM": 147, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.2, "Num_TM": 63, "Name_RN": "европий-147", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 24.0, "Vid_izluch": "бета", "Name_RN_Lat": "Eu-147", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'прометий-141', 'прометий-141', '{"AM": 141, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 61, "Name_RN": "прометий-141", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.348, "Vid_izluch": "бета", "Name_RN_Lat": "Pm-141", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'неодим-151', 'неодим-151', '{"AM": 151, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 60, "Name_RN": "неодим-151", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.207, "Vid_izluch": "бета", "Name_RN_Lat": "Nd-151", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'неодим-141', 'неодим-141', '{"AM": 141, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 60, "Name_RN": "неодим-141", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.49, "Vid_izluch": "бета", "Name_RN_Lat": "Nd-141", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'неодим-139м', 'неодим-139м', '{"AM": 139, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 60, "Name_RN": "неодим-139м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 5.5, "Vid_izluch": "гамма", "Name_RN_Lat": "Nd-139m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'самарий-153', 'самарий-153', '{"AM": 153, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 2, "Num_TM": 62, "Name_RN": "самарий-153", "UdA_GRO": 19.0, "UdA_TRO": 100.0, "ObA_GaRO": 170.0, "Kod_gruppy": "б", "Period_p_r": 1.95, "Vid_izluch": "бета", "Name_RN_Lat": "Sm-153", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'самарий-151', 'самарий-151', '{"AM": 151, "MOI": 1000.0, "MZA": 100000000.0, "MZUA": 10000000.0, "D_val": 500, "Num_TM": 62, "Name_RN": "самарий-151", "UdA_GRO": 140.0, "UdA_TRO": 10000.0, "ObA_GaRO": 31.0, "Kod_gruppy": "б", "Period_p_r": 90.0, "Vid_izluch": "бета", "Name_RN_Lat": "Sm-151", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'неодим-139', 'неодим-139', '{"AM": 139, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 60, "Name_RN": "неодим-139", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.495, "Vid_izluch": "бета", "Name_RN_Lat": "Nd-139", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'самарий-145', 'самарий-145', '{"AM": 145, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": 4, "Num_TM": 62, "Name_RN": "самарий-145", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 340.0, "Vid_izluch": "бета", "Name_RN_Lat": "Sm-145", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '324.0', 'ГВИНЕЯ', '{"2": "ГВИНЕЯ", "v": 324.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'неодим-138', 'неодим-138', '{"AM": 138, "MOI": "", "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 60, "Name_RN": "неодим-138", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 5.04, "Vid_izluch": "гамма", "Name_RN_Lat": "Nd-138", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'неодим-136', 'неодим-136', '{"AM": 136, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 60, "Name_RN": "неодим-136", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.844, "Vid_izluch": "бета", "Name_RN_Lat": "Nd-136", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'празеодим-147', 'празеодим-147', '{"AM": 147, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 59, "Name_RN": "празеодим-147", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.227, "Vid_izluch": "бета", "Name_RN_Lat": "Pr-147", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'прометий-151', 'прометий-151', '{"AM": 151, "MOI": 1000.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.2, "Num_TM": 61, "Name_RN": "прометий-151", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.18, "Vid_izluch": "бета", "Name_RN_Lat": "Pm-151", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'празеодим-145', 'празеодим-145', '{"AM": 145, "MOI": "", "MZA": 100000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 59, "Name_RN": "празеодим-145", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 5.98, "Vid_izluch": "бета", "Name_RN_Lat": "Pr-145", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'прометий-149', 'прометий-149', '{"AM": 149, "MOI": 1000.0, "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": 6, "Num_TM": 61, "Name_RN": "прометий-149", "UdA_GRO": 14.0, "UdA_TRO": 1000.0, "ObA_GaRO": 150.0, "Kod_gruppy": "б", "Period_p_r": 2.21, "Vid_izluch": "бета", "Name_RN_Lat": "Pm-149", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'празеодим-144', 'празеодим-144', '{"AM": 144, "MOI": "", "MZA": 100000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 59, "Name_RN": "празеодим-144", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.288, "Vid_izluch": "бета", "Name_RN_Lat": "Pr-144", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'прометий-147', 'прометий-147', '{"AM": 147, "MOI": 1000.0, "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": 40, "Num_TM": 61, "Name_RN": "прометий-147", "UdA_GRO": 53.0, "UdA_TRO": 10000.0, "ObA_GaRO": 24.0, "Kod_gruppy": "б", "Period_p_r": 2.62, "Vid_izluch": "бета", "Name_RN_Lat": "Pm-147", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'празеодим-142м', 'празеодим-142м', '{"AM": 142, "MOI": "", "MZA": 1000000000.0, "MZUA": 10000000000.0, "D_val": "", "Num_TM": 59, "Name_RN": "празеодим-142м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.243, "Vid_izluch": "гамма", "Name_RN_Lat": "Pr-142m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'прометий-145', 'прометий-145', '{"AM": 145, "MOI": "", "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": 10, "Num_TM": 61, "Name_RN": "прометий-145", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 17.7, "Vid_izluch": "гамма", "Name_RN_Lat": "Pm-145", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'прометий-144', 'прометий-144', '{"AM": 144, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.04, "Num_TM": 61, "Name_RN": "прометий-144", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 363.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Pm-144", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'прометий-143', 'прометий-143', '{"AM": 143, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.2, "Num_TM": 61, "Name_RN": "прометий-143", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 265.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Pm-143", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'празеодим-139', 'празеодим-139', '{"AM": 139, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 59, "Name_RN": "празеодим-139", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 4.51, "Vid_izluch": "бета", "Name_RN_Lat": "Pr-139", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'празеодим-138м', 'празеодим-138м', '{"AM": 138, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 59, "Name_RN": "празеодим-138м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.1, "Vid_izluch": "бета", "Name_RN_Lat": "Pr-138m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'неодим-149', 'неодим-149', '{"AM": 149, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.2, "Num_TM": 60, "Name_RN": "неодим-149", "UdA_GRO": "", "UdA_TRO": 100.0, "ObA_GaRO": 1000.0, "Kod_gruppy": "б", "Period_p_r": 1.73, "Vid_izluch": "бета", "Name_RN_Lat": "Nd-149", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'неодим-147', 'неодим-147', '{"AM": 147, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.6, "Num_TM": 60, "Name_RN": "неодим-147", "UdA_GRO": 12.0, "UdA_TRO": 100.0, "ObA_GaRO": 46.0, "Kod_gruppy": "б", "Period_p_r": 11.0, "Vid_izluch": "бета", "Name_RN_Lat": "Nd-147", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'празеодим-137', 'празеодим-137', '{"AM": 137, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 59, "Name_RN": "празеодим-137", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.28, "Vid_izluch": "бета", "Name_RN_Lat": "Pr-137", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'празеодим-136', 'празеодим-136', '{"AM": 136, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 59, "Name_RN": "празеодим-136", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.218, "Vid_izluch": "бета", "Name_RN_Lat": "Pr-136", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'церий-137м', 'церий-137м', '{"AM": 137, "MOI": "", "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 58, "Name_RN": "церий-137м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.43, "Vid_izluch": "гамма", "Name_RN_Lat": "Ce-137m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'церий-137', 'церий-137', '{"AM": 137, "MOI": "", "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 58, "Name_RN": "церий-137", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 9.0, "Vid_izluch": "бета", "Name_RN_Lat": "Ce-137", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'церий-135', 'церий-135', '{"AM": 135, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 58, "Name_RN": "церий-135", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 17.6, "Vid_izluch": "гамма", "Name_RN_Lat": "Ce-135", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'лантан-143', 'лантан-143', '{"AM": 143, "MOI": "", "MZA": 100000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 57, "Name_RN": "лантан-143", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.237, "Vid_izluch": "бета", "Name_RN_Lat": "La-143", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'лантан-142', 'лантан-142', '{"AM": 142, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 57, "Name_RN": "лантан-142", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.54, "Vid_izluch": "бета", "Name_RN_Lat": "La-142", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'празеодим-143', 'празеодим-143', '{"AM": 143, "MOI": 1000.0, "MZA": 1000000.0, "MZUA": 10000000.0, "D_val": 30, "Num_TM": 59, "Name_RN": "празеодим-143", "UdA_GRO": 11.0, "UdA_TRO": 10000.0, "ObA_GaRO": 46.0, "Kod_gruppy": "б", "Period_p_r": 13.6, "Vid_izluch": "бета", "Name_RN_Lat": "Pr-143", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'лантан-141', 'лантан-141', '{"AM": 141, "MOI": "", "MZA": 100000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 57, "Name_RN": "лантан-141", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.93, "Vid_izluch": "бета", "Name_RN_Lat": "La-141", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'празеодим-142', 'празеодим-142', '{"AM": 142, "MOI": 100.0, "MZA": 100000.0, "MZUA": 100000.0, "D_val": 1, "Num_TM": 59, "Name_RN": "празеодим-142", "UdA_GRO": 10.0, "UdA_TRO": 100.0, "ObA_GaRO": 140.0, "Kod_gruppy": "б", "Period_p_r": 19.1, "Vid_izluch": "бета", "Name_RN_Lat": "Pr-142", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'лантан-138', 'лантан-138', '{"AM": 138, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 57, "Name_RN": "лантан-138", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 103000000000.0, "Vid_izluch": "гамма", "Name_RN_Lat": "La-138", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'лантан-135', 'лантан-135', '{"AM": 135, "MOI": "", "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 57, "Name_RN": "лантан-135", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 19.5, "Vid_izluch": "бета", "Name_RN_Lat": "La-135", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'лантан-132', 'лантан-132', '{"AM": 132, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 57, "Name_RN": "лантан-132", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 4.8, "Vid_izluch": "бета", "Name_RN_Lat": "La-132", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'лантан-131', 'лантан-131', '{"AM": 131, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 57, "Name_RN": "лантан-131", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.983, "Vid_izluch": "бета", "Name_RN_Lat": "La-131", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'церий-144', 'церий-144', '{"AM": 144, "MOI": 10.0, "MZA": 100000.0, "MZUA": 100000.0, "D_val": 0.9, "Num_TM": 58, "Name_RN": "церий-144", "UdA_GRO": 2.6, "UdA_TRO": 100.0, "ObA_GaRO": 3.3, "Kod_gruppy": "б", "Period_p_r": 284.0, "Vid_izluch": "бета", "Name_RN_Lat": "Ce-144", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'церий-143', 'церий-143', '{"AM": 143, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.3, "Num_TM": 58, "Name_RN": "церий-143", "UdA_GRO": 1.2, "UdA_TRO": 100.0, "ObA_GaRO": 130.0, "Kod_gruppy": "б", "Period_p_r": 1.38, "Vid_izluch": "бета", "Name_RN_Lat": "Ce-143", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'церий-141', 'церий-141', '{"AM": 141, "MOI": 100.0, "MZA": 10000000.0, "MZUA": 100000.0, "D_val": 1, "Num_TM": 58, "Name_RN": "церий-141", "UdA_GRO": 1.9, "UdA_TRO": 100.0, "ObA_GaRO": 33.0, "Kod_gruppy": "б", "Period_p_r": 32.5, "Vid_izluch": "бета", "Name_RN_Lat": "Ce-141", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'барий-142', 'барий-142', '{"AM": 142, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 56, "Name_RN": "барий-142", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.177, "Vid_izluch": "бета", "Name_RN_Lat": "Ba-142", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'барий-141', 'барий-141', '{"AM": 141, "MOI": "", "MZA": 100000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 56, "Name_RN": "барий-141", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.305, "Vid_izluch": "бета", "Name_RN_Lat": "Ba-141", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'барий-139', 'барий-139', '{"AM": 139, "MOI": "", "MZA": 100000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 56, "Name_RN": "барий-139", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.38, "Vid_izluch": "бета", "Name_RN_Lat": "Ba-139", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'барий-135м', 'барий-135м', '{"AM": 135, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 56, "Name_RN": "барий-135м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.2, "Vid_izluch": "гамма", "Name_RN_Lat": "Ba-135m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'барий-131м', 'барий-131м', '{"AM": 131, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 56, "Name_RN": "барий-131м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.243, "Vid_izluch": "гамма", "Name_RN_Lat": "Ba-131m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'барий-128', 'барий-128', '{"AM": 128, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 56, "Name_RN": "барий-128", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.43, "Vid_izluch": "бета", "Name_RN_Lat": "Ba-128", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'барий-126', 'барий-126', '{"AM": 126, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 56, "Name_RN": "барий-126", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.61, "Vid_izluch": "бета", "Name_RN_Lat": "Ba-126", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'лантан-140', 'лантан-140', '{"AM": 140, "MOI": 1.0, "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.03, "Num_TM": 57, "Name_RN": "лантан-140", "UdA_GRO": 0.6, "UdA_TRO": 10.0, "ObA_GaRO": 84.0, "Kod_gruppy": "б", "Period_p_r": 1.68, "Vid_izluch": "бета", "Name_RN_Lat": "La-140", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цезий-138', 'цезий-138', '{"AM": 138, "MOI": 10.0, "MZA": 10000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 55, "Name_RN": "цезий-138", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.536, "Vid_izluch": "бета", "Name_RN_Lat": "Cs-138", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'лантан-137', 'лантан-137', '{"AM": 137, "MOI": "", "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": 20, "Num_TM": 57, "Name_RN": "лантан-137", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 60000.0, "Vid_izluch": "гамма", "Name_RN_Lat": "La-137", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цезий-135м', 'цезий-135м', '{"AM": 135, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 55, "Name_RN": "цезий-135м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.883, "Vid_izluch": "гамма", "Name_RN_Lat": "Cs-135m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цезий-135', 'цезий-135', '{"AM": 135, "MOI": 100.0, "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": "неогран.", "Num_TM": 55, "Name_RN": "цезий-135", "UdA_GRO": 6.9, "UdA_TRO": 10000.0, "ObA_GaRO": 180.0, "Kod_gruppy": "б", "Period_p_r": 2300000.0, "Vid_izluch": "бета", "Name_RN_Lat": "Cs-135", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цезий-130', 'цезий-130', '{"AM": 130, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 55, "Name_RN": "цезий-130", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.498, "Vid_izluch": "бета", "Name_RN_Lat": "Cs-130", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цезий-127', 'цезий-127', '{"AM": 127, "MOI": "", "MZA": 100000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 55, "Name_RN": "цезий-127", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 6.25, "Vid_izluch": "бета", "Name_RN_Lat": "Cs-127", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цезий-125', 'цезий-125', '{"AM": 125, "MOI": "", "MZA": 10000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 55, "Name_RN": "цезий-125", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.75, "Vid_izluch": "бета", "Name_RN_Lat": "Cs-125", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'барий-140', 'барий-140', '{"AM": 140, "MOI": 1.0, "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.03, "Num_TM": 56, "Name_RN": "барий-140", "UdA_GRO": 0.5, "UdA_TRO": 10.0, "ObA_GaRO": 22.0, "Kod_gruppy": "б", "Period_p_r": 12.7, "Vid_izluch": "бета", "Name_RN_Lat": "Ba-140", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ксенон-138', 'ксенон-138', '{"AM": 138, "MOI": "", "MZA": 1000000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 54, "Name_RN": "ксенон-138", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 14.17, "Vid_izluch": "бета", "Name_RN_Lat": "Xe-138", "Edinica_izmer_p_r": "мин", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'барий-133м', 'барий-133м', '{"AM": 133, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.3, "Num_TM": 56, "Name_RN": "барий-133м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.62, "Vid_izluch": "гамма", "Name_RN_Lat": "Ba-133m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'барий-133', 'барий-133', '{"AM": 133, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.2, "Num_TM": 56, "Name_RN": "барий-133", "UdA_GRO": 9.3, "UdA_TRO": 10.0, "ObA_GaRO": 25.0, "Kod_gruppy": "б", "Period_p_r": 10.7, "Vid_izluch": "гамма", "Name_RN_Lat": "Ba-133", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ксенон-137', 'ксенон-137', '{"AM": 137, "MOI": "", "MZA": "", "MZUA": "", "D_val": "", "Num_TM": 54, "Name_RN": "ксенон-137", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.9, "Vid_izluch": "бета", "Name_RN_Lat": "Xe-137", "Edinica_izmer_p_r": "мин", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'барий-131', 'барий-131', '{"AM": 131, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.2, "Num_TM": 56, "Name_RN": "барий-131", "UdA_GRO": 3.0, "UdA_TRO": 100.0, "ObA_GaRO": 140.0, "Kod_gruppy": "б", "Period_p_r": 11.8, "Vid_izluch": "бета", "Name_RN_Lat": "Ba-131", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ксенон-135м', 'ксенон-135м', '{"AM": 135, "MOI": "", "MZA": 1000000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 54, "Name_RN": "ксенон-135м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 15.29, "Vid_izluch": "гамма", "Name_RN_Lat": "Xe-135m", "Edinica_izmer_p_r": "мин", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ксенон-133м', 'ксенон-133м', '{"AM": 133, "MOI": "", "MZA": 10000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 54, "Name_RN": "ксенон-133м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.19, "Vid_izluch": "гамма", "Name_RN_Lat": "Xe-133m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иод-132м', 'иод-132м', '{"AM": 132, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 53, "Name_RN": "иод-132м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.39, "Vid_izluch": "гамма", "Name_RN_Lat": "I-132m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цезий-137', 'цезий-137', '{"AM": 137, "MOI": 0.1, "MZA": 10000.0, "MZUA": 10000.0, "D_val": 0.1, "Num_TM": 55, "Name_RN": "цезий-137", "UdA_GRO": 1.1, "UdA_TRO": 10.0, "ObA_GaRO": 27.0, "Kod_gruppy": "б", "Period_p_r": 30.2, "Vid_izluch": "бета", "Name_RN_Lat": "Cs-137", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цезий-136', 'цезий-136', '{"AM": 136, "MOI": 1.0, "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.03, "Num_TM": 55, "Name_RN": "цезий-136", "UdA_GRO": 4.6, "UdA_TRO": 10.0, "ObA_GaRO": 96.0, "Kod_gruppy": "б", "Period_p_r": 13.1, "Vid_izluch": "бета", "Name_RN_Lat": "Cs-136", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иод-130', 'иод-130', '{"AM": 130, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 53, "Name_RN": "иод-130", "UdA_GRO": 6.9, "UdA_TRO": 10.0, "ObA_GaRO": 71.0, "Kod_gruppy": "б", "Period_p_r": 12.4, "Vid_izluch": "бета", "Name_RN_Lat": "I-130", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иод-129', 'иод-129', '{"AM": 129, "MOI": 0.01, "MZA": 100000.0, "MZUA": 100000.0, "D_val": "неогран.", "Num_TM": 53, "Name_RN": "иод-129", "UdA_GRO": 0.13, "UdA_TRO": 100.0, "ObA_GaRO": 2.9, "Kod_gruppy": "б", "Period_p_r": 15700000.0, "Vid_izluch": "бета", "Name_RN_Lat": "I-129", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цезий-134', 'цезий-134', '{"AM": 134, "MOI": 0.1, "MZA": 10000.0, "MZUA": 10000.0, "D_val": 0.04, "Num_TM": 55, "Name_RN": "цезий-134", "UdA_GRO": 0.72, "UdA_TRO": 10.0, "ObA_GaRO": 19.0, "Kod_gruppy": "б", "Period_p_r": 2.06, "Vid_izluch": "бета", "Name_RN_Lat": "Cs-134", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цезий-132', 'цезий-132', '{"AM": 132, "MOI": 10.0, "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.1, "Num_TM": 55, "Name_RN": "цезий-132", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 440.0, "Kod_gruppy": "б", "Period_p_r": 6.48, "Vid_izluch": "бета", "Name_RN_Lat": "Cs-132", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цезий-131', 'цезий-131', '{"AM": 131, "MOI": 1000.0, "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": 20, "Num_TM": 55, "Name_RN": "цезий-131", "UdA_GRO": 24.0, "UdA_TRO": 1000.0, "ObA_GaRO": 3100.0, "Kod_gruppy": "б", "Period_p_r": 9.69, "Vid_izluch": "гамма", "Name_RN_Lat": "Cs-131", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иод-128', 'иод-128', '{"AM": 128, "MOI": "", "MZA": 100000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 53, "Name_RN": "иод-128", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.416, "Vid_izluch": "бета", "Name_RN_Lat": "I-128", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цезий-129', 'цезий-129', '{"AM": 129, "MOI": 10.0, "MZA": 100000.0, "MZUA": 100000.0, "D_val": 0.3, "Num_TM": 55, "Name_RN": "цезий-129", "UdA_GRO": 23.0, "UdA_TRO": 100.0, "ObA_GaRO": 1900.0, "Kod_gruppy": "б", "Period_p_r": 1.34, "Vid_izluch": "бета", "Name_RN_Lat": "Cs-129", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иод-121', 'иод-121', '{"AM": 121, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 53, "Name_RN": "иод-121", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.12, "Vid_izluch": "бета", "Name_RN_Lat": "I-121", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иод-120м', 'иод-120м', '{"AM": 120, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 53, "Name_RN": "иод-120м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.883, "Vid_izluch": "бета", "Name_RN_Lat": "I-120m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иод-120', 'иод-120', '{"AM": 120, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 53, "Name_RN": "иод-120", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.35, "Vid_izluch": "бета", "Name_RN_Lat": "I-120", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'теллур-134', 'теллур-134', '{"AM": 134, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 52, "Name_RN": "теллур-134", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.696, "Vid_izluch": "бета", "Name_RN_Lat": "Te-134", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'теллур-133м', 'теллур-133м', '{"AM": 133, "MOI": 10.0, "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 52, "Name_RN": "теллур-133м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.923, "Vid_izluch": "бета", "Name_RN_Lat": "Te-133m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ксенон-135', 'ксенон-135', '{"AM": 135, "MOI": "", "MZA": 10000000000.0, "MZUA": 1000000.0, "D_val": 0.3, "Num_TM": 54, "Name_RN": "ксенон-135", "UdA_GRO": "", "UdA_TRO": 1000.0, "ObA_GaRO": 2800.0, "Kod_gruppy": "б", "Period_p_r": 9.09, "Vid_izluch": "бета", "Name_RN_Lat": "Xe-135", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'теллур-133', 'теллур-133', '{"AM": 133, "MOI": 10.0, "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 52, "Name_RN": "теллур-133", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.207, "Vid_izluch": "бета", "Name_RN_Lat": "Te-133", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иод-135', 'иод-135', '{"AM": 135, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.04, "Num_TM": 53, "Name_RN": "иод-135", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 140.0, "Kod_gruppy": "б", "Period_p_r": 6.61, "Vid_izluch": "бета", "Name_RN_Lat": "I-135", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иод-134', 'иод-134', '{"AM": 134, "MOI": 10.0, "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.03, "Num_TM": 53, "Name_RN": "иод-134", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.876, "Vid_izluch": "бета", "Name_RN_Lat": "I-134", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иод-133', 'иод-133', '{"AM": 133, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.1, "Num_TM": 53, "Name_RN": "иод-133", "UdA_GRO": 3.1, "UdA_TRO": 10.0, "ObA_GaRO": 29.0, "Kod_gruppy": "б", "Period_p_r": 20.8, "Vid_izluch": "бета", "Name_RN_Lat": "I-133", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'теллур-131', 'теллур-131', '{"AM": 131, "MOI": 100.0, "MZA": 100000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 52, "Name_RN": "теллур-131", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.417, "Vid_izluch": "бета", "Name_RN_Lat": "Te-131", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иод-132', 'иод-132', '{"AM": 132, "MOI": 10.0, "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.03, "Num_TM": 53, "Name_RN": "иод-132", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 540.0, "Kod_gruppy": "б", "Period_p_r": 2.3, "Vid_izluch": "бета", "Name_RN_Lat": "I-132", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иод-131', 'иод-131', '{"AM": 131, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.2, "Num_TM": 53, "Name_RN": "иод-131", "UdA_GRO": 0.62, "UdA_TRO": 100.0, "ObA_GaRO": 7.3, "Kod_gruppy": "б", "Period_p_r": 8.04, "Vid_izluch": "бета", "Name_RN_Lat": "I-131", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'теллур-123', 'теллур-123', '{"AM": 123, "MOI": "", "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 52, "Name_RN": "теллур-123", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 92000000000000000, "Vid_izluch": "гамма", "Name_RN_Lat": "Te-123", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'теллур-116', 'теллур-116', '{"AM": 116, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 52, "Name_RN": "теллур-116", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.49, "Vid_izluch": "бета", "Name_RN_Lat": "Te-116", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сурьма-131', 'сурьма-131', '{"AM": 131, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 51, "Name_RN": "сурьма-131", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.383, "Vid_izluch": "бета", "Name_RN_Lat": "Sb-131", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '328.0', 'ГАЙАНА', '{"2": "ГАЙАНА", "v": 328.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100200', '4100200 Фонды', '{"2": "Фонды", "m": 4100200}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иод-126', 'иод-126', '{"AM": 126, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.1, "Num_TM": 53, "Name_RN": "иод-126", "UdA_GRO": 0.47, "UdA_TRO": 100.0, "ObA_GaRO": 6.3, "Kod_gruppy": "б", "Period_p_r": 13.0, "Vid_izluch": "бета", "Name_RN_Lat": "I-126", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иод-125', 'иод-125', '{"AM": 125, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": 0.2, "Num_TM": 53, "Name_RN": "иод-125", "UdA_GRO": 0.91, "UdA_TRO": 1000.0, "ObA_GaRO": 17.0, "Kod_gruppy": "б", "Period_p_r": 60.1, "Vid_izluch": "гамма", "Name_RN_Lat": "I-125", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иод-123', 'иод-123', '{"AM": 123, "MOI": 100.0, "MZA": 10000000.0, "MZUA": 100000.0, "D_val": 0.5, "Num_TM": 53, "Name_RN": "иод-123", "UdA_GRO": 65.0, "UdA_TRO": 100.0, "ObA_GaRO": 660.0, "Kod_gruppy": "б", "Period_p_r": 13.2, "Vid_izluch": "гамма", "Name_RN_Lat": "I-123", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сурьма-130', 'сурьма-130', '{"AM": 130, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 51, "Name_RN": "сурьма-130", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.667, "Vid_izluch": "бета", "Name_RN_Lat": "Sb-130", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сурьма-129', 'сурьма-129', '{"AM": 129, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 51, "Name_RN": "сурьма-129", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 4.32, "Vid_izluch": "бета", "Name_RN_Lat": "Sb-129", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сурьма-128м', 'сурьма-128м', '{"AM": 128, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 51, "Name_RN": "сурьма-128м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.173, "Vid_izluch": "гамма", "Name_RN_Lat": "Sb-128m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сурьма-128', 'сурьма-128', '{"AM": 128, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 51, "Name_RN": "сурьма-128", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 9.01, "Vid_izluch": "бета", "Name_RN_Lat": "Sb-128", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сурьма-127', 'сурьма-127', '{"AM": 127, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 51, "Name_RN": "сурьма-127", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.85, "Vid_izluch": "бета", "Name_RN_Lat": "Sb-127", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сурьма-126м', 'сурьма-126м', '{"AM": 126, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 51, "Name_RN": "сурьма-126м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.317, "Vid_izluch": "бета", "Name_RN_Lat": "Sb-126m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'теллур-132', 'теллур-132', '{"AM": 132, "MOI": 1.0, "MZA": 10000000.0, "MZUA": 100000.0, "D_val": 0.03, "Num_TM": 52, "Name_RN": "теллур-132", "UdA_GRO": 3.6, "UdA_TRO": 100.0, "ObA_GaRO": 40.0, "Kod_gruppy": "б", "Period_p_r": 3.26, "Vid_izluch": "бета", "Name_RN_Lat": "Te-132", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'теллур-131м', 'теллур-131м', '{"AM": 131, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.04, "Num_TM": 52, "Name_RN": "теллур-131м", "UdA_GRO": 7.2, "UdA_TRO": 10.0, "ObA_GaRO": 91.0, "Kod_gruppy": "б", "Period_p_r": 1.25, "Vid_izluch": "гамма", "Name_RN_Lat": "Te-131m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'теллур-129м', 'теллур-129м', '{"AM": 129, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": 1, "Num_TM": 52, "Name_RN": "теллур-129м", "UdA_GRO": 4.6, "UdA_TRO": 1000.0, "ObA_GaRO": 17.0, "Kod_gruppy": "б", "Period_p_r": 33.6, "Vid_izluch": "гамма", "Name_RN_Lat": "Te-129m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'теллур-127м', 'теллур-127м', '{"AM": 127, "MOI": 10.0, "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": 3, "Num_TM": 52, "Name_RN": "теллур-127м", "UdA_GRO": 6.0, "UdA_TRO": 1000.0, "ObA_GaRO": 15.0, "Kod_gruppy": "б", "Period_p_r": 109.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Te-127m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'теллур-127', 'теллур-127', '{"AM": 127, "MOI": 1000.0, "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": 10, "Num_TM": 52, "Name_RN": "теллур-127", "UdA_GRO": 81.0, "UdA_TRO": 1000.0, "ObA_GaRO": 720.0, "Kod_gruppy": "б", "Period_p_r": 9.35, "Vid_izluch": "бета", "Name_RN_Lat": "Te-127", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'теллур-125м', 'теллур-125м', '{"AM": 125, "MOI": 1000.0, "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": 10, "Num_TM": 52, "Name_RN": "теллур-125м", "UdA_GRO": 15.0, "UdA_TRO": 1000.0, "ObA_GaRO": 32.0, "Kod_gruppy": "б", "Period_p_r": 58.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Te-125m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'теллур-123м', 'теллур-123м', '{"AM": 123, "MOI": 1.0, "MZA": 10000000.0, "MZUA": 100000.0, "D_val": 0.6, "Num_TM": 52, "Name_RN": "теллур-123м", "UdA_GRO": 8.6, "UdA_TRO": 100.0, "ObA_GaRO": 27.0, "Kod_gruppy": "б", "Period_p_r": 120.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Te-123m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'теллур-121м', 'теллур-121м', '{"AM": 121, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.1, "Num_TM": 52, "Name_RN": "теллур-121м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 154.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Te-121m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'теллур-121', 'теллур-121', '{"AM": 121, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.1, "Num_TM": 52, "Name_RN": "теллур-121", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 19.17, "Vid_izluch": "гамма", "Name_RN_Lat": "Te-121", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сурьма-120', 'сурьма-120', '{"AM": 120, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 51, "Name_RN": "сурьма-120", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.265, "Vid_izluch": "бета", "Name_RN_Lat": "Sb-120", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сурьма-119', 'сурьма-119', '{"AM": 119, "MOI": "", "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 51, "Name_RN": "сурьма-119", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.59, "Vid_izluch": "гамма", "Name_RN_Lat": "Sb-119", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сурьма-118м', 'сурьма-118м', '{"AM": 118, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 51, "Name_RN": "сурьма-118м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 5.0, "Vid_izluch": "бета", "Name_RN_Lat": "Sb-118m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сурьма-117', 'сурьма-117', '{"AM": 117, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 51, "Name_RN": "сурьма-117", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.8, "Vid_izluch": "бета", "Name_RN_Lat": "Sb-117", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сурьма-116м', 'сурьма-116м', '{"AM": 116, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 51, "Name_RN": "сурьма-116м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.0, "Vid_izluch": "бета", "Name_RN_Lat": "Sb-116m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сурьма-116', 'сурьма-116', '{"AM": 116, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 51, "Name_RN": "сурьма-116", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.263, "Vid_izluch": "бета", "Name_RN_Lat": "Sb-116", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сурьма-115', 'сурьма-115', '{"AM": 115, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 51, "Name_RN": "сурьма-115", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.53, "Vid_izluch": "бета", "Name_RN_Lat": "Sb-115", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'олово-128', 'олово-128', '{"AM": 128, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 50, "Name_RN": "олово-128", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.985, "Vid_izluch": "бета", "Name_RN_Lat": "Sn-128", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сурьма-126', 'сурьма-126', '{"AM": 126, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.02, "Num_TM": 51, "Name_RN": "сурьма-126", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 12.4, "Vid_izluch": "бета", "Name_RN_Lat": "Sb-126", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сурьма-125', 'сурьма-125', '{"AM": 125, "MOI": 0.1, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.2, "Num_TM": 51, "Name_RN": "сурьма-125", "UdA_GRO": 12.0, "UdA_TRO": 100.0, "ObA_GaRO": 24.0, "Kod_gruppy": "б", "Period_p_r": 2.77, "Vid_izluch": "бета", "Name_RN_Lat": "Sb-125", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'олово-127', 'олово-127', '{"AM": 127, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 50, "Name_RN": "олово-127", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.1, "Vid_izluch": "бета", "Name_RN_Lat": "Sn-127", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сурьма-124', 'сурьма-124', '{"AM": 124, "MOI": 1.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.04, "Num_TM": 51, "Name_RN": "сурьма-124", "UdA_GRO": 5.5, "UdA_TRO": 10.0, "ObA_GaRO": 18.0, "Kod_gruppy": "б", "Period_p_r": 60.2, "Vid_izluch": "бета", "Name_RN_Lat": "Sb-124", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сурьма-122', 'сурьма-122', '{"AM": 122, "MOI": 10.0, "MZA": 10000.0, "MZUA": 100000.0, "D_val": 0.1, "Num_TM": 51, "Name_RN": "сурьма-122", "UdA_GRO": 8.1, "UdA_TRO": 100.0, "ObA_GaRO": 92.0, "Kod_gruppy": "б", "Period_p_r": 2.7, "Vid_izluch": "бета", "Name_RN_Lat": "Sb-122", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'олово-123м', 'олово-123м', '{"AM": 123, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 50, "Name_RN": "олово-123м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.668, "Vid_izluch": "бета", "Name_RN_Lat": "Sn-123m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'олово-121', 'олово-121', '{"AM": 121, "MOI": "", "MZA": 10000000.0, "MZUA": 100000000.0, "D_val": "", "Num_TM": 50, "Name_RN": "олово-121", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 27.03, "Vid_izluch": "бета", "Name_RN_Lat": "Sn-121", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'олово-111', 'олово-111', '{"AM": 111, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 50, "Name_RN": "олово-111", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.588, "Vid_izluch": "бета", "Name_RN_Lat": "Sn-111", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'олово-110', 'олово-110', '{"AM": 110, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 50, "Name_RN": "олово-110", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 4.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Sn-110", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'индий-119м', 'индий-119м', '{"AM": 119, "MOI": "", "MZA": 100000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 49, "Name_RN": "индий-119м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.3, "Vid_izluch": "бета", "Name_RN_Lat": "In-119m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'индий-117м', 'индий-117м', '{"AM": 117, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 49, "Name_RN": "индий-117м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.94, "Vid_izluch": "бета", "Name_RN_Lat": "In-117m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'индий-117', 'индий-117', '{"AM": 117, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 49, "Name_RN": "индий-117", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.73, "Vid_izluch": "бета", "Name_RN_Lat": "In-117", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'индий-116м', 'индий-116м', '{"AM": 116, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 49, "Name_RN": "индий-116м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.902, "Vid_izluch": "бета", "Name_RN_Lat": "In-116m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'индий-112', 'индий-112', '{"AM": 112, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 49, "Name_RN": "индий-112", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.24, "Vid_izluch": "бета", "Name_RN_Lat": "In-112", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'индий-110м', 'индий-110м', '{"AM": 110, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 49, "Name_RN": "индий-110м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.15, "Vid_izluch": "бета", "Name_RN_Lat": "In-110м", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'олово-126', 'олово-126', '{"AM": 126, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.03, "Num_TM": 50, "Name_RN": "олово-126", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 230000.0, "Vid_izluch": "бета", "Name_RN_Lat": "Sn-126", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'олово-125', 'олово-125', '{"AM": 125, "MOI": 10.0, "MZA": 100000.0, "MZUA": 100000.0, "D_val": 0.1, "Num_TM": 50, "Name_RN": "олово-125", "UdA_GRO": 4.4, "UdA_TRO": 100.0, "ObA_GaRO": 35.0, "Kod_gruppy": "б", "Period_p_r": 9.64, "Vid_izluch": "бета", "Name_RN_Lat": "Sn-125", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'индий-110', 'индий-110', '{"AM": 110, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 49, "Name_RN": "индий-110", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 4.9, "Vid_izluch": "бета", "Name_RN_Lat": "In-110", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'олово-123', 'олово-123', '{"AM": 123, "MOI": "", "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": 7, "Num_TM": 50, "Name_RN": "олово-123", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 129.0, "Vid_izluch": "бета", "Name_RN_Lat": "Sn-123", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'олово-121м', 'олово-121м', '{"AM": 121, "MOI": "", "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": 70, "Num_TM": 50, "Name_RN": "олово-121м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 43.9, "Vid_izluch": "гамма", "Name_RN_Lat": "Sn-121m", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'индий-109', 'индий-109', '{"AM": 109, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 49, "Name_RN": "индий-109", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 4.2, "Vid_izluch": "бета", "Name_RN_Lat": "In-109", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'олово-119м', 'олово-119м', '{"AM": 119, "MOI": "", "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": 70, "Num_TM": 50, "Name_RN": "олово-119м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 293.1, "Vid_izluch": "гамма", "Name_RN_Lat": "Sn-119m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'олово-117м', 'олово-117м', '{"AM": 117, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.5, "Num_TM": 50, "Name_RN": "олово-117м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 14.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Sn-117m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'олово-113', 'олово-113', '{"AM": 113, "MOI": 1.0, "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": 0.3, "Num_TM": 50, "Name_RN": "олово-113", "UdA_GRO": 19.0, "UdA_TRO": 1000.0, "ObA_GaRO": 43.0, "Kod_gruppy": "б", "Period_p_r": 115.0, "Vid_izluch": "бета", "Name_RN_Lat": "Sn-113", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кадмий-117м', 'кадмий-117м', '{"AM": 117, "MOI": "", "MZA": "", "MZUA": "", "D_val": "", "Num_TM": 48, "Name_RN": "кадмий-117м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.36, "Vid_izluch": "бета", "Name_RN_Lat": "Cd-117m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кадмий-117', 'кадмий-117', '{"AM": 117, "MOI": "", "MZA": "", "MZUA": "", "D_val": "", "Num_TM": 48, "Name_RN": "кадмий-117", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.49, "Vid_izluch": "бета", "Name_RN_Lat": "Cd-117", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кадмий-113', 'кадмий-113', '{"AM": 113, "MOI": "", "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 48, "Name_RN": "кадмий-113", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 8040000000000000.0, "Vid_izluch": "бета", "Name_RN_Lat": "Cd-113", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кадмий-107', 'кадмий-107', '{"AM": 107, "MOI": "", "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 48, "Name_RN": "кадмий-107", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 6.49, "Vid_izluch": "бета", "Name_RN_Lat": "Cd-107", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кадмий-104', 'кадмий-104', '{"AM": 104, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 48, "Name_RN": "кадмий-104", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.961, "Vid_izluch": "бета", "Name_RN_Lat": "Cd-104", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'серебро-115', 'серебро-115', '{"AM": 115, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 47, "Name_RN": "серебро-115", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.333, "Vid_izluch": "бета", "Name_RN_Lat": "Ag-115", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'индий-115м', 'индий-115м', '{"AM": 115, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.4, "Num_TM": 49, "Name_RN": "индий-115м", "UdA_GRO": "", "UdA_TRO": 100.0, "ObA_GaRO": 1500.0, "Kod_gruppy": "б", "Period_p_r": 4.49, "Vid_izluch": "гамма", "Name_RN_Lat": "In-115m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'индий-114м', 'индий-114м', '{"AM": 114, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.8, "Num_TM": 49, "Name_RN": "индий-114м", "UdA_GRO": 3.3, "UdA_TRO": 100.0, "ObA_GaRO": 6.8, "Kod_gruppy": "б", "Period_p_r": 49.5, "Vid_izluch": "гамма", "Name_RN_Lat": "In-114m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'индий-113м', 'индий-113м', '{"AM": 113, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.3, "Num_TM": 49, "Name_RN": "индий-113м", "UdA_GRO": "", "UdA_TRO": 100.0, "ObA_GaRO": 4700.0, "Kod_gruppy": "б", "Period_p_r": 1.66, "Vid_izluch": "гамма", "Name_RN_Lat": "In-113m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'серебро-112', 'серебро-112', '{"AM": 112, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 47, "Name_RN": "серебро-112", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.12, "Vid_izluch": "бета", "Name_RN_Lat": "Ag-112", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'индий-111', 'индий-111', '{"AM": 111, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.2, "Num_TM": 49, "Name_RN": "индий-111", "UdA_GRO": 47.0, "UdA_TRO": 100.0, "ObA_GaRO": 440.0, "Kod_gruppy": "б", "Period_p_r": 2.83, "Vid_izluch": "гамма", "Name_RN_Lat": "In-111", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'серебро-106м', 'серебро-106м', '{"AM": 106, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 47, "Name_RN": "серебро-106м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 8.41, "Vid_izluch": "бета", "Name_RN_Lat": "Ag-106m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'серебро-106', 'серебро-106', '{"AM": 106, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 47, "Name_RN": "серебро-106", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.399, "Vid_izluch": "бета", "Name_RN_Lat": "Ag-106", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'серебро-104м', 'серебро-104м', '{"AM": 104, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 47, "Name_RN": "серебро-104м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.558, "Vid_izluch": "бета", "Name_RN_Lat": "Ag-104m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'серебро-104', 'серебро-104', '{"AM": 104, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 47, "Name_RN": "серебро-104", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.15, "Vid_izluch": "бета", "Name_RN_Lat": "Ag-104", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'серебро-103', 'серебро-103', '{"AM": 103, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 47, "Name_RN": "серебро-103", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.09, "Vid_izluch": "бета", "Name_RN_Lat": "Ag-103", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кадмий-115м', 'кадмий-115м', '{"AM": 115, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": 3, "Num_TM": 48, "Name_RN": "кадмий-115м", "UdA_GRO": 4.2, "UdA_TRO": 1000.0, "ObA_GaRO": 15.0, "Kod_gruppy": "б", "Period_p_r": 44.6, "Vid_izluch": "бета", "Name_RN_Lat": "Cd-115m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кадмий-115', 'кадмий-115', '{"AM": 115, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.2, "Num_TM": 48, "Name_RN": "кадмий-115", "UdA_GRO": 9.8, "UdA_TRO": 100.0, "ObA_GaRO": 100.0, "Kod_gruppy": "б", "Period_p_r": 2.23, "Vid_izluch": "бета", "Name_RN_Lat": "Cd-115", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кадмий-113м', 'кадмий-113м', '{"AM": 113, "MOI": "", "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": 40, "Num_TM": 48, "Name_RN": "кадмий-113м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 14.1, "Vid_izluch": "бета", "Name_RN_Lat": "Cd-113m", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'серебро-102', 'серебро-102', '{"AM": 102, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 47, "Name_RN": "серебро-102", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.215, "Vid_izluch": "бета", "Name_RN_Lat": "Ag-102", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кадмий-109', 'кадмий-109', '{"AM": 109, "MOI": 1.0, "MZA": 1000000.0, "MZUA": 10000000.0, "D_val": 20, "Num_TM": 48, "Name_RN": "кадмий-109", "UdA_GRO": 6.9, "UdA_TRO": 10000.0, "ObA_GaRO": 14.0, "Kod_gruppy": "б", "Period_p_r": 1.27, "Vid_izluch": "гамма", "Name_RN_Lat": "Cd-109", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'палладий-107', 'палладий-107', '{"AM": 107, "MOI": "", "MZA": 100000000.0, "MZUA": 100000000.0, "D_val": "неогран.", "Num_TM": 46, "Name_RN": "палладий-107", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 6500000.0, "Vid_izluch": "бета", "Name_RN_Lat": "Pd-107", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'палладий-101', 'палладий-101', '{"AM": 101, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 46, "Name_RN": "палладий-101", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 8.27, "Vid_izluch": "бета", "Name_RN_Lat": "Pd-101", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'палладий-100', 'палладий-100', '{"AM": 100, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 46, "Name_RN": "палладий-100", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.63, "Vid_izluch": "гамма", "Name_RN_Lat": "Pd-100", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'родий-106', 'родий-106', '{"AM": 106, "MOI": "", "MZA": 0.0, "MZUA": 0.0, "D_val": 0, "Num_TM": 45, "Name_RN": "родий-106", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 28.8, "Vid_izluch": "бета", "Name_RN_Lat": "Rh-106", "Edinica_izmer_p_r": "сек", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'серебро-111', 'серебро-111', '{"AM": 111, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": 2, "Num_TM": 47, "Name_RN": "серебро-111", "UdA_GRO": 11.0, "UdA_TRO": 1000.0, "ObA_GaRO": 72.0, "Kod_gruppy": "б", "Period_p_r": 7.45, "Vid_izluch": "бета", "Name_RN_Lat": "Ag-111", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'серебро-110м', 'серебро-110м', '{"AM": 110, "MOI": 0.1, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.02, "Num_TM": 47, "Name_RN": "серебро-110м", "UdA_GRO": 4.9, "UdA_TRO": 10.0, "ObA_GaRO": 15.0, "Kod_gruppy": "б", "Period_p_r": 250.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Ag-110m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'серебро-108м', 'серебро-108м', '{"AM": 108, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.04, "Num_TM": 47, "Name_RN": "серебро-108м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 438.0, "Vid_izluch": "бета", "Name_RN_Lat": "Ag-108m", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'родий-107', 'родий-107', '{"AM": 107, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 45, "Name_RN": "родий-107", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.362, "Vid_izluch": "бета", "Name_RN_Lat": "Rh-107", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'родий-106м', 'родий-106м', '{"AM": 106, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 45, "Name_RN": "родий-106м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.2, "Vid_izluch": "бета", "Name_RN_Lat": "Rh-106m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'серебро-105', 'серебро-105', '{"AM": 105, "MOI": 1.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.1, "Num_TM": 47, "Name_RN": "серебро-105", "UdA_GRO": 29.0, "UdA_TRO": 100.0, "ObA_GaRO": 150.0, "Kod_gruppy": "б", "Period_p_r": 41.0, "Vid_izluch": "бета", "Name_RN_Lat": "Ag-105", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '334.0', 'ОСТРОВ ХЕРД И ОСТРОВА МАКДОНАЛЬД', '{"2": "ОСТРОВ ХЕРД И ОСТРОВА МАКДОНАЛЬД", "v": 334.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'родий-101м', 'родий-101м', '{"AM": 101, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 45, "Name_RN": "родий-101м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 4.34, "Vid_izluch": "гамма", "Name_RN_Lat": "Rh-101m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'родий-100', 'родий-100', '{"AM": 100, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 45, "Name_RN": "родий-100", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 20.8, "Vid_izluch": "бета", "Name_RN_Lat": "Rh-100", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'родий-99м', 'родий-99м', '{"AM": 99, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 45, "Name_RN": "родий-99м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 4.7, "Vid_izluch": "бета", "Name_RN_Lat": "Rh-99m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рутений-94', 'рутений-94', '{"AM": 94, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 44, "Name_RN": "рутений-94", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.863, "Vid_izluch": "гамма", "Name_RN_Lat": "Ru-94", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'палладий-109', 'палладий-109', '{"AM": 109, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": 20, "Num_TM": 46, "Name_RN": "палладий-109", "UdA_GRO": 24.0, "UdA_TRO": 1000.0, "ObA_GaRO": 270.0, "Kod_gruppy": "б", "Period_p_r": 13.4, "Vid_izluch": "бета", "Name_RN_Lat": "Pd-109", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'технеций-104', 'технеций-104', '{"AM": 104, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 43, "Name_RN": "технеций-104", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.303, "Vid_izluch": "бета", "Name_RN_Lat": "Tc-104", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'палладий-103', 'палладий-103', '{"AM": 103, "MOI": 1000.0, "MZA": 100000000.0, "MZUA": 1000000.0, "D_val": 90, "Num_TM": 46, "Name_RN": "палладий-103", "UdA_GRO": 72.0, "UdA_TRO": 1000.0, "ObA_GaRO": 260.0, "Kod_gruppy": "б", "Period_p_r": 17.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Pd-103", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'технеций-101', 'технеций-101', '{"AM": 101, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 43, "Name_RN": "технеций-101", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.237, "Vid_izluch": "бета", "Name_RN_Lat": "Tc-101", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'технеций-97', 'технеций-97', '{"AM": 97, "MOI": 10.0, "MZA": 100000000.0, "MZUA": 1000000.0, "D_val": "неогран.", "Num_TM": 43, "Name_RN": "технеций-97", "UdA_GRO": 210.0, "UdA_TRO": 1000.0, "ObA_GaRO": 490.0, "Kod_gruppy": "б", "Period_p_r": 4210000.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Tc-97", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'технеций-95', 'технеций-95', '{"AM": 95, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 43, "Name_RN": "технеций-95", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 20.0, "Vid_izluch": "бета", "Name_RN_Lat": "Tc-95", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'технеций-94м', 'технеций-94м', '{"AM": 94, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 43, "Name_RN": "технеций-94м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.867, "Vid_izluch": "бета", "Name_RN_Lat": "Tc-94m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'родий-105', 'родий-105', '{"AM": 105, "MOI": 100.0, "MZA": 10000000.0, "MZUA": 100000.0, "D_val": 0.9, "Num_TM": 45, "Name_RN": "родий-105", "UdA_GRO": 37.0, "UdA_TRO": 100.0, "ObA_GaRO": 300.0, "Kod_gruppy": "б", "Period_p_r": 1.47, "Vid_izluch": "бета", "Name_RN_Lat": "Rh-105", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'родий-103м', 'родий-103м', '{"AM": 103, "MOI": 10000.0, "MZA": 100000000.0, "MZUA": 10000000.0, "D_val": 900, "Num_TM": 45, "Name_RN": "родий-103м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.935, "Vid_izluch": "гамма", "Name_RN_Lat": "Rh-103m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'родий-102м', 'родий-102м', '{"AM": 102, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.1, "Num_TM": 45, "Name_RN": "родий-102м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.74, "Vid_izluch": "гамма", "Name_RN_Lat": "Rh-102m", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'родий-102', 'родий-102', '{"AM": 102, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.03, "Num_TM": 45, "Name_RN": "родий-102", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 207.3, "Vid_izluch": "бета", "Name_RN_Lat": "Rh-102", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'технеций-94', 'технеций-94', '{"AM": 94, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 43, "Name_RN": "технеций-94", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 4.88, "Vid_izluch": "бета", "Name_RN_Lat": "Tc-94", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'родий-101', 'родий-101', '{"AM": 101, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": 0.3, "Num_TM": 45, "Name_RN": "родий-101", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.2, "Vid_izluch": "гамма", "Name_RN_Lat": "Rh-101", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'технеций-93м', 'технеций-93м', '{"AM": 93, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 43, "Name_RN": "технеций-93м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.725, "Vid_izluch": "гамма", "Name_RN_Lat": "Tc-93m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'технеций-93', 'технеций-93', '{"AM": 93, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 43, "Name_RN": "технеций-93", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.75, "Vid_izluch": "бета", "Name_RN_Lat": "Tc-93", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'родий-99', 'родий-99', '{"AM": 99, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.1, "Num_TM": 45, "Name_RN": "родий-99", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 16.0, "Vid_izluch": "бета", "Name_RN_Lat": "Rh-99", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рутений-106', 'рутений-106', '{"AM": 106, "MOI": 0.1, "MZA": 100000.0, "MZUA": 100000.0, "D_val": 0.3, "Num_TM": 44, "Name_RN": "рутений-106", "UdA_GRO": 2.0, "UdA_TRO": 100.0, "ObA_GaRO": 4.4, "Kod_gruppy": "б", "Period_p_r": 1.01, "Vid_izluch": "бета", "Name_RN_Lat": "Ru-106", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рутений-105', 'рутений-105', '{"AM": 105, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.08, "Num_TM": 44, "Name_RN": "рутений-105", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 570.0, "Kod_gruppy": "б", "Period_p_r": 4.44, "Vid_izluch": "бета", "Name_RN_Lat": "Ru-105", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рутений-103', 'рутений-103', '{"AM": 103, "MOI": 1.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.1, "Num_TM": 44, "Name_RN": "рутений-103", "UdA_GRO": 19.0, "UdA_TRO": 100.0, "ObA_GaRO": 46.0, "Kod_gruppy": "б", "Period_p_r": 39.3, "Vid_izluch": "бета", "Name_RN_Lat": "Ru-103", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рутений-97', 'рутений-97', '{"AM": 97, "MOI": 10.0, "MZA": 10000000.0, "MZUA": 100000.0, "D_val": 0.3, "Num_TM": 44, "Name_RN": "рутений-97", "UdA_GRO": 91.0, "UdA_TRO": 100.0, "ObA_GaRO": 860.0, "Kod_gruppy": "б", "Period_p_r": 2.9, "Vid_izluch": "гамма", "Name_RN_Lat": "Ru-97", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'молибден-101', 'молибден-101', '{"AM": 101, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 42, "Name_RN": "молибден-101", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.244, "Vid_izluch": "бета", "Name_RN_Lat": "Mo-101", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'молибден-93м', 'молибден-93м', '{"AM": 93, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 42, "Name_RN": "молибден-93м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 6.85, "Vid_izluch": "гамма", "Name_RN_Lat": "Mo-93m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'молибден-90', 'молибден-90', '{"AM": 90, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 42, "Name_RN": "молибден-90", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 260.0, "Kod_gruppy": "б", "Period_p_r": 5.67, "Vid_izluch": "бета", "Name_RN_Lat": "Mo-90", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'технеций-99м', 'технеций-99м', '{"AM": 99, "MOI": 100.0, "MZA": 10000000.0, "MZUA": 100000.0, "D_val": 0.7, "Num_TM": 43, "Name_RN": "технеций-99м", "UdA_GRO": "", "UdA_TRO": 100.0, "ObA_GaRO": 5300.0, "Kod_gruppy": "б", "Period_p_r": 6.02, "Vid_izluch": "гамма", "Name_RN_Lat": "Tc-99m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'технеций-99', 'технеций-99', '{"AM": 99, "MOI": 1.0, "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": 30, "Num_TM": 43, "Name_RN": "технеций-99", "UdA_GRO": 21.0, "UdA_TRO": 10000.0, "ObA_GaRO": 27.0, "Kod_gruppy": "б", "Period_p_r": 211000.0, "Vid_izluch": "бета", "Name_RN_Lat": "Tc-99", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'технеций-98', 'технеций-98', '{"AM": 98, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.05, "Num_TM": 43, "Name_RN": "технеций-98", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 4200000.0, "Vid_izluch": "бета", "Name_RN_Lat": "Tc-98", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'технеций-97м', 'технеций-97м', '{"AM": 97, "MOI": 100.0, "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": 40, "Num_TM": 43, "Name_RN": "технеций-97м", "UdA_GRO": 25.0, "UdA_TRO": 1000.0, "ObA_GaRO": 33.0, "Kod_gruppy": "б", "Period_p_r": 91.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Tc-97m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ниобий-98', 'ниобий-98', '{"AM": 98, "MOI": 10.0, "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 41, "Name_RN": "ниобий-98", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.858, "Vid_izluch": "бета", "Name_RN_Lat": "Nb-98", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'технеций-96м', 'технеций-96м', '{"AM": 96, "MOI": 1000.0, "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": 0.03, "Num_TM": 43, "Name_RN": "технеций-96м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.858, "Vid_izluch": "гамма", "Name_RN_Lat": "Tc-96m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'технеций-96', 'технеций-96', '{"AM": 96, "MOI": 1.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.03, "Num_TM": 43, "Name_RN": "технеций-96", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 130.0, "Kod_gruppy": "б", "Period_p_r": 4.28, "Vid_izluch": "бета", "Name_RN_Lat": "Tc-96", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'технеций-95м', 'технеций-95м', '{"AM": 95, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.1, "Num_TM": 43, "Name_RN": "технеций-95м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 61.0, "Vid_izluch": "бета", "Name_RN_Lat": "Tc-95m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ниобий-96', 'ниобий-96', '{"AM": 96, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 41, "Name_RN": "ниобий-96", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 23.3, "Vid_izluch": "бета", "Name_RN_Lat": "Nb-96", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ниобий-95м', 'ниобий-95м', '{"AM": 95, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 41, "Name_RN": "ниобий-95м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.61, "Vid_izluch": "гамма", "Name_RN_Lat": "Nb-95m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ниобий-90', 'ниобий-90', '{"AM": 90, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 41, "Name_RN": "ниобий-90", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 14.6, "Vid_izluch": "бета", "Name_RN_Lat": "Nb-90", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ниобий-89м', 'ниобий-89м', '{"AM": 89, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 41, "Name_RN": "ниобий-89м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.1, "Vid_izluch": "гамма", "Name_RN_Lat": "Nb-89m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ниобий-89', 'ниобий-89', '{"AM": 89, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 41, "Name_RN": "ниобий-89", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.03, "Vid_izluch": "бета", "Name_RN_Lat": "Nb-89", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ниобий-88', 'ниобий-88', '{"AM": 88, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 41, "Name_RN": "ниобий-88", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.238, "Vid_izluch": "бета", "Name_RN_Lat": "Nb-88", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'молибден-99', 'молибден-99', '{"AM": 99, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.3, "Num_TM": 42, "Name_RN": "молибден-99", "UdA_GRO": 22.0, "UdA_TRO": 100.0, "ObA_GaRO": 120.0, "Kod_gruppy": "б", "Period_p_r": 2.75, "Vid_izluch": "бета", "Name_RN_Lat": "Mo-99", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цирконий-93', 'цирконий-93', '{"AM": 93, "MOI": 10.0, "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": "неогран.", "Num_TM": 40, "Name_RN": "цирконий-93", "UdA_GRO": 12.0, "UdA_TRO": 1000.0, "ObA_GaRO": 12.0, "Kod_gruppy": "б", "Period_p_r": 1530000.0, "Vid_izluch": "бета", "Name_RN_Lat": "Zr-93", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'молибден-93', 'молибден-93', '{"AM": 93, "MOI": 10.0, "MZA": 100000000.0, "MZUA": 1000000.0, "D_val": 300, "Num_TM": 42, "Name_RN": "молибден-93", "UdA_GRO": 4.4, "UdA_TRO": 1000.0, "ObA_GaRO": 210.0, "Kod_gruppy": "б", "Period_p_r": 4000.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Mo-93", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цирконий-89', 'цирконий-89', '{"AM": 89, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 40, "Name_RN": "цирконий-89", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.27, "Vid_izluch": "бета", "Name_RN_Lat": "Zr-89", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цирконий-86', 'цирконий-86', '{"AM": 86, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 40, "Name_RN": "цирконий-86", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 16.5, "Vid_izluch": "гамма", "Name_RN_Lat": "Zr-86", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ниобий-97', 'ниобий-97', '{"AM": 97, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.1, "Num_TM": 41, "Name_RN": "ниобий-97", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 2100.0, "Kod_gruppy": "б", "Period_p_r": 1.2, "Vid_izluch": "бета", "Name_RN_Lat": "Nb-97", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иттрий-95', 'иттрий-95', '{"AM": 95, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 39, "Name_RN": "иттрий-95", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.178, "Vid_izluch": "бета", "Name_RN_Lat": "Y-95", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иттрий-94', 'иттрий-94', '{"AM": 94, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 39, "Name_RN": "иттрий-94", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.318, "Vid_izluch": "бета", "Name_RN_Lat": "Y-94", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ниобий-95', 'ниобий-95', '{"AM": 95, "MOI": 1.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.09, "Num_TM": 41, "Name_RN": "ниобий-95", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 72.0, "Kod_gruppy": "б", "Period_p_r": 35.1, "Vid_izluch": "бета", "Name_RN_Lat": "Nb-95", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ниобий-94', 'ниобий-94', '{"AM": 94, "MOI": 0.1, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.04, "Num_TM": 41, "Name_RN": "ниобий-94", "UdA_GRO": 8.1, "UdA_TRO": 10.0, "ObA_GaRO": 11.0, "Kod_gruppy": "б", "Period_p_r": 20300.0, "Vid_izluch": "бета", "Name_RN_Lat": "Nb-94", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ниобий-93м', 'ниобий-93м', '{"AM": 93, "MOI": 10.0, "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": 300, "Num_TM": 41, "Name_RN": "ниобий-93м", "UdA_GRO": 110.0, "UdA_TRO": 10000.0, "ObA_GaRO": 220.0, "Kod_gruppy": "б", "Period_p_r": 16.12, "Vid_izluch": "гамма", "Name_RN_Lat": "Nb-93m", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иттрий-90м', 'иттрий-90м', '{"AM": 90, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 39, "Name_RN": "иттрий-90м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.19, "Vid_izluch": "гамма", "Name_RN_Lat": "Y-90m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иттрий-86м', 'иттрий-86м', '{"AM": 86, "MOI": "", "MZA": 10000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 39, "Name_RN": "иттрий-86м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.8, "Vid_izluch": "гамма", "Name_RN_Lat": "Y-86m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иттрий-86', 'иттрий-86', '{"AM": 86, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 39, "Name_RN": "иттрий-86", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 14.7, "Vid_izluch": "бета", "Name_RN_Lat": "Y-86", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'стронций-83', 'стронций-83', '{"AM": 83, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 38, "Name_RN": "стронций-83", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.35, "Vid_izluch": "бета", "Name_RN_Lat": "Sr-83", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цирконий-97', 'цирконий-97', '{"AM": 97, "MOI": 10.0, "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.04, "Num_TM": 40, "Name_RN": "цирконий-97", "UdA_GRO": 6.5, "UdA_TRO": 10.0, "ObA_GaRO": 99.0, "Kod_gruppy": "б", "Period_p_r": 16.9, "Vid_izluch": "бета", "Name_RN_Lat": "Zr-97", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цирконий-95', 'цирконий-95', '{"AM": 95, "MOI": 1.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.04, "Num_TM": 40, "Name_RN": "цирконий-95", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 23.0, "Kod_gruppy": "б", "Period_p_r": 64.0, "Vid_izluch": "бета", "Name_RN_Lat": "Zr-95", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'стронций-81', 'стронций-81', '{"AM": 81, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 38, "Name_RN": "стронций-81", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.372, "Vid_izluch": "бета", "Name_RN_Lat": "Sr-81", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'стронций-80', 'стронций-80', '{"AM": 80, "MOI": "", "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 38, "Name_RN": "стронций-80", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.77, "Vid_izluch": "бета", "Name_RN_Lat": "Sr-80", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цирконий-88', 'цирконий-88', '{"AM": 88, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.02, "Num_TM": 40, "Name_RN": "цирконий-88", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 83.4, "Vid_izluch": "гамма", "Name_RN_Lat": "Zr-88", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рубидий-89', 'рубидий-89', '{"AM": 89, "MOI": "", "MZA": 100000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 37, "Name_RN": "рубидий-89", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.253, "Vid_izluch": "бета", "Name_RN_Lat": "Rb-89", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рубидий-88', 'рубидий-88', '{"AM": 88, "MOI": "", "MZA": 100000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 37, "Name_RN": "рубидий-88", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.297, "Vid_izluch": "бета", "Name_RN_Lat": "Rb-88", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рубидий-82м', 'рубидий-82м', '{"AM": 82, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 37, "Name_RN": "рубидий-82м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 6.2, "Vid_izluch": "бета", "Name_RN_Lat": "Rb-82m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иттрий-93', 'иттрий-93', '{"AM": 93, "MOI": 100.0, "MZA": 100000.0, "MZUA": 100000.0, "D_val": 0.6, "Num_TM": 39, "Name_RN": "иттрий-93", "UdA_GRO": 11.0, "UdA_TRO": 100.0, "ObA_GaRO": 170.0, "Kod_gruppy": "б", "Period_p_r": 10.1, "Vid_izluch": "бета", "Name_RN_Lat": "Y-93", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иттрий-92', 'иттрий-92', '{"AM": 92, "MOI": 100.0, "MZA": 100000.0, "MZUA": 100000.0, "D_val": 0.2, "Num_TM": 39, "Name_RN": "иттрий-92", "UdA_GRO": 27.0, "UdA_TRO": 100.0, "ObA_GaRO": 430.0, "Kod_gruppy": "б", "Period_p_r": 3.54, "Vid_izluch": "бета", "Name_RN_Lat": "Y-92", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иттрий-91м', 'иттрий-91м', '{"AM": 91, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.1, "Num_TM": 39, "Name_RN": "иттрий-91м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.828, "Vid_izluch": "гамма", "Name_RN_Lat": "Y-91m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иттрий-91', 'иттрий-91', '{"AM": 91, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": 8, "Num_TM": 39, "Name_RN": "иттрий-91", "UdA_GRO": 5.7, "UdA_TRO": 1000.0, "ObA_GaRO": 14.0, "Kod_gruppy": "б", "Period_p_r": 58.5, "Vid_izluch": "бета", "Name_RN_Lat": "Y-91", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рубидий-81м', 'рубидий-81м', '{"AM": 81, "MOI": "", "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 37, "Name_RN": "рубидий-81м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.533, "Vid_izluch": "гамма", "Name_RN_Lat": "Rb-81m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иттрий-90', 'иттрий-90', '{"AM": 90, "MOI": 1000.0, "MZA": 100000.0, "MZUA": 1000000.0, "D_val": 5, "Num_TM": 39, "Name_RN": "иттрий-90", "UdA_GRO": 5.1, "UdA_TRO": 1000.0, "ObA_GaRO": 60.0, "Kod_gruppy": "б", "Period_p_r": 2.67, "Vid_izluch": "бета", "Name_RN_Lat": "Y-90", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иттрий-88', 'иттрий-88', '{"AM": 88, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.03, "Num_TM": 39, "Name_RN": "иттрий-88", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 106.626, "Vid_izluch": "бета", "Name_RN_Lat": "Y-88", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иттрий-87', 'иттрий-87', '{"AM": 87, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.09, "Num_TM": 39, "Name_RN": "иттрий-87", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.35, "Vid_izluch": "бета", "Name_RN_Lat": "Y-87", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рубидий-79', 'рубидий-79', '{"AM": 79, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 37, "Name_RN": "рубидий-79", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.382, "Vid_izluch": "бета", "Name_RN_Lat": "Rb-79", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'криптон-89', 'криптон-89', '{"AM": 89, "MOI": "", "MZA": "", "MZUA": "", "D_val": "", "Num_TM": 36, "Name_RN": "криптон-89", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.18, "Vid_izluch": "бета", "Name_RN_Lat": "Kr-89", "Edinica_izmer_p_r": "мин", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'стронций-92', 'стронций-92', '{"AM": 92, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.04, "Num_TM": 38, "Name_RN": "стронций-92", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 370.0, "Kod_gruppy": "б", "Period_p_r": 2.71, "Vid_izluch": "бета", "Name_RN_Lat": "Sr-92", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'стронций-91', 'стронций-91', '{"AM": 91, "MOI": 10.0, "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.06, "Num_TM": 38, "Name_RN": "стронций-91", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 230.0, "Kod_gruppy": "б", "Period_p_r": 9.5, "Vid_izluch": "бета", "Name_RN_Lat": "Sr-91", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'стронций-90', 'стронций-90', '{"AM": 90, "MOI": 1.0, "MZA": 10000.0, "MZUA": 100000.0, "D_val": 1, "Num_TM": 38, "Name_RN": "стронций-90", "UdA_GRO": 0.49, "UdA_TRO": 100.0, "ObA_GaRO": 2.7, "Kod_gruppy": "б", "Period_p_r": 29.1, "Vid_izluch": "бета", "Name_RN_Lat": "Sr-90", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'стронций-89', 'стронций-89', '{"AM": 89, "MOI": 1000.0, "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": 20, "Num_TM": 38, "Name_RN": "стронций-89", "UdA_GRO": 5.3, "UdA_TRO": 1000.0, "ObA_GaRO": 19.0, "Kod_gruppy": "б", "Period_p_r": 50.5, "Vid_izluch": "бета", "Name_RN_Lat": "Sr-89", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'стронций-87м', 'стронций-87м', '{"AM": 87, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.2, "Num_TM": 38, "Name_RN": "стронций-87м", "UdA_GRO": "", "UdA_TRO": 100.0, "ObA_GaRO": 4300.0, "Kod_gruppy": "б", "Period_p_r": 2.8, "Vid_izluch": "гамма", "Name_RN_Lat": "Sr-87m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'стронций-85м', 'стронций-85м', '{"AM": 85, "MOI": 100.0, "MZA": 10000000.0, "MZUA": 100000.0, "D_val": 0.1, "Num_TM": 38, "Name_RN": "стронций-85м", "UdA_GRO": "", "UdA_TRO": 100.0, "ObA_GaRO": 21000.0, "Kod_gruppy": "б", "Period_p_r": 1.16, "Vid_izluch": "гамма", "Name_RN_Lat": "Sr-85m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'стронций-85', 'стронций-85', '{"AM": 85, "MOI": 1.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.1, "Num_TM": 38, "Name_RN": "стронций-85", "UdA_GRO": 24.0, "UdA_TRO": 100.0, "ObA_GaRO": 160.0, "Kod_gruppy": "б", "Period_p_r": 64.8, "Vid_izluch": "гамма", "Name_RN_Lat": "Sr-85", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'криптон-88', 'криптон-88', '{"AM": 88, "MOI": "", "MZA": 1000000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 36, "Name_RN": "криптон-88", "UdA_GRO": "", "UdA_TRO": 100.0, "ObA_GaRO": 320.0, "Kod_gruppy": "б", "Period_p_r": 2.84, "Vid_izluch": "бета", "Name_RN_Lat": "Kr-88", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'стронций-82', 'стронций-82', '{"AM": 82, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.06, "Num_TM": 38, "Name_RN": "стронций-82", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 25.35, "Vid_izluch": "бета", "Name_RN_Lat": "Sr-82", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'бром-84', 'бром-84', '{"AM": 84, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 35, "Name_RN": "бром-84", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.53, "Vid_izluch": "бета", "Name_RN_Lat": "Br-84", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'бром-83', 'бром-83', '{"AM": 83, "MOI": "", "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 35, "Name_RN": "бром-83", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.39, "Vid_izluch": "бета", "Name_RN_Lat": "Br-83", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'бром-80м', 'бром-80м', '{"AM": 80, "MOI": "", "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 35, "Name_RN": "бром-80м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 4.42, "Vid_izluch": "гамма", "Name_RN_Lat": "Br-80m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'бром-80', 'бром-80', '{"AM": 80, "MOI": "", "MZA": 100000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 35, "Name_RN": "бром-80", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.29, "Vid_izluch": "бета", "Name_RN_Lat": "Br-80", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рубидий-86', 'рубидий-86', '{"AM": 86, "MOI": 100.0, "MZA": 100000.0, "MZUA": 100000.0, "D_val": 0.7, "Num_TM": 37, "Name_RN": "рубидий-86", "UdA_GRO": 4.9, "UdA_TRO": 100.0, "ObA_GaRO": 68.0, "Kod_gruppy": "б", "Period_p_r": 18.6, "Vid_izluch": "бета", "Name_RN_Lat": "Rb-86", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рубидий-84', 'рубидий-84', '{"AM": 84, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.07, "Num_TM": 37, "Name_RN": "рубидий-84", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 32.8, "Vid_izluch": "бета", "Name_RN_Lat": "Rb-84", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рубидий-83', 'рубидий-83', '{"AM": 83, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.1, "Num_TM": 37, "Name_RN": "рубидий-83", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 86.2, "Vid_izluch": "гамма", "Name_RN_Lat": "Rb-83", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'бром-75', 'бром-75', '{"AM": 75, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 35, "Name_RN": "бром-75", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.63, "Vid_izluch": "бета", "Name_RN_Lat": "Br-75", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'бром-74м', 'бром-74м', '{"AM": 74, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 35, "Name_RN": "бром-74м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.691, "Vid_izluch": "бета", "Name_RN_Lat": "Br-74m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'рубидий-81', 'рубидий-81', '{"AM": 81, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.1, "Num_TM": 37, "Name_RN": "рубидий-81", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 4.58, "Vid_izluch": "бета", "Name_RN_Lat": "Rb-81", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'бром-74', 'бром-74', '{"AM": 74, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 35, "Name_RN": "бром-74", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.422, "Vid_izluch": "бета", "Name_RN_Lat": "Br-74", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'селен-83', 'селен-83', '{"AM": 83, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 34, "Name_RN": "селен-83", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.375, "Vid_izluch": "бета", "Name_RN_Lat": "Se-83", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'селен-81м', 'селен-81м', '{"AM": 81, "MOI": "", "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 34, "Name_RN": "селен-81м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.954, "Vid_izluch": "гамма", "Name_RN_Lat": "Se-81m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'криптон-87', 'криптон-87', '{"AM": 87, "MOI": "", "MZA": 1000000000.0, "MZUA": 100000.0, "D_val": 0.09, "Num_TM": 36, "Name_RN": "криптон-87", "UdA_GRO": "", "UdA_TRO": 100.0, "ObA_GaRO": 800.0, "Kod_gruppy": "б", "Period_p_r": 76.3, "Vid_izluch": "бета", "Name_RN_Lat": "Kr-87", "Edinica_izmer_p_r": "мин", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'криптон-85м', 'криптон-85м', '{"AM": 85, "MOI": "", "MZA": 10000000000.0, "MZUA": 1000000.0, "D_val": 0.5, "Num_TM": 36, "Name_RN": "криптон-85м", "UdA_GRO": "", "UdA_TRO": 1000.0, "ObA_GaRO": 4600.0, "Kod_gruppy": "б", "Period_p_r": 4.48, "Vid_izluch": "бета", "Name_RN_Lat": "Kr-85m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'криптон-85', 'криптон-85', '{"AM": 85, "MOI": "", "MZA": 10000.0, "MZUA": 100000000.0, "D_val": 30, "Num_TM": 36, "Name_RN": "криптон-85", "UdA_GRO": "", "UdA_TRO": 100000.0, "ObA_GaRO": 120000.0, "Kod_gruppy": "б", "Period_p_r": 10.73, "Vid_izluch": "бета", "Name_RN_Lat": "Kr-85", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'криптон-81', 'криптон-81', '{"AM": 81, "MOI": "", "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": 30, "Num_TM": 36, "Name_RN": "криптон-81", "UdA_GRO": "", "UdA_TRO": 10000.0, "ObA_GaRO": 130000.0, "Kod_gruppy": "б", "Period_p_r": 229000.0, "Vid_izluch": "бета", "Name_RN_Lat": "Kr-81", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'селен-81', 'селен-81', '{"AM": 81, "MOI": "", "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 34, "Name_RN": "селен-81", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.308, "Vid_izluch": "бета", "Name_RN_Lat": "Se-81", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'селен-73м', 'селен-73м', '{"AM": 73, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 34, "Name_RN": "селен-73м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.65, "Vid_izluch": "гамма", "Name_RN_Lat": "Se-73m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'бром-82', 'бром-82', '{"AM": 82, "MOI": 1.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.03, "Num_TM": 35, "Name_RN": "бром-82", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 170.0, "Kod_gruppy": "б", "Period_p_r": 1.47, "Vid_izluch": "бета", "Name_RN_Lat": "Br-82", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'селен-73', 'селен-73', '{"AM": 73, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 34, "Name_RN": "селен-73", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 7.15, "Vid_izluch": "бета", "Name_RN_Lat": "Se-73", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'селен-70', 'селен-70', '{"AM": 70, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 34, "Name_RN": "селен-70", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.683, "Vid_izluch": "бета", "Name_RN_Lat": "Se-70", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'бром-77', 'бром-77', '{"AM": 77, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.2, "Num_TM": 35, "Name_RN": "бром-77", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.33, "Vid_izluch": "бета", "Name_RN_Lat": "Br-77", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'бром-76', 'бром-76', '{"AM": 76, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.03, "Num_TM": 35, "Name_RN": "бром-76", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 16.2, "Vid_izluch": "бета", "Name_RN_Lat": "Br-76", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'мышьяк-78', 'мышьяк-78', '{"AM": 78, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 33, "Name_RN": "мышьяк-78", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.51, "Vid_izluch": "бета", "Name_RN_Lat": "As-78", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'мышьяк-71', 'мышьяк-71', '{"AM": 71, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 33, "Name_RN": "мышьяк-71", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.7, "Vid_izluch": "бета", "Name_RN_Lat": "As-71", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'мышьяк-70', 'мышьяк-70', '{"AM": 70, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 33, "Name_RN": "мышьяк-70", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.876, "Vid_izluch": "бета", "Name_RN_Lat": "As-70", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'мышьяк-69', 'мышьяк-69', '{"AM": 69, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 33, "Name_RN": "мышьяк-69", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.253, "Vid_izluch": "бета", "Name_RN_Lat": "As-69", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'германий-78', 'германий-78', '{"AM": 78, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 32, "Name_RN": "германий-78", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.45, "Vid_izluch": "бета", "Name_RN_Lat": "Ge-78", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'германий-75', 'германий-75', '{"AM": 75, "MOI": "", "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 32, "Name_RN": "германий-75", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.38, "Vid_izluch": "бета", "Name_RN_Lat": "Ge-75", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'селен-79', 'селен-79', '{"AM": 79, "MOI": "", "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": 200, "Num_TM": 34, "Name_RN": "селен-79", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 327000.0, "Vid_izluch": "бета", "Name_RN_Lat": "Se-79", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'селен-75', 'селен-75', '{"AM": 75, "MOI": 1.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.2, "Num_TM": 34, "Name_RN": "селен-75", "UdA_GRO": 5.3, "UdA_TRO": 100.0, "ObA_GaRO": 77.0, "Kod_gruppy": "б", "Period_p_r": 120.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Se-75", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'германий-69', 'германий-69', '{"AM": 69, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 32, "Name_RN": "германий-69", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.63, "Vid_izluch": "бета", "Name_RN_Lat": "Ge-69", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'германий-67', 'германий-67', '{"AM": 67, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 32, "Name_RN": "германий-67", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.312, "Vid_izluch": "бета", "Name_RN_Lat": "Ge-67", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'германий-66', 'германий-66', '{"AM": 66, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 32, "Name_RN": "германий-66", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.27, "Vid_izluch": "бета", "Name_RN_Lat": "Ge-66", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'галлий-73', 'галлий-73', '{"AM": 73, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 31, "Name_RN": "галлий-73", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 4.91, "Vid_izluch": "бета", "Name_RN_Lat": "Ga-73", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'мышьяк-77', 'мышьяк-77', '{"AM": 77, "MOI": 1000.0, "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": 8, "Num_TM": 33, "Name_RN": "мышьяк-77", "UdA_GRO": 34.0, "UdA_TRO": 1000.0, "ObA_GaRO": 270.0, "Kod_gruppy": "б", "Period_p_r": 1.62, "Vid_izluch": "бета", "Name_RN_Lat": "As-77", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'мышьяк-76', 'мышьяк-76', '{"AM": 76, "MOI": 10.0, "MZA": 100000.0, "MZUA": 100000.0, "D_val": 0.2, "Num_TM": 33, "Name_RN": "мышьяк-76", "UdA_GRO": 8.6, "UdA_TRO": 100.0, "ObA_GaRO": 110.0, "Kod_gruppy": "б", "Period_p_r": 1.1, "Vid_izluch": "бета", "Name_RN_Lat": "As-76", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'мышьяк-74', 'мышьяк-74', '{"AM": 74, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.09, "Num_TM": 33, "Name_RN": "мышьяк-74", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 53.0, "Kod_gruppy": "б", "Period_p_r": 17.8, "Vid_izluch": "бета", "Name_RN_Lat": "As-74", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'мышьяк-73', 'мышьяк-73', '{"AM": 73, "MOI": 1000.0, "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": 40, "Num_TM": 33, "Name_RN": "мышьяк-73", "UdA_GRO": 53.0, "UdA_TRO": 1000.0, "ObA_GaRO": 110.0, "Kod_gruppy": "б", "Period_p_r": 80.3, "Vid_izluch": "гамма", "Name_RN_Lat": "As-73", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'мышьяк-72', 'мышьяк-72', '{"AM": 72, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.04, "Num_TM": 33, "Name_RN": "мышьяк-72", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.08, "Vid_izluch": "бета", "Name_RN_Lat": "As-72", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'галлий-70', 'галлий-70', '{"AM": 70, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 31, "Name_RN": "галлий-70", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.353, "Vid_izluch": "бета", "Name_RN_Lat": "Ga-70", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'галлий-66', 'галлий-66', '{"AM": 66, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 31, "Name_RN": "галлий-66", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 9.4, "Vid_izluch": "бета", "Name_RN_Lat": "Ga-66", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'галлий-65', 'галлий-65', '{"AM": 65, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 31, "Name_RN": "галлий-65", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.253, "Vid_izluch": "бета", "Name_RN_Lat": "Ga-65", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цинк-72', 'цинк-72', '{"AM": 72, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 30, "Name_RN": "цинк-72", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.94, "Vid_izluch": "бета", "Name_RN_Lat": "Zn-72", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'германий-77', 'германий-77', '{"AM": 77, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.06, "Num_TM": 32, "Name_RN": "германий-77", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 11.3, "Vid_izluch": "бета", "Name_RN_Lat": "Ge-77", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100100', '4100100 Банки', '{"2": "Банки", "m": 4100100}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цинк-71м', 'цинк-71м', '{"AM": 71, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 30, "Name_RN": "цинк-71м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.92, "Vid_izluch": "бета", "Name_RN_Lat": "Zn-71m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'германий-71', 'германий-71', '{"AM": 71, "MOI": 10000.0, "MZA": 100000000.0, "MZUA": 10000000.0, "D_val": 1000, "Num_TM": 32, "Name_RN": "германий-71", "UdA_GRO": 1140.0, "UdA_TRO": 10000.0, "ObA_GaRO": 6100.0, "Kod_gruppy": "б", "Period_p_r": 11.8, "Vid_izluch": "гамма", "Name_RN_Lat": "Ge-71", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цинк-63', 'цинк-63', '{"AM": 63, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 30, "Name_RN": "цинк-63", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.635, "Vid_izluch": "бета", "Name_RN_Lat": "Zn-63", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'германий-68', 'германий-68', '{"AM": 68, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.07, "Num_TM": 32, "Name_RN": "германий-68", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 270.93, "Vid_izluch": "гамма", "Name_RN_Lat": "Ge-68", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цинк-62', 'цинк-62', '{"AM": 62, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 30, "Name_RN": "цинк-62", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 9.26, "Vid_izluch": "бета", "Name_RN_Lat": "Zn-62", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'медь-61', 'медь-61', '{"AM": 61, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 29, "Name_RN": "медь-61", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.41, "Vid_izluch": "бета", "Name_RN_Lat": "Cu-61", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'медь-60', 'медь-60', '{"AM": 60, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 29, "Name_RN": "медь-60", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.387, "Vid_izluch": "бета", "Name_RN_Lat": "Cu-60", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'галлий-72', 'галлий-72', '{"AM": 72, "MOI": 10.0, "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.03, "Num_TM": 31, "Name_RN": "галлий-72", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 150.0, "Kod_gruppy": "б", "Period_p_r": 14.1, "Vid_izluch": "бета", "Name_RN_Lat": "Ga-72", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'никель-66', 'никель-66', '{"AM": 66, "MOI": "", "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": "", "Num_TM": 28, "Name_RN": "никель-66", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.27, "Vid_izluch": "бета", "Name_RN_Lat": "Ni-66", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'галлий-68', 'галлий-68', '{"AM": 68, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.07, "Num_TM": 31, "Name_RN": "галлий-68", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.1285, "Vid_izluch": "бета", "Name_RN_Lat": "Ga-68", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'галлий-67', 'галлий-67', '{"AM": 67, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.5, "Num_TM": 31, "Name_RN": "галлий-67", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.26, "Vid_izluch": "гамма", "Name_RN_Lat": "Ga-67", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'никель-57', 'никель-57', '{"AM": 57, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 28, "Name_RN": "никель-57", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.5, "Vid_izluch": "бета", "Name_RN_Lat": "Ni-57", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'никель-56', 'никель-56', '{"AM": 56, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 28, "Name_RN": "никель-56", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 6.1, "Vid_izluch": "гамма", "Name_RN_Lat": "Ni-56", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кобальт-62м', 'кобальт-62м', '{"AM": 62, "MOI": 10.0, "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 27, "Name_RN": "кобальт-62м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.232, "Vid_izluch": "бета", "Name_RN_Lat": "Co-62m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кобальт-61', 'кобальт-61', '{"AM": 61, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 27, "Name_RN": "кобальт-61", "UdA_GRO": "", "UdA_TRO": 100.0, "ObA_GaRO": 1900.0, "Kod_gruppy": "б", "Period_p_r": 1.65, "Vid_izluch": "бета", "Name_RN_Lat": "Co-61", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цинк-69м', 'цинк-69м', '{"AM": 69, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.2, "Num_TM": 30, "Name_RN": "цинк-69м", "UdA_GRO": 41.0, "UdA_TRO": 100.0, "ObA_GaRO": 350.0, "Kod_gruppy": "б", "Period_p_r": 13.8, "Vid_izluch": "гамма", "Name_RN_Lat": "Zn-69m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цинк-69', 'цинк-69', '{"AM": 69, "MOI": 1000.0, "MZA": 1000000.0, "MZUA": 10000000.0, "D_val": 30, "Num_TM": 30, "Name_RN": "цинк-69", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.95, "Vid_izluch": "бета", "Name_RN_Lat": "Zn-69", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цинк-65', 'цинк-65', '{"AM": 65, "MOI": 0.1, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.1, "Num_TM": 30, "Name_RN": "цинк-65", "UdA_GRO": 3.5, "UdA_TRO": 10.0, "ObA_GaRO": 72.0, "Kod_gruppy": "б", "Period_p_r": 244.0, "Vid_izluch": "бета", "Name_RN_Lat": "Zn-65", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кобальт-60м', 'кобальт-60м', '{"AM": 60, "MOI": 1000.0, "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 27, "Name_RN": "кобальт-60м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.174, "Vid_izluch": "гамма", "Name_RN_Lat": "Co-60m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'марганец-53', 'марганец-53', '{"AM": 53, "MOI": 100.0, "MZA": 1000000000.0, "MZUA": 10000000.0, "D_val": "неогран.", "Num_TM": 25, "Name_RN": "марганец-53", "UdA_GRO": 460.0, "UdA_TRO": 10000.0, "ObA_GaRO": 1500.0, "Kod_gruppy": "б", "Period_p_r": 3700000.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Mn-53", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'медь-67', 'медь-67', '{"AM": 67, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.7, "Num_TM": 29, "Name_RN": "медь-67", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.58, "Vid_izluch": "бета", "Name_RN_Lat": "Cu-67", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'медь-64', 'медь-64', '{"AM": 64, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.3, "Num_TM": 29, "Name_RN": "медь-64", "UdA_GRO": "", "UdA_TRO": 100.0, "ObA_GaRO": 920.0, "Kod_gruppy": "б", "Period_p_r": 12.7, "Vid_izluch": "бета", "Name_RN_Lat": "Cu-64", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'марганец-52м', 'марганец-52м', '{"AM": 52, "MOI": 10.0, "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 25, "Name_RN": "марганец-52м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.352, "Vid_izluch": "бета", "Name_RN_Lat": "Mn-52m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'марганец-51', 'марганец-51', '{"AM": 51, "MOI": 10.0, "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 25, "Name_RN": "марганец-51", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.77, "Vid_izluch": "бета", "Name_RN_Lat": "Mn-51", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'хром-49', 'хром-49', '{"AM": 49, "MOI": "", "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": "", "Num_TM": 24, "Name_RN": "хром-49", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.702, "Vid_izluch": "бета", "Name_RN_Lat": "Cr-49", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'никель-65', 'никель-65', '{"AM": 65, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.1, "Num_TM": 28, "Name_RN": "никель-65", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 1000.0, "Kod_gruppy": "б", "Period_p_r": 2.52, "Vid_izluch": "бета", "Name_RN_Lat": "Ni-65", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'никель-63', 'никель-63', '{"AM": 63, "MOI": 100.0, "MZA": 100000000.0, "MZUA": 100000000.0, "D_val": 60, "Num_TM": 28, "Name_RN": "никель-63", "UdA_GRO": 91.0, "UdA_TRO": 100000.0, "ObA_GaRO": 260.0, "Kod_gruppy": "б", "Period_p_r": 101.2, "Vid_izluch": "бета", "Name_RN_Lat": "Ni-63", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'никель-59', 'никель-59', '{"AM": 59, "MOI": 100.0, "MZA": 100000000.0, "MZUA": 10000000.0, "D_val": 1000, "Num_TM": 28, "Name_RN": "никель-59", "UdA_GRO": 220.0, "UdA_TRO": 10000.0, "ObA_GaRO": 850.0, "Kod_gruppy": "б", "Period_p_r": 76000.0, "Vid_izluch": "бета", "Name_RN_Lat": "Ni-59", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'хром-48', 'хром-48', '{"AM": 48, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 24, "Name_RN": "хром-48", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 21.56, "Vid_izluch": "бета", "Name_RN_Lat": "Cr-48", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ванадий-47', 'ванадий-47', '{"AM": 47, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 23, "Name_RN": "ванадий-47", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.543, "Vid_izluch": "бета", "Name_RN_Lat": "V-47", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'титан-45', 'титан-45', '{"AM": 45, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 22, "Name_RN": "титан-45", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.08, "Vid_izluch": "бета", "Name_RN_Lat": "Ti-45", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'скандий-49', 'скандий-49', '{"AM": 49, "MOI": "", "MZA": 100000.0, "MZUA": 1000000.0, "D_val": "", "Num_TM": 21, "Name_RN": "скандий-49", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.956, "Vid_izluch": "бета", "Name_RN_Lat": "Sc-49", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'скандий-44м', 'скандий-44м', '{"AM": 44, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 21, "Name_RN": "скандий-44м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.44, "Vid_izluch": "гамма", "Name_RN_Lat": "Sc-44m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кобальт-60', 'кобальт-60', '{"AM": 60, "MOI": 0.1, "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.03, "Num_TM": 27, "Name_RN": "кобальт-60", "UdA_GRO": 4.0, "UdA_TRO": 10.0, "ObA_GaRO": 11.0, "Kod_gruppy": "б", "Period_p_r": 5.27, "Vid_izluch": "бета", "Name_RN_Lat": "Co-60", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кобальт-58м', 'кобальт-58м', '{"AM": 58, "MOI": 10000.0, "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": 0.07, "Num_TM": 27, "Name_RN": "кобальт-58м", "UdA_GRO": 570.0, "UdA_TRO": 10000.0, "ObA_GaRO": 6900.0, "Kod_gruppy": "б", "Period_p_r": 9.15, "Vid_izluch": "гамма", "Name_RN_Lat": "Co-58m", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кобальт-58', 'кобальт-58', '{"AM": 58, "MOI": 1.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.07, "Num_TM": 27, "Name_RN": "кобальт-58", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 68.0, "Kod_gruppy": "б", "Period_p_r": 70.8, "Vid_izluch": "бета", "Name_RN_Lat": "Co-58", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кобальт-57', 'кобальт-57', '{"AM": 57, "MOI": 1.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.7, "Num_TM": 27, "Name_RN": "кобальт-57", "UdA_GRO": 65.0, "UdA_TRO": 100.0, "ObA_GaRO": 200.0, "Kod_gruppy": "б", "Period_p_r": 271.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Co-57", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кобальт-56', 'кобальт-56', '{"AM": 56, "MOI": 0.1, "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.02, "Num_TM": 27, "Name_RN": "кобальт-56", "UdA_GRO": 5.5, "UdA_TRO": 10.0, "ObA_GaRO": 24.0, "Kod_gruppy": "б", "Period_p_r": 77.236, "Vid_izluch": "бета", "Name_RN_Lat": "Co-56", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кобальт-55', 'кобальт-55', '{"AM": 55, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.03, "Num_TM": 27, "Name_RN": "кобальт-55", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 160.0, "Kod_gruppy": "б", "Period_p_r": 17.5, "Vid_izluch": "бета", "Name_RN_Lat": "Co-55", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'железо-59', 'железо-59', '{"AM": 59, "MOI": 1.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.06, "Num_TM": 26, "Name_RN": "железо-59", "UdA_GRO": 7.6, "UdA_TRO": 10.0, "ObA_GaRO": 30.0, "Kod_gruppy": "б", "Period_p_r": 44.5, "Vid_izluch": "бета", "Name_RN_Lat": "Fe-59", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'железо-55', 'железо-55', '{"AM": 55, "MOI": 1000.0, "MZA": 1000000.0, "MZUA": 10000000.0, "D_val": 800, "Num_TM": 26, "Name_RN": "железо-55", "UdA_GRO": 42.0, "UdA_TRO": 10000.0, "ObA_GaRO": 310.0, "Kod_gruppy": "б", "Period_p_r": 2.7, "Vid_izluch": "гамма", "Name_RN_Lat": "Fe-55", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'железо-52', 'железо-52', '{"AM": 52, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.02, "Num_TM": 26, "Name_RN": "железо-52", "UdA_GRO": 9.7, "UdA_TRO": 10.0, "ObA_GaRO": 120.0, "Kod_gruppy": "б", "Period_p_r": 8.28, "Vid_izluch": "бета", "Name_RN_Lat": "Fe-52", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'марганец-56', 'марганец-56', '{"AM": 56, "MOI": 10.0, "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.04, "Num_TM": 25, "Name_RN": "марганец-56", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 680.0, "Kod_gruppy": "б", "Period_p_r": 2.58, "Vid_izluch": "бета", "Name_RN_Lat": "Mn-56", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'марганец-54', 'марганец-54', '{"AM": 54, "MOI": 0.1, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.08, "Num_TM": 25, "Name_RN": "марганец-54", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 72.0, "Kod_gruppy": "б", "Period_p_r": 312.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Mn-54", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'скандий-43', 'скандий-43', '{"AM": 43, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 21, "Name_RN": "скандий-43", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.89, "Vid_izluch": "бета", "Name_RN_Lat": "Sc-43", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кальций-41', 'кальций-41', '{"AM": 41, "MOI": "", "MZA": 10000000.0, "MZUA": 100000000.0, "D_val": "неогран.", "Num_TM": 20, "Name_RN": "кальций-41", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 99400.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Ca-41", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'марганец-52', 'марганец-52', '{"AM": 52, "MOI": 1.0, "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.02, "Num_TM": 25, "Name_RN": "марганец-52", "UdA_GRO": 7.6, "UdA_TRO": 10.0, "ObA_GaRO": 77.0, "Kod_gruppy": "б", "Period_p_r": 5.59, "Vid_izluch": "бета", "Name_RN_Lat": "Mn-52", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'хром-51', 'хром-51', '{"AM": 51, "MOI": 100.0, "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": 2, "Num_TM": 24, "Name_RN": "хром-51", "UdA_GRO": 360.0, "UdA_TRO": 1000.0, "ObA_GaRO": 2500.0, "Kod_gruppy": "б", "Period_p_r": 27.7, "Vid_izluch": "гамма", "Name_RN_Lat": "Cr-51", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2500000', '2500000 ДРУГИЕ ГОСУДАРСТВЕННЫЕ ОРГАНЫ СУБЪЕКТОВ РФ', '{"2": "ДРУГИЕ ГОСУДАРСТВЕННЫЕ ОРГАНЫ СУБЪЕКТОВ РФ", "m": 2500000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'калий-44', 'калий-44', '{"AM": 44, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 19, "Name_RN": "калий-44", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.369, "Vid_izluch": "бета", "Name_RN_Lat": "K-44", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'калий-40', 'калий-40', '{"AM": 40, "MOI": "", "MZA": 1000000.0, "MZUA": 100000.0, "D_val": "неогран.", "Num_TM": 19, "Name_RN": "калий-40", "UdA_GRO": 2.2, "UdA_TRO": 100.0, "ObA_GaRO": 31.0, "Kod_gruppy": "б", "Period_p_r": 1248000000.0, "Vid_izluch": "бета", "Name_RN_Lat": "K-40", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ванадий-49', 'ванадий-49', '{"AM": 49, "MOI": "", "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": 2000, "Num_TM": 23, "Name_RN": "ванадий-49", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 330.0, "Vid_izluch": "гамма", "Name_RN_Lat": "V-49", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'ванадий-48', 'ванадий-48', '{"AM": 48, "MOI": 1.0, "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.02, "Num_TM": 23, "Name_RN": "ванадий-48", "UdA_GRO": 6.9, "UdA_TRO": 10.0, "ObA_GaRO": 45.0, "Kod_gruppy": "б", "Period_p_r": 16.2, "Vid_izluch": "бета", "Name_RN_Lat": "V-48", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'аргон-42', 'аргон-42', '{"AM": 42, "MOI": "", "MZA": "", "MZUA": "", "D_val": "", "Num_TM": 18, "Name_RN": "аргон-42", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 32.98, "Vid_izluch": "бета", "Name_RN_Lat": "Ar-42", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'аргон-37', 'аргон-37', '{"AM": 37, "MOI": "", "MZA": 100000000.0, "MZUA": 1000000000.0, "D_val": "неогран.", "Num_TM": 18, "Name_RN": "аргон-37", "UdA_GRO": "", "UdA_TRO": 1000000.0, "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 35.0, "Vid_izluch": "гамма", "Name_RN_Lat": "Ar-37", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'титан-44', 'титан-44', '{"AM": 44, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.03, "Num_TM": 22, "Name_RN": "титан-44", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 59.1, "Vid_izluch": "гамма", "Name_RN_Lat": "Ti-44", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'хлор-39', 'хлор-39', '{"AM": 39, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": "", "Num_TM": 17, "Name_RN": "хлор-39", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.927, "Vid_izluch": "бета", "Name_RN_Lat": "Cl-39", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'скандий-48', 'скандий-48', '{"AM": 48, "MOI": 1.0, "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.02, "Num_TM": 21, "Name_RN": "скандий-48", "UdA_GRO": 8.1, "UdA_TRO": 10.0, "ObA_GaRO": 89.0, "Kod_gruppy": "б", "Period_p_r": 1.82, "Vid_izluch": "бета", "Name_RN_Lat": "Sc-48", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'скандий-47', 'скандий-47', '{"AM": 47, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.7, "Num_TM": 21, "Name_RN": "скандий-47", "UdA_GRO": 25.0, "UdA_TRO": 100.0, "ObA_GaRO": 150.0, "Kod_gruppy": "б", "Period_p_r": 3.35, "Vid_izluch": "бета", "Name_RN_Lat": "Sc-47", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'скандий-46', 'скандий-46', '{"AM": 46, "MOI": 0.1, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.03, "Num_TM": 21, "Name_RN": "скандий-46", "UdA_GRO": 9.1, "UdA_TRO": 10.0, "ObA_GaRO": 16.0, "Kod_gruppy": "б", "Period_p_r": 83.8, "Vid_izluch": "бета", "Name_RN_Lat": "Sc-46", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'неон-24', 'неон-24', '{"AM": 24, "MOI": "", "MZA": "", "MZUA": "", "D_val": "", "Num_TM": 10, "Name_RN": "неон-24", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 3.38, "Vid_izluch": "бета", "Name_RN_Lat": "Ne-24", "Edinica_izmer_p_r": "мин", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'фтор-17', 'фтор-17', '{"AM": 17, "MOI": "", "MZA": "", "MZUA": "", "D_val": "", "Num_TM": 9, "Name_RN": "фтор-17", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.08, "Vid_izluch": "бета", "Name_RN_Lat": "F-17", "Edinica_izmer_p_r": "мин", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кальций-47', 'кальций-47', '{"AM": 47, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.06, "Num_TM": 20, "Name_RN": "кальций-47", "UdA_GRO": 86.0, "UdA_TRO": 10.0, "ObA_GaRO": 53.0, "Kod_gruppy": "б", "Period_p_r": 4.53, "Vid_izluch": "бета", "Name_RN_Lat": "Ca-47", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кальций-45', 'кальций-45', '{"AM": 45, "MOI": 100.0, "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": 100, "Num_TM": 20, "Name_RN": "кальций-45", "UdA_GRO": 19.0, "UdA_TRO": 10000.0, "ObA_GaRO": 30.0, "Kod_gruppy": "б", "Period_p_r": 163.0, "Vid_izluch": "бета", "Name_RN_Lat": "Ca-45", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кислород-15', 'кислород-15', '{"AM": 15, "MOI": "", "MZA": 1000000000.0, "MZUA": 100000.0, "D_val": "", "Num_TM": 8, "Name_RN": "кислород-15", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.03, "Vid_izluch": "бета", "Name_RN_Lat": "O-15", "Edinica_izmer_p_r": "мин", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кислород-14', 'кислород-14', '{"AM": 14, "MOI": "", "MZA": "", "MZUA": "", "D_val": "", "Num_TM": 8, "Name_RN": "кислород-14", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1.18, "Vid_izluch": "бета", "Name_RN_Lat": "O-14", "Edinica_izmer_p_r": "мин", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'углерод-13', 'углерод-13', '{"AM": 13, "MOI": "", "MZA": 0.0, "MZUA": 0.0, "D_val": 0, "Num_TM": 6, "Name_RN": "углерод-13", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": "", "Vid_izluch": "бета", "Name_RN_Lat": "C-13", "Edinica_izmer_p_r": "", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'калий-43', 'калий-43', '{"AM": 43, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.07, "Num_TM": 19, "Name_RN": "калий-43", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 540.0, "Kod_gruppy": "б", "Period_p_r": 22.6, "Vid_izluch": "бета", "Name_RN_Lat": "K-43", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'калий-42', 'калий-42', '{"AM": 42, "MOI": 100.0, "MZA": 1000000.0, "MZUA": 100000.0, "D_val": 0.2, "Num_TM": 19, "Name_RN": "калий-42", "UdA_GRO": 31.0, "UdA_TRO": 100.0, "ObA_GaRO": 520.0, "Kod_gruppy": "б", "Period_p_r": 12.4, "Vid_izluch": "бета", "Name_RN_Lat": "K-42", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'палладий-234', 'палладий-234', '{"AM": 0, "MOI": "", "MZA": 0.0, "MZUA": 0.0, "D_val": 0, "Num_TM": 0, "Name_RN": "палладий-234", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 6.0, "Vid_izluch": "", "Name_RN_Lat": "Pd-234", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": ""}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'уран обедненный', 'уран обедненный', '{"AM": 0, "MOI": "", "MZA": "", "MZUA": "", "D_val": "неогран.", "Num_TM": 0, "Name_RN": "уран обедненный", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "а", "Period_p_r": 4470000000.0, "Vid_izluch": "альфа", "Name_RN_Lat": "U-ОБЕД", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": ""}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'аргон-41', 'аргон-41', '{"AM": 41, "MOI": "", "MZA": 1000000000.0, "MZUA": 100000.0, "D_val": 0.05, "Num_TM": 18, "Name_RN": "аргон-41", "UdA_GRO": "", "UdA_TRO": 100.0, "ObA_GaRO": 510.0, "Kod_gruppy": "б", "Period_p_r": 1.827, "Vid_izluch": "бета", "Name_RN_Lat": "Ar-41", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'аргон-39', 'аргон-39', '{"AM": 39, "MOI": "", "MZA": 10000.0, "MZUA": 10000000000.0, "D_val": 300, "Num_TM": 18, "Name_RN": "аргон-39", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 269.0, "Vid_izluch": "бета", "Name_RN_Lat": "Ar-39", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сумма бета-активных радионуклидов', 'сумма бета-активных радионуклидов', '{"AM": 0, "MOI": "", "MZA": "", "MZUA": "", "D_val": "", "Num_TM": 0, "Name_RN": "сумма бета-активных радионуклидов", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": "", "Vid_izluch": "", "Name_RN_Lat": "", "Edinica_izmer_p_r": "", "Osn_vid_raspad_text": ""}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сумма ирг', 'сумма ирг', '{"AM": 0, "MOI": "", "MZA": "", "MZUA": "", "D_val": "", "Num_TM": 0, "Name_RN": "сумма ирг", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": "", "Vid_izluch": "бета", "Name_RN_Lat": "", "Edinica_izmer_p_r": "", "Osn_vid_raspad_text": ""}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'хлор-38', 'хлор-38', '{"AM": 38, "MOI": 10.0, "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.05, "Num_TM": 17, "Name_RN": "хлор-38", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.62, "Vid_izluch": "бета", "Name_RN_Lat": "Cl-38", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'хлор-36', 'хлор-36', '{"AM": 36, "MOI": 1.0, "MZA": 1000000.0, "MZUA": 10000000.0, "D_val": 20, "Num_TM": 17, "Name_RN": "хлор-36", "UdA_GRO": 15.0, "UdA_TRO": 10000.0, "ObA_GaRO": 16.0, "Kod_gruppy": "б", "Period_p_r": 301000.0, "Vid_izluch": "бета", "Name_RN_Lat": "Cl-36", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сера-35', 'сера-35', '{"AM": 35, "MOI": 100.0, "MZA": 100000000.0, "MZUA": 100000000.0, "D_val": 60, "Num_TM": 16, "Name_RN": "сера-35", "UdA_GRO": 17.8, "UdA_TRO": 100000.0, "ObA_GaRO": 76.0, "Kod_gruppy": "б", "Period_p_r": 87.4, "Vid_izluch": "бета", "Name_RN_Lat": "S-35", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'фосфор-33', 'фосфор-33', '{"AM": 33, "MOI": 1000.0, "MZA": 100000000.0, "MZUA": 100000000.0, "D_val": 200, "Num_TM": 15, "Name_RN": "фосфор-33", "UdA_GRO": 57.0, "UdA_TRO": 100000.0, "ObA_GaRO": 72.0, "Kod_gruppy": "б", "Period_p_r": 25.4, "Vid_izluch": "бета", "Name_RN_Lat": "P-33", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'фосфор-32', 'фосфор-32', '{"AM": 32, "MOI": 1000.0, "MZA": 100000.0, "MZUA": 1000000.0, "D_val": 10, "Num_TM": 15, "Name_RN": "фосфор-32", "UdA_GRO": 5.7, "UdA_TRO": 1000.0, "ObA_GaRO": 34.0, "Kod_gruppy": "б", "Period_p_r": 14.3, "Vid_izluch": "бета", "Name_RN_Lat": "P-32", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кремний-32', 'кремний-32', '{"AM": 32, "MOI": "", "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": 7, "Num_TM": 14, "Name_RN": "кремний-32", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 153.0, "Vid_izluch": "бета", "Name_RN_Lat": "Si-32", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2400000', '2400000 ОРГАНЫ СУДЕБНОЙ ВЛАСТИ СУБЪЕКТОВ РФ', '{"2": "ОРГАНЫ СУДЕБНОЙ ВЛАСТИ СУБЪЕКТОВ РФ", "m": 2400000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кремний-31', 'кремний-31', '{"AM": 31, "MOI": 1000.0, "MZA": 1000000.0, "MZUA": 1000000.0, "D_val": 10, "Num_TM": 14, "Name_RN": "кремний-31", "UdA_GRO": 85.0, "UdA_TRO": 1000.0, "ObA_GaRO": 1100.0, "Kod_gruppy": "б", "Period_p_r": 2.62, "Vid_izluch": "бета", "Name_RN_Lat": "Si-31", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'алюминий-26', 'алюминий-26', '{"AM": 26, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.03, "Num_TM": 13, "Name_RN": "алюминий-26", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 717000.0, "Vid_izluch": "бета", "Name_RN_Lat": "Al-26", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'магний-28', 'магний-28', '{"AM": 28, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.02, "Num_TM": 12, "Name_RN": "магний-28", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 20.92, "Vid_izluch": "бета", "Name_RN_Lat": "Mg-28", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'натрий-24', 'натрий-24', '{"AM": 24, "MOI": "", "MZA": 100000.0, "MZUA": 10000.0, "D_val": 0.02, "Num_TM": 11, "Name_RN": "натрий-24", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 290.0, "Kod_gruppy": "б", "Period_p_r": 15.0, "Vid_izluch": "бета", "Name_RN_Lat": "Na-24", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'натрий-22', 'натрий-22', '{"AM": 22, "MOI": 0.1, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.03, "Num_TM": 11, "Name_RN": "натрий-22", "UdA_GRO": 4.3, "UdA_TRO": 10.0, "ObA_GaRO": 72.0, "Kod_gruppy": "б", "Period_p_r": 2.6, "Vid_izluch": "бета", "Name_RN_Lat": "Na-22", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'джн (смесь долгоживущих нуклидов)', 'джн (смесь долгоживущих нуклидов)', '{"AM": 0, "MOI": "", "MZA": "", "MZUA": "", "D_val": "", "Num_TM": 0, "Name_RN": "джн (смесь долгоживущих нуклидов)", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "а", "Period_p_r": "", "Vid_izluch": "альфа", "Name_RN_Lat": "", "Edinica_izmer_p_r": "", "Osn_vid_raspad_text": ""}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'фтор-18', 'фтор-18', '{"AM": 18, "MOI": 10.0, "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.06, "Num_TM": 9, "Name_RN": "фтор-18", "UdA_GRO": "", "UdA_TRO": 10.0, "ObA_GaRO": 1600.0, "Kod_gruppy": "б", "Period_p_r": 1.83, "Vid_izluch": "бета", "Name_RN_Lat": "F-18", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кжн (смесь короткоживущих нуклидов)', 'кжн (смесь короткоживущих нуклидов)', '{"AM": 0, "MOI": "", "MZA": "", "MZUA": "", "D_val": "", "Num_TM": 0, "Name_RN": "кжн (смесь короткоживущих нуклидов)", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": "", "Vid_izluch": "бета", "Name_RN_Lat": "", "Edinica_izmer_p_r": "", "Osn_vid_raspad_text": ""}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сумма альфа-активных радионуклидов', 'сумма альфа-активных радионуклидов', '{"AM": 0, "MOI": "", "MZA": "", "MZUA": "", "D_val": "", "Num_TM": 0, "Name_RN": "сумма альфа-активных радионуклидов", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "а", "Period_p_r": "", "Vid_izluch": "альфа", "Name_RN_Lat": "", "Edinica_izmer_p_r": "", "Osn_vid_raspad_text": ""}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'сумма нуклидов урана', 'сумма нуклидов урана', '{"AM": 0, "MOI": "", "MZA": "", "MZUA": "", "D_val": "", "Num_TM": 0, "Name_RN": "сумма нуклидов урана", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "а", "Period_p_r": "", "Vid_izluch": "альфа", "Name_RN_Lat": "U", "Edinica_izmer_p_r": "", "Osn_vid_raspad_text": ""}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'азот-13', 'азот-13', '{"AM": 13, "MOI": "", "MZA": 1000000000.0, "MZUA": 100000.0, "D_val": 0.06, "Num_TM": 7, "Name_RN": "азот-13", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 9.96, "Vid_izluch": "бета", "Name_RN_Lat": "N-13", "Edinica_izmer_p_r": "мин", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'углерод-14', 'углерод-14', '{"AM": 14, "MOI": 1.0, "MZA": 10000000.0, "MZUA": 10000000.0, "D_val": 50, "Num_TM": 6, "Name_RN": "углерод-14", "UdA_GRO": 24.0, "UdA_TRO": 10000.0, "ObA_GaRO": 55.0, "Kod_gruppy": "б", "Period_p_r": 5730.0, "Vid_izluch": "бета", "Name_RN_Lat": "C-14", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'углерод-11', 'углерод-11', '{"AM": 11, "MOI": "", "MZA": 1000000.0, "MZUA": 10000.0, "D_val": 0.06, "Num_TM": 6, "Name_RN": "углерод-11", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 0.34, "Vid_izluch": "бета", "Name_RN_Lat": "C-11", "Edinica_izmer_p_r": "час", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'бериллий-10', 'бериллий-10', '{"AM": 10, "MOI": "", "MZA": 1000000.0, "MZUA": 10000000.0, "D_val": 30, "Num_TM": 4, "Name_RN": "бериллий-10", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 1510000.0, "Vid_izluch": "бета", "Name_RN_Lat": "Be-10", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'бериллий-7', 'бериллий-7', '{"AM": 7, "MOI": 10.0, "MZA": 10000000.0, "MZUA": 1000000.0, "D_val": 1, "Num_TM": 4, "Name_RN": "бериллий-7", "UdA_GRO": 490.0, "UdA_TRO": 1000.0, "ObA_GaRO": 2000.0, "Kod_gruppy": "б", "Period_p_r": 53.3, "Vid_izluch": "гамма", "Name_RN_Lat": "Be-7", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тритий', 'тритий', '{"AM": 3, "MOI": 100.0, "MZA": 1000000000.0, "MZUA": 1000000000.0, "D_val": 2000, "Num_TM": 1, "Name_RN": "тритий", "UdA_GRO": 1000.0, "UdA_TRO": 1000000.0, "ObA_GaRO": 1900.0, "Kod_gruppy": "Тритий", "Period_p_r": 12.3, "Vid_izluch": "бета", "Name_RN_Lat": "H-3", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('112_voc', '22', '22', '{"g1_cod": "22", "g1_types": "ОРИ, кроме отдельных изделий, содержащие жидкие РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('112_voc', '21', '21', '{"g1_cod": "21", "g1_types": "ОРИ, кроме отдельных изделий, содержащие твёрдые РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('112_voc', '13', '13', '{"g1_cod": "13", "g1_types": "ОРИ в виде отдельных изделий, содержащие газообразные РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('112_voc', '12', '12', '{"g1_cod": "12", "g1_types": "ОРИ в виде отдельных изделий, содержащие жидкие РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('112_voc', '11', '11', '{"g1_cod": "11", "g1_types": "ОРИ в виде отдельных изделий, содержащие твердые РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '61', '61 Собственность государственных корпораций', '{"2": "Собственность государственных корпораций", "b": 61}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '54', '54 Собственность религиозных объединений', '{"2": "Собственность религиозных объединений", "b": 54}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '53', '53 Собственность общественных объединений', '{"2": "Собственность общественных объединений", "b": 53}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '52', '52 Собственность профессиональных союзов', '{"2": "Собственность профессиональных союзов", "b": 52}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '51', '51 Собственность политических общественных объединений', '{"2": "Собственность политических общественных объединений", "b": 51}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '50', '50 Собственность благотворительных организаций', '{"2": "Собственность благотворительных организаций", "b": 50}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '49', '49 Иная смешанная российская собственность', '{"2": "Иная смешанная российская собственность", "b": 49}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '43', '43 Смешанная российская собственность с долями федеральной собственности и собственности субъектов Российской Федерации', '{"2": "Смешанная российская собственность с долями федеральной собственности и собственности субъектов Российской Федерации", "b": 43}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '42', '42 Смешанная российская собственность с долей собственности субъектов Российской Федерации', '{"2": "Смешанная российская собственность с долей собственности субъектов Российской Федерации", "b": 42}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '41', '41 Смешанная российская собственность с долей федеральной собственности', '{"2": "Смешанная российская собственность с долей федеральной собственности", "b": 41}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '40', '40   СМЕШАННАЯ РОССИЙСКАЯ СОБСТВЕННОСТЬ С ДОЛЕЙ ГОСУДАРСТВЕННОЙ СОБСТВЕННОСТИ  (41 + 42 + 43)', '{"2": "  СМЕШАННАЯ РОССИЙСКАЯ СОБСТВЕННОСТЬ С ДОЛЕЙ ГОСУДАРСТВЕННОЙ СОБСТВЕННОСТИ  (41 + 42 + 43)", "b": 40}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '35', '35 Совместная собственность общественных и религиозных организаций (объединений) и иностранная собственность', '{"2": "Совместная собственность общественных и религиозных организаций (объединений) и иностранная собственность", "b": 35}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '34', '34 Совместная частная и иностранная собственность', '{"2": "Совместная частная и иностранная собственность", "b": 34}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '33', '33 Совместная муниципальная и иностранная собственность', '{"2": "Совместная муниципальная и иностранная собственность", "b": 33}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '32', '32 Совместная собственность субъектов Российской Федерации и иностранная собственность', '{"2": "Совместная собственность субъектов Российской Федерации и иностранная собственность", "b": 32}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '31', '31 Совместная федеральная и иностранная собственность ', '{"2": "Совместная федеральная и иностранная собственность ", "b": 31}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '30', '30   СОВМЕСТНАЯ РОССИЙСКАЯ И ИНОСТРАННАЯ СОБСТВЕННОСТЬ  (31 + 32 + 33+ 34 + 35)', '{"2": "  СОВМЕСТНАЯ РОССИЙСКАЯ И ИНОСТРАННАЯ СОБСТВЕННОСТЬ  (31 + 32 + 33+ 34 + 35)", "b": 30}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '27', '27 Смешанная иностранная собственность', '{"2": "Смешанная иностранная собственность", "b": 27}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '23', '23 Собственность иностранных юридических лиц', '{"2": "Собственность иностранных юридических лиц", "b": 23}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '22', '22 Собственность иностранных государств', '{"2": "Собственность иностранных государств", "b": 22}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '21', '21 Собственность международных организаций', '{"2": "Собственность международных организаций", "b": 21}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '20', '20  ИНОСТРАННАЯ СОБСТВЕННОСТЬ  (21 + 22 + 23+ 24 + 27)', '{"2": " ИНОСТРАННАЯ СОБСТВЕННОСТЬ  (21 + 22 + 23+ 24 + 27)", "b": 20}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '19', '19 Собственность потребительской кооперации', '{"2": "Собственность потребительской кооперации", "b": 19}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '18', '18 Собственность российских граждан, постоянно проживающих за границей', '{"2": "Собственность российских граждан, постоянно проживающих за границей", "b": 18}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '17', '17   СМЕШАННАЯ РОССИЙСКАЯ СОБСТВЕННОСТЬ  (40 + 49)', '{"2": "  СМЕШАННАЯ РОССИЙСКАЯ СОБСТВЕННОСТЬ  (40 + 49)", "b": 17}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '16', '16 Частная собственность', '{"2": "Частная собственность", "b": 16}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '15', '15   СОБСТВЕННОСТЬ ОБЩЕСТВЕННЫХ И РЕЛИГИОЗНЫХ ОРГАНИЗАЦИЙ (ОБЪЕДИНЕНИЙ)  (50 + 51 + 52+ 53 + 54)', '{"2": "  СОБСТВЕННОСТЬ ОБЩЕСТВЕННЫХ И РЕЛИГИОЗНЫХ ОРГАНИЗАЦИЙ (ОБЪЕДИНЕНИЙ)  (50 + 51 + 52+ 53 + 54)", "b": 15}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '14', '14 Муниципальная собственность', '{"2": "Муниципальная собственность", "b": 14}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '13', '13 Собственность субъектов Российской Федерации', '{"2": "Собственность субъектов Российской Федерации", "b": 13}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '12', '12 Федеральная собственность', '{"2": "Федеральная собственность", "b": 12}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '11', '11   ГОСУДАРСТВЕННАЯ СОБСТВЕННОСТЬ  (12 + 13)', '{"2": "  ГОСУДАРСТВЕННАЯ СОБСТВЕННОСТЬ  (12 + 13)", "b": 11}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '10', '10   РОССИЙСКАЯ СОБСТВЕННОСТЬ  (11 + 14 + 15+ 16 + 17 +18 + 19 + 61)', '{"2": "  РОССИЙСКАЯ СОБСТВЕННОСТЬ  (11 + 14 + 15+ 16 + 17 +18 + 19 + 61)", "b": 10}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '10000', '10000 ОРГАНИЗАЦИОННО-ПРАВОВЫЕ ФОРМЫ ЮРИДИЧЕСКИХ ЛИЦ, ЯВЛЯЮЩИХСЯ КОММЕРЧЕСКИМИ КОРПОРАТИВНЫМИ  ОРГАНИЗАЦИЯМИ', '{"2": "ОРГАНИЗАЦИОННО-ПРАВОВЫЕ ФОРМЫ ЮРИДИЧЕСКИХ ЛИЦ, ЯВЛЯЮЩИХСЯ КОММЕРЧЕСКИМИ КОРПОРАТИВНЫМИ  ОРГАНИЗАЦИЯМИ", "n": 10000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '11000', '11000 Хозяйственные товарищества', '{"2": "Хозяйственные товарищества", "n": 11000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '11051', '11051 Полные товарищества', '{"2": "Полные товарищества", "n": 11051}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '11064', '11064 Товарищества на вере (коммандитные товарищества)', '{"2": "Товарищества на вере (коммандитные товарищества)", "n": 11064}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '12000', '12000 Хозяйственные общества', '{"2": "Хозяйственные общества", "n": 12000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '12200', '12200 Акционерные общества', '{"2": "Акционерные общества", "n": 12200}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '12247', '12247 Публичные акционерные общества', '{"2": "Публичные акционерные общества", "n": 12247}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '12267', '12267 Непубличные акционерные общества', '{"2": "Непубличные акционерные общества", "n": 12267}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20108', '20108 Общества взаимного страхования', '{"2": "Общества взаимного страхования", "n": 20108}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20109', '20109 Сельскохозяйственные потребительские перерабатывающие  кооперативы', '{"2": "Сельскохозяйственные потребительские перерабатывающие  кооперативы", "n": 20109}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20110', '20110 Сельскохозяйственные потребительские сбытовые (торговые) кооперативы', '{"2": "Сельскохозяйственные потребительские сбытовые (торговые) кооперативы", "n": 20110}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20111', '20111 Сельскохозяйственные потребительские обслуживающие кооперативы', '{"2": "Сельскохозяйственные потребительские обслуживающие кооперативы", "n": 20111}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20112', '20112 Сельскохозяйственные потребительские снабженческие кооперативы', '{"2": "Сельскохозяйственные потребительские снабженческие кооперативы", "n": 20112}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20113', '20113 Сельскохозяйственные потребительские садоводческие кооперативы', '{"2": "Сельскохозяйственные потребительские садоводческие кооперативы", "n": 20113}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20114', '20114 Сельскохозяйственные потребительские огороднические кооперативы', '{"2": "Сельскохозяйственные потребительские огороднические кооперативы", "n": 20114}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20115', '20115 Сельскохозяйственные потребительские животноводческие кооперативы', '{"2": "Сельскохозяйственные потребительские животноводческие кооперативы", "n": 20115}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20120', '20120 Садоводческие, огороднические или дачные потребительские кооперативы', '{"2": "Садоводческие, огороднические или дачные потребительские кооперативы", "n": 20120}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20121', '20121 Фонды проката', '{"2": "Фонды проката", "n": 20121}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20200', '20200 Общественные организации', '{"2": "Общественные организации", "n": 20200}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20201', '20201 Политические партии', '{"2": "Политические партии", "n": 20201}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20202', '20202 Профсоюзные организации', '{"2": "Профсоюзные организации", "n": 20202}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20211', '20211 Органы общественной самодеятельности', '{"2": "Органы общественной самодеятельности", "n": 20211}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20217', '20217 Территориальные общественные самоуправления', '{"2": "Территориальные общественные самоуправления", "n": 20217}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20600', '20600 Ассоциации (союзы)', '{"2": "Ассоциации (союзы)", "n": 20600}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20601', '20601 Ассоциации (союзы) экономического взаимодействия субъектов Российской Федерации', '{"2": "Ассоциации (союзы) экономического взаимодействия субъектов Российской Федерации", "n": 20601}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20603', '20603 Советы муниципальных образований субъектов Российской Федерации', '{"2": "Советы муниципальных образований субъектов Российской Федерации", "n": 20603}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20604', '20604 Союзы (ассоциации) кредитных кооперативов', '{"2": "Союзы (ассоциации) кредитных кооперативов", "n": 20604}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20605', '20605 Союзы (ассоциации) кооперативов', '{"2": "Союзы (ассоциации) кооперативов", "n": 20605}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20606', '20606 Союзы (ассоциации) общественных объединений', '{"2": "Союзы (ассоциации) общественных объединений", "n": 20606}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20608', '20608 Союзы потребительских обществ', '{"2": "Союзы потребительских обществ", "n": 20608}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20609', '20609 Адвокатские палаты', '{"2": "Адвокатские палаты", "n": 20609}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20610', '20610 Нотариальные палаты', '{"2": "Нотариальные палаты", "n": 20610}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20611', '20611 Торгово-промышленные палаты', '{"2": "Торгово-промышленные палаты", "n": 20611}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20612', '20612 Объединения работодателей', '{"2": "Объединения работодателей", "n": 20612}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20613', '20613 Объединения фермерских хозяйств', '{"2": "Объединения фермерских хозяйств", "n": 20613}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20614', '20614 Некоммерческие партнерства', '{"2": "Некоммерческие партнерства", "n": 20614}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20615', '20615 Адвокатские бюро', '{"2": "Адвокатские бюро", "n": 20615}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20616', '20616 Коллегии адвокатов', '{"2": "Коллегии адвокатов", "n": 20616}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20617', '20617 Садоводческие, огороднические или дачные некоммерческие партнерства', '{"2": "Садоводческие, огороднические или дачные некоммерческие партнерства", "n": 20617}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20618', '20618 Ассоциации (союзы) садоводческих, огороднических и дачных некоммерческих объединений', '{"2": "Ассоциации (союзы) садоводческих, огороднических и дачных некоммерческих объединений", "n": 20618}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20619', '20619 Саморегулируемые организации', '{"2": "Саморегулируемые организации", "n": 20619}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20620', '20620 Объединения (ассоциации и союзы) благотворительных организаций', '{"2": "Объединения (ассоциации и союзы) благотворительных организаций", "n": 20620}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20700', '20700 Товарищества собственников недвижимости', '{"2": "Товарищества собственников недвижимости", "n": 20700}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20701', '20701 Садоводческие, огороднические или дачные некоммерческие товарищества', '{"2": "Садоводческие, огороднические или дачные некоммерческие товарищества", "n": 20701}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20716', '20716 Товарищества собственников жилья', '{"2": "Товарищества собственников жилья", "n": 20716}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '21100', '21100 Казачьи общества, внесенные в государственный реестр казачьих обществ в Российской Федерации', '{"2": "Казачьи общества, внесенные в государственный реестр казачьих обществ в Российской Федерации", "n": 21100}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '30000', '30000 ОРГАНИЗАЦИОННО-ПРАВОВЫЕ ФОРМЫ ОРГАНИЗАЦИЙ, СОЗДАННЫХ  БЕЗ ПРАВ ЮРИДИЧЕСКОГО ЛИЦА', '{"2": "ОРГАНИЗАЦИОННО-ПРАВОВЫЕ ФОРМЫ ОРГАНИЗАЦИЙ, СОЗДАННЫХ  БЕЗ ПРАВ ЮРИДИЧЕСКОГО ЛИЦА", "n": 30000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '30001', '30001 Представительства юридических лиц', '{"2": "Представительства юридических лиц", "n": 30001}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '30002', '30002 Филиалы юридических лиц', '{"2": "Филиалы юридических лиц", "n": 30002}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '30003', '30003 Обособленные подразделения юридических лиц', '{"2": "Обособленные подразделения юридических лиц", "n": 30003}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '30004', '30004 Структурные подразделения обособленных подразделений юридических лиц', '{"2": "Структурные подразделения обособленных подразделений юридических лиц", "n": 30004}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '30005', '30005 Паевые инвестиционные фонды', '{"2": "Паевые инвестиционные фонды", "n": 30005}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '30006', '30006 Простые товарищества', '{"2": "Простые товарищества", "n": 30006}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '30008', '30008 Районные суды, городские суды, межрайонные суды (районные суды)', '{"2": "Районные суды, городские суды, межрайонные суды (районные суды)", "n": 30008}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '40000', '40000 ОРГАНИЗАЦИОННО-ПРАВОВЫЕ ФОРМЫ МЕЖДУНАРОДНЫХ ОРГАНИЗАЦИЙ, ОСУЩЕСТВЛЯЮЩИХ ДЕЯТЕЛЬНОСТЬ НА ТЕРРИТОРИИ РОССИЙСКОЙ ФЕДЕРАЦИИ', '{"2": "ОРГАНИЗАЦИОННО-ПРАВОВЫЕ ФОРМЫ МЕЖДУНАРОДНЫХ ОРГАНИЗАЦИЙ, ОСУЩЕСТВЛЯЮЩИХ ДЕЯТЕЛЬНОСТЬ НА ТЕРРИТОРИИ РОССИЙСКОЙ ФЕДЕРАЦИИ", "n": 40000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '40001', '40001 Межправительственные международные организации', '{"2": "Межправительственные международные организации", "n": 40001}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '40002', '40002 Неправительственные международные организации', '{"2": "Неправительственные международные организации", "n": 40002}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '50000', '50000 ОРГАНИЗАЦИОННО-ПРАВОВЫЕ ФОРМЫ ДЛЯ ДЕЯТЕЛЬНОСТИ ГРАЖДАН (ФИЗИЧЕСКИХ ЛИЦ)', '{"2": "ОРГАНИЗАЦИОННО-ПРАВОВЫЕ ФОРМЫ ДЛЯ ДЕЯТЕЛЬНОСТИ ГРАЖДАН (ФИЗИЧЕСКИХ ЛИЦ)", "n": 50000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '50100', '50100 Организационно-правовые формы для коммерческой деятельности граждан', '{"2": "Организационно-правовые формы для коммерческой деятельности граждан", "n": 50100}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '50101', '50101 Главы крестьянских (фермерских) хозяйств', '{"2": "Главы крестьянских (фермерских) хозяйств", "n": 50101}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '50102', '50102 Индивидуальные предприниматели', '{"2": "Индивидуальные предприниматели", "n": 50102}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '50200', '50200 Организационно-правовые формы для деятельности граждан, не отнесенной  к предпринимательству', '{"2": "Организационно-правовые формы для деятельности граждан, не отнесенной  к предпринимательству", "n": 50200}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '50201', '50201 Адвокаты, учредившие адвокатский кабинет', '{"2": "Адвокаты, учредившие адвокатский кабинет", "n": 50201}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '50202', '50202 Нотариусы, занимающиеся частной практикой', '{"2": "Нотариусы, занимающиеся частной практикой", "n": 50202}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '60000', '60000 ОРГАНИЗАЦИОННО-ПРАВОВЫЕ ФОРМЫ ЮРИДИЧЕСКИХ ЛИЦ, ЯВЛЯЮЩИХСЯ КОММЕРЧЕСКИМИ УНИТАРНЫМИ ОРГАНИЗАЦИЯМИ', '{"2": "ОРГАНИЗАЦИОННО-ПРАВОВЫЕ ФОРМЫ ЮРИДИЧЕСКИХ ЛИЦ, ЯВЛЯЮЩИХСЯ КОММЕРЧЕСКИМИ УНИТАРНЫМИ ОРГАНИЗАЦИЯМИ", "n": 60000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '65000', '65000 Унитарные предприятия', '{"2": "Унитарные предприятия", "n": 65000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '65100', '65100 Унитарные предприятия, основанные на праве оперативного управления (казенные предприятия)', '{"2": "Унитарные предприятия, основанные на праве оперативного управления (казенные предприятия)", "n": 65100}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '65141', '65141 Федеральные казенные предприятия', '{"2": "Федеральные казенные предприятия", "n": 65141}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '65142', '65142 Казенные предприятия субъектов Российской Федерации', '{"2": "Казенные предприятия субъектов Российской Федерации", "n": 65142}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '65143', '65143 Муниципальные казенные предприятия', '{"2": "Муниципальные казенные предприятия", "n": 65143}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '65200', '65200 Унитарные предприятия, основанные на праве хозяйственного ведения', '{"2": "Унитарные предприятия, основанные на праве хозяйственного ведения", "n": 65200}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '65242', '65242 Государственные унитарные предприятия субъектов Российской Федерации', '{"2": "Государственные унитарные предприятия субъектов Российской Федерации", "n": 65242}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '65243', '65243 Муниципальные унитарные предприятия', '{"2": "Муниципальные унитарные предприятия", "n": 65243}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '70000', '70000 ОРГАНИЗАЦИОННО-ПРАВОВЫЕ ФОРМЫ ЮРИДИЧЕСКИХ ЛИЦ, ЯВЛЯЮЩИХСЯ НЕКОММЕРЧЕСКИМИ УНИТАРНЫМИ ОРГАНИЗАЦИЯМИ', '{"2": "ОРГАНИЗАЦИОННО-ПРАВОВЫЕ ФОРМЫ ЮРИДИЧЕСКИХ ЛИЦ, ЯВЛЯЮЩИХСЯ НЕКОММЕРЧЕСКИМИ УНИТАРНЫМИ ОРГАНИЗАЦИЯМИ", "n": 70000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '70400', '70400 Фонды', '{"2": "Фонды", "n": 70400}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '70401', '70401 Благотворительные фонды', '{"2": "Благотворительные фонды", "n": 70401}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '70402', '70402 Негосударственные пенсионные фонды', '{"2": "Негосударственные пенсионные фонды", "n": 70402}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '70403', '70403 Общественные фонды', '{"2": "Общественные фонды", "n": 70403}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '70404', '70404 Экологические фонды', '{"2": "Экологические фонды", "n": 70404}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '71400', '71400 Автономные некоммерческие организации', '{"2": "Автономные некоммерческие организации", "n": 71400}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '71500', '71500 Религиозные организации', '{"2": "Религиозные организации", "n": 71500}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '71600', '71600 Публично-правовые компании', '{"2": "Публично-правовые компании", "n": 71600}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '71601', '71601 Государственные корпорации', '{"2": "Государственные корпорации", "n": 71601}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '71602', '71602 Государственные компании', '{"2": "Государственные компании", "n": 71602}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '71610', '71610 Отделения иностранных некоммерческих неправительственных организаций', '{"2": "Отделения иностранных некоммерческих неправительственных организаций", "n": 71610}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '75000', '75000 Учреждения', '{"2": "Учреждения", "n": 75000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '75100', '75100 Учреждения, созданные Российской Федерацией', '{"2": "Учреждения, созданные Российской Федерацией", "n": 75100}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '75101', '75101 Федеральные государственные автономные учреждения', '{"2": "Федеральные государственные автономные учреждения", "n": 75101}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '75103', '75103 Федеральные государственные бюджетные учреждения', '{"2": "Федеральные государственные бюджетные учреждения", "n": 75103}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '75104', '75104 Федеральные государственные казенные учреждения', '{"2": "Федеральные государственные казенные учреждения", "n": 75104}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '75200', '75200 Учреждения, созданные субъектом Российской Федерации', '{"2": "Учреждения, созданные субъектом Российской Федерации", "n": 75200}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '75201', '75201 Государственные автономные учреждения субъектов Российской Федерации', '{"2": "Государственные автономные учреждения субъектов Российской Федерации", "n": 75201}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '75203', '75203 Государственные бюджетные учреждения субъектов Российской Федерации', '{"2": "Государственные бюджетные учреждения субъектов Российской Федерации", "n": 75203}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '75204', '75204 Государственные казенные учреждения субъектов Российской Федерации', '{"2": "Государственные казенные учреждения субъектов Российской Федерации", "n": 75204}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '75300', '75300 Государственные академии наук', '{"2": "Государственные академии наук", "n": 75300}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '75400', '75400 Учреждения, созданные муниципальным образованием (муниципальные учреждения)', '{"2": "Учреждения, созданные муниципальным образованием (муниципальные учреждения)", "n": 75400}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '75401', '75401 Муниципальные автономные учреждения', '{"2": "Муниципальные автономные учреждения", "n": 75401}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '75403', '75403 Муниципальные бюджетные учреждения', '{"2": "Муниципальные бюджетные учреждения", "n": 75403}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '75404', '75404 Муниципальные казенные учреждения', '{"2": "Муниципальные казенные учреждения", "n": 75404}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '75500', '75500 Частные учреждения', '{"2": "Частные учреждения", "n": 75500}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '75502', '75502 Благотворительные учреждения', '{"2": "Благотворительные учреждения", "n": 75502}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '75505', '75505 Общественные учреждения', '{"2": "Общественные учреждения", "n": 75505}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '15100', '15100 Унитарные предприятия, основанные на праве оперативного управления (казенные предприятия)', '{"2": "Унитарные предприятия, основанные на праве оперативного управления (казенные предприятия)", "n": 15100}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '15141', '15141 Федеральные казенные предприятия', '{"2": "Федеральные казенные предприятия", "n": 15141}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '15142', '15142 Казенные предприятия субъектов Российской Федерации', '{"2": "Казенные предприятия субъектов Российской Федерации", "n": 15142}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '15143', '15143 Муниципальные казенные предприятия', '{"2": "Муниципальные казенные предприятия", "n": 15143}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '15200', '15200 Унитарные предприятия, основанные на праве хозяйственного ведения', '{"2": "Унитарные предприятия, основанные на праве хозяйственного ведения", "n": 15200}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '15241', '15241 Федеральные государственные унитарные предприятия', '{"2": "Федеральные государственные унитарные предприятия", "n": 15241}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '15242', '15242 Государственные унитарные предприятия субъектов Российской Федерации', '{"2": "Государственные унитарные предприятия субъектов Российской Федерации", "n": 15242}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '15243', '15243 Муниципальные унитарные предприятия', '{"2": "Муниципальные унитарные предприятия", "n": 15243}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '12100', '12100 Общества с ограниченной или дополнительной ответственностью', '{"2": "Общества с ограниченной или дополнительной ответственностью", "n": 12100}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '12165', '12165 Общества с ограниченной ответственностью', '{"2": "Общества с ограниченной ответственностью", "n": 12165}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '12166', '12166 Общества с дополнительной ответственностью', '{"2": "Общества с дополнительной ответственностью", "n": 12166}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '21001', '21001 Садоводческие, огороднические или дачные некоммерческие товарищества', '{"2": "Садоводческие, огороднические или дачные некоммерческие товарищества", "n": 21001}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '21002', '21002 Садоводческие, огороднические или дачные потребительские кооперативы', '{"2": "Садоводческие, огороднические или дачные потребительские кооперативы", "n": 21002}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '21003', '21003 Садоводческие, огороднические или дачные некоммерческие партнерства', '{"2": "Садоводческие, огороднические или дачные некоммерческие партнерства", "n": 21003}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '28000', '28000 Юридические лица, являющиеся некоммерческими организациями, не включенные в другие группировки', '{"2": "Юридические лица, являющиеся некоммерческими организациями, не включенные в другие группировки", "n": 28000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '28002', '28002 Адвокатские бюро', '{"2": "Адвокатские бюро", "n": 28002}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '28003', '28003 Государственные академии наук', '{"2": "Государственные академии наук", "n": 28003}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '28004', '28004 Государственные компании', '{"2": "Государственные компании", "n": 28004}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '28005', '28005 Государственные корпорации', '{"2": "Государственные корпорации", "n": 28005}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '28006', '28006 Казачьи общества', '{"2": "Казачьи общества", "n": 28006}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '28007', '28007 Коллегии адвокатов', '{"2": "Коллегии адвокатов", "n": 28007}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '28008', '28008 Общества взаимного страхования', '{"2": "Общества взаимного страхования", "n": 28008}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '28010', '28010 Общины коренных малочисленных народов', '{"2": "Общины коренных малочисленных народов", "n": 28010}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '28012', '28012 Объединения работодателей', '{"2": "Объединения работодателей", "n": 28012}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '28013', '28013 Объединения (союзы) крестьянских (фермерских) хозяйств', '{"2": "Объединения (союзы) крестьянских (фермерских) хозяйств", "n": 28013}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '28014', '28014 Отделения иностранных некоммерческих неправительственных организаций', '{"2": "Отделения иностранных некоммерческих неправительственных организаций", "n": 28014}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '28016', '28016 Товарищества собственников жилья', '{"2": "Товарищества собственников жилья", "n": 28016}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '28017', '28017 Территориальные общественные самоуправления', '{"2": "Территориальные общественные самоуправления", "n": 28017}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '29000', '29000 Иные некоммерческие организации, не включенные в другие группировки', '{"2": "Иные некоммерческие организации, не включенные в другие группировки", "n": 29000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '21200', '21200 Общины коренных малочисленных народов Российской Федерации', '{"2": "Общины коренных малочисленных народов Российской Федерации", "n": 21200}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '12300', '12300 Общества с ограниченной ответственностью', '{"2": "Общества с ограниченной ответственностью", "n": 12300}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '13000', '13000 Хозяйственные партнерства', '{"2": "Хозяйственные партнерства", "n": 13000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '14000', '14000 Производственные кооперативы (артели)', '{"2": "Производственные кооперативы (артели)", "n": 14000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '14100', '14100 Сельскохозяйственные производственные кооперативы', '{"2": "Сельскохозяйственные производственные кооперативы", "n": 14100}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '14153', '14153 Сельскохозяйственные артели (колхозы)', '{"2": "Сельскохозяйственные артели (колхозы)", "n": 14153}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '14154', '14154 Рыболовецкие артели (колхозы)', '{"2": "Рыболовецкие артели (колхозы)", "n": 14154}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '14155', '14155 Кооперативные хозяйства (коопхозы)', '{"2": "Кооперативные хозяйства (коопхозы)", "n": 14155}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '14200', '14200 Производственные кооперативы (кроме сельскохозяйственных производственных кооперативов)', '{"2": "Производственные кооперативы (кроме сельскохозяйственных производственных кооперативов)", "n": 14200}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '15300', '15300 Крестьянские (фермерские) хозяйства', '{"2": "Крестьянские (фермерские) хозяйства", "n": 15300}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '19000', '19000 Прочие юридические лица, являющиеся коммерческими организациями', '{"2": "Прочие юридические лица, являющиеся коммерческими организациями", "n": 19000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20000', '20000 ОРГАНИЗАЦИОННО-ПРАВОВЫЕ ФОРМЫ ЮРИДИЧЕСКИХ ЛИЦ, ЯВЛЯЮЩИХСЯ НЕКОММЕРЧЕСКИМИ КОРПОРАТИВНЫМИ ОРГАНИЗАЦИЯМИ', '{"2": "ОРГАНИЗАЦИОННО-ПРАВОВЫЕ ФОРМЫ ЮРИДИЧЕСКИХ ЛИЦ, ЯВЛЯЮЩИХСЯ НЕКОММЕРЧЕСКИМИ КОРПОРАТИВНЫМИ ОРГАНИЗАЦИЯМИ", "n": 20000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20100', '20100 Потребительские кооперативы', '{"2": "Потребительские кооперативы", "n": 20100}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20101', '20101 Гаражные и гаражно-строительные кооперативы', '{"2": "Гаражные и гаражно-строительные кооперативы", "n": 20101}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20102', '20102 Жилищные или жилищно-строительные кооперативы', '{"2": "Жилищные или жилищно-строительные кооперативы", "n": 20102}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20103', '20103 Жилищные накопительные кооперативы', '{"2": "Жилищные накопительные кооперативы", "n": 20103}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20104', '20104 Кредитные потребительские кооперативы', '{"2": "Кредитные потребительские кооперативы", "n": 20104}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20105', '20105 Кредитные потребительские  кооперативы граждан', '{"2": "Кредитные потребительские  кооперативы граждан", "n": 20105}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20106', '20106 Кредитные кооперативы второго уровня', '{"2": "Кредитные кооперативы второго уровня", "n": 20106}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20107', '20107 Потребительские общества', '{"2": "Потребительские общества", "n": 20107}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('121_voc', '02', '02 ОРИ кроме изделий', '{"2": "ОРИ кроме изделий", "c": "02"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('121_voc', '01', '01 ОРИ в виде изделий', '{"2": "ОРИ в виде изделий", "c": "01"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('16_voc', '6', '6', '{"attrs1": null, "status": "1", "BCodCod": "6", "BCod_Name": "Бесхозяйные РАО", "BCod_Clarif": "Бесхозяйными признаются РАО в соответствии с положениями статьи 225 Гражданского кодекса Российской Федерации"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('16_voc', '4', '4', '{"BCodCod": "4", "BCod_Name": "Муниципальная собственность", "BCod_Clarif": "В примечании необходимо указать наименование муниципалитета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('16_voc', '3', '3', '{"BCodCod": "3", "BCod_Name": "Собственность субъекта Российской Федерации", "BCod_Clarif": "В примечании к ячейке необходимо указать наименование субъекта Российской Федерации"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('16_voc', '2', '2', '{"BCodCod": "2", "BCod_Name": "федеральные, кроме накопленных РАО", "BCod_Clarif": "код используется для РАО, образовавшихся после вступления в силу Федерального закона от 11.07.2011  № 190-ФЗ «Об обращении с радиоактивными отходами и о внесении изменений в отдельные законодательные акты Российской Федерации», содержащих ядерные материалы, которые могут находиться исключительно в федеральной собственности"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('16_voc', '1', '1', '{"BCodCod": "1", "BCod_Name": "накопленные РАО", "BCod_Clarif": "код используется для РАО, образовавшихся до дня вступления в силу Федерального закона от 11.07.2011  № 190-ФЗ «Об обращении с радиоактивными отходами и о внесении изменений в отдельные законодательные акты Российской Федерации»"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('113_voc', 'СЗЗ', 'СЗЗ', '{"Name": "СЗЗ", "Clarif": "Санитарно-защитная зона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('113_voc', 'ПП', 'ПП', '{"Name": "ПП", "Clarif": "Промплощадка"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('113_voc', 'ЗН', 'ЗН', '{"Name": "ЗН", "Clarif": "Зона наблюдения"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('114_voc', '12', '12 Министерство обороны Российской федерации', '{"2": "Министерство обороны Российской федерации", "a": "12"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('114_voc', '11', '11 Федеральное медико-биологическое агенство', '{"2": "Федеральное медико-биологическое агенство", "a": "11"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('114_voc', '10', '10 Федеральное агентство морского и речного транспорта', '{"2": "Федеральное агентство морского и речного транспорта", "a": "10"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('114_voc', '09', '09 Федеральное агентство по техническому регулированию и метрологии', '{"2": "Федеральное агентство по техническому регулированию и метрологии", "a": "09"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('114_voc', '08', '08 Федеральное агентство по недропользованию', '{"2": "Федеральное агентство по недропользованию", "a": "08"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '07', 'Кабардино-Балкарская республика', '{"2": "Кабардино-Балкарская республика", "3": 4, "s": "07"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '08', 'Республика Калмыкия', '{"2": "Республика Калмыкия", "3": 3, "s": "08"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '09', 'Карачаево-Черкесская республика', '{"2": "Карачаево-Черкесская республика", "3": 4, "s": "09"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '10', 'Республика Карелия', '{"2": "Республика Карелия", "3": 2, "s": 10}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '11', 'Республика Коми', '{"2": "Республика Коми", "3": 2, "s": 11}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '12', 'Республика Марий Эл', '{"2": "Республика Марий Эл", "3": 5, "s": 12}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '13', 'Республика Мордовия', '{"2": "Республика Мордовия", "3": 5, "s": 13}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '14', 'Республика Саха (Якутия)', '{"2": "Республика Саха (Якутия)", "3": 8, "s": 14}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '15', 'Республика Северная Осетия — Алания', '{"2": "Республика Северная Осетия — Алания", "3": 4, "s": 15}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '16', 'Республика Татарстан', '{"2": "Республика Татарстан", "3": 5, "s": 16}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '17', 'Республика Тыва', '{"2": "Республика Тыва", "3": 7, "s": 17}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '18', 'Удмуртская республика', '{"2": "Удмуртская республика", "3": 5, "s": 18}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '19', 'Республика Хакасия', '{"2": "Республика Хакасия", "3": 7, "s": 19}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '20', 'Чеченская республика', '{"2": "Чеченская республика", "3": 4, "s": 20}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '21', 'Чувашская республика', '{"2": "Чувашская республика", "3": 5, "s": 21}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '22', 'Алтайский край', '{"2": "Алтайский край", "3": 7, "s": 22}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '75', 'Забайкальский край', '{"2": "Забайкальский край", "3": 8, "s": 75}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '41', 'Камчатский край', '{"2": "Камчатский край", "3": 8, "s": 41}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '23', 'Краснодарский край', '{"2": "Краснодарский край", "3": 3, "s": 23}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '24', 'Красноярский край', '{"2": "Красноярский край", "3": 7, "s": 24}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '25', 'Приморский край', '{"2": "Приморский край", "3": 8, "s": 25}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '26', 'Ставропольский край', '{"2": "Ставропольский край", "3": 4, "s": 26}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '27', 'Хабаровский край', '{"2": "Хабаровский край", "3": 8, "s": 27}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '28', 'Амурская область', '{"2": "Амурская область", "3": 8, "s": 28}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '29', 'Архангельская область', '{"2": "Архангельская область", "3": 2, "s": 29}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '30', 'Астраханская область', '{"2": "Астраханская область", "3": 3, "s": 30}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '31', 'Белгородская область', '{"2": "Белгородская область", "3": 1, "s": 31}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '32', 'Брянская область', '{"2": "Брянская область", "3": 1, "s": 32}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '33', 'Владимирская область', '{"2": "Владимирская область", "3": 1, "s": 33}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '34', 'Волгоградская область', '{"2": "Волгоградская область", "3": 3, "s": 34}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '35', 'Вологодская область', '{"2": "Вологодская область", "3": 2, "s": 35}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '36', 'Воронежская область', '{"2": "Воронежская область", "3": 1, "s": 36}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '37', 'Ивановская область', '{"2": "Ивановская область", "3": 1, "s": 37}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '38', 'Иркутская область', '{"2": "Иркутская область", "3": 7, "s": 38}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '39', 'Калининградская область', '{"2": "Калининградская область", "3": 2, "s": 39}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '40', 'Калужская область', '{"2": "Калужская область", "3": 1, "s": 40}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '42', 'Кемеровская область', '{"2": "Кемеровская область", "3": 7, "s": 42}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '43', 'Кировская область', '{"2": "Кировская область", "3": 5, "s": 43}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '44', 'Костромская область', '{"2": "Костромская область", "3": 1, "s": 44}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '45', 'Курганская область', '{"2": "Курганская область", "3": 6, "s": 45}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '46', 'Курская область', '{"2": "Курская область", "3": 1, "s": 46}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '47', 'Ленинградская область', '{"2": "Ленинградская область", "3": 2, "s": 47}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '48', 'Липецкая область', '{"2": "Липецкая область", "3": 1, "s": 48}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '49', 'Магаданская область', '{"2": "Магаданская область", "3": 8, "s": 49}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '51', 'Мурманская область', '{"2": "Мурманская область", "3": 2, "s": 51}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '52', 'Нижегородская область', '{"2": "Нижегородская область", "3": 5, "s": 52}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '53', 'Новгородская область', '{"2": "Новгородская область", "3": 2, "s": 53}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '54', 'Новосибирская область', '{"2": "Новосибирская область", "3": 7, "s": 54}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '55', 'Омская область', '{"2": "Омская область", "3": 7, "s": 55}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '56', 'Оренбургская область', '{"2": "Оренбургская область", "3": 5, "s": 56}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '57', 'Орловская область', '{"2": "Орловская область", "3": 1, "s": 57}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '58', 'Пензенская область', '{"2": "Пензенская область", "3": 5, "s": 58}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '60', 'Псковская область', '{"2": "Псковская область", "3": 2, "s": 60}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '61', 'Ростовская область', '{"2": "Ростовская область", "3": 3, "s": 61}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '62', 'Рязанская область', '{"2": "Рязанская область", "3": 1, "s": 62}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '63', 'Самарская область', '{"2": "Самарская область", "3": 5, "s": 63}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '64', 'Саратовская область', '{"2": "Саратовская область", "3": 5, "s": 64}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '65', 'Сахалинская область', '{"2": "Сахалинская область", "3": 8, "s": 65}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '66', 'Свердловская область', '{"2": "Свердловская область", "3": 6, "s": 66}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '67', 'Смоленская область', '{"2": "Смоленская область", "3": 1, "s": 67}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '68', 'Тамбовская область', '{"2": "Тамбовская область", "3": 1, "s": 68}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '69', 'Тверская область', '{"2": "Тверская область", "3": 1, "s": 69}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '70', 'Томская область', '{"2": "Томская область", "3": 7, "s": 70}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '71', 'Тульская область', '{"2": "Тульская область", "3": 1, "s": 71}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '72', 'Тюменская область', '{"2": "Тюменская область", "3": 6, "s": 72}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '73', 'Ульяновская область', '{"2": "Ульяновская область", "3": 5, "s": 73}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '74', 'Челябинская область', '{"2": "Челябинская область", "3": 6, "s": 74}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '76', 'Ярославская область', '{"2": "Ярославская область", "3": 1, "s": 76}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '77', 'Москва', '{"2": "Москва", "3": 1, "s": 77}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '78', 'Санкт-Петербург', '{"2": "Санкт-Петербург", "3": 2, "s": 78}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '79', 'Еврейская автономная область', '{"2": "Еврейская автономная область", "3": 8, "s": 79}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '83', 'Ненецкий автономный округ', '{"2": "Ненецкий автономный округ", "3": 2, "s": 83}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '86', 'Ханты-Мансийский автономный округ - Югра', '{"2": "Ханты-Мансийский автономный округ - Югра", "3": 6, "s": 86}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '87', 'Чукотский автономный округ', '{"2": "Чукотский автономный округ", "3": 8, "s": 87}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '89', 'Ямало-Ненецкий автономный округ', '{"2": "Ямало-Ненецкий автономный округ", "3": 6, "s": 89}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '91', 'Республика Крым', '{"2": "Республика Крым", "3": 3, "s": 91}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '92', 'Севастополь', '{"2": "Севастополь", "3": 3, "s": 92}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '59', 'Пермский край', '{"2": "Пермский край", "3": 5, "s": 59}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '02', 'Республика Башкортостан', '{"2": "Республика Башкортостан", "3": 5, "s": "02"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('13_voc', '7', '7 Раствор', '{"ORICod": 7, "ORI_ty": "Раствор"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('13_voc', '4', '4 Руда', '{"ORICod": 4, "ORI_ty": "Руда"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('110_voc', 'Амурский', '20 Амурский', '{"PolCod": "20", "PolName": "Амурский"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('110_voc', 'Анадыро-Колымский', '19 Анадыро-Колымский', '{"PolCod": "19", "PolName": "Анадыро-Колымский"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('110_voc', 'Ленский', '18 Ленский', '{"PolCod": "18", "PolName": "Ленский"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('110_voc', 'Енисейский', '17 Енисейский', '{"PolCod": "17", "PolName": "Енисейский"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('110_voc', 'Ангаро-Байкальский', '16 Ангаро-Байкальский', '{"PolCod": "16", "PolName": "Ангаро-Байкальский"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('110_voc', 'Нижнеобский', '15 Нижнеобский', '{"PolCod": "15", "PolName": "Нижнеобский"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('110_voc', 'Иртышский', '14 Иртышский', '{"PolCod": "14", "PolName": "Иртышский"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('110_voc', 'Верхнеобский', '13 Верхнеобский', '{"PolCod": "13", "PolName": "Верхнеобский"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('110_voc', 'Уральский', '12 Уральский', '{"PolCod": "12", "PolName": "Уральский"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('110_voc', 'Нижневолжский', '11 Нижневолжский', '{"PolCod": "11", "PolName": "Нижневолжский"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('110_voc', 'Камский', '10 Камский', '{"PolCod": "10", "PolName": "Камский"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('110_voc', 'Окский', '09 Окский', '{"PolCod": "09", "PolName": "Окский"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('110_voc', 'Верхневолжский', '08 Верхневолжский', '{"PolCod": "08", "PolName": "Верхневолжский"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('110_voc', 'Западно-Каспийский', '07 Западно-Каспийский', '{"PolCod": "07", "PolName": "Западно-Каспийский"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('110_voc', 'Кубанский', '06 Кубанский', '{"PolCod": "06", "PolName": "Кубанский"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('110_voc', 'Донской', '05 Донской', '{"PolCod": "05", "PolName": "Донской"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('110_voc', 'Днепропетровский', '04 Днепропетровский', '{"PolCod": "04", "PolName": "Днепропетровский"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('110_voc', 'Двинско-Печорский', '03 Двинско-Печорский', '{"PolCod": "03", "PolName": "Двинско-Печорский"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('110_voc', 'Баренцево-Беломорский', '02 Баренцево-Беломорский', '{"PolCod": "02", "PolName": "Баренцево-Беломорский"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('110_voc', 'Балтийский', '01 Балтийский', '{"PolCod": "01", "PolName": "Балтийский"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '99', '99', '{"c1": "99", "c11": "Прочие типы установок переработки (в примечании к ячейке следует указать наименование установки и используемую технологию)"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '79', '79', '{"c1": "79", "c11": "Прочие (в примечании к ячейке следует указать способ переработки)"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '74', '74', '{"c1": "74", "c11": "Горячая камера"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '73', '73', '{"c1": "73", "c11": "Фрагментация"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '72', '72', '{"c1": "72", "c11": "Сортировка"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '71', '71', '{"c1": "71", "c11": "Цементирование"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '63', '63', '{"c1": "63", "c11": "Комплексы переработки твердых РАО прочие (в примечании к ячейке следует указать наименования установок и используемую технологию)"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '43', '43', '{"c1": "43", "c11": "Установка остекловывания"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '42', '42', '{"c1": "42", "c11": "Установка цементирования"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '41', '41', '{"c1": "41", "c11": "Установка битумирования"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '39', '39', '{"c1": "39", "c11": "Установки компактирования прочие (в примечании к ячейке следует указать наименование установки и используемую технологию)"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '32', '32', '{"c1": "32", "c11": "Установка суперкомпактирования"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '31', '31', '{"c1": "31", "c11": "Установка прессования"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '29', '29', '{"c1": "29", "c11": "Установки термообработки прочие (в примечании к ячейке следует указать наименование установки и используемую технологию)"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '24', '24', '{"c1": "24", "c11": "Установка кальцинации"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '23', '23', '{"c1": "23", "c11": "Установка плавления"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '22', '22', '{"c1": "22", "c11": "Установка сжигания"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '21', '21', '{"c1": "21", "c11": "Установка упаривания"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '19', '19', '{"c1": "19", "c11": "Установки разделения прочие (в примечании к ячейке следует указать наименование установки и используемую технологию)"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '17', '17', '{"c1": "17", "c11": "Установка коагуляционная"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '16', '16', '{"c1": "16", "c11": "Установка осадительная"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '15', '15', '{"c1": "15", "c11": "Установка фракционирования радионуклидов"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '14', '14', '{"c1": "14", "c11": "Установка фильтрации"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '13', '13', '{"c1": "13", "c11": "Установка мембранно-сорбционная"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '12', '12', '{"c1": "12", "c11": "Установка мембранная"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '762', 'ТАДЖИКИСТАН', '{"2": "ТАДЖИКИСТАН", "3": "СНГ", "v": 762, "attrs1": null, "status": "1"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '11', '11', '{"c1": "11", "c11": "Установка сорбционная (ионообменная)"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '896.0', 'ЮЖНАЯ ОСЕТИЯ', '{"2": "ЮЖНАЯ ОСЕТИЯ", "v": 896.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '895.0', 'АБХАЗИЯ', '{"2": "АБХАЗИЯ", "v": 895.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '894.0', 'ЗАМБИЯ', '{"2": "ЗАМБИЯ", "v": 894.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '887.0', 'ЙЕМЕН', '{"2": "ЙЕМЕН", "v": 887.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '882.0', 'САМОА', '{"2": "САМОА", "v": 882.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '876.0', 'УОЛЛИС И ФУТУНА', '{"2": "УОЛЛИС И ФУТУНА", "v": 876.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '862.0', 'ВЕНЕСУЭЛА (БОЛИВАРИАНСКАЯ РЕСПУБЛИКА)', '{"2": "ВЕНЕСУЭЛА (БОЛИВАРИАНСКАЯ РЕСПУБЛИКА)", "v": 862.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '858.0', 'УРУГВАЙ', '{"2": "УРУГВАЙ", "v": 858.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '854.0', 'БУРКИНА-ФАСО', '{"2": "БУРКИНА-ФАСО", "v": 854.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '850.0', 'ВИРГИНСКИЕ ОСТРОВА (США)', '{"2": "ВИРГИНСКИЕ ОСТРОВА (США)", "v": 850.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '840.0', 'СОЕДИНЕННЫЕ ШТАТЫ', '{"2": "СОЕДИНЕННЫЕ ШТАТЫ", "v": 840.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '834.0', 'ТАНЗАНИЯ (ОБЪЕДИНЕННАЯ РЕСПУБЛИКА)', '{"2": "ТАНЗАНИЯ (ОБЪЕДИНЕННАЯ РЕСПУБЛИКА)", "v": 834.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '833.0', 'ОСТРОВ МЭН', '{"2": "ОСТРОВ МЭН", "v": 833.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '832.0', 'ДЖЕРСИ', '{"2": "ДЖЕРСИ", "v": 832.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '831.0', 'ГЕРНСИ', '{"2": "ГЕРНСИ", "v": 831.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '826.0', 'СОЕДИНЕННОЕ КОРОЛЕВСТВО', '{"2": "СОЕДИНЕННОЕ КОРОЛЕВСТВО", "v": 826.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '818.0', 'ЕГИПЕТ', '{"2": "ЕГИПЕТ", "v": 818.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '807.0', 'СЕВЕРНАЯ МАКЕДОНИЯ', '{"2": "СЕВЕРНАЯ МАКЕДОНИЯ", "v": 807.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '800.0', 'УГАНДА', '{"2": "УГАНДА", "v": 800.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '798.0', 'ТУВАЛУ', '{"2": "ТУВАЛУ", "v": 798.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '796.0', 'ОСТРОВА ТЕРКС И КАЙКОС', '{"2": "ОСТРОВА ТЕРКС И КАЙКОС", "v": 796.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '795.0', 'ТУРКМЕНИСТАН', '{"2": "ТУРКМЕНИСТАН", "v": 795.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '792.0', 'ТУРЦИЯ', '{"2": "ТУРЦИЯ", "v": 792.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '788.0', 'ТУНИС', '{"2": "ТУНИС", "v": 788.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '784.0', 'ОБЪЕДИНЕННЫЕ АРАБСКИЕ ЭМИРАТЫ', '{"2": "ОБЪЕДИНЕННЫЕ АРАБСКИЕ ЭМИРАТЫ", "v": 784.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '780.0', 'ТРИНИДАД И ТОБАГО', '{"2": "ТРИНИДАД И ТОБАГО", "v": 780.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '776.0', 'ТОНГА', '{"2": "ТОНГА", "v": 776.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '772.0', 'ТОКЕЛАУ', '{"2": "ТОКЕЛАУ", "v": 772.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '768.0', 'ТОГО', '{"2": "ТОГО", "v": 768.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '764.0', 'ТАИЛАНД', '{"2": "ТАИЛАНД", "v": 764.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '760.0', 'СИРИЙСКАЯ АРАБСКАЯ РЕСПУБЛИКА', '{"2": "СИРИЙСКАЯ АРАБСКАЯ РЕСПУБЛИКА", "v": 760.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '756.0', 'ШВЕЙЦАРИЯ', '{"2": "ШВЕЙЦАРИЯ", "v": 756.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '752.0', 'ШВЕЦИЯ', '{"2": "ШВЕЦИЯ", "v": 752.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '748.0', 'ЭСВАТИНИ', '{"2": "ЭСВАТИНИ", "v": 748.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '744.0', 'ШПИЦБЕРГЕН И ЯН МАЙЕН', '{"2": "ШПИЦБЕРГЕН И ЯН МАЙЕН", "v": 744.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '740.0', 'СУРИНАМ', '{"2": "СУРИНАМ", "v": 740.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '732.0', 'ЗАПАДНАЯ САХАРА', '{"2": "ЗАПАДНАЯ САХАРА", "v": 732.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '729.0', 'СУДАН', '{"2": "СУДАН", "v": 729.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '728.0', 'ЮЖНЫЙ СУДАН', '{"2": "ЮЖНЫЙ СУДАН", "v": 728.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '724.0', 'ИСПАНИЯ', '{"2": "ИСПАНИЯ", "v": 724.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '716.0', 'ЗИМБАБВЕ', '{"2": "ЗИМБАБВЕ", "v": 716.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '710.0', 'ЮЖНАЯ АФРИКА', '{"2": "ЮЖНАЯ АФРИКА", "v": 710.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '706.0', 'СОМАЛИ', '{"2": "СОМАЛИ", "v": 706.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '705.0', 'СЛОВЕНИЯ', '{"2": "СЛОВЕНИЯ", "v": 705.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '704.0', 'ВЬЕТНАМ', '{"2": "ВЬЕТНАМ", "v": 704.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '703.0', 'СЛОВАКИЯ', '{"2": "СЛОВАКИЯ", "v": 703.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '702.0', 'СИНГАПУР', '{"2": "СИНГАПУР", "v": 702.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '694.0', 'СЬЕРРА-ЛЕОНЕ', '{"2": "СЬЕРРА-ЛЕОНЕ", "v": 694.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '690.0', 'СЕЙШЕЛЫ', '{"2": "СЕЙШЕЛЫ", "v": 690.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '688.0', 'СЕРБИЯ', '{"2": "СЕРБИЯ", "v": 688.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '686.0', 'СЕНЕГАЛ', '{"2": "СЕНЕГАЛ", "v": 686.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '682.0', 'САУДОВСКАЯ АРАВИЯ', '{"2": "САУДОВСКАЯ АРАВИЯ", "v": 682.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '678.0', 'САН-ТОМЕ И ПРИНСИПИ', '{"2": "САН-ТОМЕ И ПРИНСИПИ", "v": 678.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '674.0', 'САН-МАРИНО', '{"2": "САН-МАРИНО", "v": 674.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '670.0', 'СЕНТ-ВИНСЕНТ И ГРЕНАДИНЫ', '{"2": "СЕНТ-ВИНСЕНТ И ГРЕНАДИНЫ", "v": 670.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '666.0', 'СЕН-ПЬЕР И МИКЕЛОН', '{"2": "СЕН-ПЬЕР И МИКЕЛОН", "v": 666.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '663.0', 'СЕН-МАРТЕН (французская часть)', '{"2": "СЕН-МАРТЕН (французская часть)", "v": 663.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '662.0', 'СЕНТ-ЛЮСИЯ', '{"2": "СЕНТ-ЛЮСИЯ", "v": 662.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '660.0', 'АНГИЛЬЯ', '{"2": "АНГИЛЬЯ", "v": 660.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '659.0', 'СЕНТ-КИТС И НЕВИС', '{"2": "СЕНТ-КИТС И НЕВИС", "v": 659.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '654.0', 'СВЯТАЯ ЕЛЕНА (ОСТРОВ ВОЗНЕСЕНИЯ  ТРИСТАН-ДА-КУНЬЯ)', '{"2": "СВЯТАЯ ЕЛЕНА (ОСТРОВ ВОЗНЕСЕНИЯ  ТРИСТАН-ДА-КУНЬЯ)", "v": 654.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '652.0', 'СЕН-БАРТЕЛЕМИ', '{"2": "СЕН-БАРТЕЛЕМИ", "v": 652.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '646.0', 'РУАНДА', '{"2": "РУАНДА", "v": 646.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '642.0', 'РУМЫНИЯ', '{"2": "РУМЫНИЯ", "v": 642.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '638.0', 'РЕЮНЬОН', '{"2": "РЕЮНЬОН", "v": 638.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '634.0', 'КАТАР', '{"2": "КАТАР", "v": 634.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '630.0', 'ПУЭРТО-РИКО', '{"2": "ПУЭРТО-РИКО", "v": 630.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '626.0', 'ТИМОР-ЛЕСТЕ', '{"2": "ТИМОР-ЛЕСТЕ", "v": 626.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '624.0', 'ГВИНЕЯ-БИСАУ', '{"2": "ГВИНЕЯ-БИСАУ", "v": 624.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '620.0', 'ПОРТУГАЛИЯ', '{"2": "ПОРТУГАЛИЯ", "v": 620.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '616.0', 'ПОЛЬША', '{"2": "ПОЛЬША", "v": 616.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '612.0', 'ПИТКЕРН', '{"2": "ПИТКЕРН", "v": 612.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '608.0', 'ФИЛИППИНЫ', '{"2": "ФИЛИППИНЫ", "v": 608.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '604.0', 'ПЕРУ', '{"2": "ПЕРУ", "v": 604.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '600.0', 'ПАРАГВАЙ', '{"2": "ПАРАГВАЙ", "v": 600.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '598.0', 'ПАПУА-НОВАЯ ГВИНЕЯ', '{"2": "ПАПУА-НОВАЯ ГВИНЕЯ", "v": 598.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '591.0', 'ПАНАМА', '{"2": "ПАНАМА", "v": 591.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '586.0', 'ПАКИСТАН', '{"2": "ПАКИСТАН", "v": 586.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '585.0', 'ПАЛАУ', '{"2": "ПАЛАУ", "v": 585.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '584.0', 'МАРШАЛЛОВЫ ОСТРОВА', '{"2": "МАРШАЛЛОВЫ ОСТРОВА", "v": 584.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '583.0', 'МИКРОНЕЗИЯ (ФЕДЕРАТИВНЫЕ ШТАТЫ)', '{"2": "МИКРОНЕЗИЯ (ФЕДЕРАТИВНЫЕ ШТАТЫ)", "v": 583.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '580.0', 'СЕВЕРНЫЕ МАРИАНСКИЕ ОСТРОВА', '{"2": "СЕВЕРНЫЕ МАРИАНСКИЕ ОСТРОВА", "v": 580.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '578.0', 'НОРВЕГИЯ', '{"2": "НОРВЕГИЯ", "v": 578.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '574.0', 'ОСТРОВ НОРФОЛК', '{"2": "ОСТРОВ НОРФОЛК", "v": 574.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '566.0', 'НИГЕРИЯ', '{"2": "НИГЕРИЯ", "v": 566.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '562.0', 'НИГЕР', '{"2": "НИГЕР", "v": 562.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '558.0', 'НИКАРАГУА', '{"2": "НИКАРАГУА", "v": 558.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '535.0', 'БОНЭЙР (СИНТ-ЭСТАТИУС И САБА)', '{"2": "БОНЭЙР (СИНТ-ЭСТАТИУС И САБА)", "v": 535.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '534.0', 'СЕН-МАРТЕН (нидерландская часть)', '{"2": "СЕН-МАРТЕН (нидерландская часть)", "v": 534.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '528.0', 'НИДЕРЛАНДЫ', '{"2": "НИДЕРЛАНДЫ", "v": 528.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '508.0', 'МОЗАМБИК', '{"2": "МОЗАМБИК", "v": 508.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '496.0', 'МОНГОЛИЯ', '{"2": "МОНГОЛИЯ", "v": 496.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '492.0', 'МОНАКО', '{"2": "МОНАКО", "v": 492.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '484.0', 'МЕКСИКА', '{"2": "МЕКСИКА", "v": 484.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '440.0', 'ЛИТВА', '{"2": "ЛИТВА", "v": 440.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '430.0', 'ЛИБЕРИЯ', '{"2": "ЛИБЕРИЯ", "v": 430.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '418.0', 'ЛАОССКАЯ НАРОДНО-ДЕМОКРАТИЧЕСКАЯ РЕСПУБЛИКА', '{"2": "ЛАОССКАЯ НАРОДНО-ДЕМОКРАТИЧЕСКАЯ РЕСПУБЛИКА", "v": 418.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '404.0', 'КЕНИЯ', '{"2": "КЕНИЯ", "v": 404.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '400.0', 'ИОРДАНИЯ', '{"2": "ИОРДАНИЯ", "v": 400.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '332.0', 'ГАИТИ', '{"2": "ГАИТИ", "v": 332.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '312.0', 'ГВАДЕЛУПА', '{"2": "ГВАДЕЛУПА", "v": 312.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '304.0', 'ГРЕНЛАНДИЯ', '{"2": "ГРЕНЛАНДИЯ", "v": 304.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '276.0', 'ГЕРМАНИЯ', '{"2": "ГЕРМАНИЯ", "v": 276.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '275.0', 'ПАЛЕСТИНА (ГОСУДАРСТВО)', '{"2": "ПАЛЕСТИНА (ГОСУДАРСТВО)", "v": 275.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '254.0', 'ФРАНЦУЗСКАЯ ГВИАНА', '{"2": "ФРАНЦУЗСКАЯ ГВИАНА", "v": 254.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '246.0', 'ФИНЛЯНДИЯ', '{"2": "ФИНЛЯНДИЯ", "v": 246.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '239.0', 'ЮЖНАЯ ДЖОРДЖИЯ И ЮЖНЫЕ САНДВИЧЕВЫ ОСТРОВА', '{"2": "ЮЖНАЯ ДЖОРДЖИЯ И ЮЖНЫЕ САНДВИЧЕВЫ ОСТРОВА", "v": 239.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '238.0', 'ФОЛКЛЕНДСКИЕ ОСТРОВА (МАЛЬВИНСКИЕ)', '{"2": "ФОЛКЛЕНДСКИЕ ОСТРОВА (МАЛЬВИНСКИЕ)", "v": 238.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '214.0', 'ДОМИНИКАНСКАЯ РЕСПУБЛИКА', '{"2": "ДОМИНИКАНСКАЯ РЕСПУБЛИКА", "v": 214.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '212.0', 'ДОМИНИКА', '{"2": "ДОМИНИКА", "v": 212.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '204.0', 'БЕНИН', '{"2": "БЕНИН", "v": 204.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '203.0', 'ЧЕХИЯ', '{"2": "ЧЕХИЯ", "v": 203.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '196.0', 'КИПР', '{"2": "КИПР", "v": 196.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '188.0', 'КОСТА-РИКА', '{"2": "КОСТА-РИКА", "v": 188.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '152.0', 'ЧИЛИ', '{"2": "ЧИЛИ", "v": 152.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '148.0', 'ЧАД', '{"2": "ЧАД", "v": 148.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '144.0', 'ШРИ-ЛАНКА', '{"2": "ШРИ-ЛАНКА", "v": 144.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '140.0', 'ЦЕНТРАЛЬНО-АФРИКАНСКАЯ РЕСПУБЛИКА', '{"2": "ЦЕНТРАЛЬНО-АФРИКАНСКАЯ РЕСПУБЛИКА", "v": 140.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '136.0', 'ОСТРОВА КАЙМАН', '{"2": "ОСТРОВА КАЙМАН", "v": 136.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '124.0', 'КАНАДА', '{"2": "КАНАДА", "v": 124.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '116.0', 'КАМБОДЖА', '{"2": "КАМБОДЖА", "v": 116.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '104.0', 'МЬЯНМА', '{"2": "МЬЯНМА", "v": 104.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '096', 'БРУНЕЙ-ДАРУССАЛАМ', '{"2": "БРУНЕЙ-ДАРУССАЛАМ", "v": "096"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '092', 'ВИРГИНСКИЕ ОСТРОВА (БРИТАНСКИЕ)', '{"2": "ВИРГИНСКИЕ ОСТРОВА (БРИТАНСКИЕ)", "v": "092"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '090', 'СОЛОМОНОВЫ ОСТРОВА', '{"2": "СОЛОМОНОВЫ ОСТРОВА", "v": "090"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '086', 'БРИТАНСКАЯ ТЕРРИТОРИЯ В ИНДИЙСКОМ ОКЕАНЕ', '{"2": "БРИТАНСКАЯ ТЕРРИТОРИЯ В ИНДИЙСКОМ ОКЕАНЕ", "v": "086"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '076', 'БРАЗИЛИЯ', '{"2": "БРАЗИЛИЯ", "v": "076"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '074', 'ОСТРОВ БУВЕ', '{"2": "ОСТРОВ БУВЕ", "v": "074"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '070', 'БОСНИЯ И ГЕРЦЕГОВИНА', '{"2": "БОСНИЯ И ГЕРЦЕГОВИНА", "v": "070"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '068', 'БОЛИВИЯ (МНОГОНАЦИОНАЛЬНОЕ ГОСУДАРСТВО)', '{"2": "БОЛИВИЯ (МНОГОНАЦИОНАЛЬНОЕ ГОСУДАРСТВО)", "v": "068"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '056', 'БЕЛЬГИЯ', '{"2": "БЕЛЬГИЯ", "v": "056"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '050', 'БАНГЛАДЕШ', '{"2": "БАНГЛАДЕШ", "v": "050"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '032', 'АРГЕНТИНА', '{"2": "АРГЕНТИНА", "v": "032"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '028', 'АНТИГУА И БАРБУДА', '{"2": "АНТИГУА И БАРБУДА", "v": "028"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '020', 'АНДОРРА', '{"2": "АНДОРРА", "v": "020"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '016', 'АМЕРИКАНСКОЕ САМОА', '{"2": "АМЕРИКАНСКОЕ САМОА", "v": "016"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '012', 'АЛЖИР', '{"2": "АЛЖИР", "v": "012"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '008', 'АЛБАНИЯ', '{"2": "АЛБАНИЯ", "v": "008"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '004', 'АФГАНИСТАН', '{"2": "АФГАНИСТАН", "v": "004"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1000000', '1000000 ГОСУДАРСТВЕННАЯ ВЛАСТЬ В РФ. ДРУГИЕ ГОСУДАРСТВЕННЫЕ ОРГАНЫ РФ. ЦЕНТРАЛЬНЫЙ БАНК РФ', '{"2": "ГОСУДАРСТВЕННАЯ ВЛАСТЬ В РФ. ДРУГИЕ ГОСУДАРСТВЕННЫЕ ОРГАНЫ РФ. ЦЕНТРАЛЬНЫЙ БАНК РФ", "m": 1000000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1100000', '1100000 ПРЕЗИДЕНТ РФ', '{"2": "ПРЕЗИДЕНТ РФ", "m": 1100000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1100001', '1100001 Совет Безопасности РФ', '{"2": "Совет Безопасности РФ", "m": 1100001}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1100002', '1100002 Администрация Президента РФ', '{"2": "Администрация Президента РФ", "m": 1100002}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1100003', '1100003 Государственный совет РФ', '{"2": "Государственный совет РФ", "m": 1100003}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1200000', '1200000 ЗАКОНОДАТЕЛЬНАЯ ВЛАСТЬ В РФ', '{"2": "ЗАКОНОДАТЕЛЬНАЯ ВЛАСТЬ В РФ", "m": 1200000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1200100', '1200100 Федеральное Собрание РФ', '{"2": "Федеральное Собрание РФ", "m": 1200100}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1200101', '1200101 Совет Федерации Федерального Собрания РФ', '{"2": "Совет Федерации Федерального Собрания РФ", "m": 1200101}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1200102', '1200102 Государственная Дума Федерального Собрания РФ', '{"2": "Государственная Дума Федерального Собрания РФ", "m": 1200102}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1300000', '1300000 ИСПОЛНИТЕЛЬНАЯ ВЛАСТЬ В РФ', '{"2": "ИСПОЛНИТЕЛЬНАЯ ВЛАСТЬ В РФ", "m": 1300000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1300001', '1300001 Правительство РФ', '{"2": "Правительство РФ", "m": 1300001}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '112', 'БЕЛАРУСЬ', '{"2": "БЕЛАРУСЬ", "3": "СНГ", "v": 112, "attrs1": null, "status": "1"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '336.0', 'ПАПСКИЙ ПРЕСТОЛ (ГОСУДАРСТВО - ГОРОД ВАТИКАН)', '{"2": "ПАПСКИЙ ПРЕСТОЛ (ГОСУДАРСТВО - ГОРОД ВАТИКАН)", "v": 336.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1310000', '1310000 Федеральные министерства, федеральные службы и федеральные агентства, руководство деятельностью которых осуществляет Президент РФ, федеральные службы и федеральные агентства, подведомственные этим федеральным министерствам', '{"2": "Федеральные министерства, федеральные службы и федеральные агентства, руководство деятельностью которых осуществляет Президент РФ, федеральные службы и федеральные агентства, подведомственные этим федеральным министерствам", "m": 1310000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1310500', '1310500 Министерство внутренних дел РФ', '{"2": "Министерство внутренних дел РФ", "m": 1310500}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1311500', '1311500 Министерство РФ по делам гражданской обороны, чрезвычайным ситуациям и ликвидации последствий стихийных бедствий', '{"2": "Министерство РФ по делам гражданской обороны, чрезвычайным ситуациям и ликвидации последствий стихийных бедствий", "m": 1311500}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1312000', '1312000 Министерство иностранных дел РФ', '{"2": "Министерство иностранных дел РФ", "m": 1312000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1312055', '1312055 Федеральное агентство по делам Содружества Независимых Государств, соотечественников, проживающих за рубежом, и по международному гуманитарному сотрудничеству', '{"2": "Федеральное агентство по делам Содружества Независимых Государств, соотечественников, проживающих за рубежом, и по международному гуманитарному сотрудничеству", "m": 1312055}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1313500', '1313500 Министерство обороны РФ', '{"2": "Министерство обороны РФ", "m": 1313500}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1313505', '1313505 Федеральная служба по военно-техническому сотрудничеству', '{"2": "Федеральная служба по военно-техническому сотрудничеству", "m": 1313505}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1313520', '1313520 Федеральная служба по техническому и экспортному контролю', '{"2": "Федеральная служба по техническому и экспортному контролю", "m": 1313520}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1313570', '1313570 Федеральное агентство специального строительства', '{"2": "Федеральное агентство специального строительства", "m": 1313570}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1318000', '1318000 Министерство юстиции РФ', '{"2": "Министерство юстиции РФ", "m": 1318000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1318010', '1318010 Федеральная служба исполнения наказаний', '{"2": "Федеральная служба исполнения наказаний", "m": 1318010}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1318025', '1318025 Федеральная служба судебных приставов', '{"2": "Федеральная служба судебных приставов", "m": 1318025}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1319805', '1319805 Государственная фельдъегерская служба РФ (федеральная служба)', '{"2": "Государственная фельдъегерская служба РФ (федеральная служба)", "m": 1319805}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1319810', '1319810 Служба внешней разведки РФ (федеральная служба)', '{"2": "Служба внешней разведки РФ (федеральная служба)", "m": 1319810}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1319815', '1319815 Федеральная служба безопасности РФ (федеральная служба)', '{"2": "Федеральная служба безопасности РФ (федеральная служба)", "m": 1319815}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1319818', '1319818 Федеральная служба войск национальной гвардии РФ (федеральная служба)', '{"2": "Федеральная служба войск национальной гвардии РФ (федеральная служба)", "m": 1319818}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1319825', '1319825 Федеральная служба охраны РФ (федеральная служба)', '{"2": "Федеральная служба охраны РФ (федеральная служба)", "m": 1319825}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1319830', '1319830 Федеральная служба по финансовому мониторингу (федеральная служба)', '{"2": "Федеральная служба по финансовому мониторингу (федеральная служба)", "m": 1319830}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1319835', '1319835 Федеральное архивное агентство (федеральное агентство)', '{"2": "Федеральное архивное агентство (федеральное агентство)", "m": 1319835}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1319905', '1319905 Главное управление специальных программ Президента РФ (федеральное агентство)', '{"2": "Главное управление специальных программ Президента РФ (федеральное агентство)", "m": 1319905}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1320000', '1320000 Федеральные министерства, руководство деятельностью которых осуществляет Правительство РФ, федеральные службы и федеральные агентства, подведомственные этим федеральным министерствам', '{"2": "Федеральные министерства, руководство деятельностью которых осуществляет Правительство РФ, федеральные службы и федеральные агентства, подведомственные этим федеральным министерствам", "m": 1320000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1320705', '1320705 Федеральная служба по надзору в сфере здравоохранения', '{"2": "Федеральная служба по надзору в сфере здравоохранения", "m": 1320705}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1320760', '1320760 Федеральное медико-биологическое агентство', '{"2": "Федеральное медико-биологическое агентство", "m": 1320760}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1321000', '1321000 Министерство культуры РФ', '{"2": "Министерство культуры РФ", "m": 1321000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1321065', '1321065 Федеральное агентство по туризму', '{"2": "Федеральное агентство по туризму", "m": 1321065}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1322500', '1322500 Министерство образования и науки РФ', '{"2": "Министерство образования и науки РФ", "m": 1322500}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1322520', '1322520 Федеральная служба по надзору в сфере образования и науки', '{"2": "Федеральная служба по надзору в сфере образования и науки", "m": 1322520}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1322555', '1322555 Федеральное агентство по делам молодежи', '{"2": "Федеральное агентство по делам молодежи", "m": 1322555}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1323000', '1323000 Министерство природных ресурсов и экологии РФ', '{"2": "Министерство природных ресурсов и экологии РФ", "m": 1323000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1323005', '1323005 Федеральная служба по гидрометеорологии и мониторингу окружающей среды', '{"2": "Федеральная служба по гидрометеорологии и мониторингу окружающей среды", "m": 1323005}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1323020', '1323020 Федеральная служба по надзору в сфере природопользования', '{"2": "Федеральная служба по надзору в сфере природопользования", "m": 1323020}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1323055', '1323055 Федеральное агентство водных ресурсов', '{"2": "Федеральное агентство водных ресурсов", "m": 1323055}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1323060', '1323060 Федеральное агентство лесного хозяйства', '{"2": "Федеральное агентство лесного хозяйства", "m": 1323060}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1323065', '1323065 Федеральное агентство по недропользованию', '{"2": "Федеральное агентство по недропользованию", "m": 1323065}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1323500', '1323500 Министерство промышленности и торговли РФ', '{"2": "Министерство промышленности и торговли РФ", "m": 1323500}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1323565', '1323565 Федеральное агентство по техническому регулированию и метрологии', '{"2": "Федеральное агентство по техническому регулированию и метрологии", "m": 1323565}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1323700', '1323700 Министерство РФ по развитию Дальнего Востока', '{"2": "Министерство РФ по развитию Дальнего Востока", "m": 1323700}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1324525', '1324525 Федеральная служба по надзору в сфере связи, информационных технологий и массовых коммуникаций', '{"2": "Федеральная служба по надзору в сфере связи, информационных технологий и массовых коммуникаций", "m": 1324525}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1324570', '1324570 Федеральное агентство по печати и массовым коммуникациям', '{"2": "Федеральное агентство по печати и массовым коммуникациям", "m": 1324570}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1324575', '1324575 Федеральное агентство связи', '{"2": "Федеральное агентство связи", "m": 1324575}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1325000', '1325000 Министерство сельского хозяйства РФ', '{"2": "Министерство сельского хозяйства РФ", "m": 1325000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1325005', '1325005 Федеральная служба по ветеринарному и фитосанитарному надзору', '{"2": "Федеральная служба по ветеринарному и фитосанитарному надзору", "m": 1325005}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1325060', '1325060 Федеральное агентство по рыболовству', '{"2": "Федеральное агентство по рыболовству", "m": 1325060}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1325800', '1325800 Министерство строительства и жилищно-коммунального хозяйства РФ', '{"2": "Министерство строительства и жилищно-коммунального хозяйства РФ", "m": 1325800}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1326000', '1326000 Министерство транспорта РФ', '{"2": "Министерство транспорта РФ", "m": 1326000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1326030', '1326030 Федеральная служба по надзору в сфере транспорта', '{"2": "Федеральная служба по надзору в сфере транспорта", "m": 1326030}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '340.0', 'ГОНДУРАС', '{"2": "ГОНДУРАС", "v": 340.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1326055', '1326055 Федеральное агентство воздушного транспорта', '{"2": "Федеральное агентство воздушного транспорта", "m": 1326055}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1326065', '1326065 Федеральное агентство железнодорожного транспорта', '{"2": "Федеральное агентство железнодорожного транспорта", "m": 1326065}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2400100', '2400100 Конституционные (уставные) суды субъектов РФ', '{"2": "Конституционные (уставные) суды субъектов РФ", "m": 2400100}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2400400', '2400400 Мировые судьи', '{"2": "Мировые судьи", "m": 2400400}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2500040', '2500040 Контрольно-счетные органы субъектов РФ', '{"2": "Контрольно-счетные органы субъектов РФ", "m": 2500040}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2500050', '2500050 Избирательные комиссии субъектов РФ', '{"2": "Избирательные комиссии субъектов РФ", "m": 2500050}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3000000', '3000000 МЕСТНОЕ САМОУПРАВЛЕНИЕ В РФ. ИЗБИРАТЕЛЬНЫЕ КОМИССИИ МУНИЦИПАЛЬНЫХ ОБРАЗОВАНИЙ', '{"2": "МЕСТНОЕ САМОУПРАВЛЕНИЕ В РФ. ИЗБИРАТЕЛЬНЫЕ КОМИССИИ МУНИЦИПАЛЬНЫХ ОБРАЗОВАНИЙ", "m": 3000000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3200000', '3200000 ПРЕДСТАВИТЕЛЬНЫЕ ОРГАНЫ МУНИЦИПАЛЬНЫХ ОБРАЗОВАНИЙ', '{"2": "ПРЕДСТАВИТЕЛЬНЫЕ ОРГАНЫ МУНИЦИПАЛЬНЫХ ОБРАЗОВАНИЙ", "m": 3200000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3200100', '3200100 Представительные органы муниципальных районов', '{"2": "Представительные органы муниципальных районов", "m": 3200100}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3200200', '3200200 Представительные органы городских округов', '{"2": "Представительные органы городских округов", "m": 3200200}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3200300', '3200300 Представительные органы внутригородских территорий (внутригородских муниципальных образований) городов федерального значения', '{"2": "Представительные органы внутригородских территорий (внутригородских муниципальных образований) городов федерального значения", "m": 3200300}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3200400', '3200400 Представительные органы городских поселений', '{"2": "Представительные органы городских поселений", "m": 3200400}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3200500', '3200500 Представительные органы сельских поселений', '{"2": "Представительные органы сельских поселений", "m": 3200500}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3300000', '3300000 МЕСТНЫЕ АДМИНИСТРАЦИИ (ИСПОЛНИТЕЛЬНО-РАСПОРЯДИТЕЛЬНЫЕ ОРГАНЫ МУНИЦИПАЛЬНЫХ ОБРАЗОВАНИЙ)', '{"2": "МЕСТНЫЕ АДМИНИСТРАЦИИ (ИСПОЛНИТЕЛЬНО-РАСПОРЯДИТЕЛЬНЫЕ ОРГАНЫ МУНИЦИПАЛЬНЫХ ОБРАЗОВАНИЙ)", "m": 3300000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3300100', '3300100 Местные администрации (исполнительно-распорядительные органы) муниципальных районов', '{"2": "Местные администрации (исполнительно-распорядительные органы) муниципальных районов", "m": 3300100}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3300200', '3300200 Местные администрации (исполнительно-распорядительные органы) городских округов', '{"2": "Местные администрации (исполнительно-распорядительные органы) городских округов", "m": 3300200}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3300300', '3300300 Местные администрации (исполнительно-распорядительные органы) внутригородских территорий (внутригородских муниципальных образований) городов федерального значения', '{"2": "Местные администрации (исполнительно-распорядительные органы) внутригородских территорий (внутригородских муниципальных образований) городов федерального значения", "m": 3300300}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3300400', '3300400 Местные администрации (исполнительно-распорядительные органы) городских поселений', '{"2": "Местные администрации (исполнительно-распорядительные органы) городских поселений", "m": 3300400}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3300500', '3300500 Местные администрации (исполнительно-распорядительные органы) сельских поселений', '{"2": "Местные администрации (исполнительно-распорядительные органы) сельских поселений", "m": 3300500}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3400000', '3400000 КОНТРОЛЬНО-СЧЕТНЫЕ ОРГАНЫ МУНИЦИПАЛЬНЫХ ОБРАЗОВАНИЙ', '{"2": "КОНТРОЛЬНО-СЧЕТНЫЕ ОРГАНЫ МУНИЦИПАЛЬНЫХ ОБРАЗОВАНИЙ", "m": 3400000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3400100', '3400100 Контрольно-счетные органы муниципальных районов', '{"2": "Контрольно-счетные органы муниципальных районов", "m": 3400100}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3400200', '3400200 Контрольно-счетные органы городских округов', '{"2": "Контрольно-счетные органы городских округов", "m": 3400200}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3400300', '3400300 Контрольно-счетные органы внутригородских территорий (внутригородских муниципальных образований) городов федерального значения', '{"2": "Контрольно-счетные органы внутригородских территорий (внутригородских муниципальных образований) городов федерального значения", "m": 3400300}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3400400', '3400400 Контрольно-счетные органы городских поселений', '{"2": "Контрольно-счетные органы городских поселений", "m": 3400400}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3400500', '3400500 Контрольно-счетные органы сельских поселений', '{"2": "Контрольно-счетные органы сельских поселений", "m": 3400500}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3500000', '3500000 ИНЫЕ ОРГАНЫ МЕСТНОГО САМОУПРАВЛЕНИЯ', '{"2": "ИНЫЕ ОРГАНЫ МЕСТНОГО САМОУПРАВЛЕНИЯ", "m": 3500000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3500100', '3500100 Иные органы муниципальных районов', '{"2": "Иные органы муниципальных районов", "m": 3500100}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3500300', '3500300 Иные органы внутригородских территорий городов федерального значения', '{"2": "Иные органы внутригородских территорий городов федерального значения", "m": 3500300}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3500400', '3500400 Иные органы городских поселений', '{"2": "Иные органы городских поселений", "m": 3500400}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3500500', '3500500 Иные органы сельских поселений', '{"2": "Иные органы сельских поселений", "m": 3500500}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3900000', '3900000 ИЗБИРАТЕЛЬНЫЕ КОМИССИИ МУНИЦИПАЛЬНЫХ ОБРАЗОВАНИЙ', '{"2": "ИЗБИРАТЕЛЬНЫЕ КОМИССИИ МУНИЦИПАЛЬНЫХ ОБРАЗОВАНИЙ", "m": 3900000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3900100', '3900100 Избирательные комиссии муниципальных районов', '{"2": "Избирательные комиссии муниципальных районов", "m": 3900100}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3900200', '3900200 Избирательные комиссии городских округов', '{"2": "Избирательные комиссии городских округов", "m": 3900200}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3900300', '3900300 Избирательные комиссии внутригородских территорий городов федерального значения', '{"2": "Избирательные комиссии внутригородских территорий городов федерального значения", "m": 3900300}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3900400', '3900400 Избирательные комиссии городских поселений', '{"2": "Избирательные комиссии городских поселений", "m": 3900400}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4000000', '4000000 ОРГАНИЗАЦИИ, ПО КОТОРЫМ ОСУЩЕСТВЛЯЕТСЯ ФЕДЕРАЛЬНОЕ СТАТИСТИЧЕСКОЕ НАБЛЮДЕНИЕ. ГРУППИРОВКИ ХОЗЯЙСТВУЮЩИХ СУБЪЕКТОВ И ОБЩЕСТВЕННЫХ ОБЪЕДИНЕНИЙ, ИСПОЛЬЗУЕМЫЕ ДЛЯ ОФИЦИАЛЬНОГО СТАТИСТИЧЕСКОГО УЧЕТА', '{"2": "ОРГАНИЗАЦИИ, ПО КОТОРЫМ ОСУЩЕСТВЛЯЕТСЯ ФЕДЕРАЛЬНОЕ СТАТИСТИЧЕСКОЕ НАБЛЮДЕНИЕ. ГРУППИРОВКИ ХОЗЯЙСТВУЮЩИХ СУБЪЕКТОВ И ОБЩЕСТВЕННЫХ ОБЪЕДИНЕНИЙ, ИСПОЛЬЗУЕМЫЕ ДЛЯ ОФИЦИАЛЬНОГО СТАТИСТИЧЕСКОГО УЧЕТА", "m": 4000000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100000', '4100000 ОРГАНИЗАЦИИ, ПО КОТОРЫМ ОСУЩЕСТВЛЯЕТСЯ ФЕДЕРАЛЬНОЕ СТАТИСТИЧЕСКОЕ НАБЛЮДЕНИЕ', '{"2": "ОРГАНИЗАЦИИ, ПО КОТОРЫМ ОСУЩЕСТВЛЯЕТСЯ ФЕДЕРАЛЬНОЕ СТАТИСТИЧЕСКОЕ НАБЛЮДЕНИЕ", "m": 4100000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100102', '4100102 Банк ВТБ (открытое акционерное общество)', '{"2": "Банк ВТБ (открытое акционерное общество)", "m": 4100102}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100103', '4100103 Государственная корпорация "Банк развития и внешнеэкономической деятельности (Внешэкономбанк)"', '{"2": "Государственная корпорация \"Банк развития и внешнеэкономической деятельности (Внешэкономбанк)\"", "m": 4100103}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100104', '4100104 Публичное акционерное общество "Сбербанк России"', '{"2": "Публичное акционерное общество \"Сбербанк России\"", "m": 4100104}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100201', '4100201 Пенсионный фонд РФ', '{"2": "Пенсионный фонд РФ", "m": 4100201}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100202', '4100202 Федеральный фонд обязательного медицинского страхования', '{"2": "Федеральный фонд обязательного медицинского страхования", "m": 4100202}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100203', '4100203 Фонд социального страхования РФ', '{"2": "Фонд социального страхования РФ", "m": 4100203}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100204', '4100204 Федеральное государственное бюджетное учреждение "Российский гуманитарный научный фонд"', '{"2": "Федеральное государственное бюджетное учреждение \"Российский гуманитарный научный фонд\"", "m": 4100204}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100205', '4100205 Федеральное государственное бюджетное учреждение "Российский фонд фундаментальных исследований"', '{"2": "Федеральное государственное бюджетное учреждение \"Российский фонд фундаментальных исследований\"", "m": 4100205}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100206', '4100206 Федеральное государственное бюджетное учреждение "Фонд содействия развитию малых форм предприятий в научно-технической сфере"', '{"2": "Федеральное государственное бюджетное учреждение \"Фонд содействия развитию малых форм предприятий в научно-технической сфере\"", "m": 4100206}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100207', '4100207 Федеральный фонд содействия развитию жилищного строительства', '{"2": "Федеральный фонд содействия развитию жилищного строительства", "m": 4100207}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100208', '4100208 Фонд перспективных исследований', '{"2": "Фонд перспективных исследований", "m": 4100208}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100300', '4100300 Государственные корпорации', '{"2": "Государственные корпорации", "m": 4100300}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100301', '4100301 Государственная корпорация по атомной энергии "Росатом"', '{"2": "Государственная корпорация по атомной энергии \"Росатом\"", "m": 4100301}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100304', '4100304 Государственная корпорация по содействию разработке, производству и экспорту высокотехнологичной промышленной продукции "Ростех"', '{"2": "Государственная корпорация по содействию разработке, производству и экспорту высокотехнологичной промышленной продукции \"Ростех\"", "m": 4100304}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100307', '4100307 Государственная корпорация по космической деятельности "Роскосмос"', '{"2": "Государственная корпорация по космической деятельности \"Роскосмос\"", "m": 4100307}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100401', '4100401 Государственная компания "Российские автомобильные дороги"', '{"2": "Государственная компания \"Российские автомобильные дороги\"", "m": 4100401}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100500', '4100500 Российские академии', '{"2": "Российские академии", "m": 4100500}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100501', '4100501 Федеральное государственное бюджетное учреждение "Российская академия наук"', '{"2": "Федеральное государственное бюджетное учреждение \"Российская академия наук\"", "m": 4100501}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100504', '4100504 Российская академия архитектуры и строительных наук', '{"2": "Российская академия архитектуры и строительных наук", "m": 4100504}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100505', '4100505 Российская академия образования', '{"2": "Российская академия образования", "m": 4100505}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100506', '4100506 Российская академия художеств', '{"2": "Российская академия художеств", "m": 4100506}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100603', '4100603 Открытое акционерное общество "БИОПРЕПАРАТ"', '{"2": "Открытое акционерное общество \"БИОПРЕПАРАТ\"", "m": 4100603}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100604', '4100604 Акционерное общество открытого типа "Концерн "ДеКО"', '{"2": "Акционерное общество открытого типа \"Концерн \"ДеКО\"", "m": 4100604}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100605', '4100605 Открытое акционерное общество "Российская государственная страховая компания"', '{"2": "Открытое акционерное общество \"Российская государственная страховая компания\"", "m": 4100605}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100607', '4100607 Открытое акционерное общество "Российская топливная компания" - Ростоппром', '{"2": "Открытое акционерное общество \"Российская топливная компания\" - Ростоппром", "m": 4100607}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100608', '4100608 Открытое акционерное общество Российская автотранспортная компания "Росавтотранс"', '{"2": "Открытое акционерное общество Российская автотранспортная компания \"Росавтотранс\"", "m": 4100608}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100609', '4100609 Открытое акционерное общество "Государственное акционерное общество "Всероссийский выставочный центр"', '{"2": "Открытое акционерное общество \"Государственное акционерное общество \"Всероссийский выставочный центр\"", "m": 4100609}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100610', '4100610 Открытое акционерное общество "Россевзапстрой"', '{"2": "Открытое акционерное общество \"Россевзапстрой\"", "m": 4100610}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100612', '4100612 Открытое акционерное общество "Российские железные дороги"', '{"2": "Открытое акционерное общество \"Российские железные дороги\"", "m": 4100612}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100614', '4100614 Открытое акционерное общество "Нефтяная компания "Роснефть"', '{"2": "Открытое акционерное общество \"Нефтяная компания \"Роснефть\"", "m": 4100614}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100615', '4100615 Публичное акционерное общество "Нефтяная компания "ЛУКОЙЛ"', '{"2": "Публичное акционерное общество \"Нефтяная компания \"ЛУКОЙЛ\"", "m": 4100615}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2300280', '2300280 Органы исполнительной власти субъектов РФ, осуществляющие другие функции', '{"2": "Органы исполнительной власти субъектов РФ, осуществляющие другие функции", "m": 2300280}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100617', '4100617 Открытое акционерное общество "Акционерная компания по транспорту нефти "Транснефть"', '{"2": "Открытое акционерное общество \"Акционерная компания по транспорту нефти \"Транснефть\"", "m": 4100617}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100618', '4100618 Открытое акционерное общество "Акционерная компания трубопроводного транспорта нефтепродуктов "Транснефтепродукт"', '{"2": "Открытое акционерное общество \"Акционерная компания трубопроводного транспорта нефтепродуктов \"Транснефтепродукт\"", "m": 4100618}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100622', '4100622 Открытое акционерное общество "Газпром нефть"', '{"2": "Открытое акционерное общество \"Газпром нефть\"", "m": 4100622}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100623', '4100623 Открытое акционерное общество "РОСНАНО"', '{"2": "Открытое акционерное общество \"РОСНАНО\"", "m": 4100623}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100900', '4100900 Другие организации, по которым осуществляется федеральное статистическое наблюдение', '{"2": "Другие организации, по которым осуществляется федеральное статистическое наблюдение", "m": 4100900}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100902', '4100902 Межреспубликанская ассоциация учебных заведений по подготовке и переподготовке кадров машиностроения "Машиностроитель"', '{"2": "Межреспубликанская ассоциация учебных заведений по подготовке и переподготовке кадров машиностроения \"Машиностроитель\"", "m": 4100902}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100903', '4100903 Центральный союз потребительских обществ РФ', '{"2": "Центральный союз потребительских обществ РФ", "m": 4100903}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100907', '4100907 Агентство стратегических инициатив по продвижению новых проектов', '{"2": "Агентство стратегических инициатив по продвижению новых проектов", "m": 4100907}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4200000', '4200000 ГРУППИРОВКИ ХОЗЯЙСТВУЮЩИХ СУБЪЕКТОВ И ОБЩЕСТВЕННЫХ ОБЪЕДИНЕНИЙ, ИСПОЛЬЗУЕМЫЕ ДЛЯ ОФИЦИАЛЬНОГО СТАТИСТИЧЕСКОГО УЧЕТА', '{"2": "ГРУППИРОВКИ ХОЗЯЙСТВУЮЩИХ СУБЪЕКТОВ И ОБЩЕСТВЕННЫХ ОБЪЕДИНЕНИЙ, ИСПОЛЬЗУЕМЫЕ ДЛЯ ОФИЦИАЛЬНОГО СТАТИСТИЧЕСКОГО УЧЕТА", "m": 4200000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4210000', '4210000 Группировки хозяйствующих субъектов', '{"2": "Группировки хозяйствующих субъектов", "m": 4210000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4210001', '4210001 Хозяйственные общества, образованные из государственных предприятий, добровольных объединений государственных предприятий', '{"2": "Хозяйственные общества, образованные из государственных предприятий, добровольных объединений государственных предприятий", "m": 4210001}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4210004', '4210004 Акционерные общества, товарищества, сельскохозяйственные производственные кооперативы, образованные при реорганизации колхозов и совхозов и приватизации отдельных сельскохозяйственных предприятий', '{"2": "Акционерные общества, товарищества, сельскохозяйственные производственные кооперативы, образованные при реорганизации колхозов и совхозов и приватизации отдельных сельскохозяйственных предприятий", "m": 4210004}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4210005', '4210005 Крестьянские (фермерские) хозяйства', '{"2": "Крестьянские (фермерские) хозяйства", "m": 4210005}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4210007', '4210007 Муниципальные организации', '{"2": "Муниципальные организации", "m": 4210007}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4210008', '4210008 Приватизированные предприятия', '{"2": "Приватизированные предприятия", "m": 4210008}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4210009', '4210009 Иностранные юридические лица и их представительства, осуществляющие деятельность на территории РФ (кроме некоммерческих неправительственных организаций)', '{"2": "Иностранные юридические лица и их представительства, осуществляющие деятельность на территории РФ (кроме некоммерческих неправительственных организаций)", "m": 4210009}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4210011', '4210011 Хозяйственные общества и товарищества с участием иностранных юридических и (или) физических лиц, а также лиц без гражданства', '{"2": "Хозяйственные общества и товарищества с участием иностранных юридических и (или) физических лиц, а также лиц без гражданства", "m": 4210011}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2300236', '2300236 - архитектуры и строительства', '{"2": "- архитектуры и строительства", "m": 2300236}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4210012', '4210012 Представительства (посольства) иностранных государств, иные представительства органов управления иностранных государств, отделения, филиалы, представительства иностранных некоммерческих неправительственных организаций', '{"2": "Представительства (посольства) иностранных государств, иные представительства органов управления иностранных государств, отделения, филиалы, представительства иностранных некоммерческих неправительственных организаций", "m": 4210012}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4210014', '4210014 Организации, учрежденные юридическими лицами или гражданами, или юридическими лицами и гражданами совместно', '{"2": "Организации, учрежденные юридическими лицами или гражданами, или юридическими лицами и гражданами совместно", "m": 4210014}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4210015', '4210015 Индивидуальные предприниматели', '{"2": "Индивидуальные предприниматели", "m": 4210015}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4210020', '4210020 Казачьи общества, созданные в форме некоммерческой организации и внесенные в государственный реестр казачьих обществ в РФ', '{"2": "Казачьи общества, созданные в форме некоммерческой организации и внесенные в государственный реестр казачьих обществ в РФ", "m": 4210020}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4210021', '4210021 Добровольные объединения (ассоциации) экономического взаимодействия субъектов РФ, органов местного самоуправления', '{"2": "Добровольные объединения (ассоциации) экономического взаимодействия субъектов РФ, органов местного самоуправления", "m": 4210021}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4210090', '4210090 Другие хозяйствующие субъекты', '{"2": "Другие хозяйствующие субъекты", "m": 4210090}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '5100001', '5100001 Межгосударственный авиационный комитет - исполнительный орган Содружества Независимых Государств', '{"2": "Межгосударственный авиационный комитет - исполнительный орган Содружества Независимых Государств", "m": 5100001}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '5100002', '5100002 Комитет по делам воинов-интернационалистов при Совете глав правительств государств - участников Содружества Независимых Государств', '{"2": "Комитет по делам воинов-интернационалистов при Совете глав правительств государств - участников Содружества Независимых Государств", "m": 5100002}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '5100003', '5100003 Электроэнергетический совет Содружества Независимых Государств', '{"2": "Электроэнергетический совет Содружества Независимых Государств", "m": 5100003}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1320700', '1320700 Министерство здравоохранения Российской Федерации', '{"2": "Министерство здравоохранения Российской Федерации", "m": 1320700}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('FormSobst_voc', '1', '1 Государственная. Федеральная собственность.', '{"FSCod": "1", "FSName": "Государственная. Федеральная собственность.", "attrs1": null, "status": "1"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('12_voc', '12', '12', '{"DocCod": 12, "DocName": "Распоряжение"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('12_voc', '11', '11', '{"DocCod": 11, "DocName": "Протокол"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('12_voc', '10', '10', '{"DocCod": 10, "DocName": "Приказ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('12_voc', '9', '9', '{"DocCod": 9, "DocName": "Паспорт"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('12_voc', '8', '8', '{"DocCod": 8, "DocName": "Ордер"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('12_voc', '7', '7', '{"DocCod": 7, "DocName": "Наряд"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('12_voc', '6', '6', '{"DocCod": 6, "DocName": "Накладная"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('12_voc', '5', '5', '{"DocCod": 5, "DocName": "Карта"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('12_voc', '4', '4', '{"DocCod": 4, "DocName": "Журнал"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('12_voc', '3', '3', '{"DocCod": 3, "DocName": "Грузовая таможенная декларация"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('12_voc', '2', '2', '{"DocCod": 2, "DocName": "Ведомость"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('12_voc', '1', '1', '{"DocCod": 1, "DocName": "Акт"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '01', '01', '{"prim": "Используется для отражения наличия РАО на 31.12.2021 по учетным данным организаций", "Op_Cod": "01", "attrs1": null, "status": "1", "Op_Name": "Наличие на дату 31.12.2021", "deadline": "30– для РАО", "import_hash": null, "import_author": null, "import_src_id": null}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '344.0', 'ГОНКОНГ', '{"2": "ГОНКОНГ", "v": 344.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2300232', '2300232 - связи и средств массовой информации', '{"2": "- связи и средств массовой информации", "m": 2300232}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2300234', '2300234 - транспорта и дорожного строительства', '{"2": "- транспорта и дорожного строительства", "m": 2300234}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2300230', '2300230 - жилищно-коммунального хозяйства', '{"2": "- жилищно-коммунального хозяйства", "m": 2300230}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2300231', '2300231 - культуры', '{"2": "- культуры", "m": 2300231}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '348.0', 'ВЕНГРИЯ', '{"2": "ВЕНГРИЯ", "v": 348.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2300228', '2300228 - охраны окружающей среды', '{"2": "- охраны окружающей среды", "m": 2300228}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2300229', '2300229 - здравоохранения', '{"2": "- здравоохранения", "m": 2300229}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2300225', '2300225 - физической культуры и спорта', '{"2": "- физической культуры и спорта", "m": 2300225}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2300227', '2300227 - предупреждения и ликвидации последствий стихийных бедствий', '{"2": "- предупреждения и ликвидации последствий стихийных бедствий", "m": 2300227}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2300220', '2300220 - труда и социальных вопросов', '{"2": "- труда и социальных вопросов", "m": 2300220}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2300223', '2300223 - образования и науки', '{"2": "- образования и науки", "m": 2300223}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2300217', '2300217 - промышленности и торговли', '{"2": "- промышленности и торговли", "m": 2300217}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2300219', '2300219 - сельского хозяйства', '{"2": "- сельского хозяйства", "m": 2300219}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '356.0', 'ИНДИЯ', '{"2": "ИНДИЯ", "v": 356.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '352.0', 'ИСЛАНДИЯ', '{"2": "ИСЛАНДИЯ", "v": 352.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2300216', '2300216 - экономики', '{"2": "- экономики", "m": 2300216}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '360.0', 'ИНДОНЕЗИЯ', '{"2": "ИНДОНЕЗИЯ", "v": 360.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2300215', '2300215 Органы исполнительной власти субъектов РФ, осуществляющие функции в области финансирования', '{"2": "Органы исполнительной власти субъектов РФ, осуществляющие функции в области финансирования", "m": 2300215}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2300001', '2300001 Правительства (Администрации) субъектов РФ', '{"2": "Правительства (Администрации) субъектов РФ", "m": 2300001}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2300200', '2300200 Органы исполнительной власти субъектов РФ', '{"2": "Органы исполнительной власти субъектов РФ", "m": 2300200}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2300000', '2300000 ИСПОЛНИТЕЛЬНАЯ ВЛАСТЬ СУБЪЕКТОВ РФ', '{"2": "ИСПОЛНИТЕЛЬНАЯ ВЛАСТЬ СУБЪЕКТОВ РФ", "m": 2300000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '364.0', 'ИРАН (ИСЛАМСКАЯ РЕСПУБЛИКА)', '{"2": "ИРАН (ИСЛАМСКАЯ РЕСПУБЛИКА)", "v": 364.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2100000', '2100000 ВЫСШЕЕ ДОЛЖНОСТНОЕ ЛИЦО СУБЪЕКТА РФ', '{"2": "ВЫСШЕЕ ДОЛЖНОСТНОЕ ЛИЦО СУБЪЕКТА РФ", "m": 2100000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2200000', '2200000 ЗАКОНОДАТЕЛЬНАЯ (ПРЕДСТАВИТЕЛЬНАЯ) ВЛАСТЬ СУБЪЕКТОВ РФ', '{"2": "ЗАКОНОДАТЕЛЬНАЯ (ПРЕДСТАВИТЕЛЬНАЯ) ВЛАСТЬ СУБЪЕКТОВ РФ", "m": 2200000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '368.0', 'ИРАК', '{"2": "ИРАК", "v": 368.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1500010', '1500010 Центральный банк РФ', '{"2": "Центральный банк РФ", "m": 1500010}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2000000', '2000000 ГОСУДАРСТВЕННАЯ ВЛАСТЬ В СУБЪЕКТАХ РФ. ДРУГИЕ ГОСУДАРСТВЕННЫЕ ОРГАНЫ СУБЪЕКТОВ РФ', '{"2": "ГОСУДАРСТВЕННАЯ ВЛАСТЬ В СУБЪЕКТАХ РФ. ДРУГИЕ ГОСУДАРСТВЕННЫЕ ОРГАНЫ СУБЪЕКТОВ РФ", "m": 2000000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '372.0', 'ИРЛАНДИЯ', '{"2": "ИРЛАНДИЯ", "v": 372.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1500005', '1500005 Уполномоченный при Президенте РФ по защите прав предпринимателей', '{"2": "Уполномоченный при Президенте РФ по защите прав предпринимателей", "m": 1500005}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1500004', '1500004 Уполномоченный по правам человека в РФ', '{"2": "Уполномоченный по правам человека в РФ", "m": 1500004}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1500003', '1500003 Центральная избирательная комиссия РФ', '{"2": "Центральная избирательная комиссия РФ", "m": 1500003}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '376.0', 'ИЗРАИЛЬ', '{"2": "ИЗРАИЛЬ", "v": 376.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1500002', '1500002 Счетная палата РФ', '{"2": "Счетная палата РФ", "m": 1500002}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1400050', '1400050 Следственный комитет РФ', '{"2": "Следственный комитет РФ", "m": 1400050}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1500000', '1500000 ДРУГИЕ ГОСУДАРСТВЕННЫЕ ОРГАНЫ РФ. ЦЕНТРАЛЬНЫЙ БАНК РФ', '{"2": "ДРУГИЕ ГОСУДАРСТВЕННЫЕ ОРГАНЫ РФ. ЦЕНТРАЛЬНЫЙ БАНК РФ", "m": 1500000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1400027', '1400027 Система федеральных арбитражных судов', '{"2": "Система федеральных арбитражных судов", "m": 1400027}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1400040', '1400040 Система прокуратуры РФ', '{"2": "Система прокуратуры РФ", "m": 1400040}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1400025', '1400025 Судебный департамент при Верховном Суде РФ', '{"2": "Судебный департамент при Верховном Суде РФ", "m": 1400025}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1400026', '1400026 Система федеральных судов общей юрисдикции', '{"2": "Система федеральных судов общей юрисдикции", "m": 1400026}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('110_voc', 'Крымский', '21 Крымский', '{"PolCod": "21", "status": "0", "PolName": "Крымский"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '73', '73', '{"prim": "Постановка на учет РВ или РАО, обнаруженного (найденного), информация об учете которого отсутствовала в момент его обнаружения в организации. В этом случае необходимо представить копию документа (протокола, акта, прочего документа), характеризующего обстоятельства обнаружения учетной единицы", "Op_Cod": "73", "Op_Name": "Обнаружение неучтенного", "deadline": "5 – для РВ; 5 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '71', '71', '{"prim": "Используется при установлении факта отсутствия учетной единицы (вследствие утери, хищения, прочего). В этом случае необходимо представить копию документа (протокола, акта, прочего документа), характеризующего обстоятельства установления факта утери", "Op_Cod": "71", "Op_Name": "Утеря", "deadline": "1 – для РВ; 1 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '72', '72', '{"prim": "Используется в случаях, когда место нахождения учетной единицы известно, а возможность контроля ее состояния отсутствует (оставление ЗРИ в скважине, прочее). В этом случае необходимо представить копию документа (протокола, акта, прочего документа), характеризующего обстоятельства установления факта утери контроля над учетной единицей", "Op_Cod": "72", "Op_Name": "Утрата контроля при известном местоположении", "deadline": "10 – для РВ; 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('nuclid_voc', '1', 'Трансурановые радионуклиды', '{"status": "", "caption": "Трансурановые радионуклиды", "num_kod_rao": 1, "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('nuclid_voc', '2', 'Альфа-излучающие радионуклиды без трансурановых', '{"status": "", "caption": "Альфа-излучающие радионуклиды без трансурановых", "num_kod_rao": 2, "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('nuclid_voc', '3', 'Альфа-излучающие радионуклиды с трансурановыми', '{"status": "", "caption": "Альфа-излучающие радионуклиды с трансурановыми", "num_kod_rao": 3, "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('nuclid_voc', '4', 'Бета-, гамма-излучающие радионуклиды', '{"status": "", "caption": "Бета-, гамма-излучающие радионуклиды", "num_kod_rao": 4, "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('nuclid_voc', '5', 'Альфа-, бета-, гамма-излучающие радионуклиды без трансурановых', '{"status": "", "caption": "Альфа-, бета-, гамма-излучающие радионуклиды без трансурановых", "num_kod_rao": 5, "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('nuclid_voc', '6', 'Альфа-, бета-, гамма-излучающие радионуклиды с трансурановыми', '{"status": "", "caption": "Альфа-, бета-, гамма-излучающие радионуклиды с трансурановыми", "num_kod_rao": 6, "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('112_voc', '2:', '2:', '{"prim": "", "g1_cod": "2:", "status": "0", "g1_types": "ОРИ, кроме отдельных изделий, за исключением радиофармпрепаратов"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('nuclid_voc', '0', 'Состав неизвестен', '{"status": "", "caption": "Состав неизвестен", "num_kod_rao": 0, "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1400000', '1400000 СУДЕБНАЯ ВЛАСТЬ В РФ', '{"2": "СУДЕБНАЯ ВЛАСТЬ В РФ", "m": 1400000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1400010', '1400010 Конституционный суд РФ', '{"2": "Конституционный суд РФ", "m": 1400010}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1400020', '1400020 Верховный Суд РФ', '{"2": "Верховный Суд РФ", "m": 1400020}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'цезий-137', 'цезий-137', '{"AM": 137, "MOI": 0.1, "MZA": 10000, "MZUA": 10000, "D_val": 0.1, "Num_TM": 55, "Name_RN": "цезий-137", "UdA_GRO": 1.1, "UdA_TRO": 10, "ObA_GaRO": 27, "Kod_gruppy": "б", "Period_p_r": 30.2, "Vid_izluch": "бета", "Name_RN_Lat": "Cs-137", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('112_voc', '3:', '3:', '{"prim": "", "g1_cod": "3:", "status": "0", "g1_types": "Радиофармпрепараты"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '643', 'РОССИЯ', '{"2": "РОССИЯ", "3": "СНГ", "v": 643, "attrs1": null, "status": "1"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кобальт-56', 'кобальт-56', '{"AM": 56, "MOI": 0.1, "MZA": 100000, "MZUA": 10000, "D_val": 0.02, "Num_TM": 27, "Name_RN": "кобальт-56", "UdA_GRO": 5.5, "UdA_TRO": 10, "ObA_GaRO": 24, "Kod_gruppy": "б", "Period_p_r": 77.236, "Vid_izluch": "бета", "Name_RN_Lat": "Co-56", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'лютеций-174м', 'лютеций-174м', '{"AM": 174, "MOI": "", "MZA": 10000000, "MZUA": 100000, "D_val": 0.6, "Num_TM": 71, "Name_RN": "лютеций-174м", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 142, "Vid_izluch": "гамма", "Name_RN_Lat": "Lu-174m", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "изомерный переход"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иттербий-166', 'иттербий-166', '{"AM": 166, "MOI": "", "MZA": 10000000, "MZUA": 100000, "D_val": "", "Num_TM": 70, "Name_RN": "иттербий-166", "UdA_GRO": "", "UdA_TRO": "", "ObA_GaRO": "", "Kod_gruppy": "б", "Period_p_r": 2.36, "Vid_izluch": "гамма", "Name_RN_Lat": "Yb-166", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "захват электрона"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1330612', '1330612 Федеральное агентство научных организаций', '{"2": "Федеральное агентство научных организаций", "m": 1330612}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1330613', '1330613 Федеральное агентство по делам национальностей', '{"2": "Федеральное агентство по делам национальностей", "m": 1330613}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '804', 'УКРАИНА', '{"2": "УКРАИНА", "3": "СНГ", "v": 804, "attrs1": null, "status": "1"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'уран-238', 'уран-238', '{"AM": 238, "MOI": "", "MZA": 10000, "MZUA": 10000, "D_val": "неогран.", "Num_TM": 92, "Name_RN": "уран-238", "UdA_GRO": 0.3, "UdA_TRO": 10, "ObA_GaRO": 0.041, "Kod_gruppy": "а", "Period_p_r": 4470000000, "Vid_izluch": "альфа", "Name_RN_Lat": "U-238", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'кобальт-60', 'кобальт-60', '{"AM": 60, "MOI": 0.1, "MZA": 100000, "MZUA": 10000, "D_val": 0.03, "Num_TM": 27, "Name_RN": "кобальт-60", "UdA_GRO": 4, "UdA_TRO": 10, "ObA_GaRO": 11, "Kod_gruppy": "б", "Period_p_r": 5.27, "Vid_izluch": "бета", "Name_RN_Lat": "Co-60", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'иттрий-90', 'иттрий-90', '{"AM": 90, "MOI": 1000, "MZA": 100000, "MZUA": 1000000, "D_val": 5, "Num_TM": 39, "Name_RN": "иттрий-90", "UdA_GRO": 5.1, "UdA_TRO": 1000, "ObA_GaRO": 60, "Kod_gruppy": "б", "Period_p_r": 2.67, "Vid_izluch": "бета", "Name_RN_Lat": "Y-90", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'америций-241', 'америций-241', '{"AM": 241, "MOI": 0.1, "MZA": 10000, "MZUA": 1000, "D_val": 0.06, "Num_TM": 95, "Name_RN": "америций-241", "UdA_GRO": 0.069, "UdA_TRO": 1, "ObA_GaRO": 0.0029, "Kod_gruppy": "Трансурановый", "Period_p_r": 432, "Vid_izluch": "альфа", "Name_RN_Lat": "Am-241", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'тритий', 'тритий', '{"AM": 3, "MOI": 100, "MZA": 1000000000, "MZUA": 1000000000, "D_val": 2000, "Num_TM": 1, "Name_RN": "тритий", "UdA_GRO": 1000, "UdA_TRO": 1000000, "ObA_GaRO": 1900, "Kod_gruppy": "Тритий", "Period_p_r": 12.3, "Vid_izluch": "бета", "Name_RN_Lat": "H-3", "Edinica_izmer_p_r": "лет", "Osn_vid_raspad_text": "бета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc1', 'эйнштейний-253', 'эйнштейний-253', '{"AM": 253, "MOI": 100, "MZA": 100000, "MZUA": 100000, "D_val": "", "Num_TM": 99, "Name_RN": "эйнштейний-253", "UdA_GRO": 2.2, "UdA_TRO": 100, "ObA_GaRO": 0.04, "Kod_gruppy": "Трансурановый", "Period_p_r": 20.5, "Vid_izluch": "альфа", "Name_RN_Lat": "Es-253", "Edinica_izmer_p_r": "сут", "Osn_vid_raspad_text": "альфа"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1330455', '1330455 Федеральная служба по экологическому, технологическому и атомному надзору', '{"2": "Федеральная служба по экологическому, технологическому и атомному надзору", "m": 1330455}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1330415', '1330415 Федеральная служба по надзору в сфере защиты прав потребителей и благополучия человека', '{"2": "Федеральная служба по надзору в сфере защиты прав потребителей и благополучия человека", "m": 1330415}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1330405', '1330405 Федеральная антимонопольная служба', '{"2": "Федеральная антимонопольная служба", "m": 1330405}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1330410', '1330410 Федеральная служба государственной статистики', '{"2": "Федеральная служба государственной статистики", "m": 1330410}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1328500', '1328500 Министерство энергетики РФ', '{"2": "Министерство энергетики РФ", "m": 1328500}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1330000', '1330000 Федеральные службы и федеральные агентства, руководство деятельностью которых осуществляет Правительство РФ', '{"2": "Федеральные службы и федеральные агентства, руководство деятельностью которых осуществляет Правительство РФ", "m": 1330000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '380.0', 'ИТАЛИЯ', '{"2": "ИТАЛИЯ", "v": 380.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1328080', '1328080 Федеральное агентство по управлению государственным имуществом', '{"2": "Федеральное агентство по управлению государственным имуществом", "m": 1328080}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '384.0', 'КОТ ДИВУАР', '{"2": "КОТ ДИВУАР", "v": 384.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1328060', '1328060 Федеральное агентство по государственным резервам', '{"2": "Федеральное агентство по государственным резервам", "m": 1328060}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('no_voc', 'ФГУП "НО РАО" (центральный офис)', 'ФГУП "НО РАО" (центральный офис)', '{"Subdiv": "ФГУП \"НО РАО\" (центральный офис)", "status": "", "Subdiv_OKPO": "12004368", "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('no_voc', 'Филиал "Димитровградский" ФГУП "НО РАО"', 'Филиал "Димитровградский" ФГУП "НО РАО"', '{"Subdiv": "Филиал \"Димитровградский\" ФГУП \"НО РАО\"", "status": "", "Subdiv_OKPO": "87810689", "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('no_voc', 'Филиал "Железногорский" ФГУП "НО РАО"', 'Филиал "Железногорский" ФГУП "НО РАО"', '{"Subdiv": "Филиал \"Железногорский\" ФГУП \"НО РАО\"", "status": "", "Subdiv_OKPO": "13442643", "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('no_voc', 'ФГУП "НО РАО" филиал "Северский"', 'ФГУП "НО РАО" филиал "Северский"', '{"Subdiv": "ФГУП \"НО РАО\" филиал \"Северский\"", "status": "", "Subdiv_OKPO": "44230741", "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('no_voc', 'Отделение "Новоуральское" филиала "Северский" ФГУП "НО РАО"', 'Отделение "Новоуральское" филиала "Северский" ФГУП "НО РАО"', '{"Subdiv": "Отделение \"Новоуральское\" филиала \"Северский\" ФГУП \"НО РАО\"", "status": "", "Subdiv_OKPO": "12004368650002", "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('no_voc', 'Филиал «Озерский» ФГУП "НО РАО"', 'Филиал «Озерский» ФГУП "НО РАО"', '{"Subdiv": "Филиал «Озерский» ФГУП \"НО РАО\"", "status": "", "Subdiv_OKPO": "?", "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '12', '12', '{"prim": "Используется для постановки на учёт: 1) РВ, которые были сняты с учета в качестве ЯМ в СГУК ЯМ; 2) РАО, образовавшихся при переводе в категорию РАО ядерных материалов, которые стояли на учёте в СГУК ЯМ. Код не может быть использован для постановки на учёт ЗРИ, ОРИ или ИОУ, изготовленных (поступивших на склад готовой продукции) с использованием ЯМ, стоявших на учёте в СГУК ЯМ. Для этого следует использовать операцию с кодом 11", "Op_Cod": "12", "attrs1": null, "status": "1", "Op_Name": "Перевод из СГУК ЯМ в СГУК РВ и РАО", "kod_org": "0", "deadline": "10 – для РВ; 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '388.0', 'ЯМАЙКА', '{"2": "ЯМАЙКА", "v": 388.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '392.0', 'ЯПОНИЯ', '{"2": "ЯПОНИЯ", "v": 392.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '51', '51', '{"prim": "Операция используется для отражения изъятия РАО из пункта хранения (места сбора и/или временного хранения) при перемещении учетной единицы из пункта хранения (места сбора и/или временного хранения) организации в пункт хранения/захоронения (места сбора и/или временного хранения) той же организации. Должна использоваться совместно с операцией с кодом 52. Операция не может быть использована при перемещении РАО между пунктами хранения (местами сбора и/или временного хранения) организации, находящимися в разных территориально обособленных подразделениях. В этом случае следует использовать операцию с кодом 21.", "Op_Cod": "51", "Op_Name": "Изъятие из пункта хранения", "deadline": "10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1328040', '1328040 Федеральная служба по интеллектуальной собственности', '{"2": "Федеральная служба по интеллектуальной собственности", "m": 1328040}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '99', '99', '{"prim": "Используется в тех случаях, когда не может быть использован ни один из кодов, перечисленных выше. Обязательным является включение в форму отчета примечаний, описывающих операцию, а также указание, производится снятие или постановка на учет РВ и РАО.", "Op_Cod": "99", "Op_Name": "Прочие операции", "deadline": "10 – для РВ; 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '88', '88', '{"prim": "Используется для отражения факта возврата (ввоза) в Российскую Федерацию ЗРИ/ОЗИИИ, ранее экспортированного из Российской Федерации с регистрацией данного факта в отчетности посредством кода операции 81.", "Op_Cod": "88", "Op_Name": "Ввоз (возврат) на территорию Российской Федерации, ранее экспортированного ЗРИ (ОЗИИИ)", "deadline": "10 – для РВ 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '87', '87', '{"prim": "Используется для отражения факта возврата учетной единицы, информация о вывозе которой прежде представлена операцией с кодом 82. Учетная единица ставится на учет в организации, осуществившей возврат временно вывезенной учетной единицы.", "Op_Cod": "87", "Op_Name": "Возврат (ввоз) на территорию Российской Федерации временно вывезенного ранее", "deadline": "10 – для РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '97', '97', '{"prim": "Используется в тех случаях, когда операция связана с постановкой учетной единицы на учет, но не может быть обозначена кодами, перечисленными выше. Обязательным является включение в форму отчета примечаний, описывающих операцию и причину постановки объекта на учет.", "Op_Cod": "97", "Op_Name": "Постановка на учет по другим причинам", "deadline": "10 – для РВ; 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '98', '98', '{"prim": "Используется в тех случаях, когда операция связана со снятием учетной единицы с учета, но не может быть обозначена кодами, перечисленными выше. Обязательным является включение в форму отчета примечаний, описывающих операцию и причину снятия учетной единицы с учета.", "Op_Cod": "98", "Op_Name": "Снятие с учета по другим причинам", "deadline": "10 – для РВ; 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '18', '18', '{"prim": "Используется для постановки на учет учетной единицы, сведения о количественных или качественных характеристиках которой изменились в результате проведения измерений. При этом учетная единица, стоявшая на учете до проведения измерений, снимается с учета, и информация об этом предоставляется с кодом операции 68", "Op_Cod": "18", "Op_Name": "Постановка на учет при изменении характеристик по результатам проведения измерений", "deadline": "10 – для РВ; 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1328005', '1328005 Федеральная служба по аккредитации', '{"2": "Федеральная служба по аккредитации", "m": 1328005}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1328030', '1328030 Федеральная служба государственной регистрации, кадастра и картографии', '{"2": "Федеральная служба государственной регистрации, кадастра и картографии", "m": 1328030}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '13', '13', '{"prim": "Используется для постановки на учет РАО, образовавшихся при обслуживании и эксплуатации пунктов хранения/захоронения РАО (образование РАО за счет переноса радионуклидов из РАО, ранее поставленных на учет: на спецодежду, оборудование, в дренажные воды пункта хранения, прочее)", "Op_Cod": "13", "Op_Name": "Образование РАО при обслуживании и эксплуатации пунктов хранения/ захоронения РАО", "deadline": "10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '14', '14', '{"prim": "Используется для постановки на учет РАО, образовавшихся из РВ, которые учитываются только в организациях, их использующих, и/или не подлежат учету и контролю в СГУК РВ и РАО в соответствии с критериями и требованиями действующих нормативных документов", "Op_Cod": "14", "Op_Name": "Постановка на учет РАО, образовавшихся из РВ и объектов, загрязненных радионуклидами, не подлежащих учету в СГУК РВ и РАО", "deadline": "10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '15', '15', '{"prim": "Используется для постановки на учет РВ, образовавшихся при переработке РАО в виде ОЗИИИ. Информация о РАО, поступивших на переработку представляется с кодом операции 44", "Op_Cod": "15", "Op_Name": "Возврат в оборот РВ при переработке ОЗИИИ", "deadline": "10 – для РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '16', '16', '{"prim": "Используется для постановки на учет РАО, образовавшихся при проведении работ по выводу из эксплуатации объектов использования атомной энергии или при реабилитации загрязненных территорий", "Op_Cod": "16", "Op_Name": "Постановка на учет РАО, образовавшихся при проведении работ по выводу из эксплуатации ОИАЭ или при реабилитации загрязненных территорий", "deadline": "10- для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '17', '17', '{"prim": "Используется для постановки на учет РВ,  обращение с которыми до вступления в силу настоящего приказа осуществлялось воинскими частями и организациями Вооруженных сил Российской Федерации, а также организациями, осуществляющими деятельность в соответствии с  лицензиями, на использование ядерных материалов и радиоактивных веществ при проведении работ по использованию атомной энергии в оборонных целях, включая разработку, изготовление, испытание, эксплуатацию и утилизацию ядерного оружия и ядерных энергетических установок военного назначения", "Op_Cod": "17", "Op_Name": "Постановка на учет РВ, ранее не подлежащих учету в СГУК РВ и РАО", "deadline": "10 – для РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '26', '26', '{"prim": "Используется для снятия с учета учетной единицы при передаче одним юридическим лицом, осуществляющим деятельность на территории Российской Федерации другому юридическому лицу на территории Российской Федерации учетной единицы, с обязательствами по безопасному обращению, включая передачу федеральному государственному унитарному предприятию «Национальный оператор по обращению с радиоактивными отходами» для захоронения с оплатой захоронения (при необходимости)", "Op_Cod": "26", "Op_Name": "Передача юридическому лицу Российской Федерации с обязательствами по обеспечению безопасности при обращении, вплоть до их передачи национальному оператору", "deadline": "10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '52', '52', '{"prim": "Операция используется для отражения размещения РАО в пункт хранения (место сбора и/или временного хранения) при перемещении учетной единицы из пункта хранения (места сбора и/или временного хранения) организации в пункт хранения (место сбора и/или временного хранения) той же организации. Должна использоваться совместно с операцией с кодом 51. Операция не может быть использована при перемещении РАО между пунктами хранения (местами сбора и/или временного хранения) организации, находящимися в разных территориально обособленных подразделениях. В этом случае следует использовать операцию с кодом 31.", "Op_Cod": "52", "Op_Name": "Размещение в пункте хранения", "deadline": "10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1327035', '1327035 Федеральное казначейство (федеральная служба)', '{"2": "Федеральное казначейство (федеральная служба)", "m": 1327035}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1328000', '1328000 Министерство экономического развития РФ', '{"2": "Министерство экономического развития РФ", "m": 1328000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '29', '29', '{"prim": "Используется для снятия с учета учетной единицы при отражении факта передачи учетной единицы в тех случаях, когда нет возможности использовать коды 21-28. При выборе этого варианта в примечании необходимо дать пояснение об осуществленной операции по передаче учетной единицы", "Op_Cod": "29", "Op_Name": "Прочие операции по передаче", "deadline": "10 – для РВ; 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '38', '38', '{"prim": "Используется для постановки на учет учетной единицы при получении одним юридическим лицом на территории Российской Федерации от другого юридического лица, осуществляющим деятельность на территории Российской Федерации, учетной единицы и права собственности (иного вещного права) на нее. Данный код не может быть использован для отражения факта постановки на учет РВ или РАО, изъятых из незаконного оборота (код операции 74) или по указанию (предписанию) надзорных органов и органов исполнительной власти (код операции 76)", "Op_Cod": "38", "Op_Name": "Получение учетной единицы и права собственности на нее", "deadline": "10 – для РВ; 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '39', '39', '{"prim": "Используется для постановки на учет учетной единицы в тех случаях, когда нет возможности использовать коды 31-38. При выборе этого варианта в примечании необходимо дать пояснение об осуществленной операции по получению учетной единицы. Данный код не может быть использован для отражения факта получения РВ или РАО, изъятых из незаконного оборота (код операции 74) или по указанию (предписанию) надзорных органов и органов исполнительной власти (код операции 76)", "Op_Cod": "39", "Op_Name": "Прочие операции по получению", "deadline": "10 – для РВ; 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '58', '58', '{"prim": "Используется для отражения факта образования новых учетных единиц из учетных единиц, сведения о которых представлены с кодом операции 46", "Op_Cod": "58", "Op_Name": "Вторичное образование РВ", "deadline": "10 – для РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '62', '62', '{"prim": "Используется для представления информации о возврате на территорию организации (или обособленного подразделения) учетной единицы, о вывозе которой представлялась информация с кодом операции 61", "Op_Cod": "62", "Op_Name": "Возврат с другой территории", "deadline": "10 – для РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '67', '67', '{"prim": "Используется для факта снятия РВ с учета в связи с переводом его в РВ, обращение с которыми регулируется лицензиями, выдаваемыми организациям на использование ядерных материалов и радиоактивных веществ при проведении работ по использованию атомной энергии в оборонных целях, включая разработку, изготовление, испытание, эксплуатацию и утилизацию ядерного оружия и ядерных энергетических установок военного назначения", "Op_Cod": "67", "Op_Name": "Перевод в РВ, используемые при проведении работ по использованию атомной энергии в оборонных целях, включая разработку, изготовление, испытание, эксплуатацию и утилизацию ядерного оружия и ядерных энергетических установок военного назначения", "deadline": "10 – для РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '54', '54', '{"prim": "Используется для отражения факта перемещения отчитывающейся организацией учетной единицы из контейнера в контейнер, из контейнера в установку (прибор) и наоборот с привлечением подрядной организации. При заполнении формы указываются наименование, тип и номер прибора (установки), УКТ или иной упаковки, в которую размещена учетная единица. Подрядная организация, производившая работы, отчет не предоставляет.", "Op_Cod": "54", "Op_Name": "Зарядка (загрузка)/разрядка (выгрузка) подрядной организацией", "deadline": "10 – для РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '56', '56', '{"prim": "Операция используется для отражения постановки на учет учетной единицы, образовавшейся из учетных единиц, сведения о снятии которых с учета представлены операциями с кодом 44. В случае, если операция с кодом 44 отражена в другой форме оперативной отчетности (или другом периоде той же формы), необходимо сделать примечание к ячейке «Код операции» с указанием формы оперативной отчетности и даты операции, отражающей снятие учетной единицы при операции переработки, кондиционирования.", "Op_Cod": "56", "Op_Name": "Образование РАО после переработки", "deadline": "10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1327020', '1327020 Федеральная таможенная служба', '{"2": "Федеральная таможенная служба", "m": 1327020}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '85', '85', '{"prim": "Ввоз на территорию Российской Федерации с передачей права собственности (иного вещного права) получателю.", "Op_Cod": "85", "Op_Name": "Импорт", "deadline": "10 – для РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '83', '83', '{"prim": "Используется для отражения факта возврата учетной единицы, информация о ввозе которой прежде представлена операцией с кодом 86. Учетная единица снимается с учета в организации, осуществившей временный ввоз.", "Op_Cod": "83", "Op_Name": "Вывоз (возврат) с территории Российской Федерации временно ввезенного ранее", "deadline": "10 – для РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '84', '84', '{"prim": "Используется для отражения факта возврата (вывоза) с территории Российской Федерации ЗРИ/ОЗИИИ в страну поставщика ЗРИ. Операция применяется только к ранее импортированным ЗРИ (и соответствующим ОЗИИИ), которые зарегистрированы в СГУК РВ и РАО посредством кода операции 85.", "Op_Cod": "84", "Op_Name": "Вывоз (возврат) ранее импортированного ЗРИ (ОЗИИИ) за рубеж", "deadline": "10 – для РВ 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '86', '86', '{"prim": "Используется при временном ввозе на территорию Российской Федерации учетной единицы без передачи права собственности (иного вещного права) организации в Российской Федерации: для производства работ на территории Российской Федерации работниками иностранной организации; в связи с арендой, лизингом; по иным причинам, с гарантией вывоза (возврата) учетной единицы за пределы территории Российской Федерации. Возврат с территории Российской Федерации временно ввезенного ранее РВ отражается посредством операции с кодом 83. Учетная единица ставится на учет в организации, осуществившей временный ввоз учетной единицы.", "Op_Cod": "86", "Op_Name": "Временный ввоз на территорию Российской Федерации", "deadline": "10 – для РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '27', '27', '{"prim": "Используется для снятия с учета учетной единицы при передаче одним юридическим лицом, осуществляющим деятельность на территории Российской Федерации другому юридическому лицу на территории Российской Федерации учетной единицы без передачи права собственности на нее (на ответственное хранение, в аренду, лизинг, прочее). При передаче РАО с передачей обязательств по безопасному обращению вплоть до передачи национальному оператору с оплатой захоронения, следует использовать операцию с кодом 26. Возврат учетной единицы в дальнейшем должен быть отражен операцией с кодом 37. В том случае, если по окончании срока лизинга или по иным причинам право собственности перейдет к получателю, и учетная единица не будет возвращена, то этот факт должен быть в дальнейшем отражен операцией с кодом 63", "Op_Cod": "27", "Op_Name": "Передача юридическому лицу, зарегистрированному на территории Российской Федерации, без передачи права собственности", "deadline": "10 – для РВ; 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1327015', '1327015 Федеральная служба по регулированию алкогольного рынка', '{"2": "Федеральная служба по регулированию алкогольного рынка", "m": 1327015}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '63', '63', '{"prim": "Используется для отражения факта: 1) передачи права собственности на учетную единицу. Передача учетной единицы должна была быть ранее отражена операцией с кодом 27; 2) реорганизации организации-собственника учетной единицы, с передачей права собственности (иного вещного права) вновь образованному собственнику. 3) передаче обязательств по оплате и передаче РАО на захоронение", "Op_Cod": "63", "Op_Name": "Передача права собственности, обязательств по оплате захоронения, иного вещного права без физического перемещения учетной единицы", "deadline": "10 – для РВ; 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '64', '64', '{"prim": "Используется для отражения факта получения права собственности (иного вещного права) на учетную единицу или обязательств по оплате и передаче РАО на захоронение. Эта операция используется при получении права собственности (иного вещного права) на учетную единицу, полученную ранее (операция с кодом 35) или в результате реорганизации собственника.", "Op_Cod": "64", "Op_Name": "Получение права собственности, (иного вещного права), обязательств по передаче РАО на захоронение без физического перемещения учетной единицы", "deadline": "10 – для РВ; 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '61', '61', '{"prim": "Используется для отражения факта перемещения учетной единицы на территорию субъекта Российской Федерации, отличного от того, в котором расположено отчитывающееся юридическое лицо или его обособленное подразделение.  Оперативный отчет направляется в РИАЦ субъекта Российской Федерации, в котором расположено юридическое лицо или его обособленное подразделение только в том случае, если учетная единица покидает территорию организации на срок более 5 суток с последующим возвратом. РИАЦ субъекта Российской Федерации на территорию, которого ввозится учетная единица уведомляется копией указанного отчета без направления этой копии в ЦИАЦ. Вывоз учетной единицы не приводит к снятию ее с учета. При возврате учетной единицы обратно на территорию организации предоставляется отчет с кодом 62. Отчет не представляется, если операции с кодами 61 и 62 с одной и той же учетной единицей осуществлены в пределах периода, охватываемого одним отчётом.", "Op_Cod": "61", "Op_Name": "Вывоз на другую территорию", "deadline": "10 – для РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '42', '42', '{"prim": "Используется для отражения факта снятия с учета учетной единицы в СГУК РВ и РАО в связи с одновременной постановкой на учет в СГУК ЯМ.", "Op_Cod": "42", "Op_Name": "Перевод из СГУК РВ и РАО в СГУК ЯМ", "deadline": "10 – для РВ; 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '10', '10', '{"prim": "Используется для представления сведений о результатах проведенной инвентаризации. Срок исчисляется с момента утверждения акта инвентаризации", "Op_Cod": "10", "attrs1": null, "status": "1", "Op_Name": "Инвентаризация", "deadline": "10 – для РВ; 10 – для РАО", "import_hash": null, "import_author": null, "import_src_id": null}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '43', '43', '{"prim": "Используется для отражения факта снятия с учета учетной единицы в СГУК РВ и РАО по причине естественного распада радионуклидов до значений активности, при которых они перестают быть объектами учета в СГУК РВ и РАО.", "Op_Cod": "43", "Op_Name": "Снятие учетной единицы по причине естественного распада радионуклидов", "deadline": "10 – для РВ; 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '49', '49', '{"prim": "Операция используется для отражения снятия учетной единицы при операциях: 1) связанных с сортировкой РАО; 2) при дефрагментации (разборке) приборов или установок в целях изъятия РАО (в том числе в виде ОЗИИИ) без использования специализированных установок по переработке РАО. При этом учетная единица снимается с учета, вновь образованная учетная единица (или учетные единицы), с изменившимися характеристиками ставится на учет с кодом операции 59 или в случае подготовки РАО, приведенных в соответствие критериям приемлемости для захоронения, с кодом операции 55. При выборе этого варианта в примечании необходимо дать пояснение об осуществленной операции.", "Op_Cod": "49", "Op_Name": "Снятие с учета РАО при операциях сортировки", "deadline": "10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '44', '44', '{"prim": "Используется для отражения факта снятия с учёта РАО, направленных на переработку/ кондиционирование на установках или без установок переработки/кондиционирования. Информация о постановке на учёт учётных единиц, образовавшихся после переработки/кондиционирования, представляется с кодом 55 (если операция связана с приведением РАО в соответствие критериям приемлемости для захоронения) или с кодом 56 (в остальных случаях). В случае, если операция с кодом 56 будет отражена в другой форме оперативной отчётности, необходимо сделать примечание к ячейке «Код операции» с указанием формы оперативной отчётности и даты операции, отражающей снятие учётной единицы при операции переработки, кондиционирования. Код не может быть использован для отражения фактов: 1) сортировки РАО. В этом случае следует использовать операцию с кодом 49; 2) дефрагментации (разборке) приборов или установок с целью изъятия РАО (в том числе в виде ОИИИ) без использования установок переработки РАО. В этом случае следует использовать операцию с кодом 49.", "Op_Cod": "44", "Op_Name": "Переработка, кондиционирование", "deadline": "10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '45', '45', '{"prim": "Используется для отражения факта снятия с учета РАО при операциях упаковки/переупаковки РАО в другую упаковку (контейнер), предназначенную для транспортировки, длительного хранения или захоронения. Учетные единицы, образовавшиеся в результате переупаковки РАО, ставятся на учет в другой упаковке (контейнере), и информация о них представляется с кодом операции 57 или в случае подготовки РАО, приведенных в соответствие критериям приемлемости для захоронения, с кодом операции 55. Операция используется так же при изъятии первичных упаковок с РАО из упаковки (контейнера)", "Op_Cod": "45", "Op_Name": "Снятие с учета РАО при операциях упаковки, переупаковки", "deadline": "10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('type_of_activity', 'Использовании', 'Использовании', '{"status": "", "activity": "Использовании", "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '398', 'КАЗАХСТАН', '{"2": "КАЗАХСТАН", "3": "СНГ", "v": 398, "attrs1": null, "status": "1"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('type_of_activity', 'Хранении', 'Хранении', '{"status": "", "activity": "Хранении", "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('type_of_activity', 'Переработке', 'Переработке', '{"status": "", "activity": "Переработке", "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('type_of_activity', 'Производстве', 'Производстве', '{"status": "", "activity": "Производстве", "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('type_of_activity', 'Транспортировании', 'Транспортировании', '{"status": "", "activity": "Транспортировании", "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '46', '46', '{"prim": "Указывается при использовании учетной единицы, в том числе при разукомплектовании или расходовании, для изготовления материалов, продуктов, изделий, приборов, установок. При этом использованная учетная единица снимается с учета. Вновь образованные учетные единицы ставятся на учет, а информация о них представляется с кодом операции 58.", "Op_Cod": "46", "Op_Name": "Использование (разукомплектование, расфасовка) для изготовления, образования учетных единиц, подлежащих учету", "deadline": "10 – для РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('zri_voc', '1', 'kateg', '{"prim": "Чрезвычайно опасно", "kateg": 1, "status": "", "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('zri_voc', '2', 'kateg', '{"prim": "Очень опасно", "kateg": 2, "status": "", "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('zri_voc', '3', 'kateg', '{"prim": "Опасно", "kateg": 3, "status": "", "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('zri_voc', '4', 'kateg', '{"prim": "Опасность маловероятна", "kateg": 4, "status": "", "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('zri_voc', '5', 'kateg', '{"prim": "Опасность очень маловероятна", "kateg": 5, "status": "", "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('form_voc', '1.1', 'form_type', '{"prim": "ЗРИ", "status": "", "form_type": "1.1", "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('form_voc', '1.2', 'form_type', '{"prim": "ИОУ", "status": "", "form_type": "1.2", "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('form_voc', '1.3', 'form_type', '{"prim": "ОРИ в виде изделий", "status": "", "form_type": "1.3", "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('form_voc', '1.4', 'form_type', '{"prim": "ОРИ кроме изделий", "status": "", "form_type": "1.4", "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('form_voc', '1.5', 'form_type', '{"prim": "ОЗРИ", "status": "", "form_type": "1.5", "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('form_voc', '1.6', 'form_type', '{"prim": "РАО", "status": "", "form_type": "1.6", "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('form_voc', '1.7', 'form_type', '{"prim": "ТК РАО", "status": "", "form_type": "1.7", "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('form_voc', '1.8', 'form_type', '{"prim": "ЖК РАО", "status": "", "form_type": "1.8", "value_status": "active"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '41', '41', '{"prim": "Используется при переводе учетной единицы РВ (ЗРИ, ИОУ, ОРИ) в РАО. Операция должна одновременно найти отражение как в форме, соответствующей виду снимаемого с учета РВ, так и форме, соответствующей образовавшемуся РАО. В частности, в форме 1.5 с помощью кода 41 ставятся на учет исключительно те учетные единицы, которые предварительно снимаются с учета в форме 1.1. Сведения о РАО, образовавшихся из ИОУ и ОРИ, приводятся в форме 1.6", "Op_Cod": "41", "Op_Name": "Перевод РВ в РАО", "deadline": "10 – для РВ; 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1327010', '1327010 Федеральная налоговая служба', '{"2": "Федеральная налоговая служба", "m": 1327010}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '28', '28', '{"prim": "Используется для снятия с учета учетной единицы при передаче одним юридическим лицом, осуществляющим деятельность на территории Российской Федерации другому юридическому лицу на территории Российской Федерации учетной единицы и права собственности на нее", "Op_Cod": "28", "Op_Name": "Передача юридическому лицу, зарегистрированному на территории Российской Федерации, с передачей права собственности", "deadline": "10 – для РВ; 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '59', '59', '{"prim": "Используется для отражения факта постановки на учет учетной единицы при операциях, связанных с сортировкой РАО. Вновь образованная учетная единица ставится на учет. Учетная единица, поступившая на сортировку, снимается с учета с кодом операции 49. При выборе этого варианта в примечании необходимо дать пояснение об осуществленной операции.", "Op_Cod": "59", "Op_Name": "Постановка на учет РАО при операциях сортировки", "deadline": "10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1327000', '1327000 Министерство финансов РФ', '{"2": "Министерство финансов РФ", "m": 1327000}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '55', '55', '{"prim": "Операция используется для отражения постановки на учет в СГУК РВ и РАО учетной единицы, характеристики которой соответствуют критериям приемлемости РАО для захоронения. Сведения об учетных единицах, из которых она получена должны быть отражены операциями с кодом 44, 45, 49, 98. Сведения об операции предоставляются в формах 1.7 или 1.8.", "Op_Cod": "55", "Op_Name": "Приведение в соответствие критериям приемлемости РАО для захоронения", "deadline": "10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '35', '35', '{"prim": "Используется для отражения факта получения одним юридическим лицом на территории Российской Федерации от другого юридического лица, осуществляющего деятельность на территории Российской Федерации, учетной единицы по основаниям, не связанным с переходом права собственности (иного вещного права) на нее (получение на ответственное хранение, в аренду, лизинг, пользование, прочее). Для отражения факта возврата учетной единицы должна использоваться операция с кодом 25. При получении РАО с обязательствами по безопасному обращению вплоть до передачи национальному оператору с оплатой захоронения, следует использовать операцию с кодом 36. В том случае, если по окончании срока лизинга или по иным причинам право собственности (иное вещное право) перейдет к получателю, то этот факт должен быть отражен с использованием операции с кодом 64. Данный код не может быть использован для отражения факта получения РВ или РАО, изъятых из незаконного оборота (код операции 74) или по указанию (предписанию) надзорных органов и органов исполнительной власти (код операции 76)", "Op_Cod": "35", "Op_Name": "Получение от юридического лица, зарегистрированного на территории Российской Федерации, без перехода права собственности (иного вещного права)", "deadline": "10 – для РВ; 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '48', '48', '{"prim": "Используется для отражения факта изменения (уменьшения) количества РВ или РАО, обусловленного безвозвратными потерями.", "Op_Cod": "48", "Op_Name": "Безвозвратные потери в пределах установленных норм", "deadline": "10 – для РВ; 10 – для РАО"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('11_voc', '47', '47', '{"prim": "Указывается при расходовании учетной единицы. При этом израсходованная (использованная) учетная единица снимается с учета.", "Op_Cod": "47", "Op_Name": "Расходование", "deadline": "10 – для РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('FormSobst_voc', '2', '2 Государственная. Собственность субъектов Российской Федерации', '{"prim": "в примечании необходимо указать наименование субъекта Российской Федерации", "FSCod": "2", "FSName": "Государственная. Собственность субъектов Российской Федерации"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('FormSobst_voc', '3', '3 Муниципальная ', '{"prim": "в примечании необходимо указать наименование муниципального образования", "FSCod": "3", "FSName": "Муниципальная "}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('FormSobst_voc', '9', '9 Иная форма собственности (вещное право)', '{"prim": "в примечании необходимо указать форму собственности (вещное право)", "FSCod": "9", "FSName": "Иная форма собственности (вещное право)"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('FormSobst_voc', '6', '6 Собственность иностранного юридического лица', '{"FSCod": "6", "FSName": "Собственность иностранного юридического лица"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('FormSobst_voc', '5', '5 Собственность иностранного государства', '{"FSCod": "5", "FSName": "Собственность иностранного государства"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('FormSobst_voc', '4', '4 Частная (собственность юридических лиц)', '{"FSCod": "4", "FSName": "Частная (собственность юридических лиц)"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('12_voc', '19', '19', '{"DocCod": 19, "DocName": "Другой документ  (при выборе этого кода в примечании к ячейке формы должно быть приведено наименование документа)."}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('12_voc', '15', '15', '{"DocCod": 15, "DocName": "Сертификат"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('12_voc', '14', '14', '{"DocCod": 14, "DocName": "Требование"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('12_voc', '13', '13', '{"DocCod": 13, "DocName": "Решение о продлении НСС"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '184.0', 'ОСТРОВА КУКА', '{"2": "ОСТРОВА КУКА", "v": 184.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1326510', '1326510 Федеральная служба по труду и занятости', '{"2": "Федеральная служба по труду и занятости", "m": 1326510}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1325500', '1325500 Министерство спорта РФ', '{"2": "Министерство спорта РФ", "m": 1325500}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '1319920', '1319920 Управление делами Президента РФ (федеральное агентство)', '{"2": "Управление делами Президента РФ (федеральное агентство)", "m": 1319920}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100600', '4100600 Акционерные общества', '{"2": "Акционерные общества", "m": 4100600}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100601', '4100601 Публичное акционерное общество "ГАЗПРОМ"', '{"2": "Публичное акционерное общество \"ГАЗПРОМ\"", "m": 4100601}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100602', '4100602 Открытое акционерное общество "Российская агрохимическая компания"', '{"2": "Открытое акционерное общество \"Российская агрохимическая компания\"", "m": 4100602}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '4100901', '4100901 Общественная палата РФ', '{"2": "Общественная палата РФ", "m": 4100901}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2500030', '2500030 Уполномоченные по правам человека субъектов РФ', '{"2": "Уполномоченные по правам человека субъектов РФ", "m": 2500030}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '2500035', '2500035 Уполномоченные по защите прав предпринимателей в субъектах РФ', '{"2": "Уполномоченные по защите прав предпринимателей в субъектах РФ", "m": 2500035}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('116_voc', '3500200', '3500200 Иные органы городских округов', '{"2": "Иные органы городских округов", "m": 3500200}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('119_voc', '208.0', 'ДАНИЯ', '{"2": "ДАНИЯ", "v": 208.0}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('120_voc', '3', '3', '{"cifra": "3", "Agr_name": "Газообразное"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('120_voc', '2', '2', '{"cifra": "2", "Agr_name": "Твёрдое"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('120_voc', '1', '1', '{"cifra": "1", "Agr_name": "Жидкое"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('13_voc', '8', '8 Препарат', '{"ORICod": 8, "ORI_ty": "Препарат"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '62', '62', '{"c1": "62", "c11": "Комплексы переработки жидких РАО прочие (в примечании к ячейке следует указать наименования установок и используемую технологию)"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '61', '61', '{"c1": "61", "c11": "Комплекс спецводоочистки"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '55', '55', '{"c1": "55", "c11": "Установка утилизации жидкометаллических теплоносителей"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '54', '54', '{"c1": "54", "c11": "Установка фрагментации"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '53', '53', '{"c1": "53", "c11": "Установка измельчения"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '52', '52', '{"c1": "52", "c11": "Установка сортировки"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '51', '51', '{"c1": "51", "c11": "Установка дезактивации"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('17_voc', '49', '49', '{"c1": "49", "c11": "Установки отверждения прочие (в примечании к ячейке следует указать наименование установки и используемую технологию)"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иод-125', 'иод-125', '{"label": "иод-125"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('13_voc', '5', '5 Концентрат', '{"ORICod": 5, "ORI_ty": "Концентрат"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '03', 'Республика Бурятия', '{"2": "Республика Бурятия", "3": 8, "s": "03"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('13_voc', '12', '12 Проба', '{"ORICod": 12, "ORI_ty": "Проба"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('13_voc', '11', '11 Исследовательский образец', '{"ORICod": 11, "ORI_ty": "Исследовательский образец"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('13_voc', '10', '10 Стандартный образец', '{"ORICod": 10, "ORI_ty": "Стандартный образец"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('13_voc', '9', '9 Прочие ОРИ', '{"ORICod": 9, "ORI_ty": "Прочие ОРИ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иод-126', 'иод-126', '{"label": "иод-126"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('114_voc', '06', '06 Министерство образования и науки Российской федерации', '{"2": "Министерство образования и науки Российской федерации", "a": "06"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('114_voc', '05', '05 Министерство регионального развития Российской федерации', '{"2": "Министерство регионального развития Российской федерации", "a": "05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('114_voc', '04', '04 Министерство здравоохранения Российской федерации', '{"2": "Министерство здравоохранения Российской федерации", "a": "04"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('114_voc', '03', '03 Министерство промышленности и торговли Российской федерации', '{"2": "Министерство промышленности и торговли Российской федерации", "a": "03"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('114_voc', '02', '02 Министерство энергетики Российской федерации', '{"2": "Министерство энергетики Российской федерации", "a": "02"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('114_voc', '01', '01 Государственная корпорация по атомной энергии «Росатом»', '{"2": "Государственная корпорация по атомной энергии «Росатом»", "a": "01"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('115_voc', '05', 'Республика Дагестан', '{"2": "Республика Дагестан", "3": 4, "s": "05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('16_voc', '9', '9', '{"attrs1": null, "status": "1", "BCodCod": "9", "BCod_Name": "Прочее", "BCod_Clarif": "Код используется при неопределенности статуса РАО; в примечании необходимо указать причину, по которой статус РАО не удается определить", "import_hash": null, "import_author": null, "import_src_id": null}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('16_voc', '7', '7', '{"BCodCod": "7", "BCod_Name": "Источник излучения иностранного производства ", "BCod_Clarif": "Используется в случае, если иностранный источник излучения будет возвращен в страну поставщика"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '28001', '28001 Автономные некоммерческие организации', '{"2": "Автономные некоммерческие организации", "n": 28001}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '65241', '65241 Федеральные государственные унитарные предприятия', '{"2": "Федеральные государственные унитарные предприятия", "n": 65241}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('117_voc', '20607', '20607 Союзы (ассоциации) общин малочисленных народов', '{"2": "Союзы (ассоциации) общин малочисленных народов", "n": 20607}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('118_voc', '24', '24 Собственность иностранных граждан и лиц без гражданства', '{"2": "Собственность иностранных граждан и лиц без гражданства", "b": 24}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('112_voc', '33', '33', '{"g1_cod": "33", "g1_types": "Радиофармпрепараты, в виде ОРИ содержащие газообразные РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('112_voc', '32', '32', '{"g1_cod": "32", "g1_types": "Радиофармпрепараты, в виде ОРИ содержащие жидкие РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('112_voc', '31', '31', '{"g1_cod": "31", "g1_types": "Радиофармпрепараты, в виде ОРИ содержащие твердые РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кобальт-60м', 'кобальт-60м', '{"label": "кобальт-60м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'азот-13', 'азот-13', '{"label": "азот-13"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('112_voc', '99', '99', '{"prim": "При выборе этого варианта следует дать пояснения в примечании", "g1_cod": "99", "g1_types": "РВ основе радионуклидов с периодом полураспада до 60 (шестидесяти) суток, включая йод-125 прочие, не относящиеся к вышеуказанным типам объектов учета"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('112_voc', '23', '23', '{"g1_cod": "23", "g1_types": "ОРИ, кроме отдельных изделий, содержащие газообразные РВ"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иод-129', 'иод-129', '{"label": "иод-129"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'скандий-47', 'скандий-47', '{"label": "скандий-47"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иод-130', 'иод-130', '{"label": "иод-130"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'скандий-48', 'скандий-48', '{"label": "скандий-48"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'натрий-24', 'натрий-24', '{"label": "натрий-24"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'скандий-49', 'скандий-49', '{"label": "скандий-49"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'фосфор-32', 'фосфор-32', '{"label": "фосфор-32"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кремний-32', 'кремний-32', '{"label": "кремний-32"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кремний-31', 'кремний-31', '{"label": "кремний-31"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ванадий-47', 'ванадий-47', '{"label": "ванадий-47"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'титан-45', 'титан-45', '{"label": "титан-45"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'титан-44', 'титан-44', '{"label": "титан-44"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иод-131', 'иод-131', '{"label": "иод-131"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ванадий-48', 'ванадий-48', '{"label": "ванадий-48"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'хлор-36', 'хлор-36', '{"label": "хлор-36"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сера-35', 'сера-35', '{"label": "сера-35"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'фосфор-33', 'фосфор-33', '{"label": "фосфор-33"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цинк-72', 'цинк-72', '{"label": "цинк-72"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'хром-48', 'хром-48', '{"label": "хром-48"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ванадий-49', 'ванадий-49', '{"label": "ванадий-49"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'хлор-39', 'хлор-39', '{"label": "хлор-39"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'хлор-38', 'хлор-38', '{"label": "хлор-38"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'галлий-68', 'галлий-68', '{"label": "галлий-68"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'галлий-67', 'галлий-67', '{"label": "галлий-67"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'галлий-65', 'галлий-65', '{"label": "галлий-65"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'марганец-51', 'марганец-51', '{"label": "марганец-51"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'хром-51', 'хром-51', '{"label": "хром-51"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'хром-49', 'хром-49', '{"label": "хром-49"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'галлий-73', 'галлий-73', '{"label": "галлий-73"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'галлий-72', 'галлий-72', '{"label": "галлий-72"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'галлий-70', 'галлий-70', '{"label": "галлий-70"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'германий-67', 'германий-67', '{"label": "германий-67"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'германий-66', 'германий-66', '{"label": "германий-66"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'марганец-52м', 'марганец-52м', '{"label": "марганец-52м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'марганец-52', 'марганец-52', '{"label": "марганец-52"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'германий-68', 'германий-68', '{"label": "германий-68"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'железо-55', 'железо-55', '{"label": "железо-55"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'марганец-54', 'марганец-54', '{"label": "марганец-54"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'марганец-53', 'марганец-53', '{"label": "марганец-53"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иод-132', 'иод-132', '{"label": "иод-132"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'германий-69', 'германий-69', '{"label": "германий-69"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'марганец-56', 'марганец-56', '{"label": "марганец-56"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'германий-77', 'германий-77', '{"label": "германий-77"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'германий-78', 'германий-78', '{"label": "германий-78"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'железо-52', 'железо-52', '{"label": "железо-52"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'мышьяк-71', 'мышьяк-71', '{"label": "мышьяк-71"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'мышьяк-70', 'мышьяк-70', '{"label": "мышьяк-70"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'мышьяк-69', 'мышьяк-69', '{"label": "мышьяк-69"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'мышьяк-72', 'мышьяк-72', '{"label": "мышьяк-72"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'галлий-66', 'галлий-66', '{"label": "галлий-66"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'мышьяк-76', 'мышьяк-76', '{"label": "мышьяк-76"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'мышьяк-74', 'мышьяк-74', '{"label": "мышьяк-74"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'мышьяк-73', 'мышьяк-73', '{"label": "мышьяк-73"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кобальт-55', 'кобальт-55', '{"label": "кобальт-55"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'железо-60', 'железо-60', '{"label": "железо-60"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'железо-59', 'железо-59', '{"label": "железо-59"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кобальт-56', 'кобальт-56', '{"label": "кобальт-56"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'мышьяк-78', 'мышьяк-78', '{"label": "мышьяк-78"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'мышьяк-77', 'мышьяк-77', '{"label": "мышьяк-77"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кобальт-58', 'кобальт-58', '{"label": "кобальт-58"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кобальт-57', 'кобальт-57', '{"label": "кобальт-57"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кобальт-61', 'кобальт-61', '{"label": "кобальт-61"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сумма альфа-активных радионуклидов*', 'сумма альфа-активных радионуклидов*', '{"label": "сумма альфа-активных радионуклидов*"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'селен-73', 'селен-73', '{"label": "селен-73"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'селен-70', 'селен-70', '{"label": "селен-70"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'селен-81м', 'селен-81м', '{"label": "селен-81м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'селен-81', 'селен-81', '{"label": "селен-81"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кобальт-62м', 'кобальт-62м', '{"label": "кобальт-62м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'бром-74', 'бром-74', '{"label": "бром-74"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'селен-83', 'селен-83', '{"label": "селен-83"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'никель-59', 'никель-59', '{"label": "никель-59"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'никель-57', 'никель-57', '{"label": "никель-57"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'никель-56', 'никель-56', '{"label": "никель-56"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'бром-75', 'бром-75', '{"label": "бром-75"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'бром-74м', 'бром-74м', '{"label": "бром-74м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'никель-66', 'никель-66', '{"label": "никель-66"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'никель-65', 'никель-65', '{"label": "никель-65"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'никель-63', 'никель-63', '{"label": "никель-63"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'медь-64', 'медь-64', '{"label": "медь-64"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'медь-61', 'медь-61', '{"label": "медь-61"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'медь-60', 'медь-60', '{"label": "медь-60"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'бром-77', 'бром-77', '{"label": "бром-77"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'бром-76', 'бром-76', '{"label": "бром-76"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цинк-71м', 'цинк-71м', '{"label": "цинк-71м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'бром-80м', 'бром-80м', '{"label": "бром-80м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'бром-80', 'бром-80', '{"label": "бром-80"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'бром-83', 'бром-83', '{"label": "бром-83"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'бром-82', 'бром-82', '{"label": "бром-82"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'медь-67', 'медь-67', '{"label": "медь-67"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рубидий-79', 'рубидий-79', '{"label": "рубидий-79"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'бром-84', 'бром-84', '{"label": "бром-84"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цинк-69м', 'цинк-69м', '{"label": "цинк-69м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цинк-69', 'цинк-69', '{"label": "цинк-69"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цинк-65', 'цинк-65', '{"label": "цинк-65"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'криптон-88', 'криптон-88', '{"label": "криптон-88"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'криптон-85м', 'криптон-85м', '{"label": "криптон-85м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'калий-40', 'калий-40', '{"label": "калий-40"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'аргон-42', 'аргон-42', '{"label": "аргон-42"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'калий-42', 'калий-42', '{"label": "калий-42"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'криптон-89', 'криптон-89', '{"label": "криптон-89"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'калий-43', 'калий-43', '{"label": "калий-43"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'аргон-41', 'аргон-41', '{"label": "аргон-41"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'аргон-39', 'аргон-39', '{"label": "аргон-39"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'торий ест.', 'торий ест.', '{"label": "торий ест."}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'алюминий-26', 'алюминий-26', '{"label": "алюминий-26"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'полоний-216', 'полоний-216', '{"label": "полоний-216"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'полоний-212', 'полоний-212', '{"label": "полоний-212"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иод-123', 'иод-123', '{"label": "иод-123"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кальций-41', 'кальций-41', '{"label": "кальций-41"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'калий-45', 'калий-45', '{"label": "калий-45"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'калий-44', 'калий-44', '{"label": "калий-44"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кальций-47', 'кальций-47', '{"label": "кальций-47"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кальций-45', 'кальций-45', '{"label": "кальций-45"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'протактиний-234м', 'протактиний-234м', '{"label": "протактиний-234м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'скандий-44', 'скандий-44', '{"label": "скандий-44"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'скандий-43', 'скандий-43', '{"label": "скандий-43"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'скандий-46', 'скандий-46', '{"label": "скандий-46"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'скандий-44м', 'скандий-44м', '{"label": "скандий-44м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'магний-28', 'магний-28', '{"label": "магний-28"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'натрий-22', 'натрий-22', '{"label": "натрий-22"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'палладий-234', 'палладий-234', '{"label": "палладий-234"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'бериллий-9', 'бериллий-9', '{"label": "бериллий-9"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'углерод-13', 'углерод-13', '{"label": "углерод-13"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ксенон-125', 'ксенон-125', '{"label": "ксенон-125"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'полоний-218', 'полоний-218', '{"label": "полоний-218"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сумма бета-активных естеств. нуклидов', 'сумма бета-активных естеств. нуклидов', '{"label": "сумма бета-активных естеств. нуклидов"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сумма альфа-активных естеств. нуклидов', 'сумма альфа-активных естеств. нуклидов', '{"label": "сумма альфа-активных естеств. нуклидов"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'фтор-18', 'фтор-18', '{"label": "фтор-18"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'фтор-17', 'фтор-17', '{"label": "фтор-17"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кислород-15', 'кислород-15', '{"label": "кислород-15"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кислород-14', 'кислород-14', '{"label": "кислород-14"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'неон-24', 'неон-24', '{"label": "неон-24"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'углерод-11', 'углерод-11', '{"label": "углерод-11"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'технеций-97', 'технеций-97', '{"label": "технеций-97"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'технеций-96м', 'технеций-96м', '{"label": "технеций-96м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'технеций-95м', 'технеций-95м', '{"label": "технеций-95м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'технеций-95', 'технеций-95', '{"label": "технеций-95"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'технеций-94м', 'технеций-94м', '{"label": "технеций-94м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'технеций-94', 'технеций-94', '{"label": "технеций-94"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'технеций-93м', 'технеций-93м', '{"label": "технеций-93м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'технеций-93', 'технеций-93', '{"label": "технеций-93"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'молибден-101', 'молибден-101', '{"label": "молибден-101"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'молибден-99', 'молибден-99', '{"label": "молибден-99"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'технеций-99м', 'технеций-99м', '{"label": "технеций-99м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'молибден-93м', 'молибден-93м', '{"label": "молибден-93м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'молибден-93', 'молибден-93', '{"label": "молибден-93"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'молибден-90', 'молибден-90', '{"label": "молибден-90"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ниобий-98', 'ниобий-98', '{"label": "ниобий-98"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ниобий-97', 'ниобий-97', '{"label": "ниобий-97"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ниобий-96', 'ниобий-96', '{"label": "ниобий-96"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ниобий-95м', 'ниобий-95м', '{"label": "ниобий-95м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ниобий-94', 'ниобий-94', '{"label": "ниобий-94"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ниобий-93м', 'ниобий-93м', '{"label": "ниобий-93м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ниобий-90', 'ниобий-90', '{"label": "ниобий-90"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ниобий-89м', 'ниобий-89м', '{"label": "ниобий-89м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ниобий-89', 'ниобий-89', '{"label": "ниобий-89"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цирконий-97', 'цирконий-97', '{"label": "цирконий-97"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цирконий-95', 'цирконий-95', '{"label": "цирконий-95"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цирконий-93', 'цирконий-93', '{"label": "цирконий-93"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цирконий-89', 'цирконий-89', '{"label": "цирконий-89"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цирконий-86', 'цирконий-86', '{"label": "цирконий-86"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иттрий-95', 'иттрий-95', '{"label": "иттрий-95"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иттрий-94', 'иттрий-94', '{"label": "иттрий-94"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иттрий-93', 'иттрий-93', '{"label": "иттрий-93"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иттрий-92', 'иттрий-92', '{"label": "иттрий-92"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иттрий-91м', 'иттрий-91м', '{"label": "иттрий-91м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иттрий-91', 'иттрий-91', '{"label": "иттрий-91"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иттрий-90м', 'иттрий-90м', '{"label": "иттрий-90м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иттрий-87', 'иттрий-87', '{"label": "иттрий-87"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иттрий-86м', 'иттрий-86м', '{"label": "иттрий-86м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иттрий-86', 'иттрий-86', '{"label": "иттрий-86"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'стронций-91', 'стронций-91', '{"label": "стронций-91"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'стронций-90', 'стронций-90', '{"label": "стронций-90"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'стронций-89', 'стронций-89', '{"label": "стронций-89"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'стронций-87м', 'стронций-87м', '{"label": "стронций-87м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'стронций-85м', 'стронций-85м', '{"label": "стронций-85м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'стронций-85', 'стронций-85', '{"label": "стронций-85"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'стронций-83', 'стронций-83', '{"label": "стронций-83"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'стронций-82', 'стронций-82', '{"label": "стронций-82"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'стронций-81', 'стронций-81', '{"label": "стронций-81"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рубидий-89', 'рубидий-89', '{"label": "рубидий-89"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рубидий-88', 'рубидий-88', '{"label": "рубидий-88"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рубидий-86', 'рубидий-86', '{"label": "рубидий-86"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рубидий-84', 'рубидий-84', '{"label": "рубидий-84"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рубидий-83', 'рубидий-83', '{"label": "рубидий-83"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рубидий-81м', 'рубидий-81м', '{"label": "рубидий-81м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рубидий-81', 'рубидий-81', '{"label": "рубидий-81"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'олово-127', 'олово-127', '{"label": "олово-127"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'олово-126', 'олово-126', '{"label": "олово-126"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'олово-123м', 'олово-123м', '{"label": "олово-123м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'олово-119м', 'олово-119м', '{"label": "олово-119м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'олово-117м', 'олово-117м', '{"label": "олово-117м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'олово-111', 'олово-111', '{"label": "олово-111"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'олово-110', 'олово-110', '{"label": "олово-110"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'индий-119м', 'индий-119м', '{"label": "индий-119м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'индий-117м', 'индий-117м', '{"label": "индий-117м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'индий-117', 'индий-117', '{"label": "индий-117"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'индий-116м', 'индий-116м', '{"label": "индий-116м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'индий-115м', 'индий-115м', '{"label": "индий-115м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'индий-114м', 'индий-114м', '{"label": "индий-114м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'индий-113м', 'индий-113м', '{"label": "индий-113м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'индий-112', 'индий-112', '{"label": "индий-112"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'индий-110м', 'индий-110м', '{"label": "индий-110м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'индий-110', 'индий-110', '{"label": "индий-110"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'индий-109', 'индий-109', '{"label": "индий-109"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кадмий-117м', 'кадмий-117м', '{"label": "кадмий-117м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кадмий-117', 'кадмий-117', '{"label": "кадмий-117"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кадмий-115м', 'кадмий-115м', '{"label": "кадмий-115м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кадмий-115', 'кадмий-115', '{"label": "кадмий-115"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кадмий-113м', 'кадмий-113м', '{"label": "кадмий-113м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кадмий-113', 'кадмий-113', '{"label": "кадмий-113"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кадмий-109', 'кадмий-109', '{"label": "кадмий-109"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кадмий-107', 'кадмий-107', '{"label": "кадмий-107"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кадмий-104', 'кадмий-104', '{"label": "кадмий-104"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'серебро-112', 'серебро-112', '{"label": "серебро-112"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'серебро-111', 'серебро-111', '{"label": "серебро-111"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'серебро-106м', 'серебро-106м', '{"label": "серебро-106м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'серебро-106', 'серебро-106', '{"label": "серебро-106"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'серебро-105', 'серебро-105', '{"label": "серебро-105"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'серебро-104м', 'серебро-104м', '{"label": "серебро-104м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'серебро-104', 'серебро-104', '{"label": "серебро-104"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'серебро-103', 'серебро-103', '{"label": "серебро-103"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'серебро-102', 'серебро-102', '{"label": "серебро-102"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'палладий-109', 'палладий-109', '{"label": "палладий-109"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'палладий-107', 'палладий-107', '{"label": "палладий-107"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'палладий-103', 'палладий-103', '{"label": "палладий-103"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'палладий-101', 'палладий-101', '{"label": "палладий-101"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'палладий-100', 'палладий-100', '{"label": "палладий-100"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'родий-103м', 'родий-103м', '{"label": "родий-103м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'родий-102м', 'родий-102м', '{"label": "родий-102м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'родий-101м', 'родий-101м', '{"label": "родий-101м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'родий-101', 'родий-101', '{"label": "родий-101"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'родий-100', 'родий-100', '{"label": "родий-100"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'родий-99м', 'родий-99м', '{"label": "родий-99м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'родий-99', 'родий-99', '{"label": "родий-99"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рутений-106', 'рутений-106', '{"label": "рутений-106"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рутений-105', 'рутений-105', '{"label": "рутений-105"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рутений-103', 'рутений-103', '{"label": "рутений-103"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рутений-97', 'рутений-97', '{"label": "рутений-97"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рутений-94', 'рутений-94', '{"label": "рутений-94"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'олово-121', 'олово-121', '{"label": "олово-121"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'технеций-99', 'технеций-99', '{"label": "технеций-99"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цезий-137', 'цезий-137', '{"label": "цезий-137"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цезий-136', 'цезий-136', '{"label": "цезий-136"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цезий-135м', 'цезий-135м', '{"label": "цезий-135м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цезий-135', 'цезий-135', '{"label": "цезий-135"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цезий-134м', 'цезий-134м', '{"label": "цезий-134м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цезий-134', 'цезий-134', '{"label": "цезий-134"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цезий-131', 'цезий-131', '{"label": "цезий-131"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цезий-130', 'цезий-130', '{"label": "цезий-130"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цезий-129', 'цезий-129', '{"label": "цезий-129"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цезий-127', 'цезий-127', '{"label": "цезий-127"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цезий-125', 'цезий-125', '{"label": "цезий-125"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ксенон-138', 'ксенон-138', '{"label": "ксенон-138"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ксенон-137', 'ксенон-137', '{"label": "ксенон-137"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'неодим-151', 'неодим-151', '{"label": "неодим-151"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ксенон-135м', 'ксенон-135м', '{"label": "ксенон-135м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ксенон-135', 'ксенон-135', '{"label": "ксенон-135"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ксенон-133', 'ксенон-133', '{"label": "ксенон-133"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иод-135', 'иод-135', '{"label": "иод-135"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сурьма-129', 'сурьма-129', '{"label": "сурьма-129"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сурьма-128м', 'сурьма-128м', '{"label": "сурьма-128м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сурьма-128', 'сурьма-128', '{"label": "сурьма-128"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сурьма-127', 'сурьма-127', '{"label": "сурьма-127"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сурьма-126м', 'сурьма-126м', '{"label": "сурьма-126м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сурьма-126', 'сурьма-126', '{"label": "сурьма-126"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сурьма-125', 'сурьма-125', '{"label": "сурьма-125"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сурьма-124м', 'сурьма-124м', '{"label": "сурьма-124м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сурьма-124', 'сурьма-124', '{"label": "сурьма-124"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сурьма-122', 'сурьма-122', '{"label": "сурьма-122"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сурьма-120м', 'сурьма-120м', '{"label": "сурьма-120м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сурьма-120', 'сурьма-120', '{"label": "сурьма-120"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сурьма-119', 'сурьма-119', '{"label": "сурьма-119"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сурьма-118м', 'сурьма-118м', '{"label": "сурьма-118м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сурьма-117', 'сурьма-117', '{"label": "сурьма-117"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сурьма-116м', 'сурьма-116м', '{"label": "сурьма-116м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сурьма-116', 'сурьма-116', '{"label": "сурьма-116"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сурьма-115', 'сурьма-115', '{"label": "сурьма-115"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'олово-128', 'олово-128', '{"label": "олово-128"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'самарий-146', 'самарий-146', '{"label": "самарий-146"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'самарий-145', 'самарий-145', '{"label": "самарий-145"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'самарий-142', 'самарий-142', '{"label": "самарий-142"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'самарий-141м', 'самарий-141м', '{"label": "самарий-141м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'прометий-151', 'прометий-151', '{"label": "прометий-151"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'прометий-149', 'прометий-149', '{"label": "прометий-149"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'прометий-148м', 'прометий-148м', '{"label": "прометий-148м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'прометий-148', 'прометий-148', '{"label": "прометий-148"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'прометий-147', 'прометий-147', '{"label": "прометий-147"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'прометий-146', 'прометий-146', '{"label": "прометий-146"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'прометий-145', 'прометий-145', '{"label": "прометий-145"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'прометий-144', 'прометий-144', '{"label": "прометий-144"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'прометий-143', 'прометий-143', '{"label": "прометий-143"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'европий-158', 'европий-158', '{"label": "европий-158"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'прометий-141', 'прометий-141', '{"label": "прометий-141"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'неодим-149', 'неодим-149', '{"label": "неодим-149"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'неодим-141', 'неодим-141', '{"label": "неодим-141"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'неодим-139м', 'неодим-139м', '{"label": "неодим-139м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'неодим-139', 'неодим-139', '{"label": "неодим-139"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'неодим-138', 'неодим-138', '{"label": "неодим-138"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'неодим-136', 'неодим-136', '{"label": "неодим-136"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'празеодим-147', 'празеодим-147', '{"label": "празеодим-147"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'празеодим-145', 'празеодим-145', '{"label": "празеодим-145"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'празеодим-144', 'празеодим-144', '{"label": "празеодим-144"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'празеодим-143', 'празеодим-143', '{"label": "празеодим-143"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'празеодим-142м', 'празеодим-142м', '{"label": "празеодим-142м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'празеодим-139', 'празеодим-139', '{"label": "празеодим-139"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'празеодим-138м', 'празеодим-138м', '{"label": "празеодим-138м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'празеодим-137', 'празеодим-137', '{"label": "празеодим-137"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'празеодим-136', 'празеодим-136', '{"label": "празеодим-136"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'церий-144', 'церий-144', '{"label": "церий-144"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'церий-143', 'церий-143', '{"label": "церий-143"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'церий-141', 'церий-141', '{"label": "церий-141"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'прометий-150', 'прометий-150', '{"label": "прометий-150"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'церий-139', 'церий-139', '{"label": "церий-139"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'церий-137м', 'церий-137м', '{"label": "церий-137м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'церий-137', 'церий-137', '{"label": "церий-137"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'церий-135', 'церий-135', '{"label": "церий-135"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'церий-134', 'церий-134', '{"label": "церий-134"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'лантан-143', 'лантан-143', '{"label": "лантан-143"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'лантан-141', 'лантан-141', '{"label": "лантан-141"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'лантан-137', 'лантан-137', '{"label": "лантан-137"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'лантан-135', 'лантан-135', '{"label": "лантан-135"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'лантан-132', 'лантан-132', '{"label": "лантан-132"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'лантан-131', 'лантан-131', '{"label": "лантан-131"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'барий-142', 'барий-142', '{"label": "барий-142"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'барий-141', 'барий-141', '{"label": "барий-141"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'барий-140', 'барий-140', '{"label": "барий-140"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'барий-139', 'барий-139', '{"label": "барий-139"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'барий-135м', 'барий-135м', '{"label": "барий-135м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'барий-133', 'барий-133', '{"label": "барий-133"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'барий-131м', 'барий-131м', '{"label": "барий-131м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'барий-131', 'барий-131', '{"label": "барий-131"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'барий-128', 'барий-128', '{"label": "барий-128"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тулий-170', 'тулий-170', '{"label": "тулий-170"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тулий-166', 'тулий-166', '{"label": "тулий-166"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тулий-162', 'тулий-162', '{"label": "тулий-162"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'эрбий-172', 'эрбий-172', '{"label": "эрбий-172"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'эрбий-171', 'эрбий-171', '{"label": "эрбий-171"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'эрбий-169', 'эрбий-169', '{"label": "эрбий-169"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'эрбий-165', 'эрбий-165', '{"label": "эрбий-165"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'эрбий-161', 'эрбий-161', '{"label": "эрбий-161"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гольмий-167', 'гольмий-167', '{"label": "гольмий-167"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гольмий-166м', 'гольмий-166м', '{"label": "гольмий-166м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гольмий-166', 'гольмий-166', '{"label": "гольмий-166"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гольмий-164', 'гольмий-164', '{"label": "гольмий-164"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гольмий-162м', 'гольмий-162м', '{"label": "гольмий-162м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гольмий-161', 'гольмий-161', '{"label": "гольмий-161"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гольмий-159', 'гольмий-159', '{"label": "гольмий-159"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гольмий-157', 'гольмий-157', '{"label": "гольмий-157"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гольмий-155', 'гольмий-155', '{"label": "гольмий-155"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'диспрозий-166', 'диспрозий-166', '{"label": "диспрозий-166"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'диспрозий-165', 'диспрозий-165', '{"label": "диспрозий-165"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'диспрозий-159', 'диспрозий-159', '{"label": "диспрозий-159"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'диспрозий-157', 'диспрозий-157', '{"label": "диспрозий-157"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'диспрозий-155', 'диспрозий-155', '{"label": "диспрозий-155"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тербий-161', 'тербий-161', '{"label": "тербий-161"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тербий-160', 'тербий-160', '{"label": "тербий-160"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тербий-158', 'тербий-158', '{"label": "тербий-158"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тербий-155', 'тербий-155', '{"label": "тербий-155"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тербий-154', 'тербий-154', '{"label": "тербий-154"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тербий-151', 'тербий-151', '{"label": "тербий-151"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тербий-150', 'тербий-150', '{"label": "тербий-150"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тербий-149', 'тербий-149', '{"label": "тербий-149"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тербий-147', 'тербий-147', '{"label": "тербий-147"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'уран обогащенный', 'уран обогащенный', '{"label": "уран обогащенный"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гадолиний-159', 'гадолиний-159', '{"label": "гадолиний-159"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гадолиний-153', 'гадолиний-153', '{"label": "гадолиний-153"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гадолиний-152', 'гадолиний-152', '{"label": "гадолиний-152"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гадолиний-151', 'гадолиний-151', '{"label": "гадолиний-151"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гадолиний-148', 'гадолиний-148', '{"label": "гадолиний-148"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гадолиний-147', 'гадолиний-147', '{"label": "гадолиний-147"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'европий-156', 'европий-156', '{"label": "европий-156"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'европий-155', 'европий-155', '{"label": "европий-155"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'европий-154', 'европий-154', '{"label": "европий-154"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'европий-152м', 'европий-152м', '{"label": "европий-152м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'европий-152', 'европий-152', '{"label": "европий-152"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'европий-150м', 'европий-150м', '{"label": "европий-150м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'европий-150', 'европий-150', '{"label": "европий-150"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'европий-149', 'европий-149', '{"label": "европий-149"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'европий-148', 'европий-148', '{"label": "европий-148"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'европий-147', 'европий-147', '{"label": "европий-147"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'европий-146', 'европий-146', '{"label": "европий-146"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'европий-145', 'европий-145', '{"label": "европий-145"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'самарий-156', 'самарий-156', '{"label": "самарий-156"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'самарий-155', 'самарий-155', '{"label": "самарий-155"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'самарий-153', 'самарий-153', '{"label": "самарий-153"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'самарий-151', 'самарий-151', '{"label": "самарий-151"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рений-184м', 'рений-184м', '{"label": "рений-184м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рений-182', 'рений-182', '{"label": "рений-182"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рений-178', 'рений-178', '{"label": "рений-178"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рений-177', 'рений-177', '{"label": "рений-177"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'вольфрам-187', 'вольфрам-187', '{"label": "вольфрам-187"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'вольфрам-185', 'вольфрам-185', '{"label": "вольфрам-185"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'вольфрам-181', 'вольфрам-181', '{"label": "вольфрам-181"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'вольфрам-179', 'вольфрам-179', '{"label": "вольфрам-179"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'вольфрам-178', 'вольфрам-178', '{"label": "вольфрам-178"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'вольфрам-177', 'вольфрам-177', '{"label": "вольфрам-177"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'вольфрам-176', 'вольфрам-176', '{"label": "вольфрам-176"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тантал-186', 'тантал-186', '{"label": "тантал-186"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тантал-185', 'тантал-185', '{"label": "тантал-185"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тантал-184', 'тантал-184', '{"label": "тантал-184"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тантал-182м', 'тантал-182м', '{"label": "тантал-182м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тантал-182', 'тантал-182', '{"label": "тантал-182"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тантал-180м', 'тантал-180м', '{"label": "тантал-180м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тантал-180', 'тантал-180', '{"label": "тантал-180"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тантал-179', 'тантал-179', '{"label": "тантал-179"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тантал-178', 'тантал-178', '{"label": "тантал-178"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тантал-177', 'тантал-177', '{"label": "тантал-177"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тантал-176', 'тантал-176', '{"label": "тантал-176"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тантал-175', 'тантал-175', '{"label": "тантал-175"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тантал-174', 'тантал-174', '{"label": "тантал-174"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тантал-173', 'тантал-173', '{"label": "тантал-173"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тантал-172', 'тантал-172', '{"label": "тантал-172"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гафний-183', 'гафний-183', '{"label": "гафний-183"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гафний-182м', 'гафний-182м', '{"label": "гафний-182м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гафний-178м', 'гафний-178м', '{"label": "гафний-178м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гафний-177м', 'гафний-177м', '{"label": "гафний-177м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гафний-175', 'гафний-175', '{"label": "гафний-175"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гафний-173', 'гафний-173', '{"label": "гафний-173"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гафний-172', 'гафний-172', '{"label": "гафний-172"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гафний-170', 'гафний-170', '{"label": "гафний-170"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'лютеций-179', 'лютеций-179', '{"label": "лютеций-179"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'лютеций-178м', 'лютеций-178м', '{"label": "лютеций-178м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'лютеций-178', 'лютеций-178', '{"label": "лютеций-178"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'лютеций-177м', 'лютеций-177м', '{"label": "лютеций-177м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'лютеций-177', 'лютеций-177', '{"label": "лютеций-177"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'лютеций-176м', 'лютеций-176м', '{"label": "лютеций-176м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'лютеций-176', 'лютеций-176', '{"label": "лютеций-176"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рений-181', 'рений-181', '{"label": "рений-181"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'лютеций-172', 'лютеций-172', '{"label": "лютеций-172"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'лютеций-171', 'лютеций-171', '{"label": "лютеций-171"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'лютеций-170', 'лютеций-170', '{"label": "лютеций-170"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'лютеций-169', 'лютеций-169', '{"label": "лютеций-169"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иттербий-178', 'иттербий-178', '{"label": "иттербий-178"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иттербий-177', 'иттербий-177', '{"label": "иттербий-177"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иттербий-175', 'иттербий-175', '{"label": "иттербий-175"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иттербий-169', 'иттербий-169', '{"label": "иттербий-169"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иттербий-167', 'иттербий-167', '{"label": "иттербий-167"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иттербий-166', 'иттербий-166', '{"label": "иттербий-166"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иттербий-162', 'иттербий-162', '{"label": "иттербий-162"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тулий-175', 'тулий-175', '{"label": "тулий-175"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тулий-173', 'тулий-173', '{"label": "тулий-173"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'протактиний-230', 'протактиний-230', '{"label": "протактиний-230"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'протактиний-228', 'протактиний-228', '{"label": "протактиний-228"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'протактиний-227', 'протактиний-227', '{"label": "протактиний-227"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'торий-234', 'торий-234', '{"label": "торий-234"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'торий-232', 'торий-232', '{"label": "торий-232"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'торий-231', 'торий-231', '{"label": "торий-231"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'торий-230', 'торий-230', '{"label": "торий-230"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'торий-229', 'торий-229', '{"label": "торий-229"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'торий-228', 'торий-228', '{"label": "торий-228"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'торий-227', 'торий-227', '{"label": "торий-227"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'торий-226', 'торий-226', '{"label": "торий-226"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'актиний-228', 'актиний-228', '{"label": "актиний-228"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'полоний-207', 'полоний-207', '{"label": "полоний-207"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'полоний-205', 'полоний-205', '{"label": "полоний-205"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'полоний-203', 'полоний-203', '{"label": "полоний-203"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'висмут-214', 'висмут-214', '{"label": "висмут-214"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'висмут-212', 'висмут-212', '{"label": "висмут-212"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'висмут-207', 'висмут-207', '{"label": "висмут-207"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'висмут-206', 'висмут-206', '{"label": "висмут-206"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'висмут-205', 'висмут-205', '{"label": "висмут-205"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'висмут-203', 'висмут-203', '{"label": "висмут-203"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'висмут-202', 'висмут-202', '{"label": "висмут-202"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'висмут-201', 'висмут-201', '{"label": "висмут-201"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'висмут-200', 'висмут-200', '{"label": "висмут-200"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'свинец-214', 'свинец-214', '{"label": "свинец-214"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'таллий-200', 'таллий-200', '{"label": "таллий-200"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'таллий-198м', 'таллий-198м', '{"label": "таллий-198м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'таллий-198', 'таллий-198', '{"label": "таллий-198"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'таллий-197', 'таллий-197', '{"label": "таллий-197"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'таллий-195', 'таллий-195', '{"label": "таллий-195"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'таллий-194м', 'таллий-194м', '{"label": "таллий-194м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ртуть-199м', 'ртуть-199м', '{"label": "ртуть-199м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ртуть-197м', 'ртуть-197м', '{"label": "ртуть-197м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ртуть-197', 'ртуть-197', '{"label": "ртуть-197"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ртуть-195м', 'ртуть-195м', '{"label": "ртуть-195м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ртуть-195', 'ртуть-195', '{"label": "ртуть-195"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ртуть-194', 'ртуть-194', '{"label": "ртуть-194"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ртуть-193м', 'ртуть-193м', '{"label": "ртуть-193м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'платина-191', 'платина-191', '{"label": "платина-191"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'платина-189', 'платина-189', '{"label": "платина-189"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'платина-188', 'платина-188', '{"label": "платина-188"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'платина-186', 'платина-186', '{"label": "платина-186"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иридий-195м', 'иридий-195м', '{"label": "иридий-195м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иридий-195', 'иридий-195', '{"label": "иридий-195"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иридий-193м', 'иридий-193м', '{"label": "иридий-193м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'фермий-252', 'фермий-252', '{"label": "фермий-252"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иридий-192м', 'иридий-192м', '{"label": "иридий-192м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иридий-192', 'иридий-192', '{"label": "иридий-192"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иридий-190м', 'иридий-190м', '{"label": "иридий-190м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иридий-190м2', 'иридий-190м2', '{"label": "иридий-190м2"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иридий-190', 'иридий-190', '{"label": "иридий-190"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'осмий-180', 'осмий-180', '{"label": "осмий-180"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рений-189', 'рений-189', '{"label": "рений-189"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рений-188м', 'рений-188м', '{"label": "рений-188м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рений-188', 'рений-188', '{"label": "рений-188"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'золото-193', 'золото-193', '{"label": "золото-193"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'осмий-191м', 'осмий-191м', '{"label": "осмий-191м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'осмий-191', 'осмий-191', '{"label": "осмий-191"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'осмий-185', 'осмий-185', '{"label": "осмий-185"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'осмий-182', 'осмий-182', '{"label": "осмий-182"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'осмий-181', 'осмий-181', '{"label": "осмий-181"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'уран естественный', 'уран естественный', '{"label": "уран естественный"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сумма нуклидов иода*', 'сумма нуклидов иода*', '{"label": "сумма нуклидов иода*"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'торий-естественный', 'торий-естественный', '{"label": "торий-естественный"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сумма нуклидов плутония', 'сумма нуклидов плутония', '{"label": "сумма нуклидов плутония"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'менделеевий-258', 'менделеевий-258', '{"label": "менделеевий-258"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'менделеевий-257', 'менделеевий-257', '{"label": "менделеевий-257"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'фермий-257', 'фермий-257', '{"label": "фермий-257"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'фермий-255', 'фермий-255', '{"label": "фермий-255"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'фермий-254', 'фермий-254', '{"label": "фермий-254"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сумма ирг', 'сумма ирг', '{"label": "сумма ирг"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'фермий-253', 'фермий-253', '{"label": "фермий-253"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'эйнштейний-254м', 'эйнштейний-254м', '{"label": "эйнштейний-254м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'берклий-246', 'берклий-246', '{"label": "берклий-246"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'берклий-245', 'берклий-245', '{"label": "берклий-245"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кюрий-250', 'кюрий-250', '{"label": "кюрий-250"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кюрий-249', 'кюрий-249', '{"label": "кюрий-249"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кюрий-248', 'кюрий-248', '{"label": "кюрий-248"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сумма нуклидов урана', 'сумма нуклидов урана', '{"label": "сумма нуклидов урана"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кюрий-247', 'кюрий-247', '{"label": "кюрий-247"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кюрий-246', 'кюрий-246', '{"label": "кюрий-246"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кюрий-245', 'кюрий-245', '{"label": "кюрий-245"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кюрий-244', 'кюрий-244', '{"label": "кюрий-244"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кюрий-243', 'кюрий-243', '{"label": "кюрий-243"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кюрий-242', 'кюрий-242', '{"label": "кюрий-242"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кюрий-241', 'кюрий-241', '{"label": "кюрий-241"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кюрий-240', 'кюрий-240', '{"label": "кюрий-240"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кюрий-238', 'кюрий-238', '{"label": "кюрий-238"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'плутоний-243', 'плутоний-243', '{"label": "плутоний-243"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'плутоний-242', 'плутоний-242', '{"label": "плутоний-242"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'плутоний-241', 'плутоний-241', '{"label": "плутоний-241"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'плутоний-240', 'плутоний-240', '{"label": "плутоний-240"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'плутоний-239', 'плутоний-239', '{"label": "плутоний-239"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'плутоний-238', 'плутоний-238', '{"label": "плутоний-238"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'плутоний-237', 'плутоний-237', '{"label": "плутоний-237"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'плутоний-236', 'плутоний-236', '{"label": "плутоний-236"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'плутоний-235', 'плутоний-235', '{"label": "плутоний-235"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'плутоний-234', 'плутоний-234', '{"label": "плутоний-234"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'нептуний-240', 'нептуний-240', '{"label": "нептуний-240"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'нептуний-239', 'нептуний-239', '{"label": "нептуний-239"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'нептуний-238', 'нептуний-238', '{"label": "нептуний-238"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'нептуний-237', 'нептуний-237', '{"label": "нептуний-237"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'нептуний-236м', 'нептуний-236м', '{"label": "нептуний-236м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'протактиний-234', 'протактиний-234', '{"label": "протактиний-234"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'протактиний-233', 'протактиний-233', '{"label": "протактиний-233"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'протактиний-232', 'протактиний-232', '{"label": "протактиний-232"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'нептуний-236', 'нептуний-236', '{"label": "нептуний-236"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'нептуний-235', 'нептуний-235', '{"label": "нептуний-235"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'нептуний-234', 'нептуний-234', '{"label": "нептуний-234"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'нептуний-233', 'нептуний-233', '{"label": "нептуний-233"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'нептуний-232', 'нептуний-232', '{"label": "нептуний-232"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'уран-240', 'уран-240', '{"label": "уран-240"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'уран-239', 'уран-239', '{"label": "уран-239"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'уран-238', 'уран-238', '{"label": "уран-238"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'уран-234', 'уран-234', '{"label": "уран-234"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'уран-233', 'уран-233', '{"label": "уран-233"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'уран-231', 'уран-231', '{"label": "уран-231"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'уран-230', 'уран-230', '{"label": "уран-230"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'актиний-227', 'актиний-227', '{"label": "актиний-227"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'актиний-226', 'актиний-226', '{"label": "актиний-226"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'актиний-225', 'актиний-225', '{"label": "актиний-225"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'актиний-224', 'актиний-224', '{"label": "актиний-224"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'радий-228', 'радий-228', '{"label": "радий-228"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'радий-223', 'радий-223', '{"label": "радий-223"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'франций-223', 'франций-223', '{"label": "франций-223"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'франций-222', 'франций-222', '{"label": "франций-222"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'радон-222', 'радон-222', '{"label": "радон-222"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'астат-211', 'астат-211', '{"label": "астат-211"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'астат-207', 'астат-207', '{"label": "астат-207"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'свинец-212', 'свинец-212', '{"label": "свинец-212"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'свинец-211', 'свинец-211', '{"label": "свинец-211"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'свинец-210', 'свинец-210', '{"label": "свинец-210"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'свинец-209', 'свинец-209', '{"label": "свинец-209"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'свинец-205', 'свинец-205', '{"label": "свинец-205"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'свинец-203', 'свинец-203', '{"label": "свинец-203"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'свинец-202м', 'свинец-202м', '{"label": "свинец-202м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'свинец-202', 'свинец-202', '{"label": "свинец-202"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'свинец-200', 'свинец-200', '{"label": "свинец-200"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'свинец-199', 'свинец-199', '{"label": "свинец-199"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'свинец-198', 'свинец-198', '{"label": "свинец-198"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'таллий-204', 'таллий-204', '{"label": "таллий-204"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'таллий-202', 'таллий-202', '{"label": "таллий-202"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'таллий-201', 'таллий-201', '{"label": "таллий-201"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ртуть-193', 'ртуть-193', '{"label": "ртуть-193"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'золото-201', 'золото-201', '{"label": "золото-201"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'золото-200м', 'золото-200м', '{"label": "золото-200м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'золото-200', 'золото-200', '{"label": "золото-200"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'золото-199', 'золото-199', '{"label": "золото-199"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'золото-198м', 'золото-198м', '{"label": "золото-198м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'платина-199', 'платина-199', '{"label": "платина-199"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'платина-197м', 'платина-197м', '{"label": "платина-197м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'платина-197', 'платина-197', '{"label": "платина-197"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'платина-195м', 'платина-195м', '{"label": "платина-195м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'платина-193м', 'платина-193м', '{"label": "платина-193м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иридий-189', 'иридий-189', '{"label": "иридий-189"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иридий-188', 'иридий-188', '{"label": "иридий-188"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иридий-187', 'иридий-187', '{"label": "иридий-187"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иридий-186м', 'иридий-186м', '{"label": "иридий-186м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иридий-186', 'иридий-186', '{"label": "иридий-186"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иридий-185', 'иридий-185', '{"label": "иридий-185"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иридий-184', 'иридий-184', '{"label": "иридий-184"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иридий-182', 'иридий-182', '{"label": "иридий-182"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цинк-63', 'цинк-63', '{"label": "цинк-63"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кобальт-60', 'кобальт-60', '{"label": "кобальт-60"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'кобальт-58м', 'кобальт-58м', '{"label": "кобальт-58м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'золото-194', 'золото-194', '{"label": "золото-194"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'осмий-193', 'осмий-193', '{"label": "осмий-193"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'висмут-210м', 'висмут-210м', '{"label": "висмут-210м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'висмут-210', 'висмут-210', '{"label": "висмут-210"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'таллий-194', 'таллий-194', '{"label": "таллий-194"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ртуть-203', 'ртуть-203', '{"label": "ртуть-203"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иридий-194м', 'иридий-194м', '{"label": "иридий-194м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иридий-194', 'иридий-194', '{"label": "иридий-194"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'рений-184', 'рений-184', '{"label": "рений-184"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гафний-182', 'гафний-182', '{"label": "гафний-182"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гафний-181', 'гафний-181', '{"label": "гафний-181"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гафний-180м', 'гафний-180м', '{"label": "гафний-180м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гафний-179м', 'гафний-179м', '{"label": "гафний-179м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'лютеций-174', 'лютеций-174', '{"label": "лютеций-174"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'лютеций-173', 'лютеций-173', '{"label": "лютеций-173"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тулий-171', 'тулий-171', '{"label": "тулий-171"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гольмий-162', 'гольмий-162', '{"label": "гольмий-162"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тербий-157', 'тербий-157', '{"label": "тербий-157"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тербий-156м', 'тербий-156м', '{"label": "тербий-156м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тербий-156м2', 'тербий-156м2', '{"label": "тербий-156м2"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'полоний-208', 'полоний-208', '{"label": "полоний-208"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'родий-106', 'родий-106', '{"label": "родий-106"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'полоний-209', 'полоний-209', '{"label": "полоний-209"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'продукты деления*', 'продукты деления*', '{"label": "продукты деления*"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'эйнштейний-254', 'эйнштейний-254', '{"label": "эйнштейний-254"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'эйнштейний-253', 'эйнштейний-253', '{"label": "эйнштейний-253"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'эйнштейний-251', 'эйнштейний-251', '{"label": "эйнштейний-251"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'эйнштейний-250', 'эйнштейний-250', '{"label": "эйнштейний-250"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'калифорний-254', 'калифорний-254', '{"label": "калифорний-254"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'калифорний-253', 'калифорний-253', '{"label": "калифорний-253"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'калифорний-252', 'калифорний-252', '{"label": "калифорний-252"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'калифорний-251', 'калифорний-251', '{"label": "калифорний-251"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'калифорний-250', 'калифорний-250', '{"label": "калифорний-250"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'калифорний-249', 'калифорний-249', '{"label": "калифорний-249"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'калифорний-248', 'калифорний-248', '{"label": "калифорний-248"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'калифорний-244', 'калифорний-244', '{"label": "калифорний-244"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'берклий-250', 'берклий-250', '{"label": "берклий-250"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'берклий-249', 'берклий-249', '{"label": "берклий-249"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'берклий-247', 'берклий-247', '{"label": "берклий-247"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'америций-246', 'америций-246', '{"label": "америций-246"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'америций-245', 'америций-245', '{"label": "америций-245"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'америций-244м', 'америций-244м', '{"label": "америций-244м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'америций-244', 'америций-244', '{"label": "америций-244"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'америций-243', 'америций-243', '{"label": "америций-243"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'америций-242м', 'америций-242м', '{"label": "америций-242м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'америций-242', 'америций-242', '{"label": "америций-242"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'америций-241', 'америций-241', '{"label": "америций-241"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'америций-237', 'америций-237', '{"label": "америций-237"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'плутоний-246', 'плутоний-246', '{"label": "плутоний-246"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сумма альфа-активных искусств. нуклидов', 'сумма альфа-активных искусств. нуклидов', '{"label": "сумма альфа-активных искусств. нуклидов"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'уран обедненный', 'уран обедненный', '{"label": "уран обедненный"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сумма бета-активных искусств. нуклидов', 'сумма бета-активных искусств. нуклидов', '{"label": "сумма бета-активных искусств. нуклидов"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'сумма бета-активных радионуклидов*', 'сумма бета-активных радионуклидов*', '{"label": "сумма бета-активных радионуклидов*"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'калифорний-246', 'калифорний-246', '{"label": "калифорний-246"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'америций-240', 'америций-240', '{"label": "америций-240"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'америций-239', 'америций-239', '{"label": "америций-239"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'америций-238', 'америций-238', '{"label": "америций-238"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'уран-237', 'уран-237', '{"label": "уран-237"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'уран-236', 'уран-236', '{"label": "уран-236"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'уран-235', 'уран-235', '{"label": "уран-235"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'радий-227', 'радий-227', '{"label": "радий-227"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'радий-226', 'радий-226', '{"label": "радий-226"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'радий-225', 'радий-225', '{"label": "радий-225"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'радий-224', 'радий-224', '{"label": "радий-224"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'свинец-201', 'свинец-201', '{"label": "свинец-201"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'золото-198', 'золото-198', '{"label": "золото-198"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'золото-195', 'золото-195', '{"label": "золото-195"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цинк-62', 'цинк-62', '{"label": "цинк-62"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'аргон-37', 'аргон-37', '{"label": "аргон-37"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'бериллий-7', 'бериллий-7', '{"label": "бериллий-7"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тритий', 'тритий', '{"label": "тритий"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'Нуклид-1', 'Нуклид-1', '{"label": "Нуклид-1"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'тербий-156', 'тербий-156', '{"label": "тербий-156"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гадолиний-146', 'гадолиний-146', '{"label": "гадолиний-146"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'гадолиний-145', 'гадолиний-145', '{"label": "гадолиний-145"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'лантан-140', 'лантан-140', '{"label": "лантан-140"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'лантан-138', 'лантан-138', '{"label": "лантан-138"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'цезий-138', 'цезий-138', '{"label": "цезий-138"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'теллур-127м', 'теллур-127м', '{"label": "теллур-127м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'теллур-127', 'теллур-127', '{"label": "теллур-127"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'олово-125', 'олово-125', '{"label": "олово-125"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'олово-123', 'олово-123', '{"label": "олово-123"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'олово-121м', 'олово-121м', '{"label": "олово-121м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'серебро-110м', 'серебро-110м', '{"label": "серебро-110м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'серебро-108м', 'серебро-108м', '{"label": "серебро-108м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'родий-107', 'родий-107', '{"label": "родий-107"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'родий-106м', 'родий-106м', '{"label": "родий-106м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'родий-105', 'родий-105', '{"label": "родий-105"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'технеций-104', 'технеций-104', '{"label": "технеций-104"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'технеций-101', 'технеций-101', '{"label": "технеций-101"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иттрий-90', 'иттрий-90', '{"label": "иттрий-90"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'иттрий-88', 'иттрий-88', '{"label": "иттрий-88"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'ниобий-88', 'ниобий-88', '{"label": "ниобий-88"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'стронций-80', 'стронций-80', '{"label": "стронций-80"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'криптон-87', 'криптон-87', '{"label": "криптон-87"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'криптон-85', 'криптон-85', '{"label": "криптон-85"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'криптон-81', 'криптон-81', '{"label": "криптон-81"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'селен-79', 'селен-79', '{"label": "селен-79"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'селен-75', 'селен-75', '{"label": "селен-75"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'селен-73м', 'селен-73м', '{"label": "селен-73м"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('radionuclide_voc', 'германий-75', 'германий-75', '{"label": "германий-75"}', 1, 0, '0');

INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '10УГ', '10УГ', '{"3": "10УГ", "4": "углерод-14", "5": "2", "6": "10", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Входит в набор 10УГ.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'APu8', 'APu8', '{"3": "APu8", "4": "плутоний-238", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '11Н6383-20', '11Н6383-20', '{"3": "11Н6383-20", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "340000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '12', '12', '{"3": "12", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "14500000000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '12а', '12а', '{"3": "12а", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "13400000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1а', '1а', '{"3": "1а", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "20000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1б', '1б', '{"3": "1б", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "20000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1г', '1г', '{"3": "1г", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "5480000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1КО', '1КО', '{"3": "1КО", "4": "кобальт-60", "5": "2", "6": "1", "7": "0.0", "8": "19000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П8', '1П8', '{"3": "1П8", "4": "плутоний-238", "5": "3", "6": "", "7": "0.0", "8": "3200000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9', '1П9', '{"3": "1П9", "4": "плутоний-239", "5": "3", "6": "35(12); 1,5; пл. раб.пов., см2: 1", "7": "4.0", "8": "100000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-101', '1П9-101', '{"3": "1П9-101", "4": "плутоний-239", "5": "3", "6": "", "7": "10.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-102', '1П9-102', '{"3": "1П9-102", "4": "плутоний-239", "5": "3", "6": "", "7": "100.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-103', '1П9-103', '{"3": "1П9-103", "4": "плутоний-239", "5": "3", "6": "", "7": "1000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-104', '1П9-104', '{"3": "1П9-104", "4": "плутоний-239", "5": "3", "6": "", "7": "10000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-105', '1П9-105', '{"3": "1П9-105", "4": "плутоний-239", "5": "3", "6": "", "7": "100000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-121', '1П9-121', '{"3": "1П9-121", "4": "плутоний-239", "5": "3", "6": "", "7": "0.0", "8": "5.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-122', '1П9-122', '{"3": "1П9-122", "4": "плутоний-239", "5": "3", "6": "", "7": "0.0", "8": "100.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-123', '1П9-123', '{"3": "1П9-123", "4": "плутоний-239", "5": "3", "6": "", "7": "0.0", "8": "2100.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-161', '1П9-161', '{"3": "1П9-161", "4": "плутоний-239", "5": "3", "6": "", "7": "16.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-162', '1П9-162', '{"3": "1П9-162", "4": "плутоний-239", "5": "3", "6": "", "7": "160.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-163', '1П9-163', '{"3": "1П9-163", "4": "плутоний-239", "5": "3", "6": "", "7": "16000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-164', '1П9-164', '{"3": "1П9-164", "4": "плутоний-239", "5": "3", "6": "", "7": "16000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-251', '1П9-251', '{"3": "1П9-251", "4": "плутоний-239", "5": "3", "6": "", "7": "25.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-252', '1П9-252', '{"3": "1П9-252", "4": "плутоний-239", "5": "3", "6": "", "7": "250.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-253', '1П9-253', '{"3": "1П9-253", "4": "плутоний-239", "5": "3", "6": "", "7": "2500.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-254', '1П9-254', '{"3": "1П9-254", "4": "плутоний-239", "5": "3", "6": "", "7": "25000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-364', '1П9-364', '{"3": "1П9-364", "4": "плутоний-239", "5": "3", "6": "", "7": "", "8": "63000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-400', '1П9-400', '{"3": "1П9-400", "4": "плутоний-239", "5": "3", "6": "", "7": "4.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-401', '1П9-401', '{"3": "1П9-401", "4": "плутоний-239", "5": "3", "6": "", "7": "40.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-402', '1П9-402', '{"3": "1П9-402", "4": "плутоний-239", "5": "3", "6": "", "7": "400.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-403', '1П9-403', '{"3": "1П9-403", "4": "плутоний-239", "5": "3", "6": "", "7": "4000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-404', '1П9-404', '{"3": "1П9-404", "4": "плутоний-239", "5": "3", "6": "", "7": "40000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-630', '1П9-630', '{"3": "1П9-630", "4": "плутоний-239", "5": "3", "6": "", "7": "6.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-631', '1П9-631', '{"3": "1П9-631", "4": "плутоний-239", "5": "3", "6": "", "7": "63.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-632', '1П9-632', '{"3": "1П9-632", "4": "плутоний-239", "5": "3", "6": "", "7": "630.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-633', '1П9-633', '{"3": "1П9-633", "4": "плутоний-239", "5": "3", "6": "", "7": "6300.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-634', '1П9-634', '{"3": "1П9-634", "4": "плутоний-239", "5": "3", "6": "", "7": "63000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО', '1СО', '{"3": "1СО", "4": "стронций-90; иттрий-90", "5": "2", "6": "35(12); 1,5; пл. раб.пов., см2: 1", "7": "13.0", "8": "3200000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-131', '1СО-131', '{"3": "1СО-131", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "13.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-132', '1СО-132', '{"3": "1СО-132", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "130.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-133', '1СО-133', '{"3": "1СО-133", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "1300.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-134', '1СО-134', '{"3": "1СО-134", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "13000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-135', '1СО-135', '{"3": "1СО-135", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "130000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-136', '1СО-136', '{"3": "1СО-136", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "1300000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-211', '1СО-211', '{"3": "1СО-211", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "21.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-212', '1СО-212', '{"3": "1СО-212", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "210.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-213', '1СО-213', '{"3": "1СО-213", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "2100.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-214', '1СО-214', '{"3": "1СО-214", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "21000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-215', '1СО-215', '{"3": "1СО-215", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "210000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-216', '1СО-216', '{"3": "1СО-216", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "2100000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-321', '1СО-321', '{"3": "1СО-321", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "32.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-322', '1СО-322', '{"3": "1СО-322", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "320.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-323', '1СО-323', '{"3": "1СО-323", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "3200.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-324', '1СО-324', '{"3": "1СО-324", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "32000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-325', '1СО-325', '{"3": "1СО-325", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "320000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-326', '1СО-326', '{"3": "1СО-326", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "3200000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-34', '1СО-34', '{"3": "1СО-34", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "0.0", "8": "75500.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-531', '1СО-531', '{"3": "1СО-531", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "53.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-532', '1СО-532', '{"3": "1СО-532", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "530.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-533', '1СО-533', '{"3": "1СО-533", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "5300.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-534', '1СО-534', '{"3": "1СО-534", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "53000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-535', '1СО-535', '{"3": "1СО-535", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "530000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-801', '1СО-801', '{"3": "1СО-801", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "80.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-802', '1СО-802', '{"3": "1СО-802", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "800.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-803', '1СО-803', '{"3": "1СО-803", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "8000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-804', '1СО-804', '{"3": "1СО-804", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "80000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-805', '1СО-805', '{"3": "1СО-805", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "800000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-83', '1СО-83', '{"3": "1СО-83", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "0.0", "8": "24000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-84', '1СО-84', '{"3": "1СО-84", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "0.0", "8": "238000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1Т4', '1Т4', '{"3": "1Т4", "4": "таллий-204", "5": "2", "6": "1", "7": "", "8": "20500.0", "9": "0", "10": "", "11": "", "12": "Входит в набор 1Т4.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1У4', '1У4', '{"3": "1У4", "4": "уран-234", "5": "3", "6": "35(12)x1,5; пл. раб.пов., см2: 1", "7": "4.0", "8": "100000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1У4-101', '1У4-101', '{"3": "1У4-101", "4": "уран-234", "5": "3", "6": "", "7": "10.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1У4-161', '1У4-161', '{"3": "1У4-161", "4": "уран-234", "5": "3", "6": "", "7": "16.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1У4-251', '1У4-251', '{"3": "1У4-251", "4": "уран-234", "5": "3", "6": "", "7": "25.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1У4-400', '1У4-400', '{"3": "1У4-400", "4": "уран-234", "5": "3", "6": "", "7": "4.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1У4-401', '1У4-401', '{"3": "1У4-401", "4": "уран-234", "5": "3", "6": "", "7": "40.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1У4-630', '1У4-630', '{"3": "1У4-630", "4": "уран-234", "5": "3", "6": "", "7": "6.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1У8', '1У8', '{"3": "1У8", "4": "уран-238", "5": "3", "6": "35(12)x1,5; пл. раб.пов., см2: 1", "7": "4.0", "8": "100000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1У8-400', '1У8-400', '{"3": "1У8-400", "4": "уран естественный", "5": "3", "6": "", "7": "4.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1УГ', '1УГ', '{"3": "1УГ", "4": "углерод-14", "5": "2", "6": "1", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Входит в набор 1УГ.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2.3 СВЗТ/3', '2.3 СВЗТ/3', '{"3": "2.3 СВЗТ/3", "4": "тритий", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2.3 СВЗТ/3 "Выход"', '2.3 СВЗТ/3 "Выход"', '{"3": "2.3 СВЗТ/3 \"Выход\"", "4": "тритий", "5": "2", "6": "", "7": "", "8": "925000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2.3 СВЗТ/3 "Стрелка"', '2.3 СВЗТ/3 "Стрелка"', '{"3": "2.3 СВЗТ/3 \"Стрелка\"", "4": "тритий", "5": "2", "6": "", "7": "", "8": "575000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2г', '2г', '{"3": "2г", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "159000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2К-0', '2К-0', '{"3": "2К-0", "4": "кобальт-60", "5": "2", "6": "4", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Входит в набор 2К-0.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9', '2П9', '{"3": "2П9", "4": "плутоний-239", "5": "3", "6": "52(22); 1,5; пл. раб.пов., см2: 4", "7": "16.0", "8": "400000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-102', '2П9-102', '{"3": "2П9-102", "4": "плутоний-239", "5": "3", "6": "", "7": "100.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-103', '2П9-103', '{"3": "2П9-103", "4": "плутоний-239", "5": "3", "6": "", "7": "1000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-104', '2П9-104', '{"3": "2П9-104", "4": "плутоний-239", "5": "3", "6": "", "7": "10000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-105', '2П9-105', '{"3": "2П9-105", "4": "плутоний-239", "5": "3", "6": "", "7": "100000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-161', '2П9-161', '{"3": "2П9-161", "4": "плутоний-239", "5": "3", "6": "", "7": "16.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-162', '2П9-162', '{"3": "2П9-162", "4": "плутоний-239", "5": "3", "6": "", "7": "160.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-163', '2П9-163', '{"3": "2П9-163", "4": "плутоний-239", "5": "3", "6": "", "7": "1600.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-164', '2П9-164', '{"3": "2П9-164", "4": "плутоний-239", "5": "3", "6": "", "7": "16000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-165', '2П9-165', '{"3": "2П9-165", "4": "плутоний-239", "5": "3", "6": "", "7": "160000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-251', '2П9-251', '{"3": "2П9-251", "4": "плутоний-239", "5": "3", "6": "", "7": "25.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-252', '2П9-252', '{"3": "2П9-252", "4": "плутоний-239", "5": "3", "6": "", "7": "250.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-253', '2П9-253', '{"3": "2П9-253", "4": "плутоний-239", "5": "3", "6": "", "7": "2500.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-254', '2П9-254', '{"3": "2П9-254", "4": "плутоний-239", "5": "3", "6": "", "7": "25000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-255', '2П9-255', '{"3": "2П9-255", "4": "плутоний-239", "5": "3", "6": "", "7": "250000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-401', '2П9-401', '{"3": "2П9-401", "4": "плутоний-239", "5": "3", "6": "", "7": "40.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-402', '2П9-402', '{"3": "2П9-402", "4": "плутоний-239", "5": "3", "6": "", "7": "400.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-403', '2П9-403', '{"3": "2П9-403", "4": "плутоний-239", "5": "3", "6": "", "7": "4000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-404', '2П9-404', '{"3": "2П9-404", "4": "плутоний-239", "5": "3", "6": "", "7": "40000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-405', '2П9-405', '{"3": "2П9-405", "4": "плутоний-239", "5": "3", "6": "", "7": "400000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-631', '2П9-631', '{"3": "2П9-631", "4": "плутоний-239", "5": "3", "6": "", "7": "63.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-632', '2П9-632', '{"3": "2П9-632", "4": "плутоний-239", "5": "3", "6": "", "7": "630.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-633', '2П9-633', '{"3": "2П9-633", "4": "плутоний-239", "5": "3", "6": "", "7": "6300.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-634', '2П9-634', '{"3": "2П9-634", "4": "плутоний-239", "5": "3", "6": "", "7": "63000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО', '2СО', '{"3": "2СО", "4": "стронций-90; иттрий-90", "5": "2", "6": "52(22); 1,5; пл. раб.пов., см2: 4", "7": "21.0", "8": "13000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-132', '2СО-132', '{"3": "2СО-132", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "130.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-133', '2СО-133', '{"3": "2СО-133", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "1300.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-134', '2СО-134', '{"3": "2СО-134", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "13000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-135', '2СО-135', '{"3": "2СО-135", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "130000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-136', '2СО-136', '{"3": "2СО-136", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "1300000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-137', '2СО-137', '{"3": "2СО-137", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "13000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-211', '2СО-211', '{"3": "2СО-211", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "21.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-212', '2СО-212', '{"3": "2СО-212", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "210.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-213', '2СО-213', '{"3": "2СО-213", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "2100.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-214', '2СО-214', '{"3": "2СО-214", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "21000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-215', '2СО-215', '{"3": "2СО-215", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "210000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-216', '2СО-216', '{"3": "2СО-216", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "2100000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-321', '2СО-321', '{"3": "2СО-321", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "32.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-322', '2СО-322', '{"3": "2СО-322", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "320.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-323', '2СО-323', '{"3": "2СО-323", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "3200.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-324', '2СО-324', '{"3": "2СО-324", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "32000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-325', '2СО-325', '{"3": "2СО-325", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "320000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-326', '2СО-326', '{"3": "2СО-326", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "3200000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-531', '2СО-531', '{"3": "2СО-531", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "53.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-532', '2СО-532', '{"3": "2СО-532", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "530.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-533', '2СО-533', '{"3": "2СО-533", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "5300.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-534', '2СО-534', '{"3": "2СО-534", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "53000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-535', '2СО-535', '{"3": "2СО-535", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "530000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-536', '2СО-536', '{"3": "2СО-536", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "5300000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-801', '2СО-801', '{"3": "2СО-801", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "80.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-802', '2СО-802', '{"3": "2СО-802", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "800.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-803', '2СО-803', '{"3": "2СО-803", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "8000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-804', '2СО-804', '{"3": "2СО-804", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "80000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-805', '2СО-805', '{"3": "2СО-805", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "800000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-806', '2СО-806', '{"3": "2СО-806", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "8000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2Т4', '2Т4', '{"3": "2Т4", "4": "таллий-204", "5": "2", "6": "4", "7": "", "8": "10000.0", "9": "0", "10": "", "11": "", "12": "Входит в набор 2Т4.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2У4', '2У4', '{"3": "2У4", "4": "уран-234", "5": "3", "6": "52(22); 1,5; пл. раб.пов., см2: 4", "7": "6.3", "8": "160.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2У4-101', '2У4-101', '{"3": "2У4-101", "4": "уран-234", "5": "3", "6": "", "7": "10.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2У4-102', '2У4-102', '{"3": "2У4-102", "4": "уран-234", "5": "3", "6": "", "7": "100.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2У4-161', '2У4-161', '{"3": "2У4-161", "4": "уран-234", "5": "3", "6": "", "7": "16.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2У4-162', '2У4-162', '{"3": "2У4-162", "4": "уран-234", "5": "3", "6": "", "7": "160.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2У4-251', '2У4-251', '{"3": "2У4-251", "4": "уран-234", "5": "3", "6": "", "7": "250.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2У4-401', '2У4-401', '{"3": "2У4-401", "4": "уран-234", "5": "3", "6": "", "7": "40.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2У4-630', '2У4-630', '{"3": "2У4-630", "4": "уран-234", "5": "3", "6": "", "7": "6.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2У4-631', '2У4-631', '{"3": "2У4-631", "4": "уран-234", "5": "3", "6": "", "7": "63.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2У8', '2У8', '{"3": "2У8", "4": "уран-238", "5": "3", "6": "52(22); 1,5; пл. раб.пов., см2: 4", "7": "6.3", "8": "16.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2У8-101', '2У8-101', '{"3": "2У8-101", "4": "уран естественный", "5": "3", "6": "", "7": "10.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2У8-161', '2У8-161', '{"3": "2У8-161", "4": "уран естественный", "5": "3", "6": "", "7": "16.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2У8-630', '2У8-630', '{"3": "2У8-630", "4": "уран естественный", "5": "3", "6": "", "7": "6.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3арп-9', '3арп-9', '{"3": "3арп-9", "4": "плутоний-239", "5": "3", "6": "", "7": "", "8": "833000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3брс-0', '3брс-0', '{"3": "3брс-0", "4": "стронций-90", "5": "2", "6": "", "7": "", "8": "5160000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3КО', '3КО', '{"3": "3КО", "4": "кобальт-60", "5": "2", "6": "", "7": "0.0", "8": "4840.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9', '3П9', '{"3": "3П9", "4": "плутоний-239", "5": "3", "6": "66(36); 1,5; пл. раб.пов., см2: 10", "7": "40.0", "8": "1000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-102', '3П9-102', '{"3": "3П9-102", "4": "плутоний-239", "5": "3", "6": "", "7": "100.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-103', '3П9-103', '{"3": "3П9-103", "4": "плутоний-239", "5": "3", "6": "", "7": "1000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-104', '3П9-104', '{"3": "3П9-104", "4": "плутоний-239", "5": "3", "6": "", "7": "10000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-105', '3П9-105', '{"3": "3П9-105", "4": "плутоний-239", "5": "3", "6": "", "7": "100000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-106', '3П9-106', '{"3": "3П9-106", "4": "плутоний-239", "5": "3", "6": "", "7": "1000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-162', '3П9-162', '{"3": "3П9-162", "4": "плутоний-239", "5": "3", "6": "", "7": "160.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-163', '3П9-163', '{"3": "3П9-163", "4": "плутоний-239", "5": "3", "6": "", "7": "1600.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-164', '3П9-164', '{"3": "3П9-164", "4": "плутоний-239", "5": "3", "6": "", "7": "16000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-165', '3П9-165', '{"3": "3П9-165", "4": "плутоний-239", "5": "3", "6": "", "7": "160000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-252', '3П9-252', '{"3": "3П9-252", "4": "плутоний-239", "5": "3", "6": "", "7": "250.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-253', '3П9-253', '{"3": "3П9-253", "4": "плутоний-239", "5": "3", "6": "", "7": "2500.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-254', '3П9-254', '{"3": "3П9-254", "4": "плутоний-239", "5": "3", "6": "", "7": "25000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-255', '3П9-255', '{"3": "3П9-255", "4": "плутоний-239", "5": "3", "6": "", "7": "200000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-401', '3П9-401', '{"3": "3П9-401", "4": "плутоний-239", "5": "3", "6": "", "7": "40.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-402', '3П9-402', '{"3": "3П9-402", "4": "плутоний-239", "5": "3", "6": "", "7": "400.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-403', '3П9-403', '{"3": "3П9-403", "4": "плутоний-239", "5": "3", "6": "", "7": "4000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-404', '3П9-404', '{"3": "3П9-404", "4": "плутоний-239", "5": "3", "6": "", "7": "40000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-405', '3П9-405', '{"3": "3П9-405", "4": "плутоний-239", "5": "3", "6": "", "7": "400000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-631', '3П9-631', '{"3": "3П9-631", "4": "плутоний-239", "5": "3", "6": "", "7": "63.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-632', '3П9-632', '{"3": "3П9-632", "4": "плутоний-239", "5": "3", "6": "", "7": "630.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-633', '3П9-633', '{"3": "3П9-633", "4": "плутоний-239", "5": "3", "6": "", "7": "6300.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-634', '3П9-634', '{"3": "3П9-634", "4": "плутоний-239", "5": "3", "6": "", "7": "63000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-635', '3П9-635', '{"3": "3П9-635", "4": "плутоний-239", "5": "3", "6": "", "7": "630000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО', '3СО', '{"3": "3СО", "4": "стронций-90; иттрий-90", "5": "2", "6": "66(36); 1,5; пл. раб.пов., см2: 10", "7": "80.0", "8": "32000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-132', '3СО-132', '{"3": "3СО-132", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "130.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-133', '3СО-133', '{"3": "3СО-133", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "1300.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-134', '3СО-134', '{"3": "3СО-134", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "13000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-135', '3СО-135', '{"3": "3СО-135", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "130000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-136', '3СО-136', '{"3": "3СО-136", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "1300000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-137', '3СО-137', '{"3": "3СО-137", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "13000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-212', '3СО-212', '{"3": "3СО-212", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "210.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-213', '3СО-213', '{"3": "3СО-213", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "2100.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-214', '3СО-214', '{"3": "3СО-214", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "21000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-215', '3СО-215', '{"3": "3СО-215", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "210000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-216', '3СО-216', '{"3": "3СО-216", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "2100000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-217', '3СО-217', '{"3": "3СО-217", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "21000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-322', '3СО-322', '{"3": "3СО-322", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "320.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-323', '3СО-323', '{"3": "3СО-323", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "3200.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-324', '3СО-324', '{"3": "3СО-324", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "32000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-325', '3СО-325', '{"3": "3СО-325", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "320000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-326', '3СО-326', '{"3": "3СО-326", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "3200000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-327', '3СО-327', '{"3": "3СО-327", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "32000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-532', '3СО-532', '{"3": "3СО-532", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "530.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-533', '3СО-533', '{"3": "3СО-533", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "5300.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-534', '3СО-534', '{"3": "3СО-534", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "53000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-535', '3СО-535', '{"3": "3СО-535", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "530000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-536', '3СО-536', '{"3": "3СО-536", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "5300000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-801', '3СО-801', '{"3": "3СО-801", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "80.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-802', '3СО-802', '{"3": "3СО-802", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "800.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-803', '3СО-803', '{"3": "3СО-803", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "8000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-804', '3СО-804', '{"3": "3СО-804", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "80000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-805', '3СО-805', '{"3": "3СО-805", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "800000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-806', '3СО-806', '{"3": "3СО-806", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "8000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-807', '3СО-807', '{"3": "3СО-807", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "8000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3Т4', '3Т4', '{"3": "3Т4", "4": "таллий-204", "5": "2", "6": "10", "7": "", "8": "12000.0", "9": "0", "10": "", "11": "", "12": "Входит в набор 3Т4.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3У4', '3У4', '{"3": "3У4", "4": "уран-234", "5": "3", "6": "66(36); 1,5; пл. раб.пов., см2: 10", "7": "10.0", "8": "250.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3У4-101', '3У4-101', '{"3": "3У4-101", "4": "уран-234", "5": "3", "6": "", "7": "10.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3У4-102', '3У4-102', '{"3": "3У4-102", "4": "уран-234", "5": "3", "6": "", "7": "100.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3У4-161', '3У4-161', '{"3": "3У4-161", "4": "уран-234", "5": "3", "6": "", "7": "16.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3У4-162', '3У4-162', '{"3": "3У4-162", "4": "уран-234", "5": "3", "6": "", "7": "160.0", "8": "1190.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3У4-251', '3У4-251', '{"3": "3У4-251", "4": "уран-234", "5": "3", "6": "", "7": "25.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3У4-252', '3У4-252', '{"3": "3У4-252", "4": "уран-234", "5": "3", "6": "", "7": "250.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3У4-401', '3У4-401', '{"3": "3У4-401", "4": "уран-234", "5": "3", "6": "", "7": "40.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3У4-631', '3У4-631', '{"3": "3У4-631", "4": "уран-234", "5": "3", "6": "", "7": "63.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3У8', '3У8', '{"3": "3У8", "4": "уран-238", "5": "3", "6": "66(36); 1,5; пл. раб.пов., см2: 10", "7": "16.0", "8": "40.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3У8-161', '3У8-161', '{"3": "3У8-161", "4": "уран естественный", "5": "3", "6": "", "7": "25.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3У8-251', '3У8-251', '{"3": "3У8-251", "4": "уран естественный", "5": "3", "6": "", "7": "25.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3У8-401', '3У8-401', '{"3": "3У8-401", "4": "уран естественный", "5": "3", "6": "", "7": "40.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3Уг4', '3Уг4', '{"3": "3Уг4", "4": "углерод-14", "5": "2", "6": "", "7": "", "8": "34000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4арп-9', '4арп-9', '{"3": "4арп-9", "4": "плутоний-239", "5": "3", "6": "", "7": "", "8": "933000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4брс-0', '4брс-0', '{"3": "4брс-0", "4": "стронций-90", "5": "2", "6": "", "7": "", "8": "540000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9', '4П9', '{"3": "4П9", "4": "плутоний-239", "5": "3", "6": "101(71); 1,5; пл. раб.пов., см2: 40", "7": "40.0", "8": "4000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-102', '4П9-102', '{"3": "4П9-102", "4": "плутоний-239", "5": "3", "6": "", "7": "100.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-103', '4П9-103', '{"3": "4П9-103", "4": "плутоний-239", "5": "3", "6": "", "7": "1000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-104', '4П9-104', '{"3": "4П9-104", "4": "плутоний-239", "5": "3", "6": "", "7": "10000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-105', '4П9-105', '{"3": "4П9-105", "4": "плутоний-239", "5": "3", "6": "", "7": "100000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-106', '4П9-106', '{"3": "4П9-106", "4": "плутоний-239", "5": "3", "6": "", "7": "1000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-162', '4П9-162', '{"3": "4П9-162", "4": "плутоний-239", "5": "3", "6": "", "7": "", "8": "160.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-163', '4П9-163', '{"3": "4П9-163", "4": "плутоний-239", "5": "3", "6": "", "7": "1600.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-164', '4П9-164', '{"3": "4П9-164", "4": "плутоний-239", "5": "3", "6": "", "7": "16000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-165', '4П9-165', '{"3": "4П9-165", "4": "плутоний-239", "5": "3", "6": "", "7": "160000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-166', '4П9-166', '{"3": "4П9-166", "4": "плутоний-239", "5": "3", "6": "", "7": "1600000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-252', '4П9-252', '{"3": "4П9-252", "4": "плутоний-239", "5": "3", "6": "", "7": "250.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-253', '4П9-253', '{"3": "4П9-253", "4": "плутоний-239", "5": "3", "6": "", "7": "2500.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-254', '4П9-254', '{"3": "4П9-254", "4": "плутоний-239", "5": "3", "6": "", "7": "25000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-255', '4П9-255', '{"3": "4П9-255", "4": "плутоний-239", "5": "3", "6": "", "7": "250000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-256', '4П9-256', '{"3": "4П9-256", "4": "плутоний-239", "5": "3", "6": "", "7": "2500000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-401', '4П9-401', '{"3": "4П9-401", "4": "плутоний-239", "5": "3", "6": "", "7": "40.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-402', '4П9-402', '{"3": "4П9-402", "4": "плутоний-239", "5": "3", "6": "", "7": "400.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-403', '4П9-403', '{"3": "4П9-403", "4": "плутоний-239", "5": "3", "6": "", "7": "4000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-404', '4П9-404', '{"3": "4П9-404", "4": "плутоний-239", "5": "3", "6": "", "7": "40000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-405', '4П9-405', '{"3": "4П9-405", "4": "плутоний-239", "5": "3", "6": "", "7": "400000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-406', '4П9-406', '{"3": "4П9-406", "4": "плутоний-239", "5": "3", "6": "", "7": "4000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-631', '4П9-631', '{"3": "4П9-631", "4": "плутоний-239", "5": "3", "6": "", "7": "63.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-632', '4П9-632', '{"3": "4П9-632", "4": "плутоний-239", "5": "3", "6": "", "7": "630.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-633', '4П9-633', '{"3": "4П9-633", "4": "плутоний-239", "5": "3", "6": "", "7": "6300.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-634', '4П9-634', '{"3": "4П9-634", "4": "плутоний-239", "5": "3", "6": "", "7": "63000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-635', '4П9-635', '{"3": "4П9-635", "4": "плутоний-239", "5": "3", "6": "", "7": "630000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО', '4СО', '{"3": "4СО", "4": "стронций-90; иттрий-90", "5": "2", "6": "101(71); 1,5; пл. раб.пов., см2: 40", "7": "210.0", "8": "130000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-133', '4СО-133', '{"3": "4СО-133", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "1300.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-134', '4СО-134', '{"3": "4СО-134", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "13000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-135', '4СО-135', '{"3": "4СО-135", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "130000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-136', '4СО-136', '{"3": "4СО-136", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "1300000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-137', '4СО-137', '{"3": "4СО-137", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "13000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-138', '4СО-138', '{"3": "4СО-138", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "130000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-212', '4СО-212', '{"3": "4СО-212", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "210.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-213', '4СО-213', '{"3": "4СО-213", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "2100.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-214', '4СО-214', '{"3": "4СО-214", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "21000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-215', '4СО-215', '{"3": "4СО-215", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "210000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-216', '4СО-216', '{"3": "4СО-216", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "2100000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-217', '4СО-217', '{"3": "4СО-217", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "21000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-322', '4СО-322', '{"3": "4СО-322", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "320.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-323', '4СО-323', '{"3": "4СО-323", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "3200.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-324', '4СО-324', '{"3": "4СО-324", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "32000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-325', '4СО-325', '{"3": "4СО-325", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "320000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-326', '4СО-326', '{"3": "4СО-326", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "3200000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-327', '4СО-327', '{"3": "4СО-327", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "32000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-532', '4СО-532', '{"3": "4СО-532", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "530.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-533', '4СО-533', '{"3": "4СО-533", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "5300.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-534', '4СО-534', '{"3": "4СО-534", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "53000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-535', '4СО-535', '{"3": "4СО-535", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "530000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-536', '4СО-536', '{"3": "4СО-536", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "5300000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-537', '4СО-537', '{"3": "4СО-537", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "53000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-802', '4СО-802', '{"3": "4СО-802", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "800.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-803', '4СО-803', '{"3": "4СО-803", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "8000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-804', '4СО-804', '{"3": "4СО-804", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "80000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-805', '4СО-805', '{"3": "4СО-805", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "800000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-806', '4СО-806', '{"3": "4СО-806", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "8000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-807', '4СО-807', '{"3": "4СО-807", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "80000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4Т4', '4Т4', '{"3": "4Т4", "4": "таллий-204", "5": "2", "6": "40", "7": "", "8": "290000.0", "9": "0", "10": "", "11": "", "12": "Входит в набор 4Т4.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4У4', '4У4', '{"3": "4У4", "4": "уран-234", "5": "3", "6": "101(71); 1,5; пл. раб.пов., см2: 40", "7": "25.0", "8": "400.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4У4-102', '4У4-102', '{"3": "4У4-102", "4": "уран-234", "5": "3", "6": "", "7": "100.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4У4-162', '4У4-162', '{"3": "4У4-162", "4": "уран-234", "5": "3", "6": "", "7": "160.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4У4-251', '4У4-251', '{"3": "4У4-251", "4": "уран-234", "5": "3", "6": "", "7": "25.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4У4-252', '4У4-252', '{"3": "4У4-252", "4": "уран-234", "5": "3", "6": "", "7": "250.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4У4-401', '4У4-401', '{"3": "4У4-401", "4": "уран-234", "5": "3", "6": "", "7": "40.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4У4-402', '4У4-402', '{"3": "4У4-402", "4": "уран-234", "5": "3", "6": "", "7": "400.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4У4-631', '4У4-631', '{"3": "4У4-631", "4": "уран-234", "5": "3", "6": "", "7": "63.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4У8', '4У8', '{"3": "4У8", "4": "уран-238", "5": "3", "6": "101(71); 1,5; пл. раб.пов., см2: 40", "7": "25.0", "8": "160.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4У8-102', '4У8-102', '{"3": "4У8-102", "4": "уран естественный", "5": "3", "6": "", "7": "100.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4У8-162', '4У8-162', '{"3": "4У8-162", "4": "уран естественный", "5": "3", "6": "", "7": "160.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4У8-251', '4У8-251', '{"3": "4У8-251", "4": "уран естественный", "5": "3", "6": "", "7": "25.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4У8-401', '4У8-401', '{"3": "4У8-401", "4": "уран естественный", "5": "3", "6": "", "7": "40.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4У8-631', '4У8-631', '{"3": "4У8-631", "4": "уран естественный", "5": "3", "6": "", "7": "63.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4УГ', '4УГ', '{"3": "4УГ", "4": "углерод-14", "5": "2", "6": "4", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Входит в набор 4УГ.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4УГ4', '4УГ4', '{"3": "4УГ4", "4": "углерод-14", "5": "2", "6": "", "7": "", "8": "60000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5а', '5а', '{"3": "5а", "4": "плутоний-239", "5": "3", "6": "", "7": "", "8": "83300.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9', '5П9', '{"3": "5П9", "4": "плутоний-239", "5": "3", "6": "143(113); 1,5; пл. раб.пов., см2: 100", "7": "25.0", "8": "10000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-102', '5П9-102', '{"3": "5П9-102", "4": "плутоний-239", "5": "3", "6": "", "7": "100.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-103', '5П9-103', '{"3": "5П9-103", "4": "плутоний-239", "5": "3", "6": "", "7": "1000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-104', '5П9-104', '{"3": "5П9-104", "4": "плутоний-239", "5": "3", "6": "", "7": "10000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-105', '5П9-105', '{"3": "5П9-105", "4": "плутоний-239", "5": "3", "6": "", "7": "100000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-106', '5П9-106', '{"3": "5П9-106", "4": "плутоний-239", "5": "3", "6": "", "7": "1000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-107', '5П9-107', '{"3": "5П9-107", "4": "плутоний-239", "5": "3", "6": "", "7": "10000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-162', '5П9-162', '{"3": "5П9-162", "4": "плутоний-239", "5": "3", "6": "", "7": "160.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-163', '5П9-163', '{"3": "5П9-163", "4": "плутоний-239", "5": "3", "6": "", "7": "1600.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-164', '5П9-164', '{"3": "5П9-164", "4": "плутоний-239", "5": "3", "6": "", "7": "16000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-165', '5П9-165', '{"3": "5П9-165", "4": "плутоний-239", "5": "3", "6": "", "7": "160000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-166', '5П9-166', '{"3": "5П9-166", "4": "плутоний-239", "5": "3", "6": "", "7": "1600000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-251', '5П9-251', '{"3": "5П9-251", "4": "плутоний-239", "5": "3", "6": "", "7": "25.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-252', '5П9-252', '{"3": "5П9-252", "4": "плутоний-239", "5": "3", "6": "", "7": "250.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-253', '5П9-253', '{"3": "5П9-253", "4": "плутоний-239", "5": "3", "6": "", "7": "2500.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-254', '5П9-254', '{"3": "5П9-254", "4": "плутоний-239", "5": "3", "6": "", "7": "25000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-255', '5П9-255', '{"3": "5П9-255", "4": "плутоний-239", "5": "3", "6": "", "7": "250000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-256', '5П9-256', '{"3": "5П9-256", "4": "плутоний-239", "5": "3", "6": "", "7": "2500000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-401', '5П9-401', '{"3": "5П9-401", "4": "плутоний-239", "5": "3", "6": "", "7": "40.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-402', '5П9-402', '{"3": "5П9-402", "4": "плутоний-239", "5": "3", "6": "", "7": "400.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-403', '5П9-403', '{"3": "5П9-403", "4": "плутоний-239", "5": "3", "6": "", "7": "4000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-404', '5П9-404', '{"3": "5П9-404", "4": "плутоний-239", "5": "3", "6": "", "7": "40000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-405', '5П9-405', '{"3": "5П9-405", "4": "плутоний-239", "5": "3", "6": "", "7": "400000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-406', '5П9-406', '{"3": "5П9-406", "4": "плутоний-239", "5": "3", "6": "", "7": "4000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-631', '5П9-631', '{"3": "5П9-631", "4": "плутоний-239", "5": "3", "6": "", "7": "63.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-632', '5П9-632', '{"3": "5П9-632", "4": "плутоний-239", "5": "3", "6": "", "7": "630.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-633', '5П9-633', '{"3": "5П9-633", "4": "плутоний-239", "5": "3", "6": "", "7": "6300.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-634', '5П9-634', '{"3": "5П9-634", "4": "плутоний-239", "5": "3", "6": "", "7": "63000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-635', '5П9-635', '{"3": "5П9-635", "4": "плутоний-239", "5": "3", "6": "", "7": "630000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-636', '5П9-636', '{"3": "5П9-636", "4": "плутоний-239", "5": "3", "6": "", "7": "6300000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО', '5СО', '{"3": "5СО", "4": "стронций-90; иттрий-90", "5": "2", "6": "143(113); 1,5; пл. раб.пов., см2: 100", "7": "80.0", "8": "210000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-132', '5СО-132', '{"3": "5СО-132", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "130.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-133', '5СО-133', '{"3": "5СО-133", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "1300.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-134', '5СО-134', '{"3": "5СО-134", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "13000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-135', '5СО-135', '{"3": "5СО-135", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "130000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-136', '5СО-136', '{"3": "5СО-136", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "1300000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-137', '5СО-137', '{"3": "5СО-137", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "13000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-138', '5СО-138', '{"3": "5СО-138", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "130000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-212', '5СО-212', '{"3": "5СО-212", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "210.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-213', '5СО-213', '{"3": "5СО-213", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "2100.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-214', '5СО-214', '{"3": "5СО-214", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "21000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-215', '5СО-215', '{"3": "5СО-215", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "210000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-216', '5СО-216', '{"3": "5СО-216", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "2100000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-217', '5СО-217', '{"3": "5СО-217", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "21000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-218', '5СО-218', '{"3": "5СО-218", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "210000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-322', '5СО-322', '{"3": "5СО-322", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "320.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-323', '5СО-323', '{"3": "5СО-323", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "3200.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-324', '5СО-324', '{"3": "5СО-324", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "32000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-325', '5СО-325', '{"3": "5СО-325", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "320000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-326', '5СО-326', '{"3": "5СО-326", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "3200000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-327', '5СО-327', '{"3": "5СО-327", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "32000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-532', '5СО-532', '{"3": "5СО-532", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "530.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-533', '5СО-533', '{"3": "5СО-533", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "5300.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-534', '5СО-534', '{"3": "5СО-534", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "53000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-535', '5СО-535', '{"3": "5СО-535", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "530000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-536', '5СО-536', '{"3": "5СО-536", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "5300000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-537', '5СО-537', '{"3": "5СО-537", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "53000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-801', '5СО-801', '{"3": "5СО-801", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "80.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-802', '5СО-802', '{"3": "5СО-802", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "800.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-803', '5СО-803', '{"3": "5СО-803", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "8000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-804', '5СО-804', '{"3": "5СО-804", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "80000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-805', '5СО-805', '{"3": "5СО-805", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "800000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-806', '5СО-806', '{"3": "5СО-806", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "8000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-807', '5СО-807', '{"3": "5СО-807", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "80000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5Т4', '5Т4', '{"3": "5Т4", "4": "таллий-204", "5": "2", "6": "100", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Входит в набор 5Т4.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5У4', '5У4', '{"3": "5У4", "4": "уран-234", "5": "3", "6": "143(113); 1,5; пл. раб.пов., см2: 100", "7": "40.0", "8": "630.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5У4-102', '5У4-102', '{"3": "5У4-102", "4": "уран-234", "5": "3", "6": "", "7": "100.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5У4-162', '5У4-162', '{"3": "5У4-162", "4": "уран-234", "5": "3", "6": "", "7": "160.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5У4-252', '5У4-252', '{"3": "5У4-252", "4": "уран-234", "5": "3", "6": "", "7": "250.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5У4-401', '5У4-401', '{"3": "5У4-401", "4": "уран-234", "5": "3", "6": "", "7": "40.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5У4-402', '5У4-402', '{"3": "5У4-402", "4": "уран-234", "5": "3", "6": "", "7": "400.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5У4-631', '5У4-631', '{"3": "5У4-631", "4": "уран-234", "5": "3", "6": "", "7": "63.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5У4-632', '5У4-632', '{"3": "5У4-632", "4": "уран-234", "5": "3", "6": "", "7": "630.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5У8', '5У8', '{"3": "5У8", "4": "уран-238", "5": "3", "6": "143(113); 1,5; пл. раб.пов., см2: 100", "7": "40.0", "8": "400.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5У8-102', '5У8-102', '{"3": "5У8-102", "4": "уран-238", "5": "3", "6": "", "7": "100.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5У8-162', '5У8-162', '{"3": "5У8-162", "4": "уран-238", "5": "3", "6": "", "7": "160.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5У8-252', '5У8-252', '{"3": "5У8-252", "4": "уран-238", "5": "3", "6": "", "7": "250.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5У8-401', '5У8-401', '{"3": "5У8-401", "4": "уран-238", "5": "3", "6": "", "7": "40.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5У8-402', '5У8-402', '{"3": "5У8-402", "4": "уран-238", "5": "3", "6": "", "7": "400.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5У8-631', '5У8-631', '{"3": "5У8-631", "4": "уран-238", "5": "3", "6": "", "7": "63.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6а', '6а', '{"3": "6а", "4": "плутоний-239", "5": "3", "6": "", "7": "", "8": "49500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6арп-9', '6арп-9', '{"3": "6арп-9", "4": "плутоний-239", "5": "3", "6": "", "7": "", "8": "1330000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6брс-0', '6брс-0', '{"3": "6брс-0", "4": "стронций-90", "5": "2", "6": "", "7": "", "8": "8330000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9', '6П9', '{"3": "6П9", "4": "плутоний-239", "5": "3", "6": "185(155), 135(105)", "7": "63.0", "8": "16000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-102', '6П9-102', '{"3": "6П9-102", "4": "плутоний-239", "5": "3", "6": "", "7": "100.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-103', '6П9-103', '{"3": "6П9-103", "4": "плутоний-239", "5": "3", "6": "", "7": "1000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-104', '6П9-104', '{"3": "6П9-104", "4": "плутоний-239", "5": "3", "6": "", "7": "10000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-105', '6П9-105', '{"3": "6П9-105", "4": "плутоний-239", "5": "3", "6": "", "7": "100000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-106', '6П9-106', '{"3": "6П9-106", "4": "плутоний-239", "5": "3", "6": "", "7": "1000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-107', '6П9-107', '{"3": "6П9-107", "4": "плутоний-239", "5": "3", "6": "", "7": "10000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-162', '6П9-162', '{"3": "6П9-162", "4": "плутоний-239", "5": "3", "6": "", "7": "160.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-163', '6П9-163', '{"3": "6П9-163", "4": "плутоний-239", "5": "3", "6": "", "7": "1600.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-164', '6П9-164', '{"3": "6П9-164", "4": "плутоний-239", "5": "3", "6": "", "7": "16000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-165', '6П9-165', '{"3": "6П9-165", "4": "плутоний-239", "5": "3", "6": "", "7": "160000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-166', '6П9-166', '{"3": "6П9-166", "4": "плутоний-239", "5": "3", "6": "", "7": "1600000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-167', '6П9-167', '{"3": "6П9-167", "4": "плутоний-239", "5": "3", "6": "", "7": "16000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-252', '6П9-252', '{"3": "6П9-252", "4": "плутоний-239", "5": "3", "6": "", "7": "250.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-253', '6П9-253', '{"3": "6П9-253", "4": "плутоний-239", "5": "3", "6": "", "7": "2500.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-254', '6П9-254', '{"3": "6П9-254", "4": "плутоний-239", "5": "3", "6": "", "7": "25000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-255', '6П9-255', '{"3": "6П9-255", "4": "плутоний-239", "5": "3", "6": "", "7": "250000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-256', '6П9-256', '{"3": "6П9-256", "4": "плутоний-239", "5": "3", "6": "", "7": "2500000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-402', '6П9-402', '{"3": "6П9-402", "4": "плутоний-239", "5": "3", "6": "", "7": "400.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-403', '6П9-403', '{"3": "6П9-403", "4": "плутоний-239", "5": "3", "6": "", "7": "4000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-404', '6П9-404', '{"3": "6П9-404", "4": "плутоний-239", "5": "3", "6": "", "7": "40000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-405', '6П9-405', '{"3": "6П9-405", "4": "плутоний-239", "5": "3", "6": "", "7": "400000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-406', '6П9-406', '{"3": "6П9-406", "4": "плутоний-239", "5": "3", "6": "", "7": "4000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-631', '6П9-631', '{"3": "6П9-631", "4": "плутоний-239", "5": "3", "6": "", "7": "63.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-632', '6П9-632', '{"3": "6П9-632", "4": "плутоний-239", "5": "3", "6": "", "7": "630.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-633', '6П9-633', '{"3": "6П9-633", "4": "плутоний-239", "5": "3", "6": "", "7": "6300.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-634', '6П9-634', '{"3": "6П9-634", "4": "плутоний-239", "5": "3", "6": "", "7": "63000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-635', '6П9-635', '{"3": "6П9-635", "4": "плутоний-239", "5": "3", "6": "", "7": "630000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-636', '6П9-636', '{"3": "6П9-636", "4": "плутоний-239", "5": "3", "6": "", "7": "6300000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО', '6СО', '{"3": "6СО", "4": "стронций-90; иттрий-90", "5": "2", "6": "185(155), 135(105)", "7": "80.0", "8": "210000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-132', '6СО-132', '{"3": "6СО-132", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "130.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-133', '6СО-133', '{"3": "6СО-133", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "1300.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-134', '6СО-134', '{"3": "6СО-134", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "13000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-135', '6СО-135', '{"3": "6СО-135", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "130000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-136', '6СО-136', '{"3": "6СО-136", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "1300000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-137', '6СО-137', '{"3": "6СО-137", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "13000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-138', '6СО-138', '{"3": "6СО-138", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "130000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-212', '6СО-212', '{"3": "6СО-212", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "210.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-213', '6СО-213', '{"3": "6СО-213", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "2100.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-214', '6СО-214', '{"3": "6СО-214", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "21000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-215', '6СО-215', '{"3": "6СО-215", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "210000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-216', '6СО-216', '{"3": "6СО-216", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "2100000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-217', '6СО-217', '{"3": "6СО-217", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "21000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-218', '6СО-218', '{"3": "6СО-218", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "210000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-322', '6СО-322', '{"3": "6СО-322", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "320.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-323', '6СО-323', '{"3": "6СО-323", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "3200.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-324', '6СО-324', '{"3": "6СО-324", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "32000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-325', '6СО-325', '{"3": "6СО-325", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "320000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-326', '6СО-326', '{"3": "6СО-326", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "3200000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-327', '6СО-327', '{"3": "6СО-327", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "32000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-532', '6СО-532', '{"3": "6СО-532", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "530.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-533', '6СО-533', '{"3": "6СО-533", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "5300.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-534', '6СО-534', '{"3": "6СО-534", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "53000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMCK 445', 'AMCK 445', '{"3": "AMCK 445", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "22000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-535', '6СО-535', '{"3": "6СО-535", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "530000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-536', '6СО-536', '{"3": "6СО-536", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "5300000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-537', '6СО-537', '{"3": "6СО-537", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "53000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-801', '6СО-801', '{"3": "6СО-801", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "80.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-802', '6СО-802', '{"3": "6СО-802", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "800.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-803', '6СО-803', '{"3": "6СО-803", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "8000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-804', '6СО-804', '{"3": "6СО-804", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "80000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-805', '6СО-805', '{"3": "6СО-805", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "800000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-806', '6СО-806', '{"3": "6СО-806", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "8000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-807', '6СО-807', '{"3": "6СО-807", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "80000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6У4', '6У4', '{"3": "6У4", "4": "уран-234", "5": "3", "6": "185(155), 135(105)", "7": "63.0", "8": "1000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6У4-102', '6У4-102', '{"3": "6У4-102", "4": "уран-234", "5": "3", "6": "", "7": "100.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6У4-103', '6У4-103', '{"3": "6У4-103", "4": "уран-234", "5": "3", "6": "", "7": "1000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6У4-162', '6У4-162', '{"3": "6У4-162", "4": "уран-234", "5": "3", "6": "", "7": "160.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6У4-252', '6У4-252', '{"3": "6У4-252", "4": "уран-234", "5": "3", "6": "", "7": "250.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6У4-402', '6У4-402', '{"3": "6У4-402", "4": "уран-234", "5": "3", "6": "", "7": "400.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6У4-631', '6У4-631', '{"3": "6У4-631", "4": "уран-234", "5": "3", "6": "", "7": "63.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6У4-632', '6У4-632', '{"3": "6У4-632", "4": "уран-234", "5": "3", "6": "", "7": "630.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6У8', '6У8', '{"3": "6У8", "4": "уран-238", "5": "3", "6": "185(155), 135(105)", "7": "40.0", "8": "630.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6У8-102', '6У8-102', '{"3": "6У8-102", "4": "уран естественный", "5": "3", "6": "", "7": "100.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6У8-162', '6У8-162', '{"3": "6У8-162", "4": "уран естественный", "5": "3", "6": "", "7": "160.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6У8-252', '6У8-252', '{"3": "6У8-252", "4": "уран естественный", "5": "3", "6": "", "7": "250.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6У8-401', '6У8-401', '{"3": "6У8-401", "4": "уран естественный", "5": "3", "6": "", "7": "40.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6У8-402', '6У8-402', '{"3": "6У8-402", "4": "уран естественный", "5": "3", "6": "", "7": "400.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6У8-631', '6У8-631', '{"3": "6У8-631", "4": "уран естественный", "5": "3", "6": "", "7": "63.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6У8-632', '6У8-632', '{"3": "6У8-632", "4": "уран естественный", "5": "3", "6": "", "7": "630.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '7а', '7а', '{"3": "7а", "4": "плутоний-239", "5": "3", "6": "", "7": "", "8": "400000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '8а', '8а', '{"3": "8а", "4": "плутоний-239", "5": "3", "6": "", "7": "", "8": "1000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AAm1.1.08', 'AAm1.1.08', '{"3": "AAm1.1.08", "4": "америций-241", "5": "3", "6": "", "7": "", "8": "500.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AAm1.1.S1', 'AAm1.1.S1', '{"3": "AAm1.1.S1", "4": "америций-241", "5": "3", "6": "25x0,2. Акт. часть d : 9", "7": "", "8": "500.0", "9": "0", "10": "НПО РИ", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AAm1.1.S2', 'AAm1.1.S2', '{"3": "AAm1.1.S2", "4": "америций-241", "5": "3", "6": "25x0,2. Акт. часть d : 9", "7": "", "8": "1000.0", "9": "0", "10": "НПО РИ", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AAm1.1.S3', 'AAm1.1.S3', '{"3": "AAm1.1.S3", "4": "америций-241", "5": "3", "6": "21x0,2. Акт. часть d : 9", "7": "", "8": "3000.0", "9": "0", "10": "НПО РИ", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AC339', 'AC339', '{"3": "AC339", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "129000000000000.0", "9": "0", "10": "", "11": "1", "12": "ЛАЭС-изг", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM13', 'AM13', '{"3": "AM13", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM130150001M', 'AM130150001M', '{"3": "AM130150001M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM130150005M', 'AM130150005M', '{"3": "AM130150005M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM130150010M', 'AM130150010M', '{"3": "AM130150010M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM130150025M', 'AM130150025M', '{"3": "AM130150025M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM130150050M', 'AM130150050M', '{"3": "AM130150050M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM130150100U', 'AM130150100U', '{"3": "AM130150100U", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM130150500U', 'AM130150500U', '{"3": "AM130150500U", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132010001M', 'AM132010001M', '{"3": "AM132010001M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132010005M', 'AM132010005M', '{"3": "AM132010005M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132010010M', 'AM132010010M', '{"3": "AM132010010M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132010025M', 'AM132010025M', '{"3": "AM132010025M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132010050M', 'AM132010050M', '{"3": "AM132010050M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132010100U', 'AM132010100U', '{"3": "AM132010100U", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132010500U', 'AM132010500U', '{"3": "AM132010500U", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132040001M', 'AM132040001M', '{"3": "AM132040001M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132040005M', 'AM132040005M', '{"3": "AM132040005M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132040010M', 'AM132040010M', '{"3": "AM132040010M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132040025M', 'AM132040025M', '{"3": "AM132040025M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132040050M', 'AM132040050M', '{"3": "AM132040050M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132040100U', 'AM132040100U', '{"3": "AM132040100U", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132040500U', 'AM132040500U', '{"3": "AM132040500U", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132150001M', 'AM132150001M', '{"3": "AM132150001M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132150005M', 'AM132150005M', '{"3": "AM132150005M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132150010M', 'AM132150010M', '{"3": "AM132150010M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132150025M', 'AM132150025M', '{"3": "AM132150025M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132150050M', 'AM132150050M', '{"3": "AM132150050M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132150100U', 'AM132150100U', '{"3": "AM132150100U", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132150500U', 'AM132150500U', '{"3": "AM132150500U", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132240001M', 'AM132240001M', '{"3": "AM132240001M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132240005M', 'AM132240005M', '{"3": "AM132240005M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132240010M', 'AM132240010M', '{"3": "AM132240010M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132240025M', 'AM132240025M', '{"3": "AM132240025M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132240050M', 'AM132240050M', '{"3": "AM132240050M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132240100U', 'AM132240100U', '{"3": "AM132240100U", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132240500U', 'AM132240500U', '{"3": "AM132240500U", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132330001M', 'AM132330001M', '{"3": "AM132330001M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132330005M', 'AM132330005M', '{"3": "AM132330005M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132330010M', 'AM132330010M', '{"3": "AM132330010M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132330030M', 'AM132330030M', '{"3": "AM132330030M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132330050M', 'AM132330050M', '{"3": "AM132330050M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132330100M', 'AM132330100M', '{"3": "AM132330100M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132330100U', 'AM132330100U', '{"3": "AM132330100U", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM132330500U', 'AM132330500U', '{"3": "AM132330500U", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G', 'AM1G', '{"3": "AM1G", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Am1№09', 'Am1№09', '{"3": "Am1№09", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G100001M', 'AM1G100001M', '{"3": "AM1G100001M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G100002M', 'AM1G100002M', '{"3": "AM1G100002M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "74000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G100005M', 'AM1G100005M', '{"3": "AM1G100005M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G100010M', 'AM1G100010M', '{"3": "AM1G100010M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G100015M', 'AM1G100015M', '{"3": "AM1G100015M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "555000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G100020M', 'AM1G100020M', '{"3": "AM1G100020M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G100030M', 'AM1G100030M', '{"3": "AM1G100030M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G110001M', 'AM1G110001M', '{"3": "AM1G110001M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G110005M', 'AM1G110005M', '{"3": "AM1G110005M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G110010M', 'AM1G110010M', '{"3": "AM1G110010M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G110020M', 'AM1G110020M', '{"3": "AM1G110020M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G110050M', 'AM1G110050M', '{"3": "AM1G110050M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G110100M', 'AM1G110100M', '{"3": "AM1G110100M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G110500U', 'AM1G110500U', '{"3": "AM1G110500U", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G220050M', 'AM1G220050M', '{"3": "AM1G220050M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G220100M', 'AM1G220100M', '{"3": "AM1G220100M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G220200M', 'AM1G220200M', '{"3": "AM1G220200M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "7400000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G220300M', 'AM1G220300M', '{"3": "AM1G220300M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "11100000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G330100M', 'AM1G330100M', '{"3": "AM1G330100M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G330200M', 'AM1G330200M', '{"3": "AM1G330200M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "7400000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G330300M', 'AM1G330300M', '{"3": "AM1G330300M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "11100000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G330500M', 'AM1G330500M', '{"3": "AM1G330500M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G440250M', 'AM1G440250M', '{"3": "AM1G440250M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "9250000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G440500M', 'AM1G440500M', '{"3": "AM1G440500M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G440750M', 'AM1G440750M', '{"3": "AM1G440750M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "27800000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G441000M', 'AM1G441000M', '{"3": "AM1G441000M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G551000M', 'AM1G551000M', '{"3": "AM1G551000M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G552000M', 'AM1G552000M', '{"3": "AM1G552000M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "74000000000.0", "9": "0", "10": "", "11": "3", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1G553000M', 'AM1G553000M', '{"3": "AM1G553000M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "111000000000.0", "9": "0", "10": "", "11": "3", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1N', 'AM1N', '{"3": "AM1N", "4": "америций-241; бериллий", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1N020001M', 'AM1N020001M', '{"3": "AM1N020001M", "4": "америций-241; бериллий", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1N020010M', 'AM1N020010M', '{"3": "AM1N020010M", "4": "америций-241; бериллий", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1N020040M', 'AM1N020040M', '{"3": "AM1N020040M", "4": "америций-241; бериллий", "5": "1", "6": "", "7": "", "8": "1480000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1N020100M', 'AM1N020100M', '{"3": "AM1N020100M", "4": "америций-241; бериллий", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1N030001C', 'AM1N030001C', '{"3": "AM1N030001C", "4": "америций-241; бериллий", "5": "1", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1N030003C', 'AM1N030003C', '{"3": "AM1N030003C", "4": "америций-241; бериллий", "5": "1", "6": "", "7": "", "8": "111000000000.0", "9": "0", "10": "", "11": "3", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1N030008C', 'AM1N030008C', '{"3": "AM1N030008C", "4": "америций-241; бериллий", "5": "1", "6": "", "7": "", "8": "296000000000.0", "9": "0", "10": "", "11": "3", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1N030016C', 'AM1N030016C', '{"3": "AM1N030016C", "4": "америций-241; бериллий", "5": "1", "6": "", "7": "", "8": "592000000000.0", "9": "0", "10": "", "11": "3", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1N030018C', 'AM1N030018C', '{"3": "AM1N030018C", "4": "америций-241; бериллий", "5": "1", "6": "", "7": "", "8": "666000000000.0", "9": "0", "10": "", "11": "2", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1N090001M', 'AM1N090001M', '{"3": "AM1N090001M", "4": "америций-241; бериллий", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1N090010M', 'AM1N090010M', '{"3": "AM1N090010M", "4": "америций-241; бериллий", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1N090100M', 'AM1N090100M', '{"3": "AM1N090100M", "4": "америций-241; бериллий", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1N090350M', 'AM1N090350M', '{"3": "AM1N090350M", "4": "америций-241; бериллий", "5": "1", "6": "", "7": "", "8": "13000000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1N09-10002M', 'AM1N09-10002M', '{"3": "AM1N09-10002M", "4": "америций-241; бериллий", "5": "1", "6": "", "7": "", "8": "74000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1N09-10005M', 'AM1N09-10005M', '{"3": "AM1N09-10005M", "4": "америций-241; бериллий", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1N09-10010M', 'AM1N09-10010M', '{"3": "AM1N09-10010M", "4": "америций-241; бериллий", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1N09-10100M', 'AM1N09-10100M', '{"3": "AM1N09-10100M", "4": "америций-241; бериллий", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1N09-10200M', 'AM1N09-10200M', '{"3": "AM1N09-10200M", "4": "америций-241; бериллий", "5": "1", "6": "", "7": "", "8": "7400000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1P', 'AM1P', '{"3": "AM1P", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1P080001M', 'AM1P080001M', '{"3": "AM1P080001M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1P080025M', 'AM1P080025M', '{"3": "AM1P080025M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1P080050M', 'AM1P080050M', '{"3": "AM1P080050M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1P080100M', 'AM1P080100M', '{"3": "AM1P080100M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1P080200M', 'AM1P080200M', '{"3": "AM1P080200M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "7400000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1P0810001M', 'AM1P0810001M', '{"3": "AM1P0810001M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1P0810025M', 'AM1P0810025M', '{"3": "AM1P0810025M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1P0810050M', 'AM1P0810050M', '{"3": "AM1P0810050M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1P0810100M', 'AM1P0810100M', '{"3": "AM1P0810100M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1P0810150M', 'AM1P0810150M', '{"3": "AM1P0810150M", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "5550000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMC', 'AMC', '{"3": "AMC", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMC.13044', 'AMC.13044', '{"3": "AMC.13044", "4": "америций-241", "5": "1", "6": "", "7": "1100000000.0", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMC.13145', 'AMC.13145', '{"3": "AMC.13145", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMC.13146', 'AMC.13146', '{"3": "AMC.13146", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMC.13446', 'AMC.13446', '{"3": "AMC.13446", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMC.16', 'AMC.16', '{"3": "AMC.16", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMC.17', 'AMC.17', '{"3": "AMC.17", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "11100000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMC.18', 'AMC.18', '{"3": "AMC.18", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMC.19', 'AMC.19', '{"3": "AMC.19", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMC.21', 'AMC.21', '{"3": "AMC.21", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "74000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMC.24', 'AMC.24', '{"3": "AMC.24", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "518000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMC.25', 'AMC.25', '{"3": "AMC.25", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "1670000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMC.26', 'AMC.26', '{"3": "AMC.26", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "7400000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMC.30', 'AMC.30', '{"3": "AMC.30", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "111000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMC.36', 'AMC.36', '{"3": "AMC.36", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMC.50', 'AMC.50', '{"3": "AMC.50", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "185000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMC.6178', 'AMC.6178', '{"3": "AMC.6178", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "6140000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMC.62', 'AMC.62', '{"3": "AMC.62", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMC.63', 'AMC.63', '{"3": "AMC.63", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "111000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMC.64', 'AMC.64', '{"3": "AMC.64", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMC.65', 'AMC.65', '{"3": "AMC.65", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMC.66', 'AMC.66', '{"3": "AMC.66", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMC.D1 model', 'AMC.D1 model', '{"3": "AMC.D1 model", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "11100000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMC.K5490', 'AMC.K5490', '{"3": "AMC.K5490", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMC.P6 model', 'AMC.P6 model', '{"3": "AMC.P6 model", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "5550000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMCK', 'AMCK', '{"3": "AMCK", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMCK 316', 'AMCK 316', '{"3": "AMCK 316", "4": "америций-241", "5": "1", "6": "", "7": "37000000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMCK 336', 'AMCK 336', '{"3": "AMCK 336", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "92500000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMCK 3717', 'AMCK 3717', '{"3": "AMCK 3717", "4": "америций-241", "5": "1", "6": "", "7": "33300000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMCK 4269', 'AMCK 4269', '{"3": "AMCK 4269", "4": "америций-241", "5": "1", "6": "", "7": "11100000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMCK 5490', 'AMCK 5490', '{"3": "AMCK 5490", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "3600000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMCK 6178', 'AMCK 6178', '{"3": "AMCK 6178", "4": "америций-241", "5": "1", "6": "", "7": "6030000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMCK 6576', 'AMCK 6576', '{"3": "AMCK 6576", "4": "америций-241", "5": "1", "6": "", "7": "3330000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMCK 6693', 'AMCK 6693', '{"3": "AMCK 6693", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "4770000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMCK 7703', 'AMCK 7703', '{"3": "AMCK 7703", "4": "америций-241", "5": "1", "6": "", "7": "999000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMCK 8122', 'AMCK 8122', '{"3": "AMCK 8122", "4": "америций-241", "5": "1", "6": "", "7": "555000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMCK 8248', 'AMCK 8248', '{"3": "AMCK 8248", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMCK 8313', 'AMCK 8313', '{"3": "AMCK 8313", "4": "америций-241", "5": "1", "6": "", "7": "5550000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMCQ', 'AMCQ', '{"3": "AMCQ", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMCQ 4064', 'AMCQ 4064', '{"3": "AMCQ 4064", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "44400000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMCQ 7540', 'AMCQ 7540', '{"3": "AMCQ 7540", "4": "америций-241", "5": "1", "6": "", "7": "2960000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMMK', 'AMMK', '{"3": "AMMK", "4": "америций-241", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMMK1235', 'AMMK1235', '{"3": "AMMK1235", "4": "америций-241", "5": "3", "6": "", "7": "", "8": "29600.0", "9": "0", "10": "", "11": "5", "12": "Дымоизвещатели", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMMK3457', 'AMMK3457', '{"3": "AMMK3457", "4": "америций-241", "5": "3", "6": "", "7": "", "8": "29600.0", "9": "0", "10": "", "11": "5", "12": "Дымоизвещатели", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMMK5588', 'AMMK5588', '{"3": "AMMK5588", "4": "америций-241", "5": "3", "6": "", "7": "", "8": "18500.0", "9": "0", "10": "", "11": "5", "12": "Дымоизвещатели", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMMK5597', 'AMMK5597', '{"3": "AMMK5597", "4": "америций-241", "5": "3", "6": "", "7": "", "8": "29600.0", "9": "0", "10": "", "11": "5", "12": "Дымоизвещатели", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMMK6045', 'AMMK6045', '{"3": "AMMK6045", "4": "америций-241", "5": "3", "6": "", "7": "", "8": "29600.0", "9": "0", "10": "", "11": "5", "12": "Дымоизвещатели", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMMK7169', 'AMMK7169', '{"3": "AMMK7169", "4": "америций-241", "5": "3", "6": "", "7": "", "8": "18500.0", "9": "0", "10": "", "11": "5", "12": "Дымоизвещатели", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMMK7540', 'AMMK7540', '{"3": "AMMK7540", "4": "америций-241", "5": "3", "6": "", "7": "", "8": "18500.0", "9": "0", "10": "", "11": "5", "12": "Дымоизвещатели", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMMK7620', 'AMMK7620', '{"3": "AMMK7620", "4": "америций-241", "5": "3", "6": "", "7": "", "8": "18500.0", "9": "0", "10": "", "11": "5", "12": "Дымоизвещатели", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMMK7648', 'AMMK7648', '{"3": "AMMK7648", "4": "америций-241", "5": "3", "6": "", "7": "", "8": "18500.0", "9": "0", "10": "", "11": "5", "12": "Дымоизвещатели", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMMK7649', 'AMMK7649', '{"3": "AMMK7649", "4": "америций-241", "5": "3", "6": "", "7": "", "8": "18500.0", "9": "0", "10": "", "11": "5", "12": "Дымоизвещатели", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMMK7650', 'AMMK7650', '{"3": "AMMK7650", "4": "америций-241", "5": "3", "6": "", "7": "", "8": "29600.0", "9": "0", "10": "", "11": "5", "12": "Дымоизвещатели", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN', 'AMN', '{"3": "AMN", "4": "америций-241", "5": "4", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.11', 'AMN.11', '{"3": "AMN.11", "4": "америций-241", "5": "4", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.13', 'AMN.13', '{"3": "AMN.13", "4": "америций-241", "5": "4", "6": "", "7": "", "8": "111000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.15', 'AMN.15', '{"3": "AMN.15", "4": "америций-241", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.16', 'AMN.16', '{"3": "AMN.16", "4": "америций-241", "5": "4", "6": "", "7": "", "8": "1110000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.168', 'AMN.168', '{"3": "AMN.168", "4": "америций-241", "5": "4", "6": "", "7": "", "8": "1110000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.17', 'AMN.17', '{"3": "AMN.17", "4": "америций-241", "5": "4", "6": "", "7": "", "8": "3700000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.18', 'AMN.18', '{"3": "AMN.18", "4": "америций-241", "5": "4", "6": "", "7": "", "8": "11100000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.19', 'AMN.19', '{"3": "AMN.19", "4": "америций-241", "5": "4", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.2071', 'AMN.2071', '{"3": "AMN.2071", "4": "америций-241", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.2072', 'AMN.2072', '{"3": "AMN.2072", "4": "америций-241", "5": "4", "6": "", "7": "", "8": "740000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.22', 'AMN.22', '{"3": "AMN.22", "4": "америций-241", "5": "4", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.23', 'AMN.23', '{"3": "AMN.23", "4": "америций-241", "5": "4", "6": "", "7": "", "8": "111000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.24', 'AMN.24', '{"3": "AMN.24", "4": "америций-241", "5": "4", "6": "", "7": "", "8": "185000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.25', 'AMN.25', '{"3": "AMN.25", "4": "америций-241", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.2643', 'AMN.2643', '{"3": "AMN.2643", "4": "америций-241", "5": "4", "6": "", "7": "", "8": "111000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.2645', 'AMN.2645', '{"3": "AMN.2645", "4": "америций-241", "5": "4", "6": "", "7": "", "8": "185000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.2653', 'AMN.2653', '{"3": "AMN.2653", "4": "америций-241", "5": "4", "6": "", "7": "", "8": "111000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.2655', 'AMN.2655', '{"3": "AMN.2655", "4": "америций-241", "5": "4", "6": "", "7": "", "8": "185000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.2665', 'AMN.2665', '{"3": "AMN.2665", "4": "америций-241", "5": "4", "6": "", "7": "", "8": "185000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.2673', 'AMN.2673', '{"3": "AMN.2673", "4": "америций-241", "5": "4", "6": "", "7": "", "8": "111000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.2675', 'AMN.2675', '{"3": "AMN.2675", "4": "америций-241", "5": "4", "6": "", "7": "", "8": "185000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.CY', 'AMN.CY', '{"3": "AMN.CY", "4": "америций-241", "5": "4", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNK', 'AMNK', '{"3": "AMNK", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNK 7062', 'AMNK 7062', '{"3": "AMNK 7062", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNK 7190', 'AMNK 7190', '{"3": "AMNK 7190", "4": "америций-241", "5": "1", "6": "", "7": "111000000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNK 7431', 'AMNK 7431', '{"3": "AMNK 7431", "4": "америций-241", "5": "1", "6": "", "7": "14800000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNK 7443', 'AMNK 7443', '{"3": "AMNK 7443", "4": "америций-241", "5": "1", "6": "", "7": "17800000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNK 8253', 'AMNK 8253', '{"3": "AMNK 8253", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "777000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNK 8254', 'AMNK 8254', '{"3": "AMNK 8254", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "15200000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNQ', 'AMNQ', '{"3": "AMNQ", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNQ 4578', 'AMNQ 4578', '{"3": "AMNQ 4578", "4": "америций-241", "5": "1", "6": "", "7": "1670000000.0", "8": "2040000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNQ 6031', 'AMNQ 6031', '{"3": "AMNQ 6031", "4": "америций-241", "5": "1", "6": "", "7": "0.0", "8": "37000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNQ 9337', 'AMNQ 9337', '{"3": "AMNQ 9337", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "4070000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNQ 9927', 'AMNQ 9927', '{"3": "AMNQ 9927", "4": "америций-241", "5": "1", "6": "", "7": "1550000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNV', 'AMNV', '{"3": "AMNV", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNV 340', 'AMNV 340', '{"3": "AMNV 340", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "4070000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNV 997', 'AMNV 997', '{"3": "AMNV 997", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "4070000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'APu8.07.1', 'APu8.07.1', '{"3": "APu8.07.1", "4": "плутоний-238", "5": "3", "6": "", "7": "", "8": "999000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'APu8.2.2', 'APu8.2.2', '{"3": "APu8.2.2", "4": "плутоний-238", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'APu8.41', 'APu8.41', '{"3": "APu8.41", "4": "плутоний-238", "5": "3", "6": "", "7": "", "8": "5550000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA3', 'BA3', '{"3": "BA3", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA330000001M', 'BA330000001M', '{"3": "BA330000001M", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA330000005M', 'BA330000005M', '{"3": "BA330000005M", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA330000010M', 'BA330000010M', '{"3": "BA330000010M", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA330000100U', 'BA330000100U', '{"3": "BA330000100U", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA330110001M', 'BA330110001M', '{"3": "BA330110001M", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA330110005M', 'BA330110005M', '{"3": "BA330110005M", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA330110010M', 'BA330110010M', '{"3": "BA330110010M", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA330110100U', 'BA330110100U', '{"3": "BA330110100U", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA330150001M', 'BA330150001M', '{"3": "BA330150001M", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA330150005M', 'BA330150005M', '{"3": "BA330150005M", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA330150010M', 'BA330150010M', '{"3": "BA330150010M", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA330150100U', 'BA330150100U', '{"3": "BA330150100U", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA330240001M', 'BA330240001M', '{"3": "BA330240001M", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA330240005M', 'BA330240005M', '{"3": "BA330240005M", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA330240010M', 'BA330240010M', '{"3": "BA330240010M", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA330240100U', 'BA330240100U', '{"3": "BA330240100U", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA332240001M', 'BA332240001M', '{"3": "BA332240001M", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA332240005M', 'BA332240005M', '{"3": "BA332240005M", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA332240010M', 'BA332240010M', '{"3": "BA332240010M", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA332240100U', 'BA332240100U', '{"3": "BA332240100U", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA334020001M', 'BA334020001M', '{"3": "BA334020001M", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA334020005M', 'BA334020005M', '{"3": "BA334020005M", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA334020010M', 'BA334020010M', '{"3": "BA334020010M", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA334020100U', 'BA334020100U', '{"3": "BA334020100U", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA339020010U', 'BA339020010U', '{"3": "BA339020010U", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "370000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BA339160010U', 'BA339160010U', '{"3": "BA339160010U", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "370000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BH3', 'BH3', '{"3": "BH3", "4": "тритий", "5": "2", "6": "14x10 или 30x2 или 30x10", "7": "370000000.0", "8": "18500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BH3.1', 'BH3.1', '{"3": "BH3.1", "4": "тритий", "5": "2", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BH3.211', 'BH3.211', '{"3": "BH3.211", "4": "тритий", "5": "2", "6": "", "7": "", "8": "5000000.0", "9": "0", "10": "", "11": "0", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BH3.211', 'BH3.211', '{"3": "BH3.211", "4": "тритий", "5": "2", "6": "", "7": "135000000.0", "8": "0.0", "9": "0", "10": "Ритверц", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BH3.212', 'BH3.212', '{"3": "BH3.212", "4": "тритий", "5": "2", "6": "", "7": "6750000000.0", "8": "0.0", "9": "0", "10": "Ритверц", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BH3.223', 'BH3.223', '{"3": "BH3.223", "4": "тритий", "5": "2", "6": "", "7": "", "8": "570000000000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BH3.224', 'BH3.224', '{"3": "BH3.224", "4": "тритий", "5": "2", "6": "", "7": "", "8": "22400000000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BH3.233', 'BH3.233', '{"3": "BH3.233", "4": "тритий", "5": "2", "6": "", "7": "", "8": "1430000000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BH3.234', 'BH3.234', '{"3": "BH3.234", "4": "тритий", "5": "2", "6": "", "7": "", "8": "2240000000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BH3.324', 'BH3.324', '{"3": "BH3.324", "4": "тритий", "5": "2", "6": "", "7": "", "8": "896000000000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BH3.3x', 'BH3.3x', '{"3": "BH3.3x", "4": "тритий", "5": "2", "6": "", "7": "", "8": "22200000000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BH3.411', 'BH3.411', '{"3": "BH3.411", "4": "тритий", "5": "2", "6": "", "7": "", "8": "1420000000.0", "9": "0", "10": "", "11": "5", "12": "Плоскостный.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BH3.412', 'BH3.412', '{"3": "BH3.412", "4": "тритий", "5": "2", "6": "", "7": "60500000000.0", "8": "0.0", "9": "0", "10": "Ритверц", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BH3.x', 'BH3.x', '{"3": "BH3.x", "4": "тритий", "5": "2", "6": "", "7": "", "8": "22200000000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BH3.X.1', 'BH3.X.1', '{"3": "BH3.X.1", "4": "тритий", "5": "2", "6": "", "7": "", "8": "74000000000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BIP-50', 'BIP-50', '{"3": "BIP-50", "4": "прометий-147", "5": "2", "6": "", "7": "", "8": "2700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BIS-50', 'BIS-50', '{"3": "BIS-50", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "6200000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5', 'BKr5', '{"3": "BKr5", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.111', 'BKr5.111', '{"3": "BKr5.111", "4": "криптон-85", "5": "2", "6": "4x8", "7": "", "8": "300000000.0", "9": "0", "10": "Ритверц", "11": "5", "12": "Точечный", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.201', 'BKr5.201', '{"3": "BKr5.201", "4": "криптон-85", "5": "2", "6": "39x12", "7": "", "8": "7400000000.0", "9": "0", "10": "", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.212', 'BKr5.212', '{"3": "BKr5.212", "4": "криптон-85", "5": "2", "6": "10x15", "7": "", "8": "2000000000.0", "9": "0", "10": "", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.213', 'BKr5.213', '{"3": "BKr5.213", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.214', 'BKr5.214', '{"3": "BKr5.214", "4": "криптон-85", "5": "2", "6": "10x15", "7": "", "8": "10000000000.0", "9": "0", "10": "", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.222', 'BKr5.222', '{"3": "BKr5.222", "4": "криптон-85", "5": "2", "6": "20x10", "7": "", "8": "2000000000.0", "9": "0", "10": "Ритверц", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.223', 'BKr5.223', '{"3": "BKr5.223", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "3000000000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.223', 'BKr5.223', '{"3": "BKr5.223", "4": "криптон-85", "5": "2", "6": "", "7": "5000000000.0", "8": "0.0", "9": "0", "10": "Ритверц", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.226', 'BKr5.226', '{"3": "BKr5.226", "4": "криптон-85", "5": "2", "6": "", "7": "10000000000.0", "8": "0.0", "9": "0", "10": "Ритверц", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.232', 'BKr5.232', '{"3": "BKr5.232", "4": "криптон-85", "5": "2", "6": "", "7": "2000000000.0", "8": "0.0", "9": "0", "10": "Ритверц", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.233', 'BKr5.233', '{"3": "BKr5.233", "4": "криптон-85", "5": "2", "6": "20x15", "7": "", "8": "5000000000.0", "9": "0", "10": "", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.243', 'BKr5.243', '{"3": "BKr5.243", "4": "криптон-85", "5": "2", "6": "", "7": "5000000000.0", "8": "0.0", "9": "0", "10": "Ритверц", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.244', 'BKr5.244', '{"3": "BKr5.244", "4": "криптон-85", "5": "2", "6": "20x20", "7": "", "8": "10000000000.0", "9": "0", "10": "", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.256', 'BKr5.256', '{"3": "BKr5.256", "4": "криптон-85", "5": "2", "6": "", "7": "10000000000.0", "8": "0.0", "9": "0", "10": "Ритверц", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.262', 'BKr5.262', '{"3": "BKr5.262", "4": "криптон-85", "5": "2", "6": "30x10", "7": "", "8": "2000000000.0", "9": "0", "10": "Ритверц", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.263', 'BKr5.263', '{"3": "BKr5.263", "4": "криптон-85", "5": "2", "6": "30x10", "7": "", "8": "5000000000.0", "9": "0", "10": "Ритверц", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.267', 'BKr5.267', '{"3": "BKr5.267", "4": "криптон-85", "5": "2", "6": "", "7": "18500000000.0", "8": "0.0", "9": "0", "10": "Ритверц", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.274', 'BKr5.274', '{"3": "BKr5.274", "4": "криптон-85", "5": "2", "6": "30x15", "7": "", "8": "10000000000.0", "9": "0", "10": "", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.277', 'BKr5.277', '{"3": "BKr5.277", "4": "криптон-85", "5": "2", "6": "", "7": "18500000000.0", "8": "0.0", "9": "0", "10": "Ритверц", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.285', 'BKr5.285', '{"3": "BKr5.285", "4": "криптон-85", "5": "2", "6": "30x20", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.285', 'BKr5.285', '{"3": "BKr5.285", "4": "криптон-85", "5": "2", "6": "", "7": "7400000000.0", "8": "0.0", "9": "0", "10": "Ритверц", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.291', 'BKr5.291', '{"3": "BKr5.291", "4": "криптон-85", "5": "2", "6": "62x7", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.294', 'BKr5.294', '{"3": "BKr5.294", "4": "криптон-85", "5": "2", "6": "", "7": "3700000000.0", "8": "0.0", "9": "0", "10": "Ритверц", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.P02', 'BKr5.P02', '{"3": "BKr5.P02", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "100000000000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р02.204', 'BKr5.Р02.204', '{"3": "BKr5.Р02.204", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "11100000000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р02.212', 'BKr5.Р02.212', '{"3": "BKr5.Р02.212", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "20000000000.0", "9": "0", "10": "", "11": "5", "12": "Корпус источника из титана или нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р02.213', 'BKr5.Р02.213', '{"3": "BKr5.Р02.213", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "50000000000.0", "9": "0", "10": "", "11": "5", "12": "Корпус источника из титана или нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р02.214', 'BKr5.Р02.214', '{"3": "BKr5.Р02.214", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "100000000000.0", "9": "0", "10": "", "11": "5", "12": "Корпус источника из титана или нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р02.215', 'BKr5.Р02.215', '{"3": "BKr5.Р02.215", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "185000000000.0", "9": "0", "10": "", "11": "5", "12": "Корпус источника из титана или нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р02.215', 'BKr5.Р02.215', '{"3": "BKr5.Р02.215", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "14800000000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р02.222', 'BKr5.Р02.222', '{"3": "BKr5.Р02.222", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "20000000000.0", "9": "0", "10": "", "11": "5", "12": "Корпус источника из титана или нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р02.223', 'BKr5.Р02.223', '{"3": "BKr5.Р02.223", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "50000000000.0", "9": "0", "10": "", "11": "5", "12": "Корпус источника из титана или нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р02.224', 'BKr5.Р02.224', '{"3": "BKr5.Р02.224", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "100000000000.0", "9": "0", "10": "", "11": "5", "12": "Корпус источника из титана или нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р02.225', 'BKr5.Р02.225', '{"3": "BKr5.Р02.225", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "185000000000.0", "9": "0", "10": "", "11": "5", "12": "Корпус источника из титана или нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р02.233', 'BKr5.Р02.233', '{"3": "BKr5.Р02.233", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р02.234', 'BKr5.Р02.234', '{"3": "BKr5.Р02.234", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "100000000000.0", "9": "0", "10": "", "11": "5", "12": "Корпус источника из титана или нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr0.7.D3', 'BSr0.7.D3', '{"3": "BSr0.7.D3", "4": "стронций-90", "5": "2", "6": "", "7": "", "8": "5000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р02.235', 'BKr5.Р02.235', '{"3": "BKr5.Р02.235", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "185000000000.0", "9": "0", "10": "", "11": "5", "12": "Корпус источника из титана или нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р02.236', 'BKr5.Р02.236', '{"3": "BKr5.Р02.236", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "4", "12": "Корпус источника из титана или нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р02.244', 'BKr5.Р02.244', '{"3": "BKr5.Р02.244", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "100000000000.0", "9": "0", "10": "", "11": "5", "12": "Корпус источника из титана или нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р02.262', 'BKr5.Р02.262', '{"3": "BKr5.Р02.262", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "20000000000.0", "9": "0", "10": "", "11": "5", "12": "Корпус источника из титана или нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р02.263', 'BKr5.Р02.263', '{"3": "BKr5.Р02.263", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "50000000000.0", "9": "0", "10": "", "11": "5", "12": "Корпус источника из титана или нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р02.274', 'BKr5.Р02.274', '{"3": "BKr5.Р02.274", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "100000000000.0", "9": "0", "10": "", "11": "5", "12": "Корпус источника из титана или нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р02.284', 'BKr5.Р02.284', '{"3": "BKr5.Р02.284", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "4500000000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р02.285', 'BKr5.Р02.285', '{"3": "BKr5.Р02.285", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "185000000000.0", "9": "0", "10": "", "11": "5", "12": "Корпус источника из титана или нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р02.286', 'BKr5.Р02.286', '{"3": "BKr5.Р02.286", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "4", "12": "Корпус источника из титана или нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р02.287', 'BKr5.Р02.287', '{"3": "BKr5.Р02.287", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "555000000000.0", "9": "0", "10": "", "11": "4", "12": "Корпус источника из титана или нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р04', 'BKr5.Р04', '{"3": "BKr5.Р04", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "300000000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р04.111', 'BKr5.Р04.111', '{"3": "BKr5.Р04.111", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "3000000000.0", "9": "0", "10": "", "11": "5", "12": "Корпус источника из титана или нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р04.233', 'BKr5.Р04.233', '{"3": "BKr5.Р04.233", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "50000000000.0", "9": "0", "10": "", "11": "5", "12": "Корпус источника из титана или нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р06.201', 'BKr5.Р06.201', '{"3": "BKr5.Р06.201", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "74000000000.0", "9": "0", "10": "", "11": "5", "12": "Корпус источника из титана или нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BKr5.Р06.291', 'BKr5.Р06.291', '{"3": "BKr5.Р06.291", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "5", "12": "Корпус источника из титана или нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3', 'BNi3', '{"3": "BNi3", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.411', 'BNi3.411', '{"3": "BNi3.411", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Плоскостный.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.421', 'BNi3.421', '{"3": "BNi3.421", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "555000000.0", "9": "0", "10": "", "11": "5", "12": "Плоскостный.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.431', 'BNi3.431', '{"3": "BNi3.431", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "Плоскостный.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.441', 'BNi3.441', '{"3": "BNi3.441", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "100000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C1', 'BNi3.C1', '{"3": "BNi3.C1", "4": "никель-63", "5": "2", "6": "4,8x10", "7": "296000000.0", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Цилиндрический.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C1.1', 'BNi3.C1.1', '{"3": "BNi3.C1.1", "4": "никель-63", "5": "2", "6": "", "7": "296000000.0", "8": "0.0", "9": "0", "10": "ЗАО РИТВЕРЦ", "11": "0", "12": "из отчета Спб Изотоп", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C1.2', 'BNi3.C1.2', '{"3": "BNi3.C1.2", "4": "никель-63", "5": "2", "6": "", "7": "370000000.0", "8": "1200000000.0", "9": "0", "10": "ЗАО РИТВЕРЦ", "11": "5", "12": "из отчета Спб Изотоп", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C1.3', 'BNi3.C1.3', '{"3": "BNi3.C1.3", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C2', 'BNi3.C2', '{"3": "BNi3.C2", "4": "никель-63", "5": "2", "6": "8x10", "7": "", "8": "555000000.0", "9": "0", "10": "", "11": "5", "12": "Цилиндрический.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C2.1', 'BNi3.C2.1', '{"3": "BNi3.C2.1", "4": "никель-63", "5": "2", "6": "", "7": "555000000.0", "8": "0.0", "9": "0", "10": "ЗАО РИТВЕРЦ", "11": "0", "12": "из отчета Спб Изотоп", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C3', 'BNi3.C3', '{"3": "BNi3.C3", "4": "никель-63", "5": "2", "6": "9,6x10", "7": "100000000.0", "8": "1200000000.0", "9": "0", "10": "", "11": "5", "12": "Цилиндрический.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C3.1', 'BNi3.C3.1', '{"3": "BNi3.C3.1", "4": "никель-63", "5": "2", "6": "", "7": "100000000.0", "8": "700000000.0", "9": "0", "10": "ЗАО РИТВЕРЦ", "11": "5", "12": "из отчета Спб Изотоп", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C3.2', 'BNi3.C3.2', '{"3": "BNi3.C3.2", "4": "никель-63", "5": "2", "6": "", "7": "600000000.0", "8": "740000000.0", "9": "0", "10": "ЗАО РИТВЕРЦ", "11": "5", "12": "из отчета Спб Изотоп", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C3.3', 'BNi3.C3.3', '{"3": "BNi3.C3.3", "4": "никель-63", "5": "2", "6": "", "7": "740000000.0", "8": "0.0", "9": "0", "10": "ЗАО РИТВЕРЦ", "11": "0", "12": "из отчета Спб Изотоп", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C3.4', 'BNi3.C3.4', '{"3": "BNi3.C3.4", "4": "никель-63", "5": "2", "6": "", "7": "1200000000.0", "8": "962000000000.0", "9": "0", "10": "ЗАО РИТВЕРЦ", "11": "4", "12": "из отчета Спб Изотоп", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C3.4.R', 'BNi3.C3.4.R', '{"3": "BNi3.C3.4.R", "4": "никель-63", "5": "2", "6": "", "7": "1200000000.0", "8": "962000000000.0", "9": "0", "10": "ЗАО РИТВЕРЦ", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C3.x', 'BNi3.C3.x', '{"3": "BNi3.C3.x", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "999000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C4', 'BNi3.C4', '{"3": "BNi3.C4", "4": "никель-63", "5": "2", "6": "15,3x3", "7": "100000000.0", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Цилиндрический.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C4.1', 'BNi3.C4.1', '{"3": "BNi3.C4.1", "4": "никель-63", "5": "2", "6": "", "7": "100000000.0", "8": "0.0", "9": "0", "10": "ЗАО РИТВЕРЦ", "11": "0", "12": "из отчета Спб Изотоп", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C4.2', 'BNi3.C4.2', '{"3": "BNi3.C4.2", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "95000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C5', 'BNi3.C5', '{"3": "BNi3.C5", "4": "никель-63", "5": "2", "6": "4,7x1,7", "7": "", "8": "27000000.0", "9": "0", "10": "", "11": "5", "12": "Цилиндрический.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C5.1', 'BNi3.C5.1', '{"3": "BNi3.C5.1", "4": "никель-63", "5": "2", "6": "", "7": "27000000.0", "8": "0.0", "9": "0", "10": "ЗАО РИТВЕРЦ", "11": "5", "12": "из отчета Спб Изотоп", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C6', 'BNi3.C6', '{"3": "BNi3.C6", "4": "никель-63", "5": "2", "6": "12,3x1,7", "7": "", "8": "73000000.0", "9": "0", "10": "", "11": "5", "12": "Цилиндрический.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C6.1', 'BNi3.C6.1', '{"3": "BNi3.C6.1", "4": "никель-63", "5": "2", "6": "", "7": "73000000.0", "8": "0.0", "9": "0", "10": "ЗАО РИТВЕРЦ", "11": "5", "12": "из отчета Спб Изотоп", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.S1', 'BNi3.S1', '{"3": "BNi3.S1", "4": "никель-63", "5": "2", "6": "15x10", "7": "296000000.0", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Плоскостный. НСС 10 лет в атмосфере инертного газа.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.S1.1', 'BNi3.S1.1', '{"3": "BNi3.S1.1", "4": "никель-63", "5": "2", "6": "", "7": "296000000.0", "8": "0.0", "9": "0", "10": "ЗАО РИТВЕРЦ", "11": "5", "12": "из отчета Спб Изотоп", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.S1.2', 'BNi3.S1.2', '{"3": "BNi3.S1.2", "4": "никель-63", "5": "2", "6": "", "7": "370000000.0", "8": "0.0", "9": "0", "10": "ЗАО РИТВЕРЦ", "11": "5", "12": "из отчета Спб Изотоп", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.S2', 'BNi3.S2', '{"3": "BNi3.S2", "4": "никель-63", "5": "2", "6": "25x10", "7": "", "8": "555000000.0", "9": "0", "10": "", "11": "5", "12": "Плоскостный. НСС 10 лет в атмосфере инертного газа.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.S2.1', 'BNi3.S2.1', '{"3": "BNi3.S2.1", "4": "никель-63", "5": "2", "6": "", "7": "555000000.0", "8": "0.0", "9": "0", "10": "ЗАО РИТВЕРЦ", "11": "0", "12": "из отчета Спб Изотоп", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.S3', 'BNi3.S3', '{"3": "BNi3.S3", "4": "никель-63", "5": "2", "6": "30x10", "7": "100000000.0", "8": "1200000000.0", "9": "0", "10": "", "11": "5", "12": "Плоскостный. НСС 10 лет в атмосфере инертного газа.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.S3.1', 'BNi3.S3.1', '{"3": "BNi3.S3.1", "4": "никель-63", "5": "2", "6": "", "7": "100000000.0", "8": "0.0", "9": "0", "10": "ЗАО РИТВЕРЦ", "11": "0", "12": "из отчета Спб Изотоп", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.S3.2', 'BNi3.S3.2', '{"3": "BNi3.S3.2", "4": "никель-63", "5": "2", "6": "", "7": "600000000.0", "8": "0.0", "9": "0", "10": "ЗАО РИТВЕРЦ", "11": "0", "12": "из отчета Спб Изотоп", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.S3.3', 'BNi3.S3.3', '{"3": "BNi3.S3.3", "4": "никель-63", "5": "2", "6": "", "7": "740000000.0", "8": "0.0", "9": "0", "10": "ЗАО РИТВЕРЦ", "11": "5", "12": "из отчета Спб Изотоп", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.S3.4', 'BNi3.S3.4', '{"3": "BNi3.S3.4", "4": "никель-63", "5": "2", "6": "", "7": "1200000000.0", "8": "0.0", "9": "0", "10": "ЗАО РИТВЕРЦ", "11": "5", "12": "из отчета Спб Изотоп", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.S4', 'BNi3.S4', '{"3": "BNi3.S4", "4": "никель-63", "5": "2", "6": "48x3", "7": "100000000.0", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Плоскостный. НСС 10 лет в атмосфере инертного газа.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.S4.1', 'BNi3.S4.1', '{"3": "BNi3.S4.1", "4": "никель-63", "5": "2", "6": "", "7": "100000000.0", "8": "0.0", "9": "0", "10": "ЗАО РИТВЕРЦ", "11": "0", "12": "из отчета Спб Изотоп", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.S4.2', 'BNi3.S4.2', '{"3": "BNi3.S4.2", "4": "никель-63", "5": "2", "6": "", "7": "370000000.0", "8": "0.0", "9": "0", "10": "ЗАО РИТВЕРЦ", "11": "0", "12": "из отчета Спб Изотоп", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.S5', 'BNi3.S5', '{"3": "BNi3.S5", "4": "никель-63", "5": "2", "6": "14,5x1,7", "7": "", "8": "27000000.0", "9": "0", "10": "", "11": "5", "12": "Плоскостный. НСС 10 лет в атмосфере инертного газа.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.S5.1', 'BNi3.S5.1', '{"3": "BNi3.S5.1", "4": "никель-63", "5": "2", "6": "", "7": "27000000.0", "8": "0.0", "9": "0", "10": "ЗАО РИТВЕРЦ", "11": "0", "12": "из отчета Спб Изотоп", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.S6', 'BNi3.S6', '{"3": "BNi3.S6", "4": "никель-63", "5": "2", "6": "38,5x1,7", "7": "", "8": "73000000.0", "9": "0", "10": "", "11": "5", "12": "Плоскостный. НСС 10 лет в атмосфере инертного газа.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.S6.1', 'BNi3.S6.1', '{"3": "BNi3.S6.1", "4": "никель-63", "5": "2", "6": "", "7": "73000000.0", "8": "0.0", "9": "0", "10": "ЗАО РИТВЕРЦ", "11": "0", "12": "из отчета Спб Изотоп", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BRu6.X', 'BRu6.X', '{"3": "BRu6.X", "4": "рутений-106", "5": "2", "6": "", "7": "", "8": "830000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr0', 'BSr0', '{"3": "BSr0", "4": "стронций-90", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr0.01', 'BSr0.01', '{"3": "BSr0.01", "4": "стронций-90", "5": "2", "6": "19x11,5x23", "7": "37000000.0", "8": "18500000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Цилиндрический.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr0.02', 'BSr0.02', '{"3": "BSr0.02", "4": "стронций-90", "5": "2", "6": "22x6. Акт. часть dxh: 15x1,5", "7": "37000000.0", "8": "18500000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Дисковый (капсула а).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr0.02X', 'BSr0.02X', '{"3": "BSr0.02X", "4": "стронций-90", "5": "2", "6": "", "7": "", "8": "555000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr0.03', 'BSr0.03', '{"3": "BSr0.03", "4": "стронций-90", "5": "2", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr0.04', 'BSr0.04', '{"3": "BSr0.04", "4": "стронций-90", "5": "2", "6": "2x10. Акт. часть dxh: 1x1,5", "7": "3700000.0", "8": "370000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Дисковый (капсула d).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr0.042', 'BSr0.042', '{"3": "BSr0.042", "4": "стронций-90", "5": "2", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr0.043', 'BSr0.043', '{"3": "BSr0.043", "4": "стронций-90", "5": "2", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr0.044', 'BSr0.044', '{"3": "BSr0.044", "4": "стронций-90", "5": "2", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr0.04x', 'BSr0.04x', '{"3": "BSr0.04x", "4": "стронций-90", "5": "2", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr0.05', 'BSr0.05', '{"3": "BSr0.05", "4": "стронций-90", "5": "2", "6": "19x12. Акт. часть dxh: 5x1", "7": "37000000.0", "8": "18500000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Дисковый (капсула b).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr0.10.P3', 'BSr0.10.P3', '{"3": "BSr0.10.P3", "4": "стронций-90", "5": "2", "6": "", "7": "", "8": "296000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr0.36', 'BSr0.36', '{"3": "BSr0.36", "4": "стронций-90", "5": "2", "6": "1,65x17,5. Акт. часть dxh: 1,27x6,7", "7": "3700000.0", "8": "370000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Линейный.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr0.37', 'BSr0.37', '{"3": "BSr0.37", "4": "стронций-90", "5": "2", "6": "3x10. Акт. часть dxh: 1x1,5", "7": "3700000.0", "8": "370000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Дисковый (капсула d).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr0.38', 'BSr0.38', '{"3": "BSr0.38", "4": "стронций-90", "5": "2", "6": "4x10. Акт. часть dxh: 3x1,5", "7": "3700000.0", "8": "370000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Дисковый (капсула d).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr0.39', 'BSr0.39', '{"3": "BSr0.39", "4": "стронций-90", "5": "2", "6": "7x10. Акт. часть dxh: 5x1,5", "7": "37000000.0", "8": "18500000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Дисковый (капсула d).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr0.4.P1', 'BSr0.4.P1', '{"3": "BSr0.4.P1", "4": "стронций-90", "5": "2", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr0.40', 'BSr0.40', '{"3": "BSr0.40", "4": "стронций-90", "5": "2", "6": "8x5. Акт. часть dxh: 5x1", "7": "37000000.0", "8": "18500000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Дисковый (капсула с).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr0.401', 'BSr0.401', '{"3": "BSr0.401", "4": "стронций-90", "5": "2", "6": "", "7": "", "8": "74000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr0.404', 'BSr0.404', '{"3": "BSr0.404", "4": "стронций-90", "5": "2", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr0.P100', 'BSr0.P100', '{"3": "BSr0.P100", "4": "стронций-90", "5": "2", "6": "", "7": "", "8": "6600000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'C-146', 'C-146', '{"3": "C-146", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "Канада", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'C-41', 'C-41', '{"3": "C-41", "4": "радий-226", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'C-51-3', 'C-51-3', '{"3": "C-51-3", "4": "радий-226", "5": "3", "6": "", "7": "0.0", "8": "185000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'C-51-4', 'C-51-4', '{"3": "C-51-4", "4": "радий-226", "5": "3", "6": "", "7": "0.0", "8": "185000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'C-62-11', 'C-62-11', '{"3": "C-62-11", "4": "радий-226", "5": "3", "6": "", "7": "0.0", "8": "37000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'C-62-12', 'C-62-12', '{"3": "C-62-12", "4": "радий-226", "5": "3", "6": "", "7": "0.0", "8": "18500.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CD932', 'CD932', '{"3": "CD932", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CD932040001M', 'CD932040001M', '{"3": "CD932040001M", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CD932040010M', 'CD932040010M', '{"3": "CD932040010M", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CD932040014M', 'CD932040014M', '{"3": "CD932040014M", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "518000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CD932040020M', 'CD932040020M', '{"3": "CD932040020M", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CD932040040M', 'CD932040040M', '{"3": "CD932040040M", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "1480000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CD932050001M', 'CD932050001M', '{"3": "CD932050001M", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CD932050010M', 'CD932050010M', '{"3": "CD932050010M", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CD932050014M', 'CD932050014M', '{"3": "CD932050014M", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "518000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CD932050020M', 'CD932050020M', '{"3": "CD932050020M", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CD932050040M', 'CD932050040M', '{"3": "CD932050040M", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "1480000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CD932050050M', 'CD932050050M', '{"3": "CD932050050M", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CD932150001M', 'CD932150001M', '{"3": "CD932150001M", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CD932150010M', 'CD932150010M', '{"3": "CD932150010M", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CD932150020M', 'CD932150020M', '{"3": "CD932150020M", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CD932150050M', 'CD932150050M', '{"3": "CD932150050M", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CD932360001M', 'CD932360001M', '{"3": "CD932360001M", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CD932360005M', 'CD932360005M', '{"3": "CD932360005M", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CD932360010M', 'CD932360010M', '{"3": "CD932360010M", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC', 'CDC', '{"3": "CDC", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 3803', 'CDC 3803', '{"3": "CDC 3803", "4": "цезий-137", "5": "1", "6": "", "7": "121000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 3804', 'CDC 3804', '{"3": "CDC 3804", "4": "цезий-137", "5": "1", "6": "", "7": "209000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 3805', 'CDC 3805', '{"3": "CDC 3805", "4": "цезий-137", "5": "1", "6": "", "7": "409000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 3806', 'CDC 3806', '{"3": "CDC 3806", "4": "цезий-137", "5": "1", "6": "", "7": "828000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 3807', 'CDC 3807', '{"3": "CDC 3807", "4": "цезий-137", "5": "1", "6": "", "7": "1210000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 3808', 'CDC 3808', '{"3": "CDC 3808", "4": "цезий-137", "5": "1", "6": "", "7": "1980000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 3809', 'CDC 3809', '{"3": "CDC 3809", "4": "цезий-137", "5": "1", "6": "", "7": "4070000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 3810', 'CDC 3810', '{"3": "CDC 3810", "4": "цезий-137", "5": "1", "6": "", "7": "8850000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 3811', 'CDC 3811', '{"3": "CDC 3811", "4": "цезий-137", "5": "1", "6": "", "7": "12600000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 3820', 'CDC 3820', '{"3": "CDC 3820", "4": "цезий-137", "5": "1", "6": "", "7": "19700000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 3821', 'CDC 3821', '{"3": "CDC 3821", "4": "цезий-137", "5": "1", "6": "", "7": "39200000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 38210', 'CDC 38210', '{"3": "CDC 38210", "4": "цезий-137", "5": "1", "6": "", "7": "369000000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 3822', 'CDC 3822', '{"3": "CDC 3822", "4": "цезий-137", "5": "1", "6": "", "7": "77700000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 3823', 'CDC 3823', '{"3": "CDC 3823", "4": "цезий-137", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 3824', 'CDC 3824', '{"3": "CDC 3824", "4": "цезий-137", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 3825', 'CDC 3825', '{"3": "CDC 3825", "4": "цезий-137", "5": "1", "6": "", "7": "190000000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 3826', 'CDC 3826', '{"3": "CDC 3826", "4": "цезий-137", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 3827', 'CDC 3827', '{"3": "CDC 3827", "4": "цезий-137", "5": "1", "6": "", "7": "278000000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 3828', 'CDC 3828', '{"3": "CDC 3828", "4": "цезий-137", "5": "1", "6": "", "7": "318000000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 6202', 'CDC 6202', '{"3": "CDC 6202", "4": "цезий-137", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 706', 'CDC 706', '{"3": "CDC 706", "4": "цезий-137", "5": "1", "6": "", "7": "736000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 707', 'CDC 707', '{"3": "CDC 707", "4": "цезий-137", "5": "1", "6": "", "7": "1120000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 711', 'CDC 711', '{"3": "CDC 711", "4": "цезий-137", "5": "1", "6": "", "7": "11300000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 7901', 'CDC 7901', '{"3": "CDC 7901", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 7902', 'CDC 7902', '{"3": "CDC 7902", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 7903', 'CDC 7903', '{"3": "CDC 7903", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 7904', 'CDC 7904', '{"3": "CDC 7904", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "74000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 7905', 'CDC 7905', '{"3": "CDC 7905", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "110000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 7906', 'CDC 7906', '{"3": "CDC 7906", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 7908', 'CDC 7908', '{"3": "CDC 7908", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 7910', 'CDC 7910', '{"3": "CDC 7910", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 7911', 'CDC 7911', '{"3": "CDC 7911", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 7913', 'CDC 7913', '{"3": "CDC 7913", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 7914', 'CDC 7914', '{"3": "CDC 7914", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "2800000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 7915', 'CDC 7915', '{"3": "CDC 7915", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 7916', 'CDC 7916', '{"3": "CDC 7916", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "5500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 7917', 'CDC 7917', '{"3": "CDC 7917", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "7400000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 7918', 'CDC 7918', '{"3": "CDC 7918", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "9250000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 7922', 'CDC 7922', '{"3": "CDC 7922", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 801', 'CDC 801', '{"3": "CDC 801", "4": "цезий-137", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 804', 'CDC 804', '{"3": "CDC 804", "4": "цезий-137", "5": "1", "6": "", "7": "212000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 805', 'CDC 805', '{"3": "CDC 805", "4": "цезий-137", "5": "1", "6": "", "7": "381000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 808', 'CDC 808', '{"3": "CDC 808", "4": "цезий-137", "5": "1", "6": "", "7": "2100000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 809', 'CDC 809', '{"3": "CDC 809", "4": "цезий-137", "5": "1", "6": "", "7": "3550000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 810', 'CDC 810', '{"3": "CDC 810", "4": "цезий-137", "5": "1", "6": "", "7": "8330000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 90', 'CDC 90', '{"3": "CDC 90", "4": "цезий-137", "5": "1", "6": "", "7": "18100000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 91', 'CDC 91', '{"3": "CDC 91", "4": "цезий-137", "5": "1", "6": "", "7": "35200000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 92', 'CDC 92', '{"3": "CDC 92", "4": "цезий-137", "5": "1", "6": "", "7": "84400000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 93', 'CDC 93', '{"3": "CDC 93", "4": "цезий-137", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.1194', 'CDC.1194', '{"3": "CDC.1194", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1480000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.26205', 'CDC.26205', '{"3": "CDC.26205", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "377000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.3801', 'CDC.3801', '{"3": "CDC.3801", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.3802', 'CDC.3802', '{"3": "CDC.3802", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "74000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.3803', 'CDC.3803', '{"3": "CDC.3803", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "111000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.3804', 'CDC.3804', '{"3": "CDC.3804", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.3805', 'CDC.3805', '{"3": "CDC.3805", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.3806', 'CDC.3806', '{"3": "CDC.3806", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.3807', 'CDC.3807', '{"3": "CDC.3807", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.3808', 'CDC.3808', '{"3": "CDC.3808", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.3809', 'CDC.3809', '{"3": "CDC.3809", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.3810', 'CDC.3810', '{"3": "CDC.3810", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "7400000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.3811', 'CDC.3811', '{"3": "CDC.3811", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "11100000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.3820', 'CDC.3820', '{"3": "CDC.3820", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.3821', 'CDC.3821', '{"3": "CDC.3821", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.38210', 'CDC.38210', '{"3": "CDC.38210", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.3822', 'CDC.3822', '{"3": "CDC.3822", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "74000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.3823', 'CDC.3823', '{"3": "CDC.3823", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "111000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.3824', 'CDC.3824', '{"3": "CDC.3824", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "148000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.3825', 'CDC.3825', '{"3": "CDC.3825", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "185000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.3826', 'CDC.3826', '{"3": "CDC.3826", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "222000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.3827', 'CDC.3827', '{"3": "CDC.3827", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "259000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.3828', 'CDC.3828', '{"3": "CDC.3828", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "296000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.3829', 'CDC.3829', '{"3": "CDC.3829", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "333000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.6202', 'CDC.6202', '{"3": "CDC.6202", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.701', 'CDC.701', '{"3": "CDC.701", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.702', 'CDC.702', '{"3": "CDC.702", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "74000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.703', 'CDC.703', '{"3": "CDC.703", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "111000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.704', 'CDC.704', '{"3": "CDC.704", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.705', 'CDC.705', '{"3": "CDC.705", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.70550', 'CDC.70550', '{"3": "CDC.70550", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "555000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.706', 'CDC.706', '{"3": "CDC.706", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.70650', 'CDC.70650', '{"3": "CDC.70650", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.707', 'CDC.707', '{"3": "CDC.707", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.708', 'CDC.708', '{"3": "CDC.708", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.709', 'CDC.709', '{"3": "CDC.709", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.710', 'CDC.710', '{"3": "CDC.710", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "7400000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.711', 'CDC.711', '{"3": "CDC.711", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "11100000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.800', 'CDC.800', '{"3": "CDC.800", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.801', 'CDC.801', '{"3": "CDC.801", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.802', 'CDC.802', '{"3": "CDC.802", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "74000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.803', 'CDC.803', '{"3": "CDC.803", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "111000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.804', 'CDC.804', '{"3": "CDC.804", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.805', 'CDC.805', '{"3": "CDC.805", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.80550', 'CDC.80550', '{"3": "CDC.80550", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "555000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.806', 'CDC.806', '{"3": "CDC.806", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.80650', 'CDC.80650', '{"3": "CDC.80650", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.807', 'CDC.807', '{"3": "CDC.807", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.808', 'CDC.808', '{"3": "CDC.808", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.809', 'CDC.809', '{"3": "CDC.809", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.810', 'CDC.810', '{"3": "CDC.810", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "7400000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.811', 'CDC.811', '{"3": "CDC.811", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "11100000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.90', 'CDC.90', '{"3": "CDC.90", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.91', 'CDC.91', '{"3": "CDC.91", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.92', 'CDC.92', '{"3": "CDC.92", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "74000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.93', 'CDC.93', '{"3": "CDC.93", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "111000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.CY', 'CDC.CY', '{"3": "CDC.CY", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.CY16', 'CDC.CY16', '{"3": "CDC.CY16", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "74000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.CY3', 'CDC.CY3', '{"3": "CDC.CY3", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "65900000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.CY4', 'CDC.CY4', '{"3": "CDC.CY4", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.CY8', 'CDC.CY8', '{"3": "CDC.CY8", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCB', 'CDCB', '{"3": "CDCB", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCB 1901', 'CDCB 1901', '{"3": "CDCB 1901", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCB 1902', 'CDCB 1902', '{"3": "CDCB 1902", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "7400000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCB 1904', 'CDCB 1904', '{"3": "CDCB 1904", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCB 1905', 'CDCB 1905', '{"3": "CDCB 1905", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCB 1907', 'CDCB 1907', '{"3": "CDCB 1907", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "74000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCB 1911', 'CDCB 1911', '{"3": "CDCB 1911", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCB 1915', 'CDCB 1915', '{"3": "CDCB 1915", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCB 1918', 'CDCB 1918', '{"3": "CDCB 1918", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCB 1920', 'CDCB 1920', '{"3": "CDCB 1920", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCB 1924', 'CDCB 1924', '{"3": "CDCB 1924", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCB 1926', 'CDCB 1926', '{"3": "CDCB 1926", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "2800000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCB 1928', 'CDCB 1928', '{"3": "CDCB 1928", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCB 1929', 'CDCB 1929', '{"3": "CDCB 1929", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "5500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCB 1932', 'CDCB 1932', '{"3": "CDCB 1932", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "7400000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCB 1933', 'CDCB 1933', '{"3": "CDCB 1933", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "9250000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCB 1942', 'CDCB 1942', '{"3": "CDCB 1942", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCK', 'CDCK', '{"3": "CDCK", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCK 3287', 'CDCK 3287', '{"3": "CDCK 3287", "4": "цезий-137", "5": "1", "6": "", "7": "386000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCK 3804', 'CDCK 3804', '{"3": "CDCK 3804", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "285000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCK 3806', 'CDCK 3806', '{"3": "CDCK 3806", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "107000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCK 4103', 'CDCK 4103', '{"3": "CDCK 4103", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "381000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCK 5129', 'CDCK 5129', '{"3": "CDCK 5129", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "61600000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCK 5751', 'CDCK 5751', '{"3": "CDCK 5751", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "95100000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCK 7433', 'CDCK 7433', '{"3": "CDCK 7433", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "58600000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCK 7788', 'CDCK 7788', '{"3": "CDCK 7788", "4": "цезий-137", "5": "1", "6": "", "7": "377000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCK 8265', 'CDCK 8265', '{"3": "CDCK 8265", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "60700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCK 8266', 'CDCK 8266', '{"3": "CDCK 8266", "4": "цезий-137", "5": "1", "6": "", "7": "58800000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCK 8267', 'CDCK 8267', '{"3": "CDCK 8267", "4": "цезий-137", "5": "1", "6": "", "7": "74400000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCK 8272', 'CDCK 8272', '{"3": "CDCK 8272", "4": "цезий-137", "5": "1", "6": "", "7": "61100000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCK 8274', 'CDCK 8274', '{"3": "CDCK 8274", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "203000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCK 8275', 'CDCK 8275', '{"3": "CDCK 8275", "4": "цезий-137", "5": "1", "6": "", "7": "74400000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCK 8321', 'CDCK 8321', '{"3": "CDCK 8321", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37400000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCQ', 'CDCQ', '{"3": "CDCQ", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCQ 6202', 'CDCQ 6202', '{"3": "CDCQ 6202", "4": "цезий-137", "5": "1", "6": "", "7": "3520000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCQ 6949', 'CDCQ 6949', '{"3": "CDCQ 6949", "4": "цезий-137", "5": "1", "6": "", "7": "757000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCQ 7147', 'CDCQ 7147', '{"3": "CDCQ 7147", "4": "цезий-137", "5": "1", "6": "", "7": "1870000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDRB', 'CDRB', '{"3": "CDRB", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "333000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF2', 'CF2', '{"3": "CF2", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF230140001M', 'CF230140001M', '{"3": "CF230140001M", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF230140005U', 'CF230140005U', '{"3": "CF230140005U", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "185000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF230140010U', 'CF230140010U', '{"3": "CF230140010U", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "370000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF230140100N', 'CF230140100N', '{"3": "CF230140100N", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "3700.0", "9": "0", "10": "", "11": "0", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF230140500U', 'CF230140500U', '{"3": "CF230140500U", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF230230001M', 'CF230230001M', '{"3": "CF230230001M", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF230230005U', 'CF230230005U', '{"3": "CF230230005U", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "185000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF230230010U', 'CF230230010U', '{"3": "CF230230010U", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "370000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF230230100N', 'CF230230100N', '{"3": "CF230230100N", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "3700.0", "9": "0", "10": "", "11": "0", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF230230500U', 'CF230230500U', '{"3": "CF230230500U", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF230240001M', 'CF230240001M', '{"3": "CF230240001M", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF230240005U', 'CF230240005U', '{"3": "CF230240005U", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "185000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF230240010U', 'CF230240010U', '{"3": "CF230240010U", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "370000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF230240100N', 'CF230240100N', '{"3": "CF230240100N", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "3700.0", "9": "0", "10": "", "11": "0", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF230240500U', 'CF230240500U', '{"3": "CF230240500U", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF230260001M', 'CF230260001M', '{"3": "CF230260001M", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF230260005U', 'CF230260005U', '{"3": "CF230260005U", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "185000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF230260010U', 'CF230260010U', '{"3": "CF230260010U", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "370000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF230260100N', 'CF230260100N', '{"3": "CF230260100N", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "3700.0", "9": "0", "10": "", "11": "0", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF230260500U', 'CF230260500U', '{"3": "CF230260500U", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF230360001M', 'CF230360001M', '{"3": "CF230360001M", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF230360005U', 'CF230360005U', '{"3": "CF230360005U", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "185000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF230360010U', 'CF230360010U', '{"3": "CF230360010U", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "370000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF230360100N', 'CF230360100N', '{"3": "CF230360100N", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "3700.0", "9": "0", "10": "", "11": "0", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CF230360500U', 'CF230360500U', '{"3": "CF230360500U", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CH3 P07 2/6/3', 'CH3 P07 2/6/3', '{"3": "CH3 P07 2/6/3", "4": "тритий", "5": "2", "6": "", "7": "", "8": "5920000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CH3 Р05 13/20/З', 'CH3 Р05 13/20/З', '{"3": "CH3 Р05 13/20/З", "4": "тритий", "5": "2", "6": "", "7": "", "8": "71000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CH3 Р05 2/15/З', 'CH3 Р05 2/15/З', '{"3": "CH3 Р05 2/15/З", "4": "тритий", "5": "2", "6": "", "7": "", "8": "1440000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CH3 Р05 7/20/3', 'CH3 Р05 7/20/3', '{"3": "CH3 Р05 7/20/3", "4": "тритий", "5": "2", "6": "", "7": "", "8": "23800000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKC', 'CKC', '{"3": "CKC", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKC.6001', 'CKC.6001', '{"3": "CKC.6001", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKC.6002', 'CKC.6002', '{"3": "CKC.6002", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKC.6003', 'CKC.6003', '{"3": "CKC.6003", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKC.6004', 'CKC.6004', '{"3": "CKC.6004", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "74000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKC.6005', 'CKC.6005', '{"3": "CKC.6005", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "110000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKC.6006', 'CKC.6006', '{"3": "CKC.6006", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKC.6007', 'CKC.6007', '{"3": "CKC.6007", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "300000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKC.6008', 'CKC.6008', '{"3": "CKC.6008", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKC.6009', 'CKC.6009', '{"3": "CKC.6009", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "550000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKC.6010', 'CKC.6010', '{"3": "CKC.6010", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKC.6011', 'CKC.6011', '{"3": "CKC.6011", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKC.6012', 'CKC.6012', '{"3": "CKC.6012", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKC.6013', 'CKC.6013', '{"3": "CKC.6013", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "1500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKC.6014', 'CKC.6014', '{"3": "CKC.6014", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKC.6015', 'CKC.6015', '{"3": "CKC.6015", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "2200000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKC.6016', 'CKC.6016', '{"3": "CKC.6016", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "2800000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKC.6017', 'CKC.6017', '{"3": "CKC.6017", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKC.6018', 'CKC.6018', '{"3": "CKC.6018", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "5500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKC.6019', 'CKC.6019', '{"3": "CKC.6019", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "7400000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKC.6020', 'CKC.6020', '{"3": "CKC.6020", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "9250000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKC.6021', 'CKC.6021', '{"3": "CKC.6021", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "11100000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKC.6022', 'CKC.6022', '{"3": "CKC.6022", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "15000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKC.6023', 'CKC.6023', '{"3": "CKC.6023", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKUMZ', 'CKUMZ', '{"3": "CKUMZ", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKUMZ001', 'CKUMZ001', '{"3": "CKUMZ001", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKUMZ002', 'CKUMZ002', '{"3": "CKUMZ002", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "74000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKUMZ003', 'CKUMZ003', '{"3": "CKUMZ003", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "111000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKUMZ004', 'CKUMZ004', '{"3": "CKUMZ004", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "148000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CKUMZ005', 'CKUMZ005', '{"3": "CKUMZ005", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "185000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CLC', 'CLC', '{"3": "CLC", "4": "кюрий-244", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CLC.13074', 'CLC.13074', '{"3": "CLC.13074", "4": "кюрий-244", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CLC.13075', 'CLC.13075', '{"3": "CLC.13075", "4": "кюрий-244", "5": "1", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CLC.13076', 'CLC.13076', '{"3": "CLC.13076", "4": "кюрий-244", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CLC.13144', 'CLC.13144', '{"3": "CLC.13144", "4": "кюрий-244", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CLC.13145', 'CLC.13145', '{"3": "CLC.13145", "4": "кюрий-244", "5": "1", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CLC.13146', 'CLC.13146', '{"3": "CLC.13146", "4": "кюрий-244", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CLC.13147', 'CLC.13147', '{"3": "CLC.13147", "4": "кюрий-244", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CLC.13528', 'CLC.13528', '{"3": "CLC.13528", "4": "кюрий-244", "5": "1", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CLC.13529', 'CLC.13529', '{"3": "CLC.13529", "4": "кюрий-244", "5": "1", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CLS 15вн.', 'CLS 15вн.', '{"3": "CLS 15вн.", "4": "цезий-137", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CLS 20вн.', 'CLS 20вн.', '{"3": "CLS 20вн.", "4": "цезий-137", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO0', 'CO0', '{"3": "CO0", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030000001M', 'CO030000001M', '{"3": "CO030000001M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030000005M', 'CO030000005M', '{"3": "CO030000005M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030000010M', 'CO030000010M', '{"3": "CO030000010M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030000050M', 'CO030000050M', '{"3": "CO030000050M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030000100M', 'CO030000100M', '{"3": "CO030000100M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030000100U', 'CO030000100U', '{"3": "CO030000100U", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030000500U', 'CO030000500U', '{"3": "CO030000500U", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030110001M', 'CO030110001M', '{"3": "CO030110001M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030110005M', 'CO030110005M', '{"3": "CO030110005M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030110010M', 'CO030110010M', '{"3": "CO030110010M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030110050M', 'CO030110050M', '{"3": "CO030110050M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030110100M', 'CO030110100M', '{"3": "CO030110100M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030110100U', 'CO030110100U', '{"3": "CO030110100U", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030110500U', 'CO030110500U', '{"3": "CO030110500U", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030150001M', 'CO030150001M', '{"3": "CO030150001M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030150005M', 'CO030150005M', '{"3": "CO030150005M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030150010M', 'CO030150010M', '{"3": "CO030150010M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030150050M', 'CO030150050M', '{"3": "CO030150050M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030150100M', 'CO030150100M', '{"3": "CO030150100M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030150100U', 'CO030150100U', '{"3": "CO030150100U", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030150500M', 'CO030150500M', '{"3": "CO030150500M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030250001M', 'CO030250001M', '{"3": "CO030250001M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030250005M', 'CO030250005M', '{"3": "CO030250005M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CP17.212', 'CP17.212', '{"3": "CP17.212", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "4600000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030250010M', 'CO030250010M', '{"3": "CO030250010M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030250050M', 'CO030250050M', '{"3": "CO030250050M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030250100M', 'CO030250100M', '{"3": "CO030250100M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030250100U', 'CO030250100U', '{"3": "CO030250100U", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030250500M', 'CO030250500M', '{"3": "CO030250500M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030260001M', 'CO030260001M', '{"3": "CO030260001M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030260005M', 'CO030260005M', '{"3": "CO030260005M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030260010M', 'CO030260010M', '{"3": "CO030260010M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030260050M', 'CO030260050M', '{"3": "CO030260050M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030260100M', 'CO030260100M', '{"3": "CO030260100M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030260100U', 'CO030260100U', '{"3": "CO030260100U", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030260500M', 'CO030260500M', '{"3": "CO030260500M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030440001C', 'CO030440001C', '{"3": "CO030440001C", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "3", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030440001M', 'CO030440001M', '{"3": "CO030440001M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030440004C', 'CO030440004C', '{"3": "CO030440004C", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "148000000000.0", "9": "0", "10": "", "11": "3", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030440010M', 'CO030440010M', '{"3": "CO030440010M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030440100M', 'CO030440100M', '{"3": "CO030440100M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO030440100U', 'CO030440100U', '{"3": "CO030440100U", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO032240001M', 'CO032240001M', '{"3": "CO032240001M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO032240005M', 'CO032240005M', '{"3": "CO032240005M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO032240005U', 'CO032240005U', '{"3": "CO032240005U", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "185000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO032240010U', 'CO032240010U', '{"3": "CO032240010U", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "370000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO032240050U', 'CO032240050U', '{"3": "CO032240050U", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "1850000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO032240100U', 'CO032240100U', '{"3": "CO032240100U", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO032240500U', 'CO032240500U', '{"3": "CO032240500U", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO034020001M', 'CO034020001M', '{"3": "CO034020001M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO034020005M', 'CO034020005M', '{"3": "CO034020005M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO034020010M', 'CO034020010M', '{"3": "CO034020010M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO034020100U', 'CO034020100U', '{"3": "CO034020100U", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO034020500U', 'CO034020500U', '{"3": "CO034020500U", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO0P030001C', 'CO0P030001C', '{"3": "CO0P030001C", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "3", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO0P030010M', 'CO0P030010M', '{"3": "CO0P030010M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO0P030050M', 'CO0P030050M', '{"3": "CO0P030050M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO0P030100M', 'CO0P030100M', '{"3": "CO0P030100M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO0P030500M', 'CO0P030500M', '{"3": "CO0P030500M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO0P040001C', 'CO0P040001C', '{"3": "CO0P040001C", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "3", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO0P040010M', 'CO0P040010M', '{"3": "CO0P040010M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO0P040050M', 'CO0P040050M', '{"3": "CO0P040050M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO0P040100M', 'CO0P040100M', '{"3": "CO0P040100M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO0P040500M', 'CO0P040500M', '{"3": "CO0P040500M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO0P170001C', 'CO0P170001C', '{"3": "CO0P170001C", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "3", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO0P170002C', 'CO0P170002C', '{"3": "CO0P170002C", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "74000000000.0", "9": "0", "10": "", "11": "3", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO0P170004C', 'CO0P170004C', '{"3": "CO0P170004C", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "148000000000.0", "9": "0", "10": "", "11": "3", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO0P170050M', 'CO0P170050M', '{"3": "CO0P170050M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO0P170100M', 'CO0P170100M', '{"3": "CO0P170100M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO0P170500M', 'CO0P170500M', '{"3": "CO0P170500M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO0P17-10001C', 'CO0P17-10001C', '{"3": "CO0P17-10001C", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "3", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO0P17-10002C', 'CO0P17-10002C', '{"3": "CO0P17-10002C", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "74000000000.0", "9": "0", "10": "", "11": "3", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO0P17-10004C', 'CO0P17-10004C', '{"3": "CO0P17-10004C", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "148000000000.0", "9": "0", "10": "", "11": "3", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO0P17-10050M', 'CO0P17-10050M', '{"3": "CO0P17-10050M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO0P17-10100M', 'CO0P17-10100M', '{"3": "CO0P17-10100M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO0P17-10500M', 'CO0P17-10500M', '{"3": "CO0P17-10500M", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO7', 'CO7', '{"3": "CO7", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO730000001M', 'CO730000001M', '{"3": "CO730000001M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO730000005M', 'CO730000005M', '{"3": "CO730000005M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO730000010M', 'CO730000010M', '{"3": "CO730000010M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO730000025M', 'CO730000025M', '{"3": "CO730000025M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO730000050M', 'CO730000050M', '{"3": "CO730000050M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO730000100M', 'CO730000100M', '{"3": "CO730000100M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO730000500U', 'CO730000500U', '{"3": "CO730000500U", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO730240001M', 'CO730240001M', '{"3": "CO730240001M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO730240005M', 'CO730240005M', '{"3": "CO730240005M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO730240010M', 'CO730240010M', '{"3": "CO730240010M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO730240025M', 'CO730240025M', '{"3": "CO730240025M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO730240050M', 'CO730240050M', '{"3": "CO730240050M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO730240100M', 'CO730240100M', '{"3": "CO730240100M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO730240500U', 'CO730240500U', '{"3": "CO730240500U", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO730250001M', 'CO730250001M', '{"3": "CO730250001M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO730250005M', 'CO730250005M', '{"3": "CO730250005M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO730250010M', 'CO730250010M', '{"3": "CO730250010M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO730250025M', 'CO730250025M', '{"3": "CO730250025M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO730250050M', 'CO730250050M', '{"3": "CO730250050M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO730250100M', 'CO730250100M', '{"3": "CO730250100M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO730250500U', 'CO730250500U', '{"3": "CO730250500U", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732010001M', 'CO732010001M', '{"3": "CO732010001M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732010005M', 'CO732010005M', '{"3": "CO732010005M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732010010M', 'CO732010010M', '{"3": "CO732010010M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732010025M', 'CO732010025M', '{"3": "CO732010025M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732010050M', 'CO732010050M', '{"3": "CO732010050M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732010100M', 'CO732010100M', '{"3": "CO732010100M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732010500U', 'CO732010500U', '{"3": "CO732010500U", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732020001M', 'CO732020001M', '{"3": "CO732020001M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732020005M', 'CO732020005M', '{"3": "CO732020005M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732020010M', 'CO732020010M', '{"3": "CO732020010M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732020025M', 'CO732020025M', '{"3": "CO732020025M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732020050M', 'CO732020050M', '{"3": "CO732020050M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732020100M', 'CO732020100M', '{"3": "CO732020100M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732020500U', 'CO732020500U', '{"3": "CO732020500U", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732040001M', 'CO732040001M', '{"3": "CO732040001M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732040005M', 'CO732040005M', '{"3": "CO732040005M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732040010M', 'CO732040010M', '{"3": "CO732040010M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732040025M', 'CO732040025M', '{"3": "CO732040025M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732040050M', 'CO732040050M', '{"3": "CO732040050M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732040100M', 'CO732040100M', '{"3": "CO732040100M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732040500U', 'CO732040500U', '{"3": "CO732040500U", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732050', 'CO732050', '{"3": "CO732050", "4": "кобальт-57", "5": "5", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732050001M', 'CO732050001M', '{"3": "CO732050001M", "4": "кобальт-57", "5": "5", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732050005M', 'CO732050005M', '{"3": "CO732050005M", "4": "кобальт-57", "5": "5", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732050010M', 'CO732050010M', '{"3": "CO732050010M", "4": "кобальт-57", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732050025M', 'CO732050025M', '{"3": "CO732050025M", "4": "кобальт-57", "5": "5", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732050050M', 'CO732050050M', '{"3": "CO732050050M", "4": "кобальт-57", "5": "5", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732050100M', 'CO732050100M', '{"3": "CO732050100M", "4": "кобальт-57", "5": "5", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732050500U', 'CO732050500U', '{"3": "CO732050500U", "4": "кобальт-57", "5": "5", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732150001M', 'CO732150001M', '{"3": "CO732150001M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732150005M', 'CO732150005M', '{"3": "CO732150005M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732150010M', 'CO732150010M', '{"3": "CO732150010M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732150025M', 'CO732150025M', '{"3": "CO732150025M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732150050M', 'CO732150050M', '{"3": "CO732150050M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732150100M', 'CO732150100M', '{"3": "CO732150100M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732150500U', 'CO732150500U', '{"3": "CO732150500U", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732240001M', 'CO732240001M', '{"3": "CO732240001M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732240005M', 'CO732240005M', '{"3": "CO732240005M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732240010M', 'CO732240010M', '{"3": "CO732240010M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732240025M', 'CO732240025M', '{"3": "CO732240025M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732240050M', 'CO732240050M', '{"3": "CO732240050M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732240100M', 'CO732240100M', '{"3": "CO732240100M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO732240500U', 'CO732240500U', '{"3": "CO732240500U", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO734020001M', 'CO734020001M', '{"3": "CO734020001M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO734020005M', 'CO734020005M', '{"3": "CO734020005M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO734020010M', 'CO734020010M', '{"3": "CO734020010M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO734020025M', 'CO734020025M', '{"3": "CO734020025M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO734020050M', 'CO734020050M', '{"3": "CO734020050M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO734020100M', 'CO734020100M', '{"3": "CO734020100M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO734020500U', 'CO734020500U', '{"3": "CO734020500U", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO738070001M', 'CO738070001M', '{"3": "CO738070001M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO738070005M', 'CO738070005M', '{"3": "CO738070005M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO738070010M', 'CO738070010M', '{"3": "CO738070010M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO738070025M', 'CO738070025M', '{"3": "CO738070025M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO738070050M', 'CO738070050M', '{"3": "CO738070050M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO738070100M', 'CO738070100M', '{"3": "CO738070100M", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO738070500U', 'CO738070500U', '{"3": "CO738070500U", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CP17.10', 'CP17.10', '{"3": "CP17.10", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "1160000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CP17.111', 'CP17.111', '{"3": "CP17.111", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "2600000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CP17.140', 'CP17.140', '{"3": "CP17.140", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "39200000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CP17.20', 'CP17.20', '{"3": "CP17.20", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "636000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CP17.200', 'CP17.200', '{"3": "CP17.200", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "6700000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CP17.311', 'CP17.311', '{"3": "CP17.311", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "6550000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CP17.40', 'CP17.40', '{"3": "CP17.40", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "2750000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CP17.412', 'CP17.412', '{"3": "CP17.412", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "470000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CP17.414', 'CP17.414', '{"3": "CP17.414", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "4740000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CP17.512', 'CP17.512', '{"3": "CP17.512", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "6730000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CP17.711', 'CP17.711', '{"3": "CP17.711", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "4880000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CP17.712', 'CP17.712', '{"3": "CP17.712", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "1270000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CP17.90', 'CP17.90', '{"3": "CP17.90", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "3240000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7', 'CS7', '{"3": "CS7", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730000001M', 'CS730000001M', '{"3": "CS730000001M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730000005U', 'CS730000005U', '{"3": "CS730000005U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "185000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730000010M', 'CS730000010M', '{"3": "CS730000010M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730000050M', 'CS730000050M', '{"3": "CS730000050M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730000100M', 'CS730000100M', '{"3": "CS730000100M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730000100U', 'CS730000100U', '{"3": "CS730000100U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730000250M', 'CS730000250M', '{"3": "CS730000250M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "9250000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730110001M', 'CS730110001M', '{"3": "CS730110001M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730110005U', 'CS730110005U', '{"3": "CS730110005U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "185000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730110010M', 'CS730110010M', '{"3": "CS730110010M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730110050M', 'CS730110050M', '{"3": "CS730110050M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730110100M', 'CS730110100M', '{"3": "CS730110100M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730110100U', 'CS730110100U', '{"3": "CS730110100U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730110250M', 'CS730110250M', '{"3": "CS730110250M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "9250000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730150001M', 'CS730150001M', '{"3": "CS730150001M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730150005U', 'CS730150005U', '{"3": "CS730150005U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "185000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730150010M', 'CS730150010M', '{"3": "CS730150010M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730150050M', 'CS730150050M', '{"3": "CS730150050M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730150100M', 'CS730150100M', '{"3": "CS730150100M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CTC.2', 'CTC.2', '{"3": "CTC.2", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "3000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730150100U', 'CS730150100U', '{"3": "CS730150100U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730150250M', 'CS730150250M', '{"3": "CS730150250M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "9250000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730240001M', 'CS730240001M', '{"3": "CS730240001M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730240005U', 'CS730240005U', '{"3": "CS730240005U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "185000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730240010M', 'CS730240010M', '{"3": "CS730240010M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730240050M', 'CS730240050M', '{"3": "CS730240050M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730240050N', 'CS730240050N', '{"3": "CS730240050N", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1850.0", "9": "0", "10": "", "11": "0", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730240100M', 'CS730240100M', '{"3": "CS730240100M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730240100U', 'CS730240100U', '{"3": "CS730240100U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730240500M', 'CS730240500M', '{"3": "CS730240500M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730250001M', 'CS730250001M', '{"3": "CS730250001M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730250005U', 'CS730250005U', '{"3": "CS730250005U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "185000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730250010M', 'CS730250010M', '{"3": "CS730250010M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730250050M', 'CS730250050M', '{"3": "CS730250050M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730250050N', 'CS730250050N', '{"3": "CS730250050N", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1850.0", "9": "0", "10": "", "11": "0", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730250100M', 'CS730250100M', '{"3": "CS730250100M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS730250100U', 'CS730250100U', '{"3": "CS730250100U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS732010001M', 'CS732010001M', '{"3": "CS732010001M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS732010005U', 'CS732010005U', '{"3": "CS732010005U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "185000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS732010010M', 'CS732010010M', '{"3": "CS732010010M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS732010050M', 'CS732010050M', '{"3": "CS732010050M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS732010050N', 'CS732010050N', '{"3": "CS732010050N", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1850.0", "9": "0", "10": "", "11": "0", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS732010100M', 'CS732010100M', '{"3": "CS732010100M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS732010100U', 'CS732010100U', '{"3": "CS732010100U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS732240001M', 'CS732240001M', '{"3": "CS732240001M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS732240001M', 'CS732240001M', '{"3": "CS732240001M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS732240005M', 'CS732240005M', '{"3": "CS732240005M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS732240005U', 'CS732240005U', '{"3": "CS732240005U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "185000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS732240050U', 'CS732240050U', '{"3": "CS732240050U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1850000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS732240100U', 'CS732240100U', '{"3": "CS732240100U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS73224010U', 'CS73224010U', '{"3": "CS73224010U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "370000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS732240500U', 'CS732240500U', '{"3": "CS732240500U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS734020005M', 'CS734020005M', '{"3": "CS734020005M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS734020010M', 'CS734020010M', '{"3": "CS734020010M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS734020050M', 'CS734020050M', '{"3": "CS734020050M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS734020100M', 'CS734020100M', '{"3": "CS734020100M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS734020100U', 'CS734020100U', '{"3": "CS734020100U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS734020500U', 'CS734020500U', '{"3": "CS734020500U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7A38880001M', 'CS7A38880001M', '{"3": "CS7A38880001M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7A38880001U', 'CS7A38880001U', '{"3": "CS7A38880001U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7A38880010U', 'CS7A38880010U', '{"3": "CS7A38880010U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "370000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7A38880084N', 'CS7A38880084N', '{"3": "CS7A38880084N", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3110.0", "9": "0", "10": "", "11": "0", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7A38880100N', 'CS7A38880100N', '{"3": "CS7A38880100N", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700.0", "9": "0", "10": "", "11": "0", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7A38880100U', 'CS7A38880100U', '{"3": "CS7A38880100U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7A39160001M', 'CS7A39160001M', '{"3": "CS7A39160001M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7A39160001U', 'CS7A39160001U', '{"3": "CS7A39160001U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7A39160010U', 'CS7A39160010U', '{"3": "CS7A39160010U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "370000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7A39160084N', 'CS7A39160084N', '{"3": "CS7A39160084N", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3110.0", "9": "0", "10": "", "11": "0", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7A39160100N', 'CS7A39160100N', '{"3": "CS7A39160100N", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700.0", "9": "0", "10": "", "11": "0", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7A39160100U', 'CS7A39160100U', '{"3": "CS7A39160100U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P010001M', 'CS7P010001M', '{"3": "CS7P010001M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P010005M', 'CS7P010005M', '{"3": "CS7P010005M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P010500U', 'CS7P010500U', '{"3": "CS7P010500U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P020001M', 'CS7P020001M', '{"3": "CS7P020001M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P020005M', 'CS7P020005M', '{"3": "CS7P020005M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P020010M', 'CS7P020010M', '{"3": "CS7P020010M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P020050M', 'CS7P020050M', '{"3": "CS7P020050M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P020100M', 'CS7P020100M', '{"3": "CS7P020100M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P020250M', 'CS7P020250M', '{"3": "CS7P020250M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "9250000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P020500U', 'CS7P020500U', '{"3": "CS7P020500U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P030001M', 'CS7P030001M', '{"3": "CS7P030001M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P030005M', 'CS7P030005M', '{"3": "CS7P030005M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P030010M', 'CS7P030010M', '{"3": "CS7P030010M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P030050M', 'CS7P030050M', '{"3": "CS7P030050M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P030100M', 'CS7P030100M', '{"3": "CS7P030100M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P030250M', 'CS7P030250M', '{"3": "CS7P030250M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "9250000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P030500M', 'CS7P030500M', '{"3": "CS7P030500M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P030500U', 'CS7P030500U', '{"3": "CS7P030500U", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P040001M', 'CS7P040001M', '{"3": "CS7P040001M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P040010M', 'CS7P040010M', '{"3": "CS7P040010M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P040100M', 'CS7P040100M', '{"3": "CS7P040100M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P040500M', 'CS7P040500M', '{"3": "CS7P040500M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P040750M', 'CS7P040750M', '{"3": "CS7P040750M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "27800000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P041000M', 'CS7P041000M', '{"3": "CS7P041000M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P041500M', 'CS7P041500M', '{"3": "CS7P041500M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "55500000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P0500003C', 'CS7P0500003C', '{"3": "CS7P0500003C", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "111000000000.0", "9": "0", "10": "", "11": "3", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P050001C', 'CS7P050001C', '{"3": "CS7P050001C", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P050002C', 'CS7P050002C', '{"3": "CS7P050002C", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "74000000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P050500M', 'CS7P050500M', '{"3": "CS7P050500M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P130001M', 'CS7P130001M', '{"3": "CS7P130001M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P130002M', 'CS7P130002M', '{"3": "CS7P130002M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "74000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CTC.3', 'CTC.3', '{"3": "CTC.3", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "111000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P130005M', 'CS7P130005M', '{"3": "CS7P130005M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P130010M', 'CS7P130010M', '{"3": "CS7P130010M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P130050M', 'CS7P130050M', '{"3": "CS7P130050M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P130100M', 'CS7P130100M', '{"3": "CS7P130100M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P170001C', 'CS7P170001C', '{"3": "CS7P170001C", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P170001M', 'CS7P170001M', '{"3": "CS7P170001M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P170002C', 'CS7P170002C', '{"3": "CS7P170002C", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "74000000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P170010M', 'CS7P170010M', '{"3": "CS7P170010M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P170100M', 'CS7P170100M', '{"3": "CS7P170100M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P170500M', 'CS7P170500M', '{"3": "CS7P170500M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P170750M', 'CS7P170750M', '{"3": "CS7P170750M", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "27800000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P17-10001C', 'CS7P17-10001C', '{"3": "CS7P17-10001C", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P17-10002C', 'CS7P17-10002C', '{"3": "CS7P17-10002C", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "74000000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P17-10004C', 'CS7P17-10004C', '{"3": "CS7P17-10004C", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "148000000000.0", "9": "0", "10": "", "11": "3", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P17-10006C', 'CS7P17-10006C', '{"3": "CS7P17-10006C", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "222000000000.0", "9": "0", "10": "", "11": "3", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P17-10008C', 'CS7P17-10008C', '{"3": "CS7P17-10008C", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "296000000000.0", "9": "0", "10": "", "11": "3", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P17-10010C', 'CS7P17-10010C', '{"3": "CS7P17-10010C", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "3", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P200001C', 'CS7P200001C', '{"3": "CS7P200001C", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P200002C', 'CS7P200002C', '{"3": "CS7P200002C", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "74000000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P200004C', 'CS7P200004C', '{"3": "CS7P200004C", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "148000000000.0", "9": "0", "10": "", "11": "3", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P200006C', 'CS7P200006C', '{"3": "CS7P200006C", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "222000000000.0", "9": "0", "10": "", "11": "3", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P200008C', 'CS7P200008C', '{"3": "CS7P200008C", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "296000000000.0", "9": "0", "10": "", "11": "3", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P200010C', 'CS7P200010C', '{"3": "CS7P200010C", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "3", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CTC', 'CTC', '{"3": "CTC", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CTC.10022', 'CTC.10022', '{"3": "CTC.10022", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CTC.10033', 'CTC.10033', '{"3": "CTC.10033", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "111000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CTC.10122', 'CTC.10122', '{"3": "CTC.10122", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CTC.10123', 'CTC.10123', '{"3": "CTC.10123", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "111000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CTC.4', 'CTC.4', '{"3": "CTC.4", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CUC', 'CUC', '{"3": "CUC", "4": "кадмий-109", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CUC.8733', 'CUC.8733', '{"3": "CUC.8733", "4": "кадмий-109", "5": "1", "6": "", "7": "", "8": "111000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CUC.8734', 'CUC.8734', '{"3": "CUC.8734", "4": "кадмий-109", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CUC.8735', 'CUC.8735', '{"3": "CUC.8735", "4": "кадмий-109", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CUC.8736', 'CUC.8736', '{"3": "CUC.8736", "4": "кадмий-109", "5": "1", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CUCK', 'CUCK', '{"3": "CUCK", "4": "кадмий-109", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CUCK.7901', 'CUCK.7901', '{"3": "CUCK.7901", "4": "кадмий-109", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CUCK.7903', 'CUCK.7903', '{"3": "CUCK.7903", "4": "кадмий-109", "5": "1", "6": "", "7": "", "8": "111000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CUCK.7910', 'CUCK.7910', '{"3": "CUCK.7910", "4": "кадмий-109", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CUCK.7920', 'CUCK.7920', '{"3": "CUCK.7920", "4": "кадмий-109", "5": "1", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CVN', 'CVN', '{"3": "CVN", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CVN.1', 'CVN.1', '{"3": "CVN.1", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "2000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CVN.10', 'CVN.10', '{"3": "CVN.10", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "1000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CVN.101', 'CVN.101', '{"3": "CVN.101", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "200000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CVN.11', 'CVN.11', '{"3": "CVN.11", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "2000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CVN.12', 'CVN.12', '{"3": "CVN.12", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "4000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CVN.2', 'CVN.2', '{"3": "CVN.2", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "10000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CVN.3', 'CVN.3', '{"3": "CVN.3", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "20000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CVN.330', 'CVN.330', '{"3": "CVN.330", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "10000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CVN.331', 'CVN.331', '{"3": "CVN.331", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "20000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CVN.352', 'CVN.352', '{"3": "CVN.352", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "40000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CVN.353', 'CVN.353', '{"3": "CVN.353", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "60000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CVN.4', 'CVN.4', '{"3": "CVN.4", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "40000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CVN.5', 'CVN.5', '{"3": "CVN.5", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "100000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CVN.6', 'CVN.6', '{"3": "CVN.6", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "200000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CVN.7', 'CVN.7', '{"3": "CVN.7", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "400000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ER', 'ER', '{"3": "ER", "4": "радий-226", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ER-14', 'ER-14', '{"3": "ER-14", "4": "радий-226", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU2', 'EU2', '{"3": "EU2", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230000001M', 'EU230000001M', '{"3": "EU230000001M", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230000005M', 'EU230000005M', '{"3": "EU230000005M", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230000010M', 'EU230000010M', '{"3": "EU230000010M", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230000020M', 'EU230000020M', '{"3": "EU230000020M", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230000100U', 'EU230000100U', '{"3": "EU230000100U", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230000250U', 'EU230000250U', '{"3": "EU230000250U", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "9250000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230000500U', 'EU230000500U', '{"3": "EU230000500U", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230110001M', 'EU230110001M', '{"3": "EU230110001M", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230110005M', 'EU230110005M', '{"3": "EU230110005M", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230110010M', 'EU230110010M', '{"3": "EU230110010M", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230110020M', 'EU230110020M', '{"3": "EU230110020M", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230110100U', 'EU230110100U', '{"3": "EU230110100U", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230110250U', 'EU230110250U', '{"3": "EU230110250U", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "9250000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230110500U', 'EU230110500U', '{"3": "EU230110500U", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230150001M', 'EU230150001M', '{"3": "EU230150001M", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230150005M', 'EU230150005M', '{"3": "EU230150005M", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230150010M', 'EU230150010M', '{"3": "EU230150010M", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230150020M', 'EU230150020M', '{"3": "EU230150020M", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230150100U', 'EU230150100U', '{"3": "EU230150100U", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230150250U', 'EU230150250U', '{"3": "EU230150250U", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "9250000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230150500U', 'EU230150500U', '{"3": "EU230150500U", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230240001M', 'EU230240001M', '{"3": "EU230240001M", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230240005M', 'EU230240005M', '{"3": "EU230240005M", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230240010M', 'EU230240010M', '{"3": "EU230240010M", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230240020M', 'EU230240020M', '{"3": "EU230240020M", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230240100U', 'EU230240100U', '{"3": "EU230240100U", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230240250U', 'EU230240250U', '{"3": "EU230240250U", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "9250000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU230240500U', 'EU230240500U', '{"3": "EU230240500U", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU232010001M', 'EU232010001M', '{"3": "EU232010001M", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU232010005M', 'EU232010005M', '{"3": "EU232010005M", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU232010010M', 'EU232010010M', '{"3": "EU232010010M", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU232010100U', 'EU232010100U', '{"3": "EU232010100U", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU232010250U', 'EU232010250U', '{"3": "EU232010250U", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "9250000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU232010500U', 'EU232010500U', '{"3": "EU232010500U", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU234020001M', 'EU234020001M', '{"3": "EU234020001M", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU234020005M', 'EU234020005M', '{"3": "EU234020005M", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU234020010M', 'EU234020010M', '{"3": "EU234020010M", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU234020020M', 'EU234020020M', '{"3": "EU234020020M", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU234020100U', 'EU234020100U', '{"3": "EU234020100U", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU234020250U', 'EU234020250U', '{"3": "EU234020250U", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "9250000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU234020500U', 'EU234020500U', '{"3": "EU234020500U", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'F-220', 'F-220', '{"3": "F-220", "4": "уран-238", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE5', 'FE5', '{"3": "FE5", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532040001M', 'FE532040001M', '{"3": "FE532040001M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532040005M', 'FE532040005M', '{"3": "FE532040005M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532040010M', 'FE532040010M', '{"3": "FE532040010M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532040030M', 'FE532040030M', '{"3": "FE532040030M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532040040M', 'FE532040040M', '{"3": "FE532040040M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "1480000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532040050M', 'FE532040050M', '{"3": "FE532040050M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532040100M', 'FE532040100M', '{"3": "FE532040100M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532050001M', 'FE532050001M', '{"3": "FE532050001M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532050005M', 'FE532050005M', '{"3": "FE532050005M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532050010M', 'FE532050010M', '{"3": "FE532050010M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532050030M', 'FE532050030M', '{"3": "FE532050030M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532050040M', 'FE532050040M', '{"3": "FE532050040M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "1480000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532050050M', 'FE532050050M', '{"3": "FE532050050M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532050100M', 'FE532050100M', '{"3": "FE532050100M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532150001M', 'FE532150001M', '{"3": "FE532150001M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532150005M', 'FE532150005M', '{"3": "FE532150005M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532150010M', 'FE532150010M', '{"3": "FE532150010M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532150030M', 'FE532150030M', '{"3": "FE532150030M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532150040M', 'FE532150040M', '{"3": "FE532150040M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "1480000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532150050M', 'FE532150050M', '{"3": "FE532150050M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532150100M', 'FE532150100M', '{"3": "FE532150100M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532360001M', 'FE532360001M', '{"3": "FE532360001M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532360005M', 'FE532360005M', '{"3": "FE532360005M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532360010M', 'FE532360010M', '{"3": "FE532360010M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532360030M', 'FE532360030M', '{"3": "FE532360030M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532360040M', 'FE532360040M', '{"3": "FE532360040M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "1480000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532360050M', 'FE532360050M', '{"3": "FE532360050M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FE532360100M', 'FE532360100M', '{"3": "FE532360100M", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1', 'GAm1', '{"3": "GAm1", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.032', 'GAm1.032', '{"3": "GAm1.032", "4": "америций-241", "5": "1", "6": "30x20x3. Акт. часть d2xd1xh :x28x22x1", "7": "1850000000.0", "8": "3700000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Кольцевой", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.033', 'GAm1.033', '{"3": "GAm1.033", "4": "америций-241", "5": "1", "6": "34x24x3. Акт. часть d2xd1xh : 32x26x1", "7": "1850000000.0", "8": "3700000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Кольцевой", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.034', 'GAm1.034', '{"3": "GAm1.034", "4": "америций-241", "5": "1", "6": "26x16x3. Акт. часть d2xd1xh : 24x18x1", "7": "370000000.0", "8": "3700000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Кольцевой", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.041', 'GAm1.041', '{"3": "GAm1.041", "4": "америций-241", "5": "1", "6": "3x5,3. Акт. часть dxh : 1,6x1,5", "7": "37000000.0", "8": "185000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Точечные и дисковые источники, капсула b", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.042', 'GAm1.042', '{"3": "GAm1.042", "4": "америций-241", "5": "1", "6": "4x5,3. Акт. часть dxh : 2,5x1,5", "7": "37000000.0", "8": "555000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Точечные и дисковые источники, капсула b", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.043', 'GAm1.043', '{"3": "GAm1.043", "4": "америций-241", "5": "1", "6": "6x5,3. Акт. часть dxh : 4x1,5", "7": "370000000.0", "8": "1850000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Точечные и дисковые источники, капсула b", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.045', 'GAm1.045', '{"3": "GAm1.045", "4": "америций-241", "5": "1", "6": "4x10. Акт. часть dxh : 3x1,5", "7": "370000000.0", "8": "1850000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Точечные и дисковые источники, капсула b", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.053', 'GAm1.053', '{"3": "GAm1.053", "4": "америций-241", "5": "1", "6": "6x5,3. Акт. часть d : 2,5x1", "7": "185000000.0", "8": "555000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Дисковый источник (капсула а)", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.06', 'GAm1.06', '{"3": "GAm1.06", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.061', 'GAm1.061', '{"3": "GAm1.061", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.061.3', 'GAm1.061.3', '{"3": "GAm1.061.3", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.061.X', 'GAm1.061.X', '{"3": "GAm1.061.X", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.07.1', 'GAm1.07.1', '{"3": "GAm1.07.1", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.08', 'GAm1.08', '{"3": "GAm1.08", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.1', 'GAm1.1', '{"3": "GAm1.1", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.10', 'GAm1.10', '{"3": "GAm1.10", "4": "америций-241", "5": "1", "6": "8x5. Акт. часть d : 4x1", "7": "740000000.0", "8": "3700000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Дисковый источник (капсула а)", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.11', 'GAm1.11', '{"3": "GAm1.11", "4": "америций-241", "5": "1", "6": "10,8x5. Акт. часть d : 7,2x1", "7": "1850000000.0", "8": "3700000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Дисковый источник (капсула а)", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.12', 'GAm1.12', '{"3": "GAm1.12", "4": "америций-241", "5": "1", "6": "15x6. Акт. часть d : 12x1", "7": "3700000000.0", "8": "11100000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Дисковый источник (капсула а)", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.13', 'GAm1.13', '{"3": "GAm1.13", "4": "америций-241", "5": "1", "6": "3x10. Акт. часть dxh : 2x1,5", "7": "37000000.0", "8": "555000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Точечные и дисковые источники, капсула b", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.14', 'GAm1.14', '{"3": "GAm1.14", "4": "америций-241", "5": "1", "6": "7x10. Акт. часть dxh : 5x2", "7": "1850000000.0", "8": "3700000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Точечные и дисковые источники, капсула b", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.14.X', 'GAm1.14.X', '{"3": "GAm1.14.X", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "7400000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.18', 'GAm1.18', '{"3": "GAm1.18", "4": "америций-241", "5": "1", "6": "6x6. Акт. часть dxh : 4x1,5", "7": "370000000.0", "8": "3700000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Точечные и дисковые источники, капсула b", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.2.01', 'GAm1.2.01', '{"3": "GAm1.2.01", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "5600000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.211', 'GAm1.211', '{"3": "GAm1.211", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Дисковый", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.212', 'GAm1.212', '{"3": "GAm1.212", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "111000000.0", "9": "0", "10": "", "11": "5", "12": "Дисковый", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.213', 'GAm1.213', '{"3": "GAm1.213", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Дисковый", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.214', 'GAm1.214', '{"3": "GAm1.214", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "4", "12": "Дисковый", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.311', 'GAm1.311', '{"3": "GAm1.311", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Кольцевой", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.312', 'GAm1.312', '{"3": "GAm1.312", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "4", "12": "Кольцевой", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.313', 'GAm1.313', '{"3": "GAm1.313", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Кольцевой", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.314', 'GAm1.314', '{"3": "GAm1.314", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "4", "12": "Кольцевой", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GAm1.6.S1', 'GAm1.6.S1', '{"3": "GAm1.6.S1", "4": "америций-241", "5": "1", "6": "29x1,2. Акт. часть d : 15", "7": "100000.0", "8": "140000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GBa3', 'GBa3', '{"3": "GBa3", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GBa3.044', 'GBa3.044', '{"3": "GBa3.044", "4": "барий-133", "5": "1", "6": "3x3. Акт. часть d, мм: 2x1,5", "7": "1100000.0", "8": "111000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Точечные и дисковые источники, окно из нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GBa3.045', 'GBa3.045', '{"3": "GBa3.045", "4": "барий-133", "5": "1", "6": "4x10. Акт. часть d, мм: 3x1,5", "7": "37000000.0", "8": "370000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Точечные и дисковые источники, окно из нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GBa3.06', 'GBa3.06', '{"3": "GBa3.06", "4": "барий-133", "5": "1", "6": "8x5. Акт. часть d, мм: 4x1", "7": "37000000.0", "8": "370000000.0", "9": "0", "10": "", "11": "", "12": "Дисковые источники, бериллиевое окно.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GBa3.061', 'GBa3.061', '{"3": "GBa3.061", "4": "барий-133", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "НПО РИ", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GBa3.10', 'GBa3.10', '{"3": "GBa3.10", "4": "барий-133", "5": "1", "6": "8x5. Акт. часть d, мм: 4x1", "7": "37000000.0", "8": "370000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Дисковые источники, окно из нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GBa3.111', 'GBa3.111', '{"3": "GBa3.111", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "", "12": "Точечный", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GBa3.112', 'GBa3.112', '{"3": "GBa3.112", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "", "12": "Точечный", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GBa3.113', 'GBa3.113', '{"3": "GBa3.113", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "", "12": "Точечный", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GBa3.114', 'GBa3.114', '{"3": "GBa3.114", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "111000000.0", "9": "0", "10": "", "11": "", "12": "Точечный", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GBa3.115', 'GBa3.115', '{"3": "GBa3.115", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "", "12": "Точечный", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GBa3.116', 'GBa3.116', '{"3": "GBa3.116", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "", "12": "Дисковый", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GBa3.13', 'GBa3.13', '{"3": "GBa3.13", "4": "барий-133", "5": "1", "6": "3x10. Акт. часть d, мм: 2x1,5", "7": "1100000.0", "8": "111000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Точечные и дисковые источники, окно из нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GBa3.14', 'GBa3.14', '{"3": "GBa3.14", "4": "барий-133", "5": "1", "6": "7x10. Акт. часть d, мм: 5x2,5", "7": "37000000.0", "8": "370000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Точечные и дисковые источники, окно из нержавеющей стали.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo0', 'GCo0', '{"3": "GCo0", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo0.31', 'GCo0.31', '{"3": "GCo0.31", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "100000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo0.311', 'GCo0.311', '{"3": "GCo0.311", "4": "кобальт-60", "5": "1", "6": "30x4,5. Акт. часть d, мм: 9", "7": "", "8": "37000000.0", "9": "0", "10": "НПО РИ", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo0.312', 'GCo0.312', '{"3": "GCo0.312", "4": "кобальт-60", "5": "1", "6": "30x4,5. Акт. часть d, мм: ", "7": "", "8": "18500000.0", "9": "0", "10": "НПО РИ", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo0.54', 'GCo0.54', '{"3": "GCo0.54", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "20000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo60', 'GCo60', '{"3": "GCo60", "4": "кобальт-60", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ЛАЭС", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo60B', 'GCo60B', '{"3": "GCo60B", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "1410000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo60B.2530', 'GCo60B.2530', '{"3": "GCo60B.2530", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "93600000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo60B.2900', 'GCo60B.2900', '{"3": "GCo60B.2900", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "10.7", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo60B.3050', 'GCo60B.3050', '{"3": "GCo60B.3050", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "113000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo60B.3140', 'GCo60B.3140', '{"3": "GCo60B.3140", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "116000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo60B.3330', 'GCo60B.3330', '{"3": "GCo60B.3330", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "1230000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7', 'GCo7', '{"3": "GCo7", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.032', 'GCo7.032', '{"3": "GCo7.032", "4": "кобальт-57", "5": "1", "6": "30x20x3. Акт. часть d2xd1xh, мм: 28x22x1", "7": "111000000.0", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Кольцевой.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.033', 'GCo7.033', '{"3": "GCo7.033", "4": "кобальт-57", "5": "1", "6": "34x24x3. Акт. часть d2xd1xh, мм: 32x26x1", "7": "1850000000.0", "8": "2590000000.0", "9": "0", "10": "", "11": "5", "12": "Кольцевой.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.041', 'GCo7.041', '{"3": "GCo7.041", "4": "кобальт-57", "5": "1", "6": "3x5,3. Акт. часть dxh, мм: 1,6x1,5", "7": "37000000.0", "8": "1850000000.0", "9": "0", "10": "Ритверц", "11": "5", "12": "Точечные и дисковые источники, капсула а.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.042', 'GCo7.042', '{"3": "GCo7.042", "4": "кобальт-57", "5": "1", "6": "4x5,3. Акт. часть dxh, мм: 2,5x1,5", "7": "111000000.0", "8": "1850000000.0", "9": "0", "10": "Ритверц", "11": "5", "12": "Точечные и дисковые источники, капсула а.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.044', 'GCo7.044', '{"3": "GCo7.044", "4": "кобальт-57", "5": "1", "6": "3x3. Акт. часть dxh, мм: 2x1,5", "7": "37000000.0", "8": "1850000000.0", "9": "0", "10": "Ритверц", "11": "5", "12": "Точечные и дисковые источники, капсула а.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.045', 'GCo7.045', '{"3": "GCo7.045", "4": "кобальт-57", "5": "1", "6": "4x10. Акт. часть dxh, мм: 3x1,5", "7": "1850000000.0", "8": "2590000000.0", "9": "0", "10": "Ритверц", "11": "5", "12": "Точечные и дисковые источники, капсула а. Старое обозначение: GCo7.13 для GCo7.045.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.061', 'GCo7.061', '{"3": "GCo7.061", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.07', 'GCo7.07', '{"3": "GCo7.07", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.10', 'GCo7.10', '{"3": "GCo7.10", "4": "кобальт-57", "5": "1", "6": "8x5. Акт. часть dxh, мм: 4x1", "7": "111000000.0", "8": "1850000000.0", "9": "0", "10": "Ритверц", "11": "5", "12": "Дисковый источник (капсула b).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.11', 'GCo7.11', '{"3": "GCo7.11", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "555000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.111', 'GCo7.111', '{"3": "GCo7.111", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "Точечный", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.112', 'GCo7.112', '{"3": "GCo7.112", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "111000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "Точечный", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.113', 'GCo7.113', '{"3": "GCo7.113", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "Точечный", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.114', 'GCo7.114', '{"3": "GCo7.114", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "555000000.0", "9": "0", "10": "Ритверц", "11": "5", "12": "Точечный", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.115', 'GCo7.115', '{"3": "GCo7.115", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.12', 'GCo7.12', '{"3": "GCo7.12", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.123', 'GCo7.123', '{"3": "GCo7.123", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.13', 'GCo7.13', '{"3": "GCo7.13", "4": "кобальт-57", "5": "1", "6": "3x10. Акт. часть dxh, мм: 2x1,5", "7": "37000000.0", "8": "1850000000.0", "9": "0", "10": "Ритверц", "11": "5", "12": "Точечные и дисковые источники, капсула а. Старое обозначение: GCo7.12 для GCo7.13.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.14', 'GCo7.14', '{"3": "GCo7.14", "4": "кобальт-57", "5": "1", "6": "7x10. Акт. часть dxh, мм: 5x2,5", "7": "1850000000.0", "8": "3700000000.0", "9": "0", "10": "Ритверц", "11": "5", "12": "Точечные и дисковые источники, капсула а. Старое обозначение: GCo7.22 для GCo7.14.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.15', 'GCo7.15', '{"3": "GCo7.15", "4": "кобальт-57", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "Ритверц", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.17', 'GCo7.17', '{"3": "GCo7.17", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.21', 'GCo7.21', '{"3": "GCo7.21", "4": "кобальт-57", "5": "1", "6": "12x3. Акт. часть dxh, мм: 8,5x1", "7": "370000000.0", "8": "3700000000.0", "9": "0", "10": "Ритверц", "11": "5", "12": "Точечные и дисковые источники, капсула а.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.211', 'GCo7.211', '{"3": "GCo7.211", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "Дисковый", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.212', 'GCo7.212', '{"3": "GCo7.212", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "Дисковый", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.213', 'GCo7.213', '{"3": "GCo7.213", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "1480000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "Дисковый", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.214', 'GCo7.214', '{"3": "GCo7.214", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "Дисковый", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.23', 'GCo7.23', '{"3": "GCo7.23", "4": "кобальт-57", "5": "1", "6": "10x5. Акт. часть dxh, мм: 8,5x1", "7": "370000000.0", "8": "3700000000.0", "9": "0", "10": "Ритверц", "11": "5", "12": "Точечные и дисковые источники, капсула а.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.24', 'GCo7.24', '{"3": "GCo7.24", "4": "кобальт-57", "5": "1", "6": "12x3. Акт. часть dxh, мм: 10x1", "7": "370000000.0", "8": "3700000000.0", "9": "0", "10": "Ритверц", "11": "5", "12": "Точечные и дисковые источники, капсула а.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.32', 'GCo7.32', '{"3": "GCo7.32", "4": "кобальт-57", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "Ритверц", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.33', 'GCo7.33', '{"3": "GCo7.33", "4": "кобальт-57", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "Ритверц", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.403', 'GCo7.403', '{"3": "GCo7.403", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.404', 'GCo7.404', '{"3": "GCo7.404", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7-карандашный (без названия)', 'GCo7-карандашный (без названия)', '{"3": "GCo7-карандашный (без названия)", "4": "кобальт-57", "5": "1", "6": "8x140. Акт. часть d, мм: 1", "7": "370000.0", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Карандашные маркерные источники.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCs7', 'GCs7', '{"3": "GCs7", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCs7.12', 'GCs7.12', '{"3": "GCs7.12", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "9000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCs7.43', 'GCs7.43', '{"3": "GCs7.43", "4": "цезий-137", "5": "1", "6": "3x5,5. Акт. часть dxh : 1,8x3,3", "7": "46300000.0", "8": "925000000.0", "9": "0", "10": "Ритверц", "11": "5", "12": "Дисковый источник (капсула а)", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCs7.44', 'GCs7.44', '{"3": "GCs7.44", "4": "цезий-137", "5": "1", "6": "6x10. Акт. часть dxh : 3x3", "7": "46300000.0", "8": "925000000.0", "9": "0", "10": "Ритверц", "11": "5", "12": "Дисковый источник (капсула b)", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCs7.45', 'GCs7.45', '{"3": "GCs7.45", "4": "цезий-137", "5": "1", "6": "6x10. Акт. часть dxh : 4,5x6", "7": "46300000.0", "8": "925000000.0", "9": "0", "10": "Ритверц", "11": "5", "12": "Дисковый источник (капсула а)", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCs7.46', 'GCs7.46', '{"3": "GCs7.46", "4": "цезий-137", "5": "1", "6": "6x9,5. Акт. часть dxh : 3x4", "7": "37000.0", "8": "11100000.0", "9": "0", "10": "Ритверц", "11": "5", "12": "Дисковый источник (капсула c)", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCs7.47', 'GCs7.47', '{"3": "GCs7.47", "4": "цезий-137", "5": "1", "6": "8x11,5. Акт. часть dxh : 4,98x6", "7": "", "8": "111000000.0", "9": "0", "10": "Ритверц", "11": "5", "12": "Дисковый источник (капсула c)", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCs7.471', 'GCs7.471', '{"3": "GCs7.471", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "130000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GD3', 'GD3', '{"3": "GD3", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GD332010005M', 'GD332010005M', '{"3": "GD332010005M", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GD332010010M', 'GD332010010M', '{"3": "GD332010010M", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GD332010020M', 'GD332010020M', '{"3": "GD332010020M", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GD332010050M', 'GD332010050M', '{"3": "GD332010050M", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GD332010100M', 'GD332010100M', '{"3": "GD332010100M", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GD334020005M', 'GD334020005M', '{"3": "GD334020005M", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GD334020010M', 'GD334020010M', '{"3": "GD334020010M", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GD334020020M', 'GD334020020M', '{"3": "GD334020020M", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GD334020050M', 'GD334020050M', '{"3": "GD334020050M", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GD334020100M', 'GD334020100M', '{"3": "GD334020100M", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GDCK7530', 'GDCK7530', '{"3": "GDCK7530", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GGLS-DA', 'GGLS-DA', '{"3": "GGLS-DA", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "65900000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GK60M21', 'GK60M21', '{"3": "GK60M21", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "86300000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GK60T03', 'GK60T03', '{"3": "GK60T03", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "401000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GM-212.13.010', 'GM-212.13.010', '{"3": "GM-212.13.010", "4": "иридий-192", "5": "1", "6": "", "7": "", "8": "450000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GNa2.x', 'GNa2.x', '{"3": "GNa2.x", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "1400000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GOG', 'GOG', '{"3": "GOG", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GOG-101', 'GOG-101', '{"3": "GOG-101", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "296000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GOG-102', 'GOG-102', '{"3": "GOG-102", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "64900000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GOG-103', 'GOG-103', '{"3": "GOG-103", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "800000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GOG-104', 'GOG-104', '{"3": "GOG-104", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "1110000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GOG-111', 'GOG-111', '{"3": "GOG-111", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "2210000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GOG-112', 'GOG-112', '{"3": "GOG-112", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3940000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GOG-131', 'GOG-131', '{"3": "GOG-131", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "17600000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GOG-132', 'GOG-132', '{"3": "GOG-132", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "19500000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GPu8-C3', 'GPu8-C3', '{"3": "GPu8-C3", "4": "плутоний-238; углерод-13", "5": "1", "6": "9x20. Акт. часть dxh : 5x14,7", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GRa6.1', 'GRa6.1', '{"3": "GRa6.1", "4": "радий-226", "5": "1", "6": "12x70; 13,5x70", "7": "37000.0", "8": "370000000.0", "9": "0", "10": "ЗАО РИТВЕРЦ", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GRa6.1.2P2', 'GRa6.1.2P2', '{"3": "GRa6.1.2P2", "4": "радий-226", "5": "1", "6": "", "7": "", "8": "42200000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GRa6.1.P1', 'GRa6.1.P1', '{"3": "GRa6.1.P1", "4": "радий-226", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GRa6.1.P2', 'GRa6.1.P2', '{"3": "GRa6.1.P2", "4": "радий-226", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GSR-J', 'GSR-J', '{"3": "GSR-J", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "65900000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GSR-Z', 'GSR-Z', '{"3": "GSR-Z", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "592000000000.0", "9": "0", "10": "ФРГ,AEA Technology QSA GmbH", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GU5.P02.1206', 'GU5.P02.1206', '{"3": "GU5.P02.1206", "4": "уран-235", "5": "1", "6": "40x44. Акт. часть dxh : 39x39", "7": "", "8": "1200000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GZn5', 'GZn5', '{"3": "GZn5", "4": "цинк-65", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GZn5.151', 'GZn5.151', '{"3": "GZn5.151", "4": "цинк-65", "5": "1", "6": "5x3. Акт. часть d, мм: 4x1", "7": "185000.0", "8": "370000.0", "9": "0", "10": "", "11": "0", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GZn5.152', 'GZn5.152', '{"3": "GZn5.152", "4": "цинк-65", "5": "1", "6": "10x4. Акт. часть d, мм: 4x1", "7": "185000.0", "8": "370000.0", "9": "0", "10": "", "11": "0", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'H-1-37', 'H-1-37', '{"3": "H-1-37", "4": "радий-226", "5": "3", "6": "", "7": "0.0", "8": "368000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'H-3', 'H-3', '{"3": "H-3", "4": "тритий", "5": "2", "6": "", "7": "", "8": "2460000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'I', 'I', '{"3": "I", "4": "кобальт-57", "5": "1", "6": "12,0х2,4", "7": "0.0", "8": "185000000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'I', 'I', '{"3": "I", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "2480000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'I', 'I', '{"3": "I", "4": "полоний-210", "5": "4", "6": "", "7": "", "8": "33500000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'I', 'I', '{"3": "I", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "6100000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'I ЯГР-ИГИК', 'I ЯГР-ИГИК', '{"3": "I ЯГР-ИГИК", "4": "кобальт-57", "5": "6", "6": "10,0х12,0", "7": "", "8": "4000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IEC', 'IEC', '{"3": "IEC", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IEC.121', 'IEC.121', '{"3": "IEC.121", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IEC.122', 'IEC.122', '{"3": "IEC.122", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IEC.123', 'IEC.123', '{"3": "IEC.123", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IEC.125', 'IEC.125', '{"3": "IEC.125", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IEC.126', 'IEC.126', '{"3": "IEC.126", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IEC.1331', 'IEC.1331', '{"3": "IEC.1331", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IEC.1332', 'IEC.1332', '{"3": "IEC.1332", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IEC.1333', 'IEC.1333', '{"3": "IEC.1333", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IEC.1335', 'IEC.1335', '{"3": "IEC.1335", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IEC.1336', 'IEC.1336', '{"3": "IEC.1336", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IEC.1337', 'IEC.1337', '{"3": "IEC.1337", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IEC.3301', 'IEC.3301', '{"3": "IEC.3301", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IEC.3302', 'IEC.3302', '{"3": "IEC.3302", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IEC.3303', 'IEC.3303', '{"3": "IEC.3303", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IEC.3305', 'IEC.3305', '{"3": "IEC.3305", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IEC.8753', 'IEC.8753', '{"3": "IEC.8753", "4": "железо-55", "5": "5", "6": "", "7": "37000000.0", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'II', 'II', '{"3": "II", "4": "кадмий-109", "5": "5", "6": "3,0х3,0", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'II', 'II', '{"3": "II", "4": "кобальт-57", "5": "1", "6": "12,0х2,4", "7": "", "8": "100000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'II', 'II', '{"3": "II", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "11500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'II', 'II', '{"3": "II", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "211000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'II ЯГР-ИГИК', 'II ЯГР-ИГИК', '{"3": "II ЯГР-ИГИК", "4": "кобальт-57", "5": "6", "6": "10,0х12,0", "7": "", "8": "4000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'III', 'III', '{"3": "III", "4": "кадмий-109", "5": "5", "6": "3,0х3,0", "7": "", "8": "250000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'III', 'III', '{"3": "III", "4": "кобальт-57", "5": "1", "6": "12,0х2,4", "7": "", "8": "930000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'III', 'III', '{"3": "III", "4": "плутоний-239", "5": "4", "6": "", "7": "", "8": "5000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'III', 'III', '{"3": "III", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1140000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'III ЯГР-ИГИК', 'III ЯГР-ИГИК', '{"3": "III ЯГР-ИГИК", "4": "кобальт-57", "5": "6", "6": "10,0х12,0", "7": "", "8": "4000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'III-РИК-109р', 'III-РИК-109р', '{"3": "III-РИК-109р", "4": "кадмий-109", "5": "5", "6": "5,0х3,0", "7": "", "8": "800000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'II-РИК-109р', 'II-РИК-109р', '{"3": "II-РИК-109р", "4": "кадмий-109", "5": "5", "6": "10,0х5,0", "7": "", "8": "800000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IRIK-D', 'IRIK-D', '{"3": "IRIK-D", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IRIK-D', 'IRIK-D', '{"3": "IRIK-D", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IRIK-D', 'IRIK-D', '{"3": "IRIK-D", "4": "кобальт-57", "5": "5", "6": "", "7": "", "8": "23200000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IV', 'IV', '{"3": "IV", "4": "кадмий-109", "5": "5", "6": "3,0х3,0", "7": "", "8": "1850000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IV', 'IV', '{"3": "IV", "4": "кобальт-57", "5": "1", "6": "12,0х2,4", "7": "", "8": "500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IV', 'IV', '{"3": "IV", "4": "плутоний-239", "5": "4", "6": "", "7": "", "8": "127000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IV', 'IV', '{"3": "IV", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1920000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IV ЯГР-ИГИК', 'IV ЯГР-ИГИК', '{"3": "IV ЯГР-ИГИК", "4": "кобальт-57", "5": "6", "6": "10,0х12,0", "7": "", "8": "4000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'I-РИК-109р', 'I-РИК-109р', '{"3": "I-РИК-109р", "4": "кадмий-109", "5": "5", "6": "10,0х10,0", "7": "", "8": "800000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KAC', 'KAC', '{"3": "KAC", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KAC.10881', 'KAC.10881', '{"3": "KAC.10881", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KAC.10882', 'KAC.10882', '{"3": "KAC.10882", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "7400000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KAC.10883', 'KAC.10883', '{"3": "KAC.10883", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "11100000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KAC.10884', 'KAC.10884', '{"3": "KAC.10884", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "14800000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KAC.11401', 'KAC.11401', '{"3": "KAC.11401", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KAC.11402', 'KAC.11402', '{"3": "KAC.11402", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "7400000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KAC.11403', 'KAC.11403', '{"3": "KAC.11403", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "11100000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KAC.11405', 'KAC.11405', '{"3": "KAC.11405", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KAC.11410', 'KAC.11410', '{"3": "KAC.11410", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KAC.2310', 'KAC.2310', '{"3": "KAC.2310", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "7400000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KAC.D3', 'KAC.D3', '{"3": "KAC.D3", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "14800000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KAC.D5', 'KAC.D5', '{"3": "KAC.D5", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "14800000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KACB', 'KACB', '{"3": "KACB", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KACB 10884', 'KACB 10884', '{"3": "KACB 10884", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "14800000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KACB 11352', 'KACB 11352', '{"3": "KACB 11352", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "3000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KACB 14466', 'KACB 14466', '{"3": "KACB 14466", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "14800000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KACK', 'KACK', '{"3": "KACK", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KACK 2311', 'KACK 2311', '{"3": "KACK 2311", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "14800000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KACK 4243', 'KACK 4243', '{"3": "KACK 4243", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "14800000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KACK 5533', 'KACK 5533', '{"3": "KACK 5533", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "30000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KACK 6581', 'KACK 6581', '{"3": "KACK 6581", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KACK 7987', 'KACK 7987', '{"3": "KACK 7987", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "9900000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KACK 8004', 'KACK 8004', '{"3": "KACK 8004", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "14800000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KACK 8045', 'KACK 8045', '{"3": "KACK 8045", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "14800000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5', 'KR5', '{"3": "KR5", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Kr5.E04', 'Kr5.E04', '{"3": "Kr5.E04", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER584', 'KR5NER584', '{"3": "KR5NER584", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "81400000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER585', 'KR5NER585', '{"3": "KR5NER585", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "111000000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER586', 'KR5NER586', '{"3": "KR5NER586", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "81400000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER5870001C', 'KR5NER5870001C', '{"3": "KR5NER5870001C", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'LAB', 'LAB', '{"3": "LAB", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "153000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER5870002C', 'KR5NER5870002C', '{"3": "KR5NER5870002C", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "74000000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER5870010M', 'KR5NER5870010M', '{"3": "KR5NER5870010M", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER5870050M', 'KR5NER5870050M', '{"3": "KR5NER5870050M", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER5870100M', 'KR5NER5870100M', '{"3": "KR5NER5870100M", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER5870200M', 'KR5NER5870200M', '{"3": "KR5NER5870200M", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "7400000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER5870500M', 'KR5NER5870500M', '{"3": "KR5NER5870500M", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER58800100M', 'KR5NER58800100M', '{"3": "KR5NER58800100M", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER58800500M', 'KR5NER58800500M', '{"3": "KR5NER58800500M", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER58801000M', 'KR5NER58801000M', '{"3": "KR5NER58801000M", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER58802000M', 'KR5NER58802000M', '{"3": "KR5NER58802000M", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "74000000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER5882500M', 'KR5NER5882500M', '{"3": "KR5NER5882500M", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "92500000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER81700010M', 'KR5NER81700010M', '{"3": "KR5NER81700010M", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER81700020M', 'KR5NER81700020M', '{"3": "KR5NER81700020M", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER81700050M', 'KR5NER81700050M', '{"3": "KR5NER81700050M", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER81700100M', 'KR5NER81700100M', '{"3": "KR5NER81700100M", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER81700200M', 'KR5NER81700200M', '{"3": "KR5NER81700200M", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "7400000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER81700500M', 'KR5NER81700500M', '{"3": "KR5NER81700500M", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER81701150M', 'KR5NER81701150M', '{"3": "KR5NER81701150M", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "42600000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER81800010M', 'KR5NER81800010M', '{"3": "KR5NER81800010M", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER81800020M', 'KR5NER81800020M', '{"3": "KR5NER81800020M", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER81800050M', 'KR5NER81800050M', '{"3": "KR5NER81800050M", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER81800100M', 'KR5NER81800100M', '{"3": "KR5NER81800100M", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER81800300M', 'KR5NER81800300M', '{"3": "KR5NER81800300M", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "11100000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER81800700M', 'KR5NER81800700M', '{"3": "KR5NER81800700M", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "25900000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5NER81801500M', 'KR5NER81801500M', '{"3": "KR5NER81801500M", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "55500000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'LAA (SBZ)', 'LAA (SBZ)', '{"3": "LAA (SBZ)", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "344000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'LB 7440', 'LB 7440', '{"3": "LB 7440", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "2630000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'LB 7440B', 'LB 7440B', '{"3": "LB 7440B", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "555000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'LB 7440D', 'LB 7440D', '{"3": "LB 7440D", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'LB 7440-D-CR', 'LB 7440-D-CR', '{"3": "LB 7440-D-CR", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "130000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'LS-921', 'LS-921', '{"3": "LS-921", "4": "германий-68", "5": "1", "6": "", "7": "", "8": "120000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'LS-921/47', 'LS-921/47', '{"3": "LS-921/47", "4": "германий-68", "5": "1", "6": "", "7": "", "8": "122000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'LX-34-01', 'LX-34-01', '{"3": "LX-34-01", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'LX-34-05', 'LX-34-05', '{"3": "LX-34-05", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "84000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'LX-64-04', 'LX-64-04', '{"3": "LX-64-04", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "84000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7', 'MCo7', '{"3": "MCo7", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.07', 'MCo7.07', '{"3": "MCo7.07", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.111', 'MCo7.111', '{"3": "MCo7.111", "4": "кобальт-57", "5": "6", "6": "11,2x13. Акт. часть d : 8", "7": "", "8": "185000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Серийные источники (матрица из родия) корпус 5", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.112', 'MCo7.112', '{"3": "MCo7.112", "4": "кобальт-57", "5": "6", "6": "11,2x13. Акт. часть d : 8", "7": "", "8": "370000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Серийные источники (матрица из родия) корпус 5", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.1121', 'MCo7.1121', '{"3": "MCo7.1121", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "555000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.1122', 'MCo7.1122', '{"3": "MCo7.1122", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.113', 'MCo7.113', '{"3": "MCo7.113", "4": "кобальт-57", "5": "6", "6": "11,2x13. Акт. часть d : 8", "7": "", "8": "925000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Серийные источники (матрица из родия) корпус 5", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.1131', 'MCo7.1131', '{"3": "MCo7.1131", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.1133', 'MCo7.1133', '{"3": "MCo7.1133", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "1480000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.114', 'MCo7.114', '{"3": "MCo7.114", "4": "кобальт-57", "5": "6", "6": "11,2x13. Акт. часть d : 8", "7": "", "8": "1850000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Серийные источники (матрица из родия) корпус 5", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.1141', 'MCo7.1141', '{"3": "MCo7.1141", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "2040000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.1142', 'MCo7.1142', '{"3": "MCo7.1142", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "2960000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.1145', 'MCo7.1145', '{"3": "MCo7.1145", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "2780000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.1146', 'MCo7.1146', '{"3": "MCo7.1146", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "2960000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.115', 'MCo7.115', '{"3": "MCo7.115", "4": "кобальт-57", "5": "6", "6": "11,2x13. Акт. часть d : 8", "7": "", "8": "3700000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Серийные источники (матрица из родия) корпус 5", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.121', 'MCo7.121', '{"3": "MCo7.121", "4": "кобальт-57", "5": "6", "6": "14x4. Акт. часть d : 8", "7": "", "8": "185000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Серийные источники (матрица из родия) корпус 1", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.122', 'MCo7.122', '{"3": "MCo7.122", "4": "кобальт-57", "5": "6", "6": "14x1. Акт. часть d : 8", "7": "", "8": "370000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Серийные источники (матрица из родия) корпус 1", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.1222', 'MCo7.1222', '{"3": "MCo7.1222", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.123', 'MCo7.123', '{"3": "MCo7.123", "4": "кобальт-57", "5": "6", "6": "14x4. Акт. часть d : 8", "7": "", "8": "925000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Серийные источники (матрица из родия) корпус 1", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.1231', 'MCo7.1231', '{"3": "MCo7.1231", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.1233', 'MCo7.1233', '{"3": "MCo7.1233", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "1480000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.1235', 'MCo7.1235', '{"3": "MCo7.1235", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "1670000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.124', 'MCo7.124', '{"3": "MCo7.124", "4": "кобальт-57", "5": "6", "6": "14x4. Акт. часть d : 8", "7": "", "8": "1850000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Серийные источники (матрица из родия) корпус 1", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.1242', 'MCo7.1242', '{"3": "MCo7.1242", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "2220000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.1244', 'MCo7.1244', '{"3": "MCo7.1244", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "2220000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.1245', 'MCo7.1245', '{"3": "MCo7.1245", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "2780000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.1246', 'MCo7.1246', '{"3": "MCo7.1246", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "2960000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.125', 'MCo7.125', '{"3": "MCo7.125", "4": "кобальт-57", "5": "6", "6": "14x4. Акт. часть d : 8", "7": "", "8": "3700000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Серийные источники (матрица из родия) корпус 1", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.1254', 'MCo7.1254', '{"3": "MCo7.1254", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "4440000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.14 ВН', 'MCo7.14 ВН', '{"3": "MCo7.14 ВН", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.16', 'MCo7.16', '{"3": "MCo7.16", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.161', 'MCo7.161', '{"3": "MCo7.161", "4": "кобальт-57", "5": "6", "6": "6x13. Акт. часть d : 4", "7": "", "8": "185000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Серийные источники (матрица из родия) корпус 6", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.162', 'MCo7.162', '{"3": "MCo7.162", "4": "кобальт-57", "5": "6", "6": "6x13. Акт. часть d : 4", "7": "", "8": "370000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Серийные источники (матрица из родия) корпус 6", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.163', 'MCo7.163', '{"3": "MCo7.163", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.1631', 'MCo7.1631', '{"3": "MCo7.1631", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.164', 'MCo7.164', '{"3": "MCo7.164", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.165', 'MCo7.165', '{"3": "MCo7.165", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "4810000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.167', 'MCo7.167', '{"3": "MCo7.167", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "7400000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.17', 'MCo7.17', '{"3": "MCo7.17", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.171', 'MCo7.171', '{"3": "MCo7.171", "4": "кобальт-57", "5": "6", "6": "4x14. Акт. часть d : 1", "7": "", "8": "185000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Точечные источники", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.172', 'MCo7.172', '{"3": "MCo7.172", "4": "кобальт-57", "5": "6", "6": "4x14. Акт. часть d : 1", "7": "", "8": "370000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Точечные источники", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.18', 'MCo7.18', '{"3": "MCo7.18", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "1000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.181', 'MCo7.181', '{"3": "MCo7.181", "4": "кобальт-57", "5": "6", "6": "12x6. Акт. часть d : 10", "7": "", "8": "185000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Двусторонние источники.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.182', 'MCo7.182', '{"3": "MCo7.182", "4": "кобальт-57", "5": "6", "6": "12x6. Акт. часть d : 10", "7": "", "8": "370000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Двусторонние источники.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.183', 'MCo7.183', '{"3": "MCo7.183", "4": "кобальт-57", "5": "6", "6": "12x6. Акт. часть d : 10", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "5", "12": "Двусторонние источники.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.184', 'MCo7.184', '{"3": "MCo7.184", "4": "кобальт-57", "5": "6", "6": "12x6. Акт. часть d : 10", "7": "", "8": "1850000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Двусторонние источники.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.185', 'MCo7.185', '{"3": "MCo7.185", "4": "кобальт-57", "5": "6", "6": "12x6. Акт. часть d : 10", "7": "", "8": "3700000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Двусторонние источники.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.254', 'MCo7.254', '{"3": "MCo7.254", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.511', 'MCo7.511', '{"3": "MCo7.511", "4": "кобальт-57", "5": "6", "6": "11,2x13. Акт. часть d : 8", "7": "", "8": "185000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Серийные источники (матрица из хрома) корпус 5", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.512', 'MCo7.512', '{"3": "MCo7.512", "4": "кобальт-57", "5": "6", "6": "11,2x13. Акт. часть d : 8", "7": "", "8": "370000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Серийные источники (матрица из хрома) корпус 5", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.513', 'MCo7.513', '{"3": "MCo7.513", "4": "кобальт-57", "5": "6", "6": "11,2x13. Акт. часть d : 8", "7": "", "8": "925000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Серийные источники (матрица из хрома) корпус 5", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.514', 'MCo7.514', '{"3": "MCo7.514", "4": "кобальт-57", "5": "6", "6": "11,2x13. Акт. часть d : 8", "7": "", "8": "1850000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Серийные источники (матрица из хрома) корпус 5", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.515', 'MCo7.515', '{"3": "MCo7.515", "4": "кобальт-57", "5": "6", "6": "11,2x13. Акт. часть d : 8", "7": "", "8": "3700000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Серийные источники (матрица из хрома) корпус 5", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.521', 'MCo7.521', '{"3": "MCo7.521", "4": "кобальт-57", "5": "6", "6": "14x4. Акт. часть d : 8", "7": "", "8": "185000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Серийные источники (матрица из хрома) корпус 1", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.522', 'MCo7.522', '{"3": "MCo7.522", "4": "кобальт-57", "5": "6", "6": "14x1. Акт. часть d : 8", "7": "", "8": "370000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Серийные источники (матрица из хрома) корпус 1", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.5221', 'MCo7.5221', '{"3": "MCo7.5221", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "555000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.5222', 'MCo7.5222', '{"3": "MCo7.5222", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "1480000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.523', 'MCo7.523', '{"3": "MCo7.523", "4": "кобальт-57", "5": "6", "6": "14x4. Акт. часть d : 8", "7": "", "8": "925000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Серийные источники (матрица из хрома) корпус 1", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.5233', 'MCo7.5233', '{"3": "MCo7.5233", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.5234', 'MCo7.5234', '{"3": "MCo7.5234", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "1670000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.524', 'MCo7.524', '{"3": "MCo7.524", "4": "кобальт-57", "5": "6", "6": "14x4. Акт. часть d : 8", "7": "", "8": "1850000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Серийные источники (матрица из хрома) корпус 1", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.525', 'MCo7.525', '{"3": "MCo7.525", "4": "кобальт-57", "5": "6", "6": "14x4. Акт. часть d : 8", "7": "", "8": "3700000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Серийные источники (матрица из хрома) корпус 1", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.561', 'MCo7.561', '{"3": "MCo7.561", "4": "кобальт-57", "5": "6", "6": "6x13. Акт. часть d : 4", "7": "", "8": "185000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Серийные источники (матрица из хрома) корпус 6", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.562', 'MCo7.562', '{"3": "MCo7.562", "4": "кобальт-57", "5": "6", "6": "6x13. Акт. часть d : 4", "7": "", "8": "370000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Серийные источники (матрица из хрома) корпус 6", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.564', 'MCo7.564', '{"3": "MCo7.564", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.7', 'MCo7.7', '{"3": "MCo7.7", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "1330000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MNTR-F ', 'MNTR-F ', '{"3": "MNTR-F ", "4": "тритий", "5": "2", "6": "", "7": "", "8": "55500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Model 702', 'Model 702', '{"3": "Model 702", "4": "уран-238", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSCRA-3', 'MSCRA-3', '{"3": "MSCRA-3", "4": "кобальт-57", "5": "2", "6": "", "7": "0.0", "8": "1420000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9', 'MSn9', '{"3": "MSn9", "4": "олово-119м", "5": "6", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.111', 'MSn9.111', '{"3": "MSn9.111", "4": "олово-119м", "5": "6", "6": "14x4. Акт. часть d : 10", "7": "", "8": "74000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из SnO2", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.112', 'MSn9.112', '{"3": "MSn9.112", "4": "олово-119м", "5": "6", "6": "14x4. Акт. часть d : 10", "7": "", "8": "185000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из SnO2", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.113', 'MSn9.113', '{"3": "MSn9.113", "4": "олово-119м", "5": "6", "6": "14x4. Акт. часть d : 10", "7": "", "8": "370000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из SnO2", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.114', 'MSn9.114', '{"3": "MSn9.114", "4": "олово-119м", "5": "6", "6": "14x4. Акт. часть d : 10", "7": "", "8": "555000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из SnO2", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.133', 'MSn9.133', '{"3": "MSn9.133", "4": "олово-119м", "5": "6", "6": "18x14. Акт. часть d : 15", "7": "", "8": "370000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из SnO2", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.134', 'MSn9.134', '{"3": "MSn9.134", "4": "олово-119м", "5": "6", "6": "18x14. Акт. часть d : 15", "7": "", "8": "555000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из SnO2", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.135', 'MSn9.135', '{"3": "MSn9.135", "4": "олово-119м", "5": "6", "6": "18x14. Акт. часть d : 15", "7": "", "8": "740000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из SnO2", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.151', 'MSn9.151', '{"3": "MSn9.151", "4": "олово-119м", "5": "6", "6": "11,2x13. Акт. часть d : 10", "7": "", "8": "74000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из SnO2", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.152', 'MSn9.152', '{"3": "MSn9.152", "4": "олово-119м", "5": "6", "6": "11,2x13. Акт. часть d : 10", "7": "", "8": "185000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из SnO2", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.153', 'MSn9.153', '{"3": "MSn9.153", "4": "олово-119м", "5": "6", "6": "11,2x13. Акт. часть d : 10", "7": "", "8": "370000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из SnO2", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.154', 'MSn9.154', '{"3": "MSn9.154", "4": "олово-119м", "5": "6", "6": "11,2x13. Акт. часть d : 10", "7": "", "8": "555000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из SnO2", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.161', 'MSn9.161', '{"3": "MSn9.161", "4": "олово-119м", "5": "6", "6": "6x13. Акт. часть d : 5", "7": "", "8": "74000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из SnO2", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.162', 'MSn9.162', '{"3": "MSn9.162", "4": "олово-119м", "5": "6", "6": "6x13. Акт. часть d : 5", "7": "", "8": "185000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из SnO2", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.211', 'MSn9.211', '{"3": "MSn9.211", "4": "олово-119м", "5": "6", "6": "14x4. Акт. часть d : 10", "7": "", "8": "74000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из CaSnO3.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.212', 'MSn9.212', '{"3": "MSn9.212", "4": "олово-119м", "5": "6", "6": "14x4. Акт. часть d : 10", "7": "", "8": "185000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из CaSnO3.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.2125', 'MSn9.2125', '{"3": "MSn9.2125", "4": "олово-119м", "5": "6", "6": "", "7": "", "8": "278000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.213', 'MSn9.213', '{"3": "MSn9.213", "4": "олово-119м", "5": "6", "6": "14x4. Акт. часть d : 10", "7": "", "8": "370000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из CaSnO3.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.231', 'MSn9.231', '{"3": "MSn9.231", "4": "олово-119м", "5": "6", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.233', 'MSn9.233', '{"3": "MSn9.233", "4": "олово-119м", "5": "6", "6": "18x14. Акт. часть d : 15", "7": "", "8": "370000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из CaSnO3.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.234', 'MSn9.234', '{"3": "MSn9.234", "4": "олово-119м", "5": "6", "6": "18x14. Акт. часть d : 15", "7": "", "8": "555000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из CaSnO3.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.235', 'MSn9.235', '{"3": "MSn9.235", "4": "олово-119м", "5": "6", "6": "18x14. Акт. часть d : 15", "7": "", "8": "740000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из CaSnO3.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.2355', 'MSn9.2355', '{"3": "MSn9.2355", "4": "олово-119м", "5": "6", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.236', 'MSn9.236', '{"3": "MSn9.236", "4": "олово-119м", "5": "6", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.251', 'MSn9.251', '{"3": "MSn9.251", "4": "олово-119м", "5": "6", "6": "11,2x13. Акт. часть d : 10", "7": "", "8": "74000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из CaSnO3.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.252', 'MSn9.252', '{"3": "MSn9.252", "4": "олово-119м", "5": "6", "6": "11,2x13. Акт. часть d : 10", "7": "", "8": "185000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из CaSnO3.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.253', 'MSn9.253', '{"3": "MSn9.253", "4": "олово-119м", "5": "6", "6": "11,2x13. Акт. часть d : 10", "7": "", "8": "370000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из CaSnO3.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.254', 'MSn9.254', '{"3": "MSn9.254", "4": "олово-119м", "5": "6", "6": "", "7": "", "8": "555000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.261', 'MSn9.261', '{"3": "MSn9.261", "4": "олово-119м", "5": "6", "6": "6x13. Акт. часть d : 5", "7": "", "8": "74000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из CaSnO3.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.262', 'MSn9.262', '{"3": "MSn9.262", "4": "олово-119м", "5": "6", "6": "6x13. Акт. часть d : 5", "7": "", "8": "185000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из CaSnO3.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.311', 'MSn9.311', '{"3": "MSn9.311", "4": "олово-119м", "5": "6", "6": "14x4. Акт. часть d : 10", "7": "", "8": "74000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из BaSnO3.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.312', 'MSn9.312', '{"3": "MSn9.312", "4": "олово-119м", "5": "6", "6": "14x4. Акт. часть d : 10", "7": "", "8": "185000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из BaSnO3.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.333', 'MSn9.333', '{"3": "MSn9.333", "4": "олово-119м", "5": "6", "6": "18x14. Акт. часть d : 15", "7": "", "8": "370000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из BaSnO3.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.351', 'MSn9.351', '{"3": "MSn9.351", "4": "олово-119м", "5": "6", "6": "11,2x13. Акт. часть d : 10", "7": "", "8": "74000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из BaSnO3.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.352', 'MSn9.352', '{"3": "MSn9.352", "4": "олово-119м", "5": "6", "6": "11,2x13. Акт. часть d : 10", "7": "", "8": "185000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из BaSnO3.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.361', 'MSn9.361', '{"3": "MSn9.361", "4": "олово-119м", "5": "6", "6": "6x13. Акт. часть d : 5", "7": "", "8": "74000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из BaSnO3.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.362', 'MSn9.362', '{"3": "MSn9.362", "4": "олово-119м", "5": "6", "6": "6x13. Акт. часть d : 5", "7": "", "8": "185000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Серийные источники. Матрица из BaSnO3.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MTe5', 'MTe5', '{"3": "MTe5", "4": "теллур-125м", "5": "6", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MTe5.11', 'MTe5.11', '{"3": "MTe5.11", "4": "теллур-125м", "5": "6", "6": "14x4. Акт. часть d : 10", "7": "370000000.0", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MTe5.13', 'MTe5.13', '{"3": "MTe5.13", "4": "теллур-125м", "5": "6", "6": "18x14. Акт. часть d : 15", "7": "370000000.0", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MTe5.15', 'MTe5.15', '{"3": "MTe5.15", "4": "теллур-125м", "5": "6", "6": "11,2x13. Акт. часть d : 10", "7": "370000000.0", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MTe5.16', 'MTe5.16', '{"3": "MTe5.16", "4": "теллур-125м", "5": "6", "6": "6x13. Акт. часть d : 5", "7": "370000000.0", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NA230000001M', 'NA230000001M', '{"3": "NA230000001M", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NA230000001U', 'NA230000001U', '{"3": "NA230000001U", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "37000.0", "9": "0", "10": "", "11": "0", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NA230000010M', 'NA230000010M', '{"3": "NA230000010M", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NA230000010U', 'NA230000010U', '{"3": "NA230000010U", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "370000.0", "9": "0", "10": "", "11": "0", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NA230000100U', 'NA230000100U', '{"3": "NA230000100U", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NA230000500U', 'NA230000500U', '{"3": "NA230000500U", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NA230250001M', 'NA230250001M', '{"3": "NA230250001M", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NA230250001U', 'NA230250001U', '{"3": "NA230250001U", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "37000.0", "9": "0", "10": "", "11": "0", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NA230250010M', 'NA230250010M', '{"3": "NA230250010M", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NA230250010U', 'NA230250010U', '{"3": "NA230250010U", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "370000.0", "9": "0", "10": "", "11": "0", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NA230250100U', 'NA230250100U', '{"3": "NA230250100U", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NA230250500U', 'NA230250500U', '{"3": "NA230250500U", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NBC', 'NBC', '{"3": "NBC", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NBC.1', 'NBC.1', '{"3": "NBC.1", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NBC.11', 'NBC.11', '{"3": "NBC.11", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NBC.12', 'NBC.12', '{"3": "NBC.12", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "111000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NBC.13', 'NBC.13', '{"3": "NBC.13", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NBC.14', 'NBC.14', '{"3": "NBC.14", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "555000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NBC.2', 'NBC.2', '{"3": "NBC.2", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "111000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NBC.21', 'NBC.21', '{"3": "NBC.21", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NBC.22', 'NBC.22', '{"3": "NBC.22", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "111000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NBC.23', 'NBC.23', '{"3": "NBC.23", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NBC.24', 'NBC.24', '{"3": "NBC.24", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "555000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NBC.3', 'NBC.3', '{"3": "NBC.3", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NBC.31', 'NBC.31', '{"3": "NBC.31", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NBC.32', 'NBC.32', '{"3": "NBC.32", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "111000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NBC.33', 'NBC.33', '{"3": "NBC.33", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NBC.34', 'NBC.34', '{"3": "NBC.34", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "555000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NBC.4', 'NBC.4', '{"3": "NBC.4", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "555000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NBC.B14746', 'NBC.B14746', '{"3": "NBC.B14746", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "481000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NBCK2111', 'NBCK2111', '{"3": "NBCK2111", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "481000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NCf2.P01.1204', 'NCf2.P01.1204', '{"3": "NCf2.P01.1204", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "НПО РИ", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NCf2.P01.1205', 'NCf2.P01.1205', '{"3": "NCf2.P01.1205", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "НПО РИ", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NCf2.P01.2005', 'NCf2.P01.2005', '{"3": "NCf2.P01.2005", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "НПО РИ", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NCf2.P01.3004', 'NCf2.P01.3004', '{"3": "NCf2.P01.3004", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "НПО РИ", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NCf2.P01.4005', 'NCf2.P01.4005', '{"3": "NCf2.P01.4005", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "НПО РИ", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NCf2.P01.6004', 'NCf2.P01.6004', '{"3": "NCf2.P01.6004", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "НПО РИ", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NCm4', 'NCm4', '{"3": "NCm4", "4": "кюрий-244", "5": "4", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NCm4.15.1', 'NCm4.15.1', '{"3": "NCm4.15.1", "4": "кюрий-244", "5": "4", "6": "6x5,3. Акт. часть dxh : 2,5x1", "7": "", "8": "240000000.0", "9": "0", "10": "", "11": "5", "12": "Точечный источник.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NCm4.15.2', 'NCm4.15.2', '{"3": "NCm4.15.2", "4": "кюрий-244", "5": "4", "6": "8x5. Акт. часть dxh : 4x1", "7": "", "8": "2800000000.0", "9": "0", "10": "", "11": "4", "12": "Точечный источник.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NCm4.15.3', 'NCm4.15.3', '{"3": "NCm4.15.3", "4": "кюрий-244", "5": "4", "6": "10,8x5. Акт. часть dxh : 7.2x1", "7": "", "8": "3500000000.0", "9": "0", "10": "", "11": "4", "12": "Точечный источник.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NCm4.15.4', 'NCm4.15.4', '{"3": "NCm4.15.4", "4": "кюрий-244", "5": "4", "6": "15x6. Акт. часть dxh :12x1", "7": "", "8": "7000000000.0", "9": "0", "10": "", "11": "4", "12": "Точечный источник.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NCm4.15.X', 'NCm4.15.X', '{"3": "NCm4.15.X", "4": "кюрий-244", "5": "4", "6": "", "7": "", "8": "5550000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NER', 'NER', '{"3": "NER", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NER', 'NER', '{"3": "NER", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NER 004P', 'NER 004P', '{"3": "NER 004P", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NER 554', 'NER 554', '{"3": "NER 554", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NER 558', 'NER 558', '{"3": "NER 558", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "46300000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NER-004R-100U', 'NER-004R-100U', '{"3": "NER-004R-100U", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NER-004R-10M', 'NER-004R-10M', '{"3": "NER-004R-10M", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NER-004R-15M', 'NER-004R-15M', '{"3": "NER-004R-15M", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "555000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NER-004R-1M', 'NER-004R-1M', '{"3": "NER-004R-1M", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NER-004R-2M', 'NER-004R-2M', '{"3": "NER-004R-2M", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "74000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NER-004R-500U', 'NER-004R-500U', '{"3": "NER-004R-500U", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NER-004R-5M', 'NER-004R-5M', '{"3": "NER-004R-5M", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NER588', 'NER588', '{"3": "NER588", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "46300000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NER-8170', 'NER-8170', '{"3": "NER-8170", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "5550000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NI3', 'NI3', '{"3": "NI3", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NI345000001M', 'NI345000001M', '{"3": "NI345000001M", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NI345000002M', 'NI345000002M', '{"3": "NI345000002M", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "74000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NI345000005M', 'NI345000005M', '{"3": "NI345000005M", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NI345000010M', 'NI345000010M', '{"3": "NI345000010M", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NI345000015M', 'NI345000015M', '{"3": "NI345000015M", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "555000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NI345000100U', 'NI345000100U', '{"3": "NI345000100U", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NI345000500U', 'NI345000500U', '{"3": "NI345000500U", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NSR', 'NSR', '{"3": "NSR", "4": "америций-241; бериллий", "5": "4", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NSR-F', 'NSR-F', '{"3": "NSR-F", "4": "америций-241; бериллий", "5": "4", "6": "", "7": "", "8": "592000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NSR-L', 'NSR-L', '{"3": "NSR-L", "4": "америций-241; бериллий", "5": "4", "6": "", "7": "", "8": "592000000000.0", "9": "0", "10": "США,Каматрон", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NSR-M', 'NSR-M', '{"3": "NSR-M", "4": "америций-241; бериллий", "5": "4", "6": "", "7": "", "8": "644000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NSR-T', 'NSR-T', '{"3": "NSR-T", "4": "америций-241; бериллий", "5": "4", "6": "", "7": "", "8": "280000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NSR-U', 'NSR-U', '{"3": "NSR-U", "4": "америций-241; бериллий", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PA-11-40', 'PA-11-40', '{"3": "PA-11-40", "4": "радий-226", "5": "3", "6": "", "7": "0.0", "8": "3620000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PA-12-43', 'PA-12-43', '{"3": "PA-12-43", "4": "радий-226", "5": "3", "6": "", "7": "0.0", "8": "370000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PA-16-318', 'PA-16-318', '{"3": "PA-16-318", "4": "радий-226", "5": "3", "6": "", "7": "0.0", "8": "1050000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PHC', 'PHC', '{"3": "PHC", "4": "прометий-147", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PHC.80952', 'PHC.80952', '{"3": "PHC.80952", "4": "прометий-147", "5": "2", "6": "", "7": "", "8": "7400000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PHC.80955', 'PHC.80955', '{"3": "PHC.80955", "4": "прометий-147", "5": "2", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PHCB.1861', 'PHCB.1861', '{"3": "PHCB.1861", "4": "прометий-147", "5": "2", "6": "", "7": "", "8": "25000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PHCK6317', 'PHCK6317', '{"3": "PHCK6317", "4": "прометий-147", "5": "2", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PHCX.1385', 'PHCX.1385', '{"3": "PHCX.1385", "4": "прометий-147", "5": "2", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PHG153M', 'PHG153M', '{"3": "PHG153M", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PHG153M13.140', 'PHG153M13.140', '{"3": "PHG153M13.140", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PHG153M13.410', 'PHG153M13.410', '{"3": "PHG153M13.410", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "37000000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PHILIPS Q4', 'PHILIPS Q4', '{"3": "PHILIPS Q4", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PHILIPS Q4583', 'PHILIPS Q4583', '{"3": "PHILIPS Q4583", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "1480000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PHILIPS Q4583S', 'PHILIPS Q4583S', '{"3": "PHILIPS Q4583S", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "1480000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PHILIPS Q4675', 'PHILIPS Q4675', '{"3": "PHILIPS Q4675", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "1480000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PM7', 'PM7', '{"3": "PM7", "4": "прометий-147", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PM738820003M', 'PM738820003M', '{"3": "PM738820003M", "4": "прометий-147", "5": "2", "6": "", "7": "", "8": "111000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PM738820005M', 'PM738820005M', '{"3": "PM738820005M", "4": "прометий-147", "5": "2", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PM738820009M', 'PM738820009M', '{"3": "PM738820009M", "4": "прометий-147", "5": "2", "6": "", "7": "", "8": "333000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PM738820024M', 'PM738820024M', '{"3": "PM738820024M", "4": "прометий-147", "5": "2", "6": "", "7": "", "8": "888000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PM738820900U', 'PM738820900U', '{"3": "PM738820900U", "4": "прометий-147", "5": "2", "6": "", "7": "", "8": "33300000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PM738821200U', 'PM738821200U', '{"3": "PM738821200U", "4": "прометий-147", "5": "2", "6": "", "7": "", "8": "44400000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'QB', 'QB', '{"3": "QB", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'R 6150', 'R 6150', '{"3": "R 6150", "4": "цезий-137", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'R 6160', 'R 6160', '{"3": "R 6160", "4": "цезий-137", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ra', 'Ra', '{"3": "Ra", "4": "радий-226", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ra-14', 'Ra-14', '{"3": "Ra-14", "4": "радий-226", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL', 'RSL', '{"3": "RSL", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL /ALI2003', 'RSL /ALI2003', '{"3": "RSL /ALI2003", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "12100000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL 05268A', 'RSL 05268A', '{"3": "RSL 05268A", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "1240000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL 1700', 'RSL 1700', '{"3": "RSL 1700", "4": "америций-241", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL 1701', 'RSL 1701', '{"3": "RSL 1701", "4": "америций-241", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL 1800', 'RSL 1800', '{"3": "RSL 1800", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL 1920', 'RSL 1920', '{"3": "RSL 1920", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL 2010', 'RSL 2010', '{"3": "RSL 2010", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL 2020', 'RSL 2020', '{"3": "RSL 2020", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL 2089', 'RSL 2089', '{"3": "RSL 2089", "4": "кобальт-60", "5": "1", "6": "", "7": "450000000000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL 6000', 'RSL 6000', '{"3": "RSL 6000", "4": "цезий-137", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL 6010', 'RSL 6010', '{"3": "RSL 6010", "4": "цезий-137", "5": "1", "6": "", "7": "1820000000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL 6020', 'RSL 6020', '{"3": "RSL 6020", "4": "цезий-137", "5": "1", "6": "", "7": "3920000000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL 6030', 'RSL 6030', '{"3": "RSL 6030", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "0.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL 6040', 'RSL 6040', '{"3": "RSL 6040", "4": "цезий-137", "5": "1", "6": "", "7": "15900000000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL 6050', 'RSL 6050', '{"3": "RSL 6050", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL 6060', 'RSL 6060', '{"3": "RSL 6060", "4": "цезий-137", "5": "1", "6": "", "7": "81400000000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL 6100', 'RSL 6100', '{"3": "RSL 6100", "4": "цезий-137", "5": "1", "6": "", "7": "58100000000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL 6150', 'RSL 6150', '{"3": "RSL 6150", "4": "цезий-137", "5": "1", "6": "", "7": "23800000000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL 6160', 'RSL 6160', '{"3": "RSL 6160", "4": "цезий-137", "5": "1", "6": "", "7": "50600000000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL 6240', 'RSL 6240', '{"3": "RSL 6240", "4": "цезий-137", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL 6270', 'RSL 6270', '{"3": "RSL 6270", "4": "цезий-137", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RU6', 'RU6', '{"3": "RU6", "4": "рутений-106", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RU632220001M', 'RU632220001M', '{"3": "RU632220001M", "4": "рутений-106", "5": "2", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RU632220002M', 'RU632220002M', '{"3": "RU632220002M", "4": "рутений-106", "5": "2", "6": "", "7": "", "8": "74000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RU632220100U', 'RU632220100U', '{"3": "RU632220100U", "4": "рутений-106", "5": "2", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RU632220200U', 'RU632220200U', '{"3": "RU632220200U", "4": "рутений-106", "5": "2", "6": "", "7": "", "8": "7400000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RU632220500U', 'RU632220500U', '{"3": "RU632220500U", "4": "рутений-106", "5": "2", "6": "", "7": "", "8": "18500000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SB60', 'SB60', '{"3": "SB60", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "882000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SE5', 'SE5', '{"3": "SE5", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SE536160020M', 'SE536160020M', '{"3": "SE536160020M", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SE536160050M', 'SE536160050M', '{"3": "SE536160050M", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XAm1', 'XAm1', '{"3": "XAm1", "4": "америций-241", "5": "5", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SE536160100M', 'SE536160100M', '{"3": "SE536160100M", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SE536160200M', 'SE536160200M', '{"3": "SE536160200M", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "7400000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SE536160400M', 'SE536160400M', '{"3": "SE536160400M", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "14800000000.0", "9": "0", "10": "", "11": "4", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SIF', 'SIF', '{"3": "SIF", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SIF.1171', 'SIF.1171', '{"3": "SIF.1171", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "74000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SIF.1174', 'SIF.1174', '{"3": "SIF.1174", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SIF.1175', 'SIF.1175', '{"3": "SIF.1175", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SIF.1176', 'SIF.1176', '{"3": "SIF.1176", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SIF.1177', 'SIF.1177', '{"3": "SIF.1177", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SIF.1546', 'SIF.1546', '{"3": "SIF.1546", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SIF.4415', 'SIF.4415', '{"3": "SIF.4415", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SKDB', 'SKDB', '{"3": "SKDB", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SKDB 2121', 'SKDB 2121', '{"3": "SKDB 2121", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SKDB 2122', 'SKDB 2122', '{"3": "SKDB 2122", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "74000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SKDB 2123', 'SKDB 2123', '{"3": "SKDB 2123", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SKDB 2449', 'SKDB 2449', '{"3": "SKDB 2449", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SR0', 'SR0', '{"3": "SR0", "4": "стронций-90", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SR032030001M', 'SR032030001M', '{"3": "SR032030001M", "4": "стронций-90", "5": "2", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SR032030002M', 'SR032030002M', '{"3": "SR032030002M", "4": "стронций-90", "5": "2", "6": "", "7": "", "8": "74000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SR032030005M', 'SR032030005M', '{"3": "SR032030005M", "4": "стронций-90", "5": "2", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SR032030010M', 'SR032030010M', '{"3": "SR032030010M", "4": "стронций-90", "5": "2", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SR032030020M', 'SR032030020M', '{"3": "SR032030020M", "4": "стронций-90", "5": "2", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SR032030050M', 'SR032030050M', '{"3": "SR032030050M", "4": "стронций-90", "5": "2", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'V', 'V', '{"3": "V", "4": "кобальт-57", "5": "1", "6": "12,0х2,4", "7": "", "8": "3700000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'V', 'V', '{"3": "V", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "10100000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'V ЯГР-ИГИК', 'V ЯГР-ИГИК', '{"3": "V ЯГР-ИГИК", "4": "кобальт-57", "5": "6", "6": "10,0х12,0", "7": "", "8": "4000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'VI', 'VI', '{"3": "VI", "4": "кобальт-57", "5": "1", "6": "12,0х2,4", "7": "0.0", "8": "9250000000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'VII', 'VII', '{"3": "VII", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "90300000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'VII', 'VII', '{"3": "VII", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "85100000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'X 2153', 'X 2153', '{"3": "X 2153", "4": "цезий-137", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XAm1.06', 'XAm1.06', '{"3": "XAm1.06", "4": "америций-241", "5": "5", "6": "8x5. Акт. часть dxh: 4x1", "7": "37000000.0", "8": "1110000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Дисковый (капсула а).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XAm1.07', 'XAm1.07', '{"3": "XAm1.07", "4": "америций-241", "5": "5", "6": "10,8x8. Акт. часть dxh: 7,2x1", "7": "1110000000.0", "8": "3700000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Дисковый (капсула а).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XAm1.08', 'XAm1.08', '{"3": "XAm1.08", "4": "америций-241", "5": "5", "6": "15x5. Акт. часть dxh: 10,6x1,2", "7": "1850000000.0", "8": "3700000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Дисковый (капсула а).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XAm1.09', 'XAm1.09', '{"3": "XAm1.09", "4": "америций-241", "5": "5", "6": "22x6. Акт. часть dxh: 18x1,2", "7": "", "8": "3700000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Дисковый (капсула b).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XAm1.131', 'XAm1.131', '{"3": "XAm1.131", "4": "америций-241", "5": "5", "6": "26x16x3. Акт. часть d2xd1xh: 24x18x1", "7": "370000000.0", "8": "999000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Кольцевой.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XAm1.132', 'XAm1.132', '{"3": "XAm1.132", "4": "америций-241", "5": "5", "6": "30x20x3. Акт. часть d2xd1xh: 28x22x1", "7": "370000000.0", "8": "999000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Кольцевой.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XAm1.133', 'XAm1.133', '{"3": "XAm1.133", "4": "америций-241", "5": "5", "6": "34x24x3. Акт. часть d2xd1xh: 32x26x1", "7": "370000000.0", "8": "999000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Кольцевой.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XBa3.06', 'XBa3.06', '{"3": "XBa3.06", "4": "барий-133", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "НПО РИ", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XBa3.061.4', 'XBa3.061.4', '{"3": "XBa3.061.4", "4": "барий-133", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9', 'XCd9', '{"3": "XCd9", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.01', 'XCd9.01', '{"3": "XCd9.01", "4": "кадмий-109", "5": "5", "6": "3x3. Акт. часть dxh: 2x1", "7": "37000000.0", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Дисковый (капсула b).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.06', 'XCd9.06', '{"3": "XCd9.06", "4": "кадмий-109", "5": "5", "6": "8x5. Акт. часть dxh: 4x1", "7": "185000000.0", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "Дисковый (капсула а).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.061', 'XCd9.061', '{"3": "XCd9.061", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.061.2', 'XCd9.061.2', '{"3": "XCd9.061.2", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.061.X', 'XCd9.061.X', '{"3": "XCd9.061.X", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.131', 'XCd9.131', '{"3": "XCd9.131", "4": "кадмий-109", "5": "5", "6": "26x16x3. Акт. часть d2xd1xh: 23,5x18,5x1", "7": "370000000.0", "8": "1110000000.0", "9": "0", "10": "", "11": "5", "12": "Кольцевой (капсула а).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.132', 'XCd9.132', '{"3": "XCd9.132", "4": "кадмий-109", "5": "5", "6": "30x20x3. Акт. часть d2xd1xh: 27,5x22,5x1", "7": "370000000.0", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Кольцевой (капсула а).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.133', 'XCd9.133', '{"3": "XCd9.133", "4": "кадмий-109", "5": "5", "6": "34x24x3. Акт. часть d2xd1xh: 31,5x26,5x1", "7": "370000000.0", "8": "2590000000.0", "9": "0", "10": "", "11": "5", "12": "Кольцевой (капсула а).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.21', 'XCd9.21', '{"3": "XCd9.21", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.211', 'XCd9.211', '{"3": "XCd9.211", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.212', 'XCd9.212', '{"3": "XCd9.212", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.213', 'XCd9.213', '{"3": "XCd9.213", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "555000000.0", "9": "0", "10": "", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.214', 'XCd9.214', '{"3": "XCd9.214", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.22', 'XCd9.22', '{"3": "XCd9.22", "4": "кадмий-109", "5": "5", "6": "5x3. Акт. часть dxh: 4x1", "7": "185000000.0", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "Дисковый (капсула b).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.22.2', 'XCd9.22.2', '{"3": "XCd9.22.2", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "ЗАО РИТВЕРЦ", "11": "5", "12": "из отчета Спб Изотоп", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.231', 'XCd9.231', '{"3": "XCd9.231", "4": "кадмий-109", "5": "5", "6": "26x16x3. Акт. часть d2xd1xh: 23,5x18,5x1", "7": "370000000.0", "8": "1110000000.0", "9": "0", "10": "", "11": "5", "12": "Кольцевой (капсула b).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.07.D', 'XFe5.07.D', '{"3": "XFe5.07.D", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "2960000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.232', 'XCd9.232', '{"3": "XCd9.232", "4": "кадмий-109", "5": "5", "6": "30x20x3. Акт. часть d2xd1xh: 27,5x22,5x1", "7": "370000000.0", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Кольцевой (капсула b).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.233', 'XCd9.233', '{"3": "XCd9.233", "4": "кадмий-109", "5": "5", "6": "34x24x3. Акт. часть d2xd1xh: 31,5x26,5x1", "7": "370000000.0", "8": "2590000000.0", "9": "0", "10": "", "11": "5", "12": "Кольцевой (капсула b).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.234', 'XCd9.234', '{"3": "XCd9.234", "4": "кадмий-109", "5": "5", "6": "21x11x3. Акт. часть d2xd1xh: 19,5x12,5x1", "7": "370000000.0", "8": "1110000000.0", "9": "0", "10": "", "11": "5", "12": "Кольцевой (капсула c).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.31', 'XCd9.31', '{"3": "XCd9.31", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.311', 'XCd9.311', '{"3": "XCd9.311", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Кольцевой.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.312', 'XCd9.312', '{"3": "XCd9.312", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "Кольцевой.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.313', 'XCd9.313', '{"3": "XCd9.313", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "5", "12": "Кольцевой.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.32', 'XCd9.32', '{"3": "XCd9.32", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.32.x', 'XCd9.32.x', '{"3": "XCd9.32.x", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.322', 'XCd9.322', '{"3": "XCd9.322", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Кольцевой.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.323', 'XCd9.323', '{"3": "XCd9.323", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Кольцевой.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.6.D.1', 'XCd9.6.D.1', '{"3": "XCd9.6.D.1", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "110000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.6.D.12', 'XCd9.6.D.12', '{"3": "XCd9.6.D.12", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.6.D.14', 'XCd9.6.D.14', '{"3": "XCd9.6.D.14", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCm4', 'XCm4', '{"3": "XCm4", "4": "кюрий-244", "5": "5", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCm4.06', 'XCm4.06', '{"3": "XCm4.06", "4": "кюрий-244", "5": "5", "6": "8x5. Акт. часть dxh, мм: 4x1", "7": "370000000.0", "8": "3700000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Дисковый (капсула а).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCm4.07', 'XCm4.07', '{"3": "XCm4.07", "4": "кюрий-244", "5": "5", "6": "10,8x5. Акт. часть dxh, мм: 7,2x1", "7": "370000000.0", "8": "3700000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Дисковый (капсула а).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCm4.07.1', 'XCm4.07.1', '{"3": "XCm4.07.1", "4": "кюрий-244", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCm4.07.3', 'XCm4.07.3', '{"3": "XCm4.07.3", "4": "кюрий-244", "5": "5", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCm4.07.X', 'XCm4.07.X', '{"3": "XCm4.07.X", "4": "кюрий-244", "5": "5", "6": "", "7": "", "8": "3520000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCm4.09', 'XCm4.09', '{"3": "XCm4.09", "4": "кюрий-244", "5": "5", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCm4.091', 'XCm4.091', '{"3": "XCm4.091", "4": "кюрий-244", "5": "5", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCm4.131', 'XCm4.131', '{"3": "XCm4.131", "4": "кюрий-244", "5": "5", "6": "26x16x3. Акт. часть d2xd1xh, мм: 24x18x1", "7": "370000000.0", "8": "999000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Кольцевой.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCm4.132', 'XCm4.132', '{"3": "XCm4.132", "4": "кюрий-244", "5": "5", "6": "30x20x3. Акт. часть d2xd1xh, мм: 28x22x1", "7": "370000000.0", "8": "999000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Кольцевой.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCm4.133', 'XCm4.133', '{"3": "XCm4.133", "4": "кюрий-244", "5": "5", "6": "34x24x3. Акт. часть d2xd1xh, мм: 32x26x1", "7": "370000000.0", "8": "999000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Кольцевой.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCm4.2.06', 'XCm4.2.06', '{"3": "XCm4.2.06", "4": "кюрий-244", "5": "5", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCm4.2.09', 'XCm4.2.09', '{"3": "XCm4.2.09", "4": "кюрий-244", "5": "5", "6": "22x6. Акт. часть d2xd1xh, мм: 18x1,2", "7": "3700000000.0", "8": "18500000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Дисковый (капсула b).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5', 'XFe5', '{"3": "XFe5", "4": "железо-55", "5": "5", "6": "См. примечание", "7": "740000000.0", "8": "7400000000.0", "9": "0", "10": "", "11": "5", "12": "Кольцевой. D2xD1xH (диаметр активной части d2xd1): 15x8,5x1,3 (12x10) или 26x16x3 (23x19) или 30x20x3 (28x22) или 34x24x3 (32x26).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.07.D.4', 'XFe5.07.D.4', '{"3": "XFe5.07.D.4", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "2960000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.07.D.X', 'XFe5.07.D.X', '{"3": "XFe5.07.D.X", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "2960000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.1', 'XFe5.1', '{"3": "XFe5.1", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.131', 'XFe5.131', '{"3": "XFe5.131", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.1D', 'XFe5.1D', '{"3": "XFe5.1D", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.21', 'XFe5.21', '{"3": "XFe5.21", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "74000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.211', 'XFe5.211', '{"3": "XFe5.211", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.212', 'XFe5.212', '{"3": "XFe5.212", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.213', 'XFe5.213', '{"3": "XFe5.213", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.214', 'XFe5.214', '{"3": "XFe5.214", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.215', 'XFe5.215', '{"3": "XFe5.215", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.216', 'XFe5.216', '{"3": "XFe5.216", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.23', 'XFe5.23', '{"3": "XFe5.23", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.31', 'XFe5.31', '{"3": "XFe5.31", "4": "железо-55", "5": "5", "6": "8x5. Диаметр активной части d, мм: 4", "7": "185000000.0", "8": "2200000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.311', 'XFe5.311', '{"3": "XFe5.311", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "Кольцевой.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.312', 'XFe5.312', '{"3": "XFe5.312", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "1480000000.0", "9": "0", "10": "", "11": "5", "12": "Кольцевой.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.313', 'XFe5.313', '{"3": "XFe5.313", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "2960000000.0", "9": "0", "10": "", "11": "5", "12": "Кольцевой.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.32', 'XFe5.32', '{"3": "XFe5.32", "4": "железо-55", "5": "5", "6": "10,8x5", "7": "185000000.0", "8": "7400000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.321', 'XFe5.321', '{"3": "XFe5.321", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Кольцевой.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.322', 'XFe5.322', '{"3": "XFe5.322", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "7400000000.0", "9": "0", "10": "", "11": "5", "12": "Кольцевой.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.323', 'XFe5.323', '{"3": "XFe5.323", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "5", "12": "Кольцевой.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.324', 'XFe5.324', '{"3": "XFe5.324", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "5", "12": "Кольцевой.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.33', 'XFe5.33', '{"3": "XFe5.33", "4": "железо-55", "5": "5", "6": "15x5. Диаметр активной части d, мм: 10,6", "7": "185000000.0", "8": "4440000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.34', 'XFe5.34', '{"3": "XFe5.34", "4": "железо-55", "5": "5", "6": "12,7x5. Диаметр активной части d, мм: 9", "7": "185000000.0", "8": "7400000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.35', 'XFe5.35', '{"3": "XFe5.35", "4": "железо-55", "5": "5", "6": "22x5. Диаметр активной части d, мм: 18", "7": "370000000.0", "8": "18500000000.0", "9": "0", "10": "НПО РИ", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.42', 'XFe5.42', '{"3": "XFe5.42", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XI5', 'XI5', '{"3": "XI5", "4": "йод-125", "5": "5", "6": "3x10. Акт. часть d, мм: 1", "7": "3700000000.0", "8": "11100000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XKr5.219', 'XKr5.219', '{"3": "XKr5.219", "4": "криптон-85", "5": "5", "6": "", "7": "", "8": "181000000000.0", "9": "0", "10": "", "11": "5", "12": "Дисковый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XN146', 'XN146', '{"3": "XN146", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "1910000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XPu8', 'XPu8', '{"3": "XPu8", "4": "плутоний-238", "5": "5", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XPu8.06', 'XPu8.06', '{"3": "XPu8.06", "4": "плутоний-238", "5": "5", "6": "8x5. Акт. часть dxh, мм: 4x1", "7": "185000000.0", "8": "1110000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Дисковый (капсула а).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XPu8.07', 'XPu8.07', '{"3": "XPu8.07", "4": "плутоний-238", "5": "5", "6": "10,8x5. Акт. часть dxh, мм: 7,2x1", "7": "370000000.0", "8": "3700000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Дисковый (капсула а).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XPu8.08', 'XPu8.08', '{"3": "XPu8.08", "4": "плутоний-238", "5": "5", "6": "15x5. Акт. часть dxh, мм: 10,6x1,2", "7": "1110000000.0", "8": "3700000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Дисковый (капсула а).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XPu8.09', 'XPu8.09', '{"3": "XPu8.09", "4": "плутоний-238", "5": "5", "6": "22x6. Акт. часть dxh, мм: 18x1,2", "7": "", "8": "3700000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Дисковый (капсула b).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XPu8.131', 'XPu8.131', '{"3": "XPu8.131", "4": "плутоний-238", "5": "5", "6": "26x16x3. Акт. часть d2xd1xh, мм: 24x18x1", "7": "370000000.0", "8": "999000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Кольцевой.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XPu8.132', 'XPu8.132', '{"3": "XPu8.132", "4": "плутоний-238", "5": "5", "6": "30x20x3. Акт. часть d2xd1xh, мм: 28x22x1", "7": "370000000.0", "8": "999000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Кольцевой.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XPu8.133', 'XPu8.133', '{"3": "XPu8.133", "4": "плутоний-238", "5": "5", "6": "34x24x3. Акт. часть d2xd1xh, мм: 32x26x1", "7": "370000000.0", "8": "999000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "Кольцевой.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XSn9.133', 'XSn9.133', '{"3": "XSn9.133", "4": "олово-119м", "5": "5", "6": "34x24x3. Акт. часть d2xd1xh, мм: 32x26x1", "7": "185000000.0", "8": "1850000000.0", "9": "0", "10": "НПО РИ", "11": "", "12": "Кольцевой.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'YB9', 'YB9', '{"3": "YB9", "4": "иттербий-169", "5": "5", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'YB936030001C', 'YB936030001C', '{"3": "YB936030001C", "4": "иттербий-169", "5": "5", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'YB936030100M', 'YB936030100M', '{"3": "YB936030100M", "4": "иттербий-169", "5": "5", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'YB936030300M', 'YB936030300M', '{"3": "YB936030300M", "4": "иттербий-169", "5": "5", "6": "", "7": "", "8": "11100000000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'YB936030600M', 'YB936030600M', '{"3": "YB936030600M", "4": "иттербий-169", "5": "5", "6": "", "7": "", "8": "22200000000.0", "9": "0", "10": "", "11": "", "12": "Activity Tolerance ±15% from nominal", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Z 7', 'Z 7', '{"3": "Z 7", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Z 725', 'Z 725', '{"3": "Z 725", "4": "америций-241", "5": "1", "6": "", "7": "185000000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Z 726', 'Z 726', '{"3": "Z 726", "4": "америций-241", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Z 728', 'Z 728', '{"3": "Z 728", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "629000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Z 729', 'Z 729', '{"3": "Z 729", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "629000000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Z 730', 'Z 730', '{"3": "Z 730", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "629000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Z 743', 'Z 743', '{"3": "Z 743", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "629000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Z 744', 'Z 744', '{"3": "Z 744", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "518000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Z 745', 'Z 745', '{"3": "Z 745", "4": "америций-241", "5": "1", "6": "", "7": "518000000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Z 747', 'Z 747', '{"3": "Z 747", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "296000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Z 748', 'Z 748', '{"3": "Z 748", "4": "америций-241", "5": "1", "6": "", "7": "270000000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Z 751', 'Z 751', '{"3": "Z 751", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "333000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Z 752', 'Z 752', '{"3": "Z 752", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "629000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'А-10', 'А-10', '{"3": "А-10", "4": "плутоний-239", "5": "3", "6": "20(10); 0,7", "7": "", "8": "700000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'А-10-8', 'А-10-8', '{"3": "А-10-8", "4": "плутоний-239", "5": "3", "6": "20(10); 0,7", "7": "", "8": "700000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'А-24', 'А-24', '{"3": "А-24", "4": "плутоний-239", "5": "3", "6": "30(24); 1,2", "7": "", "8": "140000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АА243Д.26', 'АА243Д.26', '{"3": "АА243Д.26", "4": "америций-243", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "НИИАР", "11": "", "12": "Открытый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АГИ', 'АГИ', '{"3": "АГИ", "4": "барий-133", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АДИ', 'АДИ', '{"3": "АДИ", "4": "плутоний-239", "5": "3", "6": "16(16); 23,5", "7": "", "8": "21000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АДС', 'АДС', '{"3": "АДС", "4": "золото-195", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АДС-1', 'АДС-1', '{"3": "АДС-1", "4": "золото-195", "5": "1", "6": "366,0x12,0", "7": "", "8": "500000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АДС-10', 'АДС-10', '{"3": "АДС-10", "4": "золото-195", "5": "1", "6": "354x496x18", "7": "", "8": "500000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АДС-11', 'АДС-11', '{"3": "АДС-11", "4": "золото-195", "5": "1", "6": "534x635x18", "7": "", "8": "500000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АДС-2', 'АДС-2', '{"3": "АДС-2", "4": "золото-195", "5": "1", "6": "48,0x12,0", "7": "", "8": "500000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АДС-3', 'АДС-3', '{"3": "АДС-3", "4": "золото-195", "5": "1", "6": "620,0x12,0", "7": "", "8": "500000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АИА-Д1', 'АИА-Д1', '{"3": "АИА-Д1", "4": "америций-241", "5": "3", "6": "12,0x20,0", "7": "", "8": "30000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АИА-Д2', 'АИА-Д2', '{"3": "АИА-Д2", "4": "америций-241", "5": "3", "6": "6,0x0,95", "7": "", "8": "30000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АИА-Д3', 'АИА-Д3', '{"3": "АИА-Д3", "4": "америций-241", "5": "3", "6": "М8x0,75x11", "7": "", "8": "30000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АИА-Д4', 'АИА-Д4', '{"3": "АИА-Д4", "4": "америций-241", "5": "3", "6": "", "7": "", "8": "30000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АИП8-1', 'АИП8-1', '{"3": "АИП8-1", "4": "плутоний-238", "5": "3", "6": "16,0x19,0x9", "7": "", "8": "7500000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АИП8-2', 'АИП8-2', '{"3": "АИП8-2", "4": "плутоний-238", "5": "3", "6": "16,0x19,0x9", "7": "", "8": "2000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АИП8-3', 'АИП8-3', '{"3": "АИП8-3", "4": "плутоний-238", "5": "3", "6": "16,0x19,0x9", "7": "", "8": "750000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АИП8-4', 'АИП8-4', '{"3": "АИП8-4", "4": "плутоний-238", "5": "3", "6": "16,0x19,0x9", "7": "", "8": "200000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АИП8-5', 'АИП8-5', '{"3": "АИП8-5", "4": "плутоний-238", "5": "3", "6": "16,0x19,0x9", "7": "", "8": "50000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АИП8-Д', 'АИП8-Д', '{"3": "АИП8-Д", "4": "плутоний-238", "5": "3", "6": "22,0x2,0", "7": "", "8": "50000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АИП-МИР-3А', 'АИП-МИР-3А', '{"3": "АИП-МИР-3А", "4": "плутоний-239", "5": "3", "6": "41(37); 3,5", "7": "", "8": "59000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АИП-Н', 'АИП-Н', '{"3": "АИП-Н", "4": "плутоний-239", "5": "3", "6": "70,0x35,0", "7": "", "8": "300000000.0", "9": "0", "10": "", "11": "5", "12": "исправлен Степановым изотопы плутония", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АИП-Н-I', 'АИП-Н-I', '{"3": "АИП-Н-I", "4": "плутоний-239", "5": "3", "6": "", "7": "0.0", "8": "300000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АИПРА', 'АИПРА', '{"3": "АИПРА", "4": "плутоний-238", "5": "3", "6": "21,7x4,0", "7": "", "8": "550000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АИП-РИГ', 'АИП-РИГ', '{"3": "АИП-РИГ", "4": "плутоний-239", "5": "3", "6": "22(17); 30", "7": "", "8": "41000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АИП-РИД', 'АИП-РИД', '{"3": "АИП-РИД", "4": "плутоний-239", "5": "3", "6": "8,6(8,6); 9,8", "7": "", "8": "210000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АИП-ЭДГХ', 'АИП-ЭДГХ', '{"3": "АИП-ЭДГХ", "4": "плутоний-239", "5": "3", "6": "8,6 max(5,6 min); 8 max", "7": "", "8": "21000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АИПЭЛ-12', 'АИПЭЛ-12', '{"3": "АИПЭЛ-12", "4": "плутоний-238", "5": "3", "6": "72x37x3", "7": "", "8": "8000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АИПЭЛ-24', 'АИПЭЛ-24', '{"3": "АИПЭЛ-24", "4": "плутоний-238", "5": "3", "6": "72x37x3", "7": "", "8": "17000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АИПЭЛ-6', 'АИПЭЛ-6', '{"3": "АИПЭЛ-6", "4": "плутоний-238", "5": "3", "6": "72x37x3", "7": "", "8": "4000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АК244', 'АК244', '{"3": "АК244", "4": "кюрий-244", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АК244D', 'АК244D', '{"3": "АК244D", "4": "кюрий-244", "5": "3", "6": "", "7": "", "8": "380000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АК244Д', 'АК244Д', '{"3": "АК244Д", "4": "кюрий-244", "5": "3", "6": "", "7": "", "8": "180000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АК244Д.28', 'АК244Д.28', '{"3": "АК244Д.28", "4": "кюрий-244", "5": "3", "6": "8x(0,2,,,1)", "7": "0.0", "8": "220000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АК244Д.29', 'АК244Д.29', '{"3": "АК244Д.29", "4": "кюрий-244", "5": "3", "6": "", "7": "", "8": "300000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АК244Д.29 (для РФА)', 'АК244Д.29 (для РФА)', '{"3": "АК244Д.29 (для РФА)", "4": "кюрий-244", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Открытый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АК244Д.38 (для обратного рассеяния)', 'АК244Д.38 (для обратного рассеяния)', '{"3": "АК244Д.38 (для обратного рассеяния)", "4": "кюрий-244", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "НИИАР", "11": "", "12": "Открытый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АК244Д1', 'АК244Д1', '{"3": "АК244Д1", "4": "кюрий-244", "5": "3", "6": "", "7": "", "8": "35000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АК-30', 'АК-30', '{"3": "АК-30", "4": "изотопы плутония", "5": "3", "6": "5,0x32,0", "7": "", "8": "3700.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АКЛ-13', 'АКЛ-13', '{"3": "АКЛ-13", "4": "изотопы плутония", "5": "3", "6": "1,3x85,0", "7": "", "8": "3700.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АЛЬФА', 'АЛЬФА', '{"3": "АЛЬФА", "4": "плутоний-238", "5": "3", "6": "22x12", "7": "92000000.0", "8": "0.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АНИ', 'АНИ', '{"3": "АНИ", "4": "кюрий-248", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АП239. А05.105', 'АП239. А05.105', '{"3": "АП239. А05.105", "4": "плутоний-239", "5": "3", "6": "", "7": "", "8": "300000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Аппарат ЛБ', 'Аппарат ЛБ', '{"3": "Аппарат ЛБ", "4": "уран-238", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Аппарат РБ', 'Аппарат РБ', '{"3": "Аппарат РБ", "4": "уран-238", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Аппарат СА', 'Аппарат СА', '{"3": "Аппарат СА", "4": "уран-238", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Аппарат ТБ', 'Аппарат ТБ', '{"3": "Аппарат ТБ", "4": "уран-238", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АРИА-241-1', 'АРИА-241-1', '{"3": "АРИА-241-1", "4": "америций-241", "5": "3", "6": "25,0x0,2", "7": "", "8": "500.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АРИА-241-2', 'АРИА-241-2', '{"3": "АРИА-241-2", "4": "америций-241", "5": "3", "6": "25,0x0,2", "7": "", "8": "1000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АСС', 'АСС', '{"3": "АСС", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "1900000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АСС-С', 'АСС-С', '{"3": "АСС-С", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "685000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Б-10', 'Б-10', '{"3": "Б-10", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "6500.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Б-105.155', 'Б-105.155', '{"3": "Б-105.155", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "325.0", "8": "17000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Б-16', 'Б-16', '{"3": "Б-16", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "16000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Б-24', 'Б-24', '{"3": "Б-24", "4": "стронций-90; иттрий-90", "5": "2", "6": "90; 45; 2,4; диаметр раб.пов.: 24 ± 2", "7": "65.0", "8": "975.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Б-30.80', 'Б-30.80', '{"3": "Б-30.80", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "16.0", "8": "6500.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Б-8', 'Б-8', '{"3": "Б-8", "4": "стронций-90; иттрий-90", "5": "2", "6": "25; 12,5; 2,7", "7": "70000.0", "8": "200000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БSr0.P100', 'БSr0.P100', '{"3": "БSr0.P100", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "6500000000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БГР', 'БГР', '{"3": "БГР", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "90000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИК', 'БИК', '{"3": "БИК", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИК-24', 'БИК-24', '{"3": "БИК-24", "4": "криптон-85", "5": "2", "6": "24,0х5,0", "7": "0.0", "8": "370000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИК-24А', 'БИК-24А', '{"3": "БИК-24А", "4": "криптон-85", "5": "2", "6": "24,0х5,0", "7": "0.0", "8": "925000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИК-3А', 'БИК-3А', '{"3": "БИК-3А", "4": "криптон-85", "5": "2", "6": "9,0x6,0", "7": "", "8": "550000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИК-4Х24', 'БИК-4Х24', '{"3": "БИК-4Х24", "4": "криптон-85", "5": "2", "6": "34x22x10", "7": "", "8": "1300000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИК-6А', 'БИК-6А', '{"3": "БИК-6А", "4": "криптон-85", "5": "2", "6": "11,0x12,5", "7": "", "8": "2600000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИК-70Х35', 'БИК-70Х35', '{"3": "БИК-70Х35", "4": "криптон-85", "5": "2", "6": "70x35x3", "7": "", "8": "3300000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИК-8Х24', 'БИК-8Х24', '{"3": "БИК-8Х24", "4": "криптон-85", "5": "2", "6": "34x22x10", "7": "", "8": "1100000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИК-М', 'БИК-М', '{"3": "БИК-М", "4": "криптон-85", "5": "2", "6": "4,0x11,0", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИК-Ф', 'БИК-Ф', '{"3": "БИК-Ф", "4": "криптон-85", "5": "2", "6": "108x54x4", "7": "", "8": "1900000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИН', 'БИН', '{"3": "БИН", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИН-63 Тип 1', 'БИН-63 Тип 1', '{"3": "БИН-63 Тип 1", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "550000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИН-63 Тип 2', 'БИН-63 Тип 2', '{"3": "БИН-63 Тип 2", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "1150000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИН-63 Тип I', 'БИН-63 Тип I', '{"3": "БИН-63 Тип I", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "520000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИН-63 Тип II', 'БИН-63 Тип II', '{"3": "БИН-63 Тип II", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "620000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИН-63-1', 'БИН-63-1', '{"3": "БИН-63-1", "4": "никель-63", "5": "2", "6": "15,0х10,0 (0,05)(для шир-толщ)", "7": "300000000.0", "8": "900000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИН-63-2', 'БИН-63-2', '{"3": "БИН-63-2", "4": "никель-63", "5": "2", "6": "30,0х10,0 (0,05)", "7": "300000000.0", "8": "900000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИН-63-3', 'БИН-63-3', '{"3": "БИН-63-3", "4": "никель-63", "5": "2", "6": "30,0х10,0 (0,05)", "7": "600000000.0", "8": "1800000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИН-63-4', 'БИН-63-4', '{"3": "БИН-63-4", "4": "никель-63", "5": "2", "6": "d20,0х1,0", "7": "300000000.0", "8": "90000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИН-63-5', 'БИН-63-5', '{"3": "БИН-63-5", "4": "никель-63", "5": "2", "6": "15,0х1,0", "7": "600000000.0", "8": "1800000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИН-63-6', 'БИН-63-6', '{"3": "БИН-63-6", "4": "никель-63", "5": "2", "6": "30,0х2,0", "7": "1200000000.0", "8": "3600000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИН-ГХ-1', 'БИН-ГХ-1', '{"3": "БИН-ГХ-1", "4": "никель-63", "5": "2", "6": "30х10х0,25", "7": "", "8": "555000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИН-ГХ-2', 'БИН-ГХ-2', '{"3": "БИН-ГХ-2", "4": "никель-63", "5": "2", "6": "15,0х0,25", "7": "", "8": "629000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИН-ГХ-2а', 'БИН-ГХ-2а', '{"3": "БИН-ГХ-2а", "4": "никель-63", "5": "2", "6": "30х10х0,25", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИП', 'БИП', '{"3": "БИП", "4": "прометий-147", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИП-10', 'БИП-10', '{"3": "БИП-10", "4": "прометий-147", "5": "2", "6": "20(10); 3", "7": "", "8": "1200000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИП-20', 'БИП-20', '{"3": "БИП-20", "4": "прометий-147", "5": "2", "6": "30(20); 3", "7": "", "8": "2800000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИП-30', 'БИП-30', '{"3": "БИП-30", "4": "прометий-147", "5": "2", "6": "40(30); 3", "7": "", "8": "5900000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БК85.011', 'БК85.011', '{"3": "БК85.011", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "42600000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИП-40', 'БИП-40', '{"3": "БИП-40", "4": "прометий-147", "5": "2", "6": "50(40); 3", "7": "", "8": "11000000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИП-50', 'БИП-50', '{"3": "БИП-50", "4": "прометий-147", "5": "2", "6": "60(50); 3", "7": "", "8": "11000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИП-50х150', 'БИП-50х150', '{"3": "БИП-50х150", "4": "прометий-147", "5": "2", "6": "160x60x5", "7": "5600000000.0", "8": "52000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИП-7', 'БИП-7', '{"3": "БИП-7", "4": "прометий-147", "5": "2", "6": "12,7x19,0", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИП-А1', 'БИП-А1', '{"3": "БИП-А1", "4": "прометий-147", "5": "2", "6": "22,0х6,0", "7": "0.0", "8": "23100000000.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИП-А2', 'БИП-А2', '{"3": "БИП-А2", "4": "прометий-147", "5": "2", "6": "22,0х6,0", "7": "0.0", "8": "23100000000.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИП-К', 'БИП-К', '{"3": "БИП-К", "4": "прометий-147", "5": "2", "6": "63x29x2", "7": "", "8": "1900000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИП-К1', 'БИП-К1', '{"3": "БИП-К1", "4": "прометий-147", "5": "2", "6": "63x29x2", "7": "", "8": "450000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИП-К2', 'БИП-К2', '{"3": "БИП-К2", "4": "прометий-149", "5": "2", "6": "63х29х2", "7": "", "8": "1850000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИП-М', 'БИП-М', '{"3": "БИП-М", "4": "прометий-147", "5": "2", "6": "6,0x7,0", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИП-Н-1', 'БИП-Н-1', '{"3": "БИП-Н-1", "4": "прометий-147", "5": "2", "6": "70 (60); 35 (25); 2,8", "7": "", "8": "4000000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИП-Н-2', 'БИП-Н-2', '{"3": "БИП-Н-2", "4": "прометий-147", "5": "2", "6": "70 (60); 35 (25); 2,8", "7": "", "8": "20000000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИП-П', 'БИП-П', '{"3": "БИП-П", "4": "прометий-147", "5": "2", "6": "", "7": "", "8": "20000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИП-Р1', 'БИП-Р1', '{"3": "БИП-Р1", "4": "прометий-147", "5": "2", "6": "60x10x2", "7": "", "8": "1100000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИП-Р2', 'БИП-Р2', '{"3": "БИП-Р2", "4": "прометий-147", "5": "2", "6": "60x10x2", "7": "", "8": "3000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИР', 'БИР', '{"3": "БИР", "4": "рутений-106; родий-106", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИР-10', 'БИР-10', '{"3": "БИР-10", "4": "рутений-106; родий-106", "5": "2", "6": "22x10", "7": "120000000.0", "8": "2200000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИР-20', 'БИР-20', '{"3": "БИР-20", "4": "рутений-106; родий-106", "5": "2", "6": "36,0x15,0", "7": "120000000.0", "8": "6000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИР-30', 'БИР-30', '{"3": "БИР-30", "4": "рутений-106; родий-106", "5": "2", "6": "46,0x15,0", "7": "120000000.0", "8": "11000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИР-40', 'БИР-40', '{"3": "БИР-40", "4": "рутений-106; родий-106", "5": "2", "6": "58,0x16,0", "7": "120000000.0", "8": "22000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИР-50', 'БИР-50', '{"3": "БИР-50", "4": "рутений-106; родий-106", "5": "2", "6": "70,0x16,0", "7": "120000000.0", "8": "62000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС', 'БИС', '{"3": "БИС", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-1', 'БИС-1', '{"3": "БИС-1", "4": "стронций-90; иттрий-90", "5": "2", "6": "12,7 (5,5); 19", "7": "", "8": "900000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-10', 'БИС-10', '{"3": "БИС-10", "4": "стронций-90; иттрий-90", "5": "2", "6": "22 (10); 10", "7": "120000000.0", "8": "44000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-2', 'БИС-2', '{"3": "БИС-2", "4": "стронций-90; иттрий-90", "5": "2", "6": "12,7 (5,5); 19", "7": "", "8": "33000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-20', 'БИС-20', '{"3": "БИС-20", "4": "стронций-90; иттрий-90", "5": "2", "6": "36 (20); 15", "7": "120000000.0", "8": "220000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-20-ТБФУ-64', 'БИС-20-ТБФУ-64', '{"3": "БИС-20-ТБФУ-64", "4": "стронций-90; иттрий-90", "5": "2", "6": "36,0x16,0", "7": "", "8": "5180000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-3', 'БИС-3', '{"3": "БИС-3", "4": "стронций-90; иттрий-90", "5": "2", "6": "3,0х80,0", "7": "", "8": "3300000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БК85.04', 'БК85.04', '{"3": "БК85.04", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "42600000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-30', 'БИС-30', '{"3": "БИС-30", "4": "стронций-90; иттрий-90", "5": "2", "6": "46 (30); 15", "7": "120000000.0", "8": "230000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-40', 'БИС-40', '{"3": "БИС-40", "4": "стронций-90; иттрий-90", "5": "2", "6": "58 (40); 16", "7": "120000000.0", "8": "230000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-4АН', 'БИС-4АН', '{"3": "БИС-4АН", "4": "стронций-90; иттрий-90", "5": "2", "6": "10,2; 12,0", "7": "", "8": "930000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "Назначенный срок службы источников – 70000 ч с даты их выпуска.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-50', 'БИС-50', '{"3": "БИС-50", "4": "стронций-90; иттрий-90", "5": "2", "6": "70 (50); 16", "7": "120000000.0", "8": "230000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-6А', 'БИС-6А', '{"3": "БИС-6А", "4": "стронций-90; иттрий-90", "5": "2", "6": "11; 12,5", "7": "", "8": "1200000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-Б-2', 'БИС-Б-2', '{"3": "БИС-Б-2", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "96000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИСГ-10', 'БИСГ-10', '{"3": "БИСГ-10", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "120000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-К', 'БИС-К', '{"3": "БИС-К", "4": "стронций-90; иттрий-90", "5": "2", "6": "85; 4; 43", "7": "", "8": "2300000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИСЛ-1', 'БИСЛ-1', '{"3": "БИСЛ-1", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "9000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИСЛ-3', 'БИСЛ-3', '{"3": "БИСЛ-3", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "11000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-М', 'БИС-М', '{"3": "БИС-М", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "1800000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-М-1', 'БИС-М-1', '{"3": "БИС-М-1", "4": "стронций-90; иттрий-90", "5": "2", "6": "6,0х7,5", "7": "0.0", "8": "185000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-М-2', 'БИС-М-2', '{"3": "БИС-М-2", "4": "стронций-90; иттрий-90", "5": "2", "6": "6,0х7,5", "7": "0.0", "8": "185000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-М-3', 'БИС-М-3', '{"3": "БИС-М-3", "4": "стронций-90; иттрий-90", "5": "2", "6": "6,0х7,5", "7": "0.0", "8": "1300000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-М-4', 'БИС-М-4', '{"3": "БИС-М-4", "4": "стронций-90; иттрий-90", "5": "2", "6": "6,0х7,5", "7": "0.0", "8": "1300000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-М-ЛА', 'БИС-М-ЛА', '{"3": "БИС-М-ЛА", "4": "стронций-90; иттрий-90", "5": "2", "6": "6 (1,5); 7,5", "7": "", "8": "11100000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-МН', 'БИС-МН', '{"3": "БИС-МН", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-МН-1', 'БИС-МН-1', '{"3": "БИС-МН-1", "4": "стронций-90; иттрий-90", "5": "2", "6": "6,0х7,5", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-МН-2', 'БИС-МН-2', '{"3": "БИС-МН-2", "4": "стронций-90; иттрий-90", "5": "2", "6": "6 (1,5); 7,5", "7": "", "8": "1100000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-МН-4', 'БИС-МН-4', '{"3": "БИС-МН-4", "4": "стронций-90; иттрий-90", "5": "2", "6": "6 (1,5); 7,5", "7": "", "8": "1100000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-МНА', 'БИС-МНА', '{"3": "БИС-МНА", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-МНА-1', 'БИС-МНА-1', '{"3": "БИС-МНА-1", "4": "стронций-90; иттрий-90", "5": "2", "6": "6; 8", "7": "", "8": "185000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "Назначенный срок службы источников – 70000 ч с даты их выпуска.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-МНА-2', 'БИС-МНА-2', '{"3": "БИС-МНА-2", "4": "стронций-90; иттрий-90", "5": "2", "6": "6; 8", "7": "", "8": "750000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "Назначенный срок службы источников – 70000 ч с даты их выпуска.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-МНБ', 'БИС-МНБ', '{"3": "БИС-МНБ", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-МНБ-1', 'БИС-МНБ-1', '{"3": "БИС-МНБ-1", "4": "стронций-90; иттрий-90", "5": "2", "6": "6; 8", "7": "", "8": "185000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "Назначенный срок службы источников – 70000 ч с даты их выпуска.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БК85', 'БК85', '{"3": "БК85", "4": "криптон-85", "5": "2", "6": "22 (20); 22", "7": "3140000000.0", "8": "42600000000.0", "9": "0", "10": "", "11": "5", "12": "Чертеж приведен для источника типа БК 85.011", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-МНБ-2', 'БИС-МНБ-2', '{"3": "БИС-МНБ-2", "4": "стронций-90; иттрий-90", "5": "2", "6": "6; 8", "7": "", "8": "750000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "Назначенный срок службы источников – 70000 ч с даты их выпуска.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-Н1', 'БИС-Н1', '{"3": "БИС-Н1", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "1100000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-Р', 'БИС-Р', '{"3": "БИС-Р", "4": "стронций-90; иттрий-90", "5": "2", "6": "22; 10", "7": "", "8": "6000000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-Ф', 'БИС-Ф', '{"3": "БИС-Ф", "4": "стронций-90; иттрий-90", "5": "2", "6": "108; 4; 54", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИТ', 'БИТ', '{"3": "БИТ", "4": "таллий-204", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИТ-10', 'БИТ-10', '{"3": "БИТ-10", "4": "таллий-204", "5": "2", "6": "22,0x10,0", "7": "1100000000.0", "8": "4800000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИТ-14', 'БИТ-14', '{"3": "БИТ-14", "4": "таллий-204", "5": "2", "6": "18,0x8,0", "7": "", "8": "2600000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИТ-20', 'БИТ-20', '{"3": "БИТ-20", "4": "таллий-204", "5": "2", "6": "36,0x15,0", "7": "3700000000.0", "8": "22000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИТ-30', 'БИТ-30', '{"3": "БИТ-30", "4": "таллий-204", "5": "2", "6": "46,0x15,0", "7": "5500000000.0", "8": "30000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИТ-40', 'БИТ-40', '{"3": "БИТ-40", "4": "таллий-204", "5": "2", "6": "58,0x16,0", "7": "11000000000.0", "8": "55000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИТ-4х24', 'БИТ-4х24', '{"3": "БИТ-4х24", "4": "таллий-204", "5": "2", "6": "34x24х10", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИТ-6', 'БИТ-6', '{"3": "БИТ-6", "4": "таллий-204", "5": "2", "6": "12,7x19,0", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИТ-М', 'БИТ-М', '{"3": "БИТ-М", "4": "таллий-204", "5": "2", "6": "6,0x7,5", "7": "", "8": "780000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИТр', 'БИТр', '{"3": "БИТр", "4": "тритий", "5": "2", "6": "", "7": "", "8": "40000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИТр-1', 'БИТр-1', '{"3": "БИТр-1", "4": "тритий", "5": "2", "6": "", "7": "", "8": "40000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИТр-Г1', 'БИТр-Г1', '{"3": "БИТр-Г1", "4": "тритий", "5": "2", "6": "40,0х10,0", "7": "", "8": "300000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИТр-Г2', 'БИТр-Г2', '{"3": "БИТр-Г2", "4": "тритий", "5": "2", "6": "40,0х20,0", "7": "", "8": "600000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИТр-Г3', 'БИТр-Г3', '{"3": "БИТр-Г3", "4": "тритий", "5": "2", "6": "40,0х40,0", "7": "", "8": "1000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИТр-Г4', 'БИТр-Г4', '{"3": "БИТр-Г4", "4": "тритий", "5": "2", "6": "70,0х35,0", "7": "", "8": "2000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИТр-М', 'БИТр-М', '{"3": "БИТр-М", "4": "тритий", "5": "2", "6": "", "7": "", "8": "3600000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИТР-М3', 'БИТР-М3', '{"3": "БИТР-М3", "4": "тритий", "5": "2", "6": "", "7": "", "8": "9030000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИЦ', 'БИЦ', '{"3": "БИЦ", "4": "церий-144; празеодим-144", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИЦ-10', 'БИЦ-10', '{"3": "БИЦ-10", "4": "церий-144; празеодим-144", "5": "2", "6": "22,0x10,0", "7": "120000000.0", "8": "44000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИЦ-20', 'БИЦ-20', '{"3": "БИЦ-20", "4": "церий-144; празеодим-144", "5": "2", "6": "36,0x15,0", "7": "120000000.0", "8": "220000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИЦ-30', 'БИЦ-30', '{"3": "БИЦ-30", "4": "церий-144; празеодим-144", "5": "2", "6": "46,0x15,0", "7": "120000000.0", "8": "220000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИЦ-40', 'БИЦ-40', '{"3": "БИЦ-40", "4": "церий-144; празеодим-144", "5": "2", "6": "58,0x16,0", "7": "120000000.0", "8": "220000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИЦ-50', 'БИЦ-50', '{"3": "БИЦ-50", "4": "церий-144; празеодим-144", "5": "2", "6": "70,0x16,0", "7": "120000000.0", "8": "220000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИЦ-К', 'БИЦ-К', '{"3": "БИЦ-К", "4": "церий-144; празеодим-144", "5": "2", "6": "85x43x4", "7": "0.0", "8": "1500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИЦ-Р', 'БИЦ-Р', '{"3": "БИЦ-Р", "4": "церий-144; празеодим-144", "5": "2", "6": "22; 10", "7": "", "8": "4500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БК85.071', 'БК85.071', '{"3": "БК85.071", "4": "криптон-85", "5": "2", "6": "4,0x6,0", "7": "", "8": "400000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БК85.072', 'БК85.072', '{"3": "БК85.072", "4": "криптон-85", "5": "2", "6": "6,0x8,0", "7": "", "8": "1000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БК85.073', 'БК85.073', '{"3": "БК85.073", "4": "криптон-85", "5": "2", "6": "10,0x10,0", "7": "", "8": "7000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БК85.074', 'БК85.074', '{"3": "БК85.074", "4": "криптон-85", "5": "2", "6": "20,0x15,0", "7": "", "8": "17000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БК85.075', 'БК85.075', '{"3": "БК85.075", "4": "криптон-85", "5": "2", "6": "30,0x15,0", "7": "", "8": "20000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БК85.P02И.1810', 'БК85.P02И.1810', '{"3": "БК85.P02И.1810", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "1590000000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БК85.Р0', 'БК85.Р0', '{"3": "БК85.Р0", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БК85.Р0 2Х.1810', 'БК85.Р0 2Х.1810', '{"3": "БК85.Р0 2Х.1810", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "18100000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БК85.Р0.2Р.509', 'БК85.Р0.2Р.509', '{"3": "БК85.Р0.2Р.509", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "5100000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БК85.Р02', 'БК85.Р02', '{"3": "БК85.Р02", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "21100000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БК85.Р02U509', 'БК85.Р02U509', '{"3": "БК85.Р02U509", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "5550000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БК85.Р02И.209', 'БК85.Р02И.209', '{"3": "БК85.Р02И.209", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "2150000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БК85.Р02П.508', 'БК85.Р02П.508', '{"3": "БК85.Р02П.508", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "340000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БК85.Р02Р.509', 'БК85.Р02Р.509', '{"3": "БК85.Р02Р.509", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БК85.Р02С.1010', 'БК85.Р02С.1010', '{"3": "БК85.Р02С.1010", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "21100000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БК85.Р02У', 'БК85.Р02У', '{"3": "БК85.Р02У", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "4900000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БК85.Р02Х1010', 'БК85.Р02Х1010', '{"3": "БК85.Р02Х1010", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "6110000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БК85.Р06Э.379РИ', 'БК85.Р06Э.379РИ', '{"3": "БК85.Р06Э.379РИ", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БК85.Р07А.1211', 'БК85.Р07А.1211', '{"3": "БК85.Р07А.1211", "4": "криптон-85", "5": "2", "6": "", "7": "", "8": "144000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БН63', 'БН63', '{"3": "БН63", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "25900000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БН63.1', 'БН63.1', '{"3": "БН63.1", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "1200000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БН63.П', 'БН63.П', '{"3": "БН63.П", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "1200000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БН63.С', 'БН63.С', '{"3": "БН63.С", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "20700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БН63/25', 'БН63/25', '{"3": "БН63/25", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "1200000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВДАВ-05', 'ВДАВ-05', '{"3": "ВДАВ-05", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "0.0", "8": "1380000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВДАВ-06', 'ВДАВ-06', '{"3": "ВДАВ-06", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "0.0", "8": "2050000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВНТ', 'ВНТ', '{"3": "ВНТ", "4": "тритий", "5": "4", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВНТ1-26М', 'ВНТ1-26М', '{"3": "ВНТ1-26М", "4": "тритий", "5": "4", "6": "", "7": "", "8": "74000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВНТ2-26М', 'ВНТ2-26М', '{"3": "ВНТ2-26М", "4": "тритий", "5": "4", "6": "", "7": "", "8": "74000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВНТ2-32', 'ВНТ2-32', '{"3": "ВНТ2-32", "4": "тритий", "5": "4", "6": "", "7": "", "8": "111000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВНТ2-67', 'ВНТ2-67', '{"3": "ВНТ2-67", "4": "тритий", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВНТ3-26д', 'ВНТ3-26д', '{"3": "ВНТ3-26д", "4": "тритий", "5": "4", "6": "", "7": "", "8": "74000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВНТ3-26М', 'ВНТ3-26М', '{"3": "ВНТ3-26М", "4": "тритий", "5": "4", "6": "", "7": "", "8": "74000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВНT4-26М', 'ВНT4-26М', '{"3": "ВНT4-26М", "4": "тритий", "5": "4", "6": "", "7": "", "8": "74000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВНT4-26М2', 'ВНT4-26М2', '{"3": "ВНT4-26М2", "4": "тритий", "5": "4", "6": "", "7": "", "8": "74000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВНИ-2', 'ВНИ-2', '{"3": "ВНИ-2", "4": "полоний-210; бериллий", "5": "4", "6": "20,0х40,0", "7": "40000000000.0", "8": "2000000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВНТ1-193МИ', 'ВНТ1-193МИ', '{"3": "ВНТ1-193МИ", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ВНИИА", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВНТ1-26', 'ВНТ1-26', '{"3": "ВНТ1-26", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ВНИИА", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВНТ1-32', 'ВНТ1-32', '{"3": "ВНТ1-32", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ВНИИА", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВНТ2-26', 'ВНТ2-26', '{"3": "ВНТ2-26", "4": "тритий", "5": "0", "6": "", "7": "74000000000.0", "8": "0.0", "9": "0", "10": "ВНИИА", "11": "0", "12": "07623885_01 ЗРИ 150509 270509", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВНТ3-26', 'ВНТ3-26', '{"3": "ВНТ3-26", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ВНИИА", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВНТ3-67', 'ВНТ3-67', '{"3": "ВНТ3-67", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ВНИИА", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВНТ4-26', 'ВНТ4-26', '{"3": "ВНТ4-26", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ВНИИА", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВР-02', 'ВР-02', '{"3": "ВР-02", "4": "кобальт-57", "5": "2", "6": "", "7": "0.0", "8": "850000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Г-1', 'Г-1', '{"3": "Г-1", "4": "криптон-85", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГBa3.044', 'ГBa3.044', '{"3": "ГBa3.044", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "1000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГBa3.044.0', 'ГBa3.044.0', '{"3": "ГBa3.044.0", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "1000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7', 'ГCs7', '{"3": "ГCs7", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.011.12', 'ГCs7.011.12', '{"3": "ГCs7.011.12", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "50000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.012', 'ГCs7.012', '{"3": "ГCs7.012", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.012.1', 'ГCs7.012.1', '{"3": "ГCs7.012.1", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "260000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.012.10', 'ГCs7.012.10', '{"3": "ГCs7.012.10", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.012.11', 'ГCs7.012.11', '{"3": "ГCs7.012.11", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "4200000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.012.2', 'ГCs7.012.2', '{"3": "ГCs7.012.2", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "120000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.012.3', 'ГCs7.012.3', '{"3": "ГCs7.012.3", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "32000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.012.4', 'ГCs7.012.4', '{"3": "ГCs7.012.4", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "61000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.012.5', 'ГCs7.012.5', '{"3": "ГCs7.012.5", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "61000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.012.6', 'ГCs7.012.6', '{"3": "ГCs7.012.6", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "240000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.012.7', 'ГCs7.012.7', '{"3": "ГCs7.012.7", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "260000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.012.8', 'ГCs7.012.8', '{"3": "ГCs7.012.8", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "171000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.012.9', 'ГCs7.012.9', '{"3": "ГCs7.012.9", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "12000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.02.2', 'ГCs7.02.2', '{"3": "ГCs7.02.2", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "121000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.02.7', 'ГCs7.02.7', '{"3": "ГCs7.02.7", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "201000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.021', 'ГCs7.021', '{"3": "ГCs7.021", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "6100000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.021.0', 'ГCs7.021.0', '{"3": "ГCs7.021.0", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "153000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.021.1', 'ГCs7.021.1', '{"3": "ГCs7.021.1", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "260000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.021.11', 'ГCs7.021.11', '{"3": "ГCs7.021.11", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "119000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.021.2', 'ГCs7.021.2', '{"3": "ГCs7.021.2", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "120000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.021.2.1', 'ГCs7.021.2.1', '{"3": "ГCs7.021.2.1", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "20000000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.021.3', 'ГCs7.021.3', '{"3": "ГCs7.021.3", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "30000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.021.4', 'ГCs7.021.4', '{"3": "ГCs7.021.4", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "610000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.021.5', 'ГCs7.021.5', '{"3": "ГCs7.021.5", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "195000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.021.6', 'ГCs7.021.6', '{"3": "ГCs7.021.6", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "240000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.021.7', 'ГCs7.021.7', '{"3": "ГCs7.021.7", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "260000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.021.8', 'ГCs7.021.8', '{"3": "ГCs7.021.8", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "61000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.023.1', 'ГCs7.023.1', '{"3": "ГCs7.023.1", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "2600000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.024.1', 'ГCs7.024.1', '{"3": "ГCs7.024.1", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "620000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.024.2', 'ГCs7.024.2', '{"3": "ГCs7.024.2", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1200000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.024.4', 'ГCs7.024.4', '{"3": "ГCs7.024.4", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "61000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.025.1', 'ГCs7.025.1', '{"3": "ГCs7.025.1", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "2500000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.028.1', 'ГCs7.028.1', '{"3": "ГCs7.028.1", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "80000000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.030.1', 'ГCs7.030.1', '{"3": "ГCs7.030.1", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "7000000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.030.2', 'ГCs7.030.2', '{"3": "ГCs7.030.2", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "12000000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.031.1', 'ГCs7.031.1', '{"3": "ГCs7.031.1", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "41600000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.108', 'ГCs7.108', '{"3": "ГCs7.108", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "411000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.12.5', 'ГCs7.12.5', '{"3": "ГCs7.12.5", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "120000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.124', 'ГCs7.124', '{"3": "ГCs7.124", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "411000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.131', 'ГCs7.131', '{"3": "ГCs7.131", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "411000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.158', 'ГCs7.158', '{"3": "ГCs7.158", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "412000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.169', 'ГCs7.169', '{"3": "ГCs7.169", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "412000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.186', 'ГCs7.186', '{"3": "ГCs7.186", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "411000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.219', 'ГCs7.219', '{"3": "ГCs7.219", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "4010000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.912.8', 'ГCs7.912.8', '{"3": "ГCs7.912.8", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "620000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГRa6.x', 'ГRa6.x', '{"3": "ГRa6.x", "4": "радий-226", "5": "1", "6": "", "7": "", "8": "4000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГU5', 'ГU5', '{"3": "ГU5", "4": "уран-235", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГU5.P01', 'ГU5.P01', '{"3": "ГU5.P01", "4": "уран-235", "5": "1", "6": "", "7": "", "8": "110000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГU5.P02', 'ГU5.P02', '{"3": "ГU5.P02", "4": "уран-235", "5": "1", "6": "", "7": "", "8": "610000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГU5.Р02.1206', 'ГU5.Р02.1206', '{"3": "ГU5.Р02.1206", "4": "уран-235", "5": "1", "6": "", "7": "", "8": "1200000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГА241М1', 'ГА241М1', '{"3": "ГА241М1", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "300000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГАМ1', 'ГАМ1', '{"3": "ГАМ1", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГАМ1.08.1', 'ГАМ1.08.1', '{"3": "ГАМ1.08.1", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "5550000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГАМ1.101', 'ГАМ1.101', '{"3": "ГАМ1.101", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "3000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГАМ1.101.4', 'ГАМ1.101.4', '{"3": "ГАМ1.101.4", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "23700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГАМ1.101.x', 'ГАМ1.101.x', '{"3": "ГАМ1.101.x", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГАМ1.18', 'ГАМ1.18', '{"3": "ГАМ1.18", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "1300000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГАМ1.18х', 'ГАМ1.18х', '{"3": "ГАМ1.18х", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "3800000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГАМ1.Х.1', 'ГАМ1.Х.1', '{"3": "ГАМ1.Х.1", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "999000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГБ133.Р02А', 'ГБ133.Р02А', '{"3": "ГБ133.Р02А", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "94000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГБ-Р-0,2', 'ГБ-Р-0,2', '{"3": "ГБ-Р-0,2", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "110000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГГ153Г', 'ГГ153Г', '{"3": "ГГ153Г", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГГ153Г1.1', 'ГГ153Г1.1', '{"3": "ГГ153Г1.1", "4": "гадолиний-153", "5": "1", "6": "10,0x5,0", "7": "", "8": "190000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГГ153Г1.2', 'ГГ153Г1.2', '{"3": "ГГ153Г1.2", "4": "гадолиний-153", "5": "1", "6": "10,0x5,0", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГГ153Г1.3', 'ГГ153Г1.3', '{"3": "ГГ153Г1.3", "4": "гадолиний-153", "5": "1", "6": "10,0x5,0", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГГ153Г1.4', 'ГГ153Г1.4', '{"3": "ГГ153Г1.4", "4": "гадолиний-153", "5": "1", "6": "10,0x5,0", "7": "", "8": "1000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГГ153Г1.5', 'ГГ153Г1.5', '{"3": "ГГ153Г1.5", "4": "гадолиний-153", "5": "1", "6": "10,0x5,0", "7": "", "8": "1900000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГГ153Г1.6', 'ГГ153Г1.6', '{"3": "ГГ153Г1.6", "4": "гадолиний-153", "5": "1", "6": "10,0x5,0", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГГ153Г2.1', 'ГГ153Г2.1', '{"3": "ГГ153Г2.1", "4": "гадолиний-153", "5": "1", "6": "5,0x3,0", "7": "", "8": "190000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГГ153Г2.2', 'ГГ153Г2.2', '{"3": "ГГ153Г2.2", "4": "гадолиний-153", "5": "1", "6": "5,0x3,0", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГГ153Г2.3', 'ГГ153Г2.3', '{"3": "ГГ153Г2.3", "4": "гадолиний-153", "5": "1", "6": "5,0x3,0", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГГ153Г2.4', 'ГГ153Г2.4', '{"3": "ГГ153Г2.4", "4": "гадолиний-153", "5": "1", "6": "5,0x3,0", "7": "", "8": "1000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГГ153Г2.5', 'ГГ153Г2.5', '{"3": "ГГ153Г2.5", "4": "гадолиний-153", "5": "1", "6": "5,0x3,0", "7": "", "8": "1900000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГГ153Г2.6', 'ГГ153Г2.6', '{"3": "ГГ153Г2.6", "4": "гадолиний-153", "5": "1", "6": "5,0x3,0", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПФ9', 'ПФ9', '{"3": "ПФ9", "4": "", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ВНИИА", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГГ-68', 'ГГ-68', '{"3": "ГГ-68", "4": "германий-68", "5": "1", "6": "6,0x127,0", "7": "", "8": "500000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Генератор радона ГРК-1', 'Генератор радона ГРК-1', '{"3": "Генератор радона ГРК-1", "4": "радий-226", "5": "1", "6": "", "7": "", "8": "1300000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Генератор радона ГРТ-1', 'Генератор радона ГРТ-1', '{"3": "Генератор радона ГРТ-1", "4": "радий-226", "5": "1", "6": "", "7": "", "8": "1190000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Генератор радона ГРТ-1М', 'Генератор радона ГРТ-1М', '{"3": "Генератор радона ГРТ-1М", "4": "радий-226", "5": "1", "6": "", "7": "", "8": "1100000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Генератор радона ГРТ-2М', 'Генератор радона ГРТ-2М', '{"3": "Генератор радона ГРТ-2М", "4": "радий-226", "5": "1", "6": "", "7": "", "8": "1100000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М', 'ГИ192М', '{"3": "ГИ192М", "4": "иридий-192", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М11.112', 'ГИ192М11.112', '{"3": "ГИ192М11.112", "4": "иридий-192", "5": "1", "6": "", "7": "", "8": "2440000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М11.140', 'ГИ192М11.140', '{"3": "ГИ192М11.140", "4": "иридий-192", "5": "1", "6": "", "7": "", "8": "3090000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М11.90', 'ГИ192М11.90', '{"3": "ГИ192М11.90", "4": "иридий-192", "5": "1", "6": "", "7": "", "8": "2130000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М3', 'ГИ192М3', '{"3": "ГИ192М3", "4": "иридий-192", "5": "1", "6": "", "7": "", "8": "4770000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М31.112', 'ГИ192М31.112', '{"3": "ГИ192М31.112", "4": "иридий-192", "5": "1", "6": "2x2", "7": "", "8": "1700000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М31.211', 'ГИ192М31.211', '{"3": "ГИ192М31.211", "4": "иридий-192", "5": "1", "6": "1x1", "7": "", "8": "410000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М31.212', 'ГИ192М31.212', '{"3": "ГИ192М31.212", "4": "иридий-192", "5": "1", "6": "2,5x2,5", "7": "", "8": "2700000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М31.312', 'ГИ192М31.312', '{"3": "ГИ192М31.312", "4": "иридий-192", "5": "1", "6": "3x3", "7": "", "8": "4200000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М31.410', 'ГИ192М31.410', '{"3": "ГИ192М31.410", "4": "иридий-192", "5": "1", "6": "0,5x0,5", "7": "", "8": "64000000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М31.412', 'ГИ192М31.412', '{"3": "ГИ192М31.412", "4": "иридий-192", "5": "1", "6": "3,5x3,5", "7": "", "8": "5100000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М31.711', 'ГИ192М31.711', '{"3": "ГИ192М31.711", "4": "иридий-192", "5": "1", "6": "1,5x1,5", "7": "", "8": "1080000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М32.112', 'ГИ192М32.112', '{"3": "ГИ192М32.112", "4": "иридий-192", "5": "1", "6": "2x2", "7": "", "8": "1700000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М32.211', 'ГИ192М32.211', '{"3": "ГИ192М32.211", "4": "иридий-192", "5": "1", "6": "1x1", "7": "", "8": "410000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М32.212', 'ГИ192М32.212', '{"3": "ГИ192М32.212", "4": "иридий-192", "5": "1", "6": "2,5x2,5", "7": "", "8": "2700000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М32.312', 'ГИ192М32.312', '{"3": "ГИ192М32.312", "4": "иридий-192", "5": "1", "6": "3x3", "7": "", "8": "4200000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М32.410', 'ГИ192М32.410', '{"3": "ГИ192М32.410", "4": "иридий-192", "5": "1", "6": "0,5x0,5", "7": "", "8": "64000000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М32.412', 'ГИ192М32.412', '{"3": "ГИ192М32.412", "4": "иридий-192", "5": "1", "6": "3,5x3,5", "7": "", "8": "5100000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М32.711', 'ГИ192М32.711', '{"3": "ГИ192М32.711", "4": "иридий-192", "5": "1", "6": "1,5x1,5", "7": "", "8": "1080000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М33.112', 'ГИ192М33.112', '{"3": "ГИ192М33.112", "4": "иридий-192", "5": "1", "6": "2x2", "7": "", "8": "1700000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М33.211', 'ГИ192М33.211', '{"3": "ГИ192М33.211", "4": "иридий-192", "5": "1", "6": "1x1", "7": "", "8": "410000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М33.212', 'ГИ192М33.212', '{"3": "ГИ192М33.212", "4": "иридий-192", "5": "1", "6": "2,5x2,5", "7": "", "8": "2700000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС7.021.1', 'ГС7.021.1', '{"3": "ГС7.021.1", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "6100000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М33.312', 'ГИ192М33.312', '{"3": "ГИ192М33.312", "4": "иридий-192", "5": "1", "6": "3x3", "7": "", "8": "4200000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М33.410', 'ГИ192М33.410', '{"3": "ГИ192М33.410", "4": "иридий-192", "5": "1", "6": "0,5x0,5", "7": "", "8": "64000000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М33.412', 'ГИ192М33.412', '{"3": "ГИ192М33.412", "4": "иридий-192", "5": "1", "6": "3,5x3,5", "7": "", "8": "5100000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М33.711', 'ГИ192М33.711', '{"3": "ГИ192М33.711", "4": "иридий-192", "5": "1", "6": "1,5x1,5", "7": "", "8": "1080000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М5', 'ГИ192М5', '{"3": "ГИ192М5", "4": "иридий-192", "5": "1", "6": "", "7": "", "8": "10800000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М51.1', 'ГИ192М51.1', '{"3": "ГИ192М51.1", "4": "иридий-192", "5": "1", "6": "4; 5 ± 0,15; 0,5; 0,5", "7": "", "8": "37000000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "В капсуле из нержавеющей стали - C(E)65546; в капсуле из титана - C(E)65344", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М52.4', 'ГИ192М52.4', '{"3": "ГИ192М52.4", "4": "иридий-192", "5": "1", "6": "4; 5 ± 0,15; 1; 1", "7": "", "8": "260000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "В капсуле из нержавеющей стали - C(E)65546; в капсуле из титана - C(E)65344", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М53.14', 'ГИ192М53.14', '{"3": "ГИ192М53.14", "4": "иридий-192", "5": "1", "6": "4; 5 ± 0,15; 1,5; 1,5", "7": "", "8": "800000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "В капсуле из нержавеющей стали - C(E)65546; в капсуле из титана - C(E)65344", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М54.25', 'ГИ192М54.25', '{"3": "ГИ192М54.25", "4": "иридий-192", "5": "1", "6": "4; 5 ± 0,15; 2; 2", "7": "", "8": "930000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "В капсуле из нержавеющей стали - C(E)65546; в капсуле из титана - C(E)65344", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М54.30', 'ГИ192М54.30', '{"3": "ГИ192М54.30", "4": "иридий-192", "5": "1", "6": "4; 5 ± 0,15; 2; 2", "7": "", "8": "1100000000000.0", "9": "0", "10": "", "11": "2", "12": "В капсуле из нержавеющей стали - C(E)65546; в капсуле из титана - C(E)65344", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М54.35', 'ГИ192М54.35', '{"3": "ГИ192М54.35", "4": "иридий-192", "5": "1", "6": "4; 5 ± 0,15; 2; 2", "7": "", "8": "1300000000000.0", "9": "0", "10": "", "11": "2", "12": "В капсуле из нержавеющей стали - C(E)65546; в капсуле из титана - C(E)65344", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М54.40', 'ГИ192М54.40', '{"3": "ГИ192М54.40", "4": "иридий-192", "5": "1", "6": "4; 5 ± 0,15; 2; 2", "7": "", "8": "1500000000000.0", "9": "0", "10": "НИИАРНИИАР", "11": "2", "12": "В капсуле из нержавеющей стали - C(E)65546; в капсуле из титана - C(E)65344", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М54.45', 'ГИ192М54.45', '{"3": "ГИ192М54.45", "4": "иридий-192", "5": "1", "6": "4; 5 ± 0,15; 2; 2", "7": "", "8": "1700000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "В капсуле из нержавеющей стали - C(E)65546; в капсуле из титана - C(E)65344", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М55', 'ГИ192М55', '{"3": "ГИ192М55", "4": "иридий-192", "5": "1", "6": "", "7": "", "8": "7990000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М55.55', 'ГИ192М55.55', '{"3": "ГИ192М55.55", "4": "иридий-192", "5": "1", "6": "5; 6 ± 0,15; 2,5; 2,5", "7": "", "8": "2000000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "В капсуле из нержавеющей стали - C(E)65546; в капсуле из титана - C(E)65344", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М55.65', 'ГИ192М55.65', '{"3": "ГИ192М55.65", "4": "иридий-192", "5": "1", "6": "5; 6 ± 0,15; 2,5; 2,5", "7": "", "8": "2400000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "В капсуле из нержавеющей стали - C(E)65546; в капсуле из титана - C(E)65344", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М55.75', 'ГИ192М55.75', '{"3": "ГИ192М55.75", "4": "иридий-192", "5": "1", "6": "5; 6 ± 0,15; 2,5; 2,5", "7": "", "8": "2800000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "В капсуле из нержавеющей стали - C(E)65546; в капсуле из титана - C(E)65344", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М56', 'ГИ192М56', '{"3": "ГИ192М56", "4": "иридий-192", "5": "1", "6": "", "7": "", "8": "6220000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М56.105', 'ГИ192М56.105', '{"3": "ГИ192М56.105", "4": "иридий-192", "5": "1", "6": "5; 6 ± 0,15; 3; 3", "7": "", "8": "3900000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "В капсуле из нержавеющей стали - C(E)65546; в капсуле из титана - C(E)65344", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М56.125', 'ГИ192М56.125', '{"3": "ГИ192М56.125", "4": "иридий-192", "5": "1", "6": "5; 6 ± 0,15; 3; 3", "7": "", "8": "4600000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "В капсуле из нержавеющей стали - C(E)65546; в капсуле из титана - C(E)65344", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М56.85', 'ГИ192М56.85', '{"3": "ГИ192М56.85", "4": "иридий-192", "5": "1", "6": "5; 6 ± 0,15; 3; 3", "7": "", "8": "3100000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "В капсуле из нержавеющей стали - C(E)65546; в капсуле из титана - C(E)65344", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М57', 'ГИ192М57', '{"3": "ГИ192М57", "4": "иридий-192", "5": "1", "6": "", "7": "", "8": "5220000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М57.135', 'ГИ192М57.135', '{"3": "ГИ192М57.135", "4": "иридий-192", "5": "1", "6": "6; 7 ± 0,15; 3,5; 3,5", "7": "", "8": "5000000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "В капсуле из нержавеющей стали - C(E)65546; в капсуле из титана - C(E)65344", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М57.150', 'ГИ192М57.150', '{"3": "ГИ192М57.150", "4": "иридий-192", "5": "1", "6": "6; 7 ± 0,15; 3,5; 3,5", "7": "", "8": "5500000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "В капсуле из нержавеющей стали - C(E)65546; в капсуле из титана - C(E)65344", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М57.165', 'ГИ192М57.165', '{"3": "ГИ192М57.165", "4": "иридий-192", "5": "1", "6": "6; 7 ± 0,15; 3,5; 3,5", "7": "", "8": "6100000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "В капсуле из нержавеющей стали - C(E)65546; в капсуле из титана - C(E)65344", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М58', 'ГИ192М58', '{"3": "ГИ192М58", "4": "иридий-192", "5": "1", "6": "", "7": "", "8": "1140000000000000.0", "9": "0", "10": "НИИАР", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М61', 'ГИ192М61', '{"3": "ГИ192М61", "4": "иридий-192", "5": "1", "6": "7,15х19,5", "7": "370000000000.0", "8": "740000000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М62', 'ГИ192М62', '{"3": "ГИ192М62", "4": "иридий-192", "5": "1", "6": "7,15х19,5", "7": "888000000000.0", "8": "1670000000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М63', 'ГИ192М63', '{"3": "ГИ192М63", "4": "иридий-192", "5": "1", "6": "7,15х19,5", "7": "3150000000000.0", "8": "4630000000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М85', 'ГИ192М85', '{"3": "ГИ192М85", "4": "иридий-192", "5": "1", "6": "", "7": "", "8": "10500000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИГ-153-V', 'ГИГ-153-V', '{"3": "ГИГ-153-V", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "4000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИД-Ц', 'ГИД-Ц', '{"3": "ГИД-Ц", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИД-Ц-1', 'ГИД-Ц-1', '{"3": "ГИД-Ц-1", "4": "цезий-137", "5": "1", "6": "6,0х10,0", "7": "", "8": "66500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИД-Ц-1-1', 'ГИД-Ц-1-1', '{"3": "ГИД-Ц-1-1", "4": "цезий-137", "5": "1", "6": "6 (3); 10 (3)", "7": "", "8": "44400000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИД-Ц-2', 'ГИД-Ц-2', '{"3": "ГИД-Ц-2", "4": "цезий-137", "5": "1", "6": "8,0х12,0", "7": "", "8": "278000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИД-Ц-2-1', 'ГИД-Ц-2-1', '{"3": "ГИД-Ц-2-1", "4": "цезий-137", "5": "1", "6": "8 (4,9); 12 (5,5)", "7": "", "8": "207000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИД-Ц-2-4', 'ГИД-Ц-2-4', '{"3": "ГИД-Ц-2-4", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "260000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИД-Ц-2-6', 'ГИД-Ц-2-6', '{"3": "ГИД-Ц-2-6", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "2060000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИД-Ц-3', 'ГИД-Ц-3', '{"3": "ГИД-Ц-3", "4": "цезий-137", "5": "1", "6": "10,0х13,0", "7": "", "8": "665000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИД-Ц-3-1', 'ГИД-Ц-3-1', '{"3": "ГИД-Ц-3-1", "4": "цезий-137", "5": "1", "6": "10 (6); 13 (4)", "7": "", "8": "233000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИД-Ц-4', 'ГИД-Ц-4', '{"3": "ГИД-Ц-4", "4": "цезий-137", "5": "1", "6": "14,0х19,0", "7": "", "8": "2800000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИД-Ц-4-1', 'ГИД-Ц-4-1', '{"3": "ГИД-Ц-4-1", "4": "цезий-137", "5": "1", "6": "14 (10); 19 (10)", "7": "", "8": "1670000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИД-Ц-5', 'ГИД-Ц-5', '{"3": "ГИД-Ц-5", "4": "цезий-137", "5": "1", "6": "22,0х26,0", "7": "", "8": "14300000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИД-Ц-5-1', 'ГИД-Ц-5-1', '{"3": "ГИД-Ц-5-1", "4": "цезий-137", "5": "1", "6": "22 (18); 26 (16,5)", "7": "", "8": "9300000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИД-Ц-6', 'ГИД-Ц-6', '{"3": "ГИД-Ц-6", "4": "цезий-137", "5": "1", "6": "27,0х31,0", "7": "", "8": "31400000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИД-Ц-6-1', 'ГИД-Ц-6-1', '{"3": "ГИД-Ц-6-1", "4": "цезий-137", "5": "1", "6": "27 (23); 31 (22)", "7": "", "8": "18900000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД', 'ГИИД', '{"3": "ГИИД", "4": "иридий-192", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-0', 'ГИИД-0', '{"3": "ГИИД-0", "4": "иридий-192", "5": "1", "6": "2,8х4,5", "7": "", "8": "58000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-1', 'ГИИД-1', '{"3": "ГИИД-1", "4": "иридий-192", "5": "1", "6": "4 (0,5); 5 (0,5)", "7": "", "8": "100000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-10-0,5', 'ГИИД-10-0,5', '{"3": "ГИИД-10-0,5", "4": "иридий-192", "5": "1", "6": "6,4 (0,5); 26,8 (0,5)", "7": "", "8": "100000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-10-1', 'ГИИД-10-1', '{"3": "ГИИД-10-1", "4": "иридий-192", "5": "1", "6": "6,4 (1); 26,8 (1)", "7": "", "8": "590000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-10-1,5', 'ГИИД-10-1,5', '{"3": "ГИИД-10-1,5", "4": "иридий-192", "5": "1", "6": "6,4 (1,5); 26,8 (1,5)", "7": "", "8": "1600000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-10-2', 'ГИИД-10-2', '{"3": "ГИИД-10-2", "4": "иридий-192", "5": "1", "6": "6,4 (2); 26,8 (2)", "7": "", "8": "3700000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-10-3', 'ГИИД-10-3', '{"3": "ГИИД-10-3", "4": "иридий-192", "5": "1", "6": "6,4 (3); 26,8 (3)", "7": "", "8": "11000000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-10-4', 'ГИИД-10-4', '{"3": "ГИИД-10-4", "4": "иридий-192", "5": "1", "6": "6,4 (4); 26,8 (4)", "7": "", "8": "29000000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-11-0,5', 'ГИИД-11-0,5', '{"3": "ГИИД-11-0,5", "4": "иридий-192", "5": "1", "6": "7,15 (0,5); 19,7 (0,5)", "7": "", "8": "110000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-11-1', 'ГИИД-11-1', '{"3": "ГИИД-11-1", "4": "иридий-192", "5": "1", "6": "7,15 (1); 19,7 (1)", "7": "", "8": "600000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-11-1,5', 'ГИИД-11-1,5', '{"3": "ГИИД-11-1,5", "4": "иридий-192", "5": "1", "6": "7,15 (1,5); 19,7 (1,5)", "7": "", "8": "1700000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-11-2', 'ГИИД-11-2', '{"3": "ГИИД-11-2", "4": "иридий-192", "5": "1", "6": "7,15 (2); 19,7 (2)", "7": "", "8": "3800000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-11-3', 'ГИИД-11-3', '{"3": "ГИИД-11-3", "4": "иридий-192", "5": "1", "6": "7,15 (3); 19,7 (3)", "7": "", "8": "12000000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-11-4', 'ГИИД-11-4', '{"3": "ГИИД-11-4", "4": "иридий-192", "5": "1", "6": "7,15 (4); 19,7 (4)", "7": "", "8": "31000000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-11а-0,5', 'ГИИД-11а-0,5', '{"3": "ГИИД-11а-0,5", "4": "иридий-192", "5": "1", "6": "6 (0,5); 19,7 (0,5)", "7": "", "8": "110000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-11а-1', 'ГИИД-11а-1', '{"3": "ГИИД-11а-1", "4": "иридий-192", "5": "1", "6": "6 (1); 19,7 (1)", "7": "", "8": "630000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-11а-1,5', 'ГИИД-11а-1,5', '{"3": "ГИИД-11а-1,5", "4": "иридий-192", "5": "1", "6": "6 (1,5); 19,7 (1,5)", "7": "", "8": "1800000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-11а-2', 'ГИИД-11а-2', '{"3": "ГИИД-11а-2", "4": "иридий-192", "5": "1", "6": "6 (2); 19,7 (2)", "7": "", "8": "3800000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-11а-3', 'ГИИД-11а-3', '{"3": "ГИИД-11а-3", "4": "иридий-192", "5": "1", "6": "6 (3); 19,7 (3)", "7": "", "8": "12000000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-11а-4', 'ГИИД-11а-4', '{"3": "ГИИД-11а-4", "4": "иридий-192", "5": "1", "6": "6 (4); 19,7 (4)", "7": "", "8": "31000000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-12-0,5', 'ГИИД-12-0,5', '{"3": "ГИИД-12-0,5", "4": "иридий-192", "5": "1", "6": "6,7 (0,5); 27,2 (0,5)", "7": "", "8": "100000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-12-1', 'ГИИД-12-1', '{"3": "ГИИД-12-1", "4": "иридий-192", "5": "1", "6": "6,7 (1); 27,2 (1)", "7": "", "8": "560000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-12-1,5', 'ГИИД-12-1,5', '{"3": "ГИИД-12-1,5", "4": "иридий-192", "5": "1", "6": "6,7 (1,5); 27,2 (1,5)", "7": "", "8": "1600000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-12-2', 'ГИИД-12-2', '{"3": "ГИИД-12-2", "4": "иридий-192", "5": "1", "6": "6,7 (2); 27,2 (2)", "7": "", "8": "3500000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-12-3', 'ГИИД-12-3', '{"3": "ГИИД-12-3", "4": "иридий-192", "5": "1", "6": "6,7 (3); 27,2 (3)", "7": "", "8": "10000000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-12-4', 'ГИИД-12-4', '{"3": "ГИИД-12-4", "4": "иридий-192", "5": "1", "6": "6,7 (4); 27,2 (4)", "7": "", "8": "27000000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-12а-0,5', 'ГИИД-12а-0,5', '{"3": "ГИИД-12а-0,5", "4": "иридий-192", "5": "1", "6": "6 (0,5); 27,2 (0,5)", "7": "", "8": "100000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-12а-1', 'ГИИД-12а-1', '{"3": "ГИИД-12а-1", "4": "иридий-192", "5": "1", "6": "6 (1); 27,2 (1)", "7": "", "8": "560000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-12а-1,5', 'ГИИД-12а-1,5', '{"3": "ГИИД-12а-1,5", "4": "иридий-192", "5": "1", "6": "6 (1,5); 27,2 (1,5)", "7": "", "8": "1600000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС7.021.4', 'ГС7.021.4', '{"3": "ГС7.021.4", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "63700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-12а-2', 'ГИИД-12а-2', '{"3": "ГИИД-12а-2", "4": "иридий-192", "5": "1", "6": "6 (2); 27,2 (2)", "7": "", "8": "3400000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-12а-3', 'ГИИД-12а-3', '{"3": "ГИИД-12а-3", "4": "иридий-192", "5": "1", "6": "6 (3); 27,2 (3)", "7": "", "8": "10000000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-12а-4', 'ГИИД-12а-4', '{"3": "ГИИД-12а-4", "4": "иридий-192", "5": "1", "6": "6 (4); 27,2 (4)", "7": "", "8": "26000000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-13-0,5', 'ГИИД-13-0,5', '{"3": "ГИИД-13-0,5", "4": "иридий-192", "5": "1", "6": "5 (0,5); 7,5 (0,5)", "7": "", "8": "110000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-13-1', 'ГИИД-13-1', '{"3": "ГИИД-13-1", "4": "иридий-192", "5": "1", "6": "5 (1); 7,5 (1)", "7": "", "8": "630000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-13-1,5', 'ГИИД-13-1,5', '{"3": "ГИИД-13-1,5", "4": "иридий-192", "5": "1", "6": "5 (1,5); 7,5 (1,5)", "7": "", "8": "1800000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-13-2', 'ГИИД-13-2', '{"3": "ГИИД-13-2", "4": "иридий-192", "5": "1", "6": "5 (2); 7,5 (2)", "7": "", "8": "3800000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-13-3', 'ГИИД-13-3', '{"3": "ГИИД-13-3", "4": "иридий-192", "5": "1", "6": "5 (3); 7,5 (3)", "7": "", "8": "12000000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-14-0,5', 'ГИИД-14-0,5', '{"3": "ГИИД-14-0,5", "4": "иридий-192", "5": "1", "6": "5,2 (0,5); 15,1 (0,5)", "7": "", "8": "110000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-14-1', 'ГИИД-14-1', '{"3": "ГИИД-14-1", "4": "иридий-192", "5": "1", "6": "5,2 (1); 15,1 (1)", "7": "", "8": "630000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-14-1,5', 'ГИИД-14-1,5', '{"3": "ГИИД-14-1,5", "4": "иридий-192", "5": "1", "6": "5,2 (1,5); 15,1 (1,5)", "7": "", "8": "1800000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-14-2', 'ГИИД-14-2', '{"3": "ГИИД-14-2", "4": "иридий-192", "5": "1", "6": "5,2 (2); 15,1 (2)", "7": "", "8": "3700000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-14-3', 'ГИИД-14-3', '{"3": "ГИИД-14-3", "4": "иридий-192", "5": "1", "6": "5,2 (3); 15,1 (3)", "7": "", "8": "11000000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-15-0,5', 'ГИИД-15-0,5', '{"3": "ГИИД-15-0,5", "4": "иридий-192", "5": "1", "6": "5,55 (0,5); 15,2 (0,5)", "7": "", "8": "100000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-15-1', 'ГИИД-15-1', '{"3": "ГИИД-15-1", "4": "иридий-192", "5": "1", "6": "5,55 (1); 15,2 (1)", "7": "", "8": "560000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-15-1,5', 'ГИИД-15-1,5', '{"3": "ГИИД-15-1,5", "4": "иридий-192", "5": "1", "6": "5,55 (1,5); 15,2 (1,5)", "7": "", "8": "1600000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-15-2', 'ГИИД-15-2', '{"3": "ГИИД-15-2", "4": "иридий-192", "5": "1", "6": "5,55 (2); 15,2 (2)", "7": "", "8": "3700000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-15-3', 'ГИИД-15-3', '{"3": "ГИИД-15-3", "4": "иридий-192", "5": "1", "6": "5,55 (3); 15,2 (3)", "7": "", "8": "11000000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-15-4', 'ГИИД-15-4', '{"3": "ГИИД-15-4", "4": "иридий-192", "5": "1", "6": "5,55 (4); 15,2 (4)", "7": "", "8": "31000000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-2', 'ГИИД-2', '{"3": "ГИИД-2", "4": "иридий-192", "5": "1", "6": "4 (1); 5 (1)", "7": "", "8": "590000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-2-5', 'ГИИД-2-5', '{"3": "ГИИД-2-5", "4": "иридий-192", "5": "1", "6": "", "7": "", "8": "1970000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-2-6', 'ГИИД-2-6', '{"3": "ГИИД-2-6", "4": "иридий-192", "5": "1", "6": "", "7": "", "8": "2700000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-3', 'ГИИД-3', '{"3": "ГИИД-3", "4": "иридий-192", "5": "1", "6": "4 (1,5); 5 (1,5)", "7": "", "8": "1600000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-4', 'ГИИД-4', '{"3": "ГИИД-4", "4": "иридий-192", "5": "1", "6": "4 (2); 5 (2)", "7": "", "8": "3590000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-4М', 'ГИИД-4М', '{"3": "ГИИД-4М", "4": "иридий-192", "5": "1", "6": "4 (2); 6 (3)", "7": "", "8": "5740000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-5', 'ГИИД-5', '{"3": "ГИИД-5", "4": "иридий-192", "5": "1", "6": "5 (3); 6 (3)", "7": "", "8": "10800000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-6', 'ГИИД-6', '{"3": "ГИИД-6", "4": "иридий-192", "5": "1", "6": "6 (4); 7 (4)", "7": "", "8": "28800000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-7', 'ГИИД-7', '{"3": "ГИИД-7", "4": "иридий-192", "5": "1", "6": "8 (6); 9 (6)", "7": "", "8": "63000000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-8-0,5', 'ГИИД-8-0,5', '{"3": "ГИИД-8-0,5", "4": "иридий-192", "5": "1", "6": "4,5 (0,5); 8 (0,5)", "7": "", "8": "100000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-8-1', 'ГИИД-8-1', '{"3": "ГИИД-8-1", "4": "иридий-192", "5": "1", "6": "4,5 (1); 8 (1)", "7": "", "8": "590000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-8-1,5', 'ГИИД-8-1,5', '{"3": "ГИИД-8-1,5", "4": "иридий-192", "5": "1", "6": "4,5 (1,5); 8 (1,5)", "7": "", "8": "1700000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-8-2', 'ГИИД-8-2', '{"3": "ГИИД-8-2", "4": "иридий-192", "5": "1", "6": "4,5 (2); 8 (2)", "7": "", "8": "3700000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-8-3', 'ГИИД-8-3', '{"3": "ГИИД-8-3", "4": "иридий-192", "5": "1", "6": "4,5 (3); 8 (3)", "7": "", "8": "11000000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-9-0,5', 'ГИИД-9-0,5', '{"3": "ГИИД-9-0,5", "4": "иридий-192", "5": "1", "6": "6,2 (0,5); 15,1 (0,5)", "7": "", "8": "100000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-9-1', 'ГИИД-9-1', '{"3": "ГИИД-9-1", "4": "иридий-192", "5": "1", "6": "6,2 (1); 15,1 (1)", "7": "", "8": "590000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-9-1,5', 'ГИИД-9-1,5', '{"3": "ГИИД-9-1,5", "4": "иридий-192", "5": "1", "6": "6,2 (1,5); 15,1 (1,5)", "7": "", "8": "1600000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИИД-9-2', 'ГИИД-9-2', '{"3": "ГИИД-9-2", "4": "иридий-192", "5": "1", "6": "6,2 (2); 15,1 (2)", "7": "", "8": "3700000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК', 'ГИК', '{"3": "ГИК", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-10-1', 'ГИК-10-1', '{"3": "ГИК-10-1", "4": "кобальт-60", "5": "1", "6": "13 (10); 26,5 (17,8)", "7": "", "8": "17200000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-10-2', 'ГИК-10-2', '{"3": "ГИК-10-2", "4": "кобальт-60", "5": "1", "6": "13 (10); 26,5 (17,8)", "7": "", "8": "34000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-10-3', 'ГИК-10-3', '{"3": "ГИК-10-3", "4": "кобальт-60", "5": "1", "6": "13 (10); 26,5 (17,8)", "7": "", "8": "48100000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-1-1', 'ГИК-1-1', '{"3": "ГИК-1-1", "4": "кобальт-60", "5": "1", "6": "6 (3); 7 (3,3)", "7": "", "8": "5550000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-11-1', 'ГИК-11-1', '{"3": "ГИК-11-1", "4": "кобальт-60", "5": "1", "6": "11 (8); 50 (40,8)", "7": "", "8": "17800000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-11-2', 'ГИК-11-2', '{"3": "ГИК-11-2", "4": "кобальт-60", "5": "1", "6": "11 (8); 50 (40,8)", "7": "", "8": "68000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-1-2', 'ГИК-1-2', '{"3": "ГИК-1-2", "4": "кобальт-60", "5": "1", "6": "6 (3); 7 (3,3)", "7": "", "8": "14200000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-12-1', 'ГИК-12-1', '{"3": "ГИК-12-1", "4": "кобальт-60", "5": "1", "6": "11 (8); 25 (18)", "7": "", "8": "35200000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-12а-1', 'ГИК-12а-1', '{"3": "ГИК-12а-1", "4": "кобальт-60", "5": "1", "6": "11 (8); 99 (90,5)", "7": "", "8": "22200000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-12а-2', 'ГИК-12а-2', '{"3": "ГИК-12а-2", "4": "кобальт-60", "5": "1", "6": "11 (8); 99 (90,5)", "7": "", "8": "50000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-12а-3', 'ГИК-12а-3', '{"3": "ГИК-12а-3", "4": "кобальт-60", "5": "1", "6": "11 (8); 99 (90,5)", "7": "", "8": "68000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-12а-4', 'ГИК-12а-4', '{"3": "ГИК-12а-4", "4": "кобальт-60", "5": "1", "6": "11 (8); 99 (90,5)", "7": "", "8": "100000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-1-3', 'ГИК-1-3', '{"3": "ГИК-1-3", "4": "кобальт-60", "5": "1", "6": "6 (3); 7 (3,3)", "7": "", "8": "142000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-13-1', 'ГИК-13-1', '{"3": "ГИК-13-1", "4": "кобальт-60", "5": "1", "6": "4 (2,5); 5 (2,5)", "7": "", "8": "80000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-1-4', 'ГИК-1-4', '{"3": "ГИК-1-4", "4": "кобальт-60", "5": "1", "6": "6 (3); 7 (3,3)", "7": "", "8": "10700000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-14-1', 'ГИК-14-1', '{"3": "ГИК-14-1", "4": "кобальт-60", "5": "1", "6": "19 (15); 22 (14)", "7": "", "8": "74000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС7.021.5', 'ГС7.021.5', '{"3": "ГС7.021.5", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "120000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-1-5', 'ГИК-1-5', '{"3": "ГИК-1-5", "4": "кобальт-60", "5": "1", "6": "6 (3); 7 (3,3)", "7": "", "8": "22200000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-1-5м', 'ГИК-1-5м', '{"3": "ГИК-1-5м", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "15600000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-16', 'ГИК-16', '{"3": "ГИК-16", "4": "кобальт-60", "5": "1", "6": "8,2х25,0", "7": "", "8": "14000000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-17', 'ГИК-17', '{"3": "ГИК-17", "4": "кобальт-60", "5": "1", "6": "12,2х25,0", "7": "", "8": "32000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-17М', 'ГИК-17М', '{"3": "ГИК-17М", "4": "кобальт-60", "5": "1", "6": "21 (17); 34 (26)", "7": "", "8": "454000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-18', 'ГИК-18', '{"3": "ГИК-18", "4": "кобальт-60", "5": "1", "6": "16,2х25,0", "7": "", "8": "57000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-19', 'ГИК-19', '{"3": "ГИК-19", "4": "кобальт-60", "5": "1", "6": "20,2х25,0", "7": "", "8": "73000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-2', 'ГИК-2', '{"3": "ГИК-2", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "110000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-2-0', 'ГИК-2-0', '{"3": "ГИК-2-0", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "91000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-2-1', 'ГИК-2-1', '{"3": "ГИК-2-1", "4": "кобальт-60", "5": "1", "6": "6,0х7,0", "7": "", "8": "15300000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-2-10', 'ГИК-2-10', '{"3": "ГИК-2-10", "4": "кобальт-60", "5": "1", "6": "6 (4,5); 7 (4,5)", "7": "", "8": "1440000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-2-11', 'ГИК-2-11', '{"3": "ГИК-2-11", "4": "кобальт-60", "5": "1", "6": "6,0х7,0", "7": "", "8": "1700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-2-12', 'ГИК-2-12', '{"3": "ГИК-2-12", "4": "кобальт-60", "5": "1", "6": "6 (4,5); 7 (4,5)", "7": "", "8": "2890000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-2-13', 'ГИК-2-13', '{"3": "ГИК-2-13", "4": "кобальт-60", "5": "1", "6": "6 (4,5); 7 (4,5)", "7": "", "8": "7200000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-2-14', 'ГИК-2-14', '{"3": "ГИК-2-14", "4": "кобальт-60", "5": "1", "6": "6 (4,5); 7 (4,5)", "7": "", "8": "14400000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-2-15', 'ГИК-2-15', '{"3": "ГИК-2-15", "4": "кобальт-60", "5": "1", "6": "6 (4,5); 7 (4,5)", "7": "", "8": "28900000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-2-16', 'ГИК-2-16', '{"3": "ГИК-2-16", "4": "кобальт-60", "5": "1", "6": "6 (4,5); 7 (4,5)", "7": "", "8": "72000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-2-17', 'ГИК-2-17', '{"3": "ГИК-2-17", "4": "кобальт-60", "5": "1", "6": "6 (4,5); 7 (4,5)", "7": "", "8": "144000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-2-18', 'ГИК-2-18', '{"3": "ГИК-2-18", "4": "кобальт-60", "5": "1", "6": "6 (4,5); 7 (4,5)", "7": "", "8": "720000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-2-2', 'ГИК-2-2', '{"3": "ГИК-2-2", "4": "кобальт-60", "5": "1", "6": "6,0х7,0", "7": "0.0", "8": "19700000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-2-3', 'ГИК-2-3', '{"3": "ГИК-2-3", "4": "кобальт-60", "5": "1", "6": "6,0х7,0", "7": "0.0", "8": "59300000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-2-4', 'ГИК-2-4', '{"3": "ГИК-2-4", "4": "кобальт-60", "5": "1", "6": "6 (4,5); 7 (4,5)", "7": "", "8": "289000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-2-5', 'ГИК-2-5', '{"3": "ГИК-2-5", "4": "кобальт-60", "5": "1", "6": "6,0х7,0", "7": "0.0", "8": "318000000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-2-6', 'ГИК-2-6', '{"3": "ГИК-2-6", "4": "кобальт-60", "5": "1", "6": "6 (4,5); 7 (4,5)", "7": "", "8": "2890000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-2-7', 'ГИК-2-7', '{"3": "ГИК-2-7", "4": "кобальт-60", "5": "1", "6": "6 (4,5); 7 (4,5)", "7": "", "8": "72000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-2-8', 'ГИК-2-8', '{"3": "ГИК-2-8", "4": "кобальт-60", "5": "1", "6": "6 (4,5); 7 (4,5)", "7": "", "8": "289000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-2-9', 'ГИК-2-9', '{"3": "ГИК-2-9", "4": "кобальт-60", "5": "1", "6": "6 (4,5); 7 (4,5)", "7": "", "8": "720000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-2-К', 'ГИК-2-К', '{"3": "ГИК-2-К", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "241000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-3-1', 'ГИК-3-1', '{"3": "ГИК-3-1", "4": "кобальт-60", "5": "1", "6": "6 (4,5); 11,5 (7,5)", "7": "", "8": "185000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-3-2', 'ГИК-3-2', '{"3": "ГИК-3-2", "4": "кобальт-60", "5": "1", "6": "6 (4,5); 11,5 (7,5)", "7": "", "8": "4070000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-4-1', 'ГИК-4-1', '{"3": "ГИК-4-1", "4": "кобальт-60", "5": "1", "6": "11 (8); 12 (5,5)", "7": "", "8": "680000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-4-2', 'ГИК-4-2', '{"3": "ГИК-4-2", "4": "кобальт-60", "5": "1", "6": "11 (8); 12 (5,5)", "7": "", "8": "4810000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-5-1', 'ГИК-5-1', '{"3": "ГИК-5-1", "4": "кобальт-60", "5": "1", "6": "11 (8); 16,5 (9,7)", "7": "", "8": "1890000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-5-2', 'ГИК-5-2', '{"3": "ГИК-5-2", "4": "кобальт-60", "5": "1", "6": "11 (8); 16,5 (9,7)", "7": "", "8": "3740000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-5-3', 'ГИК-5-3', '{"3": "ГИК-5-3", "4": "кобальт-60", "5": "1", "6": "11 (8); 16,5 (9,7)", "7": "", "8": "7400000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-5-4', 'ГИК-5-4', '{"3": "ГИК-5-4", "4": "кобальт-60", "5": "1", "6": "11 (8); 16,5 (9,7)", "7": "", "8": "18900000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-57', 'ГИК-57', '{"3": "ГИК-57", "4": "кобальт-57", "5": "1", "6": "10x10; 10x5; пластина 40x6", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "Подложка алюминий или бериллий", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-57-I', 'ГИК-57-I', '{"3": "ГИК-57-I", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "3300000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-57-II', 'ГИК-57-II', '{"3": "ГИК-57-II", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "780000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-57-III', 'ГИК-57-III', '{"3": "ГИК-57-III", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "2000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-6-1', 'ГИК-6-1', '{"3": "ГИК-6-1", "4": "кобальт-60", "5": "1", "6": "26,2 (21); 27 (20)", "7": "", "8": "16300000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-6-2', 'ГИК-6-2', '{"3": "ГИК-6-2", "4": "кобальт-60", "5": "1", "6": "26,2 (21); 27 (20)", "7": "", "8": "80000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-6-3', 'ГИК-6-3', '{"3": "ГИК-6-3", "4": "кобальт-60", "5": "1", "6": "26,2 (21); 27 (20)", "7": "", "8": "115000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-7-1', 'ГИК-7-1', '{"3": "ГИК-7-1", "4": "кобальт-60", "5": "1", "6": "11,2 (7,4); 81,5 (69)", "7": "", "8": "6800000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-7-2', 'ГИК-7-2', '{"3": "ГИК-7-2", "4": "кобальт-60", "5": "1", "6": "11,2 (7,4); 81,5 (69)", "7": "", "8": "21800000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-7-3', 'ГИК-7-3', '{"3": "ГИК-7-3", "4": "кобальт-60", "5": "1", "6": "11,2 (7,4); 81,5 (69)", "7": "", "8": "48100000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-7-4', 'ГИК-7-4', '{"3": "ГИК-7-4", "4": "кобальт-60", "5": "1", "6": "11,2 (7,4); 81,5 (69)", "7": "", "8": "98000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-7а-1', 'ГИК-7а-1', '{"3": "ГИК-7а-1", "4": "кобальт-60", "5": "1", "6": "12,8 (10); 86 (78,4)", "7": "", "8": "22600000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-7а-2', 'ГИК-7а-2', '{"3": "ГИК-7а-2", "4": "кобальт-60", "5": "1", "6": "12,8 (10); 86 (78,4)", "7": "", "8": "51800000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-7а-3', 'ГИК-7а-3', '{"3": "ГИК-7а-3", "4": "кобальт-60", "5": "1", "6": "12,8 (10); 86 (78,4)", "7": "", "8": "70000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-7а-4', 'ГИК-7а-4', '{"3": "ГИК-7а-4", "4": "кобальт-60", "5": "1", "6": "12,8 (10); 86 (78,4)", "7": "", "8": "98000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-7м-1', 'ГИК-7м-1', '{"3": "ГИК-7м-1", "4": "кобальт-60", "5": "1", "6": "11,2 (7,4); 81,5 (69)", "7": "", "8": "6800000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-7м-2', 'ГИК-7м-2', '{"3": "ГИК-7м-2", "4": "кобальт-60", "5": "1", "6": "11,2 (7,4); 81,5 (69)", "7": "", "8": "21800000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-7м-3', 'ГИК-7м-3', '{"3": "ГИК-7м-3", "4": "кобальт-60", "5": "1", "6": "11,2 (7,4); 81,5 (69)", "7": "", "8": "48100000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-7м-4', 'ГИК-7м-4', '{"3": "ГИК-7м-4", "4": "кобальт-60", "5": "1", "6": "11,2 (7,4); 81,5 (69)", "7": "", "8": "98000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-8-1', 'ГИК-8-1', '{"3": "ГИК-8-1", "4": "кобальт-60", "5": "1", "6": "23 (21); 22,5 (14,5)", "7": "", "8": "78000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-8-2', 'ГИК-8-2', '{"3": "ГИК-8-2", "4": "кобальт-60", "5": "1", "6": "23 (21); 22,5 (14,5)", "7": "", "8": "124000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-8-3', 'ГИК-8-3', '{"3": "ГИК-8-3", "4": "кобальт-60", "5": "1", "6": "23 (21); 22,5 (14,5)", "7": "", "8": "155000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-8-4', 'ГИК-8-4', '{"3": "ГИК-8-4", "4": "кобальт-60", "5": "1", "6": "23 (21); 22,5 (14,5)", "7": "", "8": "198000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "Для гамма-терапевтических аппаратов АГАТ-С.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-9', 'ГИК-9', '{"3": "ГИК-9", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "342000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-9-1', 'ГИК-9-1', '{"3": "ГИК-9-1", "4": "кобальт-60", "5": "1", "6": "23 (21); 33 (24,5)", "7": "", "8": "179000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-9-2', 'ГИК-9-2', '{"3": "ГИК-9-2", "4": "кобальт-60", "5": "1", "6": "23 (21); 33 (24,5)", "7": "", "8": "231000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-9-3', 'ГИК-9-3', '{"3": "ГИК-9-3", "4": "кобальт-60", "5": "1", "6": "23 (21); 33 (24,5)", "7": "", "8": "289000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "Для гамма-терапевтических аппаратов АГАТ-Р.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-9-4', 'ГИК-9-4', '{"3": "ГИК-9-4", "4": "кобальт-60", "5": "1", "6": "23 (21); 33 (24,5)", "7": "", "8": "363000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "Для гамма-терапевтических аппаратов АГАТ-Р.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-9-5', 'ГИК-9-5', '{"3": "ГИК-9-5", "4": "кобальт-60", "5": "1", "6": "23,0х33,0", "7": "0.0", "8": "323000000000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-VII', 'ГИК-VII', '{"3": "ГИК-VII", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "450000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-А2Н', 'ГИК-А2Н', '{"3": "ГИК-А2Н", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "105000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-А2С', 'ГИК-А2С', '{"3": "ГИК-А2С", "4": "кобальт-60", "5": "1", "6": "", "7": "210000000000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-А3', 'ГИК-А3', '{"3": "ГИК-А3", "4": "кобальт-60", "5": "1", "6": "11,0x451,0", "7": "", "8": "185000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-А3М', 'ГИК-А3М', '{"3": "ГИК-А3М", "4": "кобальт-60", "5": "1", "6": "11,0x451,0", "7": "", "8": "185000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-А5', 'ГИК-А5', '{"3": "ГИК-А5", "4": "кобальт-60", "5": "1", "6": "11,2х451,6", "7": "", "8": "185000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-А6', 'ГИК-А6', '{"3": "ГИК-А6", "4": "кобальт-60", "5": "1", "6": "11,1 max ; 451,6max", "7": "", "8": "359000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-А6М', 'ГИК-А6М', '{"3": "ГИК-А6М", "4": "кобальт-60", "5": "1", "6": "11,1 max ; 451,6max", "7": "", "8": "359000000000000.0", "9": "0", "10": "ПО Маяк", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-К-10', 'ГИК-К-10', '{"3": "ГИК-К-10", "4": "кобальт-60", "5": "1", "6": "3,0x14,0", "7": "0.0", "8": "6600000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-К-11', 'ГИК-К-11', '{"3": "ГИК-К-11", "4": "кобальт-60", "5": "1", "6": "3,0x14,0", "7": "0.0", "8": "13000000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-К-12', 'ГИК-К-12', '{"3": "ГИК-К-12", "4": "кобальт-60", "5": "1", "6": "3,0x14,0", "7": "0.0", "8": "26000000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-Р2 005', 'ГИК-Р2 005', '{"3": "ГИК-Р2 005", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "570000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-ТС5', 'ГИК-ТС5', '{"3": "ГИК-ТС5", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "4360000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-ФХИ-1-1', 'ГИК-ФХИ-1-1', '{"3": "ГИК-ФХИ-1-1", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "10800000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-ФХИ-1-2', 'ГИК-ФХИ-1-2', '{"3": "ГИК-ФХИ-1-2", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "13400000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИН-22-1', 'ГИН-22-1', '{"3": "ГИН-22-1", "4": "натрий-22", "5": "1", "6": "20,0х4,0", "7": "", "8": "40000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИН-22-2', 'ГИН-22-2', '{"3": "ГИН-22-2", "4": "натрий-22", "5": "1", "6": "20,0х4,0", "7": "", "8": "80000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИН-22-3', 'ГИН-22-3', '{"3": "ГИН-22-3", "4": "натрий-22", "5": "1", "6": "20,0х4,0", "7": "", "8": "200000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИН-22-4', 'ГИН-22-4', '{"3": "ГИН-22-4", "4": "натрий-22", "5": "1", "6": "20,0х4,0", "7": "", "8": "400000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИО-МАК-I', 'ГИО-МАК-I', '{"3": "ГИО-МАК-I", "4": "олово-119м", "5": "6", "6": "35,0х3,0", "7": "", "8": "20000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИО-МАК-II', 'ГИО-МАК-II', '{"3": "ГИО-МАК-II", "4": "олово-119м", "5": "6", "6": "35,0х3,0", "7": "", "8": "50000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИО-МАК-III', 'ГИО-МАК-III', '{"3": "ГИО-МАК-III", "4": "олово-119м", "5": "6", "6": "35,0х3,0", "7": "", "8": "200000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИТ-К', 'ГИТ-К', '{"3": "ГИТ-К", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИТ-К-10', 'ГИТ-К-10', '{"3": "ГИТ-К-10", "4": "кобальт-60", "5": "1", "6": "3 (1,4); 14 (9,9)", "7": "", "8": "6660000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "Для гамма-терапевтических аппаратов АГАТ-В.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИТ-К-11', 'ГИТ-К-11', '{"3": "ГИТ-К-11", "4": "кобальт-60", "5": "1", "6": "3 (1,4); 14 (9,9)", "7": "", "8": "13300000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "Для гамма-терапевтических аппаратов АГАТ-В.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИТ-К-12', 'ГИТ-К-12', '{"3": "ГИТ-К-12", "4": "кобальт-60", "5": "1", "6": "3 (1,4); 14 (9,9)", "7": "", "8": "26600000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "Для гамма-терапевтических аппаратов АГАТ-В.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИЦ-139', 'ГИЦ-139', '{"3": "ГИЦ-139", "4": "церий-139", "5": "1", "6": "10x10 или 10x5", "7": "190000000.0", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "Подложка алюминий или бериллий", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИЦ-139-П', 'ГИЦ-139-П', '{"3": "ГИЦ-139-П", "4": "церий-139", "5": "1", "6": "", "7": "", "8": "170000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИЦ-65', 'ГИЦ-65', '{"3": "ГИЦ-65", "4": "цинк-65", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИЦ-65-1', 'ГИЦ-65-1', '{"3": "ГИЦ-65-1", "4": "цинк-65", "5": "1", "6": "35,0х4,0", "7": "", "8": "200000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИЦ-65-2', 'ГИЦ-65-2', '{"3": "ГИЦ-65-2", "4": "цинк-65", "5": "1", "6": "35,0х4,0", "7": "", "8": "400000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57', 'ГК57', '{"3": "ГК57", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57.ВД4.01', 'ГК57.ВД4.01', '{"3": "ГК57.ВД4.01", "4": "кобальт-57", "5": "1", "6": "7,0x3,0", "7": "", "8": "500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57.ВД4.02', 'ГК57.ВД4.02', '{"3": "ГК57.ВД4.02", "4": "кобальт-57", "5": "1", "6": "7,0x3,0", "7": "", "8": "1000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57.ВД4.03', 'ГК57.ВД4.03', '{"3": "ГК57.ВД4.03", "4": "кобальт-57", "5": "1", "6": "7,0x3,0", "7": "", "8": "2500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57.ВД4.04', 'ГК57.ВД4.04', '{"3": "ГК57.ВД4.04", "4": "кобальт-57", "5": "1", "6": "7,0x3,0", "7": "", "8": "5000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57.ВД7.01', 'ГК57.ВД7.01', '{"3": "ГК57.ВД7.01", "4": "кобальт-57", "5": "1", "6": "12,0x3,0", "7": "", "8": "500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57.ВД7.02', 'ГК57.ВД7.02', '{"3": "ГК57.ВД7.02", "4": "кобальт-57", "5": "1", "6": "12,0x3,0", "7": "", "8": "1000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57.ВД7.03', 'ГК57.ВД7.03', '{"3": "ГК57.ВД7.03", "4": "кобальт-57", "5": "1", "6": "12,0x3,0", "7": "", "8": "2500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57.ВД7.04', 'ГК57.ВД7.04', '{"3": "ГК57.ВД7.04", "4": "кобальт-57", "5": "1", "6": "12,0x3,0", "7": "", "8": "5000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57.ВТ2.01', 'ГК57.ВТ2.01', '{"3": "ГК57.ВТ2.01", "4": "кобальт-57", "5": "1", "6": "3,0x3,0", "7": "", "8": "500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57.ВТ2.02', 'ГК57.ВТ2.02', '{"3": "ГК57.ВТ2.02", "4": "кобальт-57", "5": "1", "6": "3,0x3,0", "7": "", "8": "1000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57.ВТ2.03', 'ГК57.ВТ2.03', '{"3": "ГК57.ВТ2.03", "4": "кобальт-57", "5": "1", "6": "3,0x3,0", "7": "", "8": "2500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57.ВТ2.3', 'ГК57.ВТ2.3', '{"3": "ГК57.ВТ2.3", "4": "кобальт-57", "5": "1", "6": "8,0x140(Акт. часть - шарик 1мм)", "7": "", "8": "1000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57.ВТ2.4', 'ГК57.ВТ2.4', '{"3": "ГК57.ВТ2.4", "4": "кобальт-57", "5": "1", "6": "8,0x140(Акт. часть - шарик 1мм)", "7": "", "8": "2500000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57.ВТМ.1', 'ГК57.ВТМ.1', '{"3": "ГК57.ВТМ.1", "4": "кобальт-57", "5": "1", "6": "8,0x140(Акт. часть - шарик 1мм)", "7": "", "8": "250000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57.ВТМ.2', 'ГК57.ВТМ.2', '{"3": "ГК57.ВТМ.2", "4": "кобальт-57", "5": "1", "6": "8,0x140(Акт. часть - шарик 1мм)", "7": "", "8": "500000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57Г1.1', 'ГК57Г1.1', '{"3": "ГК57Г1.1", "4": "кобальт-57", "5": "1", "6": "10,0x5,0", "7": "", "8": "7400000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57Г1.2', 'ГК57Г1.2', '{"3": "ГК57Г1.2", "4": "кобальт-57", "5": "1", "6": "10,0x5,0", "7": "", "8": "19000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57Г1.3', 'ГК57Г1.3', '{"3": "ГК57Г1.3", "4": "кобальт-57", "5": "1", "6": "10,0x5,0", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57Г1.4', 'ГК57Г1.4', '{"3": "ГК57Г1.4", "4": "кобальт-57", "5": "1", "6": "10,0x5,0", "7": "", "8": "74000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57Г1.5', 'ГК57Г1.5', '{"3": "ГК57Г1.5", "4": "кобальт-57", "5": "1", "6": "10,0x5,0", "7": "", "8": "190000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57Г1.6', 'ГК57Г1.6', '{"3": "ГК57Г1.6", "4": "кобальт-57", "5": "1", "6": "10,0x5,0", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57Г1.7', 'ГК57Г1.7', '{"3": "ГК57Г1.7", "4": "кобальт-57", "5": "1", "6": "10,0x5,0", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57Г1.8', 'ГК57Г1.8', '{"3": "ГК57Г1.8", "4": "кобальт-57", "5": "1", "6": "10,0x5,0", "7": "", "8": "1900000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57Г1.9', 'ГК57Г1.9', '{"3": "ГК57Г1.9", "4": "кобальт-57", "5": "1", "6": "10,0x5,0", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57Г2.1', 'ГК57Г2.1', '{"3": "ГК57Г2.1", "4": "кобальт-57", "5": "1", "6": "5,0x3,0", "7": "", "8": "7400000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57Г2.2', 'ГК57Г2.2', '{"3": "ГК57Г2.2", "4": "кобальт-57", "5": "1", "6": "5,0x3,0", "7": "", "8": "19000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57Г2.3', 'ГК57Г2.3', '{"3": "ГК57Г2.3", "4": "кобальт-57", "5": "1", "6": "5,0x3,0", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57Г2.4', 'ГК57Г2.4', '{"3": "ГК57Г2.4", "4": "кобальт-57", "5": "1", "6": "5,0x3,0", "7": "", "8": "74000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57Г2.5', 'ГК57Г2.5', '{"3": "ГК57Г2.5", "4": "кобальт-57", "5": "1", "6": "5,0x3,0", "7": "", "8": "190000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57Г2.6', 'ГК57Г2.6', '{"3": "ГК57Г2.6", "4": "кобальт-57", "5": "1", "6": "5,0x3,0", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57Г2.7', 'ГК57Г2.7', '{"3": "ГК57Г2.7", "4": "кобальт-57", "5": "1", "6": "5,0x3,0", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57Г2.8', 'ГК57Г2.8', '{"3": "ГК57Г2.8", "4": "кобальт-57", "5": "1", "6": "5,0x3,0", "7": "", "8": "1900000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК57Г2.9', 'ГК57Г2.9', '{"3": "ГК57Г2.9", "4": "кобальт-57", "5": "1", "6": "5,0x3,0", "7": "", "8": "7400000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60', 'ГК60', '{"3": "ГК60", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М', 'ГК60М', '{"3": "ГК60М", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М11.105', 'ГК60М11.105', '{"3": "ГК60М11.105", "4": "кобальт-60", "5": "1", "6": "20; 25; 5; 10", "7": "", "8": "105000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "В капсуле из нержавеющей стали - C65444; в капсуле из титана - C65243", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М11.110', 'ГК60М11.110', '{"3": "ГК60М11.110", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "111000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М11.115', 'ГК60М11.115', '{"3": "ГК60М11.115", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "1500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М11.125', 'ГК60М11.125', '{"3": "ГК60М11.125", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "126000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М11.75', 'ГК60М11.75', '{"3": "ГК60М11.75", "4": "кобальт-60", "5": "1", "6": "20; 25; 5; 10", "7": "", "8": "75000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "В капсуле из нержавеющей стали - C65444; в капсуле из титана - C65243", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М11.85', 'ГК60М11.85', '{"3": "ГК60М11.85", "4": "кобальт-60", "5": "1", "6": "20; 25; 5; 10", "7": "", "8": "85000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "В капсуле из нержавеющей стали - C65444; в капсуле из титана - C65243", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М11.95', 'ГК60М11.95', '{"3": "ГК60М11.95", "4": "кобальт-60", "5": "1", "6": "20; 25; 5; 10", "7": "", "8": "95000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "В капсуле из нержавеющей стали - C65444; в капсуле из титана - C65243", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М12.105', 'ГК60М12.105', '{"3": "ГК60М12.105", "4": "кобальт-60", "5": "1", "6": "30; 35; 5; 20", "7": "", "8": "105000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "В капсуле из нержавеющей стали - C65444; в капсуле из титана - C65243", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М12.115', 'ГК60М12.115', '{"3": "ГК60М12.115", "4": "кобальт-60", "5": "1", "6": "30; 35; 5; 20", "7": "", "8": "115000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "В капсуле из нержавеющей стали - C65444; в капсуле из титана - C65243", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М12.120', 'ГК60М12.120', '{"3": "ГК60М12.120", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "122000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М12.85', 'ГК60М12.85', '{"3": "ГК60М12.85", "4": "кобальт-60", "5": "1", "6": "30; 35; 5; 20", "7": "", "8": "85000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "В капсуле из нержавеющей стали - C65444; в капсуле из титана - C65243", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М12.95', 'ГК60М12.95', '{"3": "ГК60М12.95", "4": "кобальт-60", "5": "1", "6": "30; 35; 5; 20", "7": "", "8": "95000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "В капсуле из нержавеющей стали - C65444; в капсуле из титана - C65243", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М22.11', 'ГК60М22.11', '{"3": "ГК60М22.11", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "74000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М22.111', 'ГК60М22.111', '{"3": "ГК60М22.111", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "74000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М313', 'ГК60М313', '{"3": "ГК60М313", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "7220000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М313.212', 'ГК60М313.212', '{"3": "ГК60М313.212", "4": "кобальт-60", "5": "1", "6": "6х18,5", "7": "0.0", "8": "1850000000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М313.411', 'ГК60М313.411', '{"3": "ГК60М313.411", "4": "кобальт-60", "5": "1", "6": "6х18,5", "7": "0.0", "8": "370000000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М313.412', 'ГК60М313.412', '{"3": "ГК60М313.412", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3760000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М313.4121', 'ГК60М313.4121', '{"3": "ГК60М313.4121", "4": "кобальт-60", "5": "1", "6": "6х18,5", "7": "0.0", "8": "3700000000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М313.9', 'ГК60М313.9', '{"3": "ГК60М313.9", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3700000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М313.911', 'ГК60М313.911', '{"3": "ГК60М313.911", "4": "кобальт-60", "5": "1", "6": "6х18,5", "7": "0.0", "8": "925000000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М324', 'ГК60М324', '{"3": "ГК60М324", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "16000000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М324.213', 'ГК60М324.213', '{"3": "ГК60М324.213", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "15900000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М324.412', 'ГК60М324.412', '{"3": "ГК60М324.412", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3700000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М324.712', 'ГК60М324.712', '{"3": "ГК60М324.712", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "8600000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М41.207', 'ГК60М41.207', '{"3": "ГК60М41.207", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "200000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "В капсуле из нержавеющей стали - C65445; в капсуле из титана - C65444", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М41.257', 'ГК60М41.257', '{"3": "ГК60М41.257", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "250000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "В капсуле из нержавеющей стали - C65445; в капсуле из титана - C65444", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М41.307', 'ГК60М41.307', '{"3": "ГК60М41.307", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "300000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "В капсуле из нержавеющей стали - C65445; в капсуле из титана - C65444", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М41.357', 'ГК60М41.357', '{"3": "ГК60М41.357", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "350000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "В капсуле из нержавеющей стали - C65445; в капсуле из титана - C65444", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М61.29', 'ГК60М61.29', '{"3": "ГК60М61.29", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "2290000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М61.39', 'ГК60М61.39', '{"3": "ГК60М61.39", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3020000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60Р', 'ГК60Р', '{"3": "ГК60Р", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "5590000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60Р01', 'ГК60Р01', '{"3": "ГК60Р01", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "20500000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60Р01.01', 'ГК60Р01.01', '{"3": "ГК60Р01.01", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "20700000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1У5', '1У5', '{"3": "1У5", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60Р02.01', 'ГК60Р02.01', '{"3": "ГК60Р02.01", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3880000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60Р02.02', 'ГК60Р02.02', '{"3": "ГК60Р02.02", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "3960000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60Р02.04', 'ГК60Р02.04', '{"3": "ГК60Р02.04", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "1150000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60Р03', 'ГК60Р03', '{"3": "ГК60Р03", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "42900000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60Р03.02', 'ГК60Р03.02', '{"3": "ГК60Р03.02", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "9990000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60Р03.03', 'ГК60Р03.03', '{"3": "ГК60Р03.03", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "141000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60Р03.04', 'ГК60Р03.04', '{"3": "ГК60Р03.04", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "58500000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60Р03.05', 'ГК60Р03.05', '{"3": "ГК60Р03.05", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "6360000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60Р04', 'ГК60Р04', '{"3": "ГК60Р04", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "1850000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60РС', 'ГК60РС', '{"3": "ГК60РС", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "38900000000000.0", "9": "0", "10": "НИИАР", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60Т03', 'ГК60Т03', '{"3": "ГК60Т03", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "488000000000000.0", "9": "0", "10": "НИИАР", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60Т04', 'ГК60Т04', '{"3": "ГК60Т04", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "203000000000000.0", "9": "0", "10": "НИИАР", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60Т04-1', 'ГК60Т04-1', '{"3": "ГК60Т04-1", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "182000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60Т04-3', 'ГК60Т04-3', '{"3": "ГК60Т04-3", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "261000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60Т05', 'ГК60Т05', '{"3": "ГК60Т05", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "304000000000000.0", "9": "0", "10": "НИИАР", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60Т05-1', 'ГК60Т05-1', '{"3": "ГК60Т05-1", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "184000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60Т05-2', 'ГК60Т05-2', '{"3": "ГК60Т05-2", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "276000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60Т05-3', 'ГК60Т05-3', '{"3": "ГК60Т05-3", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "289000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60Т06', 'ГК60Т06', '{"3": "ГК60Т06", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "373000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60ТВ4', 'ГК60ТВ4', '{"3": "ГК60ТВ4", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "33600000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60ТВ51', 'ГК60ТВ51', '{"3": "ГК60ТВ51", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "338000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60ТВ6', 'ГК60ТВ6', '{"3": "ГК60ТВ6", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "413000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60ТВ9', 'ГК60ТВ9', '{"3": "ГК60ТВ9", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "13500000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ', 'ГНТ', '{"3": "ГНТ", "4": "тритий", "5": "4", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ1-143', 'ГНТ1-143', '{"3": "ГНТ1-143", "4": "тритий", "5": "4", "6": "", "7": "", "8": "740000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ1-143(L)', 'ГНТ1-143(L)', '{"3": "ГНТ1-143(L)", "4": "тритий", "5": "4", "6": "", "7": "", "8": "740000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ1-143(а)', 'ГНТ1-143(а)', '{"3": "ГНТ1-143(а)", "4": "тритий", "5": "4", "6": "", "7": "", "8": "74000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ1-143L', 'ГНТ1-143L', '{"3": "ГНТ1-143L", "4": "тритий", "5": "0", "6": "", "7": "740000000000.0", "8": "0.0", "9": "0", "10": "ВНИИА", "11": "0", "12": "07623885_01 ЗРИ 150509 270509", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ1-32', 'ГНТ1-32', '{"3": "ГНТ1-32", "4": "тритий", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "ВНИИА", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ1-32М', 'ГНТ1-32М', '{"3": "ГНТ1-32М", "4": "тритий", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ1-42', 'ГНТ1-42', '{"3": "ГНТ1-42", "4": "тритий", "5": "4", "6": "", "7": "", "8": "740000000000.0", "9": "0", "10": "ВНИИА", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ1-42М', 'ГНТ1-42М', '{"3": "ГНТ1-42М", "4": "тритий", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "ВНИИА", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ1-88', 'ГНТ1-88', '{"3": "ГНТ1-88", "4": "тритий", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "ВНИИА", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ-191', 'ГНТ-191', '{"3": "ГНТ-191", "4": "тритий", "5": "4", "6": "", "7": "", "8": "740000000000.0", "9": "0", "10": "ВНИИА", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ1-МД', 'ГНТ1-МД', '{"3": "ГНТ1-МД", "4": "тритий", "5": "4", "6": "", "7": "740000000000.0", "8": "0.0", "9": "0", "10": "ВНИИА", "11": "5", "12": "07623885_01 ЗРИ 150509 270509", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ2-26', 'ГНТ2-26', '{"3": "ГНТ2-26", "4": "тритий", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "ВНИИА", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ2-42', 'ГНТ2-42', '{"3": "ГНТ2-42", "4": "тритий", "5": "4", "6": "", "7": "", "8": "740000000000.0", "9": "0", "10": "ВНИИА", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ2-42м', 'ГНТ2-42м', '{"3": "ГНТ2-42м", "4": "тритий", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ3-29', 'ГНТ3-29', '{"3": "ГНТ3-29", "4": "тритий", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ3-40', 'ГНТ3-40', '{"3": "ГНТ3-40", "4": "тритий", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "ВНИИА", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ3-67', 'ГНТ3-67', '{"3": "ГНТ3-67", "4": "тритий", "5": "4", "6": "", "7": "", "8": "740000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ3-67-01', 'ГНТ3-67-01', '{"3": "ГНТ3-67-01", "4": "тритий", "5": "4", "6": "", "7": "", "8": "740000000000.0", "9": "0", "10": "ВНИИА", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ3-67М', 'ГНТ3-67М', '{"3": "ГНТ3-67М", "4": "тритий", "5": "4", "6": "", "7": "", "8": "740000000000.0", "9": "0", "10": "ВНИИА", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ5-67', 'ГНТ5-67', '{"3": "ГНТ5-67", "4": "тритий", "5": "4", "6": "", "7": "", "8": "740000000000.0", "9": "0", "10": "ВНИИА", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГО119мГ', 'ГО119мГ', '{"3": "ГО119мГ", "4": "олово-119м", "5": "6", "6": "20,0x12,0", "7": "37000000.0", "8": "370000000.0", "9": "0", "10": "", "11": "", "12": "Титановая капсула", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГО119мГ3.1', 'ГО119мГ3.1', '{"3": "ГО119мГ3.1", "4": "олово-119м", "5": "5", "6": "10,0x2,0", "7": "", "8": "190000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГО119мГ3.2', 'ГО119мГ3.2', '{"3": "ГО119мГ3.2", "4": "олово-119м", "5": "5", "6": "10,0x2,0", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГО119мГ3.3', 'ГО119мГ3.3', '{"3": "ГО119мГ3.3", "4": "олово-119м", "5": "5", "6": "10,0x2,0", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГО-21', 'ГО-21', '{"3": "ГО-21", "4": "цезий-137", "5": "2", "6": "120,0х140,0", "7": "0.0", "8": "370000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГРа6', 'ГРа6', '{"3": "ГРа6", "4": "радий-226", "5": "1", "6": "", "7": "", "8": "3710000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС7', 'ГС7', '{"3": "ГС7", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС7.012.1', 'ГС7.012.1', '{"3": "ГС7.012.1", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "6000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС7.012.11', 'ГС7.012.11', '{"3": "ГС7.012.11", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "4200000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС7.012.4', 'ГС7.012.4', '{"3": "ГС7.012.4", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1280000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС7.012.5', 'ГС7.012.5', '{"3": "ГС7.012.5", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "120000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС7.012.6', 'ГС7.012.6', '{"3": "ГС7.012.6", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "300000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС7.012.8', 'ГС7.012.8', '{"3": "ГС7.012.8", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "620000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС7.02.14', 'ГС7.02.14', '{"3": "ГС7.02.14", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "61100000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС7.02.8', 'ГС7.02.8', '{"3": "ГС7.02.8", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "630000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС7.021', 'ГС7.021', '{"3": "ГС7.021", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "6280000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС7.021.6', 'ГС7.021.6', '{"3": "ГС7.021.6", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "240000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС7.029.2', 'ГС7.029.2', '{"3": "ГС7.029.2", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "6400000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС7.12', 'ГС7.12', '{"3": "ГС7.12", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "160000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М11', 'ГС75М11', '{"3": "ГС75М11", "4": "селен-75", "5": "1", "6": "от 1x1 до 3,5x3,5", "7": "111000000000.0", "8": "5200000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М11.10', 'ГС75М11.10', '{"3": "ГС75М11.10", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "248000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М11.111', 'ГС75М11.111', '{"3": "ГС75М11.111", "4": "селен-75", "5": "1", "6": "1x1", "7": "", "8": "111000000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М11.112', 'ГС75М11.112', '{"3": "ГС75М11.112", "4": "селен-75", "5": "1", "6": "2,5x2,5", "7": "", "8": "1480000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М11.140', 'ГС75М11.140', '{"3": "ГС75М11.140", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "5070000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М11.20', 'ГС75М11.20', '{"3": "ГС75М11.20", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "7400000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М11.212', 'ГС75М11.212', '{"3": "ГС75М11.212", "4": "селен-75", "5": "1", "6": "6,0x12,0", "7": "", "8": "2200000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М11.311', 'ГС75М11.311', '{"3": "ГС75М11.311", "4": "селен-75", "5": "1", "6": "1,5x1,5", "7": "", "8": "296000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М11.312', 'ГС75М11.312', '{"3": "ГС75М11.312", "4": "селен-75", "5": "1", "6": "2,8x2,8", "7": "", "8": "2800000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М11.40', 'ГС75М11.40', '{"3": "ГС75М11.40", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "9950000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М11.411', 'ГС75М11.411', '{"3": "ГС75М11.411", "4": "селен-75", "5": "1", "6": "6,0x12,0", "7": "", "8": "370000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М11.412', 'ГС75М11.412', '{"3": "ГС75М11.412", "4": "селен-75", "5": "1", "6": "3x3", "7": "", "8": "3700000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М11.512', 'ГС75М11.512', '{"3": "ГС75М11.512", "4": "селен-75", "5": "1", "6": "3,5x3,5", "7": "", "8": "5200000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М11.611', 'ГС75М11.611', '{"3": "ГС75М11.611", "4": "селен-75", "5": "1", "6": "1,8x1,8", "7": "", "8": "560000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М11.711', 'ГС75М11.711', '{"3": "ГС75М11.711", "4": "селен-75", "5": "1", "6": "2x2", "7": "", "8": "740000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М11.811', 'ГС75М11.811', '{"3": "ГС75М11.811", "4": "селен-75", "5": "1", "6": "6,0x12,0", "7": "", "8": "830000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М11.90', 'ГС75М11.90', '{"3": "ГС75М11.90", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "3320000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М11.910', 'ГС75М11.910', '{"3": "ГС75М11.910", "4": "селен-75", "5": "1", "6": "6,0x12,0", "7": "", "8": "90000000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М12', 'ГС75М12', '{"3": "ГС75М12", "4": "селен-75", "5": "1", "6": "от 1x1 до 3,5x3,5", "7": "111000000000.0", "8": "5200000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М12.111', 'ГС75М12.111', '{"3": "ГС75М12.111", "4": "селен-75", "5": "1", "6": "1x1", "7": "", "8": "111000000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М12.112', 'ГС75М12.112', '{"3": "ГС75М12.112", "4": "селен-75", "5": "1", "6": "2,5x2,5", "7": "", "8": "1480000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М12.140', 'ГС75М12.140', '{"3": "ГС75М12.140", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "4550000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М12.212', 'ГС75М12.212', '{"3": "ГС75М12.212", "4": "селен-75", "5": "1", "6": "6x20(М3)", "7": "", "8": "2200000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М12.311', 'ГС75М12.311', '{"3": "ГС75М12.311", "4": "селен-75", "5": "1", "6": "1,5x1,5", "7": "", "8": "296000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М12.312', 'ГС75М12.312', '{"3": "ГС75М12.312", "4": "селен-75", "5": "1", "6": "2,8x2,8", "7": "", "8": "2800000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М12.411', 'ГС75М12.411', '{"3": "ГС75М12.411", "4": "селен-75", "5": "1", "6": "6x20(М3)", "7": "", "8": "370000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М12.412', 'ГС75М12.412', '{"3": "ГС75М12.412", "4": "селен-75", "5": "1", "6": "3x3", "7": "", "8": "3700000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М12.451', 'ГС75М12.451', '{"3": "ГС75М12.451", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "2960000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М12.512', 'ГС75М12.512', '{"3": "ГС75М12.512", "4": "селен-75", "5": "1", "6": "3,5x3,5", "7": "", "8": "5200000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М12.611', 'ГС75М12.611', '{"3": "ГС75М12.611", "4": "селен-75", "5": "1", "6": "1,8x1,8", "7": "", "8": "560000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М12.711', 'ГС75М12.711', '{"3": "ГС75М12.711", "4": "селен-75", "5": "1", "6": "2x2", "7": "", "8": "740000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М12.712', 'ГС75М12.712', '{"3": "ГС75М12.712", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "1140000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М12.811', 'ГС75М12.811', '{"3": "ГС75М12.811", "4": "селен-75", "5": "1", "6": "6x20(М3)", "7": "", "8": "830000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М12.910', 'ГС75М12.910', '{"3": "ГС75М12.910", "4": "селен-75", "5": "1", "6": "6x20(М3)", "7": "", "8": "90000000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М13', 'ГС75М13', '{"3": "ГС75М13", "4": "селен-75", "5": "1", "6": "от 1x1 до 3,5x3,5", "7": "111000000000.0", "8": "5200000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М13.111', 'ГС75М13.111', '{"3": "ГС75М13.111", "4": "селен-75", "5": "1", "6": "1x1", "7": "", "8": "111000000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М13.112', 'ГС75М13.112', '{"3": "ГС75М13.112", "4": "селен-75", "5": "1", "6": "2,5x2,5", "7": "", "8": "1480000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М13.140', 'ГС75М13.140', '{"3": "ГС75М13.140", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "5070000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М13.212', 'ГС75М13.212', '{"3": "ГС75М13.212", "4": "селен-75", "5": "1", "6": "6,0x20,0", "7": "", "8": "2200000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М13.311', 'ГС75М13.311', '{"3": "ГС75М13.311", "4": "селен-75", "5": "1", "6": "1,5x1,5", "7": "", "8": "296000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М13.312', 'ГС75М13.312', '{"3": "ГС75М13.312", "4": "селен-75", "5": "1", "6": "2,8x2,8", "7": "", "8": "2800000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М13.411', 'ГС75М13.411', '{"3": "ГС75М13.411", "4": "селен-75", "5": "1", "6": "6,0x20,0", "7": "", "8": "370000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М13.412', 'ГС75М13.412', '{"3": "ГС75М13.412", "4": "селен-75", "5": "1", "6": "3x3", "7": "", "8": "3700000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М13.512', 'ГС75М13.512', '{"3": "ГС75М13.512", "4": "селен-75", "5": "1", "6": "3,5x3,5", "7": "", "8": "5200000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М13.611', 'ГС75М13.611', '{"3": "ГС75М13.611", "4": "селен-75", "5": "1", "6": "1,8x1,8", "7": "", "8": "560000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М13.711', 'ГС75М13.711', '{"3": "ГС75М13.711", "4": "селен-75", "5": "1", "6": "2x2", "7": "", "8": "740000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М13.811', 'ГС75М13.811', '{"3": "ГС75М13.811", "4": "селен-75", "5": "1", "6": "6,0x20,0", "7": "", "8": "830000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М13.90', 'ГС75М13.90', '{"3": "ГС75М13.90", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "3140000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М13.910', 'ГС75М13.910', '{"3": "ГС75М13.910", "4": "селен-75", "5": "1", "6": "6,0x20,0", "7": "", "8": "90000000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М14', 'ГС75М14', '{"3": "ГС75М14", "4": "селен-75", "5": "1", "6": "от 1x1 до 3,5x3,5", "7": "111000000000.0", "8": "5200000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М14.111', 'ГС75М14.111', '{"3": "ГС75М14.111", "4": "селен-75", "5": "1", "6": "1x1", "7": "", "8": "111000000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М14.112', 'ГС75М14.112', '{"3": "ГС75М14.112", "4": "селен-75", "5": "1", "6": "2,5x2,5", "7": "", "8": "1480000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М14.140', 'ГС75М14.140', '{"3": "ГС75М14.140", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "4550000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М14.200', 'ГС75М14.200', '{"3": "ГС75М14.200", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "6110000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГЦ65Г2.4', 'ГЦ65Г2.4', '{"3": "ГЦ65Г2.4", "4": "цинк-65", "5": "1", "6": "5,0x3,0", "7": "", "8": "200000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М14.212', 'ГС75М14.212', '{"3": "ГС75М14.212", "4": "селен-75", "5": "1", "6": "6,0x27,0", "7": "", "8": "2200000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М14.311', 'ГС75М14.311', '{"3": "ГС75М14.311", "4": "селен-75", "5": "1", "6": "1,5x1,5", "7": "", "8": "296000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М14.312', 'ГС75М14.312', '{"3": "ГС75М14.312", "4": "селен-75", "5": "1", "6": "2,8x2,8", "7": "", "8": "2800000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М14.411', 'ГС75М14.411', '{"3": "ГС75М14.411", "4": "селен-75", "5": "1", "6": "6,0x27,0", "7": "", "8": "370000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М14.412', 'ГС75М14.412', '{"3": "ГС75М14.412", "4": "селен-75", "5": "1", "6": "3x3", "7": "", "8": "3700000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М14.512', 'ГС75М14.512', '{"3": "ГС75М14.512", "4": "селен-75", "5": "1", "6": "3,5x3,5", "7": "", "8": "5200000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М14.611', 'ГС75М14.611', '{"3": "ГС75М14.611", "4": "селен-75", "5": "1", "6": "1,8x1,8", "7": "", "8": "560000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М14.711', 'ГС75М14.711', '{"3": "ГС75М14.711", "4": "селен-75", "5": "1", "6": "2x2", "7": "", "8": "740000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М14.811', 'ГС75М14.811', '{"3": "ГС75М14.811", "4": "селен-75", "5": "1", "6": "6,0x27,0", "7": "", "8": "830000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М14.910', 'ГС75М14.910', '{"3": "ГС75М14.910", "4": "селен-75", "5": "1", "6": "6,0x27,0", "7": "", "8": "90000000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСOO.061.1', 'ГСOO.061.1', '{"3": "ГСOO.061.1", "4": "кобальт-60", "5": "1", "6": "0,8x5,0 с тросиками 200, 700, 1000м", "7": "0.0", "8": "22100000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСOO.061.2', 'ГСOO.061.2', '{"3": "ГСOO.061.2", "4": "кобальт-60", "5": "1", "6": "0,8x5,0 с тросиками 200, 700, 1000м", "7": "0.0", "8": "44200000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСOO.061.3', 'ГСOO.061.3', '{"3": "ГСOO.061.3", "4": "кобальт-60", "5": "1", "6": "0,8x5,0 с тросиками 200, 700, 1000м", "7": "0.0", "8": "88500000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСOO.061.4', 'ГСOO.061.4', '{"3": "ГСOO.061.4", "4": "кобальт-60", "5": "1", "6": "0,8x5,0 с тросиками 200, 700, 1000м", "7": "0.0", "8": "178000000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСS7.012.1', 'ГСS7.012.1', '{"3": "ГСS7.012.1", "4": "цезий-137", "5": "2", "6": "", "7": "0.0", "8": "24000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО0.061', 'ГСО0.061', '{"3": "ГСО0.061", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО0.061.1', 'ГСО0.061.1', '{"3": "ГСО0.061.1", "4": "кобальт-60", "5": "1", "6": "1,6 (0,8); 11,5 (5)", "7": "", "8": "22800000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "Назначенный срок службы источников 5 лет с даты их выпуска при общем количестве циклов перемещения (посылка и возврат) не более 5000.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО0.061.2', 'ГСО0.061.2', '{"3": "ГСО0.061.2", "4": "кобальт-60", "5": "1", "6": "1,6 (0,8); 11,5 (5)", "7": "", "8": "45600000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "Назначенный срок службы источников 5 лет с даты их выпуска при общем количестве циклов перемещения (посылка и возврат) не более 5000.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО0.061.3', 'ГСО0.061.3', '{"3": "ГСО0.061.3", "4": "кобальт-60", "5": "1", "6": "1,6 (0,8); 11,5 (5)", "7": "", "8": "91000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "Назначенный срок службы источников 5 лет с даты их выпуска при общем количестве циклов перемещения (посылка и возврат) не более 5000.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО0.061.4', 'ГСО0.061.4', '{"3": "ГСО0.061.4", "4": "кобальт-60", "5": "1", "6": "1,6 (0,8); 11,5 (5)", "7": "", "8": "182000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "Назначенный срок службы источников 5 лет с даты их выпуска при общем количестве циклов перемещения (посылка и возврат) не более 5000.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГЦ-139Г2.3', 'ГЦ-139Г2.3', '{"3": "ГЦ-139Г2.3", "4": "церий-139", "5": "1", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГЦ65', 'ГЦ65', '{"3": "ГЦ65", "4": "цинк-65", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГЦ65Г2.1', 'ГЦ65Г2.1', '{"3": "ГЦ65Г2.1", "4": "цинк-65", "5": "1", "6": "5,0x3,0", "7": "", "8": "20000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГЦ65Г2.2', 'ГЦ65Г2.2', '{"3": "ГЦ65Г2.2", "4": "цинк-65", "5": "1", "6": "5,0x3,0", "7": "", "8": "40000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГЦ65Г2.3', 'ГЦ65Г2.3', '{"3": "ГЦ65Г2.3", "4": "цинк-65", "5": "1", "6": "5,0x3,0", "7": "", "8": "100000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГЦ65Г2.5', 'ГЦ65Г2.5', '{"3": "ГЦ65Г2.5", "4": "цинк-65", "5": "1", "6": "5,0x3,0", "7": "", "8": "370000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГЦ65ГД2.1', 'ГЦ65ГД2.1', '{"3": "ГЦ65ГД2.1", "4": "цинк-65", "5": "1", "6": "35,0x4,0", "7": "", "8": "20000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГЦ65ГД2.2', 'ГЦ65ГД2.2', '{"3": "ГЦ65ГД2.2", "4": "цинк-65", "5": "1", "6": "35,0x4,0", "7": "", "8": "40000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГЦ65ГД2.3', 'ГЦ65ГД2.3', '{"3": "ГЦ65ГД2.3", "4": "цинк-65", "5": "1", "6": "35,0x4,0", "7": "", "8": "100000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГЦ65ГД2.4', 'ГЦ65ГД2.4', '{"3": "ГЦ65ГД2.4", "4": "цинк-65", "5": "1", "6": "35,0x4,0", "7": "", "8": "200000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГЦ65ГД2.5', 'ГЦ65ГД2.5', '{"3": "ГЦ65ГД2.5", "4": "цинк-65", "5": "1", "6": "35,0x4,0", "7": "", "8": "370000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЕР-1', 'ЕР-1', '{"3": "ЕР-1", "4": "", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЕР-14', 'ЕР-14', '{"3": "ЕР-14", "4": "радий-226", "5": "3", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЕР-15', 'ЕР-15', '{"3": "ЕР-15", "4": "радий-226", "5": "3", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЗРИ 010488', 'ЗРИ 010488', '{"3": "ЗРИ 010488", "4": "плутоний-239", "5": "3", "6": "", "7": "0.0", "8": "97000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЗРИ 01291286', 'ЗРИ 01291286', '{"3": "ЗРИ 01291286", "4": "радий-226", "5": "3", "6": "", "7": "0.0", "8": "83.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЗРИ 02291286', 'ЗРИ 02291286', '{"3": "ЗРИ 02291286", "4": "радий-226", "5": "3", "6": "", "7": "0.0", "8": "130.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЗРИ 03291286', 'ЗРИ 03291286', '{"3": "ЗРИ 03291286", "4": "радий-226", "5": "3", "6": "", "7": "0.0", "8": "979.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЗРИ 101', 'ЗРИ 101', '{"3": "ЗРИ 101", "4": "олово-119м", "5": "1", "6": "", "7": "0.0", "8": "1100000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЗРИ 180488', 'ЗРИ 180488', '{"3": "ЗРИ 180488", "4": "плутоний-239", "5": "3", "6": "", "7": "0.0", "8": "97000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЗРИ 210488', 'ЗРИ 210488', '{"3": "ЗРИ 210488", "4": "плутоний-239", "5": "3", "6": "", "7": "0.0", "8": "97000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЗРИ 300388', 'ЗРИ 300388', '{"3": "ЗРИ 300388", "4": "плутоний-239", "5": "3", "6": "", "7": "0.0", "8": "10000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЗРИ 3003881', 'ЗРИ 3003881', '{"3": "ЗРИ 3003881", "4": "плутоний-239", "5": "3", "6": "", "7": "0.0", "8": "10000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЗРИ 4140', 'ЗРИ 4140', '{"3": "ЗРИ 4140", "4": "цезий-137", "5": "2", "6": "", "7": "0.0", "8": "8570.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЗРИ 5730', 'ЗРИ 5730', '{"3": "ЗРИ 5730", "4": "олово-119м", "5": "1", "6": "", "7": "0.0", "8": "1200000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЗРИ 5731', 'ЗРИ 5731', '{"3": "ЗРИ 5731", "4": "олово-119м", "5": "1", "6": "", "7": "0.0", "8": "1200000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЗРИ 67', 'ЗРИ 67', '{"3": "ЗРИ 67", "4": "олово-119м", "5": "1", "6": "", "7": "0.0", "8": "1000000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИА 1', 'ИА 1', '{"3": "ИА 1", "4": "америций-241", "5": "3", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63', 'ИБИРЗН-63', '{"3": "ИБИРЗН-63", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "120000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63 1', 'ИБИРЗН-63 1', '{"3": "ИБИРЗН-63 1", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "1200000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63 1а', 'ИБИРЗН-63 1а', '{"3": "ИБИРЗН-63 1а", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "800000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63 1а-1', 'ИБИРЗН-63 1а-1', '{"3": "ИБИРЗН-63 1а-1", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "600000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63 2а', 'ИБИРЗН-63 2а', '{"3": "ИБИРЗН-63 2а", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "600000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63 2б', 'ИБИРЗН-63 2б', '{"3": "ИБИРЗН-63 2б", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "206000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63 2б-1', 'ИБИРЗН-63 2б-1', '{"3": "ИБИРЗН-63 2б-1", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "1200000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63 I', 'ИБИРЗН-63 I', '{"3": "ИБИРЗН-63 I", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "1200000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63 II', 'ИБИРЗН-63 II', '{"3": "ИБИРЗН-63 II", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "1200000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63 IIa', 'ИБИРЗН-63 IIa', '{"3": "ИБИРЗН-63 IIa", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "600000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63 IIб', 'ИБИРЗН-63 IIб', '{"3": "ИБИРЗН-63 IIб", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "5550000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63 IIб-1', 'ИБИРЗН-63 IIб-1', '{"3": "ИБИРЗН-63 IIб-1", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "24000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63 IIб-I', 'ИБИРЗН-63 IIб-I', '{"3": "ИБИРЗН-63 IIб-I", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "1200000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63 Iа', 'ИБИРЗН-63 Iа', '{"3": "ИБИРЗН-63 Iа", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "1200000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63 Iб', 'ИБИРЗН-63 Iб', '{"3": "ИБИРЗН-63 Iб", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "1200000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63 тип 1', 'ИБИРЗН-63 тип 1', '{"3": "ИБИРЗН-63 тип 1", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "1200000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63 тип 2б', 'ИБИРЗН-63 тип 2б', '{"3": "ИБИРЗН-63 тип 2б", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "1280000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63 тип 2б-1', 'ИБИРЗН-63 тип 2б-1', '{"3": "ИБИРЗН-63 тип 2б-1", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "1200000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63 тип IIб', 'ИБИРЗН-63 тип IIб', '{"3": "ИБИРЗН-63 тип IIб", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "1200000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63 тип IIб-1', 'ИБИРЗН-63 тип IIб-1', '{"3": "ИБИРЗН-63 тип IIб-1", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "1200000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63 тип а', 'ИБИРЗН-63 тип а', '{"3": "ИБИРЗН-63 тип а", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "1200000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63-1', 'ИБИРЗН-63-1', '{"3": "ИБИРЗН-63-1", "4": "никель-63", "5": "2", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63-2а', 'ИБИРЗН-63-2а', '{"3": "ИБИРЗН-63-2а", "4": "никель-63", "5": "2", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63-2б', 'ИБИРЗН-63-2б', '{"3": "ИБИРЗН-63-2б", "4": "никель-63", "5": "2", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63-2б-1', 'ИБИРЗН-63-2б-1', '{"3": "ИБИРЗН-63-2б-1", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63-2би', 'ИБИРЗН-63-2би', '{"3": "ИБИРЗН-63-2би", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63-I', 'ИБИРЗН-63-I', '{"3": "ИБИРЗН-63-I", "4": "никель-63", "5": "2", "6": "15x10x0,05", "7": "300000000.0", "8": "3000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63-IIа', 'ИБИРЗН-63-IIа', '{"3": "ИБИРЗН-63-IIа", "4": "никель-63", "5": "2", "6": "30x10x0,05", "7": "300000000.0", "8": "3000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63-IIб', 'ИБИРЗН-63-IIб', '{"3": "ИБИРЗН-63-IIб", "4": "никель-63", "5": "2", "6": "30x15x0,05", "7": "300000000.0", "8": "3000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-1', 'ИБН-1', '{"3": "ИБН-1", "4": "плутоний-239", "5": "4", "6": "10 (3); 19 (3)", "7": "", "8": "250000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-10', 'ИБН-10', '{"3": "ИБН-10", "4": "плутоний-239", "5": "4", "6": "35 (27); 45 (27)", "7": "", "8": "250000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-11', 'ИБН-11', '{"3": "ИБН-11", "4": "плутоний-239", "5": "4", "6": "42 (34); 52 (34)", "7": "", "8": "500000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-12', 'ИБН-12', '{"3": "ИБН-12", "4": "плутоний-239", "5": "4", "6": "54 (46); 64 (46)", "7": "", "8": "1300000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-13', 'ИБН-13', '{"3": "ИБН-13", "4": "плутоний-239", "5": "4", "6": "10 (2); 13 (2)", "7": "", "8": "25000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-14', 'ИБН-14', '{"3": "ИБН-14", "4": "плутоний-239", "5": "4", "6": "10 (2); 13 (2)", "7": "", "8": "50000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-15', 'ИБН-15', '{"3": "ИБН-15", "4": "плутоний-239", "5": "4", "6": "10 (3); 13 (3)", "7": "", "8": "130000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-16', 'ИБН-16', '{"3": "ИБН-16", "4": "плутоний-239", "5": "4", "6": "10 (3); 13 (3)", "7": "", "8": "250000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-17', 'ИБН-17', '{"3": "ИБН-17", "4": "плутоний-239", "5": "4", "6": "12 (4); 16 (4)", "7": "", "8": "500000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-18', 'ИБН-18', '{"3": "ИБН-18", "4": "плутоний-239", "5": "4", "6": "12 (5); 16 (5)", "7": "", "8": "1200000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-19', 'ИБН-19', '{"3": "ИБН-19", "4": "плутоний-239", "5": "4", "6": "15 (6); 18 (6)", "7": "", "8": "2500000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-2', 'ИБН-2', '{"3": "ИБН-2", "4": "плутоний-239", "5": "4", "6": "12 (4); 22 (4)", "7": "", "8": "500000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-20', 'ИБН-20', '{"3": "ИБН-20", "4": "плутоний-239", "5": "4", "6": "15 (8); 20 (8)", "7": "", "8": "5000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-21', 'ИБН-21', '{"3": "ИБН-21", "4": "плутоний-239", "5": "4", "6": "18 (10); 22 (10)", "7": "", "8": "12000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-22', 'ИБН-22', '{"3": "ИБН-22", "4": "плутоний-239", "5": "4", "6": "21 (13); 25 (13)", "7": "", "8": "25000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-23', 'ИБН-23', '{"3": "ИБН-23", "4": "плутоний-239", "5": "4", "6": "24 (16); 30 (16)", "7": "", "8": "50000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-24', 'ИБН-24', '{"3": "ИБН-24", "4": "плутоний-239", "5": "4", "6": "29 (21); 33 (21)", "7": "", "8": "120000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-10-1', 'ИБН-241-10-1', '{"3": "ИБН-241-10-1", "4": "америций-241", "5": "4", "6": "7,844 max (4,4); 10,00max (4,6)", "7": "", "8": "560000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-10-2', 'ИБН-241-10-2', '{"3": "ИБН-241-10-2", "4": "америций-241", "5": "4", "6": "7,844 max (4,4); 10,00max (4,6)", "7": "", "8": "1600000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-10-3', 'ИБН-241-10-3', '{"3": "ИБН-241-10-3", "4": "америций-241", "5": "4", "6": "7,844 max (4,4); 10,00max (4,6)", "7": "", "8": "1900000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-1-1', 'ИБН-241-1-1', '{"3": "ИБН-241-1-1", "4": "америций-241", "5": "4", "6": "10 (3); 13 (3)", "7": "", "8": "2400000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-11-1', 'ИБН-241-11-1', '{"3": "ИБН-241-11-1", "4": "америций-241", "5": "4", "6": "9,070 max (5,2); 12,80max (4,8)", "7": "", "8": "2300000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-11-2', 'ИБН-241-11-2', '{"3": "ИБН-241-11-2", "4": "америций-241", "5": "4", "6": "9,070 max (5,2); 12,80max (4,8)", "7": "", "8": "4800000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-12-1', 'ИБН-241-12-1', '{"3": "ИБН-241-12-1", "4": "америций-241", "5": "4", "6": "12,740 max (9,2); 13,20max (7,4)", "7": "", "8": "5300000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-12-2', 'ИБН-241-12-2', '{"3": "ИБН-241-12-2", "4": "америций-241", "5": "4", "6": "12,740 max (9,2); 13,20max (7,4)", "7": "", "8": "9500000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-13-1', 'ИБН-241-13-1', '{"3": "ИБН-241-13-1", "4": "америций-241", "5": "4", "6": "14,350 max (11,3); 31,75max (25,0)", "7": "", "8": "140000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-14-1', 'ИБН-241-14-1', '{"3": "ИБН-241-14-1", "4": "америций-241", "5": "4", "6": "17,475 max (13,9); 19,40max (11,9)", "7": "", "8": "37000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-14-2', 'ИБН-241-14-2', '{"3": "ИБН-241-14-2", "4": "америций-241", "5": "4", "6": "17,475 max (13,9); 19,40max (11,9)", "7": "", "8": "140000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-14-3', 'ИБН-241-14-3', '{"3": "ИБН-241-14-3", "4": "америций-241", "5": "4", "6": "17,475 max (13,9); 19,40max (11,9)", "7": "", "8": "480000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-14-4', 'ИБН-241-14-4', '{"3": "ИБН-241-14-4", "4": "америций-241", "5": "4", "6": "17,475 max (13,9); 19,40max (11,9)", "7": "", "8": "1400000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-14-5', 'ИБН-241-14-5', '{"3": "ИБН-241-14-5", "4": "америций-241", "5": "4", "6": "17,475 max (13,9); 19,40max (11,9)", "7": "", "8": "4800000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-14-6', 'ИБН-241-14-6', '{"3": "ИБН-241-14-6", "4": "америций-241", "5": "4", "6": "17,475 max (13,9); 19,40max (11,9)", "7": "", "8": "14000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'X.7', 'X.7', '{"3": "X.7", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-14-7', 'ИБН-241-14-7', '{"3": "ИБН-241-14-7", "4": "америций-241", "5": "4", "6": "17,475 max (13,9); 19,40max (11,9)", "7": "", "8": "21000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-14-8', 'ИБН-241-14-8', '{"3": "ИБН-241-14-8", "4": "америций-241", "5": "4", "6": "17,475 max (13,9); 19,40max (11,9)", "7": "", "8": "43000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-15-1', 'ИБН-241-15-1', '{"3": "ИБН-241-15-1", "4": "америций-241", "5": "4", "6": "22,475 max (17,4); 31,20max (17,7)", "7": "", "8": "480000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-15-2', 'ИБН-241-15-2', '{"3": "ИБН-241-15-2", "4": "америций-241", "5": "4", "6": "22,475 max (17,4); 31,20max (17,7)", "7": "", "8": "9500000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-15-3', 'ИБН-241-15-3', '{"3": "ИБН-241-15-3", "4": "америций-241", "5": "4", "6": "22,475 max (17,4); 31,20max (17,7)", "7": "", "8": "23000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-15-4', 'ИБН-241-15-4', '{"3": "ИБН-241-15-4", "4": "америций-241", "5": "4", "6": "22,475 max (17,4); 31,20max (17,7)", "7": "", "8": "48000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-15-5', 'ИБН-241-15-5', '{"3": "ИБН-241-15-5", "4": "америций-241", "5": "4", "6": "22,475 max (17,4); 31,20max (17,7)", "7": "", "8": "52000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-16-1', 'ИБН-241-16-1', '{"3": "ИБН-241-16-1", "4": "америций-241", "5": "4", "6": "22,475 max (17,4); 48,60max (35,1)", "7": "", "8": "140000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-17-1', 'ИБН-241-17-1', '{"3": "ИБН-241-17-1", "4": "америций-241", "5": "4", "6": "30,131 max (25,1); 60,20max (46,6)", "7": "", "8": "230000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-17-2', 'ИБН-241-17-2', '{"3": "ИБН-241-17-2", "4": "америций-241", "5": "4", "6": "30,131 max (25,1); 60,20max (46,6)", "7": "", "8": "470000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-17-3', 'ИБН-241-17-3', '{"3": "ИБН-241-17-3", "4": "америций-241", "5": "4", "6": "30,131 max (25,1); 60,20max (46,6)", "7": "", "8": "640000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-18-1', 'ИБН-241-18-1', '{"3": "ИБН-241-18-1", "4": "америций-241", "5": "4", "6": "7,920 max (4,4); 15,20max (9,0)", "7": "", "8": "480000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-18-2', 'ИБН-241-18-2', '{"3": "ИБН-241-18-2", "4": "америций-241", "5": "4", "6": "7,920 max (4,4); 15,20max (9,0)", "7": "", "8": "1400000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-18-3', 'ИБН-241-18-3', '{"3": "ИБН-241-18-3", "4": "америций-241", "5": "4", "6": "7,920 max (4,4); 15,20max (9,0)", "7": "", "8": "2300000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-18-4', 'ИБН-241-18-4', '{"3": "ИБН-241-18-4", "4": "америций-241", "5": "4", "6": "7,920 max (4,4); 15,20max (9,0)", "7": "", "8": "3700000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-18-5', 'ИБН-241-18-5', '{"3": "ИБН-241-18-5", "4": "америций-241", "5": "4", "6": "7,920 max (4,4); 15,20max (9,0)", "7": "", "8": "4800000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-19-1', 'ИБН-241-19-1', '{"3": "ИБН-241-19-1", "4": "америций-241", "5": "4", "6": "49,255 max (41,0); 81,15max (70,0)", "7": "", "8": "1000000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-19-2', 'ИБН-241-19-2', '{"3": "ИБН-241-19-2", "4": "америций-241", "5": "4", "6": "49,255 max (41,0); 81,15max (70,0)", "7": "", "8": "2100000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-2-1', 'ИБН-241-2-1', '{"3": "ИБН-241-2-1", "4": "америций-241", "5": "4", "6": "12 (4); 16 (4)", "7": "", "8": "4800000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-2-2', 'ИБН-241-2-2', '{"3": "ИБН-241-2-2", "4": "америций-241", "5": "4", "6": "12 (5); 16 (5)", "7": "", "8": "12000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-4-1', 'ИБН-241-4-1', '{"3": "ИБН-241-4-1", "4": "америций-241", "5": "4", "6": "15 (6); 18 (6)", "7": "", "8": "24000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-5-1', 'ИБН-241-5-1', '{"3": "ИБН-241-5-1", "4": "америций-241", "5": "4", "6": "15 (8); 20 (8)", "7": "", "8": "48000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-6-1', 'ИБН-241-6-1', '{"3": "ИБН-241-6-1", "4": "америций-241", "5": "4", "6": "18 (10); 22 (10)", "7": "", "8": "120000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-7-1', 'ИБН-241-7-1', '{"3": "ИБН-241-7-1", "4": "америций-241", "5": "4", "6": "21 (13); 25 (13)", "7": "", "8": "240000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-8-1', 'ИБН-241-8-1', '{"3": "ИБН-241-8-1", "4": "америций-241", "5": "4", "6": "24 (16); 30 (16)", "7": "", "8": "480000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EG1', 'EG1', '{"3": "EG1", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-9-1', 'ИБН-241-9-1', '{"3": "ИБН-241-9-1", "4": "америций-241", "5": "4", "6": "29 (21); 33 (21)", "7": "", "8": "1200000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-В-1', 'ИБН-241-В-1', '{"3": "ИБН-241-В-1", "4": "америций-241", "5": "4", "6": "25 (17); 46 (36)", "7": "", "8": "120000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-В1-1', 'ИБН-241-В1-1', '{"3": "ИБН-241-В1-1", "4": "америций-241", "5": "4", "6": "35 (27); 45 (27)", "7": "", "8": "240000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-В-2', 'ИБН-241-В-2', '{"3": "ИБН-241-В-2", "4": "америций-241", "5": "4", "6": "25 (17); 46 (36)", "7": "", "8": "200000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-В-3', 'ИБН-241-В-3', '{"3": "ИБН-241-В-3", "4": "америций-241", "5": "4", "6": "25 (17); 46 (36)", "7": "", "8": "240000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-В-4', 'ИБН-241-В-4', '{"3": "ИБН-241-В-4", "4": "америций-241", "5": "4", "6": "35 (27); 45 (27)", "7": "", "8": "240000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-К-1', 'ИБН-241-К-1', '{"3": "ИБН-241-К-1", "4": "америций-241", "5": "4", "6": "50,5; 56,5", "7": "", "8": "440000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-241-К-2', 'ИБН-241-К-2', '{"3": "ИБН-241-К-2", "4": "америций-241", "5": "4", "6": "50,5; 56,5", "7": "", "8": "960000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-25', 'ИБН-25', '{"3": "ИБН-25", "4": "плутоний-239", "5": "4", "6": "35 (27); 39 (27)", "7": "", "8": "250000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-26', 'ИБН-26', '{"3": "ИБН-26", "4": "плутоний-239", "5": "4", "6": "42 (34); 46 (34)", "7": "", "8": "500000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-27', 'ИБН-27', '{"3": "ИБН-27", "4": "плутоний-239", "5": "4", "6": "54 (46); 58 (46)", "7": "", "8": "1200000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-28', 'ИБН-28', '{"3": "ИБН-28", "4": "плутоний-239", "5": "4", "6": "3,2 (30)", "7": "", "8": "290000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-3', 'ИБН-3', '{"3": "ИБН-3", "4": "плутоний-239", "5": "4", "6": "12 (5); 22 (5)", "7": "", "8": "1300000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-4', 'ИБН-4', '{"3": "ИБН-4", "4": "плутоний-239", "5": "4", "6": "15 (6); 24 (6)", "7": "", "8": "2500000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-5', 'ИБН-5', '{"3": "ИБН-5", "4": "плутоний-239", "5": "4", "6": "15 (8); 26 (8)", "7": "", "8": "5000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-6', 'ИБН-6', '{"3": "ИБН-6", "4": "плутоний-239", "5": "4", "6": "18 (10); 28 (10)", "7": "", "8": "13000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-7', 'ИБН-7', '{"3": "ИБН-7", "4": "плутоний-239", "5": "4", "6": "21 (13); 31 (13)", "7": "", "8": "25000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-8', 'ИБН-8', '{"3": "ИБН-8", "4": "плутоний-239", "5": "4", "6": "24 (16); 36 (16)", "7": "", "8": "50000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-8-1', 'ИБН-8-1', '{"3": "ИБН-8-1", "4": "плутоний-238", "5": "4", "6": "10 (4); 13 (4,2)", "7": "", "8": "1200000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-8-2', 'ИБН-8-2', '{"3": "ИБН-8-2", "4": "плутоний-238", "5": "4", "6": "12 (5); 16 (5,2)", "7": "", "8": "24000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-8-3', 'ИБН-8-3', '{"3": "ИБН-8-3", "4": "плутоний-238", "5": "4", "6": "15 (6); 18 (6,2)", "7": "", "8": "48000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-8-4', 'ИБН-8-4', '{"3": "ИБН-8-4", "4": "плутоний-238", "5": "4", "6": "15 (8); 20 (8,2)", "7": "", "8": "120000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-8-5', 'ИБН-8-5', '{"3": "ИБН-8-5", "4": "плутоний-238", "5": "4", "6": "18 (10); 22 (10,2)", "7": "", "8": "240000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-8-6', 'ИБН-8-6', '{"3": "ИБН-8-6", "4": "плутоний-238", "5": "4", "6": "21 (13); 25 (13,5)", "7": "", "8": "480000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-8-7', 'ИБН-8-7', '{"3": "ИБН-8-7", "4": "плутоний-238", "5": "4", "6": "19,5 (12,3); 40 (30)", "7": "", "8": "1200000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-8-8', 'ИБН-8-8', '{"3": "ИБН-8-8", "4": "плутоний-238", "5": "4", "6": "24 (17); 30 (17)", "7": "", "8": "1200000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-8-9', 'ИБН-8-9', '{"3": "ИБН-8-9", "4": "плутоний-238", "5": "4", "6": "30 (22); 24 (10)", "7": "", "8": "1200000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-9', 'ИБН-9', '{"3": "ИБН-9", "4": "плутоний-239", "5": "4", "6": "29 (21); 39 (21)", "7": "", "8": "130000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИВГИ', 'ИВГИ', '{"3": "ИВГИ", "4": "калифорний-252", "5": "3", "6": "", "7": "0.0", "8": "32800000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА', 'ИГИА', '{"3": "ИГИА", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-1', 'ИГИА-1', '{"3": "ИГИА-1", "4": "америций-241", "5": "1", "6": "6,0x6,0", "7": "", "8": "4800000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-10', 'ИГИА-10', '{"3": "ИГИА-10", "4": "америций-241", "5": "1", "6": "15 (12); 6 (0,20-0,25)", "7": "", "8": "13000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-11', 'ИГИА-11', '{"3": "ИГИА-11", "4": "америций-241", "5": "1", "6": "22 (18); 6 (0,20-0,25)", "7": "", "8": "21000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-1-1', 'ИГИА-1-1', '{"3": "ИГИА-1-1", "4": "америций-241", "5": "1", "6": "6,0x6,0", "7": "", "8": "48000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-12', 'ИГИА-12', '{"3": "ИГИА-12", "4": "америций-241", "5": "1", "6": "30 (25); 6 (0,20-0,25)", "7": "", "8": "41000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-1-2', 'ИГИА-1-2', '{"3": "ИГИА-1-2", "4": "америций-241", "5": "1", "6": "6,0x6,0", "7": "", "8": "240000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-13', 'ИГИА-13', '{"3": "ИГИА-13", "4": "америций-241", "5": "1", "6": "36 (31); 8 (0,25-0,30)", "7": "", "8": "130000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-1-3', 'ИГИА-1-3', '{"3": "ИГИА-1-3", "4": "америций-241", "5": "1", "6": "6,0x6,0", "7": "", "8": "480000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-14', 'ИГИА-14', '{"3": "ИГИА-14", "4": "америций-241", "5": "1", "6": "45,0х8,0", "7": "", "8": "210000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-1-4', 'ИГИА-1-4', '{"3": "ИГИА-1-4", "4": "америций-241", "5": "1", "6": "6,0x6,0", "7": "", "8": "1300000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-1-5', 'ИГИА-1-5', '{"3": "ИГИА-1-5", "4": "америций-241", "5": "1", "6": "6,0x6,0", "7": "", "8": "3800000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-1м', 'ИГИА-1м', '{"3": "ИГИА-1м", "4": "америций-241", "5": "1", "6": "6 (4); 6,5 (0,20-0,25)", "7": "", "8": "5300000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-1м-1', 'ИГИА-1м-1', '{"3": "ИГИА-1м-1", "4": "америций-241", "5": "1", "6": "6 (4); 6,5 (0,20-0,25)", "7": "", "8": "53000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-1м-2', 'ИГИА-1м-2', '{"3": "ИГИА-1м-2", "4": "америций-241", "5": "1", "6": "6 (4); 6,5 (0,20-0,25)", "7": "", "8": "290000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-1м-3', 'ИГИА-1м-3', '{"3": "ИГИА-1м-3", "4": "америций-241", "5": "1", "6": "6 (4); 6,5 (0,20-0,25)", "7": "", "8": "530000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-1м-4', 'ИГИА-1м-4', '{"3": "ИГИА-1м-4", "4": "америций-241", "5": "1", "6": "6 (4); 6,5 (0,20-0,25)", "7": "", "8": "1200000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-1м-5', 'ИГИА-1м-5', '{"3": "ИГИА-1м-5", "4": "америций-241", "5": "1", "6": "6 (4); 6,5 (0,20-0,25)", "7": "", "8": "3900000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-2', 'ИГИА-2', '{"3": "ИГИА-2", "4": "америций-241", "5": "1", "6": "8,0x6,0", "7": "", "8": "8200000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-2м', 'ИГИА-2м', '{"3": "ИГИА-2м", "4": "америций-241", "5": "1", "6": "8 (6); 6,5 (0,20-0,25)", "7": "", "8": "9100000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-3', 'ИГИА-3', '{"3": "ИГИА-3", "4": "америций-241", "5": "1", "6": "10,0x6,0", "7": "", "8": "14000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-3м', 'ИГИА-3м', '{"3": "ИГИА-3м", "4": "америций-241", "5": "1", "6": "10 (8); 6,5 (0,20-0,25)", "7": "", "8": "18000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-4', 'ИГИА-4', '{"3": "ИГИА-4", "4": "америций-241", "5": "1", "6": "12,0x6,0", "7": "", "8": "4100000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-4-1', 'ИГИА-4-1', '{"3": "ИГИА-4-1", "4": "америций-241", "5": "1", "6": "12,0x6,0", "7": "", "8": "22000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-4м', 'ИГИА-4м', '{"3": "ИГИА-4м", "4": "америций-241", "5": "1", "6": "12 (10); 6,5 (0,20-0,25)", "7": "", "8": "5200000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-4м-1', 'ИГИА-4м-1', '{"3": "ИГИА-4м-1", "4": "америций-241", "5": "1", "6": "12 (10); 6,5 (0,20-0,25)", "7": "", "8": "26000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-5', 'ИГИА-5', '{"3": "ИГИА-5", "4": "америций-241", "5": "1", "6": "20,0x6,0", "7": "", "8": "14000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-5-1', 'ИГИА-5-1', '{"3": "ИГИА-5-1", "4": "америций-241", "5": "1", "6": "20,0x6,0", "7": "", "8": "71000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-5м', 'ИГИА-5м', '{"3": "ИГИА-5м", "4": "америций-241", "5": "1", "6": "20 (18); 6,5 (0,20-0,25)", "7": "", "8": "16000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-5м-1', 'ИГИА-5м-1', '{"3": "ИГИА-5м-1", "4": "америций-241", "5": "1", "6": "20 (18); 6,5 (0,20-0,25)", "7": "", "8": "83000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-6-1', 'ИГИА-6-1', '{"3": "ИГИА-6-1", "4": "америций-241", "5": "1", "6": "8 (4,2); 5 (0,20-0,25)", "7": "", "8": "47000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-6-2', 'ИГИА-6-2', '{"3": "ИГИА-6-2", "4": "америций-241", "5": "1", "6": "8 (4,2); 5 (0,20-0,25)", "7": "", "8": "170000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-6-3', 'ИГИА-6-3', '{"3": "ИГИА-6-3", "4": "америций-241", "5": "1", "6": "8 (4,2); 5 (0,20-0,25)", "7": "", "8": "470000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-8', 'ИГИА-8', '{"3": "ИГИА-8", "4": "америций-241", "5": "1", "6": "10,8 (8); 5 (0,20-0,25)", "7": "", "8": "4100000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА-9', 'ИГИА-9', '{"3": "ИГИА-9", "4": "америций-241", "5": "1", "6": "10,8 (7,5); 6 (0,20-0,25)", "7": "", "8": "4100000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИГ', 'ИГИГ', '{"3": "ИГИГ", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИГ-1', 'ИГИГ-1', '{"3": "ИГИГ-1", "4": "гадолиний-153", "5": "1", "6": "6,0х12,0", "7": "0.0", "8": "37000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИГ-2', 'ИГИГ-2', '{"3": "ИГИГ-2", "4": "гадолиний-153", "5": "1", "6": "6,0х12,0", "7": "", "8": "200000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИГ-3', 'ИГИГ-3', '{"3": "ИГИГ-3", "4": "гадолиний-153", "5": "1", "6": "6,0х12,0", "7": "", "8": "400000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИГ-4', 'ИГИГ-4', '{"3": "ИГИГ-4", "4": "гадолиний-153", "5": "1", "6": "6,0х12,0", "7": "", "8": "800000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИГ-5', 'ИГИГ-5', '{"3": "ИГИГ-5", "4": "гадолиний-153", "5": "1", "6": "6,0х12,0", "7": "", "8": "1000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИГ-6', 'ИГИГ-6', '{"3": "ИГИГ-6", "4": "гадолиний-153", "5": "1", "6": "6,0х12,0", "7": "", "8": "2000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИГ-7', 'ИГИГ-7', '{"3": "ИГИГ-7", "4": "гадолиний-153", "5": "1", "6": "6,0х12,0", "7": "", "8": "4000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Е', 'ИГИ-Е', '{"3": "ИГИ-Е", "4": "европий-152; европий-154", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Е-1', 'ИГИ-Е-1', '{"3": "ИГИ-Е-1", "4": "европий-152; европий-154", "5": "1", "6": "7,5х8,0", "7": "64000000000.0", "8": "248000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Е-1-1', 'ИГИ-Е-1-1', '{"3": "ИГИ-Е-1-1", "4": "европий-152; европий-154", "5": "1", "6": "7,5x8,0", "7": "", "8": "48000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Е-1-2', 'ИГИ-Е-1-2', '{"3": "ИГИ-Е-1-2", "4": "европий-152; европий-154", "5": "1", "6": "7,5x8,0", "7": "", "8": "96000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Е-1-3', 'ИГИ-Е-1-3', '{"3": "ИГИ-Е-1-3", "4": "европий-152; европий-154", "5": "1", "6": "7,5x8,0", "7": "", "8": "190000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ир', 'ИГИ-Ир', '{"3": "ИГИ-Ир", "4": "иридий-192", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ир-0', 'ИГИ-Ир-0', '{"3": "ИГИ-Ир-0", "4": "иридий-192", "5": "1", "6": "4,5х5,0", "7": "1320000000.0", "8": "692000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ир-0-1', 'ИГИ-Ир-0-1', '{"3": "ИГИ-Ир-0-1", "4": "иридий-192", "5": "1", "6": "4,5x5,0", "7": "", "8": "1400000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ир-0-2', 'ИГИ-Ир-0-2', '{"3": "ИГИ-Ир-0-2", "4": "иридий-192", "5": "1", "6": "4,5x5,0", "7": "", "8": "14000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ир-0-3', 'ИГИ-Ир-0-3', '{"3": "ИГИ-Ир-0-3", "4": "иридий-192", "5": "1", "6": "4,5x5,0", "7": "", "8": "66000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ир-0-4', 'ИГИ-Ир-0-4', '{"3": "ИГИ-Ир-0-4", "4": "иридий-192", "5": "1", "6": "4,5x5,0", "7": "", "8": "270000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ир-0-5', 'ИГИ-Ир-0-5', '{"3": "ИГИ-Ир-0-5", "4": "иридий-192", "5": "1", "6": "4,5x5,0", "7": "", "8": "660000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ир-1', 'ИГИ-Ир-1', '{"3": "ИГИ-Ир-1", "4": "иридий-192", "5": "1", "6": "7,5х8,0", "7": "1320000000000.0", "8": "2720000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ир-1-1', 'ИГИ-Ир-1-1', '{"3": "ИГИ-Ир-1-1", "4": "иридий-192", "5": "1", "6": "7,5x8,0", "7": "", "8": "1400000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИН-22', 'ИГИН-22', '{"3": "ИГИН-22", "4": "натрий-22", "5": "1", "6": "20,0х4,0", "7": "0.0", "8": "370000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-РСД-А-1', 'ИГИ-РСД-А-1', '{"3": "ИГИ-РСД-А-1", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "193000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-РСД-А-2', 'ИГИ-РСД-А-2', '{"3": "ИГИ-РСД-А-2", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "19000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-РСД-К-1', 'ИГИ-РСД-К-1', '{"3": "ИГИ-РСД-К-1", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-РСД-К-2', 'ИГИ-РСД-К-2', '{"3": "ИГИ-РСД-К-2", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "203000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-РСД-Ц-1', 'ИГИ-РСД-Ц-1', '{"3": "ИГИ-РСД-Ц-1", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "183000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-РСД-Ц-2', 'ИГИ-РСД-Ц-2', '{"3": "ИГИ-РСД-Ц-2", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "19000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-С-0', 'ИГИ-С-0', '{"3": "ИГИ-С-0", "4": "серебро-110м", "5": "1", "6": "4,0х5,0", "7": "42800000.0", "8": "6520000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-С-0-1', 'ИГИ-С-0-1', '{"3": "ИГИ-С-0-1", "4": "серебро-110м", "5": "1", "6": "4,5x5,0", "7": "", "8": "42000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-С-0-2', 'ИГИ-С-0-2', '{"3": "ИГИ-С-0-2", "4": "серебро-110м", "5": "1", "6": "4,5x5,0", "7": "", "8": "210000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-С-0-3', 'ИГИ-С-0-3', '{"3": "ИГИ-С-0-3", "4": "серебро-110м", "5": "1", "6": "4,5x5,0", "7": "", "8": "420000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-С-0-4', 'ИГИ-С-0-4', '{"3": "ИГИ-С-0-4", "4": "серебро-110м", "5": "1", "6": "4,5x5,0", "7": "", "8": "2100000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-С-0-5', 'ИГИ-С-0-5', '{"3": "ИГИ-С-0-5", "4": "серебро-110м", "5": "1", "6": "4,5x5,0", "7": "", "8": "4200000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-С-0-6', 'ИГИ-С-0-6', '{"3": "ИГИ-С-0-6", "4": "серебро-110м", "5": "1", "6": "4,5x5,0", "7": "", "8": "6200000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Се', 'ИГИ-Се', '{"3": "ИГИ-Се", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Се-0-1', 'ИГИ-Се-0-1', '{"3": "ИГИ-Се-0-1", "4": "селен-75", "5": "1", "6": "4,5x5,0", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Се-2', 'ИГИ-Се-2', '{"3": "ИГИ-Се-2", "4": "селен-75", "5": "1", "6": "7,5х9,5", "7": "1600000000.0", "8": "30600000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Се-2-1', 'ИГИ-Се-2-1', '{"3": "ИГИ-Се-2-1", "4": "селен-75", "5": "1", "6": "7,5x9,5", "7": "", "8": "1700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Се-2-2', 'ИГИ-Се-2-2', '{"3": "ИГИ-Се-2-2", "4": "селен-75", "5": "1", "6": "7,5x9,5", "7": "", "8": "3100000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Се-2-3', 'ИГИ-Се-2-3', '{"3": "ИГИ-Се-2-3", "4": "селен-75", "5": "1", "6": "7,5x9,5", "7": "", "8": "6500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Се-2-4', 'ИГИ-Се-2-4', '{"3": "ИГИ-Се-2-4", "4": "селен-75", "5": "1", "6": "7,5x9,5", "7": "", "8": "17000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Се-2-5', 'ИГИ-Се-2-5', '{"3": "ИГИ-Се-2-5", "4": "селен-75", "5": "1", "6": "7,5x9,5", "7": "", "8": "33000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Се-4', 'ИГИ-Се-4', '{"3": "ИГИ-Се-4", "4": "селен-75", "5": "1", "6": "12,5х12,5", "7": "77500000000.0", "8": "160000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Се-4-1', 'ИГИ-Се-4-1', '{"3": "ИГИ-Се-4-1", "4": "селен-75", "5": "1", "6": "12,5x12,5", "7": "", "8": "83000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Се-4-3', 'ИГИ-Се-4-3', '{"3": "ИГИ-Се-4-3", "4": "селен-75", "5": "1", "6": "12,5x12,5", "7": "", "8": "160000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Се-5', 'ИГИ-Се-5', '{"3": "ИГИ-Се-5", "4": "селен-75", "5": "1", "6": "16,5х16,5", "7": "306000000000.0", "8": "620000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Се-5-1', 'ИГИ-Се-5-1', '{"3": "ИГИ-Се-5-1", "4": "селен-75", "5": "1", "6": "16,5x16,5", "7": "", "8": "330000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Се-5-2', 'ИГИ-Се-5-2', '{"3": "ИГИ-Се-5-2", "4": "селен-75", "5": "1", "6": "16,5x16,5", "7": "", "8": "520000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Се-5-3', 'ИГИ-Се-5-3', '{"3": "ИГИ-Се-5-3", "4": "селен-75", "5": "1", "6": "16,5x16,5", "7": "", "8": "650000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Се-6-1', 'ИГИ-Се-6-1', '{"3": "ИГИ-Се-6-1", "4": "селен-75", "5": "1", "6": "8,0x11,0", "7": "", "8": "16000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Се-6-2', 'ИГИ-Се-6-2', '{"3": "ИГИ-Се-6-2", "4": "селен-75", "5": "1", "6": "8,0x11,0", "7": "", "8": "31000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Се-7-1', 'ИГИ-Се-7-1', '{"3": "ИГИ-Се-7-1", "4": "селен-75", "5": "1", "6": "13,0x13,5", "7": "", "8": "78000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Се-7-2', 'ИГИ-Се-7-2', '{"3": "ИГИ-Се-7-2", "4": "селен-75", "5": "1", "6": "13,0x13,5", "7": "", "8": "160000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Се-8-1', 'ИГИ-Се-8-1', '{"3": "ИГИ-Се-8-1", "4": "селен-75", "5": "1", "6": "17,0x17,5", "7": "", "8": "310000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Се-8-2', 'ИГИ-Се-8-2', '{"3": "ИГИ-Се-8-2", "4": "селен-75", "5": "1", "6": "17,0x17,5", "7": "", "8": "470000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Се-8-3', 'ИГИ-Се-8-3', '{"3": "ИГИ-Се-8-3", "4": "селен-75", "5": "1", "6": "17,0x17,5", "7": "", "8": "620000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Су', 'ИГИ-Су', '{"3": "ИГИ-Су", "4": "сурьма-124", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Су-1', 'ИГИ-Су-1', '{"3": "ИГИ-Су-1", "4": "сурьма-124", "5": "1", "6": "4,0х55,0", "7": "", "8": "33700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Су-1-1', 'ИГИ-Су-1-1', '{"3": "ИГИ-Су-1-1", "4": "сурьма-124", "5": "1", "6": "4,0x55,0", "7": "", "8": "6500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Су-1-2', 'ИГИ-Су-1-2', '{"3": "ИГИ-Су-1-2", "4": "сурьма-124", "5": "1", "6": "4,0x55,0", "7": "", "8": "9700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Су-1-3', 'ИГИ-Су-1-3', '{"3": "ИГИ-Су-1-3", "4": "сурьма-124", "5": "1", "6": "4,0x55,0", "7": "", "8": "13000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Су-1-4', 'ИГИ-Су-1-4', '{"3": "ИГИ-Су-1-4", "4": "сурьма-124", "5": "1", "6": "4,0x55,0", "7": "", "8": "20000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Су-1-5', 'ИГИ-Су-1-5', '{"3": "ИГИ-Су-1-5", "4": "сурьма-124", "5": "1", "6": "4,0x55,0", "7": "", "8": "33000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Су-6-1', 'ИГИ-Су-6-1', '{"3": "ИГИ-Су-6-1", "4": "сурьма-124", "5": "1", "6": "6,2x25,0", "7": "", "8": "640000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Су-6-2', 'ИГИ-Су-6-2', '{"3": "ИГИ-Су-6-2", "4": "сурьма-124", "5": "1", "6": "6,2x25,0", "7": "", "8": "1300000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Су-6-3', 'ИГИ-Су-6-3', '{"3": "ИГИ-Су-6-3", "4": "сурьма-124", "5": "1", "6": "6,2x25,0", "7": "", "8": "3400000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Су-6-4', 'ИГИ-Су-6-4', '{"3": "ИГИ-Су-6-4", "4": "сурьма-124", "5": "1", "6": "6,2x25,0", "7": "", "8": "6400000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Су-6-5', 'ИГИ-Су-6-5', '{"3": "ИГИ-Су-6-5", "4": "сурьма-124", "5": "1", "6": "6,2x25,0", "7": "", "8": "13000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Су-6-6', 'ИГИ-Су-6-6', '{"3": "ИГИ-Су-6-6", "4": "сурьма-124", "5": "1", "6": "6,2x25,0", "7": "", "8": "34000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Су-7', 'ИГИ-Су-7', '{"3": "ИГИ-Су-7", "4": "сурьма-124", "5": "1", "6": "6,2x25,0", "7": "", "8": "130000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Су-8', 'ИГИ-Су-8', '{"3": "ИГИ-Су-8", "4": "сурьма-124", "5": "1", "6": "10,0x95,0", "7": "", "8": "2400000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ту', 'ИГИ-Ту', '{"3": "ИГИ-Ту", "4": "тулий-170", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ту-0', 'ИГИ-Ту-0', '{"3": "ИГИ-Ту-0", "4": "тулий-170", "5": "1", "6": "4,5х5,0", "7": "", "8": "258000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ту-0-1', 'ИГИ-Ту-0-1', '{"3": "ИГИ-Ту-0-1", "4": "тулий-170", "5": "1", "6": "4,5x5,0", "7": "", "8": "24000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ту-0-2', 'ИГИ-Ту-0-2', '{"3": "ИГИ-Ту-0-2", "4": "тулий-170", "5": "1", "6": "4,5x5,0", "7": "", "8": "46000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ту-0-3', 'ИГИ-Ту-0-3', '{"3": "ИГИ-Ту-0-3", "4": "тулий-170", "5": "1", "6": "4,5x5,0", "7": "", "8": "91000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ту-0-4', 'ИГИ-Ту-0-4', '{"3": "ИГИ-Ту-0-4", "4": "тулий-170", "5": "1", "6": "4,5x5,0", "7": "", "8": "240000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ту-1', 'ИГИ-Ту-1', '{"3": "ИГИ-Ту-1", "4": "тулий-170", "5": "1", "6": "7,5х8,0", "7": "", "8": "6320000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ту-1-1', 'ИГИ-Ту-1-1', '{"3": "ИГИ-Ту-1-1", "4": "тулий-170", "5": "1", "6": "7,5x8,0", "7": "", "8": "480000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ту-1-2', 'ИГИ-Ту-1-2', '{"3": "ИГИ-Ту-1-2", "4": "тулий-170", "5": "1", "6": "7,5x8,0", "7": "", "8": "2400000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ту-3', 'ИГИ-Ту-3', '{"3": "ИГИ-Ту-3", "4": "тулий-170", "5": "1", "6": "12,0х10,0", "7": "", "8": "32200000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ту-3-2', 'ИГИ-Ту-3-2', '{"3": "ИГИ-Ту-3-2", "4": "тулий-170", "5": "1", "6": "12,0x10,0", "7": "", "8": "12000000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц', 'ИГИ-Ц', '{"3": "ИГИ-Ц", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-1', 'ИГИ-Ц-1', '{"3": "ИГИ-Ц-1", "4": "цезий-137", "5": "1", "6": "6,1х10,0", "7": "", "8": "730000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-10', 'ИГИ-Ц-10', '{"3": "ИГИ-Ц-10", "4": "цезий-137", "5": "1", "6": "11,0х16,5", "7": "", "8": "885000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-10-1', 'ИГИ-Ц-10-1', '{"3": "ИГИ-Ц-10-1", "4": "цезий-137", "5": "1", "6": "11 (7); 16,5 (7,5)", "7": "", "8": "592000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-11', 'ИГИ-Ц-11', '{"3": "ИГИ-Ц-11", "4": "цезий-137", "5": "1", "6": "15,0х25,0", "7": "", "8": "5180000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-11-1', 'ИГИ-Ц-11-1', '{"3": "ИГИ-Ц-11-1", "4": "цезий-137", "5": "1", "6": "15 (11); 25 (16)", "7": "", "8": "3220000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-12', 'ИГИ-Ц-12', '{"3": "ИГИ-Ц-12", "4": "цезий-137", "5": "1", "6": "15,0х81,5", "7": "", "8": "22600000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-12-1', 'ИГИ-Ц-12-1', '{"3": "ИГИ-Ц-12-1", "4": "цезий-137", "5": "1", "6": "15 (11); 81,5 (72,5)", "7": "", "8": "14400000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-13', 'ИГИ-Ц-13', '{"3": "ИГИ-Ц-13", "4": "цезий-137", "5": "1", "6": "20,0х81,5", "7": "", "8": "46200000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-13-1', 'ИГИ-Ц-13-1', '{"3": "ИГИ-Ц-13-1", "4": "цезий-137", "5": "1", "6": "20 (16); 81,5 (72,5)", "7": "", "8": "31800000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-14', 'ИГИ-Ц-14', '{"3": "ИГИ-Ц-14", "4": "цезий-137", "5": "1", "6": "11,5х84,0", "7": "2760000000000.0", "8": "11800000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-14-1', 'ИГИ-Ц-14-1', '{"3": "ИГИ-Ц-14-1", "4": "цезий-137", "5": "1", "6": "11,5 (8,5); 84 (75)", "7": "", "8": "8900000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-14-2', 'ИГИ-Ц-14-2', '{"3": "ИГИ-Ц-14-2", "4": "цезий-137", "5": "1", "6": "11,5 (8,5); 84 (75)", "7": "", "8": "6290000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-14-3', 'ИГИ-Ц-14-3', '{"3": "ИГИ-Ц-14-3", "4": "цезий-137", "5": "1", "6": "11,5 (8,5); 84 (75)", "7": "", "8": "2630000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-1-5', 'ИГИ-Ц-1-5', '{"3": "ИГИ-Ц-1-5", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1000000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-15-1', 'ИГИ-Ц-15-1', '{"3": "ИГИ-Ц-15-1", "4": "цезий-137", "5": "1", "6": "", "7": "337000000000000.0", "8": "", "9": "0", "10": "ПО Маяк", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-16-1', 'ИГИ-Ц-16-1', '{"3": "ИГИ-Ц-16-1", "4": "цезий-137", "5": "1", "6": "12,49 max (8,9); 17,85max (9,5)", "7": "", "8": "1300000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-17-1', 'ИГИ-Ц-17-1', '{"3": "ИГИ-Ц-17-1", "4": "цезий-137", "5": "1", "6": "14,69 max (11,1); 20,4max (11,5)", "7": "", "8": "2040000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-18-1', 'ИГИ-Ц-18-1', '{"3": "ИГИ-Ц-18-1", "4": "цезий-137", "5": "1", "6": "17,49 max (13,9); 22,9max (14)", "7": "", "8": "4440000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-19-1', 'ИГИ-Ц-19-1', '{"3": "ИГИ-Ц-19-1", "4": "цезий-137", "5": "1", "6": "21,29 max (17,7); 26,35max (17,5)", "7": "", "8": "8900000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-2', 'ИГИ-Ц-2', '{"3": "ИГИ-Ц-2", "4": "цезий-137", "5": "1", "6": "6,1х10,0", "7": "", "8": "2500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-20-1', 'ИГИ-Ц-20-1', '{"3": "ИГИ-Ц-20-1", "4": "цезий-137", "5": "1", "6": "27,1 max (23,5); 28,4max (19,5)", "7": "", "8": "17800000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-2-1', 'ИГИ-Ц-2-1', '{"3": "ИГИ-Ц-2-1", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "207000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-21-1', 'ИГИ-Ц-21-1', '{"3": "ИГИ-Ц-21-1", "4": "цезий-137", "5": "1", "6": "30,15 max (25,5); 70,35max (52,3)", "7": "", "8": "51800000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-22-1', 'ИГИ-Ц-22-1', '{"3": "ИГИ-Ц-22-1", "4": "цезий-137", "5": "1", "6": "36,15 max (32,1); 65,35max (56,8)", "7": "", "8": "96000000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-3', 'ИГИ-Ц-3', '{"3": "ИГИ-Ц-3", "4": "цезий-137", "5": "1", "6": "6,0х10,0", "7": "6400000.0", "8": "4500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-3-0', 'ИГИ-Ц-3-0', '{"3": "ИГИ-Ц-3-0", "4": "цезий-137", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-3-1', 'ИГИ-Ц-3-1', '{"3": "ИГИ-Ц-3-1", "4": "цезий-137", "5": "1", "6": "6 (4,5); 10 (6,1)", "7": "", "8": "4200000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-3-10', 'ИГИ-Ц-3-10', '{"3": "ИГИ-Ц-3-10", "4": "цезий-137", "5": "1", "6": "6 (4,5); 10 (6,1)", "7": "", "8": "4200000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-3-11', 'ИГИ-Ц-3-11', '{"3": "ИГИ-Ц-3-11", "4": "цезий-137", "5": "1", "6": "6 (4,5); 10 (6,1)", "7": "", "8": "4200000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-3-2', 'ИГИ-Ц-3-2', '{"3": "ИГИ-Ц-3-2", "4": "цезий-137", "5": "1", "6": "6 (4,5); 10 (6,1)", "7": "", "8": "4200000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-3-3', 'ИГИ-Ц-3-3', '{"3": "ИГИ-Ц-3-3", "4": "цезий-137", "5": "1", "6": "6 (4,5); 10 (6,1)", "7": "", "8": "4200000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-3-4', 'ИГИ-Ц-3-4', '{"3": "ИГИ-Ц-3-4", "4": "цезий-137", "5": "1", "6": "6 (4,5); 10 (6,1)", "7": "", "8": "4200000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-3-5', 'ИГИ-Ц-3-5', '{"3": "ИГИ-Ц-3-5", "4": "цезий-137", "5": "1", "6": "6 (4,5); 10 (6,1)", "7": "", "8": "4200000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-3-6', 'ИГИ-Ц-3-6', '{"3": "ИГИ-Ц-3-6", "4": "цезий-137", "5": "1", "6": "6 (4,5); 10 (6,1)", "7": "", "8": "4200000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-3-7', 'ИГИ-Ц-3-7', '{"3": "ИГИ-Ц-3-7", "4": "цезий-137", "5": "1", "6": "6 (4,5); 10 (6,1)", "7": "", "8": "4200000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-3-8', 'ИГИ-Ц-3-8', '{"3": "ИГИ-Ц-3-8", "4": "цезий-137", "5": "1", "6": "6 (4,5); 10 (6,1)", "7": "", "8": "4200000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-3-9', 'ИГИ-Ц-3-9', '{"3": "ИГИ-Ц-3-9", "4": "цезий-137", "5": "1", "6": "6 (4,5); 10 (6,1)", "7": "", "8": "4200000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-4', 'ИГИ-Ц-4', '{"3": "ИГИ-Ц-4", "4": "цезий-137", "5": "1", "6": "8,0х12,0", "7": "6650000000.0", "8": "244000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-4-1', 'ИГИ-Ц-4-1', '{"3": "ИГИ-Ц-4-1", "4": "цезий-137", "5": "1", "6": "8 (4,9); 12 (5,5)", "7": "", "8": "207000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-4-2', 'ИГИ-Ц-4-2', '{"3": "ИГИ-Ц-4-2", "4": "цезий-137", "5": "1", "6": "8 (4,9); 12 (5,5)", "7": "", "8": "207000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-4-3', 'ИГИ-Ц-4-3', '{"3": "ИГИ-Ц-4-3", "4": "цезий-137", "5": "1", "6": "8 (4,9); 12 (5,5)", "7": "", "8": "207000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-4-4', 'ИГИ-Ц-4-4', '{"3": "ИГИ-Ц-4-4", "4": "цезий-137", "5": "1", "6": "8 (4,9); 12 (5,5)", "7": "", "8": "207000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-4-5', 'ИГИ-Ц-4-5', '{"3": "ИГИ-Ц-4-5", "4": "цезий-137", "5": "1", "6": "8 (4,9); 12 (5,5)", "7": "", "8": "207000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-4-6', 'ИГИ-Ц-4-6', '{"3": "ИГИ-Ц-4-6", "4": "цезий-137", "5": "1", "6": "8 (4,9); 12 (5,5)", "7": "", "8": "207000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-5', 'ИГИ-Ц-5', '{"3": "ИГИ-Ц-5", "4": "цезий-137", "5": "1", "6": "16,0х18,0", "7": "650000000000.0", "8": "1250000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-5-1', 'ИГИ-Ц-5-1', '{"3": "ИГИ-Ц-5-1", "4": "цезий-137", "5": "1", "6": "16 (10); 18 (9)", "7": "", "8": "1520000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-5-2', 'ИГИ-Ц-5-2', '{"3": "ИГИ-Ц-5-2", "4": "цезий-137", "5": "1", "6": "16 (10); 18 (9)", "7": "", "8": "1520000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-6', 'ИГИ-Ц-6', '{"3": "ИГИ-Ц-6", "4": "цезий-137", "5": "1", "6": "16,0х24,0", "7": "", "8": "2560000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-6-1', 'ИГИ-Ц-6-1', '{"3": "ИГИ-Ц-6-1", "4": "цезий-137", "5": "1", "6": "16 (10); 24 (15)", "7": "", "8": "2550000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-7', 'ИГИ-Ц-7', '{"3": "ИГИ-Ц-7", "4": "цезий-137", "5": "1", "6": "19,0х31,0", "7": "7250000000000.0", "8": "13900000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-7-1', 'ИГИ-Ц-7-1', '{"3": "ИГИ-Ц-7-1", "4": "цезий-137", "5": "1", "6": "19 (15); 31 (22)", "7": "", "8": "8500000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-7-2', 'ИГИ-Ц-7-2', '{"3": "ИГИ-Ц-7-2", "4": "цезий-137", "5": "1", "6": "19 (15); 31 (22)", "7": "", "8": "8500000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-8', 'ИГИ-Ц-8', '{"3": "ИГИ-Ц-8", "4": "цезий-137", "5": "1", "6": "35,0х48,0", "7": "30800000000000.0", "8": "93500000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-8-1', 'ИГИ-Ц-8-1', '{"3": "ИГИ-Ц-8-1", "4": "цезий-137", "5": "1", "6": "35 (31); 48 (39)", "7": "", "8": "51800000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-8-2', 'ИГИ-Ц-8-2', '{"3": "ИГИ-Ц-8-2", "4": "цезий-137", "5": "1", "6": "35 (31); 48 (39)", "7": "", "8": "51800000000000.0", "9": "0", "10": "ПО Маяк", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-9', 'ИГИ-Ц-9', '{"3": "ИГИ-Ц-9", "4": "цезий-137", "5": "1", "6": "38,0х49,0", "7": "93500000000000.0", "8": "120000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-9-1', 'ИГИ-Ц-9-1', '{"3": "ИГИ-Ц-9-1", "4": "цезий-137", "5": "1", "6": "38 (33); 49 (40)", "7": "", "8": "74000000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИЦ-I', 'ИГИЦ-I', '{"3": "ИГИЦ-I", "4": "цинк-65", "5": "1", "6": "35,0х4,0", "7": "0.0", "8": "185000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИЦ-II', 'ИГИЦ-II', '{"3": "ИГИЦ-II", "4": "цинк-65", "5": "1", "6": "35,0х4,0", "7": "0.0", "8": "370000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Це', 'ИГИ-Це', '{"3": "ИГИ-Це", "4": "цезий-134", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Це-1', 'ИГИ-Це-1', '{"3": "ИГИ-Це-1", "4": "цезий-134", "5": "1", "6": "7,5х8,0", "7": "712000000.0", "8": "37400000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Це-1-1', 'ИГИ-Це-1-1', '{"3": "ИГИ-Це-1-1", "4": "цезий-134", "5": "1", "6": "7,5x8,0", "7": "", "8": "710000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Це-1-2', 'ИГИ-Це-1-2', '{"3": "ИГИ-Це-1-2", "4": "цезий-134", "5": "1", "6": "7,5x8,0", "7": "", "8": "36000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Це-3', 'ИГИ-Це-3', '{"3": "ИГИ-Це-3", "4": "цезий-134", "5": "1", "6": "12,0х10,0", "7": "", "8": "14500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Це-3-1', 'ИГИ-Це-3-1', '{"3": "ИГИ-Це-3-1", "4": "цезий-134", "5": "1", "6": "12,0x10,0", "7": "", "8": "150000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-ЦН', 'ИГИ-ЦН', '{"3": "ИГИ-ЦН", "4": "цинк-65", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-ЦН-1', 'ИГИ-ЦН-1', '{"3": "ИГИ-ЦН-1", "4": "цинк-65", "5": "1", "6": "2,5х12,0", "7": "20200000.0", "8": "410000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-ЦН-1-1', 'ИГИ-ЦН-1-1', '{"3": "ИГИ-ЦН-1-1", "4": "цинк-65", "5": "1", "6": "2,5x12,0", "7": "", "8": "20000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-ЦН-1-2', 'ИГИ-ЦН-1-2', '{"3": "ИГИ-ЦН-1-2", "4": "цинк-65", "5": "1", "6": "2,5x12,0", "7": "", "8": "50000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-ЦН-1-3', 'ИГИ-ЦН-1-3', '{"3": "ИГИ-ЦН-1-3", "4": "цинк-65", "5": "1", "6": "2,5x12,0", "7": "", "8": "100000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-ЦН-1-4', 'ИГИ-ЦН-1-4', '{"3": "ИГИ-ЦН-1-4", "4": "цинк-65", "5": "1", "6": "2,5x12,0", "7": "", "8": "200000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-ЦН-1-5', 'ИГИ-ЦН-1-5', '{"3": "ИГИ-ЦН-1-5", "4": "цинк-65", "5": "1", "6": "2,5x12,0", "7": "", "8": "400000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-ЦН-2', 'ИГИ-ЦН-2', '{"3": "ИГИ-ЦН-2", "4": "цинк-65", "5": "1", "6": "3,5х12,0", "7": "890000000.0", "8": "4100000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-ЦН-2-1', 'ИГИ-ЦН-2-1', '{"3": "ИГИ-ЦН-2-1", "4": "цинк-65", "5": "1", "6": "3,5x12,0", "7": "", "8": "880000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-ЦН-2-2', 'ИГИ-ЦН-2-2', '{"3": "ИГИ-ЦН-2-2", "4": "цинк-65", "5": "1", "6": "3,5x12,0", "7": "", "8": "1000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-ЦН-2-3', 'ИГИ-ЦН-2-3', '{"3": "ИГИ-ЦН-2-3", "4": "цинк-65", "5": "1", "6": "3,5x12,0", "7": "", "8": "2000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-ЦН-2-4', 'ИГИ-ЦН-2-4', '{"3": "ИГИ-ЦН-2-4", "4": "цинк-65", "5": "1", "6": "3,5x12,0", "7": "", "8": "3000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-ЦН-2-5', 'ИГИ-ЦН-2-5', '{"3": "ИГИ-ЦН-2-5", "4": "цинк-65", "5": "1", "6": "3,5x12,0", "7": "", "8": "4000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Цр', 'ИГИ-Цр', '{"3": "ИГИ-Цр", "4": "церий-144; празеодим-144", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Цр-1', 'ИГИ-Цр-1', '{"3": "ИГИ-Цр-1", "4": "церий-144; празеодим-144", "5": "1", "6": "8,0x12,0", "7": "40000000000.0", "8": "80000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Цр-2', 'ИГИ-Цр-2', '{"3": "ИГИ-Цр-2", "4": "церий-144; празеодим-144", "5": "1", "6": "16,0x18,0", "7": "240000000000.0", "8": "470000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Цр-3', 'ИГИ-Цр-3', '{"3": "ИГИ-Цр-3", "4": "церий-144; празеодим-144", "5": "1", "6": "19,0x31,0", "7": "2500000000000.0", "8": "5000000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Цр-4', 'ИГИ-Цр-4', '{"3": "ИГИ-Цр-4", "4": "церий-144; празеодим-144", "5": "1", "6": "35,0x42,0", "7": "12000000000000.0", "8": "28000000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИИР-6', 'ИИР-6', '{"3": "ИИР-6", "4": "иридий-192", "5": "1", "6": "", "7": "6710000000000.0", "8": "0.0", "9": "0", "10": "ЦНИИ КМ Прометей", "11": "0", "12": "из отчета Спб Изотоп", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИИР-8', 'ИИР-8', '{"3": "ИИР-8", "4": "иридий-192", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ЦНИИ КМ Прометей", "11": "0", "12": "из отчета Спб Изотоп", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИК', 'ИМГИК', '{"3": "ИМГИК", "4": "кобальт-60", "5": "1", "6": "6,4х16,0", "7": "", "8": "111000000000.0", "9": "0", "10": "ПО Маяк", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ', 'ИМГИЦ', '{"3": "ИМГИЦ", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ-1', 'ИМГИЦ-1', '{"3": "ИМГИЦ-1", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "37000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ-10', 'ИМГИЦ-10', '{"3": "ИМГИЦ-10", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "370000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ-100', 'ИМГИЦ-100', '{"3": "ИМГИЦ-100", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "3700000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ-15', 'ИМГИЦ-15', '{"3": "ИМГИЦ-15", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "555000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ-150', 'ИМГИЦ-150', '{"3": "ИМГИЦ-150", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "5550000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ-2', 'ИМГИЦ-2', '{"3": "ИМГИЦ-2", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "74000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ-20', 'ИМГИЦ-20', '{"3": "ИМГИЦ-20", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ-200', 'ИМГИЦ-200', '{"3": "ИМГИЦ-200", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "7400000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ2-1', 'ИМГИЦ2-1', '{"3": "ИМГИЦ2-1", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "37000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ2-10', 'ИМГИЦ2-10', '{"3": "ИМГИЦ2-10", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "370000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ2-100', 'ИМГИЦ2-100', '{"3": "ИМГИЦ2-100", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "3700000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ2-15', 'ИМГИЦ2-15', '{"3": "ИМГИЦ2-15", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "555000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ2-150', 'ИМГИЦ2-150', '{"3": "ИМГИЦ2-150", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "5550000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ2-2', 'ИМГИЦ2-2', '{"3": "ИМГИЦ2-2", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "74000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ2-20', 'ИМГИЦ2-20', '{"3": "ИМГИЦ2-20", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "740000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ2-200', 'ИМГИЦ2-200', '{"3": "ИМГИЦ2-200", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "7400000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ2-250', 'ИМГИЦ2-250', '{"3": "ИМГИЦ2-250", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "9250000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ2-3', 'ИМГИЦ2-3', '{"3": "ИМГИЦ2-3", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "111000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ2-30', 'ИМГИЦ2-30', '{"3": "ИМГИЦ2-30", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "1110000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ2-300', 'ИМГИЦ2-300', '{"3": "ИМГИЦ2-300", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "11100000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ2-5', 'ИМГИЦ2-5', '{"3": "ИМГИЦ2-5", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "185000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ-250', 'ИМГИЦ-250', '{"3": "ИМГИЦ-250", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "9250000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ2-50', 'ИМГИЦ2-50', '{"3": "ИМГИЦ2-50", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "1850000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ-3', 'ИМГИЦ-3', '{"3": "ИМГИЦ-3", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "111000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ-30', 'ИМГИЦ-30', '{"3": "ИМГИЦ-30", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "1110000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ-300', 'ИМГИЦ-300', '{"3": "ИМГИЦ-300", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "11100000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ-5', 'ИМГИЦ-5', '{"3": "ИМГИЦ-5", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМГИЦ-50', 'ИМГИЦ-50', '{"3": "ИМГИЦ-50", "4": "цезий-137", "5": "1", "6": "6,4 max (3,2); 16max (4)", "7": "", "8": "1850000000.0", "9": "0", "10": "ПО Маяк", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНГ', 'ИНГ', '{"3": "ИНГ", "4": "тритий", "5": "4", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНГ-06-10', 'ИНГ-06-10', '{"3": "ИНГ-06-10", "4": "тритий", "5": "4", "6": "", "7": "", "8": "70000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНГ-061-10', 'ИНГ-061-10', '{"3": "ИНГ-061-10", "4": "тритий", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНГ-101ТБТ', 'ИНГ-101ТБТ', '{"3": "ИНГ-101ТБТ", "4": "тритий", "5": "4", "6": "", "7": "", "8": "70000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНГ-10-20-120 ТБТ БТ 090076', 'ИНГ-10-20-120 ТБТ БТ 090076', '{"3": "ИНГ-10-20-120 ТБТ БТ 090076", "4": "тритий", "5": "4", "6": "", "7": "", "8": "70000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ', 'ИНИТ', '{"3": "ИНИТ", "4": "тритий", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-1', 'ИНИТ-1', '{"3": "ИНИТ-1", "4": "тритий", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-1, Т1', 'ИНИТ-1, Т1', '{"3": "ИНИТ-1, Т1", "4": "тритий", "5": "2", "6": "", "7": "", "8": "35000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-10', 'ИНИТ-10', '{"3": "ИНИТ-10", "4": "тритий", "5": "2", "6": "", "7": "", "8": "160000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-2', 'ИНИТ-2', '{"3": "ИНИТ-2", "4": "тритий", "5": "2", "6": "", "7": "", "8": "149000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-2, Т2', 'ИНИТ-2, Т2', '{"3": "ИНИТ-2, Т2", "4": "тритий", "5": "2", "6": "", "7": "", "8": "52500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-3', 'ИНИТ-3', '{"3": "ИНИТ-3", "4": "тритий", "5": "2", "6": "", "7": "", "8": "531000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-3, Т3', 'ИНИТ-3, Т3', '{"3": "ИНИТ-3, Т3", "4": "тритий", "5": "2", "6": "", "7": "", "8": "228000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-4', 'ИНИТ-4', '{"3": "ИНИТ-4", "4": "тритий", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-4, Т4', 'ИНИТ-4, Т4', '{"3": "ИНИТ-4, Т4", "4": "тритий", "5": "2", "6": "", "7": "", "8": "59400000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-5', 'ИНИТ-5', '{"3": "ИНИТ-5", "4": "тритий", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-6', 'ИНИТ-6', '{"3": "ИНИТ-6", "4": "тритий", "5": "2", "6": "", "7": "", "8": "3200000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-7', 'ИНИТ-7', '{"3": "ИНИТ-7", "4": "тритий", "5": "2", "6": "", "7": "", "8": "172000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-8', 'ИНИТ-8', '{"3": "ИНИТ-8", "4": "тритий", "5": "2", "6": "", "7": "", "8": "80000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-9', 'ИНИТ-9', '{"3": "ИНИТ-9", "4": "тритий", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-АС-1', 'ИНИТ-АС-1', '{"3": "ИНИТ-АС-1", "4": "тритий", "5": "2", "6": "", "7": "", "8": "360000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-АС-2', 'ИНИТ-АС-2', '{"3": "ИНИТ-АС-2", "4": "тритий", "5": "2", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-АС-6', 'ИНИТ-АС-6', '{"3": "ИНИТ-АС-6", "4": "тритий", "5": "2", "6": "", "7": "", "8": "22200000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-Т1', 'ИНИТ-Т1', '{"3": "ИНИТ-Т1", "4": "тритий", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-Т10', 'ИНИТ-Т10', '{"3": "ИНИТ-Т10", "4": "тритий", "5": "2", "6": "", "7": "", "8": "320000000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-Т2', 'ИНИТ-Т2', '{"3": "ИНИТ-Т2", "4": "тритий", "5": "2", "6": "", "7": "", "8": "50000000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-Т3', 'ИНИТ-Т3', '{"3": "ИНИТ-Т3", "4": "тритий", "5": "2", "6": "", "7": "", "8": "2700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-Т4', 'ИНИТ-Т4', '{"3": "ИНИТ-Т4", "4": "тритий", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-Т5', 'ИНИТ-Т5', '{"3": "ИНИТ-Т5", "4": "тритий", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-Т6', 'ИНИТ-Т6', '{"3": "ИНИТ-Т6", "4": "тритий", "5": "2", "6": "", "7": "", "8": "3200000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-Т7', 'ИНИТ-Т7', '{"3": "ИНИТ-Т7", "4": "тритий", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-Т8', 'ИНИТ-Т8', '{"3": "ИНИТ-Т8", "4": "тритий", "5": "2", "6": "", "7": "", "8": "1600000000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНИТ-Т9', 'ИНИТ-Т9', '{"3": "ИНИТ-Т9", "4": "тритий", "5": "2", "6": "", "7": "", "8": "124000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНТ', 'ИНТ', '{"3": "ИНТ", "4": "тритий", "5": "2", "6": "", "7": "", "8": "124000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНК', 'ИНК', '{"3": "ИНК", "4": "калифорний-252", "5": "4", "6": "", "7": "0.0", "8": "100000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНК-1', 'ИНК-1', '{"3": "ИНК-1", "4": "калифорний-252", "5": "4", "6": "7,0х14,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНК-10', 'ИНК-10', '{"3": "ИНК-10", "4": "калифорний-252", "5": "4", "6": "7,0х14,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНК-1-3', 'ИНК-1-3', '{"3": "ИНК-1-3", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "13100000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНК-2', 'ИНК-2', '{"3": "ИНК-2", "4": "калифорний-252", "5": "4", "6": "7,0х14,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНК-3', 'ИНК-3', '{"3": "ИНК-3", "4": "калифорний-252", "5": "4", "6": "7,0х14,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНК-3-6', 'ИНК-3-6', '{"3": "ИНК-3-6", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "15000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНК-3-7', 'ИНК-3-7', '{"3": "ИНК-3-7", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "990000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНК-4', 'ИНК-4', '{"3": "ИНК-4", "4": "калифорний-252", "5": "4", "6": "7,0х14,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНК-4-3', 'ИНК-4-3', '{"3": "ИНК-4-3", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "426000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНК-5', 'ИНК-5', '{"3": "ИНК-5", "4": "калифорний-252", "5": "4", "6": "7,0х14,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНК-6', 'ИНК-6', '{"3": "ИНК-6", "4": "калифорний-252", "5": "4", "6": "7,0х14,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНК-7', 'ИНК-7', '{"3": "ИНК-7", "4": "калифорний-252", "5": "4", "6": "7,0х14,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНК-8', 'ИНК-8', '{"3": "ИНК-8", "4": "калифорний-252", "5": "4", "6": "7,0х14,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНК-9', 'ИНК-9', '{"3": "ИНК-9", "4": "калифорний-252", "5": "4", "6": "7,0х14,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНК-М', 'ИНК-М', '{"3": "ИНК-М", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "7450000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНКМ-3 30-2', 'ИНКМ-3 30-2', '{"3": "ИНКМ-3 30-2", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "140000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНКМ-3 30-2А', 'ИНКМ-3 30-2А', '{"3": "ИНКМ-3 30-2А", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "170000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНКМ-Ш10-1', 'ИНКМ-Ш10-1', '{"3": "ИНКМ-Ш10-1", "4": "калифорний-252", "5": "4", "6": "1,2х15,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНКМ-Ш10-2', 'ИНКМ-Ш10-2', '{"3": "ИНКМ-Ш10-2", "4": "калифорний-252", "5": "4", "6": "1,2х15,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНКМ-Ш20-1', 'ИНКМ-Ш20-1', '{"3": "ИНКМ-Ш20-1", "4": "калифорний-252", "5": "4", "6": "1,2х25,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНКМ-Ш20-2', 'ИНКМ-Ш20-2', '{"3": "ИНКМ-Ш20-2", "4": "калифорний-252", "5": "4", "6": "1,2х25,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНКМ-Ш30-1', 'ИНКМ-Ш30-1', '{"3": "ИНКМ-Ш30-1", "4": "калифорний-252", "5": "4", "6": "1,2х35,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНКМ-Ш30-2', 'ИНКМ-Ш30-2', '{"3": "ИНКМ-Ш30-2", "4": "калифорний-252", "5": "4", "6": "1,2х35,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Иридиевый диск', 'Иридиевый диск', '{"3": "Иридиевый диск", "4": "иридий-192", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИЖ', 'ИРИЖ', '{"3": "ИРИЖ", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИЖ-1', 'ИРИЖ-1', '{"3": "ИРИЖ-1", "4": "железо-55", "5": "5", "6": "14 (10); 1", "7": "", "8": "140000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИЖ-2', 'ИРИЖ-2', '{"3": "ИРИЖ-2", "4": "железо-55", "5": "5", "6": "14 (10); 1", "7": "", "8": "710000000.0", "9": "0", "10": "", "11": "5", "12": "Открытый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИЖ-3', 'ИРИЖ-3', '{"3": "ИРИЖ-3", "4": "железо-55", "5": "5", "6": "14 (10); 1", "7": "", "8": "3500000000.0", "9": "0", "10": "", "11": "5", "12": "Открытый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИЖ-30', 'ИРИЖ-30', '{"3": "ИРИЖ-30", "4": "железо-55", "5": "5", "6": "0,6х47,0", "7": "", "8": "55500.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИЖ-55-1', 'ИРИЖ-55-1', '{"3": "ИРИЖ-55-1", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "210000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИЖ-V', 'ИРИЖ-V', '{"3": "ИРИЖ-V", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "3500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИЖ-Р', 'ИРИЖ-Р', '{"3": "ИРИЖ-Р", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "5470.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-АД-1', 'ИРИК-АД-1', '{"3": "ИРИК-АД-1", "4": "кадмий-109", "5": "5", "6": "7,0x2,2", "7": "", "8": "100000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-АД-2', 'ИРИК-АД-2', '{"3": "ИРИК-АД-2", "4": "кадмий-109", "5": "5", "6": "7,0x2,2", "7": "", "8": "250000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-АД-3', 'ИРИК-АД-3', '{"3": "ИРИК-АД-3", "4": "кадмий-109", "5": "5", "6": "7,0x2,2", "7": "", "8": "500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-АД-4', 'ИРИК-АД-4', '{"3": "ИРИК-АД-4", "4": "кадмий-109", "5": "5", "6": "7,0x2,2", "7": "", "8": "1000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-АД-5', 'ИРИК-АД-5', '{"3": "ИРИК-АД-5", "4": "кадмий-109", "5": "5", "6": "7,0x2,2", "7": "", "8": "2500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-АД-6', 'ИРИК-АД-6', '{"3": "ИРИК-АД-6", "4": "кадмий-109", "5": "5", "6": "7,0x2,2", "7": "", "8": "5000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-АК-1', 'ИРИК-АК-1', '{"3": "ИРИК-АК-1", "4": "кадмий-109", "5": "5", "6": "30x20х2,2", "7": "", "8": "250000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-АК-2', 'ИРИК-АК-2', '{"3": "ИРИК-АК-2", "4": "кадмий-109", "5": "5", "6": "30x20х2,2", "7": "", "8": "500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-АК-3', 'ИРИК-АК-3', '{"3": "ИРИК-АК-3", "4": "кадмий-109", "5": "5", "6": "30x20х2,2", "7": "", "8": "1000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-АК-4', 'ИРИК-АК-4', '{"3": "ИРИК-АК-4", "4": "кадмий-109", "5": "5", "6": "30x20х2,2", "7": "", "8": "2500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-АТ-1', 'ИРИК-АТ-1', '{"3": "ИРИК-АТ-1", "4": "кадмий-109", "5": "5", "6": "3,0x3,0", "7": "", "8": "100000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-АТ-2', 'ИРИК-АТ-2', '{"3": "ИРИК-АТ-2", "4": "кадмий-109", "5": "5", "6": "3,0x3,0", "7": "", "8": "250000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-АТ-3', 'ИРИК-АТ-3', '{"3": "ИРИК-АТ-3", "4": "кадмий-109", "5": "5", "6": "3,0x3,0", "7": "", "8": "500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-АТ-4', 'ИРИК-АТ-4', '{"3": "ИРИК-АТ-4", "4": "кадмий-109", "5": "5", "6": "3,0x3,0", "7": "", "8": "1000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-АТ-5', 'ИРИК-АТ-5', '{"3": "ИРИК-АТ-5", "4": "кадмий-109", "5": "5", "6": "3,0x3,0", "7": "", "8": "2000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-Д', 'ИРИК-Д', '{"3": "ИРИК-Д", "4": "железо-55", "5": "1", "6": "14x2", "7": "", "8": "59200000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-Д', 'ИРИК-Д', '{"3": "ИРИК-Д", "4": "кадмий-109", "5": "1", "6": "3x3", "7": "", "8": "11100000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-Д', 'ИРИК-Д', '{"3": "ИРИК-Д", "4": "кобальт-57", "5": "1", "6": "3x3", "7": "", "8": "11100000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-Д1', 'ИРИК-Д1', '{"3": "ИРИК-Д1", "4": "железо-55", "5": "5", "6": "4x1,5 или 4x4", "7": "", "8": "200000000.0", "9": "0", "10": "", "11": "5", "12": "Титановая капсула с бериллиевым окном d:2мм", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-Д1', 'ИРИК-Д1', '{"3": "ИРИК-Д1", "4": "кадмий-109", "5": "5", "6": "4x4 или 4x1,5", "7": "", "8": "100000000.0", "9": "0", "10": "", "11": "5", "12": "Титановая капсула с бериллиевым окном D2мм", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-Д1', 'ИРИК-Д1', '{"3": "ИРИК-Д1", "4": "кобальт-57", "5": "1", "6": "4x4; 4x1,5", "7": "", "8": "500000000.0", "9": "0", "10": "", "11": "5", "12": "Титановая капсула с бериллиевым окном d -2мм.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-Д2', 'ИРИК-Д2', '{"3": "ИРИК-Д2", "4": "железо-55", "5": "5", "6": "4,0x1,5", "7": "", "8": "500000000.0", "9": "0", "10": "", "11": "5", "12": "Титановая капсула с бериллиевым окном D2мм", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-Д2', 'ИРИК-Д2', '{"3": "ИРИК-Д2", "4": "кадмий-109", "5": "5", "6": "4x4 или 4x1,5", "7": "", "8": "250000000.0", "9": "0", "10": "", "11": "5", "12": "Титановая капсула с бериллиевым окном D2мм", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-Д2', 'ИРИК-Д2', '{"3": "ИРИК-Д2", "4": "кобальт-57", "5": "1", "6": "4x4; 4x1,5", "7": "", "8": "1000000000.0", "9": "0", "10": "", "11": "5", "12": "Титановая капсула с бериллиевым окном d -2мм.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-Д3', 'ИРИК-Д3', '{"3": "ИРИК-Д3", "4": "кадмий-109", "5": "5", "6": "4x4 или 4x1,5", "7": "", "8": "500000000.0", "9": "0", "10": "", "11": "5", "12": "Титановая капсула с бериллиевым окном D2мм", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-Д3', 'ИРИК-Д3', '{"3": "ИРИК-Д3", "4": "кобальт-57", "5": "1", "6": "4x4; 4x1,5", "7": "", "8": "2000000000.0", "9": "0", "10": "", "11": "5", "12": "Титановая капсула с бериллиевым окном d -2мм.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-Д4', 'ИРИК-Д4', '{"3": "ИРИК-Д4", "4": "кадмий-109", "5": "5", "6": "4x4 или 4x1,5", "7": "", "8": "1000000000.0", "9": "0", "10": "", "11": "5", "12": "Титановая капсула с бериллиевым окном D2мм", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-Д4', 'ИРИК-Д4', '{"3": "ИРИК-Д4", "4": "кобальт-57", "5": "1", "6": "4x4; 4x1,5", "7": "", "8": "3000000000.0", "9": "0", "10": "", "11": "5", "12": "Титановая капсула с бериллиевым окном d -2мм.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-Д5', 'ИРИК-Д5', '{"3": "ИРИК-Д5", "4": "кадмий-109", "5": "5", "6": "4x4 или 4x1,5", "7": "", "8": "2500000000.0", "9": "0", "10": "", "11": "5", "12": "Титановая капсула с бериллиевым окном D2мм", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-Д5', 'ИРИК-Д5', '{"3": "ИРИК-Д5", "4": "кобальт-57", "5": "1", "6": "4x4; 4x1,5", "7": "", "8": "4000000000.0", "9": "0", "10": "", "11": "5", "12": "Титановая капсула с бериллиевым окном d -2мм.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК-Д6', 'ИРИК-Д6', '{"3": "ИРИК-Д6", "4": "кадмий-109", "5": "5", "6": "4x4 или 4x1,5", "7": "", "8": "5000000000.0", "9": "0", "10": "", "11": "5", "12": "Титановая капсула с бериллиевым окном D2мм", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИП', 'ИРИП', '{"3": "ИРИП", "4": "прометий-147", "5": "5", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИП-1', 'ИРИП-1', '{"3": "ИРИП-1", "4": "прометий-147", "5": "5", "6": "10 (5,9); 14", "7": "", "8": "24000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИП-2', 'ИРИП-2', '{"3": "ИРИП-2", "4": "прометий-147", "5": "5", "6": "10 (5,9); 14", "7": "", "8": "120000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИП-3', 'ИРИП-3', '{"3": "ИРИП-3", "4": "прометий-147", "5": "5", "6": "10 (5,9); 14", "7": "", "8": "590000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИП-4', 'ИРИП-4', '{"3": "ИРИП-4", "4": "прометий-147", "5": "5", "6": "10 (5,9); 14", "7": "", "8": "1600000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИПЛ-1', 'ИРИПЛ-1', '{"3": "ИРИПЛ-1", "4": "плутоний-238", "5": "5", "6": "12 (7); 10", "7": "", "8": "660000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИПЛ-2', 'ИРИПЛ-2', '{"3": "ИРИПЛ-2", "4": "плутоний-238", "5": "5", "6": "12 (7); 10", "7": "", "8": "1300000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИПЛ-3', 'ИРИПЛ-3', '{"3": "ИРИПЛ-3", "4": "плутоний-238", "5": "5", "6": "12 (7); 10", "7": "", "8": "2600000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИС-1', 'ИРИС-1', '{"3": "ИРИС-1", "4": "стронций-90; иттрий-90", "5": "5", "6": "14,2 (9); 17,6", "7": "", "8": "55000000000.0", "9": "0", "10": "", "11": "4", "12": "Открытый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИС-2', 'ИРИС-2', '{"3": "ИРИС-2", "4": "стронций-90; иттрий-90", "5": "5", "6": "14,2 (9); 17,6", "7": "", "8": "250000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИС-3', 'ИРИС-3', '{"3": "ИРИС-3", "4": "стронций-90; иттрий-90", "5": "5", "6": "14,2 (9); 17,6", "7": "", "8": "1200000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИТ-1', 'ИРИТ-1', '{"3": "ИРИТ-1", "4": "таллий-204", "5": "5", "6": "15,5 (10); 8,5", "7": "", "8": "38000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИТ-2', 'ИРИТ-2', '{"3": "ИРИТ-2", "4": "таллий-204", "5": "5", "6": "15,5 (10); 8,5", "7": "", "8": "1500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИТ-3', 'ИРИТ-3', '{"3": "ИРИТ-3", "4": "таллий-204", "5": "5", "6": "15,5 (10); 8,5", "7": "", "8": "7500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИТ-4', 'ИРИТ-4', '{"3": "ИРИТ-4", "4": "таллий-204", "5": "5", "6": "15,5 (10); 8,5", "7": "", "8": "38000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРУС-1-1', 'ИРУС-1-1', '{"3": "ИРУС-1-1", "4": "стронций-90; иттрий-90", "5": "2", "6": "38,0x9,0", "7": "", "8": "500000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРУС-1-2', 'ИРУС-1-2', '{"3": "ИРУС-1-2", "4": "стронций-90; иттрий-90", "5": "2", "6": "38,0x9,0", "7": "", "8": "1500000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРУС-1-3', 'ИРУС-1-3', '{"3": "ИРУС-1-3", "4": "стронций-90; иттрий-90", "5": "2", "6": "38,0x9,0", "7": "", "8": "3000000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРУС-2', 'ИРУС-2', '{"3": "ИРУС-2", "4": "стронций-90; иттрий-90", "5": "2", "6": "87,0x12,0", "7": "", "8": "30000000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРУС-М-1', 'ИРУС-М-1', '{"3": "ИРУС-М-1", "4": "стронций-90; иттрий-90", "5": "2", "6": "11,0x87,0", "7": "", "8": "3000000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРУС-М-2', 'ИРУС-М-2', '{"3": "ИРУС-М-2", "4": "стронций-90; иттрий-90", "5": "2", "6": "11,0x87,0", "7": "", "8": "30000000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИТОР-1', 'ИТОР-1', '{"3": "ИТОР-1", "4": "кобальт-60", "5": "1", "6": "53,0х23,0х49,0", "7": "0.0", "8": "2020000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К', 'К', '{"3": "К", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-106', 'К-106', '{"3": "К-106", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "14400.0", "9": "0", "10": "", "11": "0", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-110', 'К-110', '{"3": "К-110", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "24000.0", "9": "0", "10": "", "11": "0", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-117', 'К-117', '{"3": "К-117", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "240000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-12', 'К-12', '{"3": "К-12", "4": "кобальт-60", "5": "2", "6": "", "7": "0.0", "8": "3500.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-122', 'К-122', '{"3": "К-122", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "18800.0", "9": "0", "10": "", "11": "0", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-14', 'К-14', '{"3": "К-14", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "320000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-146', 'К-146', '{"3": "К-146", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "190000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-17', 'К-17', '{"3": "К-17", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "220000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-229-91', 'К-229-91', '{"3": "К-229-91", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "20000.0", "9": "0", "10": "", "11": "0", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-3', 'К-3', '{"3": "К-3", "4": "кобальт-60", "5": "1", "6": "30(9); 4,5", "7": "", "8": "22000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-39', 'К-39', '{"3": "К-39", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "21000.0", "9": "0", "10": "", "11": "0", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-3А', 'К-3А', '{"3": "К-3А", "4": "кобальт-60", "5": "1", "6": "30(9); 4,5", "7": "", "8": "44000.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-4', 'К-4', '{"3": "К-4", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "190000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-40', 'К-40', '{"3": "К-40", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "29000.0", "9": "0", "10": "", "11": "0", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-419-84', 'К-419-84', '{"3": "К-419-84", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "14000.0", "9": "0", "10": "", "11": "0", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-422', 'К-422', '{"3": "К-422", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "14000.0", "9": "0", "10": "", "11": "0", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-43', 'К-43', '{"3": "К-43", "4": "кобальт-60", "5": "2", "6": "", "7": "0.0", "8": "29000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-47', 'К-47', '{"3": "К-47", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "30000.0", "9": "0", "10": "", "11": "0", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-48', 'К-48', '{"3": "К-48", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "22000.0", "9": "0", "10": "", "11": "0", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-5', 'К-5', '{"3": "К-5", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "220000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-51', 'К-51', '{"3": "К-51", "4": "кобальт-60", "5": "2", "6": "", "7": "0.0", "8": "320.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-56', 'К-56', '{"3": "К-56", "4": "кобальт-60", "5": "2", "6": "", "7": "0.0", "8": "270.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-58', 'К-58', '{"3": "К-58", "4": "кобальт-60", "5": "2", "6": "", "7": "0.0", "8": "3500.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-619', 'К-619', '{"3": "К-619", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "29000.0", "9": "0", "10": "", "11": "0", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-65', 'К-65', '{"3": "К-65", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "19000.0", "9": "0", "10": "", "11": "0", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К7Л', 'К7Л', '{"3": "К7Л", "4": "кобальт-60", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-8', 'К-8', '{"3": "К-8", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "190000.0", "9": "0", "10": "", "11": "5", "12": "Вариант", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Кактус', 'Кактус', '{"3": "Кактус", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Кактус Р1/1', 'Кактус Р1/1', '{"3": "Кактус Р1/1", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "400000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Кактус Р1/14', 'Кактус Р1/14', '{"3": "Кактус Р1/14", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "55000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Кактус Р1/3', 'Кактус Р1/3', '{"3": "Кактус Р1/3", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Кактус Р1/4', 'Кактус Р1/4', '{"3": "Кактус Р1/4", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "48000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Кактус Р1/5', 'Кактус Р1/5', '{"3": "Кактус Р1/5", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "300000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Кактус Р1/6', 'Кактус Р1/6', '{"3": "Кактус Р1/6", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "1000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Кактус Р1/7', 'Кактус Р1/7', '{"3": "Кактус Р1/7", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Кактус Р1/8', 'Кактус Р1/8', '{"3": "Кактус Р1/8", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "50000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Кактус Р1/9', 'Кактус Р1/9', '{"3": "Кактус Р1/9", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "990000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Кактус Р2/4', 'Кактус Р2/4', '{"3": "Кактус Р2/4", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "79600000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Кактус Р2/5', 'Кактус Р2/5', '{"3": "Кактус Р2/5", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "168000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Кактус Р21/3', 'Кактус Р21/3', '{"3": "Кактус Р21/3", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "3000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Калифорниевый', 'Калифорниевый', '{"3": "Калифорниевый", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Камера измерительная ОИСН', 'Камера измерительная ОИСН', '{"3": "Камера измерительная ОИСН", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "77900.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КГИ', 'КГИ', '{"3": "КГИ", "4": "цезий-137", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КИГ', 'КИГ', '{"3": "КИГ", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КИГ-АД-1', 'КИГ-АД-1', '{"3": "КИГ-АД-1", "4": "кобальт-57", "5": "1", "6": "12,0x3,0", "7": "", "8": "500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КИГ-АД-2', 'КИГ-АД-2', '{"3": "КИГ-АД-2", "4": "кобальт-57", "5": "1", "6": "12,0x3,0", "7": "", "8": "1000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КИГ-АД-3', 'КИГ-АД-3', '{"3": "КИГ-АД-3", "4": "кобальт-57", "5": "1", "6": "12,0x3,0", "7": "", "8": "2500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КИГ-АД-4', 'КИГ-АД-4', '{"3": "КИГ-АД-4", "4": "кобальт-57", "5": "1", "6": "12,0x3,0", "7": "", "8": "5000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КИГ-АДМ-1', 'КИГ-АДМ-1', '{"3": "КИГ-АДМ-1", "4": "кобальт-57", "5": "1", "6": "7,0x2,2", "7": "", "8": "500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КИГ-АДМ-2', 'КИГ-АДМ-2', '{"3": "КИГ-АДМ-2", "4": "кобальт-57", "5": "1", "6": "7,0x2,2", "7": "", "8": "1000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КИГ-АДМ-3', 'КИГ-АДМ-3', '{"3": "КИГ-АДМ-3", "4": "кобальт-57", "5": "1", "6": "7,0x2,2", "7": "", "8": "2500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КИГ-АДМ-4', 'КИГ-АДМ-4', '{"3": "КИГ-АДМ-4", "4": "кобальт-57", "5": "1", "6": "7,0x2,2", "7": "", "8": "5000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КИГ-АК-1', 'КИГ-АК-1', '{"3": "КИГ-АК-1", "4": "кобальт-57", "5": "1", "6": "30x2,2(d=20)", "7": "", "8": "500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КИГ-АК-2', 'КИГ-АК-2', '{"3": "КИГ-АК-2", "4": "кобальт-57", "5": "1", "6": "30,0x2,2(d=20)", "7": "", "8": "1000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КИГ-АК-3', 'КИГ-АК-3', '{"3": "КИГ-АК-3", "4": "кобальт-57", "5": "1", "6": "30x2,2(d=20)", "7": "", "8": "2500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КИГ-АК-4', 'КИГ-АК-4', '{"3": "КИГ-АК-4", "4": "кобальт-57", "5": "1", "6": "30x2,2(d=20)", "7": "", "8": "5000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КИГ-АТ-1', 'КИГ-АТ-1', '{"3": "КИГ-АТ-1", "4": "кобальт-57", "5": "1", "6": "3,0x3,0", "7": "", "8": "500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КИГ-АТ-2', 'КИГ-АТ-2', '{"3": "КИГ-АТ-2", "4": "кобальт-57", "5": "1", "6": "3,0x3,0", "7": "", "8": "1000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КИГ-АТ-3', 'КИГ-АТ-3', '{"3": "КИГ-АТ-3", "4": "кобальт-57", "5": "1", "6": "3,0x3,0", "7": "", "8": "2500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ', 'КМ', '{"3": "КМ", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ 200', 'КМ 200', '{"3": "КМ 200", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "11000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ13-6', 'КМ13-6', '{"3": "КМ13-6", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "4600000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ1-4', 'КМ1-4', '{"3": "КМ1-4", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "24000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ1-5', 'КМ1-5', '{"3": "КМ1-5", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "2100000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ2-3', 'КМ2-3', '{"3": "КМ2-3", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "2700000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ2-4', 'КМ2-4', '{"3": "КМ2-4", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "28000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ2-5', 'КМ2-5', '{"3": "КМ2-5", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "6030000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ2-6', 'КМ2-6', '{"3": "КМ2-6", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "300000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ3-2', 'КМ3-2', '{"3": "КМ3-2", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "1870000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ3-3', 'КМ3-3', '{"3": "КМ3-3", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "890000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ3-4', 'КМ3-4', '{"3": "КМ3-4", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "182000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ3-5', 'КМ3-5', '{"3": "КМ3-5", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "16000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ3-6', 'КМ3-6', '{"3": "КМ3-6", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "390000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ3-6, тип П', 'КМ3-6, тип П', '{"3": "КМ3-6, тип П", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "360000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ3-7', 'КМ3-7', '{"3": "КМ3-7", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "95300000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ3-7, тип П', 'КМ3-7, тип П', '{"3": "КМ3-7, тип П", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "360000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ5-4', 'КМ5-4', '{"3": "КМ5-4", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "870000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ5-7', 'КМ5-7', '{"3": "КМ5-7", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "11000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ6-4', 'КМ6-4', '{"3": "КМ6-4", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "86000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ6-5', 'КМ6-5', '{"3": "КМ6-5", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "10000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ6-6', 'КМ6-6', '{"3": "КМ6-6", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "788000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ6-8', 'КМ6-8', '{"3": "КМ6-8", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "29500000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ-8', 'КМ-8', '{"3": "КМ-8", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "380000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ-И-14', 'КМ-И-14', '{"3": "КМ-И-14", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "5300000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ-И-17', 'КМ-И-17', '{"3": "КМ-И-17", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "5800000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ-И-18', 'КМ-И-18', '{"3": "КМ-И-18", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "7100000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ-И-2', 'КМ-И-2', '{"3": "КМ-И-2", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "12000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ-И-56', 'КМ-И-56', '{"3": "КМ-И-56", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "22000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНК-15-1', 'КНК-15-1', '{"3": "КНК-15-1", "4": "уран-235", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "камера нейтронная с компенсацией гамма-фона", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ', 'КНТ', '{"3": "КНТ", "4": "уран-235", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ-15', 'КНТ-15', '{"3": "КНТ-15", "4": "уран-235", "5": "1", "6": "", "7": "", "8": "8600.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ-15-1-0177', 'КНТ-15-1-0177', '{"3": "КНТ-15-1-0177", "4": "уран-235", "5": "1", "6": "", "7": "", "8": "8600.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ-15-1276', 'КНТ-15-1276', '{"3": "КНТ-15-1276", "4": "уран-235", "5": "1", "6": "", "7": "", "8": "34800.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ-15-75', 'КНТ-15-75', '{"3": "КНТ-15-75", "4": "уран-235", "5": "1", "6": "", "7": "", "8": "34800.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ-16-0577', 'КНТ-16-0577', '{"3": "КНТ-16-0577", "4": "уран-235", "5": "1", "6": "", "7": "", "8": "8600.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ-16-1276', 'КНТ-16-1276', '{"3": "КНТ-16-1276", "4": "уран-235", "5": "1", "6": "", "7": "", "8": "8600.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ-16-76', 'КНТ-16-76', '{"3": "КНТ-16-76", "4": "уран-235", "5": "1", "6": "", "7": "", "8": "8600.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ-23-1', 'КНТ-23-1', '{"3": "КНТ-23-1", "4": "уран-235", "5": "1", "6": "", "7": "", "8": "240000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ-23-1-8', 'КНТ-23-1-8', '{"3": "КНТ-23-1-8", "4": "уран-235", "5": "1", "6": "", "7": "", "8": "240000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ-23-8', 'КНТ-23-8', '{"3": "КНТ-23-8", "4": "уран-235", "5": "1", "6": "", "7": "", "8": "240000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ-24-1', 'КНТ-24-1', '{"3": "КНТ-24-1", "4": "уран-235", "5": "1", "6": "", "7": "", "8": "181000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ-31', 'КНТ-31', '{"3": "КНТ-31", "4": "уран-235", "5": "1", "6": "", "7": "", "8": "290000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ-31-1', 'КНТ-31-1', '{"3": "КНТ-31-1", "4": "уран-235", "5": "1", "6": "", "7": "", "8": "185000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ-31-68', 'КНТ-31-68', '{"3": "КНТ-31-68", "4": "уран-235", "5": "1", "6": "", "7": "", "8": "39600.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ-5', 'КНТ-5', '{"3": "КНТ-5", "4": "уран-235", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ-54', 'КНТ-54', '{"3": "КНТ-54", "4": "уран-235", "5": "1", "6": "", "7": "", "8": "310000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ-54-1', 'КНТ-54-1', '{"3": "КНТ-54-1", "4": "уран-235", "5": "1", "6": "", "7": "", "8": "84800.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ-6', 'КНТ-6', '{"3": "КНТ-6", "4": "уран-235", "5": "1", "6": "", "7": "", "8": "121000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КтВ-К', 'КтВ-К', '{"3": "КтВ-К", "4": "уран-235", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МИКХ', 'МИКХ', '{"3": "МИКХ", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МИКХ ЭДГХ', 'МИКХ ЭДГХ', '{"3": "МИКХ ЭДГХ", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "2300000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МИКХ-1', 'МИКХ-1', '{"3": "МИКХ-1", "4": "кобальт-57", "5": "6", "6": "16,0x13,0", "7": "", "8": "500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МИКХ-2', 'МИКХ-2', '{"3": "МИКХ-2", "4": "кобальт-57", "5": "6", "6": "16,0x13,0", "7": "", "8": "1000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МИКХ-3', 'МИКХ-3', '{"3": "МИКХ-3", "4": "кобальт-57", "5": "6", "6": "16,0x13,0", "7": "", "8": "2500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МИКХ-4', 'МИКХ-4', '{"3": "МИКХ-4", "4": "кобальт-57", "5": "6", "6": "16,0x13,0", "7": "", "8": "5000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МИКХ-5', 'МИКХ-5', '{"3": "МИКХ-5", "4": "кобальт-57", "5": "6", "6": "16,0x13,0", "7": "", "8": "10000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МИКХ-6', 'МИКХ-6', '{"3": "МИКХ-6", "4": "кобальт-57", "5": "6", "6": "16,0x13,0", "7": "", "8": "5000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МИКХ-7', 'МИКХ-7', '{"3": "МИКХ-7", "4": "кобальт-57", "5": "6", "6": "16,0x13,0", "7": "", "8": "10000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МИКХ-II', 'МИКХ-II', '{"3": "МИКХ-II", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "5040000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МИКХ-III', 'МИКХ-III', '{"3": "МИКХ-III", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "2500000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МИР-3А', 'МИР-3А', '{"3": "МИР-3А", "4": "плутоний-239", "5": "3", "6": "", "7": "0.0", "8": "125000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МК57.ВР01.80/94', 'МК57.ВР01.80/94', '{"3": "МК57.ВР01.80/94", "4": "кобальт-57", "5": "2", "6": "", "7": "0.0", "8": "500000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МК57.ВР1, 5', 'МК57.ВР1, 5', '{"3": "МК57.ВР1, 5", "4": "кобальт-57", "5": "4", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МК57.ВР1.01', 'МК57.ВР1.01', '{"3": "МК57.ВР1.01", "4": "кобальт-57", "5": "4", "6": "0,7", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "По согласованию с заказчиком капсула может быть изготовлена: • из титана с бериллиевым выходным окном (ИСО классификация С 54334); • из алюминия с выходным окном из радиационностойкого каптона толщиной 20 мкм (ИСО классификация С 22221). В качестве материала подложки могут быть использованы хром, палладий, платина или медь. По согласованию с заказчиком диаметр активной части, а также габаритные размеры капсулы могут быть изменены, а активность Co-57 увеличена до 1,11e; 10 Бк (300 мКи).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МК57.ВР5.01', 'МК57.ВР5.01', '{"3": "МК57.ВР5.01", "4": "кобальт-57", "5": "4", "6": "10х10", "7": "0.0", "8": "370000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МК57.ВР5.02', 'МК57.ВР5.02', '{"3": "МК57.ВР5.02", "4": "кобальт-57", "5": "4", "6": "10х10", "7": "0.0", "8": "925000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МК57.ВР5.03', 'МК57.ВР5.03', '{"3": "МК57.ВР5.03", "4": "кобальт-57", "5": "4", "6": "10х10", "7": "0.0", "8": "1850000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МК57.ВР5.04', 'МК57.ВР5.04', '{"3": "МК57.ВР5.04", "4": "кобальт-57", "5": "4", "6": "10x10", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "По согласованию с заказчиком капсула может быть изготовлена: • из титана с бериллиевым выходным окном (ИСО классификация С 54334); • из алюминия с выходным окном из радиационностойкого каптона толщиной 20 мкм (ИСО классификация С 22221). В качестве материала подложки могут быть использованы хром, палладий, платина или медь. По согласованию с заказчиком диаметр активной части, а также габаритные размеры капсулы могут быть изменены, а активность Co-57 увеличена до 1,11e; 10 Бк (300 мКи).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МК57.ВР5.05', 'МК57.ВР5.05', '{"3": "МК57.ВР5.05", "4": "кобальт-57", "5": "4", "6": "10х10", "7": "0.0", "8": "5550000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МК57ВР.4, 7', 'МК57ВР.4, 7', '{"3": "МК57ВР.4, 7", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МК57ВР.4.01', 'МК57ВР.4.01', '{"3": "МК57ВР.4.01", "4": "кобальт-57", "5": "6", "6": "10,0x10,0", "7": "", "8": "5000000000.0", "9": "0", "10": "", "11": "5", "12": "Матрица - родий, хром, палладий, платина", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МК57ВР.4.02', 'МК57ВР.4.02', '{"3": "МК57ВР.4.02", "4": "кобальт-57", "5": "6", "6": "10,0x10,0", "7": "", "8": "10000000000.0", "9": "0", "10": "", "11": "4", "12": "Матрица - родий, хром, палладий, платина", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МК57ВР.7.01', 'МК57ВР.7.01', '{"3": "МК57ВР.7.01", "4": "кобальт-57", "5": "6", "6": "10,0x10,0", "7": "", "8": "500000000.0", "9": "0", "10": "", "11": "5", "12": "Матрица - родий, хром, палладий, платина", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МК57ВР.7.02', 'МК57ВР.7.02', '{"3": "МК57ВР.7.02", "4": "кобальт-57", "5": "6", "6": "10,0x10,0", "7": "", "8": "1000000000.0", "9": "0", "10": "", "11": "5", "12": "Матрица - родий, хром, палладий, платина", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МК57ВР.7.03', 'МК57ВР.7.03', '{"3": "МК57ВР.7.03", "4": "кобальт-57", "5": "6", "6": "10,0x10,0", "7": "", "8": "2500000000.0", "9": "0", "10": "", "11": "5", "12": "Матрица - родий, хром, палладий, платина", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МК57ВР.7.04', 'МК57ВР.7.04', '{"3": "МК57ВР.7.04", "4": "кобальт-57", "5": "6", "6": "10,0x10,0", "7": "", "8": "5000000000.0", "9": "0", "10": "", "11": "5", "12": "Матрица - родий, хром, палладий, платина", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МК57ВР.7.05', 'МК57ВР.7.05', '{"3": "МК57ВР.7.05", "4": "кобальт-57", "5": "6", "6": "10,0x10,0", "7": "", "8": "10000000000.0", "9": "0", "10": "", "11": "4", "12": "Матрица - родий, хром, палладий, платина", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МК57Г', 'МК57Г', '{"3": "МК57Г", "4": "кобальт-57", "5": "6", "6": "14,0x12,0", "7": "200000000.0", "8": "8000000000.0", "9": "0", "10": "", "11": "4", "12": "Титановая капсула", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Н-1', 'Н-1', '{"3": "Н-1", "4": "радий-226", "5": "3", "6": "", "7": "0.0", "8": "372000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НCf2', 'НCf2', '{"3": "НCf2", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НCf2.01.03', 'НCf2.01.03', '{"3": "НCf2.01.03", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "6000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НCf2.011.1', 'НCf2.011.1', '{"3": "НCf2.011.1", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "12000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НCf2.011.11', 'НCf2.011.11', '{"3": "НCf2.011.11", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "6000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НCf2.011.13', 'НCf2.011.13', '{"3": "НCf2.011.13", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "8560000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НCf2.011.3', 'НCf2.011.3', '{"3": "НCf2.011.3", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "60000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НCf2.011.6', 'НCf2.011.6', '{"3": "НCf2.011.6", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "240000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НCf2.011.7', 'НCf2.011.7', '{"3": "НCf2.011.7", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "600000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НCf2.011.9', 'НCf2.011.9', '{"3": "НCf2.011.9", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "107000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НCf2.012', 'НCf2.012', '{"3": "НCf2.012", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "60000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НCf2.022.2', 'НCf2.022.2', '{"3": "НCf2.022.2", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "19400000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НCf2.023.4', 'НCf2.023.4', '{"3": "НCf2.023.4", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "90000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НCf2.023.5', 'НCf2.023.5', '{"3": "НCf2.023.5", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "85000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НCf2.024', 'НCf2.024', '{"3": "НCf2.024", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "1500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НCf2.041', 'НCf2.041', '{"3": "НCf2.041", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "13000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НCf2.041.9', 'НCf2.041.9', '{"3": "НCf2.041.9", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "7000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НCf2.P01', 'НCf2.P01', '{"3": "НCf2.P01", "4": "калифорний-252", "5": "4", "6": "6x5. Акт. часть d : 4", "7": "12000.0", "8": "400000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НCf2.P01.1204', 'НCf2.P01.1204', '{"3": "НCf2.P01.1204", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "100000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НCf2.P01.3004', 'НCf2.P01.3004', '{"3": "НCf2.P01.3004", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "30000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НCf2.P01.4005', 'НCf2.P01.4005', '{"3": "НCf2.P01.4005", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "1000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НCf2.P01.6004', 'НCf2.P01.6004', '{"3": "НCf2.P01.6004", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "60000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Набор "1" или "1а" или "1аи"', 'Набор "1" или "1а" или "1аи"', '{"3": "Набор \"1\" или \"1а\" или \"1аи\"", "4": "плутоний-239", "5": "3", "6": "1; 4; 10; 40; 100; 160", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Состоит из нескольких комплектов источников (1П9; 2П9; 3П9; 4П9; 5П9; 6П9).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Набор "2а"', 'Набор "2а"', '{"3": "Набор \"2а\"", "4": "уран естественный", "5": "3", "6": "1; 10; 160", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Состоит из нескольких источников (1У8, 3У8, 6У8).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Набор "2аи"', 'Набор "2аи"', '{"3": "Набор \"2аи\"", "4": "уран естественный", "5": "3", "6": "10; 160", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Состоит из нескольких источников (3У8, 6У8).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Набор "2б"', 'Набор "2б"', '{"3": "Набор \"2б\"", "4": "таллий-204", "5": "2", "6": "1; 4; 10; 40", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Состоит из нескольких источников.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Набор "3б"', 'Набор "3б"', '{"3": "Набор \"3б\"", "4": "кобальт-60", "5": "2", "6": "1; 10", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Состоит из нескольких источников.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Набор "4" или "4а"', 'Набор "4" или "4а"', '{"3": "Набор \"4\" или \"4а\"", "4": "уран-234", "5": "3", "6": "1; 10", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Состоит из нескольких источников 1У4 и 3У4.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Набор "4аи"', 'Набор "4аи"', '{"3": "Набор \"4аи\"", "4": "уран-234", "5": "3", "6": "1; 10", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Состоит из нескольких источников (1У4, 3У4).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Набор "б" или "1б" или "1би"', 'Набор "б" или "1б" или "1би"', '{"3": "Набор \"б\" или \"1б\" или \"1би\"", "4": "стронций-90; иттрий-90", "5": "2", "6": "1; 4; 10; 40; 100; 160", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Состоит из нескольких комплектов источников (1СО; 2СО; 3СО; 4СО; 5СО; 6СО)", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Набор 10УГ', 'Набор 10УГ', '{"3": "Набор 10УГ", "4": "углерод-14", "5": "2", "6": "10", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Состоит из нескольких источников 10УГ.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Набор 1К-0', 'Набор 1К-0', '{"3": "Набор 1К-0", "4": "кобальт-60", "5": "2", "6": "1", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Состоит из нескольких источников.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Набор 1Т4', 'Набор 1Т4', '{"3": "Набор 1Т4", "4": "таллий-204", "5": "2", "6": "1; 4; 10; 40; 100", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Состоит из нескольких источников 2Т4.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Набор 1У4', 'Набор 1У4', '{"3": "Набор 1У4", "4": "уран-234", "5": "3", "6": "1", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Состоит из нескольких источников. Входит в набор 4 и 4а (1У4; 3У4)..", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Набор 1У8', 'Набор 1У8', '{"3": "Набор 1У8", "4": "уран естественный", "5": "3", "6": "1", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Состоит из нескольких источников.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Набор 1УГ', 'Набор 1УГ', '{"3": "Набор 1УГ", "4": "углерод-14", "5": "2", "6": "1", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Состоит из нескольких источников 1УГ.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Набор 2К-0', 'Набор 2К-0', '{"3": "Набор 2К-0", "4": "кобальт-60", "5": "2", "6": "4", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Состоит из нескольких источников.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Набор 3К-0', 'Набор 3К-0', '{"3": "Набор 3К-0", "4": "кобальт-60", "5": "2", "6": "10", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Состоит из нескольких источников.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Набор 3У4', 'Набор 3У4', '{"3": "Набор 3У4", "4": "уран-234", "5": "3", "6": "10", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Состоит из нескольких источников. Входит в набор 4 и 4а (1У4; 3У4).", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Набор 3У8', 'Набор 3У8', '{"3": "Набор 3У8", "4": "уран естественный", "5": "3", "6": "10", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Состоит из нескольких источников.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Набор 4УГ', 'Набор 4УГ', '{"3": "Набор 4УГ", "4": "углерод-14", "5": "2", "6": "4", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Состоит из нескольких источников 4УГ.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Набор 6У8', 'Набор 6У8', '{"3": "Набор 6У8", "4": "уран естественный", "5": "3", "6": "160", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Состоит из нескольких источников.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НИ-1', 'НИ-1', '{"3": "НИ-1", "4": "никель-63", "5": "2", "6": "0,4x20,0", "7": "200000000.0", "8": "8000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НИ-3', 'НИ-3', '{"3": "НИ-3", "4": "никель-63", "5": "2", "6": "1,6x30,0", "7": "", "8": "1100000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НИ-4', 'НИ-4', '{"3": "НИ-4", "4": "никель-63", "5": "2", "6": "0,8х13,0", "7": "", "8": "277000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НИ-5', 'НИ-5', '{"3": "НИ-5", "4": "никель-63", "5": "2", "6": "110,0x1,0", "7": "", "8": "1850000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК244М12', 'НК244М12', '{"3": "НК244М12", "4": "кюрий-244", "5": "4", "6": "", "7": "", "8": "29400000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК248М11.26', 'НК248М11.26', '{"3": "НК248М11.26", "4": "кюрий-248", "5": "4", "6": "", "7": "", "8": "10000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК248М11.44', 'НК248М11.44', '{"3": "НК248М11.44", "4": "кюрий-248", "5": "4", "6": "", "7": "", "8": "230000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252Д.26', 'НК252Д.26', '{"3": "НК252Д.26", "4": "калифорний-252", "5": "4", "6": "", "7": "210000.0", "8": "27000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "Открытый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М', 'НК252М', '{"3": "НК252М", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М01.07', 'НК252М01.07', '{"3": "НК252М01.07", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "240000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М01.106', 'НК252М01.106', '{"3": "НК252М01.106", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "112000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М01.108', 'НК252М01.108', '{"3": "НК252М01.108", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "17100000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М01.109', 'НК252М01.109', '{"3": "НК252М01.109", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "9800000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М01.207', 'НК252М01.207', '{"3": "НК252М01.207", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "240000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М01.506', 'НК252М01.506', '{"3": "НК252М01.506", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "5700000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М01.507', 'НК252М01.507', '{"3": "НК252М01.507", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "600000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М01.508', 'НК252М01.508', '{"3": "НК252М01.508", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "6000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М11.13', 'НК252М11.13', '{"3": "НК252М11.13", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "2590000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М11.16', 'НК252М11.16', '{"3": "НК252М11.16", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "12000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М11.17', 'НК252М11.17', '{"3": "НК252М11.17", "4": "калифорний-252", "5": "4", "6": "7x15", "7": "", "8": "120000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М11.18', 'НК252М11.18', '{"3": "НК252М11.18", "4": "калифорний-252", "5": "4", "6": "7x15", "7": "", "8": "1200000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М11.19', 'НК252М11.19', '{"3": "НК252М11.19", "4": "калифорний-252", "5": "4", "6": "7x15", "7": "", "8": "12000000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М11.26', 'НК252М11.26', '{"3": "НК252М11.26", "4": "калифорний-252", "5": "4", "6": "7x15", "7": "", "8": "24000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М11.27', 'НК252М11.27', '{"3": "НК252М11.27", "4": "калифорний-252", "5": "4", "6": "7x15", "7": "", "8": "240000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М11.28', 'НК252М11.28', '{"3": "НК252М11.28", "4": "калифорний-252", "5": "4", "6": "7x15", "7": "", "8": "2400000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М11.56', 'НК252М11.56', '{"3": "НК252М11.56", "4": "калифорний-252", "5": "4", "6": "7x15", "7": "", "8": "60000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М11.57', 'НК252М11.57', '{"3": "НК252М11.57", "4": "калифорний-252", "5": "4", "6": "7x15", "7": "", "8": "600000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М11.58', 'НК252М11.58', '{"3": "НК252М11.58", "4": "калифорний-252", "5": "4", "6": "7x15", "7": "", "8": "6000000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М11.76', 'НК252М11.76', '{"3": "НК252М11.76", "4": "калифорний-252", "5": "4", "6": "7x15", "7": "", "8": "90000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М11.77', 'НК252М11.77', '{"3": "НК252М11.77", "4": "калифорний-252", "5": "4", "6": "7x15", "7": "", "8": "900000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М11.78', 'НК252М11.78', '{"3": "НК252М11.78", "4": "калифорний-252", "5": "4", "6": "7x15", "7": "", "8": "9000000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М12.110', 'НК252М12.110', '{"3": "НК252М12.110", "4": "калифорний-252", "5": "4", "6": "7x25", "7": "", "8": "120000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М12.210', 'НК252М12.210', '{"3": "НК252М12.210", "4": "калифорний-252", "5": "4", "6": "7x25", "7": "", "8": "240000000000.0", "9": "0", "10": "НИИАР", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М12.29', 'НК252М12.29', '{"3": "НК252М12.29", "4": "калифорний-252", "5": "4", "6": "7x25", "7": "", "8": "24000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М12.59', 'НК252М12.59', '{"3": "НК252М12.59", "4": "калифорний-252", "5": "4", "6": "7x25", "7": "", "8": "60000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М12.79', 'НК252М12.79', '{"3": "НК252М12.79", "4": "калифорний-252", "5": "4", "6": "7x25", "7": "", "8": "90000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М13.100', 'НК252М13.100', '{"3": "НК252М13.100", "4": "калифорний-252", "5": "4", "6": "3,7; 4,27; 2,3; 1", "7": "", "8": "2800000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М13.150', 'НК252М13.150', '{"3": "НК252М13.150", "4": "калифорний-252", "5": "4", "6": "3,7; 4,27; 2,3; 1", "7": "", "8": "4100000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М13.200', 'НК252М13.200', '{"3": "НК252М13.200", "4": "калифорний-252", "5": "4", "6": "3,7; 4,27; 2,3; 1", "7": "", "8": "5500000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М13.300', 'НК252М13.300', '{"3": "НК252М13.300", "4": "калифорний-252", "5": "4", "6": "3,7; 4,27; 2,3; 1", "7": "", "8": "8300000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М13.500', 'НК252М13.500', '{"3": "НК252М13.500", "4": "калифорний-252", "5": "4", "6": "3,7; 4,27; 2,3; 1", "7": "", "8": "13800000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М14.100', 'НК252М14.100', '{"3": "НК252М14.100", "4": "калифорний-252", "5": "4", "6": "5,5; 6; 4; 2", "7": "", "8": "2800000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М14.150', 'НК252М14.150', '{"3": "НК252М14.150", "4": "калифорний-252", "5": "4", "6": "5,5; 6; 4; 2", "7": "", "8": "4100000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М14.200', 'НК252М14.200', '{"3": "НК252М14.200", "4": "калифорний-252", "5": "4", "6": "5,5; 6; 4; 2", "7": "", "8": "5500000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М14.300', 'НК252М14.300', '{"3": "НК252М14.300", "4": "калифорний-252", "5": "4", "6": "5,5; 6; 4; 2", "7": "", "8": "8300000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М14.500', 'НК252М14.500', '{"3": "НК252М14.500", "4": "калифорний-252", "5": "4", "6": "5,5; 6; 4; 2", "7": "", "8": "13800000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М211.16', 'НК252М211.16', '{"3": "НК252М211.16", "4": "калифорний-252", "5": "4", "6": "15; 10", "7": "", "8": "12000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М211.25', 'НК252М211.25', '{"3": "НК252М211.25", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "16900000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М211.26', 'НК252М211.26', '{"3": "НК252М211.26", "4": "калифорний-252", "5": "4", "6": "15; 10", "7": "", "8": "24000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М211.56', 'НК252М211.56', '{"3": "НК252М211.56", "4": "калифорний-252", "5": "4", "6": "15; 10", "7": "", "8": "60000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М211.76', 'НК252М211.76', '{"3": "НК252М211.76", "4": "калифорний-252", "5": "4", "6": "15; 10", "7": "", "8": "90000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М212.16', 'НК252М212.16', '{"3": "НК252М212.16", "4": "калифорний-252", "5": "4", "6": "15; 10", "7": "", "8": "12000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М212.26', 'НК252М212.26', '{"3": "НК252М212.26", "4": "калифорний-252", "5": "4", "6": "15; 10", "7": "", "8": "24000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М212.56', 'НК252М212.56', '{"3": "НК252М212.56", "4": "калифорний-252", "5": "4", "6": "15; 10", "7": "", "8": "60000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М212.76', 'НК252М212.76', '{"3": "НК252М212.76", "4": "калифорний-252", "5": "4", "6": "15; 10", "7": "", "8": "90000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М213.16', 'НК252М213.16', '{"3": "НК252М213.16", "4": "калифорний-252", "5": "4", "6": "15; 10", "7": "", "8": "12000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М213.26', 'НК252М213.26', '{"3": "НК252М213.26", "4": "калифорний-252", "5": "4", "6": "15; 10", "7": "", "8": "24000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М213.56', 'НК252М213.56', '{"3": "НК252М213.56", "4": "калифорний-252", "5": "4", "6": "15; 10", "7": "", "8": "60000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М213.76', 'НК252М213.76', '{"3": "НК252М213.76", "4": "калифорний-252", "5": "4", "6": "15; 10", "7": "", "8": "90000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М221.16', 'НК252М221.16', '{"3": "НК252М221.16", "4": "калифорний-252", "5": "4", "6": "20; 15", "7": "", "8": "12000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М221.26', 'НК252М221.26', '{"3": "НК252М221.26", "4": "калифорний-252", "5": "4", "6": "20; 15", "7": "", "8": "24000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М221.56', 'НК252М221.56', '{"3": "НК252М221.56", "4": "калифорний-252", "5": "4", "6": "20; 15", "7": "", "8": "60000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М221.76', 'НК252М221.76', '{"3": "НК252М221.76", "4": "калифорний-252", "5": "4", "6": "20; 15", "7": "", "8": "90000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М222.16', 'НК252М222.16', '{"3": "НК252М222.16", "4": "калифорний-252", "5": "4", "6": "20; 15", "7": "", "8": "12000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М222.26', 'НК252М222.26', '{"3": "НК252М222.26", "4": "калифорний-252", "5": "4", "6": "20; 15", "7": "", "8": "24000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М222.56', 'НК252М222.56', '{"3": "НК252М222.56", "4": "калифорний-252", "5": "4", "6": "20; 15", "7": "", "8": "60000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М222.76', 'НК252М222.76', '{"3": "НК252М222.76", "4": "калифорний-252", "5": "4", "6": "20; 15", "7": "", "8": "90000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М223.16', 'НК252М223.16', '{"3": "НК252М223.16", "4": "калифорний-252", "5": "4", "6": "20; 15", "7": "", "8": "12000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М223.26', 'НК252М223.26', '{"3": "НК252М223.26", "4": "калифорний-252", "5": "4", "6": "20; 15", "7": "", "8": "24000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М223.56', 'НК252М223.56', '{"3": "НК252М223.56", "4": "калифорний-252", "5": "4", "6": "20; 15", "7": "", "8": "60000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М223.76', 'НК252М223.76', '{"3": "НК252М223.76", "4": "калифорний-252", "5": "4", "6": "20; 15", "7": "", "8": "90000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М231.17', 'НК252М231.17', '{"3": "НК252М231.17", "4": "калифорний-252", "5": "4", "6": "25; 20", "7": "", "8": "120000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М231.27', 'НК252М231.27', '{"3": "НК252М231.27", "4": "калифорний-252", "5": "4", "6": "25; 20", "7": "", "8": "240000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М231.56', 'НК252М231.56', '{"3": "НК252М231.56", "4": "калифорний-252", "5": "4", "6": "25; 20", "7": "", "8": "60000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М231.57', 'НК252М231.57', '{"3": "НК252М231.57", "4": "калифорний-252", "5": "4", "6": "25; 20", "7": "", "8": "600000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М231.76', 'НК252М231.76', '{"3": "НК252М231.76", "4": "калифорний-252", "5": "4", "6": "25; 20", "7": "", "8": "90000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М231.77', 'НК252М231.77', '{"3": "НК252М231.77", "4": "калифорний-252", "5": "4", "6": "25; 20", "7": "", "8": "900000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М232.17', 'НК252М232.17', '{"3": "НК252М232.17", "4": "калифорний-252", "5": "4", "6": "25; 20", "7": "", "8": "120000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М232.27', 'НК252М232.27', '{"3": "НК252М232.27", "4": "калифорний-252", "5": "4", "6": "25; 20", "7": "", "8": "240000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М232.56', 'НК252М232.56', '{"3": "НК252М232.56", "4": "калифорний-252", "5": "4", "6": "25; 20", "7": "", "8": "60000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М232.57', 'НК252М232.57', '{"3": "НК252М232.57", "4": "калифорний-252", "5": "4", "6": "25; 20", "7": "", "8": "600000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М232.76', 'НК252М232.76', '{"3": "НК252М232.76", "4": "калифорний-252", "5": "4", "6": "25; 20", "7": "", "8": "90000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М232.77', 'НК252М232.77', '{"3": "НК252М232.77", "4": "калифорний-252", "5": "4", "6": "25; 20", "7": "", "8": "900000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М233.17', 'НК252М233.17', '{"3": "НК252М233.17", "4": "калифорний-252", "5": "4", "6": "25; 20", "7": "", "8": "120000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М6', 'НК252М6', '{"3": "НК252М6", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "8990000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М233.27', 'НК252М233.27', '{"3": "НК252М233.27", "4": "калифорний-252", "5": "4", "6": "25; 20", "7": "", "8": "240000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М233.56', 'НК252М233.56', '{"3": "НК252М233.56", "4": "калифорний-252", "5": "4", "6": "25; 20", "7": "", "8": "60000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М233.57', 'НК252М233.57', '{"3": "НК252М233.57", "4": "калифорний-252", "5": "4", "6": "25; 20", "7": "", "8": "600000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М233.76', 'НК252М233.76', '{"3": "НК252М233.76", "4": "калифорний-252", "5": "4", "6": "25; 20", "7": "", "8": "90000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М233.77', 'НК252М233.77', '{"3": "НК252М233.77", "4": "калифорний-252", "5": "4", "6": "25; 20", "7": "", "8": "900000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М241.17', 'НК252М241.17', '{"3": "НК252М241.17", "4": "калифорний-252", "5": "4", "6": "35; 30", "7": "", "8": "120000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М241.27', 'НК252М241.27', '{"3": "НК252М241.27", "4": "калифорний-252", "5": "4", "6": "35; 30", "7": "", "8": "240000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М241.56', 'НК252М241.56', '{"3": "НК252М241.56", "4": "калифорний-252", "5": "4", "6": "35; 30", "7": "", "8": "60000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М241.57', 'НК252М241.57', '{"3": "НК252М241.57", "4": "калифорний-252", "5": "4", "6": "35; 30", "7": "", "8": "600000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М241.76', 'НК252М241.76', '{"3": "НК252М241.76", "4": "калифорний-252", "5": "4", "6": "35; 30", "7": "", "8": "90000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М241.77', 'НК252М241.77', '{"3": "НК252М241.77", "4": "калифорний-252", "5": "4", "6": "35; 30", "7": "", "8": "900000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М242.17', 'НК252М242.17', '{"3": "НК252М242.17", "4": "калифорний-252", "5": "4", "6": "35; 30", "7": "", "8": "120000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М242.27', 'НК252М242.27', '{"3": "НК252М242.27", "4": "калифорний-252", "5": "4", "6": "35; 30", "7": "", "8": "240000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М242.56', 'НК252М242.56', '{"3": "НК252М242.56", "4": "калифорний-252", "5": "4", "6": "35; 30", "7": "", "8": "60000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М242.57', 'НК252М242.57', '{"3": "НК252М242.57", "4": "калифорний-252", "5": "4", "6": "35; 30", "7": "", "8": "600000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М242.76', 'НК252М242.76', '{"3": "НК252М242.76", "4": "калифорний-252", "5": "4", "6": "35; 30", "7": "", "8": "90000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М242.77', 'НК252М242.77', '{"3": "НК252М242.77", "4": "калифорний-252", "5": "4", "6": "35; 30", "7": "", "8": "900000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М243.17', 'НК252М243.17', '{"3": "НК252М243.17", "4": "калифорний-252", "5": "4", "6": "35; 30", "7": "", "8": "120000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М243.27', 'НК252М243.27', '{"3": "НК252М243.27", "4": "калифорний-252", "5": "4", "6": "35; 30", "7": "", "8": "240000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М243.56', 'НК252М243.56', '{"3": "НК252М243.56", "4": "калифорний-252", "5": "4", "6": "35; 30", "7": "", "8": "60000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М243.57', 'НК252М243.57', '{"3": "НК252М243.57", "4": "калифорний-252", "5": "4", "6": "35; 30", "7": "", "8": "600000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М243.76', 'НК252М243.76', '{"3": "НК252М243.76", "4": "калифорний-252", "5": "4", "6": "35; 30", "7": "", "8": "90000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М243.77', 'НК252М243.77', '{"3": "НК252М243.77", "4": "калифорний-252", "5": "4", "6": "35; 30", "7": "", "8": "900000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М31.16', 'НК252М31.16', '{"3": "НК252М31.16", "4": "калифорний-252", "5": "4", "6": "0,8x6,4", "7": "", "8": "12000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "Гибкие сборки закрытых калифорниевых источников нейтронного излучения предназначены для использования в медицине. Сборка представляет собой стальную пружину или эластичную полимерную трубку, в которую помещены вплотную друг к другу или через инертные вставки миниатюрные источники на основе радионуклида калифорния-252 типа НК252М3. Сборка содержит от 4 до 11 источников и может иметь жесткий держатель.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М57', 'НК252М57', '{"3": "НК252М57", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "630000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М31.26', 'НК252М31.26', '{"3": "НК252М31.26", "4": "калифорний-252", "5": "4", "6": "0,8x6,4", "7": "", "8": "24000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "Гибкие сборки закрытых калифорниевых источников нейтронного излучения предназначены для использования в медицине. Сборка представляет собой стальную пружину или эластичную полимерную трубку, в которую помещены вплотную друг к другу или через инертные вставки миниатюрные источники на основе радионуклида калифорния-252 типа НК252М3. Сборка содержит от 4 до 11 источников и может иметь жесткий держатель.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М31.56', 'НК252М31.56', '{"3": "НК252М31.56", "4": "калифорний-252", "5": "4", "6": "0,8x6,4", "7": "", "8": "60000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "Гибкие сборки закрытых калифорниевых источников нейтронного излучения предназначены для использования в медицине. Сборка представляет собой стальную пружину или эластичную полимерную трубку, в которую помещены вплотную друг к другу или через инертные вставки миниатюрные источники на основе радионуклида калифорния-252 типа НК252М3. Сборка содержит от 4 до 11 источников и может иметь жесткий держатель.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М31.76', 'НК252М31.76', '{"3": "НК252М31.76", "4": "калифорний-252", "5": "4", "6": "0,8x6,4", "7": "", "8": "90000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "Гибкие сборки закрытых калифорниевых источников нейтронного излучения предназначены для использования в медицине. Сборка представляет собой стальную пружину или эластичную полимерную трубку, в которую помещены вплотную друг к другу или через инертные вставки миниатюрные источники на основе радионуклида калифорния-252 типа НК252М3. Сборка содержит от 4 до 11 источников и может иметь жесткий держатель.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М32.16', 'НК252М32.16', '{"3": "НК252М32.16", "4": "калифорний-252", "5": "4", "6": "0,6x6,4", "7": "", "8": "12000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М32.26', 'НК252М32.26', '{"3": "НК252М32.26", "4": "калифорний-252", "5": "4", "6": "0,6x6,4", "7": "", "8": "24000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М32.56', 'НК252М32.56', '{"3": "НК252М32.56", "4": "калифорний-252", "5": "4", "6": "0,6x6,4", "7": "", "8": "60000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М32.76', 'НК252М32.76', '{"3": "НК252М32.76", "4": "калифорний-252", "5": "4", "6": "0,6x6,4", "7": "", "8": "90000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М41.17', 'НК252М41.17', '{"3": "НК252М41.17", "4": "калифорний-252", "5": "4", "6": "3x15", "7": "", "8": "120000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М41.18', 'НК252М41.18', '{"3": "НК252М41.18", "4": "калифорний-252", "5": "4", "6": "3x15", "7": "", "8": "1200000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М41.19', 'НК252М41.19', '{"3": "НК252М41.19", "4": "калифорний-252", "5": "4", "6": "3x15", "7": "", "8": "12000000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М41.27', 'НК252М41.27', '{"3": "НК252М41.27", "4": "калифорний-252", "5": "4", "6": "3x15", "7": "", "8": "240000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М41.28', 'НК252М41.28', '{"3": "НК252М41.28", "4": "калифорний-252", "5": "4", "6": "3x15", "7": "", "8": "2400000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М41.29', 'НК252М41.29', '{"3": "НК252М41.29", "4": "калифорний-252", "5": "4", "6": "3x15", "7": "", "8": "24000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М41.57', 'НК252М41.57', '{"3": "НК252М41.57", "4": "калифорний-252", "5": "4", "6": "3x15", "7": "", "8": "600000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М41.58', 'НК252М41.58', '{"3": "НК252М41.58", "4": "калифорний-252", "5": "4", "6": "3x15", "7": "", "8": "6000000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М41.59', 'НК252М41.59', '{"3": "НК252М41.59", "4": "калифорний-252", "5": "4", "6": "3x15", "7": "", "8": "60000000000.0", "9": "0", "10": "НИИАР", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М41.77', 'НК252М41.77', '{"3": "НК252М41.77", "4": "калифорний-252", "5": "4", "6": "3x15", "7": "", "8": "900000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М41.78', 'НК252М41.78', '{"3": "НК252М41.78", "4": "калифорний-252", "5": "4", "6": "3x15", "7": "", "8": "9000000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М44', 'НК252М44', '{"3": "НК252М44", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "2420000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М5', 'НК252М5', '{"3": "НК252М5", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "1730000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252МГ', 'НК252МГ', '{"3": "НК252МГ", "4": "калифорний-252", "5": "4", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Открытый.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НП210.ГО1.106', 'НП210.ГО1.106', '{"3": "НП210.ГО1.106", "4": "полоний-210", "5": "4", "6": "20,0x40,0", "7": "", "8": "21000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НП210.ГО1.107', 'НП210.ГО1.107', '{"3": "НП210.ГО1.107", "4": "полоний-210", "5": "4", "6": "", "7": "", "8": "21200000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НП210.ГО1.108', 'НП210.ГО1.108', '{"3": "НП210.ГО1.108", "4": "полоний-210", "5": "4", "6": "20,0x40,0", "7": "", "8": "2100000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НП210.ГО2.106', 'НП210.ГО2.106', '{"3": "НП210.ГО2.106", "4": "полоний-210", "5": "4", "6": "8,0x40,0", "7": "", "8": "21000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НП210.ГО2.207', 'НП210.ГО2.207', '{"3": "НП210.ГО2.207", "4": "полоний-210", "5": "4", "6": "8,0x40,0", "7": "", "8": "420000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НП210.ГО3.106', 'НП210.ГО3.106', '{"3": "НП210.ГО3.106", "4": "полоний-210", "5": "4", "6": "8,0x60,0", "7": "", "8": "21000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НП210.ГО3.507', 'НП210.ГО3.507', '{"3": "НП210.ГО3.507", "4": "полоний-210", "5": "4", "6": "8,0x60,0", "7": "", "8": "1100000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НП210.ГО4.106', 'НП210.ГО4.106', '{"3": "НП210.ГО4.106", "4": "полоний-210", "5": "4", "6": "8,0x100,0", "7": "", "8": "21000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НП210.ГО4.707', 'НП210.ГО4.707', '{"3": "НП210.ГО4.707", "4": "полоний-210", "5": "4", "6": "8,0x100,0", "7": "", "8": "1500000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НП210.ГО5.106', 'НП210.ГО5.106', '{"3": "НП210.ГО5.106", "4": "полоний-210; бор", "5": "4", "6": "20,0x40,0", "7": "", "8": "200000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НП210.ГО5.107', 'НП210.ГО5.107', '{"3": "НП210.ГО5.107", "4": "полоний-210; бор", "5": "4", "6": "20,0x40,0", "7": "", "8": "2000000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НТ-16', 'НТ-16', '{"3": "НТ-16", "4": "тритий", "5": "4", "6": "", "7": "", "8": "185000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОВСГИ', 'ОВСГИ', '{"3": "ОВСГИ", "4": "цезий-137", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИГИСН', 'ОИГИСН', '{"3": "ОИГИСН", "4": "уран-235", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИДК', 'ОИДК', '{"3": "ОИДК", "4": "барий-133", "5": "1", "6": "27x61. Акт. часть dxh : 25x20", "7": "37000.0", "8": "37000000.0", "9": "0", "10": "", "11": "", "12": "Цветовой код: коричневый", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИДК', 'ОИДК', '{"3": "ОИДК", "4": "кобальт-57", "5": "1", "6": "27x61. Акт. часть dxh : 25x20", "7": "37000.0", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Цветовой код: синий", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИДК', 'ОИДК', '{"3": "ОИДК", "4": "кобальт-60", "5": "1", "6": "27x61. Акт. часть dxh : 25x20", "7": "37000.0", "8": "3700000.0", "9": "0", "10": "", "11": "5", "12": "Цветовой код: светло-зеленый", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИДК', 'ОИДК', '{"3": "ОИДК", "4": "цезий-137", "5": "1", "6": "27x61. Акт. часть dxh : 25x20", "7": "37000.0", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "Цветовой код: желтый", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИСН', 'ОИСН', '{"3": "ОИСН", "4": "барий-133", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИСУ', 'ОИСУ', '{"3": "ОИСУ", "4": "", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН (Co)', 'ОМАСН (Co)', '{"3": "ОМАСН (Co)", "4": "кобальт-60", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН(Sr)', 'ОМАСН(Sr)', '{"3": "ОМАСН(Sr)", "4": "стронций-90; иттрий-90", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН(Th)', 'ОМАСН(Th)', '{"3": "ОМАСН(Th)", "4": "торий-228", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Опытный источник', 'Опытный источник', '{"3": "Опытный источник", "4": "олово-121м", "5": "2", "6": "", "7": "0.0", "8": "37000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОРБИ', 'ОРБИ', '{"3": "ОРБИ", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОРИБИ', 'ОРИБИ', '{"3": "ОРИБИ", "4": "кобальт-60", "5": "2", "6": "Подл. А: 35x1. Подл. B: 25x(1 или 3)", "7": "10.0", "8": "50000.0", "9": "0", "10": "", "11": "0", "12": "Материал защитного покрытия: Хром. Толщина покрытия, мкм: 2,0", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ (Т)', 'ОСАИ (Т)', '{"3": "ОСАИ (Т)", "4": "плутоний-238", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОРИБИ', 'ОРИБИ', '{"3": "ОРИБИ", "4": "никель-63", "5": "2", "6": "Подл. А: 35x1. Подл. B: 25x(1 или 3)", "7": "10.0", "8": "50000.0", "9": "0", "10": "", "11": "0", "12": "Материал защитного покрытия: Хром. Толщина покрытия, мкм: 0,5", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОРИБИ', 'ОРИБИ', '{"3": "ОРИБИ", "4": "прометий-147", "5": "2", "6": "Подл. А: 35x1. Подл. B: 25x(1 или 3)", "7": "10.0", "8": "50000.0", "9": "0", "10": "", "11": "0", "12": "Материал защитного покрытия: Хром. Толщина покрытия, мкм: 2,0", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОРИБИ', 'ОРИБИ', '{"3": "ОРИБИ", "4": "рутений-106; родий-106", "5": "2", "6": "Подл. А: 35x1. Подл. B: 25x(1 или 3)", "7": "10.0", "8": "50000.0", "9": "0", "10": "", "11": "0", "12": "Материал защитного покрытия: Хром. Толщина покрытия, мкм: 2,0", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОРИБИ', 'ОРИБИ', '{"3": "ОРИБИ", "4": "стронций-90; иттрий-90", "5": "2", "6": "Подл. А: 35x1. Подл. B: 25x(1 или 3)", "7": "10.0", "8": "50000.0", "9": "0", "10": "", "11": "5", "12": "Материал защитного покрытия: Хром. Толщина покрытия, мкм: 2,0", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОРИБИ', 'ОРИБИ', '{"3": "ОРИБИ", "4": "таллий-204", "5": "2", "6": "Подл. А: 35x1. Подл. B: 25x(1 или 3)", "7": "10.0", "8": "50000.0", "9": "0", "10": "", "11": "5", "12": "Материал защитного покрытия: Цинк; Никель. Толщина покрытия, мкм: 8,0", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОРИБИ', 'ОРИБИ', '{"3": "ОРИБИ", "4": "углерод-14", "5": "2", "6": "Подл. А: 35x1. Подл. B: 25x(1 или 3)", "7": "10.0", "8": "50000.0", "9": "0", "10": "", "11": "5", "12": "Материал защитного покрытия: Олово. Толщина покрытия, мкм: 0,2", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОРИБИ', 'ОРИБИ', '{"3": "ОРИБИ", "4": "цезий-137", "5": "2", "6": "Подл. А: 35x1. Подл. B: 25x(1 или 3)", "7": "10.0", "8": "50000.0", "9": "0", "10": "", "11": "5", "12": "Материал защитного покрытия: Хром. Толщина покрытия, мкм: 2,0", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ', 'ОСАИ', '{"3": "ОСАИ", "4": "америций-241", "5": "3", "6": "", "7": "1.0", "8": "100000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ', 'ОСАИ', '{"3": "ОСАИ", "4": "кюрий-244", "5": "3", "6": "А: 24x(1 или 2). B: 25x0,5x7 или 25x3", "7": "", "8": "30000.0", "9": "0", "10": "", "11": "5", "12": "Набор образцовых спектрометрических источников альфа-излучения", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ', 'ОСАИ', '{"3": "ОСАИ", "4": "нептуний-237", "5": "3", "6": "", "7": "1.0", "8": "100000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ', 'ОСАИ', '{"3": "ОСАИ", "4": "плутоний-238", "5": "3", "6": "А: 24x(1 или 2). B: 25x0,5x7 или 25x3", "7": "", "8": "30000.0", "9": "0", "10": "", "11": "5", "12": "Набор образцовых спектрометрических источников альфа-излучения", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ', 'ОСАИ', '{"3": "ОСАИ", "4": "плутоний-238; плутоний-239; кюрий-244", "5": "3", "6": "А: 24x(1 или 2). B: 25x0,5x7 или 25x3", "7": "", "8": "30000.0", "9": "0", "10": "", "11": "5", "12": "Набор образцовых спектрометрических источников альфа-излучения", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ', 'ОСАИ', '{"3": "ОСАИ", "4": "плутоний-239", "5": "3", "6": "А: 24x(1 или 2). B: 25x0,5x7 или 25x3", "7": "", "8": "30000.0", "9": "0", "10": "", "11": "5", "12": "Набор образцовых спектрометрических источников альфа-излучения", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ', 'ОСАИ', '{"3": "ОСАИ", "4": "радий-226", "5": "3", "6": "А: 24x(1 или 2). B: 25x0,5x7 или 25x3", "7": "", "8": "30000.0", "9": "0", "10": "", "11": "5", "12": "Набор образцовых спектрометрических источников альфа-излучения", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ', 'ОСАИ', '{"3": "ОСАИ", "4": "торий-232", "5": "3", "6": "", "7": "1.0", "8": "100000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ', 'ОСАИ', '{"3": "ОСАИ", "4": "уран-233", "5": "3", "6": "", "7": "1.0", "8": "100000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ', 'ОСАИ', '{"3": "ОСАИ", "4": "уран-233; плутоний-238; плутоний-239", "5": "3", "6": "А: 24x(1 или 2). B: 25x0,5x7 или 25x3", "7": "", "8": "30000.0", "9": "0", "10": "", "11": "5", "12": "Набор образцовых спектрометрических источников альфа-излучения", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ', 'ОСАИ', '{"3": "ОСАИ", "4": "уран-234", "5": "3", "6": "", "7": "1.0", "8": "100000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ', 'ОСАИ', '{"3": "ОСАИ", "4": "уран-235", "5": "3", "6": "", "7": "1.0", "8": "100000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ', 'ОСАИ', '{"3": "ОСАИ", "4": "уран-238", "5": "3", "6": "", "7": "1.0", "8": "100000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ (Т)', 'ОСАИ (Т)', '{"3": "ОСАИ (Т)", "4": "америций-241", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ (Т)', 'ОСАИ (Т)', '{"3": "ОСАИ (Т)", "4": "кюрий-244", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ (Т)', 'ОСАИ (Т)', '{"3": "ОСАИ (Т)", "4": "нептуний-237", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ (Т)', 'ОСАИ (Т)', '{"3": "ОСАИ (Т)", "4": "плутоний-238; плутоний-239; кюрий-244", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ (Т)', 'ОСАИ (Т)', '{"3": "ОСАИ (Т)", "4": "плутоний-239", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ (Т)', 'ОСАИ (Т)', '{"3": "ОСАИ (Т)", "4": "радий-226", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ (Т)', 'ОСАИ (Т)', '{"3": "ОСАИ (Т)", "4": "торий-232", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ (Т)', 'ОСАИ (Т)', '{"3": "ОСАИ (Т)", "4": "уран-233", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ (Т)', 'ОСАИ (Т)', '{"3": "ОСАИ (Т)", "4": "уран-233; плутоний-238; плутоний-239", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ (Т)', 'ОСАИ (Т)', '{"3": "ОСАИ (Т)", "4": "уран-234", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ (Т)', 'ОСАИ (Т)', '{"3": "ОСАИ (Т)", "4": "уран-235", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ (Т)', 'ОСАИ (Т)', '{"3": "ОСАИ (Т)", "4": "уран-238", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-2', 'ОСАИ-2', '{"3": "ОСАИ-2", "4": "америций-241", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-2', 'ОСАИ-2', '{"3": "ОСАИ-2", "4": "кюрий-244", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-2', 'ОСАИ-2', '{"3": "ОСАИ-2", "4": "нептуний-237", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-2', 'ОСАИ-2', '{"3": "ОСАИ-2", "4": "плутоний-238", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-2', 'ОСАИ-2', '{"3": "ОСАИ-2", "4": "плутоний-238; плутоний-239; кюрий-244", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-2', 'ОСАИ-2', '{"3": "ОСАИ-2", "4": "плутоний-239", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-2', 'ОСАИ-2', '{"3": "ОСАИ-2", "4": "радий-226", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-2', 'ОСАИ-2', '{"3": "ОСАИ-2", "4": "торий-232", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-2', 'ОСАИ-2', '{"3": "ОСАИ-2", "4": "уран-233", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-2', 'ОСАИ-2', '{"3": "ОСАИ-2", "4": "уран-233; плутоний-238; плутоний-239", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-2', 'ОСАИ-2', '{"3": "ОСАИ-2", "4": "уран-234", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-2', 'ОСАИ-2', '{"3": "ОСАИ-2", "4": "уран-235", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-2', 'ОСАИ-2', '{"3": "ОСАИ-2", "4": "уран-238", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-3-2', 'ОСАИ-3-2', '{"3": "ОСАИ-3-2", "4": "америций-241", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-3-2', 'ОСАИ-3-2', '{"3": "ОСАИ-3-2", "4": "кюрий-244", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-3-2', 'ОСАИ-3-2', '{"3": "ОСАИ-3-2", "4": "нептуний-237", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-3-2', 'ОСАИ-3-2', '{"3": "ОСАИ-3-2", "4": "плутоний-238", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-3-2', 'ОСАИ-3-2', '{"3": "ОСАИ-3-2", "4": "плутоний-238; плутоний-239; кюрий-244", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-3-2', 'ОСАИ-3-2', '{"3": "ОСАИ-3-2", "4": "плутоний-239", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-3-2', 'ОСАИ-3-2', '{"3": "ОСАИ-3-2", "4": "радий-226", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-3-2', 'ОСАИ-3-2', '{"3": "ОСАИ-3-2", "4": "торий-232", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-3-2', 'ОСАИ-3-2', '{"3": "ОСАИ-3-2", "4": "уран-233", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-3-2', 'ОСАИ-3-2', '{"3": "ОСАИ-3-2", "4": "уран-233; плутоний-238; плутоний-239", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-3-2', 'ОСАИ-3-2', '{"3": "ОСАИ-3-2", "4": "уран-234", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-3-2', 'ОСАИ-3-2', '{"3": "ОСАИ-3-2", "4": "уран-235", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-3-2', 'ОСАИ-3-2', '{"3": "ОСАИ-3-2", "4": "уран-238", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-II', 'ОСАИ-II', '{"3": "ОСАИ-II", "4": "америций-241", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-II', 'ОСАИ-II', '{"3": "ОСАИ-II", "4": "кюрий-244", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-II', 'ОСАИ-II', '{"3": "ОСАИ-II", "4": "нептуний-237", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-II', 'ОСАИ-II', '{"3": "ОСАИ-II", "4": "плутоний-238", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-II', 'ОСАИ-II', '{"3": "ОСАИ-II", "4": "плутоний-238; плутоний-239; кюрий-244", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-II', 'ОСАИ-II', '{"3": "ОСАИ-II", "4": "плутоний-239", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-II', 'ОСАИ-II', '{"3": "ОСАИ-II", "4": "радий-226", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-II', 'ОСАИ-II', '{"3": "ОСАИ-II", "4": "торий-232", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-II', 'ОСАИ-II', '{"3": "ОСАИ-II", "4": "уран-233", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-II', 'ОСАИ-II', '{"3": "ОСАИ-II", "4": "уран-233; плутоний-238; плутоний-239", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-II', 'ОСАИ-II', '{"3": "ОСАИ-II", "4": "уран-234", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-II', 'ОСАИ-II', '{"3": "ОСАИ-II", "4": "уран-235", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-II', 'ОСАИ-II', '{"3": "ОСАИ-II", "4": "уран-238", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-М', 'ОСАИ-М', '{"3": "ОСАИ-М", "4": "америций-241", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-М', 'ОСАИ-М', '{"3": "ОСАИ-М", "4": "кюрий-244", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-М', 'ОСАИ-М', '{"3": "ОСАИ-М", "4": "нептуний-237", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-М', 'ОСАИ-М', '{"3": "ОСАИ-М", "4": "плутоний-238", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-М', 'ОСАИ-М', '{"3": "ОСАИ-М", "4": "плутоний-238; плутоний-239; кюрий-244", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-М', 'ОСАИ-М', '{"3": "ОСАИ-М", "4": "плутоний-239", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-М', 'ОСАИ-М', '{"3": "ОСАИ-М", "4": "радий-226", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-М', 'ОСАИ-М', '{"3": "ОСАИ-М", "4": "торий-232", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-М', 'ОСАИ-М', '{"3": "ОСАИ-М", "4": "уран-233", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-М', 'ОСАИ-М', '{"3": "ОСАИ-М", "4": "уран-233; плутоний-238; плутоний-239", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-М', 'ОСАИ-М', '{"3": "ОСАИ-М", "4": "уран-234", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-М', 'ОСАИ-М', '{"3": "ОСАИ-М", "4": "уран-235", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ-М', 'ОСАИ-М', '{"3": "ОСАИ-М", "4": "уран-238", "5": "3", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ', 'ОСГИ', '{"3": "ОСГИ", "4": "цезий-137", "5": "1", "6": "Держатель А: 25x3x3. Держатель B: 29x3x3", "7": "1000.0", "8": "1000000.0", "9": "0", "10": "", "11": "5", "12": "Межпроверочный интервал: 2 года", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ', 'ОСГИ', '{"3": "ОСГИ", "4": "америций-241", "5": "1", "6": "Держатель А: 25x3x3. Держатель B: 29x3x3", "7": "1000.0", "8": "100000.0", "9": "0", "10": "", "11": "5", "12": "С активностью менее 10e; 3: НСС - 5 лет, межпроверочный интервал - 2 года.С активностью более 10e; 3: НСС - 3 года, межпроверочный интервал - 1 год.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ', 'ОСГИ', '{"3": "ОСГИ", "4": "америций-243", "5": "1", "6": "Держатель А: 25x3x3. Держатель B: 29x3x3", "7": "1000.0", "8": "50000.0", "9": "0", "10": "", "11": "5", "12": "С активностью менее 10e; 3: НСС - 5 лет, межпроверочный интервал - 2 года.С активностью более 10e; 3: НСС - 3 года, межпроверочный интервал - 1 год.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ', 'ОСГИ', '{"3": "ОСГИ", "4": "барий-133", "5": "1", "6": "Держатель А: 25x3x3. Держатель B: 29x3x3", "7": "1000.0", "8": "1000000.0", "9": "0", "10": "", "11": "", "12": "Межпроверочный интервал: 2 года", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ', 'ОСГИ', '{"3": "ОСГИ", "4": "висмут-207", "5": "1", "6": "Держатель А: 25x3x3. Держатель B: 29x3x3", "7": "1000.0", "8": "50000.0", "9": "0", "10": "", "11": "0", "12": "Межпроверочный интервал: 2 года", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ', 'ОСГИ', '{"3": "ОСГИ", "4": "гадолиний-153", "5": "1", "6": "Держатель А: 25x3x3. Держатель B: 29x3x3", "7": "1000.0", "8": "100000.0", "9": "0", "10": "", "11": "0", "12": "Межпроверочный интервал: 1 год", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ', 'ОСГИ', '{"3": "ОСГИ", "4": "европий-152", "5": "1", "6": "Держатель А: 25x3x3. Держатель B: 29x3x3", "7": "1000.0", "8": "100000.0", "9": "0", "10": "", "11": "0", "12": "Межпроверочный интервал: 2 года", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ', 'ОСГИ', '{"3": "ОСГИ", "4": "железо-55", "5": "1", "6": "Держатель А: 25x3x3. Держатель B: 29x3x3", "7": "1000.0", "8": "100000.0", "9": "0", "10": "", "11": "0", "12": "Межпроверочный интервал: 2 года", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ', 'ОСГИ', '{"3": "ОСГИ", "4": "иттрий-88", "5": "1", "6": "Держатель А: 25x3x3. Держатель B: 29x3x3", "7": "1000.0", "8": "1000000.0", "9": "0", "10": "", "11": "", "12": "Межпроверочный интервал: 1 год", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ', 'ОСГИ', '{"3": "ОСГИ", "4": "йод-125", "5": "1", "6": "", "7": "100.0", "8": "100000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ', 'ОСГИ', '{"3": "ОСГИ", "4": "кадмий-109", "5": "1", "6": "Держатель А: 25x3x3. Держатель B: 29x3x3", "7": "1000.0", "8": "1000000.0", "9": "0", "10": "", "11": "5", "12": "Межпроверочный интервал: 1 год", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ', 'ОСГИ', '{"3": "ОСГИ", "4": "кобальт-57", "5": "1", "6": "Держатель А: 25x3x3. Держатель B: 29x3x3", "7": "1000.0", "8": "100000.0", "9": "0", "10": "", "11": "0", "12": "Межпроверочный интервал: 1 год", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ', 'ОСГИ', '{"3": "ОСГИ", "4": "кобальт-60", "5": "1", "6": "Держатель А: 25x3x3. Держатель B: 29x3x3", "7": "1000.0", "8": "100000.0", "9": "0", "10": "", "11": "5", "12": "Межпроверочный интервал: 2 года", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ', 'ОСГИ', '{"3": "ОСГИ", "4": "марганец-54", "5": "1", "6": "Держатель А: 25x3x3. Держатель B: 29x3x3", "7": "1000.0", "8": "100000.0", "9": "0", "10": "", "11": "0", "12": "Межпроверочный интервал: 1 год", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ', 'ОСГИ', '{"3": "ОСГИ", "4": "натрий-22", "5": "1", "6": "Держатель А: 25x3x3. Держатель B: 29x3x3", "7": "1000.0", "8": "1000000.0", "9": "0", "10": "", "11": "5", "12": "Межпроверочный интервал: 2 года", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ', 'ОСГИ', '{"3": "ОСГИ", "4": "олово-113", "5": "1", "6": "Держатель А: 25x3x3. Держатель B: 29x3x3", "7": "1000.0", "8": "100000.0", "9": "0", "10": "", "11": "0", "12": "Межпроверочный интервал: 1 год", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ', 'ОСГИ', '{"3": "ОСГИ", "4": "ртуть-203", "5": "1", "6": "", "7": "100.0", "8": "100000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ', 'ОСГИ', '{"3": "ОСГИ", "4": "титан-44", "5": "1", "6": "Держатель А: 25x3x3. Держатель B: 29x3x3", "7": "1000.0", "8": "100000.0", "9": "0", "10": "", "11": "", "12": "Межпроверочный интервал: 2 года", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ', 'ОСГИ', '{"3": "ОСГИ", "4": "торий-228", "5": "1", "6": "Держатель А: 25x3x3. Держатель B: 29x3x3", "7": "1000.0", "8": "50000.0", "9": "0", "10": "", "11": "5", "12": "С активностью менее 10e; 3: НСС - 5 лет, межпроверочный интервал - 2 года.С активностью более 10e; 3: НСС - 3 года, межпроверочный интервал - 1 год.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ', 'ОСГИ', '{"3": "ОСГИ", "4": "тулий-170", "5": "1", "6": "", "7": "100.0", "8": "100000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ', 'ОСГИ', '{"3": "ОСГИ", "4": "хлор-39", "5": "1", "6": "", "7": "100.0", "8": "100000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ', 'ОСГИ', '{"3": "ОСГИ", "4": "цезий-134", "5": "1", "6": "Держатель А: 25x3x3. Держатель B: 29x3x3", "7": "1000.0", "8": "100000.0", "9": "0", "10": "", "11": "5", "12": "Межпроверочный интервал: 2 года", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-1', 'ОСГИ-3-1', '{"3": "ОСГИ-3-1", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ', 'ОСГИ', '{"3": "ОСГИ", "4": "церий-139", "5": "1", "6": "Держатель А: 25x3x3. Держатель B: 29x3x3", "7": "1000.0", "8": "1000000.0", "9": "0", "10": "", "11": "5", "12": "Межпроверочный интервал: 1 год", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ', 'ОСГИ', '{"3": "ОСГИ", "4": "цинк-65", "5": "1", "6": "Держатель А: 25x3x3. Держатель B: 29x3x3", "7": "1000.0", "8": "100000.0", "9": "0", "10": "", "11": "0", "12": "Межпроверочный интервал: 1 год", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-2', 'ОСГИ-2', '{"3": "ОСГИ-2", "4": "барий-133", "5": "2", "6": "", "7": "0.0", "8": "100000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3', 'ОСГИ-3', '{"3": "ОСГИ-3", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3', 'ОСГИ-3', '{"3": "ОСГИ-3", "4": "америций-243", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3', 'ОСГИ-3', '{"3": "ОСГИ-3", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3', 'ОСГИ-3', '{"3": "ОСГИ-3", "4": "висмут-207", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3', 'ОСГИ-3', '{"3": "ОСГИ-3", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3', 'ОСГИ-3', '{"3": "ОСГИ-3", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3', 'ОСГИ-3', '{"3": "ОСГИ-3", "4": "железо-55", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3', 'ОСГИ-3', '{"3": "ОСГИ-3", "4": "иттрий-88", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3', 'ОСГИ-3', '{"3": "ОСГИ-3", "4": "йод-125", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3', 'ОСГИ-3', '{"3": "ОСГИ-3", "4": "кадмий-109", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3', 'ОСГИ-3', '{"3": "ОСГИ-3", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3', 'ОСГИ-3', '{"3": "ОСГИ-3", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3', 'ОСГИ-3', '{"3": "ОСГИ-3", "4": "марганец-54", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3', 'ОСГИ-3', '{"3": "ОСГИ-3", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3', 'ОСГИ-3', '{"3": "ОСГИ-3", "4": "олово-113", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3', 'ОСГИ-3', '{"3": "ОСГИ-3", "4": "ртуть-203", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3', 'ОСГИ-3', '{"3": "ОСГИ-3", "4": "титан-44", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3', 'ОСГИ-3', '{"3": "ОСГИ-3", "4": "торий-228", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3', 'ОСГИ-3', '{"3": "ОСГИ-3", "4": "тулий-170", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3', 'ОСГИ-3', '{"3": "ОСГИ-3", "4": "хлор-39", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3', 'ОСГИ-3', '{"3": "ОСГИ-3", "4": "цезий-134", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3', 'ОСГИ-3', '{"3": "ОСГИ-3", "4": "цезий-137", "5": "1", "6": "25,0х3,0", "7": "0.0", "8": "12000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3', 'ОСГИ-3', '{"3": "ОСГИ-3", "4": "церий-139", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3', 'ОСГИ-3', '{"3": "ОСГИ-3", "4": "цинк-65", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-1', 'ОСГИ-3-1', '{"3": "ОСГИ-3-1", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-1', 'ОСГИ-3-1', '{"3": "ОСГИ-3-1", "4": "америций-243", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-1', 'ОСГИ-3-1', '{"3": "ОСГИ-3-1", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-1', 'ОСГИ-3-1', '{"3": "ОСГИ-3-1", "4": "висмут-207", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-1', 'ОСГИ-3-1', '{"3": "ОСГИ-3-1", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-1', 'ОСГИ-3-1', '{"3": "ОСГИ-3-1", "4": "железо-55", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-1', 'ОСГИ-3-1', '{"3": "ОСГИ-3-1", "4": "иттрий-88", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-1', 'ОСГИ-3-1', '{"3": "ОСГИ-3-1", "4": "йод-125", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-1', 'ОСГИ-3-1', '{"3": "ОСГИ-3-1", "4": "кадмий-109", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-1', 'ОСГИ-3-1', '{"3": "ОСГИ-3-1", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-1', 'ОСГИ-3-1', '{"3": "ОСГИ-3-1", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-1', 'ОСГИ-3-1', '{"3": "ОСГИ-3-1", "4": "марганец-54", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-1', 'ОСГИ-3-1', '{"3": "ОСГИ-3-1", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-1', 'ОСГИ-3-1', '{"3": "ОСГИ-3-1", "4": "олово-113", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-1', 'ОСГИ-3-1', '{"3": "ОСГИ-3-1", "4": "ртуть-203", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-1', 'ОСГИ-3-1', '{"3": "ОСГИ-3-1", "4": "титан-44", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-1', 'ОСГИ-3-1', '{"3": "ОСГИ-3-1", "4": "торий-228", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-1', 'ОСГИ-3-1', '{"3": "ОСГИ-3-1", "4": "тулий-170", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-1', 'ОСГИ-3-1', '{"3": "ОСГИ-3-1", "4": "хлор-39", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-1', 'ОСГИ-3-1', '{"3": "ОСГИ-3-1", "4": "цезий-134", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-1', 'ОСГИ-3-1', '{"3": "ОСГИ-3-1", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-1', 'ОСГИ-3-1', '{"3": "ОСГИ-3-1", "4": "церий-139", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-1', 'ОСГИ-3-1', '{"3": "ОСГИ-3-1", "4": "цинк-65", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2', 'ОСГИ-3-2', '{"3": "ОСГИ-3-2", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2', 'ОСГИ-3-2', '{"3": "ОСГИ-3-2", "4": "америций-243", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2', 'ОСГИ-3-2', '{"3": "ОСГИ-3-2", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2', 'ОСГИ-3-2', '{"3": "ОСГИ-3-2", "4": "висмут-207", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2', 'ОСГИ-3-2', '{"3": "ОСГИ-3-2", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2', 'ОСГИ-3-2', '{"3": "ОСГИ-3-2", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2', 'ОСГИ-3-2', '{"3": "ОСГИ-3-2", "4": "железо-55", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2', 'ОСГИ-3-2', '{"3": "ОСГИ-3-2", "4": "иттрий-88", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2', 'ОСГИ-3-2', '{"3": "ОСГИ-3-2", "4": "йод-125", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2', 'ОСГИ-3-2', '{"3": "ОСГИ-3-2", "4": "кадмий-109", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2', 'ОСГИ-3-2', '{"3": "ОСГИ-3-2", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2', 'ОСГИ-3-2', '{"3": "ОСГИ-3-2", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2', 'ОСГИ-3-2', '{"3": "ОСГИ-3-2", "4": "марганец-54", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2', 'ОСГИ-3-2', '{"3": "ОСГИ-3-2", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2', 'ОСГИ-3-2', '{"3": "ОСГИ-3-2", "4": "олово-113", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2', 'ОСГИ-3-2', '{"3": "ОСГИ-3-2", "4": "ртуть-203", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2', 'ОСГИ-3-2', '{"3": "ОСГИ-3-2", "4": "титан-44", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2', 'ОСГИ-3-2', '{"3": "ОСГИ-3-2", "4": "торий-228", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2', 'ОСГИ-3-2', '{"3": "ОСГИ-3-2", "4": "тулий-170", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2', 'ОСГИ-3-2', '{"3": "ОСГИ-3-2", "4": "хлор-39", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2', 'ОСГИ-3-2', '{"3": "ОСГИ-3-2", "4": "цезий-134", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2', 'ОСГИ-3-2', '{"3": "ОСГИ-3-2", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2', 'ОСГИ-3-2', '{"3": "ОСГИ-3-2", "4": "церий-139", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2', 'ОСГИ-3-2', '{"3": "ОСГИ-3-2", "4": "цинк-65", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1', 'ОСГИ-3-2-1', '{"3": "ОСГИ-3-2-1", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1', 'ОСГИ-3-2-1', '{"3": "ОСГИ-3-2-1", "4": "америций-243", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1', 'ОСГИ-3-2-1', '{"3": "ОСГИ-3-2-1", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1', 'ОСГИ-3-2-1', '{"3": "ОСГИ-3-2-1", "4": "висмут-207", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1', 'ОСГИ-3-2-1', '{"3": "ОСГИ-3-2-1", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1', 'ОСГИ-3-2-1', '{"3": "ОСГИ-3-2-1", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1', 'ОСГИ-3-2-1', '{"3": "ОСГИ-3-2-1", "4": "железо-55", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1', 'ОСГИ-3-2-1', '{"3": "ОСГИ-3-2-1", "4": "иттрий-88", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1', 'ОСГИ-3-2-1', '{"3": "ОСГИ-3-2-1", "4": "йод-125", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1', 'ОСГИ-3-2-1', '{"3": "ОСГИ-3-2-1", "4": "кадмий-109", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1', 'ОСГИ-3-2-1', '{"3": "ОСГИ-3-2-1", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1', 'ОСГИ-3-2-1', '{"3": "ОСГИ-3-2-1", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1', 'ОСГИ-3-2-1', '{"3": "ОСГИ-3-2-1", "4": "марганец-54", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1', 'ОСГИ-3-2-1', '{"3": "ОСГИ-3-2-1", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1', 'ОСГИ-3-2-1', '{"3": "ОСГИ-3-2-1", "4": "олово-113", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1', 'ОСГИ-3-2-1', '{"3": "ОСГИ-3-2-1", "4": "ртуть-203", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1', 'ОСГИ-3-2-1', '{"3": "ОСГИ-3-2-1", "4": "титан-44", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1', 'ОСГИ-3-2-1', '{"3": "ОСГИ-3-2-1", "4": "торий-228", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1', 'ОСГИ-3-2-1', '{"3": "ОСГИ-3-2-1", "4": "тулий-170", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1', 'ОСГИ-3-2-1', '{"3": "ОСГИ-3-2-1", "4": "хлор-39", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1', 'ОСГИ-3-2-1', '{"3": "ОСГИ-3-2-1", "4": "цезий-134", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1', 'ОСГИ-3-2-1', '{"3": "ОСГИ-3-2-1", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1', 'ОСГИ-3-2-1', '{"3": "ОСГИ-3-2-1", "4": "церий-139", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1', 'ОСГИ-3-2-1', '{"3": "ОСГИ-3-2-1", "4": "цинк-65", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-2р', 'ОСГИ-3-2-2р', '{"3": "ОСГИ-3-2-2р", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-2р', 'ОСГИ-3-2-2р', '{"3": "ОСГИ-3-2-2р", "4": "америций-243", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-2р', 'ОСГИ-3-2-2р', '{"3": "ОСГИ-3-2-2р", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-2р', 'ОСГИ-3-2-2р', '{"3": "ОСГИ-3-2-2р", "4": "висмут-207", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-2р', 'ОСГИ-3-2-2р', '{"3": "ОСГИ-3-2-2р", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-2р', 'ОСГИ-3-2-2р', '{"3": "ОСГИ-3-2-2р", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-2р', 'ОСГИ-3-2-2р', '{"3": "ОСГИ-3-2-2р", "4": "железо-55", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-2р', 'ОСГИ-3-2-2р', '{"3": "ОСГИ-3-2-2р", "4": "иттрий-88", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-2р', 'ОСГИ-3-2-2р', '{"3": "ОСГИ-3-2-2р", "4": "йод-125", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-2р', 'ОСГИ-3-2-2р', '{"3": "ОСГИ-3-2-2р", "4": "кадмий-109", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-2р', 'ОСГИ-3-2-2р', '{"3": "ОСГИ-3-2-2р", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-2р', 'ОСГИ-3-2-2р', '{"3": "ОСГИ-3-2-2р", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-2р', 'ОСГИ-3-2-2р', '{"3": "ОСГИ-3-2-2р", "4": "марганец-54", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-2р', 'ОСГИ-3-2-2р', '{"3": "ОСГИ-3-2-2р", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-2р', 'ОСГИ-3-2-2р', '{"3": "ОСГИ-3-2-2р", "4": "олово-113", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-2р', 'ОСГИ-3-2-2р', '{"3": "ОСГИ-3-2-2р", "4": "ртуть-203", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-2р', 'ОСГИ-3-2-2р', '{"3": "ОСГИ-3-2-2р", "4": "титан-44", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-2р', 'ОСГИ-3-2-2р', '{"3": "ОСГИ-3-2-2р", "4": "торий-228", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-2р', 'ОСГИ-3-2-2р', '{"3": "ОСГИ-3-2-2р", "4": "тулий-170", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-2р', 'ОСГИ-3-2-2р', '{"3": "ОСГИ-3-2-2р", "4": "хлор-39", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-2р', 'ОСГИ-3-2-2р', '{"3": "ОСГИ-3-2-2р", "4": "цезий-134", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-2р', 'ОСГИ-3-2-2р', '{"3": "ОСГИ-3-2-2р", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-2р', 'ОСГИ-3-2-2р', '{"3": "ОСГИ-3-2-2р", "4": "церий-139", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-2р', 'ОСГИ-3-2-2р', '{"3": "ОСГИ-3-2-2р", "4": "цинк-65", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-А', 'ОСГИ-А', '{"3": "ОСГИ-А", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-А', 'ОСГИ-А', '{"3": "ОСГИ-А", "4": "америций-243", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-А', 'ОСГИ-А', '{"3": "ОСГИ-А", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-А', 'ОСГИ-А', '{"3": "ОСГИ-А", "4": "висмут-207", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-А', 'ОСГИ-А', '{"3": "ОСГИ-А", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-А', 'ОСГИ-А', '{"3": "ОСГИ-А", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-А', 'ОСГИ-А', '{"3": "ОСГИ-А", "4": "железо-55", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-А', 'ОСГИ-А', '{"3": "ОСГИ-А", "4": "иттрий-88", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-А', 'ОСГИ-А', '{"3": "ОСГИ-А", "4": "йод-125", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-А', 'ОСГИ-А', '{"3": "ОСГИ-А", "4": "кадмий-109", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-А', 'ОСГИ-А', '{"3": "ОСГИ-А", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-А', 'ОСГИ-А', '{"3": "ОСГИ-А", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-А', 'ОСГИ-А', '{"3": "ОСГИ-А", "4": "марганец-54", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-А', 'ОСГИ-А', '{"3": "ОСГИ-А", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-А', 'ОСГИ-А', '{"3": "ОСГИ-А", "4": "олово-113", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-А', 'ОСГИ-А', '{"3": "ОСГИ-А", "4": "ртуть-203", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-А', 'ОСГИ-А', '{"3": "ОСГИ-А", "4": "титан-44", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-А', 'ОСГИ-А', '{"3": "ОСГИ-А", "4": "торий-228", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-А', 'ОСГИ-А', '{"3": "ОСГИ-А", "4": "тулий-170", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-А', 'ОСГИ-А', '{"3": "ОСГИ-А", "4": "хлор-39", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-А', 'ОСГИ-А', '{"3": "ОСГИ-А", "4": "цезий-134", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-А', 'ОСГИ-А', '{"3": "ОСГИ-А", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-А', 'ОСГИ-А', '{"3": "ОСГИ-А", "4": "церий-139", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-А', 'ОСГИ-А', '{"3": "ОСГИ-А", "4": "цинк-65", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Б', 'ОСГИ-Б', '{"3": "ОСГИ-Б", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Б', 'ОСГИ-Б', '{"3": "ОСГИ-Б", "4": "америций-243", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Б', 'ОСГИ-Б', '{"3": "ОСГИ-Б", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Б', 'ОСГИ-Б', '{"3": "ОСГИ-Б", "4": "висмут-207", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Б', 'ОСГИ-Б', '{"3": "ОСГИ-Б", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Б', 'ОСГИ-Б', '{"3": "ОСГИ-Б", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Б', 'ОСГИ-Б', '{"3": "ОСГИ-Б", "4": "железо-55", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Б', 'ОСГИ-Б', '{"3": "ОСГИ-Б", "4": "иттрий-88", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Б', 'ОСГИ-Б', '{"3": "ОСГИ-Б", "4": "йод-125", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Б', 'ОСГИ-Б', '{"3": "ОСГИ-Б", "4": "кадмий-109", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Б', 'ОСГИ-Б', '{"3": "ОСГИ-Б", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Б', 'ОСГИ-Б', '{"3": "ОСГИ-Б", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Б', 'ОСГИ-Б', '{"3": "ОСГИ-Б", "4": "марганец-54", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Б', 'ОСГИ-Б', '{"3": "ОСГИ-Б", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Б', 'ОСГИ-Б', '{"3": "ОСГИ-Б", "4": "олово-113", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Б', 'ОСГИ-Б', '{"3": "ОСГИ-Б", "4": "ртуть-203", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Б', 'ОСГИ-Б', '{"3": "ОСГИ-Б", "4": "титан-44", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Б', 'ОСГИ-Б', '{"3": "ОСГИ-Б", "4": "торий-228", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Б', 'ОСГИ-Б', '{"3": "ОСГИ-Б", "4": "тулий-170", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Б', 'ОСГИ-Б', '{"3": "ОСГИ-Б", "4": "хлор-39", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Б', 'ОСГИ-Б', '{"3": "ОСГИ-Б", "4": "цезий-134", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Б', 'ОСГИ-Б', '{"3": "ОСГИ-Б", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Б', 'ОСГИ-Б', '{"3": "ОСГИ-Б", "4": "церий-139", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Б', 'ОСГИ-Б', '{"3": "ОСГИ-Б", "4": "цинк-65", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-М', 'ОСГИ-М', '{"3": "ОСГИ-М", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-М', 'ОСГИ-М', '{"3": "ОСГИ-М", "4": "америций-243", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-М', 'ОСГИ-М', '{"3": "ОСГИ-М", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-М', 'ОСГИ-М', '{"3": "ОСГИ-М", "4": "висмут-207", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-М', 'ОСГИ-М', '{"3": "ОСГИ-М", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-М', 'ОСГИ-М', '{"3": "ОСГИ-М", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-М', 'ОСГИ-М', '{"3": "ОСГИ-М", "4": "железо-55", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-М', 'ОСГИ-М', '{"3": "ОСГИ-М", "4": "иттрий-88", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-М', 'ОСГИ-М', '{"3": "ОСГИ-М", "4": "йод-125", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-М', 'ОСГИ-М', '{"3": "ОСГИ-М", "4": "кадмий-109", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-М', 'ОСГИ-М', '{"3": "ОСГИ-М", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-М', 'ОСГИ-М', '{"3": "ОСГИ-М", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-М', 'ОСГИ-М', '{"3": "ОСГИ-М", "4": "марганец-54", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-М', 'ОСГИ-М', '{"3": "ОСГИ-М", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-М', 'ОСГИ-М', '{"3": "ОСГИ-М", "4": "олово-113", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-М', 'ОСГИ-М', '{"3": "ОСГИ-М", "4": "ртуть-203", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-М', 'ОСГИ-М', '{"3": "ОСГИ-М", "4": "титан-44", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-М', 'ОСГИ-М', '{"3": "ОСГИ-М", "4": "торий-228", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-М', 'ОСГИ-М', '{"3": "ОСГИ-М", "4": "тулий-170", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-М', 'ОСГИ-М', '{"3": "ОСГИ-М", "4": "хлор-39", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-М', 'ОСГИ-М', '{"3": "ОСГИ-М", "4": "цезий-134", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-М', 'ОСГИ-М', '{"3": "ОСГИ-М", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-М', 'ОСГИ-М', '{"3": "ОСГИ-М", "4": "церий-139", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-М', 'ОСГИ-М', '{"3": "ОСГИ-М", "4": "цинк-65", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Р', 'ОСГИ-Р', '{"3": "ОСГИ-Р", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Р', 'ОСГИ-Р', '{"3": "ОСГИ-Р", "4": "америций-243", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Р', 'ОСГИ-Р', '{"3": "ОСГИ-Р", "4": "барий-133", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Р', 'ОСГИ-Р', '{"3": "ОСГИ-Р", "4": "висмут-207", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Р', 'ОСГИ-Р', '{"3": "ОСГИ-Р", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Р', 'ОСГИ-Р', '{"3": "ОСГИ-Р", "4": "европий-152", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Р', 'ОСГИ-Р', '{"3": "ОСГИ-Р", "4": "железо-55", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Р', 'ОСГИ-Р', '{"3": "ОСГИ-Р", "4": "иттрий-88", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Р', 'ОСГИ-Р', '{"3": "ОСГИ-Р", "4": "йод-125", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Р', 'ОСГИ-Р', '{"3": "ОСГИ-Р", "4": "кадмий-109", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Р', 'ОСГИ-Р', '{"3": "ОСГИ-Р", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Р', 'ОСГИ-Р', '{"3": "ОСГИ-Р", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Р', 'ОСГИ-Р', '{"3": "ОСГИ-Р", "4": "марганец-54", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Р', 'ОСГИ-Р', '{"3": "ОСГИ-Р", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Р', 'ОСГИ-Р', '{"3": "ОСГИ-Р", "4": "олово-113", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Р', 'ОСГИ-Р', '{"3": "ОСГИ-Р", "4": "ртуть-203", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Р', 'ОСГИ-Р', '{"3": "ОСГИ-Р", "4": "титан-44", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Р', 'ОСГИ-Р', '{"3": "ОСГИ-Р", "4": "торий-228", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Р', 'ОСГИ-Р', '{"3": "ОСГИ-Р", "4": "тулий-170", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Р', 'ОСГИ-Р', '{"3": "ОСГИ-Р", "4": "хлор-39", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Р', 'ОСГИ-Р', '{"3": "ОСГИ-Р", "4": "цезий-134", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Р', 'ОСГИ-Р', '{"3": "ОСГИ-Р", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Р', 'ОСГИ-Р', '{"3": "ОСГИ-Р", "4": "церий-139", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Р', 'ОСГИ-Р', '{"3": "ОСГИ-Р", "4": "цинк-65", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСИКЭ', 'ОСИКЭ', '{"3": "ОСИКЭ", "4": "калифорний-252", "5": "3", "6": "", "7": "0.0", "8": "242000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСНИГИ (А)', 'ОСНИГИ (А)', '{"3": "ОСНИГИ (А)", "4": "америций-241", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСНИГИ (Б)', 'ОСНИГИ (Б)', '{"3": "ОСНИГИ (Б)", "4": "кобальт-60", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'П', 'П', '{"3": "П", "4": "полоний-210", "5": "4", "6": "d20 Н 40-125", "7": "40000000000.0", "8": "4000000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'П-9', 'П-9', '{"3": "П-9", "4": "плутоний-239", "5": "3", "6": "35,0х1,0", "7": "0.0", "8": "1600.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'П9-490', 'П9-490', '{"3": "П9-490", "4": "плутоний-239", "5": "3", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "НПО РИ", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'П9-491', 'П9-491', '{"3": "П9-491", "4": "плутоний-239", "5": "3", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "НПО РИ", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'П9-492', 'П9-492', '{"3": "П9-492", "4": "плутоний-239", "5": "3", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "НПО РИ", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'П9-493', 'П9-493', '{"3": "П9-493", "4": "плутоний-239", "5": "3", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "НПО РИ", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'П9-494', 'П9-494', '{"3": "П9-494", "4": "плутоний-239", "5": "3", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "НПО РИ", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'П9-495', 'П9-495', '{"3": "П9-495", "4": "плутоний-239", "5": "3", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "НПО РИ", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'П9-496', 'П9-496', '{"3": "П9-496", "4": "плутоний-239", "5": "3", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "НПО РИ", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПГЛ 1', 'ПГЛ 1', '{"3": "ПГЛ 1", "4": "германий-68", "5": "1", "6": "1x192", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "", "12": "ПГЛ1 сертифицирован в системе сертификации оборудования, изделий и технологий для ядерных установок, радиационных источников и пунктов хранения. Сертификат № РОСС RU.0001.01АЭ00.40.10.0397.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПГЛ 1', 'ПГЛ 1', '{"3": "ПГЛ 1", "4": "натрий-22", "5": "1", "6": "1x192", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "4", "12": "ПГЛ1 сертифицирован в системе сертификации оборудования, изделий и технологий для ядерных установок, радиационных источников и пунктов хранения. Сертификат № РОСС RU.0001.01АЭ00.40.10.0397.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПГЛ 1', 'ПГЛ 1', '{"3": "ПГЛ 1", "4": "титан-44", "5": "1", "6": "1x192", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "", "12": "ПГЛ1 сертифицирован в системе сертификации оборудования, изделий и технологий для ядерных установок, радиационных источников и пунктов хранения. Сертификат № РОСС RU.0001.01АЭ00.40.10.0397.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПГЛ 2', 'ПГЛ 2', '{"3": "ПГЛ 2", "4": "германий-68", "5": "1", "6": "200x200", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "", "12": "ПГЛ2 сертифицирован в системе сертификации оборудования, изделий и технологий для ядерных установок, радиационных источников и пунктов хранения. Сертификат № РОСС RU.0001.01АЭ00.40.10.0266.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПГЛ З', 'ПГЛ З', '{"3": "ПГЛ З", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "13200000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПГЛ1', 'ПГЛ1', '{"3": "ПГЛ1", "4": "германий-68", "5": "1", "6": "", "7": "84000000.0", "8": "0.0", "9": "0", "10": "Циклотрон", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПГЛ2', 'ПГЛ2', '{"3": "ПГЛ2", "4": "германий-68", "5": "1", "6": "", "7": "145000000.0", "8": "0.0", "9": "0", "10": "Циклотрон", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПН-22', 'ПН-22', '{"3": "ПН-22", "4": "натрий-22", "5": "2", "6": "18,0x4,0", "7": "", "8": "200000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПТ-44', 'ПТ-44', '{"3": "ПТ-44", "4": "титан-44", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПТ-44.В01.01', 'ПТ-44.В01.01', '{"3": "ПТ-44.В01.01", "4": "титан-44", "5": "2", "6": "10,0x0,5", "7": "", "8": "250000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПТ-44.В01.02', 'ПТ-44.В01.02', '{"3": "ПТ-44.В01.02", "4": "титан-44", "5": "2", "6": "10,0x0,5", "7": "", "8": "500000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПТ-44.В01.03', 'ПТ-44.В01.03', '{"3": "ПТ-44.В01.03", "4": "титан-44", "5": "2", "6": "10,0x0,5", "7": "0.0", "8": "1000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПТ-44.В01.04', 'ПТ-44.В01.04', '{"3": "ПТ-44.В01.04", "4": "титан-44", "5": "2", "6": "10,0x0,5", "7": "0.0", "8": "2500000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПТ-44.В01.05', 'ПТ-44.В01.05', '{"3": "ПТ-44.В01.05", "4": "титан-44", "5": "2", "6": "10,0x0,5", "7": "", "8": "5000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПТ44В.012', 'ПТ44В.012', '{"3": "ПТ44В.012", "4": "титан-44", "5": "2", "6": "", "7": "", "8": "600000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПТ44В.015', 'ПТ44В.015', '{"3": "ПТ44В.015", "4": "титан-44", "5": "2", "6": "", "7": "", "8": "4630000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПФ2-120', 'ПФ2-120', '{"3": "ПФ2-120", "4": "", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ВНИИА", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПФ3-120', 'ПФ3-120', '{"3": "ПФ3-120", "4": "", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ВНИИА", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПФ7-01', 'ПФ7-01', '{"3": "ПФ7-01", "4": "", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ВНИИА", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПФ7-02', 'ПФ7-02', '{"3": "ПФ7-02", "4": "", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ВНИИА", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПФ7-02М', 'ПФ7-02М', '{"3": "ПФ7-02М", "4": "", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ВНИИА", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПФ9м', 'ПФ9м', '{"3": "ПФ9м", "4": "", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ВНИИА", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Р-1', 'Р-1', '{"3": "Р-1", "4": "радий-226", "5": "3", "6": "", "7": "34500000.0", "8": "41800000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Р2', 'Р2', '{"3": "Р2", "4": "рутений-106", "5": "2", "6": "", "7": "", "8": "16700000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Р-22', 'Р-22', '{"3": "Р-22", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Используются в агрегатах зажигания: СК-22-2К, СКН-11-1, СКН-22-1, СКНА-22-2А, СКНР-22-0,5А", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Р-22', 'Р-22', '{"3": "Р-22", "4": "прометий-147", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Используются в агрегатах зажигания: СК-22-2К, СКН-11-1, СКН-22-1, СКНА-22-2А, СКНР-22-0,5А", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Р-22', 'Р-22', '{"3": "Р-22", "4": "тритий", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Используются в агрегатах зажигания: СК-22-2К, СКН-11-1, СКН-22-1, СКНА-22-2А, СКНР-22-0,5А", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Р-26', 'Р-26', '{"3": "Р-26", "4": "никель-63", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Используются в агрегатах зажигания: СК-22-2К, СКН-11-1, СКН-22-1, СКНА-22-2А, СКНР-22-0,5А", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Р-26', 'Р-26', '{"3": "Р-26", "4": "прометий-147", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Используются в агрегатах зажигания: СК-22-2К, СКН-11-1, СКН-22-1, СКНА-22-2А, СКНР-22-0,5А", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Р-26', 'Р-26', '{"3": "Р-26", "4": "тритий", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Используются в агрегатах зажигания: СК-22-2К, СКН-11-1, СКН-22-1, СКНА-22-2А, СКНР-22-0,5А", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Р2608', 'Р2608', '{"3": "Р2608", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "259000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Р2608-100', 'Р2608-100', '{"3": "Р2608-100", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "407000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Р3', 'Р3', '{"3": "Р3", "4": "рутений-106", "5": "2", "6": "", "7": "", "8": "44100000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Р4', 'Р4', '{"3": "Р4", "4": "рутений-106", "5": "2", "6": "", "7": "", "8": "38400000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Р-4', 'Р-4', '{"3": "Р-4", "4": "радий-226", "5": "2", "6": "", "7": "", "8": "36400000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Р5', 'Р5', '{"3": "Р5", "4": "рутений-106", "5": "2", "6": "", "7": "", "8": "70600000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Р6', 'Р6', '{"3": "Р6", "4": "рутений-106", "5": "2", "6": "", "7": "", "8": "57300000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Р7', 'Р7', '{"3": "Р7", "4": "рутений-106", "5": "2", "6": "", "7": "", "8": "41700000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Р-90', 'Р-90', '{"3": "Р-90", "4": "тритий", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РCd9', 'РCd9', '{"3": "РCd9", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РCd9.061.1', 'РCd9.061.1', '{"3": "РCd9.061.1", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РCd9.061.2', 'РCd9.061.2', '{"3": "РCd9.061.2", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РCd9.061.3', 'РCd9.061.3', '{"3": "РCd9.061.3", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "480000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РCd9.061.4', 'РCd9.061.4', '{"3": "РCd9.061.4", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РCm4', 'РCm4', '{"3": "РCm4", "4": "кюрий-244", "5": "5", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РCm4.061.1', 'РCm4.061.1', '{"3": "РCm4.061.1", "4": "кюрий-244", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РCm4.061.2', 'РCm4.061.2', '{"3": "РCm4.061.2", "4": "кюрий-244", "5": "5", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РCm4.061.3', 'РCm4.061.3', '{"3": "РCm4.061.3", "4": "кюрий-244", "5": "5", "6": "", "7": "", "8": "2220000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РCm4.061.х', 'РCm4.061.х', '{"3": "РCm4.061.х", "4": "кюрий-244", "5": "5", "6": "", "7": "", "8": "2220000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РPu8', 'РPu8', '{"3": "РPu8", "4": "плутоний-238", "5": "5", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РPu8.061.2', 'РPu8.061.2', '{"3": "РPu8.061.2", "4": "плутоний-238", "5": "5", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РPu8.061.X', 'РPu8.061.X', '{"3": "РPu8.061.X", "4": "плутоний-238", "5": "5", "6": "", "7": "", "8": "1110000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РPu8.07.2', 'РPu8.07.2', '{"3": "РPu8.07.2", "4": "плутоний-238", "5": "5", "6": "", "7": "", "8": "2220000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РPu8.07.3', 'РPu8.07.3', '{"3": "РPu8.07.3", "4": "плутоний-238", "5": "5", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РPu8.07.X', 'РPu8.07.X', '{"3": "РPu8.07.X", "4": "плутоний-238", "5": "5", "6": "", "7": "", "8": "3520000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ра', 'Ра', '{"3": "Ра", "4": "радий-226", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РАИП-9-1', 'РАИП-9-1', '{"3": "РАИП-9-1", "4": "плутоний-239", "5": "3", "6": "", "7": "309.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РАИП-9-2', 'РАИП-9-2', '{"3": "РАИП-9-2", "4": "плутоний-239", "5": "3", "6": "", "7": "1040.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РАП', 'РАП', '{"3": "РАП", "4": "плутоний-239", "5": "3", "6": "", "7": "", "8": "5300.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ 5.03.А.К', 'РЖ 5.03.А.К', '{"3": "РЖ 5.03.А.К", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "70000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55', 'РЖ55', '{"3": "РЖ55", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "20000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.З01', 'РЖ55.З01', '{"3": "РЖ55.З01", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р', 'РЖ55.Р', '{"3": "РЖ55.Р", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р01', 'РЖ55.Р01', '{"3": "РЖ55.Р01", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р01А', 'РЖ55.Р01А', '{"3": "РЖ55.Р01А", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р01А.А', 'РЖ55.Р01А.А', '{"3": "РЖ55.Р01А.А", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р01В.А', 'РЖ55.Р01В.А', '{"3": "РЖ55.Р01В.А", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р01Г.А', 'РЖ55.Р01Г.А', '{"3": "РЖ55.Р01Г.А", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р01Г.В', 'РЖ55.Р01Г.В', '{"3": "РЖ55.Р01Г.В", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р02', 'РЖ55.Р02', '{"3": "РЖ55.Р02", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р02А', 'РЖ55.Р02А', '{"3": "РЖ55.Р02А", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "555000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р02А.Б.', 'РЖ55.Р02А.Б.', '{"3": "РЖ55.Р02А.Б.", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "2000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р02А.В', 'РЖ55.Р02А.В', '{"3": "РЖ55.Р02А.В", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р02А.Ж', 'РЖ55.Р02А.Ж', '{"3": "РЖ55.Р02А.Ж", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "20000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р02В.А', 'РЖ55.Р02В.А', '{"3": "РЖ55.Р02В.А", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "15000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р02Г.А', 'РЖ55.Р02Г.А', '{"3": "РЖ55.Р02Г.А", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "20000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р02Г.Б', 'РЖ55.Р02Г.Б', '{"3": "РЖ55.Р02Г.Б", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р02Г.И', 'РЖ55.Р02Г.И', '{"3": "РЖ55.Р02Г.И", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "5170000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р02Д.5С', 'РЖ55.Р02Д.5С', '{"3": "РЖ55.Р02Д.5С", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "51000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р02Е.А', 'РЖ55.Р02Е.А', '{"3": "РЖ55.Р02Е.А", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "20000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р09', 'РЖ55.Р09', '{"3": "РЖ55.Р09", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "6000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р09А', 'РЖ55.Р09А', '{"3": "РЖ55.Р09А", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р09А.Ж', 'РЖ55.Р09А.Ж', '{"3": "РЖ55.Р09А.Ж", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "2960000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р09А.И', 'РЖ55.Р09А.И', '{"3": "РЖ55.Р09А.И", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "2960000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р09А.К', 'РЖ55.Р09А.К', '{"3": "РЖ55.Р09А.К", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "6000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р0В1.А', 'РЖ55.Р0В1.А', '{"3": "РЖ55.Р0В1.А", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р10А.В', 'РЖ55.Р10А.В', '{"3": "РЖ55.Р10А.В", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р10А.Г', 'РЖ55.Р10А.Г', '{"3": "РЖ55.Р10А.Г", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "1480000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.Р10А.Е', 'РЖ55.Р10А.Е', '{"3": "РЖ55.Р10А.Е", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "1480000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЖ55.р2а', 'РЖ55.р2а', '{"3": "РЖ55.р2а", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "3000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИ125-2', 'РИ125-2', '{"3": "РИ125-2", "4": "йод-125", "5": "5", "6": "6,2x6,2", "7": "", "8": "7400000000.0", "9": "0", "10": "", "11": "4", "12": "Ковшового типа", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИ125Г1.1', 'РИ125Г1.1', '{"3": "РИ125Г1.1", "4": "йод-125", "5": "5", "6": "3,0x10,0", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИ125Г1.2', 'РИ125Г1.2', '{"3": "РИ125Г1.2", "4": "йод-125", "5": "5", "6": "3,0x10,0", "7": "", "8": "7400000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИ125Г1.3', 'РИ125Г1.3', '{"3": "РИ125Г1.3", "4": "йод-125", "5": "5", "6": "3,0x10,0", "7": "", "8": "11100000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИ125Г1.4', 'РИ125Г1.4', '{"3": "РИ125Г1.4", "4": "йод-125", "5": "5", "6": "3,0x10,0", "7": "", "8": "18500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИ-575', 'РИ-575', '{"3": "РИ-575", "4": "цезий-137", "5": "1", "6": "", "7": "960.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИБ', 'РИБ', '{"3": "РИБ", "4": "прометий-147", "5": "5", "6": "10,0х10,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИВ-181', 'РИВ-181', '{"3": "РИВ-181", "4": "вольфрам-181", "5": "1", "6": "10,0х10,0", "7": "190000000.0", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "Подложка алюминий или бериллий", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИД-1', 'РИД-1', '{"3": "РИД-1", "4": "плутоний-239", "5": "3", "6": "", "7": "210900.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИД-6М', 'РИД-6М', '{"3": "РИД-6М", "4": "плутоний-239", "5": "3", "6": "", "7": "185000.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИЖ-55', 'РИЖ-55', '{"3": "РИЖ-55", "4": "железо-55", "5": "5", "6": "", "7": "", "8": "4400000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИЖ-55-1', 'РИЖ-55-1', '{"3": "РИЖ-55-1", "4": "железо-55", "5": "5", "6": "30,0x3,5", "7": "", "8": "190000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИЖ-55-2', 'РИЖ-55-2', '{"3": "РИЖ-55-2", "4": "железо-55", "5": "5", "6": "30,0x3,5", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИЖ-55-3', 'РИЖ-55-3', '{"3": "РИЖ-55-3", "4": "железо-55", "5": "5", "6": "30,0x3,5", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИЖ-55-I', 'РИЖ-55-I', '{"3": "РИЖ-55-I", "4": "железо-55", "5": "5", "6": "30,0x3,5", "7": "0.0", "8": "190000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИЖ-55-II', 'РИЖ-55-II', '{"3": "РИЖ-55-II", "4": "железо-55", "5": "5", "6": "30,0x3,5", "7": "0.0", "8": "370000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИЖ-55-III', 'РИЖ-55-III', '{"3": "РИЖ-55-III", "4": "железо-55", "5": "5", "6": "30,0x3,5", "7": "", "8": "700000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИК', 'РИК', '{"3": "РИК", "4": "прометий-147", "5": "5", "6": "10,0х10,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИК-109', 'РИК-109', '{"3": "РИК-109", "4": "кадмий-109", "5": "1", "6": "10x10; пластина 40,0х6,0", "7": "190000000.0", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "Подложка алюминий или бериллий", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИК-109-III', 'РИК-109-III', '{"3": "РИК-109-III", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "540000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИК-109Р', 'РИК-109Р', '{"3": "РИК-109Р", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "303000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИК-109-р-111', 'РИК-109-р-111', '{"3": "РИК-109-р-111", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "100000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-238', 'РИТ-238', '{"3": "РИТ-238", "4": "плутоний-238", "5": "8", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-238.НО32.85', 'РИТ-238.НО32.85', '{"3": "РИТ-238.НО32.85", "4": "плутоний-238", "5": "8", "6": "45,0x74,0", "7": "", "8": "95000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-238.РО41.050', 'РИТ-238.РО41.050', '{"3": "РИТ-238.РО41.050", "4": "плутоний-238", "5": "8", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-238/75', 'РИТ-238/75', '{"3": "РИТ-238/75", "4": "плутоний-238", "5": "8", "6": "67,5x57,5", "7": "", "8": "90700000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-238/86', 'РИТ-238/86', '{"3": "РИТ-238/86", "4": "плутоний-238", "5": "8", "6": "70,0x63,0", "7": "", "8": "100000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-238-0,1', 'РИТ-238-0,1', '{"3": "РИТ-238-0,1", "4": "плутоний-238", "5": "3", "6": "", "7": "123000000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-238-0,14', 'РИТ-238-0,14', '{"3": "РИТ-238-0,14", "4": "плутоний-238", "5": "3", "6": "", "7": "168000000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-90', 'РИТ-90', '{"3": "РИТ-90", "4": "стронций-90; иттрий-90", "5": "8", "6": "", "7": "12000000000000.0", "8": "22000000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-90-180-45', 'РИТ-90-180-45', '{"3": "РИТ-90-180-45", "4": "стронций-90; иттрий-90", "5": "8", "6": "51,0x196,0", "7": "", "8": "2300000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-90-200-45', 'РИТ-90-200-45', '{"3": "РИТ-90-200-45", "4": "стронций-90; иттрий-90", "5": "8", "6": "110,5x104,0", "7": "", "8": "2560000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-90-220-45', 'РИТ-90-220-45', '{"3": "РИТ-90-220-45", "4": "стронций-90; иттрий-90", "5": "8", "6": "52,0x245,0", "7": "", "8": "2800000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-90-230-45', 'РИТ-90-230-45', '{"3": "РИТ-90-230-45", "4": "стронций-90; иттрий-90", "5": "8", "6": "110,5x104,0", "7": "", "8": "2940000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-90-230-95', 'РИТ-90-230-95', '{"3": "РИТ-90-230-95", "4": "стронций-90; иттрий-90", "5": "8", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-90-260-45', 'РИТ-90-260-45', '{"3": "РИТ-90-260-45", "4": "стронций-90; иттрий-90", "5": "8", "6": "110,5x104,0", "7": "", "8": "3320000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-90-290-45', 'РИТ-90-290-45', '{"3": "РИТ-90-290-45", "4": "стронций-90; иттрий-90", "5": "8", "6": "110,5x104,0", "7": "", "8": "3700000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-90-29-45', 'РИТ-90-29-45', '{"3": "РИТ-90-29-45", "4": "стронций-90; иттрий-90", "5": "8", "6": "12,0x22,0", "7": "", "8": "364000000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-90-530-45', 'РИТ-90-530-45', '{"3": "РИТ-90-530-45", "4": "стронций-90; иттрий-90", "5": "8", "6": "111,0x196,0", "7": "", "8": "6760000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-90-580-45', 'РИТ-90-580-45', '{"3": "РИТ-90-580-45", "4": "стронций-90; иттрий-90", "5": "8", "6": "136,0x144,0", "7": "", "8": "7660000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-90-650-45', 'РИТ-90-650-45', '{"3": "РИТ-90-650-45", "4": "стронций-90; иттрий-90", "5": "8", "6": "136,0x152,0", "7": "", "8": "8280000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-90-680-45', 'РИТ-90-680-45', '{"3": "РИТ-90-680-45", "4": "стронций-90; иттрий-90", "5": "8", "6": "154,0x136,0", "7": "", "8": "8700000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-90-800-45', 'РИТ-90-800-45', '{"3": "РИТ-90-800-45", "4": "стронций-90; иттрий-90", "5": "8", "6": "111,0x245,0", "7": "", "8": "1.04e+16", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-К', 'РИТ-К', '{"3": "РИТ-К", "4": "плутоний-239", "5": "3", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТМ', 'РИТМ', '{"3": "РИТМ", "4": "плутоний-238", "5": "8", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТМ-10', 'РИТМ-10', '{"3": "РИТМ-10", "4": "плутоний-238", "5": "8", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТМ-С-01', 'РИТМ-С-01', '{"3": "РИТМ-С-01", "4": "плутоний-238", "5": "8", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТМ-С-02', 'РИТМ-С-02', '{"3": "РИТМ-С-02", "4": "плутоний-238", "5": "8", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТМ-С-024', 'РИТМ-С-024', '{"3": "РИТМ-С-024", "4": "плутоний-238", "5": "8", "6": "12,0x20,0", "7": "", "8": "1900000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТМ-С-02И', 'РИТМ-С-02И', '{"3": "РИТМ-С-02И", "4": "плутоний-238", "5": "3", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90', 'РИТу-90', '{"3": "РИТу-90", "4": "стронций-90", "5": "8", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-102', 'РИТу-90-102', '{"3": "РИТу-90-102", "4": "стронций-90", "5": "8", "6": "110,5x60,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-112', 'РИТу-90-112', '{"3": "РИТу-90-112", "4": "стронций-90", "5": "8", "6": "110,5x60,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-123', 'РИТу-90-123', '{"3": "РИТу-90-123", "4": "стронций-90", "5": "8", "6": "110,5x60,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-135', 'РИТу-90-135', '{"3": "РИТу-90-135", "4": "стронций-90", "5": "8", "6": "110,5x60,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-148', 'РИТу-90-148', '{"3": "РИТу-90-148", "4": "стронций-90", "5": "8", "6": "110,5x60,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-163', 'РИТу-90-163', '{"3": "РИТу-90-163", "4": "стронций-90", "5": "8", "6": "110,5x70,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-179', 'РИТу-90-179', '{"3": "РИТу-90-179", "4": "стронций-90", "5": "8", "6": "110,5x70,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-197', 'РИТу-90-197', '{"3": "РИТу-90-197", "4": "стронций-90", "5": "8", "6": "110,5x70,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-20', 'РИТу-90-20', '{"3": "РИТу-90-20", "4": "стронций-90", "5": "8", "6": "59,0x48,5", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-217', 'РИТу-90-217', '{"3": "РИТу-90-217", "4": "стронций-90", "5": "8", "6": "110,5x90,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-22', 'РИТу-90-22', '{"3": "РИТу-90-22", "4": "стронций-90", "5": "8", "6": "59,0x59,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-239', 'РИТу-90-239', '{"3": "РИТу-90-239", "4": "стронций-90", "5": "8", "6": "110,5x90,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-24', 'РИТу-90-24', '{"3": "РИТу-90-24", "4": "стронций-90", "5": "8", "6": "59,0x59,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-263', 'РИТу-90-263', '{"3": "РИТу-90-263", "4": "стронций-90", "5": "8", "6": "110,5x90,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-27', 'РИТу-90-27', '{"3": "РИТу-90-27", "4": "стронций-90", "5": "8", "6": "59,0x59,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-30', 'РИТу-90-30', '{"3": "РИТу-90-30", "4": "стронций-90", "5": "8", "6": "59,0x59,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-300', 'РИТу-90-300', '{"3": "РИТу-90-300", "4": "стронций-90", "5": "8", "6": "110,5x104,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-320', 'РИТу-90-320', '{"3": "РИТу-90-320", "4": "стронций-90", "5": "8", "6": "110,5x104,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-33', 'РИТу-90-33', '{"3": "РИТу-90-33", "4": "стронций-90", "5": "8", "6": "59,0x79,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-352', 'РИТу-90-352', '{"3": "РИТу-90-352", "4": "стронций-90", "5": "8", "6": "110,5x141,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-36', 'РИТу-90-36', '{"3": "РИТу-90-36", "4": "стронций-90", "5": "8", "6": "59,0x79,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-387', 'РИТу-90-387', '{"3": "РИТу-90-387", "4": "стронций-90", "5": "8", "6": "110,5x141,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-40', 'РИТу-90-40', '{"3": "РИТу-90-40", "4": "стронций-90", "5": "8", "6": "59,0x79,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-425', 'РИТу-90-425', '{"3": "РИТу-90-425", "4": "стронций-90", "5": "8", "6": "110,5x141,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-44', 'РИТу-90-44', '{"3": "РИТу-90-44", "4": "стронций-90", "5": "8", "6": "59,0x79,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-467', 'РИТу-90-467', '{"3": "РИТу-90-467", "4": "стронций-90", "5": "8", "6": "110,5x141,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-48', 'РИТу-90-48', '{"3": "РИТу-90-48", "4": "стронций-90", "5": "8", "6": "59,0x99,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-513', 'РИТу-90-513', '{"3": "РИТу-90-513", "4": "стронций-90", "5": "8", "6": "110,5x141,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-53', 'РИТу-90-53', '{"3": "РИТу-90-53", "4": "стронций-90", "5": "8", "6": "59,0x99,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-564', 'РИТу-90-564', '{"3": "РИТу-90-564", "4": "стронций-90", "5": "8", "6": "110,5x182,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-58', 'РИТу-90-58', '{"3": "РИТу-90-58", "4": "стронций-90", "5": "8", "6": "59,0x99,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-620', 'РИТу-90-620', '{"3": "РИТу-90-620", "4": "стронций-90", "5": "8", "6": "110,5x182,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-64', 'РИТу-90-64', '{"3": "РИТу-90-64", "4": "стронций-90", "5": "8", "6": "59,0x99,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-682', 'РИТу-90-682', '{"3": "РИТу-90-682", "4": "стронций-90", "5": "8", "6": "110,5x182,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-70', 'РИТу-90-70', '{"3": "РИТу-90-70", "4": "стронций-90", "5": "8", "6": "59,0x129,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-750', 'РИТу-90-750', '{"3": "РИТу-90-750", "4": "стронций-90", "5": "8", "6": "110,5x232,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-77', 'РИТу-90-77', '{"3": "РИТу-90-77", "4": "стронций-90", "5": "8", "6": "59,0x129,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-825', 'РИТу-90-825', '{"3": "РИТу-90-825", "4": "стронций-90", "5": "8", "6": "110,5x232,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-85', 'РИТу-90-85', '{"3": "РИТу-90-85", "4": "стронций-90", "5": "8", "6": "59,0x129,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-908', 'РИТу-90-908', '{"3": "РИТу-90-908", "4": "стронций-90", "5": "8", "6": "110,5x232,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-93', 'РИТу-90-93', '{"3": "РИТу-90-93", "4": "стронций-90", "5": "8", "6": "110,5x60,0", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РК109', 'РК109', '{"3": "РК109", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РК109.Р03', 'РК109.Р03', '{"3": "РК109.Р03", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РК109.Р04', 'РК109.Р04', '{"3": "РК109.Р04", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РК109-III-ЦГ2.4', 'РК109-III-ЦГ2.4', '{"3": "РК109-III-ЦГ2.4", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РК109рГ1.1', 'РК109рГ1.1', '{"3": "РК109рГ1.1", "4": "кадмий-109", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РК109рГ1.2', 'РК109рГ1.2', '{"3": "РК109рГ1.2", "4": "кадмий-109", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РК109рГ1.3', 'РК109рГ1.3', '{"3": "РК109рГ1.3", "4": "кадмий-109", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РК109рГ1.4', 'РК109рГ1.4', '{"3": "РК109рГ1.4", "4": "кадмий-109", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РК109рГ1.5', 'РК109рГ1.5', '{"3": "РК109рГ1.5", "4": "кадмий-109", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РК109рГ2.1', 'РК109рГ2.1', '{"3": "РК109рГ2.1", "4": "кадмий-109", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РК109рГ2.2', 'РК109рГ2.2', '{"3": "РК109рГ2.2", "4": "кадмий-109", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РК109рГ2.3', 'РК109рГ2.3', '{"3": "РК109рГ2.3", "4": "кадмий-109", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РК109рГ2.4', 'РК109рГ2.4', '{"3": "РК109рГ2.4", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РК109рГ2.5', 'РК109рГ2.5', '{"3": "РК109рГ2.5", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РК109ЦГ2.4', 'РК109ЦГ2.4', '{"3": "РК109ЦГ2.4", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РК109ЦГ2.5', 'РК109ЦГ2.5', '{"3": "РК109ЦГ2.5", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РКГ109', 'РКГ109', '{"3": "РКГ109", "4": "кадмий-109", "5": "5", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РКГ109рГ1.1', 'РКГ109рГ1.1', '{"3": "РКГ109рГ1.1", "4": "кадмий-109", "5": "5", "6": "10,0x5,0", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РКГ109рГ1.2', 'РКГ109рГ1.2', '{"3": "РКГ109рГ1.2", "4": "кадмий-109", "5": "5", "6": "10,0x5,0", "7": "", "8": "74000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РКГ109рГ1.3', 'РКГ109рГ1.3', '{"3": "РКГ109рГ1.3", "4": "кадмий-109", "5": "5", "6": "10,0x5,0", "7": "", "8": "190000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РКГ109рГ1.4', 'РКГ109рГ1.4', '{"3": "РКГ109рГ1.4", "4": "кадмий-109", "5": "5", "6": "10,0x5,0", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РКГ109рГ1.5', 'РКГ109рГ1.5', '{"3": "РКГ109рГ1.5", "4": "кадмий-109", "5": "5", "6": "10,0x5,0", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РКГ109рГ2.1', 'РКГ109рГ2.1', '{"3": "РКГ109рГ2.1", "4": "кадмий-109", "5": "5", "6": "5,0x3,0", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РКГ109рГ2.2', 'РКГ109рГ2.2', '{"3": "РКГ109рГ2.2", "4": "кадмий-109", "5": "5", "6": "5,0x3,0", "7": "", "8": "74000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РКГ109рГ2.3', 'РКГ109рГ2.3', '{"3": "РКГ109рГ2.3", "4": "кадмий-109", "5": "5", "6": "5,0x3,0", "7": "", "8": "190000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РКГ109рГ2.4', 'РКГ109рГ2.4', '{"3": "РКГ109рГ2.4", "4": "кадмий-109", "5": "5", "6": "5,0x3,0", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РКГ109рГ2.5', 'РКГ109рГ2.5', '{"3": "РКГ109рГ2.5", "4": "кадмий-109", "5": "5", "6": "5,0x3,0", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РН93.ВКЗ.01', 'РН93.ВКЗ.01', '{"3": "РН93.ВКЗ.01", "4": "ниобий-93м", "5": "5", "6": "", "7": "", "8": "851000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РПН-1', 'РПН-1', '{"3": "РПН-1", "4": "таллий-204", "5": "2", "6": "225,0х30,0х12,0", "7": "", "8": "1480000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PPu8.07', 'PPu8.07', '{"3": "PPu8.07", "4": "плутоний-238", "5": "5", "6": "", "7": "", "8": "3700000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PPu8.07.07.3', 'PPu8.07.07.3', '{"3": "PPu8.07.07.3", "4": "плутоний-238", "5": "5", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РС145', 'РС145', '{"3": "РС145", "4": "самарий-145", "5": "5", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РС145Г1.1', 'РС145Г1.1', '{"3": "РС145Г1.1", "4": "самарий-145", "5": "5", "6": "10,0x5,0", "7": "", "8": "190000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РС145Г1.2', 'РС145Г1.2', '{"3": "РС145Г1.2", "4": "самарий-145", "5": "5", "6": "10,0x5,0", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РС145Г1.3', 'РС145Г1.3', '{"3": "РС145Г1.3", "4": "самарий-145", "5": "5", "6": "10,0x5,0", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РС145Г2.1', 'РС145Г2.1', '{"3": "РС145Г2.1", "4": "самарий-145", "5": "5", "6": "5,0x3,0", "7": "", "8": "190000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РС145Г2.2', 'РС145Г2.2', '{"3": "РС145Г2.2", "4": "самарий-145", "5": "5", "6": "5,0x3,0", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РС145Г2.3', 'РС145Г2.3', '{"3": "РС145Г2.3", "4": "самарий-145", "5": "5", "6": "5,0x3,0", "7": "", "8": "740000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЭКС', 'РЭКС', '{"3": "РЭКС", "4": "плутоний-238", "5": "5", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЭКС 200-SN', 'РЭКС 200-SN', '{"3": "РЭКС 200-SN", "4": "плутоний-238", "5": "5", "6": "", "7": "", "8": "17000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЭКС 2203', 'РЭКС 2203', '{"3": "РЭКС 2203", "4": "плутоний-238", "5": "5", "6": "", "7": "", "8": "17000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЭКС-2203-W-1М', 'РЭКС-2203-W-1М', '{"3": "РЭКС-2203-W-1М", "4": "плутоний-238", "5": "5", "6": "", "7": "", "8": "14000000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЭКС-Д-SN', 'РЭКС-Д-SN', '{"3": "РЭКС-Д-SN", "4": "плутоний-238", "5": "5", "6": "", "7": "", "8": "17000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'С-10', 'С-10', '{"3": "С-10", "4": "прометий-147", "5": "2", "6": "", "7": "", "8": "120000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'С-13', 'С-13', '{"3": "С-13", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'С-14', 'С-14', '{"3": "С-14", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'С-15', 'С-15', '{"3": "С-15", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'С-16', 'С-16', '{"3": "С-16", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'С-2', 'С-2', '{"3": "С-2", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'С-2', 'С-2', '{"3": "С-2", "4": "углерод-14", "5": "2", "6": "", "7": "0.0", "8": "21000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'С-3', 'С-3', '{"3": "С-3", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'С-4', 'С-4', '{"3": "С-4", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'С-41', 'С-41', '{"3": "С-41", "4": "радий-226", "5": "3", "6": "", "7": "3718500.0", "8": "237000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'С-41-304', 'С-41-304', '{"3": "С-41-304", "4": "радий-226", "5": "3", "6": "", "7": "0.0", "8": "3850000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'С-41-305', 'С-41-305', '{"3": "С-41-305", "4": "радий-226", "5": "3", "6": "", "7": "0.0", "8": "1850000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'С-45', 'С-45', '{"3": "С-45", "4": "прометий-147", "5": "4", "6": "", "7": "", "8": "1300.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'С-5', 'С-5', '{"3": "С-5", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'С-6', 'С-6', '{"3": "С-6", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'С-62', 'С-62', '{"3": "С-62", "4": "прометий-147", "5": "2", "6": "", "7": "", "8": "12000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'С-64', 'С-64', '{"3": "С-64", "4": "углерод-14", "5": "2", "6": "", "7": "0.0", "8": "1330.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'САДТРН-Н-10', 'САДТРН-Н-10', '{"3": "САДТРН-Н-10", "4": "нептуний-237", "5": "3", "6": "", "7": "0.0", "8": "178.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Самарий', 'Самарий', '{"3": "Самарий", "4": "самарий-151", "5": "2", "6": "", "7": "0.0", "8": "1850000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СБ60', 'СБ60', '{"3": "СБ60", "4": "кобальт-60", "5": "1", "6": "", "7": "", "8": "9220000000000000.0", "9": "0", "10": "", "11": "1", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СБИС-МН-2', 'СБИС-МН-2', '{"3": "СБИС-МН-2", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СБИС-МНА-1', 'СБИС-МНА-1', '{"3": "СБИС-МНА-1", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СВЗТ', 'СВЗТ', '{"3": "СВЗТ", "4": "тритий", "5": "2", "6": "", "7": "", "8": "1850000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СВЗТ-3-3 "Стрелка"', 'СВЗТ-3-3 "Стрелка"', '{"3": "СВЗТ-3-3 \"Стрелка\"", "4": "тритий", "5": "2", "6": "", "7": "", "8": "925000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СВЗТ-3-5 "Выход"', 'СВЗТ-3-5 "Выход"', '{"3": "СВЗТ-3-5 \"Выход\"", "4": "тритий", "5": "2", "6": "", "7": "", "8": "1850000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СВЗТ-3-5 "Стрелка"', 'СВЗТ-3-5 "Стрелка"', '{"3": "СВЗТ-3-5 \"Стрелка\"", "4": "тритий", "5": "2", "6": "", "7": "", "8": "925000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СГ-17.11.1 - СГ-17-11.2', 'СГ-17.11.1 - СГ-17-11.2', '{"3": "СГ-17.11.1 - СГ-17-11.2", "4": "калифорний-252", "5": "4", "6": "65 ;  3", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Длина держателей к сборкам равна длине активной части сборки L. Источники, входящие в сборку, отличаются друг от друга по потоку нейтронов не более чем на 5 %. Отклонение потока нейтронов от номинальных значений ;  20%. Погрешность измерения потока нейтронов менее 6%.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СГ-17-5.1 - СГ-17-5.2', 'СГ-17-5.1 - СГ-17-5.2', '{"3": "СГ-17-5.1 - СГ-17-5.2", "4": "калифорний-252", "5": "4", "6": "50 ;  3", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Длина держателей к сборкам равна длине активной части сборки L. Источники, входящие в сборку, отличаются друг от друга по потоку нейтронов не более чем на 5 %. Отклонение потока нейтронов от номинальных значений ;  20%. Погрешность измерения потока нейтронов менее 6%.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СР17.512', 'СР17.512', '{"3": "СР17.512", "4": "селен-75", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СР17.711', 'СР17.711', '{"3": "СР17.711", "4": "селен-75", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СГ-17-6.1 - СГ-17-6.2', 'СГ-17-6.1 - СГ-17-6.2', '{"3": "СГ-17-6.1 - СГ-17-6.2", "4": "калифорний-252", "5": "4", "6": "60 ;  3", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Длина держателей к сборкам равна длине активной части сборки L. Источники, входящие в сборку, отличаются друг от друга по потоку нейтронов не более чем на 5 %. Отклонение потока нейтронов от номинальных значений ;  20%. Погрешность измерения потока нейтронов менее 6%.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СГ-17-7.1 - СГ-17-7.2', 'СГ-17-7.1 - СГ-17-7.2', '{"3": "СГ-17-7.1 - СГ-17-7.2", "4": "калифорний-252", "5": "4", "6": "43 ;  3", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Длина держателей к сборкам равна длине активной части сборки L. Источники, входящие в сборку, отличаются друг от друга по потоку нейтронов не более чем на 5 %. Отклонение потока нейтронов от номинальных значений ;  20%. Погрешность измерения потока нейтронов менее 6%.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СГ-27.11.1 - СГ-27-11.2', 'СГ-27.11.1 - СГ-27-11.2', '{"3": "СГ-27.11.1 - СГ-27-11.2", "4": "калифорний-252", "5": "4", "6": "65 ;  3", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Длина держателей к сборкам равна длине активной части сборки L. Источники, входящие в сборку, отличаются друг от друга по потоку нейтронов не более чем на 5 %. Отклонение потока нейтронов от номинальных значений ;  20%. Погрешность измерения потока нейтронов менее 6%.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СГ-27-4.1 - СГ-27-4.2', 'СГ-27-4.1 - СГ-27-4.2', '{"3": "СГ-27-4.1 - СГ-27-4.2", "4": "калифорний-252", "5": "4", "6": "40 ;  3", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Длина держателей к сборкам равна длине активной части сборки L. Источники, входящие в сборку, отличаются друг от друга по потоку нейтронов не более чем на 5 %. Отклонение потока нейтронов от номинальных значений ;  20%. Погрешность измерения потока нейтронов менее 6%.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СГ-27-9.1 - СГ-27-9.2', 'СГ-27-9.1 - СГ-27-9.2', '{"3": "СГ-27-9.1 - СГ-27-9.2", "4": "калифорний-252", "5": "4", "6": "54 ;  3", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Длина держателей к сборкам равна длине активной части сборки L. Источники, входящие в сборку, отличаются друг от друга по потоку нейтронов не более чем на 5 %. Отклонение потока нейтронов от номинальных значений ;  20%. Погрешность измерения потока нейтронов менее 6%.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СГ-37-4.1 - СГ-37-4.2', 'СГ-37-4.1 - СГ-37-4.2', '{"3": "СГ-37-4.1 - СГ-37-4.2", "4": "калифорний-252", "5": "4", "6": "40 ;  3", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Длина держателей к сборкам равна длине активной части сборки L. Источники, входящие в сборку, отличаются друг от друга по потоку нейтронов не более чем на 5 %. Отклонение потока нейтронов от номинальных значений ;  20%. Погрешность измерения потока нейтронов менее 6%.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СГ-37-5.1 - СГ-37-5.2', 'СГ-37-5.1 - СГ-37-5.2', '{"3": "СГ-37-5.1 - СГ-37-5.2", "4": "калифорний-252", "5": "4", "6": "50 ;  3", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Длина держателей к сборкам равна длине активной части сборки L. Источники, входящие в сборку, отличаются друг от друга по потоку нейтронов не более чем на 5 %. Отклонение потока нейтронов от номинальных значений ;  20%. Погрешность измерения потока нейтронов менее 6%.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СГ-37-6.1 - СГ-37-6.2', 'СГ-37-6.1 - СГ-37-6.2', '{"3": "СГ-37-6.1 - СГ-37-6.2", "4": "калифорний-252", "5": "4", "6": "60 ;  3", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Длина держателей к сборкам равна длине активной части сборки L. Источники, входящие в сборку, отличаются друг от друга по потоку нейтронов не более чем на 5 %. Отклонение потока нейтронов от номинальных значений ;  20%. Погрешность измерения потока нейтронов менее 6%.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СГ-46-4.1 - СГ-46-4.2', 'СГ-46-4.1 - СГ-46-4.2', '{"3": "СГ-46-4.1 - СГ-46-4.2", "4": "калифорний-252", "5": "4", "6": "40 ;  3", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Длина держателей к сборкам равна длине активной части сборки L. Источники, входящие в сборку, отличаются друг от друга по потоку нейтронов не более чем на 5 %. Отклонение потока нейтронов от номинальных значений ;  20%. Погрешность измерения потока нейтронов менее 6%.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СГ-47-5.1 - СГ-47-5.2', 'СГ-47-5.1 - СГ-47-5.2', '{"3": "СГ-47-5.1 - СГ-47-5.2", "4": "калифорний-252", "5": "4", "6": "50 ;  3", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Длина держателей к сборкам равна длине активной части сборки L. Источники, входящие в сборку, отличаются друг от друга по потоку нейтронов не более чем на 5 %. Отклонение потока нейтронов от номинальных значений ;  20%. Погрешность измерения потока нейтронов менее 6%.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т', 'Т', '{"3": "Т", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СГ-47-6.1 - СГ-47-6.2', 'СГ-47-6.1 - СГ-47-6.2', '{"3": "СГ-47-6.1 - СГ-47-6.2", "4": "калифорний-252", "5": "4", "6": "60 ;  3", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Длина держателей к сборкам равна длине активной части сборки L. Источники, входящие в сборку, отличаются друг от друга по потоку нейтронов не более чем на 5 %. Отклонение потока нейтронов от номинальных значений ;  20%. Погрешность измерения потока нейтронов менее 6%.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СГ-47-7.1 - СГ-47-7.2', 'СГ-47-7.1 - СГ-47-7.2', '{"3": "СГ-47-7.1 - СГ-47-7.2", "4": "калифорний-252", "5": "4", "6": "43 ;  3", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Длина держателей к сборкам равна длине активной части сборки L. Источники, входящие в сборку, отличаются друг от друга по потоку нейтронов не более чем на 5 %. Отклонение потока нейтронов от номинальных значений ;  20%. Погрешность измерения потока нейтронов менее 6%.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СГ-56-5.1 - СГ-56-5.2', 'СГ-56-5.1 - СГ-56-5.2', '{"3": "СГ-56-5.1 - СГ-56-5.2", "4": "калифорний-252", "5": "4", "6": "50 ;  3", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Длина держателей к сборкам равна длине активной части сборки L. Источники, входящие в сборку, отличаются друг от друга по потоку нейтронов не более чем на 5 %. Отклонение потока нейтронов от номинальных значений ;  20%. Погрешность измерения потока нейтронов менее 6%.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СГ-57-7.1 - СГ-57-7.2', 'СГ-57-7.1 - СГ-57-7.2', '{"3": "СГ-57-7.1 - СГ-57-7.2", "4": "калифорний-252", "5": "4", "6": "43 ;  3", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Длина держателей к сборкам равна длине активной части сборки L. Источники, входящие в сборку, отличаются друг от друга по потоку нейтронов не более чем на 5 %. Отклонение потока нейтронов от номинальных значений ;  20%. Погрешность измерения потока нейтронов менее 6%.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СГ-57-9.1 - СГ-46-9.2', 'СГ-57-9.1 - СГ-46-9.2', '{"3": "СГ-57-9.1 - СГ-46-9.2", "4": "калифорний-252", "5": "4", "6": "54 ;  3", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Длина держателей к сборкам равна длине активной части сборки L. Источники, входящие в сборку, отличаются друг от друга по потоку нейтронов не более чем на 5 %. Отклонение потока нейтронов от номинальных значений ;  20%. Погрешность измерения потока нейтронов менее 6%.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СГ-66-6.1 - СГ-66-6.2', 'СГ-66-6.1 - СГ-66-6.2', '{"3": "СГ-66-6.1 - СГ-66-6.2", "4": "калифорний-252", "5": "4", "6": "60 ;  3", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Длина держателей к сборкам равна длине активной части сборки L. Источники, входящие в сборку, отличаются друг от друга по потоку нейтронов не более чем на 5 %. Отклонение потока нейтронов от номинальных значений ;  20%. Погрешность измерения потока нейтронов менее 6%.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СГ-67.11.1 - СГ-67-11.2', 'СГ-67.11.1 - СГ-67-11.2', '{"3": "СГ-67.11.1 - СГ-67-11.2", "4": "калифорний-252", "5": "4", "6": "65 ;  3", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Длина держателей к сборкам равна длине активной части сборки L. Источники, входящие в сборку, отличаются друг от друга по потоку нейтронов не более чем на 5 %. Отклонение потока нейтронов от номинальных значений ;  20%. Погрешность измерения потока нейтронов менее 6%.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СГ-67-9.1 - СГ-67-9.2', 'СГ-67-9.1 - СГ-67-9.2', '{"3": "СГ-67-9.1 - СГ-67-9.2", "4": "калифорний-252", "5": "4", "6": "54 ;  3", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Длина держателей к сборкам равна длине активной части сборки L. Источники, входящие в сборку, отличаются друг от друга по потоку нейтронов не более чем на 5 %. Отклонение потока нейтронов от номинальных значений ;  20%. Погрешность измерения потока нейтронов менее 6%.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СГ-76-7.1 - СГ-76-7.2', 'СГ-76-7.1 - СГ-76-7.2', '{"3": "СГ-76-7.1 - СГ-76-7.2", "4": "калифорний-252", "5": "4", "6": "43 ;  3", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Длина держателей к сборкам равна длине активной части сборки L. Источники, входящие в сборку, отличаются друг от друга по потоку нейтронов не более чем на 5 %. Отклонение потока нейтронов от номинальных значений ;  20%. Погрешность измерения потока нейтронов менее 6%.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СГ-86-4.1 - СГ-86-4.2', 'СГ-86-4.1 - СГ-86-4.2', '{"3": "СГ-86-4.1 - СГ-86-4.2", "4": "калифорний-252", "5": "4", "6": "40 ;  3", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Длина держателей к сборкам равна длине активной части сборки L. Источники, входящие в сборку, отличаются друг от друга по потоку нейтронов не более чем на 5 %. Отклонение потока нейтронов от номинальных значений ;  20%. Погрешность измерения потока нейтронов менее 6%.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т(3)', 'Т(3)', '{"3": "Т(3)", "4": "тритий", "5": "2", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СГ-87.11.1 - СГ-87-11.2', 'СГ-87.11.1 - СГ-87-11.2', '{"3": "СГ-87.11.1 - СГ-87-11.2", "4": "калифорний-252", "5": "4", "6": "65 ;  3", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Длина держателей к сборкам равна длине активной части сборки L. Источники, входящие в сборку, отличаются друг от друга по потоку нейтронов не более чем на 5 %. Отклонение потока нейтронов от номинальных значений ;  20%. Погрешность измерения потока нейтронов менее 6%.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СГ-96-9.1 - СГ-6-9.2', 'СГ-96-9.1 - СГ-6-9.2', '{"3": "СГ-96-9.1 - СГ-6-9.2", "4": "калифорний-252", "5": "4", "6": "54 ;  3", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "Длина держателей к сборкам равна длине активной части сборки L. Источники, входящие в сборку, отличаются друг от друга по потоку нейтронов не более чем на 5 %. Отклонение потока нейтронов от номинальных значений ;  20%. Погрешность измерения потока нейтронов менее 6%.", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СДС', 'СДС', '{"3": "СДС", "4": "кобальт-57", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СДС-1', 'СДС-1', '{"3": "СДС-1", "4": "кобальт-57", "5": "1", "6": "366,0x12,0", "7": "400000000.0", "8": "600000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СДС-10', 'СДС-10', '{"3": "СДС-10", "4": "кобальт-57", "5": "1", "6": "393x495x18", "7": "400000000.0", "8": "600000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СДС-11', 'СДС-11', '{"3": "СДС-11", "4": "кобальт-57", "5": "1", "6": "534x650x18", "7": "400000000.0", "8": "600000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СДС-2', 'СДС-2', '{"3": "СДС-2", "4": "кобальт-57", "5": "1", "6": "481,0x12,0", "7": "400000000.0", "8": "600000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СДС-3', 'СДС-3', '{"3": "СДС-3", "4": "кобальт-57", "5": "1", "6": "620,0x12,0", "7": "400000000.0", "8": "600000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Се', 'Се', '{"3": "Се", "4": "селен-75", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Се-6', 'Се-6', '{"3": "Се-6", "4": "селен-75", "5": "1", "6": "8,0х11,0", "7": "16000000000.0", "8": "37000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Се-7', 'Се-7', '{"3": "Се-7", "4": "селен-75", "5": "1", "6": "13,0х13,5", "7": "77500000000.0", "8": "160000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Се-8', 'Се-8', '{"3": "Се-8", "4": "селен-75", "5": "1", "6": "17,0х17,5", "7": "307000000000.0", "8": "625000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СИГИД', 'СИГИД', '{"3": "СИГИД", "4": "кобальт-60", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СИРИ', 'СИРИ', '{"3": "СИРИ", "4": "плутоний-238", "5": "3", "6": "", "7": "0.0", "8": "20000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СН3 РО5 2/15/3', 'СН3 РО5 2/15/3', '{"3": "СН3 РО5 2/15/3", "4": "тритий", "5": "2", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СН3 РО5 7/20/3', 'СН3 РО5 7/20/3', '{"3": "СН3 РО5 7/20/3", "4": "тритий", "5": "2", "6": "", "7": "407000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СН3 РО7 2/15/3', 'СН3 РО7 2/15/3', '{"3": "СН3 РО7 2/15/3", "4": "тритий", "5": "2", "6": "", "7": "1440000000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СН3 РО7 2/6/3', 'СН3 РО7 2/6/3', '{"3": "СН3 РО7 2/6/3", "4": "тритий", "5": "2", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "из отчета ПО маяк", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОМП-0,1', 'СОМП-0,1', '{"3": "СОМП-0,1", "4": "плутоний-239", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОМП-0,3', 'СОМП-0,3', '{"3": "СОМП-0,3", "4": "плутоний-239", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОМУ-10', 'СОМУ-10', '{"3": "СОМУ-10", "4": "уран-235", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОМУ-3', 'СОМУ-3', '{"3": "СОМУ-3", "4": "уран-235", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СР17.111', 'СР17.111', '{"3": "СР17.111", "4": "селен-75", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СР17.212', 'СР17.212', '{"3": "СР17.212", "4": "селен-75", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СР17.311', 'СР17.311', '{"3": "СР17.311", "4": "селен-75", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СР17.412', 'СР17.412', '{"3": "СР17.412", "4": "селен-75", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т(3)-08', 'Т(3)-08', '{"3": "Т(3)-08", "4": "тритий", "5": "2", "6": "", "7": "40000000000.0", "8": "0.0", "9": "0", "10": "4806898", "11": "0", "12": "08625202 ЗРИ 020609 120609", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т(3)-08М', 'Т(3)-08М', '{"3": "Т(3)-08М", "4": "тритий", "5": "2", "6": "", "7": "40000000000.0", "8": "0.0", "9": "0", "10": "4806898", "11": "0", "12": "08625202 ЗРИ 020609 120609", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т-1', 'Т-1', '{"3": "Т-1", "4": "стронций-90; иттрий-90", "5": "2", "6": "Фигурный", "7": "6600000.0", "8": "0.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т-11', 'Т-11', '{"3": "Т-11", "4": "плутоний, стронций-90; иттрий-90", "5": "10", "6": "Фигурный", "7": "", "8": "8400.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т-11', 'Т-11', '{"3": "Т-11", "4": "плутоний-239", "5": "3", "6": "", "7": "", "8": "8400.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т-11', 'Т-11', '{"3": "Т-11", "4": "стронций-90; иттрий-90", "5": "2", "6": "", "7": "", "8": "8400.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т-12', 'Т-12', '{"3": "Т-12", "4": "плутоний-239; стронций-90; иттрий-90", "5": "10", "6": "Фигурный", "7": "", "8": "5100.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т-13', 'Т-13', '{"3": "Т-13", "4": "стронций-90; иттрий-90", "5": "2", "6": "Фигурный", "7": "", "8": "6600.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т-14', 'Т-14', '{"3": "Т-14", "4": "изотопы плутония", "5": "3", "6": "Фигурный", "7": "", "8": "290000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т-14', 'Т-14', '{"3": "Т-14", "4": "плутоний-239", "5": "3", "6": "", "7": "", "8": "29000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т-15', 'Т-15', '{"3": "Т-15", "4": "плутоний-239", "5": "3", "6": "Фигурный", "7": "", "8": "110000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т-16', 'Т-16', '{"3": "Т-16", "4": "плутоний-239", "5": "3", "6": "Фигурный", "7": "", "8": "11000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т-17', 'Т-17', '{"3": "Т-17", "4": "стронций-90; иттрий-90", "5": "2", "6": "Фигурный", "7": "", "8": "66000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т-18', 'Т-18', '{"3": "Т-18", "4": "стронций-90; иттрий-90", "5": "2", "6": "Фигурный", "7": "", "8": "27000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т-19', 'Т-19', '{"3": "Т-19", "4": "стронций-90; иттрий-90", "5": "2", "6": "Фигурный", "7": "", "8": "270000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т-2', 'Т-2', '{"3": "Т-2", "4": "стронций-90; иттрий-90", "5": "2", "6": "Фигурный", "7": "", "8": "660000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т-20', 'Т-20', '{"3": "Т-20", "4": "стронций-90; иттрий-90", "5": "2", "6": "Фигурный", "7": "", "8": "2700000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т-21', 'Т-21', '{"3": "Т-21", "4": "стронций-90; иттрий-90", "5": "2", "6": "Фигурный", "7": "", "8": "3300.0", "9": "0", "10": "ПО Маяк", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т-22', 'Т-22', '{"3": "Т-22", "4": "стронций-90; иттрий-90", "5": "2", "6": "Фигурный", "7": "", "8": "410000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т-3', 'Т-3', '{"3": "Т-3", "4": "стронций-90; иттрий-90", "5": "2", "6": "Фигурный", "7": "", "8": "100000.0", "9": "0", "10": "ПО Маяк", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т-4', 'Т-4', '{"3": "Т-4", "4": "стронций-90; иттрий-90", "5": "2", "6": "Фигурный", "7": "", "8": "6600.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т-5', 'Т-5', '{"3": "Т-5", "4": "изотопы плутония", "5": "3", "6": "Фигурный", "7": "", "8": "2900.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т-5', 'Т-5', '{"3": "Т-5", "4": "плутоний-239", "5": "3", "6": "", "7": "", "8": "2900.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т-6', 'Т-6', '{"3": "Т-6", "4": "изотопы плутония", "5": "3", "6": "Фигурный", "7": "", "8": "1800.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т-88', 'Т-88', '{"3": "Т-88", "4": "таллий-204", "5": "2", "6": "", "7": "", "8": "12100.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТВНT 14142', 'ТВНT 14142', '{"3": "ТВНT 14142", "4": "тритий", "5": "4", "6": "", "7": "", "8": "111000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТГНТ 191', 'ТГНТ 191', '{"3": "ТГНТ 191", "4": "тритий", "5": "4", "6": "", "7": "", "8": "740000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТГНТ 191М', 'ТГНТ 191М', '{"3": "ТГНТ 191М", "4": "тритий", "5": "4", "6": "", "7": "", "8": "740000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Тип-1', 'Тип-1', '{"3": "Тип-1", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "470000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Тип-2', 'Тип-2', '{"3": "Тип-2", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "14000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Тип-I', 'Тип-I', '{"3": "Тип-I", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "470000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Тип-II', 'Тип-II', '{"3": "Тип-II", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "470000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТИС-1', 'ТИС-1', '{"3": "ТИС-1", "4": "тритий", "5": "4", "6": "", "7": "", "8": "200000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТИС-5', 'ТИС-5', '{"3": "ТИС-5", "4": "тритий", "5": "4", "6": "", "7": "", "8": "600000000000.0", "9": "0", "10": "ВНИИА", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ', 'ТНТ', '{"3": "ТНТ", "4": "тритий", "5": "4", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ 132', 'ТНТ 132', '{"3": "ТНТ 132", "4": "тритий", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ 1411', 'ТНТ 1411', '{"3": "ТНТ 1411", "4": "тритий", "5": "4", "6": "", "7": "", "8": "300000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ 141100.11', 'ТНТ 141100.11', '{"3": "ТНТ 141100.11", "4": "тритий", "5": "4", "6": "", "7": "", "8": "8900000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ 1412', 'ТНТ 1412', '{"3": "ТНТ 1412", "4": "тритий", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ 1413', 'ТНТ 1413', '{"3": "ТНТ 1413", "4": "тритий", "5": "4", "6": "", "7": "", "8": "444000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ 1415', 'ТНТ 1415', '{"3": "ТНТ 1415", "4": "тритий", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ 1415М', 'ТНТ 1415М', '{"3": "ТНТ 1415М", "4": "тритий", "5": "4", "6": "", "7": "", "8": "88800000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ 143', 'ТНТ 143', '{"3": "ТНТ 143", "4": "тритий", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ 143-01', 'ТНТ 143-01', '{"3": "ТНТ 143-01", "4": "тритий", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ 144', 'ТНТ 144', '{"3": "ТНТ 144", "4": "тритий", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ 145', 'ТНТ 145', '{"3": "ТНТ 145", "4": "тритий", "5": "4", "6": "", "7": "", "8": "110000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ 1450', 'ТНТ 1450', '{"3": "ТНТ 1450", "4": "тритий", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ 1450-01', 'ТНТ 1450-01', '{"3": "ТНТ 1450-01", "4": "тритий", "5": "4", "6": "", "7": "", "8": "300000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ 1451', 'ТНТ 1451', '{"3": "ТНТ 1451", "4": "тритий", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ 147', 'ТНТ 147', '{"3": "ТНТ 147", "4": "тритий", "5": "4", "6": "", "7": "", "8": "240000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ 147-01', 'ТНТ 147-01', '{"3": "ТНТ 147-01", "4": "тритий", "5": "4", "6": "", "7": "", "8": "444000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ 147-02', 'ТНТ 147-02', '{"3": "ТНТ 147-02", "4": "тритий", "5": "4", "6": "", "7": "", "8": "440000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ 147-1', 'ТНТ 147-1', '{"3": "ТНТ 147-1", "4": "тритий", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ 174', 'ТНТ 174', '{"3": "ТНТ 174", "4": "тритий", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ С1413', 'ТНТ С1413', '{"3": "ТНТ С1413", "4": "тритий", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ С147-01', 'ТНТ С147-01', '{"3": "ТНТ С147-01", "4": "тритий", "5": "4", "6": "", "7": "", "8": "30000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ С192', 'ТНТ С192', '{"3": "ТНТ С192", "4": "тритий", "5": "4", "6": "", "7": "", "8": "370000000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ1411', 'ТНТ1411', '{"3": "ТНТ1411", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ВНИИА", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ1413', 'ТНТ1413', '{"3": "ТНТ1413", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ВНИИА", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ1415', 'ТНТ1415', '{"3": "ТНТ1415", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ВНИИА", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ145', 'ТНТ145', '{"3": "ТНТ145", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ВНИИА", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ147-01', 'ТНТ147-01', '{"3": "ТНТ147-01", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ВНИИА", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ147-02', 'ТНТ147-02', '{"3": "ТНТ147-02", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ВНИИА", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТС1413', 'ТНТС1413', '{"3": "ТНТС1413", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "ВНИИА", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТС192', 'ТНТС192', '{"3": "ТНТС192", "4": "тритий", "5": "0", "6": "", "7": "259000000000.0", "8": "0.0", "9": "0", "10": "ВНИИА", "11": "0", "12": "07623885_01 ЗРИ 070509 140509", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Тритиево-титановая мишень', 'Тритиево-титановая мишень', '{"3": "Тритиево-титановая мишень", "4": "тритий", "5": "2", "6": "", "7": "0.0", "8": "5960000000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'У', 'У', '{"3": "У", "4": "цезий-137", "5": "2", "6": "", "7": "0.0", "8": "213000000000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'У13-86', 'У13-86', '{"3": "У13-86", "4": "углерод-14", "5": "2", "6": "", "7": "0.0", "8": "3300.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'У3П8П9-225', 'У3П8П9-225', '{"3": "У3П8П9-225", "4": "уран-233; плутоний-238; плутоний-239", "5": "3", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "НПО РИ", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'У3П8П9-227', 'У3П8П9-227', '{"3": "У3П8П9-227", "4": "уран-233; плутоний-238; плутоний-239", "5": "3", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "НПО РИ", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'У3П8П9-228', 'У3П8П9-228', '{"3": "У3П8П9-228", "4": "уран-233; плутоний-238; плутоний-239", "5": "3", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "НПО РИ", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'У3П8П9-229', 'У3П8П9-229', '{"3": "У3П8П9-229", "4": "уран-233; плутоний-238; плутоний-239", "5": "3", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "НПО РИ", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'У3П8П9-230', 'У3П8П9-230', '{"3": "У3П8П9-230", "4": "уран-233; плутоний-238; плутоний-239", "5": "3", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "НПО РИ", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Факт-1', 'Факт-1', '{"3": "Факт-1", "4": "полоний-210", "5": "3", "6": "12,2x4,3", "7": "", "8": "1500000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ФГ153М', 'ФГ153М', '{"3": "ФГ153М", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ФГ153М11.410', 'ФГ153М11.410', '{"3": "ФГ153М11.410", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ФГ153М12.110', 'ФГ153М12.110', '{"3": "ФГ153М12.110", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "11100000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ФГ153М13.410', 'ФГ153М13.410', '{"3": "ФГ153М13.410", "4": "гадолиний-153", "5": "1", "6": "", "7": "", "8": "37000000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ФГ153М21.150', 'ФГ153М21.150', '{"3": "ФГ153М21.150", "4": "гадолиний-153", "5": "1", "6": "3; 522; 1,8; 508", "7": "", "8": "5550000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ФГ153М22.60', 'ФГ153М22.60', '{"3": "ФГ153М22.60", "4": "гадолиний-153", "5": "1", "6": "6; 267; 5; 256", "7": "", "8": "2220000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ФГ153М23.300', 'ФГ153М23.300', '{"3": "ФГ153М23.300", "4": "гадолиний-153", "5": "1", "6": "3; 410; 1; 400", "7": "", "8": "11100000000.0", "9": "0", "10": "НИИАР", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ФГ153М24.10', 'ФГ153М24.10', '{"3": "ФГ153М24.10", "4": "гадолиний-153", "5": "1", "6": "4; 130; 1,5; 119", "7": "", "8": "370000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ФГ153М25.10', 'ФГ153М25.10', '{"3": "ФГ153М25.10", "4": "гадолиний-153", "5": "1", "6": "4; 335; 2,8; 324", "7": "", "8": "370000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ФГ153М26.1', 'ФГ153М26.1', '{"3": "ФГ153М26.1", "4": "гадолиний-153", "5": "1", "6": "3; 207; 1,5; 197", "7": "", "8": "33000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ФГ153М26.12', 'ФГ153М26.12', '{"3": "ФГ153М26.12", "4": "гадолиний-153", "5": "1", "6": "3; 207; 1,5; 197", "7": "", "8": "440000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ФГ153М26.2', 'ФГ153М26.2', '{"3": "ФГ153М26.2", "4": "гадолиний-153", "5": "1", "6": "3; 207; 1,5; 197", "7": "", "8": "56000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ФГ153М26.20', 'ФГ153М26.20', '{"3": "ФГ153М26.20", "4": "гадолиний-153", "5": "1", "6": "3; 207; 1,5; 197", "7": "", "8": "740000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ФГ153М26.3', 'ФГ153М26.3', '{"3": "ФГ153М26.3", "4": "гадолиний-153", "5": "1", "6": "3; 207; 1,5; 197", "7": "", "8": "93000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ФГ153М26.4', 'ФГ153М26.4', '{"3": "ФГ153М26.4", "4": "гадолиний-153", "5": "1", "6": "3; 207; 1,5; 197", "7": "", "8": "156000000.0", "9": "0", "10": "НИИАР", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ФГ153М26.7', 'ФГ153М26.7', '{"3": "ФГ153М26.7", "4": "гадолиний-153", "5": "1", "6": "3; 207; 1,5; 197", "7": "", "8": "260000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Фотон-А1', 'Фотон-А1', '{"3": "Фотон-А1", "4": "америций-241", "5": "1", "6": "36,0x32,0", "7": "", "8": "15000000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Фотон-А2', 'Фотон-А2', '{"3": "Фотон-А2", "4": "америций-241", "5": "1", "6": "36,0x32,0", "7": "", "8": "2300000000.0", "9": "0", "10": "НПО РИ", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Фотон-Ф2', 'Фотон-Ф2', '{"3": "Фотон-Ф2", "4": "америций-241", "5": "1", "6": "", "7": "", "8": "3000000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Фотон-Ц-2Н', 'Фотон-Ц-2Н', '{"3": "Фотон-Ц-2Н", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц1', 'Ц1', '{"3": "Ц1", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "101000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц1-5', 'Ц1-5', '{"3": "Ц1-5", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "4090000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц1-6', 'Ц1-6', '{"3": "Ц1-6", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "433000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц1-7', 'Ц1-7', '{"3": "Ц1-7", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "333000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц2-5', 'Ц2-5', '{"3": "Ц2-5", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "9250000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц2-6', 'Ц2-6', '{"3": "Ц2-6", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "590000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц2-7', 'Ц2-7', '{"3": "Ц2-7", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "95500000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц2-8', 'Ц2-8', '{"3": "Ц2-8", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "10000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц4-7', 'Ц4-7', '{"3": "Ц4-7", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "24400000000.0", "9": "0", "10": "", "11": "4", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц5-4', 'Ц5-4', '{"3": "Ц5-4", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "109000000000.0", "9": "0", "10": "", "11": "3", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц6-5', 'Ц6-5', '{"3": "Ц6-5", "4": "цезий-137", "5": "2", "6": "", "7": "0.0", "8": "30800000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц8-6', 'Ц8-6', '{"3": "Ц8-6", "4": "цезий-137", "5": "1", "6": "", "7": "", "8": "7250000000000.0", "9": "0", "10": "", "11": "2", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ШС-4,0', 'ШС-4,0', '{"3": "ШС-4,0", "4": "стронций-90; иттрий-90", "5": "1", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГР', 'ЯГР', '{"3": "ЯГР", "4": "олово-119м", "5": "6", "6": "", "7": "", "8": "370000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГР (МО119mГ2.4)', 'ЯГР (МО119mГ2.4)', '{"3": "ЯГР (МО119mГ2.4)", "4": "олово-119м", "5": "6", "6": "", "7": "", "8": "150000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГР (МО119mГ4.4)', 'ЯГР (МО119mГ4.4)', '{"3": "ЯГР (МО119mГ4.4)", "4": "олово-119м", "5": "6", "6": "", "7": "", "8": "150000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГР (МО119mГ5.4)', 'ЯГР (МО119mГ5.4)', '{"3": "ЯГР (МО119mГ5.4)", "4": "олово-119м", "5": "6", "6": "", "7": "", "8": "150000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГР (МО119mГ6.4)', 'ЯГР (МО119mГ6.4)', '{"3": "ЯГР (МО119mГ6.4)", "4": "олово-119м", "5": "6", "6": "", "7": "", "8": "150000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГР МК57Г5', 'ЯГР МК57Г5', '{"3": "ЯГР МК57Г5", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "1600000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГР-ИГИК', 'ЯГР-ИГИК', '{"3": "ЯГР-ИГИК", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "3770000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИК-57', 'ЯГРИК-57', '{"3": "ЯГРИК-57", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "925000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИК-57-III', 'ЯГРИК-57-III', '{"3": "ЯГРИК-57-III", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "750000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИК-57-IV', 'ЯГРИК-57-IV', '{"3": "ЯГРИК-57-IV", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "3600000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИК-57-l', 'ЯГРИК-57-l', '{"3": "ЯГРИК-57-l", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "460000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИК-57-V', 'ЯГРИК-57-V', '{"3": "ЯГРИК-57-V", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "2100000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИО', 'ЯГРИО', '{"3": "ЯГРИО", "4": "олово-119м", "5": "6", "6": "", "7": "", "8": "5000000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИО-119', 'ЯГРИО-119', '{"3": "ЯГРИО-119", "4": "олово-119м", "5": "6", "6": "", "7": "", "8": "185000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИО-119m', 'ЯГРИО-119m', '{"3": "ЯГРИО-119m", "4": "олово-119м", "5": "6", "6": "", "7": "", "8": "140000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИО-119м-1а', 'ЯГРИО-119м-1а', '{"3": "ЯГРИО-119м-1а", "4": "олово-119м", "5": "6", "6": "17,0х12,0", "7": "", "8": "70000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИО-119м-1б', 'ЯГРИО-119м-1б', '{"3": "ЯГРИО-119м-1б", "4": "олово-119м", "5": "6", "6": "17,0х12,0", "7": "", "8": "140000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИО-119м-2а', 'ЯГРИО-119м-2а', '{"3": "ЯГРИО-119м-2а", "4": "олово-119м", "5": "6", "6": "17,0х12,0", "7": "", "8": "30000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИО-119м-2б', 'ЯГРИО-119м-2б', '{"3": "ЯГРИО-119м-2б", "4": "олово-119м", "5": "6", "6": "17,0х12,0", "7": "", "8": "70000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИО-119м-2в', 'ЯГРИО-119м-2в', '{"3": "ЯГРИО-119м-2в", "4": "олово-119м", "5": "6", "6": "17,0х12,0", "7": "", "8": "140000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИО-119м-3а', 'ЯГРИО-119м-3а', '{"3": "ЯГРИО-119м-3а", "4": "олово-119м", "5": "6", "6": "17,0х12,0", "7": "", "8": "70000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИО-119м-3б', 'ЯГРИО-119м-3б', '{"3": "ЯГРИО-119м-3б", "4": "олово-119м", "5": "6", "6": "17,0х12,0", "7": "", "8": "140000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИО-119м-4а', 'ЯГРИО-119м-4а', '{"3": "ЯГРИО-119м-4а", "4": "олово-119м", "5": "6", "6": "17,0х12,0", "7": "", "8": "30000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИО-119м-4б', 'ЯГРИО-119м-4б', '{"3": "ЯГРИО-119м-4б", "4": "олово-119м", "5": "6", "6": "17,0х12,0", "7": "", "8": "70000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИО-119м-4в', 'ЯГРИО-119м-4в', '{"3": "ЯГРИО-119м-4в", "4": "олово-119м", "5": "6", "6": "17,0х12,0", "7": "", "8": "140000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИО-119м-5а', 'ЯГРИО-119м-5а', '{"3": "ЯГРИО-119м-5а", "4": "олово-119м", "5": "6", "6": "17,0х12,0", "7": "", "8": "30000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИО-119м-5б', 'ЯГРИО-119м-5б', '{"3": "ЯГРИО-119м-5б", "4": "олово-119м", "5": "6", "6": "17,0х12,0", "7": "", "8": "70000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИО-119м-5в', 'ЯГРИО-119м-5в', '{"3": "ЯГРИО-119м-5в", "4": "олово-119м", "5": "6", "6": "17,0х12,0", "7": "", "8": "140000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИО-119м-6а', 'ЯГРИО-119м-6а', '{"3": "ЯГРИО-119м-6а", "4": "олово-119м", "5": "6", "6": "17,0х12,0", "7": "", "8": "30000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИО-119м-6б', 'ЯГРИО-119м-6б', '{"3": "ЯГРИО-119м-6б", "4": "олово-119м", "5": "6", "6": "17,0х12,0", "7": "", "8": "70000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИО-119м-6в', 'ЯГРИО-119м-6в', '{"3": "ЯГРИО-119м-6в", "4": "олово-119м", "5": "6", "6": "17,0х11,0", "7": "", "8": "140000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИО-119м-VIв', 'ЯГРИО-119м-VIв', '{"3": "ЯГРИО-119м-VIв", "4": "олово-119м", "5": "6", "6": "", "7": "", "8": "140000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИО-119м-Шб', 'ЯГРИО-119м-Шб', '{"3": "ЯГРИО-119м-Шб", "4": "олово-119м", "5": "6", "6": "", "7": "", "8": "140000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИО-VI', 'ЯГРИО-VI', '{"3": "ЯГРИО-VI", "4": "олово-119м", "5": "6", "6": "", "7": "", "8": "120000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИО-VIв', 'ЯГРИО-VIв', '{"3": "ЯГРИО-VIв", "4": "олово-119м", "5": "6", "6": "", "7": "", "8": "140000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГРИО-VIд', 'ЯГРИО-VIд', '{"3": "ЯГРИО-VIд", "4": "олово-119м", "5": "6", "6": "", "7": "", "8": "400000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГР-МАК', 'ЯГР-МАК', '{"3": "ЯГР-МАК", "4": "олово-119м", "5": "6", "6": "", "7": "", "8": "192000000.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГР-МАК-III', 'ЯГР-МАК-III', '{"3": "ЯГР-МАК-III", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "37000000.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЯГР-МАК-III', 'ЯГР-МАК-III', '{"3": "ЯГР-МАК-III", "4": "олово-119м", "5": "6", "6": "", "7": "", "8": "0.0", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'В-07', 'В-07', '{"3": "В-07", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Am-241', 'Am-241', '{"3": "Am-241", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2083', '2083', '{"3": "2083", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Am1/G44', 'Am1/G44', '{"3": "Am1/G44", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-251', '3П9-251', '{"3": "3П9-251", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3077', '3077', '{"3": "3077", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-124', '5П9-124', '{"3": "5П9-124", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РСМ4.061.3', 'РСМ4.061.3', '{"3": "РСМ4.061.3", "4": "кюрий-244", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОРР', 'ОРР', '{"3": "ОРР", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМН-Г-3-Н (ОИСН)', 'ИМН-Г-3-Н (ОИСН)', '{"3": "ИМН-Г-3-Н (ОИСН)", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Am1.G22', 'Am1.G22', '{"3": "Am1.G22", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'F6A', 'F6A', '{"3": "F6A", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Контрольный источник', 'Контрольный источник', '{"3": "Контрольный источник", "4": "плутоний-238; плутоний-239; плутоний-242", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GBa3.061.4', 'GBa3.061.4', '{"3": "GBa3.061.4", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПЭ-1', 'ПЭ-1', '{"3": "ПЭ-1", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IND1402', 'IND1402', '{"3": "IND1402", "4": "калий-40", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОП2-01-1,4', 'ОП2-01-1,4', '{"3": "ОП2-01-1,4", "4": "уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCs7.ГЗР', 'GCs7.ГЗР', '{"3": "GCs7.ГЗР", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CoO.A86', 'CoO.A86', '{"3": "CoO.A86", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '569AN', '569AN', '{"3": "569AN", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ампула', 'Ампула', '{"3": "Ампула", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GRa6.1P2', 'GRa6.1P2', '{"3": "GRa6.1P2", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'А424-13', 'А424-13', '{"3": "А424-13", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-54', '1П9-54', '{"3": "1П9-54", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'QA', 'QA', '{"3": "QA", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'VZ-2946', 'VZ-2946', '{"3": "VZ-2946", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1T4', '1T4', '{"3": "1T4", "4": "таллий-204", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-634.11', '1П9-634.11', '{"3": "1П9-634.11", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'К-ЗА', 'К-ЗА', '{"3": "К-ЗА", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2608-100', '2608-100', '{"3": "2608-100", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ba-133', 'Ba-133', '{"3": "Ba-133", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '19513-01', '19513-01', '{"3": "19513-01", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Am-Li', 'Am-Li', '{"3": "Am-Li", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-23', '5П9-23', '{"3": "5П9-23", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ra-226', 'Ra-226', '{"3": "Ra-226", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц1-4', 'Ц1-4', '{"3": "Ц1-4", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'VZ-1508/2', 'VZ-1508/2', '{"3": "VZ-1508/2", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНТ Ж83-З919', 'ИНТ Ж83-З919', '{"3": "ИНТ Ж83-З919", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-100-2р', 'ОСГИ-100-2р', '{"3": "ОСГИ-100-2р", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.700', 'CDC.700', '{"3": "CDC.700", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'R6060', 'R6060', '{"3": "R6060", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОП2-06-1,4', 'ОП2-06-1,4', '{"3": "ОП2-06-1,4", "4": "уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NCf2.P01', 'NCf2.P01', '{"3": "NCf2.P01", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Бета-прибор', 'Бета-прибор', '{"3": "Бета-прибор", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Cs7.P02', 'Cs7.P02', '{"3": "Cs7.P02", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2б-1', '2б-1', '{"3": "2б-1", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИДКР-Р', 'ОИДКР-Р', '{"3": "ОИДКР-Р", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PHCX1385', 'PHCX1385', '{"3": "PHCX1385", "4": "прометий-147", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Со0.А87', 'Со0.А87', '{"3": "Со0.А87", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-162.11', '4П9-162.11', '{"3": "4П9-162.11", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'N', 'N', '{"3": "N", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ir2.A85-2', 'Ir2.A85-2', '{"3": "Ir2.A85-2", "4": "иридий-192", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4CO', '4CO', '{"3": "4CO", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Cf-252', 'Cf-252', '{"3": "Cf-252", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5109.11', '5109.11', '{"3": "5109.11", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '27527-01', '27527-01', '{"3": "27527-01", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ir2.A-85-2', 'Ir2.A-85-2', '{"3": "Ir2.A-85-2", "4": "иридий-192", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GNa2', 'GNa2', '{"3": "GNa2", "4": "натрий-22", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНТ Ж83-Р919', 'ИНТ Ж83-Р919', '{"3": "ИНТ Ж83-Р919", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-56', '6СО-56', '{"3": "6СО-56", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '7200А', '7200А', '{"3": "7200А", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3,69E+04', '3,69E+04', '{"3": "3,69E+04", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Co0.A86', 'Co0.A86', '{"3": "Co0.A86", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MMS07-153-100U', 'MMS07-153-100U', '{"3": "MMS07-153-100U", "4": "гадолиний-153", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-24', '5П9-24', '{"3": "5П9-24", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GM232.02-000-SFC', 'GM232.02-000-SFC', '{"3": "GM232.02-000-SFC", "4": "иридий-192", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SIF.D2', 'SIF.D2', '{"3": "SIF.D2", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CLN.PG1', 'CLN.PG1', '{"3": "CLN.PG1", "4": "кюрий-244; углерод-13", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Бетта-прибор КИ101/50', 'Бетта-прибор КИ101/50', '{"3": "Бетта-прибор КИ101/50", "4": "иттрий-90; стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСS7.012.11', 'ГСS7.012.11', '{"3": "ГСS7.012.11", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГРа6.х', 'ГРа6.х', '{"3": "ГРа6.х", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПГЛ.3', 'ПГЛ.3', '{"3": "ПГЛ.3", "4": "титан-44", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SIFQ5792', 'SIFQ5792', '{"3": "SIFQ5792", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-253.11', '4П9-253.11', '{"3": "4П9-253.11", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АРЕВ 10.000', 'АРЕВ 10.000', '{"3": "АРЕВ 10.000", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Co-57', 'Co-57', '{"3": "Co-57", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PNG-BA', 'PNG-BA', '{"3": "PNG-BA", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "1/19/2016", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C4.4', 'BNi3.C4.4', '{"3": "BNi3.C4.4", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'A3201-4', 'A3201-4', '{"3": "A3201-4", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИК-8', 'ИК-8', '{"3": "ИК-8", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SZSr-12', 'SZSr-12', '{"3": "SZSr-12", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-34', '5П9-34', '{"3": "5П9-34", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'проба ЛАС', 'проба ЛАС', '{"3": "проба ЛАС", "4": "кобальт-60; цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'R', 'R', '{"3": "R", "4": "железо-55", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3,73E+04', '3,73E+04', '{"3": "3,73E+04", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КИ', 'КИ', '{"3": "КИ", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'радиоэлемент', 'радиоэлемент', '{"3": "радиоэлемент", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EAB-PL', 'EAB-PL', '{"3": "EAB-PL", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНМ-Г-3-Т', 'ИНМ-Г-3-Т', '{"3": "ИНМ-Г-3-Т", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GGLS', 'GGLS', '{"3": "GGLS", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "11/28/2017 5:26:11 PM", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИбИРЗН-63-IIб-1', 'ИбИРЗН-63-IIб-1', '{"3": "ИбИРЗН-63-IIб-1", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'VQС-068', 'VQС-068', '{"3": "VQС-068", "4": "германий-68", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1CО-126', '1CО-126', '{"3": "1CО-126", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO0.А86', 'CO0.А86', '{"3": "CO0.А86", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВН3.731', 'ВН3.731', '{"3": "ВН3.731", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-632.11', '4П9-632.11', '{"3": "4П9-632.11", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'A3906', 'A3906', '{"3": "A3906", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Cо0.А86', 'Cо0.А86', '{"3": "Cо0.А86", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RODSOURCE', 'RODSOURCE', '{"3": "RODSOURCE", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИКД', 'ИКД', '{"3": "ИКД", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'LX-64-05', 'LX-64-05', '{"3": "LX-64-05", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КН 031', 'КН 031', '{"3": "КН 031", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЖСС LS6500', 'ЖСС LS6500', '{"3": "ЖСС LS6500", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '11', '11', '{"3": "11", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-54', '5П9-54', '{"3": "5П9-54", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'R', 'R', '{"3": "R", "4": "кадмий-109", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Тритиевая  мишень', 'Тритиевая  мишень', '{"3": "Тритиевая  мишень", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3,84E+04', '3,84E+04', '{"3": "3,84E+04", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C10.1.R', 'BNi3.C10.1.R', '{"3": "BNi3.C10.1.R", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EAB-PL', 'EAB-PL', '{"3": "EAB-PL", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSRO.40', 'BSRO.40', '{"3": "BSRO.40", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6CO', '6CO', '{"3": "6CO", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6С0-534', '6С0-534', '{"3": "6С0-534", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KACK8044', 'KACK8044', '{"3": "KACK8044", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-21', '3П9-21', '{"3": "3П9-21", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АИП-МИР-3', 'АИП-МИР-3', '{"3": "АИП-МИР-3", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КАСВ14466', 'КАСВ14466', '{"3": "КАСВ14466", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EP-14', 'EP-14', '{"3": "EP-14", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'P2608-100', 'P2608-100', '{"3": "P2608-100", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМН-Г-1 (ОСГИ)', 'ИМН-Г-1 (ОСГИ)', '{"3": "ИМН-Г-1 (ОСГИ)", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КН 032', 'КН 032', '{"3": "КН 032", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'контрольный источник в приборе', 'контрольный источник в приборе', '{"3": "контрольный источник в приборе", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МК57.ВР', 'МК57.ВР', '{"3": "МК57.ВР", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'генератор', 'генератор', '{"3": "генератор", "4": "германий-68", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСS7.021.1', 'ГСS7.021.1', '{"3": "ГСS7.021.1", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3,69E+04', '3,69E+04', '{"3": "3,69E+04", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EGSA15', 'EGSA15', '{"3": "EGSA15", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NCf.Р01', 'NCf.Р01', '{"3": "NCf.Р01", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ХАm1.06', 'ХАm1.06', '{"3": "ХАm1.06", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCs7.464', 'GCs7.464', '{"3": "GCs7.464", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMMQ9495', 'AMMQ9495', '{"3": "AMMQ9495", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SRM 4968', 'SRM 4968', '{"3": "SRM 4968", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-24', '1П9-24', '{"3": "1П9-24", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Eu-152', 'Eu-152', '{"3": "Eu-152", "4": "европий-152", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РКБ-05', 'РКБ-05', '{"3": "РКБ-05", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'I25.S061-60', 'I25.S061-60', '{"3": "I25.S061-60", "4": "иод-125", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИН', 'ИН', '{"3": "ИН", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3,69E+04', '3,69E+04', '{"3": "3,69E+04", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'A3916-1', 'A3916-1', '{"3": "A3916-1", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-25', '5СО-25', '{"3": "5СО-25", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Аm241', 'Аm241', '{"3": "Аm241", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'HEGL-137', 'HEGL-137', '{"3": "HEGL-137", "4": "Цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСS7.021.6', 'ГСS7.021.6', '{"3": "ГСS7.021.6", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MS', 'MS', '{"3": "MS", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЗП9', 'ЗП9', '{"3": "ЗП9", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-МН2', 'БИС-МН2', '{"3": "БИС-МН2", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Pu-239', 'Pu-239', '{"3": "Pu-239", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.CY4', 'AMN.CY4', '{"3": "AMN.CY4", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ж08 Р3479-01', 'Ж08 Р3479-01', '{"3": "Ж08 Р3479-01", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-26', '5СО-26', '{"3": "5СО-26", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ru6.A05', 'Ru6.A05', '{"3": "Ru6.A05", "4": "рутений-106", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМН-Г-1', 'ИМН-Г-1', '{"3": "ИМН-Г-1", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОРР', 'ОРР', '{"3": "ОРР", "4": "плутоний-239; плутоний-240", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3,73E+04', '3,73E+04', '{"3": "3,73E+04", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО-7520-99', 'ГСО-7520-99', '{"3": "ГСО-7520-99", "4": "уран-234; уран-235; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Со0.А86', 'Со0.А86', '{"3": "Со0.А86", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВН3.Х.1', 'ВН3.Х.1', '{"3": "ВН3.Х.1", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'S17F04', 'S17F04', '{"3": "S17F04", "4": "америций-241Be", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РА-226', 'РА-226', '{"3": "РА-226", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'HK252M11.17', 'HK252M11.17', '{"3": "HK252M11.17", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЗУ8', 'ЗУ8', '{"3": "ЗУ8", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C1.3.R', 'BNi3.C1.3.R', '{"3": "BNi3.C1.3.R", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Pu-Be', 'Pu-Be', '{"3": "Pu-Be", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AN-HP', 'AN-HP', '{"3": "AN-HP", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-34', '5СО-34', '{"3": "5СО-34", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'I25.S061-65', 'I25.S061-65', '{"3": "I25.S061-65", "4": "иод-125", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RU6.A03', 'RU6.A03', '{"3": "RU6.A03", "4": "рутений-106", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМН-Г-1', 'ИМН-Г-1', '{"3": "ИМН-Г-1", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3,76E+04', '3,76E+04', '{"3": "3,76E+04", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '-', '-', '{"3": "-", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО-7542-99', 'ГСО-7542-99', '{"3": "ГСО-7542-99", "4": "уран-234; уран-235; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ind1500', 'Ind1500', '{"3": "Ind1500", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ж83-2044', 'Ж83-2044', '{"3": "Ж83-2044", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'A3916-7', 'A3916-7', '{"3": "A3916-7", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "12/14/2017 12:38:21 PM", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВН3.Х.2', 'ВН3.Х.2', '{"3": "ВН3.Х.2", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'S17S15', 'S17S15', '{"3": "S17S15", "4": "америций-241Be", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RS-P6-1808-201 HELIUM-3', 'RS-P6-1808-201 HELIUM-3', '{"3": "RS-P6-1808-201 HELIUM-3", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "1/19/2016", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т48012', 'Т48012', '{"3": "Т48012", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-35', '5СО-35', '{"3": "5СО-35", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'I25.S061-55', 'I25.S061-55', '{"3": "I25.S061-55", "4": "иод-125", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RU6.A04', 'RU6.A04', '{"3": "RU6.A04", "4": "рутений-106", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМН-Г-1', 'ИМН-Г-1', '{"3": "ИМН-Г-1", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЕР-9', 'ЕР-9', '{"3": "ЕР-9", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '-', '-', '{"3": "-", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'калибров. фильтр', 'калибров. фильтр', '{"3": "калибров. фильтр", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'S3X295', 'S3X295', '{"3": "S3X295", "4": "церий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ISOXSRCE', 'ISOXSRCE', '{"3": "ISOXSRCE", "4": "европий-155; натрий-22", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'источник излучения нейтронов', 'источник излучения нейтронов', '{"3": "источник излучения нейтронов", "4": "плутоний; бериллий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Калибровоч-ный', 'Калибровоч-ный', '{"3": "Калибровоч-ный", "4": "уран-234; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'stabstrahler', 'stabstrahler', '{"3": "stabstrahler", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ж83-Р2111', 'Ж83-Р2111', '{"3": "Ж83-Р2111", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ra-226 ЕР', 'Ra-226 ЕР', '{"3": "Ra-226 ЕР", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RS-P6-1004-201 HELIUM-3', 'RS-P6-1004-201 HELIUM-3', '{"3": "RS-P6-1004-201 HELIUM-3", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "1/19/2016", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RU6.A06', 'RU6.A06', '{"3": "RU6.A06", "4": "рутений-106", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Bkr5.204', 'Bkr5.204', '{"3": "Bkr5.204", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RD-23', 'RD-23', '{"3": "RD-23", "4": "уран-234; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ra-Be', 'Ra-Be', '{"3": "Ra-Be", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ж83-Р919', 'Ж83-Р919', '{"3": "Ж83-Р919", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'САИ-4А', 'САИ-4А', '{"3": "САИ-4А", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '-', '-', '{"3": "-", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-83', '5СО-83', '{"3": "5СО-83", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Комбинированнный', 'Комбинированнный', '{"3": "Комбинированнный", "4": "уран-238; уран-234; торий-234; протактиний-234", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Комбинированный', 'Комбинированный', '{"3": "Комбинированный", "4": "уран-238; уран-234; торий-234; протактиний-234", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Nickel-63', 'Nickel-63', '{"3": "Nickel-63", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Подвеска КД РБМ-К9.Сб.235', 'Подвеска КД РБМ-К9.Сб.235', '{"3": "Подвеска КД РБМ-К9.Сб.235", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-804.08', '1СО-804.08', '{"3": "1СО-804.08", "4": "иттрий-90; стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-83', '1П9-83', '{"3": "1П9-83", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ж83-Р919-03', 'Ж83-Р919-03', '{"3": "Ж83-Р919-03", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-СП', '1СО-СП', '{"3": "1СО-СП", "4": "иттрий-90; стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '-', '-', '{"3": "-", "4": "Прометий-147", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'бленкер', 'бленкер', '{"3": "бленкер", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Натрий -22', 'Натрий -22', '{"3": "Натрий -22", "4": "натрий-22", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КАСК 8044', 'КАСК 8044', '{"3": "КАСК 8044", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Подвеска КД РБМ-К9.Сб.241', 'Подвеска КД РБМ-К9.Сб.241', '{"3": "Подвеска КД РБМ-К9.Сб.241", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RU6.A08', 'RU6.A08', '{"3": "RU6.A08", "4": "рутений-106", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БВИ', 'БВИ', '{"3": "БВИ", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EP1', 'EP1', '{"3": "EP1", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'STB A 3916-1', 'STB A 3916-1', '{"3": "STB A 3916-1", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "7/23/2015 9:28:42 AM", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'точечный', 'точечный', '{"3": "точечный", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ДКН', 'ДКН', '{"3": "ДКН", "4": "нептуний-237", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-85', '5СО-85', '{"3": "5СО-85", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '-', '-', '{"3": "-", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'бленкер', 'бленкер', '{"3": "бленкер", "4": "хлор-36", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИСН (КП-альфа)', 'ОИСН (КП-альфа)', '{"3": "ОИСН (КП-альфа)", "4": "сумма нуклидов плутония", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Прототип мишени', 'Прототип мишени', '{"3": "Прототип мишени", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-123', '1СО-123', '{"3": "1СО-123", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Подвеска СУЗ-В РБМ-К9.Сб.245', 'Подвеска СУЗ-В РБМ-К9.Сб.245', '{"3": "Подвеска СУЗ-В РБМ-К9.Сб.245", "4": "Уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RU6.A10', 'RU6.A10', '{"3": "RU6.A10", "4": "рутений-106", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-215.08', '3СО-215.08', '{"3": "3СО-215.08", "4": "иттрий-90; стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'STB IND 1402', 'STB IND 1402', '{"3": "STB IND 1402", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "7/23/2015 9:28:42 AM", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Э-4М', 'Э-4М', '{"3": "Э-4М", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '241m02', '241m02', '{"3": "241m02", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '-', '-', '{"3": "-", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'HCF2.012', 'HCF2.012', '{"3": "HCF2.012", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-86', '5СО-86', '{"3": "5СО-86", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСИАИ', 'ОСИАИ', '{"3": "ОСИАИ", "4": "плутоний-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi.C3.4', 'BNi.C3.4', '{"3": "BNi.C3.4", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КАСК8044', 'КАСК8044', '{"3": "КАСК8044", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-32', '6СО-32', '{"3": "6СО-32", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Подвеска ШДК РБМ-К7.Сб.232', 'Подвеска ШДК РБМ-К7.Сб.232', '{"3": "Подвеска ШДК РБМ-К7.Сб.232", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЗРИ из состава РИП', 'ЗРИ из состава РИП', '{"3": "ЗРИ из состава РИП", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИСН-22-4,5', 'ОИСН-22-4,5', '{"3": "ОИСН-22-4,5", "4": "натрий-22", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'еИ3.399.113', 'еИ3.399.113', '{"3": "еИ3.399.113", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'С16', 'С16', '{"3": "С16", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М 11.19', 'НК252М 11.19', '{"3": "НК252М 11.19", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-804.08', '3СО-804.08', '{"3": "3СО-804.08", "4": "иттрий-90; стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Cs7.P03', 'Cs7.P03', '{"3": "Cs7.P03", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М11', 'НК252М11', '{"3": "НК252М11", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-124', '6П9-124', '{"3": "6П9-124", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '-', '-', '{"3": "-", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСИАИ', 'ОСИАИ', '{"3": "ОСИАИ", "4": "плутоний-238; плутоний-239; уран-233", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-21', '1П9-21', '{"3": "1П9-21", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-33', '6СО-33', '{"3": "6СО-33", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'еИ3.399.113-01', 'еИ3.399.113-01', '{"3": "еИ3.399.113-01", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'С-146', 'С-146', '{"3": "С-146", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-805.08', '3СО-805.08', '{"3": "3СО-805.08", "4": "иттрий-90; стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Мессбауэровский', 'Мессбауэровский', '{"3": "Мессбауэровский", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЕР', 'ЕР', '{"3": "ЕР", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'C25745', 'C25745', '{"3": "C25745", "4": "сумма альфа-активных естеств. нуклидов", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-24', '6П9-24', '{"3": "6П9-24", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '105.002', '105.002', '{"3": "105.002", "4": "иридий-192", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСИАИ', 'ОСИАИ', '{"3": "ОСИАИ", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-22', '1П9-22', '{"3": "1П9-22", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВНТ4-32', 'ВНТ4-32', '{"3": "ВНТ4-32", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Источник бета излучения', 'Источник бета излучения', '{"3": "Источник бета излучения", "4": "Никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Калибр. фильтр', 'Калибр. фильтр', '{"3": "Калибр. фильтр", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-215.11', '4СО-215.11', '{"3": "4СО-215.11", "4": "иттрий-90; стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.CY11', 'CDC.CY11', '{"3": "CDC.CY11", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-34', '6П9-34', '{"3": "6П9-34", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ж98Р0104', 'Ж98Р0104', '{"3": "Ж98Р0104", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСИАИ', 'ОСИАИ', '{"3": "ОСИАИ", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-106', '5СО-106', '{"3": "5СО-106", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-23', '1П9-23', '{"3": "1П9-23", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-534.08', '4СО-534.08', '{"3": "4СО-534.08", "4": "иттрий-90; стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИА-И', 'РИА-И', '{"3": "РИА-И", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЕР-10', 'ЕР-10', '{"3": "ЕР-10", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.CY13', 'CDC.CY13', '{"3": "CDC.CY13", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ж98Р0105', 'Ж98Р0105', '{"3": "Ж98Р0105", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КФ-1', 'КФ-1', '{"3": "КФ-1", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-53', '6П9-53', '{"3": "6П9-53", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СКНА-22-2а', 'СКНА-22-2а', '{"3": "СКНА-22-2а", "4": "прометий-147", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КГИ  U-235', 'КГИ  U-235', '{"3": "КГИ  U-235", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-535.08', '4СО-535.08', '{"3": "4СО-535.08", "4": "иттрий-90; стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2Б', '2Б', '{"3": "2Б", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СP17.90', 'СP17.90', '{"3": "СP17.90", "4": "селен-75", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDRB 10275', 'CDRB 10275', '{"3": "CDRB 10275", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЕР-16', 'ЕР-16', '{"3": "ЕР-16", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-54', '6П9-54', '{"3": "6П9-54", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'П9-245', 'П9-245', '{"3": "П9-245", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'А3536', 'А3536', '{"3": "А3536", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИТР-М5', 'БИТР-М5', '{"3": "БИТР-М5", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-22', '1СО-22', '{"3": "1СО-22", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-536.08', '4СО-536.08', '{"3": "4СО-536.08", "4": "иттрий-90; стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDRB 10276', 'CDRB 10276', '{"3": "CDRB 10276", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-24', '1СО-24', '{"3": "1СО-24", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СИ', 'СИ', '{"3": "СИ", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЕР-8', 'ЕР-8', '{"3": "ЕР-8", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИСН-210', 'ОИСН-210', '{"3": "ОИСН-210", "4": "висмут-210; полоний-210; свинец-210", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-83', '6П9-83', '{"3": "6П9-83", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПГЛ 3', 'ПГЛ 3', '{"3": "ПГЛ 3", "4": "титан-44", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3С0-536', '3С0-536', '{"3": "3С0-536", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-254.08', '5П9-254.08', '{"3": "5П9-254.08", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'раствор', 'раствор', '{"3": "раствор", "4": "уран-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GSR-Y', 'GSR-Y', '{"3": "GSR-Y", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИСН-22-9', 'ОИСН-22-9', '{"3": "ОИСН-22-9", "4": "натрий-22", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-24', '6СО-24', '{"3": "6СО-24", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГСН', 'ИГСН', '{"3": "ИГСН", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-214.08', '6СО-214.08', '{"3": "6СО-214.08", "4": "иттрий-90; стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИСН-90-3', 'ОИСН-90-3', '{"3": "ОИСН-90-3", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПГЛ.2', 'ПГЛ.2', '{"3": "ПГЛ.2", "4": "германий-68", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-25', '6СО-25', '{"3": "6СО-25", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СР17.10', 'СР17.10', '{"3": "СР17.10", "4": "селен-75", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Калибровочн. источник', 'Калибровочн. источник', '{"3": "Калибровочн. источник", "4": "калий-40; цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BN3.C10.1.R', 'BN3.C10.1.R', '{"3": "BN3.C10.1.R", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Калибровочн. источни', 'Калибровочн. источни', '{"3": "Калибровочн. источни", "4": "калий-40; цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Сf252', 'Сf252', '{"3": "Сf252", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИДК - Р', 'ОИДК - Р', '{"3": "ОИДК - Р", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-804.08', '6СО-804.08', '{"3": "6СО-804.08", "4": "иттрий-90; стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Сs137', 'Сs137', '{"3": "Сs137", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНК-15', 'КНК-15', '{"3": "КНК-15", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТИС-5-01', 'ТИС-5-01', '{"3": "ТИС-5-01", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-5-4', 'ИГИ-Ц-5-4', '{"3": "ИГИ-Ц-5-4", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-300-2р', 'ОСГИ-300-2р', '{"3": "ОСГИ-300-2р", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'H541500', 'H541500', '{"3": "H541500", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-26', '6СО-26', '{"3": "6СО-26", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NCm4.15', 'NCm4.15', '{"3": "NCm4.15", "4": "кюрий-244", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Калибровочн. источник', 'Калибровочн. источник', '{"3": "Калибровочн. источник", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ1-40', 'ГНТ1-40', '{"3": "ГНТ1-40", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-805.08', '6СО-805.08', '{"3": "6СО-805.08", "4": "иттрий-90; стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ-23', 'КНТ-23', '{"3": "КНТ-23", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Кr-85', 'Кr-85', '{"3": "Кr-85", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СКАИ', 'СКАИ', '{"3": "СКАИ", "4": "Свинец-210", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'H541501', 'H541501', '{"3": "H541501", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-34', '6СО-34', '{"3": "6СО-34", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'A3808', 'A3808', '{"3": "A3808", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СР17.200', 'СР17.200', '{"3": "СР17.200", "4": "селен-75", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ж83-Р2114', 'Ж83-Р2114', '{"3": "Ж83-Р2114", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ1-40-02', 'ГНТ1-40-02', '{"3": "ГНТ1-40-02", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GBa 3.061.4', 'GBa 3.061.4', '{"3": "GBa 3.061.4", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'криптон-85', 'криптон-85', '{"3": "криптон-85", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ШЦ-4', 'ШЦ-4', '{"3": "ШЦ-4", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-35', '6СО-35', '{"3": "6СО-35", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СР17.90', 'СР17.90', '{"3": "СР17.90", "4": "селен-75", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОППУ-8', 'СОППУ-8', '{"3": "СОППУ-8", "4": "Уран-238, Уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3C3.4', 'BNi3C3.4', '{"3": "BNi3C3.4", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.132-90', 'ОСО 95.132-90', '{"3": "ОСО 95.132-90", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МП 0974', 'МП 0974', '{"3": "МП 0974", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2С0-135', '2С0-135', '{"3": "2С0-135", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IV', 'IV', '{"3": "IV", "4": "бериллий-10; плутоний-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ-24', 'КНТ-24', '{"3": "КНТ-24", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЛИ', 'РЛИ', '{"3": "РЛИ", "4": "углерод-14", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-36', '6СО-36', '{"3": "6СО-36", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОППУ-8', 'СОППУ-8', '{"3": "СОППУ-8", "4": "уран-238; уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ24', 'КНТ24', '{"3": "КНТ24", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.133-90', 'ОСО 95.133-90', '{"3": "ОСО 95.133-90", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МП 2864', 'МП 2864', '{"3": "МП 2864", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТГТН191', 'ТГТН191', '{"3": "ТГТН191", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IV', 'IV', '{"3": "IV", "4": "плутоний-238; бериллий-10", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Спецназ.', 'Спецназ.', '{"3": "Спецназ.", "4": "Висмут-207", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНУ-3', 'КНУ-3', '{"3": "КНУ-3", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-83', '6СО-83', '{"3": "6СО-83", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИРИК Д', 'ИРИК Д', '{"3": "ИРИК Д", "4": "Кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АРЕВ 28-3', 'АРЕВ 28-3', '{"3": "АРЕВ 28-3", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCs7.15', 'GCs7.15', '{"3": "GCs7.15", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.135-90', 'ОСО 95.135-90', '{"3": "ОСО 95.135-90", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МУ 10.389', 'МУ 10.389', '{"3": "МУ 10.389", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'радиометрический источник', 'радиометрический источник', '{"3": "радиометрический источник", "4": "таллий-204", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Стандартный образец обогащения урана', 'Стандартный образец обогащения урана', '{"3": "Стандартный образец обогащения урана", "4": "Уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-84', '6СО-84', '{"3": "6СО-84", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНУ-3-01', 'КНУ-3-01', '{"3": "КНУ-3-01", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Аппликатор', 'Аппликатор', '{"3": "Аппликатор", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'прим.', 'прим.', '{"3": "прим.", "4": "натрий-22", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОИРИ', 'СОИРИ', '{"3": "СОИРИ", "4": "висмут-207", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1АИ', '1АИ', '{"3": "1АИ", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.138-90', 'ОСО 95.138-90', '{"3": "ОСО 95.138-90", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'T-17', 'T-17', '{"3": "T-17", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МУ 3.117', 'МУ 3.117', '{"3": "МУ 3.117", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ 34 ВШЛК.418221.020', 'КНТ 34 ВШЛК.418221.020', '{"3": "КНТ 34 ВШЛК.418221.020", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМН-Г-3-Н (ОСГИ)', 'ИМН-Г-3-Н (ОСГИ)', '{"3": "ИМН-Г-3-Н (ОСГИ)", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1С0', '1С0', '{"3": "1С0", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СИГИ-С', 'СИГИ-С', '{"3": "СИГИ-С", "4": "европий-152", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-85', '6СО-85', '{"3": "6СО-85", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'прим.', 'прим.', '{"3": "прим.", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.141-90', 'ОСО 95.141-90', '{"3": "ОСО 95.141-90", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АМ1.Р08', 'АМ1.Р08', '{"3": "АМ1.Р08", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6С0-804', '6С0-804', '{"3": "6С0-804", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252МО1', 'НК252МО1', '{"3": "НК252МО1", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМН-Г-3-Н (ОСГИ)', 'ИМН-Г-3-Н (ОСГИ)', '{"3": "ИМН-Г-3-Н (ОСГИ)", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СИГИ-С', 'СИГИ-С', '{"3": "СИГИ-С", "4": "калий-40", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-86', '6СО-86', '{"3": "6СО-86", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М53', 'ГИ192М53', '{"3": "ГИ192М53", "4": "иридий-192", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'прим.', 'прим.', '{"3": "прим.", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IND1402', 'IND1402', '{"3": "IND1402", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-81', '1П9-81', '{"3": "1П9-81", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.142-90', 'ОСО 95.142-90', '{"3": "ОСО 95.142-90", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ 34-2 ВШЛК.418221.057', 'КНТ 34-2 ВШЛК.418221.057', '{"3": "КНТ 34-2 ВШЛК.418221.057", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИНИ-А', 'ОИНИ-А', '{"3": "ОИНИ-А", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМН-Г-3-Н (ОСГИ)', 'ИМН-Г-3-Н (ОСГИ)', '{"3": "ИМН-Г-3-Н (ОСГИ)", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2С0', '2С0', '{"3": "2С0", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СИГИ-С', 'СИГИ-С', '{"3": "СИГИ-С", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-87', '6СО-87', '{"3": "6СО-87", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М31.6', 'ГИ192М31.6', '{"3": "ГИ192М31.6", "4": "Иридий-192", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'прим.', 'прим.', '{"3": "прим.", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1П9-82', '1П9-82', '{"3": "1П9-82", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SRM4968', 'SRM4968', '{"3": "SRM4968", "4": "Радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.147-92', 'ОСО 95.147-92', '{"3": "ОСО 95.147-92", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6С0-805', '6С0-805', '{"3": "6С0-805", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Контрольный источник (Сосуд Маринелли)', 'Контрольный источник (Сосуд Маринелли)', '{"3": "Контрольный источник (Сосуд Маринелли)", "4": "калий-40; цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'микроисточники', 'микроисточники', '{"3": "микроисточники", "4": "иод-125", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MNTR-F 761', 'MNTR-F 761', '{"3": "MNTR-F 761", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СИГИ-С', 'СИГИ-С', '{"3": "СИГИ-С", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3С0', '3С0', '{"3": "3С0", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.152-92', 'ОСО 95.152-92', '{"3": "ОСО 95.152-92", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИ', 'ИБИ', '{"3": "ИБИ", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Контрольный источник (Сосуд Маринелли)', 'Контрольный источник (Сосуд Маринелли)', '{"3": "Контрольный источник (Сосуд Маринелли)", "4": "цезий-137; калий-40", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СИГИ-С', 'СИГИ-С', '{"3": "СИГИ-С", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4С0', '4С0', '{"3": "4С0", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-122', '1СО-122', '{"3": "1СО-122", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.156-90', 'ОСО 95.156-90', '{"3": "ОСО 95.156-90", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИ (источник бетта-излучения)', 'ИБИ (источник бетта-излучения)', '{"3": "ИБИ (источник бетта-излучения)", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6С0-137', '6С0-137', '{"3": "6С0-137", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СИГИ-П', 'СИГИ-П', '{"3": "СИГИ-П", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9 162', '2П9 162', '{"3": "2П9 162", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-125', '1СО-125', '{"3": "1СО-125", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.158-90', 'ОСО 95.158-90', '{"3": "ОСО 95.158-90", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-804.08', '1СО-804.08', '{"3": "1СО-804.08", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5С0', '5С0', '{"3": "5С0", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т48012-0236', 'Т48012-0236', '{"3": "Т48012-0236", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М34.7', 'ГИ192М34.7', '{"3": "ГИ192М34.7", "4": "иридий-192", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ5-40', 'ГНТ5-40', '{"3": "ГНТ5-40", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63 тип II', 'ИБИРЗН-63 тип II', '{"3": "ИБИРЗН-63 тип II", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.163-90', 'ОСО 95.163-90', '{"3": "ОСО 95.163-90", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6С0-327', '6С0-327', '{"3": "6С0-327", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-215.08', '3СО-215.08', '{"3": "3СО-215.08", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.1232', 'MCo7.1232', '{"3": "MCo7.1232", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "0.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6С0', '6С0', '{"3": "6С0", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'М37С02-13', 'М37С02-13', '{"3": "М37С02-13", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-23', '1СО-23', '{"3": "1СО-23", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.168-90', 'ОСО 95.168-90', '{"3": "ОСО 95.168-90", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-804.08', '3СО-804.08', '{"3": "3СО-804.08", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'радий-226', 'радий-226', '{"3": "радий-226", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.1234', 'MCo7.1234', '{"3": "MCo7.1234", "4": "кобальт-57", "5": "6", "6": "", "7": "", "8": "0.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDRB1548', 'CDRB1548', '{"3": "CDRB1548", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-25', '1СО-25', '{"3": "1СО-25", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.178-90', 'ОСО 95.178-90', '{"3": "ОСО 95.178-90", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-805.08', '3СО-805.08', '{"3": "3СО-805.08", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-126', '5П9-126', '{"3": "5П9-126", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Мишень тритид титана', 'Мишень тритид титана', '{"3": "Мишень тритид титана", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЭС', 'РЭС', '{"3": "РЭС", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCs7.13', 'GCs7.13', '{"3": "GCs7.13", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-31', '1СО-31', '{"3": "1СО-31", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.180-90', 'ОСО 95.180-90', '{"3": "ОСО 95.180-90", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-534.08', '4СО-534.08', '{"3": "4СО-534.08", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОИСУ- 0011', 'СОИСУ- 0011', '{"3": "СОИСУ- 0011", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИСН (ОСГИ)', 'ОИСН (ОСГИ)', '{"3": "ОИСН (ОСГИ)", "4": "цезий-137; калий-40", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М324.113', 'ГК60М324.113', '{"3": "ГК60М324.113", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Мишень тритид циркония', 'Мишень тритид циркония', '{"3": "Мишень тритид циркония", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН', 'ИБН', '{"3": "ИБН", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MMS05-022-10U', 'MMS05-022-10U', '{"3": "MMS05-022-10U", "4": "натрий-22", "5": "1", "6": "", "7": "", "8": "", "9": "0", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-32', '1СО-32', '{"3": "1СО-32", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ж83-Р919-02', 'Ж83-Р919-02', '{"3": "Ж83-Р919-02", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.182-90', 'ОСО 95.182-90', '{"3": "ОСО 95.182-90", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1С0-534', '1С0-534', '{"3": "1С0-534", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМН-Г-3Т', 'ИМН-Г-3Т', '{"3": "ИМН-Г-3Т", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-535.08', '4СО-535.08', '{"3": "4СО-535.08", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОИСУ- 0071', 'СОИСУ- 0071', '{"3": "СОИСУ- 0071", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IPPE Am241-Li', 'IPPE Am241-Li', '{"3": "IPPE Am241-Li", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-33', '1СО-33', '{"3": "1СО-33", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.183-90', 'ОСО 95.183-90', '{"3": "ОСО 95.183-90", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1С0-535', '1С0-535', '{"3": "1С0-535", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый источник 3-го разряда', 'Образцовый источник 3-го разряда', '{"3": "Образцовый источник 3-го разряда", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-536.08', '4СО-536.08', '{"3": "4СО-536.08", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РА23-34', 'РА23-34', '{"3": "РА23-34", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОИСУ- 0642', 'СОИСУ- 0642', '{"3": "СОИСУ- 0642", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'а-б-г источник', 'а-б-г источник', '{"3": "а-б-г источник", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТРИБ-90-29-45', 'ТРИБ-90-29-45', '{"3": "ТРИБ-90-29-45", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-32', '3СО-32', '{"3": "3СО-32", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-35', '1СО-35', '{"3": "1СО-35", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.186-90', 'ОСО 95.186-90', '{"3": "ОСО 95.186-90", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-214.08', '6СО-214.08', '{"3": "6СО-214.08", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОИСУ- 4928', 'СОИСУ- 4928', '{"3": "СОИСУ- 4928", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'P486724', 'P486724', '{"3": "P486724", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'образцы для исследований', 'образцы для исследований', '{"3": "образцы для исследований", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АHИCm244-3', 'АHИCm244-3', '{"3": "АHИCm244-3", "4": "кюрий-244", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-52', '1СО-52', '{"3": "1СО-52", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.195-90', 'ОСО 95.195-90', '{"3": "ОСО 95.195-90", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-804.08', '6СО-804.08', '{"3": "6СО-804.08", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'P585232', 'P585232', '{"3": "P585232", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОРР', 'ОРР', '{"3": "ОРР", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АHИCm244-7', 'АHИCm244-7', '{"3": "АHИCm244-7", "4": "кюрий-244", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-55', '1СО-55', '{"3": "1СО-55", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.198-90', 'ОСО 95.198-90', '{"3": "ОСО 95.198-90", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СПД', 'СПД', '{"3": "СПД", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-805.08', '6СО-805.08', '{"3": "6СО-805.08", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'P585244', 'P585244', '{"3": "P585244", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОРР', 'ОРР', '{"3": "ОРР", "4": "кюрий-244", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АГИBa133-150', 'АГИBa133-150', '{"3": "АГИBa133-150", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3У8-8', '3У8-8', '{"3": "3У8-8", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-8', '1СО-8', '{"3": "1СО-8", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'камера деления КНТ 30-5', 'камера деления КНТ 30-5', '{"3": "камера деления КНТ 30-5", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.200-90', 'ОСО 95.200-90', '{"3": "ОСО 95.200-90", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СP17.10', 'СP17.10', '{"3": "СP17.10", "4": "cелен-75", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц-1-5', 'Ц-1-5', '{"3": "Ц-1-5", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОРР', 'ОРР', '{"3": "ОРР", "4": "плутоний-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.Y997', 'AMN.Y997', '{"3": "AMN.Y997", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АГИBa133-25', 'АГИBa133-25', '{"3": "АГИBa133-25", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ-27-05', 'КНТ-27-05', '{"3": "КНТ-27-05", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-81', '1СО-81', '{"3": "1СО-81", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.205-90', 'ОСО 95.205-90', '{"3": "ОСО 95.205-90", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОЗРИ', 'ОЗРИ', '{"3": "ОЗРИ", "4": "иод-125", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СP17.10', 'СP17.10', '{"3": "СP17.10", "4": "Селен-75", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц-1-6', 'Ц-1-6', '{"3": "Ц-1-6", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОРР', 'ОРР', '{"3": "ОРР", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АГИBa133-350', 'АГИBa133-350', '{"3": "АГИBa133-350", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-82', '1СО-82', '{"3": "1СО-82", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.208-90', 'ОСО 95.208-90', '{"3": "ОСО 95.208-90", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц-1-7', 'Ц-1-7', '{"3": "Ц-1-7", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C1.2.R', 'BNi3.C1.2.R', '{"3": "BNi3.C1.2.R", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-125', '6СО-125', '{"3": "6СО-125", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'А 3916-1', 'А 3916-1', '{"3": "А 3916-1", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.212-90', 'ОСО 95.212-90', '{"3": "ОСО 95.212-90", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4С0-137', '4С0-137', '{"3": "4С0-137", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОСГИ-М', 'СОСГИ-М', '{"3": "СОСГИ-М", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц-2-5', 'Ц-2-5', '{"3": "Ц-2-5", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95 148-90', 'ОСО 95 148-90', '{"3": "ОСО 95 148-90", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.S3.4.R', 'BNi3.S3.4.R', '{"3": "BNi3.S3.4.R", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-21', '2П9-21', '{"3": "2П9-21", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НТ-32М', 'НТ-32М', '{"3": "НТ-32М", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.214-90', 'ОСО 95.214-90', '{"3": "ОСО 95.214-90", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.CY3', 'AMN.CY3', '{"3": "AMN.CY3", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'А3906', 'А3906', '{"3": "А3906", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц-2-6', 'Ц-2-6', '{"3": "Ц-2-6", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-84', '4СО-84', '{"3": "4СО-84", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95 148-90', 'ОСО 95 148-90', '{"3": "ОСО 95 148-90", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН (Сосуд Маринелли)', 'ОМАСН (Сосуд Маринелли)', '{"3": "ОМАСН (Сосуд Маринелли)", "4": "америций-241; барий-133; европий-152; кобальт-60; цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-22', '2П9-22', '{"3": "2П9-22", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.227-92', 'ОСО 95.227-92', '{"3": "ОСО 95.227-92", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6С0-216', '6С0-216', '{"3": "6С0-216", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СP17.90', 'СP17.90', '{"3": "СP17.90", "4": "Cелен-75", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц-2-7', 'Ц-2-7', '{"3": "Ц-2-7", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95 889-04', 'ОСО 95 889-04', '{"3": "ОСО 95 889-04", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-23', '2П9-23', '{"3": "2П9-23", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95.531-98', 'ОСО 95.531-98', '{"3": "ОСО 95.531-98", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6С0-217', '6С0-217', '{"3": "6С0-217", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'радионуклидный источник гамма-излучения', 'радионуклидный источник гамма-излучения', '{"3": "радионуклидный источник гамма-излучения", "4": "калий-40; цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'У-754Ф', 'У-754Ф', '{"3": "У-754Ф", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц-2-8', 'Ц-2-8', '{"3": "Ц-2-8", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95 890-04', 'ОСО 95 890-04', '{"3": "ОСО 95 890-04", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCs7.26', 'GCs7.26', '{"3": "GCs7.26", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Эталонный образец', 'Эталонный образец', '{"3": "Эталонный образец", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'альфа-источник', 'альфа-источник', '{"3": "альфа-источник", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИЦ', 'ГИЦ', '{"3": "ГИЦ", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-24', '2П9-24', '{"3": "2П9-24", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц-3-4', 'Ц-3-4', '{"3": "Ц-3-4", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95 891-04', 'ОСО 95 891-04', '{"3": "ОСО 95 891-04", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GRa 6.1', 'GRa 6.1', '{"3": "GRa 6.1", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Эталонный образец', 'Эталонный образец', '{"3": "Эталонный образец", "4": "углерод-14", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ1-143 (н/трубка)', 'ГНТ1-143 (н/трубка)', '{"3": "ГНТ1-143 (н/трубка)", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ECD -2010', 'ECD -2010', '{"3": "ECD -2010", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-25', '2П9-25', '{"3": "2П9-25", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПГЛ3', 'ПГЛ3', '{"3": "ПГЛ3", "4": "титан-44", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3С0-535', '3С0-535', '{"3": "3С0-535", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСS7.012.6', 'ГСS7.012.6', '{"3": "ГСS7.012.6", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц-4-5', 'Ц-4-5', '{"3": "Ц-4-5", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ECD -G2397-65506', 'ECD -G2397-65506', '{"3": "ECD -G2397-65506", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95 892-04', 'ОСО 95 892-04', '{"3": "ОСО 95 892-04", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ1-143а (н/трубка)', 'ГНТ1-143а (н/трубка)', '{"3": "ГНТ1-143а (н/трубка)", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-8', '2П9-8', '{"3": "2П9-8", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2С0-804', '2С0-804', '{"3": "2С0-804", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСS7.024.1', 'ГСS7.024.1', '{"3": "ГСS7.024.1", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц-4-7', 'Ц-4-7', '{"3": "Ц-4-7", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М41', 'НК252М41', '{"3": "НК252М41", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95 893-04', 'ОСО 95 893-04', '{"3": "ОСО 95 893-04", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'A3916-3', 'A3916-3', '{"3": "A3916-3", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Эталонный источник', 'Эталонный источник', '{"3": "Эталонный источник", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ1-80 (н/трубка)', 'ГНТ1-80 (н/трубка)', '{"3": "ГНТ1-80 (н/трубка)", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-81', '2П9-81', '{"3": "2П9-81", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2С0-325', '2С0-325', '{"3": "2С0-325", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМН-Г-2', 'ИМН-Г-2', '{"3": "ИМН-Г-2", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц-5-4', 'Ц-5-4', '{"3": "Ц-5-4", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95 894', 'ОСО 95 894', '{"3": "ОСО 95 894", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ2-42 (н/трубка)', 'ГНТ2-42 (н/трубка)', '{"3": "ГНТ2-42 (н/трубка)", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-82', '2П9-82', '{"3": "2П9-82", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IR-192 GAMMAMED PLUS HDR 0.9MM', 'IR-192 GAMMAMED PLUS HDR 0.9MM', '{"3": "IR-192 GAMMAMED PLUS HDR 0.9MM", "4": "иридий-192", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц-6-5', 'Ц-6-5', '{"3": "Ц-6-5", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'А424-14', 'А424-14', '{"3": "А424-14", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МК-04', 'МК-04', '{"3": "МК-04", "4": "калий-40", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ2-42М (н/трубка)', 'ГНТ2-42М (н/трубка)', '{"3": "ГНТ2-42М (н/трубка)", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6CO 215', '6CO 215', '{"3": "6CO 215", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-83', '2П9-83', '{"3": "2П9-83", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГРа-6', 'ГРа-6', '{"3": "ГРа-6", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Набор №2брс-о', 'Набор №2брс-о', '{"3": "Набор №2брс-о", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KACB10884', 'KACB10884', '{"3": "KACB10884", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МК-04', 'МК-04', '{"3": "МК-04", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС', 'ГС', '{"3": "ГС", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6CO 534', '6CO 534', '{"3": "6CO 534", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NLF 0124-003', 'NLF 0124-003', '{"3": "NLF 0124-003", "4": "иридий-192", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2П9-84', '2П9-84', '{"3": "2П9-84", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСОО.061.2', 'ГСОО.061.2', '{"3": "ГСОО.061.2", "4": "Кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'закрытая мишень', 'закрытая мишень', '{"3": "закрытая мишень", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Примечание', 'Примечание', '{"3": "Примечание", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М54', 'ГИ192М54', '{"3": "ГИ192М54", "4": "иридий-192", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KUTh Gamma Calibrator', 'KUTh Gamma Calibrator', '{"3": "KUTh Gamma Calibrator", "4": "джн (смесь долгоживущих нуклидов)*", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МК-04', 'МК-04', '{"3": "МК-04", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО', 'ГСО', '{"3": "ГСО", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-122', '2СО-122', '{"3": "2СО-122", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63 2Б1547', 'ИБИРЗН-63 2Б1547', '{"3": "ИБИРЗН-63 2Б1547", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'нетиповой источник', 'нетиповой источник', '{"3": "нетиповой источник", "4": "висмут-207", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МК-04', 'МК-04', '{"3": "МК-04", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО 7115-94', 'ГСО 7115-94', '{"3": "ГСО 7115-94", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-125', '2СО-125', '{"3": "2СО-125", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РП8-26-10', 'РП8-26-10', '{"3": "РП8-26-10", "4": "плутоний-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'нетиповой источник', 'нетиповой источник', '{"3": "нетиповой источник", "4": "гафний-181", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'в виде NpAl2', 'в виде NpAl2', '{"3": "в виде NpAl2", "4": "нептуний-237", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО 8363-2003', 'ГСО 8363-2003', '{"3": "ГСО 8363-2003", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-127', '2СО-127', '{"3": "2СО-127", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МТ-СНЕГ', 'МТ-СНЕГ', '{"3": "МТ-СНЕГ", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РП9-26-10', 'РП9-26-10', '{"3": "РП9-26-10", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'нетиповой источник', 'нетиповой источник', '{"3": "нетиповой источник", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'гамма-источник', 'гамма-источник', '{"3": "гамма-источник", "4": "алюминий-26", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСОСТЕРЭ-Т2', 'ГСОСТЕРЭ-Т2', '{"3": "ГСОСТЕРЭ-Т2", "4": "торий ествественный", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМН-Г-3-Т(ОИСН)', 'ИМН-Г-3-Т(ОИСН)', '{"3": "ИМН-Г-3-Т(ОИСН)", "4": "барий-133; кобальт-60; цезий-137; европий-152", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'нетиповой источник', 'нетиповой источник', '{"3": "нетиповой источник", "4": "плутоний-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'гамма-источник', 'гамма-источник', '{"3": "гамма-источник", "4": "висмут-207", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСОСТЕРЭ-К3', 'ГСОСТЕРЭ-К3', '{"3": "ГСОСТЕРЭ-К3", "4": "калий-40", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНК15', 'КНК15', '{"3": "КНК15", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СО-2', 'СО-2', '{"3": "СО-2", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'гамма-источник', 'гамма-источник', '{"3": "гамма-источник", "4": "гафний-181", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-23', '2СО-23', '{"3": "2СО-23", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СО-20-1', 'СО-20-1', '{"3": "СО-20-1", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'гамма-источник', 'гамма-источник', '{"3": "гамма-источник", "4": "европий-152", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СО-20-2', 'СО-20-2', '{"3": "СО-20-2", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'гамма-источник', 'гамма-источник', '{"3": "гамма-источник", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GS 192Ir 112SS', 'GS 192Ir 112SS', '{"3": "GS 192Ir 112SS", "4": "иридий-192", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-25', '2СО-25', '{"3": "2СО-25", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСИАИ (набор)', 'ОСИАИ (набор)', '{"3": "ОСИАИ (набор)", "4": "америций-241+европий-152+торий-228", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СО-20-3', 'СО-20-3', '{"3": "СО-20-3", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'гамма-источник', 'гамма-источник', '{"3": "гамма-источник", "4": "радий-226; бериллий-7", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЖОСГИ', 'ЖОСГИ', '{"3": "ЖОСГИ", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'HCF2.021.2', 'HCF2.021.2', '{"3": "HCF2.021.2", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-26', '2СО-26', '{"3": "2СО-26", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСИАИ (набор)', 'ОСИАИ (набор)', '{"3": "ОСИАИ (набор)", "4": "плутоний-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CO U', 'CO U', '{"3": "CO U", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СО-3', 'СО-3', '{"3": "СО-3", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'гамма-источник', 'гамма-источник', '{"3": "гамма-источник", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЖОСГИ', 'ЖОСГИ', '{"3": "ЖОСГИ", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-31', '2СО-31', '{"3": "2СО-31", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСИАИ (набор)', 'ОСИАИ (набор)', '{"3": "ОСИАИ (набор)", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'гексабромотехнетат р-р3-K2TcBr6', 'гексабромотехнетат р-р3-K2TcBr6', '{"3": "гексабромотехнетат р-р3-K2TcBr6", "4": "технеций-99", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЖОСГИ', 'ЖОСГИ', '{"3": "ЖОСГИ", "4": "европий-152", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-34', '2СО-34', '{"3": "2СО-34", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСИАИ (набор)', 'ОСИАИ (набор)', '{"3": "ОСИАИ (набор)", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КАСК5533', 'КАСК5533', '{"3": "КАСК5533", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЖОСГИ', 'ЖОСГИ', '{"3": "ЖОСГИ", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-52', '2СО-52', '{"3": "2СО-52", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСИАИ (набор)', 'ОСИАИ (набор)', '{"3": "ОСИАИ (набор)", "4": "уран-233; плутоний-239; плутоний-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СО-6-3', 'СО-6-3', '{"3": "СО-6-3", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'двуокись', 'двуокись', '{"3": "двуокись", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'В геометрии фильтра АФА-РСП-20', 'В геометрии фильтра АФА-РСП-20', '{"3": "В геометрии фильтра АФА-РСП-20", "4": "иттрий-90; стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-56', '2СО-56', '{"3": "2СО-56", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПФ2-120-04', 'ПФ2-120-04', '{"3": "ПФ2-120-04", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СЭМ-А', 'СЭМ-А', '{"3": "СЭМ-А", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Контрольный источник 74020 QB', 'Контрольный источник 74020 QB', '{"3": "Контрольный источник 74020 QB", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'двуокись', 'двуокись', '{"3": "двуокись", "4": "кюрий-244", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'тип IV', 'тип IV', '{"3": "тип IV", "4": "плутоний-238; бериллий-10", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Контрольный источник 767', 'Контрольный источник 767', '{"3": "Контрольный источник 767", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'двуокись', 'двуокись', '{"3": "двуокись", "4": "нептуний-237", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КИ (a,b)', 'КИ (a,b)', '{"3": "КИ (a,b)", "4": "уран-238; уран-234; торий-234; протактиний-234", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'двуокись на Al подл.', 'двуокись на Al подл.', '{"3": "двуокись на Al подл.", "4": "нептуний-237", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-83', '2СО-83', '{"3": "2СО-83", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ДКH', 'ДКH', '{"3": "ДКH", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцы для иследований', 'Образцы для иследований', '{"3": "Образцы для иследований", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т19-Л209', 'Т19-Л209', '{"3": "Т19-Л209", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'образцовый гамма-источник III разряда', 'образцовый гамма-источник III разряда', '{"3": "образцовый гамма-источник III разряда", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'для уст.МРХ-100', 'для уст.МРХ-100', '{"3": "для уст.МРХ-100", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Комбинированный контрольный источник', 'Комбинированный контрольный источник', '{"3": "Комбинированный контрольный источник", "4": "протактиний-234; торий-234; уран-234; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцы для иследований', 'Образцы для иследований', '{"3": "Образцы для иследований", "4": "кюрий-242", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2СО-85', '2СО-85', '{"3": "2СО-85", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Изокорд I25.S061', 'Изокорд I25.S061', '{"3": "Изокорд I25.S061", "4": "иод-125", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-Ам1-1р', 'ОСГИ-3-2-Ам1-1р', '{"3": "ОСГИ-3-2-Ам1-1р", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцы для иследований', 'Образцы для иследований', '{"3": "Образцы для иследований", "4": "кюрий-244", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т19-Л214', 'Т19-Л214', '{"3": "Т19-Л214", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5109.II2', '5109.II2', '{"3": "5109.II2", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-Цз7-1р', 'ОСГИ-3-2-Цз7-1р', '{"3": "ОСГИ-3-2-Цз7-1р", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'золотая фольга', 'золотая фольга', '{"3": "золотая фольга", "4": "кобальт-60; золото-194", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИСН-238', 'ОИСН-238', '{"3": "ОИСН-238", "4": "уран-238; уран-234; уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцы для иследований', 'Образцы для иследований', '{"3": "Образцы для иследований", "4": "плутоний-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т19-Л214-01', 'Т19-Л214-01', '{"3": "Т19-Л214-01", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-166', '3П9-166', '{"3": "3П9-166", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МСо7.123', 'МСо7.123', '{"3": "МСо7.123", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5109II2', '5109II2', '{"3": "5109II2", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИHК-1', 'ИHК-1', '{"3": "ИHК-1", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцы для иследований', 'Образцы для иследований', '{"3": "Образцы для иследований", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СDC.CY11', 'СDC.CY11', '{"3": "СDC.CY11", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5С0-134', '5С0-134', '{"3": "5С0-134", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9 126', '5П9 126', '{"3": "5П9 126", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИHК-3', 'ИHК-3', '{"3": "ИHК-3", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИСН-242', 'ОИСН-242', '{"3": "ОИСН-242", "4": "плутоний-238; плутоний-239; плутоний-242", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцы для иследований', 'Образцы для иследований', '{"3": "Образцы для иследований", "4": "плутоний-242", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-22', '3П9-22', '{"3": "3П9-22", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТГНТ194', 'ТГНТ194', '{"3": "ТГНТ194", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BH3.731', 'BH3.731', '{"3": "BH3.731", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5С0-135', '5С0-135', '{"3": "5С0-135", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GСо7.115', 'GСо7.115', '{"3": "GСо7.115", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМН-Г-3Н', 'ИМН-Г-3Н', '{"3": "ИМН-Г-3Н", "4": "торий-232; цезий-137", "5": "1", "6": "", "7": "0.0", "8": "3000.0", "9": "0", "10": "ФГУП \"ВНИИФТРИ\"", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцы для иследований', 'Образцы для иследований', '{"3": "Образцы для иследований", "4": "радий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-23', '3П9-23', '{"3": "3П9-23", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТЖИУ.433422.004-01', 'ТЖИУ.433422.004-01', '{"3": "ТЖИУ.433422.004-01", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KTPM-Ir-40', 'KTPM-Ir-40', '{"3": "KTPM-Ir-40", "4": "иридий-192", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИHК-5', 'ИHК-5', '{"3": "ИHК-5", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцы для иследований', 'Образцы для иследований', '{"3": "Образцы для иследований", "4": "радий-223", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-24', '3П9-24', '{"3": "3П9-24", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТЖИУ.433422.004-02', 'ТЖИУ.433422.004-02', '{"3": "ТЖИУ.433422.004-02", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИHК-7', 'ИHК-7', '{"3": "ИHК-7", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцы для иследований', 'Образцы для иследований', '{"3": "Образцы для иследований", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-25', '3П9-25', '{"3": "3П9-25", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТЖИУ.433422.004-04', 'ТЖИУ.433422.004-04', '{"3": "ТЖИУ.433422.004-04", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцы для иследований', 'Образцы для иследований', '{"3": "Образцы для иследований", "4": "торий-228", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-55', '3П9-55', '{"3": "3П9-55", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТЖИУ.433422.004-05', 'ТЖИУ.433422.004-05', '{"3": "ТЖИУ.433422.004-05", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МК 57 ВР', 'МК 57 ВР', '{"3": "МК 57 ВР", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцы для иследований', 'Образцы для иследований', '{"3": "Образцы для иследований", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-81', '3П9-81', '{"3": "3П9-81", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СО (241Am02-Li)', 'СО (241Am02-Li)', '{"3": "СО (241Am02-Li)", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'опытный источник альфа-излучения', 'опытный источник альфа-излучения', '{"3": "опытный источник альфа-излучения", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BN13.C3.4', 'BN13.C3.4', '{"3": "BN13.C3.4", "4": "Никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-82', '3П9-82', '{"3": "3П9-82", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТЖИУ.433442.004-03', 'ТЖИУ.433442.004-03', '{"3": "ТЖИУ.433442.004-03", "4": "тритий, уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-83', '3П9-83', '{"3": "3П9-83", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТЖИУ.433442.004-03', 'ТЖИУ.433442.004-03', '{"3": "ТЖИУ.433442.004-03", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi.3C3.4', 'BNi.3C3.4', '{"3": "BNi.3C3.4", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C3.4R', 'BNi3.C3.4R', '{"3": "BNi3.C3.4R", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3П9-84', '3П9-84', '{"3": "3П9-84", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'А 3201-4', 'А 3201-4', '{"3": "А 3201-4", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-122', '3СО-122', '{"3": "3СО-122", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОП УА', 'СОП УА', '{"3": "СОП УА", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-123', '3СО-123', '{"3": "3СО-123", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый ДГ-2', 'Образцовый ДГ-2', '{"3": "Образцовый ДГ-2", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСИАИ-2', 'ОСИАИ-2', '{"3": "ОСИАИ-2", "4": "плутоний-238; уран-233", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ITУ-02-2-919-87', 'ITУ-02-2-919-87', '{"3": "ITУ-02-2-919-87", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.CI.3', 'BNi3.CI.3', '{"3": "BNi3.CI.3", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-124', '3СО-124', '{"3": "3СО-124", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'радиометрич.', 'радиометрич.', '{"3": "радиометрич.", "4": "иттрий-90; стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСИАИ-2', 'ОСИАИ-2', '{"3": "ОСИАИ-2", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОП УМ', 'СОП УМ', '{"3": "СОП УМ", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Пб', 'Пб', '{"3": "Пб", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-125', '3СО-125', '{"3": "3СО-125", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т19-Л282', 'Т19-Л282', '{"3": "Т19-Л282", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '74020', '74020', '{"3": "74020", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NCm244.5.14', 'NCm244.5.14', '{"3": "NCm244.5.14", "4": "кюрий-244", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '7', '7', '{"3": "7", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-126', '3СО-126', '{"3": "3СО-126", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОП УН', 'СОП УН', '{"3": "СОП УН", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN11', 'AMN11', '{"3": "AMN11", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-127', '3СО-127', '{"3": "3СО-127", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN15', 'AMN15', '{"3": "AMN15", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-22', '3СО-22', '{"3": "3СО-22", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОППУ', 'СОППУ', '{"3": "СОППУ", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN16', 'AMN16', '{"3": "AMN16", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95 1307-2011', 'ОСО 95 1307-2011', '{"3": "ОСО 95 1307-2011", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-23', '3СО-23', '{"3": "3СО-23", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN17', 'AMN17', '{"3": "AMN17", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95 1308-2011', 'ОСО 95 1308-2011', '{"3": "ОСО 95 1308-2011", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-24', '3СО-24', '{"3": "3СО-24", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.V997', 'AMN.V997', '{"3": "AMN.V997", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BH3.4', 'BH3.4', '{"3": "BH3.4", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN18', 'AMN18', '{"3": "AMN18", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-25', '3СО-25', '{"3": "3СО-25", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN19', 'AMN19', '{"3": "AMN19", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-26', '3СО-26', '{"3": "3СО-26", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BH3.531', 'BH3.531', '{"3": "BH3.531", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN23', 'AMN23', '{"3": "AMN23", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95 674-2015П', 'ОСО 95 674-2015П', '{"3": "ОСО 95 674-2015П", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-31', '3СО-31', '{"3": "3СО-31", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KUTh Field Verifie', 'KUTh Field Verifie', '{"3": "KUTh Field Verifie", "4": "сумма альфа-активных естеств. нуклидов", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN24', 'AMN24', '{"3": "AMN24", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РБМ-К7.Сб.262-1.4', 'РБМ-К7.Сб.262-1.4', '{"3": "РБМ-К7.Сб.262-1.4", "4": "иод-125", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNK7062', 'AMNK7062', '{"3": "AMNK7062", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-33', '3СО-33', '{"3": "3СО-33", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KUTh Field Verifie', 'KUTh Field Verifie', '{"3": "KUTh Field Verifie", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNK7190', 'AMNK7190', '{"3": "AMNK7190", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-34', '3СО-34', '{"3": "3СО-34", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МК57ВР', 'МК57ВР', '{"3": "МК57ВР", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNK7318 (Z726)', 'AMNK7318 (Z726)', '{"3": "AMNK7318 (Z726)", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-35', '3СО-35', '{"3": "3СО-35", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi.C3.4R', 'BNi.C3.4R', '{"3": "BNi.C3.4R", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НK-252M11.27', 'НK-252M11.27', '{"3": "НK-252M11.27", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNK7318 (Z743)', 'AMNK7318 (Z743)', '{"3": "AMNK7318 (Z743)", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КИ101/06', 'КИ101/06', '{"3": "КИ101/06", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-36', '3СО-36', '{"3": "3СО-36", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМН-Г-3-Н', 'ИМН-Г-3-Н', '{"3": "ИМН-Г-3-Н", "4": "америций-241; европий-152; цезий-137; барий-133", "5": "1", "6": "", "7": "0.0", "8": "11000.0", "9": "0", "10": "ФГУП \"ВНИИФТРИ\"", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNK7319 (Z730)', 'AMNK7319 (Z730)', '{"3": "AMNK7319 (Z730)", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-51', '3СО-51', '{"3": "3СО-51", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КИ101/50', 'КИ101/50', '{"3": "КИ101/50", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNK7446 (Z729)', 'AMNK7446 (Z729)', '{"3": "AMNK7446 (Z729)", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-52', '3СО-52', '{"3": "3СО-52", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNK7447 (Z725)', 'AMNK7447 (Z725)', '{"3": "AMNK7447 (Z725)", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РА-4-10', 'РА-4-10', '{"3": "РА-4-10", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-Ко0-1р', 'ОСГИ-3-2-Ко0-1р', '{"3": "ОСГИ-3-2-Ко0-1р", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-53', '3СО-53', '{"3": "3СО-53", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ECD-2010', 'ECD-2010', '{"3": "ECD-2010", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNQ4578', 'AMNQ4578', '{"3": "AMNQ4578", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РА-5-10', 'РА-5-10', '{"3": "РА-5-10", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'А3201-3', 'А3201-3', '{"3": "А3201-3", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-537', '3СО-537', '{"3": "3СО-537", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Elekta 43685', 'Elekta 43685', '{"3": "Elekta 43685", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNQ6031', 'AMNQ6031', '{"3": "AMNQ6031", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РА1-63-10', 'РА1-63-10', '{"3": "РА1-63-10", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'А3201', 'А3201', '{"3": "А3201", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-54', '3СО-54', '{"3": "3СО-54", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNQ9337', 'AMNQ9337', '{"3": "AMNQ9337", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РА1-65-10', 'РА1-65-10', '{"3": "РА1-65-10", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IND - 1402', 'IND - 1402', '{"3": "IND - 1402", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-55', '3СО-55', '{"3": "3СО-55", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ147-03', 'ТНТ147-03', '{"3": "ТНТ147-03", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNQ9927', 'AMNQ9927', '{"3": "AMNQ9927", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-56', '3СО-56', '{"3": "3СО-56", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГН', 'ИГН', '{"3": "ИГН", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNV340', 'AMNV340', '{"3": "AMNV340", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-81', '3СО-81', '{"3": "3СО-81", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'из набора НГСО4', 'из набора НГСО4', '{"3": "из набора НГСО4", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr0.83', 'BSr0.83', '{"3": "BSr0.83", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMNV997', 'AMNV997', '{"3": "AMNV997", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РТ', 'РТ', '{"3": "РТ", "4": "плутоний-239; плутоний-240", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-82', '3СО-82', '{"3": "3СО-82", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Field Verifie Calibrator', 'Field Verifie Calibrator', '{"3": "Field Verifie Calibrator", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'G1223A', 'G1223A', '{"3": "G1223A", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'C66544', 'C66544', '{"3": "C66544", "4": "кадмий-109", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'из набора НГСО4', 'из набора НГСО4', '{"3": "из набора НГСО4", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СО-1', 'СО-1', '{"3": "СО-1", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-83', '3СО-83', '{"3": "3СО-83", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'G2397A', 'G2397A', '{"3": "G2397A", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Cs-137', 'Cs-137', '{"3": "Cs-137", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-84', '3СО-84', '{"3": "3СО-84", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВНТ4-32-01', 'ВНТ4-32-01', '{"3": "ВНТ4-32-01", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-85', '3СО-85', '{"3": "3СО-85", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВНТ6-26', 'ВНТ6-26', '{"3": "ВНТ6-26", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'C3100', 'C3100', '{"3": "C3100", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3СО-86', '3СО-86', '{"3": "3СО-86", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПФ2-120-01', 'ПФ2-120-01', '{"3": "ПФ2-120-01", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'C3100', 'C3100', '{"3": "C3100", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-134', '6П9-134', '{"3": "6П9-134", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПФ2-120-03', 'ПФ2-120-03', '{"3": "ПФ2-120-03", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'из набора ОСГИ', 'из набора ОСГИ', '{"3": "из набора ОСГИ", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'C3100 вн.кап.', 'C3100 вн.кап.', '{"3": "C3100 вн.кап.", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'из набора ОСГИ', 'из набора ОСГИ', '{"3": "из набора ОСГИ", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СО-6-1', 'СО-6-1', '{"3": "СО-6-1", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'из набора ОСГИ', 'из набора ОСГИ', '{"3": "из набора ОСГИ", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GBa3.70', 'GBa3.70', '{"3": "GBa3.70", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СО-6-2', 'СО-6-2', '{"3": "СО-6-2", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Gba3.11', 'Gba3.11', '{"3": "Gba3.11", "4": "Барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'индикаторный', 'индикаторный', '{"3": "индикаторный", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo0.70', 'GCo0.70', '{"3": "GCo0.70", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'индикаторный', 'индикаторный', '{"3": "индикаторный", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцы для иследований', 'Образцы для иследований', '{"3": "Образцы для иследований", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-22', '4П9-22', '{"3": "4П9-22", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo0.x', 'GCo0.x', '{"3": "GCo0.x", "4": "Кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КHК-15-1', 'КHК-15-1', '{"3": "КHК-15-1", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцы для иследований', 'Образцы для иследований', '{"3": "Образцы для иследований", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-23', '4П9-23', '{"3": "4П9-23", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КHТ-2', 'КHТ-2', '{"3": "КHТ-2", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCs7.453', 'GCs7.453', '{"3": "GCs7.453", "4": "Цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SHLG-1', 'SHLG-1', '{"3": "SHLG-1", "4": "олово-119м", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-24', '4П9-24', '{"3": "4П9-24", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КHТ-31', 'КHТ-31', '{"3": "КHТ-31", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-25', '4П9-25', '{"3": "4П9-25", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCs7.441', 'GCs7.441', '{"3": "GCs7.441", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КHТ-5', 'КHТ-5', '{"3": "КHТ-5", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCs7.70', 'GCs7.70', '{"3": "GCs7.70", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-26', '4П9-26', '{"3": "4П9-26", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 3801', 'CDC 3801', '{"3": "CDC 3801", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КHТ-54-1', 'КHТ-54-1', '{"3": "КHТ-54-1", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КHТ-54-2', 'КHТ-54-2', '{"3": "КHТ-54-2", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCs7.461', 'GCs7.461', '{"3": "GCs7.461", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КHТ-6', 'КHТ-6', '{"3": "КHТ-6", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 38070', 'CDC 38070', '{"3": "CDC 38070", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCs7.462', 'GCs7.462', '{"3": "GCs7.462", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КHТ-6М', 'КHТ-6М', '{"3": "КHТ-6М", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КHТ-8', 'КHТ-8', '{"3": "КHТ-8", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'HCf.PO1.4005', 'HCf.PO1.4005', '{"3": "HCf.PO1.4005", "4": "Калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'HCf2.P01', 'HCf2.P01', '{"3": "HCf2.P01", "4": "Калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GF-0227', 'GF-0227', '{"3": "GF-0227", "4": "натрий-22", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'кабаниковая проволока', 'кабаниковая проволока', '{"3": "кабаниковая проволока", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КГИCs137-22', 'КГИCs137-22', '{"3": "КГИCs137-22", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО ИСУ', 'ОСО ИСУ', '{"3": "ОСО ИСУ", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'контейнер', 'контейнер', '{"3": "контейнер", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-81', '4П9-81', '{"3": "4П9-81", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО ИСУ', 'ОСО ИСУ', '{"3": "ОСО ИСУ", "4": "уран-235; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'концентрат иония', 'концентрат иония', '{"3": "концентрат иония", "4": "торий-230", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-82', '4П9-82', '{"3": "4П9-82", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО ИСУ', 'ОСО ИСУ', '{"3": "ОСО ИСУ", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GU5.R-1', 'GU5.R-1', '{"3": "GU5.R-1", "4": "уран-235; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-83', '4П9-83', '{"3": "4П9-83", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GU5.R-2', 'GU5.R-2', '{"3": "GU5.R-2", "4": "уран-235; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-84', '4П9-84', '{"3": "4П9-84", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NCf2.82', 'NCf2.82', '{"3": "NCf2.82", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GU5.R-3', 'GU5.R-3', '{"3": "GU5.R-3", "4": "уран-235; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4П9-85', '4П9-85', '{"3": "4П9-85", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC3821', 'CDC3821', '{"3": "CDC3821", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-125', '4СО-125', '{"3": "4СО-125", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСК', 'ОСК', '{"3": "ОСК", "4": "свинец-210, полоний-210", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC3825', 'CDC3825', '{"3": "CDC3825", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-127', '4СО-127', '{"3": "4СО-127", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'металл', 'металл', '{"3": "металл", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC801', 'CDC801', '{"3": "CDC801", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-23', '4СО-23', '{"3": "4СО-23", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ni-63', 'Ni-63', '{"3": "Ni-63", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'металл', 'металл', '{"3": "металл", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC809', 'CDC809', '{"3": "CDC809", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-26', '4СО-26', '{"3": "4СО-26", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'металл', 'металл', '{"3": "металл", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC90', 'CDC90', '{"3": "CDC90", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-27', '4СО-27', '{"3": "4СО-27", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'металлическая фольга', 'металлическая фольга', '{"3": "металлическая фольга", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC92', 'CDC92', '{"3": "CDC92", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-35', '4СО-35', '{"3": "4СО-35", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNI63.C3.4', 'BNI63.C3.4', '{"3": "BNI63.C3.4", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'металлическая фольга', 'металлическая фольга', '{"3": "металлическая фольга", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'HEGL-0132', 'HEGL-0132', '{"3": "HEGL-0132", "4": "германий-68", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCB1936', 'CDCB1936', '{"3": "CDCB1936", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-52', '4СО-52', '{"3": "4СО-52", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'HEGL-0136', 'HEGL-0136', '{"3": "HEGL-0136", "4": "натрий-22", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'металлическая фольга', 'металлическая фольга', '{"3": "металлическая фольга", "4": "уран-236", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SIFB12173', 'SIFB12173', '{"3": "SIFB12173", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-54', '4СО-54', '{"3": "4СО-54", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SSD607', 'SSD607', '{"3": "SSD607", "4": "самарий-151", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'HEGL-0139', 'HEGL-0139', '{"3": "HEGL-0139", "4": "Ge-68", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'металлическая фольга', 'металлическая фольга', '{"3": "металлическая фольга", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDCK7788', 'CDCK7788', '{"3": "CDCK7788", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-56', '4СО-56', '{"3": "4СО-56", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'TG-4 S11', 'TG-4 S11', '{"3": "TG-4 S11", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'HEGL-0139', 'HEGL-0139', '{"3": "HEGL-0139", "4": "германий-68", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'металлический порошок', 'металлический порошок', '{"3": "металлический порошок", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-57', '4СО-57', '{"3": "4СО-57", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-83', '4СО-83', '{"3": "4СО-83", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Th-232', 'Th-232', '{"3": "Th-232", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МКД', 'МКД', '{"3": "МКД", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4СО-85', '4СО-85', '{"3": "4СО-85", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IУ', 'IУ', '{"3": "IУ", "4": "бериллий-10; плутоний-238", "5": "", "6": "", "7": "", "8": "", "9": "", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IУ', 'IУ', '{"3": "IУ", "4": "плутоний-238; бериллий-10", "5": "", "6": "", "7": "", "8": "", "9": "", "10": "", "11": "", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Протяженный источник', 'Протяженный источник', '{"3": "Протяженный источник", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.08', 'XCd9.08', '{"3": "XCd9.08", "4": "Кадмий-109", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-22', '5П9-22', '{"3": "5П9-22", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EU-NA-S', 'EU-NA-S', '{"3": "EU-NA-S", "4": "европий-155; натрий-22", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-25', '5П9-25', '{"3": "5П9-25", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GRa6.12', 'GRa6.12', '{"3": "GRa6.12", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'облуч. обр.ПИК-11', 'облуч. обр.ПИК-11', '{"3": "облуч. обр.ПИК-11", "4": "углерод-14", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-26', '5П9-26', '{"3": "5П9-26", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GRa6.12', 'GRa6.12', '{"3": "GRa6.12", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MMS-05-022', 'MMS-05-022', '{"3": "MMS-05-022", "4": "натрий-22", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'образец', 'образец', '{"3": "образец", "4": "платина-191; углерод-14", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-56', '5П9-56', '{"3": "5П9-56", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'образец', 'образец', '{"3": "образец", "4": "углерод-14", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'M37C04-19', 'M37C04-19', '{"3": "M37C04-19", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-81', '5П9-81', '{"3": "5П9-81", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'образец 1984-1', 'образец 1984-1', '{"3": "образец 1984-1", "4": "углерод-14", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-82', '5П9-82', '{"3": "5П9-82", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АС339', 'АС339', '{"3": "АС339", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'образец 1984-10', 'образец 1984-10', '{"3": "образец 1984-10", "4": "углерод-14", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-83', '5П9-83', '{"3": "5П9-83", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'образец 1984-8', 'образец 1984-8', '{"3": "образец 1984-8", "4": "углерод-14", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL 6160вн.кап.', 'RSL 6160вн.кап.', '{"3": "RSL 6160вн.кап.", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-84', '5П9-84', '{"3": "5П9-84", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'образец 1984-9', 'образец 1984-9', '{"3": "образец 1984-9", "4": "углерод-14", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-85', '5П9-85', '{"3": "5П9-85", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'окись', 'окись', '{"3": "окись", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-122', '5СО-122', '{"3": "5СО-122", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'оксид', 'оксид', '{"3": "оксид", "4": "америций-243", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL1700', 'RSL1700', '{"3": "RSL1700", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-125', '5СО-125', '{"3": "5СО-125", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'оксид', 'оксид', '{"3": "оксид", "4": "кюрий-244", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL6000', 'RSL6000', '{"3": "RSL6000", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-127', '5СО-127', '{"3": "5СО-127", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NER-584', 'NER-584', '{"3": "NER-584", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'оксид', 'оксид', '{"3": "оксид", "4": "плутоний-242", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL6010', 'RSL6010', '{"3": "RSL6010", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL6020', 'RSL6020', '{"3": "RSL6020", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NER-588', 'NER-588', '{"3": "NER-588", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL6030', 'RSL6030', '{"3": "RSL6030", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'раствор р/н', 'раствор р/н', '{"3": "раствор р/н", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL6040', 'RSL6040', '{"3": "RSL6040", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-37', '5СО-37', '{"3": "5СО-37", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'раствор р/н', 'раствор р/н', '{"3": "раствор р/н", "4": "хлор-36", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М34.3', 'ГИ192М34.3', '{"3": "ГИ192М34.3", "4": "иридий-192", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL6050', 'RSL6050', '{"3": "RSL6050", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-52', '5СО-52', '{"3": "5СО-52", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М34.4', 'ГИ192М34.4', '{"3": "ГИ192М34.4", "4": "иридий-192", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL6060', 'RSL6060', '{"3": "RSL6060", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-56', '5СО-56', '{"3": "5СО-56", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М34.6', 'ГИ192М34.6', '{"3": "ГИ192М34.6", "4": "иридий-192", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-10-1р', 'ОСГИ-10-1р', '{"3": "ОСГИ-10-1р", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL6100', 'RSL6100', '{"3": "RSL6100", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ238-Н/2', 'РИТ238-Н/2', '{"3": "РИТ238-Н/2", "4": "Торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-10-1р', 'ОСГИ-10-1р', '{"3": "ОСГИ-10-1р", "4": "марганец-54", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL6150', 'RSL6150', '{"3": "RSL6150", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-10-1р', 'ОСГИ-10-1р', '{"3": "ОСГИ-10-1р", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL6150 вн.', 'RSL6150 вн.', '{"3": "RSL6150 вн.", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-87', '5СО-87', '{"3": "5СО-87", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-100-1р', 'ОСГИ-100-1р', '{"3": "ОСГИ-100-1р", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'TCI-3077', 'TCI-3077', '{"3": "TCI-3077", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-100-1р', 'ОСГИ-100-1р', '{"3": "ОСГИ-100-1р", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL6160 вн.', 'RSL6160 вн.', '{"3": "RSL6160 вн.", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-100-1р', 'ОСГИ-100-1р', '{"3": "ОСГИ-100-1р", "4": "европий-152", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RSL6240', 'RSL6240', '{"3": "RSL6240", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'VQC-068', 'VQC-068', '{"3": "VQC-068", "4": "германий-68", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-100-1р', 'ОСГИ-100-1р', '{"3": "ОСГИ-100-1р", "4": "иттрий-88", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-100-1р', 'ОСГИ-100-1р', '{"3": "ОСГИ-100-1р", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Tl', 'Tl', '{"3": "Tl", "4": "таллий-204", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'X.117', 'X.117', '{"3": "X.117", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-100-1р', 'ОСГИ-100-1р', '{"3": "ОСГИ-100-1р", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'X2153', 'X2153', '{"3": "X2153", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-100-1р', 'ОСГИ-100-1р', '{"3": "ОСГИ-100-1р", "4": "натрий-22", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd 9.01', 'XCd 9.01', '{"3": "XCd 9.01", "4": "кадмий-109", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-100-1р', 'ОСГИ-100-1р', '{"3": "ОСГИ-100-1р", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-23', '6П9-23', '{"3": "6П9-23", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6CO-53', '6CO-53', '{"3": "6CO-53", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.22', 'XFe5.22', '{"3": "XFe5.22", "4": "железо-55", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-100-1р', 'ОСГИ-100-1р', '{"3": "ОСГИ-100-1р", "4": "церий-139", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-100-1р', 'ОСГИ-100-1р', '{"3": "ОСГИ-100-1р", "4": "цинк-65", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-25', '6П9-25', '{"3": "6П9-25", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-53', '6СО-53', '{"3": "6СО-53", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-26', '6П9-26', '{"3": "6П9-26", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АКН', 'АКН', '{"3": "АКН", "4": "хром-51", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОУ.0043-16-6 ГСО ОУ.0043', 'ОУ.0043-16-6 ГСО ОУ.0043', '{"3": "ОУ.0043-16-6 ГСО ОУ.0043", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОУ.0370-31-6 ГСО ОУ.0370', 'ОУ.0370-31-6 ГСО ОУ.0370', '{"3": "ОУ.0370-31-6 ГСО ОУ.0370", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БА-95', 'БА-95', '{"3": "БА-95", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОУ.2149-46-6 ГСО ОУ.2149', 'ОУ.2149-46-6 ГСО ОУ.2149', '{"3": "ОУ.2149-46-6 ГСО ОУ.2149", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Бета-источник', 'Бета-источник', '{"3": "Бета-источник", "4": "Стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОУ.4979-61-6 ГСО ОУ.4979', 'ОУ.4979-61-6 ГСО ОУ.4979', '{"3": "ОУ.4979-61-6 ГСО ОУ.4979", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОУ.8795-76-6 ГСО ОУ.8795', 'ОУ.8795-76-6 ГСО ОУ.8795', '{"3": "ОУ.8795-76-6 ГСО ОУ.8795", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО ОУ.0043', 'ГСО ОУ.0043', '{"3": "ГСО ОУ.0043", "4": "Уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО ОУ.0370', 'ГСО ОУ.0370', '{"3": "ГСО ОУ.0370", "4": "Уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО ОУ.2149', 'ГСО ОУ.2149', '{"3": "ГСО ОУ.2149", "4": "Уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'пертехнетат - 10.0 г.', 'пертехнетат - 10.0 г.', '{"3": "пертехнетат - 10.0 г.", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО ОУ.4979', 'ГСО ОУ.4979', '{"3": "ГСО ОУ.4979", "4": "Уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РБК-К7.Сб.262-1-10', 'РБК-К7.Сб.262-1-10', '{"3": "РБК-К7.Сб.262-1-10", "4": "иод-125", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РБМ-К7.Сб.262-10', 'РБМ-К7.Сб.262-10', '{"3": "РБМ-К7.Сб.262-10", "4": "иод-125", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'пертехнетат аммония 1-NH4TcO4', 'пертехнетат аммония 1-NH4TcO4', '{"3": "пертехнетат аммония 1-NH4TcO4", "4": "технеций-99", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО ОУ.8795', 'ГСО ОУ.8795', '{"3": "ГСО ОУ.8795", "4": "Уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'пертехнетат калия 2-KTcO4', 'пертехнетат калия 2-KTcO4', '{"3": "пертехнетат калия 2-KTcO4", "4": "технеций-99", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'газовая смесь', 'газовая смесь', '{"3": "газовая смесь", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'препарат', 'препарат', '{"3": "препарат", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВН3.223', 'ВН3.223', '{"3": "ВН3.223", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-81', '6П9-81', '{"3": "6П9-81", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'тритий', 'тритий', '{"3": "тритий", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'прибор "Гелий"', 'прибор "Гелий"', '{"3": "прибор \"Гелий\"", "4": "осмий-186", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВН3.324', 'ВН3.324', '{"3": "ВН3.324", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Pu-Li', 'Pu-Li', '{"3": "Pu-Li", "4": "плутоний-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'смесь изотопов', 'смесь изотопов', '{"3": "смесь изотопов", "4": "америций-241; америций-243", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'смесь тяжелых изотопов', 'смесь тяжелых изотопов', '{"3": "смесь тяжелых изотопов", "4": "плутоний-240", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЗиУ-4', 'ЗиУ-4', '{"3": "ЗиУ-4", "4": "уран-234", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОИСУ', 'СОИСУ', '{"3": "СОИСУ", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6П9-86', '6П9-86', '{"3": "6П9-86", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЗиУ-8', 'ЗиУ-8', '{"3": "ЗиУ-8", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РБМ-К7.Сб.262-1', 'РБМ-К7.Сб.262-1', '{"3": "РБМ-К7.Сб.262-1", "4": "молибден-99", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОИСУ-0001', 'СОИСУ-0001', '{"3": "СОИСУ-0001", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОИСУ-0011', 'СОИСУ-0011', '{"3": "СОИСУ-0011", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОК', 'ОК', '{"3": "ОК", "4": "уран-234; уран-235; уран-236; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИА141', 'ИГИА141', '{"3": "ИГИА141", "4": "Америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОИСУ-0027', 'СОИСУ-0027', '{"3": "СОИСУ-0027", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-127', '6СО-127', '{"3": "6СО-127", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОИСУ-0050', 'СОИСУ-0050', '{"3": "СОИСУ-0050", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИК-А2', 'ГИК-А2', '{"3": "ГИК-А2", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.5232', 'MCo7.5232', '{"3": "MCo7.5232", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОИСУ-0071', 'СОИСУ-0071', '{"3": "СОИСУ-0071", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОИСУ-0250', 'СОИСУ-0250', '{"3": "СОИСУ-0250", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'источник бета-излучения', 'источник бета-излучения', '{"3": "источник бета-излучения", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОИСУ-0446', 'СОИСУ-0446', '{"3": "СОИСУ-0446", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'источник гамма-излучения', 'источник гамма-излучения', '{"3": "источник гамма-излучения", "4": "гадолиний-153", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РБК-К7.Сб.262-2', 'РБК-К7.Сб.262-2', '{"3": "РБК-К7.Сб.262-2", "4": "молибден-99", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОИСУ-1023', 'СОИСУ-1023', '{"3": "СОИСУ-1023", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-37', '6СО-37', '{"3": "6СО-37", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РБК-К7.Сб.262-3', 'РБК-К7.Сб.262-3', '{"3": "РБК-К7.Сб.262-3", "4": "молибден-99", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОИСУ-2098', 'СОИСУ-2098', '{"3": "СОИСУ-2098", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6СО-52', '6СО-52', '{"3": "6СО-52", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РБК-К7.Сб.262-4', 'РБК-К7.Сб.262-4', '{"3": "РБК-К7.Сб.262-4", "4": "молибден-99", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОИСУ-4928', 'СОИСУ-4928', '{"3": "СОИСУ-4928", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО ОУ', 'ГСО ОУ', '{"3": "ГСО ОУ", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВН3.531', 'ВН3.531', '{"3": "ВН3.531", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОИСУ-9999', 'СОИСУ-9999', '{"3": "СОИСУ-9999", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'калибровочный фильтр', 'калибровочный фильтр', '{"3": "калибровочный фильтр", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КАСВ 14466', 'КАСВ 14466', '{"3": "КАСВ 14466", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'установка ГУМ', 'установка ГУМ', '{"3": "установка ГУМ", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'контр.препарат', 'контр.препарат', '{"3": "контр.препарат", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'фольга 99Tc', 'фольга 99Tc', '{"3": "фольга 99Tc", "4": "технеций-99", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГCs7.028.2', 'ГCs7.028.2', '{"3": "ГCs7.028.2", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'фольга99Tc', 'фольга99Tc', '{"3": "фольга99Tc", "4": "технеций-99", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КактусР1/11', 'КактусР1/11', '{"3": "КактусР1/11", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГАМ1.10', 'ГАМ1.10', '{"3": "ГАМ1.10", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МCo7.122', 'МCo7.122', '{"3": "МCo7.122", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КактусР1/13', 'КактусР1/13', '{"3": "КактусР1/13", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KACB 10687', 'KACB 10687', '{"3": "KACB 10687", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'электрод', 'электрод', '{"3": "электрод", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Виота', 'Виота', '{"3": "Виота", "4": "иттрий-90; стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ-2', 'КНТ-2', '{"3": "КНТ-2", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'тип 08', 'тип 08', '{"3": "тип 08", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НР-237', 'НР-237', '{"3": "НР-237", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PHCB 4745', 'PHCB 4745', '{"3": "PHCB 4745", "4": "прометий-147", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцов.2разр', 'Образцов.2разр', '{"3": "Образцов.2разр", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСS7.012.5', 'ГСS7.012.5', '{"3": "ГСS7.012.5", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый', 'Образцовый', '{"3": "Образцовый", "4": "технеций-98", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСS7.021.5', 'ГСS7.021.5', '{"3": "ГСS7.021.5", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСS7.0212.1', 'ГСS7.0212.1', '{"3": "ГСS7.0212.1", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый', 'Образцовый', '{"3": "Образцовый", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый', 'Образцовый', '{"3": "Образцовый", "4": "селен-79", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-3-14', 'ИГИ-Ц-3-14', '{"3": "ИГИ-Ц-3-14", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АО', 'АО', '{"3": "АО", "4": "плутоний-242", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый', 'Образцовый', '{"3": "Образцовый", "4": "олово-113", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый', 'Образцовый', '{"3": "Образцовый", "4": "марганец-54", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-3-21', 'ИГИ-Ц-3-21', '{"3": "ИГИ-Ц-3-21", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IPPESeed', 'IPPESeed', '{"3": "IPPESeed", "4": "иод-125", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Р1-881М', 'Р1-881М', '{"3": "Р1-881М", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый', 'Образцовый', '{"3": "Образцовый", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'HCF2.01-1.7', 'HCF2.01-1.7', '{"3": "HCF2.01-1.7", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый', 'Образцовый', '{"3": "Образцовый", "4": "кадмий-109", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РА14-262', 'РА14-262', '{"3": "РА14-262", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'HCF2.011.6', 'HCF2.011.6', '{"3": "HCF2.011.6", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый', 'Образцовый', '{"3": "Образцовый", "4": "иттрий-90; стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РА2,5.2-2', 'РА2,5.2-2', '{"3": "РА2,5.2-2", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'HCF2.011.9', 'HCF2.011.9', '{"3": "HCF2.011.9", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый', 'Образцовый', '{"3": "Образцовый", "4": "иттрий-88", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NXCm4.15', 'NXCm4.15', '{"3": "NXCm4.15", "4": "кюрий-244", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РА21-4', 'РА21-4', '{"3": "РА21-4", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'HК252.М11.18', 'HК252.М11.18', '{"3": "HК252.М11.18", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый', 'Образцовый', '{"3": "Образцовый", "4": "европий-152", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый', 'Образцовый', '{"3": "Образцовый", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИ 42.К2.17.000', 'РИ 42.К2.17.000', '{"3": "РИ 42.К2.17.000", "4": "Торий-229; Торий-227", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РА53-42', 'РА53-42', '{"3": "РА53-42", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Нестандартный, по инд. Заказу', 'Нестандартный, по инд. Заказу', '{"3": "Нестандартный, по инд. Заказу", "4": "самарий-151", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый', 'Образцовый', '{"3": "Образцовый", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NER-004P', 'NER-004P', '{"3": "NER-004P", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый', 'Образцовый', '{"3": "Образцовый", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.2.P', 'XCd9.2.P', '{"3": "XCd9.2.P", "4": "Кадмий-109", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСОО.061.3', 'ГСОО.061.3', '{"3": "ГСОО.061.3", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'образцовый источник', 'образцовый источник', '{"3": "образцовый источник", "4": "европий-152", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый источник', 'Образцовый источник', '{"3": "Образцовый источник", "4": "технеций-98", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РБА14-1', 'РБА14-1', '{"3": "РБА14-1", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'образцовый источник', 'образцовый источник', '{"3": "образцовый источник", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РБА15-2', 'РБА15-2', '{"3": "РБА15-2", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'азотнокислый', 'азотнокислый', '{"3": "азотнокислый", "4": "цезий-134", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый источник', 'Образцовый источник', '{"3": "Образцовый источник", "4": "селен-79", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'дефлектор', 'дефлектор', '{"3": "дефлектор", "4": "джн (смесь долгоживущих нуклидов)*", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РБА16-3', 'РБА16-3', '{"3": "РБА16-3", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АКH', 'АКH', '{"3": "АКH", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый источник', 'Образцовый источник', '{"3": "Образцовый источник", "4": "олово-113", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АКH-2', 'АКH-2', '{"3": "АКH-2", "4": "хром-51", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый источник', 'Образцовый источник', '{"3": "Образцовый источник", "4": "марганец-54", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АКH-Т-10', 'АКH-Т-10', '{"3": "АКH-Т-10", "4": "марганец-54", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый источник', 'Образцовый источник', '{"3": "Образцовый источник", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ - Р', 'ОСГИ - Р', '{"3": "ОСГИ - Р", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АКH-Т-10', 'АКH-Т-10', '{"3": "АКH-Т-10", "4": "натрий-22", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый источник', 'Образцовый источник', '{"3": "Образцовый источник", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ - Р', 'ОСГИ - Р', '{"3": "ОСГИ - Р", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый источник', 'Образцовый источник', '{"3": "Образцовый источник", "4": "иттрий-88", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый источник', 'Образцовый источник', '{"3": "Образцовый источник", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GU5.P01.1206', 'GU5.P01.1206', '{"3": "GU5.P01.1206", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'образцовый источник', 'образцовый источник', '{"3": "образцовый источник", "4": "америций-243", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GU5.P01.1304', 'GU5.P01.1304', '{"3": "GU5.P01.1304", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'образцовый источник', 'образцовый источник', '{"3": "образцовый источник", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GU5.P01.2704', 'GU5.P01.2704', '{"3": "GU5.P01.2704", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'образцовый источник', 'образцовый источник', '{"3": "образцовый источник", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GU5.P01.6204', 'GU5.P01.6204', '{"3": "GU5.P01.6204", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый источник', 'Образцовый источник', '{"3": "Образцовый источник", "4": "кадмий-109", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГU5.Р01', 'ГU5.Р01', '{"3": "ГU5.Р01", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КИ 101/50', 'КИ 101/50', '{"3": "КИ 101/50", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГU5.Р02', 'ГU5.Р02', '{"3": "ГU5.Р02", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСS7.012.2', 'ГСS7.012.2', '{"3": "ГСS7.012.2", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCs.7.44', 'GCs.7.44', '{"3": "GCs.7.44", "4": "Цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБИРЗН-63, 2б-1', 'ИБИРЗН-63, 2б-1', '{"3": "ИБИРЗН-63, 2б-1", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'нитрат в твердой форме', 'нитрат в твердой форме', '{"3": "нитрат в твердой форме", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'С64343', 'С64343', '{"3": "С64343", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'контрольный', 'контрольный', '{"3": "контрольный", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'контрольный', 'контрольный', '{"3": "контрольный", "4": "цезий-137; калий-40", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PPu8.07.Х', 'PPu8.07.Х', '{"3": "PPu8.07.Х", "4": "Плутоний-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'контрольный', 'контрольный', '{"3": "контрольный", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.061.Х', 'XCd9.061.Х', '{"3": "XCd9.061.Х", "4": "Кадмий-109", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНВЭД', 'ТНВЭД', '{"3": "ТНВЭД", "4": "цезий-131", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'М37С04-09', 'М37С04-09', '{"3": "М37С04-09", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCm4.061.2', 'XCm4.061.2', '{"3": "XCm4.061.2", "4": "Кюрий-244", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЭН-2В', 'ЭН-2В', '{"3": "ЭН-2В", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНВЭД', 'ТНВЭД', '{"3": "ТНВЭД", "4": "хром-51", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'М37С04-16', 'М37С04-16', '{"3": "М37С04-16", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЭН-7В', 'ЭН-7В', '{"3": "ЭН-7В", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЭЯ-1', 'ЭЯ-1', '{"3": "ЭЯ-1", "4": "углерод-14", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XPu8.07.3', 'XPu8.07.3', '{"3": "XPu8.07.3", "4": "Плутоний-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'М37С04-23', 'М37С04-23', '{"3": "М37С04-23", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "радий-226; европий-152; торий-228; цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'М37С04-27', 'М37С04-27', '{"3": "М37С04-27", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'радиометрический', 'радиометрический', '{"3": "радиометрический", "4": "Стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "торий-228", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'М37С08-03', 'М37С08-03', '{"3": "М37С08-03", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "церий-139", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1С0-134', '1С0-134', '{"3": "1С0-134", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'М37С08-05', 'М37С08-05', '{"3": "М37С08-05", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1С0-214', '1С0-214', '{"3": "1С0-214", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'М37С08-18', 'М37С08-18', '{"3": "М37С08-18", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1С0-215', '1С0-215', '{"3": "1С0-215", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'М37С08-28', 'М37С08-28', '{"3": "М37С08-28", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "Уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1С0-216', '1С0-216', '{"3": "1С0-216", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "Уран-238; уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2С0-214', '2С0-214', '{"3": "2С0-214", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "Кобальт-60; Цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2С0-215', '2С0-215', '{"3": "2С0-215", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMRB 1741', 'AMRB 1741', '{"3": "AMRB 1741", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2С0-216', '2С0-216', '{"3": "2С0-216", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'двухсторонняя мишень', 'двухсторонняя мишень', '{"3": "двухсторонняя мишень", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'М37С11-40', 'М37С11-40', '{"3": "М37С11-40", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3С0-134', '3С0-134', '{"3": "3С0-134", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'стронций-90', 'стронций-90', '{"3": "стронций-90", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'М37С11-51', 'М37С11-51', '{"3": "М37С11-51", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "америций-241; церий-139; кобальт-57; барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3С0-137', '3С0-137', '{"3": "3С0-137", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SIRB 8151', 'SIRB 8151', '{"3": "SIRB 8151", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'М37С16-74', 'М37С16-74', '{"3": "М37С16-74", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "углерод-14", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3С0-214', '3С0-214', '{"3": "3С0-214", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr.0.02', 'BSr.0.02', '{"3": "BSr.0.02", "4": "Стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КактусР1/4', 'КактусР1/4', '{"3": "КактусР1/4", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3С0-215', '3С0-215', '{"3": "3С0-215", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "кобальт-60; америций-241; цезий-137; европий-152", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3С0-324', '3С0-324', '{"3": "3С0-324", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'М85К01-29', 'М85К01-29', '{"3": "М85К01-29", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "америций-241; цезий-137; кобальт-60; европий-152", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3С0-534', '3С0-534', '{"3": "3С0-534", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4С0-214', '4С0-214', '{"3": "4С0-214", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Естественный уран на подложке', 'Естественный уран на подложке', '{"3": "Естественный уран на подложке", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "европий-152", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4С0-215', '4С0-215', '{"3": "4С0-215", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "европий-152; барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК-252', 'НК-252', '{"3": "НК-252", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4С0-216', '4С0-216', '{"3": "4С0-216", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "иттрий-88", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4С0-217', '4С0-217', '{"3": "4С0-217", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "калий-40", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4С0-534', '4С0-534', '{"3": "4С0-534", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КАСВ14466 TI-317', 'КАСВ14466 TI-317', '{"3": "КАСВ14466 TI-317", "4": "Криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "Кобальт-57; Барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5С0-325', '5С0-325', '{"3": "5С0-325", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5С0-326', '5С0-326', '{"3": "5С0-326", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН', 'ОМАСН', '{"3": "ОМАСН", "4": "америций-241; цезий-137; барий-133; иттрий-88; кобальт-60; кобальт-57; калий-40", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5С0-327', '5С0-327', '{"3": "5С0-327", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН (Am Co Ce Ba)', 'ОМАСН (Am Co Ce Ba)', '{"3": "ОМАСН (Am Co Ce Ba)", "4": "америций-241; кобальт-57; церий-139; барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr0.43', 'BSr0.43', '{"3": "BSr0.43", "4": "Стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН (Am)', 'ОМАСН (Am)', '{"3": "ОМАСН (Am)", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6С0-136', '6С0-136', '{"3": "6С0-136", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NCf.82', 'NCf.82', '{"3": "NCf.82", "4": "Калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН (Тh)', 'ОМАСН (Тh)', '{"3": "ОМАСН (Тh)", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6С0-326', '6С0-326', '{"3": "6С0-326", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН(Сs Co Y)', 'ОМАСН(Сs Co Y)', '{"3": "ОМАСН(Сs Co Y)", "4": "цезий-137; кобальт-60; иттрий-88", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6С0-537', '6С0-537', '{"3": "6С0-537", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCd9.06', 'GCd9.06', '{"3": "GCd9.06", "4": "Кадмий-109", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'U-238', 'U-238', '{"3": "U-238", "4": "Уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'аналитический источник', 'аналитический источник', '{"3": "аналитический источник", "4": "Уран-238; Плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'макет', 'макет', '{"3": "макет", "4": "Торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'металлический кобальт порошок', 'металлический кобальт порошок', '{"3": "металлический кобальт порошок", "4": "Кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Микро ЭЗД', 'Микро ЭЗД', '{"3": "Микро ЭЗД", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Cd-109', 'Cd-109', '{"3": "Cd-109", "4": "Cd-109", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'мишень', 'мишень', '{"3": "мишень", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Cd-109', 'Cd-109', '{"3": "Cd-109", "4": "Кадмий-109", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'мишень', 'мишень', '{"3": "мишень", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'мишень', 'мишень', '{"3": "мишень", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'мишень', 'мишень', '{"3": "мишень", "4": "уран-235; торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'мишень', 'мишень', '{"3": "мишень", "4": "плутоний-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'мишень', 'мишень', '{"3": "мишень", "4": "кюрий-245", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'мишень', 'мишень', '{"3": "мишень", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МСо7.1231', 'МСо7.1231', '{"3": "МСо7.1231", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МСо7.124', 'МСо7.124', '{"3": "МСо7.124", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КТВ-4', 'КТВ-4', '{"3": "КТВ-4", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Металическая фольга', 'Металическая фольга', '{"3": "Металическая фольга", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ 3-2', 'ОСГИ 3-2', '{"3": "ОСГИ 3-2", "4": "кадмий-109", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ 3-2', 'ОСГИ 3-2', '{"3": "ОСГИ 3-2", "4": "цинк-65", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ 3-2', 'ОСГИ 3-2', '{"3": "ОСГИ 3-2", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ 3-2', 'ОСГИ 3-2', '{"3": "ОСГИ 3-2", "4": "торий-228", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ 3-2', 'ОСГИ 3-2', '{"3": "ОСГИ 3-2", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ 3-2', 'ОСГИ 3-2', '{"3": "ОСГИ 3-2", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ 3-2', 'ОСГИ 3-2', '{"3": "ОСГИ 3-2", "4": "олово-113", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1р', 'ОСГИ-3-2-1р', '{"3": "ОСГИ-3-2-1р", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1р', 'ОСГИ-3-2-1р', '{"3": "ОСГИ-3-2-1р", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1р', 'ОСГИ-3-2-1р', '{"3": "ОСГИ-3-2-1р", "4": "кадмий-109", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'нейтронный источник', 'нейтронный источник', '{"3": "нейтронный источник", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3-2-1р', 'ОСГИ-3-2-1р', '{"3": "ОСГИ-3-2-1р", "4": "торий-228", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'нейтронный источник', 'нейтронный источник', '{"3": "нейтронный источник", "4": "плутоний-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСНИГИ', 'ОСНИГИ', '{"3": "ОСНИГИ", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСНИГИ', 'ОСНИГИ', '{"3": "ОСНИГИ", "4": "европий-152", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСНИГИ', 'ОСНИГИ', '{"3": "ОСНИГИ", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Опытный', 'Опытный', '{"3": "Опытный", "4": "железо-55", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95 651-97', 'ОСО 95 651-97', '{"3": "ОСО 95 651-97", "4": "Уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'опытный', 'опытный', '{"3": "опытный", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95 652-97', 'ОСО 95 652-97', '{"3": "ОСО 95 652-97", "4": "Уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95 653-97', 'ОСО 95 653-97', '{"3": "ОСО 95 653-97", "4": "Уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95 654-97', 'ОСО 95 654-97', '{"3": "ОСО 95 654-97", "4": "Уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95 655-97', 'ОСО 95 655-97', '{"3": "ОСО 95 655-97", "4": "Уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'С1610', 'С1610', '{"3": "С1610", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СВЗт-3-5', 'СВЗт-3-5', '{"3": "СВЗт-3-5", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Co', 'Co', '{"3": "Co", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Стержневой источник', 'Стержневой источник', '{"3": "Стержневой источник", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Цезиевый', 'Цезиевый', '{"3": "Цезиевый", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЭИСН', 'ЭИСН', '{"3": "ЭИСН", "4": "иод-129", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЭИСН', 'ЭИСН', '{"3": "ЭИСН", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-3 СН', 'ОСГИ-3 СН', '{"3": "ОСГИ-3 СН", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Прогресс (комбинированный источник)', 'Прогресс (комбинированный источник)', '{"3": "Прогресс (комбинированный источник)", "4": "цезий-137; калий-40", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-П', 'ОСГИ-П', '{"3": "ОСГИ-П", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-Р/М', 'ОСГИ-Р/М', '{"3": "ОСГИ-Р/М", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-238-Н/2', 'РИТ-238-Н/2', '{"3": "РИТ-238-Н/2", "4": "Торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-238.Р05', 'РИТ-238.Р05', '{"3": "РИТ-238.Р05", "4": "Плутоний-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ238.Р05', 'РИТ238.Р05', '{"3": "РИТ238.Р05", "4": "Плутоний-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РР', 'РР', '{"3": "РР", "4": "уран-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РР', 'РР', '{"3": "РР", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РР', 'РР', '{"3": "РР", "4": "цезий-134", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПГЛ.1', 'ПГЛ.1', '{"3": "ПГЛ.1", "4": "германий-68", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РР', 'РР', '{"3": "РР", "4": "уран-234; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РР', 'РР', '{"3": "РР", "4": "америций-243", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РР', 'РР', '{"3": "РР", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РР', 'РР', '{"3": "РР", "4": "нептуний-237", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РР', 'РР', '{"3": "РР", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РР', 'РР', '{"3": "РР", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'С3100', 'С3100', '{"3": "С3100", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СБ-60', 'СБ-60', '{"3": "СБ-60", "4": "Кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Со7.1.16.96', 'Со7.1.16.96', '{"3": "Со7.1.16.96", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'спецназначения', 'спецназначения', '{"3": "спецназначения", "4": "калий-40; цезий-137; стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'спецназначения', 'спецназначения', '{"3": "спецназначения", "4": "радий-226; торий-228; калий-40; цезий-137; кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'спецназначения', 'спецназначения', '{"3": "спецназначения", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СР17.140', 'СР17.140', '{"3": "СР17.140", "4": "селен-75", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т-19 (ДРГ-05)', 'Т-19 (ДРГ-05)', '{"3": "Т-19 (ДРГ-05)", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Хлорид', 'Хлорид', '{"3": "Хлорид", "4": "Радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц 4-7', 'Ц 4-7', '{"3": "Ц 4-7", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Цезий-137', 'Цезий-137', '{"3": "Цезий-137", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'APu8.12', 'APu8.12', '{"3": "APu8.12", "4": "Плутоний-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSrO.083', 'BSrO.083', '{"3": "BSrO.083", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.27', 'GCo7.27', '{"3": "GCo7.27", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РБК-К7.Сб.262-1', 'РБК-К7.Сб.262-1', '{"3": "РБК-К7.Сб.262-1", "4": "молибден-99", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.132', 'XFe5.132', '{"3": "XFe5.132", "4": "железо-55", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МCo7.124', 'МCo7.124', '{"3": "МCo7.124", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Пленка имитатор РЗ', 'Пленка имитатор РЗ', '{"3": "Пленка имитатор РЗ", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СоО.А86', 'СоО.А86', '{"3": "СоО.А86", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'техне-ций-99', 'техне-ций-99', '{"3": "техне-ций-99", "4": "технеций-99", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'тритий-титановая мишень', 'тритий-титановая мишень', '{"3": "тритий-титановая мишень", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'эталон', 'эталон', '{"3": "эталон", "4": "Цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '-', '-', '{"3": "-", "4": "нептуний-237", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '-', '-', '{"3": "-", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '-', '-', '{"3": "-", "4": "теллур-134", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '-', '-', '{"3": "-", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '-', '-', '{"3": "-", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '-', '-', '{"3": "-", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '-', '-', '{"3": "-", "4": "церий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '-', '-', '{"3": "-", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '-', '-', '{"3": "-", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '-', '-', '{"3": "-", "4": "калий-40", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '-', '-', '{"3": "-", "4": "иод-129", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '-', '-', '{"3": "-", "4": "плутоний-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Е04', 'Е04', '{"3": "Е04", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-126', '1СО-126', '{"3": "1СО-126", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИЦ-4-4', 'ИГИЦ-4-4', '{"3": "ИГИЦ-4-4", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'прим.', 'прим.', '{"3": "прим.", "4": "европий-152", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Am-241+Li', 'Am-241+Li', '{"3": "Am-241+Li", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСS7.011.13', 'ГСS7.011.13', '{"3": "ГСS7.011.13", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.S16.1.R', 'BNi3.S16.1.R', '{"3": "BNi3.S16.1.R", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GNa2.13', 'GNa2.13', '{"3": "GNa2.13", "4": "натрий-22", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО84002003', 'ГСО84002003', '{"3": "ГСО84002003", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО84012003', 'ГСО84012003', '{"3": "ГСО84012003", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'A3888', 'A3888', '{"3": "A3888", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'A3894-4', 'A3894-4', '{"3": "A3894-4", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСНИ', 'ОСНИ', '{"3": "ОСНИ", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСНИ', 'ОСНИ', '{"3": "ОСНИ", "4": "уран-235; европий-152", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСНИ', 'ОСНИ', '{"3": "ОСНИ", "4": "уран-235; кобальт-60; цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РБИ', 'РБИ', '{"3": "РБИ", "4": "углерод-14", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'B+Na2', 'B+Na2', '{"3": "B+Na2", "4": "натрий-22", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3С3.4.R', 'BNi3С3.4.R', '{"3": "BNi3С3.4.R", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BSr0.374', 'BSr0.374', '{"3": "BSr0.374", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'C42211', 'C42211', '{"3": "C42211", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDRB4339', 'CDRB4339', '{"3": "CDRB4339", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Clarus 600', 'Clarus 600', '{"3": "Clarus 600", "4": "Никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCs7.x', 'GCs7.x', '{"3": "GCs7.x", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GM232.02-000', 'GM232.02-000', '{"3": "GM232.02-000", "4": "иридий-192", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KACK5533', 'KACK5533', '{"3": "KACK5533", "4": "Криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Elekta Gammaknife Source No. 43685', 'Elekta Gammaknife Source No. 43685', '{"3": "Elekta Gammaknife Source No. 43685", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KИ-101/50', 'KИ-101/50', '{"3": "KИ-101/50", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MСо7.121', 'MСо7.121', '{"3": "MСо7.121", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NaPosSC', 'NaPosSC', '{"3": "NaPosSC", "4": "натрий-22", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NER004P', 'NER004P', '{"3": "NER004P", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SR17.140', 'SR17.140', '{"3": "SR17.140", "4": "селен-75", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5Am1.31A', 'XFe5Am1.31A', '{"3": "XFe5Am1.31A", "4": "америций-241; железо-55", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АИП-МИР-ЗА', 'АИП-МИР-ЗА', '{"3": "АИП-МИР-ЗА", "4": "Плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АИП-Н-1', 'АИП-Н-1', '{"3": "АИП-Н-1", "4": "Плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AM1.Р08', 'AM1.Р08', '{"3": "AM1.Р08", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Бета-источник', 'Бета-источник', '{"3": "Бета-источник", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИДК-Р', 'ОИДК-Р', '{"3": "ОИДК-Р", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИДК-Р', 'ОИДК-Р', '{"3": "ОИДК-Р", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВС1.0-11', 'ВС1.0-11', '{"3": "ВС1.0-11", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГС75М15.90', 'ГС75М15.90', '{"3": "ГС75М15.90", "4": "селен-75", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСS7', 'ГСS7', '{"3": "ГСS7", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСS7.012.4', 'ГСS7.012.4', '{"3": "ГСS7.012.4", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСS7.012.9', 'ГСS7.012.9', '{"3": "ГСS7.012.9", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСS7.021.2', 'ГСS7.021.2', '{"3": "ГСS7.021.2", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО ОУ 0043', 'ГСО ОУ 0043', '{"3": "ГСО ОУ 0043", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО ОУ 0370', 'ГСО ОУ 0370', '{"3": "ГСО ОУ 0370", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО ОУ 2149', 'ГСО ОУ 2149', '{"3": "ГСО ОУ 2149", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО ОУ 4979', 'ГСО ОУ 4979', '{"3": "ГСО ОУ 4979", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО ОУ 8795', 'ГСО ОУ 8795', '{"3": "ГСО ОУ 8795", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЕС GF', 'ЕС GF', '{"3": "ЕС GF", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЕС GF', 'ЕС GF', '{"3": "ЕС GF", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'закрытый', 'закрытый', '{"3": "закрытый", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SIFB11369', 'SIFB11369', '{"3": "SIFB11369", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЗРИ', 'ЗРИ', '{"3": "ЗРИ", "4": "уран-233", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИК-В', 'ИК-В', '{"3": "ИК-В", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 38015', 'CDC 38015', '{"3": "CDC 38015", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 3802', 'CDC 3802', '{"3": "CDC 3802", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 381600', 'CDC 381600', '{"3": "CDC 381600", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 38750', 'CDC 38750', '{"3": "CDC 38750", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 806', 'CDC 806', '{"3": "CDC 806", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМН-Г-3-Т', 'ИМН-Г-3-Т', '{"3": "ИМН-Г-3-Т", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC 807', 'CDC 807', '{"3": "CDC 807", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМН-Г-3-Т', 'ИМН-Г-3-Т', '{"3": "ИМН-Г-3-Т", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC2016-0010', 'CDC2016-0010', '{"3": "CDC2016-0010", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМН-Г-3-Т', 'ИМН-Г-3-Т', '{"3": "ИМН-Г-3-Т", "4": "европий-152", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC2016-0020', 'CDC2016-0020', '{"3": "CDC2016-0020", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМН-Г-3-Т', 'ИМН-Г-3-Т', '{"3": "ИМН-Г-3-Т", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC2016-0100', 'CDC2016-0100', '{"3": "CDC2016-0100", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМН-Г-3-Т', 'ИМН-Г-3-Т', '{"3": "ИМН-Г-3-Т", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC2016-0200', 'CDC2016-0200', '{"3": "CDC2016-0200", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC38015', 'CDC38015', '{"3": "CDC38015", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНK-5', 'ИНK-5', '{"3": "ИНK-5", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНK-6', 'ИНK-6', '{"3": "ИНK-6", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИСО/99/С22111', 'ИСО/99/С22111', '{"3": "ИСО/99/С22111", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'M37C02-02', 'M37C02-02', '{"3": "M37C02-02", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'M37C04-01', 'M37C04-01', '{"3": "M37C04-01", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'M37C04-09', 'M37C04-09', '{"3": "M37C04-09", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'M37C04-16', 'M37C04-16', '{"3": "M37C04-16", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'мёссбауэровский источник', 'мёссбауэровский источник', '{"3": "мёссбауэровский источник", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Мёссбауэровский источник', 'Мёссбауэровский источник', '{"3": "Мёссбауэровский источник", "4": "самарий-151", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'M37C04-23', 'M37C04-23', '{"3": "M37C04-23", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'мишени', 'мишени', '{"3": "мишени", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'M37C04-27', 'M37C04-27', '{"3": "M37C04-27", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'M37C08-29', 'M37C08-29', '{"3": "M37C08-29", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'M37C0839', 'M37C0839', '{"3": "M37C0839", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НИК', 'НИК', '{"3": "НИК", "4": "Калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'M37C09-29', 'M37C09-29', '{"3": "M37C09-29", "4": "церий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'M37C09-37', 'M37C09-37', '{"3": "M37C09-37", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК248М11', 'НК248М11', '{"3": "НК248М11", "4": "Кюрий-248", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'M85K01-29', 'M85K01-29', '{"3": "M85K01-29", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252МО.01.109', 'НК252МО.01.109', '{"3": "НК252МО.01.109", "4": "Калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НР-11Н', 'НР-11Н', '{"3": "НР-11Н", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НР-9Н', 'НР-9Н', '{"3": "НР-9Н", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОАИ-6', 'ОАИ-6', '{"3": "ОАИ-6", "4": "Стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый бета- источник ISO-135', 'Образцовый бета- источник ISO-135', '{"3": "Образцовый бета- источник ISO-135", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый источник 1Г', 'Образцовый источник 1Г', '{"3": "Образцовый источник 1Г", "4": "Кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'TSI-3077', 'TSI-3077', '{"3": "TSI-3077", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'опытный', 'опытный', '{"3": "опытный", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'опытный', 'опытный', '{"3": "опытный", "4": "плутоний-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'опытный', 'опытный', '{"3": "опытный", "4": "церий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'опытный', 'опытный', '{"3": "опытный", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'опытный', 'опытный', '{"3": "опытный", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'опытный', 'опытный', '{"3": "опытный", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'опытный', 'опытный', '{"3": "опытный", "4": "иод-129", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'опытный', 'опытный', '{"3": "опытный", "4": "уран-236", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'опытный', 'опытный', '{"3": "опытный", "4": "германий-68", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'опытный', 'опытный', '{"3": "опытный", "4": "нептуний-237", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192M53', 'ГИ192M53', '{"3": "ГИ192M53", "4": "иридий-192", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-8-156', 'ИБН-8-156', '{"3": "ИБН-8-156", "4": "плутоний-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-8-157', 'ИБН-8-157', '{"3": "ИБН-8-157", "4": "плутоний-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИБН-8-556', 'ИБН-8-556', '{"3": "ИБН-8-556", "4": "плутоний-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИГИ-Ц-3-18', 'ИГИ-Ц-3-18', '{"3": "ИГИ-Ц-3-18", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСК-238', 'ОСК-238', '{"3": "ОСК-238", "4": "уран естественный", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПИК-ЭГП', 'ПИК-ЭГП', '{"3": "ПИК-ЭГП", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Рабочий эталон', 'Рабочий эталон', '{"3": "Рабочий эталон", "4": "Кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C3.2.R', 'BNi3.C3.2.R', '{"3": "BNi3.C3.2.R", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Радиевый эталон ЕР1', 'Радиевый эталон ЕР1', '{"3": "Радиевый эталон ЕР1", "4": "Радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Радиевый эталон ЕР15', 'Радиевый эталон ЕР15', '{"3": "Радиевый эталон ЕР15", "4": "Радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5203', '5203', '{"3": "5203", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РФА', 'РФА', '{"3": "РФА", "4": "кадмий-109", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЭС-1', 'РЭС-1', '{"3": "РЭС-1", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЭС-10', 'РЭС-10', '{"3": "РЭС-10", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЭС-100', 'РЭС-100', '{"3": "РЭС-100", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЭС-160', 'РЭС-160', '{"3": "РЭС-160", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЭС-4', 'РЭС-4', '{"3": "РЭС-4", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РЭС-40', 'РЭС-40', '{"3": "РЭС-40", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'М37С02-02', 'М37С02-02', '{"3": "М37С02-02", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'М37С02-05', 'М37С02-05', '{"3": "М37С02-05", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'М37С02-19', 'М37С02-19', '{"3": "М37С02-19", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'М37С02-23', 'М37С02-23', '{"3": "М37С02-23", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'М37С04-05', 'М37С04-05', '{"3": "М37С04-05", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'М37С04-19', 'М37С04-19', '{"3": "М37С04-19", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'САИ-5', 'САИ-5', '{"3": "САИ-5", "4": "Стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОП', 'СОП', '{"3": "СОП", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'М37С09-33', 'М37С09-33', '{"3": "М37С09-33", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'М37С24-43', 'М37С24-43', '{"3": "М37С24-43", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'М37С24-44', 'М37С24-44', '{"3": "М37С24-44", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'М85К01-27', 'М85К01-27', '{"3": "М85К01-27", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'М85К01-33', 'М85К01-33', '{"3": "М85К01-33", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т19-Л69.01', 'Т19-Л69.01', '{"3": "Т19-Л69.01", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т19-Л69.010', 'Т19-Л69.010', '{"3": "Т19-Л69.010", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'VQC- 068', 'VQC- 068', '{"3": "VQC- 068", "4": "германий-68", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц-1', 'Ц-1', '{"3": "Ц-1", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЭИК', 'ЭИК', '{"3": "ЭИК", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ3', 'ОСГИ3', '{"3": "ОСГИ3", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ3', 'ОСГИ3', '{"3": "ОСГИ3", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ3', 'ОСГИ3', '{"3": "ОСГИ3", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ3', 'ОСГИ3', '{"3": "ОСГИ3", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ3', 'ОСГИ3', '{"3": "ОСГИ3", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ3', 'ОСГИ3', '{"3": "ОСГИ3", "4": "европий-152", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КАСК5565', 'КАСК5565', '{"3": "КАСК5565", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'W1020-005', 'W1020-005', '{"3": "W1020-005", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-90-180/27700-45', 'РИТ-90-180/27700-45', '{"3": "РИТ-90-180/27700-45", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-90-230/35000-95', 'РИТ-90-230/35000-95', '{"3": "РИТ-90-230/35000-95", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-90-530-95', 'РИТ-90-530-95', '{"3": "РИТ-90-530-95", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-90-530-95', 'РИТ-90-530-95', '{"3": "РИТ-90-530-95", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-90-530/81400-95', 'РИТ-90-530/81400-95', '{"3": "РИТ-90-530/81400-95", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-90-550/85000-120', 'РИТ-90-550/85000-120', '{"3": "РИТ-90-550/85000-120", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-90-580/89700-120', 'РИТ-90-580/89700-120', '{"3": "РИТ-90-580/89700-120", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-90-650-120', 'РИТ-90-650-120', '{"3": "РИТ-90-650-120", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СВЗТ-3-3', 'СВЗТ-3-3', '{"3": "СВЗТ-3-3", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СН3.РО5.7/25/3', 'СН3.РО5.7/25/3', '{"3": "СН3.РО5.7/25/3", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IEC3305', 'IEC3305', '{"3": "IEC3305", "4": "железо-55", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5E020200M', 'KR5E020200M', '{"3": "KR5E020200M", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KACK5565', 'KACK5565', '{"3": "KACK5565", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KACB14466', 'KACB14466', '{"3": "KACB14466", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.SX.Y.R', 'BNi3.SX.Y.R', '{"3": "BNi3.SX.Y.R", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PHC 80953', 'PHC 80953', '{"3": "PHC 80953", "4": "прометий-147", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМН-Г-3-Т', 'ИМН-Г-3-Т', '{"3": "ИМН-Г-3-Т", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.222', 'MSn9.222', '{"3": "MSn9.222", "4": "олово-119м", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NR-116-D-101-G', 'NR-116-D-101-G', '{"3": "NR-116-D-101-G", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНГ-27', 'ИНГ-27', '{"3": "ИНГ-27", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Образцовый альфа-источник 1П9-634', 'Образцовый альфа-источник 1П9-634', '{"3": "Образцовый альфа-источник 1П9-634", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КАСК 5565', 'КАСК 5565', '{"3": "КАСК 5565", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.S19.1.R', 'BNi3.S19.1.R', '{"3": "BNi3.S19.1.R", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.S3.2.R', 'BNi3.S3.2.R', '{"3": "BNi3.S3.2.R", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1С0-135', '1С0-135', '{"3": "1С0-135", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1С0-801', '1С0-801', '{"3": "1С0-801", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1С0-802', '1С0-802', '{"3": "1С0-802", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1С0-803', '1С0-803', '{"3": "1С0-803", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1С0-805', '1С0-805', '{"3": "1С0-805", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4С0-212', '4С0-212', '{"3": "4С0-212", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4С0-802', '4С0-802', '{"3": "4С0-802", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4С0-803', '4С0-803', '{"3": "4С0-803", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4С0-805', '4С0-805', '{"3": "4С0-805", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6С0-532', '6С0-532', '{"3": "6С0-532", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6С0-801', '6С0-801', '{"3": "6С0-801", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Cs137; Eu152', 'Cs137; Eu152', '{"3": "Cs137; Eu152", "4": "цезий-137; европий-152", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Th232;Cs137;Ra226;Raдоч.;K40', 'Th232;Cs137;Ra226;Raдоч.;K40', '{"3": "Th232;Cs137;Ra226;Raдоч.;K40", "4": "торий-232; цезий-137; радий-226; калий-40", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМН', 'ОМН', '{"3": "ОМН", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.191', 'MCo7.191', '{"3": "MCo7.191", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1С0-212', '1С0-212', '{"3": "1С0-212", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6С0-802', '6С0-802', '{"3": "6С0-802", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6С0-803', '6С0-803', '{"3": "6С0-803", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Th232', 'Th232', '{"3": "Th232", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GSR-B', 'GSR-B', '{"3": "GSR-B", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Cs137', 'Cs137', '{"3": "Cs137", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Sr-90;Y-90', 'Sr-90;Y-90', '{"3": "Sr-90;Y-90", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1С0-326', '1С0-326', '{"3": "1С0-326", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.K.C1.1.R', 'BNi3.K.C1.1.R', '{"3": "BNi3.K.C1.1.R", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-РТ', 'ОСГИ-РТ', '{"3": "ОСГИ-РТ", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Оу', 'Оу', '{"3": "Оу", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИК', 'ИК', '{"3": "ИК", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНК-15У', 'КНК-15У', '{"3": "КНК-15У", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ55', 'КНТ55', '{"3": "КНТ55", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.214', 'MSn9.214', '{"3": "MSn9.214", "4": "олово-119м", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCs7.11', 'GCs7.11', '{"3": "GCs7.11", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.224', 'MSn9.224', '{"3": "MSn9.224", "4": "олово-119м", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCs7.14', 'GCs7.14', '{"3": "GCs7.14", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СИГИ-Т', 'СИГИ-Т', '{"3": "СИГИ-Т", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'HCF 2.012.17', 'HCF 2.012.17', '{"3": "HCF 2.012.17", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PU90-499R3', 'PU90-499R3', '{"3": "PU90-499R3", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CTEPH-U1', 'CTEPH-U1', '{"3": "CTEPH-U1", "4": "уран естественный", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDRB 2719 VZ-322', 'CDRB 2719 VZ-322', '{"3": "CDRB 2719 VZ-322", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5П9-14', '5П9-14', '{"3": "5П9-14", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CТЕРН-Th', 'CТЕРН-Th', '{"3": "CТЕРН-Th", "4": "торий естественный", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ELEKTA Gamma Knife Source No: 43685', 'ELEKTA Gamma Knife Source No: 43685', '{"3": "ELEKTA Gamma Knife Source No: 43685", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GSR-D', 'GSR-D', '{"3": "GSR-D", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "1/25/2017 3:18:33 PM", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СТЕРН-U2', 'СТЕРН-U2', '{"3": "СТЕРН-U2", "4": "уран естественный", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GPu2.89', 'GPu2.89', '{"3": "GPu2.89", "4": "плутоний-242", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СТЕРН-К', 'СТЕРН-К', '{"3": "СТЕРН-К", "4": "калий-40", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Gra6.1.Р2', 'Gra6.1.Р2', '{"3": "Gra6.1.Р2", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-24', '5СО-24', '{"3": "5СО-24", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PNG-CB', 'PNG-CB', '{"3": "PNG-CB", "4": "Тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "1/25/2017 3:18:33 PM", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '132', '132', '{"3": "132", "4": "германий-68", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GU5.P03.А', 'GU5.P03.А', '{"3": "GU5.P03.А", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.С3.2.R', 'BNi3.С3.2.R', '{"3": "BNi3.С3.2.R", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БН-600', 'БН-600', '{"3": "БН-600", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М34.2', 'ГИ192М34.2', '{"3": "ГИ192М34.2", "4": "иридий-192", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Г-2', 'Г-2', '{"3": "Г-2", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.CX', 'BNi3.CX', '{"3": "BNi3.CX", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВНТ1-20М', 'ВНТ1-20М', '{"3": "ВНТ1-20М", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.CX.R', 'BNi3.CX.R', '{"3": "BNi3.CX.R", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-36', '5СО-36', '{"3": "5СО-36", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН(Eu)', 'ОМАСН(Eu)', '{"3": "ОМАСН(Eu)", "4": "европий-152", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВНТ1-32М', 'ВНТ1-32М', '{"3": "ВНТ1-32М", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РБМ-К7.Сб.262-1-10', 'РБМ-К7.Сб.262-1-10', '{"3": "РБМ-К7.Сб.262-1-10", "4": "иод-125", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5СО-84', '5СО-84', '{"3": "5СО-84", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'P 2608-100', 'P 2608-100', '{"3": "P 2608-100", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'источник гамма-излучения', 'источник гамма-излучения', '{"3": "источник гамма-излучения", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВНТ3-32', 'ВНТ3-32', '{"3": "ВНТ3-32", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AGAm1.60', 'AGAm1.60', '{"3": "AGAm1.60", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6С0-133', '6С0-133', '{"3": "6С0-133", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Am1.G10', 'Am1.G10', '{"3": "Am1.G10", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6С0-134', '6С0-134', '{"3": "6С0-134", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСS7.012.7', 'ГСS7.012.7', '{"3": "ГСS7.012.7", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FSG61-BB', 'FSG61-BB', '{"3": "FSG61-BB", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6С0-135', '6С0-135', '{"3": "6С0-135", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ1-100', 'ГНТ1-100', '{"3": "ГНТ1-100", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСS7.021.3', 'ГСS7.021.3', '{"3": "ГСS7.021.3", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ1-143-01', 'ГНТ1-143-01', '{"3": "ГНТ1-143-01", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ1-143-02', 'ГНТ1-143-02', '{"3": "ГНТ1-143-02", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GSM1.011.1A', 'GSM1.011.1A', '{"3": "GSM1.011.1A", "4": "самарий-151", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'HEGL-0140', 'HEGL-0140', '{"3": "HEGL-0140", "4": "германий-68", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ1-143-03', 'ГНТ1-143-03', '{"3": "ГНТ1-143-03", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М64', 'ГИ192М64', '{"3": "ГИ192М64", "4": "иридий-192", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНК 15-1', 'КНК 15-1', '{"3": "КНК 15-1", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ 23', 'КНТ 23', '{"3": "КНТ 23", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.CY17', 'AMN.CY17', '{"3": "AMN.CY17", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Р 2608.100', 'Р 2608.100', '{"3": "Р 2608.100", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ 24', 'КНТ 24', '{"3": "КНТ 24", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.CY19', 'AMN.CY19', '{"3": "AMN.CY19", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ1-80', 'ГНТ1-80', '{"3": "ГНТ1-80", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Рудная модель', 'Рудная модель', '{"3": "Рудная модель", "4": "торий-естественный", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ 26', 'КНТ 26', '{"3": "КНТ 26", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KAC10883', 'KAC10883', '{"3": "KAC10883", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ-26-1', 'КНТ-26-1', '{"3": "КНТ-26-1", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AMN.CY20', 'AMN.CY20', '{"3": "AMN.CY20", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'рудная модель', 'рудная модель', '{"3": "рудная модель", "4": "уран обедненный", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ 26-1', 'КНТ 26-1', '{"3": "КНТ 26-1", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ3-26', 'ГНТ3-26', '{"3": "ГНТ3-26", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCI-200', 'MCI-200', '{"3": "MCI-200", "4": "Цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ 31', 'КНТ 31', '{"3": "КНТ 31", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНУ 3', 'КНУ 3', '{"3": "КНУ 3", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.1132', 'MCo7.1132', '{"3": "MCo7.1132", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ3-26М', 'ГНТ3-26М', '{"3": "ГНТ3-26М", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4С0-134', '4С0-134', '{"3": "4С0-134", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.1249', 'MCo7.1249', '{"3": "MCo7.1249", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'С-139', 'С-139', '{"3": "С-139", "4": "иод-131", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ3-42М', 'ГНТ3-42М', '{"3": "ГНТ3-42М", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Na-22', 'Na-22', '{"3": "Na-22", "4": "натрий-22", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PHFB1028', 'PHFB1028', '{"3": "PHFB1028", "4": "прометий-147", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИС-4А', 'БИС-4А', '{"3": "БИС-4А", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ4-26', 'ГНТ4-26', '{"3": "ГНТ4-26", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Th232; Cs137; Ra226; K40', 'Th232; Cs137; Ra226; K40', '{"3": "Th232; Cs137; Ra226; K40", "4": "торий-232; цезий-137; радий-226; калий-40", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ4-67', 'ГНТ4-67', '{"3": "ГНТ4-67", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Rod source, Co-60, 2608-100', 'Rod source, Co-60, 2608-100', '{"3": "Rod source, Co-60, 2608-100", "4": "Кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'VZ-357-001', 'VZ-357-001', '{"3": "VZ-357-001", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'VZ-79-001', 'VZ-79-001', '{"3": "VZ-79-001", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М315', 'ГК60М315', '{"3": "ГК60М315", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ВНТ1-20', 'ВНТ1-20', '{"3": "ВНТ1-20", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6С0-214', '6С0-214', '{"3": "6С0-214", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4С0-135', '4С0-135', '{"3": "4С0-135", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М325', 'ГК60М325', '{"3": "ГК60М325", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CDC.CY 16', 'CDC.CY 16', '{"3": "CDC.CY 16", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6С0-324', '6С0-324', '{"3": "6С0-324", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'генератор радона', 'генератор радона', '{"3": "генератор радона", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М81', 'ГК60М81', '{"3": "ГК60М81", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М41', 'ГК60М41', '{"3": "ГК60М41", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6С0-215', '6С0-215', '{"3": "6С0-215", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КГИ  U235  НГСО-6', 'КГИ  U235  НГСО-6', '{"3": "КГИ  U235  НГСО-6", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60М83', 'ГК60М83', '{"3": "ГК60М83", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КГИ  U235  НГСО-6', 'КГИ  U235  НГСО-6', '{"3": "КГИ  U235  НГСО-6", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГК60Р02', 'ГК60Р02', '{"3": "ГК60Р02", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КГИ Cs137  НГСО-6', 'КГИ Cs137  НГСО-6', '{"3": "КГИ Cs137  НГСО-6", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ1-6', 'КМ1-6', '{"3": "КМ1-6", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМН-Г-3Т', 'ИМН-Г-3Т', '{"3": "ИМН-Г-3Т", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR85.2', 'KR85.2', '{"3": "KR85.2", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИСН-1', 'ОИСН-1', '{"3": "ОИСН-1", "4": "калий-40; радий-226; торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '6С0-325', '6С0-325', '{"3": "6С0-325", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КМ7-4', 'КМ7-4', '{"3": "КМ7-4", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНГ-061', 'ИНГ-061', '{"3": "ИНГ-061", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '2', '2', '{"3": "2", "4": "калий-40; цезий-137; радий-226; торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ-26', 'КНТ-26', '{"3": "КНТ-26", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'САИ-4Б', 'САИ-4Б', '{"3": "САИ-4Б", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КИ 101/06Э', 'КИ 101/06Э', '{"3": "КИ 101/06Э", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'C1610', 'C1610', '{"3": "C1610", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-513-95', 'РИТу-90-513-95', '{"3": "РИТу-90-513-95", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНТ-2', 'ИНТ-2', '{"3": "ИНТ-2", "4": "Тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М11.35', 'НК252М11.35', '{"3": "НК252М11.35", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТу-90-580-120', 'РИТу-90-580-120', '{"3": "РИТу-90-580-120", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS10', 'CS10', '{"3": "CS10", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Капсула BSr0.38', 'Капсула BSr0.38', '{"3": "Капсула BSr0.38", "4": "америций-241; стронций-90; цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1С0-804', '1С0-804', '{"3": "1С0-804", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ц7-5', 'Ц7-5', '{"3": "Ц7-5", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Cs7.P04', 'Cs7.P04', '{"3": "Cs7.P04", "4": "Цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Cs-137, Eu-152, Am-241', 'Cs-137, Eu-152, Am-241', '{"3": "Cs-137, Eu-152, Am-241", "4": "цезий-137; европий-152; америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'FTC-CF', 'FTC-CF', '{"3": "FTC-CF", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Co-60, Cs-137, Eu-152, Am-241', 'Co-60, Cs-137, Eu-152, Am-241', '{"3": "Co-60, Cs-137, Eu-152, Am-241", "4": "кобальт-60; цезий-137; европий-152; америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GGLS-CA', 'GGLS-CA', '{"3": "GGLS-CA", "4": "Америций-241; бериллий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ-1415', 'ТНТ-1415', '{"3": "ТНТ-1415", "4": "Тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЭКД', 'ЭКД', '{"3": "ЭКД", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОА 1.1.1', 'ОА 1.1.1', '{"3": "ОА 1.1.1", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GGLS-CA', 'GGLS-CA', '{"3": "GGLS-CA", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Эталон радия', 'Эталон радия', '{"3": "Эталон радия", "4": "Радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОА 1.1.2', 'ОА 1.1.2', '{"3": "ОА 1.1.2", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОА 1.1.5', 'ОА 1.1.5', '{"3": "ОА 1.1.5", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GGLS-FZ', 'GGLS-FZ', '{"3": "GGLS-FZ", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОА 1.2.1', 'ОА 1.2.1', '{"3": "ОА 1.2.1", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОА 2.1.1', 'ОА 2.1.1', '{"3": "ОА 2.1.1", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '5С0-215', '5С0-215', '{"3": "5С0-215", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GRa6,12', 'GRa6,12', '{"3": "GRa6,12", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОА 2.1.2', 'ОА 2.1.2', '{"3": "ОА 2.1.2", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОА 2.1.3', 'ОА 2.1.3', '{"3": "ОА 2.1.3", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОА С 16', 'ОА С 16', '{"3": "ОА С 16", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СИГИ', 'СИГИ', '{"3": "СИГИ", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИНГ-063', 'ИНГ-063', '{"3": "ИНГ-063", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОИСУ 0011', 'СОИСУ 0011', '{"3": "СОИСУ 0011", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'X9094', 'X9094', '{"3": "X9094", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОИСУ 0031', 'СОИСУ 0031', '{"3": "СОИСУ 0031", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОИСУ 0061', 'СОИСУ 0061', '{"3": "СОИСУ 0061", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОИСУ 0075', 'СОИСУ 0075', '{"3": "СОИСУ 0075", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОИСУ-0575', 'СОИСУ-0575', '{"3": "СОИСУ-0575", "4": "уран-238; уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KUTh', 'KUTh', '{"3": "KUTh", "4": "калий-40 уран-238 торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ 15', 'КНТ 15', '{"3": "КНТ 15", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОИСУ-3310', 'СОИСУ-3310', '{"3": "СОИСУ-3310", "4": "уран-238; уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОИСУ-9020', 'СОИСУ-9020', '{"3": "СОИСУ-9020", "4": "уран-235; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KUTh Field Verifier', 'KUTh Field Verifier', '{"3": "KUTh Field Verifier", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОМП-0,1 НГСО-6', 'СОМП-0,1 НГСО-6', '{"3": "СОМП-0,1 НГСО-6", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СОМП-0,3 НГСО-6', 'СОМП-0,3 НГСО-6', '{"3": "СОМП-0,3 НГСО-6", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СО', 'СО', '{"3": "СО", "4": "уран-233", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'LWD4133', 'LWD4133', '{"3": "LWD4133", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т19-Л150', 'Т19-Л150', '{"3": "Т19-Л150", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т19-Л166', 'Т19-Л166', '{"3": "Т19-Л166", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NNLS-D', 'NNLS-D', '{"3": "NNLS-D", "4": "Америций-241; бериллий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т19-Л193', 'Т19-Л193', '{"3": "Т19-Л193", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т19-Л201', 'Т19-Л201', '{"3": "Т19-Л201", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KACB 17190', 'KACB 17190', '{"3": "KACB 17190", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Т19-Л210', 'Т19-Л210', '{"3": "Т19-Л210", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНК15У1', 'КНК15У1', '{"3": "КНК15У1", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XCd9.2.Р', 'XCd9.2.Р', '{"3": "XCd9.2.Р", "4": "кадмий-109", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТЖИУ.433442.004', 'ТЖИУ.433442.004', '{"3": "ТЖИУ.433442.004", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NSRU', 'NSRU', '{"3": "NSRU", "4": "Америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ1411М', 'ТНТ1411М', '{"3": "ТНТ1411М", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ1413-01', 'ТНТ1413-01', '{"3": "ТНТ1413-01", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ1413-03', 'ТНТ1413-03', '{"3": "ТНТ1413-03", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ1413М', 'ТНТ1413М', '{"3": "ТНТ1413М", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Pu+Be', 'Pu+Be', '{"3": "Pu+Be", "4": "плутоний-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОМАСН (Cs137 Ba133 Y88 Co60)', 'ОМАСН (Cs137 Ba133 Y88 Co60)', '{"3": "ОМАСН (Cs137 Ba133 Y88 Co60)", "4": "цезий-137; барий-133; иттрий-88; кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ1415-01', 'ТНТ1415-01', '{"3": "ТНТ1415-01", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТНТ147', 'ТНТ147', '{"3": "ТНТ147", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IND1401', 'IND1401', '{"3": "IND1401", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RD-2 (Cs-137)', 'RD-2 (Cs-137)', '{"3": "RD-2 (Cs-137)", "4": "Цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'S17S20', 'S17S20', '{"3": "S17S20", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'S17X20', 'S17X20', '{"3": "S17X20", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'S2K', 'S2K', '{"3": "S2K", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'S3T25', 'S3T25', '{"3": "S3T25", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ1-40-03', 'ГНТ1-40-03', '{"3": "ГНТ1-40-03", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'F716I', 'F716I', '{"3": "F716I", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'S3X25', 'S3X25', '{"3": "S3X25", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SGD/О', 'SGD/О', '{"3": "SGD/О", "4": "Цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИК-В3', 'ИК-В3', '{"3": "ИК-В3", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БДПП-2К-1', 'БДПП-2К-1', '{"3": "БДПП-2К-1", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БДПП-3-2', 'БДПП-3-2', '{"3": "БДПП-3-2", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГИ192М34.5', 'ГИ192М34.5', '{"3": "ГИ192М34.5", "4": "иридий-192", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.S3.3.R', 'BNi3.S3.3.R', '{"3": "BNi3.S3.3.R", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'AGAm1.96', 'AGAm1.96', '{"3": "AGAm1.96", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АО 95 0001-2014', 'АО 95 0001-2014', '{"3": "АО 95 0001-2014", "4": "плутоний-242", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСО 95 1299-2011', 'ОСО 95 1299-2011', '{"3": "ОСО 95 1299-2011", "4": "уран-233", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'САИ-05И-01', 'САИ-05И-01', '{"3": "САИ-05И-01", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'никель-63', 'никель-63', '{"3": "никель-63", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ПФ5-40', 'ПФ5-40', '{"3": "ПФ5-40", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Nikel-63', 'Nikel-63', '{"3": "Nikel-63", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'М37С02-18', 'М37С02-18', '{"3": "М37С02-18", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КАСК7987', 'КАСК7987', '{"3": "КАСК7987", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ1-30', 'ГНТ1-30', '{"3": "ГНТ1-30", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Ж83-Р2044', 'Ж83-Р2044', '{"3": "Ж83-Р2044", "4": "Тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-900-2р', 'ОСГИ-900-2р', '{"3": "ОСГИ-900-2р", "4": "натрий-22", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СS10', 'СS10', '{"3": "СS10", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Сs7.P04', 'Сs7.P04', '{"3": "Сs7.P04", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Х 38/2', 'Х 38/2', '{"3": "Х 38/2", "4": "Цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '57157С', '57157С', '{"3": "57157С", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТ3-42', 'ГНТ3-42', '{"3": "ГНТ3-42", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Изокорд', 'Изокорд', '{"3": "Изокорд", "4": "иод-125", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IND 1402', 'IND 1402', '{"3": "IND 1402", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'А3201-4', 'А3201-4', '{"3": "А3201-4", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GGLS-FZ', 'GGLS-FZ', '{"3": "GGLS-FZ", "4": "церий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NSRU', 'NSRU', '{"3": "NSRU", "4": "Америций-241+бериллий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'A 3201-4', 'A 3201-4', '{"3": "A 3201-4", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'LWD4133', 'LWD4133', '{"3": "LWD4133", "4": "америций241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'LWD4173', 'LWD4173', '{"3": "LWD4173", "4": "америций 241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'LWD4173', 'LWD4173', '{"3": "LWD4173", "4": "америций241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'A3201', 'A3201', '{"3": "A3201", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '4839 FTC-CF', '4839 FTC-CF', '{"3": "4839 FTC-CF", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО 7250-96', 'ГСО 7250-96', '{"3": "ГСО 7250-96", "4": "торий-естественный", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО 7250-96', 'ГСО 7250-96', '{"3": "ГСО 7250-96", "4": "уран естественный", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'А3888', 'А3888', '{"3": "А3888", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SGD/O', 'SGD/O', '{"3": "SGD/O", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'A 3888', 'A 3888', '{"3": "A 3888", "4": "натрий-22", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'A3916', 'A3916', '{"3": "A3916", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'STB A 3916-7', 'STB A 3916-7', '{"3": "STB A 3916-7", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'A3201-3', 'A3201-3', '{"3": "A3201-3", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ELEKTA Gamma Knife Source No:43685', 'ELEKTA Gamma Knife Source No:43685', '{"3": "ELEKTA Gamma Knife Source No:43685", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РКШЕ.944450.002.', 'РКШЕ.944450.002.', '{"3": "РКШЕ.944450.002.", "4": "иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Детекторный узел', 'Детекторный узел', '{"3": "Детекторный узел", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GM-232.02.000', 'GM-232.02.000', '{"3": "GM-232.02.000", "4": "иридий-192", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'изделие ГНТ1-100', 'изделие ГНТ1-100', '{"3": "изделие ГНТ1-100", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'камера КНТ-23', 'камера КНТ-23', '{"3": "камера КНТ-23", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'камера КНТ-25', 'камера КНТ-25', '{"3": "камера КНТ-25", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Макет БДБН с кер. опор. изоляторами', 'Макет БДБН с кер. опор. изоляторами', '{"3": "Макет БДБН с кер. опор. изоляторами", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Макет КНТ30-8', 'Макет КНТ30-8', '{"3": "Макет КНТ30-8", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'нейтронная трубка ГНТ1-100-01', 'нейтронная трубка ГНТ1-100-01', '{"3": "нейтронная трубка ГНТ1-100-01", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'нейтронная трубка ГНТЗ-67=01', 'нейтронная трубка ГНТЗ-67=01', '{"3": "нейтронная трубка ГНТЗ-67=01", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'нейтронная трубка ТНТ-147-01', 'нейтронная трубка ТНТ-147-01', '{"3": "нейтронная трубка ТНТ-147-01", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Опытный образец БДБН-Н', 'Опытный образец БДБН-Н', '{"3": "Опытный образец БДБН-Н", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Опытный образец КНТ 34', 'Опытный образец КНТ 34', '{"3": "Опытный образец КНТ 34", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ №6321', 'ОСАИ №6321', '{"3": "ОСАИ №6321", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ №6321', 'ОСАИ №6321', '{"3": "ОСАИ №6321", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ №6321', 'ОСАИ №6321', '{"3": "ОСАИ №6321", "4": "плутоний-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСАИ №6321', 'ОСАИ №6321', '{"3": "ОСАИ №6321", "4": "уран-233; плутоний-238; плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Опытный образец  ДМНП с ДУ-8', 'Опытный образец  ДМНП с ДУ-8', '{"3": "Опытный образец  ДМНП с ДУ-8", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СКС', 'СКС', '{"3": "СКС", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов из 30 СРЗ", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НК252М12', 'НК252М12', '{"3": "НК252М12", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСS7.012', 'ГСS7.012', '{"3": "ГСS7.012", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НCf2.021.2', 'НCf2.021.2', '{"3": "НCf2.021.2", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГТ-2м', 'ГТ-2м', '{"3": "ГТ-2м", "4": "молибден-99; технеций-99м", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'изосид', 'изосид', '{"3": "изосид", "4": "иод-125", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'САИ-6', 'САИ-6', '{"3": "САИ-6", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GM-232.02.000-SFC', 'GM-232.02.000-SFC', '{"3": "GM-232.02.000-SFC", "4": "иридий-192", "5": "1", "6": "", "7": "428000000000.0", "8": "436000000000000.0", "9": "0", "10": "Германия", "11": "1", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СР18.140', 'СР18.140', '{"3": "СР18.140", "4": "селен-75", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СР18.90', 'СР18.90', '{"3": "СР18.90", "4": "селен-75", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СР18.40', 'СР18.40', '{"3": "СР18.40", "4": "селен-75", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИСН', 'ОИСН', '{"3": "ОИСН", "4": "цезий-137; торий-232; радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSn9.223', 'MSn9.223', '{"3": "MSn9.223", "4": "олово-119м", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RU6.A13', 'RU6.A13', '{"3": "RU6.A13", "4": "рутений-106", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RU6.A12', 'RU6.A12', '{"3": "RU6.A12", "4": "рутений-106", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.C3.1R', 'BNi3.C3.1R', '{"3": "BNi3.C3.1R", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KACK 8044', 'KACK 8044', '{"3": "KACK 8044", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИПК ТУ', 'ИПК ТУ', '{"3": "ИПК ТУ", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Р56-Л1544', 'Р56-Л1544', '{"3": "Р56-Л1544", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РМТ2', 'РМТ2', '{"3": "РМТ2", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-90-550-120', 'РИТ-90-550-120', '{"3": "РИТ-90-550-120", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИТ-90-580-120', 'РИТ-90-580-120', '{"3": "РИТ-90-580-120", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИСН', 'ОИСН', '{"3": "ОИСН", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BAG 3', 'BAG 3', '{"3": "BAG 3", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'COG 3', 'COG 3', '{"3": "COG 3", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CSG 3', 'CSG 3', '{"3": "CSG 3", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EUG 3', 'EUG 3', '{"3": "EUG 3", "4": "европий-152", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NAG 3', 'NAG 3', '{"3": "NAG 3", "4": "натрий-22", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '3H', '3H', '{"3": "3H", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'АРИА', 'АРИА', '{"3": "АРИА", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ДП-64', 'ДП-64', '{"3": "ДП-64", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'НАД-4', 'НАД-4', '{"3": "НАД-4", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'TG4 S-20', 'TG4 S-20', '{"3": "TG4 S-20", "4": "прометий-147", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SO61', 'SO61', '{"3": "SO61", "4": "иод-125", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.12', 'MCo7.12', '{"3": "MCo7.12", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИМН-Г-3', 'ИМН-Г-3', '{"3": "ИМН-Г-3", "4": "барий-133; цезий-134; цезий-137; калий-40; натрий-22; америций-241; кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'B', 'B', '{"3": "B", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '19513-11', '19513-11', '{"3": "19513-11", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Офтальмоаппликатор (ОА)', 'Офтальмоаппликатор (ОА)', '{"3": "Офтальмоаппликатор (ОА)", "4": "рутений-106", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RBa3.1.1', 'RBa3.1.1', '{"3": "RBa3.1.1", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RCo7.1.1', 'RCo7.1.1', '{"3": "RCo7.1.1", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.16', 'GCo7.16', '{"3": "GCo7.16", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RCs7.1.1', 'RCs7.1.1', '{"3": "RCs7.1.1", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCs7.T2', 'GCs7.T2', '{"3": "GCs7.T2", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RCs7.1.2', 'RCs7.1.2', '{"3": "RCs7.1.2", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCs7.T5', 'GCs7.T5', '{"3": "GCs7.T5", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'PNG-GC', 'PNG-GC', '{"3": "PNG-GC", "4": "тритий", "5": "4", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "5", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'microSelectron-HDR V2', 'microSelectron-HDR V2', '{"3": "microSelectron-HDR V2", "4": "иридий-192", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GBa3.19', 'GBa3.19', '{"3": "GBa3.19", "4": "барий-133", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo0.19', 'GCo0.19', '{"3": "GCo0.19", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo7.19', 'GCo7.19', '{"3": "GCo7.19", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Калибровочный источник', 'Калибровочный источник', '{"3": "Калибровочный источник", "4": "плутоний-239", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Калибровочный источник', 'Калибровочный источник', '{"3": "Калибровочный источник", "4": "натрий-22", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Плутон', 'Плутон', '{"3": "Плутон", "4": "плутоний-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИСН-АЛЬФА', 'ОИСН-АЛЬФА', '{"3": "ОИСН-АЛЬФА", "4": "плутоний-238; плутоний-239; плутоний-242", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИСН-БЕТА', 'ОИСН-БЕТА', '{"3": "ОИСН-БЕТА", "4": "калий-40", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Плутон', 'Плутон', '{"3": "Плутон", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Плутон', 'Плутон', '{"3": "Плутон", "4": "натрий-22", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Плутон', 'Плутон', '{"3": "Плутон", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИСН-234', 'ОИСН-234', '{"3": "ОИСН-234", "4": "уран-234; уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.192', 'MCo7.192', '{"3": "MCo7.192", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'XFe5.24', 'XFe5.24', '{"3": "XFe5.24", "4": "железо-55", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SIRB1515', 'SIRB1515', '{"3": "SIRB1515", "4": "стронций-90", "5": "2", "6": "", "7": "0.0", "8": "33300000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'SIRB1612', 'SIRB1612', '{"3": "SIRB1612", "4": "стронций-90", "5": "2", "6": "", "7": "0.0", "8": "33000000.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GNa2.M.29.1', 'GNa2.M.29.1', '{"3": "GNa2.M.29.1", "4": "натрий-22", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IIб-1', 'IIб-1', '{"3": "IIб-1", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.1134', 'MCo7.1134', '{"3": "MCo7.1134", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MSm1.X20X', 'MSm1.X20X', '{"3": "MSm1.X20X", "4": "самарий-151", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1СО-134.13', '1СО-134.13', '{"3": "1СО-134.13", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'RCo7.1.2', 'RCo7.1.2', '{"3": "RCo7.1.2", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'MCo7.1148', 'MCo7.1148', '{"3": "MCo7.1148", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КДУ-2', 'КДУ-2', '{"3": "КДУ-2", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КДУС-1', 'КДУС-1', '{"3": "КДУС-1", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EM145', 'EM145', '{"3": "EM145", "4": "стронций-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EM445', 'EM445', '{"3": "EM445", "4": "америций-241", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCs7.27', 'GCs7.27', '{"3": "GCs7.27", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Rco0.1.1', 'Rco0.1.1', '{"3": "Rco0.1.1", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo0.11', 'GCo0.11', '{"3": "GCo0.11", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GNa2.11', 'GNa2.11', '{"3": "GNa2.11", "4": "натрий-22", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IsoSeed', 'IsoSeed', '{"3": "IsoSeed", "4": "иод-125", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5E0202200M', 'KR5E0202200M', '{"3": "KR5E0202200M", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'образец-свидетель', 'образец-свидетель', '{"3": "образец-свидетель", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'CS7P030003M', 'CS7P030003M', '{"3": "CS7P030003M", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'X 119', 'X 119', '{"3": "X 119", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KR5E040500M', 'KR5E040500M', '{"3": "KR5E040500M", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БДБН', 'БДБН', '{"3": "БДБН", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'БИТ-М5', 'БИТ-М5', '{"3": "БИТ-М5", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГНТЗ-67', 'ГНТЗ-67', '{"3": "ГНТЗ-67", "4": "тритий", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ДМНП', 'ДМНП', '{"3": "ДМНП", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Закись-окись', 'Закись-окись', '{"3": "Закись-окись", "4": "уран-236", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Закись-окись', 'Закись-окись', '{"3": "Закись-окись", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ИКД КНТ 30-5', 'ИКД КНТ 30-5', '{"3": "ИКД КНТ 30-5", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КНТ-54-2-1', 'КНТ-54-2-1', '{"3": "КНТ-54-2-1", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'макет камеры', 'макет камеры', '{"3": "макет камеры", "4": "уран-235", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'макет камеры', 'макет камеры', '{"3": "макет камеры", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Макет узла', 'Макет узла', '{"3": "Макет узла", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'BNi3.K.C3.4.R', 'BNi3.K.C3.4.R', '{"3": "BNi3.K.C3.4.R", "4": "никель-63", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'МСо7', 'МСо7', '{"3": "МСо7", "4": "Кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Опытный образец БДБН нижней ВНК', 'Опытный образец БДБН нижней ВНК', '{"3": "Опытный образец БДБН нижней ВНК", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Опытный образец БДПН', 'Опытный образец БДПН', '{"3": "Опытный образец БДПН", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Опытный образец ИКД', 'Опытный образец ИКД', '{"3": "Опытный образец ИКД", "4": "уран-238", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Pip tag', 'Pip tag', '{"3": "Pip tag", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', '1CO-123', '1CO-123', '{"3": "1CO-123", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО 10964-2017', 'ГСО 10964-2017', '{"3": "ГСО 10964-2017", "4": "радий-226", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО 10966-2017', 'ГСО 10966-2017', '{"3": "ГСО 10966-2017", "4": "торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ГСО 10973-2017', 'ГСО 10973-2017', '{"3": "ГСО 10973-2017", "4": "радий-226; торий-232", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GCo0.15', 'GCo0.15', '{"3": "GCo0.15", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'KAC10882', 'KAC10882', '{"3": "KAC10882", "4": "криптон-85", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РИАН', 'РИАН', '{"3": "РИАН", "4": "железо-55", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РХЛ', 'РХЛ', '{"3": "РХЛ", "4": "железо-55", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РХЛ', 'РХЛ', '{"3": "РХЛ", "4": "кадмий-109", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'РХЛ', 'РХЛ', '{"3": "РХЛ", "4": "гафний-181", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'EG-0320', 'EG-0320', '{"3": "EG-0320", "4": "германий-68", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'черновой урано-фланцевый', 'черновой урано-фланцевый', '{"3": "черновой урано-фланцевый", "4": "торий-227; торий-228", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'IGI-Z-22-1', 'IGI-Z-22-1', '{"3": "IGI-Z-22-1", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'СР18.200', 'СР18.200', '{"3": "СР18.200", "4": "селен-75", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ЗИНК-244', 'ЗИНК-244', '{"3": "ЗИНК-244", "4": "кюрий-244", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'КИ101/06Э', 'КИ101/06Э', '{"3": "КИ101/06Э", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'GNa2.M.29.2', 'GNa2.M.29.2', '{"3": "GNa2.M.29.2", "4": "натрий-22", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'Гамма-нож Elekta', 'Гамма-нож Elekta', '{"3": "Гамма-нож Elekta", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов (в спр есть еще такой тип", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТРИБ-90-200/30800-95', 'ТРИБ-90-200/30800-95', '{"3": "ТРИБ-90-200/30800-95", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ТРИБ-90-230-95', 'ТРИБ-90-230-95', '{"3": "ТРИБ-90-230-95", "4": "стронций-90; иттрий-90", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'NCf252.07.05', 'NCf252.07.05', '{"3": "NCf252.07.05", "4": "калифорний-252", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-РТ', 'ОСГИ-РТ', '{"3": "ОСГИ-РТ", "4": "иттрий-88", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степапнов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-РТ', 'ОСГИ-РТ', '{"3": "ОСГИ-РТ", "4": "торий-228", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степапнов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-П', 'ОСГИ-П', '{"3": "ОСГИ-П", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степапнов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОСГИ-П', 'ОСГИ-П', '{"3": "ОСГИ-П", "4": "цезий-137", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степапнов", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'изокорд-изосид', 'изокорд-изосид', '{"3": "изокорд-изосид", "4": "иод-125", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОЗРИ', 'ОЗРИ', '{"3": "ОЗРИ", "4": "кобальт-60", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "", "adding_date": "2023-06-05"}', 1, 0, '0');
INSERT INTO "public"."VocValues" (voc_type, value_id, label, attrs, value_status, creator_id, kod_org) VALUES ('23_voc', 'ОИДК-Р', 'ОИДК-Р', '{"3": "ОИДК-Р", "4": "кобальт-57", "5": "0", "6": "", "7": "0.0", "8": "0.0", "9": "0", "10": "", "11": "0", "12": "Степанов", "adding_date": "2023-06-05"}', 1, 0, '0');


--
-- TOC entry 3119 (class 0 OID 19719)
-- Dependencies: 197
-- Data for Name: Vocs; Type: TABLE DATA; Schema: public; Owner: db_admin
--



--
-- TOC entry 3149 (class 0 OID 100132)
-- Dependencies: 227
-- Data for Name: acc_fc; Type: TABLE DATA; Schema: public; Owner: user1
--



--
-- TOC entry 3137 (class 0 OID 32216)
-- Dependencies: 215
-- Data for Name: acc_o1; Type: TABLE DATA; Schema: public; Owner: db_admin
--



--
-- TOC entry 3138 (class 0 OID 32224)
-- Dependencies: 216
-- Data for Name: acc_o2; Type: TABLE DATA; Schema: public; Owner: db_admin
--



--
-- TOC entry 3139 (class 0 OID 32231)
-- Dependencies: 217
-- Data for Name: acc_o3; Type: TABLE DATA; Schema: public; Owner: db_admin
--



--
-- TOC entry 3140 (class 0 OID 32238)
-- Dependencies: 218
-- Data for Name: acc_o4; Type: TABLE DATA; Schema: public; Owner: db_admin
--



--
-- TOC entry 3141 (class 0 OID 32245)
-- Dependencies: 219
-- Data for Name: acc_o5; Type: TABLE DATA; Schema: public; Owner: db_admin
--



--
-- TOC entry 3142 (class 0 OID 32252)
-- Dependencies: 220
-- Data for Name: acc_o6; Type: TABLE DATA; Schema: public; Owner: db_admin
--



--
-- TOC entry 3143 (class 0 OID 32259)
-- Dependencies: 221
-- Data for Name: acc_o7; Type: TABLE DATA; Schema: public; Owner: db_admin
--



--
-- TOC entry 3144 (class 0 OID 32266)
-- Dependencies: 222
-- Data for Name: acc_o8; Type: TABLE DATA; Schema: public; Owner: db_admin
--



--
-- TOC entry 3145 (class 0 OID 32273)
-- Dependencies: 223
-- Data for Name: acc_o9; Type: TABLE DATA; Schema: public; Owner: db_admin
--



--
-- TOC entry 3136 (class 0 OID 32209)
-- Dependencies: 214
-- Data for Name: acc_oper; Type: TABLE DATA; Schema: public; Owner: db_admin
--



--
-- TOC entry 3135 (class 0 OID 32198)
-- Dependencies: 213
-- Data for Name: acc_proto; Type: TABLE DATA; Schema: public; Owner: db_admin
--



--
-- TOC entry 3129 (class 0 OID 28183)
-- Dependencies: 207
-- Data for Name: bugreports; Type: TABLE DATA; Schema: public; Owner: db_admin
--



--
-- TOC entry 3147 (class 0 OID 40186)
-- Dependencies: 225
-- Data for Name: hints; Type: TABLE DATA; Schema: public; Owner: db_admin
--



--
-- TOC entry 3133 (class 0 OID 28228)
-- Dependencies: 211
-- Data for Name: user_logs; Type: TABLE DATA; Schema: public; Owner: db_admin
--



--
-- TOC entry 3131 (class 0 OID 28212)
-- Dependencies: 209
-- Data for Name: users; Type: TABLE DATA; Schema: public; Owner: db_admin
--

INSERT INTO public.users VALUES (1, 'Администратор', false, 'admin', 'pbkdf2:sha256:260000$oL6OAOYomz7X7PIe$11a856a533a483c7189bff0524935e95633eeeab66a37d26411cf449eae05fc6', 'rquwjGHPlt8wdpo1FD9Wj8OEZ89wDPuB4Pzwu9iL30ct7WdzDlDtzhXwQqWCGoTCxOG-cf6XQMStJ4rd3OMaLdr_DppDlryMhBXn79F2EfCFikSwlFMV_ImuaGIOVVle6L010NOowajnSpFuod438cFjjU75FG5u', 'administrator', 8191, NULL, NULL, '0', NULL, '{}', NULL, NULL);


--
-- TOC entry 3172 (class 0 OID 0)
-- Dependencies: 203
-- Name: AReports_areport_id_seq; Type: SEQUENCE SET; Schema: public; Owner: db_admin
--

SELECT pg_catalog.setval('public."AReports_areport_id_seq"', 397, true);


--
-- TOC entry 3173 (class 0 OID 0)
-- Dependencies: 201
-- Name: ReportForms_report_form_id_seq; Type: SEQUENCE SET; Schema: public; Owner: db_admin
--

SELECT pg_catalog.setval('public."ReportForms_report_form_id_seq"', 1176, true);


--
-- TOC entry 3174 (class 0 OID 0)
-- Dependencies: 198
-- Name: VocValues_record_id_seq; Type: SEQUENCE SET; Schema: public; Owner: db_admin
--

SELECT pg_catalog.setval('public."VocValues_record_id_seq"', 31890, true);


--
-- TOC entry 3175 (class 0 OID 0)
-- Dependencies: 196
-- Name: Vocs_voc_id_seq; Type: SEQUENCE SET; Schema: public; Owner: db_admin
--

SELECT pg_catalog.setval('public."Vocs_voc_id_seq"', 30, true);


--
-- TOC entry 3176 (class 0 OID 0)
-- Dependencies: 226
-- Name: acc_fc_record_id_seq; Type: SEQUENCE SET; Schema: public; Owner: user1
--

SELECT pg_catalog.setval('public.acc_fc_record_id_seq', 1, false);


--
-- TOC entry 3177 (class 0 OID 0)
-- Dependencies: 212
-- Name: acc_proto_record_id_seq; Type: SEQUENCE SET; Schema: public; Owner: db_admin
--

SELECT pg_catalog.setval('public.acc_proto_record_id_seq', 111975, true);


--
-- TOC entry 3178 (class 0 OID 0)
-- Dependencies: 206
-- Name: bugreports_record_id_seq; Type: SEQUENCE SET; Schema: public; Owner: db_admin
--

SELECT pg_catalog.setval('public.bugreports_record_id_seq', 433, true);


--
-- TOC entry 3179 (class 0 OID 0)
-- Dependencies: 224
-- Name: hints_record_id_seq; Type: SEQUENCE SET; Schema: public; Owner: db_admin
--

SELECT pg_catalog.setval('public.hints_record_id_seq', 7, true);


--
-- TOC entry 3180 (class 0 OID 0)
-- Dependencies: 210
-- Name: user_logs_record_id_seq; Type: SEQUENCE SET; Schema: public; Owner: db_admin
--

SELECT pg_catalog.setval('public.user_logs_record_id_seq', 2223, true);


--
-- TOC entry 3181 (class 0 OID 0)
-- Dependencies: 208
-- Name: users_record_id_seq; Type: SEQUENCE SET; Schema: public; Owner: db_admin
--

SELECT pg_catalog.setval('public.users_record_id_seq', 191, true);


--
-- TOC entry 2979 (class 2606 OID 19831)
-- Name: AReports AReports_pkey; Type: CONSTRAINT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public."AReports"
    ADD CONSTRAINT "AReports_pkey" PRIMARY KEY (areport_id);


--
-- TOC entry 2981 (class 2606 OID 20001)
-- Name: AccMoves AccMoves_pkey; Type: CONSTRAINT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public."AccMoves"
    ADD CONSTRAINT "AccMoves_pkey" PRIMARY KEY (record_id);


--
-- TOC entry 2977 (class 2606 OID 19804)
-- Name: ReportForms ReportForms_pkey; Type: CONSTRAINT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public."ReportForms"
    ADD CONSTRAINT "ReportForms_pkey" PRIMARY KEY (report_form_id);


--
-- TOC entry 2975 (class 2606 OID 19738)
-- Name: VocValues VocValues_pkey; Type: CONSTRAINT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public."VocValues"
    ADD CONSTRAINT "VocValues_pkey" PRIMARY KEY (record_id);


--
-- TOC entry 2973 (class 2606 OID 19727)
-- Name: Vocs Vocs_pkey; Type: CONSTRAINT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public."Vocs"
    ADD CONSTRAINT "Vocs_pkey" PRIMARY KEY (voc_id);


--
-- TOC entry 2993 (class 2606 OID 100140)
-- Name: acc_fc acc_fc_pkey; Type: CONSTRAINT; Schema: public; Owner: user1
--

ALTER TABLE ONLY public.acc_fc
    ADD CONSTRAINT acc_fc_pkey PRIMARY KEY (record_id);


--
-- TOC entry 2989 (class 2606 OID 32203)
-- Name: acc_proto acc_proto_pkey; Type: CONSTRAINT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public.acc_proto
    ADD CONSTRAINT acc_proto_pkey PRIMARY KEY (record_id);


--
-- TOC entry 2983 (class 2606 OID 28194)
-- Name: bugreports bugreports_pkey; Type: CONSTRAINT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public.bugreports
    ADD CONSTRAINT bugreports_pkey PRIMARY KEY (record_id);


--
-- TOC entry 2991 (class 2606 OID 40194)
-- Name: hints hints_pkey; Type: CONSTRAINT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public.hints
    ADD CONSTRAINT hints_pkey PRIMARY KEY (record_id);


--
-- TOC entry 2987 (class 2606 OID 28238)
-- Name: user_logs user_logs_pkey; Type: CONSTRAINT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public.user_logs
    ADD CONSTRAINT user_logs_pkey PRIMARY KEY (record_id);


--
-- TOC entry 2985 (class 2606 OID 28223)
-- Name: users users_pkey; Type: CONSTRAINT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public.users
    ADD CONSTRAINT users_pkey PRIMARY KEY (record_id);


--
-- TOC entry 2996 (class 2620 OID 20196)
-- Name: AReports tg_areport_state_update; Type: TRIGGER; Schema: public; Owner: db_admin
--

CREATE TRIGGER tg_areport_state_update BEFORE INSERT OR UPDATE ON public."AReports" FOR EACH ROW EXECUTE PROCEDURE public.areport_status_update();


--
-- TOC entry 2995 (class 2606 OID 100141)
-- Name: acc_fc acc_fc_report_form_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: user1
--

ALTER TABLE ONLY public.acc_fc
    ADD CONSTRAINT acc_fc_report_form_id_fkey FOREIGN KEY (report_form_id) REFERENCES public."ReportForms"(report_form_id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- TOC entry 2994 (class 2606 OID 32204)
-- Name: acc_proto acc_proto_report_form_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: db_admin
--

ALTER TABLE ONLY public.acc_proto
    ADD CONSTRAINT acc_proto_report_form_id_fkey FOREIGN KEY (report_form_id) REFERENCES public."ReportForms"(report_form_id) ON DELETE CASCADE;


--
-- TOC entry 3156 (class 0 OID 0)
-- Dependencies: 204
-- Name: TABLE "AReports"; Type: ACL; Schema: public; Owner: db_admin
--

GRANT SELECT ON TABLE public."AReports" TO ro_user;
GRANT SELECT,INSERT,DELETE,UPDATE ON TABLE public."AReports" TO rw_user;


--
-- TOC entry 3158 (class 0 OID 0)
-- Dependencies: 205
-- Name: TABLE "AccMoves"; Type: ACL; Schema: public; Owner: db_admin
--

GRANT SELECT ON TABLE public."AccMoves" TO ro_user;
GRANT SELECT,INSERT,DELETE,UPDATE ON TABLE public."AccMoves" TO rw_user;
GRANT SELECT,INSERT,DELETE,UPDATE ON TABLE public."AccMoves" TO PUBLIC;


--
-- TOC entry 3159 (class 0 OID 0)
-- Dependencies: 202
-- Name: TABLE "ReportForms"; Type: ACL; Schema: public; Owner: db_admin
--

GRANT SELECT,INSERT,DELETE,UPDATE ON TABLE public."ReportForms" TO rw_user;
GRANT SELECT ON TABLE public."ReportForms" TO ro_user;


--
-- TOC entry 3161 (class 0 OID 0)
-- Dependencies: 199
-- Name: TABLE "VocValues"; Type: ACL; Schema: public; Owner: db_admin
--

GRANT SELECT ON TABLE public."VocValues" TO ro_user;
GRANT SELECT,INSERT,DELETE,UPDATE ON TABLE public."VocValues" TO rw_user;


--
-- TOC entry 3162 (class 0 OID 0)
-- Dependencies: 200
-- Name: TABLE "VocOrgs"; Type: ACL; Schema: public; Owner: db_admin
--

GRANT SELECT ON TABLE public."VocOrgs" TO ro_user;
GRANT SELECT,INSERT,DELETE,UPDATE ON TABLE public."VocOrgs" TO rw_user;


--
-- TOC entry 3164 (class 0 OID 0)
-- Dependencies: 197
-- Name: TABLE "Vocs"; Type: ACL; Schema: public; Owner: db_admin
--

GRANT SELECT ON TABLE public."Vocs" TO ro_user;
GRANT SELECT,INSERT,DELETE,UPDATE ON TABLE public."Vocs" TO rw_user;


-- Completed on 2022-11-30 13:43:07 MSK

--
-- PostgreSQL database dump complete
--

